/*
 * Project    : ipv6calc/ipv6logstats
 * File       : ipv6logstatshelp.h
 * Version    : $Id: ipv6logstatshelp.h,v 1.4 2014/07/22 06:00:41 ds6peter Exp $
 * Copyright  : 2003-2014 by Peter Bieringer <pb (at) bieringer.de>
 *
 * Information:
 *  Header file for ipv6logstatshelp.c
 */

extern void printversion(void);
extern void printversion_help(void);
extern void printversion_verbose(int level_verbose);
extern void ipv6logstats_printinfo(void);
extern void ipv6logstats_printhelp(void);
