/*
 * Project    : ipv6calc
 * File       : ipv6loganonhelp.h
 * Version    : $Id: ipv6loganonhelp.h,v 1.2 2014/07/22 06:00:41 ds6peter Exp $
 * Copyright  : 2007-2014 by Peter Bieringer <pb (at) bieringer.de>
 * License    : GNU GPL v2
 *
 * Information:
 *  Header file for ipv6loganonhelp.c
 */

extern void printversion(void);
extern void printversion_help(void);
extern void ipv6loganon_printinfo(void);
extern void ipv6loganon_printhelp(void);
