 /*
 * Project       : ipv6calc
 * File          : db_cc_reg_assignment.h
 * Version       : $Id: db_cc_reg_assignment.h,v 1.18 2015/02/20 07:00:22 ds6peter Exp $
 * Generated     : Fri Feb 20 07:47:42 2015
 * Data copyright: RIPE NCC, APNIC, ARIN, LACNIC, AFRINIC
 *
 * Information:
 *  Additional header file for databases/lib/libipv6calc_db_wrapper_BuiltIn.c
 */

static const char *db_cc_registry_status = "AFRINIC/20150220 APNIC/20150219 ARIN/20150219 LACNIC/20150219 RIPENCC/20150219";

static const s_cc_reg_assignment cc_reg_assignment[] = {
	{ "", REGISTRY_ARIN     },
	{ "AD", REGISTRY_RIPENCC  },
	{ "AE", REGISTRY_RIPENCC  },
	{ "AF", REGISTRY_APNIC    },
	{ "AG", REGISTRY_ARIN     },
	{ "AI", REGISTRY_ARIN     },
	{ "AL", REGISTRY_RIPENCC  },
	{ "AM", REGISTRY_RIPENCC  },
	{ "AO", REGISTRY_AFRINIC  },
	{ "AP", REGISTRY_APNIC    },
	{ "AR", REGISTRY_LACNIC   },
	{ "AS", REGISTRY_APNIC    },
	{ "AT", REGISTRY_RIPENCC  },
	{ "AU", REGISTRY_APNIC    },
	{ "AW", REGISTRY_LACNIC   },
	{ "AZ", REGISTRY_RIPENCC  },
	{ "BA", REGISTRY_RIPENCC  },
	{ "BB", REGISTRY_ARIN     },
	{ "BD", REGISTRY_APNIC    },
	{ "BE", REGISTRY_RIPENCC  },
	{ "BF", REGISTRY_AFRINIC  },
	{ "BG", REGISTRY_RIPENCC  },
	{ "BH", REGISTRY_RIPENCC  },
	{ "BI", REGISTRY_AFRINIC  },
	{ "BJ", REGISTRY_AFRINIC  },
	{ "BL", REGISTRY_ARIN     },
	{ "BM", REGISTRY_ARIN     },
	{ "BN", REGISTRY_APNIC    },
	{ "BO", REGISTRY_LACNIC   },
	{ "BQ", REGISTRY_LACNIC   },
	{ "BR", REGISTRY_LACNIC   },
	{ "BS", REGISTRY_ARIN     },
	{ "BT", REGISTRY_APNIC    },
	{ "BW", REGISTRY_AFRINIC  },
	{ "BY", REGISTRY_RIPENCC  },
	{ "BZ", REGISTRY_LACNIC   },
	{ "CA", REGISTRY_ARIN     },
	{ "CD", REGISTRY_AFRINIC  },
	{ "CF", REGISTRY_AFRINIC  },
	{ "CG", REGISTRY_AFRINIC  },
	{ "CH", REGISTRY_RIPENCC  },
	{ "CI", REGISTRY_AFRINIC  },
	{ "CK", REGISTRY_APNIC    },
	{ "CL", REGISTRY_LACNIC   },
	{ "CM", REGISTRY_AFRINIC  },
	{ "CN", REGISTRY_APNIC    },
	{ "CO", REGISTRY_LACNIC   },
	{ "CR", REGISTRY_LACNIC   },
	{ "CU", REGISTRY_LACNIC   },
	{ "CV", REGISTRY_AFRINIC  },
	{ "CW", REGISTRY_LACNIC   },
	{ "CY", REGISTRY_RIPENCC  },
	{ "CZ", REGISTRY_RIPENCC  },
	{ "DE", REGISTRY_RIPENCC  },
	{ "DJ", REGISTRY_AFRINIC  },
	{ "DK", REGISTRY_RIPENCC  },
	{ "DM", REGISTRY_ARIN     },
	{ "DO", REGISTRY_LACNIC   },
	{ "DZ", REGISTRY_AFRINIC  },
	{ "EC", REGISTRY_LACNIC   },
	{ "EE", REGISTRY_RIPENCC  },
	{ "EG", REGISTRY_AFRINIC  },
	{ "ER", REGISTRY_AFRINIC  },
	{ "ES", REGISTRY_RIPENCC  },
	{ "ET", REGISTRY_AFRINIC  },
	{ "EU", REGISTRY_RIPENCC  },
	{ "FI", REGISTRY_RIPENCC  },
	{ "FJ", REGISTRY_APNIC    },
	{ "FM", REGISTRY_APNIC    },
	{ "FO", REGISTRY_RIPENCC  },
	{ "FR", REGISTRY_RIPENCC  },
	{ "GA", REGISTRY_AFRINIC  },
	{ "GB", REGISTRY_RIPENCC  },
	{ "GD", REGISTRY_ARIN     },
	{ "GE", REGISTRY_RIPENCC  },
	{ "GF", REGISTRY_LACNIC   },
	{ "GG", REGISTRY_RIPENCC  },
	{ "GH", REGISTRY_AFRINIC  },
	{ "GI", REGISTRY_RIPENCC  },
	{ "GL", REGISTRY_RIPENCC  },
	{ "GM", REGISTRY_AFRINIC  },
	{ "GN", REGISTRY_AFRINIC  },
	{ "GP", REGISTRY_ARIN     },
	{ "GQ", REGISTRY_AFRINIC  },
	{ "GR", REGISTRY_RIPENCC  },
	{ "GT", REGISTRY_LACNIC   },
	{ "GU", REGISTRY_APNIC    },
	{ "GW", REGISTRY_AFRINIC  },
	{ "GY", REGISTRY_LACNIC   },
	{ "HK", REGISTRY_APNIC    },
	{ "HN", REGISTRY_LACNIC   },
	{ "HR", REGISTRY_RIPENCC  },
	{ "HT", REGISTRY_LACNIC   },
	{ "HU", REGISTRY_RIPENCC  },
	{ "ID", REGISTRY_APNIC    },
	{ "IE", REGISTRY_RIPENCC  },
	{ "IL", REGISTRY_RIPENCC  },
	{ "IM", REGISTRY_RIPENCC  },
	{ "IN", REGISTRY_APNIC    },
	{ "IO", REGISTRY_APNIC    },
	{ "IQ", REGISTRY_RIPENCC  },
	{ "IR", REGISTRY_RIPENCC  },
	{ "IS", REGISTRY_RIPENCC  },
	{ "IT", REGISTRY_RIPENCC  },
	{ "JE", REGISTRY_RIPENCC  },
	{ "JM", REGISTRY_ARIN     },
	{ "JO", REGISTRY_RIPENCC  },
	{ "JP", REGISTRY_APNIC    },
	{ "KE", REGISTRY_AFRINIC  },
	{ "KG", REGISTRY_RIPENCC  },
	{ "KH", REGISTRY_APNIC    },
	{ "KI", REGISTRY_APNIC    },
	{ "KM", REGISTRY_AFRINIC  },
	{ "KN", REGISTRY_ARIN     },
	{ "KP", REGISTRY_APNIC    },
	{ "KR", REGISTRY_APNIC    },
	{ "KW", REGISTRY_RIPENCC  },
	{ "KY", REGISTRY_ARIN     },
	{ "KZ", REGISTRY_RIPENCC  },
	{ "LA", REGISTRY_APNIC    },
	{ "LB", REGISTRY_RIPENCC  },
	{ "LC", REGISTRY_ARIN     },
	{ "LI", REGISTRY_RIPENCC  },
	{ "LK", REGISTRY_APNIC    },
	{ "LR", REGISTRY_AFRINIC  },
	{ "LS", REGISTRY_AFRINIC  },
	{ "LT", REGISTRY_RIPENCC  },
	{ "LU", REGISTRY_RIPENCC  },
	{ "LV", REGISTRY_RIPENCC  },
	{ "LY", REGISTRY_AFRINIC  },
	{ "MA", REGISTRY_AFRINIC  },
	{ "MC", REGISTRY_RIPENCC  },
	{ "MD", REGISTRY_RIPENCC  },
	{ "ME", REGISTRY_RIPENCC  },
	{ "MF", REGISTRY_ARIN     },
	{ "MG", REGISTRY_AFRINIC  },
	{ "MH", REGISTRY_APNIC    },
	{ "MK", REGISTRY_RIPENCC  },
	{ "ML", REGISTRY_AFRINIC  },
	{ "MM", REGISTRY_APNIC    },
	{ "MN", REGISTRY_APNIC    },
	{ "MO", REGISTRY_APNIC    },
	{ "MP", REGISTRY_APNIC    },
	{ "MQ", REGISTRY_RIPENCC  },
	{ "MR", REGISTRY_AFRINIC  },
	{ "MS", REGISTRY_ARIN     },
	{ "MT", REGISTRY_RIPENCC  },
	{ "MU", REGISTRY_AFRINIC  },
	{ "MV", REGISTRY_APNIC    },
	{ "MW", REGISTRY_AFRINIC  },
	{ "MX", REGISTRY_LACNIC   },
	{ "MY", REGISTRY_APNIC    },
	{ "MZ", REGISTRY_AFRINIC  },
	{ "NA", REGISTRY_AFRINIC  },
	{ "NC", REGISTRY_APNIC    },
	{ "NE", REGISTRY_AFRINIC  },
	{ "NF", REGISTRY_APNIC    },
	{ "NG", REGISTRY_AFRINIC  },
	{ "NI", REGISTRY_LACNIC   },
	{ "NL", REGISTRY_RIPENCC  },
	{ "NO", REGISTRY_RIPENCC  },
	{ "NP", REGISTRY_APNIC    },
	{ "NR", REGISTRY_APNIC    },
	{ "NU", REGISTRY_APNIC    },
	{ "NZ", REGISTRY_APNIC    },
	{ "OM", REGISTRY_RIPENCC  },
	{ "PA", REGISTRY_LACNIC   },
	{ "PE", REGISTRY_LACNIC   },
	{ "PF", REGISTRY_APNIC    },
	{ "PG", REGISTRY_APNIC    },
	{ "PH", REGISTRY_APNIC    },
	{ "PK", REGISTRY_APNIC    },
	{ "PL", REGISTRY_RIPENCC  },
	{ "PM", REGISTRY_ARIN     },
	{ "PR", REGISTRY_ARIN     },
	{ "PS", REGISTRY_RIPENCC  },
	{ "PT", REGISTRY_RIPENCC  },
	{ "PW", REGISTRY_APNIC    },
	{ "PY", REGISTRY_LACNIC   },
	{ "QA", REGISTRY_RIPENCC  },
	{ "RE", REGISTRY_AFRINIC  },
	{ "RO", REGISTRY_RIPENCC  },
	{ "RS", REGISTRY_RIPENCC  },
	{ "RU", REGISTRY_RIPENCC  },
	{ "RW", REGISTRY_AFRINIC  },
	{ "SA", REGISTRY_RIPENCC  },
	{ "SB", REGISTRY_APNIC    },
	{ "SC", REGISTRY_AFRINIC  },
	{ "SD", REGISTRY_AFRINIC  },
	{ "SE", REGISTRY_RIPENCC  },
	{ "SG", REGISTRY_APNIC    },
	{ "SI", REGISTRY_RIPENCC  },
	{ "SK", REGISTRY_RIPENCC  },
	{ "SL", REGISTRY_AFRINIC  },
	{ "SM", REGISTRY_RIPENCC  },
	{ "SN", REGISTRY_AFRINIC  },
	{ "SO", REGISTRY_AFRINIC  },
	{ "SR", REGISTRY_LACNIC   },
	{ "SS", REGISTRY_AFRINIC  },
	{ "ST", REGISTRY_AFRINIC  },
	{ "SV", REGISTRY_LACNIC   },
	{ "SX", REGISTRY_LACNIC   },
	{ "SY", REGISTRY_RIPENCC  },
	{ "SZ", REGISTRY_AFRINIC  },
	{ "TC", REGISTRY_ARIN     },
	{ "TD", REGISTRY_AFRINIC  },
	{ "TG", REGISTRY_AFRINIC  },
	{ "TH", REGISTRY_APNIC    },
	{ "TJ", REGISTRY_RIPENCC  },
	{ "TK", REGISTRY_APNIC    },
	{ "TL", REGISTRY_APNIC    },
	{ "TM", REGISTRY_RIPENCC  },
	{ "TN", REGISTRY_AFRINIC  },
	{ "TO", REGISTRY_APNIC    },
	{ "TR", REGISTRY_RIPENCC  },
	{ "TT", REGISTRY_LACNIC   },
	{ "TV", REGISTRY_APNIC    },
	{ "TW", REGISTRY_APNIC    },
	{ "TZ", REGISTRY_AFRINIC  },
	{ "UA", REGISTRY_RIPENCC  },
	{ "UG", REGISTRY_AFRINIC  },
	{ "US", REGISTRY_ARIN     },
	{ "UY", REGISTRY_LACNIC   },
	{ "UZ", REGISTRY_RIPENCC  },
	{ "VA", REGISTRY_RIPENCC  },
	{ "VC", REGISTRY_ARIN     },
	{ "VE", REGISTRY_LACNIC   },
	{ "VG", REGISTRY_RIPENCC  },
	{ "VI", REGISTRY_ARIN     },
	{ "VN", REGISTRY_APNIC    },
	{ "VU", REGISTRY_APNIC    },
	{ "WF", REGISTRY_APNIC    },
	{ "WS", REGISTRY_APNIC    },
	{ "YE", REGISTRY_RIPENCC  },
	{ "YT", REGISTRY_AFRINIC  },
	{ "ZA", REGISTRY_AFRINIC  },
	{ "ZM", REGISTRY_AFRINIC  },
	{ "ZW", REGISTRY_AFRINIC  },
};

static const s_cc_index_reg_assignment cc_index_reg_assignment[] = {
	{ REGISTRY_UNKNOWN  }, //    0  A0
	{ REGISTRY_UNKNOWN  }, //    1  B0
	{ REGISTRY_UNKNOWN  }, //    2  C0
	{ REGISTRY_UNKNOWN  }, //    3  D0
	{ REGISTRY_UNKNOWN  }, //    4  E0
	{ REGISTRY_UNKNOWN  }, //    5  F0
	{ REGISTRY_UNKNOWN  }, //    6  G0
	{ REGISTRY_UNKNOWN  }, //    7  H0
	{ REGISTRY_UNKNOWN  }, //    8  I0
	{ REGISTRY_UNKNOWN  }, //    9  J0
	{ REGISTRY_UNKNOWN  }, //   10  K0
	{ REGISTRY_UNKNOWN  }, //   11  L0
	{ REGISTRY_UNKNOWN  }, //   12  M0
	{ REGISTRY_UNKNOWN  }, //   13  N0
	{ REGISTRY_UNKNOWN  }, //   14  O0
	{ REGISTRY_UNKNOWN  }, //   15  P0
	{ REGISTRY_UNKNOWN  }, //   16  Q0
	{ REGISTRY_UNKNOWN  }, //   17  R0
	{ REGISTRY_UNKNOWN  }, //   18  S0
	{ REGISTRY_UNKNOWN  }, //   19  T0
	{ REGISTRY_UNKNOWN  }, //   20  U0
	{ REGISTRY_UNKNOWN  }, //   21  V0
	{ REGISTRY_UNKNOWN  }, //   22  W0
	{ REGISTRY_UNKNOWN  }, //   23  X0
	{ REGISTRY_UNKNOWN  }, //   24  Y0
	{ REGISTRY_UNKNOWN  }, //   25  Z0
	{ REGISTRY_UNKNOWN  }, //   26  A1
	{ REGISTRY_UNKNOWN  }, //   27  B1
	{ REGISTRY_UNKNOWN  }, //   28  C1
	{ REGISTRY_UNKNOWN  }, //   29  D1
	{ REGISTRY_UNKNOWN  }, //   30  E1
	{ REGISTRY_UNKNOWN  }, //   31  F1
	{ REGISTRY_UNKNOWN  }, //   32  G1
	{ REGISTRY_UNKNOWN  }, //   33  H1
	{ REGISTRY_UNKNOWN  }, //   34  I1
	{ REGISTRY_UNKNOWN  }, //   35  J1
	{ REGISTRY_UNKNOWN  }, //   36  K1
	{ REGISTRY_UNKNOWN  }, //   37  L1
	{ REGISTRY_UNKNOWN  }, //   38  M1
	{ REGISTRY_UNKNOWN  }, //   39  N1
	{ REGISTRY_UNKNOWN  }, //   40  O1
	{ REGISTRY_UNKNOWN  }, //   41  P1
	{ REGISTRY_UNKNOWN  }, //   42  Q1
	{ REGISTRY_UNKNOWN  }, //   43  R1
	{ REGISTRY_UNKNOWN  }, //   44  S1
	{ REGISTRY_UNKNOWN  }, //   45  T1
	{ REGISTRY_UNKNOWN  }, //   46  U1
	{ REGISTRY_UNKNOWN  }, //   47  V1
	{ REGISTRY_UNKNOWN  }, //   48  W1
	{ REGISTRY_UNKNOWN  }, //   49  X1
	{ REGISTRY_UNKNOWN  }, //   50  Y1
	{ REGISTRY_UNKNOWN  }, //   51  Z1
	{ REGISTRY_UNKNOWN  }, //   52  A2
	{ REGISTRY_UNKNOWN  }, //   53  B2
	{ REGISTRY_UNKNOWN  }, //   54  C2
	{ REGISTRY_UNKNOWN  }, //   55  D2
	{ REGISTRY_UNKNOWN  }, //   56  E2
	{ REGISTRY_UNKNOWN  }, //   57  F2
	{ REGISTRY_UNKNOWN  }, //   58  G2
	{ REGISTRY_UNKNOWN  }, //   59  H2
	{ REGISTRY_UNKNOWN  }, //   60  I2
	{ REGISTRY_UNKNOWN  }, //   61  J2
	{ REGISTRY_UNKNOWN  }, //   62  K2
	{ REGISTRY_UNKNOWN  }, //   63  L2
	{ REGISTRY_UNKNOWN  }, //   64  M2
	{ REGISTRY_UNKNOWN  }, //   65  N2
	{ REGISTRY_UNKNOWN  }, //   66  O2
	{ REGISTRY_UNKNOWN  }, //   67  P2
	{ REGISTRY_UNKNOWN  }, //   68  Q2
	{ REGISTRY_UNKNOWN  }, //   69  R2
	{ REGISTRY_UNKNOWN  }, //   70  S2
	{ REGISTRY_UNKNOWN  }, //   71  T2
	{ REGISTRY_UNKNOWN  }, //   72  U2
	{ REGISTRY_UNKNOWN  }, //   73  V2
	{ REGISTRY_UNKNOWN  }, //   74  W2
	{ REGISTRY_UNKNOWN  }, //   75  X2
	{ REGISTRY_UNKNOWN  }, //   76  Y2
	{ REGISTRY_UNKNOWN  }, //   77  Z2
	{ REGISTRY_UNKNOWN  }, //   78  A3
	{ REGISTRY_UNKNOWN  }, //   79  B3
	{ REGISTRY_UNKNOWN  }, //   80  C3
	{ REGISTRY_UNKNOWN  }, //   81  D3
	{ REGISTRY_UNKNOWN  }, //   82  E3
	{ REGISTRY_UNKNOWN  }, //   83  F3
	{ REGISTRY_UNKNOWN  }, //   84  G3
	{ REGISTRY_UNKNOWN  }, //   85  H3
	{ REGISTRY_UNKNOWN  }, //   86  I3
	{ REGISTRY_UNKNOWN  }, //   87  J3
	{ REGISTRY_UNKNOWN  }, //   88  K3
	{ REGISTRY_UNKNOWN  }, //   89  L3
	{ REGISTRY_UNKNOWN  }, //   90  M3
	{ REGISTRY_UNKNOWN  }, //   91  N3
	{ REGISTRY_UNKNOWN  }, //   92  O3
	{ REGISTRY_UNKNOWN  }, //   93  P3
	{ REGISTRY_UNKNOWN  }, //   94  Q3
	{ REGISTRY_UNKNOWN  }, //   95  R3
	{ REGISTRY_UNKNOWN  }, //   96  S3
	{ REGISTRY_UNKNOWN  }, //   97  T3
	{ REGISTRY_UNKNOWN  }, //   98  U3
	{ REGISTRY_UNKNOWN  }, //   99  V3
	{ REGISTRY_UNKNOWN  }, //  100  W3
	{ REGISTRY_UNKNOWN  }, //  101  X3
	{ REGISTRY_UNKNOWN  }, //  102  Y3
	{ REGISTRY_UNKNOWN  }, //  103  Z3
	{ REGISTRY_UNKNOWN  }, //  104  A4
	{ REGISTRY_UNKNOWN  }, //  105  B4
	{ REGISTRY_UNKNOWN  }, //  106  C4
	{ REGISTRY_UNKNOWN  }, //  107  D4
	{ REGISTRY_UNKNOWN  }, //  108  E4
	{ REGISTRY_UNKNOWN  }, //  109  F4
	{ REGISTRY_UNKNOWN  }, //  110  G4
	{ REGISTRY_UNKNOWN  }, //  111  H4
	{ REGISTRY_UNKNOWN  }, //  112  I4
	{ REGISTRY_UNKNOWN  }, //  113  J4
	{ REGISTRY_UNKNOWN  }, //  114  K4
	{ REGISTRY_UNKNOWN  }, //  115  L4
	{ REGISTRY_UNKNOWN  }, //  116  M4
	{ REGISTRY_UNKNOWN  }, //  117  N4
	{ REGISTRY_UNKNOWN  }, //  118  O4
	{ REGISTRY_UNKNOWN  }, //  119  P4
	{ REGISTRY_UNKNOWN  }, //  120  Q4
	{ REGISTRY_UNKNOWN  }, //  121  R4
	{ REGISTRY_UNKNOWN  }, //  122  S4
	{ REGISTRY_UNKNOWN  }, //  123  T4
	{ REGISTRY_UNKNOWN  }, //  124  U4
	{ REGISTRY_UNKNOWN  }, //  125  V4
	{ REGISTRY_UNKNOWN  }, //  126  W4
	{ REGISTRY_UNKNOWN  }, //  127  X4
	{ REGISTRY_UNKNOWN  }, //  128  Y4
	{ REGISTRY_UNKNOWN  }, //  129  Z4
	{ REGISTRY_UNKNOWN  }, //  130  A5
	{ REGISTRY_UNKNOWN  }, //  131  B5
	{ REGISTRY_UNKNOWN  }, //  132  C5
	{ REGISTRY_UNKNOWN  }, //  133  D5
	{ REGISTRY_UNKNOWN  }, //  134  E5
	{ REGISTRY_UNKNOWN  }, //  135  F5
	{ REGISTRY_UNKNOWN  }, //  136  G5
	{ REGISTRY_UNKNOWN  }, //  137  H5
	{ REGISTRY_UNKNOWN  }, //  138  I5
	{ REGISTRY_UNKNOWN  }, //  139  J5
	{ REGISTRY_UNKNOWN  }, //  140  K5
	{ REGISTRY_UNKNOWN  }, //  141  L5
	{ REGISTRY_UNKNOWN  }, //  142  M5
	{ REGISTRY_UNKNOWN  }, //  143  N5
	{ REGISTRY_UNKNOWN  }, //  144  O5
	{ REGISTRY_UNKNOWN  }, //  145  P5
	{ REGISTRY_UNKNOWN  }, //  146  Q5
	{ REGISTRY_UNKNOWN  }, //  147  R5
	{ REGISTRY_UNKNOWN  }, //  148  S5
	{ REGISTRY_UNKNOWN  }, //  149  T5
	{ REGISTRY_UNKNOWN  }, //  150  U5
	{ REGISTRY_UNKNOWN  }, //  151  V5
	{ REGISTRY_UNKNOWN  }, //  152  W5
	{ REGISTRY_UNKNOWN  }, //  153  X5
	{ REGISTRY_UNKNOWN  }, //  154  Y5
	{ REGISTRY_UNKNOWN  }, //  155  Z5
	{ REGISTRY_UNKNOWN  }, //  156  A6
	{ REGISTRY_UNKNOWN  }, //  157  B6
	{ REGISTRY_UNKNOWN  }, //  158  C6
	{ REGISTRY_UNKNOWN  }, //  159  D6
	{ REGISTRY_UNKNOWN  }, //  160  E6
	{ REGISTRY_UNKNOWN  }, //  161  F6
	{ REGISTRY_UNKNOWN  }, //  162  G6
	{ REGISTRY_UNKNOWN  }, //  163  H6
	{ REGISTRY_UNKNOWN  }, //  164  I6
	{ REGISTRY_UNKNOWN  }, //  165  J6
	{ REGISTRY_UNKNOWN  }, //  166  K6
	{ REGISTRY_UNKNOWN  }, //  167  L6
	{ REGISTRY_UNKNOWN  }, //  168  M6
	{ REGISTRY_UNKNOWN  }, //  169  N6
	{ REGISTRY_UNKNOWN  }, //  170  O6
	{ REGISTRY_UNKNOWN  }, //  171  P6
	{ REGISTRY_UNKNOWN  }, //  172  Q6
	{ REGISTRY_UNKNOWN  }, //  173  R6
	{ REGISTRY_UNKNOWN  }, //  174  S6
	{ REGISTRY_UNKNOWN  }, //  175  T6
	{ REGISTRY_UNKNOWN  }, //  176  U6
	{ REGISTRY_UNKNOWN  }, //  177  V6
	{ REGISTRY_UNKNOWN  }, //  178  W6
	{ REGISTRY_UNKNOWN  }, //  179  X6
	{ REGISTRY_UNKNOWN  }, //  180  Y6
	{ REGISTRY_UNKNOWN  }, //  181  Z6
	{ REGISTRY_UNKNOWN  }, //  182  A7
	{ REGISTRY_UNKNOWN  }, //  183  B7
	{ REGISTRY_UNKNOWN  }, //  184  C7
	{ REGISTRY_UNKNOWN  }, //  185  D7
	{ REGISTRY_UNKNOWN  }, //  186  E7
	{ REGISTRY_UNKNOWN  }, //  187  F7
	{ REGISTRY_UNKNOWN  }, //  188  G7
	{ REGISTRY_UNKNOWN  }, //  189  H7
	{ REGISTRY_UNKNOWN  }, //  190  I7
	{ REGISTRY_UNKNOWN  }, //  191  J7
	{ REGISTRY_UNKNOWN  }, //  192  K7
	{ REGISTRY_UNKNOWN  }, //  193  L7
	{ REGISTRY_UNKNOWN  }, //  194  M7
	{ REGISTRY_UNKNOWN  }, //  195  N7
	{ REGISTRY_UNKNOWN  }, //  196  O7
	{ REGISTRY_UNKNOWN  }, //  197  P7
	{ REGISTRY_UNKNOWN  }, //  198  Q7
	{ REGISTRY_UNKNOWN  }, //  199  R7
	{ REGISTRY_UNKNOWN  }, //  200  S7
	{ REGISTRY_UNKNOWN  }, //  201  T7
	{ REGISTRY_UNKNOWN  }, //  202  U7
	{ REGISTRY_UNKNOWN  }, //  203  V7
	{ REGISTRY_UNKNOWN  }, //  204  W7
	{ REGISTRY_UNKNOWN  }, //  205  X7
	{ REGISTRY_UNKNOWN  }, //  206  Y7
	{ REGISTRY_UNKNOWN  }, //  207  Z7
	{ REGISTRY_UNKNOWN  }, //  208  A8
	{ REGISTRY_UNKNOWN  }, //  209  B8
	{ REGISTRY_UNKNOWN  }, //  210  C8
	{ REGISTRY_UNKNOWN  }, //  211  D8
	{ REGISTRY_UNKNOWN  }, //  212  E8
	{ REGISTRY_UNKNOWN  }, //  213  F8
	{ REGISTRY_UNKNOWN  }, //  214  G8
	{ REGISTRY_UNKNOWN  }, //  215  H8
	{ REGISTRY_UNKNOWN  }, //  216  I8
	{ REGISTRY_UNKNOWN  }, //  217  J8
	{ REGISTRY_UNKNOWN  }, //  218  K8
	{ REGISTRY_UNKNOWN  }, //  219  L8
	{ REGISTRY_UNKNOWN  }, //  220  M8
	{ REGISTRY_UNKNOWN  }, //  221  N8
	{ REGISTRY_UNKNOWN  }, //  222  O8
	{ REGISTRY_UNKNOWN  }, //  223  P8
	{ REGISTRY_UNKNOWN  }, //  224  Q8
	{ REGISTRY_UNKNOWN  }, //  225  R8
	{ REGISTRY_UNKNOWN  }, //  226  S8
	{ REGISTRY_UNKNOWN  }, //  227  T8
	{ REGISTRY_UNKNOWN  }, //  228  U8
	{ REGISTRY_UNKNOWN  }, //  229  V8
	{ REGISTRY_UNKNOWN  }, //  230  W8
	{ REGISTRY_UNKNOWN  }, //  231  X8
	{ REGISTRY_UNKNOWN  }, //  232  Y8
	{ REGISTRY_UNKNOWN  }, //  233  Z8
	{ REGISTRY_UNKNOWN  }, //  234  A9
	{ REGISTRY_UNKNOWN  }, //  235  B9
	{ REGISTRY_UNKNOWN  }, //  236  C9
	{ REGISTRY_UNKNOWN  }, //  237  D9
	{ REGISTRY_UNKNOWN  }, //  238  E9
	{ REGISTRY_UNKNOWN  }, //  239  F9
	{ REGISTRY_UNKNOWN  }, //  240  G9
	{ REGISTRY_UNKNOWN  }, //  241  H9
	{ REGISTRY_UNKNOWN  }, //  242  I9
	{ REGISTRY_UNKNOWN  }, //  243  J9
	{ REGISTRY_UNKNOWN  }, //  244  K9
	{ REGISTRY_UNKNOWN  }, //  245  L9
	{ REGISTRY_UNKNOWN  }, //  246  M9
	{ REGISTRY_UNKNOWN  }, //  247  N9
	{ REGISTRY_UNKNOWN  }, //  248  O9
	{ REGISTRY_UNKNOWN  }, //  249  P9
	{ REGISTRY_UNKNOWN  }, //  250  Q9
	{ REGISTRY_UNKNOWN  }, //  251  R9
	{ REGISTRY_UNKNOWN  }, //  252  S9
	{ REGISTRY_UNKNOWN  }, //  253  T9
	{ REGISTRY_UNKNOWN  }, //  254  U9
	{ REGISTRY_UNKNOWN  }, //  255  V9
	{ REGISTRY_UNKNOWN  }, //  256  W9
	{ REGISTRY_UNKNOWN  }, //  257  X9
	{ REGISTRY_UNKNOWN  }, //  258  Y9
	{ REGISTRY_UNKNOWN  }, //  259  Z9
	{ REGISTRY_UNKNOWN  }, //  260  AA
	{ REGISTRY_RIPENCC  }, //  261  BA
	{ REGISTRY_ARIN     }, //  262  CA
	{ REGISTRY_UNKNOWN  }, //  263  DA
	{ REGISTRY_UNKNOWN  }, //  264  EA
	{ REGISTRY_UNKNOWN  }, //  265  FA
	{ REGISTRY_AFRINIC  }, //  266  GA
	{ REGISTRY_UNKNOWN  }, //  267  HA
	{ REGISTRY_UNKNOWN  }, //  268  IA
	{ REGISTRY_UNKNOWN  }, //  269  JA
	{ REGISTRY_UNKNOWN  }, //  270  KA
	{ REGISTRY_APNIC    }, //  271  LA
	{ REGISTRY_AFRINIC  }, //  272  MA
	{ REGISTRY_AFRINIC  }, //  273  NA
	{ REGISTRY_UNKNOWN  }, //  274  OA
	{ REGISTRY_LACNIC   }, //  275  PA
	{ REGISTRY_RIPENCC  }, //  276  QA
	{ REGISTRY_UNKNOWN  }, //  277  RA
	{ REGISTRY_RIPENCC  }, //  278  SA
	{ REGISTRY_UNKNOWN  }, //  279  TA
	{ REGISTRY_RIPENCC  }, //  280  UA
	{ REGISTRY_RIPENCC  }, //  281  VA
	{ REGISTRY_UNKNOWN  }, //  282  WA
	{ REGISTRY_UNKNOWN  }, //  283  XA
	{ REGISTRY_UNKNOWN  }, //  284  YA
	{ REGISTRY_AFRINIC  }, //  285  ZA
	{ REGISTRY_UNKNOWN  }, //  286  AB
	{ REGISTRY_ARIN     }, //  287  BB
	{ REGISTRY_UNKNOWN  }, //  288  CB
	{ REGISTRY_UNKNOWN  }, //  289  DB
	{ REGISTRY_UNKNOWN  }, //  290  EB
	{ REGISTRY_UNKNOWN  }, //  291  FB
	{ REGISTRY_RIPENCC  }, //  292  GB
	{ REGISTRY_UNKNOWN  }, //  293  HB
	{ REGISTRY_UNKNOWN  }, //  294  IB
	{ REGISTRY_UNKNOWN  }, //  295  JB
	{ REGISTRY_UNKNOWN  }, //  296  KB
	{ REGISTRY_RIPENCC  }, //  297  LB
	{ REGISTRY_UNKNOWN  }, //  298  MB
	{ REGISTRY_UNKNOWN  }, //  299  NB
	{ REGISTRY_UNKNOWN  }, //  300  OB
	{ REGISTRY_UNKNOWN  }, //  301  PB
	{ REGISTRY_UNKNOWN  }, //  302  QB
	{ REGISTRY_UNKNOWN  }, //  303  RB
	{ REGISTRY_APNIC    }, //  304  SB
	{ REGISTRY_UNKNOWN  }, //  305  TB
	{ REGISTRY_UNKNOWN  }, //  306  UB
	{ REGISTRY_UNKNOWN  }, //  307  VB
	{ REGISTRY_UNKNOWN  }, //  308  WB
	{ REGISTRY_UNKNOWN  }, //  309  XB
	{ REGISTRY_UNKNOWN  }, //  310  YB
	{ REGISTRY_UNKNOWN  }, //  311  ZB
	{ REGISTRY_UNKNOWN  }, //  312  AC
	{ REGISTRY_UNKNOWN  }, //  313  BC
	{ REGISTRY_UNKNOWN  }, //  314  CC
	{ REGISTRY_UNKNOWN  }, //  315  DC
	{ REGISTRY_LACNIC   }, //  316  EC
	{ REGISTRY_UNKNOWN  }, //  317  FC
	{ REGISTRY_UNKNOWN  }, //  318  GC
	{ REGISTRY_UNKNOWN  }, //  319  HC
	{ REGISTRY_UNKNOWN  }, //  320  IC
	{ REGISTRY_UNKNOWN  }, //  321  JC
	{ REGISTRY_UNKNOWN  }, //  322  KC
	{ REGISTRY_ARIN     }, //  323  LC
	{ REGISTRY_RIPENCC  }, //  324  MC
	{ REGISTRY_APNIC    }, //  325  NC
	{ REGISTRY_UNKNOWN  }, //  326  OC
	{ REGISTRY_UNKNOWN  }, //  327  PC
	{ REGISTRY_UNKNOWN  }, //  328  QC
	{ REGISTRY_UNKNOWN  }, //  329  RC
	{ REGISTRY_AFRINIC  }, //  330  SC
	{ REGISTRY_ARIN     }, //  331  TC
	{ REGISTRY_UNKNOWN  }, //  332  UC
	{ REGISTRY_ARIN     }, //  333  VC
	{ REGISTRY_UNKNOWN  }, //  334  WC
	{ REGISTRY_UNKNOWN  }, //  335  XC
	{ REGISTRY_UNKNOWN  }, //  336  YC
	{ REGISTRY_UNKNOWN  }, //  337  ZC
	{ REGISTRY_RIPENCC  }, //  338  AD
	{ REGISTRY_APNIC    }, //  339  BD
	{ REGISTRY_AFRINIC  }, //  340  CD
	{ REGISTRY_UNKNOWN  }, //  341  DD
	{ REGISTRY_UNKNOWN  }, //  342  ED
	{ REGISTRY_UNKNOWN  }, //  343  FD
	{ REGISTRY_ARIN     }, //  344  GD
	{ REGISTRY_UNKNOWN  }, //  345  HD
	{ REGISTRY_APNIC    }, //  346  ID
	{ REGISTRY_UNKNOWN  }, //  347  JD
	{ REGISTRY_UNKNOWN  }, //  348  KD
	{ REGISTRY_UNKNOWN  }, //  349  LD
	{ REGISTRY_RIPENCC  }, //  350  MD
	{ REGISTRY_UNKNOWN  }, //  351  ND
	{ REGISTRY_UNKNOWN  }, //  352  OD
	{ REGISTRY_UNKNOWN  }, //  353  PD
	{ REGISTRY_UNKNOWN  }, //  354  QD
	{ REGISTRY_UNKNOWN  }, //  355  RD
	{ REGISTRY_AFRINIC  }, //  356  SD
	{ REGISTRY_AFRINIC  }, //  357  TD
	{ REGISTRY_UNKNOWN  }, //  358  UD
	{ REGISTRY_UNKNOWN  }, //  359  VD
	{ REGISTRY_UNKNOWN  }, //  360  WD
	{ REGISTRY_UNKNOWN  }, //  361  XD
	{ REGISTRY_UNKNOWN  }, //  362  YD
	{ REGISTRY_UNKNOWN  }, //  363  ZD
	{ REGISTRY_RIPENCC  }, //  364  AE
	{ REGISTRY_RIPENCC  }, //  365  BE
	{ REGISTRY_UNKNOWN  }, //  366  CE
	{ REGISTRY_RIPENCC  }, //  367  DE
	{ REGISTRY_RIPENCC  }, //  368  EE
	{ REGISTRY_UNKNOWN  }, //  369  FE
	{ REGISTRY_RIPENCC  }, //  370  GE
	{ REGISTRY_UNKNOWN  }, //  371  HE
	{ REGISTRY_RIPENCC  }, //  372  IE
	{ REGISTRY_RIPENCC  }, //  373  JE
	{ REGISTRY_AFRINIC  }, //  374  KE
	{ REGISTRY_UNKNOWN  }, //  375  LE
	{ REGISTRY_RIPENCC  }, //  376  ME
	{ REGISTRY_AFRINIC  }, //  377  NE
	{ REGISTRY_UNKNOWN  }, //  378  OE
	{ REGISTRY_LACNIC   }, //  379  PE
	{ REGISTRY_UNKNOWN  }, //  380  QE
	{ REGISTRY_AFRINIC  }, //  381  RE
	{ REGISTRY_RIPENCC  }, //  382  SE
	{ REGISTRY_UNKNOWN  }, //  383  TE
	{ REGISTRY_UNKNOWN  }, //  384  UE
	{ REGISTRY_LACNIC   }, //  385  VE
	{ REGISTRY_UNKNOWN  }, //  386  WE
	{ REGISTRY_UNKNOWN  }, //  387  XE
	{ REGISTRY_RIPENCC  }, //  388  YE
	{ REGISTRY_UNKNOWN  }, //  389  ZE
	{ REGISTRY_APNIC    }, //  390  AF
	{ REGISTRY_AFRINIC  }, //  391  BF
	{ REGISTRY_AFRINIC  }, //  392  CF
	{ REGISTRY_UNKNOWN  }, //  393  DF
	{ REGISTRY_UNKNOWN  }, //  394  EF
	{ REGISTRY_UNKNOWN  }, //  395  FF
	{ REGISTRY_LACNIC   }, //  396  GF
	{ REGISTRY_UNKNOWN  }, //  397  HF
	{ REGISTRY_UNKNOWN  }, //  398  IF
	{ REGISTRY_UNKNOWN  }, //  399  JF
	{ REGISTRY_UNKNOWN  }, //  400  KF
	{ REGISTRY_UNKNOWN  }, //  401  LF
	{ REGISTRY_ARIN     }, //  402  MF
	{ REGISTRY_APNIC    }, //  403  NF
	{ REGISTRY_UNKNOWN  }, //  404  OF
	{ REGISTRY_APNIC    }, //  405  PF
	{ REGISTRY_UNKNOWN  }, //  406  QF
	{ REGISTRY_UNKNOWN  }, //  407  RF
	{ REGISTRY_UNKNOWN  }, //  408  SF
	{ REGISTRY_UNKNOWN  }, //  409  TF
	{ REGISTRY_UNKNOWN  }, //  410  UF
	{ REGISTRY_UNKNOWN  }, //  411  VF
	{ REGISTRY_APNIC    }, //  412  WF
	{ REGISTRY_UNKNOWN  }, //  413  XF
	{ REGISTRY_UNKNOWN  }, //  414  YF
	{ REGISTRY_UNKNOWN  }, //  415  ZF
	{ REGISTRY_ARIN     }, //  416  AG
	{ REGISTRY_RIPENCC  }, //  417  BG
	{ REGISTRY_AFRINIC  }, //  418  CG
	{ REGISTRY_UNKNOWN  }, //  419  DG
	{ REGISTRY_AFRINIC  }, //  420  EG
	{ REGISTRY_UNKNOWN  }, //  421  FG
	{ REGISTRY_RIPENCC  }, //  422  GG
	{ REGISTRY_UNKNOWN  }, //  423  HG
	{ REGISTRY_UNKNOWN  }, //  424  IG
	{ REGISTRY_UNKNOWN  }, //  425  JG
	{ REGISTRY_RIPENCC  }, //  426  KG
	{ REGISTRY_UNKNOWN  }, //  427  LG
	{ REGISTRY_AFRINIC  }, //  428  MG
	{ REGISTRY_AFRINIC  }, //  429  NG
	{ REGISTRY_UNKNOWN  }, //  430  OG
	{ REGISTRY_APNIC    }, //  431  PG
	{ REGISTRY_UNKNOWN  }, //  432  QG
	{ REGISTRY_UNKNOWN  }, //  433  RG
	{ REGISTRY_APNIC    }, //  434  SG
	{ REGISTRY_AFRINIC  }, //  435  TG
	{ REGISTRY_AFRINIC  }, //  436  UG
	{ REGISTRY_RIPENCC  }, //  437  VG
	{ REGISTRY_UNKNOWN  }, //  438  WG
	{ REGISTRY_UNKNOWN  }, //  439  XG
	{ REGISTRY_UNKNOWN  }, //  440  YG
	{ REGISTRY_UNKNOWN  }, //  441  ZG
	{ REGISTRY_UNKNOWN  }, //  442  AH
	{ REGISTRY_RIPENCC  }, //  443  BH
	{ REGISTRY_RIPENCC  }, //  444  CH
	{ REGISTRY_UNKNOWN  }, //  445  DH
	{ REGISTRY_UNKNOWN  }, //  446  EH
	{ REGISTRY_UNKNOWN  }, //  447  FH
	{ REGISTRY_AFRINIC  }, //  448  GH
	{ REGISTRY_UNKNOWN  }, //  449  HH
	{ REGISTRY_UNKNOWN  }, //  450  IH
	{ REGISTRY_UNKNOWN  }, //  451  JH
	{ REGISTRY_APNIC    }, //  452  KH
	{ REGISTRY_UNKNOWN  }, //  453  LH
	{ REGISTRY_APNIC    }, //  454  MH
	{ REGISTRY_UNKNOWN  }, //  455  NH
	{ REGISTRY_UNKNOWN  }, //  456  OH
	{ REGISTRY_APNIC    }, //  457  PH
	{ REGISTRY_UNKNOWN  }, //  458  QH
	{ REGISTRY_UNKNOWN  }, //  459  RH
	{ REGISTRY_UNKNOWN  }, //  460  SH
	{ REGISTRY_APNIC    }, //  461  TH
	{ REGISTRY_UNKNOWN  }, //  462  UH
	{ REGISTRY_UNKNOWN  }, //  463  VH
	{ REGISTRY_UNKNOWN  }, //  464  WH
	{ REGISTRY_UNKNOWN  }, //  465  XH
	{ REGISTRY_UNKNOWN  }, //  466  YH
	{ REGISTRY_UNKNOWN  }, //  467  ZH
	{ REGISTRY_ARIN     }, //  468  AI
	{ REGISTRY_AFRINIC  }, //  469  BI
	{ REGISTRY_AFRINIC  }, //  470  CI
	{ REGISTRY_UNKNOWN  }, //  471  DI
	{ REGISTRY_UNKNOWN  }, //  472  EI
	{ REGISTRY_RIPENCC  }, //  473  FI
	{ REGISTRY_RIPENCC  }, //  474  GI
	{ REGISTRY_UNKNOWN  }, //  475  HI
	{ REGISTRY_UNKNOWN  }, //  476  II
	{ REGISTRY_UNKNOWN  }, //  477  JI
	{ REGISTRY_APNIC    }, //  478  KI
	{ REGISTRY_RIPENCC  }, //  479  LI
	{ REGISTRY_UNKNOWN  }, //  480  MI
	{ REGISTRY_LACNIC   }, //  481  NI
	{ REGISTRY_UNKNOWN  }, //  482  OI
	{ REGISTRY_UNKNOWN  }, //  483  PI
	{ REGISTRY_UNKNOWN  }, //  484  QI
	{ REGISTRY_UNKNOWN  }, //  485  RI
	{ REGISTRY_RIPENCC  }, //  486  SI
	{ REGISTRY_UNKNOWN  }, //  487  TI
	{ REGISTRY_UNKNOWN  }, //  488  UI
	{ REGISTRY_ARIN     }, //  489  VI
	{ REGISTRY_UNKNOWN  }, //  490  WI
	{ REGISTRY_UNKNOWN  }, //  491  XI
	{ REGISTRY_UNKNOWN  }, //  492  YI
	{ REGISTRY_UNKNOWN  }, //  493  ZI
	{ REGISTRY_UNKNOWN  }, //  494  AJ
	{ REGISTRY_AFRINIC  }, //  495  BJ
	{ REGISTRY_UNKNOWN  }, //  496  CJ
	{ REGISTRY_AFRINIC  }, //  497  DJ
	{ REGISTRY_UNKNOWN  }, //  498  EJ
	{ REGISTRY_APNIC    }, //  499  FJ
	{ REGISTRY_UNKNOWN  }, //  500  GJ
	{ REGISTRY_UNKNOWN  }, //  501  HJ
	{ REGISTRY_UNKNOWN  }, //  502  IJ
	{ REGISTRY_UNKNOWN  }, //  503  JJ
	{ REGISTRY_UNKNOWN  }, //  504  KJ
	{ REGISTRY_UNKNOWN  }, //  505  LJ
	{ REGISTRY_UNKNOWN  }, //  506  MJ
	{ REGISTRY_UNKNOWN  }, //  507  NJ
	{ REGISTRY_UNKNOWN  }, //  508  OJ
	{ REGISTRY_UNKNOWN  }, //  509  PJ
	{ REGISTRY_UNKNOWN  }, //  510  QJ
	{ REGISTRY_UNKNOWN  }, //  511  RJ
	{ REGISTRY_UNKNOWN  }, //  512  SJ
	{ REGISTRY_RIPENCC  }, //  513  TJ
	{ REGISTRY_UNKNOWN  }, //  514  UJ
	{ REGISTRY_UNKNOWN  }, //  515  VJ
	{ REGISTRY_UNKNOWN  }, //  516  WJ
	{ REGISTRY_UNKNOWN  }, //  517  XJ
	{ REGISTRY_UNKNOWN  }, //  518  YJ
	{ REGISTRY_UNKNOWN  }, //  519  ZJ
	{ REGISTRY_UNKNOWN  }, //  520  AK
	{ REGISTRY_UNKNOWN  }, //  521  BK
	{ REGISTRY_APNIC    }, //  522  CK
	{ REGISTRY_RIPENCC  }, //  523  DK
	{ REGISTRY_UNKNOWN  }, //  524  EK
	{ REGISTRY_UNKNOWN  }, //  525  FK
	{ REGISTRY_UNKNOWN  }, //  526  GK
	{ REGISTRY_APNIC    }, //  527  HK
	{ REGISTRY_UNKNOWN  }, //  528  IK
	{ REGISTRY_UNKNOWN  }, //  529  JK
	{ REGISTRY_UNKNOWN  }, //  530  KK
	{ REGISTRY_APNIC    }, //  531  LK
	{ REGISTRY_RIPENCC  }, //  532  MK
	{ REGISTRY_UNKNOWN  }, //  533  NK
	{ REGISTRY_UNKNOWN  }, //  534  OK
	{ REGISTRY_APNIC    }, //  535  PK
	{ REGISTRY_UNKNOWN  }, //  536  QK
	{ REGISTRY_UNKNOWN  }, //  537  RK
	{ REGISTRY_RIPENCC  }, //  538  SK
	{ REGISTRY_APNIC    }, //  539  TK
	{ REGISTRY_UNKNOWN  }, //  540  UK
	{ REGISTRY_UNKNOWN  }, //  541  VK
	{ REGISTRY_UNKNOWN  }, //  542  WK
	{ REGISTRY_UNKNOWN  }, //  543  XK
	{ REGISTRY_UNKNOWN  }, //  544  YK
	{ REGISTRY_UNKNOWN  }, //  545  ZK
	{ REGISTRY_RIPENCC  }, //  546  AL
	{ REGISTRY_ARIN     }, //  547  BL
	{ REGISTRY_LACNIC   }, //  548  CL
	{ REGISTRY_UNKNOWN  }, //  549  DL
	{ REGISTRY_UNKNOWN  }, //  550  EL
	{ REGISTRY_UNKNOWN  }, //  551  FL
	{ REGISTRY_RIPENCC  }, //  552  GL
	{ REGISTRY_UNKNOWN  }, //  553  HL
	{ REGISTRY_RIPENCC  }, //  554  IL
	{ REGISTRY_UNKNOWN  }, //  555  JL
	{ REGISTRY_UNKNOWN  }, //  556  KL
	{ REGISTRY_UNKNOWN  }, //  557  LL
	{ REGISTRY_AFRINIC  }, //  558  ML
	{ REGISTRY_RIPENCC  }, //  559  NL
	{ REGISTRY_UNKNOWN  }, //  560  OL
	{ REGISTRY_RIPENCC  }, //  561  PL
	{ REGISTRY_UNKNOWN  }, //  562  QL
	{ REGISTRY_UNKNOWN  }, //  563  RL
	{ REGISTRY_AFRINIC  }, //  564  SL
	{ REGISTRY_APNIC    }, //  565  TL
	{ REGISTRY_UNKNOWN  }, //  566  UL
	{ REGISTRY_UNKNOWN  }, //  567  VL
	{ REGISTRY_UNKNOWN  }, //  568  WL
	{ REGISTRY_UNKNOWN  }, //  569  XL
	{ REGISTRY_UNKNOWN  }, //  570  YL
	{ REGISTRY_UNKNOWN  }, //  571  ZL
	{ REGISTRY_RIPENCC  }, //  572  AM
	{ REGISTRY_ARIN     }, //  573  BM
	{ REGISTRY_AFRINIC  }, //  574  CM
	{ REGISTRY_ARIN     }, //  575  DM
	{ REGISTRY_UNKNOWN  }, //  576  EM
	{ REGISTRY_APNIC    }, //  577  FM
	{ REGISTRY_AFRINIC  }, //  578  GM
	{ REGISTRY_UNKNOWN  }, //  579  HM
	{ REGISTRY_RIPENCC  }, //  580  IM
	{ REGISTRY_ARIN     }, //  581  JM
	{ REGISTRY_AFRINIC  }, //  582  KM
	{ REGISTRY_UNKNOWN  }, //  583  LM
	{ REGISTRY_APNIC    }, //  584  MM
	{ REGISTRY_UNKNOWN  }, //  585  NM
	{ REGISTRY_RIPENCC  }, //  586  OM
	{ REGISTRY_ARIN     }, //  587  PM
	{ REGISTRY_UNKNOWN  }, //  588  QM
	{ REGISTRY_UNKNOWN  }, //  589  RM
	{ REGISTRY_RIPENCC  }, //  590  SM
	{ REGISTRY_RIPENCC  }, //  591  TM
	{ REGISTRY_UNKNOWN  }, //  592  UM
	{ REGISTRY_UNKNOWN  }, //  593  VM
	{ REGISTRY_UNKNOWN  }, //  594  WM
	{ REGISTRY_UNKNOWN  }, //  595  XM
	{ REGISTRY_UNKNOWN  }, //  596  YM
	{ REGISTRY_AFRINIC  }, //  597  ZM
	{ REGISTRY_UNKNOWN  }, //  598  AN
	{ REGISTRY_APNIC    }, //  599  BN
	{ REGISTRY_APNIC    }, //  600  CN
	{ REGISTRY_UNKNOWN  }, //  601  DN
	{ REGISTRY_UNKNOWN  }, //  602  EN
	{ REGISTRY_UNKNOWN  }, //  603  FN
	{ REGISTRY_AFRINIC  }, //  604  GN
	{ REGISTRY_LACNIC   }, //  605  HN
	{ REGISTRY_APNIC    }, //  606  IN
	{ REGISTRY_UNKNOWN  }, //  607  JN
	{ REGISTRY_ARIN     }, //  608  KN
	{ REGISTRY_UNKNOWN  }, //  609  LN
	{ REGISTRY_APNIC    }, //  610  MN
	{ REGISTRY_UNKNOWN  }, //  611  NN
	{ REGISTRY_UNKNOWN  }, //  612  ON
	{ REGISTRY_UNKNOWN  }, //  613  PN
	{ REGISTRY_UNKNOWN  }, //  614  QN
	{ REGISTRY_UNKNOWN  }, //  615  RN
	{ REGISTRY_AFRINIC  }, //  616  SN
	{ REGISTRY_AFRINIC  }, //  617  TN
	{ REGISTRY_UNKNOWN  }, //  618  UN
	{ REGISTRY_APNIC    }, //  619  VN
	{ REGISTRY_UNKNOWN  }, //  620  WN
	{ REGISTRY_UNKNOWN  }, //  621  XN
	{ REGISTRY_UNKNOWN  }, //  622  YN
	{ REGISTRY_UNKNOWN  }, //  623  ZN
	{ REGISTRY_AFRINIC  }, //  624  AO
	{ REGISTRY_LACNIC   }, //  625  BO
	{ REGISTRY_LACNIC   }, //  626  CO
	{ REGISTRY_LACNIC   }, //  627  DO
	{ REGISTRY_UNKNOWN  }, //  628  EO
	{ REGISTRY_RIPENCC  }, //  629  FO
	{ REGISTRY_UNKNOWN  }, //  630  GO
	{ REGISTRY_UNKNOWN  }, //  631  HO
	{ REGISTRY_APNIC    }, //  632  IO
	{ REGISTRY_RIPENCC  }, //  633  JO
	{ REGISTRY_UNKNOWN  }, //  634  KO
	{ REGISTRY_UNKNOWN  }, //  635  LO
	{ REGISTRY_APNIC    }, //  636  MO
	{ REGISTRY_RIPENCC  }, //  637  NO
	{ REGISTRY_UNKNOWN  }, //  638  OO
	{ REGISTRY_UNKNOWN  }, //  639  PO
	{ REGISTRY_UNKNOWN  }, //  640  QO
	{ REGISTRY_RIPENCC  }, //  641  RO
	{ REGISTRY_AFRINIC  }, //  642  SO
	{ REGISTRY_APNIC    }, //  643  TO
	{ REGISTRY_UNKNOWN  }, //  644  UO
	{ REGISTRY_UNKNOWN  }, //  645  VO
	{ REGISTRY_UNKNOWN  }, //  646  WO
	{ REGISTRY_UNKNOWN  }, //  647  XO
	{ REGISTRY_UNKNOWN  }, //  648  YO
	{ REGISTRY_UNKNOWN  }, //  649  ZO
	{ REGISTRY_APNIC    }, //  650  AP
	{ REGISTRY_UNKNOWN  }, //  651  BP
	{ REGISTRY_UNKNOWN  }, //  652  CP
	{ REGISTRY_UNKNOWN  }, //  653  DP
	{ REGISTRY_UNKNOWN  }, //  654  EP
	{ REGISTRY_UNKNOWN  }, //  655  FP
	{ REGISTRY_ARIN     }, //  656  GP
	{ REGISTRY_UNKNOWN  }, //  657  HP
	{ REGISTRY_UNKNOWN  }, //  658  IP
	{ REGISTRY_APNIC    }, //  659  JP
	{ REGISTRY_APNIC    }, //  660  KP
	{ REGISTRY_UNKNOWN  }, //  661  LP
	{ REGISTRY_APNIC    }, //  662  MP
	{ REGISTRY_APNIC    }, //  663  NP
	{ REGISTRY_UNKNOWN  }, //  664  OP
	{ REGISTRY_UNKNOWN  }, //  665  PP
	{ REGISTRY_UNKNOWN  }, //  666  QP
	{ REGISTRY_UNKNOWN  }, //  667  RP
	{ REGISTRY_UNKNOWN  }, //  668  SP
	{ REGISTRY_UNKNOWN  }, //  669  TP
	{ REGISTRY_UNKNOWN  }, //  670  UP
	{ REGISTRY_UNKNOWN  }, //  671  VP
	{ REGISTRY_UNKNOWN  }, //  672  WP
	{ REGISTRY_UNKNOWN  }, //  673  XP
	{ REGISTRY_UNKNOWN  }, //  674  YP
	{ REGISTRY_UNKNOWN  }, //  675  ZP
	{ REGISTRY_UNKNOWN  }, //  676  AQ
	{ REGISTRY_LACNIC   }, //  677  BQ
	{ REGISTRY_UNKNOWN  }, //  678  CQ
	{ REGISTRY_UNKNOWN  }, //  679  DQ
	{ REGISTRY_UNKNOWN  }, //  680  EQ
	{ REGISTRY_UNKNOWN  }, //  681  FQ
	{ REGISTRY_AFRINIC  }, //  682  GQ
	{ REGISTRY_UNKNOWN  }, //  683  HQ
	{ REGISTRY_RIPENCC  }, //  684  IQ
	{ REGISTRY_UNKNOWN  }, //  685  JQ
	{ REGISTRY_UNKNOWN  }, //  686  KQ
	{ REGISTRY_UNKNOWN  }, //  687  LQ
	{ REGISTRY_RIPENCC  }, //  688  MQ
	{ REGISTRY_UNKNOWN  }, //  689  NQ
	{ REGISTRY_UNKNOWN  }, //  690  OQ
	{ REGISTRY_UNKNOWN  }, //  691  PQ
	{ REGISTRY_UNKNOWN  }, //  692  QQ
	{ REGISTRY_UNKNOWN  }, //  693  RQ
	{ REGISTRY_UNKNOWN  }, //  694  SQ
	{ REGISTRY_UNKNOWN  }, //  695  TQ
	{ REGISTRY_UNKNOWN  }, //  696  UQ
	{ REGISTRY_UNKNOWN  }, //  697  VQ
	{ REGISTRY_UNKNOWN  }, //  698  WQ
	{ REGISTRY_UNKNOWN  }, //  699  XQ
	{ REGISTRY_UNKNOWN  }, //  700  YQ
	{ REGISTRY_UNKNOWN  }, //  701  ZQ
	{ REGISTRY_LACNIC   }, //  702  AR
	{ REGISTRY_LACNIC   }, //  703  BR
	{ REGISTRY_LACNIC   }, //  704  CR
	{ REGISTRY_UNKNOWN  }, //  705  DR
	{ REGISTRY_AFRINIC  }, //  706  ER
	{ REGISTRY_RIPENCC  }, //  707  FR
	{ REGISTRY_RIPENCC  }, //  708  GR
	{ REGISTRY_RIPENCC  }, //  709  HR
	{ REGISTRY_RIPENCC  }, //  710  IR
	{ REGISTRY_UNKNOWN  }, //  711  JR
	{ REGISTRY_APNIC    }, //  712  KR
	{ REGISTRY_AFRINIC  }, //  713  LR
	{ REGISTRY_AFRINIC  }, //  714  MR
	{ REGISTRY_APNIC    }, //  715  NR
	{ REGISTRY_UNKNOWN  }, //  716  OR
	{ REGISTRY_ARIN     }, //  717  PR
	{ REGISTRY_UNKNOWN  }, //  718  QR
	{ REGISTRY_UNKNOWN  }, //  719  RR
	{ REGISTRY_LACNIC   }, //  720  SR
	{ REGISTRY_RIPENCC  }, //  721  TR
	{ REGISTRY_UNKNOWN  }, //  722  UR
	{ REGISTRY_UNKNOWN  }, //  723  VR
	{ REGISTRY_UNKNOWN  }, //  724  WR
	{ REGISTRY_UNKNOWN  }, //  725  XR
	{ REGISTRY_UNKNOWN  }, //  726  YR
	{ REGISTRY_UNKNOWN  }, //  727  ZR
	{ REGISTRY_APNIC    }, //  728  AS
	{ REGISTRY_ARIN     }, //  729  BS
	{ REGISTRY_UNKNOWN  }, //  730  CS
	{ REGISTRY_UNKNOWN  }, //  731  DS
	{ REGISTRY_RIPENCC  }, //  732  ES
	{ REGISTRY_UNKNOWN  }, //  733  FS
	{ REGISTRY_UNKNOWN  }, //  734  GS
	{ REGISTRY_UNKNOWN  }, //  735  HS
	{ REGISTRY_RIPENCC  }, //  736  IS
	{ REGISTRY_UNKNOWN  }, //  737  JS
	{ REGISTRY_UNKNOWN  }, //  738  KS
	{ REGISTRY_AFRINIC  }, //  739  LS
	{ REGISTRY_ARIN     }, //  740  MS
	{ REGISTRY_UNKNOWN  }, //  741  NS
	{ REGISTRY_UNKNOWN  }, //  742  OS
	{ REGISTRY_RIPENCC  }, //  743  PS
	{ REGISTRY_UNKNOWN  }, //  744  QS
	{ REGISTRY_RIPENCC  }, //  745  RS
	{ REGISTRY_AFRINIC  }, //  746  SS
	{ REGISTRY_UNKNOWN  }, //  747  TS
	{ REGISTRY_ARIN     }, //  748  US
	{ REGISTRY_UNKNOWN  }, //  749  VS
	{ REGISTRY_APNIC    }, //  750  WS
	{ REGISTRY_UNKNOWN  }, //  751  XS
	{ REGISTRY_UNKNOWN  }, //  752  YS
	{ REGISTRY_UNKNOWN  }, //  753  ZS
	{ REGISTRY_RIPENCC  }, //  754  AT
	{ REGISTRY_APNIC    }, //  755  BT
	{ REGISTRY_UNKNOWN  }, //  756  CT
	{ REGISTRY_UNKNOWN  }, //  757  DT
	{ REGISTRY_AFRINIC  }, //  758  ET
	{ REGISTRY_UNKNOWN  }, //  759  FT
	{ REGISTRY_LACNIC   }, //  760  GT
	{ REGISTRY_LACNIC   }, //  761  HT
	{ REGISTRY_RIPENCC  }, //  762  IT
	{ REGISTRY_UNKNOWN  }, //  763  JT
	{ REGISTRY_UNKNOWN  }, //  764  KT
	{ REGISTRY_RIPENCC  }, //  765  LT
	{ REGISTRY_RIPENCC  }, //  766  MT
	{ REGISTRY_UNKNOWN  }, //  767  NT
	{ REGISTRY_UNKNOWN  }, //  768  OT
	{ REGISTRY_RIPENCC  }, //  769  PT
	{ REGISTRY_UNKNOWN  }, //  770  QT
	{ REGISTRY_UNKNOWN  }, //  771  RT
	{ REGISTRY_AFRINIC  }, //  772  ST
	{ REGISTRY_LACNIC   }, //  773  TT
	{ REGISTRY_UNKNOWN  }, //  774  UT
	{ REGISTRY_UNKNOWN  }, //  775  VT
	{ REGISTRY_UNKNOWN  }, //  776  WT
	{ REGISTRY_UNKNOWN  }, //  777  XT
	{ REGISTRY_AFRINIC  }, //  778  YT
	{ REGISTRY_UNKNOWN  }, //  779  ZT
	{ REGISTRY_APNIC    }, //  780  AU
	{ REGISTRY_UNKNOWN  }, //  781  BU
	{ REGISTRY_LACNIC   }, //  782  CU
	{ REGISTRY_UNKNOWN  }, //  783  DU
	{ REGISTRY_RIPENCC  }, //  784  EU
	{ REGISTRY_UNKNOWN  }, //  785  FU
	{ REGISTRY_APNIC    }, //  786  GU
	{ REGISTRY_RIPENCC  }, //  787  HU
	{ REGISTRY_UNKNOWN  }, //  788  IU
	{ REGISTRY_UNKNOWN  }, //  789  JU
	{ REGISTRY_UNKNOWN  }, //  790  KU
	{ REGISTRY_RIPENCC  }, //  791  LU
	{ REGISTRY_AFRINIC  }, //  792  MU
	{ REGISTRY_APNIC    }, //  793  NU
	{ REGISTRY_UNKNOWN  }, //  794  OU
	{ REGISTRY_UNKNOWN  }, //  795  PU
	{ REGISTRY_UNKNOWN  }, //  796  QU
	{ REGISTRY_RIPENCC  }, //  797  RU
	{ REGISTRY_UNKNOWN  }, //  798  SU
	{ REGISTRY_UNKNOWN  }, //  799  TU
	{ REGISTRY_UNKNOWN  }, //  800  UU
	{ REGISTRY_APNIC    }, //  801  VU
	{ REGISTRY_UNKNOWN  }, //  802  WU
	{ REGISTRY_UNKNOWN  }, //  803  XU
	{ REGISTRY_UNKNOWN  }, //  804  YU
	{ REGISTRY_UNKNOWN  }, //  805  ZU
	{ REGISTRY_UNKNOWN  }, //  806  AV
	{ REGISTRY_UNKNOWN  }, //  807  BV
	{ REGISTRY_AFRINIC  }, //  808  CV
	{ REGISTRY_UNKNOWN  }, //  809  DV
	{ REGISTRY_UNKNOWN  }, //  810  EV
	{ REGISTRY_UNKNOWN  }, //  811  FV
	{ REGISTRY_UNKNOWN  }, //  812  GV
	{ REGISTRY_UNKNOWN  }, //  813  HV
	{ REGISTRY_UNKNOWN  }, //  814  IV
	{ REGISTRY_UNKNOWN  }, //  815  JV
	{ REGISTRY_UNKNOWN  }, //  816  KV
	{ REGISTRY_RIPENCC  }, //  817  LV
	{ REGISTRY_APNIC    }, //  818  MV
	{ REGISTRY_UNKNOWN  }, //  819  NV
	{ REGISTRY_UNKNOWN  }, //  820  OV
	{ REGISTRY_UNKNOWN  }, //  821  PV
	{ REGISTRY_UNKNOWN  }, //  822  QV
	{ REGISTRY_UNKNOWN  }, //  823  RV
	{ REGISTRY_LACNIC   }, //  824  SV
	{ REGISTRY_APNIC    }, //  825  TV
	{ REGISTRY_UNKNOWN  }, //  826  UV
	{ REGISTRY_UNKNOWN  }, //  827  VV
	{ REGISTRY_UNKNOWN  }, //  828  WV
	{ REGISTRY_UNKNOWN  }, //  829  XV
	{ REGISTRY_UNKNOWN  }, //  830  YV
	{ REGISTRY_UNKNOWN  }, //  831  ZV
	{ REGISTRY_LACNIC   }, //  832  AW
	{ REGISTRY_AFRINIC  }, //  833  BW
	{ REGISTRY_LACNIC   }, //  834  CW
	{ REGISTRY_UNKNOWN  }, //  835  DW
	{ REGISTRY_UNKNOWN  }, //  836  EW
	{ REGISTRY_UNKNOWN  }, //  837  FW
	{ REGISTRY_AFRINIC  }, //  838  GW
	{ REGISTRY_UNKNOWN  }, //  839  HW
	{ REGISTRY_UNKNOWN  }, //  840  IW
	{ REGISTRY_UNKNOWN  }, //  841  JW
	{ REGISTRY_RIPENCC  }, //  842  KW
	{ REGISTRY_UNKNOWN  }, //  843  LW
	{ REGISTRY_AFRINIC  }, //  844  MW
	{ REGISTRY_UNKNOWN  }, //  845  NW
	{ REGISTRY_UNKNOWN  }, //  846  OW
	{ REGISTRY_APNIC    }, //  847  PW
	{ REGISTRY_UNKNOWN  }, //  848  QW
	{ REGISTRY_AFRINIC  }, //  849  RW
	{ REGISTRY_UNKNOWN  }, //  850  SW
	{ REGISTRY_APNIC    }, //  851  TW
	{ REGISTRY_UNKNOWN  }, //  852  UW
	{ REGISTRY_UNKNOWN  }, //  853  VW
	{ REGISTRY_UNKNOWN  }, //  854  WW
	{ REGISTRY_UNKNOWN  }, //  855  XW
	{ REGISTRY_UNKNOWN  }, //  856  YW
	{ REGISTRY_AFRINIC  }, //  857  ZW
	{ REGISTRY_UNKNOWN  }, //  858  AX
	{ REGISTRY_UNKNOWN  }, //  859  BX
	{ REGISTRY_UNKNOWN  }, //  860  CX
	{ REGISTRY_UNKNOWN  }, //  861  DX
	{ REGISTRY_UNKNOWN  }, //  862  EX
	{ REGISTRY_UNKNOWN  }, //  863  FX
	{ REGISTRY_UNKNOWN  }, //  864  GX
	{ REGISTRY_UNKNOWN  }, //  865  HX
	{ REGISTRY_UNKNOWN  }, //  866  IX
	{ REGISTRY_UNKNOWN  }, //  867  JX
	{ REGISTRY_UNKNOWN  }, //  868  KX
	{ REGISTRY_UNKNOWN  }, //  869  LX
	{ REGISTRY_LACNIC   }, //  870  MX
	{ REGISTRY_UNKNOWN  }, //  871  NX
	{ REGISTRY_UNKNOWN  }, //  872  OX
	{ REGISTRY_UNKNOWN  }, //  873  PX
	{ REGISTRY_UNKNOWN  }, //  874  QX
	{ REGISTRY_UNKNOWN  }, //  875  RX
	{ REGISTRY_LACNIC   }, //  876  SX
	{ REGISTRY_UNKNOWN  }, //  877  TX
	{ REGISTRY_UNKNOWN  }, //  878  UX
	{ REGISTRY_UNKNOWN  }, //  879  VX
	{ REGISTRY_UNKNOWN  }, //  880  WX
	{ REGISTRY_UNKNOWN  }, //  881  XX
	{ REGISTRY_UNKNOWN  }, //  882  YX
	{ REGISTRY_UNKNOWN  }, //  883  ZX
	{ REGISTRY_UNKNOWN  }, //  884  AY
	{ REGISTRY_RIPENCC  }, //  885  BY
	{ REGISTRY_RIPENCC  }, //  886  CY
	{ REGISTRY_UNKNOWN  }, //  887  DY
	{ REGISTRY_UNKNOWN  }, //  888  EY
	{ REGISTRY_UNKNOWN  }, //  889  FY
	{ REGISTRY_LACNIC   }, //  890  GY
	{ REGISTRY_UNKNOWN  }, //  891  HY
	{ REGISTRY_UNKNOWN  }, //  892  IY
	{ REGISTRY_UNKNOWN  }, //  893  JY
	{ REGISTRY_ARIN     }, //  894  KY
	{ REGISTRY_AFRINIC  }, //  895  LY
	{ REGISTRY_APNIC    }, //  896  MY
	{ REGISTRY_UNKNOWN  }, //  897  NY
	{ REGISTRY_UNKNOWN  }, //  898  OY
	{ REGISTRY_LACNIC   }, //  899  PY
	{ REGISTRY_UNKNOWN  }, //  900  QY
	{ REGISTRY_UNKNOWN  }, //  901  RY
	{ REGISTRY_RIPENCC  }, //  902  SY
	{ REGISTRY_UNKNOWN  }, //  903  TY
	{ REGISTRY_LACNIC   }, //  904  UY
	{ REGISTRY_UNKNOWN  }, //  905  VY
	{ REGISTRY_UNKNOWN  }, //  906  WY
	{ REGISTRY_UNKNOWN  }, //  907  XY
	{ REGISTRY_UNKNOWN  }, //  908  YY
	{ REGISTRY_UNKNOWN  }, //  909  ZY
	{ REGISTRY_RIPENCC  }, //  910  AZ
	{ REGISTRY_LACNIC   }, //  911  BZ
	{ REGISTRY_RIPENCC  }, //  912  CZ
	{ REGISTRY_AFRINIC  }, //  913  DZ
	{ REGISTRY_UNKNOWN  }, //  914  EZ
	{ REGISTRY_UNKNOWN  }, //  915  FZ
	{ REGISTRY_UNKNOWN  }, //  916  GZ
	{ REGISTRY_UNKNOWN  }, //  917  HZ
	{ REGISTRY_UNKNOWN  }, //  918  IZ
	{ REGISTRY_UNKNOWN  }, //  919  JZ
	{ REGISTRY_RIPENCC  }, //  920  KZ
	{ REGISTRY_UNKNOWN  }, //  921  LZ
	{ REGISTRY_AFRINIC  }, //  922  MZ
	{ REGISTRY_APNIC    }, //  923  NZ
	{ REGISTRY_UNKNOWN  }, //  924  OZ
	{ REGISTRY_UNKNOWN  }, //  925  PZ
	{ REGISTRY_UNKNOWN  }, //  926  QZ
	{ REGISTRY_UNKNOWN  }, //  927  RZ
	{ REGISTRY_AFRINIC  }, //  928  SZ
	{ REGISTRY_AFRINIC  }, //  929  TZ
	{ REGISTRY_RIPENCC  }, //  930  UZ
	{ REGISTRY_UNKNOWN  }, //  931  VZ
	{ REGISTRY_UNKNOWN  }, //  932  WZ
	{ REGISTRY_UNKNOWN  }, //  933  XZ
	{ REGISTRY_UNKNOWN  }, //  934  YZ
	{ REGISTRY_UNKNOWN  }, //  935  ZZ
	{ REGISTRY_UNKNOWN  }, //  936  reserved-special
	{ REGISTRY_UNKNOWN  }, //  937  reserved-special
	{ REGISTRY_UNKNOWN  }, //  938  reserved-special
	{ REGISTRY_UNKNOWN  }, //  939  reserved-special
	{ REGISTRY_UNKNOWN  }, //  940  reserved-special
	{ REGISTRY_UNKNOWN  }, //  941  reserved-special
	{ REGISTRY_UNKNOWN  }, //  942  reserved-special
	{ REGISTRY_UNKNOWN  }, //  943  reserved-special
	{ REGISTRY_UNKNOWN  }, //  944  reserved-special
	{ REGISTRY_UNKNOWN  }, //  945  reserved-special
	{ REGISTRY_UNKNOWN  }, //  946  reserved-special
	{ REGISTRY_UNKNOWN  }, //  947  reserved-special
	{ REGISTRY_UNKNOWN  }, //  948  reserved-special
	{ REGISTRY_UNKNOWN  }, //  949  reserved-special
	{ REGISTRY_UNKNOWN  }, //  950  reserved-special
	{ REGISTRY_UNKNOWN  }, //  951  reserved-special
	{ REGISTRY_UNKNOWN  }, //  952  reserved-special
	{ REGISTRY_UNKNOWN  }, //  953  reserved-special
	{ REGISTRY_UNKNOWN  }, //  954  reserved-special
	{ REGISTRY_UNKNOWN  }, //  955  reserved-special
	{ REGISTRY_UNKNOWN  }, //  956  reserved-special
	{ REGISTRY_UNKNOWN  }, //  957  reserved-special
	{ REGISTRY_UNKNOWN  }, //  958  reserved-special
	{ REGISTRY_UNKNOWN  }, //  959  reserved-special
	{ REGISTRY_UNKNOWN  }, //  960  reserved-special
	{ REGISTRY_UNKNOWN  }, //  961  reserved-special
	{ REGISTRY_UNKNOWN  }, //  962  reserved-special
	{ REGISTRY_UNKNOWN  }, //  963  reserved-special
	{ REGISTRY_UNKNOWN  }, //  964  reserved-special
	{ REGISTRY_UNKNOWN  }, //  965  reserved-special
	{ REGISTRY_UNKNOWN  }, //  966  reserved-special
	{ REGISTRY_UNKNOWN  }, //  967  reserved-special
	{ REGISTRY_UNKNOWN  }, //  968  reserved-special
	{ REGISTRY_UNKNOWN  }, //  969  reserved-special
	{ REGISTRY_UNKNOWN  }, //  970  reserved-special
	{ REGISTRY_UNKNOWN  }, //  971  reserved-special
	{ REGISTRY_UNKNOWN  }, //  972  reserved-special
	{ REGISTRY_UNKNOWN  }, //  973  reserved-special
	{ REGISTRY_UNKNOWN  }, //  974  reserved-special
	{ REGISTRY_UNKNOWN  }, //  975  reserved-special
	{ REGISTRY_UNKNOWN  }, //  976  reserved-special
	{ REGISTRY_UNKNOWN  }, //  977  reserved-special
	{ REGISTRY_UNKNOWN  }, //  978  reserved-special
	{ REGISTRY_UNKNOWN  }, //  979  reserved-special
	{ REGISTRY_UNKNOWN  }, //  980  reserved-special
	{ REGISTRY_UNKNOWN  }, //  981  reserved-special
	{ REGISTRY_UNKNOWN  }, //  982  reserved-special
	{ REGISTRY_UNKNOWN  }, //  983  reserved-special
	{ REGISTRY_UNKNOWN  }, //  984  reserved-special
	{ REGISTRY_UNKNOWN  }, //  985  reserved-special
	{ REGISTRY_UNKNOWN  }, //  986  reserved-special
	{ REGISTRY_UNKNOWN  }, //  987  reserved-special
	{ REGISTRY_UNKNOWN  }, //  988  reserved-special
	{ REGISTRY_UNKNOWN  }, //  989  reserved-special
	{ REGISTRY_UNKNOWN  }, //  990  reserved-special
	{ REGISTRY_UNKNOWN  }, //  991  reserved-special
	{ REGISTRY_UNKNOWN  }, //  992  reserved-special
	{ REGISTRY_UNKNOWN  }, //  993  reserved-special
	{ REGISTRY_UNKNOWN  }, //  994  reserved-special
	{ REGISTRY_UNKNOWN  }, //  995  reserved-special
	{ REGISTRY_UNKNOWN  }, //  996  reserved-special
	{ REGISTRY_UNKNOWN  }, //  997  reserved-special
	{ REGISTRY_UNKNOWN  }, //  998  reserved-special
	{ REGISTRY_UNKNOWN  }, //  999  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1000  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1001  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1002  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1003  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1004  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1005  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1006  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1007  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1008  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1009  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1010  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1011  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1012  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1013  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1014  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1015  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1016  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1017  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1018  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1019  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1020  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1021  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1022  reserved-special
	{ REGISTRY_UNKNOWN  }, // 1023  reserved-special
};
