/*
 * Project       : ipv6calc
 * File          : dbieee_oui.h
 * Version       : $Id: dbieee_oui.h,v 1.93 2015/02/20 07:00:22 ds6peter Exp $
 * Generated     : Fri Feb 20 07:39:31 2015
 * Data copyright: IEEE
 *
 * Information:
 *  Additional header file for libieee.c
 */

/*@unused@*/ static const char* libieee_oui_status __attribute__ ((__unused__)) = "OUI/20150218";


static const s_ieee_oui libieee_oui[] = {
	{ 0x000000, "XEROX CORPORATION", "XEROX" },
	{ 0x000001, "XEROX CORPORATION", "XEROX" },
	{ 0x000002, "XEROX CORPORATION", "XEROX" },
	{ 0x000003, "XEROX CORPORATION", "XEROX" },
	{ 0x000004, "XEROX CORPORATION", "XEROX" },
	{ 0x000005, "XEROX CORPORATION", "XEROX" },
	{ 0x000006, "XEROX CORPORATION", "XEROX" },
	{ 0x000007, "XEROX CORPORATION", "XEROX" },
	{ 0x000008, "XEROX CORPORATION", "XEROX" },
	{ 0x000009, "XEROX CORPORATION", "XEROX" },
	{ 0x00000A, "OMRON TATEISI ELECTRONICS CO.", "OMRON-TATEISI-ELECTRONICS" },
	{ 0x00000B, "MATRIX CORPORATION", "MATRIX" },
	{ 0x00000C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00000D, "FIBRONICS LTD.", "FIBRONICS" },
	{ 0x00000E, "FUJITSU LIMITED", "FUJITSU" },
	{ 0x00000F, "NEXT, INC.", "NEXT" },
	{ 0x000010, "SYTEK INC.", "SYTEK" },
	{ 0x000011, "NORMEREL SYSTEMES", "NORMEREL-SYSTEMES" },
	{ 0x000012, "INFORMATION TECHNOLOGY LIMITED", "INFORMATION" },
	{ 0x000013, "CAMEX", "CAMEX" },
	{ 0x000014, "NETRONIX", "NETRONIX" },
	{ 0x000015, "DATAPOINT CORPORATION", "DATAPOINT" },
	{ 0x000016, "DU PONT PIXEL SYSTEMS .", "DU-PONT-PIXEL" },
	{ 0x000017, "Oracle", "ORACLE" },
	{ 0x000018, "WEBSTER COMPUTER CORPORATION", "WEBSTER" },
	{ 0x000019, "APPLIED DYNAMICS INTERNATIONAL", "APPLIED-DYNAMICS" },
	{ 0x00001A, "ADVANCED MICRO DEVICES", "ADVANCED-MICRO-DEVICES" },
	{ 0x00001B, "NOVELL INC.", "NOVELL" },
	{ 0x00001C, "BELL TECHNOLOGIES", "BELL-TECHNOLOGIES" },
	{ 0x00001D, "CABLETRON SYSTEMS, INC.", "CABLETRON" },
	{ 0x00001E, "TELSIST INDUSTRIA ELECTRONICA", "TELSIST-INDUSTRIA-ELECTRONICA" },
	{ 0x00001F, "Telco Systems, Inc.", "TELCO" },
	{ 0x000020, "DATAINDUSTRIER DIAB AB", "DATAINDUSTRIER-DIAB-AB" },
	{ 0x000021, "SUREMAN COMP. & COMMUN. CORP.", "SUREMAN-COMP-COMMUN" },
	{ 0x000022, "VISUAL TECHNOLOGY INC.", "VISUAL" },
	{ 0x000023, "ABB INDUSTRIAL SYSTEMS AB", "ABB-INDUSTRIAL-AB" },
	{ 0x000024, "CONNECT AS", "CONNECT-AS" },
	{ 0x000025, "RAMTEK CORP.", "RAMTEK" },
	{ 0x000026, "SHA-KEN CO., LTD.", "SHA-KEN" },
	{ 0x000027, "JAPAN RADIO COMPANY", "JAPAN-RADIO-COMPANY" },
	{ 0x000028, "PRODIGY SYSTEMS CORPORATION", "PRODIGY" },
	{ 0x000029, "IMC NETWORKS CORP.", "IMC-NETWORKS" },
	{ 0x00002A, "TRW - SEDD/INP", "TRW---SEDD/INP" },
	{ 0x00002B, "CRISP AUTOMATION, INC", "CRISP-AUTOMATION" },
	{ 0x00002C, "AUTOTOTE LIMITED", "AUTOTOTE" },
	{ 0x00002D, "CHROMATICS INC", "CHROMATICS" },
	{ 0x00002E, "SOCIETE EVIRA", "SOCIETE-EVIRA" },
	{ 0x00002F, "TIMEPLEX INC.", "TIMEPLEX" },
	{ 0x000030, "VG LABORATORY SYSTEMS LTD", "VG-LABORATORY" },
	{ 0x000031, "QPSX COMMUNICATIONS PTY LTD", "QPSX-COMMUNICATION-PTY" },
	{ 0x000032, "Marconi plc", "MARCONI-PLC" },
	{ 0x000033, "EGAN MACHINERY COMPANY", "EGAN-MACHINERY-COMPANY" },
	{ 0x000034, "NETWORK RESOURCES CORPORATION", "NETWORK-RESOURCES" },
	{ 0x000035, "SPECTRAGRAPHICS CORPORATION", "SPECTRAGRAPHICS" },
	{ 0x000036, "ATARI CORPORATION", "ATARI" },
	{ 0x000037, "OXFORD METRICS LIMITED", "OXFORD-METRICS" },
	{ 0x000038, "CSS LABS", "CSS-LABS" },
	{ 0x000039, "TOSHIBA CORPORATION", "TOSHIBA" },
	{ 0x00003A, "CHYRON CORPORATION", "CHYRON" },
	{ 0x00003B, "i Controls, Inc.", "I-CONTROLS" },
	{ 0x00003C, "AUSPEX SYSTEMS INC.", "AUSPEX" },
	{ 0x00003D, "UNISYS", "UNISYS" },
	{ 0x00003E, "SIMPACT", "SIMPACT" },
	{ 0x00003F, "SYNTREX, INC.", "SYNTREX" },
	{ 0x000040, "APPLICON, INC.", "APPLICON" },
	{ 0x000041, "ICE CORPORATION", "ICE" },
	{ 0x000042, "METIER MANAGEMENT SYSTEMS LTD.", "METIER-MANAGEMENT" },
	{ 0x000043, "MICRO TECHNOLOGY", "MICRO" },
	{ 0x000044, "CASTELLE CORPORATION", "CASTELLE" },
	{ 0x000045, "FORD AEROSPACE & COMM. CORP.", "FORD-AEROSPACE-COMM" },
	{ 0x000046, "OLIVETTI NORTH AMERICA", "OLIVETTI-NORTH-AMERICA" },
	{ 0x000047, "NICOLET INSTRUMENTS CORP.", "NICOLET-INSTRUMENTS" },
	{ 0x000048, "SEIKO EPSON CORPORATION", "SEIKO-EPSON" },
	{ 0x000049, "APRICOT COMPUTERS, LTD", "APRICOT-COMPUTERS" },
	{ 0x00004A, "ADC CODENOLL TECHNOLOGY CORP.", "ADC-CODENOLL" },
	{ 0x00004B, "ICL DATA OY", "ICL-DATA-OY" },
	{ 0x00004C, "NEC CORPORATION", "NEC" },
	{ 0x00004D, "DCI CORPORATION", "DCI" },
	{ 0x00004E, "AMPEX CORPORATION", "AMPEX" },
	{ 0x00004F, "LOGICRAFT, INC.", "LOGICRAFT" },
	{ 0x000050, "RADISYS CORPORATION", "RADISYS" },
	{ 0x000051, "HOB ELECTRONIC GMBH & CO. KG", "HOB" },
	{ 0x000052, "Intrusion.com, Inc.", "INTRUSION-COM" },
	{ 0x000053, "COMPUCORP", "COMPUCORP" },
	{ 0x000054, "Schnieder Electric", "SCHNIEDER-ELECTRIC" },
	{ 0x000055, "COMMISSARIAT A L`ENERGIE ATOM.", "COMMISSARIAT-A-L`ENERGIE-ATOM" },
	{ 0x000056, "DR. B. STRUCK", "DR-B-STRUCK" },
	{ 0x000057, "SCITEX CORPORATION LTD.", "SCITEX" },
	{ 0x000058, "RACORE COMPUTER PRODUCTS INC.", "RACORE-PRODUCTS" },
	{ 0x000059, "HELLIGE GMBH", "HELLIGE" },
	{ 0x00005A, "SysKonnect GmbH", "SYSKONNECT" },
	{ 0x00005B, "ELTEC ELEKTRONIK AG", "ELTEC-ELEKTRONIK" },
	{ 0x00005C, "TELEMATICS INTERNATIONAL INC.", "TELEMATICS" },
	{ 0x00005D, "CS TELECOM", "CS-TELECOM" },
	{ 0x00005E, "ICANN, IANA Department", "ICANN-IANA-DEPARTMENT" },
	{ 0x00005F, "SUMITOMO ELECTRIC IND., LTD.", "SUMITOMO-ELECTRIC-IND" },
	{ 0x000060, "KONTRON ELEKTRONIK GMBH", "KONTRON-ELEKTRONIK" },
	{ 0x000061, "GATEWAY COMMUNICATIONS", "GATEWAY-COMMUNICATION" },
	{ 0x000062, "BULL HN INFORMATION SYSTEMS", "BULL-HN-INFORMATION" },
	{ 0x000063, "BARCO CONTROL ROOMS GMBH", "BARCO-CONTROL-ROOMS" },
	{ 0x000064, "Yokogawa Electric Corporation", "YOKOGAWA-ELECTRIC" },
	{ 0x000065, "Network General Corporation", "NETWORK-GENERAL" },
	{ 0x000066, "TALARIS SYSTEMS, INC.", "TALARIS" },
	{ 0x000067, "SOFT * RITE, INC.", "SOFT-*-RITE" },
	{ 0x000068, "ROSEMOUNT CONTROLS", "ROSEMOUNT-CONTROLS" },
	{ 0x000069, "CONCORD COMMUNICATIONS INC", "CONCORD-COMMUNICATION" },
	{ 0x00006A, "COMPUTER CONSOLES INC.", "CONSOLES" },
	{ 0x00006B, "SILICON GRAPHICS INC./MIPS", "SILICON-GRAPHICS-/MIPS" },
	{ 0x00006C, "PRIVATE", "PRIVATE" },
	{ 0x00006D, "CRAY COMMUNICATIONS, LTD.", "CRAY-COMMUNICATION" },
	{ 0x00006E, "ARTISOFT, INC.", "ARTISOFT" },
	{ 0x00006F, "Madge Ltd.", "MADGE" },
	{ 0x000070, "HCL LIMITED", "HCL" },
	{ 0x000071, "ADRA SYSTEMS INC.", "ADRA" },
	{ 0x000072, "MINIWARE TECHNOLOGY", "MINIWARE" },
	{ 0x000073, "SIECOR CORPORATION", "SIECOR" },
	{ 0x000074, "RICOH COMPANY LTD.", "RICOH-COMPANY" },
	{ 0x000075, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000076, "ABEKAS VIDEO SYSTEM", "ABEKAS-VIDEO-SYSTEM" },
	{ 0x000077, "INTERPHASE CORPORATION", "INTERPHASE" },
	{ 0x000078, "LABTAM LIMITED", "LABTAM" },
	{ 0x000079, "NETWORTH INCORPORATED", "NETWORTH-INCORPORATED" },
	{ 0x00007A, "DANA COMPUTER INC.", "DANA" },
	{ 0x00007B, "RESEARCH MACHINES", "RESEARCH-MACHINES" },
	{ 0x00007C, "AMPERE INCORPORATED", "AMPERE-INCORPORATED" },
	{ 0x00007D, "Oracle Corporation", "ORACLE" },
	{ 0x00007E, "CLUSTRIX CORPORATION", "CLUSTRIX" },
	{ 0x00007F, "LINOTYPE-HELL AG", "LINOTYPE-HELL" },
	{ 0x000080, "CRAY COMMUNICATIONS A/S", "CRAY-COMMUNICATION-A/S" },
	{ 0x000081, "BAY NETWORKS", "BAY-NETWORKS" },
	{ 0x000082, "LECTRA SYSTEMES SA", "LECTRA-SYSTEMES" },
	{ 0x000083, "TADPOLE TECHNOLOGY PLC", "TADPOLE-PLC" },
	{ 0x000084, "SUPERNET", "SUPERNET" },
	{ 0x000085, "CANON INC.", "CANON" },
	{ 0x000086, "MEGAHERTZ CORPORATION", "MEGAHERTZ" },
	{ 0x000087, "HITACHI, LTD.", "HITACHI" },
	{ 0x000088, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x000089, "CAYMAN SYSTEMS INC.", "CAYMAN" },
	{ 0x00008A, "DATAHOUSE INFORMATION SYSTEMS", "DATAHOUSE-INFORMATION" },
	{ 0x00008B, "INFOTRON", "INFOTRON" },
	{ 0x00008C, "Alloy Computer Products (Australia) Pty Ltd", "ALLOY-PRODUCTS-AUSTRALIA-PTY" },
	{ 0x00008D, "Cryptek Inc.", "CRYPTEK" },
	{ 0x00008E, "SOLBOURNE COMPUTER, INC.", "SOLBOURNE" },
	{ 0x00008F, "Raytheon", "RAYTHEON" },
	{ 0x000090, "MICROCOM", "MICROCOM" },
	{ 0x000091, "ANRITSU CORPORATION", "ANRITSU" },
	{ 0x000092, "COGENT DATA TECHNOLOGIES", "COGENT-DATA-TECHNOLOGIES" },
	{ 0x000093, "PROTEON INC.", "PROTEON" },
	{ 0x000094, "ASANTE TECHNOLOGIES", "ASANTE-TECHNOLOGIES" },
	{ 0x000095, "SONY TEKTRONIX CORP.", "SONY-TEKTRONIX" },
	{ 0x000096, "MARCONI ELECTRONICS LTD.", "MARCONI-ELECTRONICS" },
	{ 0x000097, "EMC Corporation", "EMC" },
	{ 0x000098, "CROSSCOMM CORPORATION", "CROSSCOMM" },
	{ 0x000099, "MTX, INC.", "MTX" },
	{ 0x00009A, "RC COMPUTER A/S", "RC-A/S" },
	{ 0x00009B, "INFORMATION INTERNATIONAL, INC", "INFORMATION" },
	{ 0x00009C, "ROLM MIL-SPEC COMPUTERS", "ROLM-MIL-SPEC-COMPUTERS" },
	{ 0x00009D, "LOCUS COMPUTING CORPORATION", "LOCUS-COMPUTING" },
	{ 0x00009E, "MARLI S.A.", "MARLI" },
	{ 0x00009F, "AMERISTAR TECHNOLOGIES INC.", "AMERISTAR-TECHNOLOGIES" },
	{ 0x0000A0, "SANYO Electric Co., Ltd.", "SANYO-ELECTRIC" },
	{ 0x0000A1, "MARQUETTE ELECTRIC CO.", "MARQUETTE-ELECTRIC" },
	{ 0x0000A2, "BAY NETWORKS", "BAY-NETWORKS" },
	{ 0x0000A3, "NETWORK APPLICATION TECHNOLOGY", "NETWORK-APPLICATION" },
	{ 0x0000A4, "ACORN COMPUTERS LIMITED", "ACORN-COMPUTERS" },
	{ 0x0000A5, "Tattile SRL", "TATTILE-SRL" },
	{ 0x0000A6, "NETWORK GENERAL CORPORATION", "NETWORK-GENERAL" },
	{ 0x0000A7, "NETWORK COMPUTING DEVICES INC.", "NETWORK-COMPUTING-DEVICES" },
	{ 0x0000A8, "STRATUS COMPUTER INC.", "STRATUS" },
	{ 0x0000A9, "NETWORK SYSTEMS CORP.", "NETWORK" },
	{ 0x0000AA, "XEROX CORPORATION", "XEROX" },
	{ 0x0000AB, "LOGIC MODELING CORPORATION", "LOGIC-MODELING" },
	{ 0x0000AC, "CONWARE COMPUTER CONSULTING", "CONWARE-CONSULTING" },
	{ 0x0000AD, "BRUKER INSTRUMENTS INC.", "BRUKER-INSTRUMENTS" },
	{ 0x0000AE, "DASSAULT ELECTRONIQUE", "DASSAULT-ELECTRONIQUE" },
	{ 0x0000AF, "Canberra Industries, Inc.", "CANBERRA-INDUSTRIES" },
	{ 0x0000B0, "RND-RAD NETWORK DEVICES", "RND-RAD-NETWORK-DEVICES" },
	{ 0x0000B1, "ALPHA MICROSYSTEMS INC.", "ALPHA-MICROSYSTEMS" },
	{ 0x0000B2, "TELEVIDEO SYSTEMS, INC.", "TELEVIDEO" },
	{ 0x0000B3, "CIMLINC INCORPORATED", "CIMLINC-INCORPORATED" },
	{ 0x0000B4, "EDIMAX COMPUTER COMPANY", "EDIMAX-COMPANY" },
	{ 0x0000B5, "DATABILITY SOFTWARE SYS. INC.", "DATABILITY-SOFTWARE-SYS" },
	{ 0x0000B6, "MICRO-MATIC RESEARCH", "MICRO-MATIC-RESEARCH" },
	{ 0x0000B7, "DOVE COMPUTER CORPORATION", "DOVE" },
	{ 0x0000B8, "SEIKOSHA CO., LTD.", "SEIKOSHA" },
	{ 0x0000B9, "MCDONNELL DOUGLAS COMPUTER SYS", "MCDONNELL-DOUGLAS-SYS" },
	{ 0x0000BA, "SIIG, INC.", "SIIG" },
	{ 0x0000BB, "TRI-DATA", "TRI-DATA" },
	{ 0x0000BC, "Rockwell Automation", "ROCKWELL-AUTOMATION" },
	{ 0x0000BD, "MITSUBISHI CABLE COMPANY", "MITSUBISHI-CABLE-COMPANY" },
	{ 0x0000BE, "THE NTI GROUP", "THE-NTI-GROUP" },
	{ 0x0000BF, "SYMMETRIC COMPUTER SYSTEMS", "SYMMETRIC" },
	{ 0x0000C0, "WESTERN DIGITAL CORPORATION", "WESTERN-DIGITAL" },
	{ 0x0000C1, "Madge Ltd.", "MADGE" },
	{ 0x0000C2, "INFORMATION PRESENTATION TECH.", "INFORMATION-PRESENTATION-TECH" },
	{ 0x0000C3, "HARRIS CORP COMPUTER SYS DIV", "HARRIS-SYS-DIV" },
	{ 0x0000C4, "WATERS DIV. OF MILLIPORE", "WATERS-DIV-OF-MILLIPORE" },
	{ 0x0000C5, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0000C6, "EON SYSTEMS", "EON" },
	{ 0x0000C7, "ARIX CORPORATION", "ARIX" },
	{ 0x0000C8, "ALTOS COMPUTER SYSTEMS", "ALTOS" },
	{ 0x0000C9, "Emulex Corporation", "EMULEX" },
	{ 0x0000CA, "ARRIS International", "ARRIS" },
	{ 0x0000CB, "COMPU-SHACK ELECTRONIC GMBH", "COMPU-SHACK" },
	{ 0x0000CC, "DENSAN CO., LTD.", "DENSAN" },
	{ 0x0000CD, "Allied Telesis Labs Ltd", "ALLIED-TELESIS-LABS" },
	{ 0x0000CE, "MEGADATA CORP.", "MEGADATA" },
	{ 0x0000CF, "HAYES MICROCOMPUTER PRODUCTS", "HAYES-MICROCOMPUTER-PRODUCTS" },
	{ 0x0000D0, "DEVELCON ELECTRONICS LTD.", "DEVELCON-ELECTRONICS" },
	{ 0x0000D1, "ADAPTEC INCORPORATED", "ADAPTEC-INCORPORATED" },
	{ 0x0000D2, "SBE, INC.", "SBE" },
	{ 0x0000D3, "WANG LABORATORIES INC.", "WANG-LABORATORIES" },
	{ 0x0000D4, "PURE DATA LTD.", "PURE-DATA" },
	{ 0x0000D5, "MICROGNOSIS INTERNATIONAL", "MICROGNOSIS" },
	{ 0x0000D6, "PUNCH LINE HOLDING", "PUNCH-LINE-HOLDING" },
	{ 0x0000D7, "DARTMOUTH COLLEGE", "DARTMOUTH-COLLEGE" },
	{ 0x0000D8, "NOVELL, INC.", "NOVELL" },
	{ 0x0000D9, "NIPPON TELEGRAPH & TELEPHONE", "NIPPON-TELEGRAPH-TELEPHONE" },
	{ 0x0000DA, "ATEX", "ATEX" },
	{ 0x0000DB, "British Telecommunications plc", "BRITISH-TELECOMMUNICATIONS-PLC" },
	{ 0x0000DC, "HAYES MICROCOMPUTER PRODUCTS", "HAYES-MICROCOMPUTER-PRODUCTS" },
	{ 0x0000DD, "TCL INCORPORATED", "TCL-INCORPORATED" },
	{ 0x0000DE, "CETIA", "CETIA" },
	{ 0x0000DF, "BELL & HOWELL PUB SYS DIV", "BELL-HOWELL-PUB-SYS-DIV" },
	{ 0x0000E0, "QUADRAM CORP.", "QUADRAM" },
	{ 0x0000E1, "GRID SYSTEMS", "GRID" },
	{ 0x0000E2, "ACER TECHNOLOGIES CORP.", "ACER-TECHNOLOGIES" },
	{ 0x0000E3, "INTEGRATED MICRO PRODUCTS LTD", "INTEGRATED-MICRO-PRODUCTS" },
	{ 0x0000E4, "IN2 GROUPE INTERTECHNIQUE", "IN2-GROUPE-INTERTECHNIQUE" },
	{ 0x0000E5, "SIGMEX LTD.", "SIGMEX" },
	{ 0x0000E6, "APTOR PRODUITS DE COMM INDUST", "APTOR-PRODUITS-DE-COMM-INDUST" },
	{ 0x0000E7, "STAR GATE TECHNOLOGIES", "STAR-GATE-TECHNOLOGIES" },
	{ 0x0000E8, "ACCTON TECHNOLOGY CORP.", "ACCTON" },
	{ 0x0000E9, "ISICAD, INC.", "ISICAD" },
	{ 0x0000EA, "UPNOD AB", "UPNOD-AB" },
	{ 0x0000EB, "MATSUSHITA COMM. IND. CO. LTD.", "MATSUSHITA-COMM-IND" },
	{ 0x0000EC, "MICROPROCESS", "MICROPROCESS" },
	{ 0x0000ED, "APRIL", "APRIL" },
	{ 0x0000EE, "NETWORK DESIGNERS, LTD.", "NETWORK-DESIGNERS" },
	{ 0x0000EF, "KTI", "KTI" },
	{ 0x0000F0, "SAMSUNG ELECTRONICS CO., LTD.", "SAMSUNG-ELECTRONICS" },
	{ 0x0000F1, "MAGNA COMPUTER CORPORATION", "MAGNA" },
	{ 0x0000F2, "SPIDER COMMUNICATIONS", "SPIDER-COMMUNICATION" },
	{ 0x0000F3, "GANDALF DATA LIMITED", "GANDALF-DATA" },
	{ 0x0000F4, "Allied Telesis", "ALLIED-TELESIS" },
	{ 0x0000F5, "DIAMOND SALES LIMITED", "DIAMOND-SALES" },
	{ 0x0000F6, "APPLIED MICROSYSTEMS CORP.", "APPLIED-MICROSYSTEMS" },
	{ 0x0000F7, "YOUTH KEEP ENTERPRISE CO LTD", "YOUTH-KEEP" },
	{ 0x0000F8, "DIGITAL EQUIPMENT CORPORATION", "DIGITAL-EQUIPMENT" },
	{ 0x0000F9, "QUOTRON SYSTEMS INC.", "QUOTRON" },
	{ 0x0000FA, "MICROSAGE COMPUTER SYSTEMS INC", "MICROSAGE" },
	{ 0x0000FB, "RECHNER ZUR KOMMUNIKATION", "RECHNER-ZUR-KOMMUNIKATION" },
	{ 0x0000FC, "MEIKO", "MEIKO" },
	{ 0x0000FD, "HIGH LEVEL HARDWARE", "HIGH-LEVEL-HARDWARE" },
	{ 0x0000FE, "ANNAPOLIS MICRO SYSTEMS", "ANNAPOLIS-MICRO" },
	{ 0x0000FF, "CAMTEC ELECTRONICS LTD.", "CAMTEC-ELECTRONICS" },
	{ 0x000100, "EQUIP'TRANS", "EQUIP-TRANS" },
	{ 0x000101, "PRIVATE", "PRIVATE" },
	{ 0x000102, "3COM CORPORATION", "3COM" },
	{ 0x000103, "3COM CORPORATION", "3COM" },
	{ 0x000104, "DVICO Co., Ltd.", "DVICO" },
	{ 0x000105, "Beckhoff Automation GmbH", "BECKHOFF-AUTOMATION" },
	{ 0x000106, "Tews Datentechnik GmbH", "TEWS-DATENTECHNIK" },
	{ 0x000107, "Leiser GmbH", "LEISER" },
	{ 0x000108, "AVLAB Technology, Inc.", "AVLAB" },
	{ 0x000109, "Nagano Japan Radio Co., Ltd.", "NAGANO-JAPAN-RADIO" },
	{ 0x00010A, "CIS TECHNOLOGY INC.", "CIS" },
	{ 0x00010B, "Space CyberLink, Inc.", "SPACE-CYBERLINK" },
	{ 0x00010C, "System Talks Inc.", "SYSTEM-TALKS" },
	{ 0x00010D, "CORECO, INC.", "CORECO" },
	{ 0x00010E, "Bri-Link Technologies Co., Ltd", "BRI-LINK-TECHNOLOGIES" },
	{ 0x00010F, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x000110, "Gotham Networks", "GOTHAM-NETWORKS" },
	{ 0x000111, "iDigm Inc.", "IDIGM" },
	{ 0x000112, "Shark Multimedia Inc.", "SHARK-MULTIMEDIA" },
	{ 0x000113, "OLYMPUS CORPORATION", "OLYMPUS" },
	{ 0x000114, "KANDA TSUSHIN KOGYO CO., LTD.", "KANDA-TSUSHIN-KOGYO" },
	{ 0x000115, "EXTRATECH CORPORATION", "EXTRATECH" },
	{ 0x000116, "Netspect Technologies, Inc.", "NETSPECT-TECHNOLOGIES" },
	{ 0x000117, "CANAL +", "CANAL-+" },
	{ 0x000118, "EZ Digital Co., Ltd.", "EZ-DIGITAL" },
	{ 0x000119, "RTUnet (Australia)", "RTUNET-AUSTRALIA" },
	{ 0x00011A, "Hoffmann und Burmeister GbR", "HOFFMANN-UND-BURMEISTER-GBR" },
	{ 0x00011B, "Unizone Technologies, Inc.", "UNIZONE-TECHNOLOGIES" },
	{ 0x00011C, "Universal Talkware Corporation", "UNIVERSAL-TALKWARE" },
	{ 0x00011D, "Centillium Communications", "CENTILLIUM-COMMUNICATION" },
	{ 0x00011E, "Precidia Technologies, Inc.", "PRECIDIA-TECHNOLOGIES" },
	{ 0x00011F, "RC Networks, Inc.", "RC-NETWORKS" },
	{ 0x000120, "OSCILLOQUARTZ S.A.", "OSCILLOQUARTZ" },
	{ 0x000121, "Watchguard Technologies, Inc.", "WATCHGUARD-TECHNOLOGIES" },
	{ 0x000122, "Trend Communications, Ltd.", "TREND-COMMUNICATION" },
	{ 0x000123, "DIGITAL ELECTRONICS CORP.", "DIGITAL-ELECTRONICS" },
	{ 0x000124, "Acer Incorporated", "ACER-INCORPORATED" },
	{ 0x000125, "YAESU MUSEN CO., LTD.", "YAESU-MUSEN" },
	{ 0x000126, "PAC Labs", "PAC-LABS" },
	{ 0x000127, "OPEN Networks Pty Ltd", "OPEN-NETWORKS-PTY" },
	{ 0x000128, "EnjoyWeb, Inc.", "ENJOYWEB" },
	{ 0x000129, "DFI Inc.", "DFI" },
	{ 0x00012A, "Telematica Sistems Inteligente", "TELEMATICA-SISTEMS-INTELIGENTE" },
	{ 0x00012B, "TELENET Co., Ltd.", "TELENET" },
	{ 0x00012C, "Aravox Technologies, Inc.", "ARAVOX-TECHNOLOGIES" },
	{ 0x00012D, "Komodo Technology", "KOMODO" },
	{ 0x00012E, "PC Partner Ltd.", "PC-PARTNER" },
	{ 0x00012F, "Twinhead International Corp", "TWINHEAD" },
	{ 0x000130, "Extreme Networks", "EXTREME-NETWORKS" },
	{ 0x000131, "Bosch Security Systems, Inc.", "BOSCH-SECURITY" },
	{ 0x000132, "Dranetz - BMI", "DRANETZ---BMI" },
	{ 0x000133, "KYOWA Electronic Instruments C", "KYOWA-INSTRUMENTS-C" },
	{ 0x000134, "Selectron Systems AG", "SELECTRON" },
	{ 0x000135, "KDC Corp.", "KDC" },
	{ 0x000136, "CyberTAN Technology, Inc.", "CYBERTAN" },
	{ 0x000137, "IT Farm Corporation", "IT-FARM" },
	{ 0x000138, "XAVi Technologies Corp.", "XAVI-TECHNOLOGIES" },
	{ 0x000139, "Point Multimedia Systems", "POINT-MULTIMEDIA" },
	{ 0x00013A, "SHELCAD COMMUNICATIONS, LTD.", "SHELCAD-COMMUNICATION" },
	{ 0x00013B, "BNA SYSTEMS", "BNA" },
	{ 0x00013C, "TIW SYSTEMS", "TIW" },
	{ 0x00013D, "RiscStation Ltd.", "RISCSTATION" },
	{ 0x00013E, "Ascom Tateco AB", "ASCOM-TATECO-AB" },
	{ 0x00013F, "Neighbor World Co., Ltd.", "NEIGHBOR-WORLD" },
	{ 0x000140, "Sendtek Corporation", "SENDTEK" },
	{ 0x000141, "CABLE PRINT", "CABLE-PRINT" },
	{ 0x000142, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000143, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000144, "EMC Corporation", "EMC" },
	{ 0x000145, "WINSYSTEMS, INC.", "WINSYSTEMS" },
	{ 0x000146, "Tesco Controls, Inc.", "TESCO-CONTROLS" },
	{ 0x000147, "Zhone Technologies", "ZHONE-TECHNOLOGIES" },
	{ 0x000148, "X-traWeb Inc.", "X-TRAWEB" },
	{ 0x000149, "T.D.T. Transfer Data Test GmbH", "T-D-T-TRANSFER-DATA-TEST" },
	{ 0x00014A, "Sony Corporation", "SONY" },
	{ 0x00014B, "Ennovate Networks, Inc.", "ENNOVATE-NETWORKS" },
	{ 0x00014C, "Berkeley Process Control", "BERKELEY-PROCESS-CONTROL" },
	{ 0x00014D, "Shin Kin Enterprises Co., Ltd", "SHIN-KIN-ENTERPRISES" },
	{ 0x00014E, "WIN Enterprises, Inc.", "WIN-ENTERPRISES" },
	{ 0x00014F, "ADTRAN INC", "ADTRAN" },
	{ 0x000150, "GILAT COMMUNICATIONS, LTD.", "GILAT-COMMUNICATION" },
	{ 0x000151, "Ensemble Communications", "ENSEMBLE-COMMUNICATION" },
	{ 0x000152, "CHROMATEK INC.", "CHROMATEK" },
	{ 0x000153, "ARCHTEK TELECOM CORPORATION", "ARCHTEK-TELECOM" },
	{ 0x000154, "G3M Corporation", "G3M" },
	{ 0x000155, "Promise Technology, Inc.", "PROMISE" },
	{ 0x000156, "FIREWIREDIRECT.COM, INC.", "FIREWIREDIRECT-COM" },
	{ 0x000157, "SYSWAVE CO., LTD", "SYSWAVE" },
	{ 0x000158, "Electro Industries/Gauge Tech", "ELECTRO-INDUSTRIES/GAUGE-TECH" },
	{ 0x000159, "S1 Corporation", "S1" },
	{ 0x00015A, "Digital Video Broadcasting", "DIGITAL-VIDEO-BROADCASTING" },
	{ 0x00015B, "ITALTEL S.p.A/RF-UP-I", "ITALTEL-S-P-A/RF-UP-I" },
	{ 0x00015C, "CADANT INC.", "CADANT" },
	{ 0x00015D, "Oracle Corporation", "ORACLE" },
	{ 0x00015E, "BEST TECHNOLOGY CO., LTD.", "BEST" },
	{ 0x00015F, "DIGITAL DESIGN GmbH", "DIGITAL-DESIGN" },
	{ 0x000160, "ELMEX Co., LTD.", "ELMEX" },
	{ 0x000161, "Meta Machine Technology", "META-MACHINE" },
	{ 0x000162, "Cygnet Technologies, Inc.", "CYGNET-TECHNOLOGIES" },
	{ 0x000163, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000164, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000165, "AirSwitch Corporation", "AIRSWITCH" },
	{ 0x000166, "TC GROUP A/S", "TC-GROUP-A/S" },
	{ 0x000167, "HIOKI E.E. CORPORATION", "HIOKI-E-E" },
	{ 0x000168, "VITANA CORPORATION", "VITANA" },
	{ 0x000169, "Celestix Networks Pte Ltd.", "CELESTIX-NETWORKS-PTE" },
	{ 0x00016A, "ALITEC", "ALITEC" },
	{ 0x00016B, "LightChip, Inc.", "LIGHTCHIP" },
	{ 0x00016C, "FOXCONN", "FOXCONN" },
	{ 0x00016D, "CarrierComm Inc.", "CARRIERCOMM" },
	{ 0x00016E, "Conklin Corporation", "CONKLIN" },
	{ 0x00016F, "Inkel Corp.", "INKEL" },
	{ 0x000170, "ESE Embedded System Engineer'g", "ESE-EMBEDDED-SYSTEM-ENGINEER-G" },
	{ 0x000171, "Allied Data Technologies", "ALLIED-DATA-TECHNOLOGIES" },
	{ 0x000172, "TechnoLand Co., LTD.", "TECHNOLAND" },
	{ 0x000173, "AMCC", "AMCC" },
	{ 0x000174, "CyberOptics Corporation", "CYBEROPTICS" },
	{ 0x000175, "Radiant Communications Corp.", "RADIANT-COMMUNICATION" },
	{ 0x000176, "Orient Silver Enterprises", "ORIENT-SILVER-ENTERPRISES" },
	{ 0x000177, "EDSL", "EDSL" },
	{ 0x000178, "MARGI Systems, Inc.", "MARGI" },
	{ 0x000179, "WIRELESS TECHNOLOGY, INC.", "WIRELESS" },
	{ 0x00017A, "Chengdu Maipu Electric Industrial Co., Ltd.", "CHENGDU-MAIPU-ELECTRIC-INDUSTRIAL" },
	{ 0x00017B, "Heidelberger Druckmaschinen AG", "HEIDELBERGER-DRUCKMASCHINEN" },
	{ 0x00017C, "AG-E GmbH", "-E" },
	{ 0x00017D, "ThermoQuest", "THERMOQUEST" },
	{ 0x00017E, "ADTEK System Science Co., Ltd.", "ADTEK-SYSTEM-SCIENCE" },
	{ 0x00017F, "Experience Music Project", "EXPERIENCE-MUSIC-PROJECT" },
	{ 0x000180, "AOpen, Inc.", "AOPEN" },
	{ 0x000181, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000182, "DICA TECHNOLOGIES AG", "DICA-TECHNOLOGIES" },
	{ 0x000183, "ANITE TELECOMS", "ANITE-TELECOMS" },
	{ 0x000184, "SIEB & MEYER AG", "SIEB-MEYER" },
	{ 0x000185, "Hitachi Aloka Medical, Ltd.", "HITACHI-ALOKA-MEDICAL" },
	{ 0x000186, "Uwe Disch", "UWE-DISCH" },
	{ 0x000187, "I2SE GmbH", "I2SE" },
	{ 0x000188, "LXCO Technologies ag", "LXCO-TECHNOLOGIES" },
	{ 0x000189, "Refraction Technology, Inc.", "REFRACTION" },
	{ 0x00018A, "ROI COMPUTER AG", "ROI" },
	{ 0x00018B, "NetLinks Co., Ltd.", "NETLINKS" },
	{ 0x00018C, "Mega Vision", "MEGA-VISION" },
	{ 0x00018D, "AudeSi Technologies", "AUDESI-TECHNOLOGIES" },
	{ 0x00018E, "Logitec Corporation", "LOGITEC" },
	{ 0x00018F, "Kenetec, Inc.", "KENETEC" },
	{ 0x000190, "SMK-M", "SMK-M" },
	{ 0x000191, "SYRED Data Systems", "SYRED-DATA" },
	{ 0x000192, "Texas Digital Systems", "TEXAS-DIGITAL" },
	{ 0x000193, "Hanbyul Telecom Co., Ltd.", "HANBYUL-TELECOM" },
	{ 0x000194, "Capital Equipment Corporation", "CAPITAL-EQUIPMENT" },
	{ 0x000195, "Sena Technologies, Inc.", "SENA-TECHNOLOGIES" },
	{ 0x000196, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000197, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000198, "Darim Vision", "DARIM-VISION" },
	{ 0x000199, "HeiSei Electronics", "HEISEI-ELECTRONICS" },
	{ 0x00019A, "LEUNIG GmbH", "LEUNIG" },
	{ 0x00019B, "Kyoto Microcomputer Co., Ltd.", "KYOTO-MICROCOMPUTER" },
	{ 0x00019C, "JDS Uniphase Inc.", "JDS-UNIPHASE" },
	{ 0x00019D, "E-Control Systems, Inc.", "E-CONTROL" },
	{ 0x00019E, "ESS Technology, Inc.", "ESS" },
	{ 0x00019F, "ReadyNet", "READYNET" },
	{ 0x0001A0, "Infinilink Corporation", "INFINILINK" },
	{ 0x0001A1, "Mag-Tek, Inc.", "MAG-TEK" },
	{ 0x0001A2, "Logical Co., Ltd.", "LOGICAL" },
	{ 0x0001A3, "GENESYS LOGIC, INC.", "GENESYS-LOGIC" },
	{ 0x0001A4, "Microlink Corporation", "MICROLINK" },
	{ 0x0001A5, "Nextcomm, Inc.", "NEXTCOMM" },
	{ 0x0001A6, "Scientific-Atlanta Arcodan A/S", "SCIENTIFIC-ATLANTA-ARCODAN-A/S" },
	{ 0x0001A7, "UNEX TECHNOLOGY CORPORATION", "UNEX" },
	{ 0x0001A8, "Welltech Computer Co., Ltd.", "WELLTECH" },
	{ 0x0001A9, "BMW AG", "BMW" },
	{ 0x0001AA, "Airspan Communications, Ltd.", "AIRSPAN-COMMUNICATION" },
	{ 0x0001AB, "Main Street Networks", "MAIN-STREET-NETWORKS" },
	{ 0x0001AC, "Sitara Networks, Inc.", "SITARA-NETWORKS" },
	{ 0x0001AD, "Coach Master International d.b.a. CMI Worldwide, Inc.", "COACH-MASTER-D-B-A-CMI-WORLDWIDE" },
	{ 0x0001AE, "Trex Enterprises", "TREX-ENTERPRISES" },
	{ 0x0001AF, "Artesyn Embedded Technologies", "ARTESYN-EMBEDDED-TECHNOLOGIES" },
	{ 0x0001B0, "Fulltek Technology Co., Ltd.", "FULLTEK" },
	{ 0x0001B1, "General Bandwidth", "GENERAL-BANDWIDTH" },
	{ 0x0001B2, "Digital Processing Systems, Inc.", "DIGITAL-PROCESSING" },
	{ 0x0001B3, "Precision Electronic Manufacturing", "PRECISION-MANUFACTURING" },
	{ 0x0001B4, "Wayport, Inc.", "WAYPORT" },
	{ 0x0001B5, "Turin Networks, Inc.", "TURIN-NETWORKS" },
	{ 0x0001B6, "SAEJIN T&M Co., Ltd.", "SAEJIN-T-M" },
	{ 0x0001B7, "Centos, Inc.", "CENTOS" },
	{ 0x0001B8, "Netsensity, Inc.", "NETSENSITY" },
	{ 0x0001B9, "SKF Condition Monitoring", "SKF-CONDITION-MONITORING" },
	{ 0x0001BA, "IC-Net, Inc.", "IC-NET" },
	{ 0x0001BB, "Frequentis", "FREQUENTIS" },
	{ 0x0001BC, "Brains Corporation", "BRAINS" },
	{ 0x0001BD, "Peterson Electro-Musical Products, Inc.", "PETERSON-ELECTRO-MUSICAL-PRODUCTS" },
	{ 0x0001BE, "Gigalink Co., Ltd.", "GIGALINK" },
	{ 0x0001BF, "Teleforce Co., Ltd.", "TELEFORCE" },
	{ 0x0001C0, "CompuLab, Ltd.", "COMPULAB" },
	{ 0x0001C1, "Vitesse Semiconductor Corporation", "VITESSE-SEMICONDUCTOR" },
	{ 0x0001C2, "ARK Research Corp.", "ARK-RESEARCH" },
	{ 0x0001C3, "Acromag, Inc.", "ACROMAG" },
	{ 0x0001C4, "NeoWave, Inc.", "NEOWAVE" },
	{ 0x0001C5, "Simpler Networks", "SIMPLER-NETWORKS" },
	{ 0x0001C6, "Quarry Technologies", "QUARRY-TECHNOLOGIES" },
	{ 0x0001C7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0001C8, "THOMAS CONRAD CORP.", "THOMAS-CONRAD" },
	{ 0x0001C8, "CONRAD CORP.", "CONRAD" },
	{ 0x0001C9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0001CA, "Geocast Network Systems, Inc.", "GEOCAST-NETWORK" },
	{ 0x0001CB, "EVR", "EVR" },
	{ 0x0001CC, "Japan Total Design Communication Co., Ltd.", "JAPAN-TOTAL-DESIGN-COMMUNICATION" },
	{ 0x0001CD, "ARtem", "ARTEM" },
	{ 0x0001CE, "Custom Micro Products, Ltd.", "CUSTOM-MICRO-PRODUCTS" },
	{ 0x0001CF, "Alpha Data Parallel Systems, Ltd.", "ALPHA-DATA-PARALLEL" },
	{ 0x0001D0, "VitalPoint, Inc.", "VITALPOINT" },
	{ 0x0001D1, "CoNet Communications, Inc.", "CONET-COMMUNICATION" },
	{ 0x0001D2, "inXtron, Inc.", "INXTRON" },
	{ 0x0001D3, "PAXCOMM, Inc.", "PAXCOMM" },
	{ 0x0001D4, "Leisure Time, Inc.", "LEISURE-TIME" },
	{ 0x0001D5, "HAEDONG INFO & COMM CO., LTD", "HAEDONG-INFO-COMM" },
	{ 0x0001D6, "manroland AG", "MANROLAND" },
	{ 0x0001D7, "F5 Networks, Inc.", "F5-NETWORKS" },
	{ 0x0001D8, "Teltronics, Inc.", "TELTRONICS" },
	{ 0x0001D9, "Sigma, Inc.", "SIGMA" },
	{ 0x0001DA, "WINCOMM Corporation", "WINCOMM" },
	{ 0x0001DB, "Freecom Technologies GmbH", "FREECOM-TECHNOLOGIES" },
	{ 0x0001DC, "Activetelco", "ACTIVETELCO" },
	{ 0x0001DD, "Avail Networks", "AVAIL-NETWORKS" },
	{ 0x0001DE, "Trango Systems, Inc.", "TRANGO" },
	{ 0x0001DF, "ISDN Communications, Ltd.", "ISDN-COMMUNICATION" },
	{ 0x0001E0, "Fast Systems, Inc.", "FAST" },
	{ 0x0001E1, "Kinpo Electronics, Inc.", "KINPO-ELECTRONICS" },
	{ 0x0001E2, "Ando Electric Corporation", "ANDO-ELECTRIC" },
	{ 0x0001E3, "Siemens AG", "SIEMENS" },
	{ 0x0001E4, "Sitera, Inc.", "SITERA" },
	{ 0x0001E5, "Supernet, Inc.", "SUPERNET" },
	{ 0x0001E6, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0001E7, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0001E8, "Force10 Networks, Inc.", "FORCE10-NETWORKS" },
	{ 0x0001E9, "Litton Marine Systems B.V.", "LITTON-MARINE" },
	{ 0x0001EA, "Cirilium Corp.", "CIRILIUM" },
	{ 0x0001EB, "C-COM Corporation", "C-COM" },
	{ 0x0001EC, "Ericsson Group", "ERICSSON-GROUP" },
	{ 0x0001ED, "SETA Corp.", "SETA" },
	{ 0x0001EE, "Comtrol Europe, Ltd.", "COMTROL-EUROPE" },
	{ 0x0001EF, "Camtel Technology Corp.", "CAMTEL" },
	{ 0x0001F0, "Tridium, Inc.", "TRIDIUM" },
	{ 0x0001F1, "Innovative Concepts, Inc.", "INNOVATIVE-CONCEPTS" },
	{ 0x0001F2, "Mark of the Unicorn, Inc.", "MARK-OF-THE-UNICORN" },
	{ 0x0001F3, "QPS, Inc.", "QPS" },
	{ 0x0001F4, "Enterasys Networks", "ENTERASYS-NETWORKS" },
	{ 0x0001F5, "ERIM S.A.", "ERIM" },
	{ 0x0001F6, "Association of Musical Electronics Industry", "ASSOCIATION-OF-MUSICAL-ELECTRONICS-INDUSTRY" },
	{ 0x0001F7, "Image Display Systems, Inc.", "IMAGE-DISPLAY" },
	{ 0x0001F8, "Texio Technology Corporation", "TEXIO" },
	{ 0x0001F9, "TeraGlobal Communications Corp.", "TERAGLOBAL-COMMUNICATION" },
	{ 0x0001FA, "HOROSCAS", "HOROSCAS" },
	{ 0x0001FB, "DoTop Technology, Inc.", "DOTOP" },
	{ 0x0001FC, "Keyence Corporation", "KEYENCE" },
	{ 0x0001FD, "Digital Voice Systems, Inc.", "DIGITAL-VOICE" },
	{ 0x0001FE, "DIGITAL EQUIPMENT CORPORATION", "DIGITAL-EQUIPMENT" },
	{ 0x0001FF, "Data Direct Networks, Inc.", "DATA-DIRECT-NETWORKS" },
	{ 0x000200, "Net & Sys Co., Ltd.", "NET-SYS" },
	{ 0x000201, "IFM Electronic gmbh", "IFM" },
	{ 0x000202, "Amino Communications, Ltd.", "AMINO-COMMUNICATION" },
	{ 0x000203, "Woonsang Telecom, Inc.", "WOONSANG-TELECOM" },
	{ 0x000204, "Bodmann Industries Elektronik GmbH", "BODMANN-INDUSTRIES-ELEKTRONIK" },
	{ 0x000205, "Hitachi Denshi, Ltd.", "HITACHI-DENSHI" },
	{ 0x000206, "Telital R&D Denmark A/S", "TELITAL-R-D-DENMARK-A/S" },
	{ 0x000207, "VisionGlobal Network Corp.", "VISIONGLOBAL-NETWORK" },
	{ 0x000208, "Unify Networks, Inc.", "UNIFY-NETWORKS" },
	{ 0x000209, "Shenzhen SED Information Technology Co., Ltd.", "SHENZHEN-SED-INFORMATION" },
	{ 0x00020A, "Gefran Spa", "GEFRAN-SPA" },
	{ 0x00020B, "Native Networks, Inc.", "NATIVE-NETWORKS" },
	{ 0x00020C, "Metro-Optix", "METRO-OPTIX" },
	{ 0x00020D, "Micronpc.com", "MICRONPC-COM" },
	{ 0x00020E, "ECI Telecom, Ltd", "ECI-TELECOM" },
	{ 0x00020F, "AATR", "AATR" },
	{ 0x000210, "Fenecom", "FENECOM" },
	{ 0x000211, "Nature Worldwide Technology Corp.", "NATURE-WORLDWIDE" },
	{ 0x000212, "SierraCom", "SIERRACOM" },
	{ 0x000213, "S.D.E.L.", "S-D-E-L" },
	{ 0x000214, "DTVRO", "DTVRO" },
	{ 0x000215, "Cotas Computer Technology A/B", "COTAS-A/B" },
	{ 0x000216, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000217, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000218, "Advanced Scientific Corp", "ADVANCED-SCIENTIFIC" },
	{ 0x000219, "Paralon Technologies", "PARALON-TECHNOLOGIES" },
	{ 0x00021A, "Zuma Networks", "ZUMA-NETWORKS" },
	{ 0x00021B, "Kollmorgen-Servotronix", "KOLLMORGEN-SERVOTRONIX" },
	{ 0x00021C, "Network Elements, Inc.", "NETWORK-ELEMENTS" },
	{ 0x00021D, "Data General Communication Ltd.", "DATA-GENERAL-COMMUNICATION" },
	{ 0x00021E, "SIMTEL S.R.L.", "SIMTEL-S-R-L" },
	{ 0x00021F, "Aculab PLC", "ACULAB-PLC" },
	{ 0x000220, "CANON FINETECH INC.", "CANON-FINETECH" },
	{ 0x000221, "DSP Application, Ltd.", "DSP-APPLICATION" },
	{ 0x000222, "Chromisys, Inc.", "CHROMISYS" },
	{ 0x000223, "ClickTV", "CLICKTV" },
	{ 0x000224, "C-COR", "C-COR" },
	{ 0x000225, "One Stop Systems", "ONE-STOP" },
	{ 0x000226, "XESystems, Inc.", "XESYSTEMS" },
	{ 0x000227, "ESD Electronic System Design GmbH", "ESD-SYSTEM-DESIGN" },
	{ 0x000228, "Necsom, Ltd.", "NECSOM" },
	{ 0x000229, "Adtec Corporation", "ADTEC" },
	{ 0x00022A, "Asound Electronic", "ASOUND" },
	{ 0x00022B, "SAXA, Inc.", "SAXA" },
	{ 0x00022C, "ABB Bomem, Inc.", "ABB-BOMEM" },
	{ 0x00022D, "Agere Systems", "AGERE" },
	{ 0x00022E, "TEAC Corp. R& D", "TEAC-R-D" },
	{ 0x00022F, "P-Cube, Ltd.", "P-CUBE" },
	{ 0x000230, "Intersoft Electronics", "INTERSOFT-ELECTRONICS" },
	{ 0x000231, "Ingersoll-Rand", "INGERSOLL-RAND" },
	{ 0x000232, "Avision, Inc.", "AVISION" },
	{ 0x000233, "Mantra Communications, Inc.", "MANTRA-COMMUNICATION" },
	{ 0x000234, "Imperial Technology, Inc.", "IMPERIAL" },
	{ 0x000235, "Paragon Networks International", "PARAGON-NETWORKS" },
	{ 0x000236, "INIT GmbH", "INIT" },
	{ 0x000237, "Cosmo Research Corp.", "COSMO-RESEARCH" },
	{ 0x000238, "Serome Technology, Inc.", "SEROME" },
	{ 0x000239, "Visicom", "VISICOM" },
	{ 0x00023A, "ZSK Stickmaschinen GmbH", "ZSK-STICKMASCHINEN" },
	{ 0x00023B, "Ericsson", "ERICSSON" },
	{ 0x00023C, "Creative Technology, Ltd.", "CREATIVE" },
	{ 0x00023D, "Cisco Systems, Inc.", "CISCO" },
	{ 0x00023E, "Selta Telematica S.p.a", "SELTA-TELEMATICA-S-P-A" },
	{ 0x00023F, "Compal Electronics, Inc.", "COMPAL-ELECTRONICS" },
	{ 0x000240, "Seedek Co., Ltd.", "SEEDEK" },
	{ 0x000241, "Amer.com", "AMER-COM" },
	{ 0x000242, "Videoframe Systems", "VIDEOFRAME" },
	{ 0x000243, "Raysis Co., Ltd.", "RAYSIS" },
	{ 0x000244, "SURECOM Technology Co.", "SURECOM" },
	{ 0x000245, "Lampus Co, Ltd.", "LAMPUS" },
	{ 0x000246, "All-Win Tech Co., Ltd.", "ALL-WIN-TECH" },
	{ 0x000247, "Great Dragon Information Technology (Group) Co., Ltd.", "GREAT-DRAGON-INFORMATION-GROUP" },
	{ 0x000248, "Pilz GmbH & Co.", "PILZ" },
	{ 0x000249, "Aviv Infocom Co, Ltd.", "AVIV-INFOCOM" },
	{ 0x00024A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00024B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00024C, "SiByte, Inc.", "SIBYTE" },
	{ 0x00024D, "Mannesman Dematic Colby Pty. Ltd.", "MANNESMAN-DEMATIC-COLBY-PTY" },
	{ 0x00024E, "Datacard Group", "DATACARD-GROUP" },
	{ 0x00024F, "IPM Datacom S.R.L.", "IPM-DATACOM-S-R-L" },
	{ 0x000250, "Geyser Networks, Inc.", "GEYSER-NETWORKS" },
	{ 0x000251, "Soma Networks, Inc.", "SOMA-NETWORKS" },
	{ 0x000252, "Carrier Corporation", "CARRIER" },
	{ 0x000253, "Televideo, Inc.", "TELEVIDEO" },
	{ 0x000254, "WorldGate", "WORLDGATE" },
	{ 0x000255, "IBM Corp", "IBM" },
	{ 0x000256, "Alpha Processor, Inc.", "ALPHA-PROCESSOR" },
	{ 0x000257, "Microcom Corp.", "MICROCOM" },
	{ 0x000258, "Flying Packets Communications", "FLYING-PACKETS-COMMUNICATION" },
	{ 0x000259, "Tsann Kuen China (Shanghai)Enterprise Co., Ltd. IT Group", "TSANN-KUEN-CHINA-SHANGHAI-IT-GROUP" },
	{ 0x00025A, "Catena Networks", "CATENA-NETWORKS" },
	{ 0x00025B, "Cambridge Silicon Radio", "CAMBRIDGE-SILICON-RADIO" },
	{ 0x00025C, "SCI Systems (Kunshan) Co., Ltd.", "SCI-KUNSHAN" },
	{ 0x00025D, "Calix Networks", "CALIX-NETWORKS" },
	{ 0x00025E, "High Technology Ltd", "HIGH" },
	{ 0x00025F, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000260, "Accordion Networks, Inc.", "ACCORDION-NETWORKS" },
	{ 0x000261, "Tilgin AB", "TILGIN-AB" },
	{ 0x000262, "Soyo Group Soyo Com Tech Co., Ltd", "SOYO-GROUP-SOYO-COM-TECH" },
	{ 0x000263, "UPS Manufacturing SRL", "UPS-MANUFACTURING-SRL" },
	{ 0x000264, "AudioRamp.com", "AUDIORAMP-COM" },
	{ 0x000265, "Virditech Co. Ltd.", "VIRDITECH" },
	{ 0x000266, "Thermalogic Corporation", "THERMALOGIC" },
	{ 0x000267, "NODE RUNNER, INC.", "NODE-RUNNER" },
	{ 0x000268, "Harris Government Communications", "HARRIS-GOVERNMENT-COMMUNICATION" },
	{ 0x000269, "Nadatel Co., Ltd", "NADATEL" },
	{ 0x00026A, "Cocess Telecom Co., Ltd.", "COCESS-TELECOM" },
	{ 0x00026B, "BCM Computers Co., Ltd.", "BCM-COMPUTERS" },
	{ 0x00026C, "Philips CFT", "PHILIPS-CFT" },
	{ 0x00026D, "Adept Telecom", "ADEPT-TELECOM" },
	{ 0x00026E, "NeGeN Access, Inc.", "NEGEN-ACCESS" },
	{ 0x00026F, "Senao International Co., Ltd.", "SENAO" },
	{ 0x000270, "Crewave Co., Ltd.", "CREWAVE" },
	{ 0x000271, "Zhone Technologies", "ZHONE-TECHNOLOGIES" },
	{ 0x000272, "CC&C Technologies, Inc.", "CC-C-TECHNOLOGIES" },
	{ 0x000273, "Coriolis Networks", "CORIOLIS-NETWORKS" },
	{ 0x000274, "Tommy Technologies Corp.", "TOMMY-TECHNOLOGIES" },
	{ 0x000275, "SMART Technologies, Inc.", "SMART-TECHNOLOGIES" },
	{ 0x000276, "Primax Electronics Ltd.", "PRIMAX-ELECTRONICS" },
	{ 0x000277, "Cash Systemes Industrie", "CASH-SYSTEMES-INDUSTRIE" },
	{ 0x000278, "Samsung Electro-Mechanics Co., Ltd.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x000279, "Control Applications, Ltd.", "CONTROL-APPLICATIONS" },
	{ 0x00027A, "IOI Technology Corporation", "IOI" },
	{ 0x00027B, "Amplify Net, Inc.", "AMPLIFY-NET" },
	{ 0x00027C, "Trilithic, Inc.", "TRILITHIC" },
	{ 0x00027D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00027E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00027F, "ask-technologies.com", "ASK-TECHNOLOGIES-COM" },
	{ 0x000280, "Mu Net, Inc.", "MU-NET" },
	{ 0x000281, "Madge Ltd.", "MADGE" },
	{ 0x000282, "ViaClix, Inc.", "VIACLIX" },
	{ 0x000283, "Spectrum Controls, Inc.", "SPECTRUM-CONTROLS" },
	{ 0x000284, "AREVA T&D", "AREVA-T-D" },
	{ 0x000285, "Riverstone Networks", "RIVERSTONE-NETWORKS" },
	{ 0x000286, "Occam Networks", "OCCAM-NETWORKS" },
	{ 0x000287, "Adapcom", "ADAPCOM" },
	{ 0x000288, "GLOBAL VILLAGE COMMUNICATION", "GLOBAL-VILLAGE-COMMUNICATION" },
	{ 0x000289, "DNE Technologies", "DNE-TECHNOLOGIES" },
	{ 0x00028A, "Ambit Microsystems Corporation", "AMBIT-MICROSYSTEMS" },
	{ 0x00028B, "VDSL Systems OY", "VDSL-OY" },
	{ 0x00028C, "Micrel-Synergy Semiconductor", "MICREL-SYNERGY-SEMICONDUCTOR" },
	{ 0x00028D, "Movita Technologies, Inc.", "MOVITA-TECHNOLOGIES" },
	{ 0x00028E, "Rapid 5 Networks, Inc.", "RAPID-5-NETWORKS" },
	{ 0x00028F, "Globetek, Inc.", "GLOBETEK" },
	{ 0x000290, "Woorigisool, Inc.", "WOORIGISOOL" },
	{ 0x000291, "Open Network Co., Ltd.", "OPEN-NETWORK" },
	{ 0x000292, "Logic Innovations, Inc.", "LOGIC-INNOVATIONS" },
	{ 0x000293, "Solid Data Systems", "SOLID-DATA" },
	{ 0x000294, "Tokyo Sokushin Co., Ltd.", "TOKYO-SOKUSHIN" },
	{ 0x000295, "IP.Access Limited", "IP-ACCESS" },
	{ 0x000296, "Lectron Co,. Ltd.", "LECTRON" },
	{ 0x000297, "C-COR.net", "C-COR-NET" },
	{ 0x000298, "Broadframe Corporation", "BROADFRAME" },
	{ 0x000299, "Apex, Inc.", "APEX" },
	{ 0x00029A, "Storage Apps", "STORAGE-APPS" },
	{ 0x00029B, "Kreatel Communications AB", "KREATEL-COMMUNICATION-AB" },
	{ 0x00029C, "3COM", "3COM" },
	{ 0x00029D, "Merix Corp.", "MERIX" },
	{ 0x00029E, "Information Equipment Co., Ltd.", "INFORMATION-EQUIPMENT" },
	{ 0x00029F, "L-3 Communication Aviation Recorders", "L-3-COMMUNICATION-AVIATION-RECORDERS" },
	{ 0x0002A0, "Flatstack Ltd.", "FLATSTACK" },
	{ 0x0002A1, "World Wide Packets", "WORLD-WIDE-PACKETS" },
	{ 0x0002A2, "Hilscher GmbH", "HILSCHER" },
	{ 0x0002A3, "ABB Switzerland Ltd, Power Systems", "ABB-SWITZERLAND-POWER" },
	{ 0x0002A4, "AddPac Technology Co., Ltd.", "ADDPAC" },
	{ 0x0002A5, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0002A6, "Effinet Systems Co., Ltd.", "EFFINET" },
	{ 0x0002A7, "Vivace Networks", "VIVACE-NETWORKS" },
	{ 0x0002A8, "Air Link Technology", "AIR-LINK" },
	{ 0x0002A9, "RACOM, s.r.o.", "RACOM-S-R-O" },
	{ 0x0002AA, "PLcom Co., Ltd.", "PLCOM" },
	{ 0x0002AB, "CTC Union Technologies Co., Ltd.", "CTC-UNION-TECHNOLOGIES" },
	{ 0x0002AC, "3PAR data", "3PAR-DATA" },
	{ 0x0002AD, "HOYA Corporation", "HOYA" },
	{ 0x0002AE, "Scannex Electronics Ltd.", "SCANNEX-ELECTRONICS" },
	{ 0x0002AF, "TeleCruz Technology, Inc.", "TELECRUZ" },
	{ 0x0002B0, "Hokubu Communication & Industrial Co., Ltd.", "HOKUBU-COMMUNICATION-INDUSTRIAL" },
	{ 0x0002B1, "Anritsu, Ltd.", "ANRITSU" },
	{ 0x0002B2, "Cablevision", "CABLEVISION" },
	{ 0x0002B3, "Intel Corporation", "INTEL" },
	{ 0x0002B4, "DAPHNE", "DAPHNE" },
	{ 0x0002B5, "Avnet, Inc.", "AVNET" },
	{ 0x0002B6, "Acrosser Technology Co., Ltd.", "ACROSSER" },
	{ 0x0002B7, "Watanabe Electric Industry Co., Ltd.", "WATANABE-ELECTRIC-INDUSTRY" },
	{ 0x0002B8, "WHI KONSULT AB", "WHI-KONSULT-AB" },
	{ 0x0002B9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0002BA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0002BB, "Continuous Computing Corp", "CONTINUOUS-COMPUTING" },
	{ 0x0002BC, "LVL 7 Systems, Inc.", "LVL-7" },
	{ 0x0002BD, "Bionet Co., Ltd.", "BIONET" },
	{ 0x0002BE, "Totsu Engineering, Inc.", "TOTSU-ENGINEERING" },
	{ 0x0002BF, "dotRocket, Inc.", "DOTROCKET" },
	{ 0x0002C0, "Bencent Tzeng Industry Co., Ltd.", "BENCENT-TZENG-INDUSTRY" },
	{ 0x0002C1, "Innovative Electronic Designs, Inc.", "INNOVATIVE-DESIGNS" },
	{ 0x0002C2, "Net Vision Telecom", "NET-VISION-TELECOM" },
	{ 0x0002C3, "Arelnet Ltd.", "ARELNET" },
	{ 0x0002C4, "Vector International BVBA", "VECTOR-BVBA" },
	{ 0x0002C5, "Evertz Microsystems Ltd.", "EVERTZ-MICROSYSTEMS" },
	{ 0x0002C6, "Data Track Technology PLC", "DATA-TRACK-PLC" },
	{ 0x0002C7, "ALPS ELECTRIC Co., Ltd.", "ALPS-ELECTRIC" },
	{ 0x0002C8, "Technocom Communications Technology (pte) Ltd", "TECHNOCOM-COMMUNICATION-PTE" },
	{ 0x0002C9, "Mellanox Technologies", "MELLANOX-TECHNOLOGIES" },
	{ 0x0002CA, "EndPoints, Inc.", "ENDPOINTS" },
	{ 0x0002CB, "TriState Ltd.", "TRISTATE" },
	{ 0x0002CC, "M.C.C.I", "M-C-C-I" },
	{ 0x0002CD, "TeleDream, Inc.", "TELEDREAM" },
	{ 0x0002CE, "FoxJet, Inc.", "FOXJET" },
	{ 0x0002CF, "ZyGate Communications, Inc.", "ZYGATE-COMMUNICATION" },
	{ 0x0002D0, "Comdial Corporation", "COMDIAL" },
	{ 0x0002D1, "Vivotek, Inc.", "VIVOTEK" },
	{ 0x0002D2, "Workstation AG", "WORKSTATION" },
	{ 0x0002D3, "NetBotz, Inc.", "NETBOTZ" },
	{ 0x0002D4, "PDA Peripherals, Inc.", "PDA-PERIPHERALS" },
	{ 0x0002D5, "ACR", "ACR" },
	{ 0x0002D6, "NICE Systems", "NICE" },
	{ 0x0002D7, "EMPEG Ltd", "EMPEG" },
	{ 0x0002D8, "BRECIS Communications Corporation", "BRECIS-COMMUNICATION" },
	{ 0x0002D9, "Reliable Controls", "RELIABLE-CONTROLS" },
	{ 0x0002DA, "ExiO Communications, Inc.", "EXIO-COMMUNICATION" },
	{ 0x0002DB, "NETSEC", "NETSEC" },
	{ 0x0002DC, "Fujitsu General Limited", "FUJITSU-GENERAL" },
	{ 0x0002DD, "Bromax Communications, Ltd.", "BROMAX-COMMUNICATION" },
	{ 0x0002DE, "Astrodesign, Inc.", "ASTRODESIGN" },
	{ 0x0002DF, "Net Com Systems, Inc.", "NET-COM" },
	{ 0x0002E0, "ETAS GmbH", "ETAS" },
	{ 0x0002E1, "Integrated Network Corporation", "INTEGRATED-NETWORK" },
	{ 0x0002E2, "NDC Infared Engineering", "NDC-INFARED-ENGINEERING" },
	{ 0x0002E3, "LITE-ON Communications, Inc.", "LITE-ON-COMMUNICATION" },
	{ 0x0002E4, "JC HYUN Systems, Inc.", "JC-HYUN" },
	{ 0x0002E5, "Timeware Ltd.", "TIMEWARE" },
	{ 0x0002E6, "Gould Instrument Systems, Inc.", "GOULD-INSTRUMENT" },
	{ 0x0002E7, "CAB GmbH & Co KG", "CAB" },
	{ 0x0002E8, "E.D.&A.", "E-D-A" },
	{ 0x0002E9, "CS Systemes De Securite - C3S", "CS-SYSTEMES-DE-SECURITE---C3S" },
	{ 0x0002EA, "Focus Enhancements", "FOCUS-ENHANCEMENTS" },
	{ 0x0002EB, "Pico Communications", "PICO-COMMUNICATION" },
	{ 0x0002EC, "Maschoff Design Engineering", "MASCHOFF-DESIGN-ENGINEERING" },
	{ 0x0002ED, "DXO Telecom Co., Ltd.", "DXO-TELECOM" },
	{ 0x0002EE, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0002EF, "CCC Network Systems Group Ltd.", "CCC-NETWORK-GROUP" },
	{ 0x0002F0, "AME Optimedia Technology Co., Ltd.", "AME-OPTIMEDIA" },
	{ 0x0002F1, "Pinetron Co., Ltd.", "PINETRON" },
	{ 0x0002F2, "eDevice, Inc.", "EDEVICE" },
	{ 0x0002F3, "Media Serve Co., Ltd.", "MEDIA-SERVE" },
	{ 0x0002F4, "PCTEL, Inc.", "PCTEL" },
	{ 0x0002F5, "VIVE Synergies, Inc.", "VIVE-SYNERGIES" },
	{ 0x0002F6, "Equipe Communications", "EQUIPE-COMMUNICATION" },
	{ 0x0002F7, "ARM", "ARM" },
	{ 0x0002F8, "SEAKR Engineering, Inc.", "SEAKR-ENGINEERING" },
	{ 0x0002F9, "MIMOS Berhad", "MIMOS-BERHAD" },
	{ 0x0002FA, "DX Antenna Co., Ltd.", "DX-ANTENNA" },
	{ 0x0002FB, "Baumuller Aulugen-Systemtechnik GmbH", "BAUMULLER-AULUGEN-SYSTEMTECHNIK" },
	{ 0x0002FC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0002FD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0002FE, "Viditec, Inc.", "VIDITEC" },
	{ 0x0002FF, "Handan BroadInfoCom", "HANDAN-BROADINFOCOM" },
	{ 0x000300, "Barracuda Networks, Inc.", "BARRACUDA-NETWORKS" },
	{ 0x000301, "EXFO", "EXFO" },
	{ 0x000302, "Charles Industries, Ltd.", "CHARLES-INDUSTRIES" },
	{ 0x000303, "JAMA Electronics Co., Ltd.", "JAMA-ELECTRONICS" },
	{ 0x000304, "Pacific Broadband Communications", "PACIFIC-BROADBAND-COMMUNICATION" },
	{ 0x000305, "MSC Vertriebs GmbH", "MSC-VERTRIEBS" },
	{ 0x000306, "Fusion In Tech Co., Ltd.", "FUSION-IN-TECH" },
	{ 0x000307, "Secure Works, Inc.", "SECURE-WORKS" },
	{ 0x000308, "AM Communications, Inc.", "AM-COMMUNICATION" },
	{ 0x000309, "Texcel Technology PLC", "TEXCEL-PLC" },
	{ 0x00030A, "Argus Technologies", "ARGUS-TECHNOLOGIES" },
	{ 0x00030B, "Hunter Technology, Inc.", "HUNTER" },
	{ 0x00030C, "Telesoft Technologies Ltd.", "TELESOFT-TECHNOLOGIES" },
	{ 0x00030D, "Uniwill Computer Corp.", "UNIWILL" },
	{ 0x00030E, "Core Communications Co., Ltd.", "CORE-COMMUNICATION" },
	{ 0x00030F, "Digital China (Shanghai) Networks Ltd.", "DIGITAL-CHINA-SHANGHAI-NETWORKS" },
	{ 0x000310, "E-Globaledge Corporation", "E-GLOBALEDGE" },
	{ 0x000311, "Micro Technology Co., Ltd.", "MICRO" },
	{ 0x000312, "TR-Systemtechnik GmbH", "TR-SYSTEMTECHNIK" },
	{ 0x000313, "Access Media SPA", "ACCESS-MEDIA-SPA" },
	{ 0x000314, "Teleware Network Systems", "TELEWARE-NETWORK" },
	{ 0x000315, "Cidco Incorporated", "CIDCO-INCORPORATED" },
	{ 0x000316, "Nobell Communications, Inc.", "NOBELL-COMMUNICATION" },
	{ 0x000317, "Merlin Systems, Inc.", "MERLIN" },
	{ 0x000318, "Cyras Systems, Inc.", "CYRAS" },
	{ 0x000319, "Infineon AG", "INFINEON" },
	{ 0x00031A, "Beijing Broad Telecom Ltd., China", "BEIJING-BROAD-TELECOM-CHINA" },
	{ 0x00031B, "Cellvision Systems, Inc.", "CELLVISION" },
	{ 0x00031C, "Svenska Hardvarufabriken AB", "SVENSKA-HARDVARUFABRIKEN-AB" },
	{ 0x00031D, "Taiwan Commate Computer, Inc.", "TAIWAN-COMMATE" },
	{ 0x00031E, "Optranet, Inc.", "OPTRANET" },
	{ 0x00031F, "Condev Ltd.", "CONDEV" },
	{ 0x000320, "Xpeed, Inc.", "XPEED" },
	{ 0x000321, "Reco Research Co., Ltd.", "RECO-RESEARCH" },
	{ 0x000322, "IDIS Co., Ltd.", "IDIS" },
	{ 0x000323, "Cornet Technology, Inc.", "CORNET" },
	{ 0x000324, "SANYO Consumer Electronics Co., Ltd.", "SANYO-CONSUMER-ELECTRONICS" },
	{ 0x000325, "Arima Computer Corp.", "ARIMA" },
	{ 0x000326, "Iwasaki Information Systems Co., Ltd.", "IWASAKI-INFORMATION" },
	{ 0x000327, "ACT'L", "ACT-L" },
	{ 0x000328, "Mace Group, Inc.", "MACE-GROUP" },
	{ 0x000329, "F3, Inc.", "F3" },
	{ 0x00032A, "UniData Communication Systems, Inc.", "UNIDATA-COMMUNICATION" },
	{ 0x00032B, "GAI Datenfunksysteme GmbH", "GAI-DATENFUNKSYSTEME" },
	{ 0x00032C, "ABB Switzerland Ltd", "ABB-SWITZERLAND" },
	{ 0x00032D, "IBASE Technology, Inc.", "IBASE" },
	{ 0x00032E, "Scope Information Management, Ltd.", "SCOPE-INFORMATION-MANAGEMENT" },
	{ 0x00032F, "Global Sun Technology, Inc.", "GLOBAL-SUN" },
	{ 0x000330, "Imagenics, Co., Ltd.", "IMAGENICS" },
	{ 0x000331, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000332, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000333, "Digitel Co., Ltd.", "DIGITEL" },
	{ 0x000334, "Newport Electronics", "NEWPORT-ELECTRONICS" },
	{ 0x000335, "Mirae Technology", "MIRAE" },
	{ 0x000336, "Zetes Technologies", "ZETES-TECHNOLOGIES" },
	{ 0x000337, "Vaone, Inc.", "VAONE" },
	{ 0x000338, "Oak Technology", "OAK" },
	{ 0x000339, "Eurologic Systems, Ltd.", "EUROLOGIC" },
	{ 0x00033A, "Silicon Wave, Inc.", "SILICON-WAVE" },
	{ 0x00033B, "TAMI Tech Co., Ltd.", "TAMI-TECH" },
	{ 0x00033C, "Daiden Co., Ltd.", "DAIDEN" },
	{ 0x00033D, "ILSHin Lab", "ILSHIN-LAB" },
	{ 0x00033E, "Tateyama System Laboratory Co., Ltd.", "TATEYAMA-SYSTEM-LABORATORY" },
	{ 0x00033F, "BigBand Networks, Ltd.", "BIGBAND-NETWORKS" },
	{ 0x000340, "Floware Wireless Systems, Ltd.", "FLOWARE-WIRELESS" },
	{ 0x000341, "Axon Digital Design", "AXON-DIGITAL-DESIGN" },
	{ 0x000342, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000343, "Martin Professional A/S", "MARTIN-PROFESSIONAL-A/S" },
	{ 0x000344, "Tietech.Co., Ltd.", "TIETECH" },
	{ 0x000345, "Routrek Networks Corporation", "ROUTREK-NETWORKS" },
	{ 0x000346, "Hitachi Kokusai Electric, Inc.", "HITACHI-KOKUSAI-ELECTRIC" },
	{ 0x000347, "Intel Corporation", "INTEL" },
	{ 0x000348, "Norscan Instruments, Ltd.", "NORSCAN-INSTRUMENTS" },
	{ 0x000349, "Vidicode Datacommunicatie B.V.", "VIDICODE-DATACOMMUNICATIE" },
	{ 0x00034A, "RIAS Corporation", "RIAS" },
	{ 0x00034B, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x00034C, "Shanghai DigiVision Technology Co., Ltd.", "SHANGHAI-DIGIVISION" },
	{ 0x00034D, "Chiaro Networks, Ltd.", "CHIARO-NETWORKS" },
	{ 0x00034E, "Pos Data Company, Ltd.", "POS-DATA-COMPANY" },
	{ 0x00034F, "Sur-Gard Security", "SUR-GARD-SECURITY" },
	{ 0x000350, "BTICINO SPA", "BTICINO-SPA" },
	{ 0x000351, "Diebold, Inc.", "DIEBOLD" },
	{ 0x000352, "Colubris Networks", "COLUBRIS-NETWORKS" },
	{ 0x000353, "Mitac, Inc.", "MITAC" },
	{ 0x000354, "Fiber Logic Communications", "FIBER-LOGIC-COMMUNICATION" },
	{ 0x000355, "TeraBeam Internet Systems", "TERABEAM-INTERNET" },
	{ 0x000356, "Wincor Nixdorf International GmbH", "WINCOR-NIXDORF" },
	{ 0x000357, "Intervoice-Brite, Inc.", "INTERVOICE-BRITE" },
	{ 0x000358, "Hanyang Digitech Co., Ltd.", "HANYANG-DIGITECH" },
	{ 0x000359, "DigitalSis", "DIGITALSIS" },
	{ 0x00035A, "Photron Limited", "PHOTRON" },
	{ 0x00035B, "BridgeWave Communications", "BRIDGEWAVE-COMMUNICATION" },
	{ 0x00035C, "Saint Song Corp.", "SAINT-SONG" },
	{ 0x00035D, "Bosung Hi-Net Co., Ltd.", "BOSUNG-HI-NET" },
	{ 0x00035E, "Metropolitan Area Networks, Inc.", "METROPOLITAN-AREA-NETWORKS" },
	{ 0x00035F, "Prüftechnik Condition Monitoring GmbH & Co. KG", "PRüFTECHNIK-CONDITION-MONITORING" },
	{ 0x000360, "PAC Interactive Technology, Inc.", "PAC-INTERACTIVE" },
	{ 0x000361, "Widcomm, Inc.", "WIDCOMM" },
	{ 0x000362, "Vodtel Communications, Inc.", "VODTEL-COMMUNICATION" },
	{ 0x000363, "Miraesys Co., Ltd.", "MIRAESYS" },
	{ 0x000364, "Scenix Semiconductor, Inc.", "SCENIX-SEMICONDUCTOR" },
	{ 0x000365, "Kira Information & Communications, Ltd.", "KIRA-INFORMATION-COMMUNICATION" },
	{ 0x000366, "ASM Pacific Technology", "ASM-PACIFIC" },
	{ 0x000367, "Jasmine Networks, Inc.", "JASMINE-NETWORKS" },
	{ 0x000368, "Embedone Co., Ltd.", "EMBEDONE" },
	{ 0x000369, "Nippon Antenna Co., Ltd.", "NIPPON-ANTENNA" },
	{ 0x00036A, "Mainnet, Ltd.", "MAINNET" },
	{ 0x00036B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00036C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00036D, "Runtop, Inc.", "RUNTOP" },
	{ 0x00036E, "Nicon Systems (Pty) Limited", "NICON-PTY" },
	{ 0x00036F, "Telsey SPA", "TELSEY-SPA" },
	{ 0x000370, "NXTV, Inc.", "NXTV" },
	{ 0x000371, "Acomz Networks Corp.", "ACOMZ-NETWORKS" },
	{ 0x000372, "ULAN", "ULAN" },
	{ 0x000373, "Aselsan A.S", "ASELSAN-A-S" },
	{ 0x000374, "Control Microsystems", "CONTROL-MICROSYSTEMS" },
	{ 0x000375, "NetMedia, Inc.", "NETMEDIA" },
	{ 0x000376, "Graphtec Technology, Inc.", "GRAPHTEC" },
	{ 0x000377, "Gigabit Wireless", "GIGABIT-WIRELESS" },
	{ 0x000378, "HUMAX Co., Ltd.", "HUMAX" },
	{ 0x000379, "Proscend Communications, Inc.", "PROSCEND-COMMUNICATION" },
	{ 0x00037A, "Taiyo Yuden Co., Ltd.", "TAIYO-YUDEN" },
	{ 0x00037B, "IDEC IZUMI Corporation", "IDEC-IZUMI" },
	{ 0x00037C, "Coax Media", "COAX-MEDIA" },
	{ 0x00037D, "Stellcom", "STELLCOM" },
	{ 0x00037E, "PORTech Communications, Inc.", "PORTECH-COMMUNICATION" },
	{ 0x00037F, "Atheros Communications, Inc.", "ATHEROS-COMMUNICATION" },
	{ 0x000380, "SSH Communications Security Corp.", "SSH-COMMUNICATION-SECURITY" },
	{ 0x000381, "Ingenico International", "INGENICO" },
	{ 0x000382, "A-One Co., Ltd.", "A-ONE" },
	{ 0x000383, "Metera Networks, Inc.", "METERA-NETWORKS" },
	{ 0x000384, "AETA", "AETA" },
	{ 0x000385, "Actelis Networks, Inc.", "ACTELIS-NETWORKS" },
	{ 0x000386, "Ho Net, Inc.", "HO-NET" },
	{ 0x000387, "Blaze Network Products", "BLAZE-NETWORK-PRODUCTS" },
	{ 0x000388, "Fastfame Technology Co., Ltd.", "FASTFAME" },
	{ 0x000389, "Plantronics", "PLANTRONICS" },
	{ 0x00038A, "America Online, Inc.", "AMERICA-ONLINE" },
	{ 0x00038B, "PLUS-ONE I&T, Inc.", "PLUS-ONE-I-T" },
	{ 0x00038C, "Total Impact", "TOTAL-IMPACT" },
	{ 0x00038D, "PCS Revenue Control Systems, Inc.", "PCS-REVENUE-CONTROL" },
	{ 0x00038E, "Atoga Systems, Inc.", "ATOGA" },
	{ 0x00038F, "Weinschel Corporation", "WEINSCHEL" },
	{ 0x000390, "Digital Video Communications, Inc.", "DIGITAL-VIDEO-COMMUNICATION" },
	{ 0x000391, "Advanced Digital Broadcast, Ltd.", "ADVANCED-DIGITAL-BROADCAST" },
	{ 0x000392, "Hyundai Teletek Co., Ltd.", "HYUNDAI-TELETEK" },
	{ 0x000393, "Apple", "APPLE" },
	{ 0x000394, "Connect One", "CONNECT-ONE" },
	{ 0x000395, "California Amplifier", "CALIFORNIA-AMPLIFIER" },
	{ 0x000396, "EZ Cast Co., Ltd.", "EZ-CAST" },
	{ 0x000397, "Watchfront Limited", "WATCHFRONT" },
	{ 0x000398, "WISI", "WISI" },
	{ 0x000399, "Dongju Informations & Communications Co., Ltd.", "DONGJU-INFORMATIONS-COMMUNICATION" },
	{ 0x00039A, "SiConnect", "SICONNECT" },
	{ 0x00039B, "NetChip Technology, Inc.", "NETCHIP" },
	{ 0x00039C, "OptiMight Communications, Inc.", "OPTIMIGHT-COMMUNICATION" },
	{ 0x00039D, "Qisda Corporation", "QISDA" },
	{ 0x00039E, "Tera System Co., Ltd.", "TERA-SYSTEM" },
	{ 0x00039F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0003A0, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0003A1, "HIPER Information & Communication, Inc.", "HIPER-INFORMATION-COMMUNICATION" },
	{ 0x0003A2, "Catapult Communications", "CATAPULT-COMMUNICATION" },
	{ 0x0003A3, "MAVIX, Ltd.", "MAVIX" },
	{ 0x0003A4, "Imation Corp.", "IMATION" },
	{ 0x0003A5, "Medea Corporation", "MEDEA" },
	{ 0x0003A6, "Traxit Technology, Inc.", "TRAXIT" },
	{ 0x0003A7, "Unixtar Technology, Inc.", "UNIXTAR" },
	{ 0x0003A8, "IDOT Computers, Inc.", "IDOT-COMPUTERS" },
	{ 0x0003A9, "AXCENT Media AG", "AXCENT-MEDIA" },
	{ 0x0003AA, "Watlow", "WATLOW" },
	{ 0x0003AB, "Bridge Information Systems", "BRIDGE-INFORMATION" },
	{ 0x0003AC, "Fronius Schweissmaschinen", "FRONIUS-SCHWEISSMASCHINEN" },
	{ 0x0003AD, "Emerson Energy Systems AB", "EMERSON-ENERGY-AB" },
	{ 0x0003AE, "Allied Advanced Manufacturing Pte, Ltd.", "ALLIED-ADVANCED-MANUFACTURING-PTE" },
	{ 0x0003AF, "Paragea Communications", "PARAGEA-COMMUNICATION" },
	{ 0x0003B0, "Xsense Technology Corp.", "XSENSE" },
	{ 0x0003B1, "Hospira Inc.", "HOSPIRA" },
	{ 0x0003B2, "Radware", "RADWARE" },
	{ 0x0003B3, "IA Link Systems Co., Ltd.", "IA-LINK" },
	{ 0x0003B4, "Macrotek International Corp.", "MACROTEK" },
	{ 0x0003B5, "Entra Technology Co.", "ENTRA" },
	{ 0x0003B6, "QSI Corporation", "QSI" },
	{ 0x0003B7, "ZACCESS Systems", "ZACCESS" },
	{ 0x0003B8, "NetKit Solutions, LLC", "NETKIT-SOLUTIONS-LLC" },
	{ 0x0003B9, "Hualong Telecom Co., Ltd.", "HUALONG-TELECOM" },
	{ 0x0003BA, "Oracle Corporation", "ORACLE" },
	{ 0x0003BB, "Signal Communications Limited", "SIGNAL-COMMUNICATION" },
	{ 0x0003BC, "COT GmbH", "COT" },
	{ 0x0003BD, "OmniCluster Technologies, Inc.", "OMNICLUSTER-TECHNOLOGIES" },
	{ 0x0003BE, "Netility", "NETILITY" },
	{ 0x0003BF, "Centerpoint Broadband Technologies, Inc.", "CENTERPOINT-BROADBAND-TECHNOLOGIES" },
	{ 0x0003C0, "RFTNC Co., Ltd.", "RFTNC" },
	{ 0x0003C1, "Packet Dynamics Ltd", "PACKET-DYNAMICS" },
	{ 0x0003C2, "Solphone K.K.", "SOLPHONE-K-K" },
	{ 0x0003C3, "Micronik Multimedia", "MICRONIK-MULTIMEDIA" },
	{ 0x0003C4, "Tomra Systems ASA", "TOMRA-ASA" },
	{ 0x0003C5, "Mobotix AG", "MOBOTIX" },
	{ 0x0003C6, "ICUE Systems, Inc.", "ICUE" },
	{ 0x0003C7, "hopf Elektronik GmbH", "HOPF-ELEKTRONIK" },
	{ 0x0003C8, "CML Emergency Services", "CML-EMERGENCY-SERVICES" },
	{ 0x0003C9, "TECOM Co., Ltd.", "TECOM" },
	{ 0x0003CA, "MTS Systems Corp.", "MTS" },
	{ 0x0003CB, "Nippon Systems Development Co., Ltd.", "NIPPON-DEVELOPMENT" },
	{ 0x0003CC, "Momentum Computer, Inc.", "MOMENTUM" },
	{ 0x0003CD, "Clovertech, Inc.", "CLOVERTECH" },
	{ 0x0003CE, "ETEN Technologies, Inc.", "ETEN-TECHNOLOGIES" },
	{ 0x0003CF, "Muxcom, Inc.", "MUXCOM" },
	{ 0x0003D0, "KOANKEISO Co., Ltd.", "KOANKEISO" },
	{ 0x0003D1, "Takaya Corporation", "TAKAYA" },
	{ 0x0003D2, "Crossbeam Systems, Inc.", "CROSSBEAM" },
	{ 0x0003D3, "Internet Energy Systems, Inc.", "INTERNET-ENERGY" },
	{ 0x0003D4, "Alloptic, Inc.", "ALLOPTIC" },
	{ 0x0003D5, "Advanced Communications Co., Ltd.", "ADVANCED-COMMUNICATION" },
	{ 0x0003D6, "RADVision, Ltd.", "RADVISION" },
	{ 0x0003D7, "NextNet Wireless, Inc.", "NEXTNET-WIRELESS" },
	{ 0x0003D8, "iMPath Networks, Inc.", "IMPATH-NETWORKS" },
	{ 0x0003D9, "Secheron SA", "SECHERON" },
	{ 0x0003DA, "Takamisawa Cybernetics Co., Ltd.", "TAKAMISAWA-CYBERNETICS" },
	{ 0x0003DB, "Apogee Electronics Corp.", "APOGEE-ELECTRONICS" },
	{ 0x0003DC, "Lexar Media, Inc.", "LEXAR-MEDIA" },
	{ 0x0003DD, "Comark Corp.", "COMARK" },
	{ 0x0003DE, "OTC Wireless", "OTC-WIRELESS" },
	{ 0x0003DF, "Desana Systems", "DESANA" },
	{ 0x0003E0, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0003E1, "Winmate Communication, Inc.", "WINMATE-COMMUNICATION" },
	{ 0x0003E2, "Comspace Corporation", "COMSPACE" },
	{ 0x0003E3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0003E4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0003E5, "Hermstedt SG", "HERMSTEDT-SG" },
	{ 0x0003E6, "Entone, Inc.", "ENTONE" },
	{ 0x0003E7, "Logostek Co. Ltd.", "LOGOSTEK" },
	{ 0x0003E8, "Wavelength Digital Limited", "WAVELENGTH-DIGITAL" },
	{ 0x0003E9, "Akara Canada, Inc.", "AKARA-CANADA" },
	{ 0x0003EA, "Mega System Technologies, Inc.", "MEGA-SYSTEM-TECHNOLOGIES" },
	{ 0x0003EB, "Atrica", "ATRICA" },
	{ 0x0003EC, "ICG Research, Inc.", "ICG-RESEARCH" },
	{ 0x0003ED, "Shinkawa Electric Co., Ltd.", "SHINKAWA-ELECTRIC" },
	{ 0x0003EE, "MKNet Corporation", "MKNET" },
	{ 0x0003EF, "Oneline AG", "ONELINE" },
	{ 0x0003F0, "Redfern Broadband Networks", "REDFERN-BROADBAND-NETWORKS" },
	{ 0x0003F1, "Cicada Semiconductor, Inc.", "CICADA-SEMICONDUCTOR" },
	{ 0x0003F2, "Seneca Networks", "SENECA-NETWORKS" },
	{ 0x0003F3, "Dazzle Multimedia, Inc.", "DAZZLE-MULTIMEDIA" },
	{ 0x0003F4, "NetBurner", "NETBURNER" },
	{ 0x0003F5, "Chip2Chip", "CHIP2CHIP" },
	{ 0x0003F6, "Allegro Networks, Inc.", "ALLEGRO-NETWORKS" },
	{ 0x0003F7, "Plast-Control GmbH", "PLAST-CONTROL" },
	{ 0x0003F8, "SanCastle Technologies, Inc.", "SANCASTLE-TECHNOLOGIES" },
	{ 0x0003F9, "Pleiades Communications, Inc.", "PLEIADES-COMMUNICATION" },
	{ 0x0003FA, "TiMetra Networks", "TIMETRA-NETWORKS" },
	{ 0x0003FB, "ENEGATE Co.,Ltd.", "ENEGATE" },
	{ 0x0003FC, "Intertex Data AB", "INTERTEX-DATA-AB" },
	{ 0x0003FD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0003FE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0003FF, "Microsoft Corporation (possible Hyper-V, Virtual Server, Virtual PC VM)", "MICROSOFT-VIRTUAL" },
	{ 0x000400, "LEXMARK INTERNATIONAL, INC.", "LEXMARK" },
	{ 0x000401, "Osaki Electric Co., Ltd.", "OSAKI-ELECTRIC" },
	{ 0x000402, "Nexsan Technologies, Ltd.", "NEXSAN-TECHNOLOGIES" },
	{ 0x000403, "Nexsi Corporation", "NEXSI" },
	{ 0x000404, "Makino Milling Machine Co., Ltd.", "MAKINO-MILLING-MACHINE" },
	{ 0x000405, "ACN Technologies", "ACN-TECHNOLOGIES" },
	{ 0x000406, "Fa. Metabox AG", "FA-METABOX" },
	{ 0x000407, "Topcon Positioning Systems, Inc.", "TOPCON-POSITIONING" },
	{ 0x000408, "Sanko Electronics Co., Ltd.", "SANKO-ELECTRONICS" },
	{ 0x000409, "Cratos Networks", "CRATOS-NETWORKS" },
	{ 0x00040A, "Sage Systems", "SAGE" },
	{ 0x00040B, "3com Europe Ltd.", "3COM" },
	{ 0x00040C, "Kanno Works, Ltd.", "KANNO-WORKS" },
	{ 0x00040D, "Avaya, Inc.", "AVAYA" },
	{ 0x00040E, "AVM GmbH", "AVM" },
	{ 0x00040F, "Asus Network Technologies, Inc.", "ASUS-NETWORK-TECHNOLOGIES" },
	{ 0x000410, "Spinnaker Networks, Inc.", "SPINNAKER-NETWORKS" },
	{ 0x000411, "Inkra Networks, Inc.", "INKRA-NETWORKS" },
	{ 0x000412, "WaveSmith Networks, Inc.", "WAVESMITH-NETWORKS" },
	{ 0x000413, "SNOM Technology AG", "SNOM" },
	{ 0x000414, "Umezawa Musen Denki Co., Ltd.", "UMEZAWA-MUSEN-DENKI" },
	{ 0x000415, "Rasteme Systems Co., Ltd.", "RASTEME" },
	{ 0x000416, "Parks S/A Comunicacoes Digitais", "PARKS-S/A-COMUNICACOES-DIGITAIS" },
	{ 0x000417, "ELAU AG", "ELAU" },
	{ 0x000418, "Teltronic S.A.U.", "TELTRONIC-U" },
	{ 0x000419, "Fibercycle Networks, Inc.", "FIBERCYCLE-NETWORKS" },
	{ 0x00041A, "Ines Test and Measurement GmbH & CoKG", "INES-TEST-AND-MEASUREMENT-COKG" },
	{ 0x00041B, "Bridgeworks Ltd.", "BRIDGEWORKS" },
	{ 0x00041C, "ipDialog, Inc.", "IPDIALOG" },
	{ 0x00041D, "Corega of America", "COREGA-OF-AMERICA" },
	{ 0x00041E, "Shikoku Instrumentation Co., Ltd.", "SHIKOKU-INSTRUMENTATION" },
	{ 0x00041F, "Sony Computer Entertainment, Inc.", "SONY-ENTERTAINMENT" },
	{ 0x000420, "Slim Devices, Inc.", "SLIM-DEVICES" },
	{ 0x000421, "Ocular Networks", "OCULAR-NETWORKS" },
	{ 0x000422, "Gordon Kapes, Inc.", "GORDON-KAPES" },
	{ 0x000423, "Intel Corporation", "INTEL" },
	{ 0x000424, "TMC s.r.l.", "TMC-S-R-L" },
	{ 0x000425, "Atmel Corporation", "ATMEL" },
	{ 0x000426, "Autosys", "AUTOSYS" },
	{ 0x000427, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000428, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000429, "Pixord Corporation", "PIXORD" },
	{ 0x00042A, "Wireless Networks, Inc.", "WIRELESS-NETWORKS" },
	{ 0x00042B, "IT Access Co., Ltd.", "IT-ACCESS" },
	{ 0x00042C, "Minet, Inc.", "MINET" },
	{ 0x00042D, "Sarian Systems, Ltd.", "SARIAN" },
	{ 0x00042E, "Netous Technologies, Ltd.", "NETOUS-TECHNOLOGIES" },
	{ 0x00042F, "International Communications Products, Inc.", "COMMUNICATION-PRODUCTS" },
	{ 0x000430, "Netgem", "NETGEM" },
	{ 0x000431, "GlobalStreams, Inc.", "GLOBALSTREAMS" },
	{ 0x000432, "Voyetra Turtle Beach, Inc.", "VOYETRA-TURTLE-BEACH" },
	{ 0x000433, "Cyberboard A/S", "CYBERBOARD-A/S" },
	{ 0x000434, "Accelent Systems, Inc.", "ACCELENT" },
	{ 0x000435, "Comptek International, Inc.", "COMPTEK" },
	{ 0x000436, "ELANsat Technologies, Inc.", "ELANSAT-TECHNOLOGIES" },
	{ 0x000437, "Powin Information Technology, Inc.", "POWIN-INFORMATION" },
	{ 0x000438, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000439, "Rosco Entertainment Technology, Inc.", "ROSCO-ENTERTAINMENT" },
	{ 0x00043A, "Intelligent Telecommunications, Inc.", "INTELLIGENT-TELECOMMUNICATIONS" },
	{ 0x00043B, "Lava Computer Mfg., Inc.", "LAVA-MFG" },
	{ 0x00043C, "SONOS Co., Ltd.", "SONOS" },
	{ 0x00043D, "INDEL AG", "INDEL" },
	{ 0x00043E, "Telencomm", "TELENCOMM" },
	{ 0x00043F, "ESTeem Wireless Modems, Inc", "ESTEEM-WIRELESS-MODEMS" },
	{ 0x000440, "cyberPIXIE, Inc.", "CYBERPIXIE" },
	{ 0x000441, "Half Dome Systems, Inc.", "HALF-DOME" },
	{ 0x000442, "NACT", "NACT" },
	{ 0x000443, "Agilent Technologies, Inc.", "AGILENT-TECHNOLOGIES" },
	{ 0x000444, "Western Multiplex Corporation", "WESTERN-MULTIPLEX" },
	{ 0x000445, "LMS Skalar Instruments GmbH", "LMS-SKALAR-INSTRUMENTS" },
	{ 0x000446, "CYZENTECH Co., Ltd.", "CYZENTECH" },
	{ 0x000447, "Acrowave Systems Co., Ltd.", "ACROWAVE" },
	{ 0x000448, "Polaroid Corporation", "POLAROID" },
	{ 0x000449, "Mapletree Networks", "MAPLETREE-NETWORKS" },
	{ 0x00044A, "iPolicy Networks, Inc.", "IPOLICY-NETWORKS" },
	{ 0x00044B, "NVIDIA", "NVIDIA" },
	{ 0x00044C, "JENOPTIK", "JENOPTIK" },
	{ 0x00044D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00044E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00044F, "Leukhardt Systemelektronik GmbH", "LEUKHARDT-SYSTEMELEKTRONIK" },
	{ 0x000450, "DMD Computers SRL", "DMD-COMPUTERS-SRL" },
	{ 0x000451, "Medrad, Inc.", "MEDRAD" },
	{ 0x000452, "RocketLogix, Inc.", "ROCKETLOGIX" },
	{ 0x000453, "YottaYotta, Inc.", "YOTTAYOTTA" },
	{ 0x000454, "Quadriga UK", "QUADRIGA-UK" },
	{ 0x000455, "ANTARA.net", "ANTARA-NET" },
	{ 0x000456, "Cambium Networks Limited", "CAMBIUM-NETWORKS" },
	{ 0x000457, "Universal Access Technology, Inc.", "UNIVERSAL-ACCESS" },
	{ 0x000458, "Fusion X Co., Ltd.", "FUSION-X" },
	{ 0x000459, "Veristar Corporation", "VERISTAR" },
	{ 0x00045A, "The Linksys Group, Inc.", "THE-LINKSYS-GROUP" },
	{ 0x00045B, "Techsan Electronics Co., Ltd.", "TECHSAN-ELECTRONICS" },
	{ 0x00045C, "Mobiwave Pte Ltd", "MOBIWAVE-PTE" },
	{ 0x00045D, "BEKA Elektronik", "BEKA-ELEKTRONIK" },
	{ 0x00045E, "PolyTrax Information Technology AG", "POLYTRAX-INFORMATION" },
	{ 0x00045F, "Avalue Technology, Inc.", "AVALUE" },
	{ 0x000460, "Knilink Technology, Inc.", "KNILINK" },
	{ 0x000461, "EPOX Computer Co., Ltd.", "EPOX" },
	{ 0x000462, "DAKOS Data & Communication Co., Ltd.", "DAKOS-DATA-COMMUNICATION" },
	{ 0x000463, "Bosch Security Systems", "BOSCH-SECURITY" },
	{ 0x000464, "Pulse-Link Inc", "PULSE-LINK" },
	{ 0x000465, "i.s.t isdn-support technik GmbH", "I-S-T-ISDN-SUPPORT-TECHNIK" },
	{ 0x000466, "ARMITEL Co.", "ARMITEL" },
	{ 0x000467, "Wuhan Research Institute of MII", "WUHAN-RESEARCH-INSTITUTE-OF-MII" },
	{ 0x000468, "Vivity, Inc.", "VIVITY" },
	{ 0x000469, "Innocom, Inc.", "INNOCOM" },
	{ 0x00046A, "Navini Networks", "NAVINI-NETWORKS" },
	{ 0x00046B, "Palm Wireless, Inc.", "PALM-WIRELESS" },
	{ 0x00046C, "Cyber Technology Co., Ltd.", "CYBER" },
	{ 0x00046D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00046E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00046F, "Digitel S/A Industria Eletronica", "DIGITEL-S/A-INDUSTRIA-ELETRONICA" },
	{ 0x000470, "ipUnplugged AB", "IPUNPLUGGED-AB" },
	{ 0x000471, "IPrad", "IPRAD" },
	{ 0x000472, "Telelynx, Inc.", "TELELYNX" },
	{ 0x000473, "Photonex Corporation", "PHOTONEX" },
	{ 0x000474, "LEGRAND", "LEGRAND" },
	{ 0x000475, "3 Com Corporation", "3COM" },
	{ 0x000476, "3 Com Corporation", "3COM" },
	{ 0x000477, "Scalant Systems, Inc.", "SCALANT" },
	{ 0x000478, "G. Star Technology Corporation", "G-STAR" },
	{ 0x000479, "Radius Co., Ltd.", "RADIUS" },
	{ 0x00047A, "AXXESSIT ASA", "AXXESSIT-ASA" },
	{ 0x00047B, "Schlumberger", "SCHLUMBERGER" },
	{ 0x00047C, "Skidata AG", "SKIDATA" },
	{ 0x00047D, "Pelco", "PELCO" },
	{ 0x00047E, "Siqura B.V.", "SIQURA" },
	{ 0x00047F, "Chr. Mayr GmbH & Co. KG", "CHR-MAYR" },
	{ 0x000480, "Brocade Communications Systems, Inc", "BROCADE-COMMUNICATION" },
	{ 0x000481, "Econolite Control Products, Inc.", "ECONOLITE-CONTROL-PRODUCTS" },
	{ 0x000482, "Medialogic Corp.", "MEDIALOGIC" },
	{ 0x000483, "Deltron Technology, Inc.", "DELTRON" },
	{ 0x000484, "Amann GmbH", "AMANN" },
	{ 0x000485, "PicoLight", "PICOLIGHT" },
	{ 0x000486, "ITTC, University of Kansas", "ITTC-UNIVERSITY-OF-KANSAS" },
	{ 0x000487, "Cogency Semiconductor, Inc.", "COGENCY-SEMICONDUCTOR" },
	{ 0x000488, "Eurotherm Controls", "EUROTHERM-CONTROLS" },
	{ 0x000489, "YAFO Networks, Inc.", "YAFO-NETWORKS" },
	{ 0x00048A, "Temia Vertriebs GmbH", "TEMIA-VERTRIEBS" },
	{ 0x00048B, "Poscon Corporation", "POSCON" },
	{ 0x00048C, "Nayna Networks, Inc.", "NAYNA-NETWORKS" },
	{ 0x00048D, "Tone Commander Systems, Inc.", "TONE-COMMANDER" },
	{ 0x00048E, "Ohm Tech Labs, Inc.", "OHM-TECH-LABS" },
	{ 0x00048F, "TD Systems Corporation", "TD" },
	{ 0x000490, "Optical Access", "OPTICAL-ACCESS" },
	{ 0x000491, "Technovision, Inc.", "TECHNOVISION" },
	{ 0x000492, "Hive Internet, Ltd.", "HIVE-INTERNET" },
	{ 0x000493, "Tsinghua Unisplendour Co., Ltd.", "TSINGHUA-UNISPLENDOUR" },
	{ 0x000494, "Breezecom, Ltd.", "BREEZECOM" },
	{ 0x000495, "Tejas Networks India Limited", "TEJAS-NETWORKS-INDIA" },
	{ 0x000496, "Extreme Networks", "EXTREME-NETWORKS" },
	{ 0x000497, "MacroSystem Digital Video AG", "MACROSYSTEM-DIGITAL-VIDEO" },
	{ 0x000498, "Mahi Networks", "MAHI-NETWORKS" },
	{ 0x000499, "Chino Corporation", "CHINO" },
	{ 0x00049A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00049B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00049C, "Surgient Networks, Inc.", "SURGIENT-NETWORKS" },
	{ 0x00049D, "Ipanema Technologies", "IPANEMA-TECHNOLOGIES" },
	{ 0x00049E, "Wirelink Co., Ltd.", "WIRELINK" },
	{ 0x00049F, "Freescale Semiconductor", "FREESCALE-SEMICONDUCTOR" },
	{ 0x0004A0, "Verity Instruments, Inc.", "VERITY-INSTRUMENTS" },
	{ 0x0004A1, "Pathway Connectivity", "PATHWAY-CONNECTIVITY" },
	{ 0x0004A2, "L.S.I. Japan Co., Ltd.", "L-S-I-JAPAN" },
	{ 0x0004A3, "Microchip Technology, Inc.", "MICROCHIP" },
	{ 0x0004A4, "NetEnabled, Inc.", "NETENABLED" },
	{ 0x0004A5, "Barco Projection Systems NV", "BARCO-PROJECTION-NV" },
	{ 0x0004A6, "SAF Tehnika Ltd.", "SAF-TEHNIKA" },
	{ 0x0004A7, "FabiaTech Corporation", "FABIATECH" },
	{ 0x0004A8, "Broadmax Technologies, Inc.", "BROADMAX-TECHNOLOGIES" },
	{ 0x0004A9, "SandStream Technologies, Inc.", "SANDSTREAM-TECHNOLOGIES" },
	{ 0x0004AA, "Jetstream Communications", "JETSTREAM-COMMUNICATION" },
	{ 0x0004AB, "Comverse Network Systems, Inc.", "COMVERSE-NETWORK" },
	{ 0x0004AC, "IBM Corp", "IBM" },
	{ 0x0004AD, "Malibu Networks", "MALIBU-NETWORKS" },
	{ 0x0004AE, "Sullair Corporation", "SULLAIR" },
	{ 0x0004AF, "Digital Fountain, Inc.", "DIGITAL-FOUNTAIN" },
	{ 0x0004B0, "ELESIGN Co., Ltd.", "ELESIGN" },
	{ 0x0004B1, "Signal Technology, Inc.", "SIGNAL" },
	{ 0x0004B2, "ESSEGI SRL", "ESSEGI-SRL" },
	{ 0x0004B3, "Videotek, Inc.", "VIDEOTEK" },
	{ 0x0004B4, "CIAC", "CIAC" },
	{ 0x0004B5, "Equitrac Corporation", "EQUITRAC" },
	{ 0x0004B6, "Stratex Networks, Inc.", "STRATEX-NETWORKS" },
	{ 0x0004B7, "AMB i.t. Holding", "AMB-I-T-HOLDING" },
	{ 0x0004B8, "Kumahira Co., Ltd.", "KUMAHIRA" },
	{ 0x0004B9, "S.I. Soubou, Inc.", "S-I-SOUBOU" },
	{ 0x0004BA, "KDD Media Will Corporation", "KDD-MEDIA-WILL" },
	{ 0x0004BB, "Bardac Corporation", "BARDAC" },
	{ 0x0004BC, "Giantec, Inc.", "GIANTEC" },
	{ 0x0004BD, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0004BE, "OptXCon, Inc.", "OPTXCON" },
	{ 0x0004BF, "VersaLogic Corp.", "VERSALOGIC" },
	{ 0x0004C0, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0004C1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0004C2, "Magnipix, Inc.", "MAGNIPIX" },
	{ 0x0004C3, "CASTOR Informatique", "CASTOR-INFORMATIQUE" },
	{ 0x0004C4, "Allen & Heath Limited", "ALLEN-HEATH" },
	{ 0x0004C5, "ASE Technologies, USA", "ASE-TECHNOLOGIES-USA" },
	{ 0x0004C6, "Yamaha Motor Co., Ltd.", "YAMAHA-MOTOR" },
	{ 0x0004C7, "NetMount", "NETMOUNT" },
	{ 0x0004C8, "LIBA Maschinenfabrik GmbH", "LIBA-MASCHINENFABRIK" },
	{ 0x0004C9, "Micro Electron Co., Ltd.", "MICRO-ELECTRON" },
	{ 0x0004CA, "FreeMs Corp.", "FREEMS" },
	{ 0x0004CB, "Tdsoft Communication, Ltd.", "TDSOFT-COMMUNICATION" },
	{ 0x0004CC, "Peek Traffic B.V.", "PEEK-TRAFFIC" },
	{ 0x0004CD, "Extenway Solutions Inc", "EXTENWAY-SOLUTIONS" },
	{ 0x0004CE, "Patria Ailon", "PATRIA-AILON" },
	{ 0x0004CF, "Seagate Technology", "SEAGATE" },
	{ 0x0004D0, "Softlink s.r.o.", "SOFTLINK-S-R-O" },
	{ 0x0004D1, "Drew Technologies, Inc.", "DREW-TECHNOLOGIES" },
	{ 0x0004D2, "Adcon Telemetry GmbH", "ADCON-TELEMETRY" },
	{ 0x0004D3, "Toyokeiki Co., Ltd.", "TOYOKEIKI" },
	{ 0x0004D4, "Proview Electronics Co., Ltd.", "PROVIEW-ELECTRONICS" },
	{ 0x0004D5, "Hitachi Information & Communication Engineering, Ltd.", "HITACHI-INFORMATION-COMMUNICATION-ENGINEERING" },
	{ 0x0004D6, "Takagi Industrial Co., Ltd.", "TAKAGI-INDUSTRIAL" },
	{ 0x0004D7, "Omitec Instrumentation Ltd.", "OMITEC-INSTRUMENTATION" },
	{ 0x0004D8, "IPWireless, Inc.", "IPWIRELESS" },
	{ 0x0004D9, "Titan Electronics, Inc.", "TITAN-ELECTRONICS" },
	{ 0x0004DA, "Relax Technology, Inc.", "RELAX" },
	{ 0x0004DB, "Tellus Group Corp.", "TELLUS-GROUP" },
	{ 0x0004DC, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x0004DD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0004DE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0004DF, "Teracom Telematica Ltda.", "TERACOM-TELEMATICA-LTDA" },
	{ 0x0004E0, "Procket Networks", "PROCKET-NETWORKS" },
	{ 0x0004E1, "Infinior Microsystems", "INFINIOR-MICROSYSTEMS" },
	{ 0x0004E2, "SMC Networks, Inc.", "SMC-NETWORKS" },
	{ 0x0004E3, "Accton Technology Corp.", "ACCTON" },
	{ 0x0004E4, "Daeryung Ind., Inc.", "DAERYUNG-IND" },
	{ 0x0004E5, "Glonet Systems, Inc.", "GLONET" },
	{ 0x0004E6, "Banyan Network Private Limited", "BANYAN-NETWORK-PRIVATE" },
	{ 0x0004E7, "Lightpointe Communications, Inc", "LIGHTPOINTE-COMMUNICATION" },
	{ 0x0004E8, "IER, Inc.", "IER" },
	{ 0x0004E9, "Infiniswitch Corporation", "INFINISWITCH" },
	{ 0x0004EA, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0004EB, "Paxonet Communications, Inc.", "PAXONET-COMMUNICATION" },
	{ 0x0004EC, "Memobox SA", "MEMOBOX" },
	{ 0x0004ED, "Billion Electric Co., Ltd.", "BILLION-ELECTRIC" },
	{ 0x0004EE, "Lincoln Electric Company", "LINCOLN-ELECTRIC-COMPANY" },
	{ 0x0004EF, "Polestar Corp.", "POLESTAR" },
	{ 0x0004F0, "International Computers, Ltd", "COMPUTERS" },
	{ 0x0004F1, "WhereNet", "WHERENET" },
	{ 0x0004F2, "Polycom", "POLYCOM" },
	{ 0x0004F3, "FS FORTH-SYSTEME GmbH", "FS-FORTH-SYSTEME" },
	{ 0x0004F4, "Infinite Electronics Inc.", "INFINITE-ELECTRONICS" },
	{ 0x0004F5, "SnowShore Networks, Inc.", "SNOWSHORE-NETWORKS" },
	{ 0x0004F6, "Amphus", "AMPHUS" },
	{ 0x0004F7, "Omega Band, Inc.", "OMEGA-BAND" },
	{ 0x0004F8, "QUALICABLE TV Industria E Com., Ltda", "QUALICABLE-TV-INDUSTRIA-E-COM-LTDA" },
	{ 0x0004F9, "Xtera Communications, Inc.", "XTERA-COMMUNICATION" },
	{ 0x0004FA, "NBS Technologies Inc.", "NBS-TECHNOLOGIES" },
	{ 0x0004FB, "Commtech, Inc.", "COMMTECH" },
	{ 0x0004FC, "Stratus Computer (DE), Inc.", "STRATUS-DE" },
	{ 0x0004FD, "Japan Control Engineering Co., Ltd.", "JAPAN-CONTROL-ENGINEERING" },
	{ 0x0004FE, "Pelago Networks", "PELAGO-NETWORKS" },
	{ 0x0004FF, "Acronet Co., Ltd.", "ACRONET" },
	{ 0x000500, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000501, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000502, "Apple", "APPLE" },
	{ 0x000503, "ICONAG", "ICONAG" },
	{ 0x000504, "Naray Information & Communication Enterprise", "NARAY-INFORMATION-COMMUNICATION" },
	{ 0x000505, "Systems Integration Solutions, Inc.", "INTEGRATION-SOLUTIONS" },
	{ 0x000506, "Reddo Networks AB", "REDDO-NETWORKS-AB" },
	{ 0x000507, "Fine Appliance Corp.", "FINE-APPLIANCE" },
	{ 0x000508, "Inetcam, Inc.", "INETCAM" },
	{ 0x000509, "AVOC Nishimura Ltd.", "AVOC-NISHIMURA" },
	{ 0x00050A, "ICS Spa", "ICS-SPA" },
	{ 0x00050B, "SICOM Systems, Inc.", "SICOM" },
	{ 0x00050C, "Network Photonics, Inc.", "NETWORK-PHOTONICS" },
	{ 0x00050D, "Midstream Technologies, Inc.", "MIDSTREAM-TECHNOLOGIES" },
	{ 0x00050E, "3ware, Inc.", "3WARE" },
	{ 0x00050F, "Tanaka S/S Ltd.", "TANAKA-S/S" },
	{ 0x000510, "Infinite Shanghai Communication Terminals Ltd.", "INFINITE-SHANGHAI-COMMUNICATION-TERMINALS" },
	{ 0x000511, "Complementary Technologies Ltd", "COMPLEMENTARY-TECHNOLOGIES" },
	{ 0x000512, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0x000513, "VTLinx Multimedia Systems, Inc.", "VTLINX-MULTIMEDIA" },
	{ 0x000514, "KDT Systems Co., Ltd.", "KDT" },
	{ 0x000515, "Nuark Co., Ltd.", "NUARK" },
	{ 0x000516, "SMART Modular Technologies", "SMART-MODULAR-TECHNOLOGIES" },
	{ 0x000517, "Shellcomm, Inc.", "SHELLCOMM" },
	{ 0x000518, "Jupiters Technology", "JUPITERS" },
	{ 0x000519, "Siemens Building Technologies AG,", "SIEMENS-BUILDING-TECHNOLOGIES" },
	{ 0x00051A, "3Com Europe Ltd.", "3COM" },
	{ 0x00051B, "Magic Control Technology Corporation", "MAGIC-CONTROL" },
	{ 0x00051C, "Xnet Technology Corp.", "XNET" },
	{ 0x00051D, "Airocon, Inc.", "AIROCON" },
	{ 0x00051E, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x00051F, "Taijin Media Co., Ltd.", "TAIJIN-MEDIA" },
	{ 0x000520, "Smartronix, Inc.", "SMARTRONIX" },
	{ 0x000521, "Control Microsystems", "CONTROL-MICROSYSTEMS" },
	{ 0x000522, "LEA*D Corporation, Inc.", "LEA*D" },
	{ 0x000523, "AVL List GmbH", "AVL-LIST" },
	{ 0x000524, "BTL System (HK) Limited", "BTL-SYSTEM-HK" },
	{ 0x000525, "Puretek Industrial Co., Ltd.", "PURETEK-INDUSTRIAL" },
	{ 0x000526, "IPAS GmbH", "IPAS" },
	{ 0x000527, "SJ Tek Co. Ltd", "SJ-TEK" },
	{ 0x000528, "New Focus, Inc.", "NEW-FOCUS" },
	{ 0x000529, "Shanghai Broadan Communication Technology Co., Ltd", "SHANGHAI-BROADAN-COMMUNICATION" },
	{ 0x00052A, "Ikegami Tsushinki Co., Ltd.", "IKEGAMI-TSUSHINKI" },
	{ 0x00052B, "HORIBA, Ltd.", "HORIBA" },
	{ 0x00052C, "Supreme Magic Corporation", "SUPREME-MAGIC" },
	{ 0x00052D, "Zoltrix International Limited", "ZOLTRIX" },
	{ 0x00052E, "Cinta Networks", "CINTA-NETWORKS" },
	{ 0x00052F, "Leviton Network Solutions", "LEVITON-NETWORK-SOLUTIONS" },
	{ 0x000530, "Andiamo Systems, Inc.", "ANDIAMO" },
	{ 0x000531, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000532, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000533, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x000534, "Northstar Engineering Ltd.", "NORTHSTAR-ENGINEERING" },
	{ 0x000535, "Chip PC Ltd.", "CHIP-PC" },
	{ 0x000536, "Danam Communications, Inc.", "DANAM-COMMUNICATION" },
	{ 0x000537, "Nets Technology Co., Ltd.", "NETS" },
	{ 0x000538, "Merilus, Inc.", "MERILUS" },
	{ 0x000539, "A Brand New World in Sweden AB", "A-BRAND-NEW-WORLD-IN-SWEDEN-AB" },
	{ 0x00053A, "Willowglen Services Pte Ltd", "WILLOWGLEN-SERVICES-PTE" },
	{ 0x00053B, "Harbour Networks Ltd., Co. Beijing", "HARBOUR-NETWORKS-BEIJING" },
	{ 0x00053C, "Xircom", "XIRCOM" },
	{ 0x00053D, "Agere Systems", "AGERE" },
	{ 0x00053E, "KID Systeme GmbH", "KID-SYSTEME" },
	{ 0x00053F, "VisionTek, Inc.", "VISIONTEK" },
	{ 0x000540, "FAST Corporation", "FAST" },
	{ 0x000541, "Advanced Systems Co., Ltd.", "ADVANCED" },
	{ 0x000542, "Otari, Inc.", "OTARI" },
	{ 0x000543, "IQ Wireless GmbH", "IQ-WIRELESS" },
	{ 0x000544, "Valley Technologies, Inc.", "VALLEY-TECHNOLOGIES" },
	{ 0x000545, "Internet Photonics", "INTERNET-PHOTONICS" },
	{ 0x000546, "KDDI Network & Solultions Inc.", "KDDI-NETWORK-SOLULTIONS" },
	{ 0x000547, "Starent Networks", "STARENT-NETWORKS" },
	{ 0x000548, "Disco Corporation", "DISCO" },
	{ 0x000549, "Salira Optical Network Systems", "SALIRA-OPTICAL-NETWORK" },
	{ 0x00054A, "Ario Data Networks, Inc.", "ARIO-DATA-NETWORKS" },
	{ 0x00054B, "Eaton Automation AG", "EATON-AUTOMATION" },
	{ 0x00054C, "RF Innovations Pty Ltd", "RF-INNOVATIONS-PTY" },
	{ 0x00054D, "Brans Technologies, Inc.", "BRANS-TECHNOLOGIES" },
	{ 0x00054E, "Philips", "PHILIPS" },
	{ 0x00054F, "PRIVATE", "PRIVATE" },
	{ 0x000550, "Vcomms Connect Limited", "VCOMMS-CONNECT" },
	{ 0x000551, "F & S Elektronik Systeme GmbH", "F-S-ELEKTRONIK-SYSTEME" },
	{ 0x000552, "Xycotec Computer GmbH", "XYCOTEC" },
	{ 0x000553, "DVC Company, Inc.", "DVC-COMPANY" },
	{ 0x000554, "Rangestar Wireless", "RANGESTAR-WIRELESS" },
	{ 0x000555, "Japan Cash Machine Co., Ltd.", "JAPAN-CASH-MACHINE" },
	{ 0x000556, "360 Systems", "360" },
	{ 0x000557, "Agile TV Corporation", "AGILE-TV" },
	{ 0x000558, "Synchronous, Inc.", "SYNCHRONOUS" },
	{ 0x000559, "Intracom S.A.", "INTRACOM" },
	{ 0x00055A, "Power Dsine Ltd.", "POWER-DSINE" },
	{ 0x00055B, "Charles Industries, Ltd.", "CHARLES-INDUSTRIES" },
	{ 0x00055C, "Kowa Company, Ltd.", "KOWA-COMPANY" },
	{ 0x00055D, "D-Link Systems, Inc.", "D-LINK" },
	{ 0x00055E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00055F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000560, "LEADER COMM.CO., LTD", "LEADER-COMM" },
	{ 0x000561, "nac Image Technology, Inc.", "NAC-IMAGE" },
	{ 0x000562, "Digital View Limited", "DIGITAL-VIEW" },
	{ 0x000563, "J-Works, Inc.", "J-WORKS" },
	{ 0x000564, "Tsinghua Bitway Co., Ltd.", "TSINGHUA-BITWAY" },
	{ 0x000565, "Tailyn Communication Company Ltd.", "TAILYN-COMMUNICATION-COMPANY" },
	{ 0x000566, "Secui.com Corporation", "SECUI-COM" },
	{ 0x000567, "Etymonic Design, Inc.", "ETYMONIC-DESIGN" },
	{ 0x000568, "Piltofish Networks AB", "PILTOFISH-NETWORKS-AB" },
	{ 0x000569, "VMware, Inc. (possible VMware VM)", "VMWARE-VIRTUAL" },
	{ 0x00056A, "Heuft Systemtechnik GmbH", "HEUFT-SYSTEMTECHNIK" },
	{ 0x00056B, "C.P. Technology Co., Ltd.", "C-P" },
	{ 0x00056C, "Hung Chang Co., Ltd.", "HUNG-CHANG" },
	{ 0x00056D, "Pacific Corporation", "PACIFIC" },
	{ 0x00056E, "National Enhance Technology, Inc.", "NATIONAL-ENHANCE" },
	{ 0x00056F, "Innomedia Technologies Pvt. Ltd.", "INNOMEDIA-TECHNOLOGIES-PVT" },
	{ 0x000570, "Baydel Ltd.", "BAYDEL" },
	{ 0x000571, "Seiwa Electronics Co.", "SEIWA-ELECTRONICS" },
	{ 0x000572, "Deonet Co., Ltd.", "DEONET" },
	{ 0x000573, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000574, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000575, "CDS-Electronics BV", "CDS-ELECTRONICS" },
	{ 0x000576, "NSM Technology Ltd.", "NSM" },
	{ 0x000577, "SM Information & Communication", "SM-INFORMATION-COMMUNICATION" },
	{ 0x000578, "PRIVATE", "PRIVATE" },
	{ 0x000579, "Universal Control Solution Corp.", "UNIVERSAL-CONTROL-SOLUTION" },
	{ 0x00057A, "Overture Networks", "OVERTURE-NETWORKS" },
	{ 0x00057B, "Chung Nam Electronic Co., Ltd.", "CHUNG-NAM" },
	{ 0x00057C, "RCO Security AB", "RCO-SECURITY-AB" },
	{ 0x00057D, "Sun Communications, Inc.", "SUN-COMMUNICATION" },
	{ 0x00057E, "Eckelmann Steuerungstechnik GmbH", "ECKELMANN-STEUERUNGSTECHNIK" },
	{ 0x00057F, "Acqis Technology", "ACQIS" },
	{ 0x000580, "FibroLAN Ltd.", "FIBROLAN" },
	{ 0x000581, "Snell", "SNELL" },
	{ 0x000582, "ClearCube Technology", "CLEARCUBE" },
	{ 0x000583, "ImageCom Limited", "IMAGECOM" },
	{ 0x000584, "AbsoluteValue Systems, Inc.", "ABSOLUTEVALUE" },
	{ 0x000585, "Juniper Networks, Inc.", "JUNIPER-NETWORKS" },
	{ 0x000586, "Lucent Technologies", "LUCENT-TECHNOLOGIES" },
	{ 0x000587, "Locus, Incorporated", "LOCUS-INCORPORATED" },
	{ 0x000588, "Sensoria Corp.", "SENSORIA" },
	{ 0x000589, "National Datacomputer", "NATIONAL-DATACOMPUTER" },
	{ 0x00058A, "Netcom Co., Ltd.", "NETCOM" },
	{ 0x00058B, "IPmental, Inc.", "IPMENTAL" },
	{ 0x00058C, "Opentech Inc.", "OPENTECH" },
	{ 0x00058D, "Lynx Photonic Networks, Inc.", "LYNX-PHOTONIC-NETWORKS" },
	{ 0x00058E, "Flextronics International GmbH & Co. Nfg. KG", "FLEXTRONICS-NFG" },
	{ 0x00058F, "CLCsoft co.", "CLCSOFT" },
	{ 0x000590, "Swissvoice Ltd.", "SWISSVOICE" },
	{ 0x000591, "Active Silicon Ltd", "ACTIVE-SILICON" },
	{ 0x000592, "Pultek Corp.", "PULTEK" },
	{ 0x000593, "Grammar Engine Inc.", "GRAMMAR-ENGINE" },
	{ 0x000594, "IXXAT Automation GmbH", "IXXAT-AUTOMATION" },
	{ 0x000595, "Alesis Corporation", "ALESIS" },
	{ 0x000596, "Genotech Co., Ltd.", "GENOTECH" },
	{ 0x000597, "Eagle Traffic Control Systems", "EAGLE-TRAFFIC-CONTROL" },
	{ 0x000598, "CRONOS S.r.l.", "CRONOS-S-R-L" },
	{ 0x000599, "DRS Test and Energy Management or DRS-TEM", "DRS-TEST-AND-ENERGY-MANAGEMENT-OR-DRS-TEM" },
	{ 0x00059A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00059B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00059C, "Kleinknecht GmbH, Ing. Büro", "KLEINKNECHT-ING-BüRO" },
	{ 0x00059D, "Daniel Computing Systems, Inc.", "DANIEL-COMPUTING" },
	{ 0x00059E, "Zinwell Corporation", "ZINWELL" },
	{ 0x00059F, "Yotta Networks, Inc.", "YOTTA-NETWORKS" },
	{ 0x0005A0, "MOBILINE Kft.", "MOBILINE-KFT" },
	{ 0x0005A1, "Zenocom", "ZENOCOM" },
	{ 0x0005A2, "CELOX Networks", "CELOX-NETWORKS" },
	{ 0x0005A3, "QEI, Inc.", "QEI" },
	{ 0x0005A4, "Lucid Voice Ltd.", "LUCID-VOICE" },
	{ 0x0005A5, "KOTT", "KOTT" },
	{ 0x0005A6, "Extron Electronics", "EXTRON-ELECTRONICS" },
	{ 0x0005A7, "Hyperchip, Inc.", "HYPERCHIP" },
	{ 0x0005A8, "WYLE ELECTRONICS", "WYLE-ELECTRONICS" },
	{ 0x0005A9, "Princeton Networks, Inc.", "PRINCETON-NETWORKS" },
	{ 0x0005AA, "Moore Industries International Inc.", "MOORE-INDUSTRIES" },
	{ 0x0005AB, "Cyber Fone, Inc.", "CYBER-FONE" },
	{ 0x0005AC, "Northern Digital, Inc.", "NORTHERN-DIGITAL" },
	{ 0x0005AD, "Topspin Communications, Inc.", "TOPSPIN-COMMUNICATION" },
	{ 0x0005AE, "Mediaport USA", "MEDIAPORT-USA" },
	{ 0x0005AF, "InnoScan Computing A/S", "INNOSCAN-COMPUTING-A/S" },
	{ 0x0005B0, "Korea Computer Technology Co., Ltd.", "KOREA" },
	{ 0x0005B1, "ASB Technology BV", "ASB" },
	{ 0x0005B2, "Medison Co., Ltd.", "MEDISON" },
	{ 0x0005B3, "Asahi-Engineering Co., Ltd.", "ASAHI-ENGINEERING" },
	{ 0x0005B4, "Aceex Corporation", "ACEEX" },
	{ 0x0005B5, "Broadcom Technologies", "BROADCOM-TECHNOLOGIES" },
	{ 0x0005B6, "INSYS Microelectronics GmbH", "INSYS-MICROELECTRONICS" },
	{ 0x0005B7, "Arbor Technology Corp.", "ARBOR" },
	{ 0x0005B8, "Electronic Design Associates, Inc.", "DESIGN-ASSOCIATES" },
	{ 0x0005B9, "Airvana, Inc.", "AIRVANA" },
	{ 0x0005BA, "Area Netwoeks, Inc.", "AREA-NETWOEKS" },
	{ 0x0005BB, "Myspace AB", "MYSPACE-AB" },
	{ 0x0005BC, "Resource Data Management Ltd", "RESOURCE-DATA-MANAGEMENT" },
	{ 0x0005BD, "ROAX BV", "ROAX" },
	{ 0x0005BE, "Kongsberg Seatex AS", "KONGSBERG-SEATEX-AS" },
	{ 0x0005BF, "JustEzy Technology, Inc.", "JUSTEZY" },
	{ 0x0005C0, "Digital Network Alacarte Co., Ltd.", "DIGITAL-NETWORK-ALACARTE" },
	{ 0x0005C1, "A-Kyung Motion, Inc.", "A-KYUNG-MOTION" },
	{ 0x0005C2, "Soronti, Inc.", "SORONTI" },
	{ 0x0005C3, "Pacific Instruments, Inc.", "PACIFIC-INSTRUMENTS" },
	{ 0x0005C4, "Telect, Inc.", "TELECT" },
	{ 0x0005C5, "Flaga HF", "FLAGA-HF" },
	{ 0x0005C6, "Triz Communications", "TRIZ-COMMUNICATION" },
	{ 0x0005C7, "I/F-COM A/S", "I/F-COM-A/S" },
	{ 0x0005C8, "VERYTECH", "VERYTECH" },
	{ 0x0005C9, "LG Innotek Co., Ltd.", "LG-INNOTEK" },
	{ 0x0005CA, "Hitron Technology, Inc.", "HITRON" },
	{ 0x0005CB, "ROIS Technologies, Inc.", "ROIS-TECHNOLOGIES" },
	{ 0x0005CC, "Sumtel Communications, Inc.", "SUMTEL-COMMUNICATION" },
	{ 0x0005CD, "Denon, Ltd.", "DENON" },
	{ 0x0005CE, "Prolink Microsystems Corporation", "PROLINK-MICROSYSTEMS" },
	{ 0x0005CF, "Thunder River Technologies, Inc.", "THUNDER-RIVER-TECHNOLOGIES" },
	{ 0x0005D0, "Solinet Systems", "SOLINET" },
	{ 0x0005D1, "Metavector Technologies", "METAVECTOR-TECHNOLOGIES" },
	{ 0x0005D2, "DAP Technologies", "DAP-TECHNOLOGIES" },
	{ 0x0005D3, "eProduction Solutions, Inc.", "EPRODUCTION-SOLUTIONS" },
	{ 0x0005D4, "FutureSmart Networks, Inc.", "FUTURESMART-NETWORKS" },
	{ 0x0005D5, "Speedcom Wireless", "SPEEDCOM-WIRELESS" },
	{ 0x0005D6, "L-3 Linkabit", "L-3-LINKABIT" },
	{ 0x0005D7, "Vista Imaging, Inc.", "VISTA-IMAGING" },
	{ 0x0005D8, "Arescom, Inc.", "ARESCOM" },
	{ 0x0005D9, "Techno Valley, Inc.", "TECHNO-VALLEY" },
	{ 0x0005DA, "Apex Automationstechnik", "APEX-AUTOMATIONSTECHNIK" },
	{ 0x0005DB, "PSI Nentec GmbH", "PSI-NENTEC" },
	{ 0x0005DC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0005DD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0005DE, "Gi Fone Korea, Inc.", "GI-FONE-KOREA" },
	{ 0x0005DF, "Electronic Innovation, Inc.", "INNOVATION" },
	{ 0x0005E0, "Empirix Corp.", "EMPIRIX" },
	{ 0x0005E1, "Trellis Photonics, Ltd.", "TRELLIS-PHOTONICS" },
	{ 0x0005E2, "Creativ Network Technologies", "CREATIV-NETWORK-TECHNOLOGIES" },
	{ 0x0005E3, "LightSand Communications, Inc.", "LIGHTSAND-COMMUNICATION" },
	{ 0x0005E4, "Red Lion Controls Inc.", "RED-LION-CONTROLS" },
	{ 0x0005E5, "Renishaw PLC", "RENISHAW-PLC" },
	{ 0x0005E6, "Egenera, Inc.", "EGENERA" },
	{ 0x0005E7, "Netrake an AudioCodes Company", "NETRAKE-AN-AUDIOCODES-COMPANY" },
	{ 0x0005E8, "TurboWave, Inc.", "TURBOWAVE" },
	{ 0x0005E9, "Unicess Network, Inc.", "UNICESS-NETWORK" },
	{ 0x0005EA, "Rednix", "REDNIX" },
	{ 0x0005EB, "Blue Ridge Networks, Inc.", "BLUE-RIDGE-NETWORKS" },
	{ 0x0005EC, "Mosaic Systems Inc.", "MOSAIC" },
	{ 0x0005ED, "Technikum Joanneum GmbH", "TECHNIKUM-JOANNEUM" },
	{ 0x0005EE, "Siemens AB, Infrastructure & Cities, Building Technologies Division, IC BT SSP SP BA PR", "SIEMENS-AB-INFRASTRUCTURE-CITIES-BUILDING-TECHNOLOGIES-DIVISION-IC-BT-SSP-SP-BA-PR" },
	{ 0x0005EF, "ADOIR Digital Technology", "ADOIR-DIGITAL" },
	{ 0x0005F0, "SATEC", "SATEC" },
	{ 0x0005F1, "Vrcom, Inc.", "VRCOM" },
	{ 0x0005F2, "Power R, Inc.", "POWER-R" },
	{ 0x0005F3, "Webyn", "WEBYN" },
	{ 0x0005F4, "System Base Co., Ltd.", "SYSTEM-BASE" },
	{ 0x0005F5, "Geospace Technologies", "GEOSPACE-TECHNOLOGIES" },
	{ 0x0005F6, "Young Chang Co. Ltd.", "YOUNG-CHANG" },
	{ 0x0005F7, "Analog Devices, Inc.", "ANALOG-DEVICES" },
	{ 0x0005F8, "Real Time Access, Inc.", "REAL-TIME-ACCESS" },
	{ 0x0005F9, "TOA Corporation", "TOA" },
	{ 0x0005FA, "IPOptical, Inc.", "IPOPTICAL" },
	{ 0x0005FB, "ShareGate, Inc.", "SHAREGATE" },
	{ 0x0005FC, "Schenck Pegasus Corp.", "SCHENCK-PEGASUS" },
	{ 0x0005FD, "PacketLight Networks Ltd.", "PACKETLIGHT-NETWORKS" },
	{ 0x0005FE, "Traficon N.V.", "TRAFICON-N-V" },
	{ 0x0005FF, "SNS Solutions, Inc.", "SNS-SOLUTIONS" },
	{ 0x000600, "Toshiba Teli Corporation", "TOSHIBA-TELI" },
	{ 0x000601, "Otanikeiki Co., Ltd.", "OTANIKEIKI" },
	{ 0x000602, "Cirkitech Electronics Co.", "CIRKITECH-ELECTRONICS" },
	{ 0x000603, "Baker Hughes Inc.", "BAKER-HUGHES" },
	{ 0x000604, "@Track Communications, Inc.", "@TRACK-COMMUNICATION" },
	{ 0x000605, "Inncom International, Inc.", "INNCOM" },
	{ 0x000606, "RapidWAN, Inc.", "RAPIDWAN" },
	{ 0x000607, "Omni Directional Control Technology Inc.", "OMNI-DIRECTIONAL-CONTROL" },
	{ 0x000608, "At-Sky SAS", "AT-SKY-SAS" },
	{ 0x000609, "Crossport Systems", "CROSSPORT" },
	{ 0x00060A, "Blue2space", "BLUE2SPACE" },
	{ 0x00060B, "Artesyn Embedded Technologies", "ARTESYN-EMBEDDED-TECHNOLOGIES" },
	{ 0x00060C, "Melco Industries, Inc.", "MELCO-INDUSTRIES" },
	{ 0x00060D, "Wave7 Optics", "WAVE7-OPTICS" },
	{ 0x00060E, "IGYS Systems, Inc.", "IGYS" },
	{ 0x00060F, "Narad Networks Inc", "NARAD-NETWORKS" },
	{ 0x000610, "Abeona Networks Inc", "ABEONA-NETWORKS" },
	{ 0x000611, "Zeus Wireless, Inc.", "ZEUS-WIRELESS" },
	{ 0x000612, "Accusys, Inc.", "ACCUSYS" },
	{ 0x000613, "Kawasaki Microelectronics Incorporated", "KAWASAKI-MICROELECTRONICS-INCORPORATED" },
	{ 0x000614, "Prism Holdings", "PRISM-HOLDINGS" },
	{ 0x000615, "Kimoto Electric Co., Ltd.", "KIMOTO-ELECTRIC" },
	{ 0x000616, "Tel Net Co., Ltd.", "TEL-NET" },
	{ 0x000617, "Redswitch Inc.", "REDSWITCH" },
	{ 0x000618, "DigiPower Manufacturing Inc.", "DIGIPOWER-MANUFACTURING" },
	{ 0x000619, "Connection Technology Systems", "CONNECTION" },
	{ 0x00061A, "Zetari Inc.", "ZETARI" },
	{ 0x00061B, "Notebook Development Lab. Lenovo Japan Ltd.", "NOTEBOOK-DEVELOPMENT-LAB-LENOVO-JAPAN" },
	{ 0x00061C, "Hoshino Metal Industries, Ltd.", "HOSHINO-METAL-INDUSTRIES" },
	{ 0x00061D, "MIP Telecom, Inc.", "MIP-TELECOM" },
	{ 0x00061E, "Maxan Systems", "MAXAN" },
	{ 0x00061F, "Vision Components GmbH", "VISION-COMPONENTS" },
	{ 0x000620, "Serial System Ltd.", "SERIAL-SYSTEM" },
	{ 0x000621, "Hinox, Co., Ltd.", "HINOX" },
	{ 0x000622, "Chung Fu Chen Yeh Enterprise Corp.", "CHUNG-FU-CHEN-YEH" },
	{ 0x000623, "MGE UPS Systems France", "MGE-UPS-FRANCE" },
	{ 0x000624, "Gentner Communications Corp.", "GENTNER-COMMUNICATION" },
	{ 0x000625, "The Linksys Group, Inc.", "THE-LINKSYS-GROUP" },
	{ 0x000626, "MWE GmbH", "MWE" },
	{ 0x000627, "Uniwide Technologies, Inc.", "UNIWIDE-TECHNOLOGIES" },
	{ 0x000628, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000629, "IBM Corp", "IBM" },
	{ 0x00062A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00062B, "INTRASERVER TECHNOLOGY", "INTRASERVER" },
	{ 0x00062C, "Bivio Networks", "BIVIO-NETWORKS" },
	{ 0x00062D, "TouchStar Technologies, L.L.C.", "TOUCHSTAR-TECHNOLOGIES-L-L-C" },
	{ 0x00062E, "Aristos Logic Corp.", "ARISTOS-LOGIC" },
	{ 0x00062F, "Pivotech Systems Inc.", "PIVOTECH" },
	{ 0x000630, "Adtranz Sweden", "ADTRANZ-SWEDEN" },
	{ 0x000631, "Calix", "CALIX" },
	{ 0x000632, "Mesco Engineering GmbH", "MESCO-ENGINEERING" },
	{ 0x000633, "Cross Match Technologies GmbH", "CROSS-MATCH-TECHNOLOGIES" },
	{ 0x000634, "GTE Airfone Inc.", "GTE-AIRFONE" },
	{ 0x000635, "PacketAir Networks, Inc.", "PACKETAIR-NETWORKS" },
	{ 0x000636, "Jedai Broadband Networks", "JEDAI-BROADBAND-NETWORKS" },
	{ 0x000637, "Toptrend-Meta Information (ShenZhen) Inc.", "TOPTREND-META-INFORMATION-SHENZHEN" },
	{ 0x000638, "Sungjin C&C Co., Ltd.", "SUNGJIN-C-C" },
	{ 0x000639, "Newtec", "NEWTEC" },
	{ 0x00063A, "Dura Micro, Inc.", "DURA-MICRO" },
	{ 0x00063B, "Arcturus Networks Inc.", "ARCTURUS-NETWORKS" },
	{ 0x00063C, "Intrinsyc Software International Inc.", "INTRINSYC-SOFTWARE" },
	{ 0x00063D, "Microwave Data Systems Inc.", "MICROWAVE-DATA" },
	{ 0x00063E, "Opthos Inc.", "OPTHOS" },
	{ 0x00063F, "Everex Communications Inc.", "EVEREX-COMMUNICATION" },
	{ 0x000640, "White Rock Networks", "WHITE-ROCK-NETWORKS" },
	{ 0x000641, "ITCN", "ITCN" },
	{ 0x000642, "Genetel Systems Inc.", "GENETEL" },
	{ 0x000643, "SONO Computer Co., Ltd.", "SONO" },
	{ 0x000644, "neix,Inc", "NEIX" },
	{ 0x000645, "Meisei Electric Co. Ltd.", "MEISEI-ELECTRIC" },
	{ 0x000646, "ShenZhen XunBao Network Technology Co Ltd", "SHENZHEN-XUNBAO-NETWORK" },
	{ 0x000647, "Etrali S.A.", "ETRALI" },
	{ 0x000648, "Seedsware, Inc.", "SEEDSWARE" },
	{ 0x000649, "3M Deutschland GmbH", "3M-DEUTSCHLAND" },
	{ 0x00064A, "Honeywell Co., Ltd. (KOREA)", "HONEYWELL-KOREA" },
	{ 0x00064B, "Alexon Co., Ltd.", "ALEXON" },
	{ 0x00064C, "Invicta Networks, Inc.", "INVICTA-NETWORKS" },
	{ 0x00064D, "Sencore", "SENCORE" },
	{ 0x00064E, "Broad Net Technology Inc.", "BROAD-NET" },
	{ 0x00064F, "PRO-NETS Technology Corporation", "PRO-NETS" },
	{ 0x000650, "Tiburon Networks, Inc.", "TIBURON-NETWORKS" },
	{ 0x000651, "Aspen Networks Inc.", "ASPEN-NETWORKS" },
	{ 0x000652, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000653, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000654, "Winpresa Building Automation Technologies GmbH", "WINPRESA-BUILDING-AUTOMATION-TECHNOLOGIES" },
	{ 0x000655, "Yipee, Inc.", "YIPEE" },
	{ 0x000656, "Tactel AB", "TACTEL-AB" },
	{ 0x000657, "Market Central, Inc.", "MARKET-CENTRAL" },
	{ 0x000658, "Helmut Fischer GmbH Institut für Elektronik und Messtechnik", "HELMUT-FISCHER-INSTITUT-FüR-ELEKTRONIK-UND-MESSTECHNIK" },
	{ 0x000659, "EAL (Apeldoorn) B.V.", "EAL-APELDOORN" },
	{ 0x00065A, "Strix Systems", "STRIX" },
	{ 0x00065B, "Dell Computer Corp.", "DELL" },
	{ 0x00065C, "Malachite Technologies, Inc.", "MALACHITE-TECHNOLOGIES" },
	{ 0x00065D, "Heidelberg Web Systems", "HEIDELBERG-WEB" },
	{ 0x00065E, "Photuris, Inc.", "PHOTURIS" },
	{ 0x00065F, "ECI Telecom - NGTS Ltd.", "ECI-TELECOM---NGTS" },
	{ 0x000660, "NADEX Co., Ltd.", "NADEX" },
	{ 0x000661, "NIA Home Technologies Corp.", "NIA-HOME-TECHNOLOGIES" },
	{ 0x000662, "MBM Technology Ltd.", "MBM" },
	{ 0x000663, "Human Technology Co., Ltd.", "HUMAN" },
	{ 0x000664, "Fostex Corporation", "FOSTEX" },
	{ 0x000665, "Sunny Giken, Inc.", "SUNNY-GIKEN" },
	{ 0x000666, "Roving Networks", "ROVING-NETWORKS" },
	{ 0x000667, "Tripp Lite", "TRIPP-LITE" },
	{ 0x000668, "Vicon Industries Inc.", "VICON-INDUSTRIES" },
	{ 0x000669, "Datasound Laboratories Ltd", "DATASOUND-LABORATORIES" },
	{ 0x00066A, "InfiniCon Systems, Inc.", "INFINICON" },
	{ 0x00066B, "Sysmex Corporation", "SYSMEX" },
	{ 0x00066C, "Robinson Corporation", "ROBINSON" },
	{ 0x00066D, "Compuprint S.P.A.", "COMPUPRINT-S-P-A" },
	{ 0x00066E, "Delta Electronics, Inc.", "DELTA-ELECTRONICS" },
	{ 0x00066F, "Korea Data Systems", "KOREA-DATA" },
	{ 0x000670, "Upponetti Oy", "UPPONETTI-OY" },
	{ 0x000671, "Softing AG", "SOFTING" },
	{ 0x000672, "Netezza", "NETEZZA" },
	{ 0x000673, "TKH Security Solutions USA", "TKH-SECURITY-SOLUTIONS-USA" },
	{ 0x000674, "Spectrum Control, Inc.", "SPECTRUM-CONTROL" },
	{ 0x000675, "Banderacom, Inc.", "BANDERACOM" },
	{ 0x000676, "Novra Technologies Inc.", "NOVRA-TECHNOLOGIES" },
	{ 0x000677, "SICK AG", "SICK" },
	{ 0x000678, "Marantz Brand Company", "MARANTZ-BRAND-COMPANY" },
	{ 0x000679, "Konami Corporation", "KONAMI" },
	{ 0x00067A, "JMP Systems", "JMP" },
	{ 0x00067B, "Toplink C&C Corporation", "TOPLINK-C-C" },
	{ 0x00067C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00067D, "Takasago Ltd.", "TAKASAGO" },
	{ 0x00067E, "WinCom Systems, Inc.", "WINCOM" },
	{ 0x00067F, "Digeo, Inc.", "DIGEO" },
	{ 0x000680, "Card Access, Inc.", "CARD-ACCESS" },
	{ 0x000681, "Goepel Electronic GmbH", "GOEPEL" },
	{ 0x000682, "Convedia", "CONVEDIA" },
	{ 0x000683, "Bravara Communications, Inc.", "BRAVARA-COMMUNICATION" },
	{ 0x000684, "Biacore AB", "BIACORE-AB" },
	{ 0x000685, "NetNearU Corporation", "NETNEARU" },
	{ 0x000686, "ZARDCOM Co., Ltd.", "ZARDCOM" },
	{ 0x000687, "Omnitron Systems Technology, Inc.", "OMNITRON" },
	{ 0x000688, "Telways Communication Co., Ltd.", "TELWAYS-COMMUNICATION" },
	{ 0x000689, "yLez Technologies Pte Ltd", "YLEZ-TECHNOLOGIES-PTE" },
	{ 0x00068A, "NeuronNet Co. Ltd. R&D Center", "NEURONNET-R-D-CENTER" },
	{ 0x00068B, "AirRunner Technologies, Inc.", "AIRRUNNER-TECHNOLOGIES" },
	{ 0x00068C, "3Com Corporation", "3COM" },
	{ 0x00068D, "SEPATON, Inc.", "SEPATON" },
	{ 0x00068E, "HID Corporation", "HID" },
	{ 0x00068F, "Telemonitor, Inc.", "TELEMONITOR" },
	{ 0x000690, "Euracom Communication GmbH", "EURACOM-COMMUNICATION" },
	{ 0x000691, "PT Inovacao", "PT-INOVACAO" },
	{ 0x000692, "Intruvert Networks, Inc.", "INTRUVERT-NETWORKS" },
	{ 0x000693, "Flexus Computer Technology, Inc.", "FLEXUS" },
	{ 0x000694, "Mobillian Corporation", "MOBILLIAN" },
	{ 0x000695, "Ensure Technologies, Inc.", "ENSURE-TECHNOLOGIES" },
	{ 0x000696, "Advent Networks", "ADVENT-NETWORKS" },
	{ 0x000697, "R & D Center", "R-D-CENTER" },
	{ 0x000698, "egnite GmbH", "EGNITE" },
	{ 0x000699, "Vida Design Co.", "VIDA-DESIGN" },
	{ 0x00069A, "e & Tel", "E-TEL" },
	{ 0x00069B, "AVT Audio Video Technologies GmbH", "AVT-AUDIO-VIDEO-TECHNOLOGIES" },
	{ 0x00069C, "Transmode Systems AB", "TRANSMODE-AB" },
	{ 0x00069D, "Petards Ltd", "PETARDS" },
	{ 0x00069E, "UNIQA, Inc.", "UNIQA" },
	{ 0x00069F, "Kuokoa Networks", "KUOKOA-NETWORKS" },
	{ 0x0006A0, "Mx Imaging", "MX-IMAGING" },
	{ 0x0006A1, "Celsian Technologies, Inc.", "CELSIAN-TECHNOLOGIES" },
	{ 0x0006A2, "Microtune, Inc.", "MICROTUNE" },
	{ 0x0006A3, "Bitran Corporation", "BITRAN" },
	{ 0x0006A4, "INNOWELL Corp.", "INNOWELL" },
	{ 0x0006A5, "PINON Corp.", "PINON" },
	{ 0x0006A6, "Artistic Licence Engineering Ltd", "ARTISTIC-LICENCE-ENGINEERING" },
	{ 0x0006A7, "Primarion", "PRIMARION" },
	{ 0x0006A8, "KC Technology, Inc.", "KC" },
	{ 0x0006A9, "Universal Instruments Corp.", "UNIVERSAL-INSTRUMENTS" },
	{ 0x0006AA, "VT Miltope", "VT-MILTOPE" },
	{ 0x0006AB, "W-Link Systems, Inc.", "W-LINK" },
	{ 0x0006AC, "Intersoft Co.", "INTERSOFT" },
	{ 0x0006AD, "KB Electronics Ltd.", "KB-ELECTRONICS" },
	{ 0x0006AE, "Himachal Futuristic Communications Ltd", "HIMACHAL-FUTURISTIC-COMMUNICATION" },
	{ 0x0006AF, "Xalted Networks", "XALTED-NETWORKS" },
	{ 0x0006B0, "Comtech EF Data Corp.", "COMTECH-EF-DATA" },
	{ 0x0006B1, "Sonicwall", "SONICWALL" },
	{ 0x0006B2, "Linxtek Co.", "LINXTEK" },
	{ 0x0006B3, "Diagraph Corporation", "DIAGRAPH" },
	{ 0x0006B4, "Vorne Industries, Inc.", "VORNE-INDUSTRIES" },
	{ 0x0006B5, "Source Photonics, Inc.", "SOURCE-PHOTONICS" },
	{ 0x0006B6, "Nir-Or Israel Ltd.", "NIR-OR-ISRAEL" },
	{ 0x0006B7, "TELEM GmbH", "TELEM" },
	{ 0x0006B8, "Bandspeed Pty Ltd", "BANDSPEED-PTY" },
	{ 0x0006B9, "A5TEK Corp.", "A5TEK" },
	{ 0x0006BA, "Westwave Communications", "WESTWAVE-COMMUNICATION" },
	{ 0x0006BB, "ATI Technologies Inc.", "ATI-TECHNOLOGIES" },
	{ 0x0006BC, "Macrolink, Inc.", "MACROLINK" },
	{ 0x0006BD, "BNTECHNOLOGY Co., Ltd.", "BNTECHNOLOGY" },
	{ 0x0006BE, "Baumer Optronic GmbH", "BAUMER-OPTRONIC" },
	{ 0x0006BF, "Accella Technologies Co., Ltd.", "ACCELLA-TECHNOLOGIES" },
	{ 0x0006C0, "United Internetworks, Inc.", "UNITED-INTERNETWORKS" },
	{ 0x0006C1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0006C2, "Smartmatic Corporation", "SMARTMATIC" },
	{ 0x0006C3, "Schindler Elevator Ltd.", "SCHINDLER-ELEVATOR" },
	{ 0x0006C4, "Piolink Inc.", "PIOLINK" },
	{ 0x0006C5, "INNOVI Technologies Limited", "INNOVI-TECHNOLOGIES" },
	{ 0x0006C6, "lesswire AG", "LESSWIRE" },
	{ 0x0006C7, "RFNET Technologies Pte Ltd (S)", "RFNET-TECHNOLOGIES-PTE-S" },
	{ 0x0006C8, "Sumitomo Metal Micro Devices, Inc.", "SUMITOMO-METAL-MICRO-DEVICES" },
	{ 0x0006C9, "Technical Marketing Research, Inc.", "TECHNICAL-MARKETING-RESEARCH" },
	{ 0x0006CA, "American Computer & Digital Components, Inc. (ACDC)", "AMERICAN-DIGITAL-COMPONENTS-ACDC" },
	{ 0x0006CB, "Jotron Electronics A/S", "JOTRON-ELECTRONICS-A/S" },
	{ 0x0006CC, "JMI Electronics Co., Ltd.", "JMI-ELECTRONICS" },
	{ 0x0006CD, "Leaf Imaging Ltd.", "LEAF-IMAGING" },
	{ 0x0006CE, "DATENO", "DATENO" },
	{ 0x0006CF, "Thales Avionics In-Flight Systems, LLC", "THALES-AVIONICS-IN-FLIGHT-LLC" },
	{ 0x0006D0, "Elgar Electronics Corp.", "ELGAR-ELECTRONICS" },
	{ 0x0006D1, "Tahoe Networks, Inc.", "TAHOE-NETWORKS" },
	{ 0x0006D2, "Tundra Semiconductor Corp.", "TUNDRA-SEMICONDUCTOR" },
	{ 0x0006D3, "Alpha Telecom, Inc. U.S.A.", "ALPHA-TELECOM-U" },
	{ 0x0006D4, "Interactive Objects, Inc.", "INTERACTIVE-OBJECTS" },
	{ 0x0006D5, "Diamond Systems Corp.", "DIAMOND" },
	{ 0x0006D6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0006D7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0006D8, "Maple Optical Systems", "MAPLE-OPTICAL" },
	{ 0x0006D9, "IPM-Net S.p.A.", "IPM-NET-S-P-A" },
	{ 0x0006DA, "ITRAN Communications Ltd.", "ITRAN-COMMUNICATION" },
	{ 0x0006DB, "ICHIPS Co., Ltd.", "ICHIPS" },
	{ 0x0006DC, "Syabas Technology (Amquest)", "SYABAS-AMQUEST" },
	{ 0x0006DD, "AT & T Laboratories - Cambridge Ltd", "AT-T-LABORATORIES---CAMBRIDGE" },
	{ 0x0006DE, "Flash Technology", "FLASH" },
	{ 0x0006DF, "AIDONIC Corporation", "AIDONIC" },
	{ 0x0006E0, "MAT Co., Ltd.", "MAT" },
	{ 0x0006E1, "Techno Trade s.a", "TECHNO-TRADE" },
	{ 0x0006E2, "Ceemax Technology Co., Ltd.", "CEEMAX" },
	{ 0x0006E3, "Quantitative Imaging Corporation", "QUANTITATIVE-IMAGING" },
	{ 0x0006E4, "Citel Technologies Ltd.", "CITEL-TECHNOLOGIES" },
	{ 0x0006E5, "Fujian Newland Computer Ltd. Co.", "FUJIAN-NEWLAND" },
	{ 0x0006E6, "DongYang Telecom Co., Ltd.", "DONGYANG-TELECOM" },
	{ 0x0006E7, "Bit Blitz Communications Inc.", "BIT-BLITZ-COMMUNICATION" },
	{ 0x0006E8, "Optical Network Testing, Inc.", "OPTICAL-NETWORK-TESTING" },
	{ 0x0006E9, "Intime Corp.", "INTIME" },
	{ 0x0006EA, "ELZET80 Mikrocomputer GmbH&Co. KG", "ELZET80-MIKROCOMPUTER" },
	{ 0x0006EB, "Global Data", "GLOBAL-DATA" },
	{ 0x0006EC, "Harris Corporation", "HARRIS" },
	{ 0x0006ED, "Inara Networks", "INARA-NETWORKS" },
	{ 0x0006EE, "Shenyang Neu-era Information & Technology Stock Co., Ltd", "SHENYANG-NEU-ERA-INFORMATION-STOCK" },
	{ 0x0006EF, "Maxxan Systems, Inc.", "MAXXAN" },
	{ 0x0006F0, "Digeo, Inc.", "DIGEO" },
	{ 0x0006F1, "Optillion", "OPTILLION" },
	{ 0x0006F2, "Platys Communications", "PLATYS-COMMUNICATION" },
	{ 0x0006F3, "AcceLight Networks", "ACCELIGHT-NETWORKS" },
	{ 0x0006F4, "Prime Electronics & Satellitics Inc.", "PRIME-ELECTRONICS-SATELLITICS" },
	{ 0x0006F5, "ALPS Co,. Ltd.", "ALPS" },
	{ 0x0006F6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0006F7, "ALPS Co,. Ltd.", "ALPS" },
	{ 0x0006F8, "The Boeing Company", "THE-BOEING-COMPANY" },
	{ 0x0006F9, "Mitsui Zosen Systems Research Inc.", "MITSUI-ZOSEN-RESEARCH" },
	{ 0x0006FA, "IP SQUARE Co, Ltd.", "IP-SQUARE" },
	{ 0x0006FB, "Hitachi Printing Solutions, Ltd.", "HITACHI-PRINTING-SOLUTIONS" },
	{ 0x0006FC, "Fnet Co., Ltd.", "FNET" },
	{ 0x0006FD, "Comjet Information Systems Corp.", "COMJET-INFORMATION" },
	{ 0x0006FE, "Ambrado, Inc", "AMBRADO" },
	{ 0x0006FF, "Sheba Systems Co., Ltd.", "SHEBA" },
	{ 0x000700, "Zettamedia Korea", "ZETTAMEDIA-KOREA" },
	{ 0x000701, "RACAL-DATACOM", "RACAL-DATACOM" },
	{ 0x000702, "Varian Medical Systems", "VARIAN-MEDICAL" },
	{ 0x000703, "CSEE Transport", "CSEE-TRANSPORT" },
	{ 0x000704, "ALPS Co,. Ltd.", "ALPS" },
	{ 0x000705, "Endress & Hauser GmbH & Co", "ENDRESS-HAUSER" },
	{ 0x000706, "Sanritz Corporation", "SANRITZ" },
	{ 0x000707, "Interalia Inc.", "INTERALIA" },
	{ 0x000708, "Bitrage Inc.", "BITRAGE" },
	{ 0x000709, "Westerstrand Urfabrik AB", "WESTERSTRAND-URFABRIK-AB" },
	{ 0x00070A, "Unicom Automation Co., Ltd.", "UNICOM-AUTOMATION" },
	{ 0x00070B, "Novabase SGPS, SA", "NOVABASE-SGPS" },
	{ 0x00070C, "SVA-Intrusion.com Co. Ltd.", "SVA-INTRUSION-COM" },
	{ 0x00070D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00070E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00070F, "Fujant, Inc.", "FUJANT" },
	{ 0x000710, "Adax, Inc.", "ADAX" },
	{ 0x000711, "Acterna", "ACTERNA" },
	{ 0x000712, "JAL Information Technology", "JAL-INFORMATION" },
	{ 0x000713, "IP One, Inc.", "IP-ONE" },
	{ 0x000714, "Brightcom", "BRIGHTCOM" },
	{ 0x000715, "General Research of Electronics, Inc.", "GENERAL-RESEARCH-OF-ELECTRONICS" },
	{ 0x000716, "J & S Marine Ltd.", "J-S-MARINE" },
	{ 0x000717, "Wieland Electric GmbH", "WIELAND-ELECTRIC" },
	{ 0x000718, "iCanTek Co., Ltd.", "ICANTEK" },
	{ 0x000719, "Mobiis Co., Ltd.", "MOBIIS" },
	{ 0x00071A, "Finedigital Inc.", "FINEDIGITAL" },
	{ 0x00071B, "CDVI Americas Ltd", "CDVI-AMERICAS" },
	{ 0x00071C, "AT&T Fixed Wireless Services", "AT-T-FIXED-WIRELESS-SERVICES" },
	{ 0x00071D, "Satelsa Sistemas Y Aplicaciones De Telecomunicaciones, S.A.", "SATELSA-SISTEMAS-Y-APLICACIONES-DE-TELECOMUNICACIONES" },
	{ 0x00071E, "Tri-M Engineering / Nupak Dev. Corp.", "TRI-M-ENGINEERING-/-NUPAK-DEV" },
	{ 0x00071F, "European Systems Integration", "EUROPEAN-INTEGRATION" },
	{ 0x000720, "Trutzschler GmbH & Co. KG", "TRUTZSCHLER" },
	{ 0x000721, "Formac Elektronik GmbH", "FORMAC-ELEKTRONIK" },
	{ 0x000722, "The Nielsen Company", "THE-NIELSEN-COMPANY" },
	{ 0x000723, "ELCON Systemtechnik GmbH", "ELCON-SYSTEMTECHNIK" },
	{ 0x000724, "Telemax Co., Ltd.", "TELEMAX" },
	{ 0x000725, "Bematech International Corp.", "BEMATECH" },
	{ 0x000726, "Shenzhen Gongjin Electronics Co., Ltd.", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0x000727, "Zi Corporation (HK) Ltd.", "ZI-HK" },
	{ 0x000728, "Neo Telecom", "NEO-TELECOM" },
	{ 0x000729, "Kistler Instrumente AG", "KISTLER-INSTRUMENTE" },
	{ 0x00072A, "Innovance Networks", "INNOVANCE-NETWORKS" },
	{ 0x00072B, "Jung Myung Telecom Co., Ltd.", "JUNG-MYUNG-TELECOM" },
	{ 0x00072C, "Fabricom", "FABRICOM" },
	{ 0x00072D, "CNSystems", "CNSYSTEMS" },
	{ 0x00072E, "North Node AB", "NORTH-NODE-AB" },
	{ 0x00072F, "Intransa, Inc.", "INTRANSA" },
	{ 0x000730, "Hutchison OPTEL Telecom Technology Co., Ltd.", "HUTCHISON-OPTEL-TELECOM" },
	{ 0x000731, "Ophir-Spiricon LLC", "OPHIR-SPIRICON-LLC" },
	{ 0x000732, "AAEON Technology Inc.", "AAEON" },
	{ 0x000733, "DANCONTROL Engineering", "DANCONTROL-ENGINEERING" },
	{ 0x000734, "ONStor, Inc.", "ONSTOR" },
	{ 0x000735, "Flarion Technologies, Inc.", "FLARION-TECHNOLOGIES" },
	{ 0x000736, "Data Video Technologies Co., Ltd.", "DATA-VIDEO-TECHNOLOGIES" },
	{ 0x000737, "Soriya Co. Ltd.", "SORIYA" },
	{ 0x000738, "Young Technology Co., Ltd.", "YOUNG" },
	{ 0x000739, "Scotty Group Austria Gmbh", "SCOTTY-GROUP-AUSTRIA" },
	{ 0x00073A, "Inventel Systemes", "INVENTEL-SYSTEMES" },
	{ 0x00073B, "Tenovis GmbH & Co KG", "TENOVIS" },
	{ 0x00073C, "Telecom Design", "TELECOM-DESIGN" },
	{ 0x00073D, "Nanjing Postel Telecommunications Co., Ltd.", "NANJING-POSTEL-TELECOMMUNICATIONS" },
	{ 0x00073E, "China Great-Wall Computer Shenzhen Co., Ltd.", "CHINA-GREAT-WALL-SHENZHEN" },
	{ 0x00073F, "Woojyun Systec Co., Ltd.", "WOOJYUN-SYSTEC" },
	{ 0x000740, "Buffalo Inc.", "BUFFALO" },
	{ 0x000741, "Sierra Automated Systems", "SIERRA-AUTOMATED" },
	{ 0x000742, "Ormazabal", "ORMAZABAL" },
	{ 0x000743, "Chelsio Communications", "CHELSIO-COMMUNICATION" },
	{ 0x000744, "Unico, Inc.", "UNICO" },
	{ 0x000745, "Radlan Computer Communications Ltd.", "RADLAN-COMMUNICATION" },
	{ 0x000746, "TURCK, Inc.", "TURCK" },
	{ 0x000747, "Mecalc", "MECALC" },
	{ 0x000748, "The Imaging Source Europe", "THE-IMAGING-SOURCE-EUROPE" },
	{ 0x000749, "CENiX Inc.", "CENIX" },
	{ 0x00074A, "Carl Valentin GmbH", "CARL-VALENTIN" },
	{ 0x00074B, "Daihen Corporation", "DAIHEN" },
	{ 0x00074C, "Beicom Inc.", "BEICOM" },
	{ 0x00074D, "Zebra Technologies Corp.", "ZEBRA-TECHNOLOGIES" },
	{ 0x00074E, "IPFRONT Inc", "IPFRONT" },
	{ 0x00074F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000750, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000751, "m-u-t AG", "M-U-T" },
	{ 0x000752, "Rhythm Watch Co., Ltd.", "RHYTHM-WATCH" },
	{ 0x000753, "Beijing Qxcomm Technology Co., Ltd.", "BEIJING-QXCOMM" },
	{ 0x000754, "Xyterra Computing, Inc.", "XYTERRA-COMPUTING" },
	{ 0x000755, "Lafon", "LAFON" },
	{ 0x000756, "Juyoung Telecom", "JUYOUNG-TELECOM" },
	{ 0x000757, "Topcall International AG", "TOPCALL" },
	{ 0x000758, "Dragonwave", "DRAGONWAVE" },
	{ 0x000759, "Boris Manufacturing Corp.", "BORIS-MANUFACTURING" },
	{ 0x00075A, "Air Products and Chemicals, Inc.", "AIR-PRODUCTS-AND-CHEMICALS" },
	{ 0x00075B, "Gibson Guitars", "GIBSON-GUITARS" },
	{ 0x00075C, "Eastman Kodak Company", "EASTMAN-KODAK-COMPANY" },
	{ 0x00075D, "Celleritas Inc.", "CELLERITAS" },
	{ 0x00075E, "Ametek Power Instruments", "AMETEK-POWER-INSTRUMENTS" },
	{ 0x00075F, "VCS Video Communication Systems AG", "VCS-VIDEO-COMMUNICATION" },
	{ 0x000760, "TOMIS Information & Telecom Corp.", "TOMIS-INFORMATION-TELECOM" },
	{ 0x000761, "Logitech Europe SA", "LOGITECH-EUROPE" },
	{ 0x000762, "Group Sense Limited", "GROUP-SENSE" },
	{ 0x000763, "Sunniwell Cyber Tech. Co., Ltd.", "SUNNIWELL-CYBER-TECH" },
	{ 0x000764, "YoungWoo Telecom Co. Ltd.", "YOUNGWOO-TELECOM" },
	{ 0x000765, "Jade Quantum Technologies, Inc.", "JADE-QUANTUM-TECHNOLOGIES" },
	{ 0x000766, "Chou Chin Industrial Co., Ltd.", "CHOU-CHIN-INDUSTRIAL" },
	{ 0x000767, "Yuxing Electronics Company Limited", "YUXING-ELECTRONICS-COMPANY" },
	{ 0x000768, "Danfoss A/S", "DANFOSS-A/S" },
	{ 0x000769, "Italiana Macchi SpA", "ITALIANA-MACCHI-SPA" },
	{ 0x00076A, "NEXTEYE Co., Ltd.", "NEXTEYE" },
	{ 0x00076B, "Stralfors AB", "STRALFORS-AB" },
	{ 0x00076C, "Daehanet, Inc.", "DAEHANET" },
	{ 0x00076D, "Flexlight Networks", "FLEXLIGHT-NETWORKS" },
	{ 0x00076E, "Sinetica Corporation Limited", "SINETICA" },
	{ 0x00076F, "Synoptics Limited", "SYNOPTICS" },
	{ 0x000770, "Ubiquoss Inc", "UBIQUOSS" },
	{ 0x000771, "Embedded System Corporation", "EMBEDDED-SYSTEM" },
	{ 0x000772, "Alcatel Shanghai Bell Co., Ltd.", "ALCATEL-SHANGHAI-BELL" },
	{ 0x000773, "Ascom Powerline Communications Ltd.", "ASCOM-POWERLINE-COMMUNICATION" },
	{ 0x000774, "GuangZhou Thinker Technology Co. Ltd.", "GUANGZHOU-THINKER" },
	{ 0x000775, "Valence Semiconductor, Inc.", "VALENCE-SEMICONDUCTOR" },
	{ 0x000776, "Federal APD", "FEDERAL-APD" },
	{ 0x000777, "Motah Ltd.", "MOTAH" },
	{ 0x000778, "GERSTEL GmbH & Co. KG", "GERSTEL" },
	{ 0x000779, "Sungil Telecom Co., Ltd.", "SUNGIL-TELECOM" },
	{ 0x00077A, "Infoware System Co., Ltd.", "INFOWARE-SYSTEM" },
	{ 0x00077B, "Millimetrix Broadband Networks", "MILLIMETRIX-BROADBAND-NETWORKS" },
	{ 0x00077C, "Westermo Teleindustri AB", "WESTERMO-TELEINDUSTRI-AB" },
	{ 0x00077D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00077E, "Elrest GmbH", "ELREST" },
	{ 0x00077F, "J Communications Co., Ltd.", "J-COMMUNICATION" },
	{ 0x000780, "Bluegiga Technologies OY", "BLUEGIGA-TECHNOLOGIES-OY" },
	{ 0x000781, "Itron Inc.", "ITRON" },
	{ 0x000782, "Oracle Corporation", "ORACLE" },
	{ 0x000783, "SynCom Network, Inc.", "SYNCOM-NETWORK" },
	{ 0x000784, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000785, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000786, "Wireless Networks Inc.", "WIRELESS-NETWORKS" },
	{ 0x000787, "Idea System Co., Ltd.", "IDEA-SYSTEM" },
	{ 0x000788, "Clipcomm, Inc.", "CLIPCOMM" },
	{ 0x000789, "DONGWON SYSTEMS", "DONGWON" },
	{ 0x00078A, "Mentor Data System Inc.", "MENTOR-DATA-SYSTEM" },
	{ 0x00078B, "Wegener Communications, Inc.", "WEGENER-COMMUNICATION" },
	{ 0x00078C, "Elektronikspecialisten i Borlange AB", "ELEKTRONIKSPECIALISTEN-I-BORLANGE-AB" },
	{ 0x00078D, "NetEngines Ltd.", "NETENGINES" },
	{ 0x00078E, "Garz & Friche GmbH", "GARZ-FRICHE" },
	{ 0x00078F, "Emkay Innovative Products", "EMKAY-INNOVATIVE-PRODUCTS" },
	{ 0x000790, "Tri-M Technologies (s) Limited", "TRI-M-TECHNOLOGIES-S" },
	{ 0x000791, "International Data Communications, Inc.", "DATA-COMMUNICATION" },
	{ 0x000792, "Sütron Electronic GmbH", "SüTRON" },
	{ 0x000793, "Shin Satellite Public Company Limited", "SHIN-SATELLITE-PUBLIC-COMPANY" },
	{ 0x000794, "Simple Devices, Inc.", "SIMPLE-DEVICES" },
	{ 0x000795, "Elitegroup Computer System Co. (ECS)", "ELITEGROUP-SYSTEM-ECS" },
	{ 0x000796, "LSI Systems, Inc.", "LSI" },
	{ 0x000797, "Netpower Co., Ltd.", "NETPOWER" },
	{ 0x000798, "Selea SRL", "SELEA-SRL" },
	{ 0x000799, "Tipping Point Technologies, Inc.", "TIPPING-POINT-TECHNOLOGIES" },
	{ 0x00079A, "Verint Systems Inc", "VERINT" },
	{ 0x00079B, "Aurora Networks", "AURORA-NETWORKS" },
	{ 0x00079C, "Golden Electronics Technology Co., Ltd.", "GOLDEN-ELECTRONICS" },
	{ 0x00079D, "Musashi Co., Ltd.", "MUSASHI" },
	{ 0x00079E, "Ilinx Co., Ltd.", "ILINX" },
	{ 0x00079F, "Action Digital Inc.", "ACTION-DIGITAL" },
	{ 0x0007A0, "e-Watch Inc.", "E-WATCH" },
	{ 0x0007A1, "VIASYS Healthcare GmbH", "VIASYS-HEALTHCARE" },
	{ 0x0007A2, "Opteon Corporation", "OPTEON" },
	{ 0x0007A3, "Ositis Software, Inc.", "OSITIS-SOFTWARE" },
	{ 0x0007A4, "GN Netcom Ltd.", "GN-NETCOM" },
	{ 0x0007A5, "Y.D.K Co. Ltd.", "Y-D-K" },
	{ 0x0007A6, "Home Automation, Inc.", "HOME-AUTOMATION" },
	{ 0x0007A7, "A-Z Inc.", "A-Z" },
	{ 0x0007A8, "Haier Group Technologies Ltd.", "HAIER-GROUP-TECHNOLOGIES" },
	{ 0x0007A9, "Novasonics", "NOVASONICS" },
	{ 0x0007AA, "Quantum Data Inc.", "QUANTUM-DATA" },
	{ 0x0007AB, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x0007AC, "Eolring", "EOLRING" },
	{ 0x0007AD, "Pentacon GmbH Foto-und Feinwerktechnik", "PENTACON-FOTO-UND-FEINWERKTECHNIK" },
	{ 0x0007AE, "Britestream Networks, Inc.", "BRITESTREAM-NETWORKS" },
	{ 0x0007AF, "Red Lion Controls, LP", "RED-LION-CONTROLS-LP" },
	{ 0x0007B0, "Office Details, Inc.", "OFFICE-DETAILS" },
	{ 0x0007B1, "Equator Technologies", "EQUATOR-TECHNOLOGIES" },
	{ 0x0007B2, "Transaccess S.A.", "TRANSACCESS" },
	{ 0x0007B3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0007B4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0007B5, "Any One Wireless Ltd.", "ANY-ONE-WIRELESS" },
	{ 0x0007B6, "Telecom Technology Ltd.", "TELECOM" },
	{ 0x0007B7, "Samurai Ind. Prods Eletronicos Ltda", "SAMURAI-IND-PRODS-ELETRONICOS-LTDA" },
	{ 0x0007B8, "Corvalent Corporation", "CORVALENT" },
	{ 0x0007B9, "Ginganet Corporation", "GINGANET" },
	{ 0x0007BA, "UTStarcom, Inc.", "UTSTARCOM" },
	{ 0x0007BB, "Candera Inc.", "CANDERA" },
	{ 0x0007BC, "Identix Inc.", "IDENTIX" },
	{ 0x0007BD, "Radionet Ltd.", "RADIONET" },
	{ 0x0007BE, "DataLogic SpA", "DATALOGIC-SPA" },
	{ 0x0007BF, "Armillaire Technologies, Inc.", "ARMILLAIRE-TECHNOLOGIES" },
	{ 0x0007C0, "NetZerver Inc.", "NETZERVER" },
	{ 0x0007C1, "Overture Networks, Inc.", "OVERTURE-NETWORKS" },
	{ 0x0007C2, "Netsys Telecom", "NETSYS-TELECOM" },
	{ 0x0007C3, "Thomson", "THOMSON" },
	{ 0x0007C4, "JEAN Co. Ltd.", "JEAN" },
	{ 0x0007C5, "Gcom, Inc.", "GCOM" },
	{ 0x0007C6, "VDS Vosskuhler GmbH", "VDS-VOSSKUHLER" },
	{ 0x0007C7, "Synectics Systems Limited", "SYNECTICS" },
	{ 0x0007C8, "Brain21, Inc.", "BRAIN21" },
	{ 0x0007C9, "Technol Seven Co., Ltd.", "TECHNOL-SEVEN" },
	{ 0x0007CA, "Creatix Polymedia Ges Fur Kommunikaitonssysteme", "CREATIX-POLYMEDIA-GES-FUR-KOMMUNIKAITONSSYSTEME" },
	{ 0x0007CB, "Freebox SA", "FREEBOX" },
	{ 0x0007CC, "Kaba Benzing GmbH", "KABA-BENZING" },
	{ 0x0007CD, "Kumoh Electronic Co, Ltd", "KUMOH" },
	{ 0x0007CE, "Cabletime Limited", "CABLETIME" },
	{ 0x0007CF, "Anoto AB", "ANOTO-AB" },
	{ 0x0007D0, "Automat Engenharia de Automação Ltda.", "AUTOMAT-ENGENHARIA-DE-AUTOMAçãO-LTDA" },
	{ 0x0007D1, "Spectrum Signal Processing Inc.", "SPECTRUM-SIGNAL-PROCESSING" },
	{ 0x0007D2, "Logopak Systeme GmbH & Co. KG", "LOGOPAK-SYSTEME" },
	{ 0x0007D3, "SPGPrints B.V.", "SPGPRINTS" },
	{ 0x0007D4, "Zhejiang Yutong Network Communication Co Ltd.", "ZHEJIANG-YUTONG-NETWORK-COMMUNICATION" },
	{ 0x0007D5, "3e Technologies Int;., Inc.", "3E-TECHNOLOGIES-INT" },
	{ 0x0007D6, "Commil Ltd.", "COMMIL" },
	{ 0x0007D7, "Caporis Networks AG", "CAPORIS-NETWORKS" },
	{ 0x0007D8, "Hitron Systems Inc.", "HITRON" },
	{ 0x0007D9, "Splicecom", "SPLICECOM" },
	{ 0x0007DA, "Neuro Telecom Co., Ltd.", "NEURO-TELECOM" },
	{ 0x0007DB, "Kirana Networks, Inc.", "KIRANA-NETWORKS" },
	{ 0x0007DC, "Atek Co, Ltd.", "ATEK" },
	{ 0x0007DD, "Cradle Technologies", "CRADLE-TECHNOLOGIES" },
	{ 0x0007DE, "eCopilt AB", "ECOPILT-AB" },
	{ 0x0007DF, "Vbrick Systems Inc.", "VBRICK" },
	{ 0x0007E0, "Palm Inc.", "PALM" },
	{ 0x0007E1, "WIS Communications Co. Ltd.", "WIS-COMMUNICATION" },
	{ 0x0007E2, "Bitworks, Inc.", "BITWORKS" },
	{ 0x0007E3, "Navcom Technology, Inc.", "NAVCOM" },
	{ 0x0007E4, "SoftRadio Co., Ltd.", "SOFTRADIO" },
	{ 0x0007E5, "Coup Corporation", "COUP" },
	{ 0x0007E6, "edgeflow Canada Inc.", "EDGEFLOW-CANADA" },
	{ 0x0007E7, "FreeWave Technologies", "FREEWAVE-TECHNOLOGIES" },
	{ 0x0007E8, "EdgeWave", "EDGEWAVE" },
	{ 0x0007E9, "Intel Corporation", "INTEL" },
	{ 0x0007EA, "Massana, Inc.", "MASSANA" },
	{ 0x0007EB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0007EC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0007ED, "Altera Corporation", "ALTERA" },
	{ 0x0007EE, "telco Informationssysteme GmbH", "TELCO-INFORMATIONSSYSTEME" },
	{ 0x0007EF, "Lockheed Martin Tactical Systems", "LOCKHEED-MARTIN-TACTICAL" },
	{ 0x0007F0, "LogiSync LLC", "LOGISYNC-LLC" },
	{ 0x0007F1, "TeraBurst Networks Inc.", "TERABURST-NETWORKS" },
	{ 0x0007F2, "IOA Corporation", "IOA" },
	{ 0x0007F3, "Thinkengine Networks", "THINKENGINE-NETWORKS" },
	{ 0x0007F4, "Eletex Co., Ltd.", "ELETEX" },
	{ 0x0007F5, "Bridgeco Co AG", "BRIDGECO" },
	{ 0x0007F6, "Qqest Software Systems", "QQEST-SOFTWARE" },
	{ 0x0007F7, "Galtronics", "GALTRONICS" },
	{ 0x0007F8, "ITDevices, Inc.", "ITDEVICES" },
	{ 0x0007F9, "Sensaphone", "SENSAPHONE" },
	{ 0x0007FA, "ITT Co., Ltd.", "ITT" },
	{ 0x0007FB, "Giga Stream UMTS Technologies GmbH", "GIGA-STREAM-UMTS-TECHNOLOGIES" },
	{ 0x0007FC, "Adept Systems Inc.", "ADEPT" },
	{ 0x0007FD, "LANergy Ltd.", "LANERGY" },
	{ 0x0007FE, "Rigaku Corporation", "RIGAKU" },
	{ 0x0007FF, "Gluon Networks", "GLUON-NETWORKS" },
	{ 0x000800, "MULTITECH SYSTEMS, INC.", "MULTITECH" },
	{ 0x000801, "HighSpeed Surfing Inc.", "HIGHSPEED-SURFING" },
	{ 0x000802, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x000803, "Cos Tron", "COS-TRON" },
	{ 0x000804, "ICA Inc.", "ICA" },
	{ 0x000805, "Techno-Holon Corporation", "TECHNO-HOLON" },
	{ 0x000806, "Raonet Systems, Inc.", "RAONET" },
	{ 0x000807, "Access Devices Limited", "ACCESS-DEVICES" },
	{ 0x000808, "PPT Vision, Inc.", "PPT-VISION" },
	{ 0x000809, "Systemonic AG", "SYSTEMONIC" },
	{ 0x00080A, "Espera-Werke GmbH", "ESPERA-WERKE" },
	{ 0x00080B, "Birka BPA Informationssystem AB", "BIRKA-BPA-INFORMATIONSSYSTEM-AB" },
	{ 0x00080C, "VDA Elettronica spa", "VDA-ELETTRONICA-SPA" },
	{ 0x00080D, "Toshiba", "TOSHIBA" },
	{ 0x00080E, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00080F, "Proximion Fiber Optics AB", "PROXIMION-FIBER-OPTICS-AB" },
	{ 0x000810, "Key Technology, Inc.", "KEY" },
	{ 0x000811, "VOIX Corporation", "VOIX" },
	{ 0x000812, "GM-2 Corporation", "GM-2" },
	{ 0x000813, "Diskbank, Inc.", "DISKBANK" },
	{ 0x000814, "TIL Technologies", "TIL-TECHNOLOGIES" },
	{ 0x000815, "CATS Co., Ltd.", "CATS" },
	{ 0x000816, "Bluelon ApS", "BLUELON-APS" },
	{ 0x000817, "EmergeCore Networks LLC", "EMERGECORE-NETWORKS-LLC" },
	{ 0x000818, "Pixelworks, Inc.", "PIXELWORKS" },
	{ 0x000819, "Banksys", "BANKSYS" },
	{ 0x00081A, "Sanrad Intelligence Storage Communications (2000) Ltd.", "SANRAD-INTELLIGENCE-STORAGE-COMMUNICATION-2000" },
	{ 0x00081B, "Windigo Systems", "WINDIGO" },
	{ 0x00081C, "@pos.com", "@POS-COM" },
	{ 0x00081D, "Ipsil, Incorporated", "IPSIL-INCORPORATED" },
	{ 0x00081E, "Repeatit AB", "REPEATIT-AB" },
	{ 0x00081F, "Pou Yuen Tech Corp. Ltd.", "POU-YUEN-TECH" },
	{ 0x000820, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000821, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000822, "InPro Comm", "INPRO-COMM" },
	{ 0x000823, "Texa Corp.", "TEXA" },
	{ 0x000824, "Nuance Document Imaging", "NUANCE-DOCUMENT-IMAGING" },
	{ 0x000825, "Acme Packet", "ACME-PACKET" },
	{ 0x000826, "Colorado Med Tech", "COLORADO-MED-TECH" },
	{ 0x000827, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x000828, "Koei Engineering Ltd.", "KOEI-ENGINEERING" },
	{ 0x000829, "Aval Nagasaki Corporation", "AVAL-NAGASAKI" },
	{ 0x00082A, "Powerwallz Network Security", "POWERWALLZ-NETWORK-SECURITY" },
	{ 0x00082B, "Wooksung Electronics, Inc.", "WOOKSUNG-ELECTRONICS" },
	{ 0x00082C, "Homag AG", "HOMAG" },
	{ 0x00082D, "Indus Teqsite Private Limited", "INDUS-TEQSITE-PRIVATE" },
	{ 0x00082E, "Multitone Electronics PLC", "MULTITONE-ELECTRONICS-PLC" },
	{ 0x00082F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000830, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000831, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000832, "Cisco", "CISCO" },
	{ 0x00084E, "DivergeNet, Inc.", "DIVERGENET" },
	{ 0x00084F, "Qualstar Corporation", "QUALSTAR" },
	{ 0x000850, "Arizona Instrument Corp.", "ARIZONA-INSTRUMENT" },
	{ 0x000851, "Canadian Bank Note Company, Ltd.", "CANADIAN-BANK-NOTE-COMPANY" },
	{ 0x000852, "Davolink Co. Inc.", "DAVOLINK" },
	{ 0x000853, "Schleicher GmbH & Co. Relaiswerke KG", "SCHLEICHER-RELAISWERKE" },
	{ 0x000854, "Netronix, Inc.", "NETRONIX" },
	{ 0x000855, "NASA-Goddard Space Flight Center", "NASA-GODDARD-SPACE-FLIGHT-CENTER" },
	{ 0x000856, "Gamatronic Electronic Industries Ltd.", "GAMATRONIC-INDUSTRIES" },
	{ 0x000857, "Polaris Networks, Inc.", "POLARIS-NETWORKS" },
	{ 0x000858, "Novatechnology Inc.", "NOVATECHNOLOGY" },
	{ 0x000859, "ShenZhen Unitone Electronics Co., Ltd.", "SHENZHEN-UNITONE-ELECTRONICS" },
	{ 0x00085A, "IntiGate Inc.", "INTIGATE" },
	{ 0x00085B, "Hanbit Electronics Co., Ltd.", "HANBIT-ELECTRONICS" },
	{ 0x00085C, "Shanghai Dare Technologies Co. Ltd.", "SHANGHAI-DARE-TECHNOLOGIES" },
	{ 0x00085D, "Aastra", "AASTRA" },
	{ 0x00085E, "PCO AG", "PCO" },
	{ 0x00085F, "Picanol N.V.", "PICANOL-N-V" },
	{ 0x000860, "LodgeNet Entertainment Corp.", "LODGENET-ENTERTAINMENT" },
	{ 0x000861, "SoftEnergy Co., Ltd.", "SOFTENERGY" },
	{ 0x000862, "NEC Eluminant Technologies, Inc.", "NEC-ELUMINANT-TECHNOLOGIES" },
	{ 0x000863, "Entrisphere Inc.", "ENTRISPHERE" },
	{ 0x000864, "Fasy S.p.A.", "FASY-S-P-A" },
	{ 0x000865, "JASCOM CO., LTD", "JASCOM" },
	{ 0x000866, "DSX Access Systems, Inc.", "DSX-ACCESS" },
	{ 0x000867, "Uptime Devices", "UPTIME-DEVICES" },
	{ 0x000868, "PurOptix", "PUROPTIX" },
	{ 0x000869, "Command-e Technology Co.,Ltd.", "COMMAND-E" },
	{ 0x00086A, "Securiton Gmbh", "SECURITON" },
	{ 0x00086B, "MIPSYS", "MIPSYS" },
	{ 0x00086C, "Plasmon LMS", "PLASMON-LMS" },
	{ 0x00086D, "Missouri FreeNet", "MISSOURI-FREENET" },
	{ 0x00086E, "Hyglo AB", "HYGLO-AB" },
	{ 0x00086F, "Resources Computer Network Ltd.", "RESOURCES-NETWORK" },
	{ 0x000870, "Rasvia Systems, Inc.", "RASVIA" },
	{ 0x000871, "NORTHDATA Co., Ltd.", "NORTHDATA" },
	{ 0x000872, "Sorenson Communications", "SORENSON-COMMUNICATION" },
	{ 0x000873, "DapTechnology B.V.", "DAPTECHNOLOGY" },
	{ 0x000874, "Dell Computer Corp.", "DELL" },
	{ 0x000875, "Acorp Electronics Corp.", "ACORP-ELECTRONICS" },
	{ 0x000876, "SDSystem", "SDSYSTEM" },
	{ 0x000877, "Liebert-Hiross Spa", "LIEBERT-HIROSS-SPA" },
	{ 0x000878, "Benchmark Storage Innovations", "BENCHMARK-STORAGE-INNOVATIONS" },
	{ 0x000879, "CEM Corporation", "CEM" },
	{ 0x00087A, "Wipotec GmbH", "WIPOTEC" },
	{ 0x00087B, "RTX Telecom A/S", "RTX-TELECOM-A/S" },
	{ 0x00087C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00087D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00087E, "Bon Electro-Telecom Inc.", "BON-ELECTRO-TELECOM" },
	{ 0x00087F, "SPAUN electronic GmbH & Co. KG", "SPAUN" },
	{ 0x000880, "BroadTel Canada Communications inc.", "BROADTEL-CANADA-COMMUNICATION" },
	{ 0x000881, "DIGITAL HANDS CO.,LTD.", "DIGITAL-HANDS" },
	{ 0x000882, "SIGMA CORPORATION", "SIGMA" },
	{ 0x000883, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x000884, "Index Braille AB", "INDEX-BRAILLE-AB" },
	{ 0x000885, "EMS Dr. Thomas Wünsche", "EMS-DR-THOMAS-WüNSCHE" },
	{ 0x000886, "Hansung Teliann, Inc.", "HANSUNG-TELIANN" },
	{ 0x000887, "Maschinenfabrik Reinhausen GmbH", "MASCHINENFABRIK-REINHAUSEN" },
	{ 0x000888, "OULLIM Information Technology Inc,.", "OULLIM-INFORMATION" },
	{ 0x000889, "Echostar Technologies Corp", "ECHOSTAR-TECHNOLOGIES" },
	{ 0x00088A, "Minds@Work", "MINDS@WORK" },
	{ 0x00088B, "Tropic Networks Inc.", "TROPIC-NETWORKS" },
	{ 0x00088C, "Quanta Network Systems Inc.", "QUANTA-NETWORK" },
	{ 0x00088D, "Sigma-Links Inc.", "SIGMA-LINKS" },
	{ 0x00088E, "Nihon Computer Co., Ltd.", "NIHON" },
	{ 0x00088F, "ADVANCED DIGITAL TECHNOLOGY", "ADVANCED-DIGITAL" },
	{ 0x000890, "AVILINKS SA", "AVILINKS" },
	{ 0x000891, "Lyan Inc.", "LYAN" },
	{ 0x000892, "EM Solutions", "EM-SOLUTIONS" },
	{ 0x000893, "LE INFORMATION COMMUNICATION INC.", "LE-INFORMATION-COMMUNICATION" },
	{ 0x000894, "InnoVISION Multimedia Ltd.", "INNOVISION-MULTIMEDIA" },
	{ 0x000895, "DIRC Technologie GmbH & Co.KG", "DIRC-TECHNOLOGIE" },
	{ 0x000896, "Printronix, Inc.", "PRINTRONIX" },
	{ 0x000897, "Quake Technologies", "QUAKE-TECHNOLOGIES" },
	{ 0x000898, "Gigabit Optics Corporation", "GIGABIT-OPTICS" },
	{ 0x000899, "Netbind, Inc.", "NETBIND" },
	{ 0x00089A, "Alcatel Microelectronics", "ALCATEL-MICROELECTRONICS" },
	{ 0x00089B, "ICP Electronics Inc.", "ICP-ELECTRONICS" },
	{ 0x00089C, "Elecs Industry Co., Ltd.", "ELECS-INDUSTRY" },
	{ 0x00089D, "UHD-Elektronik", "UHD-ELEKTRONIK" },
	{ 0x00089E, "Beijing Enter-Net co.LTD", "BEIJING-ENTER-NET" },
	{ 0x00089F, "EFM Networks", "EFM-NETWORKS" },
	{ 0x0008A0, "Stotz Feinmesstechnik GmbH", "STOTZ-FEINMESSTECHNIK" },
	{ 0x0008A1, "CNet Technology Inc.", "CNET" },
	{ 0x0008A2, "ADI Engineering, Inc.", "ADI-ENGINEERING" },
	{ 0x0008A3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0008A4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0008A5, "Peninsula Systems Inc.", "PENINSULA" },
	{ 0x0008A6, "Multiware & Image Co., Ltd.", "MULTIWARE-IMAGE" },
	{ 0x0008A7, "iLogic Inc.", "ILOGIC" },
	{ 0x0008A8, "Systec Co., Ltd.", "SYSTEC" },
	{ 0x0008A9, "SangSang Technology, Inc.", "SANGSANG" },
	{ 0x0008AA, "KARAM", "KARAM" },
	{ 0x0008AB, "EnerLinx.com, Inc.", "ENERLINX-COM" },
	{ 0x0008AC, "Eltromat GmbH", "ELTROMAT" },
	{ 0x0008AD, "Toyo-Linx Co., Ltd.", "TOYO-LINX" },
	{ 0x0008AE, "PacketFront Network Products AB", "PACKETFRONT-NETWORK-PRODUCTS-AB" },
	{ 0x0008AF, "Novatec Corporation", "NOVATEC" },
	{ 0x0008B0, "BKtel communications GmbH", "BKTEL-COMMUNICATION" },
	{ 0x0008B1, "ProQuent Systems", "PROQUENT" },
	{ 0x0008B2, "SHENZHEN COMPASS TECHNOLOGY DEVELOPMENT CO.,LTD", "SHENZHEN-COMPASS-DEVELOPMENT" },
	{ 0x0008B3, "Fastwel", "FASTWEL" },
	{ 0x0008B4, "SYSPOL", "SYSPOL" },
	{ 0x0008B5, "TAI GUEN ENTERPRISE CO., LTD", "TAI-GUEN" },
	{ 0x0008B6, "RouteFree, Inc.", "ROUTEFREE" },
	{ 0x0008B7, "HIT Incorporated", "HIT-INCORPORATED" },
	{ 0x0008B8, "E.F. Johnson", "E-F-JOHNSON" },
	{ 0x0008B9, "KAON MEDIA Co., Ltd.", "KAON-MEDIA" },
	{ 0x0008BA, "Erskine Systems Ltd", "ERSKINE" },
	{ 0x0008BB, "NetExcell", "NETEXCELL" },
	{ 0x0008BC, "Ilevo AB", "ILEVO-AB" },
	{ 0x0008BD, "TEPG-US", "TEPG-US" },
	{ 0x0008BE, "XENPAK MSA Group", "XENPAK-MSA-GROUP" },
	{ 0x0008BF, "Aptus Elektronik AB", "APTUS-ELEKTRONIK-AB" },
	{ 0x0008C0, "ASA SYSTEMS", "ASA" },
	{ 0x0008C1, "Avistar Communications Corporation", "AVISTAR-COMMUNICATION" },
	{ 0x0008C2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0008C3, "Contex A/S", "CONTEX-A/S" },
	{ 0x0008C4, "Hikari Co.,Ltd.", "HIKARI" },
	{ 0x0008C5, "Liontech Co., Ltd.", "LIONTECH" },
	{ 0x0008C6, "Philips Consumer Communications", "PHILIPS-CONSUMER-COMMUNICATION" },
	{ 0x0008C7, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0008C8, "Soneticom, Inc.", "SONETICOM" },
	{ 0x0008C9, "TechniSat Digital GmbH", "TECHNISAT-DIGITAL" },
	{ 0x0008CA, "TwinHan Technology Co.,Ltd", "TWINHAN" },
	{ 0x0008CB, "Zeta Broadband Inc.", "ZETA-BROADBAND" },
	{ 0x0008CC, "Remotec, Inc.", "REMOTEC" },
	{ 0x0008CD, "With-Net Inc", "WITH-NET" },
	{ 0x0008CE, "IPMobileNet Inc.", "IPMOBILENET" },
	{ 0x0008CF, "Nippon Koei Power Systems Co., Ltd.", "NIPPON-KOEI-POWER" },
	{ 0x0008D0, "Musashi Engineering Co., LTD.", "MUSASHI-ENGINEERING" },
	{ 0x0008D1, "KAREL INC.", "KAREL" },
	{ 0x0008D2, "ZOOM Networks Inc.", "ZOOM-NETWORKS" },
	{ 0x0008D3, "Hercules Technologies S.A.S.", "HERCULES-TECHNOLOGIES-S" },
	{ 0x0008D4, "IneoQuest Technologies, Inc", "INEOQUEST-TECHNOLOGIES" },
	{ 0x0008D5, "Vanguard Networks Solutions, LLC", "VANGUARD-NETWORKS-SOLUTIONS-LLC" },
	{ 0x0008D6, "HASSNET Inc.", "HASSNET" },
	{ 0x0008D7, "HOW CORPORATION", "HOW" },
	{ 0x0008D8, "Dowkey Microwave", "DOWKEY-MICROWAVE" },
	{ 0x0008D9, "Mitadenshi Co.,LTD", "MITADENSHI" },
	{ 0x0008DA, "SofaWare Technologies Ltd.", "SOFAWARE-TECHNOLOGIES" },
	{ 0x0008DB, "Corrigent Systems", "CORRIGENT" },
	{ 0x0008DC, "Wiznet", "WIZNET" },
	{ 0x0008DD, "Telena Communications, Inc.", "TELENA-COMMUNICATION" },
	{ 0x0008DE, "3UP Systems", "3UP" },
	{ 0x0008DF, "Alistel Inc.", "ALISTEL" },
	{ 0x0008E0, "ATO Technology Ltd.", "ATO" },
	{ 0x0008E1, "Barix AG", "BARIX" },
	{ 0x0008E2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0008E3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0008E4, "Envenergy Inc", "ENVENERGY" },
	{ 0x0008E5, "IDK Corporation", "IDK" },
	{ 0x0008E6, "Littlefeet", "LITTLEFEET" },
	{ 0x0008E7, "SHI ControlSystems,Ltd.", "SHI-CONTROLSYSTEMS" },
	{ 0x0008E8, "Excel Master Ltd.", "EXCEL-MASTER" },
	{ 0x0008E9, "NextGig", "NEXTGIG" },
	{ 0x0008EA, "Motion Control Engineering, Inc", "MOTION-CONTROL-ENGINEERING" },
	{ 0x0008EB, "ROMWin Co.,Ltd.", "ROMWIN" },
	{ 0x0008EC, "Optical Zonu Corporation", "OPTICAL-ZONU" },
	{ 0x0008ED, "ST&T Instrument Corp.", "ST-T-INSTRUMENT" },
	{ 0x0008EE, "Logic Product Development", "LOGIC-PRODUCT-DEVELOPMENT" },
	{ 0x0008EF, "DIBAL,S.A.", "DIBAL" },
	{ 0x0008F0, "Next Generation Systems, Inc.", "NEXT-GENERATION" },
	{ 0x0008F1, "Voltaire", "VOLTAIRE" },
	{ 0x0008F2, "C&S Technology", "C-S" },
	{ 0x0008F3, "WANY", "WANY" },
	{ 0x0008F4, "Bluetake Technology Co., Ltd.", "BLUETAKE" },
	{ 0x0008F5, "YESTECHNOLOGY Co.,Ltd.", "YESTECHNOLOGY" },
	{ 0x0008F6, "Sumitomo Electric System Solutions Co., Ltd.", "SUMITOMO-ELECTRIC-SYSTEM-SOLUTIONS" },
	{ 0x0008F7, "Hitachi Ltd, Semiconductor & Integrated Circuits Gr", "HITACHI-SEMICONDUCTOR-INTEGRATED-CIRCUITS-GR" },
	{ 0x0008F8, "UTC CCS", "UTC-CCS" },
	{ 0x0008F9, "Artesyn Embedded Technologies", "ARTESYN-EMBEDDED-TECHNOLOGIES" },
	{ 0x0008FA, "Karl E.Brinkmann GmbH", "KARL-E-BRINKMANN" },
	{ 0x0008FB, "SonoSite, Inc.", "SONOSITE" },
	{ 0x0008FC, "Gigaphoton Inc.", "GIGAPHOTON" },
	{ 0x0008FD, "BlueKorea Co., Ltd.", "BLUEKOREA" },
	{ 0x0008FE, "UNIK C&C Co.,Ltd.", "UNIK-C-C" },
	{ 0x0008FF, "Trilogy Communications Ltd", "TRILOGY-COMMUNICATION" },
	{ 0x000900, "TMT", "TMT" },
	{ 0x000901, "Shenzhen Shixuntong Information & Technoligy Co", "SHENZHEN-SHIXUNTONG-INFORMATION-TECHNOLIGY" },
	{ 0x000902, "Redline Communications Inc.", "REDLINE-COMMUNICATION" },
	{ 0x000903, "Panasas, Inc", "PANASAS" },
	{ 0x000904, "MONDIAL electronic", "MONDIAL" },
	{ 0x000905, "iTEC Technologies Ltd.", "ITEC-TECHNOLOGIES" },
	{ 0x000906, "Esteem Networks", "ESTEEM-NETWORKS" },
	{ 0x000907, "Chrysalis Development", "CHRYSALIS-DEVELOPMENT" },
	{ 0x000908, "VTech Technology Corp.", "VTECH" },
	{ 0x000909, "Telenor Connect A/S", "TELENOR-CONNECT-A/S" },
	{ 0x00090A, "SnedFar Technology Co., Ltd.", "SNEDFAR" },
	{ 0x00090B, "MTL Instruments PLC", "MTL-INSTRUMENTS-PLC" },
	{ 0x00090C, "Mayekawa Mfg. Co. Ltd.", "MAYEKAWA-MFG" },
	{ 0x00090D, "LEADER ELECTRONICS CORP.", "LEADER-ELECTRONICS" },
	{ 0x00090E, "Helix Technology Inc.", "HELIX" },
	{ 0x00090F, "Fortinet Inc.", "FORTINET" },
	{ 0x000910, "Simple Access Inc.", "SIMPLE-ACCESS" },
	{ 0x000911, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000912, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000913, "SystemK Corporation", "SYSTEMK" },
	{ 0x000914, "COMPUTROLS INC.", "COMPUTROLS" },
	{ 0x000915, "CAS Corp.", "CAS" },
	{ 0x000916, "Listman Home Technologies, Inc.", "LISTMAN-HOME-TECHNOLOGIES" },
	{ 0x000917, "WEM Technology Inc", "WEM" },
	{ 0x000918, "SAMSUNG TECHWIN CO.,LTD", "SAMSUNG-TECHWIN" },
	{ 0x000919, "MDS Gateways", "MDS-GATEWAYS" },
	{ 0x00091A, "Macat Optics & Electronics Co., Ltd.", "MACAT-OPTICS-ELECTRONICS" },
	{ 0x00091B, "Digital Generation Inc.", "DIGITAL-GENERATION" },
	{ 0x00091C, "CacheVision, Inc", "CACHEVISION" },
	{ 0x00091D, "Proteam Computer Corporation", "PROTEAM" },
	{ 0x00091E, "Firstech Technology Corp.", "FIRSTECH" },
	{ 0x00091F, "A&D Co., Ltd.", "A-D" },
	{ 0x000920, "EpoX COMPUTER CO.,LTD.", "EPOX" },
	{ 0x000921, "Planmeca Oy", "PLANMECA-OY" },
	{ 0x000922, "TST Biometrics GmbH", "TST-BIOMETRICS" },
	{ 0x000923, "Heaman System Co., Ltd", "HEAMAN-SYSTEM" },
	{ 0x000924, "Telebau GmbH", "TELEBAU" },
	{ 0x000925, "VSN Systemen BV", "VSN-SYSTEMEN" },
	{ 0x000926, "YODA COMMUNICATIONS, INC.", "YODA-COMMUNICATION" },
	{ 0x000927, "TOYOKEIKI CO.,LTD.", "TOYOKEIKI" },
	{ 0x000928, "Telecore", "TELECORE" },
	{ 0x000929, "Sanyo Industries (UK) Limited", "SANYO-INDUSTRIES-UK" },
	{ 0x00092A, "MYTECS Co.,Ltd.", "MYTECS" },
	{ 0x00092B, "iQstor Networks, Inc.", "IQSTOR-NETWORKS" },
	{ 0x00092C, "Hitpoint Inc.", "HITPOINT" },
	{ 0x00092D, "HTC Corporation", "HTC" },
	{ 0x00092E, "B&Tech System Inc.", "B-TECH-SYSTEM" },
	{ 0x00092F, "Akom Technology Corporation", "AKOM" },
	{ 0x000930, "AeroConcierge Inc.", "AEROCONCIERGE" },
	{ 0x000931, "Future Internet, Inc.", "FUTURE-INTERNET" },
	{ 0x000932, "Omnilux", "OMNILUX" },
	{ 0x000933, "Ophit Co.Ltd.", "OPHIT" },
	{ 0x000934, "Dream-Multimedia-Tv GmbH", "DREAM-MULTIMEDIA-TV" },
	{ 0x000935, "Sandvine Incorporated", "SANDVINE-INCORPORATED" },
	{ 0x000936, "Ipetronik GmbH & Co. KG", "IPETRONIK" },
	{ 0x000937, "Inventec Appliance Corp", "INVENTEC-APPLIANCE" },
	{ 0x000938, "Allot Communications", "ALLOT-COMMUNICATION" },
	{ 0x000939, "ShibaSoku Co.,Ltd.", "SHIBASOKU" },
	{ 0x00093A, "Molex Fiber Optics", "MOLEX-FIBER-OPTICS" },
	{ 0x00093B, "HYUNDAI NETWORKS INC.", "HYUNDAI-NETWORKS" },
	{ 0x00093C, "Jacques Technologies P/L", "JACQUES-TECHNOLOGIES-P/L" },
	{ 0x00093D, "Newisys,Inc.", "NEWISYS" },
	{ 0x00093E, "C&I Technologies", "C-I-TECHNOLOGIES" },
	{ 0x00093F, "Double-Win Enterpirse CO., LTD", "DOUBLE-WIN-ENTERPIRSE" },
	{ 0x000940, "AGFEO GmbH & Co. KG", "AGFEO" },
	{ 0x000941, "Allied Telesis K.K.", "ALLIED-TELESIS-K-K" },
	{ 0x000942, "Wireless Technologies, Inc", "WIRELESS-TECHNOLOGIES" },
	{ 0x000943, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000944, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000945, "Palmmicro Communications Inc", "PALMMICRO-COMMUNICATION" },
	{ 0x000946, "Cluster Labs GmbH", "CLUSTER-LABS" },
	{ 0x000947, "Aztek, Inc.", "AZTEK" },
	{ 0x000948, "Vista Control Systems, Corp.", "VISTA-CONTROL" },
	{ 0x000949, "Glyph Technologies Inc.", "GLYPH-TECHNOLOGIES" },
	{ 0x00094A, "Homenet Communications", "HOMENET-COMMUNICATION" },
	{ 0x00094B, "FillFactory NV", "FILLFACTORY-NV" },
	{ 0x00094C, "Communication Weaver Co.,Ltd.", "COMMUNICATION-WEAVER" },
	{ 0x00094D, "Braintree Communications Pty Ltd", "BRAINTREE-COMMUNICATION-PTY" },
	{ 0x00094E, "BARTECH SYSTEMS INTERNATIONAL, INC", "BARTECH" },
	{ 0x00094F, "elmegt GmbH & Co. KG", "ELMEGT" },
	{ 0x000950, "Independent Storage Corporation", "INDEPENDENT-STORAGE" },
	{ 0x000951, "Apogee Imaging Systems", "APOGEE-IMAGING" },
	{ 0x000952, "Auerswald GmbH & Co. KG", "AUERSWALD" },
	{ 0x000953, "Linkage System Integration Co.Ltd.", "LINKAGE-SYSTEM-INTEGRATION" },
	{ 0x000954, "AMiT spol. s. r. o.", "AMIT-SPOL-S-R-O" },
	{ 0x000955, "Young Generation International Corp.", "YOUNG-GENERATION" },
	{ 0x000956, "Network Systems Group, Ltd. (NSG)", "NETWORK-GROUP-NSG" },
	{ 0x000957, "Supercaller, Inc.", "SUPERCALLER" },
	{ 0x000958, "INTELNET S.A.", "INTELNET" },
	{ 0x000959, "Sitecsoft", "SITECSOFT" },
	{ 0x00095A, "RACEWOOD TECHNOLOGY", "RACEWOOD" },
	{ 0x00095B, "Netgear, Inc.", "NETGEAR" },
	{ 0x00095C, "Philips Medical Systems - Cardiac and Monitoring Systems (CM", "PHILIPS-MEDICAL---CARDIAC-AND-MONITORING-CM" },
	{ 0x00095D, "Dialogue Technology Corp.", "DIALOGUE" },
	{ 0x00095E, "Masstech Group Inc.", "MASSTECH-GROUP" },
	{ 0x00095F, "Telebyte, Inc.", "TELEBYTE" },
	{ 0x000960, "YOZAN Inc.", "YOZAN" },
	{ 0x000961, "Switchgear and Instrumentation Ltd", "SWITCHGEAR-AND-INSTRUMENTATION" },
	{ 0x000962, "Sonitor Technologies AS", "SONITOR-TECHNOLOGIES-AS" },
	{ 0x000963, "Dominion Lasercom Inc.", "DOMINION-LASERCOM" },
	{ 0x000964, "Hi-Techniques, Inc.", "HI-TECHNIQUES" },
	{ 0x000965, "HyunJu Computer Co., Ltd.", "HYUNJU" },
	{ 0x000966, "Thales Navigation", "THALES-NAVIGATION" },
	{ 0x000967, "Tachyon, Inc", "TACHYON" },
	{ 0x000968, "TECHNOVENTURE, INC.", "TECHNOVENTURE" },
	{ 0x000969, "Meret Optical Communications", "MERET-OPTICAL-COMMUNICATION" },
	{ 0x00096A, "Cloverleaf Communications Inc.", "CLOVERLEAF-COMMUNICATION" },
	{ 0x00096B, "IBM Corp", "IBM" },
	{ 0x00096C, "Imedia Semiconductor Corp.", "IMEDIA-SEMICONDUCTOR" },
	{ 0x00096D, "Powernet Technologies Corp.", "POWERNET-TECHNOLOGIES" },
	{ 0x00096E, "GIANT ELECTRONICS LTD.", "GIANT-ELECTRONICS" },
	{ 0x00096F, "Beijing Zhongqing Elegant Tech. Corp.,Limited", "BEIJING-ZHONGQING-ELEGANT-TECH" },
	{ 0x000970, "Vibration Research Corporation", "VIBRATION-RESEARCH" },
	{ 0x000971, "Time Management, Inc.", "TIME-MANAGEMENT" },
	{ 0x000972, "Securebase,Inc", "SECUREBASE" },
	{ 0x000973, "Lenten Technology Co., Ltd.", "LENTEN" },
	{ 0x000974, "Innopia Technologies, Inc.", "INNOPIA-TECHNOLOGIES" },
	{ 0x000975, "fSONA Communications Corporation", "FSONA-COMMUNICATION" },
	{ 0x000976, "Datasoft ISDN Systems GmbH", "DATASOFT-ISDN" },
	{ 0x000977, "Brunner Elektronik AG", "BRUNNER-ELEKTRONIK" },
	{ 0x000978, "AIJI System Co., Ltd.", "AIJI-SYSTEM" },
	{ 0x000979, "Advanced Television Systems Committee, Inc.", "ADVANCED-TELEVISION-COMMITTEE" },
	{ 0x00097A, "Louis Design Labs.", "LOUIS-DESIGN-LABS" },
	{ 0x00097B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00097C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00097D, "SecWell Networks Oy", "SECWELL-NETWORKS-OY" },
	{ 0x00097E, "IMI TECHNOLOGY CO., LTD", "IMI" },
	{ 0x00097F, "Vsecure 2000 LTD.", "VSECURE-2000" },
	{ 0x000980, "Power Zenith Inc.", "POWER-ZENITH" },
	{ 0x000981, "Newport Networks", "NEWPORT-NETWORKS" },
	{ 0x000982, "Loewe Opta GmbH", "LOEWE-OPTA" },
	{ 0x000983, "GlobalTop Technology, Inc.", "GLOBALTOP" },
	{ 0x000984, "MyCasa Network Inc.", "MYCASA-NETWORK" },
	{ 0x000985, "Auto Telecom Company", "AUTO-TELECOM-COMPANY" },
	{ 0x000986, "Metalink LTD.", "METALINK" },
	{ 0x000987, "NISHI NIPPON ELECTRIC WIRE & CABLE CO.,LTD.", "NISHI-NIPPON-ELECTRIC-WIRE-CABLE" },
	{ 0x000988, "Nudian Electron Co., Ltd.", "NUDIAN-ELECTRON" },
	{ 0x000989, "VividLogic Inc.", "VIVIDLOGIC" },
	{ 0x00098A, "EqualLogic Inc", "EQUALLOGIC" },
	{ 0x00098B, "Entropic Communications, Inc.", "ENTROPIC-COMMUNICATION" },
	{ 0x00098C, "Option Wireless Sweden", "OPTION-WIRELESS-SWEDEN" },
	{ 0x00098D, "Velocity Semiconductor", "VELOCITY-SEMICONDUCTOR" },
	{ 0x00098E, "ipcas GmbH", "IPCAS" },
	{ 0x00098F, "Cetacean Networks", "CETACEAN-NETWORKS" },
	{ 0x000990, "ACKSYS Communications & systems", "ACKSYS-COMMUNICATION" },
	{ 0x000991, "GE Fanuc Automation Manufacturing, Inc.", "GE-FANUC-AUTOMATION-MANUFACTURING" },
	{ 0x000992, "InterEpoch Technology,INC.", "INTEREPOCH" },
	{ 0x000993, "Visteon Corporation", "VISTEON" },
	{ 0x000994, "Cronyx Engineering", "CRONYX-ENGINEERING" },
	{ 0x000995, "Castle Technology Ltd", "CASTLE" },
	{ 0x000996, "RDI", "RDI" },
	{ 0x000997, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000998, "Capinfo Company Limited", "CAPINFO-COMPANY" },
	{ 0x000999, "CP GEORGES RENAULT", "CP-GEORGES-RENAULT" },
	{ 0x00099A, "ELMO COMPANY, LIMITED", "ELMO-COMPANY" },
	{ 0x00099B, "Western Telematic Inc.", "WESTERN-TELEMATIC" },
	{ 0x00099C, "Naval Research Laboratory", "NAVAL-RESEARCH-LABORATORY" },
	{ 0x00099D, "Haliplex Communications", "HALIPLEX-COMMUNICATION" },
	{ 0x00099E, "Testech, Inc.", "TESTECH" },
	{ 0x00099F, "VIDEX INC.", "VIDEX" },
	{ 0x0009A0, "Microtechno Corporation", "MICROTECHNO" },
	{ 0x0009A1, "Telewise Communications, Inc.", "TELEWISE-COMMUNICATION" },
	{ 0x0009A2, "Interface Co., Ltd.", "INTERFACE" },
	{ 0x0009A3, "Leadfly Techologies Corp. Ltd.", "LEADFLY-TECHOLOGIES" },
	{ 0x0009A4, "HARTEC Corporation", "HARTEC" },
	{ 0x0009A5, "HANSUNG ELETRONIC INDUSTRIES DEVELOPMENT CO., LTD", "HANSUNG-ELETRONIC-INDUSTRIES-DEVELOPMENT" },
	{ 0x0009A6, "Ignis Optics, Inc.", "IGNIS-OPTICS" },
	{ 0x0009A7, "Bang & Olufsen A/S", "BANG-OLUFSEN-A/S" },
	{ 0x0009A8, "Eastmode Pte Ltd", "EASTMODE-PTE" },
	{ 0x0009A9, "Ikanos Communications", "IKANOS-COMMUNICATION" },
	{ 0x0009AA, "Data Comm for Business, Inc.", "DATA-COMM-FOR-BUSINESS" },
	{ 0x0009AB, "Netcontrol Oy", "NETCONTROL-OY" },
	{ 0x0009AC, "LANVOICE", "LANVOICE" },
	{ 0x0009AD, "HYUNDAI SYSCOMM, INC.", "HYUNDAI-SYSCOMM" },
	{ 0x0009AE, "OKANO ELECTRIC CO.,LTD", "OKANO-ELECTRIC" },
	{ 0x0009AF, "e-generis", "E-GENERIS" },
	{ 0x0009B0, "Onkyo Corporation", "ONKYO" },
	{ 0x0009B1, "Kanematsu Electronics, Ltd.", "KANEMATSU-ELECTRONICS" },
	{ 0x0009B2, "L&F Inc.", "L-F" },
	{ 0x0009B3, "MCM Systems Ltd", "MCM" },
	{ 0x0009B4, "KISAN TELECOM CO., LTD.", "KISAN-TELECOM" },
	{ 0x0009B5, "3J Tech. Co., Ltd.", "3J-TECH" },
	{ 0x0009B6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0009B7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0009B8, "Entise Systems", "ENTISE" },
	{ 0x0009B9, "Action Imaging Solutions", "ACTION-IMAGING-SOLUTIONS" },
	{ 0x0009BA, "MAKU Informationstechik GmbH", "MAKU-INFORMATIONSTECHIK" },
	{ 0x0009BB, "MathStar, Inc.", "MATHSTAR" },
	{ 0x0009BC, "Digital Safety Technologies, Inc", "DIGITAL-SAFETY-TECHNOLOGIES" },
	{ 0x0009BD, "Epygi Technologies, Ltd.", "EPYGI-TECHNOLOGIES" },
	{ 0x0009BE, "Mamiya-OP Co.,Ltd.", "MAMIYA-OP" },
	{ 0x0009BF, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x0009C0, "6WIND", "6WIND" },
	{ 0x0009C1, "PROCES-DATA A/S", "PROCES-DATA-A/S" },
	{ 0x0009C2, "Onity, Inc.", "ONITY" },
	{ 0x0009C3, "NETAS", "NETAS" },
	{ 0x0009C4, "Medicore Co., Ltd", "MEDICORE" },
	{ 0x0009C5, "KINGENE Technology Corporation", "KINGENE" },
	{ 0x0009C6, "Visionics Corporation", "VISIONICS" },
	{ 0x0009C7, "Movistec", "MOVISTEC" },
	{ 0x0009C8, "SINAGAWA TSUSHIN KEISOU SERVICE", "SINAGAWA-TSUSHIN-KEISOU-SERVICE" },
	{ 0x0009C9, "BlueWINC Co., Ltd.", "BLUEWINC" },
	{ 0x0009CA, "iMaxNetworks(Shenzhen)Limited.", "IMAXNETWORKS-SHENZHEN" },
	{ 0x0009CB, "HBrain", "HBRAIN" },
	{ 0x0009CC, "Moog GmbH", "MOOG" },
	{ 0x0009CD, "HUDSON SOFT CO.,LTD.", "HUDSON-SOFT" },
	{ 0x0009CE, "SpaceBridge Semiconductor Corp.", "SPACEBRIDGE-SEMICONDUCTOR" },
	{ 0x0009CF, "iAd GmbH", "IAD" },
	{ 0x0009D0, "Solacom Technologies Inc.", "SOLACOM-TECHNOLOGIES" },
	{ 0x0009D1, "SERANOA NETWORKS INC", "SERANOA-NETWORKS" },
	{ 0x0009D2, "Mai Logic Inc.", "MAI-LOGIC" },
	{ 0x0009D3, "Western DataCom Co., Inc.", "WESTERN-DATACOM" },
	{ 0x0009D4, "Transtech Networks", "TRANSTECH-NETWORKS" },
	{ 0x0009D5, "Signal Communication, Inc.", "SIGNAL-COMMUNICATION" },
	{ 0x0009D6, "KNC One GmbH", "KNC-ONE" },
	{ 0x0009D7, "DC Security Products", "DC-SECURITY-PRODUCTS" },
	{ 0x0009D8, "Fält Communications AB", "FäLT-COMMUNICATION-AB" },
	{ 0x0009D9, "Neoscale Systems, Inc", "NEOSCALE" },
	{ 0x0009DA, "Control Module Inc.", "CONTROL-MODULE" },
	{ 0x0009DB, "eSpace", "ESPACE" },
	{ 0x0009DC, "Galaxis Technology AG", "GALAXIS" },
	{ 0x0009DD, "Mavin Technology Inc.", "MAVIN" },
	{ 0x0009DE, "Samjin Information & Communications Co., Ltd.", "SAMJIN-INFORMATION-COMMUNICATION" },
	{ 0x0009DF, "Vestel Komunikasyon Sanayi ve Ticaret A.S.", "VESTEL-KOMUNIKASYON-SANAYI-VE-TICARET-A-S" },
	{ 0x0009E0, "XEMICS S.A.", "XEMICS" },
	{ 0x0009E1, "Gemtek Technology Co., Ltd.", "GEMTEK" },
	{ 0x0009E2, "Sinbon Electronics Co., Ltd.", "SINBON-ELECTRONICS" },
	{ 0x0009E3, "Angel Iglesias S.A.", "ANGEL-IGLESIAS" },
	{ 0x0009E4, "K Tech Infosystem Inc.", "K-TECH-INFOSYSTEM" },
	{ 0x0009E5, "Hottinger Baldwin Messtechnik GmbH", "HOTTINGER-BALDWIN-MESSTECHNIK" },
	{ 0x0009E6, "Cyber Switching Inc.", "CYBER-SWITCHING" },
	{ 0x0009E7, "ADC Techonology", "ADC-TECHONOLOGY" },
	{ 0x0009E8, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0009E9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0009EA, "YEM Inc.", "YEM" },
	{ 0x0009EB, "HuMANDATA LTD.", "HUMANDATA" },
	{ 0x0009EC, "Daktronics, Inc.", "DAKTRONICS" },
	{ 0x0009ED, "CipherOptics", "CIPHEROPTICS" },
	{ 0x0009EE, "MEIKYO ELECTRIC CO.,LTD", "MEIKYO-ELECTRIC" },
	{ 0x0009EF, "Vocera Communications", "VOCERA-COMMUNICATION" },
	{ 0x0009F0, "Shimizu Technology Inc.", "SHIMIZU" },
	{ 0x0009F1, "Yamaki Electric Corporation", "YAMAKI-ELECTRIC" },
	{ 0x0009F2, "Cohu, Inc., Electronics Division", "COHU-ELECTRONICS-DIVISION" },
	{ 0x0009F3, "WELL Communication Corp.", "WELL-COMMUNICATION" },
	{ 0x0009F4, "Alcon Laboratories, Inc.", "ALCON-LABORATORIES" },
	{ 0x0009F5, "Emerson Network Power Co.,Ltd", "EMERSON-NETWORK-POWER" },
	{ 0x0009F6, "Shenzhen Eastern Digital Tech Ltd.", "SHENZHEN-EASTERN-DIGITAL-TECH" },
	{ 0x0009F7, "SED, a division of Calian", "SED-A-DIVISION-OF-CALIAN" },
	{ 0x0009F8, "UNIMO TECHNOLOGY CO., LTD.", "UNIMO" },
	{ 0x0009F9, "ART JAPAN CO., LTD.", "ART-JAPAN" },
	{ 0x0009FB, "Philips Patient Monitoring", "PHILIPS-PATIENT-MONITORING" },
	{ 0x0009FC, "IPFLEX Inc.", "IPFLEX" },
	{ 0x0009FD, "Ubinetics Limited", "UBINETICS" },
	{ 0x0009FE, "Daisy Technologies, Inc.", "DAISY-TECHNOLOGIES" },
	{ 0x0009FF, "X.net 2000 GmbH", "X-NET-2000" },
	{ 0x000A00, "Mediatek Corp.", "MEDIATEK" },
	{ 0x000A01, "SOHOware, Inc.", "SOHOWARE" },
	{ 0x000A02, "ANNSO CO., LTD.", "ANNSO" },
	{ 0x000A03, "ENDESA SERVICIOS, S.L.", "ENDESA-SERVICIOS-S-L" },
	{ 0x000A04, "3Com Ltd", "3COM" },
	{ 0x000A05, "Widax Corp.", "WIDAX" },
	{ 0x000A06, "Teledex LLC", "TELEDEX-LLC" },
	{ 0x000A07, "WebWayOne Ltd", "WEBWAYONE" },
	{ 0x000A08, "ALPINE ELECTRONICS, INC.", "ALPINE-ELECTRONICS" },
	{ 0x000A09, "TaraCom Integrated Products, Inc.", "TARACOM-INTEGRATED-PRODUCTS" },
	{ 0x000A0A, "SUNIX Co., Ltd.", "SUNIX" },
	{ 0x000A0B, "Sealevel Systems, Inc.", "SEALEVEL" },
	{ 0x000A0C, "Scientific Research Corporation", "SCIENTIFIC-RESEARCH" },
	{ 0x000A0D, "FCI Deutschland GmbH", "FCI-DEUTSCHLAND" },
	{ 0x000A0E, "Invivo Research Inc.", "INVIVO-RESEARCH" },
	{ 0x000A0F, "Ilryung Telesys, Inc", "ILRYUNG-TELESYS" },
	{ 0x000A10, "FAST media integrations AG", "FAST-MEDIA-INTEGRATIONS" },
	{ 0x000A11, "ExPet Technologies, Inc", "EXPET-TECHNOLOGIES" },
	{ 0x000A12, "Azylex Technology, Inc", "AZYLEX" },
	{ 0x000A13, "Honeywell Video Systems", "HONEYWELL-VIDEO" },
	{ 0x000A14, "TECO a.s.", "TECO-A-S" },
	{ 0x000A15, "Silicon Data, Inc", "SILICON-DATA" },
	{ 0x000A16, "Lassen Research", "LASSEN-RESEARCH" },
	{ 0x000A17, "NESTAR COMMUNICATIONS, INC", "NESTAR-COMMUNICATION" },
	{ 0x000A18, "Vichel Inc.", "VICHEL" },
	{ 0x000A19, "Valere Power, Inc.", "VALERE-POWER" },
	{ 0x000A1A, "Imerge Ltd", "IMERGE" },
	{ 0x000A1B, "Stream Labs", "STREAM-LABS" },
	{ 0x000A1C, "Bridge Information Co., Ltd.", "BRIDGE-INFORMATION" },
	{ 0x000A1D, "Optical Communications Products Inc.", "OPTICAL-COMMUNICATION-PRODUCTS" },
	{ 0x000A1E, "Red-M Products Limited", "RED-M-PRODUCTS" },
	{ 0x000A1F, "ART WARE Telecommunication Co., Ltd.", "ART-WARE-TELECOMMUNICATION" },
	{ 0x000A20, "SVA Networks, Inc.", "SVA-NETWORKS" },
	{ 0x000A21, "Integra Telecom Co. Ltd", "INTEGRA-TELECOM" },
	{ 0x000A22, "Amperion Inc", "AMPERION" },
	{ 0x000A23, "Parama Networks Inc", "PARAMA-NETWORKS" },
	{ 0x000A24, "Octave Communications", "OCTAVE-COMMUNICATION" },
	{ 0x000A25, "CERAGON NETWORKS", "CERAGON-NETWORKS" },
	{ 0x000A26, "CEIA S.p.A.", "CEIA-S-P-A" },
	{ 0x000A27, "Apple", "APPLE" },
	{ 0x000A28, "Motorola", "MOTOROLA" },
	{ 0x000A29, "Pan Dacom Networking AG", "PAN-DACOM-NETWORKING" },
	{ 0x000A2A, "QSI Systems Inc.", "QSI" },
	{ 0x000A2B, "Etherstuff", "ETHERSTUFF" },
	{ 0x000A2C, "Active Tchnology Corporation", "ACTIVE-TCHNOLOGY" },
	{ 0x000A2D, "Cabot Communications Limited", "CABOT-COMMUNICATION" },
	{ 0x000A2E, "MAPLE NETWORKS CO., LTD", "MAPLE-NETWORKS" },
	{ 0x000A2F, "Artnix Inc.", "ARTNIX" },
	{ 0x000A30, "Visteon Corporation", "VISTEON" },
	{ 0x000A31, "HCV Consulting", "HCV-CONSULTING" },
	{ 0x000A32, "Xsido Corporation", "XSIDO" },
	{ 0x000A33, "Emulex Corporation", "EMULEX" },
	{ 0x000A34, "Identicard Systems Incorporated", "IDENTICARD-INCORPORATED" },
	{ 0x000A35, "Xilinx", "XILINX" },
	{ 0x000A36, "Synelec Telecom Multimedia", "SYNELEC-TELECOM-MULTIMEDIA" },
	{ 0x000A37, "Procera Networks, Inc.", "PROCERA-NETWORKS" },
	{ 0x000A38, "Apani Networks", "APANI-NETWORKS" },
	{ 0x000A39, "LoPA Information Technology", "LOPA-INFORMATION" },
	{ 0x000A3A, "J-THREE INTERNATIONAL Holding Co., Ltd.", "J-THREE-HOLDING" },
	{ 0x000A3B, "GCT Semiconductor, Inc", "GCT-SEMICONDUCTOR" },
	{ 0x000A3C, "Enerpoint Ltd.", "ENERPOINT" },
	{ 0x000A3D, "Elo Sistemas Eletronicos S.A.", "ELO-SISTEMAS-ELETRONICOS" },
	{ 0x000A3E, "EADS Telecom", "EADS-TELECOM" },
	{ 0x000A3F, "Data East Corporation", "DATA-EAST" },
	{ 0x000A40, "Crown Audio -- Harmanm International", "CROWN-AUDIO----HARMANM" },
	{ 0x000A41, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000A42, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000A43, "Chunghwa Telecom Co., Ltd.", "CHUNGHWA-TELECOM" },
	{ 0x000A44, "Avery Dennison Deutschland GmbH", "AVERY-DENNISON-DEUTSCHLAND" },
	{ 0x000A45, "Audio-Technica Corp.", "AUDIO-TECHNICA" },
	{ 0x000A46, "ARO WELDING TECHNOLOGIES SAS", "ARO-WELDING-TECHNOLOGIES-SAS" },
	{ 0x000A47, "Allied Vision Technologies", "ALLIED-VISION-TECHNOLOGIES" },
	{ 0x000A48, "Albatron Technology", "ALBATRON" },
	{ 0x000A49, "F5 Networks, Inc.", "F5-NETWORKS" },
	{ 0x000A4A, "Targa Systems Ltd.", "TARGA" },
	{ 0x000A4B, "DataPower Technology, Inc.", "DATAPOWER" },
	{ 0x000A4C, "Molecular Devices Corporation", "MOLECULAR-DEVICES" },
	{ 0x000A4D, "Noritz Corporation", "NORITZ" },
	{ 0x000A4E, "UNITEK Electronics INC.", "UNITEK-ELECTRONICS" },
	{ 0x000A4F, "Brain Boxes Limited", "BRAIN-BOXES" },
	{ 0x000A50, "REMOTEK CORPORATION", "REMOTEK" },
	{ 0x000A51, "GyroSignal Technology Co., Ltd.", "GYROSIGNAL" },
	{ 0x000A52, "AsiaRF Ltd.", "ASIARF" },
	{ 0x000A53, "Intronics, Incorporated", "INTRONICS-INCORPORATED" },
	{ 0x000A54, "Laguna Hills, Inc.", "LAGUNA-HILLS" },
	{ 0x000A55, "MARKEM Corporation", "MARKEM" },
	{ 0x000A56, "HITACHI Maxell Ltd.", "HITACHI-MAXELL" },
	{ 0x000A57, "Hewlett-Packard Company - Standards", "HEWLETT-PACKARD-COMPANY---STANDARDS" },
	{ 0x000A58, "Freyer & Siegel Elektronik GmbH & Co. KG", "FREYER-SIEGEL-ELEKTRONIK" },
	{ 0x000A59, "HW server", "HW-SERVER" },
	{ 0x000A5A, "GreenNET Technologies Co.,Ltd.", "GREENNET-TECHNOLOGIES" },
	{ 0x000A5B, "Power-One as", "POWER-ONE-AS" },
	{ 0x000A5C, "Carel s.p.a.", "CAREL-S-P-A" },
	{ 0x000A5D, "FingerTec Worldwide Sdn Bhd", "FINGERTEC-WORLDWIDE-SDN-BHD" },
	{ 0x000A5E, "3COM Corporation", "3COM" },
	{ 0x000A5F, "almedio inc.", "ALMEDIO" },
	{ 0x000A60, "Autostar Technology Pte Ltd", "AUTOSTAR-PTE" },
	{ 0x000A61, "Cellinx Systems Inc.", "CELLINX" },
	{ 0x000A62, "Crinis Networks, Inc.", "CRINIS-NETWORKS" },
	{ 0x000A63, "DHD GmbH", "DHD" },
	{ 0x000A64, "Eracom Technologies", "ERACOM-TECHNOLOGIES" },
	{ 0x000A65, "GentechMedia.co.,ltd.", "GENTECHMEDIA" },
	{ 0x000A66, "MITSUBISHI ELECTRIC SYSTEM & SERVICE CO.,LTD.", "MITSUBISHI-ELECTRIC-SYSTEM-SERVICE" },
	{ 0x000A67, "OngCorp", "ONGCORP" },
	{ 0x000A68, "SolarFlare Communications, Inc.", "SOLARFLARE-COMMUNICATION" },
	{ 0x000A69, "SUNNY bell Technology Co., Ltd.", "SUNNY-BELL" },
	{ 0x000A6A, "SVM Microwaves s.r.o.", "SVM-MICROWAVES-S-R-O" },
	{ 0x000A6B, "Tadiran Telecom Business Systems LTD", "TADIRAN-TELECOM-BUSINESS" },
	{ 0x000A6C, "Walchem Corporation", "WALCHEM" },
	{ 0x000A6D, "EKS Elektronikservice GmbH", "EKS-ELEKTRONIKSERVICE" },
	{ 0x000A6E, "Harmonic, Inc", "HARMONIC" },
	{ 0x000A6F, "ZyFLEX Technologies Inc", "ZYFLEX-TECHNOLOGIES" },
	{ 0x000A70, "MPLS Forum", "MPLS-FORUM" },
	{ 0x000A71, "Avrio Technologies, Inc", "AVRIO-TECHNOLOGIES" },
	{ 0x000A72, "STEC, INC.", "STEC" },
	{ 0x000A73, "Scientific Atlanta", "SCIENTIFIC-ATLANTA" },
	{ 0x000A74, "Manticom Networks Inc.", "MANTICOM-NETWORKS" },
	{ 0x000A75, "Caterpillar, Inc", "CATERPILLAR" },
	{ 0x000A76, "Beida Jade Bird Huaguang Technology Co.,Ltd", "BEIDA-JADE-BIRD-HUAGUANG" },
	{ 0x000A77, "Bluewire Technologies LLC", "BLUEWIRE-TECHNOLOGIES-LLC" },
	{ 0x000A78, "OLITEC", "OLITEC" },
	{ 0x000A79, "corega K.K", "COREGA-K-K" },
	{ 0x000A7A, "Kyoritsu Electric Co., Ltd.", "KYORITSU-ELECTRIC" },
	{ 0x000A7B, "Cornelius Consult", "CORNELIUS-CONSULT" },
	{ 0x000A7C, "Tecton Ltd", "TECTON" },
	{ 0x000A7D, "Valo, Inc.", "VALO" },
	{ 0x000A7E, "The Advantage Group", "THE-ADVANTAGE-GROUP" },
	{ 0x000A7F, "Teradon Industries, Inc", "TERADON-INDUSTRIES" },
	{ 0x000A80, "Telkonet Inc.", "TELKONET" },
	{ 0x000A81, "TEIMA Audiotex S.L.", "TEIMA-AUDIOTEX-S-L" },
	{ 0x000A82, "TATSUTA SYSTEM ELECTRONICS CO.,LTD.", "TATSUTA-SYSTEM-ELECTRONICS" },
	{ 0x000A83, "SALTO SYSTEMS S.L.", "SALTO-S-L" },
	{ 0x000A84, "Rainsun Enterprise Co., Ltd.", "RAINSUN" },
	{ 0x000A85, "PLAT'C2,Inc", "PLAT-C2" },
	{ 0x000A86, "Lenze", "LENZE" },
	{ 0x000A87, "Integrated Micromachines Inc.", "INTEGRATED-MICROMACHINES" },
	{ 0x000A88, "InCypher S.A.", "INCYPHER" },
	{ 0x000A89, "Creval Systems, Inc.", "CREVAL" },
	{ 0x000A8A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000A8B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000A8C, "Guardware Systems Ltd.", "GUARDWARE" },
	{ 0x000A8D, "EUROTHERM LIMITED", "EUROTHERM" },
	{ 0x000A8E, "Invacom Ltd", "INVACOM" },
	{ 0x000A8F, "Aska International Inc.", "ASKA" },
	{ 0x000A90, "Bayside Interactive, Inc.", "BAYSIDE-INTERACTIVE" },
	{ 0x000A91, "HemoCue AB", "HEMOCUE-AB" },
	{ 0x000A92, "Presonus Corporation", "PRESONUS" },
	{ 0x000A93, "W2 Networks, Inc.", "W2-NETWORKS" },
	{ 0x000A94, "ShangHai cellink CO., LTD", "SHANGHAI-CELLINK" },
	{ 0x000A95, "Apple", "APPLE" },
	{ 0x000A96, "MEWTEL TECHNOLOGY INC.", "MEWTEL" },
	{ 0x000A97, "SONICblue, Inc.", "SONICBLUE" },
	{ 0x000A98, "M+F Gwinner GmbH & Co", "M+F-GWINNER" },
	{ 0x000A99, "Calamp Wireless Networks Inc", "CALAMP-WIRELESS-NETWORKS" },
	{ 0x000A9A, "Aiptek International Inc", "AIPTEK" },
	{ 0x000A9B, "TB Group Inc", "TB-GROUP" },
	{ 0x000A9C, "Server Technology, Inc.", "SERVER" },
	{ 0x000A9D, "King Young Technology Co. Ltd.", "KING-YOUNG" },
	{ 0x000A9E, "BroadWeb Corportation", "BROADWEB-CORPORTATION" },
	{ 0x000A9F, "Pannaway Technologies, Inc.", "PANNAWAY-TECHNOLOGIES" },
	{ 0x000AA0, "Cedar Point Communications", "CEDAR-POINT-COMMUNICATION" },
	{ 0x000AA1, "V V S Limited", "V-V-S" },
	{ 0x000AA2, "SYSTEK INC.", "SYSTEK" },
	{ 0x000AA3, "SHIMAFUJI ELECTRIC CO.,LTD.", "SHIMAFUJI-ELECTRIC" },
	{ 0x000AA4, "SHANGHAI SURVEILLANCE TECHNOLOGY CO,LTD", "SHANGHAI-SURVEILLANCE" },
	{ 0x000AA5, "MAXLINK INDUSTRIES LIMITED", "MAXLINK-INDUSTRIES" },
	{ 0x000AA6, "Hochiki Corporation", "HOCHIKI" },
	{ 0x000AA7, "FEI Electron Optics", "FEI-ELECTRON-OPTICS" },
	{ 0x000AA8, "ePipe Pty. Ltd.", "EPIPE-PTY" },
	{ 0x000AA9, "Brooks Automation GmbH", "BROOKS-AUTOMATION" },
	{ 0x000AAA, "AltiGen Communications Inc.", "ALTIGEN-COMMUNICATION" },
	{ 0x000AAB, "Toyota Technical Development Corporation", "TOYOTA-TECHNICAL-DEVELOPMENT" },
	{ 0x000AAC, "TerraTec Electronic GmbH", "TERRATEC" },
	{ 0x000AAD, "Stargames Corporation", "STARGAMES" },
	{ 0x000AAE, "Rosemount Process Analytical", "ROSEMOUNT-PROCESS-ANALYTICAL" },
	{ 0x000AAF, "Pipal Systems", "PIPAL" },
	{ 0x000AB0, "LOYTEC electronics GmbH", "LOYTEC-ELECTRONICS" },
	{ 0x000AB1, "GENETEC Corporation", "GENETEC" },
	{ 0x000AB2, "Fresnel Wireless Systems", "FRESNEL-WIRELESS" },
	{ 0x000AB3, "Fa. GIRA", "FA-GIRA" },
	{ 0x000AB4, "ETIC Telecommunications", "ETIC-TELECOMMUNICATIONS" },
	{ 0x000AB5, "Digital Electronic Network", "DIGITAL-NETWORK" },
	{ 0x000AB6, "COMPUNETIX, INC", "COMPUNETIX" },
	{ 0x000AB7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000AB8, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000AB9, "Astera Technologies Corp.", "ASTERA-TECHNOLOGIES" },
	{ 0x000ABA, "Arcon Technology Limited", "ARCON" },
	{ 0x000ABB, "Taiwan Secom Co,. Ltd", "TAIWAN-SECOM" },
	{ 0x000ABC, "Seabridge Ltd.", "SEABRIDGE" },
	{ 0x000ABD, "Rupprecht & Patashnick Co.", "RUPPRECHT-PATASHNICK" },
	{ 0x000ABE, "OPNET Technologies CO., LTD.", "OPNET-TECHNOLOGIES" },
	{ 0x000ABF, "HIROTA SS", "HIROTA-SS" },
	{ 0x000AC0, "Fuyoh Video Industry CO., LTD.", "FUYOH-VIDEO-INDUSTRY" },
	{ 0x000AC1, "Futuretel", "FUTURETEL" },
	{ 0x000AC2, "FiberHome Telecommunication Technologies CO.,LTD", "FIBERHOME-TELECOMMUNICATION-TECHNOLOGIES" },
	{ 0x000AC3, "eM Technics Co., Ltd.", "EM-TECHNICS" },
	{ 0x000AC4, "Daewoo Teletech Co., Ltd", "DAEWOO-TELETECH" },
	{ 0x000AC5, "Color Kinetics", "COLOR-KINETICS" },
	{ 0x000AC6, "Overture Networks.", "OVERTURE-NETWORKS" },
	{ 0x000AC7, "Unication Group", "UNICATION-GROUP" },
	{ 0x000AC8, "ZPSYS CO.,LTD. (Planning&Management)", "ZPSYS-PLANNING-MANAGEMENT" },
	{ 0x000AC9, "Zambeel Inc", "ZAMBEEL" },
	{ 0x000ACA, "YOKOYAMA SHOKAI CO.,Ltd.", "YOKOYAMA-SHOKAI" },
	{ 0x000ACB, "XPAK MSA Group", "XPAK-MSA-GROUP" },
	{ 0x000ACC, "Winnow Networks, Inc.", "WINNOW-NETWORKS" },
	{ 0x000ACD, "Sunrich Technology Limited", "SUNRICH" },
	{ 0x000ACE, "RADIANTECH, INC.", "RADIANTECH" },
	{ 0x000ACF, "PROVIDEO Multimedia Co. Ltd.", "PROVIDEO-MULTIMEDIA" },
	{ 0x000AD0, "Niigata Develoment Center, F.I.T. Co., Ltd.", "NIIGATA-DEVELOMENT-CENTER-F-I-T" },
	{ 0x000AD1, "MWS", "MWS" },
	{ 0x000AD2, "JEPICO Corporation", "JEPICO" },
	{ 0x000AD3, "INITECH Co., Ltd", "INITECH" },
	{ 0x000AD4, "CoreBell Systems Inc.", "COREBELL" },
	{ 0x000AD5, "Brainchild Electronic Co., Ltd.", "BRAINCHILD" },
	{ 0x000AD6, "BeamReach Networks", "BEAMREACH-NETWORKS" },
	{ 0x000AD7, "Origin ELECTRIC CO.,LTD.", "ORIGIN-ELECTRIC" },
	{ 0x000AD8, "IPCserv Technology Corp.", "IPCSERV" },
	{ 0x000AD9, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x000ADA, "Vindicator Technologies", "VINDICATOR-TECHNOLOGIES" },
	{ 0x000ADB, "SkyPilot Network, Inc", "SKYPILOT-NETWORK" },
	{ 0x000ADC, "RuggedCom Inc.", "RUGGEDCOM" },
	{ 0x000ADD, "Allworx Corp.", "ALLWORX" },
	{ 0x000ADE, "Happy Communication Co., Ltd.", "HAPPY-COMMUNICATION" },
	{ 0x000ADF, "Gennum Corporation", "GENNUM" },
	{ 0x000AE0, "Fujitsu Softek", "FUJITSU-SOFTEK" },
	{ 0x000AE1, "EG Technology", "EG" },
	{ 0x000AE2, "Binatone Electronics International, Ltd", "BINATONE-ELECTRONICS" },
	{ 0x000AE3, "YANG MEI TECHNOLOGY CO., LTD", "YANG-MEI" },
	{ 0x000AE4, "Wistron Corp.", "WISTRON" },
	{ 0x000AE5, "ScottCare Corporation", "SCOTTCARE" },
	{ 0x000AE6, "Elitegroup Computer System Co. (ECS)", "ELITEGROUP-SYSTEM-ECS" },
	{ 0x000AE7, "ELIOP S.A.", "ELIOP" },
	{ 0x000AE8, "Cathay Roxus Information Technology Co. LTD", "CATHAY-ROXUS-INFORMATION" },
	{ 0x000AE9, "AirVast Technology Inc.", "AIRVAST" },
	{ 0x000AEA, "ADAM ELEKTRONIK LTD. ŞTI", "ADAM-ELEKTRONIK-ŞTI" },
	{ 0x000AEB, "Shenzhen Tp-Link Technology Co; Ltd.", "SHENZHEN-TP-LINK" },
	{ 0x000AEC, "Koatsu Gas Kogyo Co., Ltd.", "KOATSU-GAS-KOGYO" },
	{ 0x000AED, "HARTING Systems GmbH & Co KG", "HARTING" },
	{ 0x000AEE, "GCD Hard- & Software GmbH", "GCD-HARD--SOFTWARE" },
	{ 0x000AEF, "OTRUM ASA", "OTRUM-ASA" },
	{ 0x000AF0, "SHIN-OH ELECTRONICS CO., LTD. R&D", "SHIN-OH-ELECTRONICS-R-D" },
	{ 0x000AF1, "Clarity Design, Inc.", "CLARITY-DESIGN" },
	{ 0x000AF2, "NeoAxiom Corp.", "NEOAXIOM" },
	{ 0x000AF3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000AF4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000AF5, "Airgo Networks, Inc.", "AIRGO-NETWORKS" },
	{ 0x000AF6, "Emerson Climate Technologies Retail Solutions, Inc.", "EMERSON-CLIMATE-TECHNOLOGIES-RETAIL-SOLUTIONS" },
	{ 0x000AF7, "Broadcom Corp.", "BROADCOM" },
	{ 0x000AF8, "American Telecare Inc.", "AMERICAN-TELECARE" },
	{ 0x000AF9, "HiConnect, Inc.", "HICONNECT" },
	{ 0x000AFA, "Traverse Technologies Australia", "TRAVERSE-TECHNOLOGIES-AUSTRALIA" },
	{ 0x000AFB, "Ambri Limited", "AMBRI" },
	{ 0x000AFC, "Core Tec Communications, LLC", "CORE-TEC-COMMUNICATION-LLC" },
	{ 0x000AFD, "Kentec Electronics", "KENTEC-ELECTRONICS" },
	{ 0x000AFE, "NovaPal Ltd", "NOVAPAL" },
	{ 0x000AFF, "Kilchherr Elektronik AG", "KILCHHERR-ELEKTRONIK" },
	{ 0x000B00, "FUJIAN START COMPUTER EQUIPMENT CO.,LTD", "FUJIAN-START-EQUIPMENT" },
	{ 0x000B01, "DAIICHI ELECTRONICS CO., LTD.", "DAIICHI-ELECTRONICS" },
	{ 0x000B02, "Dallmeier electronic", "DALLMEIER" },
	{ 0x000B03, "Taekwang Industrial Co., Ltd", "TAEKWANG-INDUSTRIAL" },
	{ 0x000B04, "Volktek Corporation", "VOLKTEK" },
	{ 0x000B05, "Pacific Broadband Networks", "PACIFIC-BROADBAND-NETWORKS" },
	{ 0x000B06, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x000B07, "Voxpath Networks", "VOXPATH-NETWORKS" },
	{ 0x000B08, "Pillar Data Systems", "PILLAR-DATA" },
	{ 0x000B09, "Ifoundry Systems Singapore", "IFOUNDRY-SINGAPORE" },
	{ 0x000B0A, "dBm Optics", "DBM-OPTICS" },
	{ 0x000B0B, "Corrent Corporation", "CORRENT" },
	{ 0x000B0C, "Agile Systems Inc.", "AGILE" },
	{ 0x000B0D, "Air2U, Inc.", "AIR2U" },
	{ 0x000B0E, "Trapeze Networks", "TRAPEZE-NETWORKS" },
	{ 0x000B0F, "Bosch Rexroth", "BOSCH-REXROTH" },
	{ 0x000B10, "11wave Technonlogy Co.,Ltd", "11WAVE-TECHNONLOGY" },
	{ 0x000B11, "HIMEJI ABC TRADING CO.,LTD.", "HIMEJI-ABC-TRADING" },
	{ 0x000B12, "NURI Telecom Co., Ltd.", "NURI-TELECOM" },
	{ 0x000B13, "ZETRON INC", "ZETRON" },
	{ 0x000B14, "ViewSonic Corporation", "VIEWSONIC" },
	{ 0x000B15, "Platypus Technology", "PLATYPUS" },
	{ 0x000B16, "Communication Machinery Corporation", "COMMUNICATION-MACHINERY" },
	{ 0x000B17, "MKS Instruments", "MKS-INSTRUMENTS" },
	{ 0x000B18, "PRIVATE", "PRIVATE" },
	{ 0x000B19, "Vernier Networks, Inc.", "VERNIER-NETWORKS" },
	{ 0x000B1A, "Industrial Defender, Inc.", "INDUSTRIAL-DEFENDER" },
	{ 0x000B1B, "Systronix, Inc.", "SYSTRONIX" },
	{ 0x000B1C, "SIBCO bv", "SIBCO" },
	{ 0x000B1D, "LayerZero Power Systems, Inc.", "LAYERZERO-POWER" },
	{ 0x000B1E, "KAPPA opto-electronics GmbH", "KAPPA-OPTO-ELECTRONICS" },
	{ 0x000B1F, "I CON Computer Co.", "I-CON" },
	{ 0x000B20, "Hirata corporation", "HIRATA" },
	{ 0x000B21, "G-Star Communications Inc.", "G-STAR-COMMUNICATION" },
	{ 0x000B22, "Environmental Systems and Services", "ENVIRONMENTAL-AND-SERVICES" },
	{ 0x000B23, "Siemens Subscriber Networks", "SIEMENS-SUBSCRIBER-NETWORKS" },
	{ 0x000B24, "AirLogic", "AIRLOGIC" },
	{ 0x000B25, "Aeluros", "AELUROS" },
	{ 0x000B26, "Wetek Corporation", "WETEK" },
	{ 0x000B27, "Scion Corporation", "SCION" },
	{ 0x000B28, "Quatech Inc.", "QUATECH" },
	{ 0x000B29, "LS(LG) Industrial Systems co.,Ltd", "LS-LG-INDUSTRIAL" },
	{ 0x000B2A, "HOWTEL Co., Ltd.", "HOWTEL" },
	{ 0x000B2B, "HOSTNET CORPORATION", "HOSTNET" },
	{ 0x000B2C, "Eiki Industrial Co. Ltd.", "EIKI-INDUSTRIAL" },
	{ 0x000B2D, "Danfoss Inc.", "DANFOSS" },
	{ 0x000B2E, "Cal-Comp Electronics (Thailand) Public Company Limited Taipe", "CAL-COMP-ELECTRONICS-THAILAND-PUBLIC-COMPANY-TAIPE" },
	{ 0x000B2F, "bplan GmbH", "BPLAN" },
	{ 0x000B30, "Beijing Gongye Science & Technology Co.,Ltd", "BEIJING-GONGYE-SCIENCE" },
	{ 0x000B31, "Yantai ZhiYang Scientific and technology industry CO., LTD", "YANTAI-ZHIYANG-SCIENTIFIC-AND-INDUSTRY" },
	{ 0x000B32, "VORMETRIC, INC.", "VORMETRIC" },
	{ 0x000B33, "Vivato Technologies", "VIVATO-TECHNOLOGIES" },
	{ 0x000B34, "ShangHai Broadband Technologies CO.LTD", "SHANGHAI-BROADBAND-TECHNOLOGIES" },
	{ 0x000B35, "Quad Bit System co., Ltd.", "QUAD-BIT-SYSTEM" },
	{ 0x000B36, "Productivity Systems, Inc.", "PRODUCTIVITY" },
	{ 0x000B37, "MANUFACTURE DES MONTRES ROLEX SA", "MANUFACTURE-DES-MONTRES-ROLEX" },
	{ 0x000B38, "Knürr GmbH", "KNüRR" },
	{ 0x000B39, "Keisoku Giken Co.,Ltd.", "KEISOKU-GIKEN" },
	{ 0x000B3A, "QuStream Corporation", "QUSTREAM" },
	{ 0x000B3B, "devolo AG", "DEVOLO" },
	{ 0x000B3C, "Cygnal Integrated Products, Inc.", "CYGNAL-INTEGRATED-PRODUCTS" },
	{ 0x000B3D, "CONTAL OK Ltd.", "CONTAL-OK" },
	{ 0x000B3E, "BittWare, Inc", "BITTWARE" },
	{ 0x000B3F, "Anthology Solutions Inc.", "ANTHOLOGY-SOLUTIONS" },
	{ 0x000B40, "Oclaro", "OCLARO" },
	{ 0x000B41, "Ing. Büro Dr. Beutlhauser", "ING-BüRO-DR-BEUTLHAUSER" },
	{ 0x000B42, "commax Co., Ltd.", "COMMAX" },
	{ 0x000B43, "Microscan Systems, Inc.", "MICROSCAN" },
	{ 0x000B44, "Concord IDea Corp.", "CONCORD-IDEA" },
	{ 0x000B45, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000B46, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000B47, "Advanced Energy", "ADVANCED-ENERGY" },
	{ 0x000B48, "sofrel", "SOFREL" },
	{ 0x000B49, "RF-Link System Inc.", "RF-LINK-SYSTEM" },
	{ 0x000B4A, "Visimetrics (UK) Ltd", "VISIMETRICS-UK" },
	{ 0x000B4B, "VISIOWAVE SA", "VISIOWAVE" },
	{ 0x000B4C, "Clarion (M) Sdn Bhd", "CLARION-M-SDN-BHD" },
	{ 0x000B4D, "Emuzed", "EMUZED" },
	{ 0x000B4E, "VertexRSI, General Dynamics SatCOM Technologies, Inc.", "VERTEXRSI-GENERAL-DYNAMICS-SATCOM-TECHNOLOGIES" },
	{ 0x000B4F, "Verifone, INC.", "VERIFONE" },
	{ 0x000B50, "Oxygnet", "OXYGNET" },
	{ 0x000B51, "Micetek International Inc.", "MICETEK" },
	{ 0x000B52, "JOYMAX ELECTRONICS CO. LTD.", "JOYMAX-ELECTRONICS" },
	{ 0x000B53, "INITIUM Co., Ltd.", "INITIUM" },
	{ 0x000B54, "BiTMICRO Networks, Inc.", "BITMICRO-NETWORKS" },
	{ 0x000B55, "ADInstruments", "ADINSTRUMENTS" },
	{ 0x000B56, "Cybernetics", "CYBERNETICS" },
	{ 0x000B57, "Silicon Laboratories", "SILICON-LABORATORIES" },
	{ 0x000B58, "Astronautics C.A LTD", "ASTRONAUTICS-C-A" },
	{ 0x000B59, "ScriptPro, LLC", "SCRIPTPRO-LLC" },
	{ 0x000B5A, "HyperEdge", "HYPEREDGE" },
	{ 0x000B5B, "Rincon Research Corporation", "RINCON-RESEARCH" },
	{ 0x000B5C, "Newtech Co.,Ltd", "NEWTECH" },
	{ 0x000B5D, "FUJITSU LIMITED", "FUJITSU" },
	{ 0x000B5E, "Audio Engineering Society Inc.", "AUDIO-ENGINEERING-SOCIETY" },
	{ 0x000B5F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000B60, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000B61, "Friedrich Lütze GmbH & Co. KG", "FRIEDRICH-LüTZE" },
	{ 0x000B62, "ib-mohnen KG", "IB-MOHNEN" },
	{ 0x000B63, "Kaleidescape", "KALEIDESCAPE" },
	{ 0x000B64, "Kieback & Peter GmbH & Co KG", "KIEBACK-PETER" },
	{ 0x000B65, "Sy.A.C. srl", "SY-A-C-SRL" },
	{ 0x000B66, "Teralink Communications", "TERALINK-COMMUNICATION" },
	{ 0x000B67, "Topview Technology Corporation", "TOPVIEW" },
	{ 0x000B68, "Addvalue Communications Pte Ltd", "ADDVALUE-COMMUNICATION-PTE" },
	{ 0x000B69, "Franke Finland Oy", "FRANKE-FINLAND-OY" },
	{ 0x000B6A, "Asiarock Incorporation", "ASIAROCK-INCORPORATION" },
	{ 0x000B6B, "Wistron Neweb Corp.", "WISTRON-NEWEB" },
	{ 0x000B6C, "Sychip Inc.", "SYCHIP" },
	{ 0x000B6D, "SOLECTRON JAPAN NAKANIIDA", "SOLECTRON-JAPAN-NAKANIIDA" },
	{ 0x000B6E, "Neff Instrument Corp.", "NEFF-INSTRUMENT" },
	{ 0x000B6F, "Media Streaming Networks Inc", "MEDIA-STREAMING-NETWORKS" },
	{ 0x000B70, "Load Technology, Inc.", "LOAD" },
	{ 0x000B71, "Litchfield Communications Inc.", "LITCHFIELD-COMMUNICATION" },
	{ 0x000B72, "Lawo AG", "LAWO" },
	{ 0x000B73, "Kodeos Communications", "KODEOS-COMMUNICATION" },
	{ 0x000B74, "Kingwave Technology Co., Ltd.", "KINGWAVE" },
	{ 0x000B75, "Iosoft Ltd.", "IOSOFT" },
	{ 0x000B76, "ET&T Technology Co. Ltd.", "ET-T" },
	{ 0x000B77, "Cogent Systems, Inc.", "COGENT" },
	{ 0x000B78, "TAIFATECH INC.", "TAIFATECH" },
	{ 0x000B79, "X-COM, Inc.", "X-COM" },
	{ 0x000B7A, "L-3 Linkabit", "L-3-LINKABIT" },
	{ 0x000B7B, "Test-Um Inc.", "TEST-UM" },
	{ 0x000B7C, "Telex Communications", "TELEX-COMMUNICATION" },
	{ 0x000B7D, "SOLOMON EXTREME INTERNATIONAL LTD.", "SOLOMON-EXTREME" },
	{ 0x000B7E, "SAGINOMIYA Seisakusho Inc.", "SAGINOMIYA-SEISAKUSHO" },
	{ 0x000B7F, "Align Engineering LLC", "ALIGN-ENGINEERING-LLC" },
	{ 0x000B80, "Lycium Networks", "LYCIUM-NETWORKS" },
	{ 0x000B81, "Kaparel Corporation", "KAPAREL" },
	{ 0x000B82, "Grandstream Networks, Inc.", "GRANDSTREAM-NETWORKS" },
	{ 0x000B83, "DATAWATT B.V.", "DATAWATT" },
	{ 0x000B84, "BODET", "BODET" },
	{ 0x000B85, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000B86, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0x000B87, "American Reliance Inc.", "AMERICAN-RELIANCE" },
	{ 0x000B88, "Vidisco ltd.", "VIDISCO" },
	{ 0x000B89, "Top Global Technology, Ltd.", "TOP-GLOBAL" },
	{ 0x000B8A, "MITEQ Inc.", "MITEQ" },
	{ 0x000B8B, "KERAJET, S.A.", "KERAJET" },
	{ 0x000B8C, "Flextronics", "FLEXTRONICS" },
	{ 0x000B8D, "Avvio Networks", "AVVIO-NETWORKS" },
	{ 0x000B8E, "Ascent Corporation", "ASCENT" },
	{ 0x000B8F, "AKITA ELECTRONICS SYSTEMS CO.,LTD.", "AKITA-ELECTRONICS" },
	{ 0x000B90, "ADVA Optical Networking Ltd.", "ADVA-OPTICAL-NETWORKING" },
	{ 0x000B91, "Aglaia Gesellschaft für Bildverarbeitung und Kommunikation mbH", "AGLAIA-GESELLSCHAFT-FüR-BILDVERARBEITUNG-UND-KOMMUNIKATION-MBH" },
	{ 0x000B92, "Ascom Danmark A/S", "ASCOM-DANMARK-A/S" },
	{ 0x000B93, "Ritter Elektronik", "RITTER-ELEKTRONIK" },
	{ 0x000B94, "Digital Monitoring Products, Inc.", "DIGITAL-MONITORING-PRODUCTS" },
	{ 0x000B95, "eBet Gaming Systems Pty Ltd", "EBET-GAMING-PTY" },
	{ 0x000B96, "Innotrac Diagnostics Oy", "INNOTRAC-DIAGNOSTICS-OY" },
	{ 0x000B97, "Matsushita Electric Industrial Co.,Ltd.", "MATSUSHITA-ELECTRIC-INDUSTRIAL" },
	{ 0x000B98, "NiceTechVision", "NICETECHVISION" },
	{ 0x000B99, "SensAble Technologies, Inc.", "SENSABLE-TECHNOLOGIES" },
	{ 0x000B9A, "Shanghai Ulink Telecom Equipment Co. Ltd.", "SHANGHAI-ULINK-TELECOM-EQUIPMENT" },
	{ 0x000B9B, "Sirius System Co, Ltd.", "SIRIUS-SYSTEM" },
	{ 0x000B9C, "TriBeam Technologies, Inc.", "TRIBEAM-TECHNOLOGIES" },
	{ 0x000B9D, "TwinMOS Technologies Inc.", "TWINMOS-TECHNOLOGIES" },
	{ 0x000B9E, "Yasing Technology Corp.", "YASING" },
	{ 0x000B9F, "Neue ELSA GmbH", "NEUE-ELSA" },
	{ 0x000BA0, "T&L Information Inc.", "T-L-INFORMATION" },
	{ 0x000BA1, "SYSCOM Ltd.", "SYSCOM" },
	{ 0x000BA2, "Sumitomo Electric Networks, Inc", "SUMITOMO-ELECTRIC-NETWORKS" },
	{ 0x000BA3, "Siemens AG, I&S", "SIEMENS-I-S" },
	{ 0x000BA4, "Shiron Satellite Communications Ltd. (1996)", "SHIRON-SATELLITE-COMMUNICATION-1996" },
	{ 0x000BA5, "Quasar Cipta Mandiri, PT", "QUASAR-CIPTA-MANDIRI-PT" },
	{ 0x000BA6, "Miyakawa Electric Works Ltd.", "MIYAKAWA-ELECTRIC-WORKS" },
	{ 0x000BA7, "Maranti Networks", "MARANTI-NETWORKS" },
	{ 0x000BA8, "HANBACK ELECTRONICS CO., LTD.", "HANBACK-ELECTRONICS" },
	{ 0x000BA9, "CloudShield Technologies, Inc.", "CLOUDSHIELD-TECHNOLOGIES" },
	{ 0x000BAA, "Aiphone co.,Ltd", "AIPHONE" },
	{ 0x000BAB, "Advantech Technology (CHINA) Co., Ltd.", "ADVANTECH-CHINA" },
	{ 0x000BAC, "3Com Ltd", "3COM" },
	{ 0x000BAD, "PC-PoS Inc.", "PC-POS" },
	{ 0x000BAE, "Vitals System Inc.", "VITALS-SYSTEM" },
	{ 0x000BAF, "WOOJU COMMUNICATIONS Co,.Ltd", "WOOJU-COMMUNICATION" },
	{ 0x000BB0, "Sysnet Telematica srl", "SYSNET-TELEMATICA-SRL" },
	{ 0x000BB1, "Super Star Technology Co., Ltd.", "SUPER-STAR" },
	{ 0x000BB2, "SMALLBIG TECHNOLOGY", "SMALLBIG" },
	{ 0x000BB3, "RiT technologies Ltd.", "RIT-TECHNOLOGIES" },
	{ 0x000BB4, "RDC Semiconductor Inc.,", "RDC-SEMICONDUCTOR" },
	{ 0x000BB5, "nStor Technologies, Inc.", "NSTOR-TECHNOLOGIES" },
	{ 0x000BB6, "Metalligence Technology Corp.", "METALLIGENCE" },
	{ 0x000BB7, "Micro Systems Co.,Ltd.", "MICRO" },
	{ 0x000BB8, "Kihoku Electronic Co.", "KIHOKU" },
	{ 0x000BB9, "Imsys AB", "IMSYS-AB" },
	{ 0x000BBA, "Harmonic, Inc", "HARMONIC" },
	{ 0x000BBB, "Etin Systems Co., Ltd", "ETIN" },
	{ 0x000BBC, "En Garde Systems, Inc.", "EN-GARDE" },
	{ 0x000BBD, "Connexionz Limited", "CONNEXIONZ" },
	{ 0x000BBE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000BBF, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000BC0, "China IWNComm Co., Ltd.", "CHINA-IWNCOMM" },
	{ 0x000BC1, "Bay Microsystems, Inc.", "BAY-MICROSYSTEMS" },
	{ 0x000BC2, "Corinex Communication Corp.", "CORINEX-COMMUNICATION" },
	{ 0x000BC3, "Multiplex, Inc.", "MULTIPLEX" },
	{ 0x000BC4, "BIOTRONIK GmbH & Co", "BIOTRONIK" },
	{ 0x000BC5, "SMC Networks, Inc.", "SMC-NETWORKS" },
	{ 0x000BC6, "ISAC, Inc.", "ISAC" },
	{ 0x000BC7, "ICET S.p.A.", "ICET-S-P-A" },
	{ 0x000BC8, "AirFlow Networks", "AIRFLOW-NETWORKS" },
	{ 0x000BC9, "Electroline Equipment", "ELECTROLINE-EQUIPMENT" },
	{ 0x000BCA, "DATAVAN International Corporation", "DATAVAN" },
	{ 0x000BCB, "Fagor Automation , S. Coop", "FAGOR-AUTOMATION-S-COOP" },
	{ 0x000BCC, "JUSAN, S.A.", "JUSAN" },
	{ 0x000BCD, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x000BCE, "Free2move AB", "FREE2MOVE-AB" },
	{ 0x000BCF, "AGFA NDT INC.", "AGFA-NDT" },
	{ 0x000BD0, "XiMeta Technology Americas Inc.", "XIMETA-AMERICAS" },
	{ 0x000BD1, "Aeronix, Inc.", "AERONIX" },
	{ 0x000BD2, "Remopro Technology Inc.", "REMOPRO" },
	{ 0x000BD3, "cd3o", "CD3O" },
	{ 0x000BD4, "Beijing Wise Technology & Science Development Co.Ltd", "BEIJING-WISE-SCIENCE-DEVELOPMENT" },
	{ 0x000BD5, "Nvergence, Inc.", "NVERGENCE" },
	{ 0x000BD6, "Paxton Access Ltd", "PAXTON-ACCESS" },
	{ 0x000BD7, "DORMA Time + Access GmbH", "DORMA-TIME-+-ACCESS" },
	{ 0x000BD8, "Industrial Scientific Corp.", "INDUSTRIAL-SCIENTIFIC" },
	{ 0x000BD9, "General Hydrogen", "GENERAL-HYDROGEN" },
	{ 0x000BDA, "EyeCross Co.,Inc.", "EYECROSS" },
	{ 0x000BDB, "Dell Inc", "DELL" },
	{ 0x000BDC, "AKCP", "AKCP" },
	{ 0x000BDD, "TOHOKU RICOH Co., LTD.", "TOHOKU-RICOH" },
	{ 0x000BDE, "TELDIX GmbH", "TELDIX" },
	{ 0x000BDF, "Shenzhen RouterD Networks Limited", "SHENZHEN-ROUTERD-NETWORKS" },
	{ 0x000BE0, "SercoNet Ltd.", "SERCONET" },
	{ 0x000BE1, "Nokia NET Product Operations", "NOKIA-NET-PRODUCT-OPERATIONS" },
	{ 0x000BE2, "Lumenera Corporation", "LUMENERA" },
	{ 0x000BE3, "Key Stream Co., Ltd.", "KEY-STREAM" },
	{ 0x000BE4, "Hosiden Corporation", "HOSIDEN" },
	{ 0x000BE5, "HIMS International Corporation", "HIMS" },
	{ 0x000BE6, "Datel Electronics", "DATEL-ELECTRONICS" },
	{ 0x000BE7, "COMFLUX TECHNOLOGY INC.", "COMFLUX" },
	{ 0x000BE8, "AOIP", "AOIP" },
	{ 0x000BE9, "Actel Corporation", "ACTEL" },
	{ 0x000BEA, "Zultys Technologies", "ZULTYS-TECHNOLOGIES" },
	{ 0x000BEB, "Systegra AG", "SYSTEGRA" },
	{ 0x000BEC, "NIPPON ELECTRIC INSTRUMENT, INC.", "NIPPON-ELECTRIC-INSTRUMENT" },
	{ 0x000BED, "ELM Inc.", "ELM" },
	{ 0x000BEE, "inc.jet, Incorporated", "JET-INCORPORATED" },
	{ 0x000BEF, "Code Corporation", "CODE" },
	{ 0x000BF0, "MoTEX Products Co., Ltd.", "MOTEX-PRODUCTS" },
	{ 0x000BF1, "LAP Laser Applikations", "LAP-LASER-APPLIKATIONS" },
	{ 0x000BF2, "Chih-Kan Technology Co., Ltd.", "CHIH-KAN" },
	{ 0x000BF3, "BAE SYSTEMS", "BAE" },
	{ 0x000BF4, "PRIVATE", "PRIVATE" },
	{ 0x000BF5, "Shanghai Sibo Telecom Technology Co.,Ltd", "SHANGHAI-SIBO-TELECOM" },
	{ 0x000BF6, "Nitgen Co., Ltd", "NITGEN" },
	{ 0x000BF7, "NIDEK CO.,LTD", "NIDEK" },
	{ 0x000BF8, "Infinera", "INFINERA" },
	{ 0x000BF9, "Gemstone communications, Inc.", "GEMSTONE-COMMUNICATION" },
	{ 0x000BFA, "EXEMYS SRL", "EXEMYS-SRL" },
	{ 0x000BFB, "D-NET International Corporation", "D-NET" },
	{ 0x000BFC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000BFD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000BFE, "CASTEL Broadband Limited", "CASTEL-BROADBAND" },
	{ 0x000BFF, "Berkeley Camera Engineering", "BERKELEY-CAMERA-ENGINEERING" },
	{ 0x000C00, "BEB Industrie-Elektronik AG", "BEB-INDUSTRIE-ELEKTRONIK" },
	{ 0x000C01, "Abatron AG", "ABATRON" },
	{ 0x000C02, "ABB Oy", "ABB-OY" },
	{ 0x000C03, "HDMI Licensing, LLC", "HDMI-LICENSING-LLC" },
	{ 0x000C04, "Tecnova", "TECNOVA" },
	{ 0x000C05, "RPA Reserch Co., Ltd.", "RPA-RESERCH" },
	{ 0x000C06, "Nixvue Systems Pte Ltd", "NIXVUE-PTE" },
	{ 0x000C07, "Iftest AG", "IFTEST" },
	{ 0x000C08, "HUMEX Technologies Corp.", "HUMEX-TECHNOLOGIES" },
	{ 0x000C09, "Hitachi IE Systems Co., Ltd", "HITACHI-IE" },
	{ 0x000C0A, "Guangdong Province Electronic Technology Research Institute", "GUANGDONG-PROVINCE-RESEARCH-INSTITUTE" },
	{ 0x000C0B, "Broadbus Technologies", "BROADBUS-TECHNOLOGIES" },
	{ 0x000C0C, "APPRO TECHNOLOGY INC.", "APPRO" },
	{ 0x000C0D, "Communications & Power Industries / Satcom Division", "COMMUNICATION-POWER-INDUSTRIES-/-SATCOM-DIVISION" },
	{ 0x000C0E, "XtremeSpectrum, Inc.", "XTREMESPECTRUM" },
	{ 0x000C0F, "Techno-One Co., Ltd", "TECHNO-ONE" },
	{ 0x000C10, "PNI Corporation", "PNI" },
	{ 0x000C11, "NIPPON DEMPA CO.,LTD.", "NIPPON-DEMPA" },
	{ 0x000C12, "Micro-Optronic-Messtechnik GmbH", "MICRO-OPTRONIC-MESSTECHNIK" },
	{ 0x000C13, "MediaQ", "MEDIAQ" },
	{ 0x000C14, "Diagnostic Instruments, Inc.", "DIAGNOSTIC-INSTRUMENTS" },
	{ 0x000C15, "CyberPower Systems, Inc.", "CYBERPOWER" },
	{ 0x000C16, "Concorde Microsystems Inc.", "CONCORDE-MICROSYSTEMS" },
	{ 0x000C17, "AJA Video Systems Inc", "AJA-VIDEO" },
	{ 0x000C18, "Zenisu Keisoku Inc.", "ZENISU-KEISOKU" },
	{ 0x000C19, "Telio Communications GmbH", "TELIO-COMMUNICATION" },
	{ 0x000C1A, "Quest Technical Solutions Inc.", "QUEST-TECHNICAL-SOLUTIONS" },
	{ 0x000C1B, "ORACOM Co, Ltd.", "ORACOM" },
	{ 0x000C1C, "MicroWeb Co., Ltd.", "MICROWEB" },
	{ 0x000C1D, "Mettler & Fuchs AG", "METTLER-FUCHS" },
	{ 0x000C1E, "Global Cache", "GLOBAL-CACHE" },
	{ 0x000C1F, "Glimmerglass Networks", "GLIMMERGLASS-NETWORKS" },
	{ 0x000C20, "Fi WIn, Inc.", "FI-WIN" },
	{ 0x000C21, "Faculty of Science and Technology, Keio University", "FACULTY-OF-SCIENCE-AND-KEIO-UNIVERSITY" },
	{ 0x000C22, "Double D Electronics Ltd", "DOUBLE-D-ELECTRONICS" },
	{ 0x000C23, "Beijing Lanchuan Tech. Co., Ltd.", "BEIJING-LANCHUAN-TECH" },
	{ 0x000C24, "ANATOR", "ANATOR" },
	{ 0x000C25, "Allied Telesis Labs, Inc.", "ALLIED-TELESIS-LABS" },
	{ 0x000C26, "Weintek Labs. Inc.", "WEINTEK-LABS" },
	{ 0x000C27, "Sammy Corporation", "SAMMY" },
	{ 0x000C28, "RIFATRON", "RIFATRON" },
	{ 0x000C29, "VMware, Inc. (possible VMware VM)", "VMWARE-VIRTUAL" },
	{ 0x000C2A, "OCTTEL Communication Co., Ltd.", "OCTTEL-COMMUNICATION" },
	{ 0x000C2B, "ELIAS Technology, Inc.", "ELIAS" },
	{ 0x000C2C, "Enwiser Inc.", "ENWISER" },
	{ 0x000C2D, "FullWave Technology Co., Ltd.", "FULLWAVE" },
	{ 0x000C2E, "Openet information technology(shenzhen) Co., Ltd.", "OPENET-INFORMATION-SHENZHEN" },
	{ 0x000C2F, "SeorimTechnology Co.,Ltd.", "SEORIMTECHNOLOGY" },
	{ 0x000C30, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000C31, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000C32, "Avionic Design Development GmbH", "AVIONIC-DESIGN-DEVELOPMENT" },
	{ 0x000C33, "Compucase Enterprise Co. Ltd.", "COMPUCASE" },
	{ 0x000C34, "Vixen Co., Ltd.", "VIXEN" },
	{ 0x000C35, "KaVo Dental GmbH & Co. KG", "KAVO-DENTAL" },
	{ 0x000C36, "SHARP TAKAYA ELECTRONICS INDUSTRY CO.,LTD.", "SHARP-TAKAYA-ELECTRONICS-INDUSTRY" },
	{ 0x000C37, "Geomation, Inc.", "GEOMATION" },
	{ 0x000C38, "TelcoBridges Inc.", "TELCOBRIDGES" },
	{ 0x000C39, "Sentinel Wireless Inc.", "SENTINEL-WIRELESS" },
	{ 0x000C3A, "Oxance", "OXANCE" },
	{ 0x000C3B, "Orion Electric Co., Ltd.", "ORION-ELECTRIC" },
	{ 0x000C3C, "MediaChorus, Inc.", "MEDIACHORUS" },
	{ 0x000C3D, "Glsystech Co., Ltd.", "GLSYSTECH" },
	{ 0x000C3E, "Crest Audio", "CREST-AUDIO" },
	{ 0x000C3F, "Cogent Defence & Security Networks,", "COGENT-DEFENCE-SECURITY-NETWORKS" },
	{ 0x000C40, "Altech Controls", "ALTECH-CONTROLS" },
	{ 0x000C41, "Cisco-Linksys", "CISCO-LINKSYS" },
	{ 0x000C42, "Routerboard.com", "ROUTERBOARD-COM" },
	{ 0x000C43, "Ralink Technology, Corp.", "RALINK" },
	{ 0x000C44, "Automated Interfaces, Inc.", "AUTOMATED-INTERFACES" },
	{ 0x000C45, "Animation Technologies Inc.", "ANIMATION-TECHNOLOGIES" },
	{ 0x000C46, "Allied Telesyn Inc.", "ALLIED-TELESYN" },
	{ 0x000C47, "SK Teletech(R&D Planning Team)", "SK-TELETECH-R-D-PLANNING-TEAM" },
	{ 0x000C48, "QoStek Corporation", "QOSTEK" },
	{ 0x000C49, "Dangaard Telecom RTC Division A/S", "DANGAARD-TELECOM-RTC-DIVISION-A/S" },
	{ 0x000C4A, "Cygnus Microsystems (P) Limited", "CYGNUS-MICROSYSTEMS-P" },
	{ 0x000C4B, "Cheops Elektronik", "CHEOPS-ELEKTRONIK" },
	{ 0x000C4C, "Arcor AG&Co.", "ARCOR" },
	{ 0x000C4D, "Curtiss-Wright Controls Avionics & Electronics", "CURTISS-WRIGHT-CONTROLS-AVIONICS-ELECTRONICS" },
	{ 0x000C4E, "Winbest Technology CO,LT", "WINBEST-LT" },
	{ 0x000C4F, "UDTech Japan Corporation", "UDTECH-JAPAN" },
	{ 0x000C50, "Seagate Technology", "SEAGATE" },
	{ 0x000C51, "Scientific Technologies Inc.", "SCIENTIFIC-TECHNOLOGIES" },
	{ 0x000C52, "Roll Systems Inc.", "ROLL" },
	{ 0x000C53, "PRIVATE", "PRIVATE" },
	{ 0x000C54, "Pedestal Networks, Inc", "PEDESTAL-NETWORKS" },
	{ 0x000C55, "Microlink Communications Inc.", "MICROLINK-COMMUNICATION" },
	{ 0x000C56, "Megatel Computer (1986) Corp.", "MEGATEL-1986" },
	{ 0x000C57, "MACKIE Engineering Services Belgium BVBA", "MACKIE-ENGINEERING-SERVICES-BELGIUM-BVBA" },
	{ 0x000C58, "M&S Systems", "M-S" },
	{ 0x000C59, "Indyme Electronics, Inc.", "INDYME-ELECTRONICS" },
	{ 0x000C5A, "IBSmm Embedded Electronics Consulting", "IBSMM-EMBEDDED-ELECTRONICS-CONSULTING" },
	{ 0x000C5B, "HANWANG TECHNOLOGY CO.,LTD", "HANWANG" },
	{ 0x000C5C, "GTN Systems B.V.", "GTN" },
	{ 0x000C5D, "CHIC TECHNOLOGY (CHINA) CORP.", "CHIC-CHINA" },
	{ 0x000C5E, "Calypso Medical", "CALYPSO-MEDICAL" },
	{ 0x000C5F, "Avtec, Inc.", "AVTEC" },
	{ 0x000C60, "ACM Systems", "ACM" },
	{ 0x000C61, "AC Tech corporation DBA Advanced Digital", "AC-TECH-DBA-ADVANCED-DIGITAL" },
	{ 0x000C62, "ABB AB, Cewe-Control", "ABB-AB-CEWE-CONTROL" },
	{ 0x000C63, "Zenith Electronics Corporation", "ZENITH-ELECTRONICS" },
	{ 0x000C64, "X2 MSA Group", "X2-MSA-GROUP" },
	{ 0x000C65, "Sunin Telecom", "SUNIN-TELECOM" },
	{ 0x000C66, "Pronto Networks Inc", "PRONTO-NETWORKS" },
	{ 0x000C67, "OYO ELECTRIC CO.,LTD", "OYO-ELECTRIC" },
	{ 0x000C68, "SigmaTel, Inc.", "SIGMATEL" },
	{ 0x000C69, "National Radio Astronomy Observatory", "NATIONAL-RADIO-ASTRONOMY-OBSERVATORY" },
	{ 0x000C6A, "MBARI", "MBARI" },
	{ 0x000C6B, "Kurz Industrie-Elektronik GmbH", "KURZ-INDUSTRIE-ELEKTRONIK" },
	{ 0x000C6C, "Elgato Systems LLC", "ELGATO-LLC" },
	{ 0x000C6D, "Edwards Ltd.", "EDWARDS" },
	{ 0x000C6E, "ASUSTEK COMPUTER INC.", "ASUSTEK" },
	{ 0x000C6F, "Amtek system co.,LTD.", "AMTEK-SYSTEM" },
	{ 0x000C70, "ACC GmbH", "ACC" },
	{ 0x000C71, "Wybron, Inc", "WYBRON" },
	{ 0x000C72, "Tempearl Industrial Co., Ltd.", "TEMPEARL-INDUSTRIAL" },
	{ 0x000C73, "TELSON ELECTRONICS CO., LTD", "TELSON-ELECTRONICS" },
	{ 0x000C74, "RIVERTEC CORPORATION", "RIVERTEC" },
	{ 0x000C75, "Oriental integrated electronics. LTD", "ORIENTAL-INTEGRATED-ELECTRONICS" },
	{ 0x000C76, "MICRO-STAR INTERNATIONAL CO., LTD.", "MICRO-STAR" },
	{ 0x000C77, "Life Racing Ltd", "LIFE-RACING" },
	{ 0x000C78, "In-Tech Electronics Limited", "IN-TECH-ELECTRONICS" },
	{ 0x000C79, "Extel Communications P/L", "EXTEL-COMMUNICATION-P/L" },
	{ 0x000C7A, "DaTARIUS Technologies GmbH", "DATARIUS-TECHNOLOGIES" },
	{ 0x000C7B, "ALPHA PROJECT Co.,Ltd.", "ALPHA-PROJECT" },
	{ 0x000C7C, "Internet Information Image Inc.", "INTERNET-INFORMATION-IMAGE" },
	{ 0x000C7D, "TEIKOKU ELECTRIC MFG. CO., LTD", "TEIKOKU-ELECTRIC-MFG" },
	{ 0x000C7E, "Tellium Incorporated", "TELLIUM-INCORPORATED" },
	{ 0x000C7F, "synertronixx GmbH", "SYNERTRONIXX" },
	{ 0x000C80, "Opelcomm Inc.", "OPELCOMM" },
	{ 0x000C81, "Schneider Electric (Australia)", "SCHNEIDER-ELECTRIC-AUSTRALIA" },
	{ 0x000C82, "NETWORK TECHNOLOGIES INC", "NETWORK-TECHNOLOGIES" },
	{ 0x000C83, "Logical Solutions", "LOGICAL-SOLUTIONS" },
	{ 0x000C84, "Eazix, Inc.", "EAZIX" },
	{ 0x000C85, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000C86, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000C87, "AMD", "AMD" },
	{ 0x000C88, "Apache Micro Peripherals, Inc.", "APACHE-MICRO-PERIPHERALS" },
	{ 0x000C89, "AC Electric Vehicles, Ltd.", "AC-ELECTRIC-VEHICLES" },
	{ 0x000C8A, "Bose Corporation", "BOSE" },
	{ 0x000C8B, "Connect Tech Inc", "CONNECT-TECH" },
	{ 0x000C8C, "KODICOM CO.,LTD.", "KODICOM" },
	{ 0x000C8D, "MATRIX VISION GmbH", "MATRIX-VISION" },
	{ 0x000C8E, "Mentor Engineering Inc", "MENTOR-ENGINEERING" },
	{ 0x000C8F, "Nergal s.r.l.", "NERGAL-S-R-L" },
	{ 0x000C90, "Octasic Inc.", "OCTASIC" },
	{ 0x000C91, "Riverhead Networks Inc.", "RIVERHEAD-NETWORKS" },
	{ 0x000C92, "WolfVision Gmbh", "WOLFVISION" },
	{ 0x000C93, "Xeline Co., Ltd.", "XELINE" },
	{ 0x000C94, "United Electronic Industries, Inc. (EUI)", "UNITED-INDUSTRIES-EUI" },
	{ 0x000C95, "PrimeNet", "PRIMENET" },
	{ 0x000C96, "OQO, Inc.", "OQO" },
	{ 0x000C97, "NV ADB TTV Technologies SA", "NV-ADB-TTV-TECHNOLOGIES" },
	{ 0x000C98, "LETEK Communications Inc.", "LETEK-COMMUNICATION" },
	{ 0x000C99, "HITEL LINK Co.,Ltd", "HITEL-LINK" },
	{ 0x000C9A, "Hitech Electronics Corp.", "HITECH-ELECTRONICS" },
	{ 0x000C9B, "EE Solutions, Inc", "EE-SOLUTIONS" },
	{ 0x000C9C, "Chongho information & communications", "CHONGHO-INFORMATION-COMMUNICATION" },
	{ 0x000C9D, "UbeeAirWalk, Inc.", "UBEEAIRWALK" },
	{ 0x000C9E, "MemoryLink Corp.", "MEMORYLINK" },
	{ 0x000C9F, "NKE Corporation", "NKE" },
	{ 0x000CA0, "StorCase Technology, Inc.", "STORCASE" },
	{ 0x000CA1, "SIGMACOM Co., LTD.", "SIGMACOM" },
	{ 0x000CA2, "Harmonic Video Network", "HARMONIC-VIDEO-NETWORK" },
	{ 0x000CA3, "Rancho Technology, Inc.", "RANCHO" },
	{ 0x000CA4, "Prompttec Product Management GmbH", "PROMPTTEC-PRODUCT-MANAGEMENT" },
	{ 0x000CA5, "Naman NZ LTd", "NAMAN-NZ" },
	{ 0x000CA6, "Mintera Corporation", "MINTERA" },
	{ 0x000CA7, "Metro (Suzhou) Technologies Co., Ltd.", "METRO-SUZHOU-TECHNOLOGIES" },
	{ 0x000CA8, "Garuda Networks Corporation", "GARUDA-NETWORKS" },
	{ 0x000CA9, "Ebtron Inc.", "EBTRON" },
	{ 0x000CAA, "Cubic Transportation Systems Inc", "CUBIC-TRANSPORTATION" },
	{ 0x000CAB, "COMMEND International", "COMMEND" },
	{ 0x000CAC, "Citizen Watch Co., Ltd.", "CITIZEN-WATCH" },
	{ 0x000CAD, "BTU International", "BTU" },
	{ 0x000CAE, "Ailocom Oy", "AILOCOM-OY" },
	{ 0x000CAF, "TRI TERM CO.,LTD.", "TRI-TERM" },
	{ 0x000CB0, "Star Semiconductor Corporation", "STAR-SEMICONDUCTOR" },
	{ 0x000CB1, "Salland Engineering (Europe) BV", "SALLAND-ENGINEERING-EUROPE" },
	{ 0x000CB2, "UNION co., ltd.", "UNION" },
	{ 0x000CB3, "ROUND Co.,Ltd.", "ROUND" },
	{ 0x000CB4, "AutoCell Laboratories, Inc.", "AUTOCELL-LABORATORIES" },
	{ 0x000CB5, "Premier Technolgies, Inc", "PREMIER-TECHNOLGIES" },
	{ 0x000CB6, "NANJING SEU MOBILE & INTERNET TECHNOLOGY CO.,LTD", "NANJING-SEU-MOBILE-INTERNET" },
	{ 0x000CB7, "Nanjing Huazhuo Electronics Co., Ltd.", "NANJING-HUAZHUO-ELECTRONICS" },
	{ 0x000CB8, "MEDION AG", "MEDION" },
	{ 0x000CB9, "LEA", "LEA" },
	{ 0x000CBA, "Jamex, Inc.", "JAMEX" },
	{ 0x000CBB, "ISKRAEMECO", "ISKRAEMECO" },
	{ 0x000CBC, "Iscutum", "ISCUTUM" },
	{ 0x000CBD, "Interface Masters, Inc", "INTERFACE-MASTERS" },
	{ 0x000CBE, "Innominate Security Technologies AG", "INNOMINATE-SECURITY-TECHNOLOGIES" },
	{ 0x000CBF, "Holy Stone Ent. Co., Ltd.", "HOLY-STONE-ENT" },
	{ 0x000CC0, "Genera Oy", "GENERA-OY" },
	{ 0x000CC1, "Cooper Industries Inc.", "COOPER-INDUSTRIES" },
	{ 0x000CC2, "ControlNet (India) Private Limited", "CONTROLNET-INDIA-PRIVATE" },
	{ 0x000CC3, "BeWAN systems", "BEWAN" },
	{ 0x000CC4, "Tiptel AG", "TIPTEL" },
	{ 0x000CC5, "Nextlink Co., Ltd.", "NEXTLINK" },
	{ 0x000CC6, "Ka-Ro electronics GmbH", "KA-RO-ELECTRONICS" },
	{ 0x000CC7, "Intelligent Computer Solutions Inc.", "INTELLIGENT-SOLUTIONS" },
	{ 0x000CC8, "Xytronix Research & Design, Inc.", "XYTRONIX-RESEARCH-DESIGN" },
	{ 0x000CC9, "ILWOO DATA & TECHNOLOGY CO.,LTD", "ILWOO-DATA" },
	{ 0x000CCA, "HGST a Western Digital Company", "HGST-A-WESTERN-DIGITAL-COMPANY" },
	{ 0x000CCB, "Design Combus Ltd", "DESIGN-COMBUS" },
	{ 0x000CCC, "Aeroscout Ltd.", "AEROSCOUT" },
	{ 0x000CCD, "IEC - TC57", "IEC---TC57" },
	{ 0x000CCE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000CCF, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000CD0, "Symetrix", "SYMETRIX" },
	{ 0x000CD1, "SFOM Technology Corp.", "SFOM" },
	{ 0x000CD2, "Schaffner EMV AG", "SCHAFFNER-EMV" },
	{ 0x000CD3, "Prettl Elektronik Radeberg GmbH", "PRETTL-ELEKTRONIK-RADEBERG" },
	{ 0x000CD4, "Positron Public Safety Systems inc.", "POSITRON-PUBLIC-SAFETY" },
	{ 0x000CD5, "Passave Inc.", "PASSAVE" },
	{ 0x000CD6, "PARTNER TECH", "PARTNER-TECH" },
	{ 0x000CD7, "Nallatech Ltd", "NALLATECH" },
	{ 0x000CD8, "M. K. Juchheim GmbH & Co", "M-K-JUCHHEIM" },
	{ 0x000CD9, "Itcare Co., Ltd", "ITCARE" },
	{ 0x000CDA, "FreeHand Systems, Inc.", "FREEHAND" },
	{ 0x000CDB, "Brocade Communications Systems, Inc", "BROCADE-COMMUNICATION" },
	{ 0x000CDC, "BECS Technology, Inc", "BECS" },
	{ 0x000CDD, "AOS Technologies AG", "AOS-TECHNOLOGIES" },
	{ 0x000CDE, "ABB STOTZ-KONTAKT GmbH", "ABB-STOTZ-KONTAKT" },
	{ 0x000CDF, "PULNiX America, Inc", "PULNIX-AMERICA" },
	{ 0x000CE0, "Trek Diagnostics Inc.", "TREK-DIAGNOSTICS" },
	{ 0x000CE1, "The Open Group", "THE-OPEN-GROUP" },
	{ 0x000CE2, "Rolls-Royce", "ROLLS-ROYCE" },
	{ 0x000CE3, "Option International N.V.", "OPTION-N-V" },
	{ 0x000CE4, "NeuroCom International, Inc.", "NEUROCOM" },
	{ 0x000CE5, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x000CE6, "Meru Networks Inc", "MERU-NETWORKS" },
	{ 0x000CE7, "MediaTek Inc.", "MEDIATEK" },
	{ 0x000CE8, "GuangZhou AnJuBao Co., Ltd", "GUANGZHOU-ANJUBAO" },
	{ 0x000CE9, "BLOOMBERG L.P.", "BLOOMBERG-L-P" },
	{ 0x000CEA, "aphona Kommunikationssysteme", "APHONA-KOMMUNIKATIONSSYSTEME" },
	{ 0x000CEB, "CNMP Networks, Inc.", "CNMP-NETWORKS" },
	{ 0x000CEC, "Spectracom Corp.", "SPECTRACOM" },
	{ 0x000CED, "Real Digital Media", "REAL-DIGITAL-MEDIA" },
	{ 0x000CEE, "jp-embedded", "JP-EMBEDDED" },
	{ 0x000CEF, "Open Networks Engineering Ltd", "OPEN-NETWORKS-ENGINEERING" },
	{ 0x000CF0, "M & N GmbH", "M-N" },
	{ 0x000CF1, "Intel Corporation", "INTEL" },
	{ 0x000CF2, "GAMESA Eólica", "GAMESA-EóLICA" },
	{ 0x000CF3, "CALL IMAGE SA", "CALL-IMAGE" },
	{ 0x000CF4, "AKATSUKI ELECTRIC MFG.CO.,LTD.", "AKATSUKI-ELECTRIC-MFG" },
	{ 0x000CF5, "InfoExpress", "INFOEXPRESS" },
	{ 0x000CF6, "Sitecom Europe BV", "SITECOM-EUROPE" },
	{ 0x000CF7, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000CF8, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000CF9, "Xylem Water Solutions", "XYLEM-WATER-SOLUTIONS" },
	{ 0x000CFA, "Digital Systems Corp", "DIGITAL" },
	{ 0x000CFB, "Korea Network Systems", "KOREA-NETWORK" },
	{ 0x000CFC, "S2io Technologies Corp", "S2IO-TECHNOLOGIES" },
	{ 0x000CFD, "Hyundai ImageQuest Co.,Ltd.", "HYUNDAI-IMAGEQUEST" },
	{ 0x000CFE, "Grand Electronic Co., Ltd", "GRAND" },
	{ 0x000CFF, "MRO-TEK LIMITED", "MRO-TEK" },
	{ 0x000D00, "Seaway Networks Inc.", "SEAWAY-NETWORKS" },
	{ 0x000D01, "P&E Microcomputer Systems, Inc.", "P-E-MICROCOMPUTER" },
	{ 0x000D02, "NEC Platforms, Ltd.", "NEC-PLATFORMS" },
	{ 0x000D03, "Matrics, Inc.", "MATRICS" },
	{ 0x000D04, "Foxboro Eckardt Development GmbH", "FOXBORO-ECKARDT-DEVELOPMENT" },
	{ 0x000D05, "cybernet manufacturing inc.", "CYBERNET-MANUFACTURING" },
	{ 0x000D06, "Compulogic Limited", "COMPULOGIC" },
	{ 0x000D07, "Calrec Audio Ltd", "CALREC-AUDIO" },
	{ 0x000D08, "AboveCable, Inc.", "ABOVECABLE" },
	{ 0x000D09, "Yuehua(Zhuhai) Electronic CO. LTD", "YUEHUA-ZHUHAI" },
	{ 0x000D0A, "Projectiondesign as", "PROJECTIONDESIGN-AS" },
	{ 0x000D0B, "Buffalo Inc.", "BUFFALO" },
	{ 0x000D0C, "MDI Security Systems", "MDI-SECURITY" },
	{ 0x000D0D, "ITSupported, LLC", "ITSUPPORTED-LLC" },
	{ 0x000D0E, "Inqnet Systems, Inc.", "INQNET" },
	{ 0x000D0F, "Finlux Ltd", "FINLUX" },
	{ 0x000D10, "Embedtronics Oy", "EMBEDTRONICS-OY" },
	{ 0x000D11, "DENTSPLY - Gendex", "DENTSPLY---GENDEX" },
	{ 0x000D12, "AXELL Corporation", "AXELL" },
	{ 0x000D13, "Wilhelm Rutenbeck GmbH&Co.KG", "WILHELM-RUTENBECK" },
	{ 0x000D14, "Vtech Innovation LP dba Advanced American Telephones", "VTECH-INNOVATION-LP-DBA-ADVANCED-AMERICAN-TELEPHONES" },
	{ 0x000D15, "Voipac s.r.o.", "VOIPAC-S-R-O" },
	{ 0x000D16, "UHS Systems Pty Ltd", "UHS-PTY" },
	{ 0x000D17, "Turbo Networks Co.Ltd", "TURBO-NETWORKS" },
	{ 0x000D18, "Mega-Trend Electronics CO., LTD.", "MEGA-TREND-ELECTRONICS" },
	{ 0x000D19, "ROBE Show lighting", "ROBE-SHOW-LIGHTING" },
	{ 0x000D1A, "Mustek System Inc.", "MUSTEK-SYSTEM" },
	{ 0x000D1B, "Kyoto Electronics Manufacturing Co., Ltd.", "KYOTO-ELECTRONICS-MANUFACTURING" },
	{ 0x000D1C, "Amesys Defense", "AMESYS-DEFENSE" },
	{ 0x000D1D, "HIGH-TEK HARNESS ENT. CO., LTD.", "HIGH-TEK-HARNESS-ENT" },
	{ 0x000D1E, "Control Techniques", "CONTROL-TECHNIQUES" },
	{ 0x000D1F, "AV Digital", "AV-DIGITAL" },
	{ 0x000D20, "ASAHIKASEI TECHNOSYSTEM CO.,LTD.", "ASAHIKASEI-TECHNOSYSTEM" },
	{ 0x000D21, "WISCORE Inc.", "WISCORE" },
	{ 0x000D22, "Unitronics LTD", "UNITRONICS" },
	{ 0x000D23, "Smart Solution, Inc", "SMART-SOLUTION" },
	{ 0x000D24, "SENTEC E&E CO., LTD.", "SENTEC-E-E" },
	{ 0x000D25, "SANDEN CORPORATION", "SANDEN" },
	{ 0x000D26, "Primagraphics Limited", "PRIMAGRAPHICS" },
	{ 0x000D27, "MICROPLEX Printware AG", "MICROPLEX-PRINTWARE" },
	{ 0x000D28, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000D29, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000D2A, "Scanmatic AS", "SCANMATIC-AS" },
	{ 0x000D2B, "Racal Instruments", "RACAL-INSTRUMENTS" },
	{ 0x000D2C, "Patapsco Designs Ltd", "PATAPSCO-DESIGNS" },
	{ 0x000D2D, "NCT Deutschland GmbH", "NCT-DEUTSCHLAND" },
	{ 0x000D2E, "Matsushita Avionics Systems Corporation", "MATSUSHITA-AVIONICS" },
	{ 0x000D2F, "AIN Comm.Tech.Co., LTD", "AIN-COMM-TECH" },
	{ 0x000D30, "IceFyre Semiconductor", "ICEFYRE-SEMICONDUCTOR" },
	{ 0x000D31, "Compellent Technologies, Inc.", "COMPELLENT-TECHNOLOGIES" },
	{ 0x000D32, "DispenseSource, Inc.", "DISPENSESOURCE" },
	{ 0x000D33, "Prediwave Corp.", "PREDIWAVE" },
	{ 0x000D34, "Shell International Exploration and Production, Inc.", "SHELL-EXPLORATION-AND-PRODUCTION" },
	{ 0x000D35, "PAC International Ltd", "PAC" },
	{ 0x000D36, "Wu Han Routon Electronic Co., Ltd", "WU-HAN-ROUTON" },
	{ 0x000D37, "WIPLUG", "WIPLUG" },
	{ 0x000D38, "NISSIN INC.", "NISSIN" },
	{ 0x000D39, "Network Electronics", "NETWORK-ELECTRONICS" },
	{ 0x000D3A, "Microsoft Corp.", "MICROSOFT" },
	{ 0x000D3B, "Microelectronics Technology Inc.", "MICROELECTRONICS" },
	{ 0x000D3C, "i.Tech Dynamic Ltd", "I-TECH-DYNAMIC" },
	{ 0x000D3D, "Hammerhead Systems, Inc.", "HAMMERHEAD" },
	{ 0x000D3E, "APLUX Communications Ltd.", "APLUX-COMMUNICATION" },
	{ 0x000D3F, "VTI Instruments Corporation", "VTI-INSTRUMENTS" },
	{ 0x000D40, "Verint Loronix Video Solutions", "VERINT-LORONIX-VIDEO-SOLUTIONS" },
	{ 0x000D41, "Siemens AG ICM MP UC RD IT KLF1", "SIEMENS-ICM-MP-UC-RD-IT-KLF1" },
	{ 0x000D42, "Newbest Development Limited", "NEWBEST-DEVELOPMENT" },
	{ 0x000D43, "DRS Tactical Systems Inc.", "DRS-TACTICAL" },
	{ 0x000D44, "Audio BU - Logitech", "AUDIO-BU---LOGITECH" },
	{ 0x000D45, "Tottori SANYO Electric Co., Ltd.", "TOTTORI-SANYO-ELECTRIC" },
	{ 0x000D46, "Parker SSD Drives", "PARKER-SSD-DRIVES" },
	{ 0x000D47, "Collex", "COLLEX" },
	{ 0x000D48, "AEWIN Technologies Co., Ltd.", "AEWIN-TECHNOLOGIES" },
	{ 0x000D49, "Triton Systems of Delaware, Inc.", "TRITON-OF-DELAWARE" },
	{ 0x000D4A, "Steag ETA-Optik", "STEAG-ETA-OPTIK" },
	{ 0x000D4B, "Roku, LLC", "ROKU-LLC" },
	{ 0x000D4C, "Outline Electronics Ltd.", "OUTLINE-ELECTRONICS" },
	{ 0x000D4D, "Ninelanes", "NINELANES" },
	{ 0x000D4E, "NDR Co.,LTD.", "NDR" },
	{ 0x000D4F, "Kenwood Corporation", "KENWOOD" },
	{ 0x000D50, "Galazar Networks", "GALAZAR-NETWORKS" },
	{ 0x000D51, "DIVR Systems, Inc.", "DIVR" },
	{ 0x000D52, "Comart system", "COMART-SYSTEM" },
	{ 0x000D53, "Beijing 5w Communication Corp.", "BEIJING-5W-COMMUNICATION" },
	{ 0x000D54, "3Com Ltd", "3COM" },
	{ 0x000D55, "SANYCOM Technology Co.,Ltd", "SANYCOM" },
	{ 0x000D56, "Dell Inc", "DELL" },
	{ 0x000D57, "Fujitsu I-Network Systems Limited.", "FUJITSU-I-NETWORK" },
	{ 0x000D58, "PRIVATE", "PRIVATE" },
	{ 0x000D59, "Amity Systems, Inc.", "AMITY" },
	{ 0x000D5A, "Tiesse SpA", "TIESSE-SPA" },
	{ 0x000D5B, "Smart Empire Investments Limited", "SMART-EMPIRE-INVESTMENTS" },
	{ 0x000D5C, "Robert Bosch GmbH, VT-ATMO", "ROBERT-BOSCH-VT-ATMO" },
	{ 0x000D5D, "Raritan Computer, Inc", "RARITAN" },
	{ 0x000D5E, "NEC Personal Products", "NEC-PERSONAL-PRODUCTS" },
	{ 0x000D5F, "Minds Inc", "MINDS" },
	{ 0x000D60, "IBM Corp", "IBM" },
	{ 0x000D61, "Giga-Byte Technology Co., Ltd.", "GIGA-BYTE" },
	{ 0x000D62, "Funkwerk Dabendorf GmbH", "FUNKWERK-DABENDORF" },
	{ 0x000D63, "DENT Instruments, Inc.", "DENT-INSTRUMENTS" },
	{ 0x000D64, "COMAG Handels AG", "COMAG-HANDELS" },
	{ 0x000D65, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000D66, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000D67, "Ericsson", "ERICSSON" },
	{ 0x000D68, "Vinci Systems, Inc.", "VINCI" },
	{ 0x000D69, "TMT&D Corporation", "TMT-D" },
	{ 0x000D6A, "Redwood Technologies LTD", "REDWOOD-TECHNOLOGIES" },
	{ 0x000D6B, "Mita-Teknik A/S", "MITA-TEKNIK-A/S" },
	{ 0x000D6C, "M-Audio", "M-AUDIO" },
	{ 0x000D6D, "K-Tech Devices Corp.", "K-TECH-DEVICES" },
	{ 0x000D6E, "K-Patents Oy", "K-PATENTS-OY" },
	{ 0x000D6F, "Ember Corporation", "EMBER" },
	{ 0x000D70, "Datamax Corporation", "DATAMAX" },
	{ 0x000D71, "boca systems", "BOCA" },
	{ 0x000D72, "2Wire, Inc", "2WIRE" },
	{ 0x000D73, "Technical Support, Inc.", "TECHNICAL-SUPPORT" },
	{ 0x000D74, "Sand Network Systems, Inc.", "SAND-NETWORK" },
	{ 0x000D75, "Kobian Pte Ltd - Taiwan Branch", "KOBIAN-PTE---TAIWAN-BRANCH" },
	{ 0x000D76, "Hokuto Denshi Co,. Ltd.", "HOKUTO-DENSHI" },
	{ 0x000D77, "FalconStor Software", "FALCONSTOR-SOFTWARE" },
	{ 0x000D78, "Engineering & Security", "ENGINEERING-SECURITY" },
	{ 0x000D79, "Dynamic Solutions Co,.Ltd.", "DYNAMIC-SOLUTIONS" },
	{ 0x000D7A, "DiGATTO Asia Pacific Pte Ltd", "DIGATTO-ASIA-PACIFIC-PTE" },
	{ 0x000D7B, "Consensys Computers Inc.", "CONSENSYS-COMPUTERS" },
	{ 0x000D7C, "Codian Ltd", "CODIAN" },
	{ 0x000D7D, "Afco Systems", "AFCO" },
	{ 0x000D7E, "Axiowave Networks, Inc.", "AXIOWAVE-NETWORKS" },
	{ 0x000D7F, "MIDAS COMMUNICATION TECHNOLOGIES PTE LTD ( Foreign Branch)", "MIDAS-COMMUNICATION-TECHNOLOGIES-PTE-FOREIGN-BRANCH" },
	{ 0x000D80, "Online Development Inc", "ONLINE-DEVELOPMENT" },
	{ 0x000D81, "Pepperl+Fuchs GmbH", "PEPPERL+FUCHS" },
	{ 0x000D82, "PHS srl", "PHS-SRL" },
	{ 0x000D83, "Sanmina-SCI Hungary Ltd.", "SANMINA-SCI-HUNGARY" },
	{ 0x000D84, "Makus Inc.", "MAKUS" },
	{ 0x000D85, "Tapwave, Inc.", "TAPWAVE" },
	{ 0x000D86, "Huber + Suhner AG", "HUBER-+-SUHNER" },
	{ 0x000D87, "Elitegroup Computer System Co. (ECS)", "ELITEGROUP-SYSTEM-ECS" },
	{ 0x000D88, "D-Link Corporation", "D-LINK" },
	{ 0x000D89, "Bils Technology Inc", "BILS" },
	{ 0x000D8A, "Winners Electronics Co., Ltd.", "WINNERS-ELECTRONICS" },
	{ 0x000D8B, "T&D Corporation", "T-D" },
	{ 0x000D8C, "Shanghai Wedone Digital Ltd. CO.", "SHANGHAI-WEDONE-DIGITAL" },
	{ 0x000D8D, "Prosoft Technology, Inc", "PROSOFT" },
	{ 0x000D8E, "Koden Electronics Co., Ltd.", "KODEN-ELECTRONICS" },
	{ 0x000D8F, "King Tsushin Kogyo Co., LTD.", "KING-TSUSHIN-KOGYO" },
	{ 0x000D90, "Factum Electronics AB", "FACTUM-ELECTRONICS-AB" },
	{ 0x000D91, "Eclipse (HQ Espana) S.L.", "ECLIPSE-HQ-ESPANA-S-L" },
	{ 0x000D92, "Arima Communication Corporation", "ARIMA-COMMUNICATION" },
	{ 0x000D93, "Apple", "APPLE" },
	{ 0x000D94, "AFAR Communications,Inc", "AFAR-COMMUNICATION" },
	{ 0x000D95, "Opti-cell, Inc.", "OPTI-CELL" },
	{ 0x000D96, "Vtera Technology Inc.", "VTERA" },
	{ 0x000D97, "Tropos Networks, Inc.", "TROPOS-NETWORKS" },
	{ 0x000D98, "S.W.A.C. Schmitt-Walter Automation Consult GmbH", "S-W-A-C-SCHMITT-WALTER-AUTOMATION-CONSULT" },
	{ 0x000D99, "Orbital Sciences Corp.; Launch Systems Group", "ORBITAL-SCIENCES-LAUNCH-GROUP" },
	{ 0x000D9A, "INFOTEC LTD", "INFOTEC" },
	{ 0x000D9B, "Heraeus Electro-Nite International N.V.", "HERAEUS-ELECTRO-NITE-N-V" },
	{ 0x000D9C, "Elan GmbH & Co KG", "ELAN" },
	{ 0x000D9D, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x000D9E, "TOKUDEN OHIZUMI SEISAKUSYO Co.,Ltd.", "TOKUDEN-OHIZUMI-SEISAKUSYO" },
	{ 0x000D9F, "RF Micro Devices", "RF-MICRO-DEVICES" },
	{ 0x000DA0, "NEDAP N.V.", "NEDAP-N-V" },
	{ 0x000DA1, "MIRAE ITS Co.,LTD.", "MIRAE-ITS" },
	{ 0x000DA2, "Infrant Technologies, Inc.", "INFRANT-TECHNOLOGIES" },
	{ 0x000DA3, "Emerging Technologies Limited", "EMERGING-TECHNOLOGIES" },
	{ 0x000DA4, "DOSCH & AMAND SYSTEMS AG", "DOSCH-AMAND" },
	{ 0x000DA5, "Fabric7 Systems, Inc", "FABRIC7" },
	{ 0x000DA6, "Universal Switching Corporation", "UNIVERSAL-SWITCHING" },
	{ 0x000DA7, "PRIVATE", "PRIVATE" },
	{ 0x000DA8, "Teletronics Technology Corporation", "TELETRONICS" },
	{ 0x000DA9, "T.E.A.M. S.L.", "T-E-A-M-S-L" },
	{ 0x000DAA, "S.A.Tehnology co.,Ltd.", "TEHNOLOGY" },
	{ 0x000DAB, "Parker Hannifin GmbH Electromechanical Division Europe", "PARKER-HANNIFIN-ELECTROMECHANICAL-DIVISION-EUROPE" },
	{ 0x000DAC, "Japan CBM Corporation", "JAPAN-CBM" },
	{ 0x000DAD, "Dataprobe, Inc.", "DATAPROBE" },
	{ 0x000DAE, "SAMSUNG HEAVY INDUSTRIES CO., LTD.", "SAMSUNG-HEAVY-INDUSTRIES" },
	{ 0x000DAF, "Plexus Corp (UK) Ltd", "PLEXUS-UK" },
	{ 0x000DB0, "Olym-tech Co.,Ltd.", "OLYM-TECH" },
	{ 0x000DB1, "Japan Network Service Co., Ltd.", "JAPAN-NETWORK-SERVICE" },
	{ 0x000DB2, "Ammasso, Inc.", "AMMASSO" },
	{ 0x000DB3, "SDO Communication Corperation", "SDO-COMMUNICATION-CORPERATION" },
	{ 0x000DB4, "NETASQ", "NETASQ" },
	{ 0x000DB5, "GLOBALSAT TECHNOLOGY CORPORATION", "GLOBALSAT" },
	{ 0x000DB6, "Broadcom Corporation", "BROADCOM" },
	{ 0x000DB7, "SANKO ELECTRIC CO,.LTD", "SANKO-ELECTRIC" },
	{ 0x000DB8, "SCHILLER AG", "SCHILLER" },
	{ 0x000DB9, "PC Engines GmbH", "PC-ENGINES" },
	{ 0x000DBA, "Océ Document Technologies GmbH", "OCé-DOCUMENT-TECHNOLOGIES" },
	{ 0x000DBB, "Nippon Dentsu Co.,Ltd.", "NIPPON-DENTSU" },
	{ 0x000DBC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000DBD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000DBE, "Bel Fuse Europe Ltd.,UK", "BEL-FUSE-EUROPE-UK" },
	{ 0x000DBF, "TekTone Sound & Signal Mfg., Inc.", "TEKTONE-SOUND-SIGNAL-MFG" },
	{ 0x000DC0, "Spagat AS", "SPAGAT-AS" },
	{ 0x000DC1, "SafeWeb Inc", "SAFEWEB" },
	{ 0x000DC2, "PRIVATE", "PRIVATE" },
	{ 0x000DC3, "First Communication, Inc.", "FIRST-COMMUNICATION" },
	{ 0x000DC4, "Emcore Corporation", "EMCORE" },
	{ 0x000DC5, "EchoStar Global B.V.", "ECHOSTAR-GLOBAL" },
	{ 0x000DC6, "DigiRose Technology Co., Ltd.", "DIGIROSE" },
	{ 0x000DC7, "COSMIC ENGINEERING INC.", "COSMIC-ENGINEERING" },
	{ 0x000DC8, "AirMagnet, Inc", "AIRMAGNET" },
	{ 0x000DC9, "THALES Elektronik Systeme GmbH", "THALES-ELEKTRONIK-SYSTEME" },
	{ 0x000DCA, "Tait Electronics", "TAIT-ELECTRONICS" },
	{ 0x000DCB, "Petcomkorea Co., Ltd.", "PETCOMKOREA" },
	{ 0x000DCC, "NEOSMART Corp.", "NEOSMART" },
	{ 0x000DCD, "GROUPE TXCOM", "GROUPE-TXCOM" },
	{ 0x000DCE, "Dynavac Technology Pte Ltd", "DYNAVAC-PTE" },
	{ 0x000DCF, "Cidra Corp.", "CIDRA" },
	{ 0x000DD0, "TetraTec Instruments GmbH", "TETRATEC-INSTRUMENTS" },
	{ 0x000DD1, "Stryker Corporation", "STRYKER" },
	{ 0x000DD2, "Simrad Optronics ASA", "SIMRAD-OPTRONICS-ASA" },
	{ 0x000DD3, "SAMWOO Telecommunication Co.,Ltd.", "SAMWOO-TELECOMMUNICATION" },
	{ 0x000DD4, "Symantec Corporation", "SYMANTEC" },
	{ 0x000DD5, "O'RITE TECHNOLOGY CO.,LTD", "O-RITE" },
	{ 0x000DD6, "ITI LTD", "ITI" },
	{ 0x000DD7, "Bright", "BRIGHT" },
	{ 0x000DD8, "BBN", "BBN" },
	{ 0x000DD9, "Anton Paar GmbH", "ANTON-PAAR" },
	{ 0x000DDA, "ALLIED TELESIS K.K.", "ALLIED-TELESIS-K-K" },
	{ 0x000DDB, "AIRWAVE TECHNOLOGIES INC.", "AIRWAVE-TECHNOLOGIES" },
	{ 0x000DDC, "VAC", "VAC" },
	{ 0x000DDD, "Profilo Telra Elektronik Sanayi ve Ticaret. A.Ş", "PROFILO-TELRA-ELEKTRONIK-SANAYI-VE-TICARET-A-Ş" },
	{ 0x000DDE, "Joyteck Co., Ltd.", "JOYTECK" },
	{ 0x000DDF, "Japan Image & Network Inc.", "JAPAN-IMAGE-NETWORK" },
	{ 0x000DE0, "ICPDAS Co.,LTD", "ICPDAS" },
	{ 0x000DE1, "Control Products, Inc.", "CONTROL-PRODUCTS" },
	{ 0x000DE2, "CMZ Sistemi Elettronici", "CMZ-SISTEMI-ELETTRONICI" },
	{ 0x000DE3, "AT Sweden AB", "AT-SWEDEN-AB" },
	{ 0x000DE4, "DIGINICS, Inc.", "DIGINICS" },
	{ 0x000DE5, "Samsung Thales", "SAMSUNG-THALES" },
	{ 0x000DE6, "YOUNGBO ENGINEERING CO.,LTD", "YOUNGBO-ENGINEERING" },
	{ 0x000DE7, "Snap-on OEM Group", "SNAP-ON-OEM-GROUP" },
	{ 0x000DE8, "Nasaco Electronics Pte. Ltd", "NASACO-ELECTRONICS-PTE" },
	{ 0x000DE9, "Napatech Aps", "NAPATECH-APS" },
	{ 0x000DEA, "Kingtel Telecommunication Corp.", "KINGTEL-TELECOMMUNICATION" },
	{ 0x000DEB, "CompXs Limited", "COMPXS" },
	{ 0x000DEC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000DED, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000DEE, "Andrew RF Power Amplifier Group", "ANDREW-RF-POWER-AMPLIFIER-GROUP" },
	{ 0x000DEF, "Soc. Coop. Bilanciai", "SOC-COOP-BILANCIAI" },
	{ 0x000DF0, "QCOM TECHNOLOGY INC.", "QCOM" },
	{ 0x000DF1, "IONIX INC.", "IONIX" },
	{ 0x000DF2, "PRIVATE", "PRIVATE" },
	{ 0x000DF3, "Asmax Solutions", "ASMAX-SOLUTIONS" },
	{ 0x000DF4, "Watertek Co.", "WATERTEK" },
	{ 0x000DF5, "Teletronics International Inc.", "TELETRONICS" },
	{ 0x000DF6, "Technology Thesaurus Corp.", "THESAURUS" },
	{ 0x000DF7, "Space Dynamics Lab", "SPACE-DYNAMICS-LAB" },
	{ 0x000DF8, "ORGA Kartensysteme GmbH", "ORGA-KARTENSYSTEME" },
	{ 0x000DF9, "NDS Limited", "NDS" },
	{ 0x000DFA, "Micro Control Systems Ltd.", "MICRO-CONTROL" },
	{ 0x000DFB, "Komax AG", "KOMAX" },
	{ 0x000DFC, "ITFOR Inc.", "ITFOR" },
	{ 0x000DFD, "Huges Hi-Tech Inc.,", "HUGES-HI-TECH" },
	{ 0x000DFE, "Hauppauge Computer Works, Inc.", "HAUPPAUGE-WORKS" },
	{ 0x000DFF, "CHENMING MOLD INDUSTRY CORP.", "CHENMING-MOLD-INDUSTRY" },
	{ 0x000E00, "Atrie", "ATRIE" },
	{ 0x000E01, "ASIP Technologies Inc.", "ASIP-TECHNOLOGIES" },
	{ 0x000E02, "Advantech AMT Inc.", "ADVANTECH-AMT" },
	{ 0x000E03, "Emulex Corporation", "EMULEX" },
	{ 0x000E04, "CMA/Microdialysis AB", "CMA/MICRODIALYSIS-AB" },
	{ 0x000E05, "WIRELESS MATRIX CORP.", "WIRELESS-MATRIX" },
	{ 0x000E06, "Team Simoco Ltd", "TEAM-SIMOCO" },
	{ 0x000E07, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x000E08, "Cisco Linksys LLC", "CISCO-LINKSYS-LLC" },
	{ 0x000E09, "Shenzhen Coship Software Co.,LTD.", "SHENZHEN-COSHIP-SOFTWARE" },
	{ 0x000E0A, "SAKUMA DESIGN OFFICE", "SAKUMA-DESIGN-OFFICE" },
	{ 0x000E0B, "Netac Technology Co., Ltd.", "NETAC" },
	{ 0x000E0C, "Intel Corporation", "INTEL" },
	{ 0x000E0D, "Hesch Schröder GmbH", "HESCH-SCHRöDER" },
	{ 0x000E0E, "ESA elettronica S.P.A.", "ESA-ELETTRONICA-S-P-A" },
	{ 0x000E0F, "ERMME", "ERMME" },
	{ 0x000E10, "C-guys, Inc.", "C-GUYS" },
	{ 0x000E11, "BDT Büro und Datentechnik GmbH & Co.KG", "BDT-BüRO-UND-DATENTECHNIK" },
	{ 0x000E12, "Adaptive Micro Systems Inc.", "ADAPTIVE-MICRO" },
	{ 0x000E13, "Accu-Sort Systems inc.", "ACCU-SORT" },
	{ 0x000E14, "Visionary Solutions, Inc.", "VISIONARY-SOLUTIONS" },
	{ 0x000E15, "Tadlys LTD", "TADLYS" },
	{ 0x000E16, "SouthWing S.L.", "SOUTHWING-S-L" },
	{ 0x000E17, "PRIVATE", "PRIVATE" },
	{ 0x000E18, "MyA Technology", "MYA" },
	{ 0x000E19, "LogicaCMG Pty Ltd", "LOGICACMG-PTY" },
	{ 0x000E1A, "JPS Communications", "JPS-COMMUNICATION" },
	{ 0x000E1B, "IAV GmbH", "IAV" },
	{ 0x000E1C, "Hach Company", "HACH-COMPANY" },
	{ 0x000E1D, "ARION Technology Inc.", "ARION" },
	{ 0x000E1E, "QLogic Corporation", "QLOGIC" },
	{ 0x000E1F, "TCL Networks Equipment Co., Ltd.", "TCL-NETWORKS-EQUIPMENT" },
	{ 0x000E20, "ACCESS Systems Americas, Inc.", "ACCESS-AMERICAS" },
	{ 0x000E21, "MTU Friedrichshafen GmbH", "MTU-FRIEDRICHSHAFEN" },
	{ 0x000E22, "PRIVATE", "PRIVATE" },
	{ 0x000E23, "Incipient, Inc.", "INCIPIENT" },
	{ 0x000E24, "Huwell Technology Inc.", "HUWELL" },
	{ 0x000E25, "Hannae Technology Co., Ltd", "HANNAE" },
	{ 0x000E26, "Gincom Technology Corp.", "GINCOM" },
	{ 0x000E27, "Crere Networks, Inc.", "CRERE-NETWORKS" },
	{ 0x000E28, "Dynamic Ratings P/L", "DYNAMIC-RATINGS-P/L" },
	{ 0x000E29, "Shester Communications Inc", "SHESTER-COMMUNICATION" },
	{ 0x000E2A, "PRIVATE", "PRIVATE" },
	{ 0x000E2B, "Safari Technologies", "SAFARI-TECHNOLOGIES" },
	{ 0x000E2C, "Netcodec co.", "NETCODEC" },
	{ 0x000E2D, "Hyundai Digital Technology Co.,Ltd.", "HYUNDAI-DIGITAL" },
	{ 0x000E2E, "Edimax Technology Co., Ltd.", "EDIMAX" },
	{ 0x000E2F, "Roche Diagnostics GmbH", "ROCHE-DIAGNOSTICS" },
	{ 0x000E30, "AERAS Networks, Inc.", "AERAS-NETWORKS" },
	{ 0x000E31, "Olympus Soft Imaging Solutions GmbH", "OLYMPUS-SOFT-IMAGING-SOLUTIONS" },
	{ 0x000E32, "Kontron Medical", "KONTRON-MEDICAL" },
	{ 0x000E33, "Shuko Electronics Co.,Ltd", "SHUKO-ELECTRONICS" },
	{ 0x000E34, "NexGen City, LP", "NEXGEN-CITY-LP" },
	{ 0x000E35, "Intel Corp", "INTEL" },
	{ 0x000E36, "HEINESYS, Inc.", "HEINESYS" },
	{ 0x000E37, "Harms & Wende GmbH & Co.KG", "HARMS-WENDE" },
	{ 0x000E38, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000E39, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000E3A, "Cirrus Logic", "CIRRUS-LOGIC" },
	{ 0x000E3B, "Hawking Technologies, Inc.", "HAWKING-TECHNOLOGIES" },
	{ 0x000E3C, "Transact Technologies Inc", "TRANSACT-TECHNOLOGIES" },
	{ 0x000E3D, "Televic N.V.", "TELEVIC-N-V" },
	{ 0x000E3E, "Sun Optronics Inc", "SUN-OPTRONICS" },
	{ 0x000E3F, "Soronti, Inc.", "SORONTI" },
	{ 0x000E40, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000E41, "NIHON MECHATRONICS CO.,LTD.", "NIHON-MECHATRONICS" },
	{ 0x000E42, "Motic Incoporation Ltd.", "MOTIC-INCOPORATION" },
	{ 0x000E43, "G-Tek Electronics Sdn. Bhd.", "G-TEK-ELECTRONICS-SDN-BHD" },
	{ 0x000E44, "Digital 5, Inc.", "DIGITAL-5" },
	{ 0x000E45, "Beijing Newtry Electronic Technology Ltd", "BEIJING-NEWTRY" },
	{ 0x000E46, "Niigata Seimitsu Co.,Ltd.", "NIIGATA-SEIMITSU" },
	{ 0x000E47, "NCI System Co.,Ltd.", "NCI-SYSTEM" },
	{ 0x000E48, "Lipman TransAction Solutions", "LIPMAN-TRANSACTION-SOLUTIONS" },
	{ 0x000E49, "Forsway Scandinavia AB", "FORSWAY-SCANDINAVIA-AB" },
	{ 0x000E4A, "Changchun Huayu WEBPAD Co.,LTD", "CHANGCHUN-HUAYU-WEBPAD" },
	{ 0x000E4B, "atrium c and i", "ATRIUM-C-AND-I" },
	{ 0x000E4C, "Bermai Inc.", "BERMAI" },
	{ 0x000E4D, "Numesa Inc.", "NUMESA" },
	{ 0x000E4E, "Waveplus Technology Co., Ltd.", "WAVEPLUS" },
	{ 0x000E4F, "Trajet GmbH", "TRAJET" },
	{ 0x000E50, "Thomson Telecom Belgium", "THOMSON-TELECOM-BELGIUM" },
	{ 0x000E51, "tecna elettronica srl", "TECNA-ELETTRONICA-SRL" },
	{ 0x000E52, "Optium Corporation", "OPTIUM" },
	{ 0x000E53, "AV TECH CORPORATION", "AV-TECH" },
	{ 0x000E54, "AlphaCell Wireless Ltd.", "ALPHACELL-WIRELESS" },
	{ 0x000E55, "AUVITRAN", "AUVITRAN" },
	{ 0x000E56, "4G Systems GmbH & Co. KG", "4G" },
	{ 0x000E57, "Iworld Networking, Inc.", "IWORLD-NETWORKING" },
	{ 0x000E58, "Sonos, Inc.", "SONOS" },
	{ 0x000E59, "SAGEM SA", "SAGEM" },
	{ 0x000E5A, "TELEFIELD inc.", "TELEFIELD" },
	{ 0x000E5B, "ParkerVision - Direct2Data", "PARKERVISION---DIRECT2DATA" },
	{ 0x000E5C, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x000E5D, "Triple Play Technologies A/S", "TRIPLE-PLAY-TECHNOLOGIES-A/S" },
	{ 0x000E5E, "Raisecom Technology", "RAISECOM" },
	{ 0x000E5F, "activ-net GmbH & Co. KG", "ACTIV-NET" },
	{ 0x000E60, "360SUN Digital Broadband Corporation", "360SUN-DIGITAL-BROADBAND" },
	{ 0x000E61, "MICROTROL LIMITED", "MICROTROL" },
	{ 0x000E62, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000E63, "Lemke Diagnostics GmbH", "LEMKE-DIAGNOSTICS" },
	{ 0x000E64, "Elphel, Inc", "ELPHEL" },
	{ 0x000E65, "TransCore", "TRANSCORE" },
	{ 0x000E66, "Hitachi Industry & Control Solutions, Ltd.", "HITACHI-INDUSTRY-CONTROL-SOLUTIONS" },
	{ 0x000E67, "Eltis Microelectronics Ltd.", "ELTIS-MICROELECTRONICS" },
	{ 0x000E68, "E-TOP Network Technology Inc.", "E-TOP-NETWORK" },
	{ 0x000E69, "China Electric Power Research Institute", "CHINA-ELECTRIC-POWER-RESEARCH-INSTITUTE" },
	{ 0x000E6A, "3Com Ltd", "3COM" },
	{ 0x000E6B, "Janitza electronics GmbH", "JANITZA-ELECTRONICS" },
	{ 0x000E6C, "Device Drivers Limited", "DEVICE-DRIVERS" },
	{ 0x000E6D, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x000E6E, "MAT S.A. (Mircrelec Advanced Technology)", "MAT-MIRCRELEC-ADVANCED" },
	{ 0x000E6F, "IRIS Corporation Berhad", "IRIS-BERHAD" },
	{ 0x000E70, "in2 Networks", "IN2-NETWORKS" },
	{ 0x000E71, "Gemstar Technology Development Ltd.", "GEMSTAR-DEVELOPMENT" },
	{ 0x000E72, "CTS electronics", "CTS-ELECTRONICS" },
	{ 0x000E73, "Tpack A/S", "TPACK-A/S" },
	{ 0x000E74, "Solar Telecom. Tech", "SOLAR-TELECOM-TECH" },
	{ 0x000E75, "New York Air Brake Corp.", "NEW-YORK-AIR-BRAKE" },
	{ 0x000E76, "GEMSOC INNOVISION INC.", "GEMSOC-INNOVISION" },
	{ 0x000E77, "Decru, Inc.", "DECRU" },
	{ 0x000E78, "Amtelco", "AMTELCO" },
	{ 0x000E79, "Ample Communications Inc.", "AMPLE-COMMUNICATION" },
	{ 0x000E7A, "GemWon Communications Co., Ltd.", "GEMWON-COMMUNICATION" },
	{ 0x000E7B, "Toshiba", "TOSHIBA" },
	{ 0x000E7C, "Televes S.A.", "TELEVES" },
	{ 0x000E7D, "Electronics Line 3000 Ltd.", "ELECTRONICS-LINE-3000" },
	{ 0x000E7E, "ionSign Oy", "IONSIGN-OY" },
	{ 0x000E7F, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x000E80, "Thomson Technology Inc", "THOMSON" },
	{ 0x000E81, "Devicescape Software, Inc.", "DEVICESCAPE-SOFTWARE" },
	{ 0x000E82, "Commtech Wireless", "COMMTECH-WIRELESS" },
	{ 0x000E83, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000E84, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000E85, "Catalyst Enterprises, Inc.", "CATALYST-ENTERPRISES" },
	{ 0x000E86, "Alcatel North America", "ALCATEL-NORTH-AMERICA" },
	{ 0x000E87, "adp Gauselmann GmbH", "ADP-GAUSELMANN" },
	{ 0x000E88, "VIDEOTRON CORP.", "VIDEOTRON" },
	{ 0x000E89, "CLEMATIC", "CLEMATIC" },
	{ 0x000E8A, "Avara Technologies Pty. Ltd.", "AVARA-TECHNOLOGIES-PTY" },
	{ 0x000E8B, "Astarte Technology Co, Ltd.", "ASTARTE" },
	{ 0x000E8C, "Siemens AG A&D ET", "SIEMENS-A-D-ET" },
	{ 0x000E8D, "Systems in Progress Holding GmbH", "IN-PROGRESS-HOLDING" },
	{ 0x000E8E, "SparkLAN Communications, Inc.", "SPARKLAN-COMMUNICATION" },
	{ 0x000E8F, "Sercomm Corp.", "SERCOMM" },
	{ 0x000E90, "PONICO CORP.", "PONICO" },
	{ 0x000E91, "Navico Auckland Ltd", "NAVICO-AUCKLAND" },
	{ 0x000E92, "Open Telecom", "OPEN-TELECOM" },
	{ 0x000E93, "Milénio 3 Sistemas Electrónicos, Lda.", "MILéNIO-3-SISTEMAS-ELECTRóNICOS-LDA" },
	{ 0x000E94, "Maas International BV", "MAAS" },
	{ 0x000E95, "Fujiya Denki Seisakusho Co.,Ltd.", "FUJIYA-DENKI-SEISAKUSHO" },
	{ 0x000E96, "Cubic Defense Applications, Inc.", "CUBIC-DEFENSE-APPLICATIONS" },
	{ 0x000E97, "Ultracker Technology CO., Inc", "ULTRACKER" },
	{ 0x000E98, "HME Clear-Com LTD.", "HME-CLEAR-COM" },
	{ 0x000E99, "Spectrum Digital, Inc", "SPECTRUM-DIGITAL" },
	{ 0x000E9A, "BOE TECHNOLOGY GROUP CO.,LTD", "BOE-GROUP" },
	{ 0x000E9B, "Ambit Microsystems Corporation", "AMBIT-MICROSYSTEMS" },
	{ 0x000E9C, "Benchmark Electronics", "BENCHMARK-ELECTRONICS" },
	{ 0x000E9D, "Tiscali UK Ltd", "TISCALI-UK" },
	{ 0x000E9E, "Topfield Co., Ltd", "TOPFIELD" },
	{ 0x000E9F, "TEMIC SDS GmbH", "TEMIC-SDS" },
	{ 0x000EA0, "NetKlass Technology Inc.", "NETKLASS" },
	{ 0x000EA1, "Formosa Teletek Corporation", "FORMOSA-TELETEK" },
	{ 0x000EA2, "McAfee, Inc", "MCAFEE" },
	{ 0x000EA3, "CNCR-IT CO.,LTD,HangZhou P.R.CHINA", "CNCR-IT-HANGZHOU-P-R-CHINA" },
	{ 0x000EA4, "Certance Inc.", "CERTANCE" },
	{ 0x000EA5, "BLIP Systems", "BLIP" },
	{ 0x000EA6, "ASUSTEK COMPUTER INC.", "ASUSTEK" },
	{ 0x000EA7, "Endace Technology", "ENDACE" },
	{ 0x000EA8, "United Technologists Europe Limited", "UNITED-TECHNOLOGISTS-EUROPE" },
	{ 0x000EA9, "Shanghai Xun Shi Communications Equipment Ltd. Co.", "SHANGHAI-XUN-SHI-COMMUNICATION-EQUIPMENT" },
	{ 0x000EAA, "Scalent Systems, Inc.", "SCALENT" },
	{ 0x000EAB, "Cray Inc", "CRAY" },
	{ 0x000EAC, "MINTRON ENTERPRISE CO., LTD.", "MINTRON" },
	{ 0x000EAD, "Metanoia Technologies, Inc.", "METANOIA-TECHNOLOGIES" },
	{ 0x000EAE, "GAWELL TECHNOLOGIES CORP.", "GAWELL-TECHNOLOGIES" },
	{ 0x000EAF, "CASTEL", "CASTEL" },
	{ 0x000EB0, "Solutions Radio BV", "SOLUTIONS-RADIO" },
	{ 0x000EB1, "Newcotech,Ltd", "NEWCOTECH" },
	{ 0x000EB2, "Micro-Research Finland Oy", "MICRO-RESEARCH-FINLAND-OY" },
	{ 0x000EB3, "Hewlett-Packard", "HEWLETT-PACKARD" },
	{ 0x000EB4, "GUANGZHOU GAOKE COMMUNICATIONS TECHNOLOGY CO.LTD.", "GUANGZHOU-GAOKE-COMMUNICATION" },
	{ 0x000EB5, "Ecastle Electronics Co., Ltd.", "ECASTLE-ELECTRONICS" },
	{ 0x000EB6, "Riverbed Technology, Inc.", "RIVERBED" },
	{ 0x000EB7, "Knovative, Inc.", "KNOVATIVE" },
	{ 0x000EB8, "Iiga co.,Ltd", "IIGA" },
	{ 0x000EB9, "HASHIMOTO Electronics Industry Co.,Ltd.", "HASHIMOTO-ELECTRONICS-INDUSTRY" },
	{ 0x000EBA, "HANMI SEMICONDUCTOR CO., LTD.", "HANMI-SEMICONDUCTOR" },
	{ 0x000EBB, "Everbee Networks", "EVERBEE-NETWORKS" },
	{ 0x000EBC, "Paragon Fidelity GmbH", "PARAGON-FIDELITY" },
	{ 0x000EBD, "Burdick, a Quinton Compny", "BURDICK-A-QUINTON-COMPNY" },
	{ 0x000EBE, "B&B Electronics Manufacturing Co.", "B-B-ELECTRONICS-MANUFACTURING" },
	{ 0x000EBF, "Remsdaq Limited", "REMSDAQ" },
	{ 0x000EC0, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000EC1, "MYNAH Technologies", "MYNAH-TECHNOLOGIES" },
	{ 0x000EC2, "Lowrance Electronics, Inc.", "LOWRANCE-ELECTRONICS" },
	{ 0x000EC3, "Logic Controls, Inc.", "LOGIC-CONTROLS" },
	{ 0x000EC4, "Iskra Transmission d.d.", "ISKRA-TRANSMISSION-D-D" },
	{ 0x000EC5, "Digital Multitools Inc", "DIGITAL-MULTITOOLS" },
	{ 0x000EC6, "ASIX ELECTRONICS CORP.", "ASIX-ELECTRONICS" },
	{ 0x000EC7, "Motorola Korea", "MOTOROLA-KOREA" },
	{ 0x000EC8, "Zoran Corporation", "ZORAN" },
	{ 0x000EC9, "YOKO Technology Corp.", "YOKO" },
	{ 0x000ECA, "WTSS Inc", "WTSS" },
	{ 0x000ECB, "VineSys Technology", "VINESYS" },
	{ 0x000ECC, "Tableau, LLC", "TABLEAU-LLC" },
	{ 0x000ECD, "SKOV A/S", "SKOV-A/S" },
	{ 0x000ECE, "S.I.T.T.I. S.p.A.", "S-I-T-T-I-S-P-A" },
	{ 0x000ECF, "PROFIBUS Nutzerorganisation e.V.", "PROFIBUS-NUTZERORGANISATION-E-V" },
	{ 0x000ED0, "Privaris, Inc.", "PRIVARIS" },
	{ 0x000ED1, "Osaka Micro Computer.", "OSAKA-MICRO" },
	{ 0x000ED2, "Filtronic plc", "FILTRONIC-PLC" },
	{ 0x000ED3, "Epicenter, Inc.", "EPICENTER" },
	{ 0x000ED4, "CRESITT INDUSTRIE", "CRESITT-INDUSTRIE" },
	{ 0x000ED5, "COPAN Systems Inc.", "COPAN" },
	{ 0x000ED6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000ED7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000ED8, "Aktino, Inc.", "AKTINO" },
	{ 0x000ED9, "Aksys, Ltd.", "AKSYS" },
	{ 0x000EDA, "C-TECH UNITED CORP.", "C-TECH-UNITED" },
	{ 0x000EDB, "XiNCOM Corp.", "XINCOM" },
	{ 0x000EDC, "Tellion INC.", "TELLION" },
	{ 0x000EDD, "SHURE INCORPORATED", "SHURE-INCORPORATED" },
	{ 0x000EDE, "REMEC, Inc.", "REMEC" },
	{ 0x000EDF, "PLX Technology", "PLX" },
	{ 0x000EE0, "Mcharge", "MCHARGE" },
	{ 0x000EE1, "ExtremeSpeed Inc.", "EXTREMESPEED" },
	{ 0x000EE2, "Custom Engineering", "CUSTOM-ENGINEERING" },
	{ 0x000EE3, "Chiyu Technology Co.,Ltd", "CHIYU" },
	{ 0x000EE4, "BOE TECHNOLOGY GROUP CO.,LTD", "BOE-GROUP" },
	{ 0x000EE5, "bitWallet, Inc.", "BITWALLET" },
	{ 0x000EE6, "Adimos Systems LTD", "ADIMOS" },
	{ 0x000EE7, "AAC ELECTRONICS CORP.", "AAC-ELECTRONICS" },
	{ 0x000EE8, "zioncom", "ZIONCOM" },
	{ 0x000EE9, "WayTech Development, Inc.", "WAYTECH-DEVELOPMENT" },
	{ 0x000EEA, "Shadong Luneng Jicheng Electronics,Co.,Ltd", "SHADONG-LUNENG-JICHENG-ELECTRONICS" },
	{ 0x000EEB, "Sandmartin(zhong shan)Electronics Co.,Ltd", "SANDMARTIN-ZHONG-SHAN-ELECTRONICS" },
	{ 0x000EEC, "Orban", "ORBAN" },
	{ 0x000EED, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x000EEE, "Muco Industrie BV", "MUCO-INDUSTRIE" },
	{ 0x000EEF, "PRIVATE", "PRIVATE" },
	{ 0x000EF0, "Festo AG & Co. KG", "FESTO" },
	{ 0x000EF1, "EZQUEST INC.", "EZQUEST" },
	{ 0x000EF2, "Infinico Corporation", "INFINICO" },
	{ 0x000EF3, "Smarthome", "SMARTHOME" },
	{ 0x000EF4, "Kasda Networks Inc", "KASDA-NETWORKS" },
	{ 0x000EF5, "iPAC Technology Co., Ltd.", "IPAC" },
	{ 0x000EF6, "E-TEN Information Systems Co., Ltd.", "E-TEN-INFORMATION" },
	{ 0x000EF7, "Vulcan Portals Inc", "VULCAN-PORTALS" },
	{ 0x000EF8, "SBC ASI", "SBC-ASI" },
	{ 0x000EF9, "REA Elektronik GmbH", "REA-ELEKTRONIK" },
	{ 0x000EFA, "Optoway Technology Incorporation", "OPTOWAY-INCORPORATION" },
	{ 0x000EFB, "Macey Enterprises", "MACEY-ENTERPRISES" },
	{ 0x000EFC, "JTAG Technologies B.V.", "JTAG-TECHNOLOGIES" },
	{ 0x000EFD, "FUJINON CORPORATION", "FUJINON" },
	{ 0x000EFE, "EndRun Technologies LLC", "ENDRUN-TECHNOLOGIES-LLC" },
	{ 0x000EFF, "Megasolution,Inc.", "MEGASOLUTION" },
	{ 0x000F00, "Legra Systems, Inc.", "LEGRA" },
	{ 0x000F01, "DIGITALKS INC", "DIGITALKS" },
	{ 0x000F02, "Digicube Technology Co., Ltd", "DIGICUBE" },
	{ 0x000F03, "COM&C CO., LTD", "COM-C" },
	{ 0x000F04, "cim-usa inc", "CIM-USA" },
	{ 0x000F05, "3B SYSTEM INC.", "3B-SYSTEM" },
	{ 0x000F06, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000F07, "Mangrove Systems, Inc.", "MANGROVE" },
	{ 0x000F08, "Indagon Oy", "INDAGON-OY" },
	{ 0x000F09, "PRIVATE", "PRIVATE" },
	{ 0x000F0A, "Clear Edge Networks", "CLEAR-EDGE-NETWORKS" },
	{ 0x000F0B, "Kentima Technologies AB", "KENTIMA-TECHNOLOGIES-AB" },
	{ 0x000F0C, "SYNCHRONIC ENGINEERING", "SYNCHRONIC-ENGINEERING" },
	{ 0x000F0D, "Hunt Electronic Co., Ltd.", "HUNT" },
	{ 0x000F0E, "WaveSplitter Technologies, Inc.", "WAVESPLITTER-TECHNOLOGIES" },
	{ 0x000F0F, "Real ID Technology Co., Ltd.", "REAL-ID" },
	{ 0x000F10, "RDM Corporation", "RDM" },
	{ 0x000F11, "Prodrive B.V.", "PRODRIVE" },
	{ 0x000F12, "Panasonic Europe Ltd.", "PANASONIC-EUROPE" },
	{ 0x000F13, "Nisca corporation", "NISCA" },
	{ 0x000F14, "Mindray Co., Ltd.", "MINDRAY" },
	{ 0x000F15, "Kjaerulff1 A/S", "KJAERULFF1-A/S" },
	{ 0x000F16, "JAY HOW TECHNOLOGY CO.,", "JAY-HOW" },
	{ 0x000F17, "Insta Elektro GmbH", "INSTA-ELEKTRO" },
	{ 0x000F18, "Industrial Control Systems", "INDUSTRIAL-CONTROL" },
	{ 0x000F19, "Boston Scientific", "BOSTON-SCIENTIFIC" },
	{ 0x000F1A, "Gaming Support B.V.", "GAMING-SUPPORT" },
	{ 0x000F1B, "Ego Systems Inc.", "EGO" },
	{ 0x000F1C, "DigitAll World Co., Ltd", "DIGITALL-WORLD" },
	{ 0x000F1D, "Cosmo Techs Co., Ltd.", "COSMO-TECHS" },
	{ 0x000F1E, "Chengdu KT Electric Co.of High & New Technology", "CHENGDU-KT-ELECTRIC-OF-HIGH-NEW" },
	{ 0x000F1F, "Dell Inc", "DELL" },
	{ 0x000F20, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x000F21, "Scientific Atlanta, Inc", "SCIENTIFIC-ATLANTA" },
	{ 0x000F22, "Helius, Inc.", "HELIUS" },
	{ 0x000F23, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000F24, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000F25, "AimValley B.V.", "AIMVALLEY" },
	{ 0x000F26, "WorldAccxx LLC", "WORLDACCXX-LLC" },
	{ 0x000F27, "TEAL Electronics, Inc.", "TEAL-ELECTRONICS" },
	{ 0x000F28, "Itronix Corporation", "ITRONIX" },
	{ 0x000F29, "Augmentix Corporation", "AUGMENTIX" },
	{ 0x000F2A, "Cableware Electronics", "CABLEWARE-ELECTRONICS" },
	{ 0x000F2B, "GREENBELL SYSTEMS", "GREENBELL" },
	{ 0x000F2C, "Uplogix, Inc.", "UPLOGIX" },
	{ 0x000F2D, "CHUNG-HSIN ELECTRIC & MACHINERY MFG.CORP.", "CHUNG-HSIN-ELECTRIC-MACHINERY-MFG" },
	{ 0x000F2E, "Megapower International Corp.", "MEGAPOWER" },
	{ 0x000F2F, "W-LINX TECHNOLOGY CO., LTD.", "W-LINX" },
	{ 0x000F30, "Raza Microelectronics Inc", "RAZA-MICROELECTRONICS" },
	{ 0x000F31, "Allied Vision Technologies Canada Inc", "ALLIED-VISION-TECHNOLOGIES-CANADA" },
	{ 0x000F32, "Lootom Telcovideo Network Wuxi Co Ltd", "LOOTOM-TELCOVIDEO-NETWORK-WUXI" },
	{ 0x000F33, "DUALi Inc.", "DUALI" },
	{ 0x000F34, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000F35, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000F36, "Accurate Techhnologies, Inc.", "ACCURATE-TECHHNOLOGIES" },
	{ 0x000F37, "Xambala Incorporated", "XAMBALA-INCORPORATED" },
	{ 0x000F38, "Netstar", "NETSTAR" },
	{ 0x000F39, "IRIS SENSORS", "IRIS-SENSORS" },
	{ 0x000F3A, "HISHARP", "HISHARP" },
	{ 0x000F3B, "Fuji System Machines Co., Ltd.", "FUJI-SYSTEM-MACHINES" },
	{ 0x000F3C, "Endeleo Limited", "ENDELEO" },
	{ 0x000F3D, "D-Link Corporation", "D-LINK" },
	{ 0x000F3E, "CardioNet, Inc", "CARDIONET" },
	{ 0x000F3F, "Big Bear Networks", "BIG-BEAR-NETWORKS" },
	{ 0x000F40, "Optical Internetworking Forum", "OPTICAL-INTERNETWORKING-FORUM" },
	{ 0x000F41, "Zipher Ltd", "ZIPHER" },
	{ 0x000F42, "Xalyo Systems", "XALYO" },
	{ 0x000F43, "Wasabi Systems Inc.", "WASABI" },
	{ 0x000F44, "Tivella Inc.", "TIVELLA" },
	{ 0x000F45, "Stretch, Inc.", "STRETCH" },
	{ 0x000F46, "SINAR AG", "SINAR" },
	{ 0x000F47, "ROBOX SPA", "ROBOX-SPA" },
	{ 0x000F48, "Polypix Inc.", "POLYPIX" },
	{ 0x000F49, "Northover Solutions Limited", "NORTHOVER-SOLUTIONS" },
	{ 0x000F4A, "Kyushu-kyohan co.,ltd", "KYUSHU-KYOHAN" },
	{ 0x000F4B, "Oracle Corporation (possible Virtual Iron VM)", "ORACLE-VIRTUAL" },
	{ 0x000F4C, "Elextech INC", "ELEXTECH" },
	{ 0x000F4D, "TalkSwitch", "TALKSWITCH" },
	{ 0x000F4E, "Cellink", "CELLINK" },
	{ 0x000F4F, "Cadmus Technology Ltd", "CADMUS" },
	{ 0x000F50, "StreamScale Limited", "STREAMSCALE" },
	{ 0x000F51, "Azul Systems, Inc.", "AZUL" },
	{ 0x000F52, "YORK Refrigeration, Marine & Controls", "YORK-REFRIGERATION-MARINE-CONTROLS" },
	{ 0x000F53, "Solarflare Communications Inc", "SOLARFLARE-COMMUNICATION" },
	{ 0x000F54, "Entrelogic Corporation", "ENTRELOGIC" },
	{ 0x000F55, "Datawire Communication Networks Inc.", "DATAWIRE-COMMUNICATION-NETWORKS" },
	{ 0x000F56, "Continuum Photonics Inc", "CONTINUUM-PHOTONICS" },
	{ 0x000F57, "CABLELOGIC Co., Ltd.", "CABLELOGIC" },
	{ 0x000F58, "Adder Technology Limited", "ADDER" },
	{ 0x000F59, "Phonak Communications AG", "PHONAK-COMMUNICATION" },
	{ 0x000F5A, "Peribit Networks", "PERIBIT-NETWORKS" },
	{ 0x000F5B, "Delta Information Systems, Inc.", "DELTA-INFORMATION" },
	{ 0x000F5C, "Day One Digital Media Limited", "DAY-ONE-DIGITAL-MEDIA" },
	{ 0x000F5D, "Genexis BV", "GENEXIS" },
	{ 0x000F5E, "Veo", "VEO" },
	{ 0x000F5F, "Nicety Technologies Inc. (NTS)", "NICETY-TECHNOLOGIES-NTS" },
	{ 0x000F60, "Lifetron Co.,Ltd", "LIFETRON" },
	{ 0x000F61, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x000F62, "Alcatel Bell Space N.V.", "ALCATEL-BELL-SPACE-N-V" },
	{ 0x000F63, "Obzerv Technologies", "OBZERV-TECHNOLOGIES" },
	{ 0x000F64, "D&R Electronica Weesp BV", "D-R-ELECTRONICA-WEESP" },
	{ 0x000F65, "icube Corp.", "ICUBE" },
	{ 0x000F66, "Cisco-Linksys", "CISCO-LINKSYS" },
	{ 0x000F67, "West Instruments", "WEST-INSTRUMENTS" },
	{ 0x000F68, "Vavic Network Technology, Inc.", "VAVIC-NETWORK" },
	{ 0x000F69, "SEW Eurodrive GmbH & Co. KG", "SEW-EURODRIVE" },
	{ 0x000F6A, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000F6B, "GateWare Communications GmbH", "GATEWARE-COMMUNICATION" },
	{ 0x000F6C, "ADDI-DATA GmbH", "ADDI-DATA" },
	{ 0x000F6D, "Midas Engineering", "MIDAS-ENGINEERING" },
	{ 0x000F6E, "BBox", "BBOX" },
	{ 0x000F6F, "FTA Communication Technologies", "FTA-COMMUNICATION-TECHNOLOGIES" },
	{ 0x000F70, "Wintec Industries, inc.", "WINTEC-INDUSTRIES" },
	{ 0x000F71, "Sanmei Electronics Co.,Ltd", "SANMEI-ELECTRONICS" },
	{ 0x000F72, "Sandburst", "SANDBURST" },
	{ 0x000F73, "RS Automation Co., Ltd", "RS-AUTOMATION" },
	{ 0x000F74, "Qamcom Technology AB", "QAMCOM-AB" },
	{ 0x000F75, "First Silicon Solutions", "FIRST-SILICON-SOLUTIONS" },
	{ 0x000F76, "Digital Keystone, Inc.", "DIGITAL-KEYSTONE" },
	{ 0x000F77, "DENTUM CO.,LTD", "DENTUM" },
	{ 0x000F78, "Datacap Systems Inc", "DATACAP" },
	{ 0x000F79, "Bluetooth Interest Group Inc.", "BLUETOOTH-INTEREST-GROUP" },
	{ 0x000F7A, "BeiJing NuQX Technology CO.,LTD", "BEIJING-NUQX" },
	{ 0x000F7B, "Arce Sistemas, S.A.", "ARCE-SISTEMAS" },
	{ 0x000F7C, "ACTi Corporation", "ACTI" },
	{ 0x000F7D, "Xirrus", "XIRRUS" },
	{ 0x000F7E, "Ablerex Electronics Co., LTD", "ABLEREX-ELECTRONICS" },
	{ 0x000F7F, "UBSTORAGE Co.,Ltd.", "UBSTORAGE" },
	{ 0x000F80, "Trinity Security Systems,Inc.", "TRINITY-SECURITY" },
	{ 0x000F81, "PAL Pacific Inc.", "PAL-PACIFIC" },
	{ 0x000F82, "Mortara Instrument, Inc.", "MORTARA-INSTRUMENT" },
	{ 0x000F83, "Brainium Technologies Inc.", "BRAINIUM-TECHNOLOGIES" },
	{ 0x000F84, "Astute Networks, Inc.", "ASTUTE-NETWORKS" },
	{ 0x000F85, "ADDO-Japan Corporation", "ADDO-JAPAN" },
	{ 0x000F86, "Research In Motion Limited", "RESEARCH-IN-MOTION" },
	{ 0x000F87, "Maxcess International", "MAXCESS" },
	{ 0x000F88, "AMETEK, Inc.", "AMETEK" },
	{ 0x000F89, "Winnertec System Co., Ltd.", "WINNERTEC-SYSTEM" },
	{ 0x000F8A, "WideView", "WIDEVIEW" },
	{ 0x000F8B, "Orion MultiSystems Inc", "ORION-MULTISYSTEMS" },
	{ 0x000F8C, "Gigawavetech Pte Ltd", "GIGAWAVETECH-PTE" },
	{ 0x000F8D, "FAST TV-Server AG", "FAST-TV-SERVER" },
	{ 0x000F8E, "DONGYANG TELECOM CO.,LTD.", "DONGYANG-TELECOM" },
	{ 0x000F8F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000F90, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000F91, "Aerotelecom Co.,Ltd.", "AEROTELECOM" },
	{ 0x000F92, "Microhard Systems Inc.", "MICROHARD" },
	{ 0x000F93, "Landis+Gyr Ltd.", "LANDIS+GYR" },
	{ 0x000F94, "Genexis BV", "GENEXIS" },
	{ 0x000F95, "ELECOM Co.,LTD Laneed Division", "ELECOM-LANEED-DIVISION" },
	{ 0x000F96, "Telco Systems, Inc.", "TELCO" },
	{ 0x000F97, "Avanex Corporation", "AVANEX" },
	{ 0x000F98, "Avamax Co. Ltd.", "AVAMAX" },
	{ 0x000F99, "APAC opto Electronics Inc.", "APAC-OPTO-ELECTRONICS" },
	{ 0x000F9A, "Synchrony, Inc.", "SYNCHRONY" },
	{ 0x000F9B, "Ross Video Limited", "ROSS-VIDEO" },
	{ 0x000F9C, "Panduit Corp", "PANDUIT" },
	{ 0x000F9D, "DisplayLink (UK) Ltd", "DISPLAYLINK-UK" },
	{ 0x000F9E, "Murrelektronik GmbH", "MURRELEKTRONIK" },
	{ 0x000F9F, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x000FA0, "CANON KOREA BUSINESS SOLUTIONS INC.", "CANON-KOREA-BUSINESS-SOLUTIONS" },
	{ 0x000FA1, "Gigabit Systems Inc.", "GIGABIT" },
	{ 0x000FA2, "2xWireless", "2XWIRELESS" },
	{ 0x000FA3, "Alpha Networks Inc.", "ALPHA-NETWORKS" },
	{ 0x000FA4, "Sprecher Automation GmbH", "SPRECHER-AUTOMATION" },
	{ 0x000FA5, "BWA Technology GmbH", "BWA" },
	{ 0x000FA6, "S2 Security Corporation", "S2-SECURITY" },
	{ 0x000FA7, "Raptor Networks Technology", "RAPTOR-NETWORKS" },
	{ 0x000FA8, "Photometrics, Inc.", "PHOTOMETRICS" },
	{ 0x000FA9, "PC Fabrik", "PC-FABRIK" },
	{ 0x000FAA, "Nexus Technologies", "NEXUS-TECHNOLOGIES" },
	{ 0x000FAB, "Kyushu Electronics Systems Inc.", "KYUSHU-ELECTRONICS" },
	{ 0x000FAC, "IEEE 802.11", "IEEE-802-11" },
	{ 0x000FAD, "FMN communications GmbH", "FMN-COMMUNICATION" },
	{ 0x000FAE, "E2O Communications", "E2O-COMMUNICATION" },
	{ 0x000FAF, "Dialog Inc.", "DIALOG" },
	{ 0x000FB0, "Compal Electronics,INC.", "COMPAL-ELECTRONICS" },
	{ 0x000FB1, "Cognio Inc.", "COGNIO" },
	{ 0x000FB2, "Broadband Pacenet (India) Pvt. Ltd.", "BROADBAND-PACENET-INDIA-PVT" },
	{ 0x000FB3, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x000FB4, "Timespace Technology", "TIMESPACE" },
	{ 0x000FB5, "NETGEAR Inc", "NETGEAR" },
	{ 0x000FB6, "Europlex Technologies", "EUROPLEX-TECHNOLOGIES" },
	{ 0x000FB7, "Cavium Networks", "CAVIUM-NETWORKS" },
	{ 0x000FB8, "CallURL Inc.", "CALLURL" },
	{ 0x000FB9, "Adaptive Instruments", "ADAPTIVE-INSTRUMENTS" },
	{ 0x000FBA, "Tevebox AB", "TEVEBOX-AB" },
	{ 0x000FBB, "Nokia Siemens Networks GmbH & Co. KG.", "NOKIA-SIEMENS-NETWORKS" },
	{ 0x000FBC, "Onkey Technologies, Inc.", "ONKEY-TECHNOLOGIES" },
	{ 0x000FBD, "MRV Communications (Networks) LTD", "MRV-COMMUNICATION-NETWORKS" },
	{ 0x000FBE, "e-w/you Inc.", "E-W/YOU" },
	{ 0x000FBF, "DGT Sp. z o.o.", "DGT-SP-Z-O-O" },
	{ 0x000FC0, "DELCOMp", "DELCOMP" },
	{ 0x000FC1, "WAVE Corporation", "WAVE" },
	{ 0x000FC2, "Uniwell Corporation", "UNIWELL" },
	{ 0x000FC3, "PalmPalm Technology, Inc.", "PALMPALM" },
	{ 0x000FC4, "NST co.,LTD.", "NST" },
	{ 0x000FC5, "KeyMed Ltd", "KEYMED" },
	{ 0x000FC6, "Eurocom Industries A/S", "EUROCOM-INDUSTRIES-A/S" },
	{ 0x000FC7, "Dionica R&D Ltd.", "DIONICA-R-D" },
	{ 0x000FC8, "Chantry Networks", "CHANTRY-NETWORKS" },
	{ 0x000FC9, "Allnet GmbH", "ALLNET" },
	{ 0x000FCA, "A-JIN TECHLINE CO, LTD", "A-JIN-TECHLINE" },
	{ 0x000FCB, "3Com Ltd", "3COM" },
	{ 0x000FCC, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x000FCD, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x000FCE, "Kikusui Electronics Corp.", "KIKUSUI-ELECTRONICS" },
	{ 0x000FCF, "Datawind Research", "DATAWIND-RESEARCH" },
	{ 0x000FD0, "ASTRI", "ASTRI" },
	{ 0x000FD1, "Applied Wireless Identifications Group, Inc.", "APPLIED-WIRELESS-IDENTIFICATIONS-GROUP" },
	{ 0x000FD2, "EWA Technologies, Inc.", "EWA-TECHNOLOGIES" },
	{ 0x000FD3, "Digium", "DIGIUM" },
	{ 0x000FD4, "Soundcraft", "SOUNDCRAFT" },
	{ 0x000FD5, "Schwechat - RISE", "SCHWECHAT---RISE" },
	{ 0x000FD6, "Sarotech Co., Ltd", "SAROTECH" },
	{ 0x000FD7, "Harman Music Group", "HARMAN-MUSIC-GROUP" },
	{ 0x000FD8, "Force, Inc.", "FORCE" },
	{ 0x000FD9, "FlexDSL Telecommunications AG", "FLEXDSL-TELECOMMUNICATIONS" },
	{ 0x000FDA, "YAZAKI CORPORATION", "YAZAKI" },
	{ 0x000FDB, "Westell Technologies", "WESTELL-TECHNOLOGIES" },
	{ 0x000FDC, "Ueda Japan Radio Co., Ltd.", "UEDA-JAPAN-RADIO" },
	{ 0x000FDD, "SORDIN AB", "SORDIN-AB" },
	{ 0x000FDE, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x000FDF, "SOLOMON Technology Corp.", "SOLOMON" },
	{ 0x000FE0, "NComputing Co.,Ltd.", "NCOMPUTING" },
	{ 0x000FE1, "ID DIGITAL CORPORATION", "ID-DIGITAL" },
	{ 0x000FE2, "Hangzhou H3C Technologies Co., Ltd.", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x000FE3, "Damm Cellular Systems A/S", "DAMM-CELLULAR-A/S" },
	{ 0x000FE4, "Pantech Co.,Ltd", "PANTECH" },
	{ 0x000FE5, "MERCURY SECURITY CORPORATION", "MERCURY-SECURITY" },
	{ 0x000FE6, "MBTech Systems, Inc.", "MBTECH" },
	{ 0x000FE7, "Lutron Electronics Co., Inc.", "LUTRON-ELECTRONICS" },
	{ 0x000FE8, "Lobos, Inc.", "LOBOS" },
	{ 0x000FE9, "GW TECHNOLOGIES CO.,LTD.", "GW-TECHNOLOGIES" },
	{ 0x000FEA, "Giga-Byte Technology Co.,LTD.", "GIGA-BYTE" },
	{ 0x000FEB, "Cylon Controls", "CYLON-CONTROLS" },
	{ 0x000FEC, "ARKUS Inc.", "ARKUS" },
	{ 0x000FED, "Anam Electronics Co., Ltd", "ANAM-ELECTRONICS" },
	{ 0x000FEE, "XTec, Incorporated", "XTEC-INCORPORATED" },
	{ 0x000FEF, "Thales e-Transactions GmbH", "THALES-E-TRANSACTIONS" },
	{ 0x000FF0, "Sunray Co. Ltd.", "SUNRAY" },
	{ 0x000FF1, "nex-G Systems Pte.Ltd", "NEX-G-PTE" },
	{ 0x000FF2, "Loud Technologies Inc.", "LOUD-TECHNOLOGIES" },
	{ 0x000FF3, "Jung Myoung Communications&Technology", "JUNG-MYOUNG-COMMUNICATION" },
	{ 0x000FF4, "Guntermann & Drunck GmbH", "GUNTERMANN-DRUNCK" },
	{ 0x000FF5, "GN&S company", "GN-S-COMPANY" },
	{ 0x000FF6, "Darfon Electronics Corp.", "DARFON-ELECTRONICS" },
	{ 0x000FF7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000FF8, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x000FF9, "Valcretec, Inc.", "VALCRETEC" },
	{ 0x000FFA, "Optinel Systems, Inc.", "OPTINEL" },
	{ 0x000FFB, "Nippon Denso Industry Co., Ltd.", "NIPPON-DENSO-INDUSTRY" },
	{ 0x000FFC, "Merit Li-Lin Ent.", "MERIT-LI-LIN-ENT" },
	{ 0x000FFD, "Glorytek Network Inc.", "GLORYTEK-NETWORK" },
	{ 0x000FFE, "G-PRO COMPUTER", "G-PRO" },
	{ 0x000FFF, "Control4", "CONTROL4" },
	{ 0x001000, "CABLE TELEVISION LABORATORIES, INC.", "CABLE-TELEVISION-LABORATORIES" },
	{ 0x001001, "Citel", "CITEL" },
	{ 0x001002, "ACTIA", "ACTIA" },
	{ 0x001003, "IMATRON, INC.", "IMATRON" },
	{ 0x001004, "THE BRANTLEY COILE COMPANY,INC", "THE-BRANTLEY-COILE-COMPANY" },
	{ 0x001005, "UEC COMMERCIAL", "UEC-COMMERCIAL" },
	{ 0x001006, "Thales Contact Solutions Ltd.", "THALES-CONTACT-SOLUTIONS" },
	{ 0x001007, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001008, "VIENNA SYSTEMS CORPORATION", "VIENNA" },
	{ 0x001009, "HORO QUARTZ", "HORO-QUARTZ" },
	{ 0x00100A, "WILLIAMS COMMUNICATIONS GROUP", "WILLIAMS-COMMUNICATION-GROUP" },
	{ 0x00100B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00100C, "ITO CO., LTD.", "ITO" },
	{ 0x00100D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00100E, "MICRO LINEAR COPORATION", "MICRO-LINEAR-COPORATION" },
	{ 0x00100F, "INDUSTRIAL CPU SYSTEMS", "INDUSTRIAL-CPU" },
	{ 0x001010, "INITIO CORPORATION", "INITIO" },
	{ 0x001011, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001012, "PROCESSOR SYSTEMS (I) PVT LTD", "PROCESSOR-I-PVT" },
	{ 0x001013, "Kontron America, Inc.", "KONTRON-AMERICA" },
	{ 0x001014, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001015, "OOmon Inc.", "OOMON" },
	{ 0x001016, "T.SQWARE", "T-SQWARE" },
	{ 0x001017, "Bosch Access Systems GmbH", "BOSCH-ACCESS" },
	{ 0x001018, "BROADCOM CORPORATION", "BROADCOM" },
	{ 0x001019, "SIRONA DENTAL SYSTEMS GmbH & Co. KG", "SIRONA-DENTAL" },
	{ 0x00101A, "PictureTel Corp.", "PICTURETEL" },
	{ 0x00101B, "CORNET TECHNOLOGY, INC.", "CORNET" },
	{ 0x00101C, "OHM TECHNOLOGIES INTL, LLC", "OHM-TECHNOLOGIES-INTL-LLC" },
	{ 0x00101D, "WINBOND ELECTRONICS CORP.", "WINBOND-ELECTRONICS" },
	{ 0x00101E, "MATSUSHITA ELECTRONIC INSTRUMENTS CORP.", "MATSUSHITA-INSTRUMENTS" },
	{ 0x00101F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001020, "Hand Held Products Inc", "HAND-HELD-PRODUCTS" },
	{ 0x001021, "ENCANTO NETWORKS, INC.", "ENCANTO-NETWORKS" },
	{ 0x001022, "SatCom Media Corporation", "SATCOM-MEDIA" },
	{ 0x001023, "Network Equipment Technologies", "NETWORK-EQUIPMENT-TECHNOLOGIES" },
	{ 0x001024, "NAGOYA ELECTRIC WORKS CO., LTD", "NAGOYA-ELECTRIC-WORKS" },
	{ 0x001025, "Grayhill, Inc", "GRAYHILL" },
	{ 0x001026, "ACCELERATED NETWORKS, INC.", "ACCELERATED-NETWORKS" },
	{ 0x001027, "L-3 COMMUNICATIONS EAST", "L-3-COMMUNICATION-EAST" },
	{ 0x001028, "COMPUTER TECHNICA, INC.", "TECHNICA" },
	{ 0x001029, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00102A, "ZF MICROSYSTEMS, INC.", "ZF-MICROSYSTEMS" },
	{ 0x00102B, "UMAX DATA SYSTEMS, INC.", "UMAX-DATA" },
	{ 0x00102C, "Lasat Networks A/S", "LASAT-NETWORKS-A/S" },
	{ 0x00102D, "HITACHI SOFTWARE ENGINEERING", "HITACHI-SOFTWARE-ENGINEERING" },
	{ 0x00102E, "NETWORK SYSTEMS & TECHNOLOGIES PVT. LTD.", "NETWORK-TECHNOLOGIES-PVT" },
	{ 0x00102F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001030, "EION Inc.", "EION" },
	{ 0x001031, "OBJECTIVE COMMUNICATIONS, INC.", "OBJECTIVE-COMMUNICATION" },
	{ 0x001032, "ALTA TECHNOLOGY", "ALTA" },
	{ 0x001033, "ACCESSLAN COMMUNICATIONS, INC.", "ACCESSLAN-COMMUNICATION" },
	{ 0x001034, "GNP Computers", "GNP-COMPUTERS" },
	{ 0x001035, "ELITEGROUP COMPUTER SYSTEMS CO., LTD", "ELITEGROUP" },
	{ 0x001036, "INTER-TEL INTEGRATED SYSTEMS", "INTER-TEL-INTEGRATED" },
	{ 0x001037, "CYQ've Technology Co., Ltd.", "CYQ-VE" },
	{ 0x001038, "MICRO RESEARCH INSTITUTE, INC.", "MICRO-RESEARCH-INSTITUTE" },
	{ 0x001039, "Vectron Systems AG", "VECTRON" },
	{ 0x00103A, "DIAMOND NETWORK TECH", "DIAMOND-NETWORK-TECH" },
	{ 0x00103B, "HIPPI NETWORKING FORUM", "HIPPI-NETWORKING-FORUM" },
	{ 0x00103C, "IC ENSEMBLE, INC.", "IC-ENSEMBLE" },
	{ 0x00103D, "PHASECOM, LTD.", "PHASECOM" },
	{ 0x00103E, "NETSCHOOLS CORPORATION", "NETSCHOOLS" },
	{ 0x00103F, "TOLLGRADE COMMUNICATIONS, INC.", "TOLLGRADE-COMMUNICATION" },
	{ 0x001040, "INTERMEC CORPORATION", "INTERMEC" },
	{ 0x001041, "BRISTOL BABCOCK, INC.", "BRISTOL-BABCOCK" },
	{ 0x001042, "Alacritech, Inc.", "ALACRITECH" },
	{ 0x001043, "A2 CORPORATION", "A2" },
	{ 0x001044, "InnoLabs Corporation", "INNOLABS" },
	{ 0x001045, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x001046, "ALCORN MCBRIDE INC.", "ALCORN-MCBRIDE" },
	{ 0x001047, "ECHO ELETRIC CO. LTD.", "ECHO-ELETRIC" },
	{ 0x001048, "HTRC AUTOMATION, INC.", "HTRC-AUTOMATION" },
	{ 0x001049, "ShoreTel, Inc", "SHORETEL" },
	{ 0x00104A, "The Parvus Corporation", "THE-PARVUS" },
	{ 0x00104B, "3COM CORPORATION", "3COM" },
	{ 0x00104C, "Teledyne LeCroy, Inc", "TELEDYNE-LECROY" },
	{ 0x00104D, "SURTEC INDUSTRIES, INC.", "SURTEC-INDUSTRIES" },
	{ 0x00104E, "CEOLOGIC", "CEOLOGIC" },
	{ 0x00104F, "Oracle Corporation", "ORACLE" },
	{ 0x001050, "RION CO., LTD.", "RION" },
	{ 0x001051, "CMICRO CORPORATION", "CMICRO" },
	{ 0x001052, "METTLER-TOLEDO (ALBSTADT) GMBH", "METTLER-TOLEDO-ALBSTADT" },
	{ 0x001053, "COMPUTER TECHNOLOGY CORP.", "" },
	{ 0x001054, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001055, "FUJITSU MICROELECTRONICS, INC.", "FUJITSU-MICROELECTRONICS" },
	{ 0x001056, "SODICK CO., LTD.", "SODICK" },
	{ 0x001057, "Rebel.com, Inc.", "REBEL-COM" },
	{ 0x001058, "ArrowPoint Communications", "ARROWPOINT-COMMUNICATION" },
	{ 0x001059, "DIABLO RESEARCH CO. LLC", "DIABLO-RESEARCH-LLC" },
	{ 0x00105A, "3COM CORPORATION", "3COM" },
	{ 0x00105B, "NET INSIGHT AB", "NET-INSIGHT-AB" },
	{ 0x00105C, "QUANTUM DESIGNS (H.K.) LTD.", "QUANTUM-DESIGNS-H-K" },
	{ 0x00105D, "Draeger Medical", "DRAEGER-MEDICAL" },
	{ 0x00105E, "Spirent plc, Service Assurance Broadband", "SPIRENT-PLC-SERVICE-ASSURANCE-BROADBAND" },
	{ 0x00105F, "ZODIAC DATA SYSTEMS", "ZODIAC-DATA" },
	{ 0x001060, "BILLIONTON SYSTEMS, INC.", "BILLIONTON" },
	{ 0x001061, "HOSTLINK CORP.", "HOSTLINK" },
	{ 0x001062, "NX SERVER, ILNC.", "NX-SERVER-ILNC" },
	{ 0x001063, "STARGUIDE DIGITAL NETWORKS", "STARGUIDE-DIGITAL-NETWORKS" },
	{ 0x001064, "DNPG, LLC", "DNPG-LLC" },
	{ 0x001065, "RADYNE CORPORATION", "RADYNE" },
	{ 0x001066, "ADVANCED CONTROL SYSTEMS, INC.", "ADVANCED-CONTROL" },
	{ 0x001067, "Ericsson", "ERICSSON" },
	{ 0x001068, "COMOS TELECOM", "COMOS-TELECOM" },
	{ 0x001069, "HELIOSS COMMUNICATIONS, INC.", "HELIOSS-COMMUNICATION" },
	{ 0x00106A, "DIGITAL MICROWAVE CORPORATION", "DIGITAL-MICROWAVE" },
	{ 0x00106B, "SONUS NETWORKS, INC.", "SONUS-NETWORKS" },
	{ 0x00106C, "EDNT GmbH", "EDNT" },
	{ 0x00106D, "Axxcelera Broadband Wireless", "AXXCELERA-BROADBAND-WIRELESS" },
	{ 0x00106E, "TADIRAN COM. LTD.", "TADIRAN-COM" },
	{ 0x00106F, "TRENTON TECHNOLOGY INC.", "TRENTON" },
	{ 0x001070, "CARADON TREND LTD.", "CARADON-TREND" },
	{ 0x001071, "ADVANET INC.", "ADVANET" },
	{ 0x001072, "GVN TECHNOLOGIES, INC.", "GVN-TECHNOLOGIES" },
	{ 0x001073, "Technobox, Inc.", "TECHNOBOX" },
	{ 0x001074, "ATEN INTERNATIONAL CO., LTD.", "ATEN" },
	{ 0x001075, "Segate Technology LLC", "SEGATE-LLC" },
	{ 0x001076, "EUREM GmbH", "EUREM" },
	{ 0x001077, "SAF DRIVE SYSTEMS, LTD.", "SAF-DRIVE" },
	{ 0x001078, "NUERA COMMUNICATIONS, INC.", "NUERA-COMMUNICATION" },
	{ 0x001079, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00107A, "AmbiCom, Inc.", "AMBICOM" },
	{ 0x00107B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00107C, "P-COM, INC.", "P-COM" },
	{ 0x00107D, "AURORA COMMUNICATIONS, LTD.", "AURORA-COMMUNICATION" },
	{ 0x00107E, "BACHMANN ELECTRONIC GmbH", "BACHMANN" },
	{ 0x00107F, "CRESTRON ELECTRONICS, INC.", "CRESTRON-ELECTRONICS" },
	{ 0x001080, "METAWAVE COMMUNICATIONS", "METAWAVE-COMMUNICATION" },
	{ 0x001081, "DPS, INC.", "DPS" },
	{ 0x001082, "JNA TELECOMMUNICATIONS LIMITED", "JNA-TELECOMMUNICATIONS" },
	{ 0x001083, "HEWLETT-PACKARD COMPANY", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001084, "K-BOT COMMUNICATIONS", "K-BOT-COMMUNICATION" },
	{ 0x001085, "POLARIS COMMUNICATIONS, INC.", "POLARIS-COMMUNICATION" },
	{ 0x001086, "ATTO Technology, Inc.", "ATTO" },
	{ 0x001087, "Xstreamis PLC", "XSTREAMIS-PLC" },
	{ 0x001088, "AMERICAN NETWORKS INC.", "AMERICAN-NETWORKS" },
	{ 0x001089, "WebSonic", "WEBSONIC" },
	{ 0x00108A, "TeraLogic, Inc.", "TERALOGIC" },
	{ 0x00108B, "LASERANIMATION SOLLINGER GmbH", "LASERANIMATION-SOLLINGER" },
	{ 0x00108C, "FUJITSU TELECOMMUNICATIONS EUROPE, LTD.", "FUJITSU-TELECOMMUNICATIONS-EUROPE" },
	{ 0x00108D, "Johnson Controls, Inc.", "JOHNSON-CONTROLS" },
	{ 0x00108E, "HUGH SYMONS CONCEPT Technologies Ltd.", "HUGH-SYMONS-CONCEPT-TECHNOLOGIES" },
	{ 0x00108F, "RAPTOR SYSTEMS", "RAPTOR" },
	{ 0x001090, "CIMETRICS, INC.", "CIMETRICS" },
	{ 0x001091, "NO WIRES NEEDED BV", "NO-WIRES-NEEDED" },
	{ 0x001092, "NETCORE INC.", "NETCORE" },
	{ 0x001093, "CMS COMPUTERS, LTD.", "CMS-COMPUTERS" },
	{ 0x001094, "Performance Analysis Broadband, Spirent plc", "PERFORMANCE-ANALYSIS-BROADBAND-SPIRENT-PLC" },
	{ 0x001095, "Thomson Inc.", "THOMSON" },
	{ 0x001096, "TRACEWELL SYSTEMS, INC.", "TRACEWELL" },
	{ 0x001097, "WinNet Metropolitan Communications Systems, Inc.", "WINNET-METROPOLITAN-COMMUNICATION" },
	{ 0x001098, "STARNET TECHNOLOGIES, INC.", "STARNET-TECHNOLOGIES" },
	{ 0x001099, "InnoMedia, Inc.", "INNOMEDIA" },
	{ 0x00109A, "NETLINE", "NETLINE" },
	{ 0x00109B, "Emulex Corporation", "EMULEX" },
	{ 0x00109C, "M-SYSTEM CO., LTD.", "M-SYSTEM" },
	{ 0x00109D, "CLARINET SYSTEMS, INC.", "CLARINET" },
	{ 0x00109E, "AWARE, INC.", "AWARE" },
	{ 0x00109F, "PAVO, INC.", "PAVO" },
	{ 0x0010A0, "INNOVEX TECHNOLOGIES, INC.", "INNOVEX-TECHNOLOGIES" },
	{ 0x0010A1, "KENDIN SEMICONDUCTOR, INC.", "KENDIN-SEMICONDUCTOR" },
	{ 0x0010A2, "TNS", "TNS" },
	{ 0x0010A3, "OMNITRONIX, INC.", "OMNITRONIX" },
	{ 0x0010A4, "XIRCOM", "XIRCOM" },
	{ 0x0010A5, "OXFORD INSTRUMENTS", "OXFORD-INSTRUMENTS" },
	{ 0x0010A6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0010A7, "UNEX TECHNOLOGY CORPORATION", "UNEX" },
	{ 0x0010A8, "RELIANCE COMPUTER CORP.", "RELIANCE" },
	{ 0x0010A9, "ADHOC TECHNOLOGIES", "ADHOC-TECHNOLOGIES" },
	{ 0x0010AA, "MEDIA4, INC.", "MEDIA4" },
	{ 0x0010AB, "KOITO ELECTRIC INDUSTRIES, LTD.", "KOITO-ELECTRIC-INDUSTRIES" },
	{ 0x0010AC, "IMCI TECHNOLOGIES", "IMCI-TECHNOLOGIES" },
	{ 0x0010AD, "SOFTRONICS USB, INC.", "SOFTRONICS-USB" },
	{ 0x0010AE, "SHINKO ELECTRIC INDUSTRIES CO.", "SHINKO-ELECTRIC-INDUSTRIES" },
	{ 0x0010AF, "TAC SYSTEMS, INC.", "TAC" },
	{ 0x0010B0, "MERIDIAN TECHNOLOGY CORP.", "MERIDIAN" },
	{ 0x0010B1, "FOR-A CO., LTD.", "FOR-A" },
	{ 0x0010B2, "COACTIVE AESTHETICS", "COACTIVE-AESTHETICS" },
	{ 0x0010B3, "NOKIA MULTIMEDIA TERMINALS", "NOKIA-MULTIMEDIA-TERMINALS" },
	{ 0x0010B4, "ATMOSPHERE NETWORKS", "ATMOSPHERE-NETWORKS" },
	{ 0x0010B5, "ACCTON TECHNOLOGY CORPORATION", "ACCTON" },
	{ 0x0010B6, "ENTRATA COMMUNICATIONS CORP.", "ENTRATA-COMMUNICATION" },
	{ 0x0010B7, "COYOTE TECHNOLOGIES, LLC", "COYOTE-TECHNOLOGIES-LLC" },
	{ 0x0010B8, "ISHIGAKI COMPUTER SYSTEM CO.", "ISHIGAKI-SYSTEM" },
	{ 0x0010B9, "MAXTOR CORP.", "MAXTOR" },
	{ 0x0010BA, "MARTINHO-DAVIS SYSTEMS, INC.", "MARTINHO-DAVIS" },
	{ 0x0010BB, "DATA & INFORMATION TECHNOLOGY", "DATA-INFORMATION" },
	{ 0x0010BC, "Aastra Telecom", "AASTRA-TELECOM" },
	{ 0x0010BD, "THE TELECOMMUNICATION TECHNOLOGY COMMITTEE (TTC)", "THE-TELECOMMUNICATION-COMMITTEE-TTC" },
	{ 0x0010BE, "MARCH NETWORKS CORPORATION", "MARCH-NETWORKS" },
	{ 0x0010BF, "InterAir Wireless", "INTERAIR-WIRELESS" },
	{ 0x0010C0, "ARMA, Inc.", "ARMA" },
	{ 0x0010C1, "OI ELECTRIC CO., LTD.", "OI-ELECTRIC" },
	{ 0x0010C2, "WILLNET, INC.", "WILLNET" },
	{ 0x0010C3, "CSI-CONTROL SYSTEMS", "CSI-CONTROL" },
	{ 0x0010C4, "MEDIA GLOBAL LINKS CO., LTD.", "MEDIA-GLOBAL-LINKS" },
	{ 0x0010C5, "PROTOCOL TECHNOLOGIES, INC.", "PROTOCOL-TECHNOLOGIES" },
	{ 0x0010C6, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x0010C7, "DATA TRANSMISSION NETWORK", "DATA-TRANSMISSION-NETWORK" },
	{ 0x0010C8, "COMMUNICATIONS ELECTRONICS SECURITY GROUP", "COMMUNICATION-ELECTRONICS-SECURITY-GROUP" },
	{ 0x0010C9, "MITSUBISHI ELECTRONICS LOGISTIC SUPPORT CO.", "MITSUBISHI-ELECTRONICS-LOGISTIC-SUPPORT" },
	{ 0x0010CA, "Telco Systems, Inc.", "TELCO" },
	{ 0x0010CB, "FACIT K.K.", "FACIT-K-K" },
	{ 0x0010CC, "CLP COMPUTER LOGISTIK PLANUNG GmbH", "CLP-LOGISTIK-PLANUNG" },
	{ 0x0010CD, "INTERFACE CONCEPT", "INTERFACE-CONCEPT" },
	{ 0x0010CE, "VOLAMP, LTD.", "VOLAMP" },
	{ 0x0010CF, "FIBERLANE COMMUNICATIONS", "FIBERLANE-COMMUNICATION" },
	{ 0x0010D0, "WITCOM, LTD.", "WITCOM" },
	{ 0x0010D1, "Top Layer Networks, Inc.", "TOP-LAYER-NETWORKS" },
	{ 0x0010D2, "NITTO TSUSHINKI CO., LTD", "NITTO-TSUSHINKI" },
	{ 0x0010D3, "GRIPS ELECTRONIC GMBH", "GRIPS" },
	{ 0x0010D4, "STORAGE COMPUTER CORPORATION", "STORAGE" },
	{ 0x0010D5, "IMASDE CANARIAS, S.A.", "IMASDE-CANARIAS" },
	{ 0x0010D6, "Exelis", "EXELIS" },
	{ 0x0010D7, "ARGOSY RESEARCH INC.", "ARGOSY-RESEARCH" },
	{ 0x0010D8, "CALISTA", "CALISTA" },
	{ 0x0010D9, "IBM JAPAN, FUJISAWA MT+D", "IBM-JAPAN-FUJISAWA-MT+D" },
	{ 0x0010DA, "Kollmorgen Corp", "KOLLMORGEN" },
	{ 0x0010DB, "Juniper Networks, Inc.", "JUNIPER-NETWORKS" },
	{ 0x0010DC, "MICRO-STAR INTERNATIONAL CO., LTD.", "MICRO-STAR" },
	{ 0x0010DD, "ENABLE SEMICONDUCTOR, INC.", "ENABLE-SEMICONDUCTOR" },
	{ 0x0010DE, "INTERNATIONAL DATACASTING CORPORATION", "DATACASTING" },
	{ 0x0010DF, "RISE COMPUTER INC.", "RISE" },
	{ 0x0010E0, "Oracle Corporation", "ORACLE" },
	{ 0x0010E1, "S.I. TECH, INC.", "S-I-TECH" },
	{ 0x0010E2, "ArrayComm, Inc.", "ARRAYCOMM" },
	{ 0x0010E3, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0010E4, "NSI CORPORATION", "NSI" },
	{ 0x0010E5, "SOLECTRON TEXAS", "SOLECTRON-TEXAS" },
	{ 0x0010E6, "APPLIED INTELLIGENT SYSTEMS, INC.", "APPLIED-INTELLIGENT" },
	{ 0x0010E7, "BreezeCom", "BREEZECOM" },
	{ 0x0010E8, "TELOCITY, INCORPORATED", "TELOCITY-INCORPORATED" },
	{ 0x0010E9, "RAIDTEC LTD.", "RAIDTEC" },
	{ 0x0010EA, "ADEPT TECHNOLOGY", "ADEPT" },
	{ 0x0010EB, "SELSIUS SYSTEMS, INC.", "SELSIUS" },
	{ 0x0010EC, "RPCG, LLC", "RPCG-LLC" },
	{ 0x0010ED, "SUNDANCE TECHNOLOGY, INC.", "SUNDANCE" },
	{ 0x0010EE, "CTI PRODUCTS, INC.", "CTI-PRODUCTS" },
	{ 0x0010EF, "DBTEL INCORPORATED", "DBTEL-INCORPORATED" },
	{ 0x0010F0, "RITTAL-WERK RUDOLF LOH GmbH & Co.", "RITTAL-WERK-RUDOLF-LOH" },
	{ 0x0010F1, "I-O CORPORATION", "I-O" },
	{ 0x0010F2, "ANTEC", "ANTEC" },
	{ 0x0010F3, "Nexcom International Co., Ltd.", "NEXCOM" },
	{ 0x0010F4, "Vertical Communications", "VERTICAL-COMMUNICATION" },
	{ 0x0010F5, "AMHERST SYSTEMS, INC.", "AMHERST" },
	{ 0x0010F6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0010F7, "IRIICHI TECHNOLOGIES Inc.", "IRIICHI-TECHNOLOGIES" },
	{ 0x0010F8, "TEXIO TECHNOLOGY CORPORATION", "TEXIO" },
	{ 0x0010F9, "UNIQUE SYSTEMS, INC.", "UNIQUE" },
	{ 0x0010FA, "Apple", "APPLE" },
	{ 0x0010FB, "ZIDA TECHNOLOGIES LIMITED", "ZIDA-TECHNOLOGIES" },
	{ 0x0010FC, "BROADBAND NETWORKS, INC.", "BROADBAND-NETWORKS" },
	{ 0x0010FD, "COCOM A/S", "COCOM-A/S" },
	{ 0x0010FE, "DIGITAL EQUIPMENT CORPORATION", "DIGITAL-EQUIPMENT" },
	{ 0x0010FF, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001100, "Schneider Electric", "SCHNEIDER-ELECTRIC" },
	{ 0x001101, "CET Technologies Pte Ltd", "CET-TECHNOLOGIES-PTE" },
	{ 0x001102, "Aurora Multimedia Corp.", "AURORA-MULTIMEDIA" },
	{ 0x001103, "kawamura electric inc.", "KAWAMURA-ELECTRIC" },
	{ 0x001104, "TELEXY", "TELEXY" },
	{ 0x001105, "Sunplus Technology Co., Ltd.", "SUNPLUS" },
	{ 0x001106, "Siemens NV (Belgium)", "SIEMENS-NV-BELGIUM" },
	{ 0x001107, "RGB Networks Inc.", "RGB-NETWORKS" },
	{ 0x001108, "Orbital Data Corporation", "ORBITAL-DATA" },
	{ 0x001109, "Micro-Star International", "MICRO-STAR" },
	{ 0x00110A, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x00110B, "Franklin Technology Systems", "FRANKLIN" },
	{ 0x00110C, "Atmark Techno, Inc.", "ATMARK-TECHNO" },
	{ 0x00110D, "SANBlaze Technology, Inc.", "SANBLAZE" },
	{ 0x00110E, "Tsurusaki Sealand Transportation Co. Ltd.", "TSURUSAKI-SEALAND-TRANSPORTATION" },
	{ 0x00110F, "netplat,Inc.", "NETPLAT" },
	{ 0x001110, "Maxanna Technology Co., Ltd.", "MAXANNA" },
	{ 0x001111, "Intel Corporation", "INTEL" },
	{ 0x001112, "Honeywell CMSS", "HONEYWELL-CMSS" },
	{ 0x001113, "Fraunhofer FOKUS", "FRAUNHOFER-FOKUS" },
	{ 0x001114, "EverFocus Electronics Corp.", "EVERFOCUS-ELECTRONICS" },
	{ 0x001115, "EPIN Technologies, Inc.", "EPIN-TECHNOLOGIES" },
	{ 0x001116, "COTEAU VERT CO., LTD.", "COTEAU-VERT" },
	{ 0x001117, "CESNET", "CESNET" },
	{ 0x001118, "BLX IC Design Corp., Ltd.", "BLX-IC-DESIGN" },
	{ 0x001119, "Solteras, Inc.", "SOLTERAS" },
	{ 0x00111A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00111B, "Targa Systems Div L-3 Communications Canada", "TARGA-DIV-L-3-COMMUNICATION-CANADA" },
	{ 0x00111C, "Pleora Technologies Inc.", "PLEORA-TECHNOLOGIES" },
	{ 0x00111D, "Hectrix Limited", "HECTRIX" },
	{ 0x00111E, "EPSG (Ethernet Powerlink Standardization Group)", "EPSG-ETHERNET-POWERLINK-STANDARDIZATION-GROUP" },
	{ 0x00111F, "Doremi Labs, Inc.", "DOREMI-LABS" },
	{ 0x001120, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001121, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001122, "CIMSYS Inc", "CIMSYS" },
	{ 0x001123, "Appointech, Inc.", "APPOINTECH" },
	{ 0x001124, "Apple", "APPLE" },
	{ 0x001125, "IBM Corp", "IBM" },
	{ 0x001126, "Venstar Inc.", "VENSTAR" },
	{ 0x001127, "TASI, Inc", "TASI" },
	{ 0x001128, "Streamit", "STREAMIT" },
	{ 0x001129, "Paradise Datacom Ltd.", "PARADISE-DATACOM" },
	{ 0x00112A, "Niko NV", "NIKO-NV" },
	{ 0x00112B, "NetModule AG", "NETMODULE" },
	{ 0x00112C, "IZT GmbH", "IZT" },
	{ 0x00112D, "iPulse Systems", "IPULSE" },
	{ 0x00112E, "CEICOM", "CEICOM" },
	{ 0x00112F, "ASUSTek Computer Inc.", "ASUSTEK" },
	{ 0x001130, "Allied Telesis (Hong Kong) Ltd.", "ALLIED-TELESIS-HONG-KONG" },
	{ 0x001131, "UNATECH. CO.,LTD", "UNATECH" },
	{ 0x001132, "Synology Incorporated", "SYNOLOGY-INCORPORATED" },
	{ 0x001133, "Siemens Austria SIMEA", "SIEMENS-AUSTRIA-SIMEA" },
	{ 0x001134, "MediaCell, Inc.", "MEDIACELL" },
	{ 0x001135, "Grandeye Ltd", "GRANDEYE" },
	{ 0x001136, "Goodrich Sensor Systems", "GOODRICH-SENSOR" },
	{ 0x001137, "AICHI ELECTRIC CO., LTD.", "AICHI-ELECTRIC" },
	{ 0x001138, "TAISHIN CO., LTD.", "TAISHIN" },
	{ 0x001139, "STOEBER ANTRIEBSTECHNIK GmbH + Co. KG.", "STOEBER-ANTRIEBSTECHNIK-+" },
	{ 0x00113A, "SHINBORAM", "SHINBORAM" },
	{ 0x00113B, "Micronet Communications Inc.", "MICRONET-COMMUNICATION" },
	{ 0x00113C, "Micronas GmbH", "MICRONAS" },
	{ 0x00113D, "KN SOLTEC CO.,LTD.", "KN-SOLTEC" },
	{ 0x00113E, "JL Corporation", "JL" },
	{ 0x00113F, "Alcatel DI", "ALCATEL-DI" },
	{ 0x001140, "Nanometrics Inc.", "NANOMETRICS" },
	{ 0x001141, "GoodMan Corporation", "GOODMAN" },
	{ 0x001142, "e-SMARTCOM INC.", "E-SMARTCOM" },
	{ 0x001143, "Dell Inc", "DELL" },
	{ 0x001144, "Assurance Technology Corp", "ASSURANCE" },
	{ 0x001145, "ValuePoint Networks", "VALUEPOINT-NETWORKS" },
	{ 0x001146, "Telecard-Pribor Ltd", "TELECARD-PRIBOR" },
	{ 0x001147, "Secom-Industry co.LTD.", "SECOM-INDUSTRY" },
	{ 0x001148, "Prolon Control Systems", "PROLON-CONTROL" },
	{ 0x001149, "Proliphix Inc.", "PROLIPHIX" },
	{ 0x00114A, "KAYABA INDUSTRY Co,.Ltd.", "KAYABA-INDUSTRY" },
	{ 0x00114B, "Francotyp-Postalia GmbH", "FRANCOTYP-POSTALIA" },
	{ 0x00114C, "caffeina applied research ltd.", "CAFFEINA-APPLIED-RESEARCH" },
	{ 0x00114D, "Atsumi Electric Co.,LTD.", "ATSUMI-ELECTRIC" },
	{ 0x00114E, "690885 Ontario Inc.", "690885-ONTARIO" },
	{ 0x00114F, "US Digital Television, Inc", "US-DIGITAL-TELEVISION" },
	{ 0x001150, "Belkin Corporation", "BELKIN" },
	{ 0x001151, "Mykotronx", "MYKOTRONX" },
	{ 0x001152, "Eidsvoll Electronics AS", "EIDSVOLL-ELECTRONICS-AS" },
	{ 0x001153, "Trident Tek, Inc.", "TRIDENT-TEK" },
	{ 0x001154, "Webpro Technologies Inc.", "WEBPRO-TECHNOLOGIES" },
	{ 0x001155, "Sevis Systems", "SEVIS" },
	{ 0x001156, "Pharos Systems NZ", "PHAROS-NZ" },
	{ 0x001157, "OF Networks Co., Ltd.", "OF-NETWORKS" },
	{ 0x001158, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x001159, "MATISSE NETWORKS INC", "MATISSE-NETWORKS" },
	{ 0x00115A, "Ivoclar Vivadent AG", "IVOCLAR-VIVADENT" },
	{ 0x00115B, "Elitegroup Computer System Co. (ECS)", "ELITEGROUP-SYSTEM-ECS" },
	{ 0x00115C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00115D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00115E, "ProMinent Dosiertechnik GmbH", "PROMINENT-DOSIERTECHNIK" },
	{ 0x00115F, "ITX Security Co., Ltd.", "ITX-SECURITY" },
	{ 0x001160, "ARTDIO Company Co., LTD", "ARTDIO-COMPANY" },
	{ 0x001161, "NetStreams, LLC", "NETSTREAMS-LLC" },
	{ 0x001162, "STAR MICRONICS CO.,LTD.", "STAR-MICRONICS" },
	{ 0x001163, "SYSTEM SPA DEPT. ELECTRONICS", "SYSTEM-SPA-DEPT-ELECTRONICS" },
	{ 0x001164, "ACARD Technology Corp.", "ACARD" },
	{ 0x001165, "Znyx Networks", "ZNYX-NETWORKS" },
	{ 0x001166, "Taelim Electronics Co., Ltd.", "TAELIM-ELECTRONICS" },
	{ 0x001167, "Integrated System Solution Corp.", "INTEGRATED-SYSTEM-SOLUTION" },
	{ 0x001168, "HomeLogic LLC", "HOMELOGIC-LLC" },
	{ 0x001169, "EMS Satcom", "EMS-SATCOM" },
	{ 0x00116A, "Domo Ltd", "DOMO" },
	{ 0x00116B, "Digital Data Communications Asia Co.,Ltd", "DIGITAL-DATA-COMMUNICATION-ASIA" },
	{ 0x00116C, "Nanwang Multimedia Inc.,Ltd", "NANWANG-MULTIMEDIA" },
	{ 0x00116D, "American Time and Signal", "AMERICAN-TIME-AND-SIGNAL" },
	{ 0x00116E, "PePLink Ltd.", "PEPLINK" },
	{ 0x00116F, "Netforyou Co., LTD.", "NETFORYOU" },
	{ 0x001170, "GSC SRL", "GSC-SRL" },
	{ 0x001171, "DEXTER Communications, Inc.", "DEXTER-COMMUNICATION" },
	{ 0x001172, "COTRON CORPORATION", "COTRON" },
	{ 0x001173, "SMART Storage Systems", "SMART-STORAGE" },
	{ 0x001174, "Wibhu Technologies, Inc.", "WIBHU-TECHNOLOGIES" },
	{ 0x001175, "Intel Corporation", "INTEL" },
	{ 0x001176, "Intellambda Systems, Inc.", "INTELLAMBDA" },
	{ 0x001177, "Coaxial Networks, Inc.", "COAXIAL-NETWORKS" },
	{ 0x001178, "Chiron Technology Ltd", "CHIRON" },
	{ 0x001179, "Singular Technology Co. Ltd.", "SINGULAR" },
	{ 0x00117A, "Singim International Corp.", "SINGIM" },
	{ 0x00117B, "Büchi Labortechnik AG", "BüCHI-LABORTECHNIK" },
	{ 0x00117C, "e-zy.net", "E-ZY-NET" },
	{ 0x00117D, "ZMD America, Inc.", "ZMD-AMERICA" },
	{ 0x00117E, "Progeny, A division of Midmark Corp", "PROGENY-A-DIVISION-OF-MIDMARK" },
	{ 0x00117F, "Neotune Information Technology Corporation,.LTD", "NEOTUNE-INFORMATION" },
	{ 0x001180, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001181, "InterEnergy Co.Ltd,", "INTERENERGY" },
	{ 0x001182, "IMI Norgren Ltd", "IMI-NORGREN" },
	{ 0x001183, "Datalogic ADC, Inc.", "DATALOGIC-ADC" },
	{ 0x001184, "Humo Laboratory,Ltd.", "HUMO-LABORATORY" },
	{ 0x001185, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001186, "Prime Systems, Inc.", "PRIME" },
	{ 0x001187, "Category Solutions, Inc", "CATEGORY-SOLUTIONS" },
	{ 0x001188, "Enterasys", "ENTERASYS" },
	{ 0x001189, "Aerotech Inc", "AEROTECH" },
	{ 0x00118A, "Viewtran Technology Limited", "VIEWTRAN" },
	{ 0x00118B, "Alcatel-Lucent, Enterprise Business Group", "ALCATEL-LUCENT-BUSINESS-GROUP" },
	{ 0x00118C, "Missouri Department of Transportation", "MISSOURI-DEPARTMENT-OF-TRANSPORTATION" },
	{ 0x00118D, "Hanchang System Corp.", "HANCHANG-SYSTEM" },
	{ 0x00118E, "Halytech Mace", "HALYTECH-MACE" },
	{ 0x00118F, "EUTECH INSTRUMENTS PTE. LTD.", "EUTECH-INSTRUMENTS-PTE" },
	{ 0x001190, "Digital Design Corporation", "DIGITAL-DESIGN" },
	{ 0x001191, "CTS-Clima Temperatur Systeme GmbH", "CTS-CLIMA-TEMPERATUR-SYSTEME" },
	{ 0x001192, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001193, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001194, "Chi Mei Communication Systems, Inc.", "CHI-MEI-COMMUNICATION" },
	{ 0x001195, "D-Link Corporation", "D-LINK" },
	{ 0x001196, "Actuality Systems, Inc.", "ACTUALITY" },
	{ 0x001197, "Monitoring Technologies Limited", "MONITORING-TECHNOLOGIES" },
	{ 0x001198, "Prism Media Products Limited", "PRISM-MEDIA-PRODUCTS" },
	{ 0x001199, "2wcom Systems GmbH", "2WCOM" },
	{ 0x00119A, "Alkeria srl", "ALKERIA-SRL" },
	{ 0x00119B, "Telesynergy Research Inc.", "TELESYNERGY-RESEARCH" },
	{ 0x00119C, "EP&T Energy", "EP-T-ENERGY" },
	{ 0x00119D, "Diginfo Technology Corporation", "DIGINFO" },
	{ 0x00119E, "Solectron Brazil", "SOLECTRON-BRAZIL" },
	{ 0x00119F, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0011A0, "Vtech Engineering Canada Ltd", "VTECH-ENGINEERING-CANADA" },
	{ 0x0011A1, "VISION NETWARE CO.,LTD", "VISION-NETWARE" },
	{ 0x0011A2, "Manufacturing Technology Inc", "MANUFACTURING" },
	{ 0x0011A3, "LanReady Technologies Inc.", "LANREADY-TECHNOLOGIES" },
	{ 0x0011A4, "JStream Technologies Inc.", "JSTREAM-TECHNOLOGIES" },
	{ 0x0011A5, "Fortuna Electronic Corp.", "FORTUNA" },
	{ 0x0011A6, "Sypixx Networks", "SYPIXX-NETWORKS" },
	{ 0x0011A7, "Infilco Degremont Inc.", "INFILCO-DEGREMONT" },
	{ 0x0011A8, "Quest Technologies", "QUEST-TECHNOLOGIES" },
	{ 0x0011A9, "MOIMSTONE Co., LTD", "MOIMSTONE" },
	{ 0x0011AA, "Uniclass Technology, Co., LTD", "UNICLASS" },
	{ 0x0011AB, "TRUSTABLE TECHNOLOGY CO.,LTD.", "TRUSTABLE" },
	{ 0x0011AC, "Simtec Electronics", "SIMTEC-ELECTRONICS" },
	{ 0x0011AD, "Shanghai Ruijie Technology", "SHANGHAI-RUIJIE" },
	{ 0x0011AE, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0011AF, "Medialink-i,Inc", "MEDIALINK-I" },
	{ 0x0011B0, "Fortelink Inc.", "FORTELINK" },
	{ 0x0011B1, "BlueExpert Technology Corp.", "BLUEEXPERT" },
	{ 0x0011B2, "2001 Technology Inc.", "2001" },
	{ 0x0011B3, "YOSHIMIYA CO.,LTD.", "YOSHIMIYA" },
	{ 0x0011B4, "Westermo Teleindustri AB", "WESTERMO-TELEINDUSTRI-AB" },
	{ 0x0011B5, "Shenzhen Powercom Co.,Ltd", "SHENZHEN-POWERCOM" },
	{ 0x0011B6, "Open Systems International", "OPEN" },
	{ 0x0011B7, "Octalix B.V.", "OCTALIX" },
	{ 0x0011B8, "Liebherr - Elektronik GmbH", "LIEBHERR---ELEKTRONIK" },
	{ 0x0011B9, "Inner Range Pty. Ltd.", "INNER-RANGE-PTY" },
	{ 0x0011BA, "Elexol Pty Ltd", "ELEXOL-PTY" },
	{ 0x0011BB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0011BC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0011BD, "Bombardier Transportation", "BOMBARDIER-TRANSPORTATION" },
	{ 0x0011BE, "AGP Telecom Co. Ltd", "AGP-TELECOM" },
	{ 0x0011BF, "AESYS S.p.A.", "AESYS-S-P-A" },
	{ 0x0011C0, "Aday Technology Inc", "ADAY" },
	{ 0x0011C1, "4P MOBILE DATA PROCESSING", "4P-MOBILE-DATA-PROCESSING" },
	{ 0x0011C2, "United Fiber Optic Communication", "UNITED-FIBER-OPTIC-COMMUNICATION" },
	{ 0x0011C3, "Transceiving System Technology Corporation", "TRANSCEIVING-SYSTEM" },
	{ 0x0011C4, "Terminales de Telecomunicacion Terrestre, S.L.", "TERMINALES-DE-TELECOMUNICACION-TERRESTRE-S-L" },
	{ 0x0011C5, "TEN Technology", "TEN" },
	{ 0x0011C6, "Seagate Technology", "SEAGATE" },
	{ 0x0011C7, "Raymarine UK Ltd", "RAYMARINE-UK" },
	{ 0x0011C8, "Powercom Co., Ltd.", "POWERCOM" },
	{ 0x0011C9, "MTT Corporation", "MTT" },
	{ 0x0011CA, "Long Range Systems, Inc.", "LONG-RANGE" },
	{ 0x0011CB, "Jacobsons AB", "JACOBSONS-AB" },
	{ 0x0011CC, "Guangzhou Jinpeng Group Co.,Ltd.", "GUANGZHOU-JINPENG-GROUP" },
	{ 0x0011CD, "Axsun Technologies", "AXSUN-TECHNOLOGIES" },
	{ 0x0011CE, "Ubisense Limited", "UBISENSE" },
	{ 0x0011CF, "Thrane & Thrane A/S", "THRANE-THRANE-A/S" },
	{ 0x0011D0, "Tandberg Data ASA", "TANDBERG-DATA-ASA" },
	{ 0x0011D1, "Soft Imaging System GmbH", "SOFT-IMAGING-SYSTEM" },
	{ 0x0011D2, "Perception Digital Ltd", "PERCEPTION-DIGITAL" },
	{ 0x0011D3, "NextGenTel Holding ASA", "NEXTGENTEL-HOLDING-ASA" },
	{ 0x0011D4, "NetEnrich, Inc", "NETENRICH" },
	{ 0x0011D5, "Hangzhou Sunyard System Engineering Co.,Ltd.", "HANGZHOU-SUNYARD-SYSTEM-ENGINEERING" },
	{ 0x0011D6, "HandEra, Inc.", "HANDERA" },
	{ 0x0011D7, "eWerks Inc", "EWERKS" },
	{ 0x0011D8, "ASUSTek Computer Inc.", "ASUSTEK" },
	{ 0x0011D9, "TiVo", "TIVO" },
	{ 0x0011DA, "Vivaas Technology Inc.", "VIVAAS" },
	{ 0x0011DB, "Land-Cellular Corporation", "LAND-CELLULAR" },
	{ 0x0011DC, "Glunz & Jensen", "GLUNZ-JENSEN" },
	{ 0x0011DD, "FROMUS TEC. Co., Ltd.", "FROMUS-TEC" },
	{ 0x0011DE, "EURILOGIC", "EURILOGIC" },
	{ 0x0011DF, "Current Energy", "CURRENT-ENERGY" },
	{ 0x0011E0, "U-MEDIA Communications, Inc.", "U-MEDIA-COMMUNICATION" },
	{ 0x0011E1, "Arcelik A.S", "ARCELIK-A-S" },
	{ 0x0011E2, "Hua Jung Components Co., Ltd.", "HUA-JUNG-COMPONENTS" },
	{ 0x0011E3, "Thomson, Inc.", "THOMSON" },
	{ 0x0011E4, "Danelec Electronics A/S", "DANELEC-ELECTRONICS-A/S" },
	{ 0x0011E5, "KCodes Corporation", "KCODES" },
	{ 0x0011E6, "Scientific Atlanta", "SCIENTIFIC-ATLANTA" },
	{ 0x0011E7, "WORLDSAT - Texas de France", "WORLDSAT---TEXAS-DE-FRANCE" },
	{ 0x0011E8, "Tixi.Com", "TIXI-COM" },
	{ 0x0011E9, "STARNEX CO., LTD.", "STARNEX" },
	{ 0x0011EA, "IWICS Inc.", "IWICS" },
	{ 0x0011EB, "Innovative Integration", "INNOVATIVE-INTEGRATION" },
	{ 0x0011EC, "AVIX INC.", "AVIX" },
	{ 0x0011ED, "802 Global", "802-GLOBAL" },
	{ 0x0011EE, "Estari, Inc.", "ESTARI" },
	{ 0x0011EF, "Conitec Datensysteme GmbH", "CONITEC-DATENSYSTEME" },
	{ 0x0011F0, "Wideful Limited", "WIDEFUL" },
	{ 0x0011F1, "QinetiQ Ltd", "QINETIQ" },
	{ 0x0011F2, "Institute of Network Technologies", "INSTITUTE-OF-NETWORK-TECHNOLOGIES" },
	{ 0x0011F3, "NeoMedia Europe AG", "NEOMEDIA-EUROPE" },
	{ 0x0011F4, "woori-net", "WOORI-NET" },
	{ 0x0011F5, "ASKEY COMPUTER CORP.", "ASKEY" },
	{ 0x0011F6, "Asia Pacific Microsystems , Inc.", "ASIA-PACIFIC-MICROSYSTEMS" },
	{ 0x0011F7, "Shenzhen Forward Industry Co., Ltd", "SHENZHEN-FORWARD-INDUSTRY" },
	{ 0x0011F8, "AIRAYA Corp", "AIRAYA" },
	{ 0x0011F9, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x0011FA, "Rane Corporation", "RANE" },
	{ 0x0011FB, "Heidelberg Engineering GmbH", "HEIDELBERG-ENGINEERING" },
	{ 0x0011FC, "HARTING Electric Gmbh & Co.KG", "HARTING-ELECTRIC" },
	{ 0x0011FD, "KORG INC.", "KORG" },
	{ 0x0011FE, "Keiyo System Research, Inc.", "KEIYO-SYSTEM-RESEARCH" },
	{ 0x0011FF, "Digitro Tecnologia Ltda", "DIGITRO-TECNOLOGIA-LTDA" },
	{ 0x001200, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001201, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001202, "Decrane Aerospace - Audio International Inc.", "DECRANE-AEROSPACE---AUDIO" },
	{ 0x001203, "ActivNetworks", "ACTIVNETWORKS" },
	{ 0x001204, "u10 Networks, Inc.", "U10-NETWORKS" },
	{ 0x001205, "Terrasat Communications, Inc.", "TERRASAT-COMMUNICATION" },
	{ 0x001206, "iQuest (NZ) Ltd", "IQUEST-NZ" },
	{ 0x001207, "Head Strong International Limited", "HEAD-STRONG" },
	{ 0x001208, "Gantner Instruments GmbH", "GANTNER-INSTRUMENTS" },
	{ 0x001209, "Fastrax Ltd", "FASTRAX" },
	{ 0x00120A, "Emerson Climate Technologies GmbH", "EMERSON-CLIMATE-TECHNOLOGIES" },
	{ 0x00120B, "Chinasys Technologies Limited", "CHINASYS-TECHNOLOGIES" },
	{ 0x00120C, "CE-Infosys Pte Ltd", "CE-INFOSYS-PTE" },
	{ 0x00120D, "Advanced Telecommunication Technologies, Inc.", "ADVANCED-TELECOMMUNICATION-TECHNOLOGIES" },
	{ 0x00120E, "AboCom", "ABOCOM" },
	{ 0x00120F, "IEEE 802.3", "IEEE-802-3" },
	{ 0x001210, "WideRay Corp", "WIDERAY" },
	{ 0x001211, "Protechna Herbst GmbH & Co. KG", "PROTECHNA-HERBST" },
	{ 0x001212, "PLUS Corporation", "PLUS" },
	{ 0x001213, "Metrohm AG", "METROHM" },
	{ 0x001214, "Koenig & Bauer AG", "KOENIG-BAUER" },
	{ 0x001215, "iStor Networks, Inc.", "ISTOR-NETWORKS" },
	{ 0x001216, "ICP Internet Communication Payment AG", "ICP-INTERNET-COMMUNICATION-PAYMENT" },
	{ 0x001217, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x001218, "ARUZE Corporation", "ARUZE" },
	{ 0x001219, "Ahead Communication Systems Inc", "AHEAD-COMMUNICATION" },
	{ 0x00121A, "Techno Soft Systemnics Inc.", "TECHNO-SOFT-SYSTEMNICS" },
	{ 0x00121B, "Sound Devices, LLC", "SOUND-DEVICES-LLC" },
	{ 0x00121C, "PARROT S.A.", "PARROT" },
	{ 0x00121D, "Netfabric Corporation", "NETFABRIC" },
	{ 0x00121E, "Juniper Networks, Inc.", "JUNIPER-NETWORKS" },
	{ 0x00121F, "Harding Instruments", "HARDING-INSTRUMENTS" },
	{ 0x001220, "Cadco Systems", "CADCO" },
	{ 0x001221, "B.Braun Melsungen AG", "B-BRAUN-MELSUNGEN" },
	{ 0x001222, "Skardin (UK) Ltd", "SKARDIN-UK" },
	{ 0x001223, "Pixim", "PIXIM" },
	{ 0x001224, "NexQL Corporation", "NEXQL" },
	{ 0x001225, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001226, "Japan Direx Corporation", "JAPAN-DIREX" },
	{ 0x001227, "Franklin Electric Co., Inc.", "FRANKLIN-ELECTRIC" },
	{ 0x001228, "Data Ltd.", "DATA" },
	{ 0x001229, "BroadEasy Technologies Co.,Ltd", "BROADEASY-TECHNOLOGIES" },
	{ 0x00122A, "VTech Telecommunications Ltd.", "VTECH-TELECOMMUNICATIONS" },
	{ 0x00122B, "Virbiage Pty Ltd", "VIRBIAGE-PTY" },
	{ 0x00122C, "Soenen Controls N.V.", "SOENEN-CONTROLS-N-V" },
	{ 0x00122D, "SiNett Corporation", "SINETT" },
	{ 0x00122E, "Signal Technology - AISD", "SIGNAL---AISD" },
	{ 0x00122F, "Sanei Electric Inc.", "SANEI-ELECTRIC" },
	{ 0x001230, "Picaso Infocommunication CO., LTD.", "PICASO-INFOCOMMUNICATION" },
	{ 0x001231, "Motion Control Systems, Inc.", "MOTION-CONTROL" },
	{ 0x001232, "LeWiz Communications Inc.", "LEWIZ-COMMUNICATION" },
	{ 0x001233, "JRC TOKKI Co.,Ltd.", "JRC-TOKKI" },
	{ 0x001234, "Camille Bauer", "CAMILLE-BAUER" },
	{ 0x001235, "Andrew Corporation", "ANDREW" },
	{ 0x001236, "ConSentry Networks", "CONSENTRY-NETWORKS" },
	{ 0x001237, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x001238, "SetaBox Technology Co., Ltd.", "SETABOX" },
	{ 0x001239, "S Net Systems Inc.", "S-NET" },
	{ 0x00123A, "Posystech Inc., Co.", "POSYSTECH" },
	{ 0x00123B, "KeRo Systems ApS", "KERO-APS" },
	{ 0x00123C, "Second Rule LLC", "SECOND-RULE-LLC" },
	{ 0x00123D, "GES Co, Ltd", "GES" },
	{ 0x00123E, "ERUNE technology Co., Ltd.", "ERUNE" },
	{ 0x00123F, "Dell Inc", "DELL" },
	{ 0x001240, "AMOI ELECTRONICS CO.,LTD", "AMOI-ELECTRONICS" },
	{ 0x001241, "a2i marketing center", "A2I-MARKETING-CENTER" },
	{ 0x001242, "Millennial Net", "MILLENNIAL-NET" },
	{ 0x001243, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001244, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001245, "Zellweger Analytics, Inc.", "ZELLWEGER-ANALYTICS" },
	{ 0x001246, "T.O.M TECHNOLOGY INC..", "T-O-M" },
	{ 0x001247, "Samsung Electronics Co., Ltd.", "SAMSUNG-ELECTRONICS" },
	{ 0x001248, "EMC Corporation (Kashya)", "EMC-KASHYA" },
	{ 0x001249, "Delta Elettronica S.p.A.", "DELTA-ELETTRONICA-S-P-A" },
	{ 0x00124A, "Dedicated Devices, Inc.", "DEDICATED-DEVICES" },
	{ 0x00124B, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x00124C, "BBWM Corporation", "BBWM" },
	{ 0x00124D, "Inducon BV", "INDUCON" },
	{ 0x00124E, "XAC AUTOMATION CORP.", "XAC-AUTOMATION" },
	{ 0x00124F, "Pentair Thermal Management", "PENTAIR-THERMAL-MANAGEMENT" },
	{ 0x001250, "Tokyo Aircaft Instrument Co., Ltd.", "TOKYO-AIRCAFT-INSTRUMENT" },
	{ 0x001251, "SILINK", "SILINK" },
	{ 0x001252, "Citronix, LLC", "CITRONIX-LLC" },
	{ 0x001253, "AudioDev AB", "AUDIODEV-AB" },
	{ 0x001254, "Spectra Technologies Holdings Company Ltd", "SPECTRA-TECHNOLOGIES-HOLDINGS-COMPANY" },
	{ 0x001255, "NetEffect Incorporated", "NETEFFECT-INCORPORATED" },
	{ 0x001256, "LG INFORMATION & COMM.", "LG-INFORMATION-COMM" },
	{ 0x001257, "LeapComm Communication Technologies Inc.", "LEAPCOMM-COMMUNICATION-TECHNOLOGIES" },
	{ 0x001258, "Activis Polska", "ACTIVIS-POLSKA" },
	{ 0x001259, "THERMO ELECTRON KARLSRUHE", "THERMO-ELECTRON-KARLSRUHE" },
	{ 0x00125A, "Microsoft Corporation", "MICROSOFT" },
	{ 0x00125B, "KAIMEI ELECTRONI", "KAIMEI-ELECTRONI" },
	{ 0x00125C, "Green Hills Software, Inc.", "GREEN-HILLS-SOFTWARE" },
	{ 0x00125D, "CyberNet Inc.", "CYBERNET" },
	{ 0x00125E, "CAEN", "CAEN" },
	{ 0x00125F, "AWIND Inc.", "AWIND" },
	{ 0x001260, "Stanton Magnetics,inc.", "STANTON-MAGNETICS" },
	{ 0x001261, "Adaptix, Inc", "ADAPTIX" },
	{ 0x001262, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001263, "Data Voice Technologies GmbH", "DATA-VOICE-TECHNOLOGIES" },
	{ 0x001264, "daum electronic gmbh", "DAUM" },
	{ 0x001265, "Enerdyne Technologies, Inc.", "ENERDYNE-TECHNOLOGIES" },
	{ 0x001266, "Swisscom Hospitality Services SA", "SWISSCOM-HOSPITALITY-SERVICES" },
	{ 0x001267, "Panasonic Corporation", "PANASONIC" },
	{ 0x001268, "IPS d.o.o.", "IPS-D-O-O" },
	{ 0x001269, "Value Electronics", "VALUE-ELECTRONICS" },
	{ 0x00126A, "OPTOELECTRONICS Co., Ltd.", "OPTOELECTRONICS" },
	{ 0x00126B, "Ascalade Communications Limited", "ASCALADE-COMMUNICATION" },
	{ 0x00126C, "Visonic Ltd.", "VISONIC" },
	{ 0x00126D, "University of California, Berkeley", "UNIVERSITY-OF-CALIFORNIA-BERKELEY" },
	{ 0x00126E, "Seidel Elektronik GmbH Nfg.KG", "SEIDEL-ELEKTRONIK-NFG" },
	{ 0x00126F, "Rayson Technology Co., Ltd.", "RAYSON" },
	{ 0x001270, "NGES Denro Systems", "NGES-DENRO" },
	{ 0x001271, "Measurement Computing Corp", "MEASUREMENT-COMPUTING" },
	{ 0x001272, "Redux Communications Ltd.", "REDUX-COMMUNICATION" },
	{ 0x001273, "Stoke Inc", "STOKE" },
	{ 0x001274, "NIT lab", "NIT-LAB" },
	{ 0x001275, "Sentilla Corporation", "SENTILLA" },
	{ 0x001276, "CG Power Systems Ireland Limited", "CG-POWER-IRELAND" },
	{ 0x001277, "Korenix Technologies Co., Ltd.", "KORENIX-TECHNOLOGIES" },
	{ 0x001278, "International Bar Code", "BAR-CODE" },
	{ 0x001279, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x00127A, "Sanyu Industry Co.,Ltd.", "SANYU-INDUSTRY" },
	{ 0x00127B, "VIA Networking Technologies, Inc.", "VIA-NETWORKING-TECHNOLOGIES" },
	{ 0x00127C, "SWEGON AB", "SWEGON-AB" },
	{ 0x00127D, "MobileAria", "MOBILEARIA" },
	{ 0x00127E, "Digital Lifestyles Group, Inc.", "DIGITAL-LIFESTYLES-GROUP" },
	{ 0x00127F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001280, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001281, "March Networks S.p.A.", "MARCH-NETWORKS-S-P-A" },
	{ 0x001282, "Qovia", "QOVIA" },
	{ 0x001283, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x001284, "Lab33 Srl", "LAB33-SRL" },
	{ 0x001285, "Gizmondo Europe Ltd", "GIZMONDO-EUROPE" },
	{ 0x001286, "ENDEVCO CORP", "ENDEVCO" },
	{ 0x001287, "Digital Everywhere Unterhaltungselektronik GmbH", "DIGITAL-EVERYWHERE-UNTERHALTUNGSELEKTRONIK" },
	{ 0x001288, "2Wire, Inc", "2WIRE" },
	{ 0x001289, "Advance Sterilization Products", "ADVANCE-STERILIZATION-PRODUCTS" },
	{ 0x00128A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00128B, "Sensory Networks Inc", "SENSORY-NETWORKS" },
	{ 0x00128C, "Woodward Governor", "WOODWARD-GOVERNOR" },
	{ 0x00128D, "STB Datenservice GmbH", "STB-DATENSERVICE" },
	{ 0x00128E, "Q-Free ASA", "Q-FREE-ASA" },
	{ 0x00128F, "Montilio", "MONTILIO" },
	{ 0x001290, "KYOWA Electric & Machinery Corp.", "KYOWA-ELECTRIC-MACHINERY" },
	{ 0x001291, "KWS Computersysteme GmbH", "KWS-COMPUTERSYSTEME" },
	{ 0x001292, "Griffin Technology", "GRIFFIN" },
	{ 0x001293, "GE Energy", "GE-ENERGY" },
	{ 0x001294, "SUMITOMO ELECTRIC DEVICE INNOVATIONS, INC", "SUMITOMO-ELECTRIC-DEVICE-INNOVATIONS" },
	{ 0x001295, "Aiware Inc.", "AIWARE" },
	{ 0x001296, "Addlogix", "ADDLOGIX" },
	{ 0x001297, "O2Micro, Inc.", "O2MICRO" },
	{ 0x001298, "MICO ELECTRIC(SHENZHEN) LIMITED", "MICO-ELECTRIC-SHENZHEN" },
	{ 0x001299, "Ktech Telecommunications Inc", "KTECH-TELECOMMUNICATIONS" },
	{ 0x00129A, "IRT Electronics Pty Ltd", "IRT-ELECTRONICS-PTY" },
	{ 0x00129B, "E2S Electronic Engineering Solutions, S.L.", "E2S-ENGINEERING-SOLUTIONS-S-L" },
	{ 0x00129C, "Yulinet", "YULINET" },
	{ 0x00129D, "First International Computer do Brasil", "FIRST-DO-BRASIL" },
	{ 0x00129E, "Surf Communications Inc.", "SURF-COMMUNICATION" },
	{ 0x00129F, "RAE Systems", "RAE" },
	{ 0x0012A0, "NeoMeridian Sdn Bhd", "NEOMERIDIAN-SDN-BHD" },
	{ 0x0012A1, "BluePacket Communications Co., Ltd.", "BLUEPACKET-COMMUNICATION" },
	{ 0x0012A2, "VITA", "VITA" },
	{ 0x0012A3, "Trust International B.V.", "TRUST" },
	{ 0x0012A4, "ThingMagic, LLC", "THINGMAGIC-LLC" },
	{ 0x0012A5, "Stargen, Inc.", "STARGEN" },
	{ 0x0012A6, "Dolby Australia", "DOLBY-AUSTRALIA" },
	{ 0x0012A7, "ISR TECHNOLOGIES Inc", "ISR-TECHNOLOGIES" },
	{ 0x0012A8, "intec GmbH", "INTEC" },
	{ 0x0012A9, "3Com Ltd", "3COM" },
	{ 0x0012AA, "IEE, Inc.", "IEE" },
	{ 0x0012AB, "WiLife, Inc.", "WILIFE" },
	{ 0x0012AC, "ONTIMETEK INC.", "ONTIMETEK" },
	{ 0x0012AD, "IDS GmbH", "IDS" },
	{ 0x0012AE, "HLS HARD-LINE Solutions Inc.", "HLS-HARD-LINE-SOLUTIONS" },
	{ 0x0012AF, "ELPRO Technologies", "ELPRO-TECHNOLOGIES" },
	{ 0x0012B0, "Efore Oyj (Plc)", "EFORE-OYJ-PLC" },
	{ 0x0012B1, "Dai Nippon Printing Co., Ltd", "DAI-NIPPON-PRINTING" },
	{ 0x0012B2, "AVOLITES LTD.", "AVOLITES" },
	{ 0x0012B3, "Advance Wireless Technology Corp.", "ADVANCE-WIRELESS" },
	{ 0x0012B4, "Work Microwave GmbH", "WORK-MICROWAVE" },
	{ 0x0012B5, "Vialta, Inc.", "VIALTA" },
	{ 0x0012B6, "Santa Barbara Infrared, Inc.", "SANTA-BARBARA-INFRARED" },
	{ 0x0012B7, "PTW Freiburg", "PTW-FREIBURG" },
	{ 0x0012B8, "G2 Microsystems", "G2-MICROSYSTEMS" },
	{ 0x0012B9, "Fusion Digital Technology", "FUSION-DIGITAL" },
	{ 0x0012BA, "FSI Systems, Inc.", "FSI" },
	{ 0x0012BB, "Telecommunications Industry Association TR-41 Committee", "TELECOMMUNICATIONS-INDUSTRY-ASSOCIATION-TR-41-COMMITTEE" },
	{ 0x0012BC, "Echolab LLC", "ECHOLAB-LLC" },
	{ 0x0012BD, "Avantec Manufacturing Limited", "AVANTEC-MANUFACTURING" },
	{ 0x0012BE, "Astek Corporation", "ASTEK" },
	{ 0x0012BF, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x0012C0, "HotLava Systems, Inc.", "HOTLAVA" },
	{ 0x0012C1, "Check Point Software Technologies", "CHECK-POINT-SOFTWARE-TECHNOLOGIES" },
	{ 0x0012C2, "Apex Electronics Factory", "APEX-ELECTRONICS-FACTORY" },
	{ 0x0012C3, "WIT S.A.", "WIT" },
	{ 0x0012C4, "Viseon, Inc.", "VISEON" },
	{ 0x0012C5, "V-Show Technology (China) Co.,Ltd", "V-SHOW-CHINA" },
	{ 0x0012C6, "TGC America, Inc", "TGC-AMERICA" },
	{ 0x0012C7, "SECURAY Technologies Ltd.Co.", "SECURAY-TECHNOLOGIES" },
	{ 0x0012C8, "Perfect tech", "PERFECT-TECH" },
	{ 0x0012C9, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0012CA, "Mechatronic Brick Aps", "MECHATRONIC-BRICK-APS" },
	{ 0x0012CB, "CSS Inc.", "CSS" },
	{ 0x0012CC, "Bitatek CO., LTD", "BITATEK" },
	{ 0x0012CD, "ASEM SpA", "ASEM-SPA" },
	{ 0x0012CE, "Advanced Cybernetics Group", "ADVANCED-CYBERNETICS-GROUP" },
	{ 0x0012CF, "Accton Technology Corporation", "ACCTON" },
	{ 0x0012D0, "Gossen-Metrawatt-GmbH", "GOSSEN-METRAWATT" },
	{ 0x0012D1, "Texas Instruments Inc", "TEXAS-INSTRUMENTS" },
	{ 0x0012D2, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0012D3, "Zetta Systems, Inc.", "ZETTA" },
	{ 0x0012D4, "Princeton Technology, Ltd", "PRINCETON" },
	{ 0x0012D5, "Motion Reality Inc.", "MOTION-REALITY" },
	{ 0x0012D6, "Jiangsu Yitong High-Tech Co.,Ltd", "JIANGSU-YITONG-HIGH-TECH" },
	{ 0x0012D7, "Invento Networks, Inc.", "INVENTO-NETWORKS" },
	{ 0x0012D8, "International Games System Co., Ltd.", "GAMES-SYSTEM" },
	{ 0x0012D9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0012DA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0012DB, "ZIEHL industrie-elektronik GmbH + Co KG", "ZIEHL-INDUSTRIE-ELEKTRONIK-+" },
	{ 0x0012DC, "SunCorp Industrial Limited", "SUNCORP-INDUSTRIAL" },
	{ 0x0012DD, "Shengqu Information Technology (Shanghai) Co., Ltd.", "SHENGQU-INFORMATION-SHANGHAI" },
	{ 0x0012DE, "Radio Components Sweden AB", "RADIO-COMPONENTS-SWEDEN-AB" },
	{ 0x0012DF, "Novomatic AG", "NOVOMATIC" },
	{ 0x0012E0, "Codan Limited", "CODAN" },
	{ 0x0012E1, "Alliant Networks, Inc", "ALLIANT-NETWORKS" },
	{ 0x0012E2, "ALAXALA Networks Corporation", "ALAXALA-NETWORKS" },
	{ 0x0012E3, "Agat-RT, Ltd.", "AGAT-RT" },
	{ 0x0012E4, "ZIEHL industrie-electronik GmbH + Co KG", "ZIEHL-INDUSTRIE-ELECTRONIK-+" },
	{ 0x0012E5, "Time America, Inc.", "TIME-AMERICA" },
	{ 0x0012E6, "SPECTEC COMPUTER CO., LTD.", "SPECTEC" },
	{ 0x0012E7, "Projectek Networking Electronics Corp.", "PROJECTEK-NETWORKING-ELECTRONICS" },
	{ 0x0012E8, "Fraunhofer IMS", "FRAUNHOFER-IMS" },
	{ 0x0012E9, "Abbey Systems Ltd", "ABBEY" },
	{ 0x0012EA, "Trane", "TRANE" },
	{ 0x0012EB, "PDH Solutions, LLC", "PDH-SOLUTIONS-LLC" },
	{ 0x0012EC, "Movacolor b.v.", "MOVACOLOR" },
	{ 0x0012ED, "AVG Advanced Technologies", "AVG-ADVANCED-TECHNOLOGIES" },
	{ 0x0012EE, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x0012EF, "OneAccess SA", "ONEACCESS" },
	{ 0x0012F0, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0012F1, "IFOTEC", "IFOTEC" },
	{ 0x0012F2, "Brocade Communications Systems, Inc", "BROCADE-COMMUNICATION" },
	{ 0x0012F3, "connectBlue AB", "CONNECTBLUE-AB" },
	{ 0x0012F4, "Belco International Co.,Ltd.", "BELCO" },
	{ 0x0012F5, "Imarda New Zealand Limited", "IMARDA-NEW-ZEALAND" },
	{ 0x0012F6, "MDK CO.,LTD.", "MDK" },
	{ 0x0012F7, "Xiamen Xinglian Electronics Co., Ltd.", "XIAMEN-XINGLIAN-ELECTRONICS" },
	{ 0x0012F8, "WNI Resources, LLC", "WNI-RESOURCES-LLC" },
	{ 0x0012F9, "URYU SEISAKU, LTD.", "URYU-SEISAKU" },
	{ 0x0012FA, "THX LTD", "THX" },
	{ 0x0012FB, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x0012FC, "PLANET System Co.,LTD", "PLANET-SYSTEM" },
	{ 0x0012FD, "OPTIMUS IC S.A.", "OPTIMUS-IC" },
	{ 0x0012FE, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x0012FF, "Lely Industries N.V.", "LELY-INDUSTRIES-N-V" },
	{ 0x001300, "IT-FACTORY, INC.", "IT-FACTORY" },
	{ 0x001301, "IronGate S.L.", "IRONGATE-S-L" },
	{ 0x001302, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001303, "GateConnect", "GATECONNECT" },
	{ 0x001304, "Flaircomm Technologies Co. LTD", "FLAIRCOMM-TECHNOLOGIES" },
	{ 0x001305, "Epicom, Inc.", "EPICOM" },
	{ 0x001306, "Always On Wireless", "ALWAYS-ON-WIRELESS" },
	{ 0x001307, "Paravirtual Corporation", "PARAVIRTUAL" },
	{ 0x001308, "Nuvera Fuel Cells", "NUVERA-FUEL-CELLS" },
	{ 0x001309, "Ocean Broadband Networks", "OCEAN-BROADBAND-NETWORKS" },
	{ 0x00130A, "Nortel", "NORTEL" },
	{ 0x00130B, "Mextal B.V.", "MEXTAL" },
	{ 0x00130C, "HF System Corporation", "HF-SYSTEM" },
	{ 0x00130D, "GALILEO AVIONICA", "GALILEO-AVIONICA" },
	{ 0x00130E, "Focusrite Audio Engineering Limited", "FOCUSRITE-AUDIO-ENGINEERING" },
	{ 0x00130F, "EGEMEN Bilgisayar Muh San ve Tic LTD STI", "EGEMEN-BILGISAYAR-MUH-SAN-VE-TIC-STI" },
	{ 0x001310, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x001311, "ARRIS International", "ARRIS" },
	{ 0x001312, "Amedia Networks Inc.", "AMEDIA-NETWORKS" },
	{ 0x001313, "GuangZhou Post & Telecom Equipment ltd", "GUANGZHOU-POST-TELECOM-EQUIPMENT" },
	{ 0x001314, "Asiamajor Inc.", "ASIAMAJOR" },
	{ 0x001315, "SONY Computer Entertainment inc,", "SONY-ENTERTAINMENT" },
	{ 0x001316, "L-S-B Broadcast Technologies GmbH", "L-S-B-BROADCAST-TECHNOLOGIES" },
	{ 0x001317, "GN Netcom as", "GN-NETCOM-AS" },
	{ 0x001318, "DGSTATION Co., Ltd.", "DGSTATION" },
	{ 0x001319, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00131A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00131B, "BeCell Innovations Corp.", "BECELL-INNOVATIONS" },
	{ 0x00131C, "LiteTouch, Inc.", "LITETOUCH" },
	{ 0x00131D, "Scanvaegt International A/S", "SCANVAEGT-A/S" },
	{ 0x00131E, "Peiker acustic GmbH & Co. KG", "PEIKER-ACUSTIC" },
	{ 0x00131F, "NxtPhase T&D, Corp.", "NXTPHASE-T-D" },
	{ 0x001320, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001321, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001322, "DAQ Electronics, Inc.", "DAQ-ELECTRONICS" },
	{ 0x001323, "Cap Co., Ltd.", "CAP" },
	{ 0x001324, "Schneider Electric Ultra Terminal", "SCHNEIDER-ELECTRIC-ULTRA-TERMINAL" },
	{ 0x001325, "Cortina Systems Inc", "CORTINA" },
	{ 0x001326, "ECM Systems Ltd", "ECM" },
	{ 0x001327, "Data Acquisitions limited", "DATA-ACQUISITIONS" },
	{ 0x001328, "Westech Korea Inc.,", "WESTECH-KOREA" },
	{ 0x001329, "VSST Co., LTD", "VSST" },
	{ 0x00132A, "Sitronics Telecom Solutions", "SITRONICS-TELECOM-SOLUTIONS" },
	{ 0x00132B, "Phoenix Digital", "PHOENIX-DIGITAL" },
	{ 0x00132C, "MAZ Brandenburg GmbH", "MAZ-BRANDENBURG" },
	{ 0x00132D, "iWise Communications", "IWISE-COMMUNICATION" },
	{ 0x00132E, "ITian Coporation", "ITIAN-COPORATION" },
	{ 0x00132F, "Interactek", "INTERACTEK" },
	{ 0x001330, "EURO PROTECTION SURVEILLANCE", "EURO-PROTECTION-SURVEILLANCE" },
	{ 0x001331, "CellPoint Connect", "CELLPOINT-CONNECT" },
	{ 0x001332, "Beijing Topsec Network Security Technology Co., Ltd.", "BEIJING-TOPSEC-NETWORK-SECURITY" },
	{ 0x001333, "BaudTec Corporation", "BAUDTEC" },
	{ 0x001334, "Arkados, Inc.", "ARKADOS" },
	{ 0x001335, "VS Industry Berhad", "VS-INDUSTRY-BERHAD" },
	{ 0x001336, "Tianjin 712 Communication Broadcasting co., ltd.", "TIANJIN-712-COMMUNICATION-BROADCASTING" },
	{ 0x001337, "Orient Power Home Network Ltd.", "ORIENT-POWER-HOME-NETWORK" },
	{ 0x001338, "FRESENIUS-VIAL", "FRESENIUS-VIAL" },
	{ 0x001339, "CCV Deutschland GmbH", "CCV-DEUTSCHLAND" },
	{ 0x00133A, "VadaTech Inc.", "VADATECH" },
	{ 0x00133B, "Speed Dragon Multimedia Limited", "SPEED-DRAGON-MULTIMEDIA" },
	{ 0x00133C, "QUINTRON SYSTEMS INC.", "QUINTRON" },
	{ 0x00133D, "Micro Memory Curtiss Wright Co", "MICRO-MEMORY-CURTISS-WRIGHT" },
	{ 0x00133E, "MetaSwitch", "METASWITCH" },
	{ 0x00133F, "Eppendorf Instrumente GmbH", "EPPENDORF-INSTRUMENTE" },
	{ 0x001340, "AD.EL s.r.l.", "AD-EL-S-R-L" },
	{ 0x001341, "Shandong New Beiyang Information Technology Co.,Ltd", "SHANDONG-NEW-BEIYANG-INFORMATION" },
	{ 0x001342, "Vision Research, Inc.", "VISION-RESEARCH" },
	{ 0x001343, "Matsushita Electronic Components (Europe) GmbH", "MATSUSHITA-COMPONENTS-EUROPE" },
	{ 0x001344, "Fargo Electronics Inc.", "FARGO-ELECTRONICS" },
	{ 0x001345, "Eaton Corporation", "EATON" },
	{ 0x001346, "D-Link Corporation", "D-LINK" },
	{ 0x001347, "Red Lion Controls, LP", "RED-LION-CONTROLS-LP" },
	{ 0x001348, "Artila Electronics Co., Ltd.", "ARTILA-ELECTRONICS" },
	{ 0x001349, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0x00134A, "Engim, Inc.", "ENGIM" },
	{ 0x00134B, "ToGoldenNet Technology Inc.", "TOGOLDENNET" },
	{ 0x00134C, "YDT Technology International", "YDT" },
	{ 0x00134D, "Inepro BV", "INEPRO" },
	{ 0x00134E, "Valox Systems, Inc.", "VALOX" },
	{ 0x00134F, "Tranzeo Wireless Technologies Inc.", "TRANZEO-WIRELESS-TECHNOLOGIES" },
	{ 0x001350, "Silver Spring Networks, Inc", "SILVER-SPRING-NETWORKS" },
	{ 0x001351, "Niles Audio Corporation", "NILES-AUDIO" },
	{ 0x001352, "Naztec, Inc.", "NAZTEC" },
	{ 0x001353, "HYDAC Filtertechnik GMBH", "HYDAC-FILTERTECHNIK" },
	{ 0x001354, "Zcomax Technologies, Inc.", "ZCOMAX-TECHNOLOGIES" },
	{ 0x001355, "TOMEN Cyber-business Solutions, Inc.", "TOMEN-CYBER-BUSINESS-SOLUTIONS" },
	{ 0x001356, "FLIR Radiation Inc", "FLIR-RADIATION" },
	{ 0x001357, "Soyal Technology Co., Ltd.", "SOYAL" },
	{ 0x001358, "Realm Systems, Inc.", "REALM" },
	{ 0x001359, "ProTelevision Technologies A/S", "PROTELEVISION-TECHNOLOGIES-A/S" },
	{ 0x00135A, "Project T&E Limited", "PROJECT-T-E" },
	{ 0x00135B, "PanelLink Cinema, LLC", "PANELLINK-CINEMA-LLC" },
	{ 0x00135C, "OnSite Systems, Inc.", "ONSITE" },
	{ 0x00135D, "NTTPC Communications, Inc.", "NTTPC-COMMUNICATION" },
	{ 0x00135E, "EAB/RWI/K", "EAB/RWI/K" },
	{ 0x00135F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001360, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001361, "Biospace Co., Ltd.", "BIOSPACE" },
	{ 0x001362, "ShinHeung Precision Co., Ltd.", "SHINHEUNG-PRECISION" },
	{ 0x001363, "Verascape, Inc.", "VERASCAPE" },
	{ 0x001364, "Paradigm Technology Inc..", "PARADIGM" },
	{ 0x001365, "Nortel", "NORTEL" },
	{ 0x001366, "Neturity Technologies Inc.", "NETURITY-TECHNOLOGIES" },
	{ 0x001367, "Narayon. Co., Ltd.", "NARAYON" },
	{ 0x001368, "Saab Danmark A/S", "SAAB-DANMARK-A/S" },
	{ 0x001369, "Honda Electron Co., LED.", "HONDA-ELECTRON-LED" },
	{ 0x00136A, "Hach Lange Sarl", "HACH-LANGE-SARL" },
	{ 0x00136B, "E-TEC", "E-TEC" },
	{ 0x00136C, "TomTom", "TOMTOM" },
	{ 0x00136D, "Tentaculus AB", "TENTACULUS-AB" },
	{ 0x00136E, "Techmetro Corp.", "TECHMETRO" },
	{ 0x00136F, "PacketMotion, Inc.", "PACKETMOTION" },
	{ 0x001370, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001371, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001372, "Dell Inc", "DELL" },
	{ 0x001373, "BLwave Electronics Co., Ltd", "BLWAVE-ELECTRONICS" },
	{ 0x001374, "Atheros Communications, Inc.", "ATHEROS-COMMUNICATION" },
	{ 0x001375, "American Security Products Co.", "AMERICAN-SECURITY-PRODUCTS" },
	{ 0x001376, "Tabor Electronics Ltd.", "TABOR-ELECTRONICS" },
	{ 0x001377, "Samsung Electronics CO., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x001378, "Qsan Technology, Inc.", "QSAN" },
	{ 0x001379, "PONDER INFORMATION INDUSTRIES LTD.", "PONDER-INFORMATION-INDUSTRIES" },
	{ 0x00137A, "Netvox Technology Co., Ltd.", "NETVOX" },
	{ 0x00137B, "Movon Corporation", "MOVON" },
	{ 0x00137C, "Kaicom co., Ltd.", "KAICOM" },
	{ 0x00137D, "Dynalab, Inc.", "DYNALAB" },
	{ 0x00137E, "CorEdge Networks, Inc.", "COREDGE-NETWORKS" },
	{ 0x00137F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001380, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001381, "CHIPS & Systems, Inc.", "CHIPS" },
	{ 0x001382, "Cetacea Networks Corporation", "CETACEA-NETWORKS" },
	{ 0x001383, "Application Technologies and Engineering Research Laboratory", "APPLICATION-TECHNOLOGIES-AND-ENGINEERING-RESEARCH-LABORATORY" },
	{ 0x001384, "Advanced Motion Controls", "ADVANCED-MOTION-CONTROLS" },
	{ 0x001385, "Add-On Technology Co., LTD.", "ADD-ON" },
	{ 0x001386, "ABB Inc./Totalflow", "ABB-/TOTALFLOW" },
	{ 0x001387, "27M Technologies AB", "27M-TECHNOLOGIES-AB" },
	{ 0x001388, "WiMedia Alliance", "WIMEDIA-ALLIANCE" },
	{ 0x001389, "Redes de Telefonía Móvil S.A.", "REDES-DE-TELEFONíA-MóVIL" },
	{ 0x00138A, "QINGDAO GOERTEK ELECTRONICS CO.,LTD.", "QINGDAO-GOERTEK-ELECTRONICS" },
	{ 0x00138B, "Phantom Technologies LLC", "PHANTOM-TECHNOLOGIES-LLC" },
	{ 0x00138C, "Kumyoung.Co.Ltd", "KUMYOUNG" },
	{ 0x00138D, "Kinghold", "KINGHOLD" },
	{ 0x00138E, "FOAB Elektronik AB", "FOAB-ELEKTRONIK-AB" },
	{ 0x00138F, "Asiarock Incorporation", "ASIAROCK-INCORPORATION" },
	{ 0x001390, "Termtek Computer Co., Ltd", "TERMTEK" },
	{ 0x001391, "OUEN CO.,LTD.", "OUEN" },
	{ 0x001392, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x001393, "Panta Systems, Inc.", "PANTA" },
	{ 0x001394, "Infohand Co.,Ltd", "INFOHAND" },
	{ 0x001395, "congatec AG", "CONGATEC" },
	{ 0x001396, "Acbel Polytech Inc.", "ACBEL-POLYTECH" },
	{ 0x001397, "Oracle Corporation", "ORACLE" },
	{ 0x001398, "TrafficSim Co.,Ltd", "TRAFFICSIM" },
	{ 0x001399, "STAC Corporation.", "STAC" },
	{ 0x00139A, "K-ubique ID Corp.", "K-UBIQUE-ID" },
	{ 0x00139B, "ioIMAGE Ltd.", "IOIMAGE" },
	{ 0x00139C, "Exavera Technologies, Inc.", "EXAVERA-TECHNOLOGIES" },
	{ 0x00139D, "Marvell Hispana S.L.", "MARVELL-HISPANA-S-L" },
	{ 0x00139E, "Ciara Technologies Inc.", "CIARA-TECHNOLOGIES" },
	{ 0x00139F, "Electronics Design Services, Co., Ltd.", "ELECTRONICS-DESIGN-SERVICES" },
	{ 0x0013A0, "ALGOSYSTEM Co., Ltd.", "ALGOSYSTEM" },
	{ 0x0013A1, "Crow Electronic Engeneering", "CROW-ENGENEERING" },
	{ 0x0013A2, "MaxStream, Inc", "MAXSTREAM" },
	{ 0x0013A3, "Siemens Com CPE Devices", "SIEMENS-COM-CPE-DEVICES" },
	{ 0x0013A4, "KeyEye Communications", "KEYEYE-COMMUNICATION" },
	{ 0x0013A5, "General Solutions, LTD.", "GENERAL-SOLUTIONS" },
	{ 0x0013A6, "Extricom Ltd", "EXTRICOM" },
	{ 0x0013A7, "BATTELLE MEMORIAL INSTITUTE", "BATTELLE-MEMORIAL-INSTITUTE" },
	{ 0x0013A8, "Tanisys Technology", "TANISYS" },
	{ 0x0013A9, "Sony Corporation", "SONY" },
	{ 0x0013AA, "ALS & TEC Ltd.", "ALS-TEC" },
	{ 0x0013AB, "Telemotive AG", "TELEMOTIVE" },
	{ 0x0013AC, "Sunmyung Electronics Co., LTD", "SUNMYUNG-ELECTRONICS" },
	{ 0x0013AD, "Sendo Ltd", "SENDO" },
	{ 0x0013AE, "Radiance Technologies, Inc.", "RADIANCE-TECHNOLOGIES" },
	{ 0x0013AF, "NUMA Technology,Inc.", "NUMA" },
	{ 0x0013B0, "Jablotron", "JABLOTRON" },
	{ 0x0013B1, "Intelligent Control Systems (Asia) Pte Ltd", "INTELLIGENT-CONTROL-ASIA-PTE" },
	{ 0x0013B2, "Carallon Limited", "CARALLON" },
	{ 0x0013B3, "Ecom Communications Technology Co., Ltd.", "ECOM-COMMUNICATION" },
	{ 0x0013B4, "Appear TV", "APPEAR-TV" },
	{ 0x0013B5, "Wavesat", "WAVESAT" },
	{ 0x0013B6, "Sling Media, Inc.", "SLING-MEDIA" },
	{ 0x0013B7, "Scantech ID", "SCANTECH-ID" },
	{ 0x0013B8, "RyCo Electronic Systems Limited", "RYCO" },
	{ 0x0013B9, "BM SPA", "BM-SPA" },
	{ 0x0013BA, "ReadyLinks Inc", "READYLINKS" },
	{ 0x0013BB, "Smartvue Corporation", "SMARTVUE" },
	{ 0x0013BC, "Artimi Ltd", "ARTIMI" },
	{ 0x0013BD, "HYMATOM SA", "HYMATOM" },
	{ 0x0013BE, "Virtual Conexions", "VIRTUAL-CONEXIONS" },
	{ 0x0013BF, "Media System Planning Corp.", "MEDIA-SYSTEM-PLANNING" },
	{ 0x0013C0, "Trix Tecnologia Ltda.", "TRIX-TECNOLOGIA-LTDA" },
	{ 0x0013C1, "Asoka USA Corporation", "ASOKA-USA" },
	{ 0x0013C2, "WACOM Co.,Ltd", "WACOM" },
	{ 0x0013C3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0013C4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0013C5, "LIGHTRON FIBER-OPTIC DEVICES INC.", "LIGHTRON-FIBER-OPTIC-DEVICES" },
	{ 0x0013C6, "OpenGear, Inc", "OPENGEAR" },
	{ 0x0013C7, "IONOS Co.,Ltd.", "IONOS" },
	{ 0x0013C8, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x0013C9, "Beyond Achieve Enterprises Ltd.", "BEYOND-ACHIEVE-ENTERPRISES" },
	{ 0x0013CA, "Pico Digital", "PICO-DIGITAL" },
	{ 0x0013CB, "Zenitel Norway AS", "ZENITEL-NORWAY-AS" },
	{ 0x0013CC, "Tall Maple Systems", "TALL-MAPLE" },
	{ 0x0013CD, "MTI co. LTD", "MTI" },
	{ 0x0013CE, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0013CF, "4Access Communications", "4ACCESS-COMMUNICATION" },
	{ 0x0013D0, "t+ Medical Ltd", "T+-MEDICAL" },
	{ 0x0013D1, "KIRK telecom A/S", "KIRK-TELECOM-A/S" },
	{ 0x0013D2, "PAGE IBERICA, S.A.", "PAGE-IBERICA" },
	{ 0x0013D3, "MICRO-STAR INTERNATIONAL CO., LTD.", "MICRO-STAR" },
	{ 0x0013D4, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x0013D5, "RuggedCom", "RUGGEDCOM" },
	{ 0x0013D6, "TII NETWORK TECHNOLOGIES, INC.", "TII-NETWORK-TECHNOLOGIES" },
	{ 0x0013D7, "SPIDCOM Technologies SA", "SPIDCOM-TECHNOLOGIES" },
	{ 0x0013D8, "Princeton Instruments", "PRINCETON-INSTRUMENTS" },
	{ 0x0013D9, "Matrix Product Development, Inc.", "MATRIX-PRODUCT-DEVELOPMENT" },
	{ 0x0013DA, "Diskware Co., Ltd", "DISKWARE" },
	{ 0x0013DB, "SHOEI Electric Co.,Ltd", "SHOEI-ELECTRIC" },
	{ 0x0013DC, "IBTEK INC.", "IBTEK" },
	{ 0x0013DD, "Abbott Diagnostics", "ABBOTT-DIAGNOSTICS" },
	{ 0x0013DE, "Adapt4, LLC", "ADAPT4-LLC" },
	{ 0x0013DF, "Ryvor Corp.", "RYVOR" },
	{ 0x0013E0, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x0013E1, "Iprobe AB", "IPROBE-AB" },
	{ 0x0013E2, "GeoVision Inc.", "GEOVISION" },
	{ 0x0013E3, "CoVi Technologies, Inc.", "COVI-TECHNOLOGIES" },
	{ 0x0013E4, "YANGJAE SYSTEMS CORP.", "YANGJAE" },
	{ 0x0013E5, "TENOSYS, INC.", "TENOSYS" },
	{ 0x0013E6, "Technolution", "TECHNOLUTION" },
	{ 0x0013E7, "Halcro", "HALCRO" },
	{ 0x0013E8, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0013E9, "VeriWave, Inc.", "VERIWAVE" },
	{ 0x0013EA, "Kamstrup A/S", "KAMSTRUP-A/S" },
	{ 0x0013EB, "Sysmaster Corporation", "SYSMASTER" },
	{ 0x0013EC, "Sunbay Software AG", "SUNBAY-SOFTWARE" },
	{ 0x0013ED, "PSIA", "PSIA" },
	{ 0x0013EE, "JBX Designs Inc.", "JBX-DESIGNS" },
	{ 0x0013EF, "Kingjon Digital Technology Co.,Ltd", "KINGJON-DIGITAL" },
	{ 0x0013F0, "Wavefront Semiconductor", "WAVEFRONT-SEMICONDUCTOR" },
	{ 0x0013F1, "AMOD Technology Co., Ltd.", "AMOD" },
	{ 0x0013F2, "Klas Ltd", "KLAS" },
	{ 0x0013F3, "Giga-byte Communications Inc.", "GIGA-BYTE-COMMUNICATION" },
	{ 0x0013F4, "Psitek (Pty) Ltd", "PSITEK-PTY" },
	{ 0x0013F5, "Akimbi Systems", "AKIMBI" },
	{ 0x0013F6, "Cintech", "CINTECH" },
	{ 0x0013F7, "SMC Networks, Inc.", "SMC-NETWORKS" },
	{ 0x0013F8, "Dex Security Solutions", "DEX-SECURITY-SOLUTIONS" },
	{ 0x0013F9, "Cavera Systems", "CAVERA" },
	{ 0x0013FA, "LifeSize Communications, Inc", "LIFESIZE-COMMUNICATION" },
	{ 0x0013FB, "RKC INSTRUMENT INC.", "RKC-INSTRUMENT" },
	{ 0x0013FC, "SiCortex, Inc", "SICORTEX" },
	{ 0x0013FD, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0013FE, "GRANDTEC ELECTRONIC CORP.", "GRANDTEC" },
	{ 0x0013FF, "Dage-MTI of MC, Inc.", "DAGE-MTI-OF-MC" },
	{ 0x001400, "MINERVA KOREA CO., LTD", "MINERVA-KOREA" },
	{ 0x001401, "Rivertree Networks Corp.", "RIVERTREE-NETWORKS" },
	{ 0x001402, "kk-electronic a/s", "KK--A/S" },
	{ 0x001403, "Renasis, LLC", "RENASIS-LLC" },
	{ 0x001404, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001405, "OpenIB, Inc.", "OPENIB" },
	{ 0x001406, "Go Networks", "GO-NETWORKS" },
	{ 0x001407, "Sperian Protection Instrumentation", "SPERIAN-PROTECTION-INSTRUMENTATION" },
	{ 0x001408, "Eka Systems Inc.", "EKA" },
	{ 0x001409, "MAGNETI MARELLI S.E. S.p.A.", "MAGNETI-MARELLI-S-E-S-P-A" },
	{ 0x00140A, "WEPIO Co., Ltd.", "WEPIO" },
	{ 0x00140B, "FIRST INTERNATIONAL COMPUTER, INC.", "FIRST" },
	{ 0x00140C, "GKB CCTV CO., LTD.", "GKB-CCTV" },
	{ 0x00140D, "Nortel", "NORTEL" },
	{ 0x00140E, "Nortel", "NORTEL" },
	{ 0x00140F, "Federal State Unitary Enterprise Leningrad R&D Institute of", "FEDERAL-STATE-UNITARY-LENINGRAD-R-D-INSTITUTE-OF" },
	{ 0x001410, "Suzhou Keda Technology CO.,Ltd", "SUZHOU-KEDA" },
	{ 0x001411, "Deutschmann Automation GmbH & Co. KG", "DEUTSCHMANN-AUTOMATION" },
	{ 0x001412, "S-TEC electronics AG", "S-TEC-ELECTRONICS" },
	{ 0x001413, "Trebing & Himstedt Prozeßautomation GmbH & Co. KG", "TREBING-HIMSTEDT-PROZEßAUTOMATION" },
	{ 0x001414, "Jumpnode Systems LLC.", "JUMPNODE-LLC" },
	{ 0x001415, "Intec Automation Inc.", "INTEC-AUTOMATION" },
	{ 0x001416, "Scosche Industries, Inc.", "SCOSCHE-INDUSTRIES" },
	{ 0x001417, "RSE Informations Technologie GmbH", "RSE-INFORMATIONS-TECHNOLOGIE" },
	{ 0x001418, "C4Line", "C4LINE" },
	{ 0x001419, "SIDSA", "SIDSA" },
	{ 0x00141A, "DEICY CORPORATION", "DEICY" },
	{ 0x00141B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00141C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00141D, "LTi DRIVES GmbH", "LTI-DRIVES" },
	{ 0x00141E, "P.A. Semi, Inc.", "P-A-SEMI" },
	{ 0x00141F, "SunKwang Electronics Co., Ltd", "SUNKWANG-ELECTRONICS" },
	{ 0x001420, "G-Links networking company", "G-LINKS-NETWORKING-COMPANY" },
	{ 0x001421, "Total Wireless Technologies Pte. Ltd.", "TOTAL-WIRELESS-TECHNOLOGIES-PTE" },
	{ 0x001422, "Dell Inc", "DELL" },
	{ 0x001423, "J-S Co. NEUROCOM", "J-S-NEUROCOM" },
	{ 0x001424, "Merry Electrics CO., LTD.", "MERRY-ELECTRICS" },
	{ 0x001425, "Galactic Computing Corp.", "GALACTIC-COMPUTING" },
	{ 0x001426, "NL Technology", "NL" },
	{ 0x001427, "JazzMutant", "JAZZMUTANT" },
	{ 0x001428, "Vocollect, Inc", "VOCOLLECT" },
	{ 0x001429, "V Center Technologies Co., Ltd.", "V-CENTER-TECHNOLOGIES" },
	{ 0x00142A, "Elitegroup Computer System Co., Ltd", "ELITEGROUP-SYSTEM" },
	{ 0x00142B, "Edata Communication Inc.", "EDATA-COMMUNICATION" },
	{ 0x00142C, "Koncept International, Inc.", "KONCEPT" },
	{ 0x00142D, "Toradex AG", "TORADEX" },
	{ 0x00142E, "77 Elektronika Kft.", "77-ELEKTRONIKA-KFT" },
	{ 0x00142F, "WildPackets", "WILDPACKETS" },
	{ 0x001430, "ViPowER, Inc", "VIPOWER" },
	{ 0x001431, "PDL Electronics Ltd", "PDL-ELECTRONICS" },
	{ 0x001432, "Tarallax Wireless, Inc.", "TARALLAX-WIRELESS" },
	{ 0x001433, "Empower Technologies(Canada) Inc.", "EMPOWER-TECHNOLOGIES-CANADA" },
	{ 0x001434, "Keri Systems, Inc", "KERI" },
	{ 0x001435, "CityCom Corp.", "CITYCOM" },
	{ 0x001436, "Qwerty Elektronik AB", "QWERTY-ELEKTRONIK-AB" },
	{ 0x001437, "GSTeletech Co.,Ltd.", "GSTELETECH" },
	{ 0x001438, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001439, "Blonder Tongue Laboratories, Inc.", "BLONDER-TONGUE-LABORATORIES" },
	{ 0x00143A, "RAYTALK INTERNATIONAL SRL", "RAYTALK-SRL" },
	{ 0x00143B, "Sensovation AG", "SENSOVATION" },
	{ 0x00143C, "Rheinmetall Canada Inc.", "RHEINMETALL-CANADA" },
	{ 0x00143D, "Aevoe Inc.", "AEVOE" },
	{ 0x00143E, "AirLink Communications, Inc.", "AIRLINK-COMMUNICATION" },
	{ 0x00143F, "Hotway Technology Corporation", "HOTWAY" },
	{ 0x001440, "ATOMIC Corporation", "ATOMIC" },
	{ 0x001441, "Innovation Sound Technology Co., LTD.", "INNOVATION-SOUND" },
	{ 0x001442, "ATTO CORPORATION", "ATTO" },
	{ 0x001443, "Consultronics Europe Ltd", "CONSULTRONICS-EUROPE" },
	{ 0x001444, "Grundfos Holding", "GRUNDFOS-HOLDING" },
	{ 0x001445, "Telefon-Gradnja d.o.o.", "TELEFON-GRADNJA-D-O-O" },
	{ 0x001446, "SuperVision Solutions LLC", "SUPERVISION-SOLUTIONS-LLC" },
	{ 0x001447, "BOAZ Inc.", "BOAZ" },
	{ 0x001448, "Inventec Multimedia & Telecom Corporation", "INVENTEC-MULTIMEDIA-TELECOM" },
	{ 0x001449, "Sichuan Changhong Electric Ltd.", "SICHUAN-CHANGHONG-ELECTRIC" },
	{ 0x00144A, "Taiwan Thick-Film Ind. Corp.", "TAIWAN-THICK-FILM-IND" },
	{ 0x00144B, "Hifn, Inc.", "HIFN" },
	{ 0x00144C, "General Meters Corp.", "GENERAL-METERS" },
	{ 0x00144D, "Intelligent Systems", "INTELLIGENT" },
	{ 0x00144E, "SRISA", "SRISA" },
	{ 0x00144F, "Oracle Corporation", "ORACLE" },
	{ 0x001450, "Heim Systems GmbH", "HEIM" },
	{ 0x001451, "Apple", "APPLE" },
	{ 0x001452, "CALCULEX,INC.", "CALCULEX" },
	{ 0x001453, "ADVANTECH TECHNOLOGIES CO.,LTD", "ADVANTECH-TECHNOLOGIES" },
	{ 0x001454, "Symwave", "SYMWAVE" },
	{ 0x001455, "Coder Electronics Corporation", "CODER-ELECTRONICS" },
	{ 0x001456, "Edge Products", "EDGE-PRODUCTS" },
	{ 0x001457, "T-VIPS AS", "T-VIPS-AS" },
	{ 0x001458, "HS Automatic ApS", "HS-AUTOMATIC-APS" },
	{ 0x001459, "Moram Co., Ltd.", "MORAM" },
	{ 0x00145A, "Neratec Solutions AG", "NERATEC-SOLUTIONS" },
	{ 0x00145B, "SeekerNet Inc.", "SEEKERNET" },
	{ 0x00145C, "Intronics B.V.", "INTRONICS" },
	{ 0x00145D, "WJ Communications, Inc.", "WJ-COMMUNICATION" },
	{ 0x00145E, "IBM Corp", "IBM" },
	{ 0x00145F, "ADITEC CO. LTD", "ADITEC" },
	{ 0x001460, "Kyocera Wireless Corp.", "KYOCERA-WIRELESS" },
	{ 0x001461, "CORONA CORPORATION", "CORONA" },
	{ 0x001462, "Digiwell Technology, inc", "DIGIWELL" },
	{ 0x001463, "IDCS N.V.", "IDCS-N-V" },
	{ 0x001464, "Cryptosoft", "CRYPTOSOFT" },
	{ 0x001465, "Novo Nordisk A/S", "NOVO-NORDISK-A/S" },
	{ 0x001466, "Kleinhenz Elektronik GmbH", "KLEINHENZ-ELEKTRONIK" },
	{ 0x001467, "ArrowSpan Inc.", "ARROWSPAN" },
	{ 0x001468, "CelPlan International, Inc.", "CELPLAN" },
	{ 0x001469, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00146A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00146B, "Anagran, Inc.", "ANAGRAN" },
	{ 0x00146C, "Netgear Inc.", "NETGEAR" },
	{ 0x00146D, "RF Technologies", "RF-TECHNOLOGIES" },
	{ 0x00146E, "H. Stoll GmbH & Co. KG", "H-STOLL" },
	{ 0x00146F, "Kohler Co", "KOHLER" },
	{ 0x001470, "Prokom Software SA", "PROKOM-SOFTWARE" },
	{ 0x001471, "Eastern Asia Technology Limited", "EASTERN-ASIA" },
	{ 0x001472, "China Broadband Wireless IP Standard Group", "CHINA-BROADBAND-WIRELESS-IP-STANDARD-GROUP" },
	{ 0x001473, "Bookham Inc", "BOOKHAM" },
	{ 0x001474, "K40 Electronics", "K40-ELECTRONICS" },
	{ 0x001475, "Wiline Networks, Inc.", "WILINE-NETWORKS" },
	{ 0x001476, "MultiCom Industries Limited", "MULTICOM-INDUSTRIES" },
	{ 0x001477, "Nertec Inc.", "NERTEC" },
	{ 0x001478, "ShenZhen TP-LINK Technologies Co., Ltd.", "SHENZHEN-TP-LINK-TECHNOLOGIES" },
	{ 0x001479, "NEC Magnus Communications,Ltd.", "NEC-MAGNUS-COMMUNICATION" },
	{ 0x00147A, "Eubus GmbH", "EUBUS" },
	{ 0x00147B, "Iteris, Inc.", "ITERIS" },
	{ 0x00147C, "3Com Ltd", "3COM" },
	{ 0x00147D, "Aeon Digital International", "AEON-DIGITAL" },
	{ 0x00147E, "InnerWireless", "INNERWIRELESS" },
	{ 0x00147F, "Thomson Telecom Belgium", "THOMSON-TELECOM-BELGIUM" },
	{ 0x001480, "Hitachi-LG Data Storage Korea, Inc", "HITACHI-LG-DATA-STORAGE-KOREA" },
	{ 0x001481, "Multilink Inc", "MULTILINK" },
	{ 0x001482, "Aurora Networks", "AURORA-NETWORKS" },
	{ 0x001483, "eXS Inc.", "EXS" },
	{ 0x001484, "Cermate Technologies Inc.", "CERMATE-TECHNOLOGIES" },
	{ 0x001485, "Giga-Byte", "GIGA-BYTE" },
	{ 0x001486, "Echo Digital Audio Corporation", "ECHO-DIGITAL-AUDIO" },
	{ 0x001487, "American Technology Integrators", "AMERICAN-INTEGRATORS" },
	{ 0x001488, "Akorri", "AKORRI" },
	{ 0x001489, "B15402100 - JANDEI, S.L.", "B15402100---JANDEI-S-L" },
	{ 0x00148A, "Elin Ebg Traction Gmbh", "ELIN-EBG-TRACTION" },
	{ 0x00148B, "Globo Electronic GmbH & Co. KG", "GLOBO" },
	{ 0x00148C, "Fortress Technologies", "FORTRESS-TECHNOLOGIES" },
	{ 0x00148D, "Cubic Defense Simulation Systems", "CUBIC-DEFENSE-SIMULATION" },
	{ 0x00148E, "Tele Power Inc.", "TELE-POWER" },
	{ 0x00148F, "Protronic (Far East) Ltd.", "PROTRONIC-FAR-EAST" },
	{ 0x001490, "ASP Corporation", "ASP" },
	{ 0x001491, "Daniels Electronics Ltd. dbo Codan Rado Communications", "DANIELS-ELECTRONICS-DBO-CODAN-RADO-COMMUNICATION" },
	{ 0x001492, "Liteon, Mobile Media Solution SBU", "LITEON-MOBILE-MEDIA-SOLUTION-SBU" },
	{ 0x001493, "Systimax Solutions", "SYSTIMAX-SOLUTIONS" },
	{ 0x001494, "ESU AG", "ESU" },
	{ 0x001495, "2Wire, Inc.", "2WIRE" },
	{ 0x001496, "Phonic Corp.", "PHONIC" },
	{ 0x001497, "ZHIYUAN Eletronics co.,ltd.", "ZHIYUAN-ELETRONICS" },
	{ 0x001498, "Viking Design Technology", "VIKING-DESIGN" },
	{ 0x001499, "Helicomm Inc", "HELICOMM" },
	{ 0x00149A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00149B, "Nokota Communications, LLC", "NOKOTA-COMMUNICATION-LLC" },
	{ 0x00149C, "HF Company", "HF-COMPANY" },
	{ 0x00149D, "Sound ID Inc.", "SOUND-ID" },
	{ 0x00149E, "UbONE Co., Ltd", "UBONE" },
	{ 0x00149F, "System and Chips, Inc.", "SYSTEM-AND-CHIPS" },
	{ 0x0014A0, "Accsense, Inc.", "ACCSENSE" },
	{ 0x0014A1, "Synchronous Communication Corp", "SYNCHRONOUS-COMMUNICATION" },
	{ 0x0014A2, "Core Micro Systems Inc.", "CORE-MICRO" },
	{ 0x0014A3, "Vitelec BV", "VITELEC" },
	{ 0x0014A4, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x0014A5, "Gemtek Technology Co., Ltd.", "GEMTEK" },
	{ 0x0014A6, "Teranetics, Inc.", "TERANETICS" },
	{ 0x0014A7, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0014A8, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0014A9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0014AA, "Ashly Audio, Inc.", "ASHLY-AUDIO" },
	{ 0x0014AB, "Senhai Electronic Technology Co., Ltd.", "SENHAI" },
	{ 0x0014AC, "Bountiful WiFi", "BOUNTIFUL-WIFI" },
	{ 0x0014AD, "Gassner Wiege- und Meßtechnik GmbH", "GASSNER-WIEGE--UND-MEßTECHNIK" },
	{ 0x0014AE, "Wizlogics Co., Ltd.", "WIZLOGICS" },
	{ 0x0014AF, "Datasym POS Inc.", "DATASYM-POS" },
	{ 0x0014B0, "Naeil Community", "NAEIL-COMMUNITY" },
	{ 0x0014B1, "Axell Wireless Limited", "AXELL-WIRELESS" },
	{ 0x0014B2, "mCubelogics Corporation", "MCUBELOGICS" },
	{ 0x0014B3, "CoreStar International Corp", "CORESTAR" },
	{ 0x0014B4, "General Dynamics United Kingdom Ltd", "GENERAL-DYNAMICS-UNITED-KINGDOM" },
	{ 0x0014B5, "PHYSIOMETRIX,INC", "PHYSIOMETRIX" },
	{ 0x0014B6, "Enswer Technology Inc.", "ENSWER" },
	{ 0x0014B7, "AR Infotek Inc.", "AR-INFOTEK" },
	{ 0x0014B8, "Hill-Rom", "HILL-ROM" },
	{ 0x0014B9, "MSTAR SEMICONDUCTOR", "MSTAR-SEMICONDUCTOR" },
	{ 0x0014BA, "Carvers SA de CV", "CARVERS-DE-CV" },
	{ 0x0014BB, "Open Interface North America", "OPEN-INTERFACE-NORTH-AMERICA" },
	{ 0x0014BC, "SYNECTIC TELECOM EXPORTS PVT. LTD.", "SYNECTIC-TELECOM-EXPORTS-PVT" },
	{ 0x0014BD, "incNETWORKS, Inc", "INCNETWORKS" },
	{ 0x0014BE, "Wink communication technology CO.LTD", "WINK-COMMUNICATION" },
	{ 0x0014BF, "Cisco-Linksys LLC", "CISCO-LINKSYS-LLC" },
	{ 0x0014C0, "Symstream Technology Group Ltd", "SYMSTREAM-GROUP" },
	{ 0x0014C1, "U.S. Robotics Corporation", "U-S-ROBOTICS" },
	{ 0x0014C2, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0014C3, "Seagate Technology", "SEAGATE" },
	{ 0x0014C4, "Vitelcom Mobile Technology", "VITELCOM-MOBILE" },
	{ 0x0014C5, "Alive Technologies Pty Ltd", "ALIVE-TECHNOLOGIES-PTY" },
	{ 0x0014C6, "Quixant Ltd", "QUIXANT" },
	{ 0x0014C7, "Nortel", "NORTEL" },
	{ 0x0014C8, "Contemporary Research Corp", "CONTEMPORARY-RESEARCH" },
	{ 0x0014C9, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x0014CA, "Key Radio Systems Limited", "KEY-RADIO" },
	{ 0x0014CB, "LifeSync Corporation", "LIFESYNC" },
	{ 0x0014CC, "Zetec, Inc.", "ZETEC" },
	{ 0x0014CD, "DigitalZone Co., Ltd.", "DIGITALZONE" },
	{ 0x0014CE, "NF CORPORATION", "NF" },
	{ 0x0014CF, "INVISIO Communications", "INVISIO-COMMUNICATION" },
	{ 0x0014D0, "BTI Systems Inc.", "BTI" },
	{ 0x0014D1, "TRENDnet", "TRENDNET" },
	{ 0x0014D2, "Kyuden Technosystems Corporation", "KYUDEN-TECHNOSYSTEMS" },
	{ 0x0014D3, "SEPSA", "SEPSA" },
	{ 0x0014D4, "K Technology Corporation", "K" },
	{ 0x0014D5, "Datang Telecom Technology CO. , LCD,Optical Communication Br", "DATANG-TELECOM-LCD-OPTICAL-COMMUNICATION-BR" },
	{ 0x0014D6, "Jeongmin Electronics Co.,Ltd.", "JEONGMIN-ELECTRONICS" },
	{ 0x0014D7, "Datastore Technology Corp", "DATASTORE" },
	{ 0x0014D8, "bio-logic SA", "BIO-LOGIC" },
	{ 0x0014D9, "IP Fabrics, Inc.", "IP-FABRICS" },
	{ 0x0014DA, "Huntleigh Healthcare", "HUNTLEIGH-HEALTHCARE" },
	{ 0x0014DB, "Elma Trenew Electronic GmbH", "ELMA-TRENEW" },
	{ 0x0014DC, "Communication System Design & Manufacturing (CSDM)", "COMMUNICATION-SYSTEM-DESIGN-MANUFACTURING-CSDM" },
	{ 0x0014DD, "Covergence Inc.", "COVERGENCE" },
	{ 0x0014DE, "Sage Instruments Inc.", "SAGE-INSTRUMENTS" },
	{ 0x0014DF, "HI-P Tech Corporation", "HI-P-TECH" },
	{ 0x0014E0, "LET'S Corporation", "LET-S" },
	{ 0x0014E1, "Data Display AG", "DATA-DISPLAY" },
	{ 0x0014E2, "datacom systems inc.", "DATACOM" },
	{ 0x0014E3, "mm-lab GmbH", "MM-LAB" },
	{ 0x0014E4, "infinias, LLC", "INFINIAS-LLC" },
	{ 0x0014E5, "Alticast", "ALTICAST" },
	{ 0x0014E6, "AIM Infrarotmodule GmbH", "AIM-INFRAROTMODULE" },
	{ 0x0014E7, "Stolinx,. Inc", "STOLINX" },
	{ 0x0014E8, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0014E9, "Nortech International", "NORTECH" },
	{ 0x0014EA, "S Digm Inc. (Safe Paradigm Inc.)", "S-DIGM-SAFE-PARADIGM" },
	{ 0x0014EB, "AwarePoint Corporation", "AWAREPOINT" },
	{ 0x0014EC, "Acro Telecom", "ACRO-TELECOM" },
	{ 0x0014ED, "Airak, Inc.", "AIRAK" },
	{ 0x0014EE, "Western Digital Technologies, Inc.", "WESTERN-DIGITAL-TECHNOLOGIES" },
	{ 0x0014EF, "TZero Technologies, Inc.", "TZERO-TECHNOLOGIES" },
	{ 0x0014F0, "Business Security OL AB", "BUSINESS-SECURITY-OL-AB" },
	{ 0x0014F1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0014F2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0014F3, "ViXS Systems Inc", "VIXS" },
	{ 0x0014F4, "DekTec Digital Video B.V.", "DEKTEC-DIGITAL-VIDEO" },
	{ 0x0014F5, "OSI Security Devices", "OSI-SECURITY-DEVICES" },
	{ 0x0014F6, "Juniper Networks, Inc.", "JUNIPER-NETWORKS" },
	{ 0x0014F7, "CREVIS Co., LTD", "CREVIS" },
	{ 0x0014F8, "Scientific Atlanta", "SCIENTIFIC-ATLANTA" },
	{ 0x0014F9, "Vantage Controls", "VANTAGE-CONTROLS" },
	{ 0x0014FA, "AsGa S.A.", "ASGA" },
	{ 0x0014FB, "Technical Solutions Inc.", "TECHNICAL-SOLUTIONS" },
	{ 0x0014FC, "Extandon, Inc.", "EXTANDON" },
	{ 0x0014FD, "Thecus Technology Corp.", "THECUS" },
	{ 0x0014FE, "Artech Electronics", "ARTECH-ELECTRONICS" },
	{ 0x0014FF, "Precise Automation, Inc.", "PRECISE-AUTOMATION" },
	{ 0x001500, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001501, "LexBox", "LEXBOX" },
	{ 0x001502, "BETA tech", "BETA-TECH" },
	{ 0x001503, "PROFIcomms s.r.o.", "PROFICOMMS-S-R-O" },
	{ 0x001504, "GAME PLUS CO., LTD.", "GAME-PLUS" },
	{ 0x001505, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x001506, "Neo Photonics", "NEO-PHOTONICS" },
	{ 0x001507, "Renaissance Learning Inc", "RENAISSANCE-LEARNING" },
	{ 0x001508, "Global Target Enterprise Inc", "GLOBAL-TARGET" },
	{ 0x001509, "Plus Technology Co., Ltd", "PLUS" },
	{ 0x00150A, "Sonoa Systems, Inc", "SONOA" },
	{ 0x00150B, "SAGE INFOTECH LTD.", "SAGE-INFOTECH" },
	{ 0x00150C, "AVM GmbH", "AVM" },
	{ 0x00150D, "Hoana Medical, Inc.", "HOANA-MEDICAL" },
	{ 0x00150E, "OPENBRAIN TECHNOLOGIES CO., LTD.", "OPENBRAIN-TECHNOLOGIES" },
	{ 0x00150F, "mingjong", "MINGJONG" },
	{ 0x001510, "Techsphere Co., Ltd", "TECHSPHERE" },
	{ 0x001511, "Data Center Systems", "DATA-CENTER" },
	{ 0x001512, "Zurich University of Applied Sciences", "ZURICH-UNIVERSITY-OF-APPLIED-SCIENCES" },
	{ 0x001513, "EFS sas", "EFS-SAS" },
	{ 0x001514, "Hu Zhou NAVA Networks&Electronics Ltd.", "HU-ZHOU-NAVA-NETWORKS-ELECTRONICS" },
	{ 0x001515, "Leipold+Co.GmbH", "LEIPOLD+" },
	{ 0x001516, "URIEL SYSTEMS INC.", "URIEL" },
	{ 0x001517, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001518, "Shenzhen 10MOONS Technology Development CO.,Ltd", "SHENZHEN-10MOONS-DEVELOPMENT" },
	{ 0x001519, "StoreAge Networking Technologies", "STOREAGE-NETWORKING-TECHNOLOGIES" },
	{ 0x00151A, "Hunter Engineering Company", "HUNTER-ENGINEERING-COMPANY" },
	{ 0x00151B, "Isilon Systems Inc.", "ISILON" },
	{ 0x00151C, "LENECO", "LENECO" },
	{ 0x00151D, "M2I CORPORATION", "M2I" },
	{ 0x00151E, "Ethernet Powerlink Standardization Group (EPSG)", "ETHERNET-POWERLINK-STANDARDIZATION-GROUP-EPSG" },
	{ 0x00151F, "Multivision Intelligent Surveillance (Hong Kong) Ltd", "MULTIVISION-INTELLIGENT-SURVEILLANCE-HONG-KONG" },
	{ 0x001520, "Radiocrafts AS", "RADIOCRAFTS-AS" },
	{ 0x001521, "Horoquartz", "HOROQUARTZ" },
	{ 0x001522, "Dea Security", "DEA-SECURITY" },
	{ 0x001523, "Meteor Communications Corporation", "METEOR-COMMUNICATION" },
	{ 0x001524, "Numatics, Inc.", "NUMATICS" },
	{ 0x001525, "Chamberlain Access Solutions", "CHAMBERLAIN-ACCESS-SOLUTIONS" },
	{ 0x001526, "Remote Technologies Inc", "REMOTE-TECHNOLOGIES" },
	{ 0x001527, "Balboa Instruments", "BALBOA-INSTRUMENTS" },
	{ 0x001528, "Beacon Medical Products LLC d.b.a. BeaconMedaes", "BEACON-MEDICAL-PRODUCTS-LLC-D-B-A-BEACONMEDAES" },
	{ 0x001529, "N3 Corporation", "N3" },
	{ 0x00152A, "Nokia GmbH", "NOKIA" },
	{ 0x00152B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00152C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00152D, "TenX Networks, LLC", "TENX-NETWORKS-LLC" },
	{ 0x00152E, "PacketHop, Inc.", "PACKETHOP" },
	{ 0x00152F, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001530, "EMC Corporation", "EMC" },
	{ 0x001531, "KOCOM", "KOCOM" },
	{ 0x001532, "Consumer Technologies Group, LLC", "CONSUMER-TECHNOLOGIES-GROUP-LLC" },
	{ 0x001533, "NADAM.CO.,LTD", "NADAM" },
	{ 0x001534, "A Beltrónica-Companhia de Comunicações, Lda", "A-BELTRóNICA-COMPANHIA-DE-COMUNICAçõES-LDA" },
	{ 0x001535, "OTE Spa", "OTE-SPA" },
	{ 0x001536, "Powertech co.,Ltd", "POWERTECH" },
	{ 0x001537, "Ventus Networks", "VENTUS-NETWORKS" },
	{ 0x001538, "RFID, Inc.", "RFID" },
	{ 0x001539, "Technodrive SRL", "TECHNODRIVE-SRL" },
	{ 0x00153A, "Shenzhen Syscan Technology Co.,Ltd.", "SHENZHEN-SYSCAN" },
	{ 0x00153B, "EMH metering GmbH & Co. KG", "EMH-METERING" },
	{ 0x00153C, "Kprotech Co., Ltd.", "KPROTECH" },
	{ 0x00153D, "ELIM PRODUCT CO.", "ELIM-PRODUCT" },
	{ 0x00153E, "Q-Matic Sweden AB", "Q-MATIC-SWEDEN-AB" },
	{ 0x00153F, "Alcatel Alenia Space Italia", "ALCATEL-ALENIA-SPACE-ITALIA" },
	{ 0x001540, "Nortel", "NORTEL" },
	{ 0x001541, "StrataLight Communications, Inc.", "STRATALIGHT-COMMUNICATION" },
	{ 0x001542, "MICROHARD S.R.L.", "MICROHARD-S-R-L" },
	{ 0x001543, "Aberdeen Test Center", "ABERDEEN-TEST-CENTER" },
	{ 0x001544, "coM.s.a.t. AG", "COM-T" },
	{ 0x001545, "SEECODE Co., Ltd.", "SEECODE" },
	{ 0x001546, "ITG Worldwide Sdn Bhd", "ITG-WORLDWIDE-SDN-BHD" },
	{ 0x001547, "AiZen Solutions Inc.", "AIZEN-SOLUTIONS" },
	{ 0x001548, "CUBE TECHNOLOGIES", "CUBE-TECHNOLOGIES" },
	{ 0x001549, "Dixtal Biomedica Ind. Com. Ltda", "DIXTAL-BIOMEDICA-IND-COM-LTDA" },
	{ 0x00154A, "WANSHIH ELECTRONIC CO., LTD", "WANSHIH" },
	{ 0x00154B, "Wonde Proud Technology Co., Ltd", "WONDE-PROUD" },
	{ 0x00154C, "Saunders Electronics", "SAUNDERS-ELECTRONICS" },
	{ 0x00154D, "Netronome Systems, Inc.", "NETRONOME" },
	{ 0x00154E, "IEC", "IEC" },
	{ 0x00154F, "one RF Technology", "ONE-RF" },
	{ 0x001550, "Nits Technology Inc", "NITS" },
	{ 0x001551, "RadioPulse Inc.", "RADIOPULSE" },
	{ 0x001552, "Wi-Gear Inc.", "WI-GEAR" },
	{ 0x001553, "Cytyc Corporation", "CYTYC" },
	{ 0x001554, "Atalum Wireless S.A.", "ATALUM-WIRELESS" },
	{ 0x001555, "DFM GmbH", "DFM" },
	{ 0x001556, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x001557, "Olivetti", "OLIVETTI" },
	{ 0x001558, "FOXCONN", "FOXCONN" },
	{ 0x001559, "Securaplane Technologies, Inc.", "SECURAPLANE-TECHNOLOGIES" },
	{ 0x00155A, "DAINIPPON PHARMACEUTICAL CO., LTD.", "DAINIPPON-PHARMACEUTICAL" },
	{ 0x00155B, "Sampo Corporation", "SAMPO" },
	{ 0x00155C, "Dresser Wayne", "DRESSER-WAYNE" },
	{ 0x00155D, "Microsoft Corporation", "MICROSOFT" },
	{ 0x00155E, "Morgan Stanley", "MORGAN-STANLEY" },
	{ 0x00155F, "GreenPeak Technologies", "GREENPEAK-TECHNOLOGIES" },
	{ 0x001560, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001561, "JJPlus Corporation", "JJPLUS" },
	{ 0x001562, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001563, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001564, "BEHRINGER Spezielle Studiotechnik GmbH", "BEHRINGER-SPEZIELLE-STUDIOTECHNIK" },
	{ 0x001565, "XIAMEN YEALINK NETWORK TECHNOLOGY CO.,LTD", "XIAMEN-YEALINK-NETWORK" },
	{ 0x001566, "A-First Technology Co., Ltd.", "A-FIRST" },
	{ 0x001567, "RADWIN Inc.", "RADWIN" },
	{ 0x001568, "Dilithium Networks", "DILITHIUM-NETWORKS" },
	{ 0x001569, "PECO II, Inc.", "PECO-II" },
	{ 0x00156A, "DG2L Technologies Pvt. Ltd.", "DG2L-TECHNOLOGIES-PVT" },
	{ 0x00156B, "Perfisans Networks Corp.", "PERFISANS-NETWORKS" },
	{ 0x00156C, "SANE SYSTEM CO., LTD", "SANE-SYSTEM" },
	{ 0x00156D, "Ubiquiti Networks Inc.", "UBIQUITI-NETWORKS" },
	{ 0x00156E, "A. W. Communication Systems Ltd", "A-W-COMMUNICATION" },
	{ 0x00156F, "Xiranet Communications GmbH", "XIRANET-COMMUNICATION" },
	{ 0x001570, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0x001571, "Nolan Systems", "NOLAN" },
	{ 0x001572, "Red-Lemon", "RED-LEMON" },
	{ 0x001573, "NewSoft Technology Corporation", "NEWSOFT" },
	{ 0x001574, "Horizon Semiconductors Ltd.", "HORIZON-SEMICONDUCTORS" },
	{ 0x001575, "Nevis Networks Inc.", "NEVIS-NETWORKS" },
	{ 0x001576, "LABiTec - Labor Biomedical Technologies GmbH", "LABITEC---LABOR-BIOMEDICAL-TECHNOLOGIES" },
	{ 0x001577, "Allied Telesis", "ALLIED-TELESIS" },
	{ 0x001578, "Audio / Video Innovations", "AUDIO-/-VIDEO-INNOVATIONS" },
	{ 0x001579, "Lunatone Industrielle Elektronik GmbH", "LUNATONE-INDUSTRIELLE-ELEKTRONIK" },
	{ 0x00157A, "Telefin S.p.A.", "TELEFIN-S-P-A" },
	{ 0x00157B, "Leuze electronic GmbH + Co. KG", "LEUZE-+" },
	{ 0x00157C, "Dave Networks, Inc.", "DAVE-NETWORKS" },
	{ 0x00157D, "POSDATA CO., LTD.", "POSDATA" },
	{ 0x00157E, "Weidmüller Interface GmbH & Co. KG", "WEIDMüLLER-INTERFACE" },
	{ 0x00157F, "ChuanG International Holding CO.,LTD.", "CHUANG-HOLDING" },
	{ 0x001580, "U-WAY CORPORATION", "U-WAY" },
	{ 0x001581, "MAKUS Inc.", "MAKUS" },
	{ 0x001582, "Pulse Eight Limited", "PULSE-EIGHT" },
	{ 0x001583, "IVT corporation", "IVT" },
	{ 0x001584, "Schenck Process GmbH", "SCHENCK-PROCESS" },
	{ 0x001585, "Aonvision Technolopy Corp.", "AONVISION-TECHNOLOPY" },
	{ 0x001586, "Xiamen Overseas Chinese Electronic Co., Ltd.", "XIAMEN-OVERSEAS-CHINESE" },
	{ 0x001587, "Takenaka Seisakusho Co.,Ltd", "TAKENAKA-SEISAKUSHO" },
	{ 0x001588, "Salutica Allied Solutions Sdn Bhd", "SALUTICA-ALLIED-SOLUTIONS-SDN-BHD" },
	{ 0x001589, "D-MAX Technology Co.,Ltd", "D-MAX" },
	{ 0x00158A, "SURECOM Technology Corp.", "SURECOM" },
	{ 0x00158B, "Park Air Systems Ltd", "PARK-AIR" },
	{ 0x00158C, "Liab ApS", "LIAB-APS" },
	{ 0x00158D, "Jennic Ltd", "JENNIC" },
	{ 0x00158E, "Plustek.INC", "PLUSTEK" },
	{ 0x00158F, "NTT Advanced Technology Corporation", "NTT-ADVANCED" },
	{ 0x001590, "Hectronic GmbH", "HECTRONIC" },
	{ 0x001591, "RLW Inc.", "RLW" },
	{ 0x001592, "Facom UK Ltd (Melksham)", "FACOM-UK-MELKSHAM" },
	{ 0x001593, "U4EA Technologies Inc.", "U4EA-TECHNOLOGIES" },
	{ 0x001594, "BIXOLON CO.,LTD", "BIXOLON" },
	{ 0x001595, "Quester Tangent Corporation", "QUESTER-TANGENT" },
	{ 0x001596, "ARRIS International", "ARRIS" },
	{ 0x001597, "AETA AUDIO SYSTEMS", "AETA-AUDIO" },
	{ 0x001598, "Kolektor group", "KOLEKTOR-GROUP" },
	{ 0x001599, "Samsung Electronics Co., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x00159A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00159B, "Nortel", "NORTEL" },
	{ 0x00159C, "B-KYUNG SYSTEM Co.,Ltd.", "B-KYUNG-SYSTEM" },
	{ 0x00159D, "Tripp Lite", "TRIPP-LITE" },
	{ 0x00159E, "Mad Catz Interactive Inc", "MAD-CATZ-INTERACTIVE" },
	{ 0x00159F, "Terascala, Inc.", "TERASCALA" },
	{ 0x0015A0, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0015A1, "ECA-SINTERS", "ECA-SINTERS" },
	{ 0x0015A2, "ARRIS International", "ARRIS" },
	{ 0x0015A3, "ARRIS International", "ARRIS" },
	{ 0x0015A4, "ARRIS International", "ARRIS" },
	{ 0x0015A5, "DCI Co., Ltd.", "DCI" },
	{ 0x0015A6, "Digital Electronics Products Ltd.", "DIGITAL-ELECTRONICS-PRODUCTS" },
	{ 0x0015A7, "Robatech AG", "ROBATECH" },
	{ 0x0015A8, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0015A9, "KWANG WOO I&C CO.,LTD", "KWANG-WOO-I-C" },
	{ 0x0015AA, "Rextechnik International Co.,", "REXTECHNIK" },
	{ 0x0015AB, "PRO CO SOUND INC", "PRO-SOUND" },
	{ 0x0015AC, "Capelon AB", "CAPELON-AB" },
	{ 0x0015AD, "Accedian Networks", "ACCEDIAN-NETWORKS" },
	{ 0x0015AE, "kyung il", "KYUNG-IL" },
	{ 0x0015AF, "AzureWave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x0015B0, "AUTOTELENET CO.,LTD", "AUTOTELENET" },
	{ 0x0015B1, "Ambient Corporation", "AMBIENT" },
	{ 0x0015B2, "Advanced Industrial Computer, Inc.", "ADVANCED-INDUSTRIAL" },
	{ 0x0015B3, "Caretech AB", "CARETECH-AB" },
	{ 0x0015B4, "Polymap Wireless LLC", "POLYMAP-WIRELESS-LLC" },
	{ 0x0015B5, "CI Network Corp.", "CI-NETWORK" },
	{ 0x0015B6, "ShinMaywa Industries, Ltd.", "SHINMAYWA-INDUSTRIES" },
	{ 0x0015B7, "Toshiba", "TOSHIBA" },
	{ 0x0015B8, "Tahoe", "TAHOE" },
	{ 0x0015B9, "Samsung Electronics Co., Ltd.", "SAMSUNG-ELECTRONICS" },
	{ 0x0015BA, "iba AG", "IBA" },
	{ 0x0015BB, "SMA Solar Technology AG", "SMA-SOLAR" },
	{ 0x0015BC, "Develco", "DEVELCO" },
	{ 0x0015BD, "Group 4 Technology Ltd", "GROUP-4" },
	{ 0x0015BE, "Iqua Ltd.", "IQUA" },
	{ 0x0015BF, "technicob", "TECHNICOB" },
	{ 0x0015C0, "DIGITAL TELEMEDIA CO.,LTD.", "DIGITAL-TELEMEDIA" },
	{ 0x0015C1, "SONY Computer Entertainment inc,", "SONY-ENTERTAINMENT" },
	{ 0x0015C2, "3M Germany", "3M-GERMANY" },
	{ 0x0015C3, "Ruf Telematik AG", "RUF-TELEMATIK" },
	{ 0x0015C4, "FLOVEL CO., LTD.", "FLOVEL" },
	{ 0x0015C5, "Dell Inc", "DELL" },
	{ 0x0015C6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0015C7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0015C8, "FlexiPanel Ltd", "FLEXIPANEL" },
	{ 0x0015C9, "Gumstix, Inc", "GUMSTIX" },
	{ 0x0015CA, "TeraRecon, Inc.", "TERARECON" },
	{ 0x0015CB, "Surf Communication Solutions Ltd.", "SURF-COMMUNICATION-SOLUTIONS" },
	{ 0x0015CC, "UQUEST, LTD.", "UQUEST" },
	{ 0x0015CD, "Exartech International Corp.", "EXARTECH" },
	{ 0x0015CE, "ARRIS International", "ARRIS" },
	{ 0x0015CF, "ARRIS International", "ARRIS" },
	{ 0x0015D0, "ARRIS International", "ARRIS" },
	{ 0x0015D1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0015D2, "Xantech Corporation", "XANTECH" },
	{ 0x0015D3, "Pantech&Curitel Communications, Inc.", "PANTECH-CURITEL-COMMUNICATION" },
	{ 0x0015D4, "Emitor AB", "EMITOR-AB" },
	{ 0x0015D5, "NICEVT", "NICEVT" },
	{ 0x0015D6, "OSLiNK Sp. z o.o.", "OSLINK-SP-Z-O-O" },
	{ 0x0015D7, "Reti Corporation", "RETI" },
	{ 0x0015D8, "Interlink Electronics", "INTERLINK-ELECTRONICS" },
	{ 0x0015D9, "PKC Electronics Oy", "PKC-ELECTRONICS-OY" },
	{ 0x0015DA, "IRITEL A.D.", "IRITEL-A-D" },
	{ 0x0015DB, "Canesta Inc.", "CANESTA" },
	{ 0x0015DC, "KT&C Co., Ltd.", "KT-C" },
	{ 0x0015DD, "IP Control Systems Ltd.", "IP-CONTROL" },
	{ 0x0015DE, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0015DF, "Clivet S.p.A.", "CLIVET-S-P-A" },
	{ 0x0015E0, "Ericsson", "ERICSSON" },
	{ 0x0015E1, "Picochip Ltd", "PICOCHIP" },
	{ 0x0015E2, "Dr.Ing. Herbert Knauer GmbH", "DR-ING-HERBERT-KNAUER" },
	{ 0x0015E3, "Dream Technologies Corporation", "DREAM-TECHNOLOGIES" },
	{ 0x0015E4, "Zimmer Elektromedizin", "ZIMMER-ELEKTROMEDIZIN" },
	{ 0x0015E5, "Cheertek Inc.", "CHEERTEK" },
	{ 0x0015E6, "MOBILE TECHNIKA Inc.", "MOBILE-TECHNIKA" },
	{ 0x0015E7, "Quantec Tontechnik", "QUANTEC-TONTECHNIK" },
	{ 0x0015E8, "Nortel", "NORTEL" },
	{ 0x0015E9, "D-Link Corporation", "D-LINK" },
	{ 0x0015EA, "Tellumat (Pty) Ltd", "TELLUMAT-PTY" },
	{ 0x0015EB, "ZTE CORPORATION", "ZTE" },
	{ 0x0015EC, "Boca Devices LLC", "BOCA-DEVICES-LLC" },
	{ 0x0015ED, "Fulcrum Microsystems, Inc.", "FULCRUM-MICROSYSTEMS" },
	{ 0x0015EE, "Omnex Control Systems", "OMNEX-CONTROL" },
	{ 0x0015EF, "NEC TOKIN Corporation", "NEC-TOKIN" },
	{ 0x0015F0, "EGO BV", "EGO" },
	{ 0x0015F1, "KYLINK Communications Corp.", "KYLINK-COMMUNICATION" },
	{ 0x0015F2, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x0015F3, "PELTOR AB", "PELTOR-AB" },
	{ 0x0015F4, "Eventide", "EVENTIDE" },
	{ 0x0015F5, "Sustainable Energy Systems", "SUSTAINABLE-ENERGY" },
	{ 0x0015F6, "SCIENCE AND ENGINEERING SERVICES, INC.", "SCIENCE-AND-ENGINEERING-SERVICES" },
	{ 0x0015F7, "Wintecronics Ltd.", "WINTECRONICS" },
	{ 0x0015F8, "Kingtronics Industrial Co. Ltd.", "KINGTRONICS-INDUSTRIAL" },
	{ 0x0015F9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0015FA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0015FB, "setex schermuly textile computer gmbh", "SETEX-SCHERMULY-TEXTILE" },
	{ 0x0015FC, "Littelfuse Startco", "LITTELFUSE-STARTCO" },
	{ 0x0015FD, "Complete Media Systems", "COMPLETE-MEDIA" },
	{ 0x0015FE, "SCHILLING ROBOTICS LLC", "SCHILLING-ROBOTICS-LLC" },
	{ 0x0015FF, "Novatel Wireless, Inc.", "NOVATEL-WIRELESS" },
	{ 0x001600, "CelleBrite Mobile Synchronization", "CELLEBRITE-MOBILE-SYNCHRONIZATION" },
	{ 0x001601, "Buffalo Inc.", "BUFFALO" },
	{ 0x001602, "CEYON TECHNOLOGY CO.,LTD.", "CEYON" },
	{ 0x001603, "COOLKSKY Co., LTD", "COOLKSKY" },
	{ 0x001604, "Sigpro", "SIGPRO" },
	{ 0x001605, "YORKVILLE SOUND INC.", "YORKVILLE-SOUND" },
	{ 0x001606, "Ideal Industries", "IDEAL-INDUSTRIES" },
	{ 0x001607, "Curves International Inc.", "CURVES" },
	{ 0x001608, "Sequans Communications", "SEQUANS-COMMUNICATION" },
	{ 0x001609, "Unitech electronics co., ltd.", "UNITECH-ELECTRONICS" },
	{ 0x00160A, "SWEEX Europe BV", "SWEEX-EUROPE" },
	{ 0x00160B, "TVWorks LLC", "TVWORKS-LLC" },
	{ 0x00160C, "LPL DEVELOPMENT S.A. DE C.V", "LPL-DEVELOPMENT-DE-C-V" },
	{ 0x00160D, "Be Here Corporation", "BE-HERE" },
	{ 0x00160E, "Optica Technologies Inc.", "OPTICA-TECHNOLOGIES" },
	{ 0x00160F, "BADGER METER INC", "BADGER-METER" },
	{ 0x001610, "Carina Technology", "CARINA" },
	{ 0x001611, "Altecon Srl", "ALTECON-SRL" },
	{ 0x001612, "Otsuka Electronics Co., Ltd.", "OTSUKA-ELECTRONICS" },
	{ 0x001613, "LibreStream Technologies Inc.", "LIBRESTREAM-TECHNOLOGIES" },
	{ 0x001614, "Picosecond Pulse Labs", "PICOSECOND-PULSE-LABS" },
	{ 0x001615, "Nittan Company, Limited", "NITTAN-COMPANY" },
	{ 0x001616, "BROWAN COMMUNICATION INC.", "BROWAN-COMMUNICATION" },
	{ 0x001617, "MSI", "MSI" },
	{ 0x001618, "HIVION Co., Ltd.", "HIVION" },
	{ 0x001619, "Lancelan Technologies S.L.", "LANCELAN-TECHNOLOGIES-S-L" },
	{ 0x00161A, "Dametric AB", "DAMETRIC-AB" },
	{ 0x00161B, "Micronet Corporation", "MICRONET" },
	{ 0x00161C, "e:cue", "E:CUE" },
	{ 0x00161D, "Innovative Wireless Technologies, Inc.", "INNOVATIVE-WIRELESS-TECHNOLOGIES" },
	{ 0x00161E, "Woojinnet", "WOOJINNET" },
	{ 0x00161F, "SUNWAVETEC Co., Ltd.", "SUNWAVETEC" },
	{ 0x001620, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x001621, "Colorado Vnet", "COLORADO-VNET" },
	{ 0x001622, "BBH SYSTEMS GMBH", "BBH" },
	{ 0x001623, "Interval Media", "INTERVAL-MEDIA" },
	{ 0x001624, "Teneros, Inc.", "TENEROS" },
	{ 0x001625, "Impinj, Inc.", "IMPINJ" },
	{ 0x001626, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001627, "embedded-logic DESIGN AND MORE GmbH", "EMBEDDED-LOGIC-DESIGN-AND-MORE" },
	{ 0x001628, "Ultra Electronics Manufacturing and Card Systems", "ULTRA-ELECTRONICS-MANUFACTURING-AND-CARD" },
	{ 0x001629, "Nivus GmbH", "NIVUS" },
	{ 0x00162A, "Antik computers & communications s.r.o.", "ANTIK-COMPUTERS-COMMUNICATION-S-R-O" },
	{ 0x00162B, "Togami Electric Mfg.co.,Ltd.", "TOGAMI-ELECTRIC-MFG" },
	{ 0x00162C, "Xanboo", "XANBOO" },
	{ 0x00162D, "STNet Co., Ltd.", "STNET" },
	{ 0x00162E, "Space Shuttle Hi-Tech Co., Ltd.", "SPACE-SHUTTLE-HI-TECH" },
	{ 0x00162F, "Geutebrück GmbH", "GEUTEBRüCK" },
	{ 0x001630, "Vativ Technologies", "VATIV-TECHNOLOGIES" },
	{ 0x001631, "Xteam", "XTEAM" },
	{ 0x001632, "SAMSUNG ELECTRONICS CO., LTD.", "SAMSUNG-ELECTRONICS" },
	{ 0x001633, "Oxford Diagnostics Ltd.", "OXFORD-DIAGNOSTICS" },
	{ 0x001634, "Mathtech, Inc.", "MATHTECH" },
	{ 0x001635, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001636, "Quanta Computer Inc.", "QUANTA" },
	{ 0x001637, "CITEL SpA", "CITEL-SPA" },
	{ 0x001638, "TECOM Co., Ltd.", "TECOM" },
	{ 0x001639, "UBIQUAM Co.,Ltd", "UBIQUAM" },
	{ 0x00163A, "YVES TECHNOLOGY CO., LTD.", "YVES" },
	{ 0x00163B, "VertexRSI/General Dynamics", "VERTEXRSI/GENERAL-DYNAMICS" },
	{ 0x00163C, "Rebox B.V.", "REBOX" },
	{ 0x00163D, "Tsinghua Tongfang Legend Silicon Tech. Co., Ltd.", "TSINGHUA-TONGFANG-LEGEND-SILICON-TECH" },
	{ 0x00163E, "Xensource, Inc. (possible Xen VM)", "XENSOURCE-VIRTUAL" },
	{ 0x00163F, "CReTE SYSTEMS Inc.", "CRETE" },
	{ 0x001640, "Asmobile Communication Inc.", "ASMOBILE-COMMUNICATION" },
	{ 0x001641, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x001642, "Pangolin", "PANGOLIN" },
	{ 0x001643, "Sunhillo Corporation", "SUNHILLO" },
	{ 0x001644, "LITE-ON Technology Corp.", "LITE-ON" },
	{ 0x001645, "Power Distribution, Inc.", "POWER-DISTRIBUTION" },
	{ 0x001646, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001647, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001648, "SSD Company Limited", "SSD-COMPANY" },
	{ 0x001649, "SetOne GmbH", "SETONE" },
	{ 0x00164A, "Vibration Technology Limited", "VIBRATION" },
	{ 0x00164B, "Quorion Data Systems GmbH", "QUORION-DATA" },
	{ 0x00164C, "PLANET INT Co., Ltd", "PLANET-INT" },
	{ 0x00164D, "Alcatel North America IP Division", "ALCATEL-NORTH-AMERICA-IP-DIVISION" },
	{ 0x00164E, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x00164F, "World Ethnic Broadcastin Inc.", "WORLD-ETHNIC-BROADCASTIN" },
	{ 0x001650, "Herley General Microwave Israel.", "HERLEY-GENERAL-MICROWAVE-ISRAEL" },
	{ 0x001651, "Exeo Systems", "EXEO" },
	{ 0x001652, "Hoatech Technologies, Inc.", "HOATECH-TECHNOLOGIES" },
	{ 0x001653, "LEGO System A/S IE Electronics Division", "LEGO-SYSTEM-A/S-IE-ELECTRONICS-DIVISION" },
	{ 0x001654, "Flex-P Industries Sdn. Bhd.", "FLEX-P-INDUSTRIES-SDN-BHD" },
	{ 0x001655, "FUHO TECHNOLOGY Co., LTD", "FUHO" },
	{ 0x001656, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001657, "Aegate Ltd", "AEGATE" },
	{ 0x001658, "Fusiontech Technologies Inc.", "FUSIONTECH-TECHNOLOGIES" },
	{ 0x001659, "Z.M.P. RADWAG", "Z-M-P-RADWAG" },
	{ 0x00165A, "Harman Specialty Group", "HARMAN-SPECIALTY-GROUP" },
	{ 0x00165B, "Grip Audio", "GRIP-AUDIO" },
	{ 0x00165C, "Trackflow Ltd", "TRACKFLOW" },
	{ 0x00165D, "AirDefense, Inc.", "AIRDEFENSE" },
	{ 0x00165E, "Precision I/O", "PRECISION-I/O" },
	{ 0x00165F, "Fairmount Automation", "FAIRMOUNT-AUTOMATION" },
	{ 0x001660, "Nortel", "NORTEL" },
	{ 0x001661, "Novatium Solutions (P) Ltd", "NOVATIUM-SOLUTIONS-P" },
	{ 0x001662, "Liyuh Technology Ltd.", "LIYUH" },
	{ 0x001663, "KBT Mobile", "KBT-MOBILE" },
	{ 0x001664, "Prod-El SpA", "PROD-EL-SPA" },
	{ 0x001665, "Cellon France", "CELLON-FRANCE" },
	{ 0x001666, "Quantier Communication Inc.", "QUANTIER-COMMUNICATION" },
	{ 0x001667, "A-TEC Subsystem INC.", "A-TEC-SUBSYSTEM" },
	{ 0x001668, "Eishin Electronics", "EISHIN-ELECTRONICS" },
	{ 0x001669, "MRV Communication (Networks) LTD", "MRV-COMMUNICATION-NETWORKS" },
	{ 0x00166A, "TPS", "TPS" },
	{ 0x00166B, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x00166C, "Samsung Electonics Digital Video System Division", "SAMSUNG-ELECTONICS-DIGITAL-VIDEO-SYSTEM-DIVISION" },
	{ 0x00166D, "Yulong Computer Telecommunication Scientific(shenzhen)Co.,Lt", "YULONG-TELECOMMUNICATION-SCIENTIFIC-SHENZHEN-LT" },
	{ 0x00166E, "Arbitron Inc.", "ARBITRON" },
	{ 0x00166F, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001670, "SKNET Corporation", "SKNET" },
	{ 0x001671, "Symphox Information Co.", "SYMPHOX-INFORMATION" },
	{ 0x001672, "Zenway enterprise ltd", "ZENWAY" },
	{ 0x001673, "Bury GmbH & Co. KG", "BURY" },
	{ 0x001674, "EuroCB (Phils.), Inc.", "EUROCB-PHILS" },
	{ 0x001675, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001676, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001677, "Bihl + Wiedemann GmbH", "BIHL-+-WIEDEMANN" },
	{ 0x001678, "SHENZHEN BAOAN GAOKE ELECTRONICS CO., LTD", "SHENZHEN-BAOAN-GAOKE-ELECTRONICS" },
	{ 0x001679, "eOn Communications", "EON-COMMUNICATION" },
	{ 0x00167A, "Skyworth Overseas Dvelopment Ltd.", "SKYWORTH-OVERSEAS-DVELOPMENT" },
	{ 0x00167B, "Haver&Boecker", "HAVER-BOECKER" },
	{ 0x00167C, "iRex Technologies BV", "IREX-TECHNOLOGIES" },
	{ 0x00167D, "Sky-Line Information Co., Ltd.", "SKY-LINE-INFORMATION" },
	{ 0x00167E, "DIBOSS.CO.,LTD", "DIBOSS" },
	{ 0x00167F, "Bluebird Soft Inc.", "BLUEBIRD-SOFT" },
	{ 0x001680, "Bally Gaming + Systems", "BALLY-GAMING-+" },
	{ 0x001681, "Vector Informatik GmbH", "VECTOR-INFORMATIK" },
	{ 0x001682, "Pro Dex, Inc", "PRO-DEX" },
	{ 0x001683, "WEBIO International Co.,.Ltd.", "WEBIO" },
	{ 0x001684, "Donjin Co.,Ltd.", "DONJIN" },
	{ 0x001685, "Elisa Oyj", "ELISA-OYJ" },
	{ 0x001686, "Karl Storz Imaging", "KARL-STORZ-IMAGING" },
	{ 0x001687, "Chubb CSC-Vendor AP", "CHUBB-CSC-VENDOR-AP" },
	{ 0x001688, "ServerEngines LLC", "SERVERENGINES-LLC" },
	{ 0x001689, "Pilkor Electronics Co., Ltd", "PILKOR-ELECTRONICS" },
	{ 0x00168A, "id-Confirm Inc", "ID-CONFIRM" },
	{ 0x00168B, "Paralan Corporation", "PARALAN" },
	{ 0x00168C, "DSL Partner AS", "DSL-PARTNER-AS" },
	{ 0x00168D, "KORWIN CO., Ltd.", "KORWIN" },
	{ 0x00168E, "Vimicro corporation", "VIMICRO" },
	{ 0x00168F, "GN Netcom as", "GN-NETCOM-AS" },
	{ 0x001690, "J-TEK INCORPORATION", "J-TEK-INCORPORATION" },
	{ 0x001691, "Moser-Baer AG", "MOSER-BAER" },
	{ 0x001692, "Scientific-Atlanta, Inc.", "SCIENTIFIC-ATLANTA" },
	{ 0x001693, "PowerLink Technology Inc.", "POWERLINK" },
	{ 0x001694, "Sennheiser Communications A/S", "SENNHEISER-COMMUNICATION-A/S" },
	{ 0x001695, "AVC Technology (International) Limited", "AVC" },
	{ 0x001696, "QDI Technology (H.K.) Limited", "QDI-H-K" },
	{ 0x001697, "NEC Corporation", "NEC" },
	{ 0x001698, "T&A Mobile Phones", "T-A-MOBILE-PHONES" },
	{ 0x001699, "Tonic DVB Marketing Ltd", "TONIC-DVB-MARKETING" },
	{ 0x00169A, "Quadrics Ltd", "QUADRICS" },
	{ 0x00169B, "Alstom Transport", "ALSTOM-TRANSPORT" },
	{ 0x00169C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00169D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00169E, "TV One Ltd", "TV-ONE" },
	{ 0x00169F, "Vimtron Electronics Co., Ltd.", "VIMTRON-ELECTRONICS" },
	{ 0x0016A0, "Auto-Maskin", "AUTO-MASKIN" },
	{ 0x0016A1, "3Leaf Networks", "3LEAF-NETWORKS" },
	{ 0x0016A2, "CentraLite Systems, Inc.", "CENTRALITE" },
	{ 0x0016A3, "Ingeteam Transmission&Distribution, S.A.", "INGETEAM-TRANSMISSION-DISTRIBUTION" },
	{ 0x0016A4, "Ezurio Ltd", "EZURIO" },
	{ 0x0016A5, "Tandberg Storage ASA", "TANDBERG-STORAGE-ASA" },
	{ 0x0016A6, "Dovado FZ-LLC", "DOVADO-FZ-LLC" },
	{ 0x0016A7, "AWETA G&P", "AWETA-G-P" },
	{ 0x0016A8, "CWT CO., LTD.", "CWT" },
	{ 0x0016A9, "2EI", "2EI" },
	{ 0x0016AA, "Kei Communication Technology Inc.", "KEI-COMMUNICATION" },
	{ 0x0016AB, "Dansensor A/S", "DANSENSOR-A/S" },
	{ 0x0016AC, "Toho Technology Corp.", "TOHO" },
	{ 0x0016AD, "BT-Links Company Limited", "BT-LINKS-COMPANY" },
	{ 0x0016AE, "INVENTEL", "INVENTEL" },
	{ 0x0016AF, "Shenzhen Union Networks Equipment Co.,Ltd.", "SHENZHEN-UNION-NETWORKS-EQUIPMENT" },
	{ 0x0016B0, "VK Corporation", "VK" },
	{ 0x0016B1, "KBS", "KBS" },
	{ 0x0016B2, "DriveCam Inc", "DRIVECAM" },
	{ 0x0016B3, "Photonicbridges (China) Co., Ltd.", "PHOTONICBRIDGES-CHINA" },
	{ 0x0016B4, "PRIVATE", "PRIVATE" },
	{ 0x0016B5, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0016B6, "Cisco-Linksys", "CISCO-LINKSYS" },
	{ 0x0016B7, "Seoul Commtech", "SEOUL-COMMTECH" },
	{ 0x0016B8, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x0016B9, "ProCurve Networking", "PROCURVE-NETWORKING" },
	{ 0x0016BA, "WEATHERNEWS INC.", "WEATHERNEWS" },
	{ 0x0016BB, "Law-Chain Computer Technology Co Ltd", "LAW-CHAIN" },
	{ 0x0016BC, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0016BD, "ATI Industrial Automation", "ATI-INDUSTRIAL-AUTOMATION" },
	{ 0x0016BE, "INFRANET, Inc.", "INFRANET" },
	{ 0x0016BF, "PaloDEx Group Oy", "PALODEX-GROUP-OY" },
	{ 0x0016C0, "Semtech Corporation", "SEMTECH" },
	{ 0x0016C1, "Eleksen Ltd", "ELEKSEN" },
	{ 0x0016C2, "Avtec Systems Inc", "AVTEC" },
	{ 0x0016C3, "BA Systems Inc", "BA" },
	{ 0x0016C4, "SiRF Technology, Inc.", "SIRF" },
	{ 0x0016C5, "Shenzhen Xing Feng Industry Co.,Ltd", "SHENZHEN-XING-FENG-INDUSTRY" },
	{ 0x0016C6, "North Atlantic Industries", "NORTH-ATLANTIC-INDUSTRIES" },
	{ 0x0016C7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0016C8, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0016C9, "NAT Seattle, Inc.", "NAT-SEATTLE" },
	{ 0x0016CA, "Nortel", "NORTEL" },
	{ 0x0016CB, "Apple", "APPLE" },
	{ 0x0016CC, "Xcute Mobile Corp.", "XCUTE-MOBILE" },
	{ 0x0016CD, "HIJI HIGH-TECH CO., LTD.", "HIJI-HIGH-TECH" },
	{ 0x0016CE, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x0016CF, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x0016D0, "ATech elektronika d.o.o.", "ATECH-ELEKTRONIKA-D-O-O" },
	{ 0x0016D1, "ZAT a.s.", "ZAT-A-S" },
	{ 0x0016D2, "Caspian", "CASPIAN" },
	{ 0x0016D3, "Wistron Corporation", "WISTRON" },
	{ 0x0016D4, "Compal Communications, Inc.", "COMPAL-COMMUNICATION" },
	{ 0x0016D5, "Synccom Co., Ltd", "SYNCCOM" },
	{ 0x0016D6, "TDA Tech Pty Ltd", "TDA-TECH-PTY" },
	{ 0x0016D7, "Sunways AG", "SUNWAYS" },
	{ 0x0016D8, "Senea AB", "SENEA-AB" },
	{ 0x0016D9, "NINGBO BIRD CO.,LTD.", "NINGBO-BIRD" },
	{ 0x0016DA, "Futronic Technology Co. Ltd.", "FUTRONIC" },
	{ 0x0016DB, "Samsung Electronics Co., Ltd.", "SAMSUNG-ELECTRONICS" },
	{ 0x0016DC, "ARCHOS", "ARCHOS" },
	{ 0x0016DD, "Gigabeam Corporation", "GIGABEAM" },
	{ 0x0016DE, "FAST Inc", "FAST" },
	{ 0x0016DF, "Lundinova AB", "LUNDINOVA-AB" },
	{ 0x0016E0, "3Com Ltd", "3COM" },
	{ 0x0016E1, "SiliconStor, Inc.", "SILICONSTOR" },
	{ 0x0016E2, "American Fibertek, Inc.", "AMERICAN-FIBERTEK" },
	{ 0x0016E3, "ASKEY COMPUTER CORP.", "ASKEY" },
	{ 0x0016E4, "VANGUARD SECURITY ENGINEERING CORP.", "VANGUARD-SECURITY-ENGINEERING" },
	{ 0x0016E5, "FORDLEY DEVELOPMENT LIMITED", "FORDLEY-DEVELOPMENT" },
	{ 0x0016E6, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x0016E7, "Dynamix Promotions Limited", "DYNAMIX-PROMOTIONS" },
	{ 0x0016E8, "Sigma Designs, Inc.", "SIGMA-DESIGNS" },
	{ 0x0016E9, "Tiba Medical Inc", "TIBA-MEDICAL" },
	{ 0x0016EA, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0016EB, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0016EC, "Elitegroup Computer Systems Co., Ltd.", "ELITEGROUP" },
	{ 0x0016ED, "Digital Safety Technologies, Inc", "DIGITAL-SAFETY-TECHNOLOGIES" },
	{ 0x0016EE, "RoyalDigital Inc.", "ROYALDIGITAL" },
	{ 0x0016EF, "Koko Fitness, Inc.", "KOKO-FITNESS" },
	{ 0x0016F0, "Dell", "DELL" },
	{ 0x0016F1, "OmniSense, LLC", "OMNISENSE-LLC" },
	{ 0x0016F2, "Dmobile System Co., Ltd.", "DMOBILE-SYSTEM" },
	{ 0x0016F3, "CAST Information Co., Ltd", "CAST-INFORMATION" },
	{ 0x0016F4, "Eidicom Co., Ltd.", "EIDICOM" },
	{ 0x0016F5, "Dalian Golden Hualu Digital Technology Co.,Ltd", "DALIAN-GOLDEN-HUALU-DIGITAL" },
	{ 0x0016F6, "Video Products Group", "VIDEO-PRODUCTS-GROUP" },
	{ 0x0016F7, "L-3 Communications, Aviation Recorders", "L-3-COMMUNICATION-AVIATION-RECORDERS" },
	{ 0x0016F8, "AVIQTECH TECHNOLOGY CO., LTD.", "AVIQTECH" },
	{ 0x0016F9, "CETRTA POT, d.o.o., Kranj", "CETRTA-POT-D-O-O-KRANJ" },
	{ 0x0016FA, "ECI Telecom Ltd.", "ECI-TELECOM" },
	{ 0x0016FB, "SHENZHEN MTC CO.,LTD.", "SHENZHEN-MTC" },
	{ 0x0016FC, "TOHKEN CO.,LTD.", "TOHKEN" },
	{ 0x0016FD, "Jaty Electronics", "JATY-ELECTRONICS" },
	{ 0x0016FE, "Alps Electric Co., Ltd", "ALPS-ELECTRIC" },
	{ 0x0016FF, "Wamin Optocomm Mfg Corp", "WAMIN-OPTOCOMM-MFG" },
	{ 0x001700, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001701, "KDE, Inc.", "KDE" },
	{ 0x001702, "Osung Midicom Co., Ltd", "OSUNG-MIDICOM" },
	{ 0x001703, "MOSDAN Internation Co.,Ltd", "MOSDAN-INTERNATION" },
	{ 0x001704, "Shinco Electronics Group Co.,Ltd", "SHINCO-ELECTRONICS-GROUP" },
	{ 0x001705, "Methode Electronics", "METHODE-ELECTRONICS" },
	{ 0x001706, "Techfaith Wireless Communication Technology Limited.", "TECHFAITH-WIRELESS-COMMUNICATION" },
	{ 0x001707, "InGrid, Inc", "INGRID" },
	{ 0x001708, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001709, "Exalt Communications", "EXALT-COMMUNICATION" },
	{ 0x00170A, "INEW DIGITAL COMPANY", "INEW-DIGITAL-COMPANY" },
	{ 0x00170B, "Contela, Inc.", "CONTELA" },
	{ 0x00170C, "Twig Com Ltd.", "TWIG-COM" },
	{ 0x00170D, "Dust Networks Inc.", "DUST-NETWORKS" },
	{ 0x00170E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00170F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001710, "Casa Systems Inc.", "CASA" },
	{ 0x001711, "GE Healthcare Bio-Sciences AB", "GE-HEALTHCARE-BIO-SCIENCES-AB" },
	{ 0x001712, "ISCO International", "ISCO" },
	{ 0x001713, "Tiger NetCom", "TIGER-NETCOM" },
	{ 0x001714, "BR Controls Nederland bv", "BR-CONTROLS-NEDERLAND" },
	{ 0x001715, "Qstik", "QSTIK" },
	{ 0x001716, "Qno Technology Inc.", "QNO" },
	{ 0x001717, "Leica Geosystems AG", "LEICA-GEOSYSTEMS" },
	{ 0x001718, "Vansco Electronics Oy", "VANSCO-ELECTRONICS-OY" },
	{ 0x001719, "AudioCodes USA, Inc", "AUDIOCODES-USA" },
	{ 0x00171A, "Winegard Company", "WINEGARD-COMPANY" },
	{ 0x00171B, "Innovation Lab Corp.", "INNOVATION-LAB" },
	{ 0x00171C, "NT MicroSystems, Inc.", "NT-MICROSYSTEMS" },
	{ 0x00171D, "DIGIT", "DIGIT" },
	{ 0x00171E, "Theo Benning GmbH & Co. KG", "THEO-BENNING" },
	{ 0x00171F, "IMV Corporation", "IMV" },
	{ 0x001720, "Image Sensing Systems, Inc.", "IMAGE-SENSING" },
	{ 0x001721, "FITRE S.p.A.", "FITRE-S-P-A" },
	{ 0x001722, "Hanazeder Electronic GmbH", "HANAZEDER" },
	{ 0x001723, "Summit Data Communications", "SUMMIT-DATA-COMMUNICATION" },
	{ 0x001724, "Studer Professional Audio GmbH", "STUDER-PROFESSIONAL-AUDIO" },
	{ 0x001725, "Liquid Computing", "LIQUID-COMPUTING" },
	{ 0x001726, "m2c Electronic Technology Ltd.", "M2C" },
	{ 0x001727, "Thermo Ramsey Italia s.r.l.", "THERMO-RAMSEY-ITALIA-S-R-L" },
	{ 0x001728, "Selex Communications", "SELEX-COMMUNICATION" },
	{ 0x001729, "Ubicod Co.LTD", "UBICOD" },
	{ 0x00172A, "Proware Technology Corp.(By Unifosa)", "PROWARE-BY-UNIFOSA" },
	{ 0x00172B, "Global Technologies Inc.", "GLOBAL-TECHNOLOGIES" },
	{ 0x00172C, "TAEJIN INFOTECH", "TAEJIN-INFOTECH" },
	{ 0x00172D, "Axcen Photonics Corporation", "AXCEN-PHOTONICS" },
	{ 0x00172E, "FXC Inc.", "FXC" },
	{ 0x00172F, "NeuLion Incorporated", "NEULION-INCORPORATED" },
	{ 0x001730, "Automation Electronics", "AUTOMATION-ELECTRONICS" },
	{ 0x001731, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x001732, "Science-Technical Center \"RISSA\"", "SCIENCE-TECHNICAL-CENTER-RISSA" },
	{ 0x001733, "SFR", "SFR" },
	{ 0x001734, "ADC Telecommunications", "ADC-TELECOMMUNICATIONS" },
	{ 0x001735, "PRIVATE", "PRIVATE" },
	{ 0x001736, "iiTron Inc.", "IITRON" },
	{ 0x001737, "Industrie Dial Face S.p.A.", "INDUSTRIE-DIAL-FACE-S-P-A" },
	{ 0x001738, "International Business Machines", "BUSINESS-MACHINES" },
	{ 0x001739, "Bright Headphone Electronics Company", "BRIGHT-HEADPHONE-ELECTRONICS-COMPANY" },
	{ 0x00173A, "Reach Systems Inc.", "REACH" },
	{ 0x00173B, "Cisco Systems, Inc.", "CISCO" },
	{ 0x00173C, "Extreme Engineering Solutions", "EXTREME-ENGINEERING-SOLUTIONS" },
	{ 0x00173D, "Neology", "NEOLOGY" },
	{ 0x00173E, "LeucotronEquipamentos Ltda.", "LEUCOTRONEQUIPAMENTOS-LTDA" },
	{ 0x00173F, "Belkin Corporation", "BELKIN" },
	{ 0x001740, "Bluberi Gaming Technologies Inc", "BLUBERI-GAMING-TECHNOLOGIES" },
	{ 0x001741, "DEFIDEV", "DEFIDEV" },
	{ 0x001742, "FUJITSU LIMITED", "FUJITSU" },
	{ 0x001743, "Deck Srl", "DECK-SRL" },
	{ 0x001744, "Araneo Ltd.", "ARANEO" },
	{ 0x001745, "INNOTZ CO., Ltd", "INNOTZ" },
	{ 0x001746, "Freedom9 Inc.", "FREEDOM9" },
	{ 0x001747, "Trimble", "TRIMBLE" },
	{ 0x001748, "Neokoros Brasil Ltda", "NEOKOROS-BRASIL-LTDA" },
	{ 0x001749, "HYUNDAE YONG-O-SA CO.,LTD", "HYUNDAE-YONG-O" },
	{ 0x00174A, "SOCOMEC", "SOCOMEC" },
	{ 0x00174B, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x00174C, "Millipore", "MILLIPORE" },
	{ 0x00174D, "DYNAMIC NETWORK FACTORY, INC.", "DYNAMIC-NETWORK-FACTORY" },
	{ 0x00174E, "Parama-tech Co.,Ltd.", "PARAMA-TECH" },
	{ 0x00174F, "iCatch Inc.", "ICATCH" },
	{ 0x001750, "GSI Group, MicroE Systems", "GSI-GROUP-MICROE" },
	{ 0x001751, "Online Corporation", "ONLINE" },
	{ 0x001752, "DAGS, Inc", "DAGS" },
	{ 0x001753, "nFore Technology Inc.", "NFORE" },
	{ 0x001754, "Arkino HiTOP Corporation Limited", "ARKINO-HITOP" },
	{ 0x001755, "GE Security", "GE-SECURITY" },
	{ 0x001756, "Vinci Labs Oy", "VINCI-LABS-OY" },
	{ 0x001757, "RIX TECHNOLOGY LIMITED", "RIX" },
	{ 0x001758, "ThruVision Ltd", "THRUVISION" },
	{ 0x001759, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00175A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00175B, "ACS Solutions Switzerland Ltd.", "ACS-SOLUTIONS-SWITZERLAND" },
	{ 0x00175C, "SHARP CORPORATION", "SHARP" },
	{ 0x00175D, "Dongseo system.", "DONGSEO-SYSTEM" },
	{ 0x00175E, "Zed-3", "ZED-3" },
	{ 0x00175F, "XENOLINK Communications Co., Ltd.", "XENOLINK-COMMUNICATION" },
	{ 0x001760, "Naito Densei Machida MFG.CO.,LTD", "NAITO-DENSEI-MACHIDA-MFG" },
	{ 0x001761, "PRIVATE", "PRIVATE" },
	{ 0x001762, "Solar Technology, Inc.", "SOLAR" },
	{ 0x001763, "Essentia S.p.A.", "ESSENTIA-S-P-A" },
	{ 0x001764, "ATMedia GmbH", "ATMEDIA" },
	{ 0x001765, "Nortel", "NORTEL" },
	{ 0x001766, "Accense Technology, Inc.", "ACCENSE" },
	{ 0x001767, "Earforce AS", "EARFORCE-AS" },
	{ 0x001768, "Zinwave Ltd", "ZINWAVE" },
	{ 0x001769, "Cymphonix Corp", "CYMPHONIX" },
	{ 0x00176A, "Avago Technologies", "AVAGO-TECHNOLOGIES" },
	{ 0x00176B, "Kiyon, Inc.", "KIYON" },
	{ 0x00176C, "Pivot3, Inc.", "PIVOT3" },
	{ 0x00176D, "CORE CORPORATION", "CORE" },
	{ 0x00176E, "DUCATI SISTEMI", "DUCATI-SISTEMI" },
	{ 0x00176F, "PAX Computer Technology(Shenzhen) Ltd.", "PAX-SHENZHEN" },
	{ 0x001770, "Arti Industrial Electronics Ltd.", "ARTI-INDUSTRIAL-ELECTRONICS" },
	{ 0x001771, "APD Communications Ltd", "APD-COMMUNICATION" },
	{ 0x001772, "ASTRO Strobel Kommunikationssysteme GmbH", "ASTRO-STROBEL-KOMMUNIKATIONSSYSTEME" },
	{ 0x001773, "Laketune Technologies Co. Ltd", "LAKETUNE-TECHNOLOGIES" },
	{ 0x001774, "Elesta GmbH", "ELESTA" },
	{ 0x001775, "TTE Germany GmbH", "TTE-GERMANY" },
	{ 0x001776, "Meso Scale Diagnostics, LLC", "MESO-SCALE-DIAGNOSTICS-LLC" },
	{ 0x001777, "Obsidian Research Corporation", "OBSIDIAN-RESEARCH" },
	{ 0x001778, "Central Music Co.", "CENTRAL-MUSIC" },
	{ 0x001779, "QuickTel", "QUICKTEL" },
	{ 0x00177A, "ASSA ABLOY AB", "ASSA-ABLOY-AB" },
	{ 0x00177B, "Azalea Networks inc", "AZALEA-NETWORKS" },
	{ 0x00177C, "Smartlink Network Systems Limited", "SMARTLINK-NETWORK" },
	{ 0x00177D, "IDT International Limited", "IDT" },
	{ 0x00177E, "Meshcom Technologies Inc.", "MESHCOM-TECHNOLOGIES" },
	{ 0x00177F, "Worldsmart Retech", "WORLDSMART-RETECH" },
	{ 0x001780, "Applied Biosystems B.V.", "APPLIED-BIOSYSTEMS" },
	{ 0x001781, "Greystone Data System, Inc.", "GREYSTONE-DATA-SYSTEM" },
	{ 0x001782, "LoBenn Inc.", "LOBENN" },
	{ 0x001783, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x001784, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001785, "Sparr Electronics Ltd", "SPARR-ELECTRONICS" },
	{ 0x001786, "wisembed", "WISEMBED" },
	{ 0x001787, "Brother, Brother & Sons ApS", "BROTHER-BROTHER-SONS-APS" },
	{ 0x001788, "Philips Lighting BV", "PHILIPS-LIGHTING" },
	{ 0x001789, "Zenitron Corporation", "ZENITRON" },
	{ 0x00178A, "DARTS TECHNOLOGIES CORP.", "DARTS-TECHNOLOGIES" },
	{ 0x00178B, "Teledyne Technologies Incorporated", "TELEDYNE-TECHNOLOGIES-INCORPORATED" },
	{ 0x00178C, "Independent Witness, Inc", "INDEPENDENT-WITNESS" },
	{ 0x00178D, "Checkpoint Systems, Inc.", "CHECKPOINT" },
	{ 0x00178E, "Gunnebo Cash Automation AB", "GUNNEBO-CASH-AUTOMATION-AB" },
	{ 0x00178F, "NINGBO YIDONG ELECTRONIC CO.,LTD.", "NINGBO-YIDONG" },
	{ 0x001790, "HYUNDAI DIGITECH Co, Ltd.", "HYUNDAI-DIGITECH" },
	{ 0x001791, "LinTech GmbH", "LINTECH" },
	{ 0x001792, "Falcom Wireless Comunications Gmbh", "FALCOM-WIRELESS-COMUNICATIONS" },
	{ 0x001793, "Tigi Corporation", "TIGI" },
	{ 0x001794, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001795, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001796, "Rittmeyer AG", "RITTMEYER" },
	{ 0x001797, "Telsy Elettronica S.p.A.", "TELSY-ELETTRONICA-S-P-A" },
	{ 0x001798, "Azonic Technology Co., LTD", "AZONIC" },
	{ 0x001799, "SmarTire Systems Inc.", "SMARTIRE" },
	{ 0x00179A, "D-Link Corporation", "D-LINK" },
	{ 0x00179B, "Chant Sincere CO., LTD.", "CHANT-SINCERE" },
	{ 0x00179C, "DEPRAG SCHULZ GMBH u. CO.", "DEPRAG-SCHULZ-U" },
	{ 0x00179D, "Kelman Limited", "KELMAN" },
	{ 0x00179E, "Sirit Inc", "SIRIT" },
	{ 0x00179F, "Apricorn", "APRICORN" },
	{ 0x0017A0, "RoboTech srl", "ROBOTECH-SRL" },
	{ 0x0017A1, "3soft inc.", "3SOFT" },
	{ 0x0017A2, "Camrivox Ltd.", "CAMRIVOX" },
	{ 0x0017A3, "MIX s.r.l.", "MIX-S-R-L" },
	{ 0x0017A4, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0017A5, "Ralink Technology Corp", "RALINK" },
	{ 0x0017A6, "YOSIN ELECTRONICS CO., LTD.", "YOSIN-ELECTRONICS" },
	{ 0x0017A7, "Mobile Computing Promotion Consortium", "MOBILE-COMPUTING-PROMOTION-CONSORTIUM" },
	{ 0x0017A8, "EDM Corporation", "EDM" },
	{ 0x0017A9, "Sentivision", "SENTIVISION" },
	{ 0x0017AA, "elab-experience inc.", "ELAB-EXPERIENCE" },
	{ 0x0017AB, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x0017AC, "O'Neil Product Development Inc.", "O-NEIL-PRODUCT-DEVELOPMENT" },
	{ 0x0017AD, "AceNet Corporation", "ACENET" },
	{ 0x0017AE, "GAI-Tronics", "GAI-TRONICS" },
	{ 0x0017AF, "Enermet", "ENERMET" },
	{ 0x0017B0, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0017B1, "ACIST Medical Systems, Inc.", "ACIST-MEDICAL" },
	{ 0x0017B2, "SK Telesys", "SK-TELESYS" },
	{ 0x0017B3, "Aftek Infosys Limited", "AFTEK-INFOSYS" },
	{ 0x0017B4, "Remote Security Systems, LLC", "REMOTE-SECURITY-LLC" },
	{ 0x0017B5, "Peerless Systems Corporation", "PEERLESS" },
	{ 0x0017B6, "Aquantia", "AQUANTIA" },
	{ 0x0017B7, "Tonze Technology Co.", "TONZE" },
	{ 0x0017B8, "NOVATRON CO., LTD.", "NOVATRON" },
	{ 0x0017B9, "Gambro Lundia AB", "GAMBRO-LUNDIA-AB" },
	{ 0x0017BA, "SEDO CO., LTD.", "SEDO" },
	{ 0x0017BB, "Syrinx Industrial Electronics", "SYRINX-INDUSTRIAL-ELECTRONICS" },
	{ 0x0017BC, "Touchtunes Music Corporation", "TOUCHTUNES-MUSIC" },
	{ 0x0017BD, "Tibetsystem", "TIBETSYSTEM" },
	{ 0x0017BE, "Tratec Telecom B.V.", "TRATEC-TELECOM" },
	{ 0x0017BF, "Coherent Research Limited", "COHERENT-RESEARCH" },
	{ 0x0017C0, "PureTech Systems, Inc.", "PURETECH" },
	{ 0x0017C1, "CM Precision Technology LTD.", "CM-PRECISION" },
	{ 0x0017C2, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x0017C3, "KTF Technologies Inc.", "KTF-TECHNOLOGIES" },
	{ 0x0017C4, "Quanta Microsystems, INC.", "QUANTA-MICROSYSTEMS" },
	{ 0x0017C5, "SonicWALL", "SONICWALL" },
	{ 0x0017C6, "Cross Match Technologies Inc", "CROSS-MATCH-TECHNOLOGIES" },
	{ 0x0017C7, "MARA Systems Consulting AB", "MARA-CONSULTING-AB" },
	{ 0x0017C8, "KYOCERA Document Solutions Inc.", "KYOCERA-DOCUMENT-SOLUTIONS" },
	{ 0x0017C9, "Samsung Electronics Co., Ltd.", "SAMSUNG-ELECTRONICS" },
	{ 0x0017CA, "Qisda Corporation", "QISDA" },
	{ 0x0017CB, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x0017CC, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x0017CD, "CEC Wireless R&D Ltd.", "CEC-WIRELESS-R-D" },
	{ 0x0017CE, "Screen Service Spa", "SCREEN-SERVICE-SPA" },
	{ 0x0017CF, "iMCA-GmbH", "IMCA" },
	{ 0x0017D0, "Opticom Communications, LLC", "OPTICOM-COMMUNICATION-LLC" },
	{ 0x0017D1, "Nortel", "NORTEL" },
	{ 0x0017D2, "THINLINX PTY LTD", "THINLINX-PTY" },
	{ 0x0017D3, "Etymotic Research, Inc.", "ETYMOTIC-RESEARCH" },
	{ 0x0017D4, "Monsoon Multimedia, Inc", "MONSOON-MULTIMEDIA" },
	{ 0x0017D5, "Samsung Electronics Co., Ltd.", "SAMSUNG-ELECTRONICS" },
	{ 0x0017D6, "Bluechips Microhouse Co.,Ltd.", "BLUECHIPS-MICROHOUSE" },
	{ 0x0017D7, "ION Geophysical Corporation Inc.", "ION-GEOPHYSICAL" },
	{ 0x0017D8, "Magnum Semiconductor, Inc.", "MAGNUM-SEMICONDUCTOR" },
	{ 0x0017D9, "AAI Corporation", "AAI" },
	{ 0x0017DA, "Spans Logic", "SPANS-LOGIC" },
	{ 0x0017DB, "CANKO TECHNOLOGIES INC.", "CANKO-TECHNOLOGIES" },
	{ 0x0017DC, "DAEMYUNG ZERO1", "DAEMYUNG-ZERO1" },
	{ 0x0017DD, "Clipsal Australia", "CLIPSAL-AUSTRALIA" },
	{ 0x0017DE, "Advantage Six Ltd", "ADVANTAGE-SIX" },
	{ 0x0017DF, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0017E0, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0017E1, "DACOS Technologies Co., Ltd.", "DACOS-TECHNOLOGIES" },
	{ 0x0017E2, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0017E3, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017E4, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017E5, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017E6, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017E7, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017E8, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017E9, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017EA, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017EB, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017EC, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0017ED, "WooJooIT Ltd.", "WOOJOOIT" },
	{ 0x0017EE, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0017EF, "IBM Corp", "IBM" },
	{ 0x0017F0, "SZCOM Broadband Network Technology Co.,Ltd", "SZCOM-BROADBAND-NETWORK" },
	{ 0x0017F1, "Renu Electronics Pvt Ltd", "RENU-ELECTRONICS-PVT" },
	{ 0x0017F2, "Apple", "APPLE" },
	{ 0x0017F3, "Harris Corparation", "HARRIS-CORPARATION" },
	{ 0x0017F4, "ZERON ALLIANCE", "ZERON-ALLIANCE" },
	{ 0x0017F5, "LIG NEOPTEK", "LIG-NEOPTEK" },
	{ 0x0017F6, "Pyramid Meriden Inc.", "PYRAMID-MERIDEN" },
	{ 0x0017F7, "CEM Solutions Pvt Ltd", "CEM-SOLUTIONS-PVT" },
	{ 0x0017F8, "Motech Industries Inc.", "MOTECH-INDUSTRIES" },
	{ 0x0017F9, "Forcom Sp. z o.o.", "FORCOM-SP-Z-O-O" },
	{ 0x0017FA, "Microsoft Corporation", "MICROSOFT" },
	{ 0x0017FB, "FA", "FA" },
	{ 0x0017FC, "Suprema Inc.", "SUPREMA" },
	{ 0x0017FD, "Amulet Hotkey", "AMULET-HOTKEY" },
	{ 0x0017FE, "TALOS SYSTEM INC.", "TALOS-SYSTEM" },
	{ 0x0017FF, "PLAYLINE Co.,Ltd.", "PLAYLINE" },
	{ 0x001800, "UNIGRAND LTD", "UNIGRAND" },
	{ 0x001801, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x001802, "Alpha Networks Inc.", "ALPHA-NETWORKS" },
	{ 0x001803, "ArcSoft Shanghai Co. LTD", "ARCSOFT-SHANGHAI" },
	{ 0x001804, "E-TEK DIGITAL TECHNOLOGY LIMITED", "E-TEK-DIGITAL" },
	{ 0x001805, "Beijing InHand Networking Technology Co.,Ltd.", "BEIJING-INHAND-NETWORKING" },
	{ 0x001806, "Hokkei Industries Co., Ltd.", "HOKKEI-INDUSTRIES" },
	{ 0x001807, "Fanstel Corp.", "FANSTEL" },
	{ 0x001808, "SightLogix, Inc.", "SIGHTLOGIX" },
	{ 0x001809, "CRESYN", "CRESYN" },
	{ 0x00180A, "Meraki, Inc.", "MERAKI" },
	{ 0x00180B, "Brilliant Telecommunications", "BRILLIANT-TELECOMMUNICATIONS" },
	{ 0x00180C, "Optelian Access Networks", "OPTELIAN-ACCESS-NETWORKS" },
	{ 0x00180D, "Terabytes Server Storage Tech Corp", "TERABYTES-SERVER-STORAGE-TECH" },
	{ 0x00180E, "Avega Systems", "AVEGA" },
	{ 0x00180F, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001810, "IPTrade S.A.", "IPTRADE" },
	{ 0x001811, "Neuros Technology International, LLC.", "NEUROS-LLC" },
	{ 0x001812, "Beijing Xinwei Telecom Technology Co., Ltd.", "BEIJING-XINWEI-TELECOM" },
	{ 0x001813, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x001814, "Mitutoyo Corporation", "MITUTOYO" },
	{ 0x001815, "GZ Technologies, Inc.", "GZ-TECHNOLOGIES" },
	{ 0x001816, "Ubixon Co., Ltd.", "UBIXON" },
	{ 0x001817, "D. E. Shaw Research, LLC", "D-E-SHAW-RESEARCH-LLC" },
	{ 0x001818, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001819, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00181A, "AVerMedia Information Inc.", "AVERMEDIA-INFORMATION" },
	{ 0x00181B, "TaiJin Metal Co., Ltd.", "TAIJIN-METAL" },
	{ 0x00181C, "Exterity Limited", "EXTERITY" },
	{ 0x00181D, "ASIA ELECTRONICS CO.,LTD", "ASIA-ELECTRONICS" },
	{ 0x00181E, "GDX Technologies Ltd.", "GDX-TECHNOLOGIES" },
	{ 0x00181F, "Palmmicro Communications", "PALMMICRO-COMMUNICATION" },
	{ 0x001820, "w5networks", "W5NETWORKS" },
	{ 0x001821, "SINDORICOH", "SINDORICOH" },
	{ 0x001822, "CEC TELECOM CO.,LTD.", "CEC-TELECOM" },
	{ 0x001823, "Delta Electronics, Inc.", "DELTA-ELECTRONICS" },
	{ 0x001824, "Kimaldi Electronics, S.L.", "KIMALDI-ELECTRONICS-S-L" },
	{ 0x001825, "PRIVATE", "PRIVATE" },
	{ 0x001826, "Cale Access AB", "CALE-ACCESS-AB" },
	{ 0x001827, "NEC UNIFIED SOLUTIONS NEDERLAND B.V.", "NEC-UNIFIED-SOLUTIONS-NEDERLAND" },
	{ 0x001828, "e2v technologies (UK) ltd.", "E2V-TECHNOLOGIES-UK" },
	{ 0x001829, "Gatsometer", "GATSOMETER" },
	{ 0x00182A, "Taiwan Video & Monitor", "TAIWAN-VIDEO-MONITOR" },
	{ 0x00182B, "Softier", "SOFTIER" },
	{ 0x00182C, "Ascend Networks, Inc.", "ASCEND-NETWORKS" },
	{ 0x00182D, "Artec Design", "ARTEC-DESIGN" },
	{ 0x00182E, "XStreamHD, LLC", "XSTREAMHD-LLC" },
	{ 0x00182F, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x001830, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x001831, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x001832, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x001833, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x001834, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x001835, "Thoratec / ITC", "THORATEC-/-ITC" },
	{ 0x001836, "Reliance Electric Limited", "RELIANCE-ELECTRIC" },
	{ 0x001837, "Universal ABIT Co., Ltd.", "UNIVERSAL-ABIT" },
	{ 0x001838, "PanAccess Communications,Inc.", "PANACCESS-COMMUNICATION" },
	{ 0x001839, "Cisco-Linksys LLC", "CISCO-LINKSYS-LLC" },
	{ 0x00183A, "Westell Technologies", "WESTELL-TECHNOLOGIES" },
	{ 0x00183B, "CENITS Co., Ltd.", "CENITS" },
	{ 0x00183C, "Encore Software Limited", "ENCORE-SOFTWARE" },
	{ 0x00183D, "Vertex Link Corporation", "VERTEX-LINK" },
	{ 0x00183E, "Digilent, Inc", "DIGILENT" },
	{ 0x00183F, "2Wire, Inc", "2WIRE" },
	{ 0x001840, "3 Phoenix, Inc.", "3-PHOENIX" },
	{ 0x001841, "High Tech Computer Corp", "HIGH-TECH" },
	{ 0x001842, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001843, "Dawevision Ltd", "DAWEVISION" },
	{ 0x001844, "Heads Up Technologies, Inc.", "HEADS-UP-TECHNOLOGIES" },
	{ 0x001845, "Pulsar-Telecom LLC.", "PULSAR-TELECOM-LLC" },
	{ 0x001846, "Crypto S.A.", "CRYPTO" },
	{ 0x001847, "AceNet Technology Inc.", "ACENET" },
	{ 0x001848, "Vecima Networks Inc.", "VECIMA-NETWORKS" },
	{ 0x001849, "Pigeon Point Systems LLC", "PIGEON-POINT-LLC" },
	{ 0x00184A, "Catcher, Inc.", "CATCHER" },
	{ 0x00184B, "Las Vegas Gaming, Inc.", "LAS-VEGAS-GAMING" },
	{ 0x00184C, "Bogen Communications", "BOGEN-COMMUNICATION" },
	{ 0x00184D, "Netgear Inc.", "NETGEAR" },
	{ 0x00184E, "Lianhe Technologies, Inc.", "LIANHE-TECHNOLOGIES" },
	{ 0x00184F, "8 Ways Technology Corp.", "8-WAYS" },
	{ 0x001850, "Secfone Kft", "SECFONE-KFT" },
	{ 0x001851, "SWsoft", "SWSOFT" },
	{ 0x001852, "StorLink Semiconductors, Inc.", "STORLINK-SEMICONDUCTORS" },
	{ 0x001853, "Atera Networks LTD.", "ATERA-NETWORKS" },
	{ 0x001854, "Argard Co., Ltd", "ARGARD" },
	{ 0x001855, "Aeromaritime Systembau GmbH", "AEROMARITIME-SYSTEMBAU" },
	{ 0x001856, "EyeFi, Inc", "EYEFI" },
	{ 0x001857, "Unilever R&D", "UNILEVER-R-D" },
	{ 0x001858, "TagMaster AB", "TAGMASTER-AB" },
	{ 0x001859, "Strawberry Linux Co.,Ltd.", "STRAWBERRY-LINUX" },
	{ 0x00185A, "uControl, Inc.", "UCONTROL" },
	{ 0x00185B, "Network Chemistry, Inc", "NETWORK-CHEMISTRY" },
	{ 0x00185C, "EDS Lab Pte Ltd", "EDS-LAB-PTE" },
	{ 0x00185D, "TAIGUEN TECHNOLOGY (SHEN-ZHEN) CO., LTD.", "TAIGUEN-SHEN-ZHEN" },
	{ 0x00185E, "Nexterm Inc.", "NEXTERM" },
	{ 0x00185F, "TAC Inc.", "TAC" },
	{ 0x001860, "SIM Technology Group Shanghai Simcom Ltd.,", "SIM-GROUP-SHANGHAI-SIMCOM" },
	{ 0x001861, "Ooma, Inc.", "OOMA" },
	{ 0x001862, "Seagate Technology", "SEAGATE" },
	{ 0x001863, "Veritech Electronics Limited", "VERITECH-ELECTRONICS" },
	{ 0x001864, "Eaton Corporation", "EATON" },
	{ 0x001865, "Siemens Healthcare Diagnostics Manufacturing Ltd", "SIEMENS-HEALTHCARE-DIAGNOSTICS-MANUFACTURING" },
	{ 0x001866, "Leutron Vision", "LEUTRON-VISION" },
	{ 0x001867, "Datalogic ADC", "DATALOGIC-ADC" },
	{ 0x001868, "Scientific Atlanta, A Cisco Company", "SCIENTIFIC-ATLANTA-A-CISCO-COMPANY" },
	{ 0x001869, "KINGJIM", "KINGJIM" },
	{ 0x00186A, "Global Link Digital Technology Co,.LTD", "GLOBAL-LINK-DIGITAL" },
	{ 0x00186B, "Sambu Communics CO., LTD.", "SAMBU-COMMUNICS" },
	{ 0x00186C, "Neonode AB", "NEONODE-AB" },
	{ 0x00186D, "Zhenjiang Sapphire Electronic Industry CO.", "ZHENJIANG-SAPPHIRE-INDUSTRY" },
	{ 0x00186E, "3Com Ltd", "3COM" },
	{ 0x00186F, "Setha Industria Eletronica LTDA", "SETHA-INDUSTRIA-ELETRONICA-LTDA" },
	{ 0x001870, "E28 Shanghai Limited", "E28-SHANGHAI" },
	{ 0x001871, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001872, "Expertise Engineering", "EXPERTISE-ENGINEERING" },
	{ 0x001873, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001874, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001875, "AnaCise Testnology Pte Ltd", "ANACISE-TESTNOLOGY-PTE" },
	{ 0x001876, "WowWee Ltd.", "WOWWEE" },
	{ 0x001877, "Amplex A/S", "AMPLEX-A/S" },
	{ 0x001878, "Mackware GmbH", "MACKWARE" },
	{ 0x001879, "dSys", "DSYS" },
	{ 0x00187A, "Wiremold", "WIREMOLD" },
	{ 0x00187B, "4NSYS Co. Ltd.", "4NSYS" },
	{ 0x00187C, "INTERCROSS, LLC", "INTERCROSS-LLC" },
	{ 0x00187D, "Armorlink shanghai Co. Ltd", "ARMORLINK-SHANGHAI" },
	{ 0x00187E, "RGB Spectrum", "RGB-SPECTRUM" },
	{ 0x00187F, "ZODIANET", "ZODIANET" },
	{ 0x001880, "Maxim Integrated Products", "MAXIM-INTEGRATED-PRODUCTS" },
	{ 0x001881, "Buyang Electronics Industrial Co., Ltd", "BUYANG-ELECTRONICS-INDUSTRIAL" },
	{ 0x001882, "Huawei Technologies Co., Ltd.", "HUAWEI-TECHNOLOGIES" },
	{ 0x001883, "FORMOSA21 INC.", "FORMOSA21" },
	{ 0x001884, "Fon Technology S.L.", "FON-S-L" },
	{ 0x001885, "Avigilon Corporation", "AVIGILON" },
	{ 0x001886, "EL-TECH, INC.", "EL-TECH" },
	{ 0x001887, "Metasystem SpA", "METASYSTEM-SPA" },
	{ 0x001888, "GOTIVE a.s.", "GOTIVE-A-S" },
	{ 0x001889, "WinNet Solutions Limited", "WINNET-SOLUTIONS" },
	{ 0x00188A, "Infinova LLC", "INFINOVA-LLC" },
	{ 0x00188B, "Dell Inc", "DELL" },
	{ 0x00188C, "Mobile Action Technology Inc.", "MOBILE-ACTION" },
	{ 0x00188D, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x00188E, "Ekahau, Inc.", "EKAHAU" },
	{ 0x00188F, "Montgomery Technology, Inc.", "MONTGOMERY" },
	{ 0x001890, "RadioCOM, s.r.o.", "RADIOCOM-S-R-O" },
	{ 0x001891, "Zhongshan General K-mate Electronics Co., Ltd", "ZHONGSHAN-GENERAL-K-MATE-ELECTRONICS" },
	{ 0x001892, "ads-tec GmbH", "ADS-TEC" },
	{ 0x001893, "SHENZHEN PHOTON BROADBAND TECHNOLOGY CO.,LTD", "SHENZHEN-PHOTON-BROADBAND" },
	{ 0x001894, "NPCore, Inc.", "NPCORE" },
	{ 0x001895, "Hansun Technologies Inc.", "HANSUN-TECHNOLOGIES" },
	{ 0x001896, "Great Well Electronic LTD", "GREAT-WELL" },
	{ 0x001897, "JESS-LINK PRODUCTS Co., LTD", "JESS-LINK-PRODUCTS" },
	{ 0x001898, "KINGSTATE ELECTRONICS CORPORATION", "KINGSTATE-ELECTRONICS" },
	{ 0x001899, "ShenZhen jieshun Science&Technology Industry CO,LTD.", "SHENZHEN-JIESHUN-SCIENCE-INDUSTRY" },
	{ 0x00189A, "HANA Micron Inc.", "HANA-MICRON" },
	{ 0x00189B, "Thomson Inc.", "THOMSON" },
	{ 0x00189C, "Weldex Corporation", "WELDEX" },
	{ 0x00189D, "Navcast Inc.", "NAVCAST" },
	{ 0x00189E, "OMNIKEY GmbH.", "OMNIKEY" },
	{ 0x00189F, "Lenntek Corporation", "LENNTEK" },
	{ 0x0018A0, "Cierma Ascenseurs", "CIERMA-ASCENSEURS" },
	{ 0x0018A1, "Tiqit Computers, Inc.", "TIQIT-COMPUTERS" },
	{ 0x0018A2, "XIP Technology AB", "XIP-AB" },
	{ 0x0018A3, "ZIPPY TECHNOLOGY CORP.", "ZIPPY" },
	{ 0x0018A4, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0018A5, "ADigit Technologies Corp.", "ADIGIT-TECHNOLOGIES" },
	{ 0x0018A6, "Persistent Systems, LLC", "PERSISTENT-LLC" },
	{ 0x0018A7, "Yoggie Security Systems LTD.", "YOGGIE-SECURITY" },
	{ 0x0018A8, "AnNeal Technology Inc.", "ANNEAL" },
	{ 0x0018A9, "Ethernet Direct Corporation", "ETHERNET-DIRECT" },
	{ 0x0018AA, "Protec Fire Detection plc", "PROTEC-FIRE-DETECTION-PLC" },
	{ 0x0018AB, "BEIJING LHWT MICROELECTRONICS INC.", "BEIJING-LHWT-MICROELECTRONICS" },
	{ 0x0018AC, "Shanghai Jiao Da HISYS Technology Co. Ltd.", "SHANGHAI-JIAO-DA-HISYS" },
	{ 0x0018AD, "NIDEC SANKYO CORPORATION", "NIDEC-SANKYO" },
	{ 0x0018AE, "TVT CO.,LTD", "TVT" },
	{ 0x0018AF, "Samsung Electronics Co., Ltd.", "SAMSUNG-ELECTRONICS" },
	{ 0x0018B0, "Nortel", "NORTEL" },
	{ 0x0018B1, "IBM Corp", "IBM" },
	{ 0x0018B2, "ADEUNIS RF", "ADEUNIS-RF" },
	{ 0x0018B3, "TEC WizHome Co., Ltd.", "TEC-WIZHOME" },
	{ 0x0018B4, "Dawon Media Inc.", "DAWON-MEDIA" },
	{ 0x0018B5, "Magna Carta", "MAGNA-CARTA" },
	{ 0x0018B6, "S3C, Inc.", "S3C" },
	{ 0x0018B7, "D3 LED, LLC", "D3-LED-LLC" },
	{ 0x0018B8, "New Voice International AG", "NEW-VOICE" },
	{ 0x0018B9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0018BA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0018BB, "Eliwell Controls srl", "ELIWELL-CONTROLS-SRL" },
	{ 0x0018BC, "ZAO NVP Bolid", "ZAO-NVP-BOLID" },
	{ 0x0018BD, "SHENZHEN DVBWORLD TECHNOLOGY CO., LTD.", "SHENZHEN-DVBWORLD" },
	{ 0x0018BE, "ANSA Corporation", "ANSA" },
	{ 0x0018BF, "Essence Technology Solution, Inc.", "ESSENCE-SOLUTION" },
	{ 0x0018C0, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0018C1, "Almitec Informática e Comércio", "ALMITEC-INFORMáTICA-E-COMéRCIO" },
	{ 0x0018C2, "Firetide, Inc", "FIRETIDE" },
	{ 0x0018C3, "CS Corporation", "CS" },
	{ 0x0018C4, "Raba Technologies LLC", "RABA-TECHNOLOGIES-LLC" },
	{ 0x0018C5, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0018C6, "OPW Fuel Management Systems", "OPW-FUEL-MANAGEMENT" },
	{ 0x0018C7, "Real Time Automation", "REAL-TIME-AUTOMATION" },
	{ 0x0018C8, "ISONAS Inc.", "ISONAS" },
	{ 0x0018C9, "EOps Technology Limited", "EOPS" },
	{ 0x0018CA, "Viprinet GmbH", "VIPRINET" },
	{ 0x0018CB, "Tecobest Technology Limited", "TECOBEST" },
	{ 0x0018CC, "AXIOHM SAS", "AXIOHM-SAS" },
	{ 0x0018CD, "Erae Electronics Industry Co., Ltd", "ERAE-ELECTRONICS-INDUSTRY" },
	{ 0x0018CE, "Dreamtech Co., Ltd", "DREAMTECH" },
	{ 0x0018CF, "Baldor Electric Company", "BALDOR-ELECTRIC-COMPANY" },
	{ 0x0018D0, "AtRoad, A Trimble Company", "ATROAD-A-TRIMBLE-COMPANY" },
	{ 0x0018D1, "Siemens Home & Office Comm. Devices", "SIEMENS-HOME-OFFICE-COMM-DEVICES" },
	{ 0x0018D2, "High-Gain Antennas LLC", "HIGH-GAIN-ANTENNAS-LLC" },
	{ 0x0018D3, "TEAMCAST", "TEAMCAST" },
	{ 0x0018D4, "Unified Display Interface SIG", "UNIFIED-DISPLAY-INTERFACE-SIG" },
	{ 0x0018D5, "REIGNCOM", "REIGNCOM" },
	{ 0x0018D6, "Swirlnet A/S", "SWIRLNET-A/S" },
	{ 0x0018D7, "Javad Navigation Systems Inc.", "JAVAD-NAVIGATION" },
	{ 0x0018D8, "ARCH METER Corporation", "ARCH-METER" },
	{ 0x0018D9, "Santosha Internatonal, Inc", "SANTOSHA-INTERNATONAL" },
	{ 0x0018DA, "AMBER wireless GmbH", "AMBER-WIRELESS" },
	{ 0x0018DB, "EPL Technology Ltd", "EPL" },
	{ 0x0018DC, "Prostar Co., Ltd.", "PROSTAR" },
	{ 0x0018DD, "Silicondust Engineering Ltd", "SILICONDUST-ENGINEERING" },
	{ 0x0018DE, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0018DF, "The Morey Corporation", "THE-MOREY" },
	{ 0x0018E0, "ANAVEO", "ANAVEO" },
	{ 0x0018E1, "Verkerk Service Systemen", "VERKERK-SERVICE-SYSTEMEN" },
	{ 0x0018E2, "Topdata Sistemas de Automacao Ltda", "TOPDATA-SISTEMAS-DE-AUTOMACAO-LTDA" },
	{ 0x0018E3, "Visualgate Systems, Inc.", "VISUALGATE" },
	{ 0x0018E4, "YIGUANG", "YIGUANG" },
	{ 0x0018E5, "Adhoco AG", "ADHOCO" },
	{ 0x0018E6, "Computer Hardware Design SIA", "HARDWARE-DESIGN-SIA" },
	{ 0x0018E7, "Cameo Communications, INC.", "CAMEO-COMMUNICATION" },
	{ 0x0018E8, "Hacetron Corporation", "HACETRON" },
	{ 0x0018E9, "Numata Corporation", "NUMATA" },
	{ 0x0018EA, "Alltec GmbH", "ALLTEC" },
	{ 0x0018EB, "BroVis Wireless Networks", "BROVIS-WIRELESS-NETWORKS" },
	{ 0x0018EC, "Welding Technology Corporation", "WELDING" },
	{ 0x0018ED, "Accutech Ultrasystems Co., Ltd.", "ACCUTECH-ULTRASYSTEMS" },
	{ 0x0018EE, "Videology Imaging Solutions, Inc.", "VIDEOLOGY-IMAGING-SOLUTIONS" },
	{ 0x0018EF, "Escape Communications, Inc.", "ESCAPE-COMMUNICATION" },
	{ 0x0018F0, "JOYTOTO Co., Ltd.", "JOYTOTO" },
	{ 0x0018F1, "Chunichi Denshi Co.,LTD.", "CHUNICHI-DENSHI" },
	{ 0x0018F2, "Beijing Tianyu Communication Equipment Co., Ltd", "BEIJING-TIANYU-COMMUNICATION-EQUIPMENT" },
	{ 0x0018F3, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x0018F4, "EO TECHNICS Co., Ltd.", "EO-TECHNICS" },
	{ 0x0018F5, "Shenzhen Streaming Video Technology Company Limited", "SHENZHEN-STREAMING-VIDEO-COMPANY" },
	{ 0x0018F6, "Thomson Telecom Belgium", "THOMSON-TELECOM-BELGIUM" },
	{ 0x0018F7, "Kameleon Technologies", "KAMELEON-TECHNOLOGIES" },
	{ 0x0018F8, "Cisco-Linksys LLC", "CISCO-LINKSYS-LLC" },
	{ 0x0018F9, "VVOND, Inc.", "VVOND" },
	{ 0x0018FA, "Yushin Precision Equipment Co.,Ltd.", "YUSHIN-PRECISION-EQUIPMENT" },
	{ 0x0018FB, "Compro Technology", "COMPRO" },
	{ 0x0018FC, "Altec Electronic AG", "ALTEC" },
	{ 0x0018FD, "Optimal Technologies International Inc.", "OPTIMAL-TECHNOLOGIES" },
	{ 0x0018FE, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0018FF, "PowerQuattro Co.", "POWERQUATTRO" },
	{ 0x001900, "Intelliverese - DBA Voicecom", "INTELLIVERESE---DBA-VOICECOM" },
	{ 0x001901, "F1MEDIA", "F1MEDIA" },
	{ 0x001902, "Cambridge Consultants Ltd", "CAMBRIDGE-CONSULTANTS" },
	{ 0x001903, "Bigfoot Networks Inc", "BIGFOOT-NETWORKS" },
	{ 0x001904, "WB Electronics Sp. z o.o.", "WB-ELECTRONICS-SP-Z-O-O" },
	{ 0x001905, "SCHRACK Seconet AG", "SCHRACK-SECONET" },
	{ 0x001906, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001907, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001908, "Duaxes Corporation", "DUAXES" },
	{ 0x001909, "DEVI - Danfoss A/S", "DEVI---DANFOSS-A/S" },
	{ 0x00190A, "HASWARE INC.", "HASWARE" },
	{ 0x00190B, "Southern Vision Systems, Inc.", "SOUTHERN-VISION" },
	{ 0x00190C, "Encore Electronics, Inc.", "ENCORE-ELECTRONICS" },
	{ 0x00190D, "IEEE 1394c", "IEEE-1394C" },
	{ 0x00190E, "Atech Technology Co., Ltd.", "ATECH" },
	{ 0x00190F, "Advansus Corp.", "ADVANSUS" },
	{ 0x001910, "Knick Elektronische Messgeraete GmbH & Co. KG", "KNICK-ELEKTRONISCHE-MESSGERAETE" },
	{ 0x001911, "Just In Mobile Information Technologies (Shanghai) Co., Ltd.", "JUST-IN-MOBILE-INFORMATION-TECHNOLOGIES-SHANGHAI" },
	{ 0x001912, "Welcat Inc", "WELCAT" },
	{ 0x001913, "Chuang-Yi Network Equipment Co.Ltd.", "CHUANG-YI-NETWORK-EQUIPMENT" },
	{ 0x001914, "Winix Co., Ltd", "WINIX" },
	{ 0x001915, "TECOM Co., Ltd.", "TECOM" },
	{ 0x001916, "PayTec AG", "PAYTEC" },
	{ 0x001917, "Posiflex Inc.", "POSIFLEX" },
	{ 0x001918, "Interactive Wear AG", "INTERACTIVE-WEAR" },
	{ 0x001919, "ASTEL Inc.", "ASTEL" },
	{ 0x00191A, "IRLINK", "IRLINK" },
	{ 0x00191B, "Sputnik Engineering AG", "SPUTNIK-ENGINEERING" },
	{ 0x00191C, "Sensicast Systems", "SENSICAST" },
	{ 0x00191D, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x00191E, "Beyondwiz Co., Ltd.", "BEYONDWIZ" },
	{ 0x00191F, "Microlink communications Inc.", "MICROLINK-COMMUNICATION" },
	{ 0x001920, "KUME electric Co.,Ltd.", "KUME-ELECTRIC" },
	{ 0x001921, "Elitegroup Computer System Co.", "ELITEGROUP-SYSTEM" },
	{ 0x001922, "CM Comandos Lineares", "CM-COMANDOS-LINEARES" },
	{ 0x001923, "Phonex Korea Co., LTD.", "PHONEX-KOREA" },
	{ 0x001924, "LBNL Engineering", "LBNL-ENGINEERING" },
	{ 0x001925, "Intelicis Corporation", "INTELICIS" },
	{ 0x001926, "BitsGen Co., Ltd.", "BITSGEN" },
	{ 0x001927, "ImCoSys Ltd", "IMCOSYS" },
	{ 0x001928, "Siemens AG, Transportation Systems", "SIEMENS-TRANSPORTATION" },
	{ 0x001929, "2M2B Montadora de Maquinas Bahia Brasil LTDA", "2M2B-MONTADORA-DE-MAQUINAS-BAHIA-BRASIL-LTDA" },
	{ 0x00192A, "Antiope Associates", "ANTIOPE-ASSOCIATES" },
	{ 0x00192B, "Aclara RF Systems Inc.", "ACLARA-RF" },
	{ 0x00192C, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00192D, "Nokia Corporation", "NOKIA" },
	{ 0x00192E, "Spectral Instruments, Inc.", "SPECTRAL-INSTRUMENTS" },
	{ 0x00192F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001930, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001931, "Balluff GmbH", "BALLUFF" },
	{ 0x001932, "Gude Analog- und Digialsysteme GmbH", "GUDE-ANALOG--UND-DIGIALSYSTEME" },
	{ 0x001933, "Strix Systems, Inc.", "STRIX" },
	{ 0x001934, "TRENDON TOUCH TECHNOLOGY CORP.", "TRENDON-TOUCH" },
	{ 0x001935, "DUERR DENTAL AG", "DUERR-DENTAL" },
	{ 0x001936, "STERLITE OPTICAL TECHNOLOGIES LIMITED", "STERLITE-OPTICAL-TECHNOLOGIES" },
	{ 0x001937, "CommerceGuard AB", "COMMERCEGUARD-AB" },
	{ 0x001938, "UMB Communications Co., Ltd.", "UMB-COMMUNICATION" },
	{ 0x001939, "Gigamips", "GIGAMIPS" },
	{ 0x00193A, "OESOLUTIONS", "OESOLUTIONS" },
	{ 0x00193B, "Wilibox Deliberant Group LLC", "WILIBOX-DELIBERANT-GROUP-LLC" },
	{ 0x00193C, "HighPoint Technologies Incorporated", "HIGHPOINT-TECHNOLOGIES-INCORPORATED" },
	{ 0x00193D, "GMC Guardian Mobility Corp.", "GMC-GUARDIAN-MOBILITY" },
	{ 0x00193E, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x00193F, "RDI technology(Shenzhen) Co.,LTD", "RDI-SHENZHEN" },
	{ 0x001940, "Rackable Systems", "RACKABLE" },
	{ 0x001941, "Pitney Bowes, Inc", "PITNEY-BOWES" },
	{ 0x001942, "ON SOFTWARE INTERNATIONAL LIMITED", "ON-SOFTWARE" },
	{ 0x001943, "Belden", "BELDEN" },
	{ 0x001944, "Fossil Partners, L.P.", "FOSSIL-PARTNERS-L-P" },
	{ 0x001945, "RF COncepts, LLC", "RF-CONCEPTS-LLC" },
	{ 0x001946, "Cianet Industria e Comercio S/A", "CIANET-INDUSTRIA-E-COMERCIO-S/A" },
	{ 0x001947, "Scientific Atlanta, A Cisco Company", "SCIENTIFIC-ATLANTA-A-CISCO-COMPANY" },
	{ 0x001948, "AireSpider Networks", "AIRESPIDER-NETWORKS" },
	{ 0x001949, "TENTEL COMTECH CO., LTD.", "TENTEL-COMTECH" },
	{ 0x00194A, "TESTO AG", "TESTO" },
	{ 0x00194B, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x00194C, "Fujian Stelcom information & Technology CO.,Ltd", "FUJIAN-STELCOM-INFORMATION" },
	{ 0x00194D, "Avago Technologies Sdn Bhd", "AVAGO-TECHNOLOGIES-SDN-BHD" },
	{ 0x00194E, "Ultra Electronics - TCS (Tactical Communication Systems)", "ULTRA-ELECTRONICS---TCS-TACTICAL-COMMUNICATION" },
	{ 0x00194F, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001950, "Harman Multimedia", "HARMAN-MULTIMEDIA" },
	{ 0x001951, "NETCONS, s.r.o.", "NETCONS-S-R-O" },
	{ 0x001952, "ACOGITO Co., Ltd", "ACOGITO" },
	{ 0x001953, "Chainleader Communications Corp.", "CHAINLEADER-COMMUNICATION" },
	{ 0x001954, "Leaf Corporation.", "LEAF" },
	{ 0x001955, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001956, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001957, "Saafnet Canada Inc.", "SAAFNET-CANADA" },
	{ 0x001958, "Bluetooth SIG, Inc.", "BLUETOOTH-SIG" },
	{ 0x001959, "Staccato Communications Inc.", "STACCATO-COMMUNICATION" },
	{ 0x00195A, "Jenaer Antriebstechnik GmbH", "JENAER-ANTRIEBSTECHNIK" },
	{ 0x00195B, "D-Link Corporation", "D-LINK" },
	{ 0x00195C, "Innotech Corporation", "INNOTECH" },
	{ 0x00195D, "ShenZhen XinHuaTong Opto Electronics Co.,Ltd", "SHENZHEN-XINHUATONG-OPTO-ELECTRONICS" },
	{ 0x00195E, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00195F, "Valemount Networks Corporation", "VALEMOUNT-NETWORKS" },
	{ 0x001960, "DoCoMo Systems, Inc.", "DOCOMO" },
	{ 0x001961, "Blaupunkt Embedded Systems GmbH", "BLAUPUNKT-EMBEDDED" },
	{ 0x001962, "Commerciant, LP", "COMMERCIANT-LP" },
	{ 0x001963, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x001964, "Doorking Inc.", "DOORKING" },
	{ 0x001965, "YuHua TelTech (ShangHai) Co., Ltd.", "YUHUA-TELTECH-SHANGHAI" },
	{ 0x001966, "Asiarock Technology Limited", "ASIAROCK" },
	{ 0x001967, "TELDAT Sp.J.", "TELDAT-SP-J" },
	{ 0x001968, "Digital Video Networks(Shanghai) CO. LTD.", "DIGITAL-VIDEO-NETWORKS-SHANGHAI" },
	{ 0x001969, "Nortel", "NORTEL" },
	{ 0x00196A, "MikroM GmbH", "MIKROM" },
	{ 0x00196B, "Danpex Corporation", "DANPEX" },
	{ 0x00196C, "ETROVISION TECHNOLOGY", "ETROVISION" },
	{ 0x00196D, "Raybit Systems Korea, Inc", "RAYBIT-KOREA" },
	{ 0x00196E, "Metacom (Pty) Ltd.", "METACOM-PTY" },
	{ 0x00196F, "SensoPart GmbH", "SENSOPART" },
	{ 0x001970, "Z-Com, Inc.", "Z-COM" },
	{ 0x001971, "Guangzhou Unicomp Technology Co.,Ltd", "GUANGZHOU-UNICOMP" },
	{ 0x001972, "Plexus (Xiamen) Co.,ltd", "PLEXUS-XIAMEN" },
	{ 0x001973, "Zeugma Systems", "ZEUGMA" },
	{ 0x001974, "AboCom Systems, Inc.", "ABOCOM" },
	{ 0x001975, "Beijing Huisen networks technology Inc", "BEIJING-HUISEN-NETWORKS" },
	{ 0x001976, "Xipher Technologies, LLC", "XIPHER-TECHNOLOGIES-LLC" },
	{ 0x001977, "Aerohive Networks, Inc.", "AEROHIVE-NETWORKS" },
	{ 0x001978, "Datum Systems, Inc.", "DATUM" },
	{ 0x001979, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x00197A, "MAZeT GmbH", "MAZET" },
	{ 0x00197B, "Picotest Corp.", "PICOTEST" },
	{ 0x00197C, "Riedel Communications GmbH", "RIEDEL-COMMUNICATION" },
	{ 0x00197D, "Hon Hai Precision Ind. Co., Ltd", "HON-HAI-PRECISION-IND" },
	{ 0x00197E, "Hon Hai Precision Ind. Co., Ltd", "HON-HAI-PRECISION-IND" },
	{ 0x00197F, "PLANTRONICS, INC.", "PLANTRONICS" },
	{ 0x001980, "Gridpoint Systems", "GRIDPOINT" },
	{ 0x001981, "Vivox Inc", "VIVOX" },
	{ 0x001982, "SmarDTV", "SMARDTV" },
	{ 0x001983, "CCT R&D Limited", "CCT-R-D" },
	{ 0x001984, "ESTIC Corporation", "ESTIC" },
	{ 0x001985, "IT Watchdogs, Inc", "IT-WATCHDOGS" },
	{ 0x001986, "Cheng Hongjian", "CHENG-HONGJIAN" },
	{ 0x001987, "Panasonic Mobile Communications Co., Ltd.", "PANASONIC-MOBILE-COMMUNICATION" },
	{ 0x001988, "Wi2Wi, Inc", "WI2WI" },
	{ 0x001989, "Sonitrol Corporation", "SONITROL" },
	{ 0x00198A, "Northrop Grumman Systems Corp.", "NORTHROP-GRUMMAN" },
	{ 0x00198B, "Novera Optics Korea, Inc.", "NOVERA-OPTICS-KOREA" },
	{ 0x00198C, "iXSea", "IXSEA" },
	{ 0x00198D, "Ocean Optics, Inc.", "OCEAN-OPTICS" },
	{ 0x00198E, "Oticon A/S", "OTICON-A/S" },
	{ 0x00198F, "Alcatel Bell N.V.", "ALCATEL-BELL-N-V" },
	{ 0x001990, "ELM DATA Co., Ltd.", "ELM-DATA" },
	{ 0x001991, "avinfo", "AVINFO" },
	{ 0x001992, "ADTRAN INC.", "ADTRAN" },
	{ 0x001993, "Changshu Switchgear MFG. Co.,Ltd. (Former Changshu Switchgea", "CHANGSHU-SWITCHGEAR-MFG-FORMER-CHANGSHU-SWITCHGEA" },
	{ 0x001994, "Jorjin Technologies Inc.", "JORJIN-TECHNOLOGIES" },
	{ 0x001995, "Jurong Hi-Tech (Suzhou)Co.ltd", "JURONG-HI-TECH-SUZHOU" },
	{ 0x001996, "TurboChef Technologies Inc.", "TURBOCHEF-TECHNOLOGIES" },
	{ 0x001997, "Soft Device Sdn Bhd", "SOFT-DEVICE-SDN-BHD" },
	{ 0x001998, "SATO CORPORATION", "SATO" },
	{ 0x001999, "Fujitsu Technology Solutions", "FUJITSU-SOLUTIONS" },
	{ 0x00199A, "EDO-EVI", "EDO-EVI" },
	{ 0x00199B, "Diversified Technical Systems, Inc.", "DIVERSIFIED-TECHNICAL" },
	{ 0x00199C, "CTRING", "CTRING" },
	{ 0x00199D, "VIZIO, Inc.", "VIZIO" },
	{ 0x00199E, "Nifty", "NIFTY" },
	{ 0x00199F, "DKT A/S", "DKT-A/S" },
	{ 0x0019A0, "NIHON DATA SYSTENS, INC.", "NIHON-DATA-SYSTENS" },
	{ 0x0019A1, "LG INFORMATION & COMM.", "LG-INFORMATION-COMM" },
	{ 0x0019A2, "ORDYN TECHNOLOGIES", "ORDYN-TECHNOLOGIES" },
	{ 0x0019A3, "asteel electronique atlantique", "ASTEEL-ELECTRONIQUE-ATLANTIQUE" },
	{ 0x0019A4, "Austar Technology (hang zhou) Co.,Ltd", "AUSTAR-HANG-ZHOU" },
	{ 0x0019A5, "RadarFind Corporation", "RADARFIND" },
	{ 0x0019A6, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0019A7, "ITU-T", "ITU-T" },
	{ 0x0019A8, "WiQuest Communications", "WIQUEST-COMMUNICATION" },
	{ 0x0019A9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0019AA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0019AB, "Raycom CO ., LTD", "RAYCOM" },
	{ 0x0019AC, "GSP SYSTEMS Inc.", "GSP" },
	{ 0x0019AD, "BOBST SA", "BOBST" },
	{ 0x0019AE, "Hopling Technologies b.v.", "HOPLING-TECHNOLOGIES" },
	{ 0x0019AF, "Rigol Technologies, Inc.", "RIGOL-TECHNOLOGIES" },
	{ 0x0019B0, "HanYang System", "HANYANG-SYSTEM" },
	{ 0x0019B1, "Arrow7 Corporation", "ARROW7" },
	{ 0x0019B2, "XYnetsoft Co.,Ltd", "XYNETSOFT" },
	{ 0x0019B3, "Stanford Research Systems", "STANFORD-RESEARCH" },
	{ 0x0019B4, "Intellio Ltd", "INTELLIO" },
	{ 0x0019B5, "Famar Fueguina S.A.", "FAMAR-FUEGUINA" },
	{ 0x0019B6, "Euro Emme s.r.l.", "EURO-EMME-S-R-L" },
	{ 0x0019B7, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0019B8, "Boundary Devices", "BOUNDARY-DEVICES" },
	{ 0x0019B9, "Dell Inc.", "DELL" },
	{ 0x0019BA, "Paradox Security Systems Ltd", "PARADOX-SECURITY" },
	{ 0x0019BB, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0019BC, "ELECTRO CHANCE SRL", "ELECTRO-CHANCE-SRL" },
	{ 0x0019BD, "New Media Life", "NEW-MEDIA-LIFE" },
	{ 0x0019BE, "Altai Technologies Limited", "ALTAI-TECHNOLOGIES" },
	{ 0x0019BF, "Citiway technology Co.,ltd", "CITIWAY" },
	{ 0x0019C0, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0019C1, "Alps Electric Co., Ltd", "ALPS-ELECTRIC" },
	{ 0x0019C2, "Equustek Solutions, Inc.", "EQUUSTEK-SOLUTIONS" },
	{ 0x0019C3, "Qualitrol", "QUALITROL" },
	{ 0x0019C4, "Infocrypt Inc.", "INFOCRYPT" },
	{ 0x0019C5, "SONY Computer Entertainment inc,", "SONY-ENTERTAINMENT" },
	{ 0x0019C6, "ZTE Corporation", "ZTE" },
	{ 0x0019C7, "Cambridge Industries(Group) Co.,Ltd.", "CAMBRIDGE-INDUSTRIES-GROUP" },
	{ 0x0019C8, "AnyDATA Corporation", "ANYDATA" },
	{ 0x0019C9, "S&C ELECTRIC COMPANY", "S-C-ELECTRIC-COMPANY" },
	{ 0x0019CA, "Broadata Communications, Inc", "BROADATA-COMMUNICATION" },
	{ 0x0019CB, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0x0019CC, "RCG (HK) Ltd", "RCG-HK" },
	{ 0x0019CD, "Chengdu ethercom information technology Ltd.", "CHENGDU-ETHERCOM-INFORMATION" },
	{ 0x0019CE, "Progressive Gaming International", "PROGRESSIVE-GAMING" },
	{ 0x0019CF, "SALICRU, S.A.", "SALICRU" },
	{ 0x0019D0, "Cathexis", "CATHEXIS" },
	{ 0x0019D1, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0019D2, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0019D3, "TRAK Microwave", "TRAK-MICROWAVE" },
	{ 0x0019D4, "ICX Technologies", "ICX-TECHNOLOGIES" },
	{ 0x0019D5, "IP Innovations, Inc.", "IP-INNOVATIONS" },
	{ 0x0019D6, "LS Cable and System Ltd.", "LS-CABLE-AND-SYSTEM" },
	{ 0x0019D7, "FORTUNETEK CO., LTD", "FORTUNETEK" },
	{ 0x0019D8, "MAXFOR", "MAXFOR" },
	{ 0x0019D9, "Zeutschel GmbH", "ZEUTSCHEL" },
	{ 0x0019DA, "Welltrans O&E Technology Co. , Ltd.", "WELLTRANS-O-E" },
	{ 0x0019DB, "MICRO-STAR INTERNATIONAL CO., LTD.", "MICRO-STAR" },
	{ 0x0019DC, "ENENSYS Technologies", "ENENSYS-TECHNOLOGIES" },
	{ 0x0019DD, "FEI-Zyfer, Inc.", "FEI-ZYFER" },
	{ 0x0019DE, "MOBITEK", "MOBITEK" },
	{ 0x0019DF, "Thomson Inc.", "THOMSON" },
	{ 0x0019E0, "TP-LINK Technologies Co., Ltd.", "TP-LINK-TECHNOLOGIES" },
	{ 0x0019E1, "Nortel", "NORTEL" },
	{ 0x0019E2, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x0019E3, "Apple", "APPLE" },
	{ 0x0019E4, "2Wire, Inc", "2WIRE" },
	{ 0x0019E5, "Lynx Studio Technology, Inc.", "LYNX-STUDIO" },
	{ 0x0019E6, "TOYO MEDIC CO.,LTD.", "TOYO-MEDIC" },
	{ 0x0019E7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0019E8, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0019E9, "S-Information Technolgy, Co., Ltd.", "S-INFORMATION-TECHNOLGY" },
	{ 0x0019EA, "TeraMage Technologies Co., Ltd.", "TERAMAGE-TECHNOLOGIES" },
	{ 0x0019EB, "Pyronix Ltd", "PYRONIX" },
	{ 0x0019EC, "Sagamore Systems, Inc.", "SAGAMORE" },
	{ 0x0019ED, "Axesstel Inc.", "AXESSTEL" },
	{ 0x0019EE, "CARLO GAVAZZI CONTROLS SPA-Controls Division", "CARLO-GAVAZZI-CONTROLS-SPA-CONTROLS-DIVISION" },
	{ 0x0019EF, "SHENZHEN LINNKING ELECTRONICS CO.,LTD", "SHENZHEN-LINNKING-ELECTRONICS" },
	{ 0x0019F0, "UNIONMAN TECHNOLOGY CO.,LTD", "UNIONMAN" },
	{ 0x0019F1, "Star Communication Network Technology Co.,Ltd", "STAR-COMMUNICATION-NETWORK" },
	{ 0x0019F2, "Teradyne K.K.", "TERADYNE-K-K" },
	{ 0x0019F3, "Cetis, Inc", "CETIS" },
	{ 0x0019F4, "Convergens Oy Ltd", "CONVERGENS-OY" },
	{ 0x0019F5, "Imagination Technologies Ltd", "IMAGINATION-TECHNOLOGIES" },
	{ 0x0019F6, "Acconet (PTE) Ltd", "ACCONET-PTE" },
	{ 0x0019F7, "Onset Computer Corporation", "ONSET" },
	{ 0x0019F8, "Embedded Systems Design, Inc.", "EMBEDDED-DESIGN" },
	{ 0x0019F9, "TDK-Lambda", "TDK-LAMBDA" },
	{ 0x0019FA, "Cable Vision Electronics CO., LTD.", "CABLE-VISION-ELECTRONICS" },
	{ 0x0019FB, "BSkyB Ltd", "BSKYB" },
	{ 0x0019FC, "PT. Ufoakses Sukses Luarbiasa", "PT-UFOAKSES-SUKSES-LUARBIASA" },
	{ 0x0019FD, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x0019FE, "SHENZHEN SEECOMM TECHNOLOGY CO.,LTD.", "SHENZHEN-SEECOMM" },
	{ 0x0019FF, "Finnzymes", "FINNZYMES" },
	{ 0x001A00, "MATRIX INC.", "MATRIX" },
	{ 0x001A01, "Smiths Medical", "SMITHS-MEDICAL" },
	{ 0x001A02, "SECURE CARE PRODUCTS, INC", "SECURE-CARE-PRODUCTS" },
	{ 0x001A03, "Angel Electronics Co., Ltd.", "ANGEL-ELECTRONICS" },
	{ 0x001A04, "Interay Solutions BV", "INTERAY-SOLUTIONS" },
	{ 0x001A05, "OPTIBASE LTD", "OPTIBASE" },
	{ 0x001A06, "OpVista, Inc.", "OPVISTA" },
	{ 0x001A07, "Arecont Vision", "ARECONT-VISION" },
	{ 0x001A08, "Simoco Ltd.", "SIMOCO" },
	{ 0x001A09, "Wayfarer Transit Systems Ltd", "WAYFARER-TRANSIT" },
	{ 0x001A0A, "Adaptive Micro-Ware Inc.", "ADAPTIVE-MICRO-WARE" },
	{ 0x001A0B, "BONA TECHNOLOGY INC.", "BONA" },
	{ 0x001A0C, "Swe-Dish Satellite Systems AB", "SWE-DISH-SATELLITE-AB" },
	{ 0x001A0D, "HandHeld entertainment, Inc.", "HANDHELD-ENTERTAINMENT" },
	{ 0x001A0E, "Cheng Uei Precision Industry Co.,Ltd", "CHENG-UEI-PRECISION-INDUSTRY" },
	{ 0x001A0F, "Sistemas Avanzados de Control, S.A.", "SISTEMAS-AVANZADOS-DE-CONTROL" },
	{ 0x001A10, "LUCENT TRANS ELECTRONICS CO.,LTD", "LUCENT-TRANS-ELECTRONICS" },
	{ 0x001A11, "Google Inc.", "GOOGLE" },
	{ 0x001A12, "Essilor", "ESSILOR" },
	{ 0x001A13, "Wanlida Group Co., LTD", "WANLIDA-GROUP" },
	{ 0x001A14, "Xin Hua Control Engineering Co.,Ltd.", "XIN-HUA-CONTROL-ENGINEERING" },
	{ 0x001A15, "gemalto e-Payment", "GEMALTO-E-PAYMENT" },
	{ 0x001A16, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001A17, "Teak Technologies, Inc.", "TEAK-TECHNOLOGIES" },
	{ 0x001A18, "Advanced Simulation Technology inc.", "ADVANCED-SIMULATION" },
	{ 0x001A19, "Computer Engineering Limited", "ENGINEERING" },
	{ 0x001A1A, "Gentex Corporation/Electro-Acoustic Products", "GENTEX-/ELECTRO-ACOUSTIC-PRODUCTS" },
	{ 0x001A1B, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001A1C, "GT&T Engineering Pte Ltd", "GT-T-ENGINEERING-PTE" },
	{ 0x001A1D, "PChome Online Inc.", "PCHOME-ONLINE" },
	{ 0x001A1E, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0x001A1F, "Coastal Environmental Systems", "COASTAL-ENVIRONMENTAL" },
	{ 0x001A20, "CMOTECH Co. Ltd.", "CMOTECH" },
	{ 0x001A21, "Indac B.V.", "INDAC" },
	{ 0x001A22, "eQ-3 Entwicklung GmbH", "EQ-3-ENTWICKLUNG" },
	{ 0x001A23, "Ice Qube, Inc", "ICE-QUBE" },
	{ 0x001A24, "Galaxy Telecom Technologies Ltd", "GALAXY-TELECOM-TECHNOLOGIES" },
	{ 0x001A25, "DELTA DORE", "DELTA-DORE" },
	{ 0x001A26, "Deltanode Solutions AB", "DELTANODE-SOLUTIONS-AB" },
	{ 0x001A27, "Ubistar", "UBISTAR" },
	{ 0x001A28, "ASWT Co., LTD. Taiwan Branch H.K.", "ASWT-TAIWAN-BRANCH-H-K" },
	{ 0x001A29, "Johnson Outdoors Marine Electronics, Inc", "JOHNSON-OUTDOORS-MARINE-ELECTRONICS" },
	{ 0x001A2A, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x001A2B, "Ayecom Technology Co., Ltd.", "AYECOM" },
	{ 0x001A2C, "SATEC Co.,LTD", "SATEC" },
	{ 0x001A2D, "The Navvo Group", "THE-NAVVO-GROUP" },
	{ 0x001A2E, "Ziova Coporation", "ZIOVA-COPORATION" },
	{ 0x001A2F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001A30, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001A31, "SCAN COIN Industries AB", "SCAN-COIN-INDUSTRIES-AB" },
	{ 0x001A32, "ACTIVA MULTIMEDIA", "ACTIVA-MULTIMEDIA" },
	{ 0x001A33, "ASI Communications, Inc.", "ASI-COMMUNICATION" },
	{ 0x001A34, "Konka Group Co., Ltd.", "KONKA-GROUP" },
	{ 0x001A35, "BARTEC GmbH", "BARTEC" },
	{ 0x001A36, "Aipermon GmbH & Co. KG", "AIPERMON" },
	{ 0x001A37, "Lear Corporation", "LEAR" },
	{ 0x001A38, "Sanmina-SCI", "SANMINA-SCI" },
	{ 0x001A39, "Merten GmbH&CoKG", "MERTEN-COKG" },
	{ 0x001A3A, "Dongahelecomm", "DONGAHELECOMM" },
	{ 0x001A3B, "Doah Elecom Inc.", "DOAH-ELECOM" },
	{ 0x001A3C, "Technowave Ltd.", "TECHNOWAVE" },
	{ 0x001A3D, "Ajin Vision Co.,Ltd", "AJIN-VISION" },
	{ 0x001A3E, "Faster Technology LLC", "FASTER-LLC" },
	{ 0x001A3F, "intelbras", "INTELBRAS" },
	{ 0x001A40, "A-FOUR TECH CO., LTD.", "A-FOUR-TECH" },
	{ 0x001A41, "INOCOVA Co.,Ltd", "INOCOVA" },
	{ 0x001A42, "Techcity Technology co., Ltd.", "TECHCITY" },
	{ 0x001A43, "Logical Link Communications", "LOGICAL-LINK-COMMUNICATION" },
	{ 0x001A44, "JWTrading Co., Ltd", "JWTRADING" },
	{ 0x001A45, "GN Netcom as", "GN-NETCOM-AS" },
	{ 0x001A46, "Digital Multimedia Technology Co., Ltd", "DIGITAL-MULTIMEDIA" },
	{ 0x001A47, "Agami Systems, Inc.", "AGAMI" },
	{ 0x001A48, "Takacom Corporation", "TAKACOM" },
	{ 0x001A49, "Micro Vision Co.,LTD", "MICRO-VISION" },
	{ 0x001A4A, "Qumranet Inc.", "QUMRANET" },
	{ 0x001A4B, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001A4C, "Crossbow Technology, Inc", "CROSSBOW" },
	{ 0x001A4D, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x001A4E, "NTI AG / LinMot", "NTI-/-LINMOT" },
	{ 0x001A4F, "AVM GmbH", "AVM" },
	{ 0x001A50, "PheeNet Technology Corp.", "PHEENET" },
	{ 0x001A51, "Alfred Mann Foundation", "ALFRED-MANN-FOUNDATION" },
	{ 0x001A52, "Meshlinx Wireless Inc.", "MESHLINX-WIRELESS" },
	{ 0x001A53, "Zylaya", "ZYLAYA" },
	{ 0x001A54, "Hip Shing Electronics Ltd.", "HIP-SHING-ELECTRONICS" },
	{ 0x001A55, "ACA-Digital Corporation", "ACA-DIGITAL" },
	{ 0x001A56, "ViewTel Co,. Ltd.", "VIEWTEL" },
	{ 0x001A57, "Matrix Design Group, LLC", "MATRIX-DESIGN-GROUP-LLC" },
	{ 0x001A58, "CCV Deutschland GmbH - Celectronic eHealth Div.", "CCV-DEUTSCHLAND---CELECTRONIC-EHEALTH-DIV" },
	{ 0x001A59, "Ircona", "IRCONA" },
	{ 0x001A5A, "Korea Electric Power Data Network (KDN) Co., Ltd", "KOREA-ELECTRIC-POWER-DATA-NETWORK-KDN" },
	{ 0x001A5B, "NetCare Service Co., Ltd.", "NETCARE-SERVICE" },
	{ 0x001A5C, "Euchner GmbH+Co. KG", "EUCHNER-+" },
	{ 0x001A5D, "Mobinnova Corp.", "MOBINNOVA" },
	{ 0x001A5E, "Thincom Technology Co.,Ltd", "THINCOM" },
	{ 0x001A5F, "KitWorks.fi Ltd.", "KITWORKS-FI" },
	{ 0x001A60, "Wave Electronics Co.,Ltd.", "WAVE-ELECTRONICS" },
	{ 0x001A61, "PacStar Corp.", "PACSTAR" },
	{ 0x001A62, "Data Robotics, Incorporated", "DATA-ROBOTICS-INCORPORATED" },
	{ 0x001A63, "Elster Solutions, LLC,", "ELSTER-SOLUTIONS-LLC" },
	{ 0x001A64, "IBM Corp", "IBM" },
	{ 0x001A65, "Seluxit", "SELUXIT" },
	{ 0x001A66, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001A67, "Infinite QL Sdn Bhd", "INFINITE-QL-SDN-BHD" },
	{ 0x001A68, "Weltec Enterprise Co., Ltd.", "WELTEC" },
	{ 0x001A69, "Wuhan Yangtze Optical Technology CO.,Ltd.", "WUHAN-YANGTZE-OPTICAL" },
	{ 0x001A6A, "Tranzas, Inc.", "TRANZAS" },
	{ 0x001A6B, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x001A6C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001A6D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001A6E, "Impro Technologies", "IMPRO-TECHNOLOGIES" },
	{ 0x001A6F, "MI.TEL s.r.l.", "MI-TEL-S-R-L" },
	{ 0x001A70, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x001A71, "Diostech Co., Ltd.", "DIOSTECH" },
	{ 0x001A72, "Mosart Semiconductor Corp.", "MOSART-SEMICONDUCTOR" },
	{ 0x001A73, "Gemtek Technology Co., Ltd.", "GEMTEK" },
	{ 0x001A74, "Procare International Co", "PROCARE" },
	{ 0x001A75, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x001A76, "SDT information Technology Co.,LTD.", "SDT-INFORMATION" },
	{ 0x001A77, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001A78, "ubtos", "UBTOS" },
	{ 0x001A79, "TELECOMUNICATION TECHNOLOGIES LTD.", "TELECOMUNICATION-TECHNOLOGIES" },
	{ 0x001A7A, "Lismore Instruments Limited", "LISMORE-INSTRUMENTS" },
	{ 0x001A7B, "Teleco, Inc.", "TELECO" },
	{ 0x001A7C, "Hirschmann Multimedia B.V.", "HIRSCHMANN-MULTIMEDIA" },
	{ 0x001A7D, "cyber-blue(HK)Ltd", "CYBER-BLUE-HK" },
	{ 0x001A7E, "LN Srithai Comm Ltd.", "LN-SRITHAI-COMM" },
	{ 0x001A7F, "GCI Science&Technology Co.,Ltd.", "GCI-SCIENCE" },
	{ 0x001A80, "Sony Corporation", "SONY" },
	{ 0x001A81, "Zelax", "ZELAX" },
	{ 0x001A82, "PROBA Building Automation Co.,LTD", "PROBA-BUILDING-AUTOMATION" },
	{ 0x001A83, "Pegasus Technologies Inc.", "PEGASUS-TECHNOLOGIES" },
	{ 0x001A84, "V One Multimedia Pte Ltd", "V-ONE-MULTIMEDIA-PTE" },
	{ 0x001A85, "NV Michel Van de Wiele", "NV-MICHEL-VAN-DE-WIELE" },
	{ 0x001A86, "AdvancedIO Systems Inc", "ADVANCEDIO" },
	{ 0x001A87, "Canhold International Limited", "CANHOLD" },
	{ 0x001A88, "Venergy,Co,Ltd", "VENERGY" },
	{ 0x001A89, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001A8A, "Samsung Electronics Co., Ltd.", "SAMSUNG-ELECTRONICS" },
	{ 0x001A8B, "CHUNIL ELECTRIC IND., CO.", "CHUNIL-ELECTRIC-IND" },
	{ 0x001A8C, "Sophos Ltd", "SOPHOS" },
	{ 0x001A8D, "AVECS Bergen GmbH", "AVECS-BERGEN" },
	{ 0x001A8E, "3Way Networks Ltd", "3WAY-NETWORKS" },
	{ 0x001A8F, "Nortel", "NORTEL" },
	{ 0x001A90, "Trópico Sistemas e Telecomunicações da Amazônia LTDA.", "TRóPICO-SISTEMAS-E-TELECOMUNICAçõES-DA-AMAZôNIA-LTDA" },
	{ 0x001A91, "FusionDynamic Ltd.", "FUSIONDYNAMIC" },
	{ 0x001A92, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x001A93, "ERCO Leuchten GmbH", "ERCO-LEUCHTEN" },
	{ 0x001A94, "Votronic GmbH", "VOTRONIC" },
	{ 0x001A95, "Hisense Mobile Communications Technoligy Co.,Ltd.", "HISENSE-MOBILE-COMMUNICATION-TECHNOLIGY" },
	{ 0x001A96, "ECLER S.A.", "ECLER" },
	{ 0x001A97, "fitivision technology Inc.", "FITIVISION" },
	{ 0x001A98, "Asotel Communication Limited Taiwan Branch", "ASOTEL-COMMUNICATION-TAIWAN-BRANCH" },
	{ 0x001A99, "Smarty (HZ) Information Electronics Co., Ltd", "SMARTY-HZ-INFORMATION-ELECTRONICS" },
	{ 0x001A9A, "Skyworth Digital technology(shenzhen)co.ltd.", "SKYWORTH-DIGITAL-SHENZHEN" },
	{ 0x001A9B, "ADEC & Parter AG", "ADEC-PARTER" },
	{ 0x001A9C, "RightHand Technologies, Inc.", "RIGHTHAND-TECHNOLOGIES" },
	{ 0x001A9D, "Skipper Wireless, Inc.", "SKIPPER-WIRELESS" },
	{ 0x001A9E, "ICON Digital International Limited", "ICON-DIGITAL" },
	{ 0x001A9F, "A-Link Ltd", "A-LINK" },
	{ 0x001AA0, "Dell Inc", "DELL" },
	{ 0x001AA1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001AA2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001AA3, "DELORME", "DELORME" },
	{ 0x001AA4, "Future University-Hakodate", "FUTURE-UNIVERSITY-HAKODATE" },
	{ 0x001AA5, "BRN Phoenix", "BRN-PHOENIX" },
	{ 0x001AA6, "Telefunken Radio Communication Systems GmbH &CO.KG", "TELEFUNKEN-RADIO-COMMUNICATION" },
	{ 0x001AA7, "Torian Wireless", "TORIAN-WIRELESS" },
	{ 0x001AA8, "Mamiya Digital Imaging Co., Ltd.", "MAMIYA-DIGITAL-IMAGING" },
	{ 0x001AA9, "FUJIAN STAR-NET COMMUNICATION CO.,LTD", "FUJIAN-STAR-NET-COMMUNICATION" },
	{ 0x001AAA, "Analogic Corp.", "ANALOGIC" },
	{ 0x001AAB, "eWings s.r.l.", "EWINGS-S-R-L" },
	{ 0x001AAC, "Corelatus AB", "CORELATUS-AB" },
	{ 0x001AAD, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001AAE, "Savant Systems LLC", "SAVANT-LLC" },
	{ 0x001AAF, "BLUSENS TECHNOLOGY", "BLUSENS" },
	{ 0x001AB0, "Signal Networks Pvt. Ltd.,", "SIGNAL-NETWORKS-PVT" },
	{ 0x001AB1, "Asia Pacific Satellite Industries Co., Ltd.", "ASIA-PACIFIC-SATELLITE-INDUSTRIES" },
	{ 0x001AB2, "Cyber Solutions Inc.", "CYBER-SOLUTIONS" },
	{ 0x001AB3, "VISIONITE INC.", "VISIONITE" },
	{ 0x001AB4, "FFEI Ltd.", "FFEI" },
	{ 0x001AB5, "Home Network System", "HOME-NETWORK-SYSTEM" },
	{ 0x001AB6, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x001AB7, "Ethos Networks LTD.", "ETHOS-NETWORKS" },
	{ 0x001AB8, "Anseri Corporation", "ANSERI" },
	{ 0x001AB9, "PMC", "PMC" },
	{ 0x001ABA, "Caton Overseas Limited", "CATON-OVERSEAS" },
	{ 0x001ABB, "Fontal Technology Incorporation", "FONTAL-INCORPORATION" },
	{ 0x001ABC, "U4EA Technologies Ltd", "U4EA-TECHNOLOGIES" },
	{ 0x001ABD, "Impatica Inc.", "IMPATICA" },
	{ 0x001ABE, "COMPUTER HI-TECH INC.", "HI-TECH" },
	{ 0x001ABF, "TRUMPF Laser Marking Systems AG", "TRUMPF-LASER-MARKING" },
	{ 0x001AC0, "JOYBIEN TECHNOLOGIES CO., LTD.", "JOYBIEN-TECHNOLOGIES" },
	{ 0x001AC1, "3Com Ltd", "3COM" },
	{ 0x001AC2, "YEC Co.,Ltd.", "YEC" },
	{ 0x001AC3, "Scientific-Atlanta, Inc", "SCIENTIFIC-ATLANTA" },
	{ 0x001AC4, "2Wire, Inc", "2WIRE" },
	{ 0x001AC5, "BreakingPoint Systems, Inc.", "BREAKINGPOINT" },
	{ 0x001AC6, "Micro Control Designs", "MICRO-CONTROL-DESIGNS" },
	{ 0x001AC7, "UNIPOINT", "UNIPOINT" },
	{ 0x001AC8, "ISL (Instrumentation Scientifique de Laboratoire)", "ISL-INSTRUMENTATION-SCIENTIFIQUE-DE-LABORATOIRE" },
	{ 0x001AC9, "SUZUKEN CO.,LTD", "SUZUKEN" },
	{ 0x001ACA, "Tilera Corporation", "TILERA" },
	{ 0x001ACB, "Autocom Products Ltd", "AUTOCOM-PRODUCTS" },
	{ 0x001ACC, "Celestial Semiconductor, Ltd", "CELESTIAL-SEMICONDUCTOR" },
	{ 0x001ACD, "Tidel Engineering LP", "TIDEL-ENGINEERING-LP" },
	{ 0x001ACE, "YUPITERU CORPORATION", "YUPITERU" },
	{ 0x001ACF, "C.T. ELETTRONICA", "C-T-ELETTRONICA" },
	{ 0x001AD0, "Albis Technologies AG", "ALBIS-TECHNOLOGIES" },
	{ 0x001AD1, "FARGO CO., LTD.", "FARGO" },
	{ 0x001AD2, "Eletronica Nitron Ltda", "ELETRONICA-NITRON-LTDA" },
	{ 0x001AD3, "Vamp Ltd.", "VAMP" },
	{ 0x001AD4, "iPOX Technology Co., Ltd.", "IPOX" },
	{ 0x001AD5, "KMC CHAIN INDUSTRIAL CO., LTD.", "KMC-CHAIN-INDUSTRIAL" },
	{ 0x001AD6, "JIAGNSU AETNA ELECTRIC CO.,LTD", "JIAGNSU-AETNA-ELECTRIC" },
	{ 0x001AD7, "Christie Digital Systems, Inc.", "CHRISTIE-DIGITAL" },
	{ 0x001AD8, "AlsterAero GmbH", "ALSTERAERO" },
	{ 0x001AD9, "International Broadband Electric Communications, Inc.", "BROADBAND-ELECTRIC-COMMUNICATION" },
	{ 0x001ADA, "Biz-2-Me Inc.", "BIZ-2-ME" },
	{ 0x001ADB, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001ADC, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001ADD, "PePWave Ltd", "PEPWAVE" },
	{ 0x001ADE, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001ADF, "Interactivetv Pty Limited", "INTERACTIVETV-PTY" },
	{ 0x001AE0, "Mythology Tech Express Inc.", "MYTHOLOGY-TECH-EXPRESS" },
	{ 0x001AE1, "EDGE ACCESS INC", "EDGE-ACCESS" },
	{ 0x001AE2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001AE3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001AE4, "Medicis Technologies Corporation", "MEDICIS-TECHNOLOGIES" },
	{ 0x001AE5, "Mvox Technologies Inc.", "MVOX-TECHNOLOGIES" },
	{ 0x001AE6, "Atlanta Advanced Communications Holdings Limited", "ATLANTA-ADVANCED-COMMUNICATION-HOLDINGS" },
	{ 0x001AE7, "Aztek Networks, Inc.", "AZTEK-NETWORKS" },
	{ 0x001AE8, "Unify GmbH and Co KG", "UNIFY-AND" },
	{ 0x001AE9, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001AEA, "Radio Terminal Systems Pty Ltd", "RADIO-TERMINAL-PTY" },
	{ 0x001AEB, "Allied Telesis K.K.", "ALLIED-TELESIS-K-K" },
	{ 0x001AEC, "Keumbee Electronics Co.,Ltd.", "KEUMBEE-ELECTRONICS" },
	{ 0x001AED, "INCOTEC GmbH", "INCOTEC" },
	{ 0x001AEE, "Shenztech Ltd", "SHENZTECH" },
	{ 0x001AEF, "Loopcomm Technology, Inc.", "LOOPCOMM" },
	{ 0x001AF0, "Alcatel - IPD", "ALCATEL---IPD" },
	{ 0x001AF1, "Embedded Artists AB", "EMBEDDED-ARTISTS-AB" },
	{ 0x001AF2, "Dynavisions Schweiz AG", "DYNAVISIONS-SCHWEIZ" },
	{ 0x001AF3, "Samyoung Electronics", "SAMYOUNG-ELECTRONICS" },
	{ 0x001AF4, "Handreamnet", "HANDREAMNET" },
	{ 0x001AF5, "PENTAONE. CO., LTD.", "PENTAONE" },
	{ 0x001AF6, "Woven Systems, Inc.", "WOVEN" },
	{ 0x001AF7, "dataschalt e+a GmbH", "DATASCHALT-E+A" },
	{ 0x001AF8, "Copley Controls Corporation", "COPLEY-CONTROLS" },
	{ 0x001AF9, "AeroVIronment (AV Inc)", "AEROVIRONMENT-AV" },
	{ 0x001AFA, "Welch Allyn, Inc.", "WELCH-ALLYN" },
	{ 0x001AFB, "Joby Inc.", "JOBY" },
	{ 0x001AFC, "ModusLink Corporation", "MODUSLINK" },
	{ 0x001AFD, "EVOLIS", "EVOLIS" },
	{ 0x001AFE, "SOFACREAL", "SOFACREAL" },
	{ 0x001AFF, "Wizyoung Tech.", "WIZYOUNG-TECH" },
	{ 0x001B00, "Neopost Technologies", "NEOPOST-TECHNOLOGIES" },
	{ 0x001B01, "Applied Radio Technologies", "APPLIED-RADIO-TECHNOLOGIES" },
	{ 0x001B02, "ED Co.Ltd", "ED" },
	{ 0x001B03, "Action Technology (SZ) Co., Ltd", "ACTION-SZ" },
	{ 0x001B04, "Affinity International S.p.a", "AFFINITY-S-P-A" },
	{ 0x001B05, "YMC AG", "YMC" },
	{ 0x001B06, "Ateliers R. LAUMONIER", "ATELIERS-R-LAUMONIER" },
	{ 0x001B07, "Mendocino Software", "MENDOCINO-SOFTWARE" },
	{ 0x001B08, "Danfoss Drives A/S", "DANFOSS-DRIVES-A/S" },
	{ 0x001B09, "Matrix Telecom Pvt. Ltd.", "MATRIX-TELECOM-PVT" },
	{ 0x001B0A, "Intelligent Distributed Controls Ltd", "INTELLIGENT-DISTRIBUTED-CONTROLS" },
	{ 0x001B0B, "Phidgets Inc.", "PHIDGETS" },
	{ 0x001B0C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001B0D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001B0E, "InoTec GmbH Organisationssysteme", "INOTEC-ORGANISATIONSSYSTEME" },
	{ 0x001B0F, "Petratec", "PETRATEC" },
	{ 0x001B10, "ShenZhen Kang Hui Technology Co.,ltd", "SHENZHEN-KANG-HUI" },
	{ 0x001B11, "D-Link Corporation", "D-LINK" },
	{ 0x001B12, "Apprion", "APPRION" },
	{ 0x001B13, "Icron Technologies Corporation", "ICRON-TECHNOLOGIES" },
	{ 0x001B14, "Carex Lighting Equipment Factory", "CAREX-LIGHTING-EQUIPMENT-FACTORY" },
	{ 0x001B15, "Voxtel, Inc.", "VOXTEL" },
	{ 0x001B16, "Celtro Ltd.", "CELTRO" },
	{ 0x001B17, "Palo Alto Networks", "PALO-ALTO-NETWORKS" },
	{ 0x001B18, "Tsuken Electric Ind. Co.,Ltd", "TSUKEN-ELECTRIC-IND" },
	{ 0x001B19, "IEEE I&M Society TC9", "IEEE-I-M-SOCIETY-TC9" },
	{ 0x001B1A, "e-trees Japan, Inc.", "E-TREES-JAPAN" },
	{ 0x001B1B, "Siemens AG,", "SIEMENS" },
	{ 0x001B1C, "Coherent", "COHERENT" },
	{ 0x001B1D, "Phoenix International Co., Ltd", "PHOENIX" },
	{ 0x001B1E, "HART Communication Foundation", "HART-COMMUNICATION-FOUNDATION" },
	{ 0x001B1F, "DELTA - Danish Electronics, Light & Acoustics", "DELTA---DANISH-ELECTRONICS-LIGHT-ACOUSTICS" },
	{ 0x001B20, "TPine Technology", "TPINE" },
	{ 0x001B21, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001B22, "Palit Microsystems ( H.K.) Ltd.", "PALIT-MICROSYSTEMS-H-K" },
	{ 0x001B23, "SimpleComTools", "SIMPLECOMTOOLS" },
	{ 0x001B24, "Quanta Computer Inc.", "QUANTA" },
	{ 0x001B25, "Nortel", "NORTEL" },
	{ 0x001B26, "RON-Telecom ZAO", "RON-TELECOM-ZAO" },
	{ 0x001B27, "Merlin CSI", "MERLIN-CSI" },
	{ 0x001B28, "POLYGON, JSC", "POLYGON-JSC" },
	{ 0x001B29, "Avantis.Co.,Ltd", "AVANTIS" },
	{ 0x001B2A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001B2B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001B2C, "ATRON electronic GmbH", "ATRON" },
	{ 0x001B2D, "Med-Eng Systems Inc.", "MED-ENG" },
	{ 0x001B2E, "Sinkyo Electron Inc", "SINKYO-ELECTRON" },
	{ 0x001B2F, "NETGEAR Inc.", "NETGEAR" },
	{ 0x001B30, "Solitech Inc.", "SOLITECH" },
	{ 0x001B31, "Neural Image. Co. Ltd.", "NEURAL-IMAGE" },
	{ 0x001B32, "QLogic Corporation", "QLOGIC" },
	{ 0x001B33, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001B34, "Focus System Inc.", "FOCUS-SYSTEM" },
	{ 0x001B35, "ChongQing JINOU Science & Technology Development CO.,Ltd", "CHONGQING-JINOU-SCIENCE-DEVELOPMENT" },
	{ 0x001B36, "Tsubata Engineering Co.,Ltd. (Head Office)", "TSUBATA-ENGINEERING-HEAD-OFFICE" },
	{ 0x001B37, "Computec Oy", "COMPUTEC-OY" },
	{ 0x001B38, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x001B39, "Proxicast", "PROXICAST" },
	{ 0x001B3A, "SIMS Corp.", "SIMS" },
	{ 0x001B3B, "Yi-Qing CO., LTD", "YI-QING" },
	{ 0x001B3C, "Software Technologies Group,Inc.", "SOFTWARE-TECHNOLOGIES-GROUP" },
	{ 0x001B3D, "EuroTel Spa", "EUROTEL-SPA" },
	{ 0x001B3E, "Curtis, Inc.", "CURTIS" },
	{ 0x001B3F, "ProCurve Networking by HP", "PROCURVE-NETWORKING-BY-HP" },
	{ 0x001B40, "Network Automation mxc AB", "NETWORK-AUTOMATION-MXC-AB" },
	{ 0x001B41, "General Infinity Co.,Ltd.", "GENERAL-INFINITY" },
	{ 0x001B42, "Wise & Blue", "WISE-BLUE" },
	{ 0x001B43, "Beijing DG Telecommunications equipment Co.,Ltd", "BEIJING-DG-TELECOMMUNICATIONS-EQUIPMENT" },
	{ 0x001B44, "SanDisk Corporation", "SANDISK" },
	{ 0x001B45, "ABB AS, Division Automation Products", "ABB-AS-DIVISION-AUTOMATION-PRODUCTS" },
	{ 0x001B46, "Blueone Technology Co.,Ltd", "BLUEONE" },
	{ 0x001B47, "Futarque A/S", "FUTARQUE-A/S" },
	{ 0x001B48, "Shenzhen Lantech Electronics Co., Ltd.", "SHENZHEN-LANTECH-ELECTRONICS" },
	{ 0x001B49, "Roberts Radio limited", "ROBERTS-RADIO" },
	{ 0x001B4A, "W&W Communications, Inc.", "W-W-COMMUNICATION" },
	{ 0x001B4B, "SANION Co., Ltd.", "SANION" },
	{ 0x001B4C, "Signtech", "SIGNTECH" },
	{ 0x001B4D, "Areca Technology Corporation", "ARECA" },
	{ 0x001B4E, "Navman New Zealand", "NAVMAN-NEW-ZEALAND" },
	{ 0x001B4F, "Avaya Inc.", "AVAYA" },
	{ 0x001B50, "Nizhny Novgorod Factory named after M.Frunze, FSUE (NZiF)", "NIZHNY-NOVGOROD-FACTORY-NAMED-AFTER-M-FRUNZE-FSUE-NZIF" },
	{ 0x001B51, "Vector Technology Corp.", "VECTOR" },
	{ 0x001B52, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001B53, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001B54, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001B55, "Hurco Automation Ltd.", "HURCO-AUTOMATION" },
	{ 0x001B56, "Tehuti Networks Ltd.", "TEHUTI-NETWORKS" },
	{ 0x001B57, "SEMINDIA SYSTEMS PRIVATE LIMITED", "SEMINDIA-PRIVATE" },
	{ 0x001B58, "ACE CAD Enterprise Co., Ltd.", "ACE-CAD" },
	{ 0x001B59, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x001B5A, "Apollo Imaging Technologies, Inc.", "APOLLO-IMAGING-TECHNOLOGIES" },
	{ 0x001B5B, "2Wire, Inc.", "2WIRE" },
	{ 0x001B5C, "Azuretec Co., Ltd.", "AZURETEC" },
	{ 0x001B5D, "Vololink Pty Ltd", "VOLOLINK-PTY" },
	{ 0x001B5E, "BPL Limited", "BPL" },
	{ 0x001B5F, "Alien Technology", "ALIEN" },
	{ 0x001B60, "NAVIGON AG", "NAVIGON" },
	{ 0x001B61, "Digital Acoustics, LLC", "DIGITAL-ACOUSTICS-LLC" },
	{ 0x001B62, "JHT Optoelectronics Co.,Ltd.", "JHT-OPTOELECTRONICS" },
	{ 0x001B63, "Apple", "APPLE" },
	{ 0x001B64, "IsaacLandKorea Co., Ltd,", "ISAACLANDKOREA" },
	{ 0x001B65, "China Gridcom Co., Ltd", "CHINA-GRIDCOM" },
	{ 0x001B66, "Sennheiser electronic GmbH & Co. KG", "SENNHEISER" },
	{ 0x001B67, "Cisco Systems Inc", "CISCO" },
	{ 0x001B68, "Modnnet Co., Ltd", "MODNNET" },
	{ 0x001B69, "Equaline Corporation", "EQUALINE" },
	{ 0x001B6A, "Powerwave Technologies Sweden AB", "POWERWAVE-TECHNOLOGIES-SWEDEN-AB" },
	{ 0x001B6B, "Swyx Solutions AG", "SWYX-SOLUTIONS" },
	{ 0x001B6C, "LookX Digital Media BV", "LOOKX-DIGITAL-MEDIA" },
	{ 0x001B6D, "Midtronics, Inc.", "MIDTRONICS" },
	{ 0x001B6E, "Anue Systems, Inc.", "ANUE" },
	{ 0x001B6F, "Teletrak Ltd", "TELETRAK" },
	{ 0x001B70, "IRI Ubiteq, INC.", "IRI-UBITEQ" },
	{ 0x001B71, "Telular Corp.", "TELULAR" },
	{ 0x001B72, "Sicep s.p.a.", "SICEP-S-P-A" },
	{ 0x001B73, "DTL Broadcast Ltd", "DTL-BROADCAST" },
	{ 0x001B74, "MiraLink Corporation", "MIRALINK" },
	{ 0x001B75, "Hypermedia Systems", "HYPERMEDIA" },
	{ 0x001B76, "Ripcode, Inc.", "RIPCODE" },
	{ 0x001B77, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001B78, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001B79, "FAIVELEY TRANSPORT", "FAIVELEY-TRANSPORT" },
	{ 0x001B7A, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001B7B, "The Tintometer Ltd", "THE-TINTOMETER" },
	{ 0x001B7C, "A & R Cambridge", "A-R-CAMBRIDGE" },
	{ 0x001B7D, "CXR Anderson Jacobson", "CXR-ANDERSON-JACOBSON" },
	{ 0x001B7E, "Beckmann GmbH", "BECKMANN" },
	{ 0x001B7F, "TMN Technologies Telecomunicacoes Ltda", "TMN-TECHNOLOGIES-TELECOMUNICACOES-LTDA" },
	{ 0x001B80, "LORD Corporation", "LORD" },
	{ 0x001B81, "DATAQ Instruments, Inc.", "DATAQ-INSTRUMENTS" },
	{ 0x001B82, "Taiwan Semiconductor Co., Ltd.", "TAIWAN-SEMICONDUCTOR" },
	{ 0x001B83, "Finsoft Ltd", "FINSOFT" },
	{ 0x001B84, "Scan Engineering Telecom", "SCAN-ENGINEERING-TELECOM" },
	{ 0x001B85, "MAN Diesel SE", "MAN-DIESEL-SE" },
	{ 0x001B86, "Bosch Access Systems GmbH", "BOSCH-ACCESS" },
	{ 0x001B87, "Deepsound Tech. Co., Ltd", "DEEPSOUND-TECH" },
	{ 0x001B88, "Divinet Access Technologies Ltd", "DIVINET-ACCESS-TECHNOLOGIES" },
	{ 0x001B89, "EMZA Visual Sense Ltd.", "EMZA-VISUAL-SENSE" },
	{ 0x001B8A, "2M Electronic A/S", "2M-A/S" },
	{ 0x001B8B, "NEC Platforms, Ltd.", "NEC-PLATFORMS" },
	{ 0x001B8C, "JMicron Technology Corp.", "JMICRON" },
	{ 0x001B8D, "Electronic Computer Systems, Inc.", "" },
	{ 0x001B8E, "Hulu Sweden AB", "HULU-SWEDEN-AB" },
	{ 0x001B8F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001B90, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001B91, "EFKON AG", "EFKON" },
	{ 0x001B92, "l-acoustics", "L-ACOUSTICS" },
	{ 0x001B93, "JC Decaux SA DNT", "JC-DECAUX-DNT" },
	{ 0x001B94, "T.E.M.A. S.p.A.", "T-E-M-A-S-P-A" },
	{ 0x001B95, "VIDEO SYSTEMS SRL", "VIDEO-SRL" },
	{ 0x001B96, "General Sensing", "GENERAL-SENSING" },
	{ 0x001B97, "Violin Technologies", "VIOLIN-TECHNOLOGIES" },
	{ 0x001B98, "Samsung Electronics Co., Ltd.", "SAMSUNG-ELECTRONICS" },
	{ 0x001B99, "KS System GmbH", "KS-SYSTEM" },
	{ 0x001B9A, "Apollo Fire Detectors Ltd", "APOLLO-FIRE-DETECTORS" },
	{ 0x001B9B, "Hose-McCann Communications", "HOSE-MCCANN-COMMUNICATION" },
	{ 0x001B9C, "SATEL sp. z o.o.", "SATEL-SP-Z-O-O" },
	{ 0x001B9D, "Novus Security Sp. z o.o.", "NOVUS-SECURITY-SP-Z-O-O" },
	{ 0x001B9E, "ASKEY COMPUTER CORP", "ASKEY" },
	{ 0x001B9F, "Calyptech Pty Ltd", "CALYPTECH-PTY" },
	{ 0x001BA0, "Awox", "AWOX" },
	{ 0x001BA1, "Åmic AB", "ÅMIC-AB" },
	{ 0x001BA2, "IDS Imaging Development Systems GmbH", "IDS-IMAGING-DEVELOPMENT" },
	{ 0x001BA3, "Flexit Group GmbH", "FLEXIT-GROUP" },
	{ 0x001BA4, "S.A.E Afikim", "E-AFIKIM" },
	{ 0x001BA5, "MyungMin Systems, Inc.", "MYUNGMIN" },
	{ 0x001BA6, "intotech inc.", "INTOTECH" },
	{ 0x001BA7, "Lorica Solutions", "LORICA-SOLUTIONS" },
	{ 0x001BA8, "UBI&MOBI,.Inc", "UBI-MOBI" },
	{ 0x001BA9, "BROTHER INDUSTRIES, LTD.", "BROTHER-INDUSTRIES" },
	{ 0x001BAA, "XenICs nv", "XENICS-NV" },
	{ 0x001BAB, "Telchemy, Incorporated", "TELCHEMY-INCORPORATED" },
	{ 0x001BAC, "Curtiss Wright Controls Embedded Computing", "CURTISS-WRIGHT-CONTROLS-EMBEDDED-COMPUTING" },
	{ 0x001BAD, "iControl Incorporated", "ICONTROL-INCORPORATED" },
	{ 0x001BAE, "Micro Control Systems, Inc", "MICRO-CONTROL" },
	{ 0x001BAF, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001BB0, "BHARAT ELECTRONICS", "BHARAT-ELECTRONICS" },
	{ 0x001BB1, "Wistron Neweb Corp.", "WISTRON-NEWEB" },
	{ 0x001BB2, "Intellect International NV", "INTELLECT-NV" },
	{ 0x001BB3, "Condalo GmbH", "CONDALO" },
	{ 0x001BB4, "Airvod Limited", "AIRVOD" },
	{ 0x001BB5, "ZF Electronics GmbH", "ZF-ELECTRONICS" },
	{ 0x001BB6, "Bird Electronic Corp.", "BIRD" },
	{ 0x001BB7, "Alta Heights Technology Corp.", "ALTA-HEIGHTS" },
	{ 0x001BB8, "BLUEWAY ELECTRONIC CO;LTD", "BLUEWAY" },
	{ 0x001BB9, "Elitegroup Computer System Co.", "ELITEGROUP-SYSTEM" },
	{ 0x001BBA, "Nortel", "NORTEL" },
	{ 0x001BBB, "RFTech Co.,Ltd", "RFTECH" },
	{ 0x001BBC, "Silver Peak Systems, Inc.", "SILVER-PEAK" },
	{ 0x001BBD, "FMC Kongsberg Subsea AS", "FMC-KONGSBERG-SUBSEA-AS" },
	{ 0x001BBE, "ICOP Digital", "ICOP-DIGITAL" },
	{ 0x001BBF, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x001BC0, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x001BC1, "HOLUX Technology, Inc.", "HOLUX" },
	{ 0x001BC2, "Integrated Control Technology Limitied", "INTEGRATED-CONTROL-LIMITIED" },
	{ 0x001BC3, "Mobisolution Co.,Ltd", "MOBISOLUTION" },
	{ 0x001BC4, "Ultratec, Inc.", "ULTRATEC" },
	{ 0x001BC5, "IEEE REGISTRATION AUTHORITY - Please see OUI36/MA-S public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-OUI36/MA-S-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x001BC6, "Strato Rechenzentrum AG", "STRATO-RECHENZENTRUM" },
	{ 0x001BC7, "StarVedia Technology Inc.", "STARVEDIA" },
	{ 0x001BC8, "MIURA CO.,LTD", "MIURA" },
	{ 0x001BC9, "FSN DISPLAY INC", "FSN-DISPLAY" },
	{ 0x001BCA, "Beijing Run Technology LTD. Company", "BEIJING-RUN-COMPANY" },
	{ 0x001BCB, "PEMPEK SYSTEMS PTY LTD", "PEMPEK-PTY" },
	{ 0x001BCC, "KINGTEK CCTV ALLIANCE CO., LTD.", "KINGTEK-CCTV-ALLIANCE" },
	{ 0x001BCD, "DAVISCOMMS (S) PTE LTD", "DAVISCOMMS-S-PTE" },
	{ 0x001BCE, "Measurement Devices Ltd", "MEASUREMENT-DEVICES" },
	{ 0x001BCF, "Dataupia Corporation", "DATAUPIA" },
	{ 0x001BD0, "IDENTEC SOLUTIONS", "IDENTEC-SOLUTIONS" },
	{ 0x001BD1, "SOGESTMATIC", "SOGESTMATIC" },
	{ 0x001BD2, "ULTRA-X ASIA PACIFIC Inc.", "ULTRA-X-ASIA-PACIFIC" },
	{ 0x001BD3, "Panasonic Corp. AVC Company", "PANASONIC-AVC-COMPANY" },
	{ 0x001BD4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001BD5, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001BD6, "Kelvin Hughes Ltd", "KELVIN-HUGHES" },
	{ 0x001BD7, "Scientific Atlanta, A Cisco Company", "SCIENTIFIC-ATLANTA-A-CISCO-COMPANY" },
	{ 0x001BD8, "DVTel LTD", "DVTEL" },
	{ 0x001BD9, "Edgewater Computer Systems", "EDGEWATER" },
	{ 0x001BDA, "UTStarcom Inc", "UTSTARCOM" },
	{ 0x001BDB, "Valeo VECS", "VALEO-VECS" },
	{ 0x001BDC, "Vencer Co., Ltd.", "VENCER" },
	{ 0x001BDD, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001BDE, "Renkus-Heinz, Inc.", "RENKUS-HEINZ" },
	{ 0x001BDF, "Iskra Sistemi d.d.", "ISKRA-SISTEMI-D-D" },
	{ 0x001BE0, "TELENOT ELECTRONIC GmbH", "TELENOT" },
	{ 0x001BE1, "ViaLogy", "VIALOGY" },
	{ 0x001BE2, "AhnLab,Inc.", "AHNLAB" },
	{ 0x001BE3, "Health Hero Network, Inc.", "HEALTH-HERO-NETWORK" },
	{ 0x001BE4, "TOWNET SRL", "TOWNET-SRL" },
	{ 0x001BE5, "802automation Limited", "802AUTOMATION" },
	{ 0x001BE6, "VR AG", "VR" },
	{ 0x001BE7, "Postek Electronics Co., Ltd.", "POSTEK-ELECTRONICS" },
	{ 0x001BE8, "Ultratronik GmbH", "ULTRATRONIK" },
	{ 0x001BE9, "Broadcom Corporation", "BROADCOM" },
	{ 0x001BEA, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001BEB, "DMP Electronics INC.", "DMP-ELECTRONICS" },
	{ 0x001BEC, "Netio Technologies Co., Ltd", "NETIO-TECHNOLOGIES" },
	{ 0x001BED, "Brocade Communications Systems, Inc", "BROCADE-COMMUNICATION" },
	{ 0x001BEE, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001BEF, "Blossoms Digital Technology Co.,Ltd.", "BLOSSOMS-DIGITAL" },
	{ 0x001BF0, "Value Platforms Limited", "VALUE-PLATFORMS" },
	{ 0x001BF1, "Nanjing SilverNet Software Co., Ltd.", "NANJING-SILVERNET-SOFTWARE" },
	{ 0x001BF2, "KWORLD COMPUTER CO., LTD", "KWORLD" },
	{ 0x001BF3, "TRANSRADIO SenderSysteme Berlin AG", "TRANSRADIO-SENDERSYSTEME-BERLIN" },
	{ 0x001BF4, "KENWIN INDUSTRIAL(HK) LTD.", "KENWIN-INDUSTRIAL-HK" },
	{ 0x001BF5, "Tellink Sistemas de Telecomunicación S.L.", "TELLINK-SISTEMAS-DE-TELECOMUNICACIóN-S-L" },
	{ 0x001BF6, "CONWISE Technology Corporation Ltd.", "CONWISE" },
	{ 0x001BF7, "Lund IP Products AB", "LUND-IP-PRODUCTS-AB" },
	{ 0x001BF8, "Digitrax Inc.", "DIGITRAX" },
	{ 0x001BF9, "Intellitect Water Ltd", "INTELLITECT-WATER" },
	{ 0x001BFA, "G.i.N. mbH", "G-I-N-MBH" },
	{ 0x001BFB, "Alps Electric Co., Ltd", "ALPS-ELECTRIC" },
	{ 0x001BFC, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x001BFD, "Dignsys Inc.", "DIGNSYS" },
	{ 0x001BFE, "Zavio Inc.", "ZAVIO" },
	{ 0x001BFF, "Millennia Media inc.", "MILLENNIA-MEDIA" },
	{ 0x001C00, "Entry Point, LLC", "ENTRY-POINT-LLC" },
	{ 0x001C01, "ABB Oy Drives", "ABB-OY-DRIVES" },
	{ 0x001C02, "Pano Logic", "PANO-LOGIC" },
	{ 0x001C03, "Betty TV Technology AG", "BETTY-TV" },
	{ 0x001C04, "Airgain, Inc.", "AIRGAIN" },
	{ 0x001C05, "Nonin Medical Inc.", "NONIN-MEDICAL" },
	{ 0x001C06, "Siemens Numerical Control Ltd., Nanjing", "SIEMENS-NUMERICAL-CONTROL-NANJING" },
	{ 0x001C07, "Cwlinux Limited", "CWLINUX" },
	{ 0x001C08, "Echo360, Inc.", "ECHO360" },
	{ 0x001C09, "SAE Electronic Co.,Ltd.", "SAE" },
	{ 0x001C0A, "Shenzhen AEE Technology Co.,Ltd.", "SHENZHEN-AEE" },
	{ 0x001C0B, "SmartAnt Telecom", "SMARTANT-TELECOM" },
	{ 0x001C0C, "TANITA Corporation", "TANITA" },
	{ 0x001C0D, "G-Technology, Inc.", "G" },
	{ 0x001C0E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001C0F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001C10, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x001C11, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001C12, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001C13, "OPTSYS TECHNOLOGY CO., LTD.", "OPTSYS" },
	{ 0x001C14, "VMware, Inc", "VMWARE" },
	{ 0x001C15, "iPhotonix LLC", "IPHOTONIX-LLC" },
	{ 0x001C16, "ThyssenKrupp Elevator", "THYSSENKRUPP-ELEVATOR" },
	{ 0x001C17, "Nortel", "NORTEL" },
	{ 0x001C18, "Sicert S.r.L.", "SICERT-S-R-L" },
	{ 0x001C19, "secunet Security Networks AG", "SECUNET-SECURITY-NETWORKS" },
	{ 0x001C1A, "Thomas Instrumentation, Inc", "THOMAS-INSTRUMENTATION" },
	{ 0x001C1B, "Hyperstone GmbH", "HYPERSTONE" },
	{ 0x001C1C, "Center Communication Systems GmbH", "CENTER-COMMUNICATION" },
	{ 0x001C1D, "CHENZHOU GOSPELL DIGITAL TECHNOLOGY CO.,LTD", "CHENZHOU-GOSPELL-DIGITAL" },
	{ 0x001C1E, "emtrion GmbH", "EMTRION" },
	{ 0x001C1F, "Quest Retail Technology Pty Ltd", "QUEST-RETAIL-PTY" },
	{ 0x001C20, "CLB Benelux", "CLB-BENELUX" },
	{ 0x001C21, "Nucsafe Inc.", "NUCSAFE" },
	{ 0x001C22, "Aeris Elettronica s.r.l.", "AERIS-ELETTRONICA-S-R-L" },
	{ 0x001C23, "Dell Inc", "DELL" },
	{ 0x001C24, "Formosa Wireless Systems Corp.", "FORMOSA-WIRELESS" },
	{ 0x001C25, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x001C26, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x001C27, "Sunell Electronics Co.", "SUNELL-ELECTRONICS" },
	{ 0x001C28, "Sphairon Technologies GmbH", "SPHAIRON-TECHNOLOGIES" },
	{ 0x001C29, "CORE DIGITAL ELECTRONICS CO., LTD", "CORE-DIGITAL-ELECTRONICS" },
	{ 0x001C2A, "Envisacor Technologies Inc.", "ENVISACOR-TECHNOLOGIES" },
	{ 0x001C2B, "Alertme.com Limited", "ALERTME-COM" },
	{ 0x001C2C, "Synapse", "SYNAPSE" },
	{ 0x001C2D, "FlexRadio Systems", "FLEXRADIO" },
	{ 0x001C2E, "HPN Supply Chain", "HPN-SUPPLY-CHAIN" },
	{ 0x001C2F, "Pfister GmbH", "PFISTER" },
	{ 0x001C30, "Mode Lighting (UK ) Ltd.", "MODE-LIGHTING-UK" },
	{ 0x001C31, "Mobile XP Technology Co., LTD", "MOBILE-XP" },
	{ 0x001C32, "Telian Corporation", "TELIAN" },
	{ 0x001C33, "Sutron", "SUTRON" },
	{ 0x001C34, "HUEY CHIAO INTERNATIONAL CO., LTD.", "HUEY-CHIAO" },
	{ 0x001C35, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001C36, "iNEWiT NV", "INEWIT-NV" },
	{ 0x001C37, "Callpod, Inc.", "CALLPOD" },
	{ 0x001C38, "Bio-Rad Laboratories, Inc.", "BIO-RAD-LABORATORIES" },
	{ 0x001C39, "S Netsystems Inc.", "S-NETSYSTEMS" },
	{ 0x001C3A, "Element Labs, Inc.", "ELEMENT-LABS" },
	{ 0x001C3B, "AmRoad Technology Inc.", "AMROAD" },
	{ 0x001C3C, "Seon Design Inc.", "SEON-DESIGN" },
	{ 0x001C3D, "WaveStorm", "WAVESTORM" },
	{ 0x001C3E, "ECKey Corporation", "ECKEY" },
	{ 0x001C3F, "International Police Technologies, Inc.", "POLICE-TECHNOLOGIES" },
	{ 0x001C40, "VDG-Security bv", "VDG-SECURITY" },
	{ 0x001C41, "scemtec Transponder Technology GmbH", "SCEMTEC-TRANSPONDER" },
	{ 0x001C42, "Parallels, Inc. (possible Paralles Desktop, Workstation, Server, Virtuozzo VM)", "PARALLELS-VIRTUAL" },
	{ 0x001C43, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x001C44, "Bosch Security Systems BV", "BOSCH-SECURITY" },
	{ 0x001C45, "Chenbro Micom Co., Ltd.", "CHENBRO-MICOM" },
	{ 0x001C46, "QTUM", "QTUM" },
	{ 0x001C47, "Hangzhou Hollysys Automation Co., Ltd", "HANGZHOU-HOLLYSYS-AUTOMATION" },
	{ 0x001C48, "WiDeFi, Inc.", "WIDEFI" },
	{ 0x001C49, "Zoltan Technology Inc.", "ZOLTAN" },
	{ 0x001C4A, "AVM GmbH", "AVM" },
	{ 0x001C4B, "Gener8, Inc.", "GENER8" },
	{ 0x001C4C, "Petrotest Instruments", "PETROTEST-INSTRUMENTS" },
	{ 0x001C4D, "Aplix IP Holdings Corporation", "APLIX-IP-HOLDINGS" },
	{ 0x001C4E, "TASA International Limited", "TASA" },
	{ 0x001C4F, "MACAB AB", "MACAB-AB" },
	{ 0x001C50, "TCL Technoly Electronics(Huizhou)Co.,Ltd", "TCL-TECHNOLY-ELECTRONICS-HUIZHOU" },
	{ 0x001C51, "Celeno Communications", "CELENO-COMMUNICATION" },
	{ 0x001C52, "VISIONEE SRL", "VISIONEE-SRL" },
	{ 0x001C53, "Synergy Lighting Controls", "SYNERGY-LIGHTING-CONTROLS" },
	{ 0x001C54, "Hillstone Networks Inc", "HILLSTONE-NETWORKS" },
	{ 0x001C55, "Shenzhen Kaifa Technology Co.", "SHENZHEN-KAIFA" },
	{ 0x001C56, "Pado Systems, Inc.", "PADO" },
	{ 0x001C57, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001C58, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001C59, "DEVON IT", "DEVON-IT" },
	{ 0x001C5A, "Advanced Relay Corporation", "ADVANCED-RELAY" },
	{ 0x001C5B, "Chubb Electronic Security Systems Ltd", "CHUBB-SECURITY" },
	{ 0x001C5C, "Integrated Medical Systems, Inc.", "INTEGRATED-MEDICAL" },
	{ 0x001C5D, "Leica Microsystems", "LEICA-MICROSYSTEMS" },
	{ 0x001C5E, "ASTON France", "ASTON-FRANCE" },
	{ 0x001C5F, "Winland Electronics, Inc.", "WINLAND-ELECTRONICS" },
	{ 0x001C60, "CSP Frontier Technologies,Inc.", "CSP-FRONTIER-TECHNOLOGIES" },
	{ 0x001C61, "Galaxy Microsystems LImited", "GALAXY-MICROSYSTEMS" },
	{ 0x001C62, "LG Electronics Inc", "LG-ELECTRONICS" },
	{ 0x001C63, "TRUEN", "TRUEN" },
	{ 0x001C64, "Landis+Gyr", "LANDIS+GYR" },
	{ 0x001C65, "JoeScan, Inc.", "JOESCAN" },
	{ 0x001C66, "UCAMP CO.,LTD", "UCAMP" },
	{ 0x001C67, "Pumpkin Networks, Inc.", "PUMPKIN-NETWORKS" },
	{ 0x001C68, "Anhui Sun Create Electronics Co., Ltd", "ANHUI-SUN-CREATE-ELECTRONICS" },
	{ 0x001C69, "Packet Vision Ltd", "PACKET-VISION" },
	{ 0x001C6A, "Weiss Engineering Ltd.", "WEISS-ENGINEERING" },
	{ 0x001C6B, "COVAX Co. Ltd", "COVAX" },
	{ 0x001C6C, "Jabil Circuit (Guangzhou) Limited", "JABIL-CIRCUIT-GUANGZHOU" },
	{ 0x001C6D, "KYOHRITSU ELECTRONIC INDUSTRY CO., LTD.", "KYOHRITSU-INDUSTRY" },
	{ 0x001C6E, "Newbury Networks, Inc.", "NEWBURY-NETWORKS" },
	{ 0x001C6F, "Emfit Ltd", "EMFIT" },
	{ 0x001C70, "NOVACOMM LTDA", "NOVACOMM-LTDA" },
	{ 0x001C71, "Emergent Electronics", "EMERGENT-ELECTRONICS" },
	{ 0x001C72, "Mayer & Cie GmbH & Co KG", "MAYER-CIE" },
	{ 0x001C73, "Arista Networks, Inc.", "ARISTA-NETWORKS" },
	{ 0x001C74, "Syswan Technologies Inc.", "SYSWAN-TECHNOLOGIES" },
	{ 0x001C75, "Segnet Ltd.", "SEGNET" },
	{ 0x001C76, "The Wandsworth Group Ltd", "THE-WANDSWORTH-GROUP" },
	{ 0x001C77, "Prodys", "PRODYS" },
	{ 0x001C78, "WYPLAY SAS", "WYPLAY-SAS" },
	{ 0x001C79, "Cohesive Financial Technologies LLC", "COHESIVE-FINANCIAL-TECHNOLOGIES-LLC" },
	{ 0x001C7A, "Perfectone Netware Company Ltd", "PERFECTONE-NETWARE-COMPANY" },
	{ 0x001C7B, "Castlenet Technology Inc.", "CASTLENET" },
	{ 0x001C7C, "PERQ SYSTEMS CORPORATION", "PERQ" },
	{ 0x001C7D, "Excelpoint Manufacturing Pte Ltd", "EXCELPOINT-MANUFACTURING-PTE" },
	{ 0x001C7E, "Toshiba", "TOSHIBA" },
	{ 0x001C7F, "Check Point Software Technologies", "CHECK-POINT-SOFTWARE-TECHNOLOGIES" },
	{ 0x001C80, "New Business Division/Rhea-Information CO., LTD.", "NEW-BUSINESS-DIVISION/RHEA-INFORMATION" },
	{ 0x001C81, "NextGen Venturi LTD", "NEXTGEN-VENTURI" },
	{ 0x001C82, "Genew Technologies", "GENEW-TECHNOLOGIES" },
	{ 0x001C83, "New Level Telecom Co., Ltd.", "NEW-LEVEL-TELECOM" },
	{ 0x001C84, "STL Solution Co.,Ltd.", "STL-SOLUTION" },
	{ 0x001C85, "Eunicorn", "EUNICORN" },
	{ 0x001C86, "Cranite Systems, Inc.", "CRANITE" },
	{ 0x001C87, "Uriver Inc.", "URIVER" },
	{ 0x001C88, "TRANSYSTEM INC.", "TRANSYSTEM" },
	{ 0x001C89, "Force Communications, Inc.", "FORCE-COMMUNICATION" },
	{ 0x001C8A, "Cirrascale Corporation", "CIRRASCALE" },
	{ 0x001C8B, "MJ Innovations Ltd.", "MJ-INNOVATIONS" },
	{ 0x001C8C, "DIAL TECHNOLOGY LTD.", "DIAL" },
	{ 0x001C8D, "Mesa Imaging", "MESA-IMAGING" },
	{ 0x001C8E, "Alcatel-Lucent IPD", "ALCATEL-LUCENT-IPD" },
	{ 0x001C8F, "Advanced Electronic Design, Inc.", "ADVANCED-DESIGN" },
	{ 0x001C90, "Empacket Corporation", "EMPACKET" },
	{ 0x001C91, "Gefen Inc.", "GEFEN" },
	{ 0x001C92, "Tervela", "TERVELA" },
	{ 0x001C93, "ExaDigm Inc", "EXADIGM" },
	{ 0x001C94, "LI-COR Biosciences", "LI-COR-BIOSCIENCES" },
	{ 0x001C95, "Opticomm Corporation", "OPTICOMM" },
	{ 0x001C96, "Linkwise Technology Pte Ltd", "LINKWISE-PTE" },
	{ 0x001C97, "Enzytek Technology Inc.,", "ENZYTEK" },
	{ 0x001C98, "LUCKY TECHNOLOGY (HK) COMPANY LIMITED", "LUCKY-HK-COMPANY" },
	{ 0x001C99, "Shunra Software Ltd.", "SHUNRA-SOFTWARE" },
	{ 0x001C9A, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001C9B, "FEIG ELECTRONIC GmbH", "FEIG" },
	{ 0x001C9C, "Nortel", "NORTEL" },
	{ 0x001C9D, "Liecthi AG", "LIECTHI" },
	{ 0x001C9E, "Dualtech IT AB", "DUALTECH-IT-AB" },
	{ 0x001C9F, "Razorstream, LLC", "RAZORSTREAM-LLC" },
	{ 0x001CA0, "Production Resource Group, LLC", "PRODUCTION-RESOURCE-GROUP-LLC" },
	{ 0x001CA1, "AKAMAI TECHNOLOGIES, INC.", "AKAMAI-TECHNOLOGIES" },
	{ 0x001CA2, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x001CA3, "Terra", "TERRA" },
	{ 0x001CA4, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x001CA5, "Zygo Corporation", "ZYGO" },
	{ 0x001CA6, "Win4NET", "WIN4NET" },
	{ 0x001CA7, "International Quartz Limited", "QUARTZ" },
	{ 0x001CA8, "AirTies Wireless Networks", "AIRTIES-WIRELESS-NETWORKS" },
	{ 0x001CA9, "Audiomatica Srl", "AUDIOMATICA-SRL" },
	{ 0x001CAA, "Bellon Pty Ltd", "BELLON-PTY" },
	{ 0x001CAB, "Meyer Sound Laboratories, Inc.", "MEYER-SOUND-LABORATORIES" },
	{ 0x001CAC, "Qniq Technology Corp.", "QNIQ" },
	{ 0x001CAD, "Wuhan Telecommunication Devices Co.,Ltd", "WUHAN-TELECOMMUNICATION-DEVICES" },
	{ 0x001CAE, "WiChorus, Inc.", "WICHORUS" },
	{ 0x001CAF, "Plato Networks Inc.", "PLATO-NETWORKS" },
	{ 0x001CB0, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001CB1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001CB2, "BPT SPA", "BPT-SPA" },
	{ 0x001CB3, "Apple", "APPLE" },
	{ 0x001CB4, "Iridium Satellite LLC", "IRIDIUM-SATELLITE-LLC" },
	{ 0x001CB5, "Neihua Network Technology Co.,LTD.(NHN)", "NEIHUA-NETWORK-NHN" },
	{ 0x001CB6, "Duzon CNT Co., Ltd.", "DUZON-CNT" },
	{ 0x001CB7, "USC DigiArk Corporation", "USC-DIGIARK" },
	{ 0x001CB8, "CBC Co., Ltd", "CBC" },
	{ 0x001CB9, "KWANG SUNG ELECTRONICS CO., LTD.", "KWANG-SUNG-ELECTRONICS" },
	{ 0x001CBA, "VerScient, Inc.", "VERSCIENT" },
	{ 0x001CBB, "MusicianLink", "MUSICIANLINK" },
	{ 0x001CBC, "CastGrabber, LLC", "CASTGRABBER-LLC" },
	{ 0x001CBD, "Ezze Mobile Tech., Inc.", "EZZE-MOBILE-TECH" },
	{ 0x001CBE, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001CBF, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001CC0, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001CC1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001CC2, "Part II Research, Inc.", "PART-II-RESEARCH" },
	{ 0x001CC3, "Pace plc", "PACE-PLC" },
	{ 0x001CC4, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001CC5, "3COM LTD", "3COM" },
	{ 0x001CC6, "ProStor Systems", "PROSTOR" },
	{ 0x001CC7, "Rembrandt Technologies, LLC d/b/a REMSTREAM", "REMBRANDT-TECHNOLOGIES-LLC-D/B/A-REMSTREAM" },
	{ 0x001CC8, "INDUSTRONIC Industrie-Electronic GmbH & Co. KG", "INDUSTRONIC-INDUSTRIE" },
	{ 0x001CC9, "Kaise Electronic Technology Co., Ltd.", "KAISE" },
	{ 0x001CCA, "Shanghai Gaozhi Science & Technology Development Co.", "SHANGHAI-GAOZHI-SCIENCE-DEVELOPMENT" },
	{ 0x001CCB, "Forth Corporation Public Company Limited", "FORTH-PUBLIC-COMPANY" },
	{ 0x001CCC, "Research In Motion Limited", "RESEARCH-IN-MOTION" },
	{ 0x001CCD, "Alektrona Corporation", "ALEKTRONA" },
	{ 0x001CCE, "By Techdesign", "BY-TECHDESIGN" },
	{ 0x001CCF, "LIMETEK", "LIMETEK" },
	{ 0x001CD0, "Circleone Co.,Ltd.", "CIRCLEONE" },
	{ 0x001CD1, "Waves Audio LTD", "WAVES-AUDIO" },
	{ 0x001CD2, "King Champion (Hong Kong) Limited", "KING-CHAMPION-HONG-KONG" },
	{ 0x001CD3, "ZP Engineering SEL", "ZP-ENGINEERING-SEL" },
	{ 0x001CD4, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001CD5, "ZeeVee, Inc.", "ZEEVEE" },
	{ 0x001CD6, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001CD7, "Harman/Becker Automotive Systems GmbH", "HARMAN/BECKER-AUTOMOTIVE" },
	{ 0x001CD8, "BlueAnt Wireless", "BLUEANT-WIRELESS" },
	{ 0x001CD9, "GlobalTop Technology Inc.", "GLOBALTOP" },
	{ 0x001CDA, "Exegin Technologies Limited", "EXEGIN-TECHNOLOGIES" },
	{ 0x001CDB, "CARPOINT CO.,LTD", "CARPOINT" },
	{ 0x001CDC, "Custom Computer Services, Inc.", "CUSTOM-SERVICES" },
	{ 0x001CDD, "COWBELL ENGINEERING CO., LTD.", "COWBELL-ENGINEERING" },
	{ 0x001CDE, "Interactive Multimedia eXchange Inc.", "INTERACTIVE-MULTIMEDIA-EXCHANGE" },
	{ 0x001CDF, "Belkin International Inc.", "BELKIN" },
	{ 0x001CE0, "DASAN TPS", "DASAN-TPS" },
	{ 0x001CE1, "INDRA SISTEMAS, S.A.", "INDRA-SISTEMAS" },
	{ 0x001CE2, "Attero Tech, LLC.", "ATTERO-TECH-LLC" },
	{ 0x001CE3, "Optimedical Systems", "OPTIMEDICAL" },
	{ 0x001CE4, "EleSy JSC", "ELESY-JSC" },
	{ 0x001CE5, "MBS Electronic Systems GmbH", "MBS" },
	{ 0x001CE6, "INNES", "INNES" },
	{ 0x001CE7, "Rocon PLC Research Centre", "ROCON-PLC-RESEARCH-CENTRE" },
	{ 0x001CE8, "Cummins Inc", "CUMMINS" },
	{ 0x001CE9, "Galaxy Technology Limited", "GALAXY" },
	{ 0x001CEA, "Scientific-Atlanta, Inc", "SCIENTIFIC-ATLANTA" },
	{ 0x001CEB, "Nortel", "NORTEL" },
	{ 0x001CEC, "Mobilesoft (Aust.) Pty Ltd", "MOBILESOFT-AUST-PTY" },
	{ 0x001CED, "ENVIRONNEMENT SA", "ENVIRONNEMENT" },
	{ 0x001CEE, "SHARP Corporation", "SHARP" },
	{ 0x001CEF, "Primax Electronics LTD", "PRIMAX-ELECTRONICS" },
	{ 0x001CF0, "D-Link Corporation", "D-LINK" },
	{ 0x001CF1, "SUPoX Technology Co. , LTD.", "SUPOX" },
	{ 0x001CF2, "Tenlon Technology Co.,Ltd.", "TENLON" },
	{ 0x001CF3, "EVS BROADCAST EQUIPMENT", "EVS-BROADCAST-EQUIPMENT" },
	{ 0x001CF4, "Media Technology Systems Inc", "MEDIA" },
	{ 0x001CF5, "Wiseblue Technology Limited", "WISEBLUE" },
	{ 0x001CF6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001CF7, "AudioScience", "AUDIOSCIENCE" },
	{ 0x001CF8, "Parade Technologies, Ltd.", "PARADE-TECHNOLOGIES" },
	{ 0x001CF9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001CFA, "Alarm.com", "ALARM-COM" },
	{ 0x001CFB, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001CFC, "Suminet Communication Technologies (Shanghai) Co., Ltd.", "SUMINET-COMMUNICATION-TECHNOLOGIES-SHANGHAI" },
	{ 0x001CFD, "Universal Electronics", "UNIVERSAL-ELECTRONICS" },
	{ 0x001CFE, "Quartics Inc", "QUARTICS" },
	{ 0x001CFF, "Napera Networks Inc", "NAPERA-NETWORKS" },
	{ 0x001D00, "Brivo Systems, LLC", "BRIVO-LLC" },
	{ 0x001D01, "Neptune Digital", "NEPTUNE-DIGITAL" },
	{ 0x001D02, "Cybertech Telecom Development", "CYBERTECH-TELECOM-DEVELOPMENT" },
	{ 0x001D03, "Design Solutions Inc.", "DESIGN-SOLUTIONS" },
	{ 0x001D04, "Zipit Wireless, Inc.", "ZIPIT-WIRELESS" },
	{ 0x001D05, "Eaton Corporation", "EATON" },
	{ 0x001D06, "HM Electronics, Inc.", "HM-ELECTRONICS" },
	{ 0x001D07, "Shenzhen Sang Fei Consumer Communications Co.,Ltd", "SHENZHEN-SANG-FEI-CONSUMER-COMMUNICATION" },
	{ 0x001D08, "JIANGSU YINHE ELECTRONICS CO., LTD", "JIANGSU-YINHE-ELECTRONICS" },
	{ 0x001D09, "Dell Inc", "DELL" },
	{ 0x001D0A, "Davis Instruments, Inc.", "DAVIS-INSTRUMENTS" },
	{ 0x001D0B, "Power Standards Lab", "POWER-STANDARDS-LAB" },
	{ 0x001D0C, "MobileCompia", "MOBILECOMPIA" },
	{ 0x001D0D, "Sony Computer Entertainment inc.", "SONY-ENTERTAINMENT" },
	{ 0x001D0E, "Agapha Technology co., Ltd.", "AGAPHA" },
	{ 0x001D0F, "TP-LINK Technologies Co., Ltd.", "TP-LINK-TECHNOLOGIES" },
	{ 0x001D10, "LightHaus Logic, Inc.", "LIGHTHAUS-LOGIC" },
	{ 0x001D11, "Analogue & Micro Ltd", "ANALOGUE-MICRO" },
	{ 0x001D12, "ROHM CO., LTD.", "ROHM" },
	{ 0x001D13, "NextGTV", "NEXTGTV" },
	{ 0x001D14, "SPERADTONE INFORMATION TECHNOLOGY LIMITED", "SPERADTONE-INFORMATION" },
	{ 0x001D15, "Shenzhen Dolphin Electronic Co., Ltd", "SHENZHEN-DOLPHIN" },
	{ 0x001D16, "SFR", "SFR" },
	{ 0x001D17, "Digital Sky Corporation", "DIGITAL-SKY" },
	{ 0x001D18, "Power Innovation GmbH", "POWER-INNOVATION" },
	{ 0x001D19, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x001D1A, "OvisLink S.A.", "OVISLINK" },
	{ 0x001D1B, "Sangean Electronics Inc.", "SANGEAN-ELECTRONICS" },
	{ 0x001D1C, "Gennet s.a.", "GENNET" },
	{ 0x001D1D, "Inter-M Corporation", "INTER-M" },
	{ 0x001D1E, "KYUSHU TEN CO.,LTD", "KYUSHU-TEN" },
	{ 0x001D1F, "Siauliu Tauro Televizoriai, JSC", "SIAULIU-TAURO-TELEVIZORIAI-JSC" },
	{ 0x001D20, "COMTREND CO.", "COMTREND" },
	{ 0x001D21, "Alcad SL", "ALCAD-SL" },
	{ 0x001D22, "Foss Analytical A/S", "FOSS-ANALYTICAL-A/S" },
	{ 0x001D23, "SENSUS", "SENSUS" },
	{ 0x001D24, "Aclara Power-Line Systems Inc.", "ACLARA-POWER-LINE" },
	{ 0x001D25, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x001D26, "Rockridgesound Technology Co.", "ROCKRIDGESOUND" },
	{ 0x001D27, "NAC-INTERCOM", "NAC-INTERCOM" },
	{ 0x001D28, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x001D29, "Doro AB", "DORO-AB" },
	{ 0x001D2A, "SHENZHEN BUL-TECH CO.,LTD.", "SHENZHEN-BUL-TECH" },
	{ 0x001D2B, "Wuhan Pont Technology CO. , LTD", "WUHAN-PONT" },
	{ 0x001D2C, "Wavetrend Technologies (Pty) Limited", "WAVETREND-TECHNOLOGIES-PTY" },
	{ 0x001D2D, "Pylone, Inc.", "PYLONE" },
	{ 0x001D2E, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x001D2F, "QuantumVision Corporation", "QUANTUMVISION" },
	{ 0x001D30, "YX Wireless S.A.", "YX-WIRELESS" },
	{ 0x001D31, "HIGHPRO INTERNATIONAL R&D CO,.LTD.", "HIGHPRO-R-D" },
	{ 0x001D32, "Longkay Communication & Technology (Shanghai) Co. Ltd", "LONGKAY-COMMUNICATION-SHANGHAI" },
	{ 0x001D33, "Maverick Systems Inc.", "MAVERICK" },
	{ 0x001D34, "SYRIS Technology Corp", "SYRIS" },
	{ 0x001D35, "Viconics Electronics Inc.", "VICONICS-ELECTRONICS" },
	{ 0x001D36, "ELECTRONICS CORPORATION OF INDIA LIMITED", "ELECTRONICS-OF-INDIA" },
	{ 0x001D37, "Thales-Panda Transportation System", "THALES-PANDA-TRANSPORTATION-SYSTEM" },
	{ 0x001D38, "Seagate Technology", "SEAGATE" },
	{ 0x001D39, "MOOHADIGITAL CO., LTD", "MOOHADIGITAL" },
	{ 0x001D3A, "mh acoustics LLC", "MH-ACOUSTICS-LLC" },
	{ 0x001D3B, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001D3C, "Muscle Corporation", "MUSCLE" },
	{ 0x001D3D, "Avidyne Corporation", "AVIDYNE" },
	{ 0x001D3E, "SAKA TECHNO SCIENCE CO.,LTD", "SAKA-TECHNO-SCIENCE" },
	{ 0x001D3F, "Mitron Pty Ltd", "MITRON-PTY" },
	{ 0x001D40, "Intel – GE Care Innovations LLC", "INTEL-–-GE-CARE-INNOVATIONS-LLC" },
	{ 0x001D41, "Hardy Instruments", "HARDY-INSTRUMENTS" },
	{ 0x001D42, "Nortel", "NORTEL" },
	{ 0x001D43, "Shenzhen G-link Digital Technology Co., Ltd.", "SHENZHEN-G-LINK-DIGITAL" },
	{ 0x001D44, "KROHNE Messtechnik GmbH", "KROHNE-MESSTECHNIK" },
	{ 0x001D45, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001D46, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001D47, "Covote GmbH & Co KG", "COVOTE" },
	{ 0x001D48, "Sensor-Technik Wiedemann GmbH", "SENSOR-TECHNIK-WIEDEMANN" },
	{ 0x001D49, "Innovation Wireless Inc.", "INNOVATION-WIRELESS" },
	{ 0x001D4A, "Carestream Health, Inc.", "CARESTREAM-HEALTH" },
	{ 0x001D4B, "Grid Connect Inc.", "GRID-CONNECT" },
	{ 0x001D4C, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x001D4D, "Adaptive Recognition Hungary, Inc", "ADAPTIVE-RECOGNITION-HUNGARY" },
	{ 0x001D4E, "TCM Mobile LLC", "TCM-MOBILE-LLC" },
	{ 0x001D4F, "Apple", "APPLE" },
	{ 0x001D50, "SPINETIX SA", "SPINETIX" },
	{ 0x001D51, "Babcock & Wilcox Power Generation Group, Inc", "BABCOCK-WILCOX-POWER-GENERATION-GROUP" },
	{ 0x001D52, "Defzone B.V.", "DEFZONE" },
	{ 0x001D53, "S&O Electronics (Malaysia) Sdn. Bhd.", "S-O-ELECTRONICS-MALAYSIA-SDN-BHD" },
	{ 0x001D54, "Sunnic Technology & Merchandise INC.", "SUNNIC-MERCHANDISE" },
	{ 0x001D55, "ZANTAZ, Inc", "ZANTAZ" },
	{ 0x001D56, "Kramer Electronics Ltd.", "KRAMER-ELECTRONICS" },
	{ 0x001D57, "CAETEC Messtechnik", "CAETEC-MESSTECHNIK" },
	{ 0x001D58, "CQ Inc", "CQ" },
	{ 0x001D59, "Mitra Energy & Infrastructure", "MITRA-ENERGY-INFRASTRUCTURE" },
	{ 0x001D5A, "2Wire Inc.", "2WIRE" },
	{ 0x001D5B, "Tecvan Informática Ltda", "TECVAN-INFORMáTICA-LTDA" },
	{ 0x001D5C, "Tom Communication Industrial Co.,Ltd.", "TOM-COMMUNICATION-INDUSTRIAL" },
	{ 0x001D5D, "Control Dynamics Pty. Ltd.", "CONTROL-DYNAMICS-PTY" },
	{ 0x001D5E, "COMING MEDIA CORP.", "COMING-MEDIA" },
	{ 0x001D5F, "OverSpeed SARL", "OVERSPEED-SARL" },
	{ 0x001D60, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x001D61, "BIJ Corporation", "BIJ" },
	{ 0x001D62, "InPhase Technologies", "INPHASE-TECHNOLOGIES" },
	{ 0x001D63, "Miele & Cie. KG", "MIELE-CIE" },
	{ 0x001D64, "Adam Communications Systems Int Ltd", "ADAM-COMMUNICATION-INT" },
	{ 0x001D65, "Microwave Radio Communications", "MICROWAVE-RADIO-COMMUNICATION" },
	{ 0x001D66, "Hyundai Telecom", "HYUNDAI-TELECOM" },
	{ 0x001D67, "AMEC", "AMEC" },
	{ 0x001D68, "Thomson Telecom Belgium", "THOMSON-TELECOM-BELGIUM" },
	{ 0x001D69, "Knorr-Bremse IT-Services GmbH", "KNORR-BREMSE-IT-SERVICES" },
	{ 0x001D6A, "Alpha Networks Inc.", "ALPHA-NETWORKS" },
	{ 0x001D6B, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001D6C, "ClariPhy Communications, Inc.", "CLARIPHY-COMMUNICATION" },
	{ 0x001D6D, "Confidant International LLC", "CONFIDANT-LLC" },
	{ 0x001D6E, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001D6F, "Chainzone Technology Co., Ltd", "CHAINZONE" },
	{ 0x001D70, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001D71, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001D72, "Wistron Corporation", "WISTRON" },
	{ 0x001D73, "Buffalo Inc.", "BUFFALO" },
	{ 0x001D74, "Tianjin China-Silicon Microelectronics Co., Ltd.", "TIANJIN-CHINA-SILICON-MICROELECTRONICS" },
	{ 0x001D75, "Radioscape PLC", "RADIOSCAPE-PLC" },
	{ 0x001D76, "Eyeheight Ltd.", "EYEHEIGHT" },
	{ 0x001D77, "NSGate", "NSGATE" },
	{ 0x001D78, "Invengo Information Technology Co.,Ltd", "INVENGO-INFORMATION" },
	{ 0x001D79, "SIGNAMAX LLC", "SIGNAMAX-LLC" },
	{ 0x001D7A, "Wideband Semiconductor, Inc.", "WIDEBAND-SEMICONDUCTOR" },
	{ 0x001D7B, "Ice Energy, Inc.", "ICE-ENERGY" },
	{ 0x001D7C, "ABE Elettronica S.p.A.", "ABE-ELETTRONICA-S-P-A" },
	{ 0x001D7D, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x001D7E, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x001D7F, "Tekron International Ltd", "TEKRON" },
	{ 0x001D80, "Beijing Huahuan Eletronics Co.,Ltd", "BEIJING-HUAHUAN-ELETRONICS" },
	{ 0x001D81, "GUANGZHOU GATEWAY ELECTRONICS CO., LTD", "GUANGZHOU-GATEWAY-ELECTRONICS" },
	{ 0x001D82, "GN A/S (GN Netcom A/S)", "GN-A/S-GN-NETCOM-A/S" },
	{ 0x001D83, "Emitech Corporation", "EMITECH" },
	{ 0x001D84, "Gateway, Inc.", "GATEWAY" },
	{ 0x001D85, "Call Direct Cellular Solutions", "CALL-DIRECT-CELLULAR-SOLUTIONS" },
	{ 0x001D86, "Shinwa Industries(China) Ltd.", "SHINWA-INDUSTRIES-CHINA" },
	{ 0x001D87, "VigTech Labs Sdn Bhd", "VIGTECH-LABS-SDN-BHD" },
	{ 0x001D88, "Clearwire", "CLEARWIRE" },
	{ 0x001D89, "VaultStor Corporation", "VAULTSTOR" },
	{ 0x001D8A, "TechTrex Inc", "TECHTREX" },
	{ 0x001D8B, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x001D8C, "La Crosse Technology LTD", "LA-CROSSE" },
	{ 0x001D8D, "Raytek GmbH", "RAYTEK" },
	{ 0x001D8E, "Alereon, Inc.", "ALEREON" },
	{ 0x001D8F, "PureWave Networks", "PUREWAVE-NETWORKS" },
	{ 0x001D90, "EMCO Flow Systems", "EMCO-FLOW" },
	{ 0x001D91, "Digitize, Inc", "DIGITIZE" },
	{ 0x001D92, "MICRO-STAR INT'L CO.,LTD.", "MICRO-STAR" },
	{ 0x001D93, "Modacom", "MODACOM" },
	{ 0x001D94, "Climax Technology Co., Ltd", "CLIMAX" },
	{ 0x001D95, "Flash, Inc.", "FLASH" },
	{ 0x001D96, "WatchGuard Video", "WATCHGUARD-VIDEO" },
	{ 0x001D97, "Alertus Technologies LLC", "ALERTUS-TECHNOLOGIES-LLC" },
	{ 0x001D98, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001D99, "Cyan Optic, Inc.", "CYAN-OPTIC" },
	{ 0x001D9A, "GODEX INTERNATIONAL CO., LTD", "GODEX" },
	{ 0x001D9B, "Hokuyo Automatic Co., Ltd.", "HOKUYO-AUTOMATIC" },
	{ 0x001D9C, "Rockwell Automation", "ROCKWELL-AUTOMATION" },
	{ 0x001D9D, "ARTJOY INTERNATIONAL LIMITED", "ARTJOY" },
	{ 0x001D9E, "AXION TECHNOLOGIES", "AXION-TECHNOLOGIES" },
	{ 0x001D9F, "MATT R.P.Traczynscy Sp.J.", "MATT-R-P-TRACZYNSCY-SP-J" },
	{ 0x001DA0, "Heng Yu Electronic Manufacturing Company Limited", "HENG-YU-MANUFACTURING-COMPANY" },
	{ 0x001DA1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001DA2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001DA3, "SabiOso", "SABIOSO" },
	{ 0x001DA4, "Hangzhou System Technology CO., LTD", "HANGZHOU-SYSTEM" },
	{ 0x001DA5, "WB Electronics", "WB-ELECTRONICS" },
	{ 0x001DA6, "Media Numerics Limited", "MEDIA-NUMERICS" },
	{ 0x001DA7, "Seamless Internet", "SEAMLESS-INTERNET" },
	{ 0x001DA8, "Takahata Electronics Co.,Ltd", "TAKAHATA-ELECTRONICS" },
	{ 0x001DA9, "Castles Technology, Co., LTD", "CASTLES" },
	{ 0x001DAA, "DrayTek Corp.", "DRAYTEK" },
	{ 0x001DAB, "SwissQual License AG", "SWISSQUAL-LICENSE" },
	{ 0x001DAC, "Gigamon Systems LLC", "GIGAMON-LLC" },
	{ 0x001DAD, "Sinotech Engineering Consultants, Inc. Geotechnical Enginee", "SINOTECH-ENGINEERING-CONSULTANTS-GEOTECHNICAL-ENGINEE" },
	{ 0x001DAE, "CHANG TSENG TECHNOLOGY CO., LTD", "CHANG-TSENG" },
	{ 0x001DAF, "Nortel", "NORTEL" },
	{ 0x001DB0, "FuJian HengTong Information Technology Co.,Ltd", "FUJIAN-HENGTONG-INFORMATION" },
	{ 0x001DB1, "Crescendo Networks", "CRESCENDO-NETWORKS" },
	{ 0x001DB2, "HOKKAIDO ELECTRIC ENGINEERING CO.,LTD.", "HOKKAIDO-ELECTRIC-ENGINEERING" },
	{ 0x001DB3, "HPN Supply Chain", "HPN-SUPPLY-CHAIN" },
	{ 0x001DB4, "KUMHO ENG CO.,LTD", "KUMHO-ENG" },
	{ 0x001DB5, "Juniper networks", "JUNIPER-NETWORKS" },
	{ 0x001DB6, "BestComm Networks, Inc.", "BESTCOMM-NETWORKS" },
	{ 0x001DB7, "Tendril Networks, Inc.", "TENDRIL-NETWORKS" },
	{ 0x001DB8, "Intoto Inc.", "INTOTO" },
	{ 0x001DB9, "Wellspring Wireless", "WELLSPRING-WIRELESS" },
	{ 0x001DBA, "Sony Corporation", "SONY" },
	{ 0x001DBB, "Dynamic System Electronics Corp.", "DYNAMIC-SYSTEM-ELECTRONICS" },
	{ 0x001DBC, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001DBD, "Versamed Inc.", "VERSAMED" },
	{ 0x001DBE, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DBF, "Radiient Technologies, Inc.", "RADIIENT-TECHNOLOGIES" },
	{ 0x001DC0, "Enphase Energy", "ENPHASE-ENERGY" },
	{ 0x001DC1, "Audinate Pty L", "AUDINATE-PTY-L" },
	{ 0x001DC2, "XORTEC OY", "XORTEC-OY" },
	{ 0x001DC3, "RIKOR TV, Ltd", "RIKOR-TV" },
	{ 0x001DC4, "AIOI Systems Co., Ltd.", "AIOI" },
	{ 0x001DC5, "Beijing Jiaxun Feihong Electricial Co., Ltd.", "BEIJING-JIAXUN-FEIHONG-ELECTRICIAL" },
	{ 0x001DC6, "SNR Inc.", "SNR" },
	{ 0x001DC7, "L-3 Communications Geneva Aerospace", "L-3-COMMUNICATION-GENEVA-AEROSPACE" },
	{ 0x001DC8, "Navionics Research Inc., dba SCADAmetrics", "NAVIONICS-RESEARCH-DBA-SCADAMETRICS" },
	{ 0x001DC9, "GainSpan Corp.", "GAINSPAN" },
	{ 0x001DCA, "PAV Electronics Limited", "PAV-ELECTRONICS" },
	{ 0x001DCB, "Exéns Development Oy", "EXéNS-DEVELOPMENT-OY" },
	{ 0x001DCC, "Hetra Secure Solutions", "HETRA-SECURE-SOLUTIONS" },
	{ 0x001DCD, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DCE, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DCF, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DD0, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DD1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DD2, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DD3, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DD4, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DD5, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DD6, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001DD7, "Algolith", "ALGOLITH" },
	{ 0x001DD8, "Microsoft Corporation", "MICROSOFT" },
	{ 0x001DD9, "Hon Hai Precision Ind.Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x001DDA, "Mikroelektronika spol. s r. o.", "MIKROELEKTRONIKA-SPOL-S-R-O" },
	{ 0x001DDB, "C-BEL Corporation", "C-BEL" },
	{ 0x001DDC, "HangZhou DeChangLong Tech&Info Co.,Ltd", "HANGZHOU-DECHANGLONG-TECH-INFO" },
	{ 0x001DDD, "DAT H.K. LIMITED", "DAT-H-K" },
	{ 0x001DDE, "Zhejiang Broadcast&Television Technology Co.,Ltd.", "ZHEJIANG-BROADCAST-TELEVISION" },
	{ 0x001DDF, "Sunitec Enterprise Co., Ltd.", "SUNITEC" },
	{ 0x001DE0, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001DE1, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001DE2, "Radionor Communications", "RADIONOR-COMMUNICATION" },
	{ 0x001DE3, "Intuicom", "INTUICOM" },
	{ 0x001DE4, "Visioneered Image Systems", "VISIONEERED-IMAGE" },
	{ 0x001DE5, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001DE6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001DE7, "Marine Sonic Technology, Ltd.", "MARINE-SONIC" },
	{ 0x001DE8, "Nikko Denki Tsushin Corporation(NDTC)", "NIKKO-DENKI-TSUSHIN-NDTC" },
	{ 0x001DE9, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001DEA, "Commtest Instruments Ltd", "COMMTEST-INSTRUMENTS" },
	{ 0x001DEB, "DINEC International", "DINEC" },
	{ 0x001DEC, "Marusys", "MARUSYS" },
	{ 0x001DED, "Grid Net, Inc.", "GRID-NET" },
	{ 0x001DEE, "NEXTVISION SISTEMAS DIGITAIS DE TELEVISÃO LTDA.", "NEXTVISION-SISTEMAS-DIGITAIS-DE-TELEVISÃO-LTDA" },
	{ 0x001DEF, "TRIMM, INC.", "TRIMM" },
	{ 0x001DF0, "Vidient Systems, Inc.", "VIDIENT" },
	{ 0x001DF1, "Intego Systems, Inc.", "INTEGO" },
	{ 0x001DF2, "Netflix, Inc.", "NETFLIX" },
	{ 0x001DF3, "SBS Science & Technology Co., Ltd", "SBS-SCIENCE" },
	{ 0x001DF4, "Magellan Technology Pty Limited", "MAGELLAN-PTY" },
	{ 0x001DF5, "Sunshine Co,LTD", "SUNSHINE" },
	{ 0x001DF6, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x001DF7, "R. STAHL Schaltgeräte GmbH", "R-STAHL-SCHALTGERäTE" },
	{ 0x001DF8, "Webpro Vision Technology Corporation", "WEBPRO-VISION" },
	{ 0x001DF9, "Cybiotronics (Far East) Limited", "CYBIOTRONICS-FAR-EAST" },
	{ 0x001DFA, "Fujian LANDI Commercial Equipment Co.,Ltd", "FUJIAN-LANDI-COMMERCIAL-EQUIPMENT" },
	{ 0x001DFB, "NETCLEUS Systems Corporation", "NETCLEUS" },
	{ 0x001DFC, "KSIC", "KSIC" },
	{ 0x001DFD, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001DFE, "Palm, Inc", "PALM" },
	{ 0x001DFF, "Network Critical Solutions Ltd", "NETWORK-CRITICAL-SOLUTIONS" },
	{ 0x001E00, "Shantou Institute of Ultrasonic Instruments", "SHANTOU-INSTITUTE-OF-ULTRASONIC-INSTRUMENTS" },
	{ 0x001E01, "Renesas Technology Sales Co., Ltd.", "RENESAS-SALES" },
	{ 0x001E02, "Sougou Keikaku Kougyou Co.,Ltd.", "SOUGOU-KEIKAKU-KOUGYOU" },
	{ 0x001E03, "LiComm Co., Ltd.", "LICOMM" },
	{ 0x001E04, "Hanson Research Corporation", "HANSON-RESEARCH" },
	{ 0x001E05, "Xseed Technologies & Computing", "XSEED-TECHNOLOGIES-COMPUTING" },
	{ 0x001E06, "WIBRAIN", "WIBRAIN" },
	{ 0x001E07, "Winy Technology Co., Ltd.", "WINY" },
	{ 0x001E08, "Centec Networks Inc", "CENTEC-NETWORKS" },
	{ 0x001E09, "ZEFATEK Co.,LTD", "ZEFATEK" },
	{ 0x001E0A, "Syba Tech Limited", "SYBA-TECH" },
	{ 0x001E0B, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001E0C, "Sherwood Information Partners, Inc.", "SHERWOOD-INFORMATION-PARTNERS" },
	{ 0x001E0D, "Micran Ltd.", "MICRAN" },
	{ 0x001E0E, "MAXI VIEW HOLDINGS LIMITED", "MAXI-VIEW-HOLDINGS" },
	{ 0x001E0F, "Briot International", "BRIOT" },
	{ 0x001E10, "ShenZhen Huawei Communication Technologies Co.,Ltd.", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x001E11, "ELELUX INTERNATIONAL LTD", "ELELUX" },
	{ 0x001E12, "Ecolab", "ECOLAB" },
	{ 0x001E13, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001E14, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001E15, "Beech Hill Electronics", "BEECH-HILL-ELECTRONICS" },
	{ 0x001E16, "Keytronix", "KEYTRONIX" },
	{ 0x001E17, "STN BV", "STN" },
	{ 0x001E18, "Radio Activity srl", "RADIO-ACTIVITY-SRL" },
	{ 0x001E19, "GTRI", "GTRI" },
	{ 0x001E1A, "Best Source Taiwan Inc.", "BEST-SOURCE-TAIWAN" },
	{ 0x001E1B, "Digital Stream Technology, Inc.", "DIGITAL-STREAM" },
	{ 0x001E1C, "SWS Australia Pty Limited", "SWS-AUSTRALIA-PTY" },
	{ 0x001E1D, "East Coast Datacom, Inc.", "EAST-COAST-DATACOM" },
	{ 0x001E1E, "Honeywell Life Safety", "HONEYWELL-LIFE-SAFETY" },
	{ 0x001E1F, "Nortel", "NORTEL" },
	{ 0x001E20, "Intertain Inc.", "INTERTAIN" },
	{ 0x001E21, "Qisda Co.", "QISDA" },
	{ 0x001E22, "ARVOO Imaging Products BV", "ARVOO-IMAGING-PRODUCTS" },
	{ 0x001E23, "Electronic Educational Devices, Inc", "EDUCATIONAL-DEVICES" },
	{ 0x001E24, "Zhejiang Bell Technology Co.,ltd", "ZHEJIANG-BELL" },
	{ 0x001E25, "Intek Digital Inc", "INTEK-DIGITAL" },
	{ 0x001E26, "Digifriends Co. Ltd", "DIGIFRIENDS" },
	{ 0x001E27, "SBN TECH Co.,Ltd.", "SBN-TECH" },
	{ 0x001E28, "Lumexis Corporation", "LUMEXIS" },
	{ 0x001E29, "Hypertherm Inc", "HYPERTHERM" },
	{ 0x001E2A, "Netgear Inc.", "NETGEAR" },
	{ 0x001E2B, "Radio Systems Design, Inc.", "RADIO-DESIGN" },
	{ 0x001E2C, "CyVerse Corporation", "CYVERSE" },
	{ 0x001E2D, "STIM", "STIM" },
	{ 0x001E2E, "SIRTI S.p.A.", "SIRTI-S-P-A" },
	{ 0x001E2F, "DiMoto Pty Ltd", "DIMOTO-PTY" },
	{ 0x001E30, "Shireen Inc", "SHIREEN" },
	{ 0x001E31, "INFOMARK CO.,LTD.", "INFOMARK" },
	{ 0x001E32, "Zensys", "ZENSYS" },
	{ 0x001E33, "Inventec Corporation", "INVENTEC" },
	{ 0x001E34, "CryptoMetrics", "CRYPTOMETRICS" },
	{ 0x001E35, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001E36, "IPTE", "IPTE" },
	{ 0x001E37, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x001E38, "Bluecard Software Technology Co., Ltd.", "BLUECARD-SOFTWARE" },
	{ 0x001E39, "Comsys Communication Ltd.", "COMSYS-COMMUNICATION" },
	{ 0x001E3A, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001E3B, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001E3C, "Lyngbox Media AB", "LYNGBOX-MEDIA-AB" },
	{ 0x001E3D, "Alps Electric Co., Ltd", "ALPS-ELECTRIC" },
	{ 0x001E3E, "KMW Inc.", "KMW" },
	{ 0x001E3F, "TrellisWare Technologies, Inc.", "TRELLISWARE-TECHNOLOGIES" },
	{ 0x001E40, "Shanghai DareGlobal Technologies Co.,Ltd.", "SHANGHAI-DAREGLOBAL-TECHNOLOGIES" },
	{ 0x001E41, "Microwave Communication & Component, Inc.", "MICROWAVE-COMMUNICATION-COMPONENT" },
	{ 0x001E42, "Teltonika", "TELTONIKA" },
	{ 0x001E43, "AISIN AW CO.,LTD.", "AISIN-AW" },
	{ 0x001E44, "SANTEC", "SANTEC" },
	{ 0x001E45, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x001E46, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001E47, "PT. Hariff Daya Tunggal Engineering", "PT-HARIFF-DAYA-TUNGGAL-ENGINEERING" },
	{ 0x001E48, "Wi-Links", "WI-LINKS" },
	{ 0x001E49, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001E4A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001E4B, "City Theatrical", "CITY-THEATRICAL" },
	{ 0x001E4C, "Hon Hai Precision Ind.Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x001E4D, "Welkin Sciences, LLC", "WELKIN-SCIENCES-LLC" },
	{ 0x001E4E, "DAKO EDV-Ingenieur- und Systemhaus GmbH", "DAKO-EDV-INGENIEUR--UND-SYSTEMHAUS" },
	{ 0x001E4F, "Dell Inc.", "DELL" },
	{ 0x001E50, "BATTISTONI RESEARCH", "BATTISTONI-RESEARCH" },
	{ 0x001E51, "Converter Industry Srl", "CONVERTER-INDUSTRY-SRL" },
	{ 0x001E52, "Apple", "APPLE" },
	{ 0x001E53, "Further Tech Co., LTD", "FURTHER-TECH" },
	{ 0x001E54, "TOYO ELECTRIC Corporation", "TOYO-ELECTRIC" },
	{ 0x001E55, "COWON SYSTEMS,Inc.", "COWON" },
	{ 0x001E56, "Bally Wulff Entertainment GmbH", "BALLY-WULFF-ENTERTAINMENT" },
	{ 0x001E57, "ALCOMA, spol. s r.o.", "ALCOMA-SPOL-S-R-O" },
	{ 0x001E58, "D-Link Corporation", "D-LINK" },
	{ 0x001E59, "Silicon Turnkey Express, LLC", "SILICON-TURNKEY-EXPRESS-LLC" },
	{ 0x001E5A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001E5B, "Unitron Company, Inc.", "UNITRON-COMPANY" },
	{ 0x001E5C, "RB GeneralEkonomik", "RB-GENERALEKONOMIK" },
	{ 0x001E5D, "Holosys d.o.o.", "HOLOSYS-D-O-O" },
	{ 0x001E5E, "COmputime Ltd.", "COMPUTIME" },
	{ 0x001E5F, "KwikByte, LLC", "KWIKBYTE-LLC" },
	{ 0x001E60, "Digital Lighting Systems, Inc", "DIGITAL-LIGHTING" },
	{ 0x001E61, "ITEC GmbH", "ITEC" },
	{ 0x001E62, "Siemon", "SIEMON" },
	{ 0x001E63, "Vibro-Meter SA", "VIBRO-METER" },
	{ 0x001E64, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001E65, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001E66, "RESOL Elektronische Regelungen GmbH", "RESOL-ELEKTRONISCHE-REGELUNGEN" },
	{ 0x001E67, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001E68, "Quanta Computer", "QUANTA" },
	{ 0x001E69, "Thomson Inc.", "THOMSON" },
	{ 0x001E6A, "Beijing Bluexon Technology Co.,Ltd", "BEIJING-BLUEXON" },
	{ 0x001E6B, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x001E6C, "Opaque Systems", "OPAQUE" },
	{ 0x001E6D, "IT R&D Center", "IT-R-D-CENTER" },
	{ 0x001E6E, "Shenzhen First Mile Communications Ltd", "SHENZHEN-FIRST-MILE-COMMUNICATION" },
	{ 0x001E6F, "Magna-Power Electronics, Inc.", "MAGNA-POWER-ELECTRONICS" },
	{ 0x001E70, "Cobham Defence Communications Ltd", "COBHAM-DEFENCE-COMMUNICATION" },
	{ 0x001E71, "MIrcom Group of Companies", "MIRCOM-GROUP-OF-COMPANIES" },
	{ 0x001E72, "PCS", "PCS" },
	{ 0x001E73, "ZTE CORPORATION", "ZTE" },
	{ 0x001E74, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x001E75, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x001E76, "Thermo Fisher Scientific", "THERMO-FISHER-SCIENTIFIC" },
	{ 0x001E77, "Air2App", "AIR2APP" },
	{ 0x001E78, "Owitek Technology Ltd.,", "OWITEK" },
	{ 0x001E79, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001E7A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001E7B, "R.I.CO. S.r.l.", "R-I-S-R-L" },
	{ 0x001E7C, "Taiwick Limited", "TAIWICK" },
	{ 0x001E7D, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x001E7E, "Nortel", "NORTEL" },
	{ 0x001E7F, "CBM of America", "CBM-OF-AMERICA" },
	{ 0x001E80, "Last Mile Ltd.", "LAST-MILE" },
	{ 0x001E81, "CNB Technology Inc.", "CNB" },
	{ 0x001E82, "SanDisk Corporation", "SANDISK" },
	{ 0x001E83, "LAN/MAN Standards Association (LMSC)", "LAN/MAN-STANDARDS-ASSOCIATION-LMSC" },
	{ 0x001E84, "Pika Technologies Inc.", "PIKA-TECHNOLOGIES" },
	{ 0x001E85, "Lagotek Corporation", "LAGOTEK" },
	{ 0x001E86, "MEL Co.,Ltd.", "MEL" },
	{ 0x001E87, "Realease Limited", "REALEASE" },
	{ 0x001E88, "ANDOR SYSTEM SUPPORT CO., LTD.", "ANDOR-SYSTEM-SUPPORT" },
	{ 0x001E89, "CRFS Limited", "CRFS" },
	{ 0x001E8A, "eCopy, Inc", "ECOPY" },
	{ 0x001E8B, "Infra Access Korea Co., Ltd.", "INFRA-ACCESS-KOREA" },
	{ 0x001E8C, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x001E8D, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001E8E, "Hunkeler AG", "HUNKELER" },
	{ 0x001E8F, "CANON INC.", "CANON" },
	{ 0x001E90, "Elitegroup Computer Systems Co", "ELITEGROUP" },
	{ 0x001E91, "KIMIN Electronic Co., Ltd.", "KIMIN" },
	{ 0x001E92, "JEULIN S.A.", "JEULIN" },
	{ 0x001E93, "CiriTech Systems Inc", "CIRITECH" },
	{ 0x001E94, "SUPERCOM TECHNOLOGY CORPORATION", "SUPERCOM" },
	{ 0x001E95, "SIGMALINK", "SIGMALINK" },
	{ 0x001E96, "Sepura Plc", "SEPURA-PLC" },
	{ 0x001E97, "Medium Link System Technology CO., LTD,", "MEDIUM-LINK-SYSTEM" },
	{ 0x001E98, "GreenLine Communications", "GREENLINE-COMMUNICATION" },
	{ 0x001E99, "Vantanol Industrial Corporation", "VANTANOL-INDUSTRIAL" },
	{ 0x001E9A, "HAMILTON Bonaduz AG", "HAMILTON-BONADUZ" },
	{ 0x001E9B, "San-Eisha, Ltd.", "SAN-EISHA" },
	{ 0x001E9C, "Fidustron INC", "FIDUSTRON" },
	{ 0x001E9D, "Recall Technologies, Inc.", "RECALL-TECHNOLOGIES" },
	{ 0x001E9E, "ddm hopt + schuler Gmbh + Co. KG", "DDM-HOPT-+-SCHULER-+" },
	{ 0x001E9F, "Visioneering Systems, Inc.", "VISIONEERING" },
	{ 0x001EA0, "XLN-t", "XLN-T" },
	{ 0x001EA1, "Brunata a/s", "BRUNATA-A/S" },
	{ 0x001EA2, "Symx Systems, Inc.", "SYMX" },
	{ 0x001EA3, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001EA4, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001EA5, "ROBOTOUS, Inc.", "ROBOTOUS" },
	{ 0x001EA6, "Best IT World (India) Pvt. Ltd.", "BEST-IT-WORLD-INDIA-PVT" },
	{ 0x001EA7, "ActionTec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x001EA8, "Datang Mobile Communications Equipment CO.,LTD", "DATANG-MOBILE-COMMUNICATION-EQUIPMENT" },
	{ 0x001EA9, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001EAA, "E-Senza Technologies GmbH", "E-SENZA-TECHNOLOGIES" },
	{ 0x001EAB, "TeleWell Oy", "TELEWELL-OY" },
	{ 0x001EAC, "Armadeus Systems", "ARMADEUS" },
	{ 0x001EAD, "Wingtech Group Limited", "WINGTECH-GROUP" },
	{ 0x001EAE, "Continental Automotive Systems", "CONTINENTAL-AUTOMOTIVE" },
	{ 0x001EAF, "Ophir Optronics Ltd", "OPHIR-OPTRONICS" },
	{ 0x001EB0, "ImesD Electronica S.L.", "IMESD-ELECTRONICA-S-L" },
	{ 0x001EB1, "Cryptsoft Pty Ltd", "CRYPTSOFT-PTY" },
	{ 0x001EB2, "LG innotek", "LG-INNOTEK" },
	{ 0x001EB3, "Primex Wireless", "PRIMEX-WIRELESS" },
	{ 0x001EB4, "UNIFAT TECHNOLOGY LTD.", "UNIFAT" },
	{ 0x001EB5, "Ever Sparkle Technologies Ltd", "EVER-SPARKLE-TECHNOLOGIES" },
	{ 0x001EB6, "TAG Heuer SA", "TAG-HEUER" },
	{ 0x001EB7, "TBTech, Co., Ltd.", "TBTECH" },
	{ 0x001EB8, "Fortis, Inc.", "FORTIS" },
	{ 0x001EB9, "Sing Fai Technology Limited", "SING-FAI" },
	{ 0x001EBA, "High Density Devices AS", "HIGH-DENSITY-DEVICES-AS" },
	{ 0x001EBB, "BLUELIGHT TECHNOLOGY INC.", "BLUELIGHT" },
	{ 0x001EBC, "WINTECH AUTOMATION CO.,LTD.", "WINTECH-AUTOMATION" },
	{ 0x001EBD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001EBE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001EBF, "Haas Automation Inc.", "HAAS-AUTOMATION" },
	{ 0x001EC0, "Microchip Technology Inc.", "MICROCHIP" },
	{ 0x001EC1, "3COM EUROPE LTD", "3COM" },
	{ 0x001EC2, "Apple", "APPLE" },
	{ 0x001EC3, "Kozio, Inc.", "KOZIO" },
	{ 0x001EC4, "Celio Corp", "CELIO" },
	{ 0x001EC5, "Middle Atlantic Products Inc", "MIDDLE-ATLANTIC-PRODUCTS" },
	{ 0x001EC6, "Obvius Holdings LLC", "OBVIUS-HOLDINGS-LLC" },
	{ 0x001EC7, "2Wire, Inc.", "2WIRE" },
	{ 0x001EC8, "Rapid Mobile (Pty) Ltd", "RAPID-MOBILE-PTY" },
	{ 0x001EC9, "Dell Inc", "DELL" },
	{ 0x001ECA, "Nortel", "NORTEL" },
	{ 0x001ECB, "\"RPC \"Energoautomatika\" Ltd", "RPC-ENERGOAUTOMATIKA" },
	{ 0x001ECC, "CDVI", "CDVI" },
	{ 0x001ECD, "KYLAND Technology Co. LTD", "KYLAND" },
	{ 0x001ECE, "BISA Technologies (Hong Kong) Limited", "BISA-TECHNOLOGIES-HONG-KONG" },
	{ 0x001ECF, "PHILIPS ELECTRONICS UK LTD", "PHILIPS-ELECTRONICS-UK" },
	{ 0x001ED0, "Ingespace", "INGESPACE" },
	{ 0x001ED1, "Keyprocessor B.V.", "KEYPROCESSOR" },
	{ 0x001ED2, "Ray Shine Video Technology Inc", "RAY-SHINE-VIDEO" },
	{ 0x001ED3, "Dot Technology Int'l Co., Ltd.", "DOT" },
	{ 0x001ED4, "Doble Engineering", "DOBLE-ENGINEERING" },
	{ 0x001ED5, "Tekon-Automatics", "TEKON-AUTOMATICS" },
	{ 0x001ED6, "Alentec & Orion AB", "ALENTEC-ORION-AB" },
	{ 0x001ED7, "H-Stream Wireless, Inc.", "H-STREAM-WIRELESS" },
	{ 0x001ED8, "Digital United Inc.", "DIGITAL-UNITED" },
	{ 0x001ED9, "Mitsubishi Precision Co.,LTd.", "MITSUBISHI-PRECISION" },
	{ 0x001EDA, "Wesemann Elektrotechniek B.V.", "WESEMANN-ELEKTROTECHNIEK" },
	{ 0x001EDB, "Giken Trastem Co., Ltd.", "GIKEN-TRASTEM" },
	{ 0x001EDC, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x001EDD, "WASKO S.A.", "WASKO" },
	{ 0x001EDE, "BYD COMPANY LIMITED", "BYD-COMPANY" },
	{ 0x001EDF, "Master Industrialization Center Kista", "MASTER-INDUSTRIALIZATION-CENTER-KISTA" },
	{ 0x001EE0, "Urmet Domus SpA", "URMET-DOMUS-SPA" },
	{ 0x001EE1, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x001EE2, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x001EE3, "T&W Electronics (ShenZhen) Co.,Ltd", "T-W-ELECTRONICS-SHENZHEN" },
	{ 0x001EE4, "ACS Solutions France", "ACS-SOLUTIONS-FRANCE" },
	{ 0x001EE5, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x001EE6, "Shenzhen Advanced Video Info-Tech Co., Ltd.", "SHENZHEN-ADVANCED-VIDEO-INFO-TECH" },
	{ 0x001EE7, "Epic Systems Inc", "EPIC" },
	{ 0x001EE8, "Mytek", "MYTEK" },
	{ 0x001EE9, "Stoneridge Electronics AB", "STONERIDGE-ELECTRONICS-AB" },
	{ 0x001EEA, "Sensor Switch, Inc.", "SENSOR-SWITCH" },
	{ 0x001EEB, "Talk-A-Phone Co.", "TALK-A-PHONE" },
	{ 0x001EEC, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x001EED, "Adventiq Ltd.", "ADVENTIQ" },
	{ 0x001EEE, "ETL Systems Ltd", "ETL" },
	{ 0x001EEF, "Cantronic International Limited", "CANTRONIC" },
	{ 0x001EF0, "Gigafin Networks", "GIGAFIN-NETWORKS" },
	{ 0x001EF1, "Servimat", "SERVIMAT" },
	{ 0x001EF2, "Micro Motion Inc", "MICRO-MOTION" },
	{ 0x001EF3, "From2", "FROM2" },
	{ 0x001EF4, "L-3 Communications Display Systems", "L-3-COMMUNICATION-DISPLAY" },
	{ 0x001EF5, "Hitek Automated Inc.", "HITEK-AUTOMATED" },
	{ 0x001EF6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001EF7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001EF8, "Emfinity Inc.", "EMFINITY" },
	{ 0x001EF9, "Pascom Kommunikations systeme GmbH.", "PASCOM-KOMMUNIKATIONS-SYSTEME" },
	{ 0x001EFA, "PROTEI Ltd.", "PROTEI" },
	{ 0x001EFB, "Trio Motion Technology Ltd", "TRIO-MOTION" },
	{ 0x001EFC, "JSC \"MASSA-K\"", "JSC-MASSA-K" },
	{ 0x001EFD, "Microbit 2.0 AB", "MICROBIT-2-0-AB" },
	{ 0x001EFE, "LEVEL s.r.o.", "LEVEL-S-R-O" },
	{ 0x001EFF, "Mueller-Elektronik GmbH & Co. KG", "MUELLER-ELEKTRONIK" },
	{ 0x001F00, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001F01, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001F02, "Pixelmetrix Corporation Pte Ltd", "PIXELMETRIX-PTE" },
	{ 0x001F03, "NUM AG", "NUM" },
	{ 0x001F04, "Granch Ltd.", "GRANCH" },
	{ 0x001F05, "iTAS Technology Corp.", "ITAS" },
	{ 0x001F06, "Integrated Dispatch Solutions", "INTEGRATED-DISPATCH-SOLUTIONS" },
	{ 0x001F07, "AZTEQ Mobile", "AZTEQ-MOBILE" },
	{ 0x001F08, "RISCO LTD", "RISCO" },
	{ 0x001F09, "JASTEC CO., LTD.", "JASTEC" },
	{ 0x001F0A, "Nortel", "NORTEL" },
	{ 0x001F0B, "Federal State Unitary Enterprise Industrial Union\"Electropribor\"", "FEDERAL-STATE-UNITARY-INDUSTRIAL-UNION-ELECTROPRIBOR" },
	{ 0x001F0C, "Intelligent Digital Services GmbH", "INTELLIGENT-DIGITAL-SERVICES" },
	{ 0x001F0D, "L3 Communications - Telemetry West", "L3-COMMUNICATION---TELEMETRY-WEST" },
	{ 0x001F0E, "Japan Kyastem Co., Ltd", "JAPAN-KYASTEM" },
	{ 0x001F0F, "Select Engineered Systems", "SELECT-ENGINEERED" },
	{ 0x001F10, "TOLEDO DO BRASIL INDUSTRIA DE BALANCAS LTDA", "TOLEDO-DO-BRASIL-INDUSTRIA-DE-BALANCAS-LTDA" },
	{ 0x001F11, "OPENMOKO, INC.", "OPENMOKO" },
	{ 0x001F12, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x001F13, "S.& A.S. Ltd.", "S-A-S" },
	{ 0x001F14, "NexG", "NEXG" },
	{ 0x001F15, "Bioscrypt Inc", "BIOSCRYPT" },
	{ 0x001F16, "Wistron Corporation", "WISTRON" },
	{ 0x001F17, "IDX Company, Ltd.", "IDX-COMPANY" },
	{ 0x001F18, "Hakusan.Mfg.Co,.Ltd", "HAKUSAN-MFG" },
	{ 0x001F19, "BEN-RI ELECTRONICA S.A.", "BEN-RI-ELECTRONICA" },
	{ 0x001F1A, "Prominvest", "PROMINVEST" },
	{ 0x001F1B, "RoyalTek Company Ltd.", "ROYALTEK-COMPANY" },
	{ 0x001F1C, "KOBISHI ELECTRIC Co.,Ltd.", "KOBISHI-ELECTRIC" },
	{ 0x001F1D, "Atlas Material Testing Technology LLC", "ATLAS-MATERIAL-TESTING-LLC" },
	{ 0x001F1E, "Astec Technology Co., Ltd", "ASTEC" },
	{ 0x001F1F, "Edimax Technology Co. Ltd.", "EDIMAX" },
	{ 0x001F20, "Logitech Europe SA", "LOGITECH-EUROPE" },
	{ 0x001F21, "Inner Mongolia Yin An Science & Technology Development Co.,L", "INNER-MONGOLIA-YIN-AN-SCIENCE-DEVELOPMENT-L" },
	{ 0x001F22, "Source Photonics, Inc.", "SOURCE-PHOTONICS" },
	{ 0x001F23, "Interacoustics", "INTERACOUSTICS" },
	{ 0x001F24, "DIGITVIEW TECHNOLOGY CO., LTD.", "DIGITVIEW" },
	{ 0x001F25, "MBS GmbH", "MBS" },
	{ 0x001F26, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001F27, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001F28, "HPN Supply Chain", "HPN-SUPPLY-CHAIN" },
	{ 0x001F29, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x001F2A, "ACCM", "ACCM" },
	{ 0x001F2B, "Orange Logic", "ORANGE-LOGIC" },
	{ 0x001F2C, "Starbridge Networks", "STARBRIDGE-NETWORKS" },
	{ 0x001F2D, "Electro-Optical Imaging, Inc.", "ELECTRO-OPTICAL-IMAGING" },
	{ 0x001F2E, "Triangle Research Int'l Pte Ltd", "TRIANGLE-RESEARCH-PTE" },
	{ 0x001F2F, "Berker GmbH & Co. KG", "BERKER" },
	{ 0x001F30, "Travelping", "TRAVELPING" },
	{ 0x001F31, "Radiocomp", "RADIOCOMP" },
	{ 0x001F32, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001F33, "Netgear Inc.", "NETGEAR" },
	{ 0x001F34, "Lung Hwa Electronics Co., Ltd.", "LUNG-HWA-ELECTRONICS" },
	{ 0x001F35, "AIR802 LLC", "AIR802-LLC" },
	{ 0x001F36, "Bellwin Information Co. Ltd.,", "BELLWIN-INFORMATION" },
	{ 0x001F37, "Genesis I&C", "GENESIS-I-C" },
	{ 0x001F38, "POSITRON", "POSITRON" },
	{ 0x001F39, "Construcciones y Auxiliar de Ferrocarriles, S.A.", "CONSTRUCCIONES-Y-AUXILIAR-DE-FERROCARRILES" },
	{ 0x001F3A, "Hon Hai Precision Ind.Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x001F3B, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001F3C, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x001F3D, "Qbit GmbH", "QBIT" },
	{ 0x001F3E, "RP-Technik e.K.", "RP-TECHNIK-E-K" },
	{ 0x001F3F, "AVM GmbH", "AVM" },
	{ 0x001F40, "Speakercraft Inc.", "SPEAKERCRAFT" },
	{ 0x001F41, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x001F42, "Etherstack plc", "ETHERSTACK-PLC" },
	{ 0x001F43, "ENTES ELEKTRONIK", "ENTES-ELEKTRONIK" },
	{ 0x001F44, "GE Transportation Systems", "GE-TRANSPORTATION" },
	{ 0x001F45, "Enterasys", "ENTERASYS" },
	{ 0x001F46, "Nortel", "NORTEL" },
	{ 0x001F47, "MCS Logic Inc.", "MCS-LOGIC" },
	{ 0x001F48, "Mojix Inc.", "MOJIX" },
	{ 0x001F49, "Eurosat Distribution Ltd", "EUROSAT-DISTRIBUTION" },
	{ 0x001F4A, "Albentia Systems S.A.", "ALBENTIA" },
	{ 0x001F4B, "Lineage Power", "LINEAGE-POWER" },
	{ 0x001F4C, "Roseman Engineering Ltd", "ROSEMAN-ENGINEERING" },
	{ 0x001F4D, "Segnetics LLC", "SEGNETICS-LLC" },
	{ 0x001F4E, "ConMed Linvatec", "CONMED-LINVATEC" },
	{ 0x001F4F, "Thinkware Co. Ltd.", "THINKWARE" },
	{ 0x001F50, "Swissdis AG", "SWISSDIS" },
	{ 0x001F51, "HD Communications Corp", "HD-COMMUNICATION" },
	{ 0x001F52, "UVT Unternehmensberatung fur Verkehr und Technik GmbH", "UVT-UNTERNEHMENSBERATUNG-FUR-VERKEHR-UND-TECHNIK" },
	{ 0x001F53, "GEMAC Gesellschaft für Mikroelektronikanwendung Chemnitz mbH", "GEMAC-GESELLSCHAFT-FüR-MIKROELEKTRONIKANWENDUNG-CHEMNITZ-MBH" },
	{ 0x001F54, "Lorex Technology Inc.", "LOREX" },
	{ 0x001F55, "Honeywell Security (China) Co., Ltd.", "HONEYWELL-SECURITY-CHINA" },
	{ 0x001F56, "DIGITAL FORECAST", "DIGITAL-FORECAST" },
	{ 0x001F57, "Phonik Innovation Co.,LTD", "PHONIK-INNOVATION" },
	{ 0x001F58, "EMH Energiemesstechnik GmbH", "EMH-ENERGIEMESSTECHNIK" },
	{ 0x001F59, "Kronback Tracers", "KRONBACK-TRACERS" },
	{ 0x001F5A, "Beckwith Electric Co.", "BECKWITH-ELECTRIC" },
	{ 0x001F5B, "Apple", "APPLE" },
	{ 0x001F5C, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001F5D, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001F5E, "Dyna Technology Co.,Ltd.", "DYNA" },
	{ 0x001F5F, "Blatand GmbH", "BLATAND" },
	{ 0x001F60, "COMPASS SYSTEMS CORP.", "COMPASS" },
	{ 0x001F61, "Talent Communication Networks Inc.", "TALENT-COMMUNICATION-NETWORKS" },
	{ 0x001F62, "JSC \"Stilsoft\"", "JSC-STILSOFT" },
	{ 0x001F63, "JSC Goodwin-Europa", "JSC-GOODWIN-EUROPA" },
	{ 0x001F64, "Beijing Autelan Technology Inc.", "BEIJING-AUTELAN" },
	{ 0x001F65, "KOREA ELECTRIC TERMINAL CO., LTD.", "KOREA-ELECTRIC-TERMINAL" },
	{ 0x001F66, "PLANAR LLC", "PLANAR-LLC" },
	{ 0x001F67, "Hitachi,Ltd.", "HITACHI" },
	{ 0x001F68, "Martinsson Elektronik AB", "MARTINSSON-ELEKTRONIK-AB" },
	{ 0x001F69, "Pingood Technology Co., Ltd.", "PINGOOD" },
	{ 0x001F6A, "PacketFlux Technologies, Inc.", "PACKETFLUX-TECHNOLOGIES" },
	{ 0x001F6B, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x001F6C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001F6D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001F6E, "Vtech Engineering Corporation", "VTECH-ENGINEERING" },
	{ 0x001F6F, "Fujian Sunnada Communication Co.,Ltd.", "FUJIAN-SUNNADA-COMMUNICATION" },
	{ 0x001F70, "Botik Technologies LTD", "BOTIK-TECHNOLOGIES" },
	{ 0x001F71, "xG Technology, Inc.", "XG" },
	{ 0x001F72, "QingDao Hiphone Technology Co,.Ltd", "QINGDAO-HIPHONE" },
	{ 0x001F73, "Teraview Technology Co., Ltd.", "TERAVIEW" },
	{ 0x001F74, "Eigen Development", "EIGEN-DEVELOPMENT" },
	{ 0x001F75, "GiBahn Media", "GIBAHN-MEDIA" },
	{ 0x001F76, "AirLogic Systems Inc.", "AIRLOGIC" },
	{ 0x001F77, "HEOL DESIGN", "HEOL-DESIGN" },
	{ 0x001F78, "Blue Fox Porini Textile", "BLUE-FOX-PORINI-TEXTILE" },
	{ 0x001F79, "Lodam Electronics A/S", "LODAM-ELECTRONICS-A/S" },
	{ 0x001F7A, "WiWide Inc.", "WIWIDE" },
	{ 0x001F7B, "TechNexion Ltd.", "TECHNEXION" },
	{ 0x001F7C, "Witelcom AS", "WITELCOM-AS" },
	{ 0x001F7D, "embedded wireless GmbH", "EMBEDDED-WIRELESS" },
	{ 0x001F7E, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001F7F, "Phabrix Limited", "PHABRIX" },
	{ 0x001F80, "Lucas Holding bv", "LUCAS-HOLDING" },
	{ 0x001F81, "Accel Semiconductor Corp", "ACCEL-SEMICONDUCTOR" },
	{ 0x001F82, "Cal-Comp Electronics & Communications Co., Ltd", "CAL-COMP-ELECTRONICS-COMMUNICATION" },
	{ 0x001F83, "Teleplan Technology Services Sdn Bhd", "TELEPLAN-SERVICES-SDN-BHD" },
	{ 0x001F84, "Gigle Semiconductor", "GIGLE-SEMICONDUCTOR" },
	{ 0x001F85, "Apriva ISS, LLC", "APRIVA-ISS-LLC" },
	{ 0x001F86, "digEcor", "DIGECOR" },
	{ 0x001F87, "Skydigital Inc.", "SKYDIGITAL" },
	{ 0x001F88, "FMS Force Measuring Systems AG", "FMS-FORCE-MEASURING" },
	{ 0x001F89, "Signalion GmbH", "SIGNALION" },
	{ 0x001F8A, "Ellion Digital Inc.", "ELLION-DIGITAL" },
	{ 0x001F8B, "Cache IQ", "CACHE-IQ" },
	{ 0x001F8C, "CCS Inc.", "CCS" },
	{ 0x001F8D, "Ingenieurbuero Stark GmbH und Ko. KG", "INGENIEURBUERO-STARK-UND-KO" },
	{ 0x001F8E, "Metris USA Inc.", "METRIS-USA" },
	{ 0x001F8F, "Shanghai Bellmann Digital Source Co.,Ltd.", "SHANGHAI-BELLMANN-DIGITAL-SOURCE" },
	{ 0x001F90, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x001F91, "DBS Lodging Technologies, LLC", "DBS-LODGING-TECHNOLOGIES-LLC" },
	{ 0x001F92, "VideoIQ, Inc.", "VIDEOIQ" },
	{ 0x001F93, "Xiotech Corporation", "XIOTECH" },
	{ 0x001F94, "Lascar Electronics Ltd", "LASCAR-ELECTRONICS" },
	{ 0x001F95, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x001F96, "APROTECH CO.LTD", "APROTECH" },
	{ 0x001F97, "BERTANA SRL", "BERTANA-SRL" },
	{ 0x001F98, "DAIICHI-DENTSU LTD.", "DAIICHI-DENTSU" },
	{ 0x001F99, "SERONICS co.ltd", "SERONICS" },
	{ 0x001F9A, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x001F9B, "POSBRO", "POSBRO" },
	{ 0x001F9C, "LEDCO", "LEDCO" },
	{ 0x001F9D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001F9E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001F9F, "Thomson Telecom Belgium", "THOMSON-TELECOM-BELGIUM" },
	{ 0x001FA0, "A10 Networks", "A10-NETWORKS" },
	{ 0x001FA1, "Gtran Inc", "GTRAN" },
	{ 0x001FA2, "Datron World Communications, Inc.", "DATRON-WORLD-COMMUNICATION" },
	{ 0x001FA3, "T&W Electronics(Shenzhen)Co.,Ltd.", "T-W-ELECTRONICS-SHENZHEN" },
	{ 0x001FA4, "ShenZhen Gongjin Electronics Co.,Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0x001FA5, "Blue-White Industries", "BLUE-WHITE-INDUSTRIES" },
	{ 0x001FA6, "Stilo srl", "STILO-SRL" },
	{ 0x001FA7, "Sony Computer Entertainment Inc.", "SONY-ENTERTAINMENT" },
	{ 0x001FA8, "Smart Energy Instruments Inc.", "SMART-ENERGY-INSTRUMENTS" },
	{ 0x001FA9, "Atlanta DTH, Inc.", "ATLANTA-DTH" },
	{ 0x001FAA, "Taseon, Inc.", "TASEON" },
	{ 0x001FAB, "I.S HIGH TECH.INC", "I-S-HIGH-TECH" },
	{ 0x001FAC, "Goodmill Systems Ltd", "GOODMILL" },
	{ 0x001FAD, "Brown Innovations, Inc", "BROWN-INNOVATIONS" },
	{ 0x001FAE, "Blick South Africa (Pty) Ltd", "BLICK-SOUTH-AFRICA-PTY" },
	{ 0x001FAF, "NextIO, Inc.", "NEXTIO" },
	{ 0x001FB0, "TimeIPS, Inc.", "TIMEIPS" },
	{ 0x001FB1, "Cybertech Inc.", "CYBERTECH" },
	{ 0x001FB2, "Sontheim Industrie Elektronik GmbH", "SONTHEIM-INDUSTRIE-ELEKTRONIK" },
	{ 0x001FB3, "2Wire", "2WIRE" },
	{ 0x001FB4, "SmartShare Systems", "SMARTSHARE" },
	{ 0x001FB5, "I/O Interconnect Inc.", "I/O-INTERCONNECT" },
	{ 0x001FB6, "Chi Lin Technology Co., Ltd.", "CHI-LIN" },
	{ 0x001FB7, "WiMate Technologies Corp.", "WIMATE-TECHNOLOGIES" },
	{ 0x001FB8, "Universal Remote Control, Inc.", "UNIVERSAL-REMOTE-CONTROL" },
	{ 0x001FB9, "Paltronics", "PALTRONICS" },
	{ 0x001FBA, "BoYoung Tech. & Marketing, Inc.", "BOYOUNG-TECH-MARKETING" },
	{ 0x001FBB, "Xenatech Co.,LTD", "XENATECH" },
	{ 0x001FBC, "EVGA Corporation", "EVGA" },
	{ 0x001FBD, "Kyocera Wireless Corp.", "KYOCERA-WIRELESS" },
	{ 0x001FBE, "Shenzhen Mopnet Industrial Co.,Ltd", "SHENZHEN-MOPNET-INDUSTRIAL" },
	{ 0x001FBF, "Fulhua Microelectronics Corp. Taiwan Branch", "FULHUA-MICROELECTRONICS-TAIWAN-BRANCH" },
	{ 0x001FC0, "Control Express Finland Oy", "CONTROL-EXPRESS-FINLAND-OY" },
	{ 0x001FC1, "Hanlong Technology Co.,LTD", "HANLONG" },
	{ 0x001FC2, "Jow Tong Technology Co Ltd", "JOW-TONG" },
	{ 0x001FC3, "SmartSynch, Inc", "SMARTSYNCH" },
	{ 0x001FC4, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x001FC5, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x001FC6, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x001FC7, "Casio Hitachi Mobile Comunications Co., Ltd.", "CASIO-HITACHI-MOBILE-COMUNICATIONS" },
	{ 0x001FC8, "Up-Today Industrial Co., Ltd.", "UP-TODAY-INDUSTRIAL" },
	{ 0x001FC9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001FCA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x001FCB, "NIW Solutions", "NIW-SOLUTIONS" },
	{ 0x001FCC, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x001FCD, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x001FCE, "QTECH LLC", "QTECH-LLC" },
	{ 0x001FCF, "MSI Technology GmbH", "MSI" },
	{ 0x001FD0, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x001FD1, "OPTEX CO.,LTD.", "OPTEX" },
	{ 0x001FD2, "COMMTECH TECHNOLOGY MACAO COMMERCIAL OFFSHORE LTD.", "COMMTECH-MACAO-COMMERCIAL-OFFSHORE" },
	{ 0x001FD3, "RIVA Networks Inc.", "RIVA-NETWORKS" },
	{ 0x001FD4, "4IPNET, INC.", "4IPNET" },
	{ 0x001FD5, "MICRORISC s.r.o.", "MICRORISC-S-R-O" },
	{ 0x001FD6, "Shenzhen Allywll", "SHENZHEN-ALLYWLL" },
	{ 0x001FD7, "TELERAD SA", "TELERAD" },
	{ 0x001FD8, "A-TRUST COMPUTER CORPORATION", "A-TRUST" },
	{ 0x001FD9, "RSD Communications Ltd", "RSD-COMMUNICATION" },
	{ 0x001FDA, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x001FDB, "Network Supply Corp.,", "NETWORK-SUPPLY" },
	{ 0x001FDC, "Mobile Safe Track Ltd", "MOBILE-SAFE-TRACK" },
	{ 0x001FDD, "GDI LLC", "GDI-LLC" },
	{ 0x001FDE, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001FDF, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x001FE0, "EdgeVelocity Corp", "EDGEVELOCITY" },
	{ 0x001FE1, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x001FE2, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x001FE3, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x001FE4, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x001FE5, "In-Circuit GmbH", "IN-CIRCUIT" },
	{ 0x001FE6, "Alphion Corporation", "ALPHION" },
	{ 0x001FE7, "Simet", "SIMET" },
	{ 0x001FE8, "KURUSUGAWA Electronics Industry Inc,.", "KURUSUGAWA-ELECTRONICS-INDUSTRY" },
	{ 0x001FE9, "Printrex, Inc.", "PRINTREX" },
	{ 0x001FEA, "Applied Media Technologies Corporation", "APPLIED-MEDIA-TECHNOLOGIES" },
	{ 0x001FEB, "Trio Datacom Pty Ltd", "TRIO-DATACOM-PTY" },
	{ 0x001FEC, "Synapse Électronique", "SYNAPSE-ÉLECTRONIQUE" },
	{ 0x001FED, "Tecan Systems Inc.", "TECAN" },
	{ 0x001FEE, "ubisys technologies GmbH", "UBISYS-TECHNOLOGIES" },
	{ 0x001FEF, "SHINSEI INDUSTRIES CO.,LTD", "SHINSEI-INDUSTRIES" },
	{ 0x001FF0, "Audio Partnership", "AUDIO-PARTNERSHIP" },
	{ 0x001FF1, "Paradox Hellas S.A.", "PARADOX-HELLAS" },
	{ 0x001FF2, "VIA Technologies, Inc.", "VIA-TECHNOLOGIES" },
	{ 0x001FF3, "Apple", "APPLE" },
	{ 0x001FF4, "Power Monitors, Inc.", "POWER-MONITORS" },
	{ 0x001FF5, "Kongsberg Defence & Aerospace", "KONGSBERG-DEFENCE-AEROSPACE" },
	{ 0x001FF6, "PS Audio International", "PS-AUDIO" },
	{ 0x001FF7, "Nakajima All Precision Co., Ltd.", "NAKAJIMA-ALL-PRECISION" },
	{ 0x001FF8, "Siemens AG, Sector Industry, Drive Technologies, Motion Control Systems", "SIEMENS-SECTOR-INDUSTRY-DRIVE-TECHNOLOGIES-MOTION-CONTROL" },
	{ 0x001FF9, "Advanced Knowledge Associates", "ADVANCED-KNOWLEDGE-ASSOCIATES" },
	{ 0x001FFA, "Coretree, Co, Ltd", "CORETREE" },
	{ 0x001FFB, "Green Packet Bhd", "GREEN-PACKET-BHD" },
	{ 0x001FFC, "Riccius+Sohn GmbH", "RICCIUS+SOHN" },
	{ 0x001FFD, "Indigo Mobile Technologies Corp.", "INDIGO-MOBILE-TECHNOLOGIES" },
	{ 0x001FFE, "HPN Supply Chain", "HPN-SUPPLY-CHAIN" },
	{ 0x001FFF, "Respironics, Inc.", "RESPIRONICS" },
	{ 0x002000, "LEXMARK INTERNATIONAL, INC.", "LEXMARK" },
	{ 0x002001, "DSP SOLUTIONS, INC.", "DSP-SOLUTIONS" },
	{ 0x002002, "SERITECH ENTERPRISE CO., LTD.", "SERITECH" },
	{ 0x002003, "PIXEL POWER LTD.", "PIXEL-POWER" },
	{ 0x002004, "YAMATAKE-HONEYWELL CO., LTD.", "YAMATAKE-HONEYWELL" },
	{ 0x002005, "SIMPLE TECHNOLOGY", "SIMPLE" },
	{ 0x002006, "GARRETT COMMUNICATIONS, INC.", "GARRETT-COMMUNICATION" },
	{ 0x002007, "SFA, INC.", "SFA" },
	{ 0x002008, "CABLE & COMPUTER TECHNOLOGY", "CABLE" },
	{ 0x002009, "PACKARD BELL ELEC., INC.", "PACKARD-BELL-ELEC" },
	{ 0x00200A, "SOURCE-COMM CORP.", "SOURCE-COMM" },
	{ 0x00200B, "OCTAGON SYSTEMS CORP.", "OCTAGON" },
	{ 0x00200C, "ADASTRA SYSTEMS CORP.", "ADASTRA" },
	{ 0x00200D, "CARL ZEISS", "CARL-ZEISS" },
	{ 0x00200E, "SATELLITE TECHNOLOGY MGMT, INC", "SATELLITE-MGMT" },
	{ 0x00200F, "TANBAC CO., LTD.", "TANBAC" },
	{ 0x002010, "JEOL SYSTEM TECHNOLOGY CO. LTD", "JEOL-SYSTEM" },
	{ 0x002011, "CANOPUS CO., LTD.", "CANOPUS" },
	{ 0x002012, "CAMTRONICS MEDICAL SYSTEMS", "CAMTRONICS-MEDICAL" },
	{ 0x002013, "DIVERSIFIED TECHNOLOGY, INC.", "DIVERSIFIED" },
	{ 0x002014, "GLOBAL VIEW CO., LTD.", "GLOBAL-VIEW" },
	{ 0x002015, "ACTIS COMPUTER SA", "ACTIS" },
	{ 0x002016, "SHOWA ELECTRIC WIRE & CABLE CO", "SHOWA-ELECTRIC-WIRE-CABLE" },
	{ 0x002017, "ORBOTECH", "ORBOTECH" },
	{ 0x002018, "CIS TECHNOLOGY INC.", "CIS" },
	{ 0x002019, "OHLER GmbH", "OHLER" },
	{ 0x00201A, "MRV Communications, Inc.", "MRV-COMMUNICATION" },
	{ 0x00201B, "NORTHERN TELECOM/NETWORK", "NORTHERN-TELECOM/NETWORK" },
	{ 0x00201C, "EXCEL, INC.", "EXCEL" },
	{ 0x00201D, "KATANA PRODUCTS", "KATANA-PRODUCTS" },
	{ 0x00201E, "NETQUEST CORPORATION", "NETQUEST" },
	{ 0x00201F, "BEST POWER TECHNOLOGY, INC.", "BEST-POWER" },
	{ 0x002020, "MEGATRON COMPUTER INDUSTRIES PTY, LTD.", "MEGATRON-INDUSTRIES-PTY" },
	{ 0x002021, "ALGORITHMS SOFTWARE PVT. LTD.", "ALGORITHMS-SOFTWARE-PVT" },
	{ 0x002022, "NMS Communications", "NMS-COMMUNICATION" },
	{ 0x002023, "T.C. TECHNOLOGIES PTY. LTD", "T-C-TECHNOLOGIES-PTY" },
	{ 0x002024, "PACIFIC COMMUNICATION SCIENCES", "PACIFIC-COMMUNICATION-SCIENCES" },
	{ 0x002025, "CONTROL TECHNOLOGY, INC.", "CONTROL" },
	{ 0x002026, "AMKLY SYSTEMS, INC.", "AMKLY" },
	{ 0x002027, "MING FORTUNE INDUSTRY CO., LTD", "MING-FORTUNE-INDUSTRY" },
	{ 0x002028, "WEST EGG SYSTEMS, INC.", "WEST-EGG" },
	{ 0x002029, "TELEPROCESSING PRODUCTS, INC.", "TELEPROCESSING-PRODUCTS" },
	{ 0x00202A, "N.V. DZINE", "N-V-DZINE" },
	{ 0x00202B, "ADVANCED TELECOMMUNICATIONS MODULES, LTD.", "ADVANCED-TELECOMMUNICATIONS-MODULES" },
	{ 0x00202C, "WELLTRONIX CO., LTD.", "WELLTRONIX" },
	{ 0x00202D, "TAIYO CORPORATION", "TAIYO" },
	{ 0x00202E, "DAYSTAR DIGITAL", "DAYSTAR-DIGITAL" },
	{ 0x00202F, "ZETA COMMUNICATIONS, LTD.", "ZETA-COMMUNICATION" },
	{ 0x002030, "ANALOG & DIGITAL SYSTEMS", "ANALOG-DIGITAL" },
	{ 0x002031, "Tattile SRL", "TATTILE-SRL" },
	{ 0x002032, "ALCATEL TAISEL", "ALCATEL-TAISEL" },
	{ 0x002033, "SYNAPSE TECHNOLOGIES, INC.", "SYNAPSE-TECHNOLOGIES" },
	{ 0x002034, "ROTEC INDUSTRIEAUTOMATION GMBH", "ROTEC-INDUSTRIEAUTOMATION" },
	{ 0x002035, "IBM Corp", "IBM" },
	{ 0x002036, "BMC SOFTWARE", "BMC-SOFTWARE" },
	{ 0x002037, "SEAGATE TECHNOLOGY", "SEAGATE" },
	{ 0x002038, "VME MICROSYSTEMS INTERNATIONAL CORPORATION", "VME-MICROSYSTEMS" },
	{ 0x002039, "SCINETS", "SCINETS" },
	{ 0x00203A, "DIGITAL BI0METRICS INC.", "DIGITAL-BI0METRICS" },
	{ 0x00203B, "WISDM LTD.", "WISDM" },
	{ 0x00203C, "EUROTIME AB", "EUROTIME-AB" },
	{ 0x00203D, "Honeywell ECC", "HONEYWELL-ECC" },
	{ 0x00203E, "LogiCan Technologies, Inc.", "LOGICAN-TECHNOLOGIES" },
	{ 0x00203F, "JUKI CORPORATION", "JUKI" },
	{ 0x002040, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002041, "DATA NET", "DATA-NET" },
	{ 0x002042, "DATAMETRICS CORP.", "DATAMETRICS" },
	{ 0x002043, "NEURON COMPANY LIMITED", "NEURON-COMPANY" },
	{ 0x002044, "GENITECH PTY LTD", "GENITECH-PTY" },
	{ 0x002045, "ION Networks, Inc.", "ION-NETWORKS" },
	{ 0x002046, "CIPRICO, INC.", "CIPRICO" },
	{ 0x002047, "STEINBRECHER CORP.", "STEINBRECHER" },
	{ 0x002048, "Marconi Communications", "MARCONI-COMMUNICATION" },
	{ 0x002049, "COMTRON, INC.", "COMTRON" },
	{ 0x00204A, "PRONET GMBH", "PRONET" },
	{ 0x00204B, "AUTOCOMPUTER CO., LTD.", "AUTOCOMPUTER" },
	{ 0x00204C, "MITRON COMPUTER PTE LTD.", "MITRON-PTE" },
	{ 0x00204D, "INOVIS GMBH", "INOVIS" },
	{ 0x00204E, "NETWORK SECURITY SYSTEMS, INC.", "NETWORK-SECURITY" },
	{ 0x00204F, "DEUTSCHE AEROSPACE AG", "DEUTSCHE-AEROSPACE" },
	{ 0x002050, "KOREA COMPUTER INC.", "KOREA" },
	{ 0x002051, "Verilink Corporation", "VERILINK" },
	{ 0x002052, "RAGULA SYSTEMS", "RAGULA" },
	{ 0x002053, "HUNTSVILLE MICROSYSTEMS, INC.", "HUNTSVILLE-MICROSYSTEMS" },
	{ 0x002054, "Sycamore Networks", "SYCAMORE-NETWORKS" },
	{ 0x002055, "ALTECH CO., LTD.", "ALTECH" },
	{ 0x002056, "NEOPRODUCTS", "NEOPRODUCTS" },
	{ 0x002057, "TITZE DATENTECHNIK GmbH", "TITZE-DATENTECHNIK" },
	{ 0x002058, "ALLIED SIGNAL INC.", "ALLIED-SIGNAL" },
	{ 0x002059, "MIRO COMPUTER PRODUCTS AG", "MIRO-PRODUCTS" },
	{ 0x00205A, "COMPUTER IDENTICS", "IDENTICS" },
	{ 0x00205B, "Kentrox, LLC", "KENTROX-LLC" },
	{ 0x00205C, "InterNet Systems of Florida, Inc.", "INTERNET-OF-FLORIDA" },
	{ 0x00205D, "NANOMATIC OY", "NANOMATIC-OY" },
	{ 0x00205E, "CASTLE ROCK, INC.", "CASTLE-ROCK" },
	{ 0x00205F, "GAMMADATA COMPUTER GMBH", "GAMMADATA" },
	{ 0x002060, "ALCATEL ITALIA S.p.A.", "ALCATEL-ITALIA-S-P-A" },
	{ 0x002061, "GarrettCom, Inc.", "GARRETTCOM" },
	{ 0x002062, "SCORPION LOGIC, LTD.", "SCORPION-LOGIC" },
	{ 0x002063, "WIPRO INFOTECH LTD.", "WIPRO-INFOTECH" },
	{ 0x002064, "PROTEC MICROSYSTEMS, INC.", "PROTEC-MICROSYSTEMS" },
	{ 0x002065, "SUPERNET NETWORKING INC.", "SUPERNET-NETWORKING" },
	{ 0x002066, "GENERAL MAGIC, INC.", "GENERAL-MAGIC" },
	{ 0x002067, "PRIVATE", "PRIVATE" },
	{ 0x002068, "ISDYNE", "ISDYNE" },
	{ 0x002069, "ISDN SYSTEMS CORPORATION", "ISDN" },
	{ 0x00206A, "OSAKA COMPUTER CORP.", "OSAKA" },
	{ 0x00206B, "KONICA MINOLTA HOLDINGS, INC.", "KONICA-MINOLTA-HOLDINGS" },
	{ 0x00206C, "EVERGREEN TECHNOLOGY CORP.", "EVERGREEN" },
	{ 0x00206D, "DATA RACE, INC.", "DATA-RACE" },
	{ 0x00206E, "XACT, INC.", "XACT" },
	{ 0x00206F, "FLOWPOINT CORPORATION", "FLOWPOINT" },
	{ 0x002070, "HYNET, LTD.", "HYNET" },
	{ 0x002071, "IBR GMBH", "IBR" },
	{ 0x002072, "WORKLINK INNOVATIONS", "WORKLINK-INNOVATIONS" },
	{ 0x002073, "FUSION SYSTEMS CORPORATION", "FUSION" },
	{ 0x002074, "SUNGWOON SYSTEMS", "SUNGWOON" },
	{ 0x002075, "MOTOROLA COMMUNICATION ISRAEL", "MOTOROLA-COMMUNICATION-ISRAEL" },
	{ 0x002076, "REUDO CORPORATION", "REUDO" },
	{ 0x002077, "KARDIOS SYSTEMS CORP.", "KARDIOS" },
	{ 0x002078, "RUNTOP, INC.", "RUNTOP" },
	{ 0x002079, "MIKRON GMBH", "MIKRON" },
	{ 0x00207A, "WiSE Communications, Inc.", "WISE-COMMUNICATION" },
	{ 0x00207B, "Intel Corporation", "INTEL" },
	{ 0x00207C, "AUTEC GmbH", "AUTEC" },
	{ 0x00207D, "ADVANCED COMPUTER APPLICATIONS", "ADVANCED-APPLICATIONS" },
	{ 0x00207E, "FINECOM Co., Ltd.", "FINECOM" },
	{ 0x00207F, "KYOEI SANGYO CO., LTD.", "KYOEI-SANGYO" },
	{ 0x002080, "SYNERGY (UK) LTD.", "SYNERGY-UK" },
	{ 0x002081, "TITAN ELECTRONICS", "TITAN-ELECTRONICS" },
	{ 0x002082, "ONEAC CORPORATION", "ONEAC" },
	{ 0x002083, "PRESTICOM INCORPORATED", "PRESTICOM-INCORPORATED" },
	{ 0x002084, "OCE PRINTING SYSTEMS, GMBH", "OCE-PRINTING" },
	{ 0x002085, "Eaton Corporation", "EATON" },
	{ 0x002086, "MICROTECH ELECTRONICS LIMITED", "MICROTECH-ELECTRONICS" },
	{ 0x002087, "MEMOTEC, INC.", "MEMOTEC" },
	{ 0x002088, "GLOBAL VILLAGE COMMUNICATION", "GLOBAL-VILLAGE-COMMUNICATION" },
	{ 0x002089, "T3PLUS NETWORKING, INC.", "T3PLUS-NETWORKING" },
	{ 0x00208A, "SONIX COMMUNICATIONS, LTD.", "SONIX-COMMUNICATION" },
	{ 0x00208B, "LAPIS TECHNOLOGIES, INC.", "LAPIS-TECHNOLOGIES" },
	{ 0x00208C, "GALAXY NETWORKS, INC.", "GALAXY-NETWORKS" },
	{ 0x00208D, "CMD TECHNOLOGY", "CMD" },
	{ 0x00208E, "CHEVIN SOFTWARE ENG. LTD.", "CHEVIN-SOFTWARE-ENG" },
	{ 0x00208F, "ECI TELECOM LTD.", "ECI-TELECOM" },
	{ 0x002090, "ADVANCED COMPRESSION TECHNOLOGY, INC.", "ADVANCED-COMPRESSION" },
	{ 0x002091, "J125, NATIONAL SECURITY AGENCY", "J125-NATIONAL-SECURITY-AGENCY" },
	{ 0x002092, "CHESS ENGINEERING B.V.", "CHESS-ENGINEERING" },
	{ 0x002093, "LANDINGS TECHNOLOGY CORP.", "LANDINGS" },
	{ 0x002094, "CUBIX CORPORATION", "CUBIX" },
	{ 0x002095, "RIVA ELECTRONICS", "RIVA-ELECTRONICS" },
	{ 0x002096, "Invensys", "INVENSYS" },
	{ 0x002097, "APPLIED SIGNAL TECHNOLOGY", "APPLIED-SIGNAL" },
	{ 0x002098, "HECTRONIC AB", "HECTRONIC-AB" },
	{ 0x002099, "BON ELECTRIC CO., LTD.", "BON-ELECTRIC" },
	{ 0x00209A, "THE 3DO COMPANY", "THE-3DO-COMPANY" },
	{ 0x00209B, "ERSAT ELECTRONIC GMBH", "ERSAT" },
	{ 0x00209C, "PRIMARY ACCESS CORP.", "PRIMARY-ACCESS" },
	{ 0x00209D, "LIPPERT AUTOMATIONSTECHNIK", "LIPPERT-AUTOMATIONSTECHNIK" },
	{ 0x00209E, "BROWN'S OPERATING SYSTEM SERVICES, LTD.", "BROWN-S-OPERATING-SYSTEM-SERVICES" },
	{ 0x00209F, "MERCURY COMPUTER SYSTEMS, INC.", "MERCURY" },
	{ 0x0020A0, "OA LABORATORY CO., LTD.", "OA-LABORATORY" },
	{ 0x0020A1, "DOVATRON", "DOVATRON" },
	{ 0x0020A2, "GALCOM NETWORKING LTD.", "GALCOM-NETWORKING" },
	{ 0x0020A3, "Harmonic, Inc", "HARMONIC" },
	{ 0x0020A4, "MULTIPOINT NETWORKS", "MULTIPOINT-NETWORKS" },
	{ 0x0020A5, "API ENGINEERING", "API-ENGINEERING" },
	{ 0x0020A6, "Proxim Wireless", "PROXIM-WIRELESS" },
	{ 0x0020A7, "PAIRGAIN TECHNOLOGIES, INC.", "PAIRGAIN-TECHNOLOGIES" },
	{ 0x0020A8, "SAST TECHNOLOGY CORP.", "SAST" },
	{ 0x0020A9, "WHITE HORSE INDUSTRIAL", "WHITE-HORSE-INDUSTRIAL" },
	{ 0x0020AA, "Ericsson Television Limited", "ERICSSON-TELEVISION" },
	{ 0x0020AB, "MICRO INDUSTRIES CORP.", "MICRO-INDUSTRIES" },
	{ 0x0020AC, "INTERFLEX DATENSYSTEME GMBH", "INTERFLEX-DATENSYSTEME" },
	{ 0x0020AD, "LINQ SYSTEMS", "LINQ" },
	{ 0x0020AE, "ORNET DATA COMMUNICATION TECH.", "ORNET-DATA-COMMUNICATION-TECH" },
	{ 0x0020AF, "3COM CORPORATION", "3COM" },
	{ 0x0020B0, "GATEWAY DEVICES, INC.", "GATEWAY-DEVICES" },
	{ 0x0020B1, "COMTECH RESEARCH INC.", "COMTECH-RESEARCH" },
	{ 0x0020B2, "GKD Gesellschaft Fur Kommunikation Und Datentechnik", "GKD-GESELLSCHAFT-FUR-KOMMUNIKATION-UND-DATENTECHNIK" },
	{ 0x0020B3, "Tattile SRL", "TATTILE-SRL" },
	{ 0x0020B4, "TERMA ELEKTRONIK AS", "TERMA-ELEKTRONIK-AS" },
	{ 0x0020B5, "YASKAWA ELECTRIC CORPORATION", "YASKAWA-ELECTRIC" },
	{ 0x0020B6, "AGILE NETWORKS, INC.", "AGILE-NETWORKS" },
	{ 0x0020B7, "NAMAQUA COMPUTERWARE", "NAMAQUA-COMPUTERWARE" },
	{ 0x0020B8, "PRIME OPTION, INC.", "PRIME-OPTION" },
	{ 0x0020B9, "METRICOM, INC.", "METRICOM" },
	{ 0x0020BA, "CENTER FOR HIGH PERFORMANCE", "CENTER-FOR-HIGH-PERFORMANCE" },
	{ 0x0020BB, "ZAX CORPORATION", "ZAX" },
	{ 0x0020BC, "Long Reach Networks Pty Ltd", "LONG-REACH-NETWORKS-PTY" },
	{ 0x0020BD, "NIOBRARA R & D CORPORATION", "NIOBRARA-R-D" },
	{ 0x0020BE, "LAN ACCESS CORP.", "LAN-ACCESS" },
	{ 0x0020BF, "AEHR TEST SYSTEMS", "AEHR-TEST" },
	{ 0x0020C0, "PULSE ELECTRONICS, INC.", "PULSE-ELECTRONICS" },
	{ 0x0020C1, "SAXA, Inc.", "SAXA" },
	{ 0x0020C2, "TEXAS MEMORY SYSTEMS, INC.", "TEXAS-MEMORY" },
	{ 0x0020C3, "COUNTER SOLUTIONS LTD.", "COUNTER-SOLUTIONS" },
	{ 0x0020C4, "INET,INC.", "INET" },
	{ 0x0020C5, "EAGLE TECHNOLOGY", "EAGLE" },
	{ 0x0020C6, "NECTEC", "NECTEC" },
	{ 0x0020C7, "AKAI Professional M.I. Corp.", "AKAI-PROFESSIONAL-M-I" },
	{ 0x0020C8, "LARSCOM INCORPORATED", "LARSCOM-INCORPORATED" },
	{ 0x0020C9, "VICTRON BV", "VICTRON" },
	{ 0x0020CA, "DIGITAL OCEAN", "DIGITAL-OCEAN" },
	{ 0x0020CB, "PRETEC ELECTRONICS CORP.", "PRETEC-ELECTRONICS" },
	{ 0x0020CC, "DIGITAL SERVICES, LTD.", "DIGITAL-SERVICES" },
	{ 0x0020CD, "HYBRID NETWORKS, INC.", "HYBRID-NETWORKS" },
	{ 0x0020CE, "LOGICAL DESIGN GROUP, INC.", "LOGICAL-DESIGN-GROUP" },
	{ 0x0020CF, "TEST & MEASUREMENT SYSTEMS INC", "TEST-MEASUREMENT" },
	{ 0x0020D0, "VERSALYNX CORPORATION", "VERSALYNX" },
	{ 0x0020D1, "MICROCOMPUTER SYSTEMS (M) SDN.", "MICROCOMPUTER-M-SDN" },
	{ 0x0020D2, "RAD DATA COMMUNICATIONS, LTD.", "RAD-DATA-COMMUNICATION" },
	{ 0x0020D3, "OST (OUEST STANDARD TELEMATIQU", "OST-OUEST-STANDARD-TELEMATIQU" },
	{ 0x0020D4, "CABLETRON - ZEITTNET INC.", "CABLETRON---ZEITTNET" },
	{ 0x0020D5, "VIPA GMBH", "VIPA" },
	{ 0x0020D6, "BREEZECOM", "BREEZECOM" },
	{ 0x0020D7, "JAPAN MINICOMPUTER SYSTEMS CO., Ltd.", "JAPAN-MINICOMPUTER" },
	{ 0x0020D8, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x0020D9, "PANASONIC TECHNOLOGIES, INC./MIECO-US", "PANASONIC-TECHNOLOGIES-/MIECO-US" },
	{ 0x0020DA, "Alcatel North America ESD", "ALCATEL-NORTH-AMERICA-ESD" },
	{ 0x0020DB, "XNET TECHNOLOGY, INC.", "XNET" },
	{ 0x0020DC, "DENSITRON TAIWAN LTD.", "DENSITRON-TAIWAN" },
	{ 0x0020DD, "Cybertec Pty Ltd", "CYBERTEC-PTY" },
	{ 0x0020DE, "JAPAN DIGITAL LABORAT'Y CO.LTD", "JAPAN-DIGITAL-LABORAT-Y" },
	{ 0x0020DF, "KYOSAN ELECTRIC MFG. CO., LTD.", "KYOSAN-ELECTRIC-MFG" },
	{ 0x0020E0, "Actiontec Electronics, Inc.", "ACTIONTEC-ELECTRONICS" },
	{ 0x0020E1, "ALAMAR ELECTRONICS", "ALAMAR-ELECTRONICS" },
	{ 0x0020E2, "INFORMATION RESOURCE ENGINEERING", "INFORMATION-RESOURCE-ENGINEERING" },
	{ 0x0020E3, "MCD KENCOM CORPORATION", "MCD-KENCOM" },
	{ 0x0020E4, "HSING TECH ENTERPRISE CO., LTD", "HSING-TECH" },
	{ 0x0020E5, "APEX DATA, INC.", "APEX-DATA" },
	{ 0x0020E6, "LIDKOPING MACHINE TOOLS AB", "LIDKOPING-MACHINE-TOOLS-AB" },
	{ 0x0020E7, "B&W NUCLEAR SERVICE COMPANY", "B-W-NUCLEAR-SERVICE-COMPANY" },
	{ 0x0020E8, "DATATREK CORPORATION", "DATATREK" },
	{ 0x0020E9, "DANTEL", "DANTEL" },
	{ 0x0020EA, "EFFICIENT NETWORKS, INC.", "EFFICIENT-NETWORKS" },
	{ 0x0020EB, "CINCINNATI MICROWAVE, INC.", "CINCINNATI-MICROWAVE" },
	{ 0x0020EC, "TECHWARE SYSTEMS CORP.", "TECHWARE" },
	{ 0x0020ED, "GIGA-BYTE TECHNOLOGY CO., LTD.", "GIGA-BYTE" },
	{ 0x0020EE, "GTECH CORPORATION", "GTECH" },
	{ 0x0020EF, "USC CORPORATION", "USC" },
	{ 0x0020F0, "UNIVERSAL MICROELECTRONICS CO.", "UNIVERSAL-MICROELECTRONICS" },
	{ 0x0020F1, "ALTOS INDIA LIMITED", "ALTOS-INDIA" },
	{ 0x0020F2, "Oracle Corporation", "ORACLE" },
	{ 0x0020F3, "RAYNET CORPORATION", "RAYNET" },
	{ 0x0020F4, "SPECTRIX CORPORATION", "SPECTRIX" },
	{ 0x0020F5, "PANDATEL AG", "PANDATEL" },
	{ 0x0020F6, "NET TEK AND KARLNET, INC.", "NET-TEK-AND-KARLNET" },
	{ 0x0020F7, "CYBERDATA CORPORATION", "CYBERDATA" },
	{ 0x0020F8, "CARRERA COMPUTERS, INC.", "CARRERA-COMPUTERS" },
	{ 0x0020F9, "PARALINK NETWORKS, INC.", "PARALINK-NETWORKS" },
	{ 0x0020FA, "GDE SYSTEMS, INC.", "GDE" },
	{ 0x0020FB, "OCTEL COMMUNICATIONS CORP.", "OCTEL-COMMUNICATION" },
	{ 0x0020FC, "MATROX", "MATROX" },
	{ 0x0020FD, "ITV TECHNOLOGIES, INC.", "ITV-TECHNOLOGIES" },
	{ 0x0020FE, "TOPWARE INC. / GRAND COMPUTER", "TOPWARE-/-GRAND" },
	{ 0x0020FF, "SYMMETRICAL TECHNOLOGIES", "SYMMETRICAL-TECHNOLOGIES" },
	{ 0x002100, "GemTek Technology Co., Ltd.", "GEMTEK" },
	{ 0x002101, "Aplicaciones Electronicas Quasar (AEQ)", "APLICACIONES-ELECTRONICAS-QUASAR-AEQ" },
	{ 0x002102, "UpdateLogic Inc.", "UPDATELOGIC" },
	{ 0x002103, "GHI Electronics, LLC", "GHI-ELECTRONICS-LLC" },
	{ 0x002104, "Gigaset Communications GmbH", "GIGASET-COMMUNICATION" },
	{ 0x002105, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x002106, "RIM Testing Services", "RIM-TESTING-SERVICES" },
	{ 0x002107, "Seowonintech Co Ltd.", "SEOWONINTECH" },
	{ 0x002108, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x002109, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x00210A, "byd:sign Corporation", "BYD:SIGN" },
	{ 0x00210B, "GEMINI TRAZE RFID PVT. LTD.", "GEMINI-TRAZE-RFID-PVT" },
	{ 0x00210C, "Cymtec Systems, Inc.", "CYMTEC" },
	{ 0x00210D, "SAMSIN INNOTEC", "SAMSIN-INNOTEC" },
	{ 0x00210E, "Orpak Systems L.T.D.", "ORPAK-L-T-D" },
	{ 0x00210F, "Cernium Corp", "CERNIUM" },
	{ 0x002110, "Clearbox Systems", "CLEARBOX" },
	{ 0x002111, "Uniphone Inc.", "UNIPHONE" },
	{ 0x002112, "WISCOM SYSTEM CO.,LTD", "WISCOM-SYSTEM" },
	{ 0x002113, "Padtec S/A", "PADTEC-S/A" },
	{ 0x002114, "Hylab Technology Inc.", "HYLAB" },
	{ 0x002115, "PHYWE Systeme GmbH & Co. KG", "PHYWE-SYSTEME" },
	{ 0x002116, "Transcon Electronic Systems, spol. s r. o.", "TRANSCON-SPOL-S-R-O" },
	{ 0x002117, "Tellord", "TELLORD" },
	{ 0x002118, "Athena Tech, Inc.", "ATHENA-TECH" },
	{ 0x002119, "Samsung Electro-Mechanics", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x00211A, "LInTech Corporation", "LINTECH" },
	{ 0x00211B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00211C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00211D, "Dataline AB", "DATALINE-AB" },
	{ 0x00211E, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00211F, "SHINSUNG DELTATECH CO.,LTD.", "SHINSUNG-DELTATECH" },
	{ 0x002120, "Sequel Technologies", "SEQUEL-TECHNOLOGIES" },
	{ 0x002121, "VRmagic GmbH", "VRMAGIC" },
	{ 0x002122, "Chip-pro Ltd.", "CHIP-PRO" },
	{ 0x002123, "Aerosat Avionics", "AEROSAT-AVIONICS" },
	{ 0x002124, "Optos Plc", "OPTOS-PLC" },
	{ 0x002125, "KUK JE TONG SHIN Co.,LTD", "KUK-JE-TONG-SHIN" },
	{ 0x002126, "Shenzhen Torch Equipment Co., Ltd.", "SHENZHEN-TORCH-EQUIPMENT" },
	{ 0x002127, "TP-LINK Technology Co., Ltd.", "TP-LINK" },
	{ 0x002128, "Oracle Corporation", "ORACLE" },
	{ 0x002129, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x00212A, "Audiovox Corporation", "AUDIOVOX" },
	{ 0x00212B, "MSA Auer", "MSA-AUER" },
	{ 0x00212C, "SemIndia System Private Limited", "SEMINDIA-SYSTEM-PRIVATE" },
	{ 0x00212D, "SCIMOLEX CORPORATION", "SCIMOLEX" },
	{ 0x00212E, "dresden-elektronik", "DRESDEN-ELEKTRONIK" },
	{ 0x00212F, "Phoebe Micro Inc.", "PHOEBE-MICRO" },
	{ 0x002130, "Keico Hightech Inc.", "KEICO-HIGHTECH" },
	{ 0x002131, "Blynke Inc.", "BLYNKE" },
	{ 0x002132, "Masterclock, Inc.", "MASTERCLOCK" },
	{ 0x002133, "Building B, Inc", "BUILDING-B" },
	{ 0x002134, "Brandywine Communications", "BRANDYWINE-COMMUNICATION" },
	{ 0x002135, "ALCATEL-LUCENT", "ALCATEL-LUCENT" },
	{ 0x002136, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002137, "Bay Controls, LLC", "BAY-CONTROLS-LLC" },
	{ 0x002138, "Cepheid", "CEPHEID" },
	{ 0x002139, "Escherlogic Inc.", "ESCHERLOGIC" },
	{ 0x00213A, "Winchester Systems Inc.", "WINCHESTER" },
	{ 0x00213B, "Berkshire Products, Inc", "BERKSHIRE-PRODUCTS" },
	{ 0x00213C, "AliphCom", "ALIPHCOM" },
	{ 0x00213D, "Cermetek Microelectronics, Inc.", "CERMETEK-MICROELECTRONICS" },
	{ 0x00213E, "TomTom", "TOMTOM" },
	{ 0x00213F, "A-Team Technology Ltd.", "A-TEAM" },
	{ 0x002140, "EN Technologies Inc.", "EN-TECHNOLOGIES" },
	{ 0x002141, "RADLIVE", "RADLIVE" },
	{ 0x002142, "Advanced Control Systems doo", "ADVANCED-CONTROL-DOO" },
	{ 0x002143, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002144, "SS Telecoms", "SS-TELECOMS" },
	{ 0x002145, "Semptian Technologies Ltd.", "SEMPTIAN-TECHNOLOGIES" },
	{ 0x002146, "Sanmina-SCI", "SANMINA-SCI" },
	{ 0x002147, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x002148, "Kaco Solar Korea", "KACO-SOLAR-KOREA" },
	{ 0x002149, "China Daheng Group ,Inc.", "CHINA-DAHENG-GROUP" },
	{ 0x00214A, "Pixel Velocity, Inc", "PIXEL-VELOCITY" },
	{ 0x00214B, "Shenzhen HAMP Science & Technology Co.,Ltd", "SHENZHEN-HAMP-SCIENCE" },
	{ 0x00214C, "SAMSUNG ELECTRONICS CO., LTD.", "SAMSUNG-ELECTRONICS" },
	{ 0x00214D, "Guangzhou Skytone Transmission Technology Com. Ltd.", "GUANGZHOU-SKYTONE-TRANSMISSION-COM" },
	{ 0x00214E, "GS Yuasa Power Supply Ltd.", "GS-YUASA-POWER-SUPPLY" },
	{ 0x00214F, "ALPS Electric Co., Ltd", "ALPS-ELECTRIC" },
	{ 0x002150, "EYEVIEW ELECTRONICS", "EYEVIEW-ELECTRONICS" },
	{ 0x002151, "Millinet Co., Ltd.", "MILLINET" },
	{ 0x002152, "General Satellite Research & Development Limited", "GENERAL-SATELLITE-RESEARCH-DEVELOPMENT" },
	{ 0x002153, "SeaMicro Inc.", "SEAMICRO" },
	{ 0x002154, "D-TACQ Solutions Ltd", "D-TACQ-SOLUTIONS" },
	{ 0x002155, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002156, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002157, "National Datacast, Inc.", "NATIONAL-DATACAST" },
	{ 0x002158, "Style Flying Technology Co.", "STYLE-FLYING" },
	{ 0x002159, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x00215A, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x00215B, "Inotive", "INOTIVE" },
	{ 0x00215C, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x00215D, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x00215E, "IBM Corp", "IBM" },
	{ 0x00215F, "IHSE GmbH", "IHSE" },
	{ 0x002160, "Hidea Solutions Co. Ltd.", "HIDEA-SOLUTIONS" },
	{ 0x002161, "Yournet Inc.", "YOURNET" },
	{ 0x002162, "Nortel", "NORTEL" },
	{ 0x002163, "ASKEY COMPUTER CORP", "ASKEY" },
	{ 0x002164, "Special Design Bureau for Seismic Instrumentation", "SPECIAL-DESIGN-BUREAU-FOR-SEISMIC-INSTRUMENTATION" },
	{ 0x002165, "Presstek Inc.", "PRESSTEK" },
	{ 0x002166, "NovAtel Inc.", "NOVATEL" },
	{ 0x002167, "HWA JIN T&I Corp.", "HWA-JIN-T-I" },
	{ 0x002168, "iVeia, LLC", "IVEIA-LLC" },
	{ 0x002169, "Prologix, LLC.", "PROLOGIX-LLC" },
	{ 0x00216A, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x00216B, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x00216C, "ODVA", "ODVA" },
	{ 0x00216D, "Soltech Co., Ltd.", "SOLTECH" },
	{ 0x00216E, "Function ATI (Huizhou) Telecommunications Co., Ltd.", "FUNCTION-ATI-HUIZHOU-TELECOMMUNICATIONS" },
	{ 0x00216F, "SymCom, Inc.", "SYMCOM" },
	{ 0x002170, "Dell Inc", "DELL" },
	{ 0x002171, "Wesung TNC Co., Ltd.", "WESUNG-TNC" },
	{ 0x002172, "Seoultek Valley", "SEOULTEK-VALLEY" },
	{ 0x002173, "Ion Torrent Systems, Inc.", "ION-TORRENT" },
	{ 0x002174, "AvaLAN Wireless", "AVALAN-WIRELESS" },
	{ 0x002175, "Pacific Satellite International Ltd.", "PACIFIC-SATELLITE" },
	{ 0x002176, "YMax Telecom Ltd.", "YMAX-TELECOM" },
	{ 0x002177, "W. L. Gore & Associates", "W-L-GORE-ASSOCIATES" },
	{ 0x002178, "Matuschek Messtechnik GmbH", "MATUSCHEK-MESSTECHNIK" },
	{ 0x002179, "IOGEAR, Inc.", "IOGEAR" },
	{ 0x00217A, "Sejin Electron, Inc.", "SEJIN-ELECTRON" },
	{ 0x00217B, "Bastec AB", "BASTEC-AB" },
	{ 0x00217C, "2Wire", "2WIRE" },
	{ 0x00217D, "PYXIS S.R.L.", "PYXIS-S-R-L" },
	{ 0x00217E, "Telit Communication s.p.a", "TELIT-COMMUNICATION-S-P-A" },
	{ 0x00217F, "Intraco Technology Pte Ltd", "INTRACO-PTE" },
	{ 0x002180, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002181, "Si2 Microsystems Limited", "SI2-MICROSYSTEMS" },
	{ 0x002182, "SandLinks Systems, Ltd.", "SANDLINKS" },
	{ 0x002183, "VATECH HYDRO", "VATECH-HYDRO" },
	{ 0x002184, "POWERSOFT SRL", "POWERSOFT-SRL" },
	{ 0x002185, "MICRO-STAR INT'L CO.,LTD.", "MICRO-STAR" },
	{ 0x002186, "Universal Global Scientific Industrial Co., Ltd", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x002187, "Imacs GmbH", "IMACS" },
	{ 0x002188, "EMC Corporation", "EMC" },
	{ 0x002189, "AppTech, Inc.", "APPTECH" },
	{ 0x00218A, "Electronic Design and Manufacturing Company", "DESIGN-AND-MANUFACTURING-COMPANY" },
	{ 0x00218B, "Wescon Technology, Inc.", "WESCON" },
	{ 0x00218C, "TopControl GMBH", "TOPCONTROL" },
	{ 0x00218D, "AP Router Ind. Eletronica LTDA", "AP-ROUTER-IND-ELETRONICA-LTDA" },
	{ 0x00218E, "MEKICS CO., LTD.", "MEKICS" },
	{ 0x00218F, "Avantgarde Acoustic Lautsprechersysteme GmbH", "AVANTGARDE-ACOUSTIC-LAUTSPRECHERSYSTEME" },
	{ 0x002190, "Goliath Solutions", "GOLIATH-SOLUTIONS" },
	{ 0x002191, "D-Link Corporation", "D-LINK" },
	{ 0x002192, "Baoding Galaxy Electronic Technology Co.,Ltd", "BAODING-GALAXY" },
	{ 0x002193, "Videofon MV", "VIDEOFON-MV" },
	{ 0x002194, "Ping Communication", "PING-COMMUNICATION" },
	{ 0x002195, "GWD Media Limited", "GWD-MEDIA" },
	{ 0x002196, "Telsey S.p.A.", "TELSEY-S-P-A" },
	{ 0x002197, "ELITEGROUP COMPUTER SYSTEM", "ELITEGROUP-SYSTEM" },
	{ 0x002198, "Thai Radio Co, LTD", "THAI-RADIO" },
	{ 0x002199, "Vacon Plc", "VACON-PLC" },
	{ 0x00219A, "Cambridge Visual Networks Ltd", "CAMBRIDGE-VISUAL-NETWORKS" },
	{ 0x00219B, "Dell Inc", "DELL" },
	{ 0x00219C, "Honeywld Technology Corp.", "HONEYWLD" },
	{ 0x00219D, "Adesys BV", "ADESYS" },
	{ 0x00219E, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x00219F, "SATEL OY", "SATEL-OY" },
	{ 0x0021A0, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0021A1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0021A2, "EKE-Electronics Ltd.", "EKE-ELECTRONICS" },
	{ 0x0021A3, "Micromint", "MICROMINT" },
	{ 0x0021A4, "Dbii Networks", "DBII-NETWORKS" },
	{ 0x0021A5, "ERLPhase Power Technologies Ltd.", "ERLPHASE-POWER-TECHNOLOGIES" },
	{ 0x0021A6, "Videotec Spa", "VIDEOTEC-SPA" },
	{ 0x0021A7, "Hantle System Co., Ltd.", "HANTLE-SYSTEM" },
	{ 0x0021A8, "Telephonics Corporation", "TELEPHONICS" },
	{ 0x0021A9, "Mobilink Telecom Co.,Ltd", "MOBILINK-TELECOM" },
	{ 0x0021AA, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0021AB, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0021AC, "Infrared Integrated Systems Ltd", "INFRARED-INTEGRATED" },
	{ 0x0021AD, "Nordic ID Oy", "NORDIC-ID-OY" },
	{ 0x0021AE, "ALCATEL-LUCENT FRANCE - WTD", "ALCATEL-LUCENT-FRANCE---WTD" },
	{ 0x0021AF, "Radio Frequency Systems", "RADIO-FREQUENCY" },
	{ 0x0021B0, "Tyco Telecommunications", "TYCO-TELECOMMUNICATIONS" },
	{ 0x0021B1, "DIGITAL SOLUTIONS LTD", "DIGITAL-SOLUTIONS" },
	{ 0x0021B2, "Fiberblaze A/S", "FIBERBLAZE-A/S" },
	{ 0x0021B3, "Ross Controls", "ROSS-CONTROLS" },
	{ 0x0021B4, "APRO MEDIA CO., LTD", "APRO-MEDIA" },
	{ 0x0021B5, "Galvanic Ltd", "GALVANIC" },
	{ 0x0021B6, "Triacta Power Technologies Inc.", "TRIACTA-POWER-TECHNOLOGIES" },
	{ 0x0021B7, "Lexmark International Inc.", "LEXMARK" },
	{ 0x0021B8, "Inphi Corporation", "INPHI" },
	{ 0x0021B9, "Universal Devices Inc.", "UNIVERSAL-DEVICES" },
	{ 0x0021BA, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0021BB, "Riken Keiki Co., Ltd.", "RIKEN-KEIKI" },
	{ 0x0021BC, "ZALA COMPUTER", "ZALA" },
	{ 0x0021BD, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x0021BE, "Cisco, Service Provider Video Technology Group", "CISCO-SERVICE-PROVIDER-VIDEO-GROUP" },
	{ 0x0021BF, "Hitachi High-Tech Control Systems Corporation", "HITACHI-HIGH-TECH-CONTROL" },
	{ 0x0021C0, "Mobile Appliance, Inc.", "MOBILE-APPLIANCE" },
	{ 0x0021C1, "ABB Oy / Medium Voltage Products", "ABB-OY-/-MEDIUM-VOLTAGE-PRODUCTS" },
	{ 0x0021C2, "GL Communications Inc", "GL-COMMUNICATION" },
	{ 0x0021C3, "CORNELL Communications, Inc.", "CORNELL-COMMUNICATION" },
	{ 0x0021C4, "Consilium AB", "CONSILIUM-AB" },
	{ 0x0021C5, "3DSP Corp", "3DSP" },
	{ 0x0021C6, "CSJ Global, Inc.", "CSJ-GLOBAL" },
	{ 0x0021C7, "Russound", "RUSSOUND" },
	{ 0x0021C8, "LOHUIS Networks", "LOHUIS-NETWORKS" },
	{ 0x0021C9, "Wavecom Asia Pacific Limited", "WAVECOM-ASIA-PACIFIC" },
	{ 0x0021CA, "ART System Co., Ltd.", "ART-SYSTEM" },
	{ 0x0021CB, "SMS TECNOLOGIA ELETRONICA LTDA", "SMS-TECNOLOGIA-ELETRONICA-LTDA" },
	{ 0x0021CC, "Flextronics International", "FLEXTRONICS" },
	{ 0x0021CD, "LiveTV", "LIVETV" },
	{ 0x0021CE, "NTC-Metrotek", "NTC-METROTEK" },
	{ 0x0021CF, "The Crypto Group", "THE-CRYPTO-GROUP" },
	{ 0x0021D0, "Global Display Solutions Spa", "GLOBAL-DISPLAY-SOLUTIONS-SPA" },
	{ 0x0021D1, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x0021D2, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x0021D3, "BOCOM SECURITY(ASIA PACIFIC) LIMITED", "BOCOM-SECURITY-ASIA-PACIFIC" },
	{ 0x0021D4, "Vollmer Werke GmbH", "VOLLMER-WERKE" },
	{ 0x0021D5, "X2E GmbH", "X2E" },
	{ 0x0021D6, "LXI Consortium", "LXI-CONSORTIUM" },
	{ 0x0021D7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0021D8, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0021D9, "SEKONIC CORPORATION", "SEKONIC" },
	{ 0x0021DA, "Automation Products Group Inc.", "AUTOMATION-PRODUCTS-GROUP" },
	{ 0x0021DB, "Santachi Video Technology (Shenzhen) Co., Ltd.", "SANTACHI-VIDEO-SHENZHEN" },
	{ 0x0021DC, "TECNOALARM S.r.l.", "TECNOALARM-S-R-L" },
	{ 0x0021DD, "Northstar Systems Corp", "NORTHSTAR" },
	{ 0x0021DE, "Firepro Wireless", "FIREPRO-WIRELESS" },
	{ 0x0021DF, "Martin Christ GmbH", "MARTIN-CHRIST" },
	{ 0x0021E0, "CommAgility Ltd", "COMMAGILITY" },
	{ 0x0021E1, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x0021E2, "Creative Electronic GmbH", "CREATIVE" },
	{ 0x0021E3, "SerialTek LLC", "SERIALTEK-LLC" },
	{ 0x0021E4, "I-WIN", "I-WIN" },
	{ 0x0021E5, "Display Solution AG", "DISPLAY-SOLUTION" },
	{ 0x0021E6, "Starlight Video Limited", "STARLIGHT-VIDEO" },
	{ 0x0021E7, "Informatics Services Corporation", "INFORMATICS-SERVICES" },
	{ 0x0021E8, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x0021E9, "Apple", "APPLE" },
	{ 0x0021EA, "Bystronic Laser AG", "BYSTRONIC-LASER" },
	{ 0x0021EB, "ESP SYSTEMS, LLC", "ESP-LLC" },
	{ 0x0021EC, "Solutronic GmbH", "SOLUTRONIC" },
	{ 0x0021ED, "Telegesis", "TELEGESIS" },
	{ 0x0021EE, "Full Spectrum Inc.", "FULL-SPECTRUM" },
	{ 0x0021EF, "Kapsys", "KAPSYS" },
	{ 0x0021F0, "EW3 Technologies LLC", "EW3-TECHNOLOGIES-LLC" },
	{ 0x0021F1, "Tutus Data AB", "TUTUS-DATA-AB" },
	{ 0x0021F2, "EASY3CALL Technology Limited", "EASY3CALL" },
	{ 0x0021F3, "Si14 SpA", "SI14-SPA" },
	{ 0x0021F4, "INRange Systems, Inc", "INRANGE" },
	{ 0x0021F5, "Western Engravers Supply, Inc.", "WESTERN-ENGRAVERS-SUPPLY" },
	{ 0x0021F6, "Oracle Corporation", "ORACLE" },
	{ 0x0021F7, "HPN Supply Chain", "HPN-SUPPLY-CHAIN" },
	{ 0x0021F8, "Enseo, Inc.", "ENSEO" },
	{ 0x0021F9, "WIRECOM Technologies", "WIRECOM-TECHNOLOGIES" },
	{ 0x0021FA, "A4SP Technologies Ltd.", "A4SP-TECHNOLOGIES" },
	{ 0x0021FB, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x0021FC, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0021FD, "DSTA S.L.", "DSTA-S-L" },
	{ 0x0021FE, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0021FF, "Cyfrowy Polsat SA", "CYFROWY-POLSAT" },
	{ 0x002200, "IBM Corp", "IBM" },
	{ 0x002201, "Aksys Networks Inc", "AKSYS-NETWORKS" },
	{ 0x002202, "Excito Elektronik i Skåne AB", "EXCITO-ELEKTRONIK-I-SKåNE-AB" },
	{ 0x002203, "Glensound Electronics Ltd", "GLENSOUND-ELECTRONICS" },
	{ 0x002204, "KORATEK", "KORATEK" },
	{ 0x002205, "WeLink Solutions, Inc.", "WELINK-SOLUTIONS" },
	{ 0x002206, "Cyberdyne Inc.", "CYBERDYNE" },
	{ 0x002207, "Inteno Broadband Technology AB", "INTENO-BROADBAND-AB" },
	{ 0x002208, "Certicom Corp", "CERTICOM" },
	{ 0x002209, "Omron Healthcare Co., Ltd", "OMRON-HEALTHCARE" },
	{ 0x00220A, "OnLive, Inc", "ONLIVE" },
	{ 0x00220B, "National Source Coding Center", "NATIONAL-SOURCE-CODING-CENTER" },
	{ 0x00220C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00220D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00220E, "Indigo Security Co., Ltd.", "INDIGO-SECURITY" },
	{ 0x00220F, "MoCA (Multimedia over Coax Alliance)", "MOCA-MULTIMEDIA-OVER-COAX-ALLIANCE" },
	{ 0x002210, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002211, "Rohati Systems", "ROHATI" },
	{ 0x002212, "CAI Networks, Inc.", "CAI-NETWORKS" },
	{ 0x002213, "PCI CORPORATION", "PCI" },
	{ 0x002214, "RINNAI KOREA", "RINNAI-KOREA" },
	{ 0x002215, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x002216, "SHIBAURA VENDING MACHINE CORPORATION", "SHIBAURA-VENDING-MACHINE" },
	{ 0x002217, "Neat Electronics", "NEAT-ELECTRONICS" },
	{ 0x002218, "Verivue Inc.", "VERIVUE" },
	{ 0x002219, "Dell Inc", "DELL" },
	{ 0x00221A, "Audio Precision", "AUDIO-PRECISION" },
	{ 0x00221B, "Morega Systems", "MOREGA" },
	{ 0x00221C, "PRIVATE", "PRIVATE" },
	{ 0x00221D, "Freegene Technology LTD", "FREEGENE" },
	{ 0x00221E, "Media Devices Co., Ltd.", "MEDIA-DEVICES" },
	{ 0x00221F, "eSang Technologies Co., Ltd.", "ESANG-TECHNOLOGIES" },
	{ 0x002220, "Mitac Technology Corp", "MITAC" },
	{ 0x002221, "ITOH DENKI CO,LTD.", "ITOH-DENKI" },
	{ 0x002222, "Schaffner Deutschland GmbH", "SCHAFFNER-DEUTSCHLAND" },
	{ 0x002223, "TimeKeeping Systems, Inc.", "TIMEKEEPING" },
	{ 0x002224, "Good Will Instrument Co., Ltd.", "GOOD-WILL-INSTRUMENT" },
	{ 0x002225, "Thales Avionics Ltd", "THALES-AVIONICS" },
	{ 0x002226, "Avaak, Inc.", "AVAAK" },
	{ 0x002227, "uv-electronic GmbH", "UV" },
	{ 0x002228, "Breeze Innovations Ltd.", "BREEZE-INNOVATIONS" },
	{ 0x002229, "Compumedics Ltd", "COMPUMEDICS" },
	{ 0x00222A, "SoundEar A/S", "SOUNDEAR-A/S" },
	{ 0x00222B, "Nucomm, Inc.", "NUCOMM" },
	{ 0x00222C, "Ceton Corp", "CETON" },
	{ 0x00222D, "SMC Networks Inc.", "SMC-NETWORKS" },
	{ 0x00222E, "maintech GmbH", "MAINTECH" },
	{ 0x00222F, "Open Grid Computing, Inc.", "OPEN-GRID-COMPUTING" },
	{ 0x002230, "FutureLogic Inc.", "FUTURELOGIC" },
	{ 0x002231, "SMT&C Co., Ltd.", "SMT-C" },
	{ 0x002232, "Design Design Technology Ltd", "DESIGN-DESIGN" },
	{ 0x002233, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x002234, "Corventis Inc.", "CORVENTIS" },
	{ 0x002235, "Strukton Systems bv", "STRUKTON" },
	{ 0x002236, "VECTOR SP. Z O.O.", "VECTOR-SP-Z-O-O" },
	{ 0x002237, "Shinhint Group", "SHINHINT-GROUP" },
	{ 0x002238, "LOGIPLUS", "LOGIPLUS" },
	{ 0x002239, "Indiana Life Sciences Incorporated", "INDIANA-LIFE-SCIENCES-INCORPORATED" },
	{ 0x00223A, "Scientific Atlanta, Cisco SPVT Group", "SCIENTIFIC-ATLANTA-CISCO-SPVT-GROUP" },
	{ 0x00223B, "Communication Networks, LLC", "COMMUNICATION-NETWORKS-LLC" },
	{ 0x00223C, "RATIO Entwicklungen GmbH", "RATIO-ENTWICKLUNGEN" },
	{ 0x00223D, "JumpGen Systems, LLC", "JUMPGEN-LLC" },
	{ 0x00223E, "IRTrans GmbH", "IRTRANS" },
	{ 0x00223F, "Netgear Inc.", "NETGEAR" },
	{ 0x002240, "Universal Telecom S/A", "UNIVERSAL-TELECOM-S/A" },
	{ 0x002241, "Apple", "APPLE" },
	{ 0x002242, "Alacron Inc.", "ALACRON" },
	{ 0x002243, "AzureWave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x002244, "Chengdu Linkon Communications Device Co., Ltd", "CHENGDU-LINKON-COMMUNICATION-DEVICE" },
	{ 0x002245, "Leine & Linde AB", "LEINE-LINDE-AB" },
	{ 0x002246, "Evoc Intelligent Technology Co.,Ltd.", "EVOC-INTELLIGENT" },
	{ 0x002247, "DAC ENGINEERING CO., LTD.", "DAC-ENGINEERING" },
	{ 0x002248, "Microsoft Corporation", "MICROSOFT" },
	{ 0x002249, "HOME MULTIENERGY SL", "HOME-MULTIENERGY-SL" },
	{ 0x00224A, "RAYLASE AG", "RAYLASE" },
	{ 0x00224B, "AIRTECH TECHNOLOGIES, INC.", "AIRTECH-TECHNOLOGIES" },
	{ 0x00224C, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x00224D, "MITAC INTERNATIONAL CORP.", "MITAC" },
	{ 0x00224E, "SEEnergy Corp.", "SEENERGY" },
	{ 0x00224F, "Byzoro Networks Ltd.", "BYZORO-NETWORKS" },
	{ 0x002250, "Point Six Wireless, LLC", "POINT-SIX-WIRELESS-LLC" },
	{ 0x002251, "Lumasense Technologies", "LUMASENSE-TECHNOLOGIES" },
	{ 0x002252, "ZOLL Lifecor Corporation", "ZOLL-LIFECOR" },
	{ 0x002253, "Entorian Technologies", "ENTORIAN-TECHNOLOGIES" },
	{ 0x002254, "Bigelow Aerospace", "BIGELOW-AEROSPACE" },
	{ 0x002255, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002256, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002257, "3Com Europe Ltd", "3COM" },
	{ 0x002258, "Taiyo Yuden Co., Ltd.", "TAIYO-YUDEN" },
	{ 0x002259, "Guangzhou New Postcom Equipment Co.,Ltd.", "GUANGZHOU-NEW-POSTCOM-EQUIPMENT" },
	{ 0x00225A, "Garde Security AB", "GARDE-SECURITY-AB" },
	{ 0x00225B, "Teradici Corporation", "TERADICI" },
	{ 0x00225C, "Multimedia & Communication Technology", "MULTIMEDIA-COMMUNICATION" },
	{ 0x00225D, "Digicable Network India Pvt. Ltd.", "DIGICABLE-NETWORK-INDIA-PVT" },
	{ 0x00225E, "Uwin Technologies Co.,LTD", "UWIN-TECHNOLOGIES" },
	{ 0x00225F, "Liteon Technology Corporation", "LITEON" },
	{ 0x002260, "AFREEY Inc.", "AFREEY" },
	{ 0x002261, "Frontier Silicon Ltd", "FRONTIER-SILICON" },
	{ 0x002262, "BEP Marine", "BEP-MARINE" },
	{ 0x002263, "Koos Technical Services, Inc.", "KOOS-TECHNICAL-SERVICES" },
	{ 0x002264, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x002265, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x002266, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x002267, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x002268, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x002269, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x00226A, "Honeywell", "HONEYWELL" },
	{ 0x00226B, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x00226C, "LinkSprite Technologies, Inc.", "LINKSPRITE-TECHNOLOGIES" },
	{ 0x00226D, "Shenzhen GIEC Electronics Co., Ltd.", "SHENZHEN-GIEC-ELECTRONICS" },
	{ 0x00226E, "Gowell Electronic Limited", "GOWELL" },
	{ 0x00226F, "3onedata Technology Co. Ltd.", "3ONEDATA" },
	{ 0x002270, "ABK North America, LLC", "ABK-NORTH-AMERICA-LLC" },
	{ 0x002271, "Jäger Computergesteuerte Meßtechnik GmbH.", "JäGER-COMPUTERGESTEUERTE-MEßTECHNIK" },
	{ 0x002272, "American Micro-Fuel Device Corp.", "AMERICAN-MICRO-FUEL-DEVICE" },
	{ 0x002273, "Techway", "TECHWAY" },
	{ 0x002274, "FamilyPhone AB", "FAMILYPHONE-AB" },
	{ 0x002275, "Belkin International Inc.", "BELKIN" },
	{ 0x002276, "Triple EYE B.V.", "TRIPLE-EYE" },
	{ 0x002277, "NEC Australia Pty Ltd", "NEC-AUSTRALIA-PTY" },
	{ 0x002278, "Shenzhen Tongfang Multimedia Technology Co.,Ltd.", "SHENZHEN-TONGFANG-MULTIMEDIA" },
	{ 0x002279, "Nippon Conlux Co., Ltd.", "NIPPON-CONLUX" },
	{ 0x00227A, "Telecom Design", "TELECOM-DESIGN" },
	{ 0x00227B, "Apogee Labs, Inc.", "APOGEE-LABS" },
	{ 0x00227C, "Woori SMT Co.,ltd", "WOORI-SMT" },
	{ 0x00227D, "YE DATA INC.", "YE-DATA" },
	{ 0x00227E, "Chengdu 30Kaitian Communication Industry Co.Ltd", "CHENGDU-30KAITIAN-COMMUNICATION-INDUSTRY" },
	{ 0x00227F, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x002280, "A2B Electronics AB", "A2B-ELECTRONICS-AB" },
	{ 0x002281, "Daintree Networks Pty", "DAINTREE-NETWORKS-PTY" },
	{ 0x002282, "8086 Consultancy", "8086-CONSULTANCY" },
	{ 0x002283, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x002284, "DESAY A&V SCIENCE AND TECHNOLOGY CO.,LTD", "DESAY-A-V-SCIENCE-AND" },
	{ 0x002285, "NOMUS COMM SYSTEMS", "NOMUS-COMM" },
	{ 0x002286, "ASTRON", "ASTRON" },
	{ 0x002287, "Titan Wireless LLC", "TITAN-WIRELESS-LLC" },
	{ 0x002288, "Sagrad, Inc.", "SAGRAD" },
	{ 0x002289, "Optosecurity Inc.", "OPTOSECURITY" },
	{ 0x00228A, "Teratronik elektronische systeme gmbh", "TERATRONIK-ELEKTRONISCHE-SYSTEME" },
	{ 0x00228B, "Kensington Computer Products Group", "KENSINGTON-PRODUCTS-GROUP" },
	{ 0x00228C, "Photon Europe GmbH", "PHOTON-EUROPE" },
	{ 0x00228D, "GBS Laboratories LLC", "GBS-LABORATORIES-LLC" },
	{ 0x00228E, "TV-NUMERIC", "TV-NUMERIC" },
	{ 0x00228F, "CNRS", "CNRS" },
	{ 0x002290, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002291, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002292, "Cinetal", "CINETAL" },
	{ 0x002293, "ZTE Corporation", "ZTE" },
	{ 0x002294, "Kyocera Corporation", "KYOCERA" },
	{ 0x002295, "SGM Technology for lighting spa", "SGM-FOR-LIGHTING-SPA" },
	{ 0x002296, "LinoWave Corporation", "LINOWAVE" },
	{ 0x002297, "XMOS Semiconductor", "XMOS-SEMICONDUCTOR" },
	{ 0x002298, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x002299, "SeaMicro Inc.", "SEAMICRO" },
	{ 0x00229A, "Lastar, Inc.", "LASTAR" },
	{ 0x00229B, "AverLogic Technologies, Inc.", "AVERLOGIC-TECHNOLOGIES" },
	{ 0x00229C, "Verismo Networks Inc", "VERISMO-NETWORKS" },
	{ 0x00229D, "PYUNG-HWA IND.CO.,LTD", "PYUNG-HWA-IND" },
	{ 0x00229E, "Social Aid Research Co., Ltd.", "SOCIAL-AID-RESEARCH" },
	{ 0x00229F, "Sensys Traffic AB", "SENSYS-TRAFFIC-AB" },
	{ 0x0022A0, "Delphi Corporation", "DELPHI" },
	{ 0x0022A1, "Huawei Symantec Technologies Co.,Ltd.", "HUAWEI-SYMANTEC-TECHNOLOGIES" },
	{ 0x0022A2, "Xtramus Technologies", "XTRAMUS-TECHNOLOGIES" },
	{ 0x0022A3, "California Eastern Laboratories", "CALIFORNIA-EASTERN-LABORATORIES" },
	{ 0x0022A4, "2Wire", "2WIRE" },
	{ 0x0022A5, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0022A6, "Sony Computer Entertainment America", "SONY-ENTERTAINMENT-AMERICA" },
	{ 0x0022A7, "Tyco Electronics AMP GmbH", "TYCO-ELECTRONICS-AMP" },
	{ 0x0022A8, "Ouman Oy", "OUMAN-OY" },
	{ 0x0022A9, "LG Electronics Inc", "LG-ELECTRONICS" },
	{ 0x0022AA, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x0022AB, "Shenzhen Turbosight Technology Ltd", "SHENZHEN-TURBOSIGHT" },
	{ 0x0022AC, "Hangzhou Siyuan Tech. Co., Ltd", "HANGZHOU-SIYUAN-TECH" },
	{ 0x0022AD, "TELESIS TECHNOLOGIES, INC.", "TELESIS-TECHNOLOGIES" },
	{ 0x0022AE, "Mattel Inc.", "MATTEL" },
	{ 0x0022AF, "Safety Vision", "SAFETY-VISION" },
	{ 0x0022B0, "D-Link Corporation", "D-LINK" },
	{ 0x0022B1, "Elbit Systems", "ELBIT" },
	{ 0x0022B2, "4RF Communications Ltd", "4RF-COMMUNICATION" },
	{ 0x0022B3, "Sei S.p.A.", "SEI-S-P-A" },
	{ 0x0022B4, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0022B5, "NOVITA", "NOVITA" },
	{ 0x0022B6, "Superflow Technologies Group", "SUPERFLOW-TECHNOLOGIES-GROUP" },
	{ 0x0022B7, "GSS Grundig SAT-Systems GmbH", "GSS-GRUNDIG-SAT" },
	{ 0x0022B8, "Norcott", "NORCOTT" },
	{ 0x0022B9, "Analogix Seminconductor, Inc", "ANALOGIX-SEMINCONDUCTOR" },
	{ 0x0022BA, "HUTH Elektronik Systeme GmbH", "HUTH-ELEKTRONIK-SYSTEME" },
	{ 0x0022BB, "beyerdynamic GmbH & Co. KG", "BEYERDYNAMIC" },
	{ 0x0022BC, "JDSU France SAS", "JDSU-FRANCE-SAS" },
	{ 0x0022BD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0022BE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0022BF, "SieAmp Group of Companies", "SIEAMP-GROUP-OF-COMPANIES" },
	{ 0x0022C0, "Shenzhen Forcelink Electronic Co, Ltd", "SHENZHEN-FORCELINK" },
	{ 0x0022C1, "Active Storage Inc.", "ACTIVE-STORAGE" },
	{ 0x0022C2, "Proview Eletrônica do Brasil LTDA", "PROVIEW-ELETRôNICA-DO-BRASIL-LTDA" },
	{ 0x0022C3, "Zeeport Technology Inc.", "ZEEPORT" },
	{ 0x0022C4, "epro GmbH", "EPRO" },
	{ 0x0022C5, "INFORSON Co,Ltd.", "INFORSON" },
	{ 0x0022C6, "Sutus Inc", "SUTUS" },
	{ 0x0022C7, "SEGGER Microcontroller GmbH & Co. KG", "SEGGER-MICROCONTROLLER" },
	{ 0x0022C8, "Applied Instruments B.V.", "APPLIED-INSTRUMENTS" },
	{ 0x0022C9, "Lenord, Bauer & Co GmbH", "LENORD-BAUER" },
	{ 0x0022CA, "Anviz Biometric Tech. Co., Ltd.", "ANVIZ-BIOMETRIC-TECH" },
	{ 0x0022CB, "IONODES Inc.", "IONODES" },
	{ 0x0022CC, "SciLog, Inc.", "SCILOG" },
	{ 0x0022CD, "Ared Technology Co., Ltd.", "ARED" },
	{ 0x0022CE, "Cisco, Service Provider Video Technology Group", "CISCO-SERVICE-PROVIDER-VIDEO-GROUP" },
	{ 0x0022CF, "PLANEX Communications INC", "PLANEX-COMMUNICATION" },
	{ 0x0022D0, "Polar Electro Oy", "POLAR-ELECTRO-OY" },
	{ 0x0022D1, "Albrecht Jung GmbH & Co. KG", "ALBRECHT-JUNG" },
	{ 0x0022D2, "All Earth Comércio de Eletrônicos LTDA.", "ALL-EARTH-COMéRCIO-DE-ELETRôNICOS-LTDA" },
	{ 0x0022D3, "Hub-Tech", "HUB-TECH" },
	{ 0x0022D4, "ComWorth Co., Ltd.", "COMWORTH" },
	{ 0x0022D5, "Eaton Corp. Electrical Group Data Center Solutions - Pulizzi", "EATON-ELECTRICAL-GROUP-DATA-CENTER-SOLUTIONS---PULIZZI" },
	{ 0x0022D6, "Cypak AB", "CYPAK-AB" },
	{ 0x0022D7, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x0022D8, "Shenzhen GST Security and Safety Technology Limited", "SHENZHEN-GST-SECURITY-AND-SAFETY" },
	{ 0x0022D9, "Fortex Industrial Ltd.", "FORTEX-INDUSTRIAL" },
	{ 0x0022DA, "ANATEK, LLC", "ANATEK-LLC" },
	{ 0x0022DB, "Translogic Corporation", "TRANSLOGIC" },
	{ 0x0022DC, "Vigil Health Solutions Inc.", "VIGIL-HEALTH-SOLUTIONS" },
	{ 0x0022DD, "Protecta Electronics Ltd", "PROTECTA-ELECTRONICS" },
	{ 0x0022DE, "OPPO Digital, Inc.", "OPPO-DIGITAL" },
	{ 0x0022DF, "TAMUZ Monitors", "TAMUZ-MONITORS" },
	{ 0x0022E0, "Atlantic Software Technologies S.r.L.", "ATLANTIC-SOFTWARE-TECHNOLOGIES-S-R-L" },
	{ 0x0022E1, "ZORT Labs, LLC.", "ZORT-LABS-LLC" },
	{ 0x0022E2, "WABTEC Transit Division", "WABTEC-TRANSIT-DIVISION" },
	{ 0x0022E3, "Amerigon", "AMERIGON" },
	{ 0x0022E4, "APASS TECHNOLOGY CO., LTD.", "APASS" },
	{ 0x0022E5, "Fisher-Rosemount Systems Inc.", "FISHER-ROSEMOUNT" },
	{ 0x0022E6, "Intelligent Data", "INTELLIGENT-DATA" },
	{ 0x0022E7, "WPS Parking Systems", "WPS-PARKING" },
	{ 0x0022E8, "Applition Co., Ltd.", "APPLITION" },
	{ 0x0022E9, "ProVision Communications", "PROVISION-COMMUNICATION" },
	{ 0x0022EA, "Rustelcom Inc.", "RUSTELCOM" },
	{ 0x0022EB, "Data Respons A/S", "DATA-RESPONS-A/S" },
	{ 0x0022EC, "IDEALBT TECHNOLOGY CORPORATION", "IDEALBT" },
	{ 0x0022ED, "TSI Power Corporation", "TSI-POWER" },
	{ 0x0022EE, "Algo Communication Products Ltd", "ALGO-COMMUNICATION-PRODUCTS" },
	{ 0x0022EF, "iWDL Technologies", "IWDL-TECHNOLOGIES" },
	{ 0x0022F0, "3 Greens Aviation Limited", "3-GREENS-AVIATION" },
	{ 0x0022F1, "PRIVATE", "PRIVATE" },
	{ 0x0022F2, "SunPower Corp", "SUNPOWER" },
	{ 0x0022F3, "SHARP Corporation", "SHARP" },
	{ 0x0022F4, "AMPAK Technology, Inc.", "AMPAK" },
	{ 0x0022F5, "Advanced Realtime Tracking GmbH", "ADVANCED-REALTIME-TRACKING" },
	{ 0x0022F6, "Syracuse Research Corporation", "SYRACUSE-RESEARCH" },
	{ 0x0022F7, "Conceptronic", "CONCEPTRONIC" },
	{ 0x0022F8, "PIMA Electronic Systems Ltd.", "PIMA" },
	{ 0x0022F9, "Pollin Electronic GmbH", "POLLIN" },
	{ 0x0022FA, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0022FB, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0022FC, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0022FD, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0022FE, "Microprocessor Designs Inc", "MICROPROCESSOR-DESIGNS" },
	{ 0x0022FF, "iWDL Technologies", "IWDL-TECHNOLOGIES" },
	{ 0x002300, "Cayee Computer Ltd.", "CAYEE" },
	{ 0x002301, "Witron Technology Limited", "WITRON" },
	{ 0x002302, "Cobalt Digital, Inc.", "COBALT-DIGITAL" },
	{ 0x002303, "LITE-ON IT Corporation", "LITE-ON-IT" },
	{ 0x002304, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002305, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002306, "ALPS Electric Co., Ltd", "ALPS-ELECTRIC" },
	{ 0x002307, "FUTURE INNOVATION TECH CO.,LTD", "FUTURE-INNOVATION-TECH" },
	{ 0x002308, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x002309, "Janam Technologies LLC", "JANAM-TECHNOLOGIES-LLC" },
	{ 0x00230A, "ARBURG GmbH & Co KG", "ARBURG" },
	{ 0x00230B, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00230C, "CLOVER ELECTRONICS CO.,LTD.", "CLOVER-ELECTRONICS" },
	{ 0x00230D, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x00230E, "Gorba AG", "GORBA" },
	{ 0x00230F, "Hirsch Electronics Corporation", "HIRSCH-ELECTRONICS" },
	{ 0x002310, "LNC Technology Co., Ltd.", "LNC" },
	{ 0x002311, "Gloscom Co., Ltd.", "GLOSCOM" },
	{ 0x002312, "Apple", "APPLE" },
	{ 0x002313, "Qool Technologies Ltd.", "QOOL-TECHNOLOGIES" },
	{ 0x002314, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x002315, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x002316, "KISAN ELECTRONICS CO", "KISAN-ELECTRONICS" },
	{ 0x002317, "Lasercraft Inc", "LASERCRAFT" },
	{ 0x002318, "Toshiba", "TOSHIBA" },
	{ 0x002319, "Sielox LLC", "SIELOX-LLC" },
	{ 0x00231A, "ITF Co., Ltd.", "ITF" },
	{ 0x00231B, "Danaher Motion - Kollmorgen", "DANAHER-MOTION---KOLLMORGEN" },
	{ 0x00231C, "Fourier Systems Ltd.", "FOURIER" },
	{ 0x00231D, "Deltacom Electronics Ltd", "DELTACOM-ELECTRONICS" },
	{ 0x00231E, "Cezzer Multimedia Technologies", "CEZZER-MULTIMEDIA-TECHNOLOGIES" },
	{ 0x00231F, "Guangda Electronic & Telecommunication Technology Development Co., Ltd.", "GUANGDA-TELECOMMUNICATION-DEVELOPMENT" },
	{ 0x002320, "Nicira Networks", "NICIRA-NETWORKS" },
	{ 0x002321, "Avitech International Corp", "AVITECH" },
	{ 0x002322, "KISS Teknical Solutions, Inc.", "KISS-TEKNICAL-SOLUTIONS" },
	{ 0x002323, "Zylin AS", "ZYLIN-AS" },
	{ 0x002324, "G-PRO COMPUTER", "G-PRO" },
	{ 0x002325, "IOLAN Holding", "IOLAN-HOLDING" },
	{ 0x002326, "Fujitsu Limited", "FUJITSU" },
	{ 0x002327, "Shouyo Electronics CO., LTD", "SHOUYO-ELECTRONICS" },
	{ 0x002328, "ALCON TELECOMMUNICATIONS CO., LTD.", "ALCON-TELECOMMUNICATIONS" },
	{ 0x002329, "DDRdrive LLC", "DDRDRIVE-LLC" },
	{ 0x00232A, "eonas IT-Beratung und -Entwicklung GmbH", "EONAS-IT-BERATUNG-UND--ENTWICKLUNG" },
	{ 0x00232B, "IRD A/S", "IRD-A/S" },
	{ 0x00232C, "Senticare", "SENTICARE" },
	{ 0x00232D, "SandForce", "SANDFORCE" },
	{ 0x00232E, "Kedah Electronics Engineering, LLC", "KEDAH-ELECTRONICS-ENGINEERING-LLC" },
	{ 0x00232F, "Advanced Card Systems Ltd.", "ADVANCED-CARD" },
	{ 0x002330, "DIZIPIA, INC.", "DIZIPIA" },
	{ 0x002331, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x002332, "Apple", "APPLE" },
	{ 0x002333, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002334, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002335, "Linkflex Co.,Ltd", "LINKFLEX" },
	{ 0x002336, "METEL s.r.o.", "METEL-S-R-O" },
	{ 0x002337, "Global Star Solutions ULC", "GLOBAL-STAR-SOLUTIONS-ULC" },
	{ 0x002338, "OJ-Electronics A/S", "OJ-ELECTRONICS-A/S" },
	{ 0x002339, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x00233A, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x00233B, "C-Matic Systems Ltd", "C-MATIC" },
	{ 0x00233C, "Alflex", "ALFLEX" },
	{ 0x00233D, "Novero holding B.V.", "NOVERO-HOLDING" },
	{ 0x00233E, "Alcatel-Lucent-IPD", "ALCATEL-LUCENT-IPD" },
	{ 0x00233F, "Purechoice Inc", "PURECHOICE" },
	{ 0x002340, "MiX Telematics", "MIX-TELEMATICS" },
	{ 0x002341, "Siemens AB, Infrastructure & Cities, Building Technologies Division, IC BT SSP SP BA PR", "SIEMENS-AB-INFRASTRUCTURE-CITIES-BUILDING-TECHNOLOGIES-DIVISION-IC-BT-SSP-SP-BA-PR" },
	{ 0x002342, "Coffee Equipment Company", "COFFEE-EQUIPMENT-COMPANY" },
	{ 0x002343, "TEM AG", "TEM" },
	{ 0x002344, "Objective Interface Systems, Inc.", "OBJECTIVE-INTERFACE" },
	{ 0x002345, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x002346, "Vestac", "VESTAC" },
	{ 0x002347, "ProCurve Networking by HP", "PROCURVE-NETWORKING-BY-HP" },
	{ 0x002348, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x002349, "Helmholtz Centre Berlin for Material and Energy", "HELMHOLTZ-CENTRE-BERLIN-FOR-MATERIAL-AND-ENERGY" },
	{ 0x00234A, "PRIVATE", "PRIVATE" },
	{ 0x00234B, "Inyuan Technology Inc.", "INYUAN" },
	{ 0x00234C, "KTC AB", "KTC-AB" },
	{ 0x00234D, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x00234E, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x00234F, "Luminous Power Technologies Pvt. Ltd.", "LUMINOUS-POWER-TECHNOLOGIES-PVT" },
	{ 0x002350, "LynTec", "LYNTEC" },
	{ 0x002351, "2Wire", "2WIRE" },
	{ 0x002352, "DATASENSOR S.p.A.", "DATASENSOR-S-P-A" },
	{ 0x002353, "F E T Elettronica snc", "F-E-T-ELETTRONICA-SNC" },
	{ 0x002354, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x002355, "Kinco Automation(Shanghai) Ltd.", "KINCO-AUTOMATION-SHANGHAI" },
	{ 0x002356, "Packet Forensics LLC", "PACKET-FORENSICS-LLC" },
	{ 0x002357, "Pitronot Technologies and Engineering P.T.E. Ltd.", "PITRONOT-TECHNOLOGIES-AND-ENGINEERING-P-T-E" },
	{ 0x002358, "SYSTEL SA", "SYSTEL" },
	{ 0x002359, "Benchmark Electronics ( Thailand ) Public Company Limited", "BENCHMARK-ELECTRONICS-THAILAND-PUBLIC-COMPANY" },
	{ 0x00235A, "COMPAL INFORMATION (KUNSHAN) CO., Ltd.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x00235B, "Gulfstream", "GULFSTREAM" },
	{ 0x00235C, "Aprius, Inc.", "APRIUS" },
	{ 0x00235D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00235E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00235F, "Silicon Micro Sensors GmbH", "SILICON-MICRO-SENSORS" },
	{ 0x002360, "Lookit Technology Co., Ltd", "LOOKIT" },
	{ 0x002361, "Unigen Corporation", "UNIGEN" },
	{ 0x002362, "Goldline Controls", "GOLDLINE-CONTROLS" },
	{ 0x002363, "Zhuhai RaySharp Technology Co., Ltd.", "ZHUHAI-RAYSHARP" },
	{ 0x002364, "Power Instruments Pte Ltd", "POWER-INSTRUMENTS-PTE" },
	{ 0x002365, "ELKA-Elektronik GmbH", "ELKA-ELEKTRONIK" },
	{ 0x002366, "Beijing Siasun Electronic System Co.,Ltd.", "BEIJING-SIASUN-SYSTEM" },
	{ 0x002367, "UniControls a.s.", "UNICONTROLS-A-S" },
	{ 0x002368, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0x002369, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x00236A, "SmartRG Inc", "SMARTRG" },
	{ 0x00236B, "Xembedded, Inc.", "XEMBEDDED" },
	{ 0x00236C, "Apple", "APPLE" },
	{ 0x00236D, "ResMed Ltd", "RESMED" },
	{ 0x00236E, "Burster GmbH & Co KG", "BURSTER" },
	{ 0x00236F, "DAQ System", "DAQ-SYSTEM" },
	{ 0x002370, "Snell", "SNELL" },
	{ 0x002371, "SOAM Systel", "SOAM-SYSTEL" },
	{ 0x002372, "MORE STAR INDUSTRIAL GROUP LIMITED", "MORE-STAR-INDUSTRIAL-GROUP" },
	{ 0x002373, "GridIron Systems, Inc.", "GRIDIRON" },
	{ 0x002374, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002375, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002376, "HTC Corporation", "HTC" },
	{ 0x002377, "Isotek Electronics Ltd", "ISOTEK-ELECTRONICS" },
	{ 0x002378, "GN Netcom A/S", "GN-NETCOM-A/S" },
	{ 0x002379, "Union Business Machines Co. Ltd.", "UNION-BUSINESS-MACHINES" },
	{ 0x00237A, "RIM", "RIM" },
	{ 0x00237B, "WHDI LLC", "WHDI-LLC" },
	{ 0x00237C, "NEOTION", "NEOTION" },
	{ 0x00237D, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x00237E, "ELSTER GMBH", "ELSTER" },
	{ 0x00237F, "PLANTRONICS, INC.", "PLANTRONICS" },
	{ 0x002380, "Nanoteq", "NANOTEQ" },
	{ 0x002381, "Lengda Technology(Xiamen) Co.,Ltd.", "LENGDA-XIAMEN" },
	{ 0x002382, "Lih Rong Electronic Enterprise Co., Ltd.", "LIH-RONG" },
	{ 0x002383, "InMage Systems Inc", "INMAGE" },
	{ 0x002384, "GGH Engineering s.r.l.", "GGH-ENGINEERING-S-R-L" },
	{ 0x002385, "ANTIPODE", "ANTIPODE" },
	{ 0x002386, "Tour & Andersson AB", "TOUR-ANDERSSON-AB" },
	{ 0x002387, "ThinkFlood, Inc.", "THINKFLOOD" },
	{ 0x002388, "V.T. Telematica S.p.a.", "V-T-TELEMATICA-S-P-A" },
	{ 0x002389, "HANGZHOU H3C Technologies Co., Ltd.", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x00238A, "Ciena Corporation", "CIENA" },
	{ 0x00238B, "Quanta Computer Inc.", "QUANTA" },
	{ 0x00238C, "PRIVATE", "PRIVATE" },
	{ 0x00238D, "Techno Design Co., Ltd.", "TECHNO-DESIGN" },
	{ 0x00238E, "Pirelli Tyre S.p.A.", "PIRELLI-TYRE-S-P-A" },
	{ 0x00238F, "NIDEC COPAL CORPORATION", "NIDEC-COPAL" },
	{ 0x002390, "Algolware Corporation", "ALGOLWARE" },
	{ 0x002391, "Maxian", "MAXIAN" },
	{ 0x002392, "Proteus Industries Inc.", "PROTEUS-INDUSTRIES" },
	{ 0x002393, "AJINEXTEK", "AJINEXTEK" },
	{ 0x002394, "Samjeon", "SAMJEON" },
	{ 0x002395, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002396, "ANDES TECHNOLOGY CORPORATION", "ANDES" },
	{ 0x002397, "Westell Technologies Inc.", "WESTELL-TECHNOLOGIES" },
	{ 0x002398, "Sky Control", "SKY-CONTROL" },
	{ 0x002399, "VD Division, Samsung Electronics Co.", "VD-DIVISION-SAMSUNG-ELECTRONICS" },
	{ 0x00239A, "EasyData Hardware GmbH", "EASYDATA-HARDWARE" },
	{ 0x00239B, "Elster Solutions, LLC", "ELSTER-SOLUTIONS-LLC" },
	{ 0x00239C, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x00239D, "Mapower Electronics Co., Ltd", "MAPOWER-ELECTRONICS" },
	{ 0x00239E, "Jiangsu Lemote Technology Corporation Limited", "JIANGSU-LEMOTE" },
	{ 0x00239F, "Institut für Prüftechnik", "INSTITUT-FüR-PRüFTECHNIK" },
	{ 0x0023A0, "Hana CNS Co., LTD.", "HANA-CNS" },
	{ 0x0023A1, "Trend Electronics Ltd", "TREND-ELECTRONICS" },
	{ 0x0023A2, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0023A3, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0023A4, "New Concepts Development Corp.", "NEW-CONCEPTS-DEVELOPMENT" },
	{ 0x0023A5, "SageTV, LLC", "SAGETV-LLC" },
	{ 0x0023A6, "E-Mon", "E-MON" },
	{ 0x0023A7, "Redpine Signals, Inc.", "REDPINE-SIGNALS" },
	{ 0x0023A8, "Marshall Electronics", "MARSHALL-ELECTRONICS" },
	{ 0x0023A9, "Beijing Detianquan Electromechanical Equipment Co., Ltd", "BEIJING-DETIANQUAN-ELECTROMECHANICAL-EQUIPMENT" },
	{ 0x0023AA, "HFR, Inc.", "HFR" },
	{ 0x0023AB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0023AC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0023AD, "Xmark Corporation", "XMARK" },
	{ 0x0023AE, "Dell Inc.", "DELL" },
	{ 0x0023AF, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0023B0, "COMXION Technology Inc.", "COMXION" },
	{ 0x0023B1, "Longcheer Technology (Singapore) Pte Ltd", "LONGCHEER-SINGAPORE-PTE" },
	{ 0x0023B2, "Intelligent Mechatronic Systems Inc", "INTELLIGENT-MECHATRONIC" },
	{ 0x0023B3, "Lyyn AB", "LYYN-AB" },
	{ 0x0023B4, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0023B5, "ORTANA LTD", "ORTANA" },
	{ 0x0023B6, "SECURITE COMMUNICATIONS / HONEYWELL", "SECURITE-COMMUNICATION-/-HONEYWELL" },
	{ 0x0023B7, "Q-Light Co., Ltd.", "Q-LIGHT" },
	{ 0x0023B8, "Sichuan Jiuzhou Electronic Technology Co.,Ltd", "SICHUAN-JIUZHOU" },
	{ 0x0023B9, "EADS Deutschland GmbH", "EADS-DEUTSCHLAND" },
	{ 0x0023BA, "Chroma", "CHROMA" },
	{ 0x0023BB, "Schmitt Industries", "SCHMITT-INDUSTRIES" },
	{ 0x0023BC, "EQ-SYS GmbH", "EQ-SYS" },
	{ 0x0023BD, "Digital Ally, Inc.", "DIGITAL-ALLY" },
	{ 0x0023BE, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x0023BF, "Mainpine, Inc.", "MAINPINE" },
	{ 0x0023C0, "Broadway Networks", "BROADWAY-NETWORKS" },
	{ 0x0023C1, "Securitas Direct AB", "SECURITAS-DIRECT-AB" },
	{ 0x0023C2, "SAMSUNG Electronics. Co. LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x0023C3, "LogMeIn, Inc.", "LOGMEIN" },
	{ 0x0023C4, "Lux Lumen", "LUX-LUMEN" },
	{ 0x0023C5, "Radiation Safety and Control Services Inc", "RADIATION-SAFETY-AND-CONTROL-SERVICES" },
	{ 0x0023C6, "SMC Corporation", "SMC" },
	{ 0x0023C7, "AVSystem", "AVSYSTEM" },
	{ 0x0023C8, "TEAM-R", "TEAM-R" },
	{ 0x0023C9, "Sichuan Tianyi Information Science & Technology Stock CO.,LTD", "SICHUAN-TIANYI-INFORMATION-SCIENCE-STOCK" },
	{ 0x0023CA, "Behind The Set, LLC", "BEHIND-THE-SET-LLC" },
	{ 0x0023CB, "Shenzhen Full-join Technology Co.,Ltd", "SHENZHEN-FULL-JOIN" },
	{ 0x0023CC, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x0023CD, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x0023CE, "KITA DENSHI CORPORATION", "KITA-DENSHI" },
	{ 0x0023CF, "CUMMINS-ALLISON CORP.", "CUMMINS-ALLISON" },
	{ 0x0023D0, "Uniloc USA Inc.", "UNILOC-USA" },
	{ 0x0023D1, "TRG", "TRG" },
	{ 0x0023D2, "Inhand Electronics, Inc.", "INHAND-ELECTRONICS" },
	{ 0x0023D3, "AirLink WiFi Networking Corp.", "AIRLINK-WIFI-NETWORKING" },
	{ 0x0023D4, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0023D5, "WAREMA electronic GmbH", "WAREMA" },
	{ 0x0023D6, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x0023D7, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x0023D8, "Ball-It Oy", "BALL-IT-OY" },
	{ 0x0023D9, "Banner Engineering", "BANNER-ENGINEERING" },
	{ 0x0023DA, "Industrial Computer Source (Deutschland)GmbH", "INDUSTRIAL-SOURCE-DEUTSCHLAND" },
	{ 0x0023DB, "saxnet gmbh", "SAXNET" },
	{ 0x0023DC, "Benein, Inc", "BENEIN" },
	{ 0x0023DD, "ELGIN S.A.", "ELGIN" },
	{ 0x0023DE, "Ansync Inc.", "ANSYNC" },
	{ 0x0023DF, "Apple", "APPLE" },
	{ 0x0023E0, "INO Therapeutics LLC", "INO-THERAPEUTICS-LLC" },
	{ 0x0023E1, "Cavena Image Products AB", "CAVENA-IMAGE-PRODUCTS-AB" },
	{ 0x0023E2, "SEA Signalisation", "SEA-SIGNALISATION" },
	{ 0x0023E3, "Microtronic AG", "MICROTRONIC" },
	{ 0x0023E4, "IPnect co. ltd.", "IPNECT" },
	{ 0x0023E5, "IPaXiom Networks", "IPAXIOM-NETWORKS" },
	{ 0x0023E6, "Pirkus, Inc.", "PIRKUS" },
	{ 0x0023E7, "Hinke A/S", "HINKE-A/S" },
	{ 0x0023E8, "Demco Corp.", "DEMCO" },
	{ 0x0023E9, "F5 Networks, Inc.", "F5-NETWORKS" },
	{ 0x0023EA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0023EB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0023EC, "Algorithmix GmbH", "ALGORITHMIX" },
	{ 0x0023ED, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0023EE, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0023EF, "Zuend Systemtechnik AG", "ZUEND-SYSTEMTECHNIK" },
	{ 0x0023F0, "Shanghai Jinghan Weighing Apparatus Co. Ltd.", "SHANGHAI-JINGHAN-WEIGHING-APPARATUS" },
	{ 0x0023F1, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x0023F2, "TVLogic", "TVLOGIC" },
	{ 0x0023F3, "Glocom, Inc.", "GLOCOM" },
	{ 0x0023F4, "Masternaut", "MASTERNAUT" },
	{ 0x0023F5, "WILO SE", "WILO-SE" },
	{ 0x0023F6, "Softwell Technology Co., Ltd.", "SOFTWELL" },
	{ 0x0023F7, "PRIVATE", "PRIVATE" },
	{ 0x0023F8, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0x0023F9, "Double-Take Software, INC.", "DOUBLE-TAKE-SOFTWARE" },
	{ 0x0023FA, "RG Nets, Inc.", "RG-NETS" },
	{ 0x0023FB, "IP Datatel, LLC.", "IP-DATATEL-LLC" },
	{ 0x0023FC, "Ultra Stereo Labs, Inc", "ULTRA-STEREO-LABS" },
	{ 0x0023FD, "AFT Atlas Fahrzeugtechnik GmbH", "AFT-ATLAS-FAHRZEUGTECHNIK" },
	{ 0x0023FE, "Biodevices, SA", "BIODEVICES" },
	{ 0x0023FF, "Beijing HTTC Technology Ltd.", "BEIJING-HTTC" },
	{ 0x002400, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x002401, "D-Link Corporation", "D-LINK" },
	{ 0x002402, "Op-Tection GmbH", "OP-TECTION" },
	{ 0x002403, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x002404, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x002405, "Dilog Nordic AB", "DILOG-NORDIC-AB" },
	{ 0x002406, "Pointmobile", "POINTMOBILE" },
	{ 0x002407, "TELEM SAS", "TELEM-SAS" },
	{ 0x002408, "Pacific Biosciences", "PACIFIC-BIOSCIENCES" },
	{ 0x002409, "The Toro Company", "THE-TORO-COMPANY" },
	{ 0x00240A, "US Beverage Net", "US-BEVERAGE-NET" },
	{ 0x00240B, "Virtual Computer Inc.", "VIRTUAL" },
	{ 0x00240C, "DELEC GmbH", "DELEC" },
	{ 0x00240D, "OnePath Networks LTD.", "ONEPATH-NETWORKS" },
	{ 0x00240E, "Inventec Besta Co., Ltd.", "INVENTEC-BESTA" },
	{ 0x00240F, "Ishii Tool & Engineering Corporation", "ISHII-TOOL-ENGINEERING" },
	{ 0x002410, "NUETEQ Technology,Inc.", "NUETEQ" },
	{ 0x002411, "PharmaSmart LLC", "PHARMASMART-LLC" },
	{ 0x002412, "Benign Technologies Co, Ltd.", "BENIGN-TECHNOLOGIES" },
	{ 0x002413, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002414, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002415, "Magnetic Autocontrol GmbH", "MAGNETIC-AUTOCONTROL" },
	{ 0x002416, "Any Use", "ANY-USE" },
	{ 0x002417, "Thomson Telecom Belgium", "THOMSON-TELECOM-BELGIUM" },
	{ 0x002418, "Nextwave Semiconductor", "NEXTWAVE-SEMICONDUCTOR" },
	{ 0x002419, "PRIVATE", "PRIVATE" },
	{ 0x00241A, "Red Beetle Inc.", "RED-BEETLE" },
	{ 0x00241B, "iWOW Communications Pte Ltd", "IWOW-COMMUNICATION-PTE" },
	{ 0x00241C, "FuGang Electronic (DG) Co.,Ltd", "FUGANG-DG" },
	{ 0x00241D, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x00241E, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x00241F, "DCT-Delta GmbH", "DCT-DELTA" },
	{ 0x002420, "NetUP Inc.", "NETUP" },
	{ 0x002421, "MICRO-STAR INT'L CO., LTD.", "MICRO-STAR" },
	{ 0x002422, "Knapp Logistik Automation GmbH", "KNAPP-LOGISTIK-AUTOMATION" },
	{ 0x002423, "AzureWave Technologies (Shanghai) Inc.", "AZUREWAVE-TECHNOLOGIES-SHANGHAI" },
	{ 0x002424, "Axis Network Technology", "AXIS-NETWORK" },
	{ 0x002425, "Shenzhenshi chuangzhicheng Technology Co.,Ltd", "SHENZHENSHI-CHUANGZHICHENG" },
	{ 0x002426, "NOHMI BOSAI LTD.", "NOHMI-BOSAI" },
	{ 0x002427, "SSI COMPUTER CORP", "SSI" },
	{ 0x002428, "EnergyICT", "ENERGYICT" },
	{ 0x002429, "MK MASTER INC.", "MK-MASTER" },
	{ 0x00242A, "Hittite Microwave Corporation", "HITTITE-MICROWAVE" },
	{ 0x00242B, "Hon Hai Precision Ind.Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x00242C, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x00242E, "Datastrip Inc.", "DATASTRIP" },
	{ 0x00242F, "Micron", "MICRON" },
	{ 0x002430, "Ruby Tech Corp.", "RUBY-TECH" },
	{ 0x002431, "Uni-v co.,ltd", "UNI-V" },
	{ 0x002432, "Neostar Technology Co.,LTD", "NEOSTAR" },
	{ 0x002433, "Alps Electric Co., Ltd", "ALPS-ELECTRIC" },
	{ 0x002434, "Lectrosonics, Inc.", "LECTROSONICS" },
	{ 0x002435, "WIDE CORPORATION", "WIDE" },
	{ 0x002436, "Apple", "APPLE" },
	{ 0x002437, "Motorola - BSG", "MOTOROLA---BSG" },
	{ 0x002438, "Brocade Communications Systems, Inc", "BROCADE-COMMUNICATION" },
	{ 0x002439, "Digital Barriers Advanced Technologies", "DIGITAL-BARRIERS-ADVANCED-TECHNOLOGIES" },
	{ 0x00243A, "Ludl Electronic Products", "LUDL-PRODUCTS" },
	{ 0x00243B, "CSSI (S) Pte Ltd", "CSSI-S-PTE" },
	{ 0x00243C, "S.A.A.A.", "A-A" },
	{ 0x00243D, "Emerson Appliance Motors and Controls", "EMERSON-APPLIANCE-MOTORS-AND-CONTROLS" },
	{ 0x00243F, "Storwize, Inc.", "STORWIZE" },
	{ 0x002440, "Halo Monitoring, Inc.", "HALO-MONITORING" },
	{ 0x002441, "Wanzl Metallwarenfabrik GmbH", "WANZL-METALLWARENFABRIK" },
	{ 0x002442, "Axona Limited", "AXONA" },
	{ 0x002443, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x002444, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x002445, "CommScope Canada Inc.", "COMMSCOPE-CANADA" },
	{ 0x002446, "MMB Research Inc.", "MMB-RESEARCH" },
	{ 0x002447, "Kaztek Systems", "KAZTEK" },
	{ 0x002448, "SpiderCloud Wireless, Inc", "SPIDERCLOUD-WIRELESS" },
	{ 0x002449, "Shen Zhen Lite Star Electronics Technology Co., Ltd", "SHEN-ZHEN-LITE-STAR-ELECTRONICS" },
	{ 0x00244A, "Voyant International", "VOYANT" },
	{ 0x00244B, "PERCEPTRON INC", "PERCEPTRON" },
	{ 0x00244C, "Solartron Metrology Ltd", "SOLARTRON-METROLOGY" },
	{ 0x00244D, "Hokkaido Electronics Corporation", "HOKKAIDO-ELECTRONICS" },
	{ 0x00244E, "RadChips, Inc.", "RADCHIPS" },
	{ 0x00244F, "Asantron Technologies Ltd.", "ASANTRON-TECHNOLOGIES" },
	{ 0x002450, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002451, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002452, "Silicon Software GmbH", "SILICON-SOFTWARE" },
	{ 0x002453, "Initra d.o.o.", "INITRA-D-O-O" },
	{ 0x002454, "Samsung Electronics CO., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x002455, "MuLogic BV", "MULOGIC" },
	{ 0x002456, "2Wire", "2WIRE" },
	{ 0x002458, "PA Bastion CC", "PA-BASTION-CC" },
	{ 0x002459, "ABB STOTZ-KONTAKT GmbH", "ABB-STOTZ-KONTAKT" },
	{ 0x00245A, "Nanjing Panda Electronics Company Limited", "NANJING-PANDA-ELECTRONICS-COMPANY" },
	{ 0x00245B, "RAIDON TECHNOLOGY, INC.", "RAIDON" },
	{ 0x00245C, "Design-Com Technologies Pty. Ltd.", "DESIGN-COM-TECHNOLOGIES-PTY" },
	{ 0x00245D, "Terberg besturingstechniek B.V.", "TERBERG-BESTURINGSTECHNIEK" },
	{ 0x00245E, "Hivision Co.,ltd", "HIVISION" },
	{ 0x00245F, "Vine Telecom CO.,Ltd.", "VINE-TELECOM" },
	{ 0x002460, "Giaval Science Development Co. Ltd.", "GIAVAL-SCIENCE-DEVELOPMENT" },
	{ 0x002461, "Shin Wang Tech.", "SHIN-WANG-TECH" },
	{ 0x002462, "Rayzone Corporation", "RAYZONE" },
	{ 0x002463, "Phybridge Inc", "PHYBRIDGE" },
	{ 0x002464, "Bridge Technologies Co AS", "BRIDGE-TECHNOLOGIES-AS" },
	{ 0x002465, "Elentec", "ELENTEC" },
	{ 0x002466, "Unitron nv", "UNITRON-NV" },
	{ 0x002467, "AOC International (Europe) GmbH", "AOC-EUROPE" },
	{ 0x002468, "Sumavision Technologies Co.,Ltd", "SUMAVISION-TECHNOLOGIES" },
	{ 0x002469, "Smart Doorphones", "SMART-DOORPHONES" },
	{ 0x00246A, "Solid Year Co., Ltd.", "SOLID-YEAR" },
	{ 0x00246B, "Covia, Inc.", "COVIA" },
	{ 0x00246C, "ARUBA NETWORKS, INC.", "ARUBA-NETWORKS" },
	{ 0x00246D, "Weinzierl Engineering GmbH", "WEINZIERL-ENGINEERING" },
	{ 0x00246E, "Phihong USA Corp.", "PHIHONG-USA" },
	{ 0x00246F, "Onda Communication spa", "ONDA-COMMUNICATION-SPA" },
	{ 0x002470, "AUROTECH ultrasound AS.", "AUROTECH-ULTRASOUND-AS" },
	{ 0x002471, "Fusion MultiSystems dba Fusion-io", "FUSION-MULTISYSTEMS-DBA-FUSION-IO" },
	{ 0x002472, "ReDriven Power Inc.", "REDRIVEN-POWER" },
	{ 0x002473, "3Com Europe Ltd", "3COM" },
	{ 0x002474, "Autronica Fire And Securirty", "AUTRONICA-FIRE-AND-SECURIRTY" },
	{ 0x002475, "Compass System(Embedded Dept.)", "COMPASS-SYSTEM-EMBEDDED-DEPT" },
	{ 0x002476, "TAP.tv", "TAP-TV" },
	{ 0x002477, "Tibbo Technology", "TIBBO" },
	{ 0x002478, "Mag Tech Electronics Co Limited", "MAG-TECH-ELECTRONICS" },
	{ 0x002479, "Optec Displays, Inc.", "OPTEC-DISPLAYS" },
	{ 0x00247A, "FU YI CHENG Technology Co., Ltd.", "FU-YI-CHENG" },
	{ 0x00247B, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x00247C, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x00247D, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x00247E, "Universal Global Scientific Industrial Co., Ltd", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x00247F, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x002480, "Meteocontrol GmbH", "METEOCONTROL" },
	{ 0x002481, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x002482, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x002483, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x002484, "Bang and Olufsen Medicom a/s", "BANG-AND-OLUFSEN-MEDICOM-A/S" },
	{ 0x002485, "ConteXtream Ltd", "CONTEXTREAM" },
	{ 0x002486, "DesignArt Networks", "DESIGNART-NETWORKS" },
	{ 0x002487, "Blackboard Inc.", "BLACKBOARD" },
	{ 0x002488, "Centre For Development Of Telematics", "CENTRE-FOR-DEVELOPMENT-OF-TELEMATICS" },
	{ 0x002489, "Vodafone Omnitel N.V.", "VODAFONE-OMNITEL-N-V" },
	{ 0x00248A, "Kaga Electronics Co., Ltd.", "KAGA-ELECTRONICS" },
	{ 0x00248B, "HYBUS CO., LTD.", "HYBUS" },
	{ 0x00248C, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x00248D, "Sony Computer Entertainment Inc.", "SONY-ENTERTAINMENT" },
	{ 0x00248E, "Infoware ZRt.", "INFOWARE-ZRT" },
	{ 0x00248F, "DO-MONIX", "DO-MONIX" },
	{ 0x002490, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x002491, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x002492, "Motorola, Broadband Solutions Group", "MOTOROLA-BROADBAND-SOLUTIONS-GROUP" },
	{ 0x002493, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002494, "Shenzhen Baoxin Tech CO., Ltd.", "SHENZHEN-BAOXIN-TECH" },
	{ 0x002495, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002496, "Ginzinger electronic systems", "GINZINGER" },
	{ 0x002497, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002498, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002499, "Aquila Technologies", "AQUILA-TECHNOLOGIES" },
	{ 0x00249A, "Beijing Zhongchuang Telecommunication Test Co., Ltd.", "BEIJING-ZHONGCHUANG-TELECOMMUNICATION-TEST" },
	{ 0x00249B, "Action Star Enterprise Co., Ltd.", "ACTION-STAR" },
	{ 0x00249C, "Bimeng Comunication System Co. Ltd", "BIMENG-COMUNICATION-SYSTEM" },
	{ 0x00249D, "NES Technology Inc.", "NES" },
	{ 0x00249E, "ADC-Elektronik GmbH", "ADC-ELEKTRONIK" },
	{ 0x00249F, "RIM Testing Services", "RIM-TESTING-SERVICES" },
	{ 0x0024A0, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0024A1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0024A2, "Hong Kong Middleware Technology Limited", "HONG-KONG-MIDDLEWARE" },
	{ 0x0024A3, "Sonim Technologies Inc", "SONIM-TECHNOLOGIES" },
	{ 0x0024A4, "Siklu Communication", "SIKLU-COMMUNICATION" },
	{ 0x0024A5, "Buffalo Inc.", "BUFFALO" },
	{ 0x0024A6, "TELESTAR DIGITAL GmbH", "TELESTAR-DIGITAL" },
	{ 0x0024A7, "Advanced Video Communications Inc.", "ADVANCED-VIDEO-COMMUNICATION" },
	{ 0x0024A8, "ProCurve Networking by HP", "PROCURVE-NETWORKING-BY-HP" },
	{ 0x0024A9, "Ag Leader Technology", "LEADER" },
	{ 0x0024AA, "Dycor Technologies Ltd.", "DYCOR-TECHNOLOGIES" },
	{ 0x0024AB, "A7 Engineering, Inc.", "A7-ENGINEERING" },
	{ 0x0024AC, "Hangzhou DPtech Technologies Co., Ltd.", "HANGZHOU-DPTECH-TECHNOLOGIES" },
	{ 0x0024AD, "Adolf Thies Gmbh & Co. KG", "ADOLF-THIES" },
	{ 0x0024AE, "Morpho", "MORPHO" },
	{ 0x0024AF, "EchoStar Technologies", "ECHOSTAR-TECHNOLOGIES" },
	{ 0x0024B0, "ESAB AB", "ESAB-AB" },
	{ 0x0024B1, "Coulomb Technologies", "COULOMB-TECHNOLOGIES" },
	{ 0x0024B2, "Netgear", "NETGEAR" },
	{ 0x0024B3, "Graf-Syteco GmbH & Co. KG", "GRAF-SYTECO" },
	{ 0x0024B4, "ESCATRONIC GmbH", "ESCATRONIC" },
	{ 0x0024B5, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x0024B6, "Seagate Technology", "SEAGATE" },
	{ 0x0024B7, "GridPoint, Inc.", "GRIDPOINT" },
	{ 0x0024B8, "free alliance sdn bhd", "FREE-ALLIANCE-SDN-BHD" },
	{ 0x0024B9, "Wuhan Higheasy Electronic Technology Development Co.Ltd", "WUHAN-HIGHEASY-DEVELOPMENT" },
	{ 0x0024BA, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x0024BB, "CENTRAL Corporation", "CENTRAL" },
	{ 0x0024BC, "HuRob Co.,Ltd", "HUROB" },
	{ 0x0024BD, "Hainzl Industriesysteme GmbH", "HAINZL-INDUSTRIESYSTEME" },
	{ 0x0024BE, "Sony Corporation", "SONY" },
	{ 0x0024BF, "CIAT", "CIAT" },
	{ 0x0024C0, "NTI COMODO INC", "NTI-COMODO" },
	{ 0x0024C1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0024C2, "Asumo Co.,Ltd.", "ASUMO" },
	{ 0x0024C3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0024C4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0024C5, "Meridian Audio Limited", "MERIDIAN-AUDIO" },
	{ 0x0024C6, "Hager Electro SAS", "HAGER-ELECTRO-SAS" },
	{ 0x0024C7, "Mobilarm Ltd", "MOBILARM" },
	{ 0x0024C8, "Broadband Solutions Group", "BROADBAND-SOLUTIONS-GROUP" },
	{ 0x0024C9, "Broadband Solutions Group", "BROADBAND-SOLUTIONS-GROUP" },
	{ 0x0024CA, "Tobii Technology AB", "TOBII-AB" },
	{ 0x0024CB, "Autonet Mobile", "AUTONET-MOBILE" },
	{ 0x0024CC, "Fascinations Toys and Gifts, Inc.", "FASCINATIONS-TOYS-AND-GIFTS" },
	{ 0x0024CD, "Willow Garage, Inc.", "WILLOW-GARAGE" },
	{ 0x0024CE, "Exeltech Inc", "EXELTECH" },
	{ 0x0024CF, "Inscape Data Corporation", "INSCAPE-DATA" },
	{ 0x0024D0, "Shenzhen SOGOOD Industry CO.,LTD.", "SHENZHEN-SOGOOD-INDUSTRY" },
	{ 0x0024D1, "Thomson Inc.", "THOMSON" },
	{ 0x0024D2, "Askey Computer", "ASKEY" },
	{ 0x0024D3, "QUALICA Inc.", "QUALICA" },
	{ 0x0024D4, "FREEBOX SA", "FREEBOX" },
	{ 0x0024D5, "Winward Industrial Limited", "WINWARD-INDUSTRIAL" },
	{ 0x0024D6, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0024D7, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0024D8, "IlSung Precision", "ILSUNG-PRECISION" },
	{ 0x0024D9, "BICOM, Inc.", "BICOM" },
	{ 0x0024DA, "Innovar Systems Limited", "INNOVAR" },
	{ 0x0024DB, "Alcohol Monitoring Systems", "ALCOHOL-MONITORING" },
	{ 0x0024DC, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x0024DD, "Centrak, Inc.", "CENTRAK" },
	{ 0x0024DE, "GLOBAL Technology Inc.", "GLOBAL" },
	{ 0x0024DF, "Digitalbox Europe GmbH", "DIGITALBOX-EUROPE" },
	{ 0x0024E0, "DS Tech, LLC", "DS-TECH-LLC" },
	{ 0x0024E1, "Convey Computer Corp.", "CONVEY" },
	{ 0x0024E2, "HASEGAWA ELECTRIC CO.,LTD.", "HASEGAWA-ELECTRIC" },
	{ 0x0024E3, "CAO Group", "CAO-GROUP" },
	{ 0x0024E4, "Withings", "WITHINGS" },
	{ 0x0024E5, "Seer Technology, Inc", "SEER" },
	{ 0x0024E6, "In Motion Technology Inc.", "IN-MOTION" },
	{ 0x0024E7, "Plaster Networks", "PLASTER-NETWORKS" },
	{ 0x0024E8, "Dell Inc.", "DELL" },
	{ 0x0024E9, "Samsung Electronics Co., Ltd., Storage System Division", "SAMSUNG-ELECTRONICS-STORAGE-SYSTEM-DIVISION" },
	{ 0x0024EA, "iris-GmbH infrared & intelligent sensors", "IRIS--INFRARED-INTELLIGENT-SENSORS" },
	{ 0x0024EB, "ClearPath Networks, Inc.", "CLEARPATH-NETWORKS" },
	{ 0x0024EC, "United Information Technology Co.,Ltd.", "UNITED-INFORMATION" },
	{ 0x0024ED, "YT Elec. Co,.Ltd.", "YT-ELEC" },
	{ 0x0024EE, "Wynmax Inc.", "WYNMAX" },
	{ 0x0024EF, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x0024F0, "Seanodes", "SEANODES" },
	{ 0x0024F1, "Shenzhen Fanhai Sanjiang Electronics Co., Ltd.", "SHENZHEN-FANHAI-SANJIANG-ELECTRONICS" },
	{ 0x0024F2, "Uniphone Telecommunication Co., Ltd.", "UNIPHONE-TELECOMMUNICATION" },
	{ 0x0024F3, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x0024F4, "Kaminario Technologies Ltd.", "KAMINARIO-TECHNOLOGIES" },
	{ 0x0024F5, "NDS Surgical Imaging", "NDS-SURGICAL-IMAGING" },
	{ 0x0024F6, "MIYOSHI ELECTRONICS CORPORATION", "MIYOSHI-ELECTRONICS" },
	{ 0x0024F7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0024F8, "Technical Solutions Company Ltd.", "TECHNICAL-SOLUTIONS-COMPANY" },
	{ 0x0024F9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0024FA, "Hilger u. Kern GMBH", "HILGER-U-KERN" },
	{ 0x0024FB, "PRIVATE", "PRIVATE" },
	{ 0x0024FC, "QuoPin Co., Ltd.", "QUOPIN" },
	{ 0x0024FD, "Accedian Networks Inc", "ACCEDIAN-NETWORKS" },
	{ 0x0024FE, "AVM GmbH", "AVM" },
	{ 0x0024FF, "QLogic Corporation", "QLOGIC" },
	{ 0x002500, "Apple", "APPLE" },
	{ 0x002501, "JSC \"Supertel\"", "JSC-SUPERTEL" },
	{ 0x002502, "NaturalPoint", "NATURALPOINT" },
	{ 0x002503, "IBM Corp", "IBM" },
	{ 0x002504, "Valiant Communications Limited", "VALIANT-COMMUNICATION" },
	{ 0x002505, "eks Engel GmbH & Co. KG", "EKS-ENGEL" },
	{ 0x002506, "A.I. ANTITACCHEGGIO ITALIA SRL", "A-I-ANTITACCHEGGIO-ITALIA-SRL" },
	{ 0x002507, "ASTAK Inc.", "ASTAK" },
	{ 0x002508, "Maquet Cardiopulmonary AG", "MAQUET-CARDIOPULMONARY" },
	{ 0x002509, "SHARETRONIC Group LTD", "SHARETRONIC-GROUP" },
	{ 0x00250A, "Security Expert Co. Ltd", "SECURITY-EXPERT" },
	{ 0x00250B, "CENTROFACTOR INC", "CENTROFACTOR" },
	{ 0x00250C, "Enertrac", "ENERTRAC" },
	{ 0x00250D, "GZT Telkom-Telmor sp. z o.o.", "GZT-TELKOM-TELMOR-SP-Z-O-O" },
	{ 0x00250E, "gt german telematics gmbh", "GT-GERMAN-TELEMATICS" },
	{ 0x00250F, "On-Ramp Wireless, Inc.", "ON-RAMP-WIRELESS" },
	{ 0x002510, "Pico-Tesla Magnetic Therapies", "PICO-TESLA-MAGNETIC-THERAPIES" },
	{ 0x002511, "ELITEGROUP COMPUTER SYSTEM CO., LTD.", "ELITEGROUP-SYSTEM" },
	{ 0x002512, "ZTE Corporation", "ZTE" },
	{ 0x002513, "CXP DIGITAL BV", "CXP-DIGITAL" },
	{ 0x002514, "PC Worth Int'l Co., Ltd.", "PC-WORTH" },
	{ 0x002515, "SFR", "SFR" },
	{ 0x002516, "Integrated Design Tools, Inc.", "INTEGRATED-DESIGN-TOOLS" },
	{ 0x002517, "Venntis, LLC", "VENNTIS-LLC" },
	{ 0x002518, "Power PLUS Communications AG", "POWER-PLUS-COMMUNICATION" },
	{ 0x002519, "Viaas Inc", "VIAAS" },
	{ 0x00251A, "Psiber Data Systems Inc.", "PSIBER-DATA" },
	{ 0x00251B, "Philips CareServant", "PHILIPS-CARESERVANT" },
	{ 0x00251C, "EDT", "EDT" },
	{ 0x00251D, "DSA Encore, LLC", "DSA-ENCORE-LLC" },
	{ 0x00251E, "ROTEL TECHNOLOGIES", "ROTEL-TECHNOLOGIES" },
	{ 0x00251F, "ZYNUS VISION INC.", "ZYNUS-VISION" },
	{ 0x002520, "SMA Railway Technology GmbH", "SMA-RAILWAY" },
	{ 0x002521, "Logitek Electronic Systems, Inc.", "LOGITEK" },
	{ 0x002522, "ASRock Incorporation", "ASROCK-INCORPORATION" },
	{ 0x002523, "OCP Inc.", "OCP" },
	{ 0x002524, "Lightcomm Technology Co., Ltd", "LIGHTCOMM" },
	{ 0x002525, "CTERA Networks Ltd.", "CTERA-NETWORKS" },
	{ 0x002526, "Genuine Technologies Co., Ltd.", "GENUINE-TECHNOLOGIES" },
	{ 0x002527, "Bitrode Corp.", "BITRODE" },
	{ 0x002528, "Daido Signal Co., Ltd.", "DAIDO-SIGNAL" },
	{ 0x002529, "COMELIT GROUP S.P.A", "COMELIT-GROUP-S-P-A" },
	{ 0x00252A, "Chengdu GeeYa Technology Co.,LTD", "CHENGDU-GEEYA" },
	{ 0x00252B, "Stirling Energy Systems", "STIRLING-ENERGY" },
	{ 0x00252C, "Entourage Systems, Inc.", "ENTOURAGE" },
	{ 0x00252D, "Kiryung Electronics", "KIRYUNG-ELECTRONICS" },
	{ 0x00252E, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x00252F, "Energy, Inc.", "ENERGY" },
	{ 0x002530, "Aetas Systems Inc.", "AETAS" },
	{ 0x002531, "Cloud Engines, Inc.", "CLOUD-ENGINES" },
	{ 0x002532, "Digital Recorders", "DIGITAL-RECORDERS" },
	{ 0x002533, "WITTENSTEIN AG", "WITTENSTEIN" },
	{ 0x002535, "Minimax GmbH & Co KG", "MINIMAX" },
	{ 0x002536, "Oki Electric Industry Co., Ltd.", "OKI-ELECTRIC-INDUSTRY" },
	{ 0x002537, "Runcom Technologies Ltd.", "RUNCOM-TECHNOLOGIES" },
	{ 0x002538, "Samsung Electronics Co., Ltd., Memory Division", "SAMSUNG-ELECTRONICS-MEMORY-DIVISION" },
	{ 0x002539, "IfTA GmbH", "IFTA" },
	{ 0x00253A, "CEVA, Ltd.", "CEVA" },
	{ 0x00253B, "din Dietmar Nocker Facilitymanagement GmbH", "DIN-DIETMAR-NOCKER-FACILITYMANAGEMENT" },
	{ 0x00253C, "2Wire", "2WIRE" },
	{ 0x00253D, "DRS Consolidated Controls", "DRS-CONSOLIDATED-CONTROLS" },
	{ 0x00253E, "Sensus Metering Systems", "SENSUS-METERING" },
	{ 0x002540, "Quasar Technologies, Inc.", "QUASAR-TECHNOLOGIES" },
	{ 0x002541, "Maquet Critical Care AB", "MAQUET-CRITICAL-CARE-AB" },
	{ 0x002542, "Pittasoft", "PITTASOFT" },
	{ 0x002543, "MONEYTECH", "MONEYTECH" },
	{ 0x002544, "LoJack Corporation", "LOJACK" },
	{ 0x002545, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002546, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002547, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x002548, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x002549, "Jeorich Tech. Co.,Ltd.", "JEORICH-TECH" },
	{ 0x00254A, "RingCube Technologies, Inc.", "RINGCUBE-TECHNOLOGIES" },
	{ 0x00254B, "Apple", "APPLE" },
	{ 0x00254C, "Videon Central, Inc.", "VIDEON-CENTRAL" },
	{ 0x00254D, "Singapore Technologies Electronics Limited", "SINGAPORE-TECHNOLOGIES-ELECTRONICS" },
	{ 0x00254E, "Vertex Wireless Co., Ltd.", "VERTEX-WIRELESS" },
	{ 0x00254F, "ELETTROLAB Srl", "ELETTROLAB-SRL" },
	{ 0x002550, "Riverbed Technology", "RIVERBED" },
	{ 0x002551, "SE-Elektronic GmbH", "SE-ELEKTRONIC" },
	{ 0x002552, "VXI CORPORATION", "VXI" },
	{ 0x002553, "Pirelli Tyre S.p.A.", "PIRELLI-TYRE-S-P-A" },
	{ 0x002554, "Pixel8 Networks", "PIXEL8-NETWORKS" },
	{ 0x002555, "Visonic Technologies 1993 Ltd", "VISONIC-TECHNOLOGIES-1993" },
	{ 0x002556, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x002557, "Research In Motion", "RESEARCH-IN-MOTION" },
	{ 0x002558, "MPEDIA", "MPEDIA" },
	{ 0x002559, "Syphan Technologies Ltd", "SYPHAN-TECHNOLOGIES" },
	{ 0x00255A, "Tantalus Systems Corp.", "TANTALUS" },
	{ 0x00255B, "CoachComm, LLC", "COACHCOMM-LLC" },
	{ 0x00255C, "NEC Corporation", "NEC" },
	{ 0x00255D, "Morningstar Corporation", "MORNINGSTAR" },
	{ 0x00255E, "Shanghai Dare Technologies Co.,Ltd.", "SHANGHAI-DARE-TECHNOLOGIES" },
	{ 0x00255F, "SenTec AG", "SENTEC" },
	{ 0x002560, "Ibridge Networks & Communications Ltd.", "IBRIDGE-NETWORKS-COMMUNICATION" },
	{ 0x002561, "ProCurve Networking by HP", "PROCURVE-NETWORKING-BY-HP" },
	{ 0x002562, "interbro Co. Ltd.", "INTERBRO" },
	{ 0x002563, "Luxtera Inc", "LUXTERA" },
	{ 0x002564, "Dell Inc.", "DELL" },
	{ 0x002565, "Vizimax Inc.", "VIZIMAX" },
	{ 0x002566, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x002567, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x002568, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x002569, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x00256A, "inIT - Institut Industrial IT", "INIT---INSTITUT-INDUSTRIAL-IT" },
	{ 0x00256B, "ATENIX E.E. s.r.l.", "ATENIX-E-E-S-R-L" },
	{ 0x00256C, "\"Azimut\" Production Association JSC", "AZIMUT-PRODUCTION-ASSOCIATION-JSC" },
	{ 0x00256D, "Broadband Forum", "BROADBAND-FORUM" },
	{ 0x00256E, "Van Breda B.V.", "VAN-BREDA" },
	{ 0x00256F, "Dantherm Power", "DANTHERM-POWER" },
	{ 0x002570, "Eastern Communications Company Limited", "EASTERN-COMMUNICATION-COMPANY" },
	{ 0x002571, "Zhejiang Tianle Digital Electric Co.,Ltd", "ZHEJIANG-TIANLE-DIGITAL-ELECTRIC" },
	{ 0x002572, "Nemo-Q International AB", "NEMO-Q-AB" },
	{ 0x002573, "ST Electronics (Info-Security) Pte Ltd", "ST-ELECTRONICS-INFO-SECURITY-PTE" },
	{ 0x002574, "KUNIMI MEDIA DEVICE Co., Ltd.", "KUNIMI-MEDIA-DEVICE" },
	{ 0x002575, "FiberPlex Technologies, LLC", "FIBERPLEX-TECHNOLOGIES-LLC" },
	{ 0x002576, "NELI TECHNOLOGIES", "NELI-TECHNOLOGIES" },
	{ 0x002577, "D-BOX Technologies", "D-BOX-TECHNOLOGIES" },
	{ 0x002578, "JSC \"Concern \"Sozvezdie\"", "JSC-CONCERN-SOZVEZDIE" },
	{ 0x002579, "J & F Labs", "J-F-LABS" },
	{ 0x00257A, "CAMCO Produktions- und Vertriebs-GmbH für Beschallungs- und Beleuchtungsanlagen", "CAMCO-PRODUKTIONS--UND-VERTRIEBS--FüR-BESCHALLUNGS--UND-BELEUCHTUNGSANLAGEN" },
	{ 0x00257B, "STJ ELECTRONICS PVT LTD", "STJ-ELECTRONICS-PVT" },
	{ 0x00257C, "Huachentel Technology Development Co., Ltd", "HUACHENTEL-DEVELOPMENT" },
	{ 0x00257D, "PointRed Telecom Private Ltd.", "POINTRED-TELECOM-PRIVATE" },
	{ 0x00257E, "NEW POS Technology Limited", "NEW-POS" },
	{ 0x00257F, "CallTechSolution Co.,Ltd", "CALLTECHSOLUTION" },
	{ 0x002580, "Equipson S.A.", "EQUIPSON" },
	{ 0x002581, "x-star networks Inc.", "X-STAR-NETWORKS" },
	{ 0x002582, "Maksat Technologies (P) Ltd", "MAKSAT-TECHNOLOGIES-P" },
	{ 0x002583, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002584, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002585, "KOKUYO S&T Co., Ltd.", "KOKUYO-S-T" },
	{ 0x002586, "TP-LINK Technologies Co., Ltd.", "TP-LINK-TECHNOLOGIES" },
	{ 0x002587, "Vitality, Inc.", "VITALITY" },
	{ 0x002588, "Genie Industries, Inc.", "GENIE-INDUSTRIES" },
	{ 0x002589, "Hills Industries Limited", "HILLS-INDUSTRIES" },
	{ 0x00258A, "Pole/Zero Corporation", "POLE/ZERO" },
	{ 0x00258B, "Mellanox Technologies Ltd", "MELLANOX-TECHNOLOGIES" },
	{ 0x00258C, "ESUS ELEKTRONIK SAN. VE DIS. TIC. LTD. STI.", "ESUS-ELEKTRONIK-SAN-VE-DIS-TIC-STI" },
	{ 0x00258D, "Haier", "HAIER" },
	{ 0x00258E, "The Weather Channel", "THE-WEATHER-CHANNEL" },
	{ 0x00258F, "Trident Microsystems, Inc.", "TRIDENT-MICROSYSTEMS" },
	{ 0x002590, "Super Micro Computer, Inc.", "SUPER-MICRO" },
	{ 0x002591, "NEXTEK, Inc.", "NEXTEK" },
	{ 0x002592, "Guangzhou Shirui Electronic Co., Ltd", "GUANGZHOU-SHIRUI" },
	{ 0x002593, "DatNet Informatikai Kft.", "DATNET-INFORMATIKAI-KFT" },
	{ 0x002594, "Eurodesign BG LTD", "EURODESIGN-BG" },
	{ 0x002595, "Northwest Signal Supply, Inc", "NORTHWEST-SIGNAL-SUPPLY" },
	{ 0x002596, "GIGAVISION srl", "GIGAVISION-SRL" },
	{ 0x002597, "Kalki Communication Technologies", "KALKI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x002598, "Zhong Shan City Litai Electronic Industrial Co. Ltd", "ZHONG-SHAN-CITY-LITAI-INDUSTRIAL" },
	{ 0x002599, "Hedon e.d. B.V.", "HEDON-E-D" },
	{ 0x00259A, "CEStronics GmbH", "CESTRONICS" },
	{ 0x00259B, "Beijing PKUNITY Microsystems Technology Co., Ltd", "BEIJING-PKUNITY-MICROSYSTEMS" },
	{ 0x00259C, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x00259D, "PRIVATE", "PRIVATE" },
	{ 0x00259E, "Huawei Technologies Co., Ltd.", "HUAWEI-TECHNOLOGIES" },
	{ 0x00259F, "TechnoDigital Technologies GmbH", "TECHNODIGITAL-TECHNOLOGIES" },
	{ 0x0025A0, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x0025A1, "Enalasys", "ENALASYS" },
	{ 0x0025A2, "Alta Definicion LINCEO S.L.", "ALTA-DEFINICION-LINCEO-S-L" },
	{ 0x0025A3, "Trimax Wireless, Inc.", "TRIMAX-WIRELESS" },
	{ 0x0025A4, "EuroDesign embedded technologies GmbH", "EURODESIGN-EMBEDDED-TECHNOLOGIES" },
	{ 0x0025A5, "Walnut Media Network", "WALNUT-MEDIA-NETWORK" },
	{ 0x0025A6, "Central Network Solution Co., Ltd.", "CENTRAL-NETWORK-SOLUTION" },
	{ 0x0025A7, "Comverge, Inc.", "COMVERGE" },
	{ 0x0025A8, "Kontron (BeiJing) Technology Co.,Ltd", "KONTRON-BEIJING" },
	{ 0x0025A9, "Shanghai Embedway Information Technologies Co.,Ltd", "SHANGHAI-EMBEDWAY-INFORMATION-TECHNOLOGIES" },
	{ 0x0025AA, "Beijing Soul Technology Co.,Ltd.", "BEIJING-SOUL" },
	{ 0x0025AB, "AIO LCD PC BU / TPV", "AIO-LCD-PC-BU-/-TPV" },
	{ 0x0025AC, "I-Tech corporation", "I-TECH" },
	{ 0x0025AD, "Manufacturing Resources International", "MANUFACTURING-RESOURCES" },
	{ 0x0025AE, "Microsoft Corporation", "MICROSOFT" },
	{ 0x0025AF, "COMFILE Technology", "COMFILE" },
	{ 0x0025B0, "Schmartz Inc", "SCHMARTZ" },
	{ 0x0025B1, "Maya-Creation Corporation", "MAYA-CREATION" },
	{ 0x0025B2, "MBDA Deutschland GmbH", "MBDA-DEUTSCHLAND" },
	{ 0x0025B3, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x0025B4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0025B5, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0025B6, "Telecom FM", "TELECOM-FM" },
	{ 0x0025B7, "Costar electronics, inc.,", "COSTAR-ELECTRONICS" },
	{ 0x0025B8, "Agile Communications, Inc.", "AGILE-COMMUNICATION" },
	{ 0x0025B9, "Cypress Solutions Inc", "CYPRESS-SOLUTIONS" },
	{ 0x0025BA, "Alcatel-Lucent IPD", "ALCATEL-LUCENT-IPD" },
	{ 0x0025BB, "INNERINT Co., Ltd.", "INNERINT" },
	{ 0x0025BC, "Apple", "APPLE" },
	{ 0x0025BD, "Italdata Ingegneria dell'Idea S.p.A.", "ITALDATA-INGEGNERIA-DELL-IDEA-S-P-A" },
	{ 0x0025BE, "Tektrap Systems Inc.", "TEKTRAP" },
	{ 0x0025BF, "Wireless Cables Inc.", "WIRELESS-CABLES" },
	{ 0x0025C0, "ZillionTV Corporation", "ZILLIONTV" },
	{ 0x0025C1, "Nawoo Korea Corp.", "NAWOO-KOREA" },
	{ 0x0025C2, "RingBell Co.,Ltd.", "RINGBELL" },
	{ 0x0025C3, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x0025C4, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x0025C5, "Star Link Communication Pvt. Ltd.", "STAR-LINK-COMMUNICATION-PVT" },
	{ 0x0025C6, "kasercorp, ltd", "KASERCORP" },
	{ 0x0025C7, "altek Corporation", "ALTEK" },
	{ 0x0025C8, "S-Access GmbH", "S-ACCESS" },
	{ 0x0025C9, "SHENZHEN HUAPU DIGITAL CO., LTD", "SHENZHEN-HUAPU-DIGITAL" },
	{ 0x0025CA, "LS Research, LLC", "LS-RESEARCH-LLC" },
	{ 0x0025CB, "Reiner SCT", "REINER-SCT" },
	{ 0x0025CC, "Mobile Communications Korea Incorporated", "MOBILE-COMMUNICATION-KOREA-INCORPORATED" },
	{ 0x0025CD, "Skylane Optics", "SKYLANE-OPTICS" },
	{ 0x0025CE, "InnerSpace", "INNERSPACE" },
	{ 0x0025CF, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0025D0, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0025D1, "Eastern Asia Technology Limited", "EASTERN-ASIA" },
	{ 0x0025D2, "InpegVision Co., Ltd", "INPEGVISION" },
	{ 0x0025D3, "AzureWave Technologies, Inc", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x0025D4, "Fortress Technologies", "FORTRESS-TECHNOLOGIES" },
	{ 0x0025D5, "Robonica (Pty) Ltd", "ROBONICA-PTY" },
	{ 0x0025D6, "The Kroger Co.", "THE-KROGER" },
	{ 0x0025D7, "CEDO", "CEDO" },
	{ 0x0025D8, "KOREA MAINTENANCE", "KOREA-MAINTENANCE" },
	{ 0x0025D9, "DataFab Systems Inc.", "DATAFAB" },
	{ 0x0025DA, "Secura Key", "SECURA-KEY" },
	{ 0x0025DB, "ATI Electronics(Shenzhen) Co., LTD", "ATI-ELECTRONICS-SHENZHEN" },
	{ 0x0025DC, "Sumitomo Electric Networks, Inc", "SUMITOMO-ELECTRIC-NETWORKS" },
	{ 0x0025DD, "SUNNYTEK INFORMATION CO., LTD.", "SUNNYTEK-INFORMATION" },
	{ 0x0025DE, "Probits Co., LTD.", "PROBITS" },
	{ 0x0025DF, "PRIVATE", "PRIVATE" },
	{ 0x0025E0, "CeedTec Sdn Bhd", "CEEDTEC-SDN-BHD" },
	{ 0x0025E1, "SHANGHAI SEEYOO ELECTRONIC & TECHNOLOGY CO., LTD", "SHANGHAI-SEEYOO" },
	{ 0x0025E2, "Everspring Industry Co., Ltd.", "EVERSPRING-INDUSTRY" },
	{ 0x0025E3, "Hanshinit Inc.", "HANSHINIT" },
	{ 0x0025E4, "OMNI-WiFi, LLC", "OMNI-WIFI-LLC" },
	{ 0x0025E5, "LG Electronics Inc", "LG-ELECTRONICS" },
	{ 0x0025E6, "Belgian Monitoring Systems bvba", "BELGIAN-MONITORING-BVBA" },
	{ 0x0025E7, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x0025E8, "Idaho Technology", "IDAHO" },
	{ 0x0025E9, "i-mate Development, Inc.", "I-MATE-DEVELOPMENT" },
	{ 0x0025EA, "Iphion BV", "IPHION" },
	{ 0x0025EB, "Reutech Radar Systems (PTY) Ltd", "REUTECH-RADAR-PTY" },
	{ 0x0025EC, "Humanware", "HUMANWARE" },
	{ 0x0025ED, "NuVo Technologies LLC", "NUVO-TECHNOLOGIES-LLC" },
	{ 0x0025EE, "Avtex Ltd", "AVTEX" },
	{ 0x0025EF, "I-TEC Co., Ltd.", "I-TEC" },
	{ 0x0025F0, "Suga Electronics Limited", "SUGA-ELECTRONICS" },
	{ 0x0025F1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0025F2, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0025F3, "Nordwestdeutsche Zählerrevision", "NORDWESTDEUTSCHE-ZäHLERREVISION" },
	{ 0x0025F4, "KoCo Connector AG", "KOCO-CONNECTOR" },
	{ 0x0025F5, "DVS Korea, Co., Ltd", "DVS-KOREA" },
	{ 0x0025F6, "netTALK.com, Inc.", "NETTALK-COM" },
	{ 0x0025F7, "Ansaldo STS USA", "ANSALDO-STS-USA" },
	{ 0x0025F9, "GMK electronic design GmbH", "GMK-DESIGN" },
	{ 0x0025FA, "J&M Analytik AG", "J-M-ANALYTIK" },
	{ 0x0025FB, "Tunstall Healthcare A/S", "TUNSTALL-HEALTHCARE-A/S" },
	{ 0x0025FC, "ENDA ENDUSTRIYEL ELEKTRONIK LTD. STI.", "ENDA-ENDUSTRIYEL-ELEKTRONIK-STI" },
	{ 0x0025FD, "OBR Centrum Techniki Morskiej S.A.", "OBR-CENTRUM-TECHNIKI-MORSKIEJ" },
	{ 0x0025FE, "Pilot Electronics Corporation", "PILOT-ELECTRONICS" },
	{ 0x0025FF, "CreNova Multimedia Co., Ltd", "CRENOVA-MULTIMEDIA" },
	{ 0x002600, "TEAC Australia Pty Ltd.", "TEAC-AUSTRALIA-PTY" },
	{ 0x002601, "Cutera Inc", "CUTERA" },
	{ 0x002602, "SMART Temps LLC", "SMART-TEMPS-LLC" },
	{ 0x002603, "Shenzhen Wistar Technology Co., Ltd", "SHENZHEN-WISTAR" },
	{ 0x002604, "Audio Processing Technology Ltd", "AUDIO-PROCESSING" },
	{ 0x002605, "CC Systems AB", "CC-AB" },
	{ 0x002606, "RAUMFELD GmbH", "RAUMFELD" },
	{ 0x002607, "Enabling Technology Pty Ltd", "ENABLING-PTY" },
	{ 0x002608, "Apple", "APPLE" },
	{ 0x002609, "Phyllis Co., Ltd.", "PHYLLIS" },
	{ 0x00260A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00260B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00260C, "Dataram", "DATARAM" },
	{ 0x00260D, "Mercury Systems, Inc.", "MERCURY" },
	{ 0x00260E, "Ablaze Systems, LLC", "ABLAZE-LLC" },
	{ 0x00260F, "Linn Products Ltd", "LINN-PRODUCTS" },
	{ 0x002610, "Apacewave Technologies", "APACEWAVE-TECHNOLOGIES" },
	{ 0x002611, "Licera AB", "LICERA-AB" },
	{ 0x002612, "Space Exploration Technologies", "SPACE-EXPLORATION-TECHNOLOGIES" },
	{ 0x002613, "Engel Axil S.L.", "ENGEL-AXIL-S-L" },
	{ 0x002614, "KTNF", "KTNF" },
	{ 0x002615, "Teracom Limited", "TERACOM" },
	{ 0x002616, "Rosemount Inc.", "ROSEMOUNT" },
	{ 0x002617, "OEM Worldwide", "OEM-WORLDWIDE" },
	{ 0x002618, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x002619, "FRC", "FRC" },
	{ 0x00261A, "Femtocomm System Technology Corp.", "FEMTOCOMM-SYSTEM" },
	{ 0x00261B, "LAUREL BANK MACHINES CO., LTD.", "LAUREL-BANK-MACHINES" },
	{ 0x00261C, "NEOVIA INC.", "NEOVIA" },
	{ 0x00261D, "COP SECURITY SYSTEM CORP.", "COP-SECURITY-SYSTEM" },
	{ 0x00261E, "QINGBANG ELEC(SZ) CO., LTD", "QINGBANG-ELEC-SZ" },
	{ 0x00261F, "SAE Magnetics (H.K.) Ltd.", "SAE-MAGNETICS-H-K" },
	{ 0x002620, "ISGUS GmbH", "ISGUS" },
	{ 0x002621, "InteliCloud Technology Inc.", "INTELICLOUD" },
	{ 0x002622, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x002623, "JRD Communication Inc", "JRD-COMMUNICATION" },
	{ 0x002624, "Thomson Inc.", "THOMSON" },
	{ 0x002625, "MediaSputnik", "MEDIASPUTNIK" },
	{ 0x002626, "Geophysical Survey Systems, Inc.", "GEOPHYSICAL-SURVEY" },
	{ 0x002627, "Truesell", "TRUESELL" },
	{ 0x002628, "companytec automação e controle ltda.", "COMPANYTEC-AUTOMAçãO-E-CONTROLE-LTDA" },
	{ 0x002629, "Juphoon System Software Inc.", "JUPHOON-SYSTEM-SOFTWARE" },
	{ 0x00262A, "Proxense, LLC", "PROXENSE-LLC" },
	{ 0x00262B, "Wongs Electronics Co. Ltd.", "WONGS-ELECTRONICS" },
	{ 0x00262C, "IKT Advanced Technologies s.r.o.", "IKT-ADVANCED-TECHNOLOGIES-S-R-O" },
	{ 0x00262D, "Wistron Corporation", "WISTRON" },
	{ 0x00262E, "Chengdu Jiuzhou Electronic Technology Inc", "CHENGDU-JIUZHOU" },
	{ 0x00262F, "HAMAMATSU TOA ELECTRONICS", "HAMAMATSU-TOA-ELECTRONICS" },
	{ 0x002630, "ACOREL S.A.S", "ACOREL-S" },
	{ 0x002631, "COMMTACT LTD", "COMMTACT" },
	{ 0x002632, "Instrumentation Technologies d.d.", "INSTRUMENTATION-TECHNOLOGIES-D-D" },
	{ 0x002633, "MIR - Medical International Research", "MIR---MEDICAL-RESEARCH" },
	{ 0x002634, "Infineta Systems, Inc", "INFINETA" },
	{ 0x002635, "Bluetechnix GmbH", "BLUETECHNIX" },
	{ 0x002636, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002637, "Samsung Electro-Mechanics", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x002638, "Xia Men Joyatech Co., Ltd.", "XIA-MEN-JOYATECH" },
	{ 0x002639, "T.M. Electronics, Inc.", "T-M-ELECTRONICS" },
	{ 0x00263A, "Digitec Systems", "DIGITEC" },
	{ 0x00263B, "Onbnetech", "ONBNETECH" },
	{ 0x00263C, "Bachmann Technology GmbH & Co. KG", "BACHMANN" },
	{ 0x00263D, "MIA Corporation", "MIA" },
	{ 0x00263E, "Trapeze Networks", "TRAPEZE-NETWORKS" },
	{ 0x00263F, "LIOS Technology GmbH", "LIOS" },
	{ 0x002640, "Baustem Broadband Technologies, Ltd.", "BAUSTEM-BROADBAND-TECHNOLOGIES" },
	{ 0x002641, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002642, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x002643, "Alps Electric Co., Ltd", "ALPS-ELECTRIC" },
	{ 0x002644, "Thomson Telecom Belgium", "THOMSON-TELECOM-BELGIUM" },
	{ 0x002645, "Circontrol S.A.", "CIRCONTROL" },
	{ 0x002646, "SHENYANG TONGFANG MULTIMEDIA TECHNOLOGY COMPANY LIMITED", "SHENYANG-TONGFANG-MULTIMEDIA-COMPANY" },
	{ 0x002647, "WFE TECHNOLOGY CORP.", "WFE" },
	{ 0x002648, "Emitech Corp.", "EMITECH" },
	{ 0x00264A, "Apple", "APPLE" },
	{ 0x00264C, "Shanghai DigiVision Technology Co., Ltd.", "SHANGHAI-DIGIVISION" },
	{ 0x00264D, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x00264E, "Rail & Road Protec GmbH", "RAIL-ROAD-PROTEC" },
	{ 0x00264F, "Krüger &Gothe GmbH", "KRüGER-GOTHE" },
	{ 0x002650, "2Wire", "2WIRE" },
	{ 0x002651, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002652, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002653, "DaySequerra Corporation", "DAYSEQUERRA" },
	{ 0x002654, "3Com Corporation", "3COM" },
	{ 0x002655, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x002656, "Sansonic Electronics USA", "SANSONIC-ELECTRONICS-USA" },
	{ 0x002657, "OOO NPP EKRA", "OOO-NPP-EKRA" },
	{ 0x002658, "T-Platforms (Cyprus) Limited", "T-PLATFORMS-CYPRUS" },
	{ 0x002659, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x00265A, "D-Link Corporation", "D-LINK" },
	{ 0x00265B, "Hitron Technologies. Inc", "HITRON-TECHNOLOGIES" },
	{ 0x00265C, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x00265D, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x00265E, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x00265F, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x002660, "Logiways", "LOGIWAYS" },
	{ 0x002661, "Irumtek Co., Ltd.", "IRUMTEK" },
	{ 0x002662, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x002663, "Shenzhen Huitaiwei Tech. Ltd, co.", "SHENZHEN-HUITAIWEI-TECH" },
	{ 0x002664, "Core System Japan", "CORE-SYSTEM-JAPAN" },
	{ 0x002665, "ProtectedLogic Corporation", "PROTECTEDLOGIC" },
	{ 0x002666, "EFM Networks", "EFM-NETWORKS" },
	{ 0x002667, "CARECOM CO.,LTD.", "CARECOM" },
	{ 0x002668, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x002669, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x00266A, "ESSENSIUM NV", "ESSENSIUM-NV" },
	{ 0x00266B, "SHINE UNION ENTERPRISE LIMITED", "SHINE-UNION" },
	{ 0x00266C, "Inventec", "INVENTEC" },
	{ 0x00266D, "MobileAccess Networks", "MOBILEACCESS-NETWORKS" },
	{ 0x00266E, "Nissho-denki Co.,LTD.", "NISSHO-DENKI" },
	{ 0x00266F, "Coordiwise Technology Corp.", "COORDIWISE" },
	{ 0x002670, "Cinch Connectors", "CINCH-CONNECTORS" },
	{ 0x002671, "AUTOVISION Co., Ltd", "AUTOVISION" },
	{ 0x002672, "AAMP of America", "AAMP-OF-AMERICA" },
	{ 0x002673, "RICOH COMPANY,LTD.", "RICOH-COMPANY" },
	{ 0x002674, "Electronic Solutions, Inc.", "SOLUTIONS" },
	{ 0x002675, "Aztech Electronics Pte Ltd", "AZTECH-ELECTRONICS-PTE" },
	{ 0x002676, "COMMidt AS", "COMMIDT-AS" },
	{ 0x002677, "DEIF A/S", "DEIF-A/S" },
	{ 0x002678, "Logic Instrument SA", "LOGIC-INSTRUMENT" },
	{ 0x002679, "Euphonic Technologies, Inc.", "EUPHONIC-TECHNOLOGIES" },
	{ 0x00267A, "wuhan hongxin telecommunication technologies co.,ltd", "WUHAN-HONGXIN-TELECOMMUNICATION-TECHNOLOGIES" },
	{ 0x00267B, "GSI Helmholtzzentrum für Schwerionenforschung GmbH", "GSI-HELMHOLTZZENTRUM-FüR-SCHWERIONENFORSCHUNG" },
	{ 0x00267C, "Metz-Werke GmbH & Co KG", "METZ-WERKE" },
	{ 0x00267D, "A-Max Technology Macao Commercial Offshore Company Limited", "A-MAX-MACAO-COMMERCIAL-OFFSHORE-COMPANY" },
	{ 0x00267E, "Parrot SA", "PARROT" },
	{ 0x00267F, "Zenterio AB", "ZENTERIO-AB" },
	{ 0x002680, "Lockie Innovation Pty Ltd", "LOCKIE-INNOVATION-PTY" },
	{ 0x002681, "Interspiro AB", "INTERSPIRO-AB" },
	{ 0x002682, "Gemtek Technology Co., Ltd.", "GEMTEK" },
	{ 0x002683, "Ajoho Enterprise Co., Ltd.", "AJOHO" },
	{ 0x002684, "KISAN SYSTEM", "KISAN-SYSTEM" },
	{ 0x002685, "Digital Innovation", "DIGITAL-INNOVATION" },
	{ 0x002686, "Quantenna Communcations, Inc.", "QUANTENNA-COMMUNCATIONS" },
	{ 0x002687, "Corega K.K", "COREGA-K-K" },
	{ 0x002688, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x002689, "General Dynamics Robotic Systems", "GENERAL-DYNAMICS-ROBOTIC" },
	{ 0x00268A, "Terrier SC Ltd", "TERRIER-SC" },
	{ 0x00268B, "Guangzhou Escene Computer Technology Limited", "GUANGZHOU-ESCENE" },
	{ 0x00268C, "StarLeaf Ltd.", "STARLEAF" },
	{ 0x00268D, "CellTel S.p.A.", "CELLTEL-S-P-A" },
	{ 0x00268E, "Alta Solutions, Inc.", "ALTA-SOLUTIONS" },
	{ 0x00268F, "MTA SpA", "MTA-SPA" },
	{ 0x002690, "I DO IT", "I-DO-IT" },
	{ 0x002691, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x002692, "Mitsubishi Electric Co.", "MITSUBISHI-ELECTRIC" },
	{ 0x002693, "QVidium Technologies, Inc.", "QVIDIUM-TECHNOLOGIES" },
	{ 0x002694, "Senscient Ltd", "SENSCIENT" },
	{ 0x002695, "ZT Group Int'l Inc", "ZT-GROUP" },
	{ 0x002696, "NOOLIX Co., Ltd", "NOOLIX" },
	{ 0x002697, "Cheetah Technologies, L.P.", "CHEETAH-TECHNOLOGIES-L-P" },
	{ 0x002698, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002699, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00269A, "Carina System Co., Ltd.", "CARINA-SYSTEM" },
	{ 0x00269B, "SOKRAT Ltd.", "SOKRAT" },
	{ 0x00269C, "ITUS JAPAN CO. LTD", "ITUS-JAPAN" },
	{ 0x00269D, "M2Mnet Co., Ltd.", "M2MNET" },
	{ 0x00269E, "Quanta Computer Inc", "QUANTA" },
	{ 0x00269F, "PRIVATE", "PRIVATE" },
	{ 0x0026A0, "moblic", "MOBLIC" },
	{ 0x0026A1, "Megger", "MEGGER" },
	{ 0x0026A2, "Instrumentation Technology Systems", "INSTRUMENTATION" },
	{ 0x0026A3, "FQ Ingenieria Electronica S.A.", "FQ-INGENIERIA-ELECTRONICA" },
	{ 0x0026A4, "Novus Produtos Eletronicos Ltda", "NOVUS-PRODUTOS-ELETRONICOS-LTDA" },
	{ 0x0026A5, "MICROROBOT.CO.,LTD", "MICROROBOT" },
	{ 0x0026A6, "TRIXELL", "TRIXELL" },
	{ 0x0026A7, "CONNECT SRL", "CONNECT-SRL" },
	{ 0x0026A8, "DAEHAP HYPER-TECH", "DAEHAP-HYPER-TECH" },
	{ 0x0026A9, "Strong Technologies Pty Ltd", "STRONG-TECHNOLOGIES-PTY" },
	{ 0x0026AA, "Kenmec Mechanical Engineering Co., Ltd.", "KENMEC-MECHANICAL-ENGINEERING" },
	{ 0x0026AB, "SEIKO EPSON CORPORATION", "SEIKO-EPSON" },
	{ 0x0026AC, "Shanghai LUSTER Teraband photonic Co., Ltd.", "SHANGHAI-LUSTER-TERABAND-PHOTONIC" },
	{ 0x0026AD, "Arada Systems, Inc.", "ARADA" },
	{ 0x0026AE, "Wireless Measurement Ltd", "WIRELESS-MEASUREMENT" },
	{ 0x0026AF, "Duelco A/S", "DUELCO-A/S" },
	{ 0x0026B0, "Apple", "APPLE" },
	{ 0x0026B1, "Navis Auto Motive Systems, Inc.", "NAVIS-AUTO-MOTIVE" },
	{ 0x0026B2, "Setrix GmbH", "SETRIX" },
	{ 0x0026B3, "Thales Communications Inc", "THALES-COMMUNICATION" },
	{ 0x0026B4, "Ford Motor Company", "FORD-MOTOR-COMPANY" },
	{ 0x0026B5, "ICOMM Tele Ltd", "ICOMM-TELE" },
	{ 0x0026B6, "Askey Computer", "ASKEY" },
	{ 0x0026B7, "Kingston Technology Company, Inc.", "KINGSTON-COMPANY" },
	{ 0x0026B8, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x0026B9, "Dell Inc", "DELL" },
	{ 0x0026BA, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0026BB, "Apple", "APPLE" },
	{ 0x0026BC, "General Jack Technology Ltd.", "GENERAL-JACK" },
	{ 0x0026BD, "JTEC Card & Communication Co., Ltd.", "JTEC-CARD-COMMUNICATION" },
	{ 0x0026BE, "Schoonderbeek Elektronica Systemen B.V.", "SCHOONDERBEEK-ELEKTRONICA-SYSTEMEN" },
	{ 0x0026BF, "ShenZhen Temobi Science&Tech Development Co.,Ltd", "SHENZHEN-TEMOBI-SCIENCE-TECH-DEVELOPMENT" },
	{ 0x0026C0, "EnergyHub", "ENERGYHUB" },
	{ 0x0026C1, "ARTRAY CO., LTD.", "ARTRAY" },
	{ 0x0026C2, "SCDI Co. LTD", "SCDI" },
	{ 0x0026C3, "Insightek Corp.", "INSIGHTEK" },
	{ 0x0026C4, "Cadmos microsystems S.r.l.", "CADMOS-MICROSYSTEMS-S-R-L" },
	{ 0x0026C5, "Guangdong Gosun Telecommunications Co.,Ltd", "GUANGDONG-GOSUN-TELECOMMUNICATIONS" },
	{ 0x0026C6, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0026C7, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0026C8, "System Sensor", "SYSTEM-SENSOR" },
	{ 0x0026C9, "Proventix Systems, Inc.", "PROVENTIX" },
	{ 0x0026CA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0026CB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0026CC, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x0026CD, "PurpleComm, Inc.", "PURPLECOMM" },
	{ 0x0026CE, "Kozumi USA Corp.", "KOZUMI-USA" },
	{ 0x0026CF, "DEKA R&D", "DEKA-R-D" },
	{ 0x0026D0, "Semihalf", "SEMIHALF" },
	{ 0x0026D1, "S Squared Innovations Inc.", "S-SQUARED-INNOVATIONS" },
	{ 0x0026D2, "Pcube Systems, Inc.", "PCUBE" },
	{ 0x0026D3, "Zeno Information System", "ZENO-INFORMATION-SYSTEM" },
	{ 0x0026D4, "IRCA SpA", "IRCA-SPA" },
	{ 0x0026D5, "Ory Solucoes em Comercio de Informatica Ltda.", "ORY-SOLUCOES-EM-COMERCIO-DE-INFORMATICA-LTDA" },
	{ 0x0026D6, "Ningbo Andy Optoelectronic Co., Ltd.", "NINGBO-ANDY-OPTOELECTRONIC" },
	{ 0x0026D7, "KM Electornic Technology Co., Ltd.", "KM-ELECTORNIC" },
	{ 0x0026D8, "Magic Point Inc.", "MAGIC-POINT" },
	{ 0x0026D9, "Pace plc", "PACE-PLC" },
	{ 0x0026DA, "Universal Media Corporation /Slovakia/ s.r.o.", "UNIVERSAL-MEDIA-/SLOVAKIA/-S-R-O" },
	{ 0x0026DB, "Ionics EMS Inc.", "IONICS-EMS" },
	{ 0x0026DC, "Optical Systems Design", "OPTICAL-DESIGN" },
	{ 0x0026DD, "Fival Science & Technology Co.,Ltd.", "FIVAL-SCIENCE" },
	{ 0x0026DE, "FDI MATELEC", "FDI-MATELEC" },
	{ 0x0026DF, "TaiDoc Technology Corp.", "TAIDOC" },
	{ 0x0026E0, "ASITEQ", "ASITEQ" },
	{ 0x0026E1, "Stanford University, OpenFlow Group", "STANFORD-UNIVERSITY-OPENFLOW-GROUP" },
	{ 0x0026E2, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x0026E3, "DTI", "DTI" },
	{ 0x0026E4, "CANAL OVERSEAS", "CANAL-OVERSEAS" },
	{ 0x0026E5, "AEG Power Solutions", "AEG-POWER-SOLUTIONS" },
	{ 0x0026E6, "Visionhitech Co., Ltd.", "VISIONHITECH" },
	{ 0x0026E7, "Shanghai ONLAN Communication Tech. Co., Ltd.", "SHANGHAI-ONLAN-COMMUNICATION-TECH" },
	{ 0x0026E8, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x0026E9, "SP Corp", "SP" },
	{ 0x0026EA, "Cheerchip Electronic Technology (ShangHai) Co., Ltd.", "CHEERCHIP-SHANGHAI" },
	{ 0x0026EB, "Advanced Spectrum Technology Co., Ltd.", "ADVANCED-SPECTRUM" },
	{ 0x0026EC, "Legrand Home Systems, Inc", "LEGRAND-HOME" },
	{ 0x0026ED, "zte corporation", "ZTE" },
	{ 0x0026EE, "TKM GmbH", "TKM" },
	{ 0x0026EF, "Technology Advancement Group, Inc.", "ADVANCEMENT-GROUP" },
	{ 0x0026F0, "cTrixs International GmbH.", "CTRIXS" },
	{ 0x0026F1, "ProCurve Networking by HP", "PROCURVE-NETWORKING-BY-HP" },
	{ 0x0026F2, "Netgear", "NETGEAR" },
	{ 0x0026F3, "SMC Networks", "SMC-NETWORKS" },
	{ 0x0026F4, "Nesslab", "NESSLAB" },
	{ 0x0026F5, "XRPLUS Inc.", "XRPLUS" },
	{ 0x0026F6, "Military Communication Institute", "MILITARY-COMMUNICATION-INSTITUTE" },
	{ 0x0026F7, "Infosys Technologies Ltd.", "INFOSYS-TECHNOLOGIES" },
	{ 0x0026F8, "Golden Highway Industry Development Co., Ltd.", "GOLDEN-HIGHWAY-INDUSTRY-DEVELOPMENT" },
	{ 0x0026F9, "S.E.M. srl", "S-E-M-SRL" },
	{ 0x0026FA, "BandRich Inc.", "BANDRICH" },
	{ 0x0026FB, "AirDio Wireless, Inc.", "AIRDIO-WIRELESS" },
	{ 0x0026FC, "AcSiP Technology Corp.", "ACSIP" },
	{ 0x0026FD, "Interactive Intelligence", "INTERACTIVE-INTELLIGENCE" },
	{ 0x0026FE, "MKD Technology Inc.", "MKD" },
	{ 0x0026FF, "Research In Motion", "RESEARCH-IN-MOTION" },
	{ 0x002700, "Shenzhen Siglent Technology Co., Ltd.", "SHENZHEN-SIGLENT" },
	{ 0x002701, "INCOstartec GmbH", "INCOSTARTEC" },
	{ 0x002702, "SolarEdge Technologies", "SOLAREDGE-TECHNOLOGIES" },
	{ 0x002703, "Testech Electronics Pte Ltd", "TESTECH-ELECTRONICS-PTE" },
	{ 0x002704, "Accelerated Concepts, Inc", "ACCELERATED-CONCEPTS" },
	{ 0x002705, "Sectronic", "SECTRONIC" },
	{ 0x002706, "YOISYS", "YOISYS" },
	{ 0x002707, "Lift Complex DS, JSC", "LIFT-COMPLEX-DS-JSC" },
	{ 0x002708, "Nordiag ASA", "NORDIAG-ASA" },
	{ 0x002709, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x00270A, "IEE S.A.", "IEE" },
	{ 0x00270B, "Adura Technologies", "ADURA-TECHNOLOGIES" },
	{ 0x00270C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00270D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00270E, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x00270F, "Envisionnovation Inc", "ENVISIONNOVATION" },
	{ 0x002710, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x002711, "LanPro Inc", "LANPRO" },
	{ 0x002712, "MaxVision LLC", "MAXVISION-LLC" },
	{ 0x002713, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x002714, "Grainmustards, Co,ltd.", "GRAINMUSTARDS" },
	{ 0x002715, "Rebound Telecom. Co., Ltd", "REBOUND-TELECOM" },
	{ 0x002716, "Adachi-Syokai Co., Ltd.", "ADACHI-SYOKAI" },
	{ 0x002717, "CE Digital(Zhenjiang)Co.,Ltd", "CE-DIGITAL-ZHENJIANG" },
	{ 0x002718, "Suzhou NEW SEAUNION Video Technology Co.,Ltd", "SUZHOU-NEW-SEAUNION-VIDEO" },
	{ 0x002719, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x00271A, "Geenovo Technology Ltd.", "GEENOVO" },
	{ 0x00271B, "Alec Sicherheitssysteme GmbH", "ALEC-SICHERHEITSSYSTEME" },
	{ 0x00271C, "MERCURY CORPORATION", "MERCURY" },
	{ 0x00271D, "Comba Telecom Systems (China) Ltd.", "COMBA-TELECOM-CHINA" },
	{ 0x00271E, "Xagyl Communications", "XAGYL-COMMUNICATION" },
	{ 0x00271F, "MIPRO Electronics Co., Ltd", "MIPRO-ELECTRONICS" },
	{ 0x002720, "NEW-SOL COM", "NEW-SOL-COM" },
	{ 0x002721, "Shenzhen Baoan Fenda Industrial Co., Ltd", "SHENZHEN-BAOAN-FENDA-INDUSTRIAL" },
	{ 0x002722, "Ubiquiti Networks", "UBIQUITI-NETWORKS" },
	{ 0x0027F8, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x002A6A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x002AAF, "LARsys-Automation GmbH", "LARSYS-AUTOMATION" },
	{ 0x002D76, "TITECH GmbH", "TITECH" },
	{ 0x003000, "ALLWELL TECHNOLOGY CORP.", "ALLWELL" },
	{ 0x003001, "SMP", "SMP" },
	{ 0x003002, "Expand Networks", "EXPAND-NETWORKS" },
	{ 0x003003, "Phasys Ltd.", "PHASYS" },
	{ 0x003004, "LEADTEK RESEARCH INC.", "LEADTEK-RESEARCH" },
	{ 0x003005, "Fujitsu Siemens Computers", "FUJITSU-SIEMENS-COMPUTERS" },
	{ 0x003006, "SUPERPOWER COMPUTER", "SUPERPOWER" },
	{ 0x003007, "OPTI, INC.", "OPTI" },
	{ 0x003008, "AVIO DIGITAL, INC.", "AVIO-DIGITAL" },
	{ 0x003009, "Tachion Networks, Inc.", "TACHION-NETWORKS" },
	{ 0x00300A, "AZTECH Electronics Pte Ltd", "AZTECH-ELECTRONICS-PTE" },
	{ 0x00300B, "mPHASE Technologies, Inc.", "MPHASE-TECHNOLOGIES" },
	{ 0x00300C, "CONGRUENCY, LTD.", "CONGRUENCY" },
	{ 0x00300D, "MMC Technology, Inc.", "MMC" },
	{ 0x00300E, "Klotz Digital AG", "KLOTZ-DIGITAL" },
	{ 0x00300F, "IMT - Information Management T", "IMT---INFORMATION-MANAGEMENT-T" },
	{ 0x003010, "VISIONETICS INTERNATIONAL", "VISIONETICS" },
	{ 0x003011, "HMS Industrial Networks", "HMS-INDUSTRIAL-NETWORKS" },
	{ 0x003012, "DIGITAL ENGINEERING LTD.", "DIGITAL-ENGINEERING" },
	{ 0x003013, "NEC Corporation", "NEC" },
	{ 0x003014, "DIVIO, INC.", "DIVIO" },
	{ 0x003015, "CP CLARE CORP.", "CP-CLARE" },
	{ 0x003016, "ISHIDA CO., LTD.", "ISHIDA" },
	{ 0x003017, "BlueArc UK Ltd", "BLUEARC-UK" },
	{ 0x003018, "Jetway Information Co., Ltd.", "JETWAY-INFORMATION" },
	{ 0x003019, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00301A, "SMARTBRIDGES PTE. LTD.", "SMARTBRIDGES-PTE" },
	{ 0x00301B, "SHUTTLE, INC.", "SHUTTLE" },
	{ 0x00301C, "ALTVATER AIRDATA SYSTEMS", "ALTVATER-AIRDATA" },
	{ 0x00301D, "SKYSTREAM, INC.", "SKYSTREAM" },
	{ 0x00301E, "3COM Europe Ltd.", "3COM" },
	{ 0x00301F, "OPTICAL NETWORKS, INC.", "OPTICAL-NETWORKS" },
	{ 0x003020, "TSI, Inc..", "TSI" },
	{ 0x003021, "HSING TECH. ENTERPRISE CO.,LTD", "HSING-TECH" },
	{ 0x003022, "Fong Kai Industrial Co., Ltd.", "FONG-KAI-INDUSTRIAL" },
	{ 0x003023, "COGENT COMPUTER SYSTEMS, INC.", "COGENT" },
	{ 0x003024, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003025, "CHECKOUT COMPUTER SYSTEMS, LTD", "CHECKOUT" },
	{ 0x003026, "HeiTel Digital Video GmbH", "HEITEL-DIGITAL-VIDEO" },
	{ 0x003027, "KERBANGO, INC.", "KERBANGO" },
	{ 0x003028, "FASE Saldatura srl", "FASE-SALDATURA-SRL" },
	{ 0x003029, "OPICOM", "OPICOM" },
	{ 0x00302A, "SOUTHERN INFORMATION", "SOUTHERN-INFORMATION" },
	{ 0x00302B, "INALP NETWORKS, INC.", "INALP-NETWORKS" },
	{ 0x00302C, "SYLANTRO SYSTEMS CORPORATION", "SYLANTRO" },
	{ 0x00302D, "QUANTUM BRIDGE COMMUNICATIONS", "QUANTUM-BRIDGE-COMMUNICATION" },
	{ 0x00302E, "Hoft & Wessel AG", "HOFT-WESSEL" },
	{ 0x00302F, "GE Aviation System", "GE-AVIATION-SYSTEM" },
	{ 0x003030, "HARMONIX CORPORATION", "HARMONIX" },
	{ 0x003031, "LIGHTWAVE COMMUNICATIONS, INC.", "LIGHTWAVE-COMMUNICATION" },
	{ 0x003032, "MagicRam, Inc.", "MAGICRAM" },
	{ 0x003033, "ORIENT TELECOM CO., LTD.", "ORIENT-TELECOM" },
	{ 0x003034, "SET ENGINEERING", "SET-ENGINEERING" },
	{ 0x003035, "Corning Incorporated", "CORNING-INCORPORATED" },
	{ 0x003036, "RMP ELEKTRONIKSYSTEME GMBH", "RMP-ELEKTRONIKSYSTEME" },
	{ 0x003037, "Packard Bell Nec Services", "PACKARD-BELL-NEC-SERVICES" },
	{ 0x003038, "XCP, INC.", "XCP" },
	{ 0x003039, "SOFTBOOK PRESS", "SOFTBOOK-PRESS" },
	{ 0x00303A, "MAATEL", "MAATEL" },
	{ 0x00303B, "PowerCom Technology", "POWERCOM" },
	{ 0x00303C, "ONNTO CORP.", "ONNTO" },
	{ 0x00303D, "IVA CORPORATION", "IVA" },
	{ 0x00303E, "Radcom Ltd.", "RADCOM" },
	{ 0x00303F, "TurboComm Tech Inc.", "TURBOCOMM-TECH" },
	{ 0x003040, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003041, "SAEJIN T & M CO., LTD.", "SAEJIN-T-M" },
	{ 0x003042, "DeTeWe-Deutsche Telephonwerke", "DETEWE-DEUTSCHE-TELEPHONWERKE" },
	{ 0x003043, "IDREAM TECHNOLOGIES, PTE. LTD.", "IDREAM-TECHNOLOGIES-PTE" },
	{ 0x003044, "CradlePoint, Inc", "CRADLEPOINT" },
	{ 0x003045, "Village Networks, Inc. (VNI)", "VILLAGE-NETWORKS-VNI" },
	{ 0x003046, "Controlled Electronic Manageme", "CONTROLLED-MANAGEME" },
	{ 0x003047, "NISSEI ELECTRIC CO., LTD.", "NISSEI-ELECTRIC" },
	{ 0x003048, "Supermicro Computer, Inc.", "SUPERMICRO" },
	{ 0x003049, "BRYANT TECHNOLOGY, LTD.", "BRYANT" },
	{ 0x00304A, "Fraunhofer IPMS", "FRAUNHOFER-IPMS" },
	{ 0x00304B, "ORBACOM SYSTEMS, INC.", "ORBACOM" },
	{ 0x00304C, "APPIAN COMMUNICATIONS, INC.", "APPIAN-COMMUNICATION" },
	{ 0x00304D, "ESI", "ESI" },
	{ 0x00304E, "BUSTEC PRODUCTION LTD.", "BUSTEC-PRODUCTION" },
	{ 0x00304F, "PLANET Technology Corporation", "PLANET" },
	{ 0x003050, "Versa Technology", "VERSA" },
	{ 0x003051, "ORBIT AVIONIC & COMMUNICATION", "ORBIT-AVIONIC-COMMUNICATION" },
	{ 0x003052, "ELASTIC NETWORKS", "ELASTIC-NETWORKS" },
	{ 0x003053, "Basler AG", "BASLER" },
	{ 0x003054, "CASTLENET TECHNOLOGY, INC.", "CASTLENET" },
	{ 0x003055, "Renesas Technology America, Inc.", "RENESAS-AMERICA" },
	{ 0x003056, "Beck IPC GmbH", "BECK-IPC" },
	{ 0x003057, "QTelNet, Inc.", "QTELNET" },
	{ 0x003058, "API MOTION", "API-MOTION" },
	{ 0x003059, "KONTRON COMPACT COMPUTERS AG", "KONTRON-COMPACT-COMPUTERS" },
	{ 0x00305A, "TELGEN CORPORATION", "TELGEN" },
	{ 0x00305B, "Toko Inc.", "TOKO" },
	{ 0x00305C, "SMAR Laboratories Corp.", "SMAR-LABORATORIES" },
	{ 0x00305D, "DIGITRA SYSTEMS, INC.", "DIGITRA" },
	{ 0x00305E, "Abelko Innovation", "ABELKO-INNOVATION" },
	{ 0x00305F, "Hasselblad", "HASSELBLAD" },
	{ 0x003060, "Powerfile, Inc.", "POWERFILE" },
	{ 0x003061, "MobyTEL", "MOBYTEL" },
	{ 0x003062, "IP Video Networks Inc", "IP-VIDEO-NETWORKS" },
	{ 0x003063, "SANTERA SYSTEMS, INC.", "SANTERA" },
	{ 0x003064, "ADLINK TECHNOLOGY, INC.", "ADLINK" },
	{ 0x003065, "Apple", "APPLE" },
	{ 0x003066, "RFM", "RFM" },
	{ 0x003067, "BIOSTAR MICROTECH INT'L CORP.", "BIOSTAR-MICROTECH" },
	{ 0x003068, "CYBERNETICS TECH. CO., LTD.", "CYBERNETICS-TECH" },
	{ 0x003069, "IMPACCT TECHNOLOGY CORP.", "IMPACCT" },
	{ 0x00306A, "PENTA MEDIA CO., LTD.", "PENTA-MEDIA" },
	{ 0x00306B, "CMOS SYSTEMS, INC.", "CMOS" },
	{ 0x00306C, "Hitex Holding GmbH", "HITEX-HOLDING" },
	{ 0x00306D, "LUCENT TECHNOLOGIES", "LUCENT-TECHNOLOGIES" },
	{ 0x00306E, "HEWLETT PACKARD", "HEWLETT-PACKARD" },
	{ 0x00306F, "SEYEON TECH. CO., LTD.", "SEYEON-TECH" },
	{ 0x003070, "1Net Corporation", "1NET" },
	{ 0x003071, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003072, "Intellibyte Inc.", "INTELLIBYTE" },
	{ 0x003073, "International Microsystems, In", "MICROSYSTEMS-IN" },
	{ 0x003074, "EQUIINET LTD.", "EQUIINET" },
	{ 0x003075, "ADTECH", "ADTECH" },
	{ 0x003076, "Akamba Corporation", "AKAMBA" },
	{ 0x003077, "ONPREM NETWORKS", "ONPREM-NETWORKS" },
	{ 0x003078, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003079, "CQOS, INC.", "CQOS" },
	{ 0x00307A, "Advanced Technology & Systems", "ADVANCED" },
	{ 0x00307B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00307C, "ADID SA", "ADID" },
	{ 0x00307D, "GRE AMERICA, INC.", "GRE-AMERICA" },
	{ 0x00307E, "Redflex Communication Systems", "REDFLEX-COMMUNICATION" },
	{ 0x00307F, "IRLAN LTD.", "IRLAN" },
	{ 0x003080, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003081, "ALTOS C&C", "ALTOS-C-C" },
	{ 0x003082, "TAIHAN ELECTRIC WIRE CO., LTD.", "TAIHAN-ELECTRIC-WIRE" },
	{ 0x003083, "Ivron Systems", "IVRON" },
	{ 0x003084, "ALLIED TELESYN INTERNAIONAL", "ALLIED-TELESYN" },
	{ 0x003085, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003086, "Transistor Devices, Inc.", "TRANSISTOR-DEVICES" },
	{ 0x003087, "VEGA GRIESHABER KG", "VEGA-GRIESHABER" },
	{ 0x003088, "Ericsson", "ERICSSON" },
	{ 0x003089, "Spectrapoint Wireless, LLC", "SPECTRAPOINT-WIRELESS-LLC" },
	{ 0x00308A, "NICOTRA SISTEMI S.P.A", "NICOTRA-SISTEMI-S-P-A" },
	{ 0x00308B, "Brix Networks", "BRIX-NETWORKS" },
	{ 0x00308C, "Quantum Corporation", "QUANTUM" },
	{ 0x00308D, "Pinnacle Systems, Inc.", "PINNACLE" },
	{ 0x00308E, "CROSS MATCH TECHNOLOGIES, INC.", "CROSS-MATCH-TECHNOLOGIES" },
	{ 0x00308F, "MICRILOR, Inc.", "MICRILOR" },
	{ 0x003090, "CYRA TECHNOLOGIES, INC.", "CYRA-TECHNOLOGIES" },
	{ 0x003091, "TAIWAN FIRST LINE ELEC. CORP.", "TAIWAN-FIRST-LINE-ELEC" },
	{ 0x003092, "ModuNORM GmbH", "MODUNORM" },
	{ 0x003093, "Sonnet Technologies, Inc", "SONNET-TECHNOLOGIES" },
	{ 0x003094, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003095, "Procomp Informatics, Ltd.", "PROCOMP-INFORMATICS" },
	{ 0x003096, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003097, "AB Regin", "AB-REGIN" },
	{ 0x003098, "Global Converging Technologies", "GLOBAL-CONVERGING-TECHNOLOGIES" },
	{ 0x003099, "BOENIG UND KALLENBACH OHG", "BOENIG-UND-KALLENBACH-OHG" },
	{ 0x00309A, "ASTRO TERRA CORP.", "ASTRO-TERRA" },
	{ 0x00309B, "Smartware", "SMARTWARE" },
	{ 0x00309C, "Timing Applications, Inc.", "TIMING-APPLICATIONS" },
	{ 0x00309D, "Nimble Microsystems, Inc.", "NIMBLE-MICROSYSTEMS" },
	{ 0x00309E, "WORKBIT CORPORATION.", "WORKBIT" },
	{ 0x00309F, "AMBER NETWORKS", "AMBER-NETWORKS" },
	{ 0x0030A0, "TYCO SUBMARINE SYSTEMS, LTD.", "TYCO-SUBMARINE" },
	{ 0x0030A1, "WEBGATE Inc.", "WEBGATE" },
	{ 0x0030A2, "Lightner Engineering", "LIGHTNER-ENGINEERING" },
	{ 0x0030A3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0030A4, "Woodwind Communications System", "WOODWIND-COMMUNICATION-SYSTEM" },
	{ 0x0030A5, "ACTIVE POWER", "ACTIVE-POWER" },
	{ 0x0030A6, "VIANET TECHNOLOGIES, LTD.", "VIANET-TECHNOLOGIES" },
	{ 0x0030A7, "SCHWEITZER ENGINEERING", "SCHWEITZER-ENGINEERING" },
	{ 0x0030A8, "OL'E COMMUNICATIONS, INC.", "OL-E-COMMUNICATION" },
	{ 0x0030A9, "Netiverse, Inc.", "NETIVERSE" },
	{ 0x0030AA, "AXUS MICROSYSTEMS, INC.", "AXUS-MICROSYSTEMS" },
	{ 0x0030AB, "DELTA NETWORKS, INC.", "DELTA-NETWORKS" },
	{ 0x0030AC, "Systeme Lauer GmbH & Co., Ltd.", "SYSTEME-LAUER" },
	{ 0x0030AD, "SHANGHAI COMMUNICATION", "SHANGHAI-COMMUNICATION" },
	{ 0x0030AE, "Times N System, Inc.", "TIMES-N-SYSTEM" },
	{ 0x0030AF, "Honeywell GmbH", "HONEYWELL" },
	{ 0x0030B0, "Convergenet Technologies", "CONVERGENET-TECHNOLOGIES" },
	{ 0x0030B1, "TrunkNet", "TRUNKNET" },
	{ 0x0030B2, "L-3 Sonoma EO", "L-3-SONOMA-EO" },
	{ 0x0030B3, "San Valley Systems, Inc.", "SAN-VALLEY" },
	{ 0x0030B4, "INTERSIL CORP.", "INTERSIL" },
	{ 0x0030B5, "Tadiran Microwave Networks", "TADIRAN-MICROWAVE-NETWORKS" },
	{ 0x0030B6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0030B7, "Teletrol Systems, Inc.", "TELETROL" },
	{ 0x0030B8, "RiverDelta Networks", "RIVERDELTA-NETWORKS" },
	{ 0x0030B9, "ECTEL", "ECTEL" },
	{ 0x0030BA, "AC&T SYSTEM CO., LTD.", "AC-T-SYSTEM" },
	{ 0x0030BB, "CacheFlow, Inc.", "CACHEFLOW" },
	{ 0x0030BC, "Optronic AG", "OPTRONIC" },
	{ 0x0030BD, "BELKIN COMPONENTS", "BELKIN-COMPONENTS" },
	{ 0x0030BE, "City-Net Technology, Inc.", "CITY-NET" },
	{ 0x0030BF, "MULTIDATA GMBH", "MULTIDATA" },
	{ 0x0030C0, "Lara Technology, Inc.", "LARA" },
	{ 0x0030C1, "HEWLETT-PACKARD", "HEWLETT-PACKARD" },
	{ 0x0030C2, "COMONE", "COMONE" },
	{ 0x0030C3, "FLUECKIGER ELEKTRONIK AG", "FLUECKIGER-ELEKTRONIK" },
	{ 0x0030C4, "Canon Imaging Systems Inc.", "CANON-IMAGING" },
	{ 0x0030C5, "CADENCE DESIGN SYSTEMS", "CADENCE-DESIGN" },
	{ 0x0030C6, "CONTROL SOLUTIONS, INC.", "CONTROL-SOLUTIONS" },
	{ 0x0030C7, "Macromate Corp.", "MACROMATE" },
	{ 0x0030C8, "GAD LINE, LTD.", "GAD-LINE" },
	{ 0x0030C9, "LuxN, N", "LUXN-N" },
	{ 0x0030CA, "Discovery Com", "DISCOVERY-COM" },
	{ 0x0030CB, "OMNI FLOW COMPUTERS, INC.", "OMNI-FLOW-COMPUTERS" },
	{ 0x0030CC, "Tenor Networks, Inc.", "TENOR-NETWORKS" },
	{ 0x0030CD, "CONEXANT SYSTEMS, INC.", "CONEXANT" },
	{ 0x0030CE, "Zaffire", "ZAFFIRE" },
	{ 0x0030CF, "TWO TECHNOLOGIES, INC.", "TWO-TECHNOLOGIES" },
	{ 0x0030D0, "Tellabs", "TELLABS" },
	{ 0x0030D1, "INOVA CORPORATION", "INOVA" },
	{ 0x0030D2, "WIN TECHNOLOGIES, CO., LTD.", "WIN-TECHNOLOGIES" },
	{ 0x0030D3, "Agilent Technologies", "AGILENT-TECHNOLOGIES" },
	{ 0x0030D4, "AAE Systems, Inc.", "AAE" },
	{ 0x0030D5, "DResearch GmbH", "DRESEARCH" },
	{ 0x0030D6, "MSC VERTRIEBS GMBH", "MSC-VERTRIEBS" },
	{ 0x0030D7, "Innovative Systems, L.L.C.", "INNOVATIVE-L-L-C" },
	{ 0x0030D8, "SITEK", "SITEK" },
	{ 0x0030D9, "DATACORE SOFTWARE CORP.", "DATACORE-SOFTWARE" },
	{ 0x0030DA, "COMTREND CO.", "COMTREND" },
	{ 0x0030DB, "Mindready Solutions, Inc.", "MINDREADY-SOLUTIONS" },
	{ 0x0030DC, "RIGHTECH CORPORATION", "RIGHTECH" },
	{ 0x0030DD, "INDIGITA CORPORATION", "INDIGITA" },
	{ 0x0030DE, "WAGO Kontakttechnik GmbH", "WAGO-KONTAKTTECHNIK" },
	{ 0x0030DF, "KB/TEL TELECOMUNICACIONES", "KB/TEL-TELECOMUNICACIONES" },
	{ 0x0030E0, "OXFORD SEMICONDUCTOR LTD.", "OXFORD-SEMICONDUCTOR" },
	{ 0x0030E1, "Network Equipment Technologies, Inc.", "NETWORK-EQUIPMENT-TECHNOLOGIES" },
	{ 0x0030E2, "GARNET SYSTEMS CO., LTD.", "GARNET" },
	{ 0x0030E3, "SEDONA NETWORKS CORP.", "SEDONA-NETWORKS" },
	{ 0x0030E4, "CHIYODA SYSTEM RIKEN", "CHIYODA-SYSTEM-RIKEN" },
	{ 0x0030E5, "Amper Datos S.A.", "AMPER-DATOS" },
	{ 0x0030E6, "Draeger Medical Systems, Inc.", "DRAEGER-MEDICAL" },
	{ 0x0030E7, "CNF MOBILE SOLUTIONS, INC.", "CNF-MOBILE-SOLUTIONS" },
	{ 0x0030E8, "ENSIM CORP.", "ENSIM" },
	{ 0x0030E9, "GMA COMMUNICATION MANUFACT'G", "GMA-COMMUNICATION-MANUFACT-G" },
	{ 0x0030EA, "TeraForce Technology Corporation", "TERAFORCE" },
	{ 0x0030EB, "TURBONET COMMUNICATIONS, INC.", "TURBONET-COMMUNICATION" },
	{ 0x0030EC, "BORGARDT", "BORGARDT" },
	{ 0x0030ED, "Expert Magnetics Corp.", "EXPERT-MAGNETICS" },
	{ 0x0030EE, "DSG Technology, Inc.", "DSG" },
	{ 0x0030EF, "NEON TECHNOLOGY, INC.", "NEON" },
	{ 0x0030F0, "Uniform Industrial Corp.", "UNIFORM-INDUSTRIAL" },
	{ 0x0030F1, "Accton Technology Corp.", "ACCTON" },
	{ 0x0030F2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0030F3, "At Work Computers", "AT-WORK-COMPUTERS" },
	{ 0x0030F4, "STARDOT TECHNOLOGIES", "STARDOT-TECHNOLOGIES" },
	{ 0x0030F5, "Wild Lab. Ltd.", "WILD-LAB" },
	{ 0x0030F6, "SECURELOGIX CORPORATION", "SECURELOGIX" },
	{ 0x0030F7, "RAMIX INC.", "RAMIX" },
	{ 0x0030F8, "Dynapro Systems, Inc.", "DYNAPRO" },
	{ 0x0030F9, "Sollae Systems Co., Ltd.", "SOLLAE" },
	{ 0x0030FA, "TELICA, INC.", "TELICA" },
	{ 0x0030FB, "AZS Technology AG", "AZS" },
	{ 0x0030FC, "Terawave Communications, Inc.", "TERAWAVE-COMMUNICATION" },
	{ 0x0030FD, "INTEGRATED SYSTEMS DESIGN", "INTEGRATED-DESIGN" },
	{ 0x0030FE, "DSA GmbH", "DSA" },
	{ 0x0030FF, "DATAFAB SYSTEMS, INC.", "DATAFAB" },
	{ 0x00336C, "SynapSense Corporation", "SYNAPSENSE" },
	{ 0x0034F1, "Radicom Research, Inc.", "RADICOM-RESEARCH" },
	{ 0x003532, "Electro-Metrics Corporation", "ELECTRO-METRICS" },
	{ 0x003560, "Rosen Aviation", "ROSEN-AVIATION" },
	{ 0x0036F8, "Conti Temic microelectronic GmbH", "CONTI-TEMIC-MICROELECTRONIC" },
	{ 0x0036FE, "SuperVision", "SUPERVISION" },
	{ 0x00376D, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x0037B7, "SAGEMCOM", "SAGEMCOM" },
	{ 0x003A98, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003A99, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003A9A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003A9B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003A9C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x003A9D, "NEC Platforms, Ltd.", "NEC-PLATFORMS" },
	{ 0x003AAF, "BlueBit Ltd.", "BLUEBIT" },
	{ 0x003CC5, "WONWOO Engineering Co., Ltd", "WONWOO-ENGINEERING" },
	{ 0x003D41, "Hatteland Computer AS", "HATTELAND-AS" },
	{ 0x003EE1, "Apple", "APPLE" },
	{ 0x004000, "PCI COMPONENTES DA AMZONIA LTD", "PCI-COMPONENTES-DA-AMZONIA" },
	{ 0x004001, "Zero One Technology Co. Ltd.", "ZERO-ONE" },
	{ 0x004002, "PERLE SYSTEMS LIMITED", "PERLE" },
	{ 0x004003, "Emerson Process Management Power & Water Solutions, Inc.", "EMERSON-PROCESS-MANAGEMENT-POWER-WATER-SOLUTIONS" },
	{ 0x004004, "ICM CO. LTD.", "ICM" },
	{ 0x004005, "ANI COMMUNICATIONS INC.", "ANI-COMMUNICATION" },
	{ 0x004006, "SAMPO TECHNOLOGY CORPORATION", "SAMPO" },
	{ 0x004007, "TELMAT INFORMATIQUE", "TELMAT-INFORMATIQUE" },
	{ 0x004008, "A PLUS INFO CORPORATION", "A-PLUS-INFO" },
	{ 0x004009, "TACHIBANA TECTRON CO., LTD.", "TACHIBANA-TECTRON" },
	{ 0x00400A, "PIVOTAL TECHNOLOGIES, INC.", "PIVOTAL-TECHNOLOGIES" },
	{ 0x00400B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00400C, "GENERAL MICRO SYSTEMS, INC.", "GENERAL-MICRO" },
	{ 0x00400D, "LANNET DATA COMMUNICATIONS,LTD", "LANNET-DATA-COMMUNICATION" },
	{ 0x00400E, "MEMOTEC, INC.", "MEMOTEC" },
	{ 0x00400F, "DATACOM TECHNOLOGIES", "DATACOM-TECHNOLOGIES" },
	{ 0x004010, "SONIC SYSTEMS, INC.", "SONIC" },
	{ 0x004011, "ANDOVER CONTROLS CORPORATION", "ANDOVER-CONTROLS" },
	{ 0x004012, "WINDATA, INC.", "WINDATA" },
	{ 0x004013, "NTT DATA COMM. SYSTEMS CORP.", "NTT-DATA-COMM" },
	{ 0x004014, "COMSOFT GMBH", "COMSOFT" },
	{ 0x004015, "ASCOM INFRASYS AG", "ASCOM-INFRASYS" },
	{ 0x004016, "ADC - Global Connectivity Solutions Division", "ADC---GLOBAL-CONNECTIVITY-SOLUTIONS-DIVISION" },
	{ 0x004017, "Silex Technology America", "SILEX-AMERICA" },
	{ 0x004018, "ADOBE SYSTEMS, INC.", "ADOBE" },
	{ 0x004019, "AEON SYSTEMS, INC.", "AEON" },
	{ 0x00401A, "FUJI ELECTRIC CO., LTD.", "FUJI-ELECTRIC" },
	{ 0x00401B, "PRINTER SYSTEMS CORP.", "PRINTER" },
	{ 0x00401C, "AST RESEARCH, INC.", "AST-RESEARCH" },
	{ 0x00401D, "INVISIBLE SOFTWARE, INC.", "INVISIBLE-SOFTWARE" },
	{ 0x00401E, "ICC", "ICC" },
	{ 0x00401F, "COLORGRAPH LTD", "COLORGRAPH" },
	{ 0x004020, "TE Connectivity Ltd.", "TE-CONNECTIVITY" },
	{ 0x004021, "RASTER GRAPHICS", "RASTER-GRAPHICS" },
	{ 0x004022, "KLEVER COMPUTERS, INC.", "KLEVER-COMPUTERS" },
	{ 0x004023, "LOGIC CORPORATION", "LOGIC" },
	{ 0x004024, "COMPAC INC.", "COMPAC" },
	{ 0x004025, "MOLECULAR DYNAMICS", "MOLECULAR-DYNAMICS" },
	{ 0x004026, "Buffalo Inc.", "BUFFALO" },
	{ 0x004027, "SMC MASSACHUSETTS, INC.", "SMC-MASSACHUSETTS" },
	{ 0x004028, "NETCOMM LIMITED", "NETCOMM" },
	{ 0x004029, "COMPEX", "COMPEX" },
	{ 0x00402A, "CANOGA-PERKINS", "CANOGA-PERKINS" },
	{ 0x00402B, "TRIGEM COMPUTER, INC.", "TRIGEM" },
	{ 0x00402C, "ISIS DISTRIBUTED SYSTEMS, INC.", "ISIS-DISTRIBUTED" },
	{ 0x00402D, "HARRIS ADACOM CORPORATION", "HARRIS-ADACOM" },
	{ 0x00402E, "PRECISION SOFTWARE, INC.", "PRECISION-SOFTWARE" },
	{ 0x00402F, "XLNT DESIGNS INC.", "XLNT-DESIGNS" },
	{ 0x004030, "GK COMPUTER", "GK" },
	{ 0x004031, "KOKUSAI ELECTRIC CO., LTD", "KOKUSAI-ELECTRIC" },
	{ 0x004032, "DIGITAL COMMUNICATIONS", "DIGITAL-COMMUNICATION" },
	{ 0x004033, "ADDTRON TECHNOLOGY CO., LTD.", "ADDTRON" },
	{ 0x004034, "BUSTEK CORPORATION", "BUSTEK" },
	{ 0x004035, "OPCOM", "OPCOM" },
	{ 0x004036, "TRIBE COMPUTER WORKS, INC.", "TRIBE-WORKS" },
	{ 0x004037, "SEA-ILAN, INC.", "SEA-ILAN" },
	{ 0x004038, "TALENT ELECTRIC INCORPORATED", "TALENT-ELECTRIC-INCORPORATED" },
	{ 0x004039, "OPTEC DAIICHI DENKO CO., LTD.", "OPTEC-DAIICHI-DENKO" },
	{ 0x00403A, "IMPACT TECHNOLOGIES", "IMPACT-TECHNOLOGIES" },
	{ 0x00403B, "SYNERJET INTERNATIONAL CORP.", "SYNERJET" },
	{ 0x00403C, "FORKS, INC.", "FORKS" },
	{ 0x00403D, "Teradata Corporation", "TERADATA" },
	{ 0x00403E, "RASTER OPS CORPORATION", "RASTER-OPS" },
	{ 0x00403F, "SSANGYONG COMPUTER SYSTEMS", "SSANGYONG" },
	{ 0x004040, "RING ACCESS, INC.", "RING-ACCESS" },
	{ 0x004041, "FUJIKURA LTD.", "FUJIKURA" },
	{ 0x004042, "N.A.T. GMBH", "N-A-T" },
	{ 0x004043, "Nokia Siemens Networks GmbH & Co. KG.", "NOKIA-SIEMENS-NETWORKS" },
	{ 0x004044, "QNIX COMPUTER CO., LTD.", "QNIX" },
	{ 0x004045, "TWINHEAD CORPORATION", "TWINHEAD" },
	{ 0x004046, "UDC RESEARCH LIMITED", "UDC-RESEARCH" },
	{ 0x004047, "WIND RIVER SYSTEMS", "WIND-RIVER" },
	{ 0x004048, "SMD INFORMATICA S.A.", "SMD-INFORMATICA" },
	{ 0x004049, "Roche Diagnostics International Ltd.", "ROCHE-DIAGNOSTICS" },
	{ 0x00404A, "WEST AUSTRALIAN DEPARTMENT", "WEST-AUSTRALIAN-DEPARTMENT" },
	{ 0x00404B, "MAPLE COMPUTER SYSTEMS", "MAPLE" },
	{ 0x00404C, "HYPERTEC PTY LTD.", "HYPERTEC-PTY" },
	{ 0x00404D, "TELECOMMUNICATIONS TECHNIQUES", "TELECOMMUNICATIONS-TECHNIQUES" },
	{ 0x00404E, "FLUENT, INC.", "FLUENT" },
	{ 0x00404F, "SPACE & NAVAL WARFARE SYSTEMS", "SPACE-NAVAL-WARFARE" },
	{ 0x004050, "IRONICS, INCORPORATED", "IRONICS-INCORPORATED" },
	{ 0x004051, "GRACILIS, INC.", "GRACILIS" },
	{ 0x004052, "STAR TECHNOLOGIES, INC.", "STAR-TECHNOLOGIES" },
	{ 0x004053, "AMPRO COMPUTERS", "AMPRO-COMPUTERS" },
	{ 0x004054, "CONNECTION MACHINES SERVICES", "CONNECTION-MACHINES-SERVICES" },
	{ 0x004055, "METRONIX GMBH", "METRONIX" },
	{ 0x004056, "MCM JAPAN LTD.", "MCM-JAPAN" },
	{ 0x004057, "LOCKHEED - SANDERS", "LOCKHEED---SANDERS" },
	{ 0x004058, "KRONOS, INC.", "KRONOS" },
	{ 0x004059, "YOSHIDA KOGYO K. K.", "YOSHIDA-KOGYO-K-K" },
	{ 0x00405A, "GOLDSTAR INFORMATION & COMM.", "GOLDSTAR-INFORMATION-COMM" },
	{ 0x00405B, "FUNASSET LIMITED", "FUNASSET" },
	{ 0x00405C, "FUTURE SYSTEMS, INC.", "FUTURE" },
	{ 0x00405D, "STAR-TEK, INC.", "STAR-TEK" },
	{ 0x00405E, "NORTH HILLS ISRAEL", "NORTH-HILLS-ISRAEL" },
	{ 0x00405F, "AFE COMPUTERS LTD.", "AFE-COMPUTERS" },
	{ 0x004060, "COMENDEC LTD", "COMENDEC" },
	{ 0x004061, "DATATECH ENTERPRISES CO., LTD.", "DATATECH-ENTERPRISES" },
	{ 0x004062, "E-SYSTEMS, INC./GARLAND DIV.", "E--/GARLAND-DIV" },
	{ 0x004063, "VIA TECHNOLOGIES, INC.", "VIA-TECHNOLOGIES" },
	{ 0x004064, "KLA INSTRUMENTS CORPORATION", "KLA-INSTRUMENTS" },
	{ 0x004065, "GTE SPACENET", "GTE-SPACENET" },
	{ 0x004066, "Hitachi Metals, Ltd.", "HITACHI-METALS" },
	{ 0x004067, "OMNIBYTE CORPORATION", "OMNIBYTE" },
	{ 0x004068, "EXTENDED SYSTEMS", "EXTENDED" },
	{ 0x004069, "LEMCOM SYSTEMS, INC.", "LEMCOM" },
	{ 0x00406A, "KENTEK INFORMATION SYSTEMS,INC", "KENTEK-INFORMATION" },
	{ 0x00406B, "SYSGEN", "SYSGEN" },
	{ 0x00406C, "COPERNIQUE", "COPERNIQUE" },
	{ 0x00406D, "LANCO, INC.", "LANCO" },
	{ 0x00406E, "COROLLARY, INC.", "COROLLARY" },
	{ 0x00406F, "SYNC RESEARCH INC.", "SYNC-RESEARCH" },
	{ 0x004070, "INTERWARE CO., LTD.", "INTERWARE" },
	{ 0x004071, "ATM COMPUTER GMBH", "ATM" },
	{ 0x004072, "Applied Innovation Inc.", "APPLIED-INNOVATION" },
	{ 0x004073, "BASS ASSOCIATES", "BASS-ASSOCIATES" },
	{ 0x004074, "CABLE AND WIRELESS", "CABLE-AND-WIRELESS" },
	{ 0x004075, "Tattile SRL", "TATTILE-SRL" },
	{ 0x004076, "Sun Conversion Technologies", "SUN-CONVERSION-TECHNOLOGIES" },
	{ 0x004077, "MAXTON TECHNOLOGY CORPORATION", "MAXTON" },
	{ 0x004078, "WEARNES AUTOMATION PTE LTD", "WEARNES-AUTOMATION-PTE" },
	{ 0x004079, "JUKO MANUFACTURE COMPANY, LTD.", "JUKO-MANUFACTURE-COMPANY" },
	{ 0x00407A, "SOCIETE D'EXPLOITATION DU CNIT", "SOCIETE-D-EXPLOITATION-DU-CNIT" },
	{ 0x00407B, "SCIENTIFIC ATLANTA", "SCIENTIFIC-ATLANTA" },
	{ 0x00407C, "QUME CORPORATION", "QUME" },
	{ 0x00407D, "EXTENSION TECHNOLOGY CORP.", "EXTENSION" },
	{ 0x00407E, "EVERGREEN SYSTEMS, INC.", "EVERGREEN" },
	{ 0x00407F, "FLIR Systems", "FLIR" },
	{ 0x004080, "ATHENIX CORPORATION", "ATHENIX" },
	{ 0x004081, "MANNESMANN SCANGRAPHIC GMBH", "MANNESMANN-SCANGRAPHIC" },
	{ 0x004082, "LABORATORY EQUIPMENT CORP.", "LABORATORY-EQUIPMENT" },
	{ 0x004083, "TDA INDUSTRIA DE PRODUTOS", "TDA-INDUSTRIA-DE-PRODUTOS" },
	{ 0x004084, "HONEYWELL ACS", "HONEYWELL-ACS" },
	{ 0x004085, "SAAB INSTRUMENTS AB", "SAAB-INSTRUMENTS-AB" },
	{ 0x004086, "MICHELS & KLEBERHOFF COMPUTER", "MICHELS-KLEBERHOFF" },
	{ 0x004087, "UBITREX CORPORATION", "UBITREX" },
	{ 0x004088, "MOBIUS TECHNOLOGIES, INC.", "MOBIUS-TECHNOLOGIES" },
	{ 0x004089, "MEIDENSHA CORPORATION", "MEIDENSHA" },
	{ 0x00408A, "TPS TELEPROCESSING SYS. GMBH", "TPS-TELEPROCESSING-SYS" },
	{ 0x00408B, "RAYLAN CORPORATION", "RAYLAN" },
	{ 0x00408C, "AXIS COMMUNICATIONS AB", "AXIS-COMMUNICATION-AB" },
	{ 0x00408D, "THE GOODYEAR TIRE & RUBBER CO.", "THE-GOODYEAR-TIRE-RUBBER" },
	{ 0x00408E, "Tattile SRL", "TATTILE-SRL" },
	{ 0x00408F, "WM-DATA MINFO AB", "WM-DATA-MINFO-AB" },
	{ 0x004090, "ANSEL COMMUNICATIONS", "ANSEL-COMMUNICATION" },
	{ 0x004091, "PROCOMP INDUSTRIA ELETRONICA", "PROCOMP-INDUSTRIA-ELETRONICA" },
	{ 0x004092, "ASP COMPUTER PRODUCTS, INC.", "ASP-PRODUCTS" },
	{ 0x004093, "PAXDATA NETWORKS LTD.", "PAXDATA-NETWORKS" },
	{ 0x004094, "SHOGRAPHICS, INC.", "SHOGRAPHICS" },
	{ 0x004095, "R.P.T. INTERGROUPS INT'L LTD.", "R-P-T-INTERGROUPS" },
	{ 0x004096, "Cisco Systems", "CISCO" },
	{ 0x004097, "DATEX DIVISION OF", "DATEX-DIVISION-OF" },
	{ 0x004098, "DRESSLER GMBH & CO.", "DRESSLER" },
	{ 0x004099, "NEWGEN SYSTEMS CORP.", "NEWGEN" },
	{ 0x00409A, "NETWORK EXPRESS, INC.", "NETWORK-EXPRESS" },
	{ 0x00409B, "HAL COMPUTER SYSTEMS INC.", "HAL" },
	{ 0x00409C, "TRANSWARE", "TRANSWARE" },
	{ 0x00409D, "DIGIBOARD, INC.", "DIGIBOARD" },
	{ 0x00409E, "CONCURRENT TECHNOLOGIES LTD.", "CONCURRENT-TECHNOLOGIES" },
	{ 0x00409F, "Telco Systems, Inc.", "TELCO" },
	{ 0x0040A0, "GOLDSTAR CO., LTD.", "GOLDSTAR" },
	{ 0x0040A1, "ERGO COMPUTING", "ERGO-COMPUTING" },
	{ 0x0040A2, "KINGSTAR TECHNOLOGY INC.", "KINGSTAR" },
	{ 0x0040A3, "MICROUNITY SYSTEMS ENGINEERING", "MICROUNITY-ENGINEERING" },
	{ 0x0040A4, "ROSE ELECTRONICS", "ROSE-ELECTRONICS" },
	{ 0x0040A5, "CLINICOMP INTL.", "CLINICOMP-INTL" },
	{ 0x0040A6, "Cray, Inc.", "CRAY" },
	{ 0x0040A7, "ITAUTEC PHILCO S.A.", "ITAUTEC-PHILCO" },
	{ 0x0040A8, "IMF INTERNATIONAL LTD.", "IMF" },
	{ 0x0040A9, "DATACOM INC.", "DATACOM" },
	{ 0x0040AA, "Metso Automation", "METSO-AUTOMATION" },
	{ 0x0040AB, "ROLAND DG CORPORATION", "ROLAND-DG" },
	{ 0x0040AC, "SUPER WORKSTATION, INC.", "SUPER-WORKSTATION" },
	{ 0x0040AD, "SMA REGELSYSTEME GMBH", "SMA-REGELSYSTEME" },
	{ 0x0040AE, "DELTA CONTROLS, INC.", "DELTA-CONTROLS" },
	{ 0x0040AF, "DIGITAL PRODUCTS, INC.", "DIGITAL-PRODUCTS" },
	{ 0x0040B0, "BYTEX CORPORATION, ENGINEERING", "BYTEX-ENGINEERING" },
	{ 0x0040B1, "CODONICS INC.", "CODONICS" },
	{ 0x0040B2, "SYSTEMFORSCHUNG", "SYSTEMFORSCHUNG" },
	{ 0x0040B3, "ParTech Inc.", "PARTECH" },
	{ 0x0040B4, "NEXTCOM K.K.", "NEXTCOM-K-K" },
	{ 0x0040B5, "VIDEO TECHNOLOGY COMPUTERS LTD", "VIDEO-COMPUTERS" },
	{ 0x0040B6, "COMPUTERM CORPORATION", "COMPUTERM" },
	{ 0x0040B7, "STEALTH COMPUTER SYSTEMS", "STEALTH" },
	{ 0x0040B8, "IDEA ASSOCIATES", "IDEA-ASSOCIATES" },
	{ 0x0040B9, "MACQ ELECTRONIQUE SA", "MACQ-ELECTRONIQUE" },
	{ 0x0040BA, "ALLIANT COMPUTER SYSTEMS CORP.", "ALLIANT" },
	{ 0x0040BB, "GOLDSTAR CABLE CO., LTD.", "GOLDSTAR-CABLE" },
	{ 0x0040BC, "ALGORITHMICS LTD.", "ALGORITHMICS" },
	{ 0x0040BD, "STARLIGHT NETWORKS, INC.", "STARLIGHT-NETWORKS" },
	{ 0x0040BE, "BOEING DEFENSE & SPACE", "BOEING-DEFENSE-SPACE" },
	{ 0x0040BF, "CHANNEL SYSTEMS INTERN'L INC.", "CHANNEL-INTERN-L" },
	{ 0x0040C0, "VISTA CONTROLS CORPORATION", "VISTA-CONTROLS" },
	{ 0x0040C1, "BIZERBA-WERKE WILHEIM KRAUT", "BIZERBA-WERKE-WILHEIM-KRAUT" },
	{ 0x0040C2, "APPLIED COMPUTING DEVICES", "APPLIED-COMPUTING-DEVICES" },
	{ 0x0040C3, "FISCHER AND PORTER CO.", "FISCHER-AND-PORTER" },
	{ 0x0040C4, "KINKEI SYSTEM CORPORATION", "KINKEI-SYSTEM" },
	{ 0x0040C5, "MICOM COMMUNICATIONS INC.", "MICOM-COMMUNICATION" },
	{ 0x0040C6, "FIBERNET RESEARCH, INC.", "FIBERNET-RESEARCH" },
	{ 0x0040C7, "RUBY TECH CORPORATION", "RUBY-TECH" },
	{ 0x0040C8, "MILAN TECHNOLOGY CORPORATION", "MILAN" },
	{ 0x0040C9, "NCUBE", "NCUBE" },
	{ 0x0040CA, "FIRST INTERNAT'L COMPUTER, INC", "FIRST-INTERNAT-L" },
	{ 0x0040CB, "LANWAN TECHNOLOGIES", "LANWAN-TECHNOLOGIES" },
	{ 0x0040CC, "SILCOM MANUF'G TECHNOLOGY INC.", "SILCOM-MANUF-G" },
	{ 0x0040CD, "TERA MICROSYSTEMS, INC.", "TERA-MICROSYSTEMS" },
	{ 0x0040CE, "NET-SOURCE, INC.", "NET-SOURCE" },
	{ 0x0040CF, "STRAWBERRY TREE, INC.", "STRAWBERRY-TREE" },
	{ 0x0040D0, "MITAC INTERNATIONAL CORP.", "MITAC" },
	{ 0x0040D1, "FUKUDA DENSHI CO., LTD.", "FUKUDA-DENSHI" },
	{ 0x0040D2, "PAGINE CORPORATION", "PAGINE" },
	{ 0x0040D3, "KIMPSION INTERNATIONAL CORP.", "KIMPSION" },
	{ 0x0040D4, "GAGE TALKER CORP.", "GAGE-TALKER" },
	{ 0x0040D5, "Sartorius Mechatronics T&H GmbH", "SARTORIUS-MECHATRONICS-T-H" },
	{ 0x0040D6, "LOCAMATION B.V.", "LOCAMATION" },
	{ 0x0040D7, "STUDIO GEN INC.", "STUDIO-GEN" },
	{ 0x0040D8, "OCEAN OFFICE AUTOMATION LTD.", "OCEAN-OFFICE-AUTOMATION" },
	{ 0x0040D9, "AMERICAN MEGATRENDS INC.", "AMERICAN-MEGATRENDS" },
	{ 0x0040DA, "TELSPEC LTD", "TELSPEC" },
	{ 0x0040DB, "ADVANCED TECHNICAL SOLUTIONS", "ADVANCED-TECHNICAL-SOLUTIONS" },
	{ 0x0040DC, "TRITEC ELECTRONIC GMBH", "TRITEC" },
	{ 0x0040DD, "HONG TECHNOLOGIES", "HONG-TECHNOLOGIES" },
	{ 0x0040DE, "Elsag Datamat spa", "ELSAG-DATAMAT-SPA" },
	{ 0x0040DF, "DIGALOG SYSTEMS, INC.", "DIGALOG" },
	{ 0x0040E0, "ATOMWIDE LTD.", "ATOMWIDE" },
	{ 0x0040E1, "MARNER INTERNATIONAL, INC.", "MARNER" },
	{ 0x0040E2, "MESA RIDGE TECHNOLOGIES, INC.", "MESA-RIDGE-TECHNOLOGIES" },
	{ 0x0040E3, "QUIN SYSTEMS LTD", "QUIN" },
	{ 0x0040E4, "E-M TECHNOLOGY, INC.", "E-M" },
	{ 0x0040E5, "SYBUS CORPORATION", "SYBUS" },
	{ 0x0040E6, "C.A.E.N.", "C-A-E-N" },
	{ 0x0040E7, "ARNOS INSTRUMENTS & COMPUTER", "ARNOS-INSTRUMENTS" },
	{ 0x0040E8, "CHARLES RIVER DATA SYSTEMS,INC", "CHARLES-RIVER-DATA" },
	{ 0x0040E9, "ACCORD SYSTEMS, INC.", "ACCORD" },
	{ 0x0040EA, "PLAIN TREE SYSTEMS INC", "PLAIN-TREE" },
	{ 0x0040EB, "MARTIN MARIETTA CORPORATION", "MARTIN-MARIETTA" },
	{ 0x0040EC, "MIKASA SYSTEM ENGINEERING", "MIKASA-SYSTEM-ENGINEERING" },
	{ 0x0040ED, "NETWORK CONTROLS INT'NATL INC.", "NETWORK-CONTROLS-INT-NATL" },
	{ 0x0040EE, "OPTIMEM", "OPTIMEM" },
	{ 0x0040EF, "HYPERCOM, INC.", "HYPERCOM" },
	{ 0x0040F0, "MicroBrain,Inc.", "MICROBRAIN" },
	{ 0x0040F1, "CHUO ELECTRONICS CO., LTD.", "CHUO-ELECTRONICS" },
	{ 0x0040F2, "JANICH & KLASS COMPUTERTECHNIK", "JANICH-KLASS-COMPUTERTECHNIK" },
	{ 0x0040F3, "NETCOR", "NETCOR" },
	{ 0x0040F4, "CAMEO COMMUNICATIONS, INC.", "CAMEO-COMMUNICATION" },
	{ 0x0040F5, "OEM ENGINES", "OEM-ENGINES" },
	{ 0x0040F6, "KATRON COMPUTERS INC.", "KATRON-COMPUTERS" },
	{ 0x0040F7, "Polaroid Corporation", "POLAROID" },
	{ 0x0040F8, "SYSTEMHAUS DISCOM", "SYSTEMHAUS-DISCOM" },
	{ 0x0040F9, "COMBINET", "COMBINET" },
	{ 0x0040FA, "MICROBOARDS, INC.", "MICROBOARDS" },
	{ 0x0040FB, "CASCADE COMMUNICATIONS CORP.", "CASCADE-COMMUNICATION" },
	{ 0x0040FC, "IBR COMPUTER TECHNIK GMBH", "IBR-TECHNIK" },
	{ 0x0040FD, "LXE", "LXE" },
	{ 0x0040FE, "SYMPLEX COMMUNICATIONS", "SYMPLEX-COMMUNICATION" },
	{ 0x0040FF, "TELEBIT CORPORATION", "TELEBIT" },
	{ 0x0041B4, "Wuxi Zhongxing Optoelectronics Technology Co.,Ltd.", "WUXI-ZHONGXING-OPTOELECTRONICS" },
	{ 0x004252, "RLX Technologies", "RLX-TECHNOLOGIES" },
	{ 0x0043FF, "KETRON S.R.L.", "KETRON-S-R-L" },
	{ 0x004501, "Versus Technology, Inc.", "VERSUS" },
	{ 0x00464B, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x004D32, "Andon Health Co.,Ltd.", "ANDON-HEALTH" },
	{ 0x005000, "NEXO COMMUNICATIONS, INC.", "NEXO-COMMUNICATION" },
	{ 0x005001, "YAMASHITA SYSTEMS CORP.", "YAMASHITA" },
	{ 0x005002, "OMNISEC AG", "OMNISEC" },
	{ 0x005003, "Xrite Inc", "XRITE" },
	{ 0x005004, "3COM CORPORATION", "3COM" },
	{ 0x005006, "TAC AB", "TAC-AB" },
	{ 0x005007, "SIEMENS TELECOMMUNICATION SYSTEMS LIMITED", "SIEMENS-TELECOMMUNICATION" },
	{ 0x005008, "TIVA MICROCOMPUTER CORP. (TMC)", "TIVA-MICROCOMPUTER-TMC" },
	{ 0x005009, "PHILIPS BROADBAND NETWORKS", "PHILIPS-BROADBAND-NETWORKS" },
	{ 0x00500A, "IRIS TECHNOLOGIES, INC.", "IRIS-TECHNOLOGIES" },
	{ 0x00500B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00500C, "e-Tek Labs, Inc.", "E-TEK-LABS" },
	{ 0x00500D, "SATORI ELECTORIC CO., LTD.", "SATORI-ELECTORIC" },
	{ 0x00500E, "CHROMATIS NETWORKS, INC.", "CHROMATIS-NETWORKS" },
	{ 0x00500F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x005010, "NovaNET Learning, Inc.", "NOVANET-LEARNING" },
	{ 0x005012, "CBL - GMBH", "CBL" },
	{ 0x005013, "Chaparral Network Storage", "CHAPARRAL-NETWORK-STORAGE" },
	{ 0x005014, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x005015, "BRIGHT STAR ENGINEERING", "BRIGHT-STAR-ENGINEERING" },
	{ 0x005016, "SST/WOODHEAD INDUSTRIES", "SST/WOODHEAD-INDUSTRIES" },
	{ 0x005017, "RSR S.R.L.", "RSR-S-R-L" },
	{ 0x005018, "AMIT, Inc.", "AMIT" },
	{ 0x005019, "SPRING TIDE NETWORKS, INC.", "SPRING-TIDE-NETWORKS" },
	{ 0x00501A, "IQinVision", "IQINVISION" },
	{ 0x00501B, "ABL CANADA, INC.", "ABL-CANADA" },
	{ 0x00501C, "JATOM SYSTEMS, INC.", "JATOM" },
	{ 0x00501E, "Miranda Technologies, Inc.", "MIRANDA-TECHNOLOGIES" },
	{ 0x00501F, "MRG SYSTEMS, LTD.", "MRG" },
	{ 0x005020, "MEDIASTAR CO., LTD.", "MEDIASTAR" },
	{ 0x005021, "EIS INTERNATIONAL, INC.", "EIS" },
	{ 0x005022, "ZONET TECHNOLOGY, INC.", "ZONET" },
	{ 0x005023, "PG DESIGN ELECTRONICS, INC.", "PG-DESIGN-ELECTRONICS" },
	{ 0x005024, "NAVIC SYSTEMS, INC.", "NAVIC" },
	{ 0x005026, "COSYSTEMS, INC.", "COSYSTEMS" },
	{ 0x005027, "GENICOM CORPORATION", "GENICOM" },
	{ 0x005028, "AVAL COMMUNICATIONS", "AVAL-COMMUNICATION" },
	{ 0x005029, "1394 PRINTER WORKING GROUP", "1394-PRINTER-WORKING-GROUP" },
	{ 0x00502A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00502B, "GENRAD LTD.", "GENRAD" },
	{ 0x00502C, "SOYO COMPUTER, INC.", "SOYO" },
	{ 0x00502D, "ACCEL, INC.", "ACCEL" },
	{ 0x00502E, "CAMBEX CORPORATION", "CAMBEX" },
	{ 0x00502F, "TollBridge Technologies, Inc.", "TOLLBRIDGE-TECHNOLOGIES" },
	{ 0x005030, "FUTURE PLUS SYSTEMS", "FUTURE-PLUS" },
	{ 0x005031, "AEROFLEX LABORATORIES, INC.", "AEROFLEX-LABORATORIES" },
	{ 0x005032, "PICAZO COMMUNICATIONS, INC.", "PICAZO-COMMUNICATION" },
	{ 0x005033, "MAYAN NETWORKS", "MAYAN-NETWORKS" },
	{ 0x005036, "NETCAM, LTD.", "NETCAM" },
	{ 0x005037, "KOGA ELECTRONICS CO.", "KOGA-ELECTRONICS" },
	{ 0x005038, "DAIN TELECOM CO., LTD.", "DAIN-TELECOM" },
	{ 0x005039, "MARINER NETWORKS", "MARINER-NETWORKS" },
	{ 0x00503A, "DATONG ELECTRONICS LTD.", "DATONG-ELECTRONICS" },
	{ 0x00503B, "MEDIAFIRE CORPORATION", "MEDIAFIRE" },
	{ 0x00503C, "TSINGHUA NOVEL ELECTRONICS", "TSINGHUA-NOVEL-ELECTRONICS" },
	{ 0x00503E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00503F, "ANCHOR GAMES", "ANCHOR-GAMES" },
	{ 0x005040, "Panasonic Electric Works Co., Ltd.", "PANASONIC-ELECTRIC-WORKS" },
	{ 0x005041, "Coretronic Corporation", "CORETRONIC" },
	{ 0x005042, "SCI MANUFACTURING SINGAPORE PTE, LTD.", "SCI-MANUFACTURING-SINGAPORE-PTE" },
	{ 0x005043, "MARVELL SEMICONDUCTOR, INC.", "MARVELL-SEMICONDUCTOR" },
	{ 0x005044, "ASACA CORPORATION", "ASACA" },
	{ 0x005045, "RIOWORKS SOLUTIONS, INC.", "RIOWORKS-SOLUTIONS" },
	{ 0x005046, "MENICX INTERNATIONAL CO., LTD.", "MENICX" },
	{ 0x005047, "PRIVATE", "PRIVATE" },
	{ 0x005048, "INFOLIBRIA", "INFOLIBRIA" },
	{ 0x005049, "Arbor Networks Inc", "ARBOR-NETWORKS" },
	{ 0x00504A, "ELTECO A.S.", "ELTECO-A-S" },
	{ 0x00504B, "BARCONET N.V.", "BARCONET-N-V" },
	{ 0x00504C, "Galil Motion Control", "GALIL-MOTION-CONTROL" },
	{ 0x00504D, "Tokyo Electron Device Limited", "TOKYO-ELECTRON-DEVICE" },
	{ 0x00504E, "SIERRA MONITOR CORP.", "SIERRA-MONITOR" },
	{ 0x00504F, "OLENCOM ELECTRONICS", "OLENCOM-ELECTRONICS" },
	{ 0x005050, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x005051, "IWATSU ELECTRIC CO., LTD.", "IWATSU-ELECTRIC" },
	{ 0x005052, "TIARA NETWORKS, INC.", "TIARA-NETWORKS" },
	{ 0x005053, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x005054, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x005055, "DOMS A/S", "DOMS-A/S" },
	{ 0x005056, "VMware, Inc. (possible VMware VM)", "VMWARE-VIRTUAL" },
	{ 0x005057, "BROADBAND ACCESS SYSTEMS", "BROADBAND-ACCESS" },
	{ 0x005058, "VegaStream Group Limted", "VEGASTREAM-GROUP-LIMTED" },
	{ 0x005059, "iBAHN", "IBAHN" },
	{ 0x00505A, "NETWORK ALCHEMY, INC.", "NETWORK-ALCHEMY" },
	{ 0x00505B, "KAWASAKI LSI U.S.A., INC.", "KAWASAKI-LSI-U" },
	{ 0x00505C, "TUNDO CORPORATION", "TUNDO" },
	{ 0x00505E, "DIGITEK MICROLOGIC S.A.", "DIGITEK-MICROLOGIC" },
	{ 0x00505F, "BRAND INNOVATORS", "BRAND-INNOVATORS" },
	{ 0x005060, "TANDBERG TELECOM AS", "TANDBERG-TELECOM-AS" },
	{ 0x005062, "KOUWELL ELECTRONICS CORP. **", "KOUWELL-ELECTRONICS-**" },
	{ 0x005063, "OY COMSEL SYSTEM AB", "OY-COMSEL-SYSTEM-AB" },
	{ 0x005064, "CAE ELECTRONICS", "CAE-ELECTRONICS" },
	{ 0x005065, "TDK-Lambda Corporation", "TDK-LAMBDA" },
	{ 0x005066, "AtecoM GmbH advanced telecomunication modules", "ATECOM-ADVANCED-TELECOMUNICATION-MODULES" },
	{ 0x005067, "AEROCOMM, INC.", "AEROCOMM" },
	{ 0x005068, "ELECTRONIC INDUSTRIES ASSOCIATION", "INDUSTRIES-ASSOCIATION" },
	{ 0x005069, "PixStream Incorporated", "PIXSTREAM-INCORPORATED" },
	{ 0x00506A, "EDEVA, INC.", "EDEVA" },
	{ 0x00506B, "SPX-ATEG", "SPX-ATEG" },
	{ 0x00506C, "Beijer Electronics Products AB", "BEIJER-ELECTRONICS-PRODUCTS-AB" },
	{ 0x00506D, "VIDEOJET SYSTEMS", "VIDEOJET" },
	{ 0x00506E, "CORDER ENGINEERING CORPORATION", "CORDER-ENGINEERING" },
	{ 0x00506F, "G-CONNECT", "G-CONNECT" },
	{ 0x005070, "CHAINTECH COMPUTER CO., LTD.", "CHAINTECH" },
	{ 0x005071, "AIWA CO., LTD.", "AIWA" },
	{ 0x005072, "CORVIS CORPORATION", "CORVIS" },
	{ 0x005073, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x005074, "ADVANCED HI-TECH CORP.", "ADVANCED-HI-TECH" },
	{ 0x005075, "KESTREL SOLUTIONS", "KESTREL-SOLUTIONS" },
	{ 0x005076, "IBM Corp", "IBM" },
	{ 0x005077, "PROLIFIC TECHNOLOGY, INC.", "PROLIFIC" },
	{ 0x005078, "MEGATON HOUSE, LTD.", "MEGATON-HOUSE" },
	{ 0x005079, "PRIVATE", "PRIVATE" },
	{ 0x00507A, "XPEED, INC.", "XPEED" },
	{ 0x00507B, "MERLOT COMMUNICATIONS", "MERLOT-COMMUNICATION" },
	{ 0x00507C, "VIDEOCON AG", "VIDEOCON" },
	{ 0x00507D, "IFP", "IFP" },
	{ 0x00507E, "NEWER TECHNOLOGY", "NEWER" },
	{ 0x00507F, "DrayTek Corp.", "DRAYTEK" },
	{ 0x005080, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x005081, "MURATA MACHINERY, LTD.", "MURATA-MACHINERY" },
	{ 0x005082, "FORESSON CORPORATION", "FORESSON" },
	{ 0x005083, "GILBARCO, INC.", "GILBARCO" },
	{ 0x005084, "ATL PRODUCTS", "ATL-PRODUCTS" },
	{ 0x005086, "TELKOM SA, LTD.", "TELKOM" },
	{ 0x005087, "TERASAKI ELECTRIC CO., LTD.", "TERASAKI-ELECTRIC" },
	{ 0x005088, "AMANO CORPORATION", "AMANO" },
	{ 0x005089, "SAFETY MANAGEMENT SYSTEMS", "SAFETY-MANAGEMENT" },
	{ 0x00508B, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x00508C, "RSI SYSTEMS", "RSI" },
	{ 0x00508D, "ABIT COMPUTER CORPORATION", "ABIT" },
	{ 0x00508E, "OPTIMATION, INC.", "OPTIMATION" },
	{ 0x00508F, "ASITA TECHNOLOGIES INT'L LTD.", "ASITA-TECHNOLOGIES" },
	{ 0x005090, "DCTRI", "DCTRI" },
	{ 0x005091, "NETACCESS, INC.", "NETACCESS" },
	{ 0x005092, "Rigaku Corporation Osaka Plant", "RIGAKU-OSAKA-PLANT" },
	{ 0x005093, "BOEING", "BOEING" },
	{ 0x005094, "PACE plc", "PACE-PLC" },
	{ 0x005095, "PERACOM NETWORKS", "PERACOM-NETWORKS" },
	{ 0x005096, "SALIX TECHNOLOGIES, INC.", "SALIX-TECHNOLOGIES" },
	{ 0x005097, "MMC-EMBEDDED COMPUTERTECHNIK GmbH", "MMC-EMBEDDED-COMPUTERTECHNIK" },
	{ 0x005098, "GLOBALOOP, LTD.", "GLOBALOOP" },
	{ 0x005099, "3COM EUROPE, LTD.", "3COM" },
	{ 0x00509A, "TAG ELECTRONIC SYSTEMS", "TAG" },
	{ 0x00509B, "SWITCHCORE AB", "SWITCHCORE-AB" },
	{ 0x00509C, "BETA RESEARCH", "BETA-RESEARCH" },
	{ 0x00509D, "THE INDUSTREE B.V.", "THE-INDUSTREE" },
	{ 0x00509E, "Les Technologies SoftAcoustik Inc.", "LES-TECHNOLOGIES-SOFTACOUSTIK" },
	{ 0x00509F, "HORIZON COMPUTER", "HORIZON" },
	{ 0x0050A0, "DELTA COMPUTER SYSTEMS, INC.", "DELTA" },
	{ 0x0050A1, "CARLO GAVAZZI, INC.", "CARLO-GAVAZZI" },
	{ 0x0050A2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0050A3, "TransMedia Communications, Inc.", "TRANSMEDIA-COMMUNICATION" },
	{ 0x0050A4, "IO TECH, INC.", "IO-TECH" },
	{ 0x0050A5, "CAPITOL BUSINESS SYSTEMS, LTD.", "CAPITOL-BUSINESS" },
	{ 0x0050A6, "OPTRONICS", "OPTRONICS" },
	{ 0x0050A7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0050A8, "OpenCon Systems, Inc.", "OPENCON" },
	{ 0x0050A9, "MOLDAT WIRELESS TECHNOLGIES", "MOLDAT-WIRELESS-TECHNOLGIES" },
	{ 0x0050AA, "KONICA MINOLTA HOLDINGS, INC.", "KONICA-MINOLTA-HOLDINGS" },
	{ 0x0050AB, "NALTEC, Inc.", "NALTEC" },
	{ 0x0050AC, "MAPLE COMPUTER CORPORATION", "MAPLE" },
	{ 0x0050AD, "CommUnique Wireless Corp.", "COMMUNIQUE-WIRELESS" },
	{ 0x0050AE, "FDK Co., Ltd", "FDK" },
	{ 0x0050AF, "INTERGON, INC.", "INTERGON" },
	{ 0x0050B0, "TECHNOLOGY ATLANTA CORPORATION", "ATLANTA" },
	{ 0x0050B1, "GIDDINGS & LEWIS", "GIDDINGS-LEWIS" },
	{ 0x0050B2, "BRODEL GmbH", "BRODEL" },
	{ 0x0050B3, "VOICEBOARD CORPORATION", "VOICEBOARD" },
	{ 0x0050B4, "SATCHWELL CONTROL SYSTEMS, LTD", "SATCHWELL-CONTROL" },
	{ 0x0050B5, "FICHET-BAUCHE", "FICHET-BAUCHE" },
	{ 0x0050B6, "GOOD WAY IND. CO., LTD.", "GOOD-WAY-IND" },
	{ 0x0050B7, "BOSER TECHNOLOGY CO., LTD.", "BOSER" },
	{ 0x0050B8, "INOVA COMPUTERS GMBH & CO. KG", "INOVA-COMPUTERS" },
	{ 0x0050B9, "XITRON TECHNOLOGIES, INC.", "XITRON-TECHNOLOGIES" },
	{ 0x0050BA, "D-LINK", "D-LINK" },
	{ 0x0050BB, "CMS TECHNOLOGIES", "CMS-TECHNOLOGIES" },
	{ 0x0050BC, "HAMMER STORAGE SOLUTIONS", "HAMMER-STORAGE-SOLUTIONS" },
	{ 0x0050BD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0050BE, "FAST MULTIMEDIA AG", "FAST-MULTIMEDIA" },
	{ 0x0050BF, "Metalligence Technology Corp.", "METALLIGENCE" },
	{ 0x0050C0, "GATAN, INC.", "GATAN" },
	{ 0x0050C1, "GEMFLEX NETWORKS, LTD.", "GEMFLEX-NETWORKS" },
	{ 0x0050C2, "IEEE REGISTRATION AUTHORITY - Please see IAB public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-IAB-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x0050C4, "IMD", "IMD" },
	{ 0x0050C5, "ADS Technologies, Inc", "ADS-TECHNOLOGIES" },
	{ 0x0050C6, "LOOP TELECOMMUNICATION INTERNATIONAL, INC.", "LOOP-TELECOMMUNICATION" },
	{ 0x0050C8, "Addonics Technologies, Inc.", "ADDONICS-TECHNOLOGIES" },
	{ 0x0050C9, "MASPRO DENKOH CORP.", "MASPRO-DENKOH" },
	{ 0x0050CA, "NET TO NET TECHNOLOGIES", "NET-TO-NET-TECHNOLOGIES" },
	{ 0x0050CB, "JETTER", "JETTER" },
	{ 0x0050CC, "XYRATEX", "XYRATEX" },
	{ 0x0050CD, "DIGIANSWER A/S", "DIGIANSWER-A/S" },
	{ 0x0050CE, "LG INTERNATIONAL CORP.", "LG" },
	{ 0x0050CF, "VANLINK COMMUNICATION TECHNOLOGY RESEARCH INSTITUTE", "VANLINK-COMMUNICATION-RESEARCH-INSTITUTE" },
	{ 0x0050D0, "MINERVA SYSTEMS", "MINERVA" },
	{ 0x0050D1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0050D2, "CMC Electronics Inc", "CMC-ELECTRONICS" },
	{ 0x0050D3, "DIGITAL AUDIO PROCESSING PTY. LTD.", "DIGITAL-AUDIO-PROCESSING-PTY" },
	{ 0x0050D4, "JOOHONG INFORMATION &", "JOOHONG-INFORMATION" },
	{ 0x0050D5, "AD SYSTEMS CORP.", "AD" },
	{ 0x0050D6, "ATLAS COPCO TOOLS AB", "ATLAS-COPCO-TOOLS-AB" },
	{ 0x0050D7, "TELSTRAT", "TELSTRAT" },
	{ 0x0050D8, "UNICORN COMPUTER CORP.", "UNICORN" },
	{ 0x0050D9, "ENGETRON-ENGENHARIA ELETRONICA IND. e COM. LTDA", "ENGETRON-ENGENHARIA-ELETRONICA-IND-E-COM-LTDA" },
	{ 0x0050DA, "3COM CORPORATION", "3COM" },
	{ 0x0050DB, "CONTEMPORARY CONTROL", "CONTEMPORARY-CONTROL" },
	{ 0x0050DC, "TAS TELEFONBAU A. SCHWABE GMBH & CO. KG", "TAS-TELEFONBAU-A-SCHWABE" },
	{ 0x0050DD, "SERRA SOLDADURA, S.A.", "SERRA-SOLDADURA" },
	{ 0x0050DE, "SIGNUM SYSTEMS CORP.", "SIGNUM" },
	{ 0x0050DF, "AirFiber, Inc.", "AIRFIBER" },
	{ 0x0050E1, "NS TECH ELECTRONICS SDN BHD", "NS-TECH-ELECTRONICS-SDN-BHD" },
	{ 0x0050E2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0050E3, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0050E4, "Apple", "APPLE" },
	{ 0x0050E6, "HAKUSAN CORPORATION", "HAKUSAN" },
	{ 0x0050E7, "PARADISE INNOVATIONS (ASIA)", "PARADISE-INNOVATIONS-ASIA" },
	{ 0x0050E8, "NOMADIX INC.", "NOMADIX" },
	{ 0x0050EA, "XEL COMMUNICATIONS, INC.", "XEL-COMMUNICATION" },
	{ 0x0050EB, "ALPHA-TOP CORPORATION", "ALPHA-TOP" },
	{ 0x0050EC, "OLICOM A/S", "OLICOM-A/S" },
	{ 0x0050ED, "ANDA NETWORKS", "ANDA-NETWORKS" },
	{ 0x0050EE, "TEK DIGITEL CORPORATION", "TEK-DIGITEL" },
	{ 0x0050EF, "SPE Systemhaus GmbH", "SPE-SYSTEMHAUS" },
	{ 0x0050F0, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0050F1, "Intel Corporation", "INTEL" },
	{ 0x0050F2, "MICROSOFT CORP.", "MICROSOFT" },
	{ 0x0050F3, "GLOBAL NET INFORMATION CO., Ltd.", "GLOBAL-NET-INFORMATION" },
	{ 0x0050F4, "SIGMATEK GMBH & CO. KG", "SIGMATEK" },
	{ 0x0050F6, "PAN-INTERNATIONAL INDUSTRIAL CORP.", "PAN--INDUSTRIAL" },
	{ 0x0050F7, "VENTURE MANUFACTURING (SINGAPORE) LTD.", "VENTURE-MANUFACTURING-SINGAPORE" },
	{ 0x0050F8, "ENTREGA TECHNOLOGIES, INC.", "ENTREGA-TECHNOLOGIES" },
	{ 0x0050F9, "Sensormatic Electronics LLC", "SENSORMATIC-ELECTRONICS-LLC" },
	{ 0x0050FA, "OXTEL, LTD.", "OXTEL" },
	{ 0x0050FB, "VSK ELECTRONICS", "VSK-ELECTRONICS" },
	{ 0x0050FC, "EDIMAX TECHNOLOGY CO., LTD.", "EDIMAX" },
	{ 0x0050FD, "VISIONCOMM CO., LTD.", "VISIONCOMM" },
	{ 0x0050FE, "PCTVnet ASA", "PCTVNET-ASA" },
	{ 0x0050FF, "HAKKO ELECTRONICS CO., LTD.", "HAKKO-ELECTRONICS" },
	{ 0x005218, "Wuxi Keboda Electron Co.Ltd", "WUXI-KEBODA-ELECTRON" },
	{ 0x0054AF, "Continental Automotive Systems Inc.", "CONTINENTAL-AUTOMOTIVE" },
	{ 0x005907, "LenovoEMC Products USA, LLC", "LENOVOEMC-PRODUCTS-USA-LLC" },
	{ 0x005A39, "SHENZHEN FAST TECHNOLOGIES CO., LTD.", "SHENZHEN-FAST-TECHNOLOGIES" },
	{ 0x005CB1, "Gospell DIGITAL TECHNOLOGY CO., LTD", "GOSPELL-DIGITAL" },
	{ 0x005D03, "Xilinx, Inc", "XILINX" },
	{ 0x006000, "XYCOM INC.", "XYCOM" },
	{ 0x006001, "InnoSys, Inc.", "INNOSYS" },
	{ 0x006002, "SCREEN SUBTITLING SYSTEMS, LTD", "SCREEN-SUBTITLING" },
	{ 0x006003, "TERAOKA WEIGH SYSTEM PTE, LTD.", "TERAOKA-WEIGH-SYSTEM-PTE" },
	{ 0x006004, "COMPUTADORES MODULARES SA", "COMPUTADORES-MODULARES" },
	{ 0x006005, "FEEDBACK DATA LTD.", "FEEDBACK-DATA" },
	{ 0x006006, "SOTEC CO., LTD", "SOTEC" },
	{ 0x006007, "ACRES GAMING, INC.", "ACRES-GAMING" },
	{ 0x006008, "3COM CORPORATION", "3COM" },
	{ 0x006009, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00600A, "SORD COMPUTER CORPORATION", "SORD" },
	{ 0x00600B, "LOGWARE GmbH", "LOGWARE" },
	{ 0x00600C, "Eurotech Inc.", "EUROTECH" },
	{ 0x00600D, "Digital Logic GmbH", "DIGITAL-LOGIC" },
	{ 0x00600E, "WAVENET INTERNATIONAL, INC.", "WAVENET" },
	{ 0x00600F, "WESTELL, INC.", "WESTELL" },
	{ 0x006010, "NETWORK MACHINES, INC.", "NETWORK-MACHINES" },
	{ 0x006011, "CRYSTAL SEMICONDUCTOR CORP.", "CRYSTAL-SEMICONDUCTOR" },
	{ 0x006012, "POWER COMPUTING CORPORATION", "POWER-COMPUTING" },
	{ 0x006013, "NETSTAL MASCHINEN AG", "NETSTAL-MASCHINEN" },
	{ 0x006014, "EDEC CO., LTD.", "EDEC" },
	{ 0x006015, "NET2NET CORPORATION", "NET2NET" },
	{ 0x006016, "CLARIION", "CLARIION" },
	{ 0x006017, "TOKIMEC INC.", "TOKIMEC" },
	{ 0x006018, "STELLAR ONE CORPORATION", "STELLAR-ONE" },
	{ 0x006019, "Roche Diagnostics", "ROCHE-DIAGNOSTICS" },
	{ 0x00601A, "KEITHLEY INSTRUMENTS", "KEITHLEY-INSTRUMENTS" },
	{ 0x00601B, "MESA ELECTRONICS", "MESA-ELECTRONICS" },
	{ 0x00601C, "TELXON CORPORATION", "TELXON" },
	{ 0x00601D, "LUCENT TECHNOLOGIES", "LUCENT-TECHNOLOGIES" },
	{ 0x00601E, "SOFTLAB, INC.", "SOFTLAB" },
	{ 0x00601F, "STALLION TECHNOLOGIES", "STALLION-TECHNOLOGIES" },
	{ 0x006020, "PIVOTAL NETWORKING, INC.", "PIVOTAL-NETWORKING" },
	{ 0x006021, "DSC CORPORATION", "DSC" },
	{ 0x006022, "VICOM SYSTEMS, INC.", "VICOM" },
	{ 0x006023, "PERICOM SEMICONDUCTOR CORP.", "PERICOM-SEMICONDUCTOR" },
	{ 0x006024, "GRADIENT TECHNOLOGIES, INC.", "GRADIENT-TECHNOLOGIES" },
	{ 0x006025, "ACTIVE IMAGING PLC", "ACTIVE-IMAGING-PLC" },
	{ 0x006026, "VIKING Modular Solutions", "VIKING-MODULAR-SOLUTIONS" },
	{ 0x006027, "Superior Modular Products", "SUPERIOR-MODULAR-PRODUCTS" },
	{ 0x006028, "MACROVISION CORPORATION", "MACROVISION" },
	{ 0x006029, "CARY PERIPHERALS INC.", "CARY-PERIPHERALS" },
	{ 0x00602A, "SYMICRON COMPUTER COMMUNICATIONS, LTD.", "SYMICRON-COMMUNICATION" },
	{ 0x00602B, "PEAK AUDIO", "PEAK-AUDIO" },
	{ 0x00602C, "LINX Data Terminals, Inc.", "LINX-DATA-TERMINALS" },
	{ 0x00602D, "ALERTON TECHNOLOGIES, INC.", "ALERTON-TECHNOLOGIES" },
	{ 0x00602E, "CYCLADES CORPORATION", "CYCLADES" },
	{ 0x00602F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x006030, "VILLAGE TRONIC ENTWICKLUNG", "VILLAGE-TRONIC-ENTWICKLUNG" },
	{ 0x006031, "HRK SYSTEMS", "HRK" },
	{ 0x006032, "I-CUBE, INC.", "I-CUBE" },
	{ 0x006033, "ACUITY IMAGING, INC.", "ACUITY-IMAGING" },
	{ 0x006034, "ROBERT BOSCH GmbH", "ROBERT-BOSCH" },
	{ 0x006035, "DALLAS SEMICONDUCTOR, INC.", "DALLAS-SEMICONDUCTOR" },
	{ 0x006036, "AIT Austrian Institute of Technology GmbH", "AIT-AUSTRIAN-INSTITUTE-OF" },
	{ 0x006037, "NXP Semiconductors", "NXP-SEMICONDUCTORS" },
	{ 0x006038, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x006039, "SanCom Technology, Inc.", "SANCOM" },
	{ 0x00603A, "QUICK CONTROLS LTD.", "QUICK-CONTROLS" },
	{ 0x00603B, "AMTEC spa", "AMTEC-SPA" },
	{ 0x00603C, "HAGIWARA SYS-COM CO., LTD.", "HAGIWARA-SYS-COM" },
	{ 0x00603D, "3CX", "3CX" },
	{ 0x00603E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00603F, "PATAPSCO DESIGNS", "PATAPSCO-DESIGNS" },
	{ 0x006040, "NETRO CORP.", "NETRO" },
	{ 0x006041, "Yokogawa Electric Corporation", "YOKOGAWA-ELECTRIC" },
	{ 0x006042, "TKS (USA), INC.", "TKS-USA" },
	{ 0x006043, "iDirect, INC.", "IDIRECT" },
	{ 0x006044, "LITTON/POLY-SCIENTIFIC", "LITTON/POLY-SCIENTIFIC" },
	{ 0x006045, "PATHLIGHT TECHNOLOGIES", "PATHLIGHT-TECHNOLOGIES" },
	{ 0x006046, "VMETRO, INC.", "VMETRO" },
	{ 0x006047, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x006048, "EMC CORPORATION", "EMC" },
	{ 0x006049, "VINA TECHNOLOGIES", "VINA-TECHNOLOGIES" },
	{ 0x00604A, "SAIC IDEAS GROUP", "SAIC-IDEAS-GROUP" },
	{ 0x00604B, "Safe-com GmbH & Co. KG", "SAFE-COM" },
	{ 0x00604C, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x00604D, "MMC NETWORKS, INC.", "MMC-NETWORKS" },
	{ 0x00604E, "CYCLE COMPUTER CORPORATION, INC.", "CYCLE" },
	{ 0x00604F, "Tattile SRL", "TATTILE-SRL" },
	{ 0x006050, "INTERNIX INC.", "INTERNIX" },
	{ 0x006051, "QUALITY SEMICONDUCTOR", "QUALITY-SEMICONDUCTOR" },
	{ 0x006052, "PERIPHERALS ENTERPRISE CO., Ltd.", "PERIPHERALS" },
	{ 0x006053, "TOYODA MACHINE WORKS, LTD.", "TOYODA-MACHINE-WORKS" },
	{ 0x006054, "CONTROLWARE GMBH", "CONTROLWARE" },
	{ 0x006055, "CORNELL UNIVERSITY", "CORNELL-UNIVERSITY" },
	{ 0x006056, "NETWORK TOOLS, INC.", "NETWORK-TOOLS" },
	{ 0x006057, "MURATA MANUFACTURING CO., LTD.", "MURATA-MANUFACTURING" },
	{ 0x006058, "COPPER MOUNTAIN COMMUNICATIONS, INC.", "COPPER-MOUNTAIN-COMMUNICATION" },
	{ 0x006059, "TECHNICAL COMMUNICATIONS CORP.", "TECHNICAL-COMMUNICATION" },
	{ 0x00605A, "CELCORE, INC.", "CELCORE" },
	{ 0x00605B, "IntraServer Technology, Inc.", "INTRASERVER" },
	{ 0x00605C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00605D, "SCANIVALVE CORP.", "SCANIVALVE" },
	{ 0x00605E, "LIBERTY TECHNOLOGY NETWORKING", "LIBERTY-NETWORKING" },
	{ 0x00605F, "NIPPON UNISOFT CORPORATION", "NIPPON-UNISOFT" },
	{ 0x006060, "Data Innovations North America", "DATA-INNOVATIONS-NORTH-AMERICA" },
	{ 0x006061, "WHISTLE COMMUNICATIONS CORP.", "WHISTLE-COMMUNICATION" },
	{ 0x006062, "TELESYNC, INC.", "TELESYNC" },
	{ 0x006063, "PSION DACOM PLC.", "PSION-DACOM-PLC" },
	{ 0x006064, "NETCOMM LIMITED", "NETCOMM" },
	{ 0x006065, "BERNECKER & RAINER INDUSTRIE-ELEKTRONIC GmbH", "BERNECKER-RAINER-INDUSTRIE-ELEKTRONIC" },
	{ 0x006066, "LACROIX Trafic", "LACROIX-TRAFIC" },
	{ 0x006067, "ACER NETXUS INC.", "ACER-NETXUS" },
	{ 0x006068, "Dialogic Corporation", "DIALOGIC" },
	{ 0x006069, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x00606A, "MITSUBISHI WIRELESS COMMUNICATIONS. INC.", "MITSUBISHI-WIRELESS-COMMUNICATION" },
	{ 0x00606B, "Synclayer Inc.", "SYNCLAYER" },
	{ 0x00606C, "ARESCOM", "ARESCOM" },
	{ 0x00606D, "DIGITAL EQUIPMENT CORP.", "DIGITAL-EQUIPMENT" },
	{ 0x00606E, "DAVICOM SEMICONDUCTOR, INC.", "DAVICOM-SEMICONDUCTOR" },
	{ 0x00606F, "CLARION CORPORATION OF AMERICA", "CLARION-OF-AMERICA" },
	{ 0x006070, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x006071, "MIDAS LAB, INC.", "MIDAS-LAB" },
	{ 0x006072, "VXL INSTRUMENTS, LIMITED", "VXL-INSTRUMENTS" },
	{ 0x006073, "REDCREEK COMMUNICATIONS, INC.", "REDCREEK-COMMUNICATION" },
	{ 0x006074, "QSC AUDIO PRODUCTS", "QSC-AUDIO-PRODUCTS" },
	{ 0x006075, "PENTEK, INC.", "PENTEK" },
	{ 0x006076, "SCHLUMBERGER TECHNOLOGIES RETAIL PETROLEUM SYSTEMS", "SCHLUMBERGER-TECHNOLOGIES-RETAIL-PETROLEUM" },
	{ 0x006077, "PRISA NETWORKS", "PRISA-NETWORKS" },
	{ 0x006078, "POWER MEASUREMENT LTD.", "POWER-MEASUREMENT" },
	{ 0x006079, "Mainstream Data, Inc.", "MAINSTREAM-DATA" },
	{ 0x00607A, "DVS GmbH", "DVS" },
	{ 0x00607B, "FORE SYSTEMS, INC.", "FORE" },
	{ 0x00607C, "WaveAccess, Ltd.", "WAVEACCESS" },
	{ 0x00607D, "SENTIENT NETWORKS INC.", "SENTIENT-NETWORKS" },
	{ 0x00607E, "GIGALABS, INC.", "GIGALABS" },
	{ 0x00607F, "AURORA TECHNOLOGIES, INC.", "AURORA-TECHNOLOGIES" },
	{ 0x006080, "MICROTRONIX DATACOM LTD.", "MICROTRONIX-DATACOM" },
	{ 0x006081, "TV/COM INTERNATIONAL", "TV/COM" },
	{ 0x006082, "NOVALINK TECHNOLOGIES, INC.", "NOVALINK-TECHNOLOGIES" },
	{ 0x006083, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x006084, "DIGITAL VIDEO", "DIGITAL-VIDEO" },
	{ 0x006085, "Storage Concepts", "STORAGE-CONCEPTS" },
	{ 0x006086, "LOGIC REPLACEMENT TECH. LTD.", "LOGIC-REPLACEMENT-TECH" },
	{ 0x006087, "KANSAI ELECTRIC CO., LTD.", "KANSAI-ELECTRIC" },
	{ 0x006088, "WHITE MOUNTAIN DSP, INC.", "WHITE-MOUNTAIN-DSP" },
	{ 0x006089, "XATA", "XATA" },
	{ 0x00608A, "CITADEL COMPUTER", "CITADEL" },
	{ 0x00608B, "ConferTech International", "CONFERTECH" },
	{ 0x00608C, "3COM CORPORATION", "3COM" },
	{ 0x00608D, "UNIPULSE CORP.", "UNIPULSE" },
	{ 0x00608E, "HE ELECTRONICS, TECHNOLOGIE & SYSTEMTECHNIK GmbH", "HE-ELECTRONICS-TECHNOLOGIE-SYSTEMTECHNIK" },
	{ 0x00608F, "TEKRAM TECHNOLOGY CO., LTD.", "TEKRAM" },
	{ 0x006090, "Artiza Networks Inc", "ARTIZA-NETWORKS" },
	{ 0x006091, "FIRST PACIFIC NETWORKS, INC.", "FIRST-PACIFIC-NETWORKS" },
	{ 0x006092, "MICRO/SYS, INC.", "MICRO/SYS" },
	{ 0x006093, "VARIAN", "VARIAN" },
	{ 0x006094, "IBM Corp", "IBM" },
	{ 0x006095, "ACCU-TIME SYSTEMS, INC.", "ACCU-TIME" },
	{ 0x006096, "T.S. MICROTECH INC.", "T-S-MICROTECH" },
	{ 0x006097, "3COM CORPORATION", "3COM" },
	{ 0x006098, "HT COMMUNICATIONS", "HT-COMMUNICATION" },
	{ 0x006099, "SBE, Inc.", "SBE" },
	{ 0x00609A, "NJK TECHNO CO.", "NJK-TECHNO" },
	{ 0x00609B, "ASTRO-MED, INC.", "ASTRO-MED" },
	{ 0x00609C, "Perkin-Elmer Incorporated", "PERKIN-ELMER-INCORPORATED" },
	{ 0x00609D, "PMI FOOD EQUIPMENT GROUP", "PMI-FOOD-EQUIPMENT-GROUP" },
	{ 0x00609E, "ASC X3 - INFORMATION TECHNOLOGY STANDARDS SECRETARIATS", "ASC-X3---INFORMATION-STANDARDS-SECRETARIATS" },
	{ 0x00609F, "PHAST CORPORATION", "PHAST" },
	{ 0x0060A0, "SWITCHED NETWORK TECHNOLOGIES, INC.", "SWITCHED-NETWORK-TECHNOLOGIES" },
	{ 0x0060A1, "VPNet, Inc.", "VPNET" },
	{ 0x0060A2, "NIHON UNISYS LIMITED CO.", "NIHON-UNISYS" },
	{ 0x0060A3, "CONTINUUM TECHNOLOGY CORP.", "CONTINUUM" },
	{ 0x0060A4, "GEW Technologies (PTY)Ltd", "GEW-TECHNOLOGIES-PTY" },
	{ 0x0060A5, "PERFORMANCE TELECOM CORP.", "PERFORMANCE-TELECOM" },
	{ 0x0060A6, "PARTICLE MEASURING SYSTEMS", "PARTICLE-MEASURING" },
	{ 0x0060A7, "MICROSENS GmbH & CO. KG", "MICROSENS" },
	{ 0x0060A8, "TIDOMAT AB", "TIDOMAT-AB" },
	{ 0x0060A9, "GESYTEC MbH", "GESYTEC-MBH" },
	{ 0x0060AA, "INTELLIGENT DEVICES INC. (IDI)", "INTELLIGENT-DEVICES-IDI" },
	{ 0x0060AB, "LARSCOM INCORPORATED", "LARSCOM-INCORPORATED" },
	{ 0x0060AC, "RESILIENCE CORPORATION", "RESILIENCE" },
	{ 0x0060AD, "MegaChips Corporation", "MEGACHIPS" },
	{ 0x0060AE, "TRIO INFORMATION SYSTEMS AB", "TRIO-INFORMATION-AB" },
	{ 0x0060AF, "PACIFIC MICRO DATA, INC.", "PACIFIC-MICRO-DATA" },
	{ 0x0060B0, "HEWLETT-PACKARD CO.", "HEWLETT-PACKARD" },
	{ 0x0060B1, "INPUT/OUTPUT, INC.", "INPUT/OUTPUT" },
	{ 0x0060B2, "PROCESS CONTROL CORP.", "PROCESS-CONTROL" },
	{ 0x0060B3, "Z-COM, INC.", "Z-COM" },
	{ 0x0060B4, "GLENAYRE R&D INC.", "GLENAYRE-R-D" },
	{ 0x0060B5, "KEBA GmbH", "KEBA" },
	{ 0x0060B6, "LAND COMPUTER CO., LTD.", "LAND" },
	{ 0x0060B7, "CHANNELMATIC, INC.", "CHANNELMATIC" },
	{ 0x0060B8, "CORELIS Inc.", "CORELIS" },
	{ 0x0060B9, "NEC Platforms, Ltd", "NEC-PLATFORMS" },
	{ 0x0060BA, "SAHARA NETWORKS, INC.", "SAHARA-NETWORKS" },
	{ 0x0060BB, "CABLETRON - NETLINK, INC.", "CABLETRON---NETLINK" },
	{ 0x0060BC, "KeunYoung Electronics & Communication Co., Ltd.", "KEUNYOUNG-ELECTRONICS-COMMUNICATION" },
	{ 0x0060BD, "HUBBELL-PULSECOM", "HUBBELL-PULSECOM" },
	{ 0x0060BE, "WEBTRONICS", "WEBTRONICS" },
	{ 0x0060BF, "MACRAIGOR SYSTEMS, INC.", "MACRAIGOR" },
	{ 0x0060C0, "Nera Networks AS", "NERA-NETWORKS-AS" },
	{ 0x0060C1, "WaveSpan Corporation", "WAVESPAN" },
	{ 0x0060C2, "MPL AG", "MPL" },
	{ 0x0060C3, "NETVISION CORPORATION", "NETVISION" },
	{ 0x0060C4, "SOLITON SYSTEMS K.K.", "SOLITON-K-K" },
	{ 0x0060C5, "ANCOT CORP.", "ANCOT" },
	{ 0x0060C6, "DCS AG", "DCS" },
	{ 0x0060C7, "AMATI COMMUNICATIONS CORP.", "AMATI-COMMUNICATION" },
	{ 0x0060C8, "KUKA WELDING SYSTEMS & ROBOTS", "KUKA-WELDING-ROBOTS" },
	{ 0x0060C9, "ControlNet, Inc.", "CONTROLNET" },
	{ 0x0060CA, "HARMONIC SYSTEMS INCORPORATED", "HARMONIC-INCORPORATED" },
	{ 0x0060CB, "HITACHI ZOSEN CORPORATION", "HITACHI-ZOSEN" },
	{ 0x0060CC, "EMTRAK, INCORPORATED", "EMTRAK-INCORPORATED" },
	{ 0x0060CD, "VideoServer, Inc.", "VIDEOSERVER" },
	{ 0x0060CE, "ACCLAIM COMMUNICATIONS", "ACCLAIM-COMMUNICATION" },
	{ 0x0060CF, "ALTEON NETWORKS, INC.", "ALTEON-NETWORKS" },
	{ 0x0060D0, "SNMP RESEARCH INCORPORATED", "SNMP-RESEARCH-INCORPORATED" },
	{ 0x0060D1, "CASCADE COMMUNICATIONS", "CASCADE-COMMUNICATION" },
	{ 0x0060D2, "LUCENT TECHNOLOGIES TAIWAN TELECOMMUNICATIONS CO., LTD.", "LUCENT-TECHNOLOGIES-TAIWAN-TELECOMMUNICATIONS" },
	{ 0x0060D3, "AT&T", "AT-T" },
	{ 0x0060D4, "ELDAT COMMUNICATION LTD.", "ELDAT-COMMUNICATION" },
	{ 0x0060D5, "MIYACHI TECHNOS CORP.", "MIYACHI-TECHNOS" },
	{ 0x0060D6, "NovAtel Wireless Technologies Ltd.", "NOVATEL-WIRELESS-TECHNOLOGIES" },
	{ 0x0060D7, "ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE (EPFL)", "ECOLE-POLYTECHNIQUE-FEDERALE-DE-LAUSANNE-EPFL" },
	{ 0x0060D8, "ELMIC SYSTEMS, INC.", "ELMIC" },
	{ 0x0060D9, "TRANSYS NETWORKS INC.", "TRANSYS-NETWORKS" },
	{ 0x0060DA, "Red Lion Controls, LP", "RED-LION-CONTROLS-LP" },
	{ 0x0060DB, "NTP ELEKTRONIK A/S", "NTP-ELEKTRONIK-A/S" },
	{ 0x0060DC, "Toyo Network Systems & System Integration Co. LTD", "TOYO-NETWORK-SYSTEM-INTEGRATION" },
	{ 0x0060DD, "MYRICOM, INC.", "MYRICOM" },
	{ 0x0060DE, "Kayser-Threde GmbH", "KAYSER-THREDE" },
	{ 0x0060DF, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x0060E0, "AXIOM TECHNOLOGY CO., LTD.", "AXIOM" },
	{ 0x0060E1, "ORCKIT COMMUNICATIONS LTD.", "ORCKIT-COMMUNICATION" },
	{ 0x0060E2, "QUEST ENGINEERING & DEVELOPMENT", "QUEST-ENGINEERING-DEVELOPMENT" },
	{ 0x0060E3, "ARBIN INSTRUMENTS", "ARBIN-INSTRUMENTS" },
	{ 0x0060E4, "COMPUSERVE, INC.", "COMPUSERVE" },
	{ 0x0060E5, "FUJI AUTOMATION CO., LTD.", "FUJI-AUTOMATION" },
	{ 0x0060E6, "SHOMITI SYSTEMS INCORPORATED", "SHOMITI-INCORPORATED" },
	{ 0x0060E7, "RANDATA", "RANDATA" },
	{ 0x0060E8, "HITACHI COMPUTER PRODUCTS (AMERICA), INC.", "HITACHI-PRODUCTS-AMERICA" },
	{ 0x0060E9, "ATOP TECHNOLOGIES, INC.", "ATOP-TECHNOLOGIES" },
	{ 0x0060EA, "StreamLogic", "STREAMLOGIC" },
	{ 0x0060EB, "FOURTHTRACK SYSTEMS", "FOURTHTRACK" },
	{ 0x0060EC, "HERMARY OPTO ELECTRONICS INC.", "HERMARY-OPTO-ELECTRONICS" },
	{ 0x0060ED, "RICARDO TEST AUTOMATION LTD.", "RICARDO-TEST-AUTOMATION" },
	{ 0x0060EE, "APOLLO", "APOLLO" },
	{ 0x0060EF, "FLYTECH TECHNOLOGY CO., LTD.", "FLYTECH" },
	{ 0x0060F0, "JOHNSON & JOHNSON MEDICAL, INC", "JOHNSON-JOHNSON-MEDICAL" },
	{ 0x0060F1, "EXP COMPUTER, INC.", "EXP" },
	{ 0x0060F2, "LASERGRAPHICS, INC.", "LASERGRAPHICS" },
	{ 0x0060F3, "Performance Analysis Broadband, Spirent plc", "PERFORMANCE-ANALYSIS-BROADBAND-SPIRENT-PLC" },
	{ 0x0060F4, "ADVANCED COMPUTER SOLUTIONS, Inc.", "ADVANCED-SOLUTIONS" },
	{ 0x0060F5, "ICON WEST, INC.", "ICON-WEST" },
	{ 0x0060F6, "NEXTEST COMMUNICATIONS PRODUCTS, INC.", "NEXTEST-COMMUNICATION-PRODUCTS" },
	{ 0x0060F7, "DATAFUSION SYSTEMS", "DATAFUSION" },
	{ 0x0060F8, "Loran International Technologies Inc.", "LORAN-TECHNOLOGIES" },
	{ 0x0060F9, "DIAMOND LANE COMMUNICATIONS", "DIAMOND-LANE-COMMUNICATION" },
	{ 0x0060FA, "EDUCATIONAL TECHNOLOGY RESOURCES, INC.", "EDUCATIONAL-RESOURCES" },
	{ 0x0060FB, "PACKETEER, INC.", "PACKETEER" },
	{ 0x0060FC, "CONSERVATION THROUGH INNOVATION LTD.", "CONSERVATION-THROUGH-INNOVATION" },
	{ 0x0060FD, "NetICs, Inc.", "NETICS" },
	{ 0x0060FE, "LYNX SYSTEM DEVELOPERS, INC.", "LYNX-SYSTEM-DEVELOPERS" },
	{ 0x0060FF, "QuVis, Inc.", "QUVIS" },
	{ 0x006171, "Apple", "APPLE" },
	{ 0x006440, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0064A6, "Maquet CardioVascular", "MAQUET-CARDIOVASCULAR" },
	{ 0x00664B, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x006B8E, "Shanghai Feixun Communication Co.,Ltd.", "SHANGHAI-FEIXUN-COMMUNICATION" },
	{ 0x006B9E, "VIZIO Inc", "VIZIO" },
	{ 0x006BA0, "SHENZHEN UNIVERSAL INTELLISYS PTE LTD", "SHENZHEN-UNIVERSAL-INTELLISYS-PTE" },
	{ 0x006DFB, "Vutrix (UK) Ltd", "VUTRIX-UK" },
	{ 0x0070B0, "M/A-COM INC. COMPANIES", "M/A-COM-COMPANIES" },
	{ 0x0070B3, "DATA RECALL LTD.", "DATA-RECALL" },
	{ 0x0071C2, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x0071CC, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x00738D, "Tinno Mobile Technology Corp", "TINNO-MOBILE" },
	{ 0x0073E0, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x007532, "INID BV", "INID" },
	{ 0x0075E1, "Ampt, LLC", "AMPT-LLC" },
	{ 0x00789E, "SAGEMCOM", "SAGEMCOM" },
	{ 0x007DFA, "Volkswagen Group of America", "VOLKSWAGEN-GROUP-OF-AMERICA" },
	{ 0x007E56, "China Dragon Technology Limited", "CHINA-DRAGON" },
	{ 0x007F28, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x008000, "MULTITECH SYSTEMS, INC.", "MULTITECH" },
	{ 0x008001, "PERIPHONICS CORPORATION", "PERIPHONICS" },
	{ 0x008002, "SATELCOM (UK) LTD", "SATELCOM-UK" },
	{ 0x008003, "HYTEC ELECTRONICS LTD.", "HYTEC-ELECTRONICS" },
	{ 0x008004, "ANTLOW COMMUNICATIONS, LTD.", "ANTLOW-COMMUNICATION" },
	{ 0x008005, "CACTUS COMPUTER INC.", "CACTUS" },
	{ 0x008006, "COMPUADD CORPORATION", "COMPUADD" },
	{ 0x008007, "DLOG NC-SYSTEME", "DLOG-NC-SYSTEME" },
	{ 0x008008, "DYNATECH COMPUTER SYSTEMS", "DYNATECH" },
	{ 0x008009, "JUPITER SYSTEMS, INC.", "JUPITER" },
	{ 0x00800A, "JAPAN COMPUTER CORP.", "JAPAN" },
	{ 0x00800B, "CSK CORPORATION", "CSK" },
	{ 0x00800C, "VIDECOM LIMITED", "VIDECOM" },
	{ 0x00800D, "VOSSWINKEL F.U.", "VOSSWINKEL-F-U" },
	{ 0x00800E, "ATLANTIX CORPORATION", "ATLANTIX" },
	{ 0x00800F, "STANDARD MICROSYSTEMS", "STANDARD-MICROSYSTEMS" },
	{ 0x008010, "COMMODORE INTERNATIONAL", "COMMODORE" },
	{ 0x008011, "DIGITAL SYSTEMS INT'L. INC.", "DIGITAL" },
	{ 0x008012, "INTEGRATED MEASUREMENT SYSTEMS", "INTEGRATED-MEASUREMENT" },
	{ 0x008013, "THOMAS-CONRAD CORPORATION", "THOMAS-CONRAD" },
	{ 0x008014, "ESPRIT SYSTEMS", "ESPRIT" },
	{ 0x008015, "SEIKO SYSTEMS, INC.", "SEIKO" },
	{ 0x008016, "WANDEL AND GOLTERMANN", "WANDEL-AND-GOLTERMANN" },
	{ 0x008017, "PFU LIMITED", "PFU" },
	{ 0x008018, "KOBE STEEL, LTD.", "KOBE-STEEL" },
	{ 0x008019, "DAYNA COMMUNICATIONS, INC.", "DAYNA-COMMUNICATION" },
	{ 0x00801A, "BELL ATLANTIC", "BELL-ATLANTIC" },
	{ 0x00801B, "KODIAK TECHNOLOGY", "KODIAK" },
	{ 0x00801C, "NEWPORT SYSTEMS SOLUTIONS", "NEWPORT-SOLUTIONS" },
	{ 0x00801D, "INTEGRATED INFERENCE MACHINES", "INTEGRATED-INFERENCE-MACHINES" },
	{ 0x00801E, "XINETRON, INC.", "XINETRON" },
	{ 0x00801F, "KRUPP ATLAS ELECTRONIK GMBH", "KRUPP-ATLAS-ELECTRONIK" },
	{ 0x008020, "NETWORK PRODUCTS", "NETWORK-PRODUCTS" },
	{ 0x008021, "Alcatel Canada Inc.", "ALCATEL-CANADA" },
	{ 0x008022, "SCAN-OPTICS", "SCAN-OPTICS" },
	{ 0x008023, "INTEGRATED BUSINESS NETWORKS", "INTEGRATED-BUSINESS-NETWORKS" },
	{ 0x008024, "KALPANA, INC.", "KALPANA" },
	{ 0x008025, "STOLLMANN GMBH", "STOLLMANN" },
	{ 0x008026, "NETWORK PRODUCTS CORPORATION", "NETWORK-PRODUCTS" },
	{ 0x008027, "ADAPTIVE SYSTEMS, INC.", "ADAPTIVE" },
	{ 0x008028, "TRADPOST (HK) LTD", "TRADPOST-HK" },
	{ 0x008029, "EAGLE TECHNOLOGY, INC.", "EAGLE" },
	{ 0x00802A, "TEST SYSTEMS & SIMULATIONS INC", "TEST-SIMULATIONS" },
	{ 0x00802B, "INTEGRATED MARKETING CO", "INTEGRATED-MARKETING" },
	{ 0x00802C, "THE SAGE GROUP PLC", "THE-SAGE-GROUP-PLC" },
	{ 0x00802D, "XYLOGICS INC", "XYLOGICS" },
	{ 0x00802E, "CASTLE ROCK COMPUTING", "CASTLE-ROCK-COMPUTING" },
	{ 0x00802F, "NATIONAL INSTRUMENTS CORP.", "NATIONAL-INSTRUMENTS" },
	{ 0x008030, "NEXUS ELECTRONICS", "NEXUS-ELECTRONICS" },
	{ 0x008031, "BASYS, CORP.", "BASYS" },
	{ 0x008032, "ACCESS CO., LTD.", "ACCESS" },
	{ 0x008033, "EMS Aviation, Inc.", "EMS-AVIATION" },
	{ 0x008034, "SMT GOUPIL", "SMT-GOUPIL" },
	{ 0x008035, "TECHNOLOGY WORKS, INC.", "WORKS" },
	{ 0x008036, "REFLEX MANUFACTURING SYSTEMS", "REFLEX-MANUFACTURING" },
	{ 0x008037, "Ericsson Group", "ERICSSON-GROUP" },
	{ 0x008038, "DATA RESEARCH & APPLICATIONS", "DATA-RESEARCH-APPLICATIONS" },
	{ 0x008039, "ALCATEL STC AUSTRALIA", "ALCATEL-STC-AUSTRALIA" },
	{ 0x00803A, "VARITYPER, INC.", "VARITYPER" },
	{ 0x00803B, "APT COMMUNICATIONS, INC.", "APT-COMMUNICATION" },
	{ 0x00803C, "TVS ELECTRONICS LTD", "TVS-ELECTRONICS" },
	{ 0x00803D, "SURIGIKEN CO., LTD.", "SURIGIKEN" },
	{ 0x00803E, "SYNERNETICS", "SYNERNETICS" },
	{ 0x00803F, "TATUNG COMPANY", "TATUNG-COMPANY" },
	{ 0x008040, "JOHN FLUKE MANUFACTURING CO.", "JOHN-FLUKE-MANUFACTURING" },
	{ 0x008041, "VEB KOMBINAT ROBOTRON", "VEB-KOMBINAT-ROBOTRON" },
	{ 0x008042, "Artesyn Embedded Technologies", "ARTESYN-EMBEDDED-TECHNOLOGIES" },
	{ 0x008043, "NETWORLD, INC.", "NETWORLD" },
	{ 0x008044, "SYSTECH COMPUTER CORP.", "SYSTECH" },
	{ 0x008045, "MATSUSHITA ELECTRIC IND. CO", "MATSUSHITA-ELECTRIC-IND" },
	{ 0x008046, "Tattile SRL", "TATTILE-SRL" },
	{ 0x008047, "IN-NET CORP.", "IN-NET" },
	{ 0x008048, "COMPEX INCORPORATED", "COMPEX-INCORPORATED" },
	{ 0x008049, "NISSIN ELECTRIC CO., LTD.", "NISSIN-ELECTRIC" },
	{ 0x00804A, "PRO-LOG", "PRO-LOG" },
	{ 0x00804B, "EAGLE TECHNOLOGIES PTY.LTD.", "EAGLE-TECHNOLOGIES-PTY" },
	{ 0x00804C, "CONTEC CO., LTD.", "CONTEC" },
	{ 0x00804D, "CYCLONE MICROSYSTEMS, INC.", "CYCLONE-MICROSYSTEMS" },
	{ 0x00804E, "APEX COMPUTER COMPANY", "APEX-COMPANY" },
	{ 0x00804F, "DAIKIN INDUSTRIES, LTD.", "DAIKIN-INDUSTRIES" },
	{ 0x008050, "ZIATECH CORPORATION", "ZIATECH" },
	{ 0x008051, "FIBERMUX", "FIBERMUX" },
	{ 0x008052, "TECHNICALLY ELITE CONCEPTS", "TECHNICALLY-ELITE-CONCEPTS" },
	{ 0x008053, "INTELLICOM, INC.", "INTELLICOM" },
	{ 0x008054, "FRONTIER TECHNOLOGIES CORP.", "FRONTIER-TECHNOLOGIES" },
	{ 0x008055, "FERMILAB", "FERMILAB" },
	{ 0x008056, "SPHINX ELEKTRONIK GMBH", "SPHINX-ELEKTRONIK" },
	{ 0x008057, "ADSOFT, LTD.", "ADSOFT" },
	{ 0x008058, "PRINTER SYSTEMS CORPORATION", "PRINTER" },
	{ 0x008059, "STANLEY ELECTRIC CO., LTD", "STANLEY-ELECTRIC" },
	{ 0x00805A, "TULIP COMPUTERS INTERNAT'L B.V", "TULIP-COMPUTERS-INTERNAT-L" },
	{ 0x00805B, "CONDOR SYSTEMS, INC.", "CONDOR" },
	{ 0x00805C, "AGILIS CORPORATION", "AGILIS" },
	{ 0x00805D, "CANSTAR", "CANSTAR" },
	{ 0x00805E, "LSI LOGIC CORPORATION", "LSI-LOGIC" },
	{ 0x00805F, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x008060, "NETWORK INTERFACE CORPORATION", "NETWORK-INTERFACE" },
	{ 0x008061, "LITTON SYSTEMS, INC.", "LITTON" },
	{ 0x008062, "INTERFACE CO.", "INTERFACE" },
	{ 0x008063, "Hirschmann Automation and Control GmbH", "HIRSCHMANN-AUTOMATION-AND-CONTROL" },
	{ 0x008064, "WYSE TECHNOLOGY LLC", "WYSE-LLC" },
	{ 0x008065, "CYBERGRAPHIC SYSTEMS PTY LTD.", "CYBERGRAPHIC-PTY" },
	{ 0x008066, "ARCOM CONTROL SYSTEMS, LTD.", "ARCOM-CONTROL" },
	{ 0x008067, "SQUARE D COMPANY", "SQUARE-D-COMPANY" },
	{ 0x008068, "YAMATECH SCIENTIFIC LTD.", "YAMATECH-SCIENTIFIC" },
	{ 0x008069, "COMPUTONE SYSTEMS", "COMPUTONE" },
	{ 0x00806A, "ERI (EMPAC RESEARCH INC.)", "ERI-EMPAC-RESEARCH" },
	{ 0x00806B, "SCHMID TELECOMMUNICATION", "SCHMID-TELECOMMUNICATION" },
	{ 0x00806C, "CEGELEC PROJECTS LTD", "CEGELEC-PROJECTS" },
	{ 0x00806D, "CENTURY SYSTEMS CORP.", "CENTURY" },
	{ 0x00806E, "NIPPON STEEL CORPORATION", "NIPPON-STEEL" },
	{ 0x00806F, "ONELAN LTD.", "ONELAN" },
	{ 0x008070, "COMPUTADORAS MICRON", "COMPUTADORAS-MICRON" },
	{ 0x008071, "SAI TECHNOLOGY", "SAI" },
	{ 0x008072, "MICROPLEX SYSTEMS LTD.", "MICROPLEX" },
	{ 0x008073, "DWB ASSOCIATES", "DWB-ASSOCIATES" },
	{ 0x008074, "FISHER CONTROLS", "FISHER-CONTROLS" },
	{ 0x008075, "PARSYTEC GMBH", "PARSYTEC" },
	{ 0x008076, "MCNC", "MCNC" },
	{ 0x008077, "BROTHER INDUSTRIES, LTD.", "BROTHER-INDUSTRIES" },
	{ 0x008078, "PRACTICAL PERIPHERALS, INC.", "PRACTICAL-PERIPHERALS" },
	{ 0x008079, "MICROBUS DESIGNS LTD.", "MICROBUS-DESIGNS" },
	{ 0x00807A, "AITECH SYSTEMS LTD.", "AITECH" },
	{ 0x00807B, "ARTEL COMMUNICATIONS CORP.", "ARTEL-COMMUNICATION" },
	{ 0x00807C, "FIBERCOM, INC.", "FIBERCOM" },
	{ 0x00807D, "EQUINOX SYSTEMS INC.", "EQUINOX" },
	{ 0x00807E, "SOUTHERN PACIFIC LTD.", "SOUTHERN-PACIFIC" },
	{ 0x00807F, "DY-4 INCORPORATED", "DY-4-INCORPORATED" },
	{ 0x008080, "DATAMEDIA CORPORATION", "DATAMEDIA" },
	{ 0x008081, "KENDALL SQUARE RESEARCH CORP.", "KENDALL-SQUARE-RESEARCH" },
	{ 0x008082, "PEP MODULAR COMPUTERS GMBH", "PEP-MODULAR-COMPUTERS" },
	{ 0x008083, "AMDAHL", "AMDAHL" },
	{ 0x008084, "THE CLOUD INC.", "THE-CLOUD" },
	{ 0x008085, "H-THREE SYSTEMS CORPORATION", "H-THREE" },
	{ 0x008086, "COMPUTER GENERATION INC.", "GENERATION" },
	{ 0x008087, "OKI ELECTRIC INDUSTRY CO., LTD", "OKI-ELECTRIC-INDUSTRY" },
	{ 0x008088, "VICTOR COMPANY OF JAPAN, LTD.", "VICTOR-COMPANY-OF-JAPAN" },
	{ 0x008089, "TECNETICS (PTY) LTD.", "TECNETICS-PTY" },
	{ 0x00808A, "SUMMIT MICROSYSTEMS CORP.", "SUMMIT-MICROSYSTEMS" },
	{ 0x00808B, "DACOLL LIMITED", "DACOLL" },
	{ 0x00808C, "NetScout Systems, Inc.", "NETSCOUT" },
	{ 0x00808D, "WESTCOAST TECHNOLOGY B.V.", "WESTCOAST" },
	{ 0x00808E, "RADSTONE TECHNOLOGY", "RADSTONE" },
	{ 0x00808F, "C. ITOH ELECTRONICS, INC.", "C-ITOH-ELECTRONICS" },
	{ 0x008090, "MICROTEK INTERNATIONAL, INC.", "MICROTEK" },
	{ 0x008091, "TOKYO ELECTRIC CO.,LTD", "TOKYO-ELECTRIC" },
	{ 0x008092, "Silex Technology, Inc.", "SILEX" },
	{ 0x008093, "XYRON CORPORATION", "XYRON" },
	{ 0x008094, "ALFA LAVAL AUTOMATION AB", "ALFA-LAVAL-AUTOMATION-AB" },
	{ 0x008095, "BASIC MERTON HANDELSGES.M.B.H.", "BASIC-MERTON-HANDELSGES-M-B-H" },
	{ 0x008096, "HUMAN DESIGNED SYSTEMS, INC.", "HUMAN-DESIGNED" },
	{ 0x008097, "CENTRALP AUTOMATISMES", "CENTRALP-AUTOMATISMES" },
	{ 0x008098, "TDK CORPORATION", "TDK" },
	{ 0x008099, "Eaton Industries GmbH", "EATON-INDUSTRIES" },
	{ 0x00809A, "NOVUS NETWORKS LTD", "NOVUS-NETWORKS" },
	{ 0x00809B, "JUSTSYSTEM CORPORATION", "JUSTSYSTEM" },
	{ 0x00809C, "LUXCOM, INC.", "LUXCOM" },
	{ 0x00809D, "Commscraft Ltd.", "COMMSCRAFT" },
	{ 0x00809E, "DATUS GMBH", "DATUS" },
	{ 0x00809F, "Alcatel-Lucent Enterprise", "ALCATEL-LUCENT" },
	{ 0x0080A0, "EDISA HEWLETT PACKARD S/A", "EDISA-HEWLETT-PACKARD-S/A" },
	{ 0x0080A1, "MICROTEST, INC.", "MICROTEST" },
	{ 0x0080A2, "CREATIVE ELECTRONIC SYSTEMS", "CREATIVE" },
	{ 0x0080A3, "Lantronix", "LANTRONIX" },
	{ 0x0080A4, "LIBERTY ELECTRONICS", "LIBERTY-ELECTRONICS" },
	{ 0x0080A5, "SPEED INTERNATIONAL", "SPEED" },
	{ 0x0080A6, "REPUBLIC TECHNOLOGY, INC.", "REPUBLIC" },
	{ 0x0080A7, "Honeywell International Inc", "HONEYWELL" },
	{ 0x0080A8, "VITACOM CORPORATION", "VITACOM" },
	{ 0x0080A9, "CLEARPOINT RESEARCH", "CLEARPOINT-RESEARCH" },
	{ 0x0080AA, "MAXPEED", "MAXPEED" },
	{ 0x0080AB, "DUKANE NETWORK INTEGRATION", "DUKANE-NETWORK-INTEGRATION" },
	{ 0x0080AC, "IMLOGIX, DIVISION OF GENESYS", "IMLOGIX-DIVISION-OF-GENESYS" },
	{ 0x0080AD, "CNET TECHNOLOGY, INC.", "CNET" },
	{ 0x0080AE, "HUGHES NETWORK SYSTEMS", "HUGHES-NETWORK" },
	{ 0x0080AF, "ALLUMER CO., LTD.", "ALLUMER" },
	{ 0x0080B0, "ADVANCED INFORMATION", "ADVANCED-INFORMATION" },
	{ 0x0080B1, "SOFTCOM A/S", "SOFTCOM-A/S" },
	{ 0x0080B2, "NETWORK EQUIPMENT TECHNOLOGIES", "NETWORK-EQUIPMENT-TECHNOLOGIES" },
	{ 0x0080B3, "AVAL DATA CORPORATION", "AVAL-DATA" },
	{ 0x0080B4, "SOPHIA SYSTEMS", "SOPHIA" },
	{ 0x0080B5, "UNITED NETWORKS INC.", "UNITED-NETWORKS" },
	{ 0x0080B6, "THEMIS COMPUTER", "THEMIS" },
	{ 0x0080B7, "STELLAR COMPUTER", "STELLAR" },
	{ 0x0080B8, "B.U.G. MORISEIKI, INCORPORATED", "B-U-G-MORISEIKI-INCORPORATED" },
	{ 0x0080B9, "ARCHE TECHNOLIGIES INC.", "ARCHE-TECHNOLIGIES" },
	{ 0x0080BA, "SPECIALIX (ASIA) PTE, LTD", "SPECIALIX-ASIA-PTE" },
	{ 0x0080BB, "HUGHES LAN SYSTEMS", "HUGHES-LAN" },
	{ 0x0080BC, "HITACHI ENGINEERING CO., LTD", "HITACHI-ENGINEERING" },
	{ 0x0080BD, "THE FURUKAWA ELECTRIC CO., LTD", "THE-FURUKAWA-ELECTRIC" },
	{ 0x0080BE, "ARIES RESEARCH", "ARIES-RESEARCH" },
	{ 0x0080BF, "TAKAOKA ELECTRIC MFG. CO. LTD.", "TAKAOKA-ELECTRIC-MFG" },
	{ 0x0080C0, "PENRIL DATACOMM", "PENRIL-DATACOMM" },
	{ 0x0080C1, "LANEX CORPORATION", "LANEX" },
	{ 0x0080C2, "IEEE 802.1 COMMITTEE", "IEEE-802-1-COMMITTEE" },
	{ 0x0080C3, "BICC INFORMATION SYSTEMS & SVC", "BICC-INFORMATION-SVC" },
	{ 0x0080C4, "DOCUMENT TECHNOLOGIES, INC.", "DOCUMENT-TECHNOLOGIES" },
	{ 0x0080C5, "NOVELLCO DE MEXICO", "NOVELLCO-DE-MEXICO" },
	{ 0x0080C6, "NATIONAL DATACOMM CORPORATION", "NATIONAL-DATACOMM" },
	{ 0x0080C7, "XIRCOM", "XIRCOM" },
	{ 0x0080C8, "D-LINK SYSTEMS, INC.", "D-LINK" },
	{ 0x0080C9, "ALBERTA MICROELECTRONIC CENTRE", "ALBERTA-MICROELECTRONIC-CENTRE" },
	{ 0x0080CA, "NETCOM RESEARCH INCORPORATED", "NETCOM-RESEARCH-INCORPORATED" },
	{ 0x0080CB, "FALCO DATA PRODUCTS", "FALCO-DATA-PRODUCTS" },
	{ 0x0080CC, "MICROWAVE BYPASS SYSTEMS", "MICROWAVE-BYPASS" },
	{ 0x0080CD, "MICRONICS COMPUTER, INC.", "MICRONICS" },
	{ 0x0080CE, "BROADCAST TELEVISION SYSTEMS", "BROADCAST-TELEVISION" },
	{ 0x0080CF, "EMBEDDED PERFORMANCE INC.", "EMBEDDED-PERFORMANCE" },
	{ 0x0080D0, "COMPUTER PERIPHERALS, INC.", "PERIPHERALS" },
	{ 0x0080D1, "KIMTRON CORPORATION", "KIMTRON" },
	{ 0x0080D2, "SHINNIHONDENKO CO., LTD.", "SHINNIHONDENKO" },
	{ 0x0080D3, "SHIVA CORP.", "SHIVA" },
	{ 0x0080D4, "CHASE RESEARCH LTD.", "CHASE-RESEARCH" },
	{ 0x0080D5, "CADRE TECHNOLOGIES", "CADRE-TECHNOLOGIES" },
	{ 0x0080D6, "NUVOTECH, INC.", "NUVOTECH" },
	{ 0x0080D7, "Fantum Engineering", "FANTUM-ENGINEERING" },
	{ 0x0080D8, "NETWORK PERIPHERALS INC.", "NETWORK-PERIPHERALS" },
	{ 0x0080D9, "EMK Elektronik GmbH & Co. KG", "EMK-ELEKTRONIK" },
	{ 0x0080DA, "Bruel & Kjaer Sound & Vibration Measurement A/S", "BRUEL-KJAER-SOUND-VIBRATION-MEASUREMENT-A/S" },
	{ 0x0080DB, "GRAPHON CORPORATION", "GRAPHON" },
	{ 0x0080DC, "PICKER INTERNATIONAL", "PICKER" },
	{ 0x0080DD, "GMX INC/GIMIX", "GMX-/GIMIX" },
	{ 0x0080DE, "GIPSI S.A.", "GIPSI" },
	{ 0x0080DF, "ADC CODENOLL TECHNOLOGY CORP.", "ADC-CODENOLL" },
	{ 0x0080E0, "XTP SYSTEMS, INC.", "XTP" },
	{ 0x0080E1, "STMICROELECTRONICS", "STMICROELECTRONICS" },
	{ 0x0080E2, "T.D.I. CO., LTD.", "T-D-I" },
	{ 0x0080E3, "CORAL NETWORK CORPORATION", "CORAL-NETWORK" },
	{ 0x0080E4, "NORTHWEST DIGITAL SYSTEMS, INC", "NORTHWEST-DIGITAL" },
	{ 0x0080E5, "NetApp, Inc", "NETAPP" },
	{ 0x0080E6, "PEER NETWORKS, INC.", "PEER-NETWORKS" },
	{ 0x0080E7, "LYNWOOD SCIENTIFIC DEV. LTD.", "LYNWOOD-SCIENTIFIC-DEV" },
	{ 0x0080E8, "CUMULUS CORPORATIION", "CUMULUS-CORPORATIION" },
	{ 0x0080E9, "Madge Ltd.", "MADGE" },
	{ 0x0080EA, "ADVA Optical Networking Ltd.", "ADVA-OPTICAL-NETWORKING" },
	{ 0x0080EB, "COMPCONTROL B.V.", "COMPCONTROL" },
	{ 0x0080EC, "SUPERCOMPUTING SOLUTIONS, INC.", "SUPERCOMPUTING-SOLUTIONS" },
	{ 0x0080ED, "IQ TECHNOLOGIES, INC.", "IQ-TECHNOLOGIES" },
	{ 0x0080EE, "THOMSON CSF", "THOMSON-CSF" },
	{ 0x0080EF, "RATIONAL", "RATIONAL" },
	{ 0x0080F0, "Panasonic Communications Co., Ltd.", "PANASONIC-COMMUNICATION" },
	{ 0x0080F1, "OPUS SYSTEMS", "OPUS" },
	{ 0x0080F2, "RAYCOM SYSTEMS INC", "RAYCOM" },
	{ 0x0080F3, "SUN ELECTRONICS CORP.", "SUN-ELECTRONICS" },
	{ 0x0080F4, "TELEMECANIQUE ELECTRIQUE", "TELEMECANIQUE-ELECTRIQUE" },
	{ 0x0080F5, "Quantel Ltd", "QUANTEL" },
	{ 0x0080F6, "SYNERGY MICROSYSTEMS", "SYNERGY-MICROSYSTEMS" },
	{ 0x0080F7, "ZENITH ELECTRONICS", "ZENITH-ELECTRONICS" },
	{ 0x0080F8, "MIZAR, INC.", "MIZAR" },
	{ 0x0080F9, "HEURIKON CORPORATION", "HEURIKON" },
	{ 0x0080FA, "RWT GMBH", "RWT" },
	{ 0x0080FB, "BVM LIMITED", "BVM" },
	{ 0x0080FC, "AVATAR CORPORATION", "AVATAR" },
	{ 0x0080FD, "EXSCEED CORPRATION", "EXSCEED-CORPRATION" },
	{ 0x0080FE, "AZURE TECHNOLOGIES, INC.", "AZURE-TECHNOLOGIES" },
	{ 0x0080FF, "SOC. DE TELEINFORMATIQUE RTC", "SOC-DE-TELEINFORMATIQUE-RTC" },
	{ 0x0086A0, "PRIVATE", "PRIVATE" },
	{ 0x008865, "Apple", "APPLE" },
	{ 0x008B43, "RFTECH", "RFTECH" },
	{ 0x008C10, "Black Box Corp.", "BLACK-BOX" },
	{ 0x008C54, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x008CFA, "Inventec Corporation", "INVENTEC" },
	{ 0x008D4E, "CJSC NII STT", "CJSC-NII-STT" },
	{ 0x008DDA, "Link One Co., Ltd.", "LINK-ONE" },
	{ 0x008EF2, "NETGEAR INC.,", "NETGEAR" },
	{ 0x009000, "DIAMOND MULTIMEDIA", "DIAMOND-MULTIMEDIA" },
	{ 0x009001, "NISHIMU ELECTRONICS INDUSTRIES CO., LTD.", "NISHIMU-ELECTRONICS-INDUSTRIES" },
	{ 0x009002, "ALLGON AB", "ALLGON-AB" },
	{ 0x009003, "APLIO", "APLIO" },
	{ 0x009004, "3COM EUROPE LTD.", "3COM" },
	{ 0x009005, "PROTECH SYSTEMS CO., LTD.", "PROTECH" },
	{ 0x009006, "HAMAMATSU PHOTONICS K.K.", "HAMAMATSU-PHOTONICS-K-K" },
	{ 0x009007, "DOMEX TECHNOLOGY CORP.", "DOMEX" },
	{ 0x009008, "HanA Systems Inc.", "HANA" },
	{ 0x009009, "I Controls, Inc.", "I-CONTROLS" },
	{ 0x00900A, "PROTON ELECTRONIC INDUSTRIAL CO., LTD.", "PROTON-INDUSTRIAL" },
	{ 0x00900B, "LANNER ELECTRONICS, INC.", "LANNER-ELECTRONICS" },
	{ 0x00900C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00900D, "Overland Storage Inc.", "OVERLAND-STORAGE" },
	{ 0x00900E, "HANDLINK TECHNOLOGIES, INC.", "HANDLINK-TECHNOLOGIES" },
	{ 0x00900F, "KAWASAKI HEAVY INDUSTRIES, LTD", "KAWASAKI-HEAVY-INDUSTRIES" },
	{ 0x009010, "SIMULATION LABORATORIES, INC.", "SIMULATION-LABORATORIES" },
	{ 0x009011, "WAVTrace, Inc.", "WAVTRACE" },
	{ 0x009012, "GLOBESPAN SEMICONDUCTOR, INC.", "GLOBESPAN-SEMICONDUCTOR" },
	{ 0x009013, "SAMSAN CORP.", "SAMSAN" },
	{ 0x009014, "ROTORK INSTRUMENTS, LTD.", "ROTORK-INSTRUMENTS" },
	{ 0x009015, "CENTIGRAM COMMUNICATIONS CORP.", "CENTIGRAM-COMMUNICATION" },
	{ 0x009016, "ZAC", "ZAC" },
	{ 0x009017, "Zypcom, Inc", "ZYPCOM" },
	{ 0x009018, "ITO ELECTRIC INDUSTRY CO, LTD.", "ITO-ELECTRIC-INDUSTRY" },
	{ 0x009019, "HERMES ELECTRONICS CO., LTD.", "HERMES-ELECTRONICS" },
	{ 0x00901A, "UNISPHERE SOLUTIONS", "UNISPHERE-SOLUTIONS" },
	{ 0x00901B, "DIGITAL CONTROLS", "DIGITAL-CONTROLS" },
	{ 0x00901C, "mps Software Gmbh", "MPS-SOFTWARE" },
	{ 0x00901D, "PEC (NZ) LTD.", "PEC-NZ" },
	{ 0x00901E, "Selesta Ingegneria S.p.A.", "SELESTA-INGEGNERIA-S-P-A" },
	{ 0x00901F, "ADTEC PRODUCTIONS, INC.", "ADTEC-PRODUCTIONS" },
	{ 0x009020, "PHILIPS ANALYTICAL X-RAY B.V.", "PHILIPS-ANALYTICAL-X-RAY" },
	{ 0x009021, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x009022, "IVEX", "IVEX" },
	{ 0x009023, "ZILOG INC.", "ZILOG" },
	{ 0x009024, "PIPELINKS, INC.", "PIPELINKS" },
	{ 0x009025, "BAE Systems Australia (Electronic Systems) Pty Ltd", "BAE-AUSTRALIA-PTY" },
	{ 0x009026, "ADVANCED SWITCHING COMMUNICATIONS, INC.", "ADVANCED-SWITCHING-COMMUNICATION" },
	{ 0x009027, "INTEL CORPORATION", "INTEL" },
	{ 0x009028, "NIPPON SIGNAL CO., LTD.", "NIPPON-SIGNAL" },
	{ 0x009029, "CRYPTO AG", "CRYPTO" },
	{ 0x00902A, "COMMUNICATION DEVICES, INC.", "COMMUNICATION-DEVICES" },
	{ 0x00902B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00902C, "DATA & CONTROL EQUIPMENT LTD.", "DATA-CONTROL-EQUIPMENT" },
	{ 0x00902D, "DATA ELECTRONICS (AUST.) PTY, LTD.", "DATA-ELECTRONICS-AUST-PTY" },
	{ 0x00902E, "NAMCO LIMITED", "NAMCO" },
	{ 0x00902F, "NETCORE SYSTEMS, INC.", "NETCORE" },
	{ 0x009030, "HONEYWELL-DATING", "HONEYWELL-DATING" },
	{ 0x009031, "MYSTICOM, LTD.", "MYSTICOM" },
	{ 0x009032, "PELCOMBE GROUP LTD.", "PELCOMBE-GROUP" },
	{ 0x009033, "INNOVAPHONE AG", "INNOVAPHONE" },
	{ 0x009034, "IMAGIC, INC.", "IMAGIC" },
	{ 0x009035, "ALPHA TELECOM, INC.", "ALPHA-TELECOM" },
	{ 0x009036, "ens, inc.", "ENS" },
	{ 0x009037, "ACUCOMM, INC.", "ACUCOMM" },
	{ 0x009038, "FOUNTAIN TECHNOLOGIES, INC.", "FOUNTAIN-TECHNOLOGIES" },
	{ 0x009039, "SHASTA NETWORKS", "SHASTA-NETWORKS" },
	{ 0x00903A, "NIHON MEDIA TOOL INC.", "NIHON-MEDIA-TOOL" },
	{ 0x00903B, "TriEMS Research Lab, Inc.", "TRIEMS-RESEARCH-LAB" },
	{ 0x00903C, "ATLANTIC NETWORK SYSTEMS", "ATLANTIC-NETWORK" },
	{ 0x00903D, "BIOPAC SYSTEMS, INC.", "BIOPAC" },
	{ 0x00903E, "N.V. PHILIPS INDUSTRIAL ACTIVITIES", "N-V-PHILIPS-INDUSTRIAL-ACTIVITIES" },
	{ 0x00903F, "AZTEC RADIOMEDIA", "AZTEC-RADIOMEDIA" },
	{ 0x009040, "Siemens Network Convergence LLC", "SIEMENS-NETWORK-CONVERGENCE-LLC" },
	{ 0x009041, "APPLIED DIGITAL ACCESS", "APPLIED-DIGITAL-ACCESS" },
	{ 0x009042, "ECCS, Inc.", "ECCS" },
	{ 0x009043, "Tattile SRL", "TATTILE-SRL" },
	{ 0x009044, "ASSURED DIGITAL, INC.", "ASSURED-DIGITAL" },
	{ 0x009045, "Marconi Communications", "MARCONI-COMMUNICATION" },
	{ 0x009046, "DEXDYNE, LTD.", "DEXDYNE" },
	{ 0x009047, "GIGA FAST E. LTD.", "GIGA-FAST-E" },
	{ 0x009048, "ZEAL CORPORATION", "ZEAL" },
	{ 0x009049, "ENTRIDIA CORPORATION", "ENTRIDIA" },
	{ 0x00904A, "CONCUR SYSTEM TECHNOLOGIES", "CONCUR-SYSTEM-TECHNOLOGIES" },
	{ 0x00904B, "GemTek Technology Co., Ltd.", "GEMTEK" },
	{ 0x00904C, "EPIGRAM, INC.", "EPIGRAM" },
	{ 0x00904D, "SPEC S.A.", "SPEC" },
	{ 0x00904E, "DELEM BV", "DELEM" },
	{ 0x00904F, "ABB POWER T&D COMPANY, INC.", "ABB-POWER-T-D-COMPANY" },
	{ 0x009050, "TELESTE OY", "TELESTE-OY" },
	{ 0x009051, "ULTIMATE TECHNOLOGY CORP.", "ULTIMATE" },
	{ 0x009052, "SELCOM ELETTRONICA S.R.L.", "SELCOM-ELETTRONICA-S-R-L" },
	{ 0x009053, "DAEWOO ELECTRONICS CO., LTD.", "DAEWOO-ELECTRONICS" },
	{ 0x009054, "INNOVATIVE SEMICONDUCTORS, INC", "INNOVATIVE-SEMICONDUCTORS" },
	{ 0x009055, "PARKER HANNIFIN CORPORATION COMPUMOTOR DIVISION", "PARKER-HANNIFIN-COMPUMOTOR-DIVISION" },
	{ 0x009056, "TELESTREAM, INC.", "TELESTREAM" },
	{ 0x009057, "AANetcom, Inc.", "AANETCOM" },
	{ 0x009058, "Ultra Electronics Ltd., Command and Control Systems", "ULTRA-ELECTRONICS-COMMAND-AND-CONTROL" },
	{ 0x009059, "TELECOM DEVICE K.K.", "TELECOM-DEVICE-K-K" },
	{ 0x00905A, "DEARBORN GROUP, INC.", "DEARBORN-GROUP" },
	{ 0x00905B, "RAYMOND AND LAE ENGINEERING", "RAYMOND-AND-LAE-ENGINEERING" },
	{ 0x00905C, "EDMI", "EDMI" },
	{ 0x00905D, "NETCOM SICHERHEITSTECHNIK GmbH", "NETCOM-SICHERHEITSTECHNIK" },
	{ 0x00905E, "RAULAND-BORG CORPORATION", "RAULAND-BORG" },
	{ 0x00905F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x009060, "SYSTEM CREATE CORP.", "SYSTEM-CREATE" },
	{ 0x009061, "PACIFIC RESEARCH & ENGINEERING CORPORATION", "PACIFIC-RESEARCH-ENGINEERING" },
	{ 0x009062, "ICP VORTEX COMPUTERSYSTEME GmbH", "ICP-VORTEX-COMPUTERSYSTEME" },
	{ 0x009063, "COHERENT COMMUNICATIONS SYSTEMS CORPORATION", "COHERENT-COMMUNICATION" },
	{ 0x009064, "Thomson Inc.", "THOMSON" },
	{ 0x009065, "FINISAR CORPORATION", "FINISAR" },
	{ 0x009066, "Troika Networks, Inc.", "TROIKA-NETWORKS" },
	{ 0x009067, "WalkAbout Computers, Inc.", "WALKABOUT-COMPUTERS" },
	{ 0x009068, "DVT CORP.", "DVT" },
	{ 0x009069, "JUNIPER NETWORKS, INC.", "JUNIPER-NETWORKS" },
	{ 0x00906A, "TURNSTONE SYSTEMS, INC.", "TURNSTONE" },
	{ 0x00906B, "APPLIED RESOURCES, INC.", "APPLIED-RESOURCES" },
	{ 0x00906C, "Sartorius Hamburg GmbH", "SARTORIUS-HAMBURG" },
	{ 0x00906D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00906E, "PRAXON, INC.", "PRAXON" },
	{ 0x00906F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x009070, "NEO NETWORKS, INC.", "NEO-NETWORKS" },
	{ 0x009071, "Applied Innovation Inc.", "APPLIED-INNOVATION" },
	{ 0x009072, "SIMRAD AS", "SIMRAD-AS" },
	{ 0x009073, "GAIO TECHNOLOGY", "GAIO" },
	{ 0x009074, "ARGON NETWORKS, INC.", "ARGON-NETWORKS" },
	{ 0x009075, "NEC DO BRASIL S.A.", "NEC-DO-BRASIL" },
	{ 0x009076, "FMT AIRCRAFT GATE SUPPORT SYSTEMS AB", "FMT-AIRCRAFT-GATE-SUPPORT-AB" },
	{ 0x009077, "ADVANCED FIBRE COMMUNICATIONS", "ADVANCED-FIBRE-COMMUNICATION" },
	{ 0x009078, "MER TELEMANAGEMENT SOLUTIONS, LTD.", "MER-TELEMANAGEMENT-SOLUTIONS" },
	{ 0x009079, "ClearOne, Inc.", "CLEARONE" },
	{ 0x00907A, "Spectralink, Inc", "SPECTRALINK" },
	{ 0x00907B, "E-TECH, INC.", "E-TECH" },
	{ 0x00907C, "DIGITALCAST, INC.", "DIGITALCAST" },
	{ 0x00907D, "Lake Communications", "LAKE-COMMUNICATION" },
	{ 0x00907E, "VETRONIX CORP.", "VETRONIX" },
	{ 0x00907F, "WatchGuard Technologies, Inc.", "WATCHGUARD-TECHNOLOGIES" },
	{ 0x009080, "NOT LIMITED, INC.", "NOT" },
	{ 0x009081, "ALOHA NETWORKS, INC.", "ALOHA-NETWORKS" },
	{ 0x009082, "FORCE INSTITUTE", "FORCE-INSTITUTE" },
	{ 0x009083, "TURBO COMMUNICATION, INC.", "TURBO-COMMUNICATION" },
	{ 0x009084, "ATECH SYSTEM", "ATECH-SYSTEM" },
	{ 0x009085, "GOLDEN ENTERPRISES, INC.", "GOLDEN-ENTERPRISES" },
	{ 0x009086, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x009087, "ITIS", "ITIS" },
	{ 0x009088, "BAXALL SECURITY LTD.", "BAXALL-SECURITY" },
	{ 0x009089, "SOFTCOM MICROSYSTEMS, INC.", "SOFTCOM-MICROSYSTEMS" },
	{ 0x00908A, "BAYLY COMMUNICATIONS, INC.", "BAYLY-COMMUNICATION" },
	{ 0x00908B, "Tattile SRL", "TATTILE-SRL" },
	{ 0x00908C, "ETREND ELECTRONICS, INC.", "ETREND-ELECTRONICS" },
	{ 0x00908D, "VICKERS ELECTRONICS SYSTEMS", "VICKERS-ELECTRONICS" },
	{ 0x00908E, "Nortel Networks Broadband Access", "NORTEL-NETWORKS-BROADBAND-ACCESS" },
	{ 0x00908F, "AUDIO CODES LTD.", "AUDIO-CODES" },
	{ 0x009090, "I-BUS", "I-BUS" },
	{ 0x009091, "DigitalScape, Inc.", "DIGITALSCAPE" },
	{ 0x009092, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x009093, "NANAO CORPORATION", "NANAO" },
	{ 0x009094, "OSPREY TECHNOLOGIES, INC.", "OSPREY-TECHNOLOGIES" },
	{ 0x009095, "UNIVERSAL AVIONICS", "UNIVERSAL-AVIONICS" },
	{ 0x009096, "ASKEY COMPUTER CORP.", "ASKEY" },
	{ 0x009097, "Sycamore Networks", "SYCAMORE-NETWORKS" },
	{ 0x009098, "SBC DESIGNS, INC.", "SBC-DESIGNS" },
	{ 0x009099, "ALLIED TELESIS, K.K.", "ALLIED-TELESIS-K-K" },
	{ 0x00909A, "ONE WORLD SYSTEMS, INC.", "ONE-WORLD" },
	{ 0x00909B, "MARKEM-IMAJE", "MARKEM-IMAJE" },
	{ 0x00909C, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00909D, "NovaTech Process Solutions, LLC", "NOVATECH-PROCESS-SOLUTIONS-LLC" },
	{ 0x00909E, "Critical IO, LLC", "CRITICAL-IO-LLC" },
	{ 0x00909F, "DIGI-DATA CORPORATION", "DIGI-DATA" },
	{ 0x0090A0, "8X8 INC.", "8X8" },
	{ 0x0090A1, "Flying Pig Systems/High End Systems Inc.", "FLYING-PIG-/HIGH-END" },
	{ 0x0090A2, "CYBERTAN TECHNOLOGY, INC.", "CYBERTAN" },
	{ 0x0090A3, "Corecess Inc.", "CORECESS" },
	{ 0x0090A4, "ALTIGA NETWORKS", "ALTIGA-NETWORKS" },
	{ 0x0090A5, "SPECTRA LOGIC", "SPECTRA-LOGIC" },
	{ 0x0090A6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0090A7, "CLIENTEC CORPORATION", "CLIENTEC" },
	{ 0x0090A8, "NineTiles Networks, Ltd.", "NINETILES-NETWORKS" },
	{ 0x0090A9, "WESTERN DIGITAL", "WESTERN-DIGITAL" },
	{ 0x0090AA, "INDIGO ACTIVE VISION SYSTEMS LIMITED", "INDIGO-ACTIVE-VISION" },
	{ 0x0090AB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0090AC, "OPTIVISION, INC.", "OPTIVISION" },
	{ 0x0090AD, "ASPECT ELECTRONICS, INC.", "ASPECT-ELECTRONICS" },
	{ 0x0090AE, "ITALTEL S.p.A.", "ITALTEL-S-P-A" },
	{ 0x0090AF, "J. MORITA MFG. CORP.", "J-MORITA-MFG" },
	{ 0x0090B0, "VADEM", "VADEM" },
	{ 0x0090B1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0090B2, "AVICI SYSTEMS INC.", "AVICI" },
	{ 0x0090B3, "AGRANAT SYSTEMS", "AGRANAT" },
	{ 0x0090B4, "WILLOWBROOK TECHNOLOGIES", "WILLOWBROOK-TECHNOLOGIES" },
	{ 0x0090B5, "NIKON CORPORATION", "NIKON" },
	{ 0x0090B6, "FIBEX SYSTEMS", "FIBEX" },
	{ 0x0090B7, "DIGITAL LIGHTWAVE, INC.", "DIGITAL-LIGHTWAVE" },
	{ 0x0090B8, "ROHDE & SCHWARZ GMBH & CO. KG", "ROHDE-SCHWARZ" },
	{ 0x0090B9, "BERAN INSTRUMENTS LTD.", "BERAN-INSTRUMENTS" },
	{ 0x0090BA, "VALID NETWORKS, INC.", "VALID-NETWORKS" },
	{ 0x0090BB, "TAINET COMMUNICATION SYSTEM Corp.", "TAINET-COMMUNICATION-SYSTEM" },
	{ 0x0090BC, "TELEMANN CO., LTD.", "TELEMANN" },
	{ 0x0090BD, "OMNIA COMMUNICATIONS, INC.", "OMNIA-COMMUNICATION" },
	{ 0x0090BE, "IBC/INTEGRATED BUSINESS COMPUTERS", "IBC/INTEGRATED-BUSINESS-COMPUTERS" },
	{ 0x0090BF, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0090C0, "K.J. LAW ENGINEERS, INC.", "K-J-LAW-ENGINEERS" },
	{ 0x0090C1, "Peco II, Inc.", "PECO-II" },
	{ 0x0090C2, "JK microsystems, Inc.", "JK-MICROSYSTEMS" },
	{ 0x0090C3, "TOPIC SEMICONDUCTOR CORP.", "TOPIC-SEMICONDUCTOR" },
	{ 0x0090C4, "JAVELIN SYSTEMS, INC.", "JAVELIN" },
	{ 0x0090C5, "INTERNET MAGIC, INC.", "INTERNET-MAGIC" },
	{ 0x0090C6, "OPTIM SYSTEMS, INC.", "OPTIM" },
	{ 0x0090C7, "ICOM INC.", "ICOM" },
	{ 0x0090C8, "WAVERIDER COMMUNICATIONS (CANADA) INC.", "WAVERIDER-COMMUNICATION-CANADA" },
	{ 0x0090C9, "DPAC Technologies", "DPAC-TECHNOLOGIES" },
	{ 0x0090CA, "ACCORD VIDEO TELECOMMUNICATIONS, LTD.", "ACCORD-VIDEO-TELECOMMUNICATIONS" },
	{ 0x0090CB, "Wireless OnLine, Inc.", "WIRELESS-ONLINE" },
	{ 0x0090CC, "Planex Communications", "PLANEX-COMMUNICATION" },
	{ 0x0090CD, "ENT-EMPRESA NACIONAL DE TELECOMMUNICACOES, S.A.", "ENT-EMPRESA-NACIONAL-DE-TELECOMMUNICACOES" },
	{ 0x0090CE, "TETRA GmbH", "TETRA" },
	{ 0x0090CF, "NORTEL", "NORTEL" },
	{ 0x0090D0, "Thomson Telecom Belgium", "THOMSON-TELECOM-BELGIUM" },
	{ 0x0090D1, "LEICHU ENTERPRISE CO., LTD.", "LEICHU" },
	{ 0x0090D2, "ARTEL VIDEO SYSTEMS", "ARTEL-VIDEO" },
	{ 0x0090D3, "GIESECKE & DEVRIENT GmbH", "GIESECKE-DEVRIENT" },
	{ 0x0090D4, "BindView Development Corp.", "BINDVIEW-DEVELOPMENT" },
	{ 0x0090D5, "EUPHONIX, INC.", "EUPHONIX" },
	{ 0x0090D6, "CRYSTAL GROUP", "CRYSTAL-GROUP" },
	{ 0x0090D7, "NetBoost Corp.", "NETBOOST" },
	{ 0x0090D8, "WHITECROSS SYSTEMS", "WHITECROSS" },
	{ 0x0090D9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0090DA, "DYNARC, INC.", "DYNARC" },
	{ 0x0090DB, "NEXT LEVEL COMMUNICATIONS", "NEXT-LEVEL-COMMUNICATION" },
	{ 0x0090DC, "TECO INFORMATION SYSTEMS", "TECO-INFORMATION" },
	{ 0x0090DD, "MIHARU COMMUNICATIONS Inc", "MIHARU-COMMUNICATION" },
	{ 0x0090DE, "CARDKEY SYSTEMS, INC.", "CARDKEY" },
	{ 0x0090DF, "MITSUBISHI CHEMICAL AMERICA, INC.", "MITSUBISHI-CHEMICAL-AMERICA" },
	{ 0x0090E0, "SYSTRAN CORP.", "SYSTRAN" },
	{ 0x0090E1, "TELENA S.P.A.", "TELENA-S-P-A" },
	{ 0x0090E2, "DISTRIBUTED PROCESSING TECHNOLOGY", "DISTRIBUTED-PROCESSING" },
	{ 0x0090E3, "AVEX ELECTRONICS INC.", "AVEX-ELECTRONICS" },
	{ 0x0090E4, "NEC AMERICA, INC.", "NEC-AMERICA" },
	{ 0x0090E5, "TEKNEMA, INC.", "TEKNEMA" },
	{ 0x0090E6, "ALi Corporation", "ALI" },
	{ 0x0090E7, "HORSCH ELEKTRONIK AG", "HORSCH-ELEKTRONIK" },
	{ 0x0090E8, "MOXA TECHNOLOGIES CORP., LTD.", "MOXA-TECHNOLOGIES" },
	{ 0x0090E9, "JANZ COMPUTER AG", "JANZ" },
	{ 0x0090EA, "ALPHA TECHNOLOGIES, INC.", "ALPHA-TECHNOLOGIES" },
	{ 0x0090EB, "SENTRY TELECOM SYSTEMS", "SENTRY-TELECOM" },
	{ 0x0090EC, "PYRESCOM", "PYRESCOM" },
	{ 0x0090ED, "CENTRAL SYSTEM RESEARCH CO., LTD.", "CENTRAL-SYSTEM-RESEARCH" },
	{ 0x0090EE, "PERSONAL COMMUNICATIONS TECHNOLOGIES", "PERSONAL-COMMUNICATION-TECHNOLOGIES" },
	{ 0x0090EF, "INTEGRIX, INC.", "INTEGRIX" },
	{ 0x0090F0, "Harmonic Video Systems Ltd.", "HARMONIC-VIDEO" },
	{ 0x0090F1, "DOT HILL SYSTEMS CORPORATION", "DOT-HILL" },
	{ 0x0090F2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0090F3, "ASPECT COMMUNICATIONS", "ASPECT-COMMUNICATION" },
	{ 0x0090F4, "LIGHTNING INSTRUMENTATION", "LIGHTNING-INSTRUMENTATION" },
	{ 0x0090F5, "CLEVO CO.", "CLEVO" },
	{ 0x0090F6, "ESCALATE NETWORKS, INC.", "ESCALATE-NETWORKS" },
	{ 0x0090F7, "NBASE COMMUNICATIONS LTD.", "NBASE-COMMUNICATION" },
	{ 0x0090F8, "MEDIATRIX TELECOM", "MEDIATRIX-TELECOM" },
	{ 0x0090F9, "LEITCH", "LEITCH" },
	{ 0x0090FA, "Emulex Corporation", "EMULEX" },
	{ 0x0090FB, "PORTWELL, INC.", "PORTWELL" },
	{ 0x0090FC, "NETWORK COMPUTING DEVICES", "NETWORK-COMPUTING-DEVICES" },
	{ 0x0090FD, "CopperCom, Inc.", "COPPERCOM" },
	{ 0x0090FE, "ELECOM CO., LTD. (LANEED DIV.)", "ELECOM-LANEED-DIV" },
	{ 0x0090FF, "TELLUS TECHNOLOGY INC.", "TELLUS" },
	{ 0x0091D6, "Crystal Group, Inc.", "CRYSTAL-GROUP" },
	{ 0x0091FA, "Synapse Product Development", "SYNAPSE-PRODUCT-DEVELOPMENT" },
	{ 0x0092FA, "SHENZHEN WISKY TECHNOLOGY CO.,LTD", "SHENZHEN-WISKY" },
	{ 0x009363, "Uni-Link Technology Co., Ltd.", "UNI-LINK" },
	{ 0x009569, "LSD Science and Technology Co.,Ltd.", "LSD-SCIENCE-AND" },
	{ 0x0097FF, "Heimann Sensor GmbH", "HEIMANN-SENSOR" },
	{ 0x009C02, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x009D8E, "CARDIAC RECORDERS, INC.", "CARDIAC-RECORDERS" },
	{ 0x009EC8, "Beijing Xiaomi Electronic Products Co., Ltd.", "BEIJING-XIAOMI-PRODUCTS" },
	{ 0x00A000, "CENTILLION NETWORKS, INC.", "CENTILLION-NETWORKS" },
	{ 0x00A001, "DRS Signal Solutions", "DRS-SIGNAL-SOLUTIONS" },
	{ 0x00A002, "LEEDS & NORTHRUP AUSTRALIA PTY LTD", "LEEDS-NORTHRUP-AUSTRALIA-PTY" },
	{ 0x00A003, "Siemens Switzerland Ltd., I B T HVP", "SIEMENS-SWITZERLAND-I-B-T-HVP" },
	{ 0x00A004, "NETPOWER, INC.", "NETPOWER" },
	{ 0x00A005, "DANIEL INSTRUMENTS, LTD.", "DANIEL-INSTRUMENTS" },
	{ 0x00A006, "IMAGE DATA PROCESSING SYSTEM GROUP", "IMAGE-DATA-PROCESSING-SYSTEM-GROUP" },
	{ 0x00A007, "APEXX TECHNOLOGY, INC.", "APEXX" },
	{ 0x00A008, "NETCORP", "NETCORP" },
	{ 0x00A009, "WHITETREE NETWORK", "WHITETREE-NETWORK" },
	{ 0x00A00A, "Airspan", "AIRSPAN" },
	{ 0x00A00B, "COMPUTEX CO., LTD.", "COMPUTEX" },
	{ 0x00A00C, "KINGMAX TECHNOLOGY, INC.", "KINGMAX" },
	{ 0x00A00D, "THE PANDA PROJECT", "THE-PANDA-PROJECT" },
	{ 0x00A00E, "VISUAL NETWORKS, INC.", "VISUAL-NETWORKS" },
	{ 0x00A00F, "Broadband Technologies", "BROADBAND-TECHNOLOGIES" },
	{ 0x00A010, "SYSLOGIC DATENTECHNIK AG", "SYSLOGIC-DATENTECHNIK" },
	{ 0x00A011, "MUTOH INDUSTRIES LTD.", "MUTOH-INDUSTRIES" },
	{ 0x00A012, "Telco Systems, Inc.", "TELCO" },
	{ 0x00A013, "TELTREND LTD.", "TELTREND" },
	{ 0x00A014, "CSIR", "CSIR" },
	{ 0x00A015, "WYLE", "WYLE" },
	{ 0x00A016, "MICROPOLIS CORP.", "MICROPOLIS" },
	{ 0x00A017, "J B M CORPORATION", "J-B-M" },
	{ 0x00A018, "CREATIVE CONTROLLERS, INC.", "CREATIVE-CONTROLLERS" },
	{ 0x00A019, "NEBULA CONSULTANTS, INC.", "NEBULA-CONSULTANTS" },
	{ 0x00A01A, "BINAR ELEKTRONIK AB", "BINAR-ELEKTRONIK-AB" },
	{ 0x00A01B, "PREMISYS COMMUNICATIONS, INC.", "PREMISYS-COMMUNICATION" },
	{ 0x00A01C, "NASCENT NETWORKS CORPORATION", "NASCENT-NETWORKS" },
	{ 0x00A01D, "Red Lion Controls, LP", "RED-LION-CONTROLS-LP" },
	{ 0x00A01E, "EST CORPORATION", "EST" },
	{ 0x00A01F, "TRICORD SYSTEMS, INC.", "TRICORD" },
	{ 0x00A020, "CITICORP/TTI", "CITICORP/TTI" },
	{ 0x00A021, "General Dynamics", "GENERAL-DYNAMICS" },
	{ 0x00A022, "CENTRE FOR DEVELOPMENT OF ADVANCED COMPUTING", "CENTRE-FOR-DEVELOPMENT-OF-ADVANCED-COMPUTING" },
	{ 0x00A023, "APPLIED CREATIVE TECHNOLOGY, INC.", "APPLIED-CREATIVE" },
	{ 0x00A024, "3COM CORPORATION", "3COM" },
	{ 0x00A025, "REDCOM LABS INC.", "REDCOM-LABS" },
	{ 0x00A026, "TELDAT, S.A.", "TELDAT" },
	{ 0x00A027, "FIREPOWER SYSTEMS, INC.", "FIREPOWER" },
	{ 0x00A028, "CONNER PERIPHERALS", "CONNER-PERIPHERALS" },
	{ 0x00A029, "COULTER CORPORATION", "COULTER" },
	{ 0x00A02A, "TRANCELL SYSTEMS", "TRANCELL" },
	{ 0x00A02B, "TRANSITIONS RESEARCH CORP.", "TRANSITIONS-RESEARCH" },
	{ 0x00A02C, "interWAVE Communications", "INTERWAVE-COMMUNICATION" },
	{ 0x00A02D, "1394 Trade Association", "1394-TRADE-ASSOCIATION" },
	{ 0x00A02E, "BRAND COMMUNICATIONS, LTD.", "BRAND-COMMUNICATION" },
	{ 0x00A02F, "PIRELLI CAVI", "PIRELLI-CAVI" },
	{ 0x00A030, "CAPTOR NV/SA", "CAPTOR-NV/" },
	{ 0x00A031, "HAZELTINE CORPORATION, MS 1-17", "HAZELTINE-MS-1-17" },
	{ 0x00A032, "GES SINGAPORE PTE. LTD.", "GES-SINGAPORE-PTE" },
	{ 0x00A033, "imc MeBsysteme GmbH", "IMC-MEBSYSTEME" },
	{ 0x00A034, "AXEL", "AXEL" },
	{ 0x00A035, "CYLINK CORPORATION", "CYLINK" },
	{ 0x00A036, "APPLIED NETWORK TECHNOLOGY", "APPLIED-NETWORK" },
	{ 0x00A037, "Mindray DS USA, Inc.", "MINDRAY-DS-USA" },
	{ 0x00A038, "EMAIL ELECTRONICS", "EMAIL-ELECTRONICS" },
	{ 0x00A039, "ROSS TECHNOLOGY, INC.", "ROSS" },
	{ 0x00A03A, "KUBOTEK CORPORATION", "KUBOTEK" },
	{ 0x00A03B, "TOSHIN ELECTRIC CO., LTD.", "TOSHIN-ELECTRIC" },
	{ 0x00A03C, "EG&G NUCLEAR INSTRUMENTS", "EG-G-NUCLEAR-INSTRUMENTS" },
	{ 0x00A03D, "OPTO-22", "OPTO-22" },
	{ 0x00A03E, "ATM FORUM", "ATM-FORUM" },
	{ 0x00A03F, "COMPUTER SOCIETY MICROPROCESSOR & MICROPROCESSOR STANDARDS C", "SOCIETY-MICROPROCESSOR-MICROPROCESSOR-STANDARDS-C" },
	{ 0x00A040, "Apple", "APPLE" },
	{ 0x00A041, "INFICON", "INFICON" },
	{ 0x00A042, "SPUR PRODUCTS CORP.", "SPUR-PRODUCTS" },
	{ 0x00A043, "AMERICAN TECHNOLOGY LABS, INC.", "AMERICAN-LABS" },
	{ 0x00A044, "NTT IT CO., LTD.", "NTT-IT" },
	{ 0x00A045, "PHOENIX CONTACT GMBH & CO.", "PHOENIX-CONTACT" },
	{ 0x00A046, "SCITEX CORP. LTD.", "SCITEX" },
	{ 0x00A047, "INTEGRATED FITNESS CORP.", "INTEGRATED-FITNESS" },
	{ 0x00A048, "QUESTECH, LTD.", "QUESTECH" },
	{ 0x00A049, "DIGITECH INDUSTRIES, INC.", "DIGITECH-INDUSTRIES" },
	{ 0x00A04A, "NISSHIN ELECTRIC CO., LTD.", "NISSHIN-ELECTRIC" },
	{ 0x00A04B, "TFL LAN INC.", "TFL-LAN" },
	{ 0x00A04C, "INNOVATIVE SYSTEMS & TECHNOLOGIES, INC.", "INNOVATIVE-TECHNOLOGIES" },
	{ 0x00A04D, "EDA INSTRUMENTS, INC.", "EDA-INSTRUMENTS" },
	{ 0x00A04E, "VOELKER TECHNOLOGIES, INC.", "VOELKER-TECHNOLOGIES" },
	{ 0x00A04F, "AMERITEC CORP.", "AMERITEC" },
	{ 0x00A050, "CYPRESS SEMICONDUCTOR", "CYPRESS-SEMICONDUCTOR" },
	{ 0x00A051, "ANGIA COMMUNICATIONS. INC.", "ANGIA-COMMUNICATION" },
	{ 0x00A052, "STANILITE ELECTRONICS PTY. LTD", "STANILITE-ELECTRONICS-PTY" },
	{ 0x00A053, "COMPACT DEVICES, INC.", "COMPACT-DEVICES" },
	{ 0x00A054, "PRIVATE", "PRIVATE" },
	{ 0x00A055, "Data Device Corporation", "DATA-DEVICE" },
	{ 0x00A056, "MICROPROSS", "MICROPROSS" },
	{ 0x00A057, "LANCOM Systems GmbH", "LANCOM" },
	{ 0x00A058, "GLORY, LTD.", "GLORY" },
	{ 0x00A059, "HAMILTON HALLMARK", "HAMILTON-HALLMARK" },
	{ 0x00A05A, "KOFAX IMAGE PRODUCTS", "KOFAX-IMAGE-PRODUCTS" },
	{ 0x00A05B, "MARQUIP, INC.", "MARQUIP" },
	{ 0x00A05C, "INVENTORY CONVERSION, INC./", "INVENTORY-CONVERSION-/" },
	{ 0x00A05D, "CS COMPUTER SYSTEME GmbH", "CS-SYSTEME" },
	{ 0x00A05E, "MYRIAD LOGIC INC.", "MYRIAD-LOGIC" },
	{ 0x00A05F, "BTG Electronics Design BV", "BTG-ELECTRONICS-DESIGN" },
	{ 0x00A060, "ACER PERIPHERALS, INC.", "ACER-PERIPHERALS" },
	{ 0x00A061, "PURITAN BENNETT", "PURITAN-BENNETT" },
	{ 0x00A062, "AES PRODATA", "AES-PRODATA" },
	{ 0x00A063, "JRL SYSTEMS, INC.", "JRL" },
	{ 0x00A064, "KVB/ANALECT", "KVB/ANALECT" },
	{ 0x00A065, "Symantec Corporation", "SYMANTEC" },
	{ 0x00A066, "ISA CO., LTD.", "ISA" },
	{ 0x00A067, "NETWORK SERVICES GROUP", "NETWORK-SERVICES-GROUP" },
	{ 0x00A068, "BHP LIMITED", "BHP" },
	{ 0x00A069, "Symmetricom, Inc.", "SYMMETRICOM" },
	{ 0x00A06A, "Verilink Corporation", "VERILINK" },
	{ 0x00A06B, "DMS DORSCH MIKROSYSTEM GMBH", "DMS-DORSCH-MIKROSYSTEM" },
	{ 0x00A06C, "SHINDENGEN ELECTRIC MFG. CO., LTD.", "SHINDENGEN-ELECTRIC-MFG" },
	{ 0x00A06D, "MANNESMANN TALLY CORPORATION", "MANNESMANN-TALLY" },
	{ 0x00A06E, "AUSTRON, INC.", "AUSTRON" },
	{ 0x00A06F, "THE APPCON GROUP, INC.", "THE-APPCON-GROUP" },
	{ 0x00A070, "COASTCOM", "COASTCOM" },
	{ 0x00A071, "VIDEO LOTTERY TECHNOLOGIES,INC", "VIDEO-LOTTERY-TECHNOLOGIES" },
	{ 0x00A072, "OVATION SYSTEMS LTD.", "OVATION" },
	{ 0x00A073, "COM21, INC.", "COM21" },
	{ 0x00A074, "PERCEPTION TECHNOLOGY", "PERCEPTION" },
	{ 0x00A075, "MICRON TECHNOLOGY, INC.", "MICRON" },
	{ 0x00A076, "CARDWARE LAB, INC.", "CARDWARE-LAB" },
	{ 0x00A077, "FUJITSU NEXION, INC.", "FUJITSU-NEXION" },
	{ 0x00A078, "Marconi Communications", "MARCONI-COMMUNICATION" },
	{ 0x00A079, "ALPS ELECTRIC (USA), INC.", "ALPS-ELECTRIC-USA" },
	{ 0x00A07A, "ADVANCED PERIPHERALS TECHNOLOGIES, INC.", "ADVANCED-PERIPHERALS-TECHNOLOGIES" },
	{ 0x00A07B, "DAWN COMPUTER INCORPORATION", "DAWN-INCORPORATION" },
	{ 0x00A07C, "TONYANG NYLON CO., LTD.", "TONYANG-NYLON" },
	{ 0x00A07D, "SEEQ TECHNOLOGY, INC.", "SEEQ" },
	{ 0x00A07E, "AVID TECHNOLOGY, INC.", "AVID" },
	{ 0x00A07F, "GSM-SYNTEL, LTD.", "GSM-SYNTEL" },
	{ 0x00A080, "Tattile SRL", "TATTILE-SRL" },
	{ 0x00A081, "ALCATEL DATA NETWORKS", "ALCATEL-DATA-NETWORKS" },
	{ 0x00A082, "NKT ELEKTRONIK A/S", "NKT-ELEKTRONIK-A/S" },
	{ 0x00A083, "ASIMMPHONY TURKEY", "ASIMMPHONY-TURKEY" },
	{ 0x00A084, "Dataplex Pty Ltd", "DATAPLEX-PTY" },
	{ 0x00A085, "PRIVATE", "PRIVATE" },
	{ 0x00A086, "AMBER WAVE SYSTEMS, INC.", "AMBER-WAVE" },
	{ 0x00A087, "Microsemi Corporation", "MICROSEMI" },
	{ 0x00A088, "ESSENTIAL COMMUNICATIONS", "ESSENTIAL-COMMUNICATION" },
	{ 0x00A089, "XPOINT TECHNOLOGIES, INC.", "XPOINT-TECHNOLOGIES" },
	{ 0x00A08A, "BROOKTROUT TECHNOLOGY, INC.", "BROOKTROUT" },
	{ 0x00A08B, "ASTON ELECTRONIC DESIGNS LTD.", "ASTON-DESIGNS" },
	{ 0x00A08C, "MultiMedia LANs, Inc.", "MULTIMEDIA-LANS" },
	{ 0x00A08D, "JACOMO CORPORATION", "JACOMO" },
	{ 0x00A08E, "Check Point Software Technologies", "CHECK-POINT-SOFTWARE-TECHNOLOGIES" },
	{ 0x00A08F, "DESKNET SYSTEMS, INC.", "DESKNET" },
	{ 0x00A090, "TimeStep Corporation", "TIMESTEP" },
	{ 0x00A091, "APPLICOM INTERNATIONAL", "APPLICOM" },
	{ 0x00A092, "H. BOLLMANN MANUFACTURERS, LTD", "H-BOLLMANN-MANUFACTURERS" },
	{ 0x00A093, "B/E AEROSPACE, Inc.", "B/E-AEROSPACE" },
	{ 0x00A094, "COMSAT CORPORATION", "COMSAT" },
	{ 0x00A095, "ACACIA NETWORKS, INC.", "ACACIA-NETWORKS" },
	{ 0x00A096, "MITSUMI ELECTRIC CO., LTD.", "MITSUMI-ELECTRIC" },
	{ 0x00A097, "JC INFORMATION SYSTEMS", "JC-INFORMATION" },
	{ 0x00A098, "NetApp", "NETAPP" },
	{ 0x00A099, "K-NET LTD.", "K-NET" },
	{ 0x00A09A, "NIHON KOHDEN AMERICA", "NIHON-KOHDEN-AMERICA" },
	{ 0x00A09B, "QPSX COMMUNICATIONS, LTD.", "QPSX-COMMUNICATION" },
	{ 0x00A09C, "Xyplex, Inc.", "XYPLEX" },
	{ 0x00A09D, "JOHNATHON FREEMAN TECHNOLOGIES", "JOHNATHON-FREEMAN-TECHNOLOGIES" },
	{ 0x00A09E, "ICTV", "ICTV" },
	{ 0x00A09F, "COMMVISION CORP.", "COMMVISION" },
	{ 0x00A0A0, "COMPACT DATA, LTD.", "COMPACT-DATA" },
	{ 0x00A0A1, "EPIC DATA INC.", "EPIC-DATA" },
	{ 0x00A0A2, "DIGICOM S.P.A.", "DIGICOM-S-P-A" },
	{ 0x00A0A3, "RELIABLE POWER METERS", "RELIABLE-POWER-METERS" },
	{ 0x00A0A4, "MICROS SYSTEMS, INC.", "MICROS" },
	{ 0x00A0A5, "TEKNOR MICROSYSTEME, INC.", "TEKNOR-MICROSYSTEME" },
	{ 0x00A0A6, "M.I. SYSTEMS, K.K.", "M-I-K-K" },
	{ 0x00A0A7, "VORAX CORPORATION", "VORAX" },
	{ 0x00A0A8, "RENEX CORPORATION", "RENEX" },
	{ 0x00A0A9, "NAVTEL COMMUNICATIONS INC.", "NAVTEL-COMMUNICATION" },
	{ 0x00A0AA, "SPACELABS MEDICAL", "SPACELABS-MEDICAL" },
	{ 0x00A0AB, "NETCS INFORMATIONSTECHNIK GMBH", "NETCS-INFORMATIONSTECHNIK" },
	{ 0x00A0AC, "GILAT SATELLITE NETWORKS, LTD.", "GILAT-SATELLITE-NETWORKS" },
	{ 0x00A0AD, "MARCONI SPA", "MARCONI-SPA" },
	{ 0x00A0AE, "NUCOM SYSTEMS, INC.", "NUCOM" },
	{ 0x00A0AF, "WMS INDUSTRIES", "WMS-INDUSTRIES" },
	{ 0x00A0B0, "I-O DATA DEVICE, INC.", "I-O-DATA-DEVICE" },
	{ 0x00A0B1, "FIRST VIRTUAL CORPORATION", "FIRST-VIRTUAL" },
	{ 0x00A0B2, "SHIMA SEIKI", "SHIMA-SEIKI" },
	{ 0x00A0B3, "ZYKRONIX", "ZYKRONIX" },
	{ 0x00A0B4, "TEXAS MICROSYSTEMS, INC.", "TEXAS-MICROSYSTEMS" },
	{ 0x00A0B5, "3H TECHNOLOGY", "3H" },
	{ 0x00A0B6, "SANRITZ AUTOMATION CO., LTD.", "SANRITZ-AUTOMATION" },
	{ 0x00A0B7, "CORDANT, INC.", "CORDANT" },
	{ 0x00A0B8, "SYMBIOS LOGIC INC.", "SYMBIOS-LOGIC" },
	{ 0x00A0B9, "EAGLE TECHNOLOGY, INC.", "EAGLE" },
	{ 0x00A0BA, "PATTON ELECTRONICS CO.", "PATTON-ELECTRONICS" },
	{ 0x00A0BB, "HILAN GMBH", "HILAN" },
	{ 0x00A0BC, "VIASAT, INCORPORATED", "VIASAT-INCORPORATED" },
	{ 0x00A0BD, "I-TECH CORP.", "I-TECH" },
	{ 0x00A0BE, "INTEGRATED CIRCUIT SYSTEMS, INC. COMMUNICATIONS GROUP", "INTEGRATED-CIRCUIT-COMMUNICATION-GROUP" },
	{ 0x00A0BF, "WIRELESS DATA GROUP MOTOROLA", "WIRELESS-DATA-GROUP-MOTOROLA" },
	{ 0x00A0C0, "DIGITAL LINK CORP.", "DIGITAL-LINK" },
	{ 0x00A0C1, "ORTIVUS MEDICAL AB", "ORTIVUS-MEDICAL-AB" },
	{ 0x00A0C2, "R.A. SYSTEMS CO., LTD.", "R-A" },
	{ 0x00A0C3, "UNICOMPUTER GMBH", "UNICOMPUTER" },
	{ 0x00A0C4, "CRISTIE ELECTRONICS LTD.", "CRISTIE-ELECTRONICS" },
	{ 0x00A0C5, "ZYXEL COMMUNICATION", "ZYXEL-COMMUNICATION" },
	{ 0x00A0C6, "QUALCOMM INCORPORATED", "QUALCOMM-INCORPORATED" },
	{ 0x00A0C7, "TADIRAN TELECOMMUNICATIONS", "TADIRAN-TELECOMMUNICATIONS" },
	{ 0x00A0C8, "ADTRAN INC.", "ADTRAN" },
	{ 0x00A0C9, "INTEL CORPORATION - HF1-06", "INTEL" },
	{ 0x00A0CA, "FUJITSU DENSO LTD.", "FUJITSU-DENSO" },
	{ 0x00A0CB, "ARK TELECOMMUNICATIONS, INC.", "ARK-TELECOMMUNICATIONS" },
	{ 0x00A0CC, "LITE-ON COMMUNICATIONS, INC.", "LITE-ON-COMMUNICATION" },
	{ 0x00A0CD, "DR. JOHANNES HEIDENHAIN GmbH", "DR-JOHANNES-HEIDENHAIN" },
	{ 0x00A0CE, "Ecessa", "ECESSA" },
	{ 0x00A0CF, "SOTAS, INC.", "SOTAS" },
	{ 0x00A0D0, "TEN X TECHNOLOGY, INC.", "TEN-X" },
	{ 0x00A0D1, "INVENTEC CORPORATION", "INVENTEC" },
	{ 0x00A0D2, "ALLIED TELESIS INTERNATIONAL CORPORATION", "ALLIED-TELESIS" },
	{ 0x00A0D3, "INSTEM COMPUTER SYSTEMS, LTD.", "INSTEM" },
	{ 0x00A0D4, "RADIOLAN, INC.", "RADIOLAN" },
	{ 0x00A0D5, "SIERRA WIRELESS INC.", "SIERRA-WIRELESS" },
	{ 0x00A0D6, "SBE, INC.", "SBE" },
	{ 0x00A0D7, "KASTEN CHASE APPLIED RESEARCH", "KASTEN-CHASE-APPLIED-RESEARCH" },
	{ 0x00A0D8, "SPECTRA - TEK", "SPECTRA---TEK" },
	{ 0x00A0D9, "CONVEX COMPUTER CORPORATION", "CONVEX" },
	{ 0x00A0DA, "INTEGRATED SYSTEMS Technology, Inc.", "INTEGRATED" },
	{ 0x00A0DB, "FISHER & PAYKEL PRODUCTION", "FISHER-PAYKEL-PRODUCTION" },
	{ 0x00A0DC, "O.N. ELECTRONIC CO., LTD.", "O-N" },
	{ 0x00A0DD, "AZONIX CORPORATION", "AZONIX" },
	{ 0x00A0DE, "YAMAHA CORPORATION", "YAMAHA" },
	{ 0x00A0DF, "STS TECHNOLOGIES, INC.", "STS-TECHNOLOGIES" },
	{ 0x00A0E0, "TENNYSON TECHNOLOGIES PTY LTD", "TENNYSON-TECHNOLOGIES-PTY" },
	{ 0x00A0E1, "WESTPORT RESEARCH ASSOCIATES, INC.", "WESTPORT-RESEARCH-ASSOCIATES" },
	{ 0x00A0E2, "Keisokugiken Corporation", "KEISOKUGIKEN" },
	{ 0x00A0E3, "XKL SYSTEMS CORP.", "XKL" },
	{ 0x00A0E4, "OPTIQUEST", "OPTIQUEST" },
	{ 0x00A0E5, "NHC COMMUNICATIONS", "NHC-COMMUNICATION" },
	{ 0x00A0E6, "DIALOGIC CORPORATION", "DIALOGIC" },
	{ 0x00A0E7, "CENTRAL DATA CORPORATION", "CENTRAL-DATA" },
	{ 0x00A0E8, "REUTERS HOLDINGS PLC", "REUTERS-HOLDINGS-PLC" },
	{ 0x00A0E9, "ELECTRONIC RETAILING SYSTEMS INTERNATIONAL", "RETAILING" },
	{ 0x00A0EA, "ETHERCOM CORP.", "ETHERCOM" },
	{ 0x00A0EB, "Encore Networks, Inc.", "ENCORE-NETWORKS" },
	{ 0x00A0EC, "TRANSMITTON LTD.", "TRANSMITTON" },
	{ 0x00A0ED, "Brooks Automation, Inc.", "BROOKS-AUTOMATION" },
	{ 0x00A0EE, "NASHOBA NETWORKS", "NASHOBA-NETWORKS" },
	{ 0x00A0EF, "LUCIDATA LTD.", "LUCIDATA" },
	{ 0x00A0F0, "TORONTO MICROELECTRONICS INC.", "TORONTO-MICROELECTRONICS" },
	{ 0x00A0F1, "MTI", "MTI" },
	{ 0x00A0F2, "INFOTEK COMMUNICATIONS, INC.", "INFOTEK-COMMUNICATION" },
	{ 0x00A0F3, "STAUBLI", "STAUBLI" },
	{ 0x00A0F4, "GE", "GE" },
	{ 0x00A0F5, "RADGUARD LTD.", "RADGUARD" },
	{ 0x00A0F6, "AutoGas Systems Inc.", "AUTOGAS" },
	{ 0x00A0F7, "V.I COMPUTER CORP.", "V-I" },
	{ 0x00A0F8, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0x00A0F9, "BINTEC COMMUNICATIONS GMBH", "BINTEC-COMMUNICATION" },
	{ 0x00A0FA, "Marconi Communication GmbH", "MARCONI-COMMUNICATION" },
	{ 0x00A0FB, "TORAY ENGINEERING CO., LTD.", "TORAY-ENGINEERING" },
	{ 0x00A0FC, "IMAGE SCIENCES, INC.", "IMAGE-SCIENCES" },
	{ 0x00A0FD, "SCITEX DIGITAL PRINTING, INC.", "SCITEX-DIGITAL-PRINTING" },
	{ 0x00A0FE, "BOSTON TECHNOLOGY, INC.", "BOSTON" },
	{ 0x00A0FF, "TELLABS OPERATIONS, INC.", "TELLABS-OPERATIONS" },
	{ 0x00A1DE, "ShenZhen ShiHua Technology CO.,LTD", "SHENZHEN-SHIHUA" },
	{ 0x00A2DA, "INAT GmbH", "INAT" },
	{ 0x00A2F5, "Guangzhou Yuanyun Network Technology Co.,Ltd", "GUANGZHOU-YUANYUN-NETWORK" },
	{ 0x00A2FF, "abatec group AG", "ABATEC-GROUP" },
	{ 0x00A509, "WigWag Inc.", "WIGWAG" },
	{ 0x00AA00, "INTEL CORPORATION", "INTEL" },
	{ 0x00AA01, "INTEL CORPORATION", "INTEL" },
	{ 0x00AA02, "INTEL CORPORATION", "INTEL" },
	{ 0x00AA3C, "OLIVETTI TELECOM SPA (OLTECO)", "OLIVETTI-TELECOM-SPA-OLTECO" },
	{ 0x00AA70, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x00ACE0, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00AEFA, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x00B009, "Grass Valley Group", "GRASS-VALLEY-GROUP" },
	{ 0x00B017, "InfoGear Technology Corp.", "INFOGEAR" },
	{ 0x00B019, "UTC CCS", "UTC-CCS" },
	{ 0x00B01C, "Westport Technologies", "WESTPORT-TECHNOLOGIES" },
	{ 0x00B01E, "Rantic Labs, Inc.", "RANTIC-LABS" },
	{ 0x00B02A, "ORSYS GmbH", "ORSYS" },
	{ 0x00B02D, "ViaGate Technologies, Inc.", "VIAGATE-TECHNOLOGIES" },
	{ 0x00B033, "OAO \"Izhevskiy radiozavod\"", "OAO-IZHEVSKIY-RADIOZAVOD" },
	{ 0x00B03B, "HiQ Networks", "HIQ-NETWORKS" },
	{ 0x00B048, "Marconi Communications Inc.", "MARCONI-COMMUNICATION" },
	{ 0x00B04A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00B052, "Atheros Communications", "ATHEROS-COMMUNICATION" },
	{ 0x00B064, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00B069, "Honewell Oy", "HONEWELL-OY" },
	{ 0x00B06D, "Jones Futurex Inc.", "JONES-FUTUREX" },
	{ 0x00B080, "Mannesmann Ipulsys B.V.", "MANNESMANN-IPULSYS" },
	{ 0x00B086, "LocSoft Limited", "LOCSOFT" },
	{ 0x00B08E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00B091, "Transmeta Corp.", "TRANSMETA" },
	{ 0x00B094, "Alaris, Inc.", "ALARIS" },
	{ 0x00B09A, "Morrow Technologies Corp.", "MORROW-TECHNOLOGIES" },
	{ 0x00B09D, "Point Grey Research Inc.", "POINT-GREY-RESEARCH" },
	{ 0x00B0AC, "SIAE-Microelettronica S.p.A.", "SIAE-MICROELETTRONICA-S-P-A" },
	{ 0x00B0AE, "Symmetricom", "SYMMETRICOM" },
	{ 0x00B0B3, "Xstreamis PLC", "XSTREAMIS-PLC" },
	{ 0x00B0C2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00B0C7, "Tellabs Operations, Inc.", "TELLABS-OPERATIONS" },
	{ 0x00B0CE, "TECHNOLOGY RESCUE", "RESCUE" },
	{ 0x00B0D0, "Dell Computer Corp.", "DELL" },
	{ 0x00B0DB, "Nextcell, Inc.", "NEXTCELL" },
	{ 0x00B0DF, "Starboard Storage Systems", "STARBOARD-STORAGE" },
	{ 0x00B0E7, "British Federal Ltd.", "BRITISH-FEDERAL" },
	{ 0x00B0EC, "EACEM", "EACEM" },
	{ 0x00B0EE, "Ajile Systems, Inc.", "AJILE" },
	{ 0x00B0F0, "CALY NETWORKS", "CALY-NETWORKS" },
	{ 0x00B0F5, "NetWorth Technologies, Inc.", "NETWORTH-TECHNOLOGIES" },
	{ 0x00B338, "Kontron Design Manufacturing Services (M) Sdn. Bhd", "KONTRON-DESIGN-MANUFACTURING-SERVICES-M-SDN-BHD" },
	{ 0x00B342, "MacroSAN Technologies Co., Ltd.", "MACROSAN-TECHNOLOGIES" },
	{ 0x00B56D, "David Electronics Co., LTD.", "DAVID-ELECTRONICS" },
	{ 0x00B5D6, "Omnibit Inc.", "OMNIBIT" },
	{ 0x00B78D, "Nanjing Shining Electric Automation Co., Ltd", "NANJING-SHINING-ELECTRIC-AUTOMATION" },
	{ 0x00B9F6, "Shenzhen Super Rich Electronics Co.,Ltd", "SHENZHEN-SUPER-RICH-ELECTRONICS" },
	{ 0x00BAC0, "Biometric Access Company", "BIOMETRIC-ACCESS-COMPANY" },
	{ 0x00BB01, "OCTOTHORPE CORP.", "OCTOTHORPE" },
	{ 0x00BB3A, "PRIVATE", "PRIVATE" },
	{ 0x00BB8E, "HME Co., Ltd.", "HME" },
	{ 0x00BBF0, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00BD27, "Exar Corp.", "EXAR" },
	{ 0x00BD3A, "Nokia Corporation", "NOKIA" },
	{ 0x00BF15, "Genetec Inc.", "GENETEC" },
	{ 0x00C000, "LANOPTICS, LTD.", "LANOPTICS" },
	{ 0x00C001, "DIATEK PATIENT MANAGMENT", "DIATEK-PATIENT-MANAGMENT" },
	{ 0x00C002, "SERCOMM CORPORATION", "SERCOMM" },
	{ 0x00C003, "GLOBALNET COMMUNICATIONS", "GLOBALNET-COMMUNICATION" },
	{ 0x00C004, "JAPAN BUSINESS COMPUTER CO.LTD", "JAPAN-BUSINESS" },
	{ 0x00C005, "LIVINGSTON ENTERPRISES, INC.", "LIVINGSTON-ENTERPRISES" },
	{ 0x00C006, "NIPPON AVIONICS CO., LTD.", "NIPPON-AVIONICS" },
	{ 0x00C007, "PINNACLE DATA SYSTEMS, INC.", "PINNACLE-DATA" },
	{ 0x00C008, "SECO SRL", "SECO-SRL" },
	{ 0x00C009, "KT TECHNOLOGY (S) PTE LTD", "KT-S-PTE" },
	{ 0x00C00A, "MICRO CRAFT", "MICRO-CRAFT" },
	{ 0x00C00B, "NORCONTROL A.S.", "NORCONTROL-A-S" },
	{ 0x00C00C, "RELIA TECHNOLGIES", "RELIA-TECHNOLGIES" },
	{ 0x00C00D, "ADVANCED LOGIC RESEARCH, INC.", "ADVANCED-LOGIC-RESEARCH" },
	{ 0x00C00E, "PSITECH, INC.", "PSITECH" },
	{ 0x00C00F, "QUANTUM SOFTWARE SYSTEMS LTD.", "QUANTUM-SOFTWARE" },
	{ 0x00C010, "HIRAKAWA HEWTECH CORP.", "HIRAKAWA-HEWTECH" },
	{ 0x00C011, "INTERACTIVE COMPUTING DEVICES", "INTERACTIVE-COMPUTING-DEVICES" },
	{ 0x00C012, "NETSPAN CORPORATION", "NETSPAN" },
	{ 0x00C013, "NETRIX", "NETRIX" },
	{ 0x00C014, "TELEMATICS CALABASAS INT'L,INC", "TELEMATICS-CALABASAS" },
	{ 0x00C015, "NEW MEDIA CORPORATION", "NEW-MEDIA" },
	{ 0x00C016, "ELECTRONIC THEATRE CONTROLS", "THEATRE-CONTROLS" },
	{ 0x00C017, "Fluke Corporation", "FLUKE" },
	{ 0x00C018, "LANART CORPORATION", "LANART" },
	{ 0x00C019, "LEAP TECHNOLOGY, INC.", "LEAP" },
	{ 0x00C01A, "COROMETRICS MEDICAL SYSTEMS", "COROMETRICS-MEDICAL" },
	{ 0x00C01B, "SOCKET COMMUNICATIONS, INC.", "SOCKET-COMMUNICATION" },
	{ 0x00C01C, "INTERLINK COMMUNICATIONS LTD.", "INTERLINK-COMMUNICATION" },
	{ 0x00C01D, "GRAND JUNCTION NETWORKS, INC.", "GRAND-JUNCTION-NETWORKS" },
	{ 0x00C01E, "LA FRANCAISE DES JEUX", "LA-FRANCAISE-DES-JEUX" },
	{ 0x00C01F, "S.E.R.C.E.L.", "S-E-R-C-E-L" },
	{ 0x00C020, "ARCO ELECTRONIC, CONTROL LTD.", "ARCO-CONTROL" },
	{ 0x00C021, "NETEXPRESS", "NETEXPRESS" },
	{ 0x00C022, "LASERMASTER TECHNOLOGIES, INC.", "LASERMASTER-TECHNOLOGIES" },
	{ 0x00C023, "TUTANKHAMON ELECTRONICS", "TUTANKHAMON-ELECTRONICS" },
	{ 0x00C024, "EDEN SISTEMAS DE COMPUTACAO SA", "EDEN-SISTEMAS-DE-COMPUTACAO" },
	{ 0x00C025, "DATAPRODUCTS CORPORATION", "DATAPRODUCTS" },
	{ 0x00C026, "LANS TECHNOLOGY CO., LTD.", "LANS" },
	{ 0x00C027, "CIPHER SYSTEMS, INC.", "CIPHER" },
	{ 0x00C028, "JASCO CORPORATION", "JASCO" },
	{ 0x00C029, "Nexans Deutschland GmbH - ANS", "NEXANS-DEUTSCHLAND---ANS" },
	{ 0x00C02A, "OHKURA ELECTRIC CO., LTD.", "OHKURA-ELECTRIC" },
	{ 0x00C02B, "GERLOFF GESELLSCHAFT FUR", "GERLOFF-GESELLSCHAFT-FUR" },
	{ 0x00C02C, "CENTRUM COMMUNICATIONS, INC.", "CENTRUM-COMMUNICATION" },
	{ 0x00C02D, "FUJI PHOTO FILM CO., LTD.", "FUJI-PHOTO-FILM" },
	{ 0x00C02E, "NETWIZ", "NETWIZ" },
	{ 0x00C02F, "OKUMA CORPORATION", "OKUMA" },
	{ 0x00C030, "INTEGRATED ENGINEERING B. V.", "INTEGRATED-ENGINEERING" },
	{ 0x00C031, "DESIGN RESEARCH SYSTEMS, INC.", "DESIGN-RESEARCH" },
	{ 0x00C032, "I-CUBED LIMITED", "I-CUBED" },
	{ 0x00C033, "TELEBIT COMMUNICATIONS APS", "TELEBIT-COMMUNICATION-APS" },
	{ 0x00C034, "TRANSACTION NETWORK", "TRANSACTION-NETWORK" },
	{ 0x00C035, "QUINTAR COMPANY", "QUINTAR-COMPANY" },
	{ 0x00C036, "RAYTECH ELECTRONIC CORP.", "RAYTECH" },
	{ 0x00C037, "DYNATEM", "DYNATEM" },
	{ 0x00C038, "RASTER IMAGE PROCESSING SYSTEM", "RASTER-IMAGE-PROCESSING-SYSTEM" },
	{ 0x00C039, "Teridian Semiconductor Corporation", "TERIDIAN-SEMICONDUCTOR" },
	{ 0x00C03A, "MEN-MIKRO ELEKTRONIK GMBH", "MEN-MIKRO-ELEKTRONIK" },
	{ 0x00C03B, "MULTIACCESS COMPUTING CORP.", "MULTIACCESS-COMPUTING" },
	{ 0x00C03C, "TOWER TECH S.R.L.", "TOWER-TECH-S-R-L" },
	{ 0x00C03D, "WIESEMANN & THEIS GMBH", "WIESEMANN-THEIS" },
	{ 0x00C03E, "FA. GEBR. HELLER GMBH", "FA-GEBR-HELLER" },
	{ 0x00C03F, "STORES AUTOMATED SYSTEMS, INC.", "STORES-AUTOMATED" },
	{ 0x00C040, "ECCI", "ECCI" },
	{ 0x00C041, "DIGITAL TRANSMISSION SYSTEMS", "DIGITAL-TRANSMISSION" },
	{ 0x00C042, "DATALUX CORP.", "DATALUX" },
	{ 0x00C043, "STRATACOM", "STRATACOM" },
	{ 0x00C044, "EMCOM CORPORATION", "EMCOM" },
	{ 0x00C045, "ISOLATION SYSTEMS, LTD.", "ISOLATION" },
	{ 0x00C046, "Blue Chip Technology Ltd", "BLUE-CHIP" },
	{ 0x00C047, "UNIMICRO SYSTEMS, INC.", "UNIMICRO" },
	{ 0x00C048, "BAY TECHNICAL ASSOCIATES", "BAY-TECHNICAL-ASSOCIATES" },
	{ 0x00C049, "U.S. ROBOTICS, INC.", "U-S-ROBOTICS" },
	{ 0x00C04A, "GROUP 2000 AG", "GROUP-2000" },
	{ 0x00C04B, "CREATIVE MICROSYSTEMS", "CREATIVE-MICROSYSTEMS" },
	{ 0x00C04C, "DEPARTMENT OF FOREIGN AFFAIRS", "DEPARTMENT-OF-FOREIGN-AFFAIRS" },
	{ 0x00C04D, "MITEC, INC.", "MITEC" },
	{ 0x00C04E, "COMTROL CORPORATION", "COMTROL" },
	{ 0x00C04F, "DELL COMPUTER CORPORATION", "DELL" },
	{ 0x00C050, "TOYO DENKI SEIZO K.K.", "TOYO-DENKI-SEIZO-K-K" },
	{ 0x00C051, "ADVANCED INTEGRATION RESEARCH", "ADVANCED-INTEGRATION-RESEARCH" },
	{ 0x00C052, "BURR-BROWN", "BURR-BROWN" },
	{ 0x00C053, "Aspect Software Inc.", "ASPECT-SOFTWARE" },
	{ 0x00C054, "NETWORK PERIPHERALS, LTD.", "NETWORK-PERIPHERALS" },
	{ 0x00C055, "MODULAR COMPUTING TECHNOLOGIES", "MODULAR-COMPUTING-TECHNOLOGIES" },
	{ 0x00C056, "SOMELEC", "SOMELEC" },
	{ 0x00C057, "MYCO ELECTRONICS", "MYCO-ELECTRONICS" },
	{ 0x00C058, "DATAEXPERT CORP.", "DATAEXPERT" },
	{ 0x00C059, "DENSO CORPORATION", "DENSO" },
	{ 0x00C05A, "SEMAPHORE COMMUNICATIONS CORP.", "SEMAPHORE-COMMUNICATION" },
	{ 0x00C05B, "NETWORKS NORTHWEST, INC.", "NETWORKS-NORTHWEST" },
	{ 0x00C05C, "ELONEX PLC", "ELONEX-PLC" },
	{ 0x00C05D, "L&N TECHNOLOGIES", "L-N-TECHNOLOGIES" },
	{ 0x00C05E, "VARI-LITE, INC.", "VARI-LITE" },
	{ 0x00C05F, "FINE-PAL COMPANY LIMITED", "FINE-PAL-COMPANY" },
	{ 0x00C060, "ID SCANDINAVIA AS", "ID-SCANDINAVIA-AS" },
	{ 0x00C061, "SOLECTEK CORPORATION", "SOLECTEK" },
	{ 0x00C062, "IMPULSE TECHNOLOGY", "IMPULSE" },
	{ 0x00C063, "MORNING STAR TECHNOLOGIES, INC", "MORNING-STAR-TECHNOLOGIES" },
	{ 0x00C064, "GENERAL DATACOMM IND. INC.", "GENERAL-DATACOMM-IND" },
	{ 0x00C065, "SCOPE COMMUNICATIONS, INC.", "SCOPE-COMMUNICATION" },
	{ 0x00C066, "DOCUPOINT, INC.", "DOCUPOINT" },
	{ 0x00C067, "UNITED BARCODE INDUSTRIES", "UNITED-BARCODE-INDUSTRIES" },
	{ 0x00C068, "HME Clear-Com LTD.", "HME-CLEAR-COM" },
	{ 0x00C069, "Axxcelera Broadband Wireless", "AXXCELERA-BROADBAND-WIRELESS" },
	{ 0x00C06A, "ZAHNER-ELEKTRIK GMBH & CO. KG", "ZAHNER-ELEKTRIK" },
	{ 0x00C06B, "OSI PLUS CORPORATION", "OSI-PLUS" },
	{ 0x00C06C, "SVEC COMPUTER CORP.", "SVEC" },
	{ 0x00C06D, "BOCA RESEARCH, INC.", "BOCA-RESEARCH" },
	{ 0x00C06E, "HAFT TECHNOLOGY, INC.", "HAFT" },
	{ 0x00C06F, "KOMATSU LTD.", "KOMATSU" },
	{ 0x00C070, "SECTRA SECURE-TRANSMISSION AB", "SECTRA-SECURE-TRANSMISSION-AB" },
	{ 0x00C071, "AREANEX COMMUNICATIONS, INC.", "AREANEX-COMMUNICATION" },
	{ 0x00C072, "KNX LTD.", "KNX" },
	{ 0x00C073, "XEDIA CORPORATION", "XEDIA" },
	{ 0x00C074, "TOYODA AUTOMATIC LOOM", "TOYODA-AUTOMATIC-LOOM" },
	{ 0x00C075, "XANTE CORPORATION", "XANTE" },
	{ 0x00C076, "I-DATA INTERNATIONAL A-S", "I-DATA-A-S" },
	{ 0x00C077, "DAEWOO TELECOM LTD.", "DAEWOO-TELECOM" },
	{ 0x00C078, "COMPUTER SYSTEMS ENGINEERING", "ENGINEERING" },
	{ 0x00C079, "FONSYS CO.,LTD.", "FONSYS" },
	{ 0x00C07A, "PRIVA B.V.", "PRIVA" },
	{ 0x00C07B, "ASCEND COMMUNICATIONS, INC.", "ASCEND-COMMUNICATION" },
	{ 0x00C07C, "HIGHTECH INFORMATION", "HIGHTECH-INFORMATION" },
	{ 0x00C07D, "RISC DEVELOPMENTS LTD.", "RISC-DEVELOPMENTS" },
	{ 0x00C07E, "KUBOTA CORPORATION ELECTRONIC", "KUBOTA" },
	{ 0x00C07F, "NUPON COMPUTING CORP.", "NUPON-COMPUTING" },
	{ 0x00C080, "NETSTAR, INC.", "NETSTAR" },
	{ 0x00C081, "METRODATA LTD.", "METRODATA" },
	{ 0x00C082, "MOORE PRODUCTS CO.", "MOORE-PRODUCTS" },
	{ 0x00C083, "TRACE MOUNTAIN PRODUCTS, INC.", "TRACE-MOUNTAIN-PRODUCTS" },
	{ 0x00C084, "DATA LINK CORP. LTD.", "DATA-LINK" },
	{ 0x00C085, "ELECTRONICS FOR IMAGING, INC.", "ELECTRONICS-FOR-IMAGING" },
	{ 0x00C086, "THE LYNK CORPORATION", "THE-LYNK" },
	{ 0x00C087, "UUNET TECHNOLOGIES, INC.", "UUNET-TECHNOLOGIES" },
	{ 0x00C088, "EKF ELEKTRONIK GMBH", "EKF-ELEKTRONIK" },
	{ 0x00C089, "TELINDUS DISTRIBUTION", "TELINDUS-DISTRIBUTION" },
	{ 0x00C08A, "Lauterbach GmbH", "LAUTERBACH" },
	{ 0x00C08B, "RISQ MODULAR SYSTEMS, INC.", "RISQ-MODULAR" },
	{ 0x00C08C, "PERFORMANCE TECHNOLOGIES, INC.", "PERFORMANCE-TECHNOLOGIES" },
	{ 0x00C08D, "TRONIX PRODUCT DEVELOPMENT", "TRONIX-PRODUCT-DEVELOPMENT" },
	{ 0x00C08E, "NETWORK INFORMATION TECHNOLOGY", "NETWORK-INFORMATION" },
	{ 0x00C08F, "Panasonic Electric Works Co., Ltd.", "PANASONIC-ELECTRIC-WORKS" },
	{ 0x00C090, "PRAIM S.R.L.", "PRAIM-S-R-L" },
	{ 0x00C091, "JABIL CIRCUIT, INC.", "JABIL-CIRCUIT" },
	{ 0x00C092, "MENNEN MEDICAL INC.", "MENNEN-MEDICAL" },
	{ 0x00C093, "ALTA RESEARCH CORP.", "ALTA-RESEARCH" },
	{ 0x00C094, "VMX INC.", "VMX" },
	{ 0x00C095, "ZNYX", "ZNYX" },
	{ 0x00C096, "TAMURA CORPORATION", "TAMURA" },
	{ 0x00C097, "ARCHIPEL SA", "ARCHIPEL" },
	{ 0x00C098, "CHUNTEX ELECTRONIC CO., LTD.", "CHUNTEX" },
	{ 0x00C099, "YOSHIKI INDUSTRIAL CO.,LTD.", "YOSHIKI-INDUSTRIAL" },
	{ 0x00C09A, "PHOTONICS CORPORATION", "PHOTONICS" },
	{ 0x00C09B, "RELIANCE COMM/TEC, R-TEC", "RELIANCE-COMM/TEC-R-TEC" },
	{ 0x00C09C, "HIOKI E.E. CORPORATION", "HIOKI-E-E" },
	{ 0x00C09D, "DISTRIBUTED SYSTEMS INT'L, INC", "DISTRIBUTED" },
	{ 0x00C09E, "CACHE COMPUTERS, INC.", "CACHE-COMPUTERS" },
	{ 0x00C09F, "QUANTA COMPUTER, INC.", "QUANTA" },
	{ 0x00C0A0, "ADVANCE MICRO RESEARCH, INC.", "ADVANCE-MICRO-RESEARCH" },
	{ 0x00C0A1, "TOKYO DENSHI SEKEI CO.", "TOKYO-DENSHI-SEKEI" },
	{ 0x00C0A2, "INTERMEDIUM A/S", "INTERMEDIUM-A/S" },
	{ 0x00C0A3, "DUAL ENTERPRISES CORPORATION", "DUAL-ENTERPRISES" },
	{ 0x00C0A4, "UNIGRAF OY", "UNIGRAF-OY" },
	{ 0x00C0A5, "DICKENS DATA SYSTEMS", "DICKENS-DATA" },
	{ 0x00C0A6, "EXICOM AUSTRALIA PTY. LTD", "EXICOM-AUSTRALIA-PTY" },
	{ 0x00C0A7, "SEEL LTD.", "SEEL" },
	{ 0x00C0A8, "GVC CORPORATION", "GVC" },
	{ 0x00C0A9, "BARRON MCCANN LTD.", "BARRON-MCCANN" },
	{ 0x00C0AA, "SILICON VALLEY COMPUTER", "SILICON-VALLEY" },
	{ 0x00C0AB, "Telco Systems, Inc.", "TELCO" },
	{ 0x00C0AC, "GAMBIT COMPUTER COMMUNICATIONS", "GAMBIT-COMMUNICATION" },
	{ 0x00C0AD, "MARBEN COMMUNICATION SYSTEMS", "MARBEN-COMMUNICATION" },
	{ 0x00C0AE, "TOWERCOM CO. INC. DBA PC HOUSE", "TOWERCOM-DBA-PC-HOUSE" },
	{ 0x00C0AF, "TEKLOGIX INC.", "TEKLOGIX" },
	{ 0x00C0B0, "GCC TECHNOLOGIES,INC.", "GCC-TECHNOLOGIES" },
	{ 0x00C0B1, "GENIUS NET CO.", "GENIUS-NET" },
	{ 0x00C0B2, "NORAND CORPORATION", "NORAND" },
	{ 0x00C0B3, "COMSTAT DATACOMM CORPORATION", "COMSTAT-DATACOMM" },
	{ 0x00C0B4, "MYSON TECHNOLOGY, INC.", "MYSON" },
	{ 0x00C0B5, "CORPORATE NETWORK SYSTEMS,INC.", "CORPORATE-NETWORK" },
	{ 0x00C0B6, "Overland Storage, Inc.", "OVERLAND-STORAGE" },
	{ 0x00C0B7, "AMERICAN POWER CONVERSION CORP", "AMERICAN-POWER-CONVERSION" },
	{ 0x00C0B8, "FRASER'S HILL LTD.", "FRASER-S-HILL" },
	{ 0x00C0B9, "FUNK SOFTWARE, INC.", "FUNK-SOFTWARE" },
	{ 0x00C0BA, "NETVANTAGE", "NETVANTAGE" },
	{ 0x00C0BB, "FORVAL CREATIVE, INC.", "FORVAL-CREATIVE" },
	{ 0x00C0BC, "TELECOM AUSTRALIA/CSSC", "TELECOM-AUSTRALIA/CSSC" },
	{ 0x00C0BD, "INEX TECHNOLOGIES, INC.", "INEX-TECHNOLOGIES" },
	{ 0x00C0BE, "ALCATEL - SEL", "ALCATEL---SEL" },
	{ 0x00C0BF, "TECHNOLOGY CONCEPTS, LTD.", "CONCEPTS" },
	{ 0x00C0C0, "SHORE MICROSYSTEMS, INC.", "SHORE-MICROSYSTEMS" },
	{ 0x00C0C1, "QUAD/GRAPHICS, INC.", "QUAD/GRAPHICS" },
	{ 0x00C0C2, "INFINITE NETWORKS LTD.", "INFINITE-NETWORKS" },
	{ 0x00C0C3, "ACUSON COMPUTED SONOGRAPHY", "ACUSON-COMPUTED-SONOGRAPHY" },
	{ 0x00C0C4, "COMPUTER OPERATIONAL", "OPERATIONAL" },
	{ 0x00C0C5, "SID INFORMATICA", "SID-INFORMATICA" },
	{ 0x00C0C6, "PERSONAL MEDIA CORP.", "PERSONAL-MEDIA" },
	{ 0x00C0C7, "SPARKTRUM MICROSYSTEMS, INC.", "SPARKTRUM-MICROSYSTEMS" },
	{ 0x00C0C8, "MICRO BYTE PTY. LTD.", "MICRO-BYTE-PTY" },
	{ 0x00C0C9, "ELSAG BAILEY PROCESS", "ELSAG-BAILEY-PROCESS" },
	{ 0x00C0CA, "ALFA, INC.", "ALFA" },
	{ 0x00C0CB, "CONTROL TECHNOLOGY CORPORATION", "CONTROL" },
	{ 0x00C0CC, "TELESCIENCES CO SYSTEMS, INC.", "TELESCIENCES" },
	{ 0x00C0CD, "COMELTA, S.A.", "COMELTA" },
	{ 0x00C0CE, "CEI SYSTEMS & ENGINEERING PTE", "CEI-ENGINEERING-PTE" },
	{ 0x00C0CF, "IMATRAN VOIMA OY", "IMATRAN-VOIMA-OY" },
	{ 0x00C0D0, "RATOC SYSTEM INC.", "RATOC-SYSTEM" },
	{ 0x00C0D1, "COMTREE TECHNOLOGY CORPORATION", "COMTREE" },
	{ 0x00C0D2, "SYNTELLECT, INC.", "SYNTELLECT" },
	{ 0x00C0D3, "OLYMPUS IMAGE SYSTEMS, INC.", "OLYMPUS-IMAGE" },
	{ 0x00C0D4, "AXON NETWORKS, INC.", "AXON-NETWORKS" },
	{ 0x00C0D5, "Werbeagentur Jürgen Siebert", "WERBEAGENTUR-JüRGEN-SIEBERT" },
	{ 0x00C0D6, "J1 SYSTEMS, INC.", "J1" },
	{ 0x00C0D7, "TAIWAN TRADING CENTER DBA", "TAIWAN-TRADING-CENTER-DBA" },
	{ 0x00C0D8, "UNIVERSAL DATA SYSTEMS", "UNIVERSAL-DATA" },
	{ 0x00C0D9, "QUINTE NETWORK CONFIDENTIALITY", "QUINTE-NETWORK-CONFIDENTIALITY" },
	{ 0x00C0DA, "NICE SYSTEMS LTD.", "NICE" },
	{ 0x00C0DB, "IPC CORPORATION (PTE) LTD.", "IPC-PTE" },
	{ 0x00C0DC, "EOS TECHNOLOGIES, INC.", "EOS-TECHNOLOGIES" },
	{ 0x00C0DD, "QLogic Corporation", "QLOGIC" },
	{ 0x00C0DE, "ZCOMM, INC.", "ZCOMM" },
	{ 0x00C0DF, "KYE Systems Corp.", "KYE" },
	{ 0x00C0E0, "DSC COMMUNICATION CORP.", "DSC-COMMUNICATION" },
	{ 0x00C0E1, "SONIC SOLUTIONS", "SONIC-SOLUTIONS" },
	{ 0x00C0E2, "CALCOMP, INC.", "CALCOMP" },
	{ 0x00C0E3, "OSITECH COMMUNICATIONS, INC.", "OSITECH-COMMUNICATION" },
	{ 0x00C0E4, "SIEMENS BUILDING", "SIEMENS-BUILDING" },
	{ 0x00C0E5, "GESPAC, S.A.", "GESPAC" },
	{ 0x00C0E6, "Verilink Corporation", "VERILINK" },
	{ 0x00C0E7, "FIBERDATA AB", "FIBERDATA-AB" },
	{ 0x00C0E8, "PLEXCOM, INC.", "PLEXCOM" },
	{ 0x00C0E9, "OAK SOLUTIONS, LTD.", "OAK-SOLUTIONS" },
	{ 0x00C0EA, "ARRAY TECHNOLOGY LTD.", "ARRAY" },
	{ 0x00C0EB, "SEH COMPUTERTECHNIK GMBH", "SEH-COMPUTERTECHNIK" },
	{ 0x00C0EC, "DAUPHIN TECHNOLOGY", "DAUPHIN" },
	{ 0x00C0ED, "US ARMY ELECTRONIC", "US-ARMY" },
	{ 0x00C0EE, "KYOCERA CORPORATION", "KYOCERA" },
	{ 0x00C0EF, "ABIT CORPORATION", "ABIT" },
	{ 0x00C0F0, "KINGSTON TECHNOLOGY CORP.", "KINGSTON" },
	{ 0x00C0F1, "SHINKO ELECTRIC CO., LTD.", "SHINKO-ELECTRIC" },
	{ 0x00C0F2, "TRANSITION NETWORKS", "TRANSITION-NETWORKS" },
	{ 0x00C0F3, "NETWORK COMMUNICATIONS CORP.", "NETWORK-COMMUNICATION" },
	{ 0x00C0F4, "INTERLINK SYSTEM CO., LTD.", "INTERLINK-SYSTEM" },
	{ 0x00C0F5, "METACOMP, INC.", "METACOMP" },
	{ 0x00C0F6, "CELAN TECHNOLOGY INC.", "CELAN" },
	{ 0x00C0F7, "ENGAGE COMMUNICATION, INC.", "ENGAGE-COMMUNICATION" },
	{ 0x00C0F8, "ABOUT COMPUTING INC.", "ABOUT-COMPUTING" },
	{ 0x00C0F9, "Artesyn Embedded Technologies", "ARTESYN-EMBEDDED-TECHNOLOGIES" },
	{ 0x00C0FA, "CANARY COMMUNICATIONS, INC.", "CANARY-COMMUNICATION" },
	{ 0x00C0FB, "ADVANCED TECHNOLOGY LABS", "ADVANCED-LABS" },
	{ 0x00C0FC, "ELASTIC REALITY, INC.", "ELASTIC-REALITY" },
	{ 0x00C0FD, "PROSUM", "PROSUM" },
	{ 0x00C0FE, "APTEC COMPUTER SYSTEMS, INC.", "APTEC" },
	{ 0x00C0FF, "DOT HILL SYSTEMS CORPORATION", "DOT-HILL" },
	{ 0x00C14F, "DDL Co,.ltd.", "DDL" },
	{ 0x00C2C6, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x00C5DB, "Datatech Sistemas Digitales Avanzados SL", "DATATECH-SISTEMAS-DIGITALES-AVANZADOS-SL" },
	{ 0x00C610, "Apple", "APPLE" },
	{ 0x00CBBD, "Cambridge Broadband Networks Ltd.", "CAMBRIDGE-BROADBAND-NETWORKS" },
	{ 0x00CD90, "MAS Elektronik AG", "MAS-ELEKTRONIK" },
	{ 0x00CF1C, "COMMUNICATION MACHINERY CORP.", "COMMUNICATION-MACHINERY" },
	{ 0x00D000, "FERRAN SCIENTIFIC, INC.", "FERRAN-SCIENTIFIC" },
	{ 0x00D001, "VST TECHNOLOGIES, INC.", "VST-TECHNOLOGIES" },
	{ 0x00D002, "DITECH CORPORATION", "DITECH" },
	{ 0x00D003, "COMDA ENTERPRISES CORP.", "COMDA-ENTERPRISES" },
	{ 0x00D004, "PENTACOM LTD.", "PENTACOM" },
	{ 0x00D005, "ZHS ZEITMANAGEMENTSYSTEME", "ZHS-ZEITMANAGEMENTSYSTEME" },
	{ 0x00D006, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D007, "MIC ASSOCIATES, INC.", "MIC-ASSOCIATES" },
	{ 0x00D008, "MACTELL CORPORATION", "MACTELL" },
	{ 0x00D009, "HSING TECH. ENTERPRISE CO. LTD", "HSING-TECH" },
	{ 0x00D00A, "LANACCESS TELECOM S.A.", "LANACCESS-TELECOM" },
	{ 0x00D00B, "RHK TECHNOLOGY, INC.", "RHK" },
	{ 0x00D00C, "SNIJDER MICRO SYSTEMS", "SNIJDER-MICRO" },
	{ 0x00D00D, "MICROMERITICS INSTRUMENT", "MICROMERITICS-INSTRUMENT" },
	{ 0x00D00E, "PLURIS, INC.", "PLURIS" },
	{ 0x00D00F, "SPEECH DESIGN GMBH", "SPEECH-DESIGN" },
	{ 0x00D010, "CONVERGENT NETWORKS, INC.", "CONVERGENT-NETWORKS" },
	{ 0x00D011, "PRISM VIDEO, INC.", "PRISM-VIDEO" },
	{ 0x00D012, "GATEWORKS CORP.", "GATEWORKS" },
	{ 0x00D013, "PRIMEX AEROSPACE COMPANY", "PRIMEX-AEROSPACE-COMPANY" },
	{ 0x00D014, "ROOT, INC.", "ROOT" },
	{ 0x00D015, "UNIVEX MICROTECHNOLOGY CORP.", "UNIVEX-MICROTECHNOLOGY" },
	{ 0x00D016, "SCM MICROSYSTEMS, INC.", "SCM-MICROSYSTEMS" },
	{ 0x00D017, "SYNTECH INFORMATION CO., LTD.", "SYNTECH-INFORMATION" },
	{ 0x00D018, "QWES. COM, INC.", "QWES-COM" },
	{ 0x00D019, "DAINIPPON SCREEN CORPORATE", "DAINIPPON-SCREEN-CORPORATE" },
	{ 0x00D01A, "URMET TLC S.P.A.", "URMET-TLC-S-P-A" },
	{ 0x00D01B, "MIMAKI ENGINEERING CO., LTD.", "MIMAKI-ENGINEERING" },
	{ 0x00D01C, "SBS TECHNOLOGIES,", "SBS-TECHNOLOGIES" },
	{ 0x00D01D, "FURUNO ELECTRIC CO., LTD.", "FURUNO-ELECTRIC" },
	{ 0x00D01E, "PINGTEL CORP.", "PINGTEL" },
	{ 0x00D01F, "Senetas Security", "SENETAS-SECURITY" },
	{ 0x00D020, "AIM SYSTEM, INC.", "AIM-SYSTEM" },
	{ 0x00D021, "REGENT ELECTRONICS CORP.", "REGENT-ELECTRONICS" },
	{ 0x00D022, "INCREDIBLE TECHNOLOGIES, INC.", "INCREDIBLE-TECHNOLOGIES" },
	{ 0x00D023, "INFORTREND TECHNOLOGY, INC.", "INFORTREND" },
	{ 0x00D024, "Cognex Corporation", "COGNEX" },
	{ 0x00D025, "XROSSTECH, INC.", "XROSSTECH" },
	{ 0x00D026, "HIRSCHMANN AUSTRIA GMBH", "HIRSCHMANN-AUSTRIA" },
	{ 0x00D027, "APPLIED AUTOMATION, INC.", "APPLIED-AUTOMATION" },
	{ 0x00D028, "Harmonic, Inc", "HARMONIC" },
	{ 0x00D029, "WAKEFERN FOOD CORPORATION", "WAKEFERN-FOOD" },
	{ 0x00D02A, "Voxent Systems Ltd.", "VOXENT" },
	{ 0x00D02B, "JETCELL, INC.", "JETCELL" },
	{ 0x00D02C, "CAMPBELL SCIENTIFIC, INC.", "CAMPBELL-SCIENTIFIC" },
	{ 0x00D02D, "ADEMCO", "ADEMCO" },
	{ 0x00D02E, "COMMUNICATION AUTOMATION CORP.", "COMMUNICATION-AUTOMATION" },
	{ 0x00D02F, "VLSI TECHNOLOGY INC.", "VLSI" },
	{ 0x00D030, "Safetran Systems Corp", "SAFETRAN" },
	{ 0x00D031, "INDUSTRIAL LOGIC CORPORATION", "INDUSTRIAL-LOGIC" },
	{ 0x00D032, "YANO ELECTRIC CO., LTD.", "YANO-ELECTRIC" },
	{ 0x00D033, "DALIAN DAXIAN NETWORK", "DALIAN-DAXIAN-NETWORK" },
	{ 0x00D034, "ORMEC SYSTEMS CORP.", "ORMEC" },
	{ 0x00D035, "BEHAVIOR TECH. COMPUTER CORP.", "BEHAVIOR-TECH" },
	{ 0x00D036, "TECHNOLOGY ATLANTA CORP.", "ATLANTA" },
	{ 0x00D037, "Pace France", "PACE-FRANCE" },
	{ 0x00D038, "FIVEMERE, LTD.", "FIVEMERE" },
	{ 0x00D039, "UTILICOM, INC.", "UTILICOM" },
	{ 0x00D03A, "ZONEWORX, INC.", "ZONEWORX" },
	{ 0x00D03B, "VISION PRODUCTS PTY. LTD.", "VISION-PRODUCTS-PTY" },
	{ 0x00D03C, "Vieo, Inc.", "VIEO" },
	{ 0x00D03D, "GALILEO TECHNOLOGY, LTD.", "GALILEO" },
	{ 0x00D03E, "ROCKETCHIPS, INC.", "ROCKETCHIPS" },
	{ 0x00D03F, "AMERICAN COMMUNICATION", "AMERICAN-COMMUNICATION" },
	{ 0x00D040, "SYSMATE CO., LTD.", "SYSMATE" },
	{ 0x00D041, "AMIGO TECHNOLOGY CO., LTD.", "AMIGO" },
	{ 0x00D042, "MAHLO GMBH & CO. UG", "MAHLO-UG" },
	{ 0x00D043, "ZONAL RETAIL DATA SYSTEMS", "ZONAL-RETAIL-DATA" },
	{ 0x00D044, "ALIDIAN NETWORKS, INC.", "ALIDIAN-NETWORKS" },
	{ 0x00D045, "KVASER AB", "KVASER-AB" },
	{ 0x00D046, "DOLBY LABORATORIES, INC.", "DOLBY-LABORATORIES" },
	{ 0x00D047, "XN TECHNOLOGIES", "XN-TECHNOLOGIES" },
	{ 0x00D048, "ECTON, INC.", "ECTON" },
	{ 0x00D049, "IMPRESSTEK CO., LTD.", "IMPRESSTEK" },
	{ 0x00D04A, "PRESENCE TECHNOLOGY GMBH", "PRESENCE" },
	{ 0x00D04B, "LA CIE GROUP S.A.", "LA-CIE-GROUP" },
	{ 0x00D04C, "EUROTEL TELECOM LTD.", "EUROTEL-TELECOM" },
	{ 0x00D04D, "DIV OF RESEARCH & STATISTICS", "DIV-OF-RESEARCH-STATISTICS" },
	{ 0x00D04E, "LOGIBAG", "LOGIBAG" },
	{ 0x00D04F, "BITRONICS, INC.", "BITRONICS" },
	{ 0x00D050, "ISKRATEL", "ISKRATEL" },
	{ 0x00D051, "O2 MICRO, INC.", "O2-MICRO" },
	{ 0x00D052, "ASCEND COMMUNICATIONS, INC.", "ASCEND-COMMUNICATION" },
	{ 0x00D053, "CONNECTED SYSTEMS", "CONNECTED" },
	{ 0x00D054, "SAS INSTITUTE INC.", "SAS-INSTITUTE" },
	{ 0x00D055, "KATHREIN-WERKE KG", "KATHREIN-WERKE" },
	{ 0x00D056, "SOMAT CORPORATION", "SOMAT" },
	{ 0x00D057, "ULTRAK, INC.", "ULTRAK" },
	{ 0x00D058, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D059, "AMBIT MICROSYSTEMS CORP.", "AMBIT-MICROSYSTEMS" },
	{ 0x00D05A, "SYMBIONICS, LTD.", "SYMBIONICS" },
	{ 0x00D05B, "ACROLOOP MOTION CONTROL", "ACROLOOP-MOTION-CONTROL" },
	{ 0x00D05C, "TECHNOTREND SYSTEMTECHNIK GMBH", "TECHNOTREND-SYSTEMTECHNIK" },
	{ 0x00D05D, "INTELLIWORXX, INC.", "INTELLIWORXX" },
	{ 0x00D05E, "STRATABEAM TECHNOLOGY, INC.", "STRATABEAM" },
	{ 0x00D05F, "VALCOM, INC.", "VALCOM" },
	{ 0x00D060, "Panasonic Europe Ltd.", "PANASONIC-EUROPE" },
	{ 0x00D061, "TREMON ENTERPRISES CO., LTD.", "TREMON-ENTERPRISES" },
	{ 0x00D062, "DIGIGRAM", "DIGIGRAM" },
	{ 0x00D063, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D064, "MULTITEL", "MULTITEL" },
	{ 0x00D065, "TOKO ELECTRIC", "TOKO-ELECTRIC" },
	{ 0x00D066, "WINTRISS ENGINEERING CORP.", "WINTRISS-ENGINEERING" },
	{ 0x00D067, "CAMPIO COMMUNICATIONS", "CAMPIO-COMMUNICATION" },
	{ 0x00D068, "IWILL CORPORATION", "IWILL" },
	{ 0x00D069, "TECHNOLOGIC SYSTEMS", "TECHNOLOGIC" },
	{ 0x00D06A, "LINKUP SYSTEMS CORPORATION", "LINKUP" },
	{ 0x00D06B, "SR TELECOM INC.", "SR-TELECOM" },
	{ 0x00D06C, "SHAREWAVE, INC.", "SHAREWAVE" },
	{ 0x00D06D, "ACRISON, INC.", "ACRISON" },
	{ 0x00D06E, "TRENDVIEW RECORDERS LTD.", "TRENDVIEW-RECORDERS" },
	{ 0x00D06F, "KMC CONTROLS", "KMC-CONTROLS" },
	{ 0x00D070, "LONG WELL ELECTRONICS CORP.", "LONG-WELL-ELECTRONICS" },
	{ 0x00D071, "ECHELON CORP.", "ECHELON" },
	{ 0x00D072, "BROADLOGIC", "BROADLOGIC" },
	{ 0x00D073, "ACN ADVANCED COMMUNICATIONS", "ACN-ADVANCED-COMMUNICATION" },
	{ 0x00D074, "TAQUA SYSTEMS, INC.", "TAQUA" },
	{ 0x00D075, "ALARIS MEDICAL SYSTEMS, INC.", "ALARIS-MEDICAL" },
	{ 0x00D076, "Bank of America", "BANK-OF-AMERICA" },
	{ 0x00D077, "LUCENT TECHNOLOGIES", "LUCENT-TECHNOLOGIES" },
	{ 0x00D078, "Eltex of Sweden AB", "ELTEX-OF-SWEDEN-AB" },
	{ 0x00D079, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D07A, "AMAQUEST COMPUTER CORP.", "AMAQUEST" },
	{ 0x00D07B, "COMCAM INTERNATIONAL INC", "COMCAM" },
	{ 0x00D07C, "KOYO ELECTRONICS INC. CO.,LTD.", "KOYO-ELECTRONICS" },
	{ 0x00D07D, "COSINE COMMUNICATIONS", "COSINE-COMMUNICATION" },
	{ 0x00D07E, "KEYCORP LTD.", "KEYCORP" },
	{ 0x00D07F, "STRATEGY & TECHNOLOGY, LIMITED", "STRATEGY" },
	{ 0x00D080, "EXABYTE CORPORATION", "EXABYTE" },
	{ 0x00D081, "RTD Embedded Technologies, Inc.", "RTD-EMBEDDED-TECHNOLOGIES" },
	{ 0x00D082, "IOWAVE INC.", "IOWAVE" },
	{ 0x00D083, "INVERTEX, INC.", "INVERTEX" },
	{ 0x00D084, "NEXCOMM SYSTEMS, INC.", "NEXCOMM" },
	{ 0x00D085, "OTIS ELEVATOR COMPANY", "OTIS-ELEVATOR-COMPANY" },
	{ 0x00D086, "FOVEON, INC.", "FOVEON" },
	{ 0x00D087, "MICROFIRST INC.", "MICROFIRST" },
	{ 0x00D088, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00D089, "DYNACOLOR, INC.", "DYNACOLOR" },
	{ 0x00D08A, "PHOTRON USA", "PHOTRON-USA" },
	{ 0x00D08B, "ADVA Optical Networking Ltd.", "ADVA-OPTICAL-NETWORKING" },
	{ 0x00D08C, "GENOA TECHNOLOGY, INC.", "GENOA" },
	{ 0x00D08D, "PHOENIX GROUP, INC.", "PHOENIX-GROUP" },
	{ 0x00D08E, "NVISION INC.", "NVISION" },
	{ 0x00D08F, "ARDENT TECHNOLOGIES, INC.", "ARDENT-TECHNOLOGIES" },
	{ 0x00D090, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D091, "SMARTSAN SYSTEMS, INC.", "SMARTSAN" },
	{ 0x00D092, "GLENAYRE WESTERN MULTIPLEX", "GLENAYRE-WESTERN-MULTIPLEX" },
	{ 0x00D093, "TQ - COMPONENTS GMBH", "TQ---COMPONENTS" },
	{ 0x00D094, "Seeion Control LLC", "SEEION-CONTROL-LLC" },
	{ 0x00D095, "Alcatel-Lucent, Enterprise Business Group", "ALCATEL-LUCENT-BUSINESS-GROUP" },
	{ 0x00D096, "3COM EUROPE LTD.", "3COM" },
	{ 0x00D097, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D098, "Photon Dynamics Canada Inc.", "PHOTON-DYNAMICS-CANADA" },
	{ 0x00D099, "Elcard Wireless Systems Oy", "ELCARD-WIRELESS-OY" },
	{ 0x00D09A, "FILANET CORPORATION", "FILANET" },
	{ 0x00D09B, "SPECTEL LTD.", "SPECTEL" },
	{ 0x00D09C, "KAPADIA COMMUNICATIONS", "KAPADIA-COMMUNICATION" },
	{ 0x00D09D, "VERIS INDUSTRIES", "VERIS-INDUSTRIES" },
	{ 0x00D09E, "2WIRE, INC.", "2WIRE" },
	{ 0x00D09F, "NOVTEK TEST SYSTEMS", "NOVTEK-TEST" },
	{ 0x00D0A0, "MIPS DENMARK", "MIPS-DENMARK" },
	{ 0x00D0A1, "OSKAR VIERLING GMBH + CO. KG", "OSKAR-VIERLING-+" },
	{ 0x00D0A2, "INTEGRATED DEVICE", "INTEGRATED-DEVICE" },
	{ 0x00D0A3, "VOCAL DATA, INC.", "VOCAL-DATA" },
	{ 0x00D0A4, "ALANTRO COMMUNICATIONS", "ALANTRO-COMMUNICATION" },
	{ 0x00D0A5, "AMERICAN ARIUM", "AMERICAN-ARIUM" },
	{ 0x00D0A6, "LANBIRD TECHNOLOGY CO., LTD.", "LANBIRD" },
	{ 0x00D0A7, "TOKYO SOKKI KENKYUJO CO., LTD.", "TOKYO-SOKKI-KENKYUJO" },
	{ 0x00D0A8, "NETWORK ENGINES, INC.", "NETWORK-ENGINES" },
	{ 0x00D0A9, "SHINANO KENSHI CO., LTD.", "SHINANO-KENSHI" },
	{ 0x00D0AA, "CHASE COMMUNICATIONS", "CHASE-COMMUNICATION" },
	{ 0x00D0AB, "DELTAKABEL TELECOM CV", "DELTAKABEL-TELECOM-CV" },
	{ 0x00D0AC, "GRAYSON WIRELESS", "GRAYSON-WIRELESS" },
	{ 0x00D0AD, "TL INDUSTRIES", "TL-INDUSTRIES" },
	{ 0x00D0AE, "ORESIS COMMUNICATIONS, INC.", "ORESIS-COMMUNICATION" },
	{ 0x00D0AF, "CUTLER-HAMMER, INC.", "CUTLER-HAMMER" },
	{ 0x00D0B0, "BITSWITCH LTD.", "BITSWITCH" },
	{ 0x00D0B1, "OMEGA ELECTRONICS SA", "OMEGA-ELECTRONICS" },
	{ 0x00D0B2, "XIOTECH CORPORATION", "XIOTECH" },
	{ 0x00D0B3, "DRS Technologies Canada Ltd", "DRS-TECHNOLOGIES-CANADA" },
	{ 0x00D0B4, "KATSUJIMA CO., LTD.", "KATSUJIMA" },
	{ 0x00D0B5, "IPricot formerly DotCom", "IPRICOT-FORMERLY-DOTCOM" },
	{ 0x00D0B6, "CRESCENT NETWORKS, INC.", "CRESCENT-NETWORKS" },
	{ 0x00D0B7, "INTEL CORPORATION", "INTEL" },
	{ 0x00D0B8, "Iomega Corporation", "IOMEGA" },
	{ 0x00D0B9, "MICROTEK INTERNATIONAL, INC.", "MICROTEK" },
	{ 0x00D0BA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D0BB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D0BC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D0BD, "Silicon Image GmbH", "SILICON-IMAGE" },
	{ 0x00D0BE, "EMUTEC INC.", "EMUTEC" },
	{ 0x00D0BF, "PIVOTAL TECHNOLOGIES", "PIVOTAL-TECHNOLOGIES" },
	{ 0x00D0C0, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D0C1, "HARMONIC DATA SYSTEMS, LTD.", "HARMONIC-DATA" },
	{ 0x00D0C2, "BALTHAZAR TECHNOLOGY AB", "BALTHAZAR-AB" },
	{ 0x00D0C3, "VIVID TECHNOLOGY PTE, LTD.", "VIVID-PTE" },
	{ 0x00D0C4, "TERATECH CORPORATION", "TERATECH" },
	{ 0x00D0C5, "COMPUTATIONAL SYSTEMS, INC.", "COMPUTATIONAL" },
	{ 0x00D0C6, "THOMAS & BETTS CORP.", "THOMAS-BETTS" },
	{ 0x00D0C7, "PATHWAY, INC.", "PATHWAY" },
	{ 0x00D0C8, "Prevas A/S", "PREVAS-A/S" },
	{ 0x00D0C9, "ADVANTECH CO., LTD.", "ADVANTECH" },
	{ 0x00D0CA, "Intrinsyc Software International Inc.", "INTRINSYC-SOFTWARE" },
	{ 0x00D0CB, "DASAN CO., LTD.", "DASAN" },
	{ 0x00D0CC, "TECHNOLOGIES LYRE INC.", "TECHNOLOGIES-LYRE" },
	{ 0x00D0CD, "ATAN TECHNOLOGY INC.", "ATAN" },
	{ 0x00D0CE, "ASYST ELECTRONIC", "ASYST" },
	{ 0x00D0CF, "MORETON BAY", "MORETON-BAY" },
	{ 0x00D0D0, "ZHONGXING TELECOM LTD.", "ZHONGXING-TELECOM" },
	{ 0x00D0D1, "Sycamore Networks", "SYCAMORE-NETWORKS" },
	{ 0x00D0D2, "EPILOG CORPORATION", "EPILOG" },
	{ 0x00D0D3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D0D4, "V-BITS, INC.", "V-BITS" },
	{ 0x00D0D5, "GRUNDIG AG", "GRUNDIG" },
	{ 0x00D0D6, "AETHRA TELECOMUNICAZIONI", "AETHRA-TELECOMUNICAZIONI" },
	{ 0x00D0D7, "B2C2, INC.", "B2C2" },
	{ 0x00D0D8, "3Com Corporation", "3COM" },
	{ 0x00D0D9, "DEDICATED MICROCOMPUTERS", "DEDICATED-MICROCOMPUTERS" },
	{ 0x00D0DA, "TAICOM DATA SYSTEMS CO., LTD.", "TAICOM-DATA" },
	{ 0x00D0DB, "MCQUAY INTERNATIONAL", "MCQUAY" },
	{ 0x00D0DC, "MODULAR MINING SYSTEMS, INC.", "MODULAR-MINING" },
	{ 0x00D0DD, "SUNRISE TELECOM, INC.", "SUNRISE-TELECOM" },
	{ 0x00D0DE, "PHILIPS MULTIMEDIA NETWORK", "PHILIPS-MULTIMEDIA-NETWORK" },
	{ 0x00D0DF, "KUZUMI ELECTRONICS, INC.", "KUZUMI-ELECTRONICS" },
	{ 0x00D0E0, "DOOIN ELECTRONICS CO.", "DOOIN-ELECTRONICS" },
	{ 0x00D0E1, "AVIONITEK ISRAEL INC.", "AVIONITEK-ISRAEL" },
	{ 0x00D0E2, "MRT MICRO, INC.", "MRT-MICRO" },
	{ 0x00D0E3, "ELE-CHEM ENGINEERING CO., LTD.", "ELE-CHEM-ENGINEERING" },
	{ 0x00D0E4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D0E5, "SOLIDUM SYSTEMS CORP.", "SOLIDUM" },
	{ 0x00D0E6, "IBOND INC.", "IBOND" },
	{ 0x00D0E7, "VCON TELECOMMUNICATION LTD.", "VCON-TELECOMMUNICATION" },
	{ 0x00D0E8, "MAC SYSTEM CO., LTD.", "MAC-SYSTEM" },
	{ 0x00D0E9, "Advantage Century Telecommunication Corp.", "ADVANTAGE-CENTURY-TELECOMMUNICATION" },
	{ 0x00D0EA, "NEXTONE COMMUNICATIONS, INC.", "NEXTONE-COMMUNICATION" },
	{ 0x00D0EB, "LIGHTERA NETWORKS, INC.", "LIGHTERA-NETWORKS" },
	{ 0x00D0EC, "NAKAYO TELECOMMUNICATIONS, INC", "NAKAYO-TELECOMMUNICATIONS" },
	{ 0x00D0ED, "XIOX", "XIOX" },
	{ 0x00D0EE, "DICTAPHONE CORPORATION", "DICTAPHONE" },
	{ 0x00D0EF, "IGT", "IGT" },
	{ 0x00D0F0, "CONVISION TECHNOLOGY GMBH", "CONVISION" },
	{ 0x00D0F1, "SEGA ENTERPRISES, LTD.", "SEGA-ENTERPRISES" },
	{ 0x00D0F2, "MONTEREY NETWORKS", "MONTEREY-NETWORKS" },
	{ 0x00D0F3, "SOLARI DI UDINE SPA", "SOLARI-DI-UDINE-SPA" },
	{ 0x00D0F4, "CARINTHIAN TECH INSTITUTE", "CARINTHIAN-TECH-INSTITUTE" },
	{ 0x00D0F5, "ORANGE MICRO, INC.", "ORANGE-MICRO" },
	{ 0x00D0F6, "Alcatel Canada", "ALCATEL-CANADA" },
	{ 0x00D0F7, "NEXT NETS CORPORATION", "NEXT-NETS" },
	{ 0x00D0F8, "FUJIAN STAR TERMINAL", "FUJIAN-STAR-TERMINAL" },
	{ 0x00D0F9, "ACUTE COMMUNICATIONS CORP.", "ACUTE-COMMUNICATION" },
	{ 0x00D0FA, "Thales e-Security Ltd.", "THALES-E-SECURITY" },
	{ 0x00D0FB, "TEK MICROSYSTEMS, INCORPORATED", "TEK-MICROSYSTEMS-INCORPORATED" },
	{ 0x00D0FC, "GRANITE MICROSYSTEMS", "GRANITE-MICROSYSTEMS" },
	{ 0x00D0FD, "OPTIMA TELE.COM, INC.", "OPTIMA-TELE-COM" },
	{ 0x00D0FE, "ASTRAL POINT", "ASTRAL-POINT" },
	{ 0x00D0FF, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00D11C, "ACETEL", "ACETEL" },
	{ 0x00D38D, "Hotel Technology Next Generation", "HOTEL-NEXT-GENERATION" },
	{ 0x00D632, "GE Energy", "GE-ENERGY" },
	{ 0x00D9D1, "Sony Computer Entertainment Inc.", "SONY-ENTERTAINMENT" },
	{ 0x00DB1E, "Albedo Telecom SL", "ALBEDO-TELECOM-SL" },
	{ 0x00DB45, "THAMWAY CO.,LTD.", "THAMWAY" },
	{ 0x00DBDF, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x00DD00, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD01, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD02, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD03, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD04, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD05, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD06, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD07, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD08, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD09, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD0A, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD0B, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD0C, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD0D, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD0E, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DD0F, "UNGERMANN-BASS INC.", "UNGERMANN-BASS" },
	{ 0x00DEFB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E000, "Fujitsu Limited", "FUJITSU" },
	{ 0x00E001, "STRAND LIGHTING LIMITED", "STRAND-LIGHTING" },
	{ 0x00E002, "CROSSROADS SYSTEMS, INC.", "CROSSROADS" },
	{ 0x00E003, "NOKIA WIRELESS BUSINESS COMMUN", "NOKIA-WIRELESS-BUSINESS-COMMUN" },
	{ 0x00E004, "PMC-SIERRA, INC.", "PMC-SIERRA" },
	{ 0x00E005, "TECHNICAL CORP.", "TECHNICAL" },
	{ 0x00E006, "SILICON INTEGRATED SYS. CORP.", "SILICON-INTEGRATED-SYS" },
	{ 0x00E007, "Avaya ECS Ltd", "AVAYA-ECS" },
	{ 0x00E008, "AMAZING CONTROLS! INC.", "AMAZING-CONTROLS!" },
	{ 0x00E009, "MARATHON TECHNOLOGIES CORP.", "MARATHON-TECHNOLOGIES" },
	{ 0x00E00A, "DIBA, INC.", "DIBA" },
	{ 0x00E00B, "ROOFTOP COMMUNICATIONS CORP.", "ROOFTOP-COMMUNICATION" },
	{ 0x00E00C, "MOTOROLA", "MOTOROLA" },
	{ 0x00E00D, "RADIANT SYSTEMS", "RADIANT" },
	{ 0x00E00E, "AVALON IMAGING SYSTEMS, INC.", "AVALON-IMAGING" },
	{ 0x00E00F, "SHANGHAI BAUD DATA", "SHANGHAI-BAUD-DATA" },
	{ 0x00E010, "HESS SB-AUTOMATENBAU GmbH", "HESS-SB-AUTOMATENBAU" },
	{ 0x00E011, "Uniden Corporation", "UNIDEN" },
	{ 0x00E012, "PLUTO TECHNOLOGIES INTERNATIONAL INC.", "PLUTO-TECHNOLOGIES" },
	{ 0x00E013, "EASTERN ELECTRONIC CO., LTD.", "EASTERN" },
	{ 0x00E014, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E015, "HEIWA CORPORATION", "HEIWA" },
	{ 0x00E016, "RAPID CITY COMMUNICATIONS", "RAPID-CITY-COMMUNICATION" },
	{ 0x00E017, "EXXACT GmbH", "EXXACT" },
	{ 0x00E018, "ASUSTEK COMPUTER INC.", "ASUSTEK" },
	{ 0x00E019, "ING. GIORDANO ELETTRONICA", "ING-GIORDANO-ELETTRONICA" },
	{ 0x00E01A, "COMTEC SYSTEMS. CO., LTD.", "COMTEC" },
	{ 0x00E01B, "SPHERE COMMUNICATIONS, INC.", "SPHERE-COMMUNICATION" },
	{ 0x00E01C, "Cradlepoint, Inc", "CRADLEPOINT" },
	{ 0x00E01D, "WebTV NETWORKS, INC.", "WEBTV-NETWORKS" },
	{ 0x00E01E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E01F, "AVIDIA Systems, Inc.", "AVIDIA" },
	{ 0x00E020, "TECNOMEN OY", "TECNOMEN-OY" },
	{ 0x00E021, "FREEGATE CORP.", "FREEGATE" },
	{ 0x00E022, "Analog Devices Inc.", "ANALOG-DEVICES" },
	{ 0x00E023, "TELRAD", "TELRAD" },
	{ 0x00E024, "GADZOOX NETWORKS", "GADZOOX-NETWORKS" },
	{ 0x00E025, "dit Co., Ltd.", "DIT" },
	{ 0x00E026, "Redlake MASD LLC", "REDLAKE-MASD-LLC" },
	{ 0x00E027, "DUX, INC.", "DUX" },
	{ 0x00E028, "APTIX CORPORATION", "APTIX" },
	{ 0x00E029, "STANDARD MICROSYSTEMS CORP.", "STANDARD-MICROSYSTEMS" },
	{ 0x00E02A, "TANDBERG TELEVISION AS", "TANDBERG-TELEVISION-AS" },
	{ 0x00E02B, "EXTREME NETWORKS", "EXTREME-NETWORKS" },
	{ 0x00E02C, "AST COMPUTER", "AST" },
	{ 0x00E02D, "InnoMediaLogic, Inc.", "INNOMEDIALOGIC" },
	{ 0x00E02E, "SPC ELECTRONICS CORPORATION", "SPC-ELECTRONICS" },
	{ 0x00E02F, "MCNS HOLDINGS, L.P.", "MCNS-HOLDINGS-L-P" },
	{ 0x00E030, "MELITA INTERNATIONAL CORP.", "MELITA" },
	{ 0x00E031, "HAGIWARA ELECTRIC CO., LTD.", "HAGIWARA-ELECTRIC" },
	{ 0x00E032, "MISYS FINANCIAL SYSTEMS, LTD.", "MISYS-FINANCIAL" },
	{ 0x00E033, "E.E.P.D. GmbH", "E-E-P-D" },
	{ 0x00E034, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E035, "Artesyn Embedded Technologies", "ARTESYN-EMBEDDED-TECHNOLOGIES" },
	{ 0x00E036, "PIONEER CORPORATION", "PIONEER" },
	{ 0x00E037, "CENTURY CORPORATION", "CENTURY" },
	{ 0x00E038, "PROXIMA CORPORATION", "PROXIMA" },
	{ 0x00E039, "PARADYNE CORP.", "PARADYNE" },
	{ 0x00E03A, "CABLETRON SYSTEMS, INC.", "CABLETRON" },
	{ 0x00E03B, "PROMINET CORPORATION", "PROMINET" },
	{ 0x00E03C, "AdvanSys", "ADVANSYS" },
	{ 0x00E03D, "FOCON ELECTRONIC SYSTEMS A/S", "FOCON-A/S" },
	{ 0x00E03E, "ALFATECH, INC.", "ALFATECH" },
	{ 0x00E03F, "JATON CORPORATION", "JATON" },
	{ 0x00E040, "DeskStation Technology, Inc.", "DESKSTATION" },
	{ 0x00E041, "CSPI", "CSPI" },
	{ 0x00E042, "Pacom Systems Ltd.", "PACOM" },
	{ 0x00E043, "VitalCom", "VITALCOM" },
	{ 0x00E044, "LSICS CORPORATION", "LSICS" },
	{ 0x00E045, "TOUCHWAVE, INC.", "TOUCHWAVE" },
	{ 0x00E046, "BENTLY NEVADA CORP.", "BENTLY-NEVADA" },
	{ 0x00E047, "InFocus Corporation", "INFOCUS" },
	{ 0x00E048, "SDL COMMUNICATIONS, INC.", "SDL-COMMUNICATION" },
	{ 0x00E049, "MICROWI ELECTRONIC GmbH", "MICROWI" },
	{ 0x00E04A, "ZX Technologies, Inc", "ZX-TECHNOLOGIES" },
	{ 0x00E04B, "JUMP INDUSTRIELLE COMPUTERTECHNIK GmbH", "JUMP-INDUSTRIELLE-COMPUTERTECHNIK" },
	{ 0x00E04C, "REALTEK SEMICONDUCTOR CORP.", "REALTEK-SEMICONDUCTOR" },
	{ 0x00E04D, "INTERNET INITIATIVE JAPAN, INC", "INTERNET-INITIATIVE-JAPAN" },
	{ 0x00E04E, "SANYO DENKI CO., LTD.", "SANYO-DENKI" },
	{ 0x00E04F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E050, "EXECUTONE INFORMATION SYSTEMS, INC.", "EXECUTONE-INFORMATION" },
	{ 0x00E051, "TALX CORPORATION", "TALX" },
	{ 0x00E052, "Brocade Communications Systems, Inc", "BROCADE-COMMUNICATION" },
	{ 0x00E053, "CELLPORT LABS, INC.", "CELLPORT-LABS" },
	{ 0x00E054, "KODAI HITEC CO., LTD.", "KODAI-HITEC" },
	{ 0x00E055, "INGENIERIA ELECTRONICA COMERCIAL INELCOM S.A.", "INGENIERIA-ELECTRONICA-COMERCIAL-INELCOM" },
	{ 0x00E056, "HOLONTECH CORPORATION", "HOLONTECH" },
	{ 0x00E057, "HAN MICROTELECOM. CO., LTD.", "HAN-MICROTELECOM" },
	{ 0x00E058, "PHASE ONE DENMARK A/S", "PHASE-ONE-DENMARK-A/S" },
	{ 0x00E059, "CONTROLLED ENVIRONMENTS, LTD.", "CONTROLLED-ENVIRONMENTS" },
	{ 0x00E05A, "GALEA NETWORK SECURITY", "GALEA-NETWORK-SECURITY" },
	{ 0x00E05B, "WEST END SYSTEMS CORP.", "WEST-END" },
	{ 0x00E05C, "MATSUSHITA KOTOBUKI ELECTRONICS INDUSTRIES, LTD.", "MATSUSHITA-KOTOBUKI-ELECTRONICS-INDUSTRIES" },
	{ 0x00E05D, "UNITEC CO., LTD.", "UNITEC" },
	{ 0x00E05E, "JAPAN AVIATION ELECTRONICS INDUSTRY, LTD.", "JAPAN-AVIATION-ELECTRONICS-INDUSTRY" },
	{ 0x00E05F, "e-Net, Inc.", "E-NET" },
	{ 0x00E060, "SHERWOOD", "SHERWOOD" },
	{ 0x00E061, "EdgePoint Networks, Inc.", "EDGEPOINT-NETWORKS" },
	{ 0x00E062, "HOST ENGINEERING", "HOST-ENGINEERING" },
	{ 0x00E063, "CABLETRON - YAGO SYSTEMS, INC.", "CABLETRON---YAGO" },
	{ 0x00E064, "SAMSUNG ELECTRONICS", "SAMSUNG-ELECTRONICS" },
	{ 0x00E065, "OPTICAL ACCESS INTERNATIONAL", "OPTICAL-ACCESS" },
	{ 0x00E066, "ProMax Systems, Inc.", "PROMAX" },
	{ 0x00E067, "eac AUTOMATION-CONSULTING GmbH", "EAC-AUTOMATION-CONSULTING" },
	{ 0x00E068, "MERRIMAC SYSTEMS INC.", "MERRIMAC" },
	{ 0x00E069, "JAYCOR", "JAYCOR" },
	{ 0x00E06A, "KAPSCH AG", "KAPSCH" },
	{ 0x00E06B, "W&G SPECIAL PRODUCTS", "W-G-SPECIAL-PRODUCTS" },
	{ 0x00E06C, "Ultra Electronics Limited (AEP Networks)", "ULTRA-ELECTRONICS-AEP-NETWORKS" },
	{ 0x00E06D, "COMPUWARE CORPORATION", "COMPUWARE" },
	{ 0x00E06E, "FAR SYSTEMS S.p.A.", "FAR-S-P-A" },
	{ 0x00E06F, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x00E070, "DH TECHNOLOGY", "DH" },
	{ 0x00E071, "EPIS MICROCOMPUTER", "EPIS-MICROCOMPUTER" },
	{ 0x00E072, "LYNK", "LYNK" },
	{ 0x00E073, "NATIONAL AMUSEMENT NETWORK, INC.", "NATIONAL-AMUSEMENT-NETWORK" },
	{ 0x00E074, "TIERNAN COMMUNICATIONS, INC.", "TIERNAN-COMMUNICATION" },
	{ 0x00E075, "Verilink Corporation", "VERILINK" },
	{ 0x00E076, "DEVELOPMENT CONCEPTS, INC.", "DEVELOPMENT-CONCEPTS" },
	{ 0x00E077, "WEBGEAR, INC.", "WEBGEAR" },
	{ 0x00E078, "BERKELEY NETWORKS", "BERKELEY-NETWORKS" },
	{ 0x00E079, "A.T.N.R.", "A-T-N-R" },
	{ 0x00E07A, "MIKRODIDAKT AB", "MIKRODIDAKT-AB" },
	{ 0x00E07B, "BAY NETWORKS", "BAY-NETWORKS" },
	{ 0x00E07C, "METTLER-TOLEDO, INC.", "METTLER-TOLEDO" },
	{ 0x00E07D, "NETRONIX, INC.", "NETRONIX" },
	{ 0x00E07E, "WALT DISNEY IMAGINEERING", "WALT-DISNEY-IMAGINEERING" },
	{ 0x00E07F, "LOGISTISTEM s.r.l.", "LOGISTISTEM-S-R-L" },
	{ 0x00E080, "CONTROL RESOURCES CORPORATION", "CONTROL-RESOURCES" },
	{ 0x00E081, "TYAN COMPUTER CORP.", "TYAN" },
	{ 0x00E082, "ANERMA", "ANERMA" },
	{ 0x00E083, "JATO TECHNOLOGIES, INC.", "JATO-TECHNOLOGIES" },
	{ 0x00E084, "COMPULITE R&D", "COMPULITE-R-D" },
	{ 0x00E085, "GLOBAL MAINTECH, INC.", "GLOBAL-MAINTECH" },
	{ 0x00E086, "Emerson Network Power, Avocent Division", "EMERSON-NETWORK-POWER-AVOCENT-DIVISION" },
	{ 0x00E087, "LeCroy - Networking Productions Division", "LECROY---NETWORKING-PRODUCTIONS-DIVISION" },
	{ 0x00E088, "LTX-Credence CORPORATION", "LTX-CREDENCE" },
	{ 0x00E089, "ION Networks, Inc.", "ION-NETWORKS" },
	{ 0x00E08A, "GEC AVERY, LTD.", "GEC-AVERY" },
	{ 0x00E08B, "QLogic Corp.", "QLOGIC" },
	{ 0x00E08C, "NEOPARADIGM LABS, INC.", "NEOPARADIGM-LABS" },
	{ 0x00E08D, "PRESSURE SYSTEMS, INC.", "PRESSURE" },
	{ 0x00E08E, "UTSTARCOM", "UTSTARCOM" },
	{ 0x00E08F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E090, "BECKMAN LAB. AUTOMATION DIV.", "BECKMAN-LAB-AUTOMATION-DIV" },
	{ 0x00E091, "LG ELECTRONICS, INC.", "LG-ELECTRONICS" },
	{ 0x00E092, "ADMTEK INCORPORATED", "ADMTEK-INCORPORATED" },
	{ 0x00E093, "ACKFIN NETWORKS", "ACKFIN-NETWORKS" },
	{ 0x00E094, "OSAI SRL", "OSAI-SRL" },
	{ 0x00E095, "ADVANCED-VISION TECHNOLGIES CORP.", "ADVANCED-VISION-TECHNOLGIES" },
	{ 0x00E096, "SHIMADZU CORPORATION", "SHIMADZU" },
	{ 0x00E097, "CARRIER ACCESS CORPORATION", "CARRIER-ACCESS" },
	{ 0x00E098, "AboCom Systems, Inc.", "ABOCOM" },
	{ 0x00E099, "SAMSON AG", "SAMSON" },
	{ 0x00E09A, "Positron Inc.", "POSITRON" },
	{ 0x00E09B, "ENGAGE NETWORKS, INC.", "ENGAGE-NETWORKS" },
	{ 0x00E09C, "MII", "MII" },
	{ 0x00E09D, "SARNOFF CORPORATION", "SARNOFF" },
	{ 0x00E09E, "QUANTUM CORPORATION", "QUANTUM" },
	{ 0x00E09F, "PIXEL VISION", "PIXEL-VISION" },
	{ 0x00E0A0, "WILTRON CO.", "WILTRON" },
	{ 0x00E0A1, "HIMA PAUL HILDEBRANDT GmbH Co. KG", "HIMA-PAUL-HILDEBRANDT" },
	{ 0x00E0A2, "MICROSLATE INC.", "MICROSLATE" },
	{ 0x00E0A3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E0A4, "ESAOTE S.p.A.", "ESAOTE-S-P-A" },
	{ 0x00E0A5, "ComCore Semiconductor, Inc.", "COMCORE-SEMICONDUCTOR" },
	{ 0x00E0A6, "TELOGY NETWORKS, INC.", "TELOGY-NETWORKS" },
	{ 0x00E0A7, "IPC INFORMATION SYSTEMS, INC.", "IPC-INFORMATION" },
	{ 0x00E0A8, "SAT GmbH & Co.", "SAT" },
	{ 0x00E0A9, "FUNAI ELECTRIC CO., LTD.", "FUNAI-ELECTRIC" },
	{ 0x00E0AA, "ELECTROSONIC LTD.", "ELECTROSONIC" },
	{ 0x00E0AB, "DIMAT S.A.", "DIMAT" },
	{ 0x00E0AC, "MIDSCO, INC.", "MIDSCO" },
	{ 0x00E0AD, "EES TECHNOLOGY, LTD.", "EES" },
	{ 0x00E0AE, "XAQTI CORPORATION", "XAQTI" },
	{ 0x00E0AF, "GENERAL DYNAMICS INFORMATION SYSTEMS", "GENERAL-DYNAMICS-INFORMATION" },
	{ 0x00E0B0, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E0B1, "Alcatel-Lucent, Enterprise Business Group", "ALCATEL-LUCENT-BUSINESS-GROUP" },
	{ 0x00E0B2, "TELMAX COMMUNICATIONS CORP.", "TELMAX-COMMUNICATION" },
	{ 0x00E0B3, "EtherWAN Systems, Inc.", "ETHERWAN" },
	{ 0x00E0B4, "TECHNO SCOPE CO., LTD.", "TECHNO-SCOPE" },
	{ 0x00E0B5, "ARDENT COMMUNICATIONS CORP.", "ARDENT-COMMUNICATION" },
	{ 0x00E0B6, "Entrada Networks", "ENTRADA-NETWORKS" },
	{ 0x00E0B7, "PI GROUP, LTD.", "PI-GROUP" },
	{ 0x00E0B8, "GATEWAY 2000", "GATEWAY-2000" },
	{ 0x00E0B9, "BYAS SYSTEMS", "BYAS" },
	{ 0x00E0BA, "BERGHOF AUTOMATIONSTECHNIK GmbH", "BERGHOF-AUTOMATIONSTECHNIK" },
	{ 0x00E0BB, "NBX CORPORATION", "NBX" },
	{ 0x00E0BC, "SYMON COMMUNICATIONS, INC.", "SYMON-COMMUNICATION" },
	{ 0x00E0BD, "INTERFACE SYSTEMS, INC.", "INTERFACE" },
	{ 0x00E0BE, "GENROCO INTERNATIONAL, INC.", "GENROCO" },
	{ 0x00E0BF, "TORRENT NETWORKING TECHNOLOGIES CORP.", "TORRENT-NETWORKING-TECHNOLOGIES" },
	{ 0x00E0C0, "SEIWA ELECTRIC MFG. CO., LTD.", "SEIWA-ELECTRIC-MFG" },
	{ 0x00E0C1, "MEMOREX TELEX JAPAN, LTD.", "MEMOREX-TELEX-JAPAN" },
	{ 0x00E0C2, "NECSY S.p.A.", "NECSY-S-P-A" },
	{ 0x00E0C3, "SAKAI SYSTEM DEVELOPMENT CORP.", "SAKAI-SYSTEM-DEVELOPMENT" },
	{ 0x00E0C4, "HORNER ELECTRIC, INC.", "HORNER-ELECTRIC" },
	{ 0x00E0C5, "BCOM ELECTRONICS INC.", "BCOM-ELECTRONICS" },
	{ 0x00E0C6, "LINK2IT, L.L.C.", "LINK2IT-L-L-C" },
	{ 0x00E0C7, "EUROTECH SRL", "EUROTECH-SRL" },
	{ 0x00E0C8, "VIRTUAL ACCESS, LTD.", "VIRTUAL-ACCESS" },
	{ 0x00E0C9, "AutomatedLogic Corporation", "AUTOMATEDLOGIC" },
	{ 0x00E0CA, "BEST DATA PRODUCTS", "BEST-DATA-PRODUCTS" },
	{ 0x00E0CB, "RESON, INC.", "RESON" },
	{ 0x00E0CC, "HERO SYSTEMS, LTD.", "HERO" },
	{ 0x00E0CD, "SAAB SENSIS CORPORATION", "SAAB-SENSIS" },
	{ 0x00E0CE, "ARN", "ARN" },
	{ 0x00E0CF, "INTEGRATED DEVICE TECHNOLOGY, INC.", "INTEGRATED-DEVICE" },
	{ 0x00E0D0, "NETSPEED, INC.", "NETSPEED" },
	{ 0x00E0D1, "TELSIS LIMITED", "TELSIS" },
	{ 0x00E0D2, "VERSANET COMMUNICATIONS, INC.", "VERSANET-COMMUNICATION" },
	{ 0x00E0D3, "DATENTECHNIK GmbH", "DATENTECHNIK" },
	{ 0x00E0D4, "EXCELLENT COMPUTER", "EXCELLENT" },
	{ 0x00E0D5, "Emulex Corporation", "EMULEX" },
	{ 0x00E0D6, "COMPUTER & COMMUNICATION RESEARCH LAB.", "COMMUNICATION-RESEARCH-LAB" },
	{ 0x00E0D7, "SUNSHINE ELECTRONICS, INC.", "SUNSHINE-ELECTRONICS" },
	{ 0x00E0D8, "LANBit Computer, Inc.", "LANBIT" },
	{ 0x00E0D9, "TAZMO CO., LTD.", "TAZMO" },
	{ 0x00E0DA, "Alcatel North America ESD", "ALCATEL-NORTH-AMERICA-ESD" },
	{ 0x00E0DB, "ViaVideo Communications, Inc.", "VIAVIDEO-COMMUNICATION" },
	{ 0x00E0DC, "NEXWARE CORP.", "NEXWARE" },
	{ 0x00E0DD, "ZENITH ELECTRONICS CORPORATION", "ZENITH-ELECTRONICS" },
	{ 0x00E0DE, "DATAX NV", "DATAX-NV" },
	{ 0x00E0DF, "KEYMILE GmbH", "KEYMILE" },
	{ 0x00E0E0, "SI ELECTRONICS, LTD.", "SI-ELECTRONICS" },
	{ 0x00E0E1, "G2 NETWORKS, INC.", "G2-NETWORKS" },
	{ 0x00E0E2, "INNOVA CORP.", "INNOVA" },
	{ 0x00E0E3, "SK-ELEKTRONIK GmbH", "SK-ELEKTRONIK" },
	{ 0x00E0E4, "FANUC ROBOTICS NORTH AMERICA, Inc.", "FANUC-ROBOTICS-NORTH-AMERICA" },
	{ 0x00E0E5, "CINCO NETWORKS, INC.", "CINCO-NETWORKS" },
	{ 0x00E0E6, "INCAA DATACOM B.V.", "INCAA-DATACOM" },
	{ 0x00E0E7, "RAYTHEON E-SYSTEMS, INC.", "RAYTHEON-E" },
	{ 0x00E0E8, "GRETACODER Data Systems AG", "GRETACODER-DATA" },
	{ 0x00E0E9, "DATA LABS, INC.", "DATA-LABS" },
	{ 0x00E0EA, "INNOVAT COMMUNICATIONS, INC.", "INNOVAT-COMMUNICATION" },
	{ 0x00E0EB, "DIGICOM SYSTEMS, INCORPORATED", "DIGICOM-INCORPORATED" },
	{ 0x00E0EC, "CELESTICA INC.", "CELESTICA" },
	{ 0x00E0ED, "SILICOM, LTD.", "SILICOM" },
	{ 0x00E0EE, "MAREL HF", "MAREL-HF" },
	{ 0x00E0EF, "DIONEX", "DIONEX" },
	{ 0x00E0F0, "ABLER TECHNOLOGY, INC.", "ABLER" },
	{ 0x00E0F1, "THAT CORPORATION", "THAT" },
	{ 0x00E0F2, "ARLOTTO COMNET, INC.", "ARLOTTO-COMNET" },
	{ 0x00E0F3, "WebSprint Communications, Inc.", "WEBSPRINT-COMMUNICATION" },
	{ 0x00E0F4, "INSIDE Technology A/S", "INSIDE-A/S" },
	{ 0x00E0F5, "TELES AG", "TELES" },
	{ 0x00E0F6, "DECISION EUROPE", "DECISION-EUROPE" },
	{ 0x00E0F7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E0F8, "DICNA CONTROL AB", "DICNA-CONTROL-AB" },
	{ 0x00E0F9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E0FA, "TRL TECHNOLOGY, LTD.", "TRL" },
	{ 0x00E0FB, "LEIGHTRONIX, INC.", "LEIGHTRONIX" },
	{ 0x00E0FC, "HUAWEI TECHNOLOGIES CO., LTD.", "HUAWEI-TECHNOLOGIES" },
	{ 0x00E0FD, "A-TREND TECHNOLOGY CO., LTD.", "A-TREND" },
	{ 0x00E0FE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x00E0FF, "SECURITY DYNAMICS TECHNOLOGIES, Inc.", "SECURITY-DYNAMICS-TECHNOLOGIES" },
	{ 0x00E16D, "Cisco", "CISCO" },
	{ 0x00E175, "AK-Systems Ltd", "AK" },
	{ 0x00E3B2, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x00E666, "ARIMA Communications Corp.", "ARIMA-COMMUNICATION" },
	{ 0x00E6D3, "NIXDORF COMPUTER CORP.", "NIXDORF" },
	{ 0x00E6E8, "Netzin Technology Corporation,.Ltd.", "NETZIN" },
	{ 0x00E8AB, "Meggitt Training Systems, Inc.", "MEGGITT-TRAINING" },
	{ 0x00EB2D, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x00EEBD, "HTC Corporation", "HTC" },
	{ 0x00F051, "KWB Gmbh", "KWB" },
	{ 0x00F3DB, "WOO Sports", "WOO-SPORTS" },
	{ 0x00F403, "Orbis Systems Oy", "ORBIS-OY" },
	{ 0x00F46F, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0x00F4B9, "Apple", "APPLE" },
	{ 0x00F76F, "Apple", "APPLE" },
	{ 0x00F860, "PT. Panggung Electric Citrabuana", "PT-PANGGUNG-ELECTRIC-CITRABUANA" },
	{ 0x00FA3B, "CLOOS ELECTRONIC GMBH", "CLOOS" },
	{ 0x00FC58, "WebSilicon Ltd.", "WEBSILICON" },
	{ 0x00FC70, "Intrepid Control Systems, Inc.", "INTREPID-CONTROL" },
	{ 0x00FD4C, "NEVATEC", "NEVATEC" },
	{ 0x020701, "RACAL-DATACOM", "RACAL-DATACOM" },
	{ 0x021C7C, "PERQ SYSTEMS CORPORATION", "PERQ" },
	{ 0x026086, "LOGIC REPLACEMENT TECH. LTD.", "LOGIC-REPLACEMENT-TECH" },
	{ 0x02608C, "3COM CORPORATION", "3COM" },
	{ 0x027001, "RACAL-DATACOM", "RACAL-DATACOM" },
	{ 0x0270B0, "M/A-COM INC. COMPANIES", "M/A-COM-COMPANIES" },
	{ 0x0270B3, "DATA RECALL LTD", "DATA-RECALL" },
	{ 0x029D8E, "CARDIAC RECORDERS INC.", "CARDIAC-RECORDERS" },
	{ 0x02AA3C, "OLIVETTI TELECOMM SPA (OLTECO)", "OLIVETTI-TELECOMM-SPA-OLTECO" },
	{ 0x02BB01, "OCTOTHORPE CORP.", "OCTOTHORPE" },
	{ 0x02C08C, "3COM CORPORATION", "3COM" },
	{ 0x02CF1C, "COMMUNICATION MACHINERY CORP.", "COMMUNICATION-MACHINERY" },
	{ 0x02E6D3, "NIXDORF COMPUTER CORPORATION", "NIXDORF" },
	{ 0x040A83, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x040AE0, "XMIT AG COMPUTER NETWORKS", "XMIT-NETWORKS" },
	{ 0x040CCE, "Apple", "APPLE" },
	{ 0x040EC2, "ViewSonic Mobile China Limited", "VIEWSONIC-MOBILE-CHINA" },
	{ 0x041552, "Apple", "APPLE" },
	{ 0x04180F, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x0418B6, "PRIVATE", "PRIVATE" },
	{ 0x0418D6, "Ubiquiti Networks", "UBIQUITI-NETWORKS" },
	{ 0x041A04, "WaveIP", "WAVEIP" },
	{ 0x041B94, "Host Mobility AB", "HOST-MOBILITY-AB" },
	{ 0x041BBA, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x041D10, "Dream Ware Inc.", "DREAM-WARE" },
	{ 0x041E64, "Apple", "APPLE" },
	{ 0x04209A, "Panasonic AVC Networks Company", "PANASONIC-AVC-NETWORKS-COMPANY" },
	{ 0x042234, "Wireless Standard Extensions", "WIRELESS-STANDARD-EXTENSIONS" },
	{ 0x042605, "GFR Gesellschaft für Regelungstechnik und Energieeinsparung mbH", "GFR-GESELLSCHAFT-FüR-REGELUNGSTECHNIK-UND-ENERGIEEINSPARUNG-MBH" },
	{ 0x042665, "Apple", "APPLE" },
	{ 0x042BBB, "PicoCELA, Inc.", "PICOCELA" },
	{ 0x042F56, "ATOCS (Shenzhen) LTD", "ATOCS-SHENZHEN" },
	{ 0x0432F4, "Partron", "PARTRON" },
	{ 0x043604, "Gyeyoung I&T", "GYEYOUNG-I-T" },
	{ 0x043D98, "ChongQing QingJia Electronics CO.,LTD", "CHONGQING-QINGJIA-ELECTRONICS" },
	{ 0x0444A1, "TELECON GALICIA,S.A.", "TELECON-GALICIA" },
	{ 0x044665, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x04489A, "Apple", "APPLE" },
	{ 0x044A50, "Ramaxel Technology (Shenzhen) limited company", "RAMAXEL-SHENZHEN-COMPANY" },
	{ 0x044BFF, "GuangZhou Hedy Digital Technology Co., Ltd", "GUANGZHOU-HEDY-DIGITAL" },
	{ 0x044CEF, "Fujian Sanao Technology Co.,Ltd", "FUJIAN-SANAO" },
	{ 0x044E06, "Ericsson AB", "ERICSSON-AB" },
	{ 0x044F8B, "Adapteva, Inc.", "ADAPTEVA" },
	{ 0x044FAA, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x045453, "Apple", "APPLE" },
	{ 0x0455CA, "BriView (Xiamen) Corp.", "BRIVIEW-XIAMEN" },
	{ 0x04572F, "Sertel Electronics UK Ltd", "SERTEL-ELECTRONICS-UK" },
	{ 0x04586F, "Sichuan Whayer information industry Co.,LTD", "SICHUAN-WHAYER-INFORMATION-INDUSTRY" },
	{ 0x045A95, "Nokia Corporation", "NOKIA" },
	{ 0x045C06, "Zmodo Technology Corporation", "ZMODO" },
	{ 0x045C8E, "gosund GROUP CO.,LTD", "GOSUND-GROUP" },
	{ 0x045D56, "camtron industrial inc.", "CAMTRON-INDUSTRIAL" },
	{ 0x045FA7, "Shenzhen Yichen Technology Development Co.,LTD", "SHENZHEN-YICHEN-DEVELOPMENT" },
	{ 0x0462D7, "ALSTOM HYDRO FRANCE", "ALSTOM-HYDRO-FRANCE" },
	{ 0x0463E0, "Nome Oy", "NOME-OY" },
	{ 0x046785, "scemtec Hard- und Software fuer Mess- und Steuerungstechnik GmbH", "SCEMTEC-HARD--UND-SOFTWARE-FUER-MESS--UND-STEUERUNGSTECHNIK" },
	{ 0x046D42, "Bryston Ltd.", "BRYSTON" },
	{ 0x046E49, "TaiYear Electronic Technology (Suzhou) Co., Ltd", "TAIYEAR-SUZHOU" },
	{ 0x0470BC, "Globalstar Inc.", "GLOBALSTAR" },
	{ 0x0474A1, "Aligera Equipamentos Digitais Ltda", "ALIGERA-EQUIPAMENTOS-DIGITAIS-LTDA" },
	{ 0x0475F5, "CSST", "CSST" },
	{ 0x04766E, "ALPS Co,. Ltd.", "ALPS" },
	{ 0x047D7B, "Quanta Computer Inc.", "QUANTA" },
	{ 0x0481AE, "Clack Corporation", "CLACK" },
	{ 0x04848A, "7INOVA TECHNOLOGY LIMITED", "7INOVA" },
	{ 0x04888C, "Eifelwerk Butler Systeme GmbH", "EIFELWERK-BUTLER-SYSTEME" },
	{ 0x0488E2, "Beats Electronics LLC", "BEATS-ELECTRONICS-LLC" },
	{ 0x048A15, "Avaya, Inc", "AVAYA" },
	{ 0x048B42, "Skspruce Technology Limited", "SKSPRUCE" },
	{ 0x048C03, "ThinPAD Technology (Shenzhen)CO.,LTD", "THINPAD-SHENZHEN" },
	{ 0x048D38, "Netcore Technology Inc.", "NETCORE" },
	{ 0x0492EE, "iway AG", "IWAY" },
	{ 0x0494A1, "CATCH THE WIND INC", "CATCH-THE-WIND" },
	{ 0x0498F3, "ALPS Electric Co,. Ltd.", "ALPS-ELECTRIC" },
	{ 0x0499E6, "Shenzhen Yoostar Technology Co., Ltd", "SHENZHEN-YOOSTAR" },
	{ 0x049B9C, "Eadingcore Intelligent Technology Co., Ltd.", "EADINGCORE-INTELLIGENT" },
	{ 0x049C62, "BMT Medical Technology s.r.o.", "BMT-MEDICAL-S-R-O" },
	{ 0x049F06, "Smobile Co., Ltd.", "SMOBILE" },
	{ 0x049F81, "Netscout Systems, Inc.", "NETSCOUT" },
	{ 0x04A151, "NETGEAR INC.,", "NETGEAR" },
	{ 0x04A3F3, "Emicon", "EMICON" },
	{ 0x04A82A, "Nokia Corporation", "NOKIA" },
	{ 0x04B3B6, "Seamap (UK) Ltd", "SEAMAP-UK" },
	{ 0x04B466, "BSP Co., Ltd.", "BSP" },
	{ 0x04BD70, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x04BD88, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0x04BFA8, "ISB Corporation", "ISB" },
	{ 0x04C05B, "Tigo Energy", "TIGO-ENERGY" },
	{ 0x04C06F, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x04C09C, "Tellabs Inc.", "TELLABS" },
	{ 0x04C1B9, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0x04C5A4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x04C880, "Samtec Inc", "SAMTEC" },
	{ 0x04C991, "Phistek INC.", "PHISTEK" },
	{ 0x04C9D9, "EchoStar Technologies Corp", "ECHOSTAR-TECHNOLOGIES" },
	{ 0x04CB1D, "Traka plc", "TRAKA-PLC" },
	{ 0x04CE14, "Wilocity LTD.", "WILOCITY" },
	{ 0x04CF25, "MANYCOLORS, INC.", "MANYCOLORS" },
	{ 0x04D437, "ZNV", "ZNV" },
	{ 0x04D783, "Y&H E&C Co.,LTD.", "Y-H-E-C" },
	{ 0x04DAD2, "Cisco", "CISCO" },
	{ 0x04DB56, "Apple, Inc.", "APPLE" },
	{ 0x04DB8A, "Suntech International Ltd.", "SUNTECH" },
	{ 0x04DD4C, "Velocytech", "VELOCYTECH" },
	{ 0x04DEDB, "Rockport Networks Inc", "ROCKPORT-NETWORKS" },
	{ 0x04DF69, "Car Connectivity Consortium", "CAR-CONNECTIVITY-CONSORTIUM" },
	{ 0x04E0C4, "TRIUMPH-ADLER AG", "TRIUMPH-ADLER" },
	{ 0x04E1C8, "IMS Soluções em Energia Ltda.", "IMS-SOLUçõES-EM-ENERGIA-LTDA" },
	{ 0x04E2F8, "AEP Ticketing solutions srl", "AEP-TICKETING-SOLUTIONS-SRL" },
	{ 0x04E451, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x04E536, "Apple", "APPLE" },
	{ 0x04E548, "Cohda Wireless Pty Ltd", "COHDA-WIRELESS-PTY" },
	{ 0x04E662, "Acroname Inc.", "ACRONAME" },
	{ 0x04E676, "AMPAK Technology Inc.", "AMPAK" },
	{ 0x04E9E5, "PJRC.COM, LLC", "PJRC-COM-LLC" },
	{ 0x04EE91, "x-fabric GmbH", "X-FABRIC" },
	{ 0x04F021, "Compex Systems Pte Ltd", "COMPEX-PTE" },
	{ 0x04F13E, "Apple", "APPLE" },
	{ 0x04F17D, "Tarana Wireless", "TARANA-WIRELESS" },
	{ 0x04F4BC, "Xena Networks", "XENA-NETWORKS" },
	{ 0x04F7E4, "Apple", "APPLE" },
	{ 0x04F8C2, "Flaircomm Microelectronics, Inc.", "FLAIRCOMM-MICROELECTRONICS" },
	{ 0x04F938, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x04FE31, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x04FE7F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x04FF51, "NOVAMEDIA INNOVISION SP. Z O.O.", "NOVAMEDIA-INNOVISION-SP-Z-O-O" },
	{ 0x080001, "COMPUTERVISION CORPORATION", "COMPUTERVISION" },
	{ 0x080002, "BRIDGE COMMUNICATIONS INC.", "BRIDGE-COMMUNICATION" },
	{ 0x080003, "ADVANCED COMPUTER COMM.", "ADVANCED-COMM" },
	{ 0x080004, "CROMEMCO INCORPORATED", "CROMEMCO-INCORPORATED" },
	{ 0x080005, "SYMBOLICS INC.", "SYMBOLICS" },
	{ 0x080006, "SIEMENS AG", "SIEMENS" },
	{ 0x080007, "Apple", "APPLE" },
	{ 0x080008, "BOLT BERANEK AND NEWMAN INC.", "BOLT-BERANEK-AND-NEWMAN" },
	{ 0x080009, "HEWLETT PACKARD", "HEWLETT-PACKARD" },
	{ 0x08000A, "NESTAR SYSTEMS INCORPORATED", "NESTAR-INCORPORATED" },
	{ 0x08000B, "UNISYS CORPORATION", "UNISYS" },
	{ 0x08000C, "MIKLYN DEVELOPMENT CO.", "MIKLYN-DEVELOPMENT" },
	{ 0x08000D, "INTERNATIONAL COMPUTERS LTD.", "COMPUTERS" },
	{ 0x08000E, "NCR CORPORATION", "NCR" },
	{ 0x08000F, "MITEL CORPORATION", "MITEL" },
	{ 0x080011, "TEKTRONIX INC.", "TEKTRONIX" },
	{ 0x080012, "BELL ATLANTIC INTEGRATED SYST.", "BELL-ATLANTIC-INTEGRATED-SYST" },
	{ 0x080013, "EXXON", "EXXON" },
	{ 0x080014, "EXCELAN", "EXCELAN" },
	{ 0x080015, "STC BUSINESS SYSTEMS", "STC-BUSINESS" },
	{ 0x080016, "BARRISTER INFO SYS CORP", "BARRISTER-INFO-SYS" },
	{ 0x080017, "NATIONAL SEMICONDUCTOR", "NATIONAL-SEMICONDUCTOR" },
	{ 0x080018, "PIRELLI FOCOM NETWORKS", "PIRELLI-FOCOM-NETWORKS" },
	{ 0x080019, "GENERAL ELECTRIC CORPORATION", "GENERAL-ELECTRIC" },
	{ 0x08001A, "TIARA/ 10NET", "TIARA/-10NET" },
	{ 0x08001B, "EMC Corporation", "EMC" },
	{ 0x08001C, "KDD-KOKUSAI DEBNSIN DENWA CO.", "KDD-KOKUSAI-DEBNSIN-DENWA" },
	{ 0x08001D, "ABLE COMMUNICATIONS INC.", "ABLE-COMMUNICATION" },
	{ 0x08001E, "APOLLO COMPUTER INC.", "APOLLO" },
	{ 0x08001F, "SHARP CORPORATION", "SHARP" },
	{ 0x080020, "Oracle Corporation", "ORACLE" },
	{ 0x080021, "3M COMPANY", "3M-COMPANY" },
	{ 0x080022, "NBI INC.", "NBI" },
	{ 0x080023, "Panasonic Communications Co., Ltd.", "PANASONIC-COMMUNICATION" },
	{ 0x080024, "10NET COMMUNICATIONS/DCA", "10NET-COMMUNICATION/DCA" },
	{ 0x080025, "CONTROL DATA", "CONTROL-DATA" },
	{ 0x080026, "NORSK DATA A.S.", "NORSK-DATA-A-S" },
	{ 0x080027, "CADMUS COMPUTER SYSTEMS (possible VirtualBox VM)", "CADMUS-VIRTUAL" },
	{ 0x080028, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x080029, "MEGATEK CORPORATION", "MEGATEK" },
	{ 0x08002A, "MOSAIC TECHNOLOGIES INC.", "MOSAIC-TECHNOLOGIES" },
	{ 0x08002B, "DIGITAL EQUIPMENT CORPORATION", "DIGITAL-EQUIPMENT" },
	{ 0x08002C, "BRITTON LEE INC.", "BRITTON-LEE" },
	{ 0x08002D, "LAN-TEC INC.", "LAN-TEC" },
	{ 0x08002E, "METAPHOR COMPUTER SYSTEMS", "METAPHOR" },
	{ 0x08002F, "PRIME COMPUTER INC.", "PRIME" },
	{ 0x080030, "NETWORK RESEARCH CORPORATION", "NETWORK-RESEARCH" },
	{ 0x080030, "CERN", "CERN" },
	{ 0x080030, "ROYAL MELBOURNE INST OF TECH", "ROYAL-MELBOURNE-INST-OF-TECH" },
	{ 0x080031, "LITTLE MACHINES INC.", "LITTLE-MACHINES" },
	{ 0x080032, "TIGAN INCORPORATED", "TIGAN-INCORPORATED" },
	{ 0x080033, "BAUSCH & LOMB", "BAUSCH-LOMB" },
	{ 0x080034, "FILENET CORPORATION", "FILENET" },
	{ 0x080035, "MICROFIVE CORPORATION", "MICROFIVE" },
	{ 0x080036, "INTERGRAPH CORPORATION", "INTERGRAPH" },
	{ 0x080037, "FUJI-XEROX CO. LTD.", "FUJI-XEROX" },
	{ 0x080038, "BULL S.A.S.", "BULL-S" },
	{ 0x080039, "SPIDER SYSTEMS LIMITED", "SPIDER" },
	{ 0x08003A, "ORCATECH INC.", "ORCATECH" },
	{ 0x08003B, "TORUS SYSTEMS LIMITED", "TORUS" },
	{ 0x08003C, "SCHLUMBERGER WELL SERVICES", "SCHLUMBERGER-WELL-SERVICES" },
	{ 0x08003D, "CADNETIX CORPORATIONS", "CADNETIX-CORPORATIONS" },
	{ 0x08003E, "CODEX CORPORATION", "CODEX" },
	{ 0x08003F, "FRED KOSCHARA ENTERPRISES", "FRED-KOSCHARA-ENTERPRISES" },
	{ 0x080040, "FERRANTI COMPUTER SYS. LIMITED", "FERRANTI-SYS" },
	{ 0x080041, "RACAL-MILGO INFORMATION SYS..", "RACAL-MILGO-INFORMATION-SYS" },
	{ 0x080042, "JAPAN MACNICS CORP.", "JAPAN-MACNICS" },
	{ 0x080043, "PIXEL COMPUTER INC.", "PIXEL" },
	{ 0x080044, "DAVID SYSTEMS INC.", "DAVID" },
	{ 0x080045, "CONCURRENT COMPUTER CORP.", "CONCURRENT" },
	{ 0x080046, "Sony Corporation", "SONY" },
	{ 0x080047, "SEQUENT COMPUTER SYSTEMS INC.", "SEQUENT" },
	{ 0x080048, "EUROTHERM GAUGING SYSTEMS", "EUROTHERM-GAUGING" },
	{ 0x080049, "UNIVATION", "UNIVATION" },
	{ 0x08004A, "BANYAN SYSTEMS INC.", "BANYAN" },
	{ 0x08004B, "PLANNING RESEARCH CORP.", "PLANNING-RESEARCH" },
	{ 0x08004C, "HYDRA COMPUTER SYSTEMS INC.", "HYDRA" },
	{ 0x08004D, "CORVUS SYSTEMS INC.", "CORVUS" },
	{ 0x08004E, "3COM EUROPE LTD.", "3COM" },
	{ 0x08004F, "CYGNET SYSTEMS", "CYGNET" },
	{ 0x080050, "DAISY SYSTEMS CORP.", "DAISY" },
	{ 0x080051, "EXPERDATA", "EXPERDATA" },
	{ 0x080052, "INSYSTEC", "INSYSTEC" },
	{ 0x080053, "MIDDLE EAST TECH. UNIVERSITY", "MIDDLE-EAST-TECH-UNIVERSITY" },
	{ 0x080055, "STANFORD TELECOMM. INC.", "STANFORD-TELECOMM" },
	{ 0x080056, "STANFORD LINEAR ACCEL. CENTER", "STANFORD-LINEAR-ACCEL-CENTER" },
	{ 0x080057, "EVANS & SUTHERLAND", "EVANS-SUTHERLAND" },
	{ 0x080058, "SYSTEMS CONCEPTS", "CONCEPTS" },
	{ 0x080059, "A/S MYCRON", "A/S-MYCRON" },
	{ 0x08005A, "IBM Corp", "IBM" },
	{ 0x08005B, "VTA TECHNOLOGIES INC.", "VTA-TECHNOLOGIES" },
	{ 0x08005C, "FOUR PHASE SYSTEMS", "FOUR-PHASE" },
	{ 0x08005D, "GOULD INC.", "GOULD" },
	{ 0x08005E, "COUNTERPOINT COMPUTER INC.", "COUNTERPOINT" },
	{ 0x08005F, "SABER TECHNOLOGY CORP.", "SABER" },
	{ 0x080060, "INDUSTRIAL NETWORKING INC.", "INDUSTRIAL-NETWORKING" },
	{ 0x080061, "JAROGATE LTD.", "JAROGATE" },
	{ 0x080062, "GENERAL DYNAMICS", "GENERAL-DYNAMICS" },
	{ 0x080063, "PLESSEY", "PLESSEY" },
	{ 0x080064, "Sitasys AG", "SITASYS" },
	{ 0x080065, "GENRAD INC.", "GENRAD" },
	{ 0x080066, "AGFA CORPORATION", "AGFA" },
	{ 0x080067, "COMDESIGN", "COMDESIGN" },
	{ 0x080068, "RIDGE COMPUTERS", "RIDGE-COMPUTERS" },
	{ 0x080069, "SILICON GRAPHICS INC.", "SILICON-GRAPHICS" },
	{ 0x08006A, "ATT BELL LABORATORIES", "ATT-BELL-LABORATORIES" },
	{ 0x08006B, "ACCEL TECHNOLOGIES INC.", "ACCEL-TECHNOLOGIES" },
	{ 0x08006C, "SUNTEK TECHNOLOGY INT'L", "SUNTEK" },
	{ 0x08006D, "WHITECHAPEL COMPUTER WORKS", "WHITECHAPEL-WORKS" },
	{ 0x08006E, "MASSCOMP", "MASSCOMP" },
	{ 0x08006F, "PHILIPS APELDOORN B.V.", "PHILIPS-APELDOORN" },
	{ 0x080070, "MITSUBISHI ELECTRIC CORP.", "MITSUBISHI-ELECTRIC" },
	{ 0x080071, "MATRA (DSIE)", "MATRA-DSIE" },
	{ 0x080072, "XEROX CORP UNIV GRANT PROGRAM", "XEROX-UNIV-GRANT-PROGRAM" },
	{ 0x080073, "TECMAR INC.", "TECMAR" },
	{ 0x080074, "CASIO COMPUTER CO. LTD.", "CASIO" },
	{ 0x080075, "DANSK DATA ELECTRONIK", "DANSK-DATA-ELECTRONIK" },
	{ 0x080076, "PC LAN TECHNOLOGIES", "PC-LAN-TECHNOLOGIES" },
	{ 0x080077, "TSL COMMUNICATIONS LTD.", "TSL-COMMUNICATION" },
	{ 0x080078, "ACCELL CORPORATION", "ACCELL" },
	{ 0x080079, "THE DROID WORKS", "THE-DROID-WORKS" },
	{ 0x08007A, "INDATA", "INDATA" },
	{ 0x08007B, "SANYO ELECTRIC CO. LTD.", "SANYO-ELECTRIC" },
	{ 0x08007C, "VITALINK COMMUNICATIONS CORP.", "VITALINK-COMMUNICATION" },
	{ 0x08007E, "AMALGAMATED WIRELESS(AUS) LTD", "AMALGAMATED-WIRELESS-AUS" },
	{ 0x08007F, "CARNEGIE-MELLON UNIVERSITY", "CARNEGIE-MELLON-UNIVERSITY" },
	{ 0x080080, "AES DATA INC.", "AES-DATA" },
	{ 0x080081, "ASTECH INC.", "ASTECH" },
	{ 0x080082, "VERITAS SOFTWARE", "VERITAS-SOFTWARE" },
	{ 0x080083, "Seiko Instruments Inc.", "SEIKO-INSTRUMENTS" },
	{ 0x080084, "TOMEN ELECTRONICS CORP.", "TOMEN-ELECTRONICS" },
	{ 0x080085, "ELXSI", "ELXSI" },
	{ 0x080086, "KONICA MINOLTA HOLDINGS, INC.", "KONICA-MINOLTA-HOLDINGS" },
	{ 0x080087, "XYPLEX", "XYPLEX" },
	{ 0x080088, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x080089, "KINETICS", "KINETICS" },
	{ 0x08008A, "PerfTech, Inc.", "PERFTECH" },
	{ 0x08008B, "PYRAMID TECHNOLOGY CORP.", "PYRAMID" },
	{ 0x08008C, "NETWORK RESEARCH CORPORATION", "NETWORK-RESEARCH" },
	{ 0x08008D, "XYVISION INC.", "XYVISION" },
	{ 0x08008E, "TANDEM COMPUTERS", "TANDEM-COMPUTERS" },
	{ 0x08008F, "CHIPCOM CORPORATION", "CHIPCOM" },
	{ 0x080090, "SONOMA SYSTEMS", "SONOMA" },
	{ 0x080371, "KRG CORPORATE", "KRG-CORPORATE" },
	{ 0x0805CD, "DongGuang EnMai Electronic Product Co.Ltd.", "DONGGUANG-ENMAI-PRODUCT" },
	{ 0x0808C2, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x0808EA, "AMSC", "AMSC" },
	{ 0x0809B6, "Masimo Corp", "MASIMO" },
	{ 0x080C0B, "SysMik GmbH Dresden", "SYSMIK-DRESDEN" },
	{ 0x080CC9, "Mission Technology Group, dba Magma", "MISSION-GROUP-DBA-MAGMA" },
	{ 0x080D84, "GECO, Inc.", "GECO" },
	{ 0x080EA8, "Velex s.r.l.", "VELEX-S-R-L" },
	{ 0x080FFA, "KSP INC.", "KSP" },
	{ 0x08115E, "Bitel Co., Ltd.", "BITEL" },
	{ 0x081196, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x081443, "UNIBRAIN S.A.", "UNIBRAIN" },
	{ 0x081651, "Shenzhen Sea Star Technology Co.,Ltd", "SHENZHEN-SEA-STAR" },
	{ 0x081735, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0817F4, "IBM Corp", "IBM" },
	{ 0x08181A, "zte corporation", "ZTE" },
	{ 0x08184C, "A. S. Thomas, Inc.", "A-S-THOMAS" },
	{ 0x0819A6, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x081DFB, "Shanghai Mexon Communication Technology Co.,Ltd", "SHANGHAI-MEXON-COMMUNICATION" },
	{ 0x081F3F, "WondaLink Inc.", "WONDALINK" },
	{ 0x081FEB, "BinCube", "BINCUBE" },
	{ 0x081FF3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x082522, "ADVANSEE", "ADVANSEE" },
	{ 0x082719, "APS systems/electronic AG", "APS-/" },
	{ 0x082AD0, "SRD Innovations Inc.", "SRD-INNOVATIONS" },
	{ 0x082E5F, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x083571, "CASwell INC.", "CASWELL" },
	{ 0x08373D, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x08379C, "Topaz Co. LTD.", "TOPAZ" },
	{ 0x0838A5, "Funkwerk plettac electronic GmbH", "FUNKWERK-PLETTAC" },
	{ 0x083AB8, "Shinoda Plasma Co., Ltd.", "SHINODA-PLASMA" },
	{ 0x083D88, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x083E0C, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x083E8E, "Hon Hai Precision Ind.Co.Ltd", "HON-HAI-PRECISION-IND" },
	{ 0x083F3E, "WSH GmbH", "WSH" },
	{ 0x083F76, "Intellian Technologies, Inc.", "INTELLIAN-TECHNOLOGIES" },
	{ 0x084027, "Gridstore Inc.", "GRIDSTORE" },
	{ 0x084656, "VODALYS Ingénierie", "VODALYS-INGéNIERIE" },
	{ 0x08482C, "Raycore Taiwan Co., LTD.", "RAYCORE-TAIWAN" },
	{ 0x084E1C, "H2A Systems, LLC", "H2A-LLC" },
	{ 0x084EBF, "Broad Net Mux Corporation", "BROAD-NET-MUX" },
	{ 0x08512E, "Orion Diagnostica Oy", "ORION-DIAGNOSTICA-OY" },
	{ 0x085240, "EbV Elektronikbau- und Vertriebs GmbH", "EBV-ELEKTRONIKBAU--UND-VERTRIEBS" },
	{ 0x085700, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x085AE0, "Recovision Technology Co., Ltd.", "RECOVISION" },
	{ 0x085B0E, "Fortinet, Inc.", "FORTINET" },
	{ 0x085DDD, "Mercury Corporation", "MERCURY" },
	{ 0x08606E, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x086266, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x086361, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x0868D0, "Japan System Design", "JAPAN-SYSTEM-DESIGN" },
	{ 0x0868EA, "EITO ELECTRONICS CO., LTD.", "EITO-ELECTRONICS" },
	{ 0x086DF2, "Shenzhen MIMOWAVE Technology Co.,Ltd", "SHENZHEN-MIMOWAVE" },
	{ 0x087045, "Apple", "APPLE" },
	{ 0x0874F6, "Winterhalter Gastronom GmbH", "WINTERHALTER-GASTRONOM" },
	{ 0x087572, "Obelux Oy", "OBELUX-OY" },
	{ 0x087618, "ViE Technologies Sdn. Bhd.", "VIE-TECHNOLOGIES-SDN-BHD" },
	{ 0x087695, "Auto Industrial Co., Ltd.", "AUTO-INDUSTRIAL" },
	{ 0x0876FF, "Thomson Telecom Belgium", "THOMSON-TELECOM-BELGIUM" },
	{ 0x087999, "AIM GmbH", "AIM" },
	{ 0x087A4C, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x087BAA, "SVYAZKOMPLEKTSERVICE, LLC", "SVYAZKOMPLEKTSERVICE-LLC" },
	{ 0x087CBE, "Quintic Corp.", "QUINTIC" },
	{ 0x087D21, "Altasec technology corporation", "ALTASEC" },
	{ 0x088039, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x0881BC, "HongKong Ipro Technology Co., Limited", "HONGKONG-IPRO" },
	{ 0x0881F4, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x08863B, "Belkin International, Inc.", "BELKIN" },
	{ 0x088DC8, "Ryowa Electronics Co.,Ltd", "RYOWA-ELECTRONICS" },
	{ 0x088E4F, "SF Software Solutions", "SF-SOFTWARE-SOLUTIONS" },
	{ 0x088F2C, "Hills Sound Vision & Lighting", "HILLS-SOUND-VISION-LIGHTING" },
	{ 0x0896D7, "AVM GmbH", "AVM" },
	{ 0x089758, "Shenzhen Strong Rising Electronics Co.,Ltd DongGuan Subsidiary", "SHENZHEN-STRONG-RISING-ELECTRONICS-DONGGUAN-SUBSIDIARY" },
	{ 0x089E01, "QUANTA COMPUTER INC.", "QUANTA" },
	{ 0x089F97, "LEROY AUTOMATION", "LEROY-AUTOMATION" },
	{ 0x08A12B, "ShenZhen EZL Technology Co., Ltd", "SHENZHEN-EZL" },
	{ 0x08A5C8, "Sunnovo International Limited", "SUNNOVO" },
	{ 0x08A95A, "Azurewave", "AZUREWAVE" },
	{ 0x08ACA5, "Benu Video, Inc.", "BENU-VIDEO" },
	{ 0x08AF78, "Totus Solutions, Inc.", "TOTUS-SOLUTIONS" },
	{ 0x08B2A3, "Cynny Italia S.r.L.", "CYNNY-ITALIA-S-R-L" },
	{ 0x08B4CF, "Abicom International", "ABICOM" },
	{ 0x08B738, "Lite-On Technogy Corp.", "LITE-ON-TECHNOGY" },
	{ 0x08B7EC, "Wireless Seismic", "WIRELESS-SEISMIC" },
	{ 0x08BBCC, "AK-NORD EDV VERTRIEBSGES. mbH", "AK-NORD-EDV-VERTRIEBSGES-MBH" },
	{ 0x08BD43, "NETGEAR INC.,", "NETGEAR" },
	{ 0x08BE09, "Astrol Electronic AG", "ASTROL" },
	{ 0x08CA45, "Toyou Feiji Electronics Co., Ltd.", "TOYOU-FEIJI-ELECTRONICS" },
	{ 0x08CC68, "Cisco", "CISCO" },
	{ 0x08CD9B, "samtec automotive electronics & software GmbH", "SAMTEC-AUTOMOTIVE-ELECTRONICS-SOFTWARE" },
	{ 0x08D09F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x08D0B7, "HISENSE ELECTRIC CO.,LTD.", "HISENSE-ELECTRIC" },
	{ 0x08D29A, "Proformatique", "PROFORMATIQUE" },
	{ 0x08D34B, "Techman Electronics (Changshu) Co., Ltd.", "TECHMAN-ELECTRONICS-CHANGSHU" },
	{ 0x08D40C, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x08D42B, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x08D5C0, "Seers Technology Co., Ltd", "SEERS" },
	{ 0x08D833, "Shenzhen RF Technology Co,.Ltd", "SHENZHEN-RF" },
	{ 0x08DF1F, "Bose Corporation", "BOSE" },
	{ 0x08E5DA, "NANJING FUJITSU COMPUTER PRODUCTS CO.,LTD.", "NANJING-FUJITSU-PRODUCTS" },
	{ 0x08E672, "JEBSEE ELECTRONICS CO.,LTD.", "JEBSEE-ELECTRONICS" },
	{ 0x08E84F, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x08EA44, "Aerohive Networks, Inc.", "AEROHIVE-NETWORKS" },
	{ 0x08EB29, "Jiangsu Huitong Group Co.,Ltd.", "JIANGSU-HUITONG-GROUP" },
	{ 0x08EB74, "Humax", "HUMAX" },
	{ 0x08EBED, "World Elite Technology Co.,LTD", "WORLD-ELITE" },
	{ 0x08EDB9, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x08EE8B, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0x08EF3B, "MCS Logic Inc.", "MCS-LOGIC" },
	{ 0x08EFAB, "SAYME WIRELESS SENSOR NETWORK", "SAYME-WIRELESS-SENSOR-NETWORK" },
	{ 0x08F1B7, "Towerstream Corpration", "TOWERSTREAM-CORPRATION" },
	{ 0x08F2F4, "Net One Partners Co.,Ltd.", "NET-ONE-PARTNERS" },
	{ 0x08F6F8, "GET Engineering", "GET-ENGINEERING" },
	{ 0x08F728, "GLOBO Multimedia Sp. z o.o. Sp.k.", "GLOBO-MULTIMEDIA-SP-Z-O-O-SP-K" },
	{ 0x08FAE0, "Fohhn Audio AG", "FOHHN-AUDIO" },
	{ 0x08FC52, "OpenXS BV", "OPENXS" },
	{ 0x08FC88, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x08FD0E, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x0C0400, "Jantar d.o.o.", "JANTAR-D-O-O" },
	{ 0x0C0535, "Juniper Systems", "JUNIPER" },
	{ 0x0C1105, "Ringslink (Xiamen) Network Communication Technologies Co., Ltd", "RINGSLINK-XIAMEN-NETWORK-COMMUNICATION-TECHNOLOGIES" },
	{ 0x0C1262, "zte corporation", "ZTE" },
	{ 0x0C130B, "Uniqoteq Ltd.", "UNIQOTEQ" },
	{ 0x0C1420, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x0C1539, "Apple", "APPLE" },
	{ 0x0C15C5, "SDTEC Co., Ltd.", "SDTEC" },
	{ 0x0C17F1, "TELECSYS", "TELECSYS" },
	{ 0x0C191F, "Inform Electronik", "INFORM-ELECTRONIK" },
	{ 0x0C1DAF, "Beijing Xiaomi communications co.,ltd", "BEIJING-XIAOMI-COMMUNICATION" },
	{ 0x0C1DC2, "SeAH Networks", "SEAH-NETWORKS" },
	{ 0x0C2026, "noax Technologies AG", "NOAX-TECHNOLOGIES" },
	{ 0x0C2724, "Cisco", "CISCO" },
	{ 0x0C2755, "Valuable Techologies Limited", "VALUABLE-TECHOLOGIES" },
	{ 0x0C2A69, "electric imp, incorporated", "ELECTRIC-IMP-INCORPORATED" },
	{ 0x0C2AE7, "Beijing General Research Institute of Mining and Metallurgy", "BEIJING-GENERAL-RESEARCH-INSTITUTE-OF-MINING-AND-METALLURGY" },
	{ 0x0C2D89, "QiiQ Communications Inc.", "QIIQ-COMMUNICATION" },
	{ 0x0C3021, "Apple", "APPLE" },
	{ 0x0C37DC, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x0C383E, "Fanvil Technology Co., Ltd.", "FANVIL" },
	{ 0x0C3956, "Observator instruments", "OBSERVATOR-INSTRUMENTS" },
	{ 0x0C3C65, "Dome Imaging Inc", "DOME-IMAGING" },
	{ 0x0C3E9F, "Apple, Inc", "APPLE" },
	{ 0x0C413E, "Microsoft Corporation", "MICROSOFT" },
	{ 0x0C469D, "MS Sedco", "MS-SEDCO" },
	{ 0x0C473D, "Hitron Technologies. Inc", "HITRON-TECHNOLOGIES" },
	{ 0x0C4885, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x0C4C39, "Mitrastar Technology", "MITRASTAR" },
	{ 0x0C4DE9, "Apple", "APPLE" },
	{ 0x0C4F5A, "ASA-RT s.r.l.", "ASA-RT-S-R-L" },
	{ 0x0C51F7, "CHAUVIN ARNOUX", "CHAUVIN-ARNOUX" },
	{ 0x0C54A5, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x0C5521, "Axiros GmbH", "AXIROS" },
	{ 0x0C565C, "HyBroad Vision (Hong Kong) Technology Co Ltd", "HYBROAD-VISION-HONG-KONG" },
	{ 0x0C57EB, "Mueller Systems", "MUELLER" },
	{ 0x0C5A19, "Axtion Sdn Bhd", "AXTION-SDN-BHD" },
	{ 0x0C5CD8, "DOLI Elektronik GmbH", "DOLI-ELEKTRONIK" },
	{ 0x0C6076, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x0C63FC, "Nanjing Signway Technology Co., Ltd", "NANJING-SIGNWAY" },
	{ 0x0C6803, "Cisco", "CISCO" },
	{ 0x0C6E4F, "PrimeVOLT Co., Ltd.", "PRIMEVOLT" },
	{ 0x0C715D, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x0C722C, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x0C74C2, "Apple", "APPLE" },
	{ 0x0C7523, "BEIJING GEHUA CATV NETWORK CO.,LTD", "BEIJING-GEHUA-CATV-NETWORK" },
	{ 0x0C771A, "Apple", "APPLE" },
	{ 0x0C7D7C, "Kexiang Information Technology Co, Ltd.", "KEXIANG-INFORMATION" },
	{ 0x0C8112, "PRIVATE", "PRIVATE" },
	{ 0x0C8230, "SHENZHEN MAGNUS TECHNOLOGIES CO.,LTD", "SHENZHEN-MAGNUS-TECHNOLOGIES" },
	{ 0x0C8268, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x0C826A, "Wuhan Huagong Genuine Optics Technology Co., Ltd", "WUHAN-HUAGONG-GENUINE-OPTICS" },
	{ 0x0C8411, "A.O. Smith Water Products", "A-O-SMITH-WATER-PRODUCTS" },
	{ 0x0C8484, "Zenovia Electronics Inc.", "ZENOVIA-ELECTRONICS" },
	{ 0x0C84DC, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x0C8525, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0C8910, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x0C8BFD, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0C8C8F, "Kamo Technology Limited", "KAMO" },
	{ 0x0C8CDC, "Suunto Oy", "SUUNTO-OY" },
	{ 0x0C8D98, "TOP EIGHT IND CORP", "TOP-EIGHT-IND" },
	{ 0x0C924E, "Rice Lake Weighing Systems", "RICE-LAKE-WEIGHING" },
	{ 0x0C9301, "PT. Prasimax Inovasi Teknologi", "PT-PRASIMAX-INOVASI-TEKNOLOGI" },
	{ 0x0C93FB, "BNS Solutions", "BNS-SOLUTIONS" },
	{ 0x0C96BF, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x0C9B13, "Shanghai Magic Mobile Telecommunication Co.Ltd.", "SHANGHAI-MAGIC-MOBILE-TELECOMMUNICATION" },
	{ 0x0C9D56, "Consort Controls Ltd", "CONSORT-CONTROLS" },
	{ 0x0C9E91, "Sankosha Corporation", "SANKOSHA" },
	{ 0x0CA138, "Blinq Wireless Inc.", "BLINQ-WIRELESS" },
	{ 0x0CA2F4, "Chameleon Technology (UK) Limited", "CHAMELEON-UK" },
	{ 0x0CA402, "Alcatel Lucent IPD", "ALCATEL-LUCENT-IPD" },
	{ 0x0CA42A, "OB Telecom Electronic Technology Co., Ltd", "OB-TELECOM" },
	{ 0x0CA694, "Sunitec Enterprise Co.,Ltd", "SUNITEC" },
	{ 0x0CAC05, "Unitend Technologies Inc.", "UNITEND-TECHNOLOGIES" },
	{ 0x0CAF5A, "GENUS POWER INFRASTRUCTURES LIMITED", "GENUS-POWER-INFRASTRUCTURES" },
	{ 0x0CB319, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0x0CB4EF, "Digience Co.,Ltd.", "DIGIENCE" },
	{ 0x0CB5DE, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x0CBC9F, "Apple", "APPLE" },
	{ 0x0CBD51, "TCT Mobile Limited", "TCT-MOBILE" },
	{ 0x0CBF15, "Genetec", "GENETEC" },
	{ 0x0CC0C0, "MAGNETI MARELLI SISTEMAS ELECTRONICOS MEXICO", "MAGNETI-MARELLI-SISTEMAS-ELECTRONICOS-MEXICO" },
	{ 0x0CC3A7, "Meritec", "MERITEC" },
	{ 0x0CC47A, "Super Micro Computer, Inc.", "SUPER-MICRO" },
	{ 0x0CC47E, "EUCAST Co., Ltd.", "EUCAST" },
	{ 0x0CC655, "Wuxi YSTen Technology Co.,Ltd.", "WUXI-YSTEN" },
	{ 0x0CC66A, "Nokia Corporation", "NOKIA" },
	{ 0x0CC6AC, "DAGS", "DAGS" },
	{ 0x0CC81F, "Summer Infant, Inc.", "SUMMER-INFANT" },
	{ 0x0CC9C6, "Samwin Hong Kong Limited", "SAMWIN-HONG-KONG" },
	{ 0x0CCB8D, "ASCO Numatics GmbH", "ASCO-NUMATICS" },
	{ 0x0CCDD3, "EASTRIVER TECHNOLOGY CO., LTD.", "EASTRIVER" },
	{ 0x0CCDFB, "EDIC Systems Inc.", "EDIC" },
	{ 0x0CCFD1, "SPRINGWAVE Co., Ltd", "SPRINGWAVE" },
	{ 0x0CD292, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x0CD2B5, "Binatone Telecommunication Pvt. Ltd", "BINATONE-TELECOMMUNICATION-PVT" },
	{ 0x0CD502, "Westell", "WESTELL" },
	{ 0x0CD696, "Amimon Ltd", "AMIMON" },
	{ 0x0CD7C2, "Axium Technologies, Inc.", "AXIUM-TECHNOLOGIES" },
	{ 0x0CD996, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x0CD9C1, "Visteon Corporation", "VISTEON" },
	{ 0x0CDA41, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x0CDCCC, "Inala Technologies", "INALA-TECHNOLOGIES" },
	{ 0x0CDDEF, "Nokia Corporation", "NOKIA" },
	{ 0x0CDFA4, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x0CE0E4, "Plantronics, Inc", "PLANTRONICS" },
	{ 0x0CE5D3, "DH electronics GmbH", "DH-ELECTRONICS" },
	{ 0x0CE709, "Fox Crypto B.V.", "FOX-CRYPTO" },
	{ 0x0CE82F, "Bonfiglioli Vectron GmbH", "BONFIGLIOLI-VECTRON" },
	{ 0x0CE936, "ELIMOS srl", "ELIMOS-SRL" },
	{ 0x0CEEE6, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x0CEF7C, "AnaCom Inc", "ANACOM" },
	{ 0x0CEFAF, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x0CF019, "Malgn Technology Co., Ltd.", "MALGN" },
	{ 0x0CF0B4, "Globalsat International Technology Ltd", "GLOBALSAT" },
	{ 0x0CF361, "Java Information", "JAVA-INFORMATION" },
	{ 0x0CF3EE, "EM Microelectronic", "EM-MICROELECTRONIC" },
	{ 0x0CF405, "Beijing Signalway Technologies Co.,Ltd", "BEIJING-SIGNALWAY-TECHNOLOGIES" },
	{ 0x0CF5A4, "Cisco", "CISCO" },
	{ 0x0CF893, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x0CFC83, "Airoha Technology Corp.,", "AIROHA" },
	{ 0x100000, "PRIVATE", "PRIVATE" },
	{ 0x10005A, "IBM Corp", "IBM" },
	{ 0x1000E8, "NATIONAL SEMICONDUCTOR", "NATIONAL-SEMICONDUCTOR" },
	{ 0x1000FD, "LaonPeople", "LAONPEOPLE" },
	{ 0x1001CA, "Ashley Butterworth", "ASHLEY-BUTTERWORTH" },
	{ 0x1005CA, "Cisco", "CISCO" },
	{ 0x100723, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x1008B1, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x10090C, "Janome Sewing Machine Co., Ltd.", "JANOME-SEWING-MACHINE" },
	{ 0x100BA9, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x100C24, "pomdevices, LLC", "POMDEVICES-LLC" },
	{ 0x100D2F, "Online Security Pty. Ltd.", "ONLINE-SECURITY-PTY" },
	{ 0x100D32, "Embedian, Inc.", "EMBEDIAN" },
	{ 0x100D7F, "NETGEAR INC.,", "NETGEAR" },
	{ 0x100E2B, "NEC CASIO Mobile Communications", "NEC-CASIO-MOBILE-COMMUNICATION" },
	{ 0x100E7E, "Juniper networks", "JUNIPER-NETWORKS" },
	{ 0x100F18, "Fu Gang Electronic(KunShan)CO.,LTD", "FU-GANG-KUNSHAN" },
	{ 0x1010B6, "McCain Inc", "MCCAIN" },
	{ 0x101212, "Vivo International Corporation Pty Ltd", "VIVO-PTY" },
	{ 0x101218, "Korins Inc.", "KORINS" },
	{ 0x101248, "ITG, Inc.", "ITG" },
	{ 0x1013EE, "Justec International Technology INC.", "JUSTEC" },
	{ 0x10189E, "Elmo Motion Control", "ELMO-MOTION-CONTROL" },
	{ 0x101B54, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x101C0C, "Apple", "APPLE" },
	{ 0x101D51, "ON-Q LLC dba ON-Q Mesh Networks", "ON-Q-LLC-DBA-ON-Q-MESH-NETWORKS" },
	{ 0x101DC0, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x101F74, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x102279, "ZeroDesktop, Inc.", "ZERODESKTOP" },
	{ 0x1027BE, "TVIP", "TVIP" },
	{ 0x102831, "Morion Inc.", "MORION" },
	{ 0x102C83, "XIMEA", "XIMEA" },
	{ 0x102D96, "Looxcie Inc.", "LOOXCIE" },
	{ 0x102EAF, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x102F6B, "Microsoft Corporation", "MICROSOFT" },
	{ 0x103047, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x103378, "FLECTRON Co., LTD", "FLECTRON" },
	{ 0x103711, "Simlink AS", "SIMLINK-AS" },
	{ 0x103B59, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x103DEA, "HFC Technology (Beijing) Ltd. Co.", "HFC-BEIJING" },
	{ 0x1040F3, "Apple", "APPLE" },
	{ 0x104369, "Soundmax Electronic Limited", "SOUNDMAX" },
	{ 0x10445A, "Shaanxi Hitech Electronic Co., LTD", "SHAANXI-HITECH" },
	{ 0x1045BE, "Norphonic AS", "NORPHONIC-AS" },
	{ 0x1045F8, "LNT-Automation GmbH", "LNT-AUTOMATION" },
	{ 0x104780, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x1048B1, "Beijing Duokan Technology Limited", "BEIJING-DUOKAN" },
	{ 0x104A7D, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x104B46, "Mitsubishi Electric Corporation", "MITSUBISHI-ELECTRIC" },
	{ 0x104D77, "Innovative Computer Engineering", "INNOVATIVE-ENGINEERING" },
	{ 0x104E07, "Shanghai Genvision Industries Co.,Ltd", "SHANGHAI-GENVISION-INDUSTRIES" },
	{ 0x105172, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x1056CA, "Peplink International Ltd.", "PEPLINK" },
	{ 0x105C3B, "Perma-Pipe, Inc.", "PERMA-PIPE" },
	{ 0x105CBF, "DuroByte Inc", "DUROBYTE" },
	{ 0x105F06, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x105F49, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x10604B, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x1062C9, "Adatis GmbH & Co. KG", "ADATIS" },
	{ 0x1064E2, "ADFweb.com s.r.l.", "ADFWEB-COM-S-R-L" },
	{ 0x1065A3, "Core Brands LLC", "CORE-BRANDS-LLC" },
	{ 0x1065CF, "IQSIM", "IQSIM" },
	{ 0x106682, "NEC Platforms, Ltd.", "NEC-PLATFORMS" },
	{ 0x10683F, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x106F3F, "Buffalo Inc.", "BUFFALO" },
	{ 0x106FEF, "Ad-Sol Nissin Corp", "AD-SOL-NISSIN" },
	{ 0x1071F9, "Cloud Telecomputers, LLC", "CLOUD-TELECOMPUTERS-LLC" },
	{ 0x10768A, "EoCell", "EOCELL" },
	{ 0x1077B1, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x107873, "Shenzhen Jinkeyi Communication Co., Ltd.", "SHENZHEN-JINKEYI-COMMUNICATION" },
	{ 0x1078CE, "Hanvit SI, Inc.", "HANVIT-SI" },
	{ 0x1078D2, "ELITEGROUP COMPUTER SYSTEM CO., LTD.", "ELITEGROUP-SYSTEM" },
	{ 0x107A86, "U&U ENGINEERING INC.", "U-U-ENGINEERING" },
	{ 0x107BEF, "ZyXEL Communications Corp", "ZYXEL-COMMUNICATION" },
	{ 0x1083D2, "Microseven Systems, LLC", "MICROSEVEN-LLC" },
	{ 0x10880F, "Daruma Telecomunicações e Informática S.A.", "DARUMA-TELECOMUNICAçõES-E-INFORMáTICA" },
	{ 0x1088CE, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0x108A1B, "RAONIX Inc.", "RAONIX" },
	{ 0x108CCF, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x109266, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x1093E9, "Apple", "APPLE" },
	{ 0x109836, "Dell Inc.", "DELL" },
	{ 0x109AB9, "Tosibox Oy", "TOSIBOX-OY" },
	{ 0x109ADD, "Apple", "APPLE" },
	{ 0x109FA9, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x10A13B, "FUJIKURA RUBBER LTD.", "FUJIKURA-RUBBER" },
	{ 0x10A5D0, "Murata Manufacturing Co.,Ltd.", "MURATA-MANUFACTURING" },
	{ 0x10A659, "Mobile Create Co.,Ltd.", "MOBILE-CREATE" },
	{ 0x10A743, "SK Mtek Limited", "SK-MTEK" },
	{ 0x10A932, "Beijing Cyber Cloud Technology Co. ,Ltd.", "BEIJING-CYBER-CLOUD" },
	{ 0x10AE60, "PRIVATE", "PRIVATE" },
	{ 0x10AF78, "Shenzhen ATUE Technology Co., Ltd", "SHENZHEN-ATUE" },
	{ 0x10B26B, "base Co.,Ltd.", "BASE" },
	{ 0x10B713, "PRIVATE", "PRIVATE" },
	{ 0x10B7F6, "Plastoform Industries Ltd.", "PLASTOFORM-INDUSTRIES" },
	{ 0x10B9FE, "Lika srl", "LIKA-SRL" },
	{ 0x10BAA5, "GANA I&C CO., LTD", "GANA-I-C" },
	{ 0x10BD18, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x10BF48, "ASUSTEK COMPUTER INC.", "ASUSTEK" },
	{ 0x10C07C, "Blu-ray Disc Association", "BLU-RAY-DISC-ASSOCIATION" },
	{ 0x10C2BA, "UTT Co., Ltd.", "UTT" },
	{ 0x10C37B, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x10C586, "BIO SOUND LAB CO., LTD.", "BIO-SOUND-LAB" },
	{ 0x10C61F, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x10C67E, "SHENZHEN JUCHIN TECHNOLOGY CO., LTD", "SHENZHEN-JUCHIN" },
	{ 0x10C6FC, "Garmin International", "GARMIN" },
	{ 0x10C73F, "Midas Klark Teknik Ltd", "MIDAS-KLARK-TEKNIK" },
	{ 0x10CA81, "PRECIA", "PRECIA" },
	{ 0x10CCDB, "AXIMUM PRODUITS ELECTRONIQUES", "AXIMUM-PRODUITS-ELECTRONIQUES" },
	{ 0x10CDAE, "Avaya, Inc", "AVAYA" },
	{ 0x10D1DC, "INSTAR Deutschland GmbH", "INSTAR-DEUTSCHLAND" },
	{ 0x10D38A, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x10D542, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x10DDB1, "Apple", "APPLE" },
	{ 0x10DDF4, "Maxway Electronics CO.,LTD", "MAXWAY-ELECTRONICS" },
	{ 0x10DEE4, "automationNEXT GmbH", "AUTOMATIONNEXT" },
	{ 0x10E2D5, "Qi Hardware Inc.", "QI-HARDWARE" },
	{ 0x10E3C7, "Seohwa Telecom", "SEOHWA-TELECOM" },
	{ 0x10E4AF, "APR, LLC", "APR-LLC" },
	{ 0x10E6AE, "Source Technologies, LLC", "SOURCE-TECHNOLOGIES-LLC" },
	{ 0x10E878, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x10E8EE, "PhaseSpace", "PHASESPACE" },
	{ 0x10EA59, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x10EED9, "Canoga Perkins Corporation", "CANOGA-PERKINS" },
	{ 0x10F311, "Cisco", "CISCO" },
	{ 0x10F3DB, "Gridco Systems, Inc.", "GRIDCO" },
	{ 0x10F49A, "T3 Innovation", "T3-INNOVATION" },
	{ 0x10F681, "vivo Mobile Communication Co., Ltd.", "VIVO-MOBILE-COMMUNICATION" },
	{ 0x10F96F, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x10F9EE, "Nokia Corporation", "NOKIA" },
	{ 0x10FACE, "Reacheng Communication Technology Co.,Ltd", "REACHENG-COMMUNICATION" },
	{ 0x10FBF0, "KangSheng LTD.", "KANGSHENG" },
	{ 0x10FC54, "Shany Electronic Co., Ltd.", "SHANY" },
	{ 0x10FEED, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x1100AA, "PRIVATE", "PRIVATE" },
	{ 0x140708, "PRIVATE", "PRIVATE" },
	{ 0x1407E0, "Abrantix AG", "ABRANTIX" },
	{ 0x140C76, "FREEBOX SAS", "FREEBOX-SAS" },
	{ 0x140D4F, "Flextronics International", "FLEXTRONICS" },
	{ 0x14109F, "Apple", "APPLE" },
	{ 0x141330, "Anakreon UK LLP", "ANAKREON-UK-LLP" },
	{ 0x14144B, "FUJIAN STAR-NET COMMUNICATION CO.,LTD", "FUJIAN-STAR-NET-COMMUNICATION" },
	{ 0x141A51, "Treetech Sistemas Digitais", "TREETECH-SISTEMAS-DIGITAIS" },
	{ 0x141AA3, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0x141BBD, "Volex Inc.", "VOLEX" },
	{ 0x141BF0, "Intellimedia Systems Ltd", "INTELLIMEDIA" },
	{ 0x141FBA, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x1422DB, "eero inc.", "EERO" },
	{ 0x1423D7, "EUTRONIX CO., LTD.", "EUTRONIX" },
	{ 0x142971, "NEMOA ELECTRONICS (HK) CO. LTD", "NEMOA-ELECTRONICS-HK" },
	{ 0x142BD2, "Armtel Ltd.", "ARMTEL" },
	{ 0x142BD6, "Guangdong Appscomm Co.,Ltd", "GUANGDONG-APPSCOMM" },
	{ 0x142D27, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x142D8B, "Incipio Technologies, Inc", "INCIPIO-TECHNOLOGIES" },
	{ 0x142DF5, "Amphitech", "AMPHITECH" },
	{ 0x14307A, "Avermetrics", "AVERMETRICS" },
	{ 0x1430C6, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0x14358B, "Mediabridge Products, LLC.", "MEDIABRIDGE-PRODUCTS-LLC" },
	{ 0x1435B3, "Future Designs, Inc.", "FUTURE-DESIGNS" },
	{ 0x143605, "Nokia Corporation", "NOKIA" },
	{ 0x1436C6, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x14373B, "PROCOM Systems", "PROCOM" },
	{ 0x143AEA, "Dynapower Company LLC", "DYNAPOWER-COMPANY-LLC" },
	{ 0x143DF2, "Beijing Shidai Hongyuan Network Communication Co.,Ltd", "BEIJING-SHIDAI-HONGYUAN-NETWORK-COMMUNICATION" },
	{ 0x143E60, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x144146, "Honeywell (China) Co., LTD", "HONEYWELL-CHINA" },
	{ 0x1441E2, "Monaco Enterprises, Inc.", "MONACO-ENTERPRISES" },
	{ 0x144319, "Creative&Link Technology Limited", "CREATIVE-LINK" },
	{ 0x1446E4, "AVISTEL", "AVISTEL" },
	{ 0x14488B, "Shenzhen Doov Technology Co.,Ltd", "SHENZHEN-DOOV" },
	{ 0x144978, "Digital Control Incorporated", "DIGITAL-CONTROL-INCORPORATED" },
	{ 0x1449E0, "Samsung Electro Mechanics co.,LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x144C1A, "Max Communication GmbH", "MAX-COMMUNICATION" },
	{ 0x145412, "Entis Co., Ltd.", "ENTIS" },
	{ 0x145645, "Savitech Corp.", "SAVITECH" },
	{ 0x1458D0, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x145A05, "Apple", "APPLE" },
	{ 0x145BD1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x146080, "zte corporation", "ZTE" },
	{ 0x146308, "JABIL CIRCUIT (SHANGHAI) LTD.", "JABIL-CIRCUIT-SHANGHAI" },
	{ 0x146A0B, "Cypress Electronics Limited", "CYPRESS-ELECTRONICS" },
	{ 0x146B72, "Shenzhen Fortune Ship Technology Co., Ltd.", "SHENZHEN-FORTUNE-SHIP" },
	{ 0x146E0A, "PRIVATE", "PRIVATE" },
	{ 0x147373, "TUBITAK UEKAE", "TUBITAK-UEKAE" },
	{ 0x147411, "RIM", "RIM" },
	{ 0x147590, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x147DB3, "JOA TELECOM.CO.,LTD", "JOA-TELECOM" },
	{ 0x147DC5, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x14825B, "Hefei Radio Communication Technology Co., Ltd", "HEFEI-RADIO-COMMUNICATION" },
	{ 0x148692, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x14893E, "VIXTEL TECHNOLOGIES LIMTED", "VIXTEL-TECHNOLOGIES-LIMTED" },
	{ 0x1489FD, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x148A70, "ADS GmbH", "ADS" },
	{ 0x148F21, "Garmin International", "GARMIN" },
	{ 0x148FC6, "Apple", "APPLE" },
	{ 0x149090, "KongTop industrial(shen zhen)CO.,LTD", "KONGTOP-INDUSTRIAL-SHEN-ZHEN" },
	{ 0x149448, "BLU CASTLE S.A.", "BLU-CASTLE" },
	{ 0x1499E2, "Apple, Inc", "APPLE" },
	{ 0x149A10, "Microsoft Corporation", "MICROSOFT" },
	{ 0x149FE8, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x14A364, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x14A62C, "S.M. Dezac S.A.", "S-M-DEZAC" },
	{ 0x14A86B, "ShenZhen Telacom Science&Technology Co., Ltd", "SHENZHEN-TELACOM-SCIENCE" },
	{ 0x14A9E3, "MST CORPORATION", "MST" },
	{ 0x14ABF0, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x14AEDB, "VTech Telecommunications Ltd.", "VTECH-TELECOMMUNICATIONS" },
	{ 0x14B126, "Industrial Software Co", "INDUSTRIAL-SOFTWARE" },
	{ 0x14B1C8, "InfiniWing, Inc.", "INFINIWING" },
	{ 0x14B484, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x14B73D, "ARCHEAN Technologies", "ARCHEAN-TECHNOLOGIES" },
	{ 0x14B968, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x14C089, "DUNE HD LTD", "DUNE-HD" },
	{ 0x14C126, "Nokia Corporation", "NOKIA" },
	{ 0x14C21D, "Sabtech Industries", "SABTECH-INDUSTRIES" },
	{ 0x14CC20, "TP-LINK TECHNOLOGIES CO.,LTD", "TP-LINK-TECHNOLOGIES" },
	{ 0x14CF8D, "OHSUNG ELECTRONICS CO., LTD.", "OHSUNG-ELECTRONICS" },
	{ 0x14CF92, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x14CFE2, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x14D4FE, "Pace plc", "PACE-PLC" },
	{ 0x14D64D, "D-Link International", "D-LINK" },
	{ 0x14D76E, "CONCH ELECTRONIC Co.,Ltd", "CONCH" },
	{ 0x14DAE9, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x14DB85, "S NET MEDIA", "S-NET-MEDIA" },
	{ 0x14E4EC, "mLogic LLC", "MLOGIC-LLC" },
	{ 0x14E6E4, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x14EB33, "BSMediasoft Co., Ltd.", "BSMEDIASOFT" },
	{ 0x14EDA5, "Wächter GmbH Sicherheitssysteme", "WÄCHTER-SICHERHEITSSYSTEME" },
	{ 0x14EDE4, "Kaiam Corporation", "KAIAM" },
	{ 0x14EE9D, "AirNav Systems LLC", "AIRNAV-LLC" },
	{ 0x14F0C5, "Xtremio Ltd.", "XTREMIO" },
	{ 0x14F28E, "ShenYang ZhongKe-Allwin Technology Co.LTD", "SHENYANG-ZHONGKE-ALLWIN" },
	{ 0x14F42A, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x14F65A, "Xiaomi inc.", "XIAOMI" },
	{ 0x14F893, "Wuhan FiberHome Digital Technology Co.,Ltd.", "WUHAN-FIBERHOME-DIGITAL" },
	{ 0x14FEAF, "SAGITTAR LIMITED", "SAGITTAR" },
	{ 0x14FEB5, "Dell Inc", "DELL" },
	{ 0x18002D, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x1800DB, "Fitbit Inc.", "FITBIT" },
	{ 0x1801E3, "Elektrobit Wireless Communications Ltd", "ELEKTROBIT-WIRELESS-COMMUNICATION" },
	{ 0x180373, "Dell Inc", "DELL" },
	{ 0x1803FA, "IBT Interfaces", "IBT-INTERFACES" },
	{ 0x180675, "DILAX Intelcom GmbH", "DILAX-INTELCOM" },
	{ 0x180B52, "Nanotron Technologies GmbH", "NANOTRON-TECHNOLOGIES" },
	{ 0x180C14, "iSonea Limited", "ISONEA" },
	{ 0x180C77, "Westinghouse Electric Company, LLC", "WESTINGHOUSE-ELECTRIC-COMPANY-LLC" },
	{ 0x180CAC, "CANON INC.", "CANON" },
	{ 0x18104E, "CEDINT-UPM", "CEDINT-UPM" },
	{ 0x181420, "TEB SAS", "TEB-SAS" },
	{ 0x181456, "Nokia Corporation", "NOKIA" },
	{ 0x181714, "DAEWOOIS", "DAEWOOIS" },
	{ 0x181725, "Cameo Communications, Inc.", "CAMEO-COMMUNICATION" },
	{ 0x18193F, "Tamtron Oy", "TAMTRON-OY" },
	{ 0x181BEB, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x181E78, "SAGEMCOM", "SAGEMCOM" },
	{ 0x181EB0, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x182012, "Aztech Associates Inc.", "AZTECH-ASSOCIATES" },
	{ 0x182032, "Apple", "APPLE" },
	{ 0x1820A6, "Sage Co., Ltd.", "SAGE" },
	{ 0x18227E, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x182666, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x182861, "AirTies Wireless Networks", "AIRTIES-WIRELESS-NETWORKS" },
	{ 0x182A7B, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x182B05, "8D Technologies", "8D-TECHNOLOGIES" },
	{ 0x182C91, "Concept Development, Inc.", "CONCEPT-DEVELOPMENT" },
	{ 0x183009, "Woojin Industrial Systems Co., Ltd.", "WOOJIN-INDUSTRIAL" },
	{ 0x1832A2, "LAON TECHNOLOGY CO., LTD.", "LAON" },
	{ 0x18339D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x183451, "Apple", "APPLE" },
	{ 0x1836FC, "Elecsys International Corporation", "ELECSYS" },
	{ 0x183825, "Wuhan Lingjiu High-tech Co.,Ltd.", "WUHAN-LINGJIU-HIGH-TECH" },
	{ 0x183864, "CAP-TECH INTERNATIONAL CO., LTD.", "CAP-TECH" },
	{ 0x183919, "Unicoi Systems", "UNICOI" },
	{ 0x183A2D, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x183BD2, "BYD Precision Manufacture Company Ltd.", "BYD-PRECISION-MANUFACTURE-COMPANY" },
	{ 0x183DA2, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x183F47, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x18421D, "PRIVATE", "PRIVATE" },
	{ 0x18422F, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x184462, "Riava Networks, Inc.", "RIAVA-NETWORKS" },
	{ 0x1844E6, "zte corporation", "ZTE" },
	{ 0x184617, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x1848D8, "Fastback Networks", "FASTBACK-NETWORKS" },
	{ 0x184A6F, "Alcatel-Lucent Shanghai Bell Co., Ltd", "ALCATEL-LUCENT-SHANGHAI-BELL" },
	{ 0x184E94, "MESSOA TECHNOLOGIES INC.", "MESSOA-TECHNOLOGIES" },
	{ 0x185253, "Pixord Corporation", "PIXORD" },
	{ 0x1853E0, "Hanyang Digitech Co.Ltd", "HANYANG-DIGITECH" },
	{ 0x18550F, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x185933, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x185936, "XIAOMI INC", "XIAOMI" },
	{ 0x185AE8, "Zenotech.Co.,Ltd", "ZENOTECH" },
	{ 0x18622C, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0x186472, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0x186571, "Top Victory Electronics (Taiwan) Co., Ltd.", "TOP-VICTORY-ELECTRONICS-TAIWAN" },
	{ 0x1866E3, "Veros Systems, Inc.", "VEROS" },
	{ 0x18673F, "Hanover Displays Limited", "HANOVER-DISPLAYS" },
	{ 0x186751, "KOMEG Industrielle Messtechnik GmbH", "KOMEG-INDUSTRIELLE-MESSTECHNIK" },
	{ 0x1867B0, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x186882, "Beward R&D Co., Ltd.", "BEWARD-R-D" },
	{ 0x186D99, "Adanis Inc.", "ADANIS" },
	{ 0x187117, "eta plus electronic gmbh", "ETA-PLUS" },
	{ 0x1879A2, "GMJ ELECTRIC LIMITED", "GMJ-ELECTRIC" },
	{ 0x187A93, "AMICCOM Electronics Corporation", "AMICCOM-ELECTRONICS" },
	{ 0x187C81, "Valeo Vision Systems", "VALEO-VISION" },
	{ 0x187ED5, "shenzhen kaism technology Co. Ltd", "SHENZHEN-KAISM" },
	{ 0x1880CE, "Barberry Solutions Ltd", "BARBERRY-SOLUTIONS" },
	{ 0x1880F5, "Alcatel-Lucent Shanghai Bell Co., Ltd", "ALCATEL-LUCENT-SHANGHAI-BELL" },
	{ 0x188219, "Alibaba Cloud Computing Ltd.", "ALIBABA-CLOUD-COMPUTING" },
	{ 0x188331, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x1883BF, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x188410, "CoreTrust Inc.", "CORETRUST" },
	{ 0x18863A, "DIGITAL ART SYSTEM", "DIGITAL-ART-SYSTEM" },
	{ 0x1886AC, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x188796, "HTC Corporation", "HTC" },
	{ 0x188857, "Beijing Jinhong Xi-Dian Information Technology Corp.", "BEIJING-JINHONG-XI-DIAN-INFORMATION" },
	{ 0x1889DF, "CerebrEX Inc.", "CEREBREX" },
	{ 0x188ED5, "TP Vision Belgium N.V. - innovation site Brugge", "TP-VISION-BELGIUM-N-V---INNOVATION-SITE-BRUGGE" },
	{ 0x18922C, "Virtual Instruments", "VIRTUAL-INSTRUMENTS" },
	{ 0x1897FF, "TechFaith Wireless Technology Limited", "TECHFAITH-WIRELESS" },
	{ 0x189A67, "CSE-Servelec Limited", "CSE-SERVELEC" },
	{ 0x189C5D, "Cisco", "CISCO" },
	{ 0x189EFC, "Apple", "APPLE" },
	{ 0x18A3E8, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0x18A905, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x18A958, "PROVISION THAI CO., LTD.", "PROVISION-THAI" },
	{ 0x18A99B, "Dell Inc", "DELL" },
	{ 0x18AA45, "Fon Technology", "FON" },
	{ 0x18ABF5, "Ultra Electronics - Electrics", "ULTRA-ELECTRONICS---ELECTRICS" },
	{ 0x18AD4D, "Polostar Technology Corporation", "POLOSTAR" },
	{ 0x18AEBB, "Siemens Convergence Creators GmbH&Co.KG", "SIEMENS-CONVERGENCE-CREATORS" },
	{ 0x18AF61, "Apple, Inc", "APPLE" },
	{ 0x18AF8F, "Apple", "APPLE" },
	{ 0x18AF9F, "DIGITRONIC Automationsanlagen GmbH", "DIGITRONIC-AUTOMATIONSANLAGEN" },
	{ 0x18B169, "Sonicwall", "SONICWALL" },
	{ 0x18B209, "Torrey Pines Logic, Inc", "TORREY-PINES-LOGIC" },
	{ 0x18B3BA, "Netlogic AB", "NETLOGIC-AB" },
	{ 0x18B430, "Nest Labs Inc.", "NEST-LABS" },
	{ 0x18B591, "I-Storm", "I-STORM" },
	{ 0x18B79E, "Invoxia", "INVOXIA" },
	{ 0x18BDAD, "L-TECH CORPORATION", "L-TECH" },
	{ 0x18C086, "Broadcom Corporation", "BROADCOM" },
	{ 0x18C451, "Tucson Embedded Systems", "TUCSON-EMBEDDED" },
	{ 0x18C58A, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x18C8E7, "Shenzhen Hualistone Technology Co.,Ltd", "SHENZHEN-HUALISTONE" },
	{ 0x18CC23, "Philio Technology Corporation", "PHILIO" },
	{ 0x18CF5E, "Liteon Technology Corporation", "LITEON" },
	{ 0x18D071, "DASAN CO., LTD.", "DASAN" },
	{ 0x18D5B6, "SMG Holdings LLC", "SMG-HOLDINGS-LLC" },
	{ 0x18D66A, "Inmarsat", "INMARSAT" },
	{ 0x18D6CF, "Kurth Electronic GmbH", "KURTH" },
	{ 0x18D949, "Qvis Labs, LLC", "QVIS-LABS-LLC" },
	{ 0x18DC56, "Yulong Computer Telecommunication Scientific(shenzhen)Co.,Lt", "YULONG-TELECOMMUNICATION-SCIENTIFIC-SHENZHEN-LT" },
	{ 0x18E288, "STT Condigi", "STT-CONDIGI" },
	{ 0x18E2C2, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x18E728, "Cisco", "CISCO" },
	{ 0x18E7F4, "Apple", "APPLE" },
	{ 0x18E80F, "Viking Electronics Inc.", "VIKING-ELECTRONICS" },
	{ 0x18E8DD, "MODULETEK", "MODULETEK" },
	{ 0x18EE69, "Apple", "APPLE" },
	{ 0x18EF63, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x18F145, "NetComm Wireless Limited", "NETCOMM-WIRELESS" },
	{ 0x18F46A, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x18F643, "Apple", "APPLE" },
	{ 0x18F650, "Multimedia Pacific Limited", "MULTIMEDIA-PACIFIC" },
	{ 0x18F87A, "i3 International Inc.", "I3" },
	{ 0x18FA6F, "ISC applied systems corp", "ISC-APPLIED" },
	{ 0x18FB7B, "Dell Inc", "DELL" },
	{ 0x18FC9F, "Changhe Electronics Co., Ltd.", "CHANGHE-ELECTRONICS" },
	{ 0x18FE34, "Espressif Inc.", "ESPRESSIF" },
	{ 0x18FF0F, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x18FF2E, "Shenzhen Rui Ying Da Technology Co., Ltd", "SHENZHEN-RUI-YING-DA" },
	{ 0x1C0656, "IDY Corporation", "IDY" },
	{ 0x1C08C1, "Lg Innotek", "LG-INNOTEK" },
	{ 0x1C0B52, "EPICOM S.A", "EPICOM" },
	{ 0x1C0FCF, "Sypro Optics GmbH", "SYPRO-OPTICS" },
	{ 0x1C11E1, "Wartsila Finland Oy", "WARTSILA-FINLAND-OY" },
	{ 0x1C129D, "IEEE PES PSRC/SUB", "IEEE-PES-PSRC/SUB" },
	{ 0x1C1448, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x1C14B3, "Pinyon Technologies", "PINYON-TECHNOLOGIES" },
	{ 0x1C17D3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x1C184A, "ShenZhen RicherLink Technologies Co.,LTD", "SHENZHEN-RICHERLINK-TECHNOLOGIES" },
	{ 0x1C19DE, "eyevis GmbH", "EYEVIS" },
	{ 0x1C1AC0, "Apple", "APPLE" },
	{ 0x1C1B68, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x1C1CFD, "Dalian Hi-Think Computer Technology, Corp", "DALIAN-HI-THINK" },
	{ 0x1C1D67, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x1C1D86, "Cisco", "CISCO" },
	{ 0x1C334D, "ITS Telecom", "ITS-TELECOM" },
	{ 0x1C3477, "Innovation Wireless", "INNOVATION-WIRELESS" },
	{ 0x1C35F1, "NEW Lift Neue Elektronische Wege Steuerungsbau GmbH", "NEW-LIFT-NEUE-ELEKTRONISCHE-WEGE-STEUERUNGSBAU" },
	{ 0x1C37BF, "Cloudium Systems Ltd.", "CLOUDIUM" },
	{ 0x1C3947, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x1C3A4F, "AccuSpec Electronics, LLC", "ACCUSPEC-ELECTRONICS-LLC" },
	{ 0x1C3DE7, "Sigma Koki Co.,Ltd.", "SIGMA-KOKI" },
	{ 0x1C3E84, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x1C4158, "Gemalto M2M GmbH", "GEMALTO-M2M" },
	{ 0x1C43EC, "JAPAN CIRCUIT CO.,LTD", "JAPAN-CIRCUIT" },
	{ 0x1C4593, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x1C4840, "IMS Messsysteme GmbH", "IMS-MESSSYSTEME" },
	{ 0x1C48F9, "GN Netcom A/S", "GN-NETCOM-A/S" },
	{ 0x1C4AF7, "AMON INC", "AMON" },
	{ 0x1C4BB9, "SMG ENTERPRISE, LLC", "SMG-LLC" },
	{ 0x1C4BD6, "AzureWave", "AZUREWAVE" },
	{ 0x1C51B5, "Techaya LTD", "TECHAYA" },
	{ 0x1C5216, "DONGGUAN HELE ELECTRONICS CO., LTD", "DONGGUAN-HELE-ELECTRONICS" },
	{ 0x1C52D6, "FLAT DISPLAY TECHNOLOGY CORPORATION", "FLAT-DISPLAY" },
	{ 0x1C5A3E, "Samsung Eletronics Co., Ltd (Visual Display Divison)", "SAMSUNG-ELETRONICS-VISUAL-DISPLAY-DIVISON" },
	{ 0x1C5A6B, "Philips Electronics Nederland BV", "PHILIPS-ELECTRONICS-NEDERLAND" },
	{ 0x1C5C55, "PRIMA Cinema, Inc", "PRIMA-CINEMA" },
	{ 0x1C5C60, "Shenzhen Belzon Technology Co.,LTD.", "SHENZHEN-BELZON" },
	{ 0x1C5FFF, "Beijing Ereneben Information Technology Co.,Ltd Shenzhen Branch", "BEIJING-ERENEBEN-INFORMATION-SHENZHEN-BRANCH" },
	{ 0x1C62B8, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x1C63B7, "OpenProducts 237 AB", "OPENPRODUCTS-237-AB" },
	{ 0x1C659D, "Liteon Technology Corporation", "LITEON" },
	{ 0x1C666D, "Hon Hai Precision Ind.Co.Ltd", "HON-HAI-PRECISION-IND" },
	{ 0x1C66AA, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x1C69A5, "Research In Motion", "RESEARCH-IN-MOTION" },
	{ 0x1C6A7A, "Cisco", "CISCO" },
	{ 0x1C6BCA, "Mitsunami Co., Ltd.", "MITSUNAMI" },
	{ 0x1C6F65, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x1C7508, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x1C76CA, "Terasic Technologies Inc.", "TERASIC-TECHNOLOGIES" },
	{ 0x1C7839, "Shenzhen Tencent Computer System Co., Ltd.", "SHENZHEN-TENCENT-SYSTEM" },
	{ 0x1C7B21, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x1C7C11, "EID", "EID" },
	{ 0x1C7C45, "Vitek Industrial Video Products, Inc.", "VITEK-INDUSTRIAL-VIDEO-PRODUCTS" },
	{ 0x1C7CC7, "Coriant GmbH", "CORIANT" },
	{ 0x1C7D22, "Fuji Xerox Co., Ltd.", "FUJI-XEROX" },
	{ 0x1C7E51, "3bumen.com", "3BUMEN-COM" },
	{ 0x1C7EE5, "D-Link International", "D-LINK" },
	{ 0x1C8341, "Hefei Bitland Information Technology Co.Ltd", "HEFEI-BITLAND-INFORMATION" },
	{ 0x1C83B0, "Linked IP GmbH", "LINKED-IP" },
	{ 0x1C8464, "FORMOSA WIRELESS COMMUNICATION CORP.", "FORMOSA-WIRELESS-COMMUNICATION" },
	{ 0x1C86AD, "MCT CO., LTD.", "MCT" },
	{ 0x1C8E5C, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x1C8E8E, "DB Communication & Systems Co., ltd.", "DB-COMMUNICATION" },
	{ 0x1C8F8A, "Phase Motion Control SpA", "PHASE-MOTION-CONTROL-SPA" },
	{ 0x1C9179, "Integrated System Technologies Ltd", "INTEGRATED-SYSTEM-TECHNOLOGIES" },
	{ 0x1C9492, "RUAG Schweiz AG", "RUAG-SCHWEIZ" },
	{ 0x1C955D, "I-LAX ELECTRONICS INC.", "I-LAX-ELECTRONICS" },
	{ 0x1C959F, "Veethree Electronics And Marine LLC", "VEETHREE-ELECTRONICS-AND-MARINE-LLC" },
	{ 0x1C965A, "Weifang goertek Electronics CO.,LTD", "WEIFANG-GOERTEK-ELECTRONICS" },
	{ 0x1C973D, "PRICOM Design", "PRICOM-DESIGN" },
	{ 0x1C994C, "Murata Manufactuaring Co.,Ltd.", "MURATA-MANUFACTUARING" },
	{ 0x1C9C26, "Zoovel Technologies", "ZOOVEL-TECHNOLOGIES" },
	{ 0x1C9ECB, "Beijing Nari Smartchip Microelectronics Company Limited", "BEIJING-NARI-SMARTCHIP-MICROELECTRONICS-COMPANY" },
	{ 0x1CA2B1, "ruwido austria gmbh", "RUWIDO-AUSTRIA" },
	{ 0x1CA532, "Shenzhen Gongjin Electronics Co.,Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0x1CA770, "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LT", "SHENZHEN-CHUANGWEI-RGB-ELECTRONICS-LT" },
	{ 0x1CAA07, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x1CAB01, "Innovolt", "INNOVOLT" },
	{ 0x1CABA7, "Apple", "APPLE" },
	{ 0x1CAF05, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x1CAFF7, "D-LINK INTERNATIONAL PTE LIMITED", "D-LINK-PTE" },
	{ 0x1CB094, "HTC Corporation", "HTC" },
	{ 0x1CB17F, "NEC Platforms, Ltd.", "NEC-PLATFORMS" },
	{ 0x1CB243, "TDC A/S", "TDC-A/S" },
	{ 0x1CB72C, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x1CBA8C, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x1CBBA8, "OJSC \"Ufimskiy Zavod \"Promsvyaz\"", "OJSC-UFIMSKIY-ZAVOD-PROMSVYAZ" },
	{ 0x1CBD0E, "Amplified Engineering Pty Ltd", "AMPLIFIED-ENGINEERING-PTY" },
	{ 0x1CBDB9, "D-LINK INTERNATIONAL PTE LIMITED", "D-LINK-PTE" },
	{ 0x1CC11A, "Wavetronix", "WAVETRONIX" },
	{ 0x1CC1DE, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x1CC316, "MileSight Technology Co., Ltd.", "MILESIGHT" },
	{ 0x1CC586, "Absolute Acoustics", "ABSOLUTE-ACOUSTICS" },
	{ 0x1CC63C, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x1CC72D, "Shenzhen Huapu Digital CO.,Ltd", "SHENZHEN-HUAPU-DIGITAL" },
	{ 0x1CCAE3, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x1CD40C, "Kriwan Industrie-Elektronik GmbH", "KRIWAN-INDUSTRIE-ELEKTRONIK" },
	{ 0x1CDEA7, "Cisco", "CISCO" },
	{ 0x1CDF0F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x1CE165, "Marshal Corporation", "MARSHAL" },
	{ 0x1CE192, "Qisda Corporation", "QISDA" },
	{ 0x1CE2CC, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x1CE62B, "Apple", "APPLE" },
	{ 0x1CE6C7, "Cisco", "CISCO" },
	{ 0x1CE85D, "Cisco", "CISCO" },
	{ 0x1CEEE8, "Ilshin Elecom", "ILSHIN-ELECOM" },
	{ 0x1CF03E, "Wearhaus Inc.", "WEARHAUS" },
	{ 0x1CF061, "SCAPS GmbH", "SCAPS" },
	{ 0x1CF4CA, "PRIVATE", "PRIVATE" },
	{ 0x1CF5E7, "Turtle Industry Co., Ltd.", "TURTLE-INDUSTRY" },
	{ 0x1CFA68, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x1CFCBB, "Realfiction ApS", "REALFICTION-APS" },
	{ 0x1CFEA7, "IDentytech Solutins Ltd.", "IDENTYTECH-SOLUTINS" },
	{ 0x20014F, "Linea Research Ltd", "LINEA-RESEARCH" },
	{ 0x2002AF, "Murata Manufactuaring Co.,Ltd.", "MURATA-MANUFACTUARING" },
	{ 0x200505, "RADMAX COMMUNICATION PRIVATE LIMITED", "RADMAX-COMMUNICATION-PRIVATE" },
	{ 0x2005E8, "OOO InProMedia", "OOO-INPROMEDIA" },
	{ 0x2008ED, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x200A5E, "Xiangshan Giant Eagle Technology Developing co.,LTD", "XIANGSHAN-GIANT-EAGLE-DEVELOPING" },
	{ 0x200BC7, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x200CC8, "NETGEAR INC.,", "NETGEAR" },
	{ 0x200E95, "IEC – TC9 WG43", "IEC-–-TC9-WG43" },
	{ 0x20107A, "Gemtek Technology Co., Ltd.", "GEMTEK" },
	{ 0x201257, "Most Lucky Trading Ltd", "MOST-LUCKY-TRADING" },
	{ 0x2012D5, "Scientech Materials Corporation", "SCIENTECH-MATERIALS" },
	{ 0x2013E0, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x2016D8, "Liteon Technology Corporation", "LITEON" },
	{ 0x20180E, "Shenzhen Sunchip Technology Co., Ltd", "SHENZHEN-SUNCHIP" },
	{ 0x201A06, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x201D03, "Elatec GmbH", "ELATEC" },
	{ 0x2021A5, "LG Electronics Inc", "LG-ELECTRONICS" },
	{ 0x202564, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x202598, "Teleview", "TELEVIEW" },
	{ 0x2028BC, "Visionscape Co,. Ltd.", "VISIONSCAPE" },
	{ 0x202BC1, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x202CB7, "Kong Yue Electronics & Information Industry (Xinhui) Ltd.", "KONG-YUE-ELECTRONICS-INFORMATION-INDUSTRY-XINHUI" },
	{ 0x2031EB, "HDSN", "HDSN" },
	{ 0x203706, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x2037BC, "Kuipers Electronic Engineering BV", "KUIPERS-ENGINEERING" },
	{ 0x203A07, "Cisco", "CISCO" },
	{ 0x204005, "feno GmbH", "FENO" },
	{ 0x20415A, "Smarteh d.o.o.", "SMARTEH-D-O-O" },
	{ 0x20443A, "Schneider Electric Asia Pacific Ltd", "SCHNEIDER-ELECTRIC-ASIA-PACIFIC" },
	{ 0x2046A1, "VECOW Co., Ltd", "VECOW" },
	{ 0x2046F9, "Advanced Network Devices (dba:AND)", "ADVANCED-NETWORK-DEVICES-DBA:AND" },
	{ 0x204747, "Dell Inc.", "DELL" },
	{ 0x204AAA, "Hanscan Spain S.A.", "HANSCAN-SPAIN" },
	{ 0x204C6D, "Hugo Brennenstuhl Gmbh & Co. KG.", "HUGO-BRENNENSTUHL" },
	{ 0x204C9E, "Cisco", "CISCO" },
	{ 0x204E6B, "Axxana(israel) ltd", "AXXANA-ISRAEL" },
	{ 0x204E7F, "NETGEAR", "NETGEAR" },
	{ 0x2053CA, "Risk Technology Ltd", "RISK" },
	{ 0x205476, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x205721, "Salix Technology CO., Ltd.", "SALIX" },
	{ 0x2059A0, "Paragon Technologies Inc.", "PARAGON-TECHNOLOGIES" },
	{ 0x205A00, "Coval", "COVAL" },
	{ 0x205B2A, "PRIVATE", "PRIVATE" },
	{ 0x205B5E, "Shenzhen Wonhe Technology Co., Ltd", "SHENZHEN-WONHE" },
	{ 0x205CFA, "Yangzhou ChangLian Network Technology Co,ltd.", "YANGZHOU-CHANGLIAN-NETWORK" },
	{ 0x206274, "Microsoft Corporation", "MICROSOFT" },
	{ 0x206432, "SAMSUNG ELECTRO MECHANICS CO.,LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x2067B1, "Pluto inc.", "PLUTO" },
	{ 0x20689D, "Liteon Technology Corporation", "LITEON" },
	{ 0x206A8A, "Wistron InfoComm Manufacturing(Kunshan)Co.,Ltd.", "WISTRON-INFOCOMM-MANUFACTURING-KUNSHAN" },
	{ 0x206AFF, "Atlas Elektronik UK Limited", "ATLAS-ELEKTRONIK-UK" },
	{ 0x206E9C, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x206FEC, "Braemac CA LLC", "BRAEMAC-CA-LLC" },
	{ 0x207355, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x2074CF, "Shenzhen Voxtech Co.,Ltd", "SHENZHEN-VOXTECH" },
	{ 0x207600, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x207693, "Lenovo (Beijing) Limited.", "LENOVO-BEIJING" },
	{ 0x207C8F, "Quanta Microsystems,Inc.", "QUANTA-MICROSYSTEMS" },
	{ 0x207D74, "Apple", "APPLE" },
	{ 0x20858C, "Assa", "ASSA" },
	{ 0x2087AC, "AES motomation", "AES-MOTOMATION" },
	{ 0x208984, "COMPAL INFORMATION (KUNSHAN) CO., LTD", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x208986, "zte corporation", "ZTE" },
	{ 0x209148, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x20918A, "PROFALUX", "PROFALUX" },
	{ 0x2091D9, "I'M SPA", "I-M-SPA" },
	{ 0x20934D, "Fujian Star-net Communication Co., Ltd", "FUJIAN-STAR-NET-COMMUNICATION" },
	{ 0x209AE9, "Volacomm Co., Ltd", "VOLACOMM" },
	{ 0x209BA5, "JIAXING GLEAD Electronics Co.,Ltd", "JIAXING-GLEAD-ELECTRONICS" },
	{ 0x20A2E4, "Apple", "APPLE" },
	{ 0x20A2E7, "Lee-Dickens Ltd", "LEE-DICKENS" },
	{ 0x20A787, "Bointec Taiwan Corporation Limited", "BOINTEC-TAIWAN" },
	{ 0x20A99B, "Microsoft Corporation", "MICROSOFT" },
	{ 0x20AA25, "IP-NET LLC", "IP-NET-LLC" },
	{ 0x20AA4B, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x20B0F7, "Enclustra GmbH", "ENCLUSTRA" },
	{ 0x20B399, "Enterasys", "ENTERASYS" },
	{ 0x20B5C6, "Mimosa Networks", "MIMOSA-NETWORKS" },
	{ 0x20B7C0, "Omicron electronics GmbH", "OMICRON-ELECTRONICS" },
	{ 0x20BBC0, "Cisco", "CISCO" },
	{ 0x20BBC6, "Jabil Circuit Hungary Ltd.", "JABIL-CIRCUIT-HUNGARY" },
	{ 0x20BFDB, "DVL", "DVL" },
	{ 0x20C06D, "SHENZHEN SPACETEK TECHNOLOGY CO.,LTD", "SHENZHEN-SPACETEK" },
	{ 0x20C1AF, "i Wit Digital Co., Limited", "I-WIT-DIGITAL" },
	{ 0x20C38F, "Texas Instruments Inc", "TEXAS-INSTRUMENTS" },
	{ 0x20C60D, "Shanghai annijie Information technology Co.,LTD", "SHANGHAI-ANNIJIE-INFORMATION" },
	{ 0x20C6EB, "Panasonic Corporation AVC Networks Company", "PANASONIC-AVC-NETWORKS-COMPANY" },
	{ 0x20C8B3, "SHENZHEN BUL-TECH CO.,LTD.", "SHENZHEN-BUL-TECH" },
	{ 0x20C9D0, "Apple", "APPLE" },
	{ 0x20CD39, "Texas Instruments, Inc", "TEXAS-INSTRUMENTS" },
	{ 0x20CEC4, "Peraso Technologies", "PERASO-TECHNOLOGIES" },
	{ 0x20CF30, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x20D21F, "Wincal Technology Corp.", "WINCAL" },
	{ 0x20D390, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x20D5AB, "Korea Infocom Co.,Ltd.", "KOREA-INFOCOM" },
	{ 0x20D5BF, "Samsung Eletronics Co., Ltd", "SAMSUNG-ELETRONICS" },
	{ 0x20D607, "Nokia Corporation", "NOKIA" },
	{ 0x20D75A, "Posh Mobile Limited", "POSH-MOBILE" },
	{ 0x20D906, "Iota, Inc.", "IOTA" },
	{ 0x20DC93, "Cheetah Hi-Tech, Inc.", "CHEETAH-HI-TECH" },
	{ 0x20DCE6, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x20DF3F, "Nanjing SAC Power Grid Automation Co., Ltd.", "NANJING-SAC-POWER-GRID-AUTOMATION" },
	{ 0x20E52A, "NETGEAR INC.,", "NETGEAR" },
	{ 0x20E564, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x20E791, "Siemens Healthcare Diagnostics, Inc", "SIEMENS-HEALTHCARE-DIAGNOSTICS" },
	{ 0x20EAC7, "SHENZHEN RIOPINE ELECTRONICS CO., LTD", "SHENZHEN-RIOPINE-ELECTRONICS" },
	{ 0x20ED74, "Ability enterprise co.,Ltd.", "ABILITY" },
	{ 0x20EEC6, "Elefirst Science & Tech Co ., ltd", "ELEFIRST-SCIENCE-TECH" },
	{ 0x20F002, "MTData Developments Pty. Ltd.", "MTDATA-DEVELOPMENTS-PTY" },
	{ 0x20F3A3, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x20F85E, "Delta Electronics", "DELTA-ELECTRONICS" },
	{ 0x20FABB, "Cambridge Executive Limited", "CAMBRIDGE-EXECUTIVE" },
	{ 0x20FDF1, "3COM EUROPE LTD", "3COM" },
	{ 0x20FECD, "System In Frontier Inc.", "SYSTEM-IN-FRONTIER" },
	{ 0x20FEDB, "M2M Solution S.A.S.", "M2M-SOLUTION-S" },
	{ 0x2401C7, "Cisco", "CISCO" },
	{ 0x24050F, "MTN Electronic Co. Ltd", "MTN" },
	{ 0x240917, "Devlin Electronics Limited", "DEVLIN-ELECTRONICS" },
	{ 0x240995, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x240A11, "TCT Mobile Limited", "TCT-MOBILE" },
	{ 0x240A64, "AzureWaveTechnologies,Inc", "AZUREWAVETECHNOLOGIES" },
	{ 0x240B2A, "Viettel Group", "VIETTEL-GROUP" },
	{ 0x240BB1, "KOSTAL Industrie Elektrik GmbH", "KOSTAL-INDUSTRIE-ELEKTRIK" },
	{ 0x241064, "Shenzhen Ecsino Tecnical Co. Ltd", "SHENZHEN-ECSINO-TECNICAL" },
	{ 0x241125, "Hutek Co., Ltd.", "HUTEK" },
	{ 0x241148, "Entropix, LLC", "ENTROPIX-LLC" },
	{ 0x2411D0, "Chongqing Ehs Science and Technology Development Co.,Ltd.", "CHONGQING-EHS-SCIENCE-AND-DEVELOPMENT" },
	{ 0x241A8C, "Squarehead Technology AS", "SQUAREHEAD-AS" },
	{ 0x241B13, "Shanghai Nutshell Electronic Co., Ltd.", "SHANGHAI-NUTSHELL" },
	{ 0x241B44, "Hangzhou Tuners Electronics Co., Ltd", "HANGZHOU-TUNERS-ELECTRONICS" },
	{ 0x241C04, "SHENZHEN JEHE TECHNOLOGY DEVELOPMENT CO., LTD.", "SHENZHEN-JEHE-DEVELOPMENT" },
	{ 0x241F2C, "Calsys, Inc.", "CALSYS" },
	{ 0x2421AB, "Sony Ericsson Mobile Communications", "SONY-ERICSSON-MOBILE-COMMUNICATION" },
	{ 0x24240E, "Apple", "APPLE" },
	{ 0x242642, "SHARP Corporation.", "SHARP" },
	{ 0x242FFA, "Toshiba Global Commerce Solutions", "TOSHIBA-GLOBAL-COMMERCE-SOLUTIONS" },
	{ 0x24336C, "PRIVATE", "PRIVATE" },
	{ 0x24374C, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x2437EF, "EMC Electronic Media Communication SA", "EMC-MEDIA-COMMUNICATION" },
	{ 0x243C20, "Dynamode Group", "DYNAMODE-GROUP" },
	{ 0x2442BC, "Alinco,incorporated", "ALINCO-INCORPORATED" },
	{ 0x244597, "GEMUE Gebr. Mueller Apparatebau", "GEMUE-GEBR-MUELLER-APPARATEBAU" },
	{ 0x24470E, "PentronicAB", "PENTRONICAB" },
	{ 0x24497B, "Innovative Converged Devices Inc", "INNOVATIVE-CONVERGED-DEVICES" },
	{ 0x244B03, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x244B81, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x244F1D, "iRule LLC", "IRULE-LLC" },
	{ 0x245FDF, "KYOCERA Corporation", "KYOCERA" },
	{ 0x246278, "sysmocom - systems for mobile communications GmbH", "SYSMOCOM---FOR-MOBILE-COMMUNICATION" },
	{ 0x2464EF, "CYG SUNRI CO.,LTD.", "CYG-SUNRI" },
	{ 0x246511, "AVM GmbH", "AVM" },
	{ 0x24693E, "innodisk Corporation", "INNODISK" },
	{ 0x24694A, "Jasmine Systems Inc.", "JASMINE" },
	{ 0x2469A5, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x246AAB, "IT-IS International", "IT-IS" },
	{ 0x247189, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x247656, "Shanghai Net Miles Fiber Optics Technology Co., LTD.", "SHANGHAI-NET-MILES-FIBER-OPTICS" },
	{ 0x24767D, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x247703, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x247F3C, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x248000, "Westcontrol AS", "WESTCONTROL-AS" },
	{ 0x2481AA, "KSH International Co., Ltd.", "KSH" },
	{ 0x24828A, "Prowave Technologies Ltd.", "PROWAVE-TECHNOLOGIES" },
	{ 0x2486F4, "Ctek, Inc.", "CTEK" },
	{ 0x248707, "SEnergy Corporation", "SENERGY" },
	{ 0x2493CA, "Voxtronic Technology Computer-Systeme GmbH", "VOXTRONIC--SYSTEME" },
	{ 0x249442, "OPEN ROAD SOLUTIONS , INC.", "OPEN-ROAD-SOLUTIONS" },
	{ 0x249504, "SFR", "SFR" },
	{ 0x2497ED, "Techvision Intelligent Technology Limited", "TECHVISION-INTELLIGENT" },
	{ 0x249EAB, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x24A074, "Apple", "APPLE" },
	{ 0x24A2E1, "Apple, Inc", "APPLE" },
	{ 0x24A42C, "KOUKAAM a.s.", "KOUKAAM-A-S" },
	{ 0x24A43C, "Ubiquiti Networks, INC", "UBIQUITI-NETWORKS" },
	{ 0x24A495, "Thales Canada Inc.", "THALES-CANADA" },
	{ 0x24A87D, "Panasonic Automotive Systems Asia Pacific(Thailand)Co.,Ltd.", "PANASONIC-AUTOMOTIVE-ASIA-PACIFIC-THAILAND" },
	{ 0x24A937, "PURE Storage", "PURE-STORAGE" },
	{ 0x24AB81, "Apple", "APPLE" },
	{ 0x24AF4A, "Alcatel-Lucent-IPD", "ALCATEL-LUCENT-IPD" },
	{ 0x24AF54, "NEXGEN Mediatech Inc.", "NEXGEN-MEDIATECH" },
	{ 0x24B0A9, "Shanghai Mobiletek Communication Ltd.", "SHANGHAI-MOBILETEK-COMMUNICATION" },
	{ 0x24B657, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x24B6B8, "FRIEM SPA", "FRIEM-SPA" },
	{ 0x24B6FD, "Dell Inc", "DELL" },
	{ 0x24B88C, "Crenus Co.,Ltd.", "CRENUS" },
	{ 0x24B8D2, "Opzoon Technology Co.,Ltd.", "OPZOON" },
	{ 0x24BA30, "Technical Consumer Products, Inc.", "TECHNICAL-CONSUMER-PRODUCTS" },
	{ 0x24BBC1, "Absolute Analysis", "ABSOLUTE-ANALYSIS" },
	{ 0x24BC82, "Dali Wireless, Inc.", "DALI-WIRELESS" },
	{ 0x24BE05, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x24BF74, "PRIVATE", "PRIVATE" },
	{ 0x24C0B3, "RSF", "RSF" },
	{ 0x24C696, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x24C848, "mywerk system GmbH", "MYWERK-SYSTEM" },
	{ 0x24C86E, "Chaney Instrument Co.", "CHANEY-INSTRUMENT" },
	{ 0x24C9A1, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x24C9DE, "Genoray", "GENORAY" },
	{ 0x24CBE7, "MYK, Inc.", "MYK" },
	{ 0x24CF21, "Shenzhen State Micro Technology Co., Ltd", "SHENZHEN-STATE-MICRO" },
	{ 0x24D13F, "MEXUS CO.,LTD", "MEXUS" },
	{ 0x24D2CC, "SmartDrive Systems Inc.", "SMARTDRIVE" },
	{ 0x24D921, "Avaya, Inc", "AVAYA" },
	{ 0x24DAB6, "Sistemas de Gestión Energética S.A. de C.V", "SISTEMAS-DE-GESTIóN-ENERGéTICA-DE-C-V" },
	{ 0x24DBAC, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x24DBAD, "ShopperTrak RCT Corporation", "SHOPPERTRAK-RCT" },
	{ 0x24DBED, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x24DEC6, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0x24E271, "Qingdao Hisense Communications Co.,Ltd", "QINGDAO-HISENSE-COMMUNICATION" },
	{ 0x24E314, "Apple", "APPLE" },
	{ 0x24E6BA, "JSC Zavod im. Kozitsky", "JSC-ZAVOD-IM-KOZITSKY" },
	{ 0x24E9B3, "Cisco", "CISCO" },
	{ 0x24EA40, "Systeme Helmholz GmbH", "SYSTEME-HELMHOLZ" },
	{ 0x24EB65, "SAET I.S. S.r.l.", "SAET-I-S-S-R-L" },
	{ 0x24EC99, "Askey Computer Corp", "ASKEY" },
	{ 0x24ECD6, "CSG Science & Technology Co.,Ltd.Hefei", "CSG-SCIENCE-HEFEI" },
	{ 0x24EE3A, "Chengdu Yingji Electronic Hi-tech Co Ltd", "CHENGDU-YINGJI-HI-TECH" },
	{ 0x24F0FF, "GHT Co., Ltd.", "GHT" },
	{ 0x24F2DD, "Radiant Zemax LLC", "RADIANT-ZEMAX-LLC" },
	{ 0x24F5AA, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x24FD52, "Liteon Technology Corporation", "LITEON" },
	{ 0x2804E0, "FERMAX ELECTRONICA S.A.U.", "FERMAX-ELECTRONICA-U" },
	{ 0x28061E, "NINGBO GLOBAL USEFUL ELECTRIC CO.,LTD", "NINGBO-GLOBAL-USEFUL-ELECTRIC" },
	{ 0x28068D, "ITL, LLC", "ITL-LLC" },
	{ 0x280B5C, "Apple", "APPLE" },
	{ 0x280CB8, "Mikrosay Yazilim ve Elektronik A.S.", "MIKROSAY-YAZILIM-VE-ELEKTRONIK-A-S" },
	{ 0x280DFC, "Sony Computer Entertainment Inc.", "SONY-ENTERTAINMENT" },
	{ 0x28107B, "D-Link International", "D-LINK" },
	{ 0x281471, "Lantis co., LTD.", "LANTIS" },
	{ 0x28162E, "2Wire", "2WIRE" },
	{ 0x2817CE, "Omnisense Ltd", "OMNISENSE" },
	{ 0x281878, "Microsoft Corporation", "MICROSOFT" },
	{ 0x2818FD, "Aditya Infotech Ltd.", "ADITYA-INFOTECH" },
	{ 0x282246, "Beijing Sinoix Communication Co., LTD", "BEIJING-SINOIX-COMMUNICATION" },
	{ 0x2826A6, "PBR electronics GmbH", "PBR-ELECTRONICS" },
	{ 0x28285D, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0x2829CC, "Corsa Technology Incorporated", "CORSA-INCORPORATED" },
	{ 0x2829D9, "GlobalBeiMing technology (Beijing)Co. Ltd", "GLOBALBEIMING-BEIJING" },
	{ 0x282CB2, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x283152, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x2832C5, "Humax.co.,ltd", "HUMAX" },
	{ 0x283410, "Enigma Diagnostics Limited", "ENIGMA-DIAGNOSTICS" },
	{ 0x2834A2, "Cisco", "CISCO" },
	{ 0x283737, "Apple", "APPLE" },
	{ 0x2838CF, "Gen2wave", "GEN2WAVE" },
	{ 0x2839E7, "Preceno Technology Pte.Ltd.", "PRECENO-PTE" },
	{ 0x283B96, "Cool Control LTD", "COOL-CONTROL" },
	{ 0x283CE4, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x28401A, "C8 MediSensors, Inc.", "C8-MEDISENSORS" },
	{ 0x284121, "OptiSense Network, LLC", "OPTISENSE-NETWORK-LLC" },
	{ 0x284430, "GenesisTechnical Systems (UK) Ltd", "GENESISTECHNICAL-UK" },
	{ 0x2847AA, "Nokia Corporation", "NOKIA" },
	{ 0x284846, "GridCentric Inc.", "GRIDCENTRIC" },
	{ 0x284C53, "Intune Networks", "INTUNE-NETWORKS" },
	{ 0x284D92, "Luminator", "LUMINATOR" },
	{ 0x284ED7, "OutSmart Power Systems, Inc.", "OUTSMART-POWER" },
	{ 0x284FCE, "Liaoning Wontel Science and Technology Development Co.,Ltd.", "LIAONING-WONTEL-SCIENCE-AND-DEVELOPMENT" },
	{ 0x285132, "Shenzhen Prayfly Technology Co.,Ltd", "SHENZHEN-PRAYFLY" },
	{ 0x2852E0, "Layon international Electronic & Telecom Co.,Ltd", "LAYON-TELECOM" },
	{ 0x285767, "Echostar Technologies Corp", "ECHOSTAR-TECHNOLOGIES" },
	{ 0x285AEB, "Apple", "APPLE" },
	{ 0x285FDB, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x286046, "Lantech Communications Global, Inc.", "LANTECH-COMMUNICATION-GLOBAL" },
	{ 0x286094, "CAPELEC", "CAPELEC" },
	{ 0x286336, "Siemens AG - Industrial Automation - EWA", "SIEMENS---INDUSTRIAL-AUTOMATION---EWA" },
	{ 0x28656B, "Keystone Microtech Corporation", "KEYSTONE-MICROTECH" },
	{ 0x286AB8, "Apple", "APPLE" },
	{ 0x286ABA, "Apple", "APPLE" },
	{ 0x286D97, "SAMJIN Co., Ltd.", "SAMJIN" },
	{ 0x286ED4, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x287184, "Spire Payments", "SPIRE-PAYMENTS" },
	{ 0x2872C5, "Smartmatic Corp", "SMARTMATIC" },
	{ 0x2872F0, "ATHENA", "ATHENA" },
	{ 0x287610, "IgniteNet", "IGNITENET" },
	{ 0x287994, "Realplay Digital Technology(Shenzhen) Co.,Ltd", "REALPLAY-DIGITAL-SHENZHEN" },
	{ 0x288023, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x2884FA, "SHARP Corporation", "SHARP" },
	{ 0x28852D, "Touch Networks", "TOUCH-NETWORKS" },
	{ 0x288915, "CashGuard Sverige AB", "CASHGUARD-SVERIGE-AB" },
	{ 0x288A1C, "Juniper networks", "JUNIPER-NETWORKS" },
	{ 0x2891D0, "Stage Tec Entwicklungsgesellschaft für professionelle Audiotechnik mbH", "STAGE-TEC-ENTWICKLUNGSGESELLSCHAFT-FüR-PROFESSIONELLE-AUDIOTECHNIK-MBH" },
	{ 0x28924A, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x2893FE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x28940F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x2894AF, "Samhwa Telecom", "SAMHWA-TELECOM" },
	{ 0x28987B, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x289A4B, "SteelSeries ApS", "STEELSERIES-APS" },
	{ 0x289AFA, "TCT Mobile Limited", "TCT-MOBILE" },
	{ 0x289EDF, "Danfoss Turbocor Compressors, Inc", "DANFOSS-TURBOCOR-COMPRESSORS" },
	{ 0x28A186, "enblink", "ENBLINK" },
	{ 0x28A192, "GERP Solution", "GERP-SOLUTION" },
	{ 0x28A1EB, "ETEK TECHNOLOGY (SHENZHEN) CO.,LTD", "ETEK-SHENZHEN" },
	{ 0x28A241, "exlar corp", "EXLAR" },
	{ 0x28A574, "Miller Electric Mfg. Co.", "MILLER-ELECTRIC-MFG" },
	{ 0x28A5EE, "Shenzhen SDGI CATV Co., Ltd", "SHENZHEN-SDGI-CATV" },
	{ 0x28AF0A, "Sirius XM Radio Inc", "SIRIUS-XM-RADIO" },
	{ 0x28B0CC, "Xenya d.o.o.", "XENYA-D-O-O" },
	{ 0x28B2BD, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x28B3AB, "Genmark Automation", "GENMARK-AUTOMATION" },
	{ 0x28BA18, "NextNav, LLC", "NEXTNAV-LLC" },
	{ 0x28BAB5, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x28BB59, "RNET Technologies, Inc.", "RNET-TECHNOLOGIES" },
	{ 0x28BE9B, "Technicolor USA Inc.", "TECHNICOLOR-USA" },
	{ 0x28C0DA, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x28C2DD, "AzureWave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x28C671, "Yota Devices OY", "YOTA-DEVICES-OY" },
	{ 0x28C68E, "NETGEAR INC.,", "NETGEAR" },
	{ 0x28C718, "Altierre", "ALTIERRE" },
	{ 0x28C7CE, "Cisco", "CISCO" },
	{ 0x28C825, "DellKing Industrial Co., Ltd", "DELLKING-INDUSTRIAL" },
	{ 0x28C914, "Taimag Corporation", "TAIMAG" },
	{ 0x28CBEB, "One", "ONE" },
	{ 0x28CC01, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x28CCFF, "Corporacion Empresarial Altra SL", "CORPORACION-EMPRESARIAL-ALTRA-SL" },
	{ 0x28CD1C, "Espotel Oy", "ESPOTEL-OY" },
	{ 0x28CD4C, "Individual Computers GmbH", "INDIVIDUAL-COMPUTERS" },
	{ 0x28CD9C, "Shenzhen Dynamax Software Development Co.,Ltd.", "SHENZHEN-DYNAMAX-SOFTWARE-DEVELOPMENT" },
	{ 0x28CFDA, "Apple", "APPLE" },
	{ 0x28CFE9, "Apple", "APPLE" },
	{ 0x28D1AF, "Nokia Corporation", "NOKIA" },
	{ 0x28D244, "LCFC(HeFei) Electronics Technology Co., Ltd.", "LCFC-HEFEI-ELECTRONICS" },
	{ 0x28D576, "Premier Wireless, Inc.", "PREMIER-WIRELESS" },
	{ 0x28D93E, "Telecor Inc.", "TELECOR" },
	{ 0x28D98A, "Hangzhou Konke Technology Co.,Ltd.", "HANGZHOU-KONKE" },
	{ 0x28D997, "Yuduan Mobile Co., Ltd.", "YUDUAN-MOBILE" },
	{ 0x28DB81, "Shanghai Guao Electronic Technology Co., Ltd", "SHANGHAI-GUAO" },
	{ 0x28DEF6, "bioMerieux Inc.", "BIOMERIEUX" },
	{ 0x28E02C, "Apple", "APPLE" },
	{ 0x28E14C, "Apple, Inc.", "APPLE" },
	{ 0x28E297, "Shanghai InfoTM Microelectronics Co.,Ltd.", "SHANGHAI-INFOTM-MICROELECTRONICS" },
	{ 0x28E31F, "Xiaomi inc.", "XIAOMI" },
	{ 0x28E347, "Liteon Technology Corporation", "LITEON" },
	{ 0x28E476, "Pi-Coral", "PI-CORAL" },
	{ 0x28E608, "Tokheim", "TOKHEIM" },
	{ 0x28E6E9, "SIS Sat Internet Services GmbH", "SIS-SAT-INTERNET-SERVICES" },
	{ 0x28E794, "Microtime Computer Inc.", "MICROTIME" },
	{ 0x28E7CF, "Apple", "APPLE" },
	{ 0x28ED58, "JAG Jakob AG", "JAG-JAKOB" },
	{ 0x28EE2C, "Frontline Test Equipment", "FRONTLINE-TEST-EQUIPMENT" },
	{ 0x28EF01, "PRIVATE", "PRIVATE" },
	{ 0x28F358, "2C - Trifonov & Co", "2C---TRIFONOV" },
	{ 0x28F532, "ADD-Engineering BV", "ADD-ENGINEERING" },
	{ 0x28F606, "Syes srl", "SYES-SRL" },
	{ 0x28FBD3, "Ragentek Technology Group", "RAGENTEK-GROUP" },
	{ 0x28FC51, "The Electric Controller and Manufacturing Co., LLC", "THE-ELECTRIC-CONTROLLER-AND-MANUFACTURING-LLC" },
	{ 0x28FCF6, "Shenzhen Xin KingBrand enterprises Co.,Ltd", "SHENZHEN-XIN-KINGBRAND-ENTERPRISES" },
	{ 0x28FD80, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x2C002C, "UNOWHY", "UNOWHY" },
	{ 0x2C0033, "EControls, LLC", "ECONTROLS-LLC" },
	{ 0x2C00F7, "XOS", "XOS" },
	{ 0x2C010B, "NASCENT Technology, LLC - RemKon", "NASCENT-LLC---REMKON" },
	{ 0x2C0623, "Win Leader Inc.", "WIN-LEADER" },
	{ 0x2C073C, "DEVLINE LIMITED", "DEVLINE" },
	{ 0x2C10C1, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x2C18AE, "Trend Electronics Co., Ltd.", "TREND-ELECTRONICS" },
	{ 0x2C1984, "IDN Telecom, Inc.", "IDN-TELECOM" },
	{ 0x2C1A31, "Electronics Company Limited", "ELECTRONICS-COMPANY" },
	{ 0x2C1EEA, "AERODEV", "AERODEV" },
	{ 0x2C1F23, "Apple", "APPLE" },
	{ 0x2C2172, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x2C245F, "Babolat VS", "BABOLAT-VS" },
	{ 0x2C265F, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x2C26C5, "zte corporation", "ZTE" },
	{ 0x2C27D7, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x2C282D, "BBK COMMUNICATIAO TECHNOLOGY CO.,LTD.", "BBK-COMMUNICATIAO" },
	{ 0x2C2997, "Microsoft Corporation", "MICROSOFT" },
	{ 0x2C2D48, "bct electronic GesmbH", "BCT-GESMBH" },
	{ 0x2C3068, "Pantech Co.,Ltd", "PANTECH" },
	{ 0x2C337A, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x2C3427, "ERCO & GENER", "ERCO-GENER" },
	{ 0x2C3557, "ELLIY Power CO..Ltd", "ELLIY-POWER" },
	{ 0x2C36A0, "Capisco Limited", "CAPISCO" },
	{ 0x2C36F8, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x2C3731, "ShenZhen Yifang Digital Technology Co.,LTD", "SHENZHEN-YIFANG-DIGITAL" },
	{ 0x2C3796, "CYBO CO.,LTD.", "CYBO" },
	{ 0x2C3996, "SAGEMCOM", "SAGEMCOM" },
	{ 0x2C39C1, "Ciena Corporation", "CIENA" },
	{ 0x2C3A28, "Fagor Electrónica", "FAGOR-ELECTRóNICA" },
	{ 0x2C3BFD, "Netstor Technology Co., Ltd.", "NETSTOR" },
	{ 0x2C3ECF, "Cisco", "CISCO" },
	{ 0x2C3F38, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x2C3F3E, "Alge-Timing GmbH", "ALGE-TIMING" },
	{ 0x2C4138, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x2C4401, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x2C441B, "Spectrum Medical Limited", "SPECTRUM-MEDICAL" },
	{ 0x2C44FD, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x2C5089, "Shenzhen Kaixuan Visual Technology Co.,Limited", "SHENZHEN-KAIXUAN-VISUAL" },
	{ 0x2C534A, "Shenzhen Winyao Electronic Limited", "SHENZHEN-WINYAO" },
	{ 0x2C542D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x2C54CF, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x2C553C, "Gainspeed, Inc.", "GAINSPEED" },
	{ 0x2C59E5, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x2C5A05, "Nokia Corporation", "NOKIA" },
	{ 0x2C5AA3, "PROMATE ELECTRONIC CO.LTD", "PROMATE" },
	{ 0x2C5BE1, "Centripetal Networks, Inc", "CENTRIPETAL-NETWORKS" },
	{ 0x2C5D93, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x2C5FF3, "Pertronic Industries", "PERTRONIC-INDUSTRIES" },
	{ 0x2C600C, "QUANTA COMPUTER INC.", "QUANTA" },
	{ 0x2C625A, "Finest Security Systems Co., Ltd", "FINEST-SECURITY" },
	{ 0x2C6289, "Regenersis (Glenrothes) Ltd", "REGENERSIS-GLENROTHES" },
	{ 0x2C67FB, "ShenZhen Zhengjili Electronics Co., LTD", "SHENZHEN-ZHENGJILI-ELECTRONICS" },
	{ 0x2C69BA, "RF Controls, LLC", "RF-CONTROLS-LLC" },
	{ 0x2C6A6F, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x2C6BF5, "Juniper networks", "JUNIPER-NETWORKS" },
	{ 0x2C7155, "HiveMotion", "HIVEMOTION" },
	{ 0x2C72C3, "Soundmatters", "SOUNDMATTERS" },
	{ 0x2C750F, "Shanghai Dongzhou-Lawton Communication Technology Co. Ltd.", "SHANGHAI-DONGZHOU-LAWTON-COMMUNICATION" },
	{ 0x2C768A, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x2C7B5A, "Milper Ltd", "MILPER" },
	{ 0x2C7B84, "OOO Petr Telegin", "OOO-PETR-TELEGIN" },
	{ 0x2C7ECF, "Onzo Ltd", "ONZO" },
	{ 0x2C8065, "HARTING Inc. of North America", "HARTING-OF-NORTH-AMERICA" },
	{ 0x2C8158, "Hon Hai Precision Ind. Co.,Ltd", "HON-HAI-PRECISION-IND" },
	{ 0x2C8A72, "HTC Corporation", "HTC" },
	{ 0x2C8BF2, "Hitachi Metals America Ltd", "HITACHI-METALS-AMERICA" },
	{ 0x2C9127, "Eintechno Corporation", "EINTECHNO" },
	{ 0x2C922C, "Kishu Giken Kogyou Company Ltd,.", "KISHU-GIKEN-KOGYOU-COMPANY" },
	{ 0x2C9464, "Cincoze Co., Ltd.", "CINCOZE" },
	{ 0x2C957F, "zte corporation", "ZTE" },
	{ 0x2C9717, "I.C.Y. B.V.", "I-C-Y" },
	{ 0x2C9AA4, "NGI SpA", "NGI-SPA" },
	{ 0x2C9E5F, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x2C9EFC, "CANON INC.", "CANON" },
	{ 0x2CA157, "acromate, Inc.", "ACROMATE" },
	{ 0x2CA2B4, "Fortify Technologies, LLC", "FORTIFY-TECHNOLOGIES-LLC" },
	{ 0x2CA30E, "POWER DRAGON DEVELOPMENT LIMITED", "POWER-DRAGON-DEVELOPMENT" },
	{ 0x2CA780, "True Technologies Inc.", "TRUE-TECHNOLOGIES" },
	{ 0x2CA835, "RIM", "RIM" },
	{ 0x2CAB25, "Shenzhen Gongjin Electronics Co.,Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0x2CABA4, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x2CAD13, "SHENZHEN ZHILU TECHNOLOGY CO.,LTD", "SHENZHEN-ZHILU" },
	{ 0x2CB05D, "NETGEAR", "NETGEAR" },
	{ 0x2CB0DF, "Soliton Technologies Pvt Ltd", "SOLITON-TECHNOLOGIES-PVT" },
	{ 0x2CB43A, "Apple", "APPLE" },
	{ 0x2CB693, "Radware", "RADWARE" },
	{ 0x2CB69D, "RED Digital Cinema", "RED-DIGITAL-CINEMA" },
	{ 0x2CBE08, "Apple", "APPLE" },
	{ 0x2CBE97, "Ingenieurbuero Bickele und Buehler GmbH", "INGENIEURBUERO-BICKELE-UND-BUEHLER" },
	{ 0x2CC260, "Ravello Systems", "RAVELLO" },
	{ 0x2CCC15, "Nokia Corporation", "NOKIA" },
	{ 0x2CCD27, "Precor Inc", "PRECOR" },
	{ 0x2CCD43, "Summit Technology Group", "SUMMIT-GROUP" },
	{ 0x2CCD69, "Aqavi.com", "AQAVI-COM" },
	{ 0x2CD05A, "Liteon Technology Corporation", "LITEON" },
	{ 0x2CD141, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x2CD1DA, "Sanjole, Inc.", "SANJOLE" },
	{ 0x2CD2E7, "Nokia Corporation", "NOKIA" },
	{ 0x2CD444, "Fujitsu Limited", "FUJITSU" },
	{ 0x2CDD0C, "Discovergy GmbH", "DISCOVERGY" },
	{ 0x2CE2A8, "DeviceDesign", "DEVICEDESIGN" },
	{ 0x2CE412, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0x2CE6CC, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x2CE871, "Alert Metalguard ApS", "ALERT-METALGUARD-APS" },
	{ 0x2CEDEB, "Alpheus Digital Company Limited", "ALPHEUS-DIGITAL-COMPANY" },
	{ 0x2CEE26, "Petroleum Geo-Services", "PETROLEUM-GEO-SERVICES" },
	{ 0x2CF0EE, "Apple", "APPLE" },
	{ 0x2CF203, "EMKO ELEKTRONIK SAN VE TIC AS", "EMKO-ELEKTRONIK-SAN-VE-TIC-AS" },
	{ 0x2CF4C5, "Avaya, Inc", "AVAYA" },
	{ 0x2CF7F1, "Seeed Technology Inc.", "SEEED" },
	{ 0x2CFAA2, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x30055C, "Brother industries, LTD.", "BROTHER-INDUSTRIES" },
	{ 0x300B9C, "Delta Mobile Systems, Inc.", "DELTA-MOBILE" },
	{ 0x300D2A, "Zhejiang Wellcom Technology Co.,Ltd.", "ZHEJIANG-WELLCOM" },
	{ 0x300ED5, "Hon Hai Precision Ind.Co.Ltd", "HON-HAI-PRECISION-IND" },
	{ 0x300EE3, "Aquantia Corporation", "AQUANTIA" },
	{ 0x3010B3, "Liteon Technology Corporation", "LITEON" },
	{ 0x3010E4, "Apple, Inc.", "APPLE" },
	{ 0x30142D, "Piciorgros GmbH", "PICIORGROS" },
	{ 0x30144A, "Wistron Neweb Corp.", "WISTRON-NEWEB" },
	{ 0x301518, "Ubiquitous Communication Co. ltd.", "UBIQUITOUS-COMMUNICATION" },
	{ 0x30168D, "ProLon", "PROLON" },
	{ 0x3017C8, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x3018CF, "DEOS control systems GmbH", "DEOS-CONTROL" },
	{ 0x301966, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x301A28, "Mako Networks Ltd", "MAKO-NETWORKS" },
	{ 0x30215B, "Shenzhen Ostar Display Electronic Co.,Ltd", "SHENZHEN-OSTAR-DISPLAY" },
	{ 0x302DE8, "JDA, LLC (JDA Systems)", "JDA-LLC-JDA" },
	{ 0x303294, "W-IE-NE-R Plein & Baus GmbH", "W-IE-NE-R-PLEIN-BAUS" },
	{ 0x3032D4, "Hanilstm Co., Ltd.", "HANILSTM" },
	{ 0x303335, "Boosty", "BOOSTY" },
	{ 0x3037A6, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x303855, "Nokia Corporation", "NOKIA" },
	{ 0x303926, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x303955, "Shenzhen Jinhengjia Electronic Co., Ltd.", "SHENZHEN-JINHENGJIA" },
	{ 0x3039F2, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x303A64, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x303D08, "GLINTT TES S.A.", "GLINTT-TES" },
	{ 0x303EAD, "Sonavox Canada Inc", "SONAVOX-CANADA" },
	{ 0x304174, "ALTEC LANSING LLC", "ALTEC-LANSING-LLC" },
	{ 0x304225, "BURG-WÄCHTER KG", "BURG-WÄCHTER" },
	{ 0x304449, "PLATH GmbH", "PLATH" },
	{ 0x30469A, "NETGEAR", "NETGEAR" },
	{ 0x30493B, "Nanjing Z-Com Wireless Co.,Ltd", "NANJING-Z-COM-WIRELESS" },
	{ 0x304C7E, "Panasonic Electric Works Automation Controls Techno Co.,Ltd.", "PANASONIC-ELECTRIC-WORKS-AUTOMATION-CONTROLS-TECHNO" },
	{ 0x304EC3, "Tianjin Techua Technology Co., Ltd.", "TIANJIN-TECHUA" },
	{ 0x3051F8, "BYK-Gardner GmbH", "BYK-GARDNER" },
	{ 0x30525A, "NST Co., LTD", "NST" },
	{ 0x3055ED, "Trex Network LLC", "TREX-NETWORK-LLC" },
	{ 0x3057AC, "IRLAB LTD.", "IRLAB" },
	{ 0x30595B, "streamnow AG", "STREAMNOW" },
	{ 0x3059B7, "Microsoft", "MICROSOFT" },
	{ 0x305D38, "Beissbarth", "BEISSBARTH" },
	{ 0x306023, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x306112, "PAV GmbH", "PAV" },
	{ 0x306118, "Paradom Inc.", "PARADOM" },
	{ 0x3065EC, "Wistron (ChongQing)", "WISTRON-CHONGQING" },
	{ 0x30688C, "Reach Technology Inc.", "REACH" },
	{ 0x30694B, "RIM", "RIM" },
	{ 0x306CBE, "Skymotion Technology (HK) Limited", "SKYMOTION-HK" },
	{ 0x306E5C, "Validus Technologies", "VALIDUS-TECHNOLOGIES" },
	{ 0x3071B2, "Hangzhou Prevail Optoelectronic Equipment Co.,LTD.", "HANGZHOU-PREVAIL-OPTOELECTRONIC-EQUIPMENT" },
	{ 0x307350, "Inpeco SA", "INPECO" },
	{ 0x307512, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x30766F, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x3077CB, "Maike Industry(Shenzhen)CO.,LTD", "MAIKE-INDUSTRY-SHENZHEN" },
	{ 0x30786B, "TIANJIN Golden Pentagon Electronics Co., Ltd.", "TIANJIN-GOLDEN-PENTAGON-ELECTRONICS" },
	{ 0x3078C2, "Innowireless, Co. Ltd.", "INNOWIRELESS" },
	{ 0x307C30, "RIM", "RIM" },
	{ 0x307ECB, "SFR", "SFR" },
	{ 0x3085A9, "Asustek Computer Inc", "ASUSTEK" },
	{ 0x308730, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x308999, "Guangdong East Power Co.,", "GUANGDONG-EAST-POWER" },
	{ 0x308CFB, "Dropcam", "DROPCAM" },
	{ 0x3090AB, "Apple", "APPLE" },
	{ 0x30918F, "Technicolor", "TECHNICOLOR" },
	{ 0x3092F6, "SHANGHAI SUNMON COMMUNICATION TECHNOGY CO.,LTD", "SHANGHAI-SUNMON-COMMUNICATION-TECHNOGY" },
	{ 0x309BAD, "BBK Electronics Corp., Ltd.,", "BBK-ELECTRONICS" },
	{ 0x30A8DB, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x30AABD, "Shanghai Reallytek Information Technology Co.,Ltd", "SHANGHAI-REALLYTEK-INFORMATION" },
	{ 0x30AE7B, "Deqing Dusun Electron CO., LTD", "DEQING-DUSUN-ELECTRON" },
	{ 0x30AEF6, "Radio Mobile Access", "RADIO-MOBILE-ACCESS" },
	{ 0x30B216, "Hytec Geraetebau GmbH", "HYTEC-GERAETEBAU" },
	{ 0x30B3A2, "Shenzhen Heguang Measurement & Control Technology Co.,Ltd", "SHENZHEN-HEGUANG-MEASUREMENT-CONTROL" },
	{ 0x30B5C2, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x30B5F1, "Aitexin Technology Co., Ltd", "AITEXIN" },
	{ 0x30C750, "MIC Technology Group", "MIC-GROUP" },
	{ 0x30C7AE, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x30C82A, "Wi-Next s.r.l.", "WI-NEXT-S-R-L" },
	{ 0x30CDA7, "Samsung Electronics ITS, Printer division", "SAMSUNG-ELECTRONICS-ITS-PRINTER-DIVISION" },
	{ 0x30D17E, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x30D357, "Logosol, Inc.", "LOGOSOL" },
	{ 0x30D46A, "Autosales Incorporated", "AUTOSALES-INCORPORATED" },
	{ 0x30D587, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x30D6C9, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x30DE86, "Cedac Software S.r.l.", "CEDAC-SOFTWARE-S-R-L" },
	{ 0x30E48E, "Vodafone UK", "VODAFONE-UK" },
	{ 0x30E4DB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x30EB25, "INTEK DIGITAL", "INTEK-DIGITAL" },
	{ 0x30EFD1, "Alstom Strongwish (Shenzhen) Co., Ltd.", "ALSTOM-STRONGWISH-SHENZHEN" },
	{ 0x30F31D, "zte corporation", "ZTE" },
	{ 0x30F335, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x30F33A, "+plugg srl", "+PLUGG-SRL" },
	{ 0x30F42F, "ESP", "ESP" },
	{ 0x30F70D, "Cisco Systems", "CISCO" },
	{ 0x30F7C5, "Apple", "APPLE" },
	{ 0x30F7D7, "Thread Technology Co., Ltd", "THREAD" },
	{ 0x30F9ED, "Sony Corporation", "SONY" },
	{ 0x30FAB7, "Tunai Creative", "TUNAI-CREATIVE" },
	{ 0x30FD11, "MACROTECH (USA) INC.", "MACROTECH-USA" },
	{ 0x3400A3, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x340286, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x34029B, "CloudBerry Technologies Private Limited", "CLOUDBERRY-TECHNOLOGIES-PRIVATE" },
	{ 0x3407FB, "Ericsson AB", "ERICSSON-AB" },
	{ 0x340804, "D-Link Corporation", "D-LINK" },
	{ 0x340A22, "TOP-ACCESS ELECTRONICS CO LTD", "TOP-ACCESS-ELECTRONICS" },
	{ 0x340AFF, "Qingdao Hisense Communications Co.,Ltd", "QINGDAO-HISENSE-COMMUNICATION" },
	{ 0x340B40, "MIOS ELETTRONICA SRL", "MIOS-ELETTRONICA-SRL" },
	{ 0x341298, "Apple", "APPLE" },
	{ 0x3413A8, "Mediplan Limited", "MEDIPLAN" },
	{ 0x3413E8, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x34159E, "Apple", "APPLE" },
	{ 0x3417EB, "Dell Inc", "DELL" },
	{ 0x341A4C, "SHENZHEN WEIBU ELECTRONICS CO.,LTD.", "SHENZHEN-WEIBU-ELECTRONICS" },
	{ 0x341B22, "Grandbeing Technology Co., Ltd", "GRANDBEING" },
	{ 0x342109, "Jensen Scandinavia AS", "JENSEN-SCANDINAVIA-AS" },
	{ 0x342387, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x3423BA, "Samsung Electro Mechanics co.,LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x34255D, "Shenzhen Loadcom Technology Co.,Ltd", "SHENZHEN-LOADCOM" },
	{ 0x3428F0, "ATN International Limited", "ATN" },
	{ 0x3429EA, "MCD ELECTRONICS SP. Z O.O.", "MCD-ELECTRONICS-SP-Z-O-O" },
	{ 0x342F6E, "Anywire corporation", "ANYWIRE" },
	{ 0x343111, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x3431C4, "AVM GmbH", "AVM" },
	{ 0x34363B, "Apple", "APPLE" },
	{ 0x3438AF, "Inlab Software GmbH", "INLAB-SOFTWARE" },
	{ 0x343D98, "Fujian JinQianMao Electronic Technology Co.,Ltd.", "FUJIAN-JINQIANMAO" },
	{ 0x3440B5, "IBM", "IBM" },
	{ 0x34466F, "HiTEM Engineering", "HITEM-ENGINEERING" },
	{ 0x344B3D, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0x344B50, "ZTE Corporation", "ZTE" },
	{ 0x344CA4, "amazipoint technology Ltd.", "AMAZIPOINT" },
	{ 0x344DEA, "zte corporation", "ZTE" },
	{ 0x344DF7, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x344F3F, "IO-Power Technology Co., Ltd.", "IO-POWER" },
	{ 0x344F5C, "R&amp;M AG", "R-AMP-M" },
	{ 0x344F69, "EKINOPS SAS", "EKINOPS-SAS" },
	{ 0x3451AA, "JID GLOBAL", "JID-GLOBAL" },
	{ 0x3451C9, "Apple", "APPLE" },
	{ 0x345B11, "EVI HEAT AB", "EVI-HEAT-AB" },
	{ 0x345C40, "Cargt Holdings LLC", "CARGT-HOLDINGS-LLC" },
	{ 0x345D10, "Wytek", "WYTEK" },
	{ 0x346178, "The Boeing Company", "THE-BOEING-COMPANY" },
	{ 0x346288, "Cisco", "CISCO" },
	{ 0x3464A9, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x34684A, "Teraworks Co., Ltd.", "TERAWORKS" },
	{ 0x346895, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x346BD3, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x346C0F, "Pramod Telecom Pvt. Ltd", "PRAMOD-TELECOM-PVT" },
	{ 0x346E8A, "Ecosense", "ECOSENSE" },
	{ 0x346F90, "Cisco", "CISCO" },
	{ 0x346F92, "White Rodgers Division", "WHITE-RODGERS-DIVISION" },
	{ 0x3475C7, "Avaya, Inc", "AVAYA" },
	{ 0x3476C5, "I-O DATA DEVICE, INC.", "I-O-DATA-DEVICE" },
	{ 0x347877, "O-NET Communications(Shenzhen) Limited", "O-NET-COMMUNICATION-SHENZHEN" },
	{ 0x347E39, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x348137, "UNICARD SA", "UNICARD" },
	{ 0x3481C4, "AVM GmbH", "AVM" },
	{ 0x3482DE, "Kayo Technology, Inc.", "KAYO" },
	{ 0x348302, "iFORCOM Co., Ltd", "IFORCOM" },
	{ 0x348446, "Ericsson AB", "ERICSSON-AB" },
	{ 0x34862A, "Heinz Lackmann GmbH & Co KG", "HEINZ-LACKMANN" },
	{ 0x34873D, "Quectel Wireless Solution Co.,Ltd.", "QUECTEL-WIRELESS-SOLUTION" },
	{ 0x34885D, "Logitech Far East", "LOGITECH-FAR-EAST" },
	{ 0x348AAE, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0x3495DB, "Logitec Corporation", "LOGITEC" },
	{ 0x3497FB, "ADVANCED RF TECHNOLOGIES INC", "ADVANCED-RF-TECHNOLOGIES" },
	{ 0x34996F, "VPI Engineering", "VPI-ENGINEERING" },
	{ 0x3499D7, "Universal Flow Monitors, Inc.", "UNIVERSAL-FLOW-MONITORS" },
	{ 0x349A0D, "ZBD Displays Ltd", "ZBD-DISPLAYS" },
	{ 0x349D90, "Heinzmann GmbH & CO. KG", "HEINZMANN" },
	{ 0x349E34, "Evervictory Electronic Co.Ltd", "EVERVICTORY" },
	{ 0x34A183, "AWare, Inc", "AWARE" },
	{ 0x34A395, "Apple", "APPLE" },
	{ 0x34A3BF, "Terewave. Inc.", "TEREWAVE" },
	{ 0x34A55D, "TECHNOSOFT INTERNATIONAL SRL", "TECHNOSOFT-SRL" },
	{ 0x34A5E1, "Sensorist ApS", "SENSORIST-APS" },
	{ 0x34A68C, "Shine Profit Development Limited", "SHINE-PROFIT-DEVELOPMENT" },
	{ 0x34A709, "Trevil srl", "TREVIL-SRL" },
	{ 0x34A7BA, "Fischer International Systems Corporation", "FISCHER" },
	{ 0x34A843, "KYOCERA Display Corporation", "KYOCERA-DISPLAY" },
	{ 0x34A84E, "Cisco", "CISCO" },
	{ 0x34AA8B, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x34AA99, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x34AAEE, "Mikrovisatos Servisas UAB", "MIKROVISATOS-SERVISAS-UAB" },
	{ 0x34ADE4, "Shanghai Chint Power Systems Co., Ltd.", "SHANGHAI-CHINT-POWER" },
	{ 0x34AF2C, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x34B1F7, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x34B571, "PLDS", "PLDS" },
	{ 0x34B7FD, "Guangzhou Younghead Electronic Technology Co.,Ltd", "GUANGZHOU-YOUNGHEAD" },
	{ 0x34BA51, "Se-Kure Controls, Inc.", "SE-KURE-CONTROLS" },
	{ 0x34BA9A, "Asiatelco Technologies Co.", "ASIATELCO-TECHNOLOGIES" },
	{ 0x34BB1F, "Research In Motion", "RESEARCH-IN-MOTION" },
	{ 0x34BB26, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0x34BCA6, "Beijing Ding Qing Technology, Ltd.", "BEIJING-DING-QING" },
	{ 0x34BDC8, "Cisco Systems", "CISCO" },
	{ 0x34BDF9, "Shanghai WDK Industrial Co.,Ltd.", "SHANGHAI-WDK-INDUSTRIAL" },
	{ 0x34BDFA, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x34BE00, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x34BF90, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0x34C059, "Apple", "APPLE" },
	{ 0x34C3AC, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x34C5D0, "Hagleitner Hygiene International GmbH", "HAGLEITNER-HYGIENE" },
	{ 0x34C69A, "Enecsys Ltd", "ENECSYS" },
	{ 0x34C731, "ALPS Co,. Ltd.", "ALPS" },
	{ 0x34C803, "Nokia Corporation", "NOKIA" },
	{ 0x34C99D, "EIDOLON COMMUNICATIONS TECHNOLOGY CO. LTD.", "EIDOLON-COMMUNICATION" },
	{ 0x34CC28, "Nexpring Co. LTD.,", "NEXPRING" },
	{ 0x34CD6D, "CommSky Technologies", "COMMSKY-TECHNOLOGIES" },
	{ 0x34CDBE, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x34CE94, "Parsec (Pty) Ltd", "PARSEC-PTY" },
	{ 0x34D09B, "MobilMAX Technology Inc.", "MOBILMAX" },
	{ 0x34D2C4, "RENA GmbH Print Systeme", "RENA-PRINT-SYSTEME" },
	{ 0x34D7B4, "Tributary Systems, Inc.", "TRIBUTARY" },
	{ 0x34DBFD, "Cisco", "CISCO" },
	{ 0x34DE1A, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x34DE34, "zte corporation", "ZTE" },
	{ 0x34DF2A, "Fujikon Industrial Co.,Limited", "FUJIKON-INDUSTRIAL" },
	{ 0x34E0CF, "zte corporation", "ZTE" },
	{ 0x34E0D7, "DONGGUAN QISHENG ELECTRONICS INDUSTRIAL CO., LTD", "DONGGUAN-QISHENG-ELECTRONICS-INDUSTRIAL" },
	{ 0x34E2FD, "Apple", "APPLE" },
	{ 0x34E42A, "Automatic Bar Controls Inc.", "AUTOMATIC-BAR-CONTROLS" },
	{ 0x34E6AD, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x34E6D7, "Dell Inc.", "DELL" },
	{ 0x34EF44, "2Wire", "2WIRE" },
	{ 0x34EF8B, "NTT Communications Corporation", "NTT-COMMUNICATION" },
	{ 0x34F0CA, "Shenzhen Linghangyuan Digital Technology Co.,Ltd.", "SHENZHEN-LINGHANGYUAN-DIGITAL" },
	{ 0x34F39B, "WizLAN Ltd.", "WIZLAN" },
	{ 0x34F62D, "SHARP Corporation", "SHARP" },
	{ 0x34F6D2, "Panasonic Taiwan Co.,Ltd.", "PANASONIC-TAIWAN" },
	{ 0x34F968, "ATEK Products, LLC", "ATEK-PRODUCTS-LLC" },
	{ 0x34FA40, "Guangzhou Robustel Technologies Co., Limited", "GUANGZHOU-ROBUSTEL-TECHNOLOGIES" },
	{ 0x34FC6F, "ALCEA", "ALCEA" },
	{ 0x34FCEF, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x380197, "Toshiba Samsung Storage Technolgoy Korea Corporation", "TOSHIBA-SAMSUNG-STORAGE-TECHNOLGOY-KOREA" },
	{ 0x3806B4, "A.D.C. GmbH", "A-D-C" },
	{ 0x3808FD, "Silca Spa", "SILCA-SPA" },
	{ 0x3809A4, "Firefly Integrations", "FIREFLY-INTEGRATIONS" },
	{ 0x380A0A, "Sky-City Communication and Electronics Limited Company", "SKY-CITY-COMMUNICATION-AND-ELECTRONICS-COMPANY" },
	{ 0x380A94, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x380B40, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x380DD4, "Primax Electronics LTD.", "PRIMAX-ELECTRONICS" },
	{ 0x380E7B, "V.P.S. Thai Co., Ltd", "V-P-S-THAI" },
	{ 0x380F4A, "Apple", "APPLE" },
	{ 0x380FE4, "Dedicated Network Partners Oy", "DEDICATED-NETWORK-PARTNERS-OY" },
	{ 0x3816D1, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x381766, "PROMZAKAZ LTD.", "PROMZAKAZ" },
	{ 0x38192F, "Nokia Corporation", "NOKIA" },
	{ 0x381C1A, "Cisco", "CISCO" },
	{ 0x381C4A, "SIMCom Wireless Solutions Co.,Ltd.", "SIMCOM-WIRELESS-SOLUTIONS" },
	{ 0x38229D, "Pirelli Tyre S.p.A.", "PIRELLI-TYRE-S-P-A" },
	{ 0x3822D6, "H3C Technologies Co., Limited", "H3C-TECHNOLOGIES" },
	{ 0x38262B, "UTran Technology", "UTRAN" },
	{ 0x3826CD, "ANDTEK", "ANDTEK" },
	{ 0x3828EA, "Fujian Netcom Technology Co., LTD", "FUJIAN-NETCOM" },
	{ 0x382C4A, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x382DD1, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x3831AC, "WEG", "WEG" },
	{ 0x383BC8, "2wire", "2WIRE" },
	{ 0x383F10, "DBL Technology Ltd.", "DBL" },
	{ 0x384233, "Wildeboer Bauteile GmbH", "WILDEBOER-BAUTEILE" },
	{ 0x3842A6, "Ingenieurbuero Stahlkopf", "INGENIEURBUERO-STAHLKOPF" },
	{ 0x384369, "Patrol Products Consortium LLC", "PATROL-PRODUCTS-CONSORTIUM-LLC" },
	{ 0x38458C, "MyCloud Technology corporation", "MYCLOUD" },
	{ 0x384608, "ZTE Corporation", "ZTE" },
	{ 0x38484C, "Apple", "APPLE" },
	{ 0x384B76, "AIRTAME ApS", "AIRTAME-APS" },
	{ 0x384FF0, "Azurewave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x38521A, "Alcatel-Lucent 7705", "ALCATEL-LUCENT-7705" },
	{ 0x38580C, "Panaccess Systems GmbH", "PANACCESS" },
	{ 0x3859F8, "MindMade sp. z o.o.", "MINDMADE-SP-Z-O-O" },
	{ 0x3859F9, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x385AA8, "Beijing Zhongdun Security Technology Development Co.", "BEIJING-ZHONGDUN-SECURITY-DEVELOPMENT" },
	{ 0x385FC3, "Yu Jeong System, Co.Ltd", "YU-JEONG-SYSTEM" },
	{ 0x386077, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x3863BB, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x3863F6, "3NOD MULTIMEDIA(SHENZHEN)CO.,LTD", "3NOD-MULTIMEDIA-SHENZHEN" },
	{ 0x386645, "OOSIC Technology CO.,Ltd", "OOSIC" },
	{ 0x386793, "Asia Optical Co., Inc.", "ASIA-OPTICAL" },
	{ 0x386BBB, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x386C9B, "Ivy Biomedical", "IVY-BIOMEDICAL" },
	{ 0x386E21, "Wasion Group Ltd.", "WASION-GROUP" },
	{ 0x3872C0, "COMTREND", "COMTREND" },
	{ 0x387B47, "AKELA, Inc.", "AKELA" },
	{ 0x388345, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x3889DC, "Opticon Sensors Europe B.V.", "OPTICON-SENSORS-EUROPE" },
	{ 0x388AB7, "ITC Networks", "ITC-NETWORKS" },
	{ 0x388EE7, "Fanhattan LLC", "FANHATTAN-LLC" },
	{ 0x3891FB, "Xenox Holding BV", "XENOX-HOLDING" },
	{ 0x389496, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0x389592, "Beijing Tendyron Corporation", "BEIJING-TENDYRON" },
	{ 0x389F83, "OTN Systems N.V.", "OTN-N-V" },
	{ 0x38A53C, "Veenstra Instruments", "VEENSTRA-INSTRUMENTS" },
	{ 0x38A5B6, "SHENZHEN MEGMEET ELECTRICAL CO.,LTD", "SHENZHEN-MEGMEET-ELECTRICAL" },
	{ 0x38A851, "Moog, Ing", "MOOG-ING" },
	{ 0x38A86B, "Orga BV", "ORGA" },
	{ 0x38A95F, "Actifio Inc", "ACTIFIO" },
	{ 0x38AA3C, "SAMSUNG ELECTRO-MECHANICS", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x38B12D, "Sonotronic Nagel GmbH", "SONOTRONIC-NAGEL" },
	{ 0x38B1DB, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x38B5BD, "E.G.O. Elektro-Ger", "E-G-O-ELEKTRO-GER" },
	{ 0x38B74D, "Fijowave Limited", "FIJOWAVE" },
	{ 0x38BB23, "OzVision America LLC", "OZVISION-AMERICA-LLC" },
	{ 0x38BB3C, "Avaya, Inc", "AVAYA" },
	{ 0x38BC1A, "Meizu technology co.,ltd", "MEIZU" },
	{ 0x38BF2F, "Espec Corp.", "ESPEC" },
	{ 0x38BF33, "NEC CASIO Mobile Communications", "NEC-CASIO-MOBILE-COMMUNICATION" },
	{ 0x38C096, "ALPS ELECTRIC CO.,LTD.", "ALPS-ELECTRIC" },
	{ 0x38C70A, "WiFiSong", "WIFISONG" },
	{ 0x38C7BA, "CS Services Co.,Ltd.", "CS-SERVICES" },
	{ 0x38C85C, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x38C9A9, "SMART High Reliability Solutions, Inc.", "SMART-HIGH-RELIABILITY-SOLUTIONS" },
	{ 0x38CA97, "Contour Design LLC", "CONTOUR-DESIGN-LLC" },
	{ 0x38D135, "EasyIO Corporation Sdn. Bhd.", "EASYIO-SDN-BHD" },
	{ 0x38D82F, "zte corporation", "ZTE" },
	{ 0x38DBBB, "Sunbow Telecom Co., Ltd.", "SUNBOW-TELECOM" },
	{ 0x38DE60, "Mohlenhoff GmbH", "MOHLENHOFF" },
	{ 0x38E08E, "Mitsubishi Electric Corporation", "MITSUBISHI-ELECTRIC" },
	{ 0x38E595, "Shenzhen Gongjin Electronics Co.,Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0x38E7D8, "HTC Corporation", "HTC" },
	{ 0x38E8DF, "b gmbh medien + datenbanken", "B-MEDIEN-+-DATENBANKEN" },
	{ 0x38E98C, "Reco S.p.A.", "RECO-S-P-A" },
	{ 0x38EAA7, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x38EC11, "Novatek Microelectronics Corp.", "NOVATEK-MICROELECTRONICS" },
	{ 0x38ECE4, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x38EE9D, "Anedo Ltd.", "ANEDO" },
	{ 0x38F098, "Vapor Stone Rail Systems", "VAPOR-STONE-RAIL" },
	{ 0x38F33F, "TATSUNO CORPORATION", "TATSUNO" },
	{ 0x38F597, "home2net GmbH", "HOME2NET" },
	{ 0x38F708, "National Resource Management, Inc.", "NATIONAL-RESOURCE-MANAGEMENT" },
	{ 0x38F889, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x38F8B7, "V2COM PARTICIPACOES S.A.", "V2COM-PARTICIPACOES" },
	{ 0x38FACA, "Skyworth Digital Technology(Shenzhen) Co.,Ltd", "SKYWORTH-DIGITAL-SHENZHEN" },
	{ 0x38FEC5, "Ellips B.V.", "ELLIPS" },
	{ 0x3C02B1, "Creation Technologies LP", "CREATION-TECHNOLOGIES-LP" },
	{ 0x3C04BF, "PRAVIS SYSTEMS Co.Ltd.,", "PRAVIS" },
	{ 0x3C05AB, "Product Creation Studio", "PRODUCT-CREATION-STUDIO" },
	{ 0x3C0754, "Apple", "APPLE" },
	{ 0x3C0771, "Sony Corporation", "SONY" },
	{ 0x3C081E, "Beijing Yupont Electric Power Technology Co.,Ltd", "BEIJING-YUPONT-ELECTRIC-POWER" },
	{ 0x3C08F6, "Cisco", "CISCO" },
	{ 0x3C096D, "Powerhouse Dynamics", "POWERHOUSE-DYNAMICS" },
	{ 0x3C0C48, "Servergy, Inc.", "SERVERGY" },
	{ 0x3C0E23, "Cisco", "CISCO" },
	{ 0x3C0FC1, "KBC Networks", "KBC-NETWORKS" },
	{ 0x3C1040, "daesung network", "DAESUNG-NETWORK" },
	{ 0x3C106F, "ALBAHITH TECHNOLOGIES", "ALBAHITH-TECHNOLOGIES" },
	{ 0x3C15C2, "Apple", "APPLE" },
	{ 0x3C15EA, "TESCOM CO., LTD.", "TESCOM" },
	{ 0x3C189F, "Nokia Corporation", "NOKIA" },
	{ 0x3C18A0, "Luxshare Precision Industry Co.,Ltd.", "LUXSHARE-PRECISION-INDUSTRY" },
	{ 0x3C1915, "GFI Chrono Time", "GFI-CHRONO-TIME" },
	{ 0x3C197D, "Ericsson AB", "ERICSSON-AB" },
	{ 0x3C1A0F, "ClearSky Data", "CLEARSKY-DATA" },
	{ 0x3C1A57, "Cardiopulmonary Corp", "CARDIOPULMONARY" },
	{ 0x3C1A79, "Huayuan Technology CO.,LTD", "HUAYUAN" },
	{ 0x3C1CBE, "JADAK LLC", "JADAK-LLC" },
	{ 0x3C1E04, "D-Link International", "D-LINK" },
	{ 0x3C1E13, "HANGZHOU SUNRISE TECHNOLOGY CO., LTD", "HANGZHOU-SUNRISE" },
	{ 0x3C25D7, "Nokia Corporation", "NOKIA" },
	{ 0x3C26D5, "Sotera Wireless", "SOTERA-WIRELESS" },
	{ 0x3C2763, "SLE quality engineering GmbH & Co. KG", "SLE-QUALITY-ENGINEERING" },
	{ 0x3C2C94, "杭州德澜科技有限公司（HangZhou Delan Technology Co.,Ltd）", "杭州德澜科技有限公司（HANGZHOU-DELAN-）" },
	{ 0x3C2DB7, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x3C2F3A, "SFORZATO Corp.", "SFORZATO" },
	{ 0x3C300C, "Dewar Electronics Pty Ltd", "DEWAR-ELECTRONICS-PTY" },
	{ 0x3C363D, "Nokia Corporation", "NOKIA" },
	{ 0x3C36E4, "Arris Group, Inc.", "ARRIS-GROUP" },
	{ 0x3C3888, "ConnectQuest, llc", "CONNECTQUEST-LLC" },
	{ 0x3C39C3, "JW Electronics Co., Ltd.", "JW-ELECTRONICS" },
	{ 0x3C39E7, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x3C3A73, "Avaya, Inc", "AVAYA" },
	{ 0x3C404F, "Guangdong Pisen Electronics Co. Ltd.", "GUANGDONG-PISEN-ELECTRONICS" },
	{ 0x3C438E, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x3C46D8, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x3C4711, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x3C4937, "ASSMANN Electronic GmbH", "ASSMANN" },
	{ 0x3C4A92, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x3C4C69, "Infinity System S.L.", "INFINITY-SYSTEM-S-L" },
	{ 0x3C4E47, "Etronic A/S", "ETRONIC-A/S" },
	{ 0x3C57BD, "Kessler Crane Inc.", "KESSLER-CRANE" },
	{ 0x3C57D5, "FiveCo", "FIVECO" },
	{ 0x3C5A37, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x3C5AB4, "Google", "GOOGLE" },
	{ 0x3C5EC3, "Cisco", "CISCO" },
	{ 0x3C5F01, "Synerchip Co., Ltd.", "SYNERCHIP" },
	{ 0x3C6104, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x3C6200, "Samsung electronics CO., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x3C6278, "SHENZHEN JETNET TECHNOLOGY CO.,LTD.", "SHENZHEN-JETNET" },
	{ 0x3C672C, "Sciovid Inc.", "SCIOVID" },
	{ 0x3C6A7D, "Niigata Power Systems Co., Ltd.", "NIIGATA-POWER" },
	{ 0x3C6A9D, "Dexatek Technology LTD.", "DEXATEK" },
	{ 0x3C6E63, "Mitron OY", "MITRON-OY" },
	{ 0x3C6F45, "Fiberpro Inc.", "FIBERPRO" },
	{ 0x3C6FF7, "EnTek Systems, Inc.", "ENTEK" },
	{ 0x3C7059, "MakerBot Industries", "MAKERBOT-INDUSTRIES" },
	{ 0x3C7437, "RIM", "RIM" },
	{ 0x3C754A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x3C77E6, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x3C7DB1, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x3C81D8, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0x3C8375, "Microsoft Corporation", "MICROSOFT" },
	{ 0x3C83B5, "Advance Vision Electronics Co. Ltd.", "ADVANCE-VISION-ELECTRONICS" },
	{ 0x3C86A8, "Sangshin elecom .co,, LTD", "SANGSHIN-ELECOM" },
	{ 0x3C89A6, "KAPELSE", "KAPELSE" },
	{ 0x3C8AB0, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x3C8AE5, "Tensun Information Technology(Hangzhou) Co.,LTD", "TENSUN-INFORMATION-HANGZHOU" },
	{ 0x3C8BFE, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x3C8C40, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x3C912B, "Vexata Inc", "VEXATA" },
	{ 0x3C9157, "Hangzhou Yulong Conmunication Co.,Ltd", "HANGZHOU-YULONG-CONMUNICATION" },
	{ 0x3C9174, "ALONG COMMUNICATION TECHNOLOGY", "ALONG-COMMUNICATION" },
	{ 0x3C94D5, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x3C970E, "Wistron InfoComm(Kunshan)Co.,Ltd.", "WISTRON-INFOCOMM-KUNSHAN" },
	{ 0x3C977E, "IPS Technology Limited", "IPS" },
	{ 0x3C98BF, "Quest Controls, Inc.", "QUEST-CONTROLS" },
	{ 0x3C99F7, "Lansentechnology AB", "LANSENTECHNOLOGY-AB" },
	{ 0x3C9F81, "Shenzhen CATIC Bit Communications Technology Co.,Ltd", "SHENZHEN-CATIC-BIT-COMMUNICATION" },
	{ 0x3CA10D, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x3CA315, "Bless Information & Communications Co., Ltd", "BLESS-INFORMATION-COMMUNICATION" },
	{ 0x3CA72B, "MRV Communications (Networks) LTD", "MRV-COMMUNICATION-NETWORKS" },
	{ 0x3CA82A, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x3CA9F4, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x3CAA3F, "iKey, Ltd.", "IKEY" },
	{ 0x3CAB8E, "Apple", "APPLE" },
	{ 0x3CAE69, "ESA Elektroschaltanlagen Grimma GmbH", "ESA-ELEKTROSCHALTANLAGEN-GRIMMA" },
	{ 0x3CB15B, "Avaya, Inc", "AVAYA" },
	{ 0x3CB17F, "Wattwatchers Pty Ld", "WATTWATCHERS-PTY-LD" },
	{ 0x3CB792, "Hitachi Maxell, Ltd., Optronics Division", "HITACHI-MAXELL-OPTRONICS-DIVISION" },
	{ 0x3CB87A, "PRIVATE", "PRIVATE" },
	{ 0x3CB9A6, "Belden Deutschland GmbH", "BELDEN-DEUTSCHLAND" },
	{ 0x3CBDD8, "LG ELECTRONICS INC", "LG-ELECTRONICS" },
	{ 0x3CC0C6, "d&b audiotechnik GmbH", "D-B-AUDIOTECHNIK" },
	{ 0x3CC12C, "AES Corporation", "AES" },
	{ 0x3CC1F6, "Melange Systems Pvt. Ltd.", "MELANGE-PVT" },
	{ 0x3CC243, "Nokia Corporation", "NOKIA" },
	{ 0x3CC2E1, "XINHUA CONTROL ENGINEERING CO.,LTD", "XINHUA-CONTROL-ENGINEERING" },
	{ 0x3CC99E, "Huiyang Technology Co., Ltd", "HUIYANG" },
	{ 0x3CCA87, "Iders Incorporated", "IDERS-INCORPORATED" },
	{ 0x3CCB7C, "TCT mobile ltd", "TCT-MOBILE" },
	{ 0x3CCD5A, "Technische Alternative GmbH", "TECHNISCHE-ALTERNATIVE" },
	{ 0x3CCD93, "LG ELECTRONICS INC", "LG-ELECTRONICS" },
	{ 0x3CCE15, "Mercedes-Benz USA, LLC", "MERCEDES-BENZ-USA-LLC" },
	{ 0x3CCE73, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x3CD0F8, "Apple", "APPLE" },
	{ 0x3CD16E, "Telepower Communication Co., Ltd", "TELEPOWER-COMMUNICATION" },
	{ 0x3CD4D6, "WirelessWERX, Inc", "WIRELESSWERX" },
	{ 0x3CD7DA, "SK Mtek microelectronics(shenzhen)limited", "SK-MTEK-MICROELECTRONICS-SHENZHEN" },
	{ 0x3CD92B, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x3CD9CE, "Eclipse WiFi", "ECLIPSE-WIFI" },
	{ 0x3CDF1E, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x3CDFA9, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x3CDFBD, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x3CE072, "Apple", "APPLE" },
	{ 0x3CE5A6, "Hangzhou H3C Technologies Co., Ltd.", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x3CE5B4, "KIDASEN INDUSTRIA E COMERCIO DE ANTENAS LTDA", "KIDASEN-INDUSTRIA-E-COMERCIO-DE-ANTENAS-LTDA" },
	{ 0x3CE624, "LG Display", "LG-DISPLAY" },
	{ 0x3CEA4F, "2Wire", "2WIRE" },
	{ 0x3CEAFB, "NSE AG", "NSE" },
	{ 0x3CF392, "Virtualtek. Co. Ltd", "VIRTUALTEK" },
	{ 0x3CF52C, "DSPECIALISTS GmbH", "DSPECIALISTS" },
	{ 0x3CF72A, "Nokia Corporation", "NOKIA" },
	{ 0x3CF748, "Shenzhen Linsn Technology Development Co.,Ltd", "SHENZHEN-LINSN-DEVELOPMENT" },
	{ 0x3CF808, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x3CFB96, "Emcraft Systems LLC", "EMCRAFT-LLC" },
	{ 0x3CFDFE, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x400107, "Arista Corp", "ARISTA" },
	{ 0x4001C6, "3COM EUROPE LTD", "3COM" },
	{ 0x40040C, "A&T", "A-T" },
	{ 0x4007C0, "Railtec Systems GmbH", "RAILTEC" },
	{ 0x400E67, "Tremol Ltd.", "TREMOL" },
	{ 0x400E85, "Samsung Electro Mechanics co.,LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x4011DC, "Sonance", "SONANCE" },
	{ 0x4012E4, "Compass-EOS", "COMPASS-EOS" },
	{ 0x4013D9, "Global ES", "GLOBAL-ES" },
	{ 0x401597, "Protect America, Inc.", "PROTECT-AMERICA" },
	{ 0x40167E, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x40169F, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x4016FA, "EKM Metering", "EKM-METERING" },
	{ 0x4018B1, "Aerohive Networks Inc.", "AEROHIVE-NETWORKS" },
	{ 0x4018D7, "Smartronix, Inc.", "SMARTRONIX" },
	{ 0x401D59, "Biometric Associates, LP", "BIOMETRIC-ASSOCIATES-LP" },
	{ 0x4022ED, "Digital Projection Ltd", "DIGITAL-PROJECTION" },
	{ 0x4025C2, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x40270B, "Mobileeco Co., Ltd", "MOBILEECO" },
	{ 0x402814, "RFI Engineering", "RFI-ENGINEERING" },
	{ 0x402BA1, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x402CF4, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x403004, "Apple", "APPLE" },
	{ 0x403067, "Conlog (Pty) Ltd", "CONLOG-PTY" },
	{ 0x40336C, "Godrej & Boyce Mfg. co. ltd", "GODREJ-BOYCE-MFG" },
	{ 0x4037AD, "Macro Image Technology, Inc.", "MACRO-IMAGE" },
	{ 0x403CFC, "Apple", "APPLE" },
	{ 0x404022, "ZIV", "ZIV" },
	{ 0x40406B, "Icomera", "ICOMERA" },
	{ 0x4045DA, "Spreadtrum Communications (Shanghai) Co., Ltd.", "SPREADTRUM-COMMUNICATION-SHANGHAI" },
	{ 0x404A03, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0x404A18, "Addrek Smart Solutions", "ADDREK-SMART-SOLUTIONS" },
	{ 0x404D8E, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x404EEB, "Higher Way Electronic Co., Ltd.", "HIGHER-WAY" },
	{ 0x4050E0, "Milton Security Group LLC", "MILTON-SECURITY-GROUP-LLC" },
	{ 0x40516C, "Grandex International Corporation", "GRANDEX" },
	{ 0x40520D, "Pico Technology", "PICO" },
	{ 0x405539, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x40560C, "In Home Displays Ltd", "IN-HOME-DISPLAYS" },
	{ 0x405A9B, "ANOVO", "ANOVO" },
	{ 0x405FBE, "RIM", "RIM" },
	{ 0x405FC2, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x40605A, "Hawkeye Tech Co. Ltd", "HAWKEYE-TECH" },
	{ 0x406186, "MICRO-STAR INT'L CO.,LTD", "MICRO-STAR" },
	{ 0x40618E, "Stella-Green Co", "STELLA-GREEN" },
	{ 0x4062B6, "Tele system communication", "TELE-SYSTEM-COMMUNICATION" },
	{ 0x40667A, "mediola - connected living AG", "MEDIOLA---CONNECTED-LIVING" },
	{ 0x406826, "Thales UK Limited", "THALES-UK" },
	{ 0x406AAB, "RIM", "RIM" },
	{ 0x406C8F, "Apple", "APPLE" },
	{ 0x406F2A, "Research In Motion", "RESEARCH-IN-MOTION" },
	{ 0x407009, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x40704A, "Power Idea Technology Limited", "POWER-IDEA" },
	{ 0x407074, "Life Technology (China) Co., Ltd", "LIFE-CHINA" },
	{ 0x407496, "aFUN TECHNOLOGY INC.", "AFUN" },
	{ 0x40786A, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0x407875, "IMBEL - Industria de Material Belico do Brasil", "IMBEL---INDUSTRIA-DE-MATERIAL-BELICO-DO-BRASIL" },
	{ 0x407A80, "Nokia Corporation", "NOKIA" },
	{ 0x407B1B, "Mettle Networks Inc.", "METTLE-NETWORKS" },
	{ 0x407FE0, "Glory Star Technics (ShenZhen) Limited", "GLORY-STAR-TECHNICS-SHENZHEN" },
	{ 0x408256, "Continental Automotive GmbH", "CONTINENTAL-AUTOMOTIVE" },
	{ 0x4083DE, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0x408493, "Clavister AB", "CLAVISTER-AB" },
	{ 0x4088E0, "Beijing Ereneben Information Technology Limited Shenzhen Branch", "BEIJING-ERENEBEN-INFORMATION-SHENZHEN-BRANCH" },
	{ 0x408A9A, "TITENG CO., Ltd.", "TITENG" },
	{ 0x408B07, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x408BF6, "Shenzhen TCL New Technology Co; Ltd.", "SHENZHEN-TCL-NEW" },
	{ 0x408D5C, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x409558, "Aisino Corporation", "AISINO" },
	{ 0x4097D1, "BK Electronics cc", "BK-ELECTRONICS-CC" },
	{ 0x40984C, "Casacom Solutions AG", "CASACOM-SOLUTIONS" },
	{ 0x40984E, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x40987B, "Aisino Corporation", "AISINO" },
	{ 0x409B0D, "Shenzhen Yourf Kwan Industrial Co., Ltd", "SHENZHEN-YOURF-KWAN-INDUSTRIAL" },
	{ 0x409FC7, "BAEKCHUN I&C Co., Ltd.", "BAEKCHUN-I-C" },
	{ 0x40A5EF, "Shenzhen Four Seas Global Link Network Technology Co., Ltd.", "SHENZHEN-FOUR-SEAS-GLOBAL-LINK-NETWORK" },
	{ 0x40A677, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x40A6A4, "PassivSystems Ltd", "PASSIVSYSTEMS" },
	{ 0x40A6D9, "Apple", "APPLE" },
	{ 0x40A6E8, "Cisco", "CISCO" },
	{ 0x40A8F0, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x40AC8D, "Data Management, Inc.", "DATA-MANAGEMENT" },
	{ 0x40B0FA, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x40B2C8, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x40B395, "Apple", "APPLE" },
	{ 0x40B3CD, "Chiyoda Electronics Co.,Ltd.", "CHIYODA-ELECTRONICS" },
	{ 0x40B3FC, "Logital Co. Limited", "LOGITAL" },
	{ 0x40B4F0, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x40B6B1, "SUNGSAM CO,.Ltd", "SUNGSAM" },
	{ 0x40B7F3, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x40B837, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x40B89A, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x40BA61, "Arima Communications Corp.", "ARIMA-COMMUNICATION" },
	{ 0x40BC73, "Cronoplast S.L.", "CRONOPLAST-S-L" },
	{ 0x40BC8B, "itelio GmbH", "ITELIO" },
	{ 0x40BD9E, "Physio-Control, Inc", "PHYSIO-CONTROL" },
	{ 0x40BF17, "Digistar Telecom. SA", "DIGISTAR-TELECOM" },
	{ 0x40C245, "Shenzhen Hexicom Technology Co., Ltd.", "SHENZHEN-HEXICOM" },
	{ 0x40C4D6, "ChongQing Camyu Technology Development Co.,Ltd.", "CHONGQING-CAMYU-DEVELOPMENT" },
	{ 0x40C62A, "Shanghai Jing Ren Electronic Technology Co., Ltd.", "SHANGHAI-JING-REN" },
	{ 0x40C7C9, "Naviit Inc.", "NAVIIT" },
	{ 0x40CBA8, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x40CD3A, "Z3 Technology", "Z3" },
	{ 0x40D28A, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x40D32D, "Apple", "APPLE" },
	{ 0x40D40E, "Biodata Ltd", "BIODATA" },
	{ 0x40D559, "MICRO S.E.R.I.", "MICRO-S-E-R-I" },
	{ 0x40D855, "IEEE REGISTRATION AUTHORITY - Please see IAB public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-IAB-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x40E230, "AzureWave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x40E730, "DEY Storage Systems, Inc.", "DEY-STORAGE" },
	{ 0x40E793, "Shenzhen Siviton Technology Co.,Ltd", "SHENZHEN-SIVITON" },
	{ 0x40EACE, "FOUNDER BROADBAND NETWORK SERVICE CO.,LTD", "FOUNDER-BROADBAND-NETWORK-SERVICE" },
	{ 0x40ECF8, "Siemens AG", "SIEMENS" },
	{ 0x40EF4C, "Fihonest communication co.,Ltd", "FIHONEST-COMMUNICATION" },
	{ 0x40F02F, "Liteon Technology Corporation", "LITEON" },
	{ 0x40F14C, "ISE Europe SPRL", "ISE-EUROPE-SPRL" },
	{ 0x40F201, "SAGEMCOM", "SAGEMCOM" },
	{ 0x40F2E9, "IBM", "IBM" },
	{ 0x40F308, "Murata Manufactuaring Co.,Ltd.", "MURATA-MANUFACTUARING" },
	{ 0x40F407, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x40F4EC, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x40F52E, "Leica Microsystems (Schweiz) AG", "LEICA-MICROSYSTEMS-SCHWEIZ" },
	{ 0x40FC89, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x4403A7, "Cisco", "CISCO" },
	{ 0x440CFD, "NetMan Co., Ltd.", "NETMAN" },
	{ 0x4411C2, "Telegartner Karl Gartner GmbH", "TELEGARTNER-KARL-GARTNER" },
	{ 0x441319, "WKK TECHNOLOGY LTD.", "WKK" },
	{ 0x44184F, "Fitview", "FITVIEW" },
	{ 0x4419B6, "Hangzhou Hikvision Digital Technology Co.,Ltd.", "HANGZHOU-HIKVISION-DIGITAL" },
	{ 0x441E91, "ARVIDA Intelligent Electronics Technology Co.,Ltd.", "ARVIDA-INTELLIGENT-ELECTRONICS" },
	{ 0x441EA1, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x4423AA, "Farmage Co., Ltd.", "FARMAGE" },
	{ 0x4425BB, "Bamboo Entertainment Corporation", "BAMBOO-ENTERTAINMENT" },
	{ 0x442938, "NietZsche enterprise Co.Ltd.", "NIETZSCHE" },
	{ 0x442A60, "Apple", "APPLE" },
	{ 0x442AFF, "E3 Technology, Inc.", "E3" },
	{ 0x442B03, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x443192, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x44322A, "Avaya, Inc", "AVAYA" },
	{ 0x4432C8, "Technicolor USA Inc.", "TECHNICOLOR-USA" },
	{ 0x44334C, "Shenzhen Bilian electronic CO.,LTD", "SHENZHEN-BILIAN" },
	{ 0x44348F, "MXT INDUSTRIAL LTDA", "MXT-INDUSTRIAL-LTDA" },
	{ 0x44356F, "Neterix", "NETERIX" },
	{ 0x443719, "2 Save Energy Ltd", "2-SAVE-ENERGY" },
	{ 0x44376F, "Young Electric Sign Co", "YOUNG-ELECTRIC-SIGN" },
	{ 0x4437E6, "Hon Hai Precision Ind.Co.Ltd", "HON-HAI-PRECISION-IND" },
	{ 0x443839, "Cumulus Networks, inc", "CUMULUS-NETWORKS" },
	{ 0x4439C4, "Universal Global Scientific Industrial Co.,Ltd", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x443C9C, "Pintsch Tiefenbach GmbH", "PINTSCH-TIEFENBACH" },
	{ 0x443D21, "Nuvolt", "NUVOLT" },
	{ 0x443EB2, "DEOTRON Co., LTD.", "DEOTRON" },
	{ 0x444891, "HDMI Licensing, LLC", "HDMI-LICENSING-LLC" },
	{ 0x444A65, "Silverflare Ltd.", "SILVERFLARE" },
	{ 0x444C0C, "Apple", "APPLE" },
	{ 0x444E1A, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x444F5E, "Pan Studios Co.,Ltd.", "PAN-STUDIOS" },
	{ 0x4451DB, "Raytheon BBN Technologies", "RAYTHEON-BBN-TECHNOLOGIES" },
	{ 0x4454C0, "Thompson Aerospace", "THOMPSON-AEROSPACE" },
	{ 0x4455B1, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x44568D, "PNC Technologies Co., Ltd.", "PNC-TECHNOLOGIES" },
	{ 0x4456B7, "Spawn Labs, Inc", "SPAWN-LABS" },
	{ 0x445829, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x44599F, "Criticare Systems, Inc", "CRITICARE" },
	{ 0x445ECD, "Razer Inc", "RAZER" },
	{ 0x445EF3, "Tonalite Holding B.V.", "TONALITE-HOLDING" },
	{ 0x445F7A, "Shihlin Electric & Engineering Corp.", "SHIHLIN-ELECTRIC-ENGINEERING" },
	{ 0x446132, "ecobee inc", "ECOBEE" },
	{ 0x44619C, "FONsystem co. ltd.", "FONSYSTEM" },
	{ 0x44666E, "IP-LINE", "IP-LINE" },
	{ 0x446755, "Orbit Irrigation", "ORBIT-IRRIGATION" },
	{ 0x4468AB, "JUIN COMPANY, LIMITED", "JUIN-COMPANY" },
	{ 0x446C24, "Reallin Electronic Co.,Ltd", "REALLIN" },
	{ 0x446D57, "Liteon Technology Corporation", "LITEON" },
	{ 0x446D6C, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0x44700B, "IFFU", "IFFU" },
	{ 0x447098, "MING HONG TECHNOLOGY (SHEN ZHEN) LIMITED", "MING-HONG-SHEN-ZHEN" },
	{ 0x44746C, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x447BC4, "DualShine Technology(SZ)Co.,Ltd", "DUALSHINE-SZ" },
	{ 0x447C7F, "Innolight Technology Corporation", "INNOLIGHT" },
	{ 0x447DA5, "VTION INFORMATION TECHNOLOGY (FUJIAN) CO.,LTD", "VTION-INFORMATION-FUJIAN" },
	{ 0x447E76, "Trek Technology (S) Pte Ltd", "TREK-S-PTE" },
	{ 0x447E95, "Alpha and Omega, Inc", "ALPHA-AND-OMEGA" },
	{ 0x4480EB, "Motorola Mobility LLC, a Lenovo Company", "MOTOROLA-MOBILITY-LLC-A-LENOVO-COMPANY" },
	{ 0x448312, "Star-Net", "STAR-NET" },
	{ 0x448500, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x4486C1, "Siemens Low Voltage & Products", "SIEMENS-LOW-VOLTAGE-PRODUCTS" },
	{ 0x4487FC, "ELITEGROUP COMPUTER SYSTEM CO., LTD.", "ELITEGROUP-SYSTEM" },
	{ 0x4488CB, "Camco Technologies NV", "CAMCO-TECHNOLOGIES-NV" },
	{ 0x448A5B, "Micro-Star INT'L CO., LTD.", "MICRO-STAR" },
	{ 0x448C52, "KTIS CO., Ltd", "KTIS" },
	{ 0x448E12, "DT Research, Inc.", "DT-RESEARCH" },
	{ 0x448E81, "VIG", "VIG" },
	{ 0x4491DB, "Shanghai Huaqin Telecom Technology Co.,Ltd", "SHANGHAI-HUAQIN-TELECOM" },
	{ 0x4494FC, "NETGEAR INC.,", "NETGEAR" },
	{ 0x4495FA, "Qingdao Santong Digital Technology Co.Ltd", "QINGDAO-SANTONG-DIGITAL" },
	{ 0x44962B, "Aidon Oy", "AIDON-OY" },
	{ 0x449B78, "The Now Factory", "THE-NOW-FACTORY" },
	{ 0x449CB5, "Alcomp, Inc", "ALCOMP" },
	{ 0x44A42D, "TCT Mobile Limited", "TCT-MOBILE" },
	{ 0x44A689, "PROMAX ELECTRONICA SA", "PROMAX-ELECTRONICA" },
	{ 0x44A6E5, "THINKING TECHNOLOGY CO.,LTD", "THINKING" },
	{ 0x44A7CF, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x44A842, "Dell Inc.", "DELL" },
	{ 0x44A8C2, "SEWOO TECH CO., LTD", "SEWOO-TECH" },
	{ 0x44AA27, "udworks Co., Ltd.", "UDWORKS" },
	{ 0x44AAE8, "Nanotec Electronic GmbH & Co. KG", "NANOTEC" },
	{ 0x44ADD9, "Cisco", "CISCO" },
	{ 0x44B382, "Kuang-chi Institute of Advanced Technology", "KUANG-CHI-INSTITUTE-OF-ADVANCED" },
	{ 0x44C15C, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x44C233, "Guangzhou Comet Technology Development Co.Ltd", "GUANGZHOU-COMET-DEVELOPMENT" },
	{ 0x44C306, "SIFROM Inc.", "SIFROM" },
	{ 0x44C39B, "OOO RUBEZH NPO", "OOO-RUBEZH-NPO" },
	{ 0x44C4A9, "Opticom Communication, LLC", "OPTICOM-COMMUNICATION-LLC" },
	{ 0x44C56F, "NGN Easy Satfinder (Tianjin) Electronic Co., Ltd", "NGN-EASY-SATFINDER-TIANJIN" },
	{ 0x44C69B, "Wuhan Feng Tian Information Network CO.,LTD", "WUHAN-FENG-TIAN-INFORMATION-NETWORK" },
	{ 0x44C9A2, "Greenwald Industries", "GREENWALD-INDUSTRIES" },
	{ 0x44CE7D, "SFR", "SFR" },
	{ 0x44D15E, "Shanghai Kingto Information Technology Ltd", "SHANGHAI-KINGTO-INFORMATION" },
	{ 0x44D244, "Seiko Epson Corporation", "SEIKO-EPSON" },
	{ 0x44D2CA, "Anvia TV Oy", "ANVIA-TV-OY" },
	{ 0x44D3CA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x44D4E0, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x44D63D, "Talari Networks", "TALARI-NETWORKS" },
	{ 0x44D832, "Azurewave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x44D884, "Apple", "APPLE" },
	{ 0x44D9E7, "Ubiquiti Networks, Inc.", "UBIQUITI-NETWORKS" },
	{ 0x44DC91, "PLANEX COMMUNICATIONS INC.", "PLANEX-COMMUNICATION" },
	{ 0x44DCCB, "SEMINDIA SYSTEMS PVT LTD", "SEMINDIA-PVT" },
	{ 0x44E08E, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x44E137, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x44E49A, "OMNITRONICS PTY LTD", "OMNITRONICS-PTY" },
	{ 0x44E4D9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x44E8A5, "Myreka Technologies Sdn. Bhd.", "MYREKA-TECHNOLOGIES-SDN-BHD" },
	{ 0x44E9DD, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0x44ED57, "Longicorn, inc.", "LONGICORN" },
	{ 0x44EE30, "Budelmann Elektronik GmbH", "BUDELMANN-ELEKTRONIK" },
	{ 0x44F459, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x44F477, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x44F849, "Union Pacific Railroad", "UNION-PACIFIC-RAILROAD" },
	{ 0x44FB42, "Apple", "APPLE" },
	{ 0x48022A, "B-Link Electronic Limited", "B-LINK" },
	{ 0x480362, "DESAY ELECTRONICS(HUIZHOU)CO.,LTD", "DESAY-ELECTRONICS-HUIZHOU" },
	{ 0x48066A, "Tempered Networks, Inc.", "TEMPERED-NETWORKS" },
	{ 0x480C49, "NAKAYO TELECOMMUNICATIONS,INC", "NAKAYO-TELECOMMUNICATIONS" },
	{ 0x481249, "Luxcom Technologies Inc.", "LUXCOM-TECHNOLOGIES" },
	{ 0x4813F3, "BBK Electronics Corp., Ltd.", "BBK-ELECTRONICS" },
	{ 0x48174C, "MicroPower technologies", "MICROPOWER-TECHNOLOGIES" },
	{ 0x481842, "Shanghai Winaas Co. Equipment Co. Ltd.", "SHANGHAI-WINAAS-EQUIPMENT" },
	{ 0x481A84, "Pointer Telocation Ltd", "POINTER-TELOCATION" },
	{ 0x481BD2, "Intron Scientific co., ltd.", "INTRON-SCIENTIFIC" },
	{ 0x481D70, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x4826E8, "Tek-Air Systems, Inc.", "TEK-AIR" },
	{ 0x48282F, "ZTE Corporation", "ZTE" },
	{ 0x482CEA, "Motorola Inc Business Light Radios", "MOTOROLA-BUSINESS-LIGHT-RADIOS" },
	{ 0x4833DD, "ZENNIO AVANCE Y TECNOLOGIA, S.L.", "ZENNIO-AVANCE-Y-TECNOLOGIA-S-L" },
	{ 0x48343D, "IEP GmbH", "IEP" },
	{ 0x483D32, "Syscor Controls &amp; Automation", "SYSCOR-CONTROLS-AMP-AUTOMATION" },
	{ 0x48437C, "Apple", "APPLE" },
	{ 0x484487, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x4844F7, "Samsung Electronics Co., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x4846F1, "Uros Oy", "UROS-OY" },
	{ 0x4846FB, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x4851B7, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x485261, "SOREEL", "SOREEL" },
	{ 0x485415, "NET RULES TECNOLOGIA EIRELI", "NET-RULES-TECNOLOGIA-EIRELI" },
	{ 0x48555F, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0x4857DD, "Facebook", "FACEBOOK" },
	{ 0x485929, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x485A3F, "WISOL", "WISOL" },
	{ 0x485AB6, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x485B39, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x485D60, "Azurewave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x4860BC, "Apple", "APPLE" },
	{ 0x4861A3, "Concern \"Axion\" JSC", "CONCERN-AXION-JSC" },
	{ 0x486276, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x486B2C, "BBK Electronics Corp., Ltd.,", "BBK-ELECTRONICS" },
	{ 0x486B91, "Fleetwood Group Inc.", "FLEETWOOD-GROUP" },
	{ 0x486E73, "Pica8, Inc.", "PICA8" },
	{ 0x486EFB, "Davit System Technology Co., Ltd.", "DAVIT-SYSTEM" },
	{ 0x486FD2, "StorSimple Inc", "STORSIMPLE" },
	{ 0x487119, "SGB GROUP LTD.", "SGB-GROUP" },
	{ 0x48746E, "Apple", "APPLE" },
	{ 0x487604, "PRIVATE", "PRIVATE" },
	{ 0x488244, "Life Fitness / Div. of Brunswick", "LIFE-FITNESS-/-DIV-OF-BRUNSWICK" },
	{ 0x4886E8, "Microsoft Corporation", "MICROSOFT" },
	{ 0x488E42, "DIGALOG GmbH", "DIGALOG" },
	{ 0x489153, "Weinmann Geräte für Medizin GmbH + Co. KG", "WEINMANN-GERäTE-FüR-MEDIZIN-+" },
	{ 0x4891F6, "Shenzhen Reach software technology CO.,LTD", "SHENZHEN-REACH-SOFTWARE" },
	{ 0x489BE2, "SCI Innovations Ltd", "SCI-INNOVATIONS" },
	{ 0x489D18, "Flashbay Limited", "FLASHBAY" },
	{ 0x489D24, "Research In Motion", "RESEARCH-IN-MOTION" },
	{ 0x48A22D, "Shenzhen Huaxuchang Telecom Technology Co.,Ltd", "SHENZHEN-HUAXUCHANG-TELECOM" },
	{ 0x48A2B7, "Kodofon JSC", "KODOFON-JSC" },
	{ 0x48A6D2, "GJsun Optical Science and Tech Co.,Ltd.", "GJSUN-OPTICAL-SCIENCE-AND-TECH" },
	{ 0x48A9D2, "Wistron Neweb Corp.", "WISTRON-NEWEB" },
	{ 0x48AA5D, "Store Electronic Systems", "STORE" },
	{ 0x48B253, "Marketaxess Corporation", "MARKETAXESS" },
	{ 0x48B5A7, "Glory Horse Industries Ltd.", "GLORY-HORSE-INDUSTRIES" },
	{ 0x48B8DE, "HOMEWINS TECHNOLOGY CO.,LTD.", "HOMEWINS" },
	{ 0x48B977, "PulseOn Oy", "PULSEON-OY" },
	{ 0x48B9C2, "Teletics Inc.", "TELETICS" },
	{ 0x48BE2D, "Symanitron", "SYMANITRON" },
	{ 0x48C093, "Xirrus, Inc.", "XIRRUS" },
	{ 0x48C1AC, "PLANTRONICS, INC.", "PLANTRONICS" },
	{ 0x48C862, "Simo Wireless,Inc.", "SIMO-WIRELESS" },
	{ 0x48C8B6, "SysTec GmbH", "SYSTEC" },
	{ 0x48CB6E, "Cello Electronics (UK) Ltd", "CELLO-ELECTRONICS-UK" },
	{ 0x48D0CF, "Universal Electronics, Inc.", "UNIVERSAL-ELECTRONICS" },
	{ 0x48D18E, "Metis Communication Co.,Ltd", "METIS-COMMUNICATION" },
	{ 0x48D224, "Liteon Technology Corporation", "LITEON" },
	{ 0x48D54C, "Jeda Networks", "JEDA-NETWORKS" },
	{ 0x48D705, "Apple", "APPLE" },
	{ 0x48D7FF, "BLANKOM Antennentechnik GmbH", "BLANKOM-ANTENNENTECHNIK" },
	{ 0x48D855, "Telvent", "TELVENT" },
	{ 0x48D8FE, "ClarIDy Solutions, Inc.", "CLARIDY-SOLUTIONS" },
	{ 0x48DCFB, "Nokia Corporation", "NOKIA" },
	{ 0x48DF1C, "Wuhan NEC Fibre Optic Communications industry Co. Ltd", "WUHAN-NEC-FIBRE-OPTIC-COMMUNICATION-INDUSTRY" },
	{ 0x48E1AF, "Vity", "VITY" },
	{ 0x48E9F1, "Apple", "APPLE" },
	{ 0x48EA63, "Zhejiang Uniview Technologies Co., Ltd.", "ZHEJIANG-UNIVIEW-TECHNOLOGIES" },
	{ 0x48EB30, "ETERNA TECHNOLOGY, INC.", "ETERNA" },
	{ 0x48ED80, "daesung eltec", "DAESUNG-ELTEC" },
	{ 0x48EE07, "Silver Palm Technologies LLC", "SILVER-PALM-TECHNOLOGIES-LLC" },
	{ 0x48EE0C, "D-Link International", "D-LINK" },
	{ 0x48EE86, "UTStarcom (China) Co.,Ltd", "UTSTARCOM-CHINA" },
	{ 0x48F230, "Ubizcore Co.,LTD", "UBIZCORE" },
	{ 0x48F317, "PRIVATE", "PRIVATE" },
	{ 0x48F47D, "TechVision Holding Internation Limited", "TECHVISION-HOLDING-INTERNATION" },
	{ 0x48F7F1, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x48F8B3, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x48F8E1, "Alcatel Lucent WT", "ALCATEL-LUCENT-WT" },
	{ 0x48F925, "Maestronic", "MAESTRONIC" },
	{ 0x48FCB8, "Woodstream Corporation", "WOODSTREAM" },
	{ 0x48FEEA, "HOMA B.V.", "HOMA" },
	{ 0x4C0082, "Cisco", "CISCO" },
	{ 0x4C022E, "CMR KOREA CO., LTD", "CMR-KOREA" },
	{ 0x4C0289, "LEX COMPUTECH CO., LTD", "LEX-COMPUTECH" },
	{ 0x4C068A, "Basler Electric Company", "BASLER-ELECTRIC-COMPANY" },
	{ 0x4C07C9, "COMPUTER OFFICE Co.,Ltd.", "OFFICE" },
	{ 0x4C09B4, "zte corporation", "ZTE" },
	{ 0x4C09D4, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x4C0B3A, "TCT Mobile Limited", "TCT-MOBILE" },
	{ 0x4C0BBE, "Microsoft", "MICROSOFT" },
	{ 0x4C0DEE, "JABIL CIRCUIT (SHANGHAI) LTD.", "JABIL-CIRCUIT-SHANGHAI" },
	{ 0x4C0F6E, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x4C0FC7, "Earda Electronics Co.,Ltd", "EARDA-ELECTRONICS" },
	{ 0x4C11BF, "ZHEJIANG DAHUA TECHNOLOGY CO.,LTD.", "ZHEJIANG-DAHUA" },
	{ 0x4C1480, "NOREGON SYSTEMS, INC", "NOREGON" },
	{ 0x4C14A3, "TCL Technoly Electronics (Huizhou) Co., Ltd.", "TCL-TECHNOLY-ELECTRONICS-HUIZHOU" },
	{ 0x4C16F1, "zte corporation", "ZTE" },
	{ 0x4C17EB, "SAGEMCOM", "SAGEMCOM" },
	{ 0x4C1A3A, "PRIMA Research And Production Enterprise Ltd.", "PRIMA-RESEARCH-AND-PRODUCTION" },
	{ 0x4C1A95, "Novakon Co., Ltd.", "NOVAKON" },
	{ 0x4C1FCC, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x4C21D0, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x4C2258, "cozybit, Inc.", "COZYBIT" },
	{ 0x4C2578, "Nokia Corporation", "NOKIA" },
	{ 0x4C26E7, "Welgate Co., Ltd.", "WELGATE" },
	{ 0x4C2C80, "Beijing Skyway Technologies Co.,Ltd", "BEIJING-SKYWAY-TECHNOLOGIES" },
	{ 0x4C2C83, "Zhejiang KaNong Network Technology Co.,Ltd.", "ZHEJIANG-KANONG-NETWORK" },
	{ 0x4C2F9D, "ICM Controls", "ICM-CONTROLS" },
	{ 0x4C3089, "Thales Transportation Systems GmbH", "THALES-TRANSPORTATION" },
	{ 0x4C322D, "TELEDATA NETWORKS", "TELEDATA-NETWORKS" },
	{ 0x4C32D9, "M Rutty Holdings Pty. Ltd.", "M-RUTTY-HOLDINGS-PTY" },
	{ 0x4C3909, "HPL Electric & Power Private Limited", "HPL-ELECTRIC-POWER-PRIVATE" },
	{ 0x4C3910, "Newtek Electronics co., Ltd.", "NEWTEK-ELECTRONICS" },
	{ 0x4C3B74, "VOGTEC(H.K.) Co., Ltd", "VOGTEC-H-K" },
	{ 0x4C3C16, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x4C48DA, "Beijing Autelan Technology Co.,Ltd", "BEIJING-AUTELAN" },
	{ 0x4C4B68, "Mobile Device, Inc.", "MOBILE-DEVICE" },
	{ 0x4C4E35, "Cisco", "CISCO" },
	{ 0x4C5427, "Linepro Sp. z o.o.", "LINEPRO-SP-Z-O-O" },
	{ 0x4C5499, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x4C5585, "Hamilton Systems", "HAMILTON" },
	{ 0x4C55B8, "Turkcell Teknoloji", "TURKCELL-TEKNOLOJI" },
	{ 0x4C55CC, "ACKme Networks Pty Ltd", "ACKME-NETWORKS-PTY" },
	{ 0x4C5DCD, "Oy Finnish Electric Vehicle Technologies Ltd", "OY-FINNISH-ELECTRIC-VEHICLE-TECHNOLOGIES" },
	{ 0x4C5E0C, "Routerboard.com", "ROUTERBOARD-COM" },
	{ 0x4C5FD2, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x4C60D5, "airPointe of New Hampshire", "AIRPOINTE-OF-NEW-HAMPSHIRE" },
	{ 0x4C60DE, "NETGEAR", "NETGEAR" },
	{ 0x4C6255, "SANMINA-SCI SYSTEM DE MEXICO S.A. DE C.V.", "SANMINA-SCI-SYSTEM-DE-MEXICO-DE-C-V" },
	{ 0x4C63EB, "Application Solutions (Electronics and Vision) Ltd", "APPLICATION-SOLUTIONS-ELECTRONICS-AND-VISION" },
	{ 0x4C64D9, "Guangdong Leawin Group Co., Ltd", "GUANGDONG-LEAWIN-GROUP" },
	{ 0x4C6E6E, "Comnect Technology CO.,LTD", "COMNECT" },
	{ 0x4C72B9, "Pegatron Corporation", "PEGATRON" },
	{ 0x4C7367, "Genius Bytes Software Solutions GmbH", "GENIUS-BYTES-SOFTWARE-SOLUTIONS" },
	{ 0x4C73A5, "KOVE", "KOVE" },
	{ 0x4C7403, "Mundo Reader (bq)", "MUNDO-READER-BQ" },
	{ 0x4C7625, "Dell Inc.", "DELL" },
	{ 0x4C774F, "Embedded Wireless Labs", "EMBEDDED-WIRELESS-LABS" },
	{ 0x4C7897, "Arrowhead Alarm Products Ltd", "ARROWHEAD-ALARM-PRODUCTS" },
	{ 0x4C79BA, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x4C7C5F, "Apple", "APPLE" },
	{ 0x4C7F62, "Nokia Corporation", "NOKIA" },
	{ 0x4C804F, "Armstrong Monitoring Corp", "ARMSTRONG-MONITORING" },
	{ 0x4C8093, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x4C82CF, "Echostar Technologies", "ECHOSTAR-TECHNOLOGIES" },
	{ 0x4C83DE, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x4C8B30, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x4C8B55, "Grupo Digicon", "GRUPO-DIGICON" },
	{ 0x4C8BEF, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x4C8D79, "Apple", "APPLE" },
	{ 0x4C8FA5, "Jastec", "JASTEC" },
	{ 0x4C9614, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x4C98EF, "Zeo", "ZEO" },
	{ 0x4C9E80, "KYOKKO ELECTRIC Co., Ltd.", "KYOKKO-ELECTRIC" },
	{ 0x4C9EE4, "Hanyang Navicom Co.,Ltd.", "HANYANG-NAVICOM" },
	{ 0x4C9EFF, "ZyXEL Communications Corp", "ZYXEL-COMMUNICATION" },
	{ 0x4CA515, "Baikal Electronics JSC", "BAIKAL-ELECTRONICS-JSC" },
	{ 0x4CA56D, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x4CA74B, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x4CA928, "Insensi", "INSENSI" },
	{ 0x4CAA16, "AzureWave Technologies (Shanghai) Inc.", "AZUREWAVE-TECHNOLOGIES-SHANGHAI" },
	{ 0x4CAB33, "KST technology", "KST" },
	{ 0x4CAC0A, "ZTE Corporation", "ZTE" },
	{ 0x4CB16C, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x4CB199, "Apple", "APPLE" },
	{ 0x4CB4EA, "HRD (S) PTE., LTD.", "HRD-S-PTE" },
	{ 0x4CB81C, "SAM Electronics GmbH", "SAM-ELECTRONICS" },
	{ 0x4CB9C8, "CONET CO., LTD.", "CONET" },
	{ 0x4CBAA3, "Bison Electronics Inc.", "BISON-ELECTRONICS" },
	{ 0x4CBB58, "Chicony Electronics Co., Ltd.", "CHICONY-ELECTRONICS" },
	{ 0x4CBC42, "Shenzhen Hangsheng Electronics Co.,Ltd.", "SHENZHEN-HANGSHENG-ELECTRONICS" },
	{ 0x4CBCA5, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x4CC452, "Shang Hai Tyd. Electon Technology Ltd.", "SHANG-HAI-TYD-ELECTON" },
	{ 0x4CC602, "Radios, Inc.", "RADIOS" },
	{ 0x4CC94F, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x4CCA53, "Skyera, Inc.", "SKYERA" },
	{ 0x4CCBF5, "zte corporation", "ZTE" },
	{ 0x4CCC34, "Motorola Solutions Inc.", "MOTOROLA-SOLUTIONS" },
	{ 0x4CD08A, "HUMAX.CO.,LTD", "HUMAX" },
	{ 0x4CD637, "Qsono Electronics Co., Ltd", "QSONO-ELECTRONICS" },
	{ 0x4CD7B6, "Helmer Scientific", "HELMER-SCIENTIFIC" },
	{ 0x4CD9C4, "Magneti Marelli Automotive Electronics (Guangzhou) Co. Ltd", "MAGNETI-MARELLI-AUTOMOTIVE-ELECTRONICS-GUANGZHOU" },
	{ 0x4CDF3D, "TEAM ENGINEERS ADVANCE TECHNOLOGIES INDIA PVT LTD", "TEAM-ENGINEERS-ADVANCE-TECHNOLOGIES-INDIA-PVT" },
	{ 0x4CE1BB, "Zhuhai HiFocus Technology Co., Ltd.", "ZHUHAI-HIFOCUS" },
	{ 0x4CE2F1, "sclak srl", "SCLAK-SRL" },
	{ 0x4CE676, "Buffalo Inc.", "BUFFALO" },
	{ 0x4CE933, "RailComm, LLC", "RAILCOMM-LLC" },
	{ 0x4CEB42, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x4CEDDE, "Askey Computer Corp", "ASKEY" },
	{ 0x4CEEB0, "SHC Netzwerktechnik GmbH", "SHC-NETZWERKTECHNIK" },
	{ 0x4CF02E, "Vifa Denmark A/S", "VIFA-DENMARK-A/S" },
	{ 0x4CF2BF, "Cambridge Industries(Group) Co.,Ltd.", "CAMBRIDGE-INDUSTRIES-GROUP" },
	{ 0x4CF45B, "Blue Clover Devices", "BLUE-CLOVER-DEVICES" },
	{ 0x4CF5A0, "Scalable Network Technologies Inc", "SCALABLE-NETWORK-TECHNOLOGIES" },
	{ 0x4CF737, "SamJi Electronics Co., Ltd", "SAMJI-ELECTRONICS" },
	{ 0x50008C, "Hong Kong Telecommunications (HKT) Limited", "HONG-KONG-TELECOMMUNICATIONS-HKT" },
	{ 0x5001BB, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x50053D, "CyWee Group Ltd", "CYWEE-GROUP" },
	{ 0x500604, "Cisco", "CISCO" },
	{ 0x500B32, "Foxda Technology Industrial(ShenZhen)Co.,LTD", "FOXDA-INDUSTRIAL-SHENZHEN" },
	{ 0x500E6D, "TrafficCast International", "TRAFFICCAST" },
	{ 0x5011EB, "SilverNet Ltd", "SILVERNET" },
	{ 0x5014B5, "Richfit Information Technology Co., Ltd", "RICHFIT-INFORMATION" },
	{ 0x5017FF, "Cisco", "CISCO" },
	{ 0x501AC5, "Microsoft", "MICROSOFT" },
	{ 0x501CBF, "Cisco", "CISCO" },
	{ 0x50206B, "Emerson Climate Technologies Transportation Solutions", "EMERSON-CLIMATE-TECHNOLOGIES-TRANSPORTATION-SOLUTIONS" },
	{ 0x502267, "PixeLINK", "PIXELINK" },
	{ 0x50252B, "Nethra Imaging Incorporated", "NETHRA-IMAGING-INCORPORATED" },
	{ 0x502690, "Fujitsu Limited", "FUJITSU" },
	{ 0x5027C7, "TECHNART Co.,Ltd", "TECHNART" },
	{ 0x50294D, "NANJING IOT SENSOR TECHNOLOGY CO,LTD", "NANJING-IOT-SENSOR" },
	{ 0x502A7E, "Smart electronic GmbH", "SMART" },
	{ 0x502A8B, "Telekom Research and Development Sdn Bhd", "TELEKOM-RESEARCH-AND-DEVELOPMENT-SDN-BHD" },
	{ 0x502D1D, "Nokia Corporation", "NOKIA" },
	{ 0x502DA2, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x502DF4, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x502E5C, "HTC Corporation", "HTC" },
	{ 0x502ECE, "Asahi Electronics Co.,Ltd", "ASAHI-ELECTRONICS" },
	{ 0x503275, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x503955, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x503CC4, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x503DE5, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x503F56, "Syncmold Enterprise Corp", "SYNCMOLD" },
	{ 0x50465D, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x5048EB, "BEIJING HAIHEJINSHENG NETWORK TECHNOLOGY CO. LTD.", "BEIJING-HAIHEJINSHENG-NETWORK" },
	{ 0x504A5E, "Masimo Corporation", "MASIMO" },
	{ 0x504A6E, "NETGEAR INC.,", "NETGEAR" },
	{ 0x504F94, "Loxone Electronics GmbH", "LOXONE-ELECTRONICS" },
	{ 0x50502A, "Egardia", "EGARDIA" },
	{ 0x505065, "TAKT Corporation", "TAKT" },
	{ 0x505527, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x505663, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x5056A8, "Jolla Ltd", "JOLLA" },
	{ 0x5056BF, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x5057A8, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x505800, "WyTec International, Inc.", "WYTEC" },
	{ 0x505AC6, "GUANGDONG SUPER TELECOM CO.,LTD.", "GUANGDONG-SUPER-TELECOM" },
	{ 0x506028, "Xirrus Inc.", "XIRRUS" },
	{ 0x506184, "Avaya, Inc", "AVAYA" },
	{ 0x5061D6, "Indu-Sol GmbH", "INDU-SOL" },
	{ 0x506313, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x506441, "Greenlee", "GREENLEE" },
	{ 0x5065F3, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x506787, "iTellus", "ITELLUS" },
	{ 0x5067AE, "Cisco", "CISCO" },
	{ 0x5067F0, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0x506F9A, "Wi-Fi Alliance", "WI-FI-ALLIANCE" },
	{ 0x5070E5, "He Shan World Fair Electronics Technology Limited", "HE-SHAN-WORLD-FAIR-ELECTRONICS" },
	{ 0x507224, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x50724D, "BEG Brueck Electronic GmbH", "BEG-BRUECK" },
	{ 0x507691, "Tekpea, Inc.", "TEKPEA" },
	{ 0x5076A6, "Ecil Informatica Ind. Com. Ltda", "ECIL-INFORMATICA-IND-COM-LTDA" },
	{ 0x50795B, "Interexport Telecomunicaciones S.A.", "INTEREXPORT-TELECOMUNICACIONES" },
	{ 0x507D02, "BIODIT", "BIODIT" },
	{ 0x507E5D, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x508569, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x508789, "Cisco", "CISCO" },
	{ 0x5087B8, "Nuvyyo Inc", "NUVYYO" },
	{ 0x508A42, "Uptmate Technology Co., LTD", "UPTMATE" },
	{ 0x508ACB, "SHENZHEN MAXMADE TECHNOLOGY CO., LTD.", "SHENZHEN-MAXMADE" },
	{ 0x508C77, "DIRMEIER Schanktechnik GmbH &Co KG", "DIRMEIER-SCHANKTECHNIK" },
	{ 0x508D6F, "CHAHOO Limited", "CHAHOO" },
	{ 0x50934F, "Gradual Tecnologia Ltda.", "GRADUAL-TECNOLOGIA-LTDA" },
	{ 0x509772, "Westinghouse Digital", "WESTINGHOUSE-DIGITAL" },
	{ 0x509871, "Inventum Technologies Private Limited", "INVENTUM-TECHNOLOGIES-PRIVATE" },
	{ 0x509F27, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x50A054, "Actineon", "ACTINEON" },
	{ 0x50A0BF, "Alba Fiber Systems Inc.", "ALBA-FIBER" },
	{ 0x50A4C8, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x50A6E3, "David Clark Company", "DAVID-CLARK-COMPANY" },
	{ 0x50A715, "Aboundi, Inc.", "ABOUNDI" },
	{ 0x50A733, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x50ABBF, "Hoseo Telecom", "HOSEO-TELECOM" },
	{ 0x50ADD5, "Dynalec Corporation", "DYNALEC" },
	{ 0x50AF73, "Shenzhen Bitland Information Technology Co., Ltd.", "SHENZHEN-BITLAND-INFORMATION" },
	{ 0x50B695, "Micropoint Biotechnologies,Inc.", "MICROPOINT-BIOTECHNOLOGIES" },
	{ 0x50B7C3, "Samsung Electronics CO., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x50B888, "wi2be Tecnologia S/A", "WI2BE-TECNOLOGIA-S/A" },
	{ 0x50B8A2, "ImTech Technologies LLC,", "IMTECH-TECHNOLOGIES-LLC" },
	{ 0x50BD5F, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x50C006, "Carmanah Signs", "CARMANAH-SIGNS" },
	{ 0x50C271, "SECURETECH INC", "SECURETECH" },
	{ 0x50C58D, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x50C7BF, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x50C971, "GN Netcom A/S", "GN-NETCOM-A/S" },
	{ 0x50C9A0, "SKIPPER Electronics AS", "SKIPPER-ELECTRONICS-AS" },
	{ 0x50CCF8, "Samsung Electro Mechanics", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x50CD32, "NanJing Chaoran Science & Technology Co.,Ltd.", "NANJING-CHAORAN-SCIENCE" },
	{ 0x50CE75, "Measy Electronics Ltd", "MEASY-ELECTRONICS" },
	{ 0x50D274, "Steffes Corporation", "STEFFES" },
	{ 0x50D6D7, "Takahata Precision", "TAKAHATA-PRECISION" },
	{ 0x50E0C7, "TurControlSystme AG", "TURCONTROLSYSTME" },
	{ 0x50E14A, "PRIVATE", "PRIVATE" },
	{ 0x50E549, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x50EAD6, "Apple", "APPLE" },
	{ 0x50EB1A, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x50ED78, "Changzhou Yongse Infotech Co.,Ltd", "CHANGZHOU-YONGSE-INFOTECH" },
	{ 0x50ED94, "Egatel SL", "EGATEL-SL" },
	{ 0x50F003, "Open Stack, Inc.", "OPEN-STACK" },
	{ 0x50F43C, "Leeo Inc", "LEEO" },
	{ 0x50F520, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x50F61A, "Kunshan JADE Technologies co., Ltd.", "KUNSHAN-JADE-TECHNOLOGIES" },
	{ 0x50FAAB, "L-tek d.o.o.", "L-TEK-D-O-O" },
	{ 0x50FC30, "Treehouse Labs", "TREEHOUSE-LABS" },
	{ 0x50FC9F, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x50FEF2, "Sify Technologies Ltd", "SIFY-TECHNOLOGIES" },
	{ 0x5403F5, "EBN Technology Corp.", "EBN" },
	{ 0x540496, "Gigawave LTD", "GIGAWAVE" },
	{ 0x5404A6, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x540536, "Vivago Oy", "VIVAGO-OY" },
	{ 0x54055F, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x54098D, "deister electronic GmbH", "DEISTER" },
	{ 0x54112F, "Sulzer Pump Solutions Finland Oy", "SULZER-PUMP-SOLUTIONS-FINLAND-OY" },
	{ 0x54115F, "Atamo Pty Ltd", "ATAMO-PTY" },
	{ 0x541473, "Wingtech Group (HongKong) Limited", "WINGTECH-GROUP-HONGKONG" },
	{ 0x541B5D, "Techno-Innov", "TECHNO-INNOV" },
	{ 0x541DFB, "Freestyle Energy Ltd", "FREESTYLE-ENERGY" },
	{ 0x541FD5, "Advantage Electronics", "ADVANTAGE-ELECTRONICS" },
	{ 0x542018, "Tely Labs", "TELY-LABS" },
	{ 0x542160, "Resolution Products", "RESOLUTION-PRODUCTS" },
	{ 0x5422F8, "zte corporation", "ZTE" },
	{ 0x542696, "Apple", "APPLE" },
	{ 0x54271E, "AzureWave Technonloies, Inc.", "AZUREWAVE-TECHNONLOIES" },
	{ 0x542A9C, "LSY Defense, LLC.", "LSY-DEFENSE-LLC" },
	{ 0x542AA2, "Alpha Networks Inc.", "ALPHA-NETWORKS" },
	{ 0x542CEA, "PROTECTRON", "PROTECTRON" },
	{ 0x542F89, "Euclid Laboratories, Inc.", "EUCLID-LABORATORIES" },
	{ 0x543131, "Raster Vision Ltd", "RASTER-VISION" },
	{ 0x543530, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x5435DF, "Symeo GmbH", "SYMEO" },
	{ 0x54369B, "In one network technology (Beijing) Co., Ltd.", "IN-ONE-NETWORK-BEIJING" },
	{ 0x543968, "Edgewater Networks Inc", "EDGEWATER-NETWORKS" },
	{ 0x5439DF, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x543D37, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x544249, "Sony Corporation", "SONY" },
	{ 0x544408, "Nokia Corporation", "NOKIA" },
	{ 0x54466B, "Shenzhen CZTIC Electronic Technology Co., Ltd", "SHENZHEN-CZTIC" },
	{ 0x544A00, "Cisco", "CISCO" },
	{ 0x544A05, "wenglor sensoric gmbh", "WENGLOR-SENSORIC" },
	{ 0x544A16, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x545146, "AMG Systems Ltd.", "AMG" },
	{ 0x5453ED, "Sony Corporation", "SONY" },
	{ 0x545414, "Digital RF Corea, Inc", "DIGITAL-RF-COREA" },
	{ 0x545EBD, "NL Technologies", "NL-TECHNOLOGIES" },
	{ 0x545FA9, "Teracom Limited", "TERACOM" },
	{ 0x5461EA, "Zaplox AB", "ZAPLOX-AB" },
	{ 0x5465DE, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x54724F, "Apple", "APPLE" },
	{ 0x547398, "Toyo Electronics Corporation", "TOYO-ELECTRONICS" },
	{ 0x5474E6, "Webtech Wireless", "WEBTECH-WIRELESS" },
	{ 0x5475D0, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x54781A, "Cisco", "CISCO" },
	{ 0x547975, "Nokia Corporation", "NOKIA" },
	{ 0x547C69, "Cisco", "CISCO" },
	{ 0x547F54, "INGENICO", "INGENICO" },
	{ 0x547FA8, "TELCO systems, s.r.o.", "TELCO-S-R-O" },
	{ 0x547FEE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x5481AD, "Eagle Research Corporation", "EAGLE-RESEARCH" },
	{ 0x54847B, "Digital Devices GmbH", "DIGITAL-DEVICES" },
	{ 0x54880E, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x548922, "Zelfy Inc", "ZELFY" },
	{ 0x548998, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x5492BE, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x549359, "SHENZHEN TWOWING TECHNOLOGIES CO.,LTD.", "SHENZHEN-TWOWING-TECHNOLOGIES" },
	{ 0x549478, "Silvershore Technology Partners", "SILVERSHORE-PARTNERS" },
	{ 0x549A16, "Uzushio Electric Co.,Ltd.", "UZUSHIO-ELECTRIC" },
	{ 0x549B12, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x549D85, "EnerAccess inc", "ENERACCESS" },
	{ 0x549F13, "Apple", "APPLE" },
	{ 0x549F35, "Dell Inc.", "DELL" },
	{ 0x54A04F, "t-mac Technologies Ltd", "T-MAC-TECHNOLOGIES" },
	{ 0x54A050, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x54A31B, "Shenzhen Linkworld Technology Co,.LTD", "SHENZHEN-LINKWORLD" },
	{ 0x54A51B, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x54A54B, "NSC Communications Siberia Ltd", "NSC-COMMUNICATION-SIBERIA" },
	{ 0x54A619, "Alcatel-Lucent Shanghai Bell Co., Ltd", "ALCATEL-LUCENT-SHANGHAI-BELL" },
	{ 0x54A9D4, "Minibar Systems", "MINIBAR" },
	{ 0x54AE27, "Apple", "APPLE" },
	{ 0x54B620, "SUHDOL E&C Co.Ltd.", "SUHDOL-E-C" },
	{ 0x54B753, "Hunan Fenghui Yinjia Science And Technology Co.,Ltd", "HUNAN-FENGHUI-YINJIA-SCIENCE-AND" },
	{ 0x54B80A, "D-Link International", "D-LINK" },
	{ 0x54BEF7, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x54C80F, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x54CD10, "Panasonic Mobile Communications Co.,Ltd.", "PANASONIC-MOBILE-COMMUNICATION" },
	{ 0x54CDA7, "Fujian Shenzhou Electronic Co.,Ltd", "FUJIAN-SHENZHOU" },
	{ 0x54CDEE, "ShenZhen Apexis Electronic Co.,Ltd", "SHENZHEN-APEXIS" },
	{ 0x54D0ED, "AXIM Communications", "AXIM-COMMUNICATION" },
	{ 0x54D163, "MAX-TECH,INC", "MAX-TECH" },
	{ 0x54D1B0, "Universal Laser Systems, Inc", "UNIVERSAL-LASER" },
	{ 0x54D46F, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x54DF00, "Ulterius Technologies, LLC", "ULTERIUS-TECHNOLOGIES-LLC" },
	{ 0x54DF63, "Intrakey technologies GmbH", "INTRAKEY-TECHNOLOGIES" },
	{ 0x54E032, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x54E140, "INGENICO", "INGENICO" },
	{ 0x54E2C8, "Dongguan Aoyuan Electronics Technology Co., Ltd", "DONGGUAN-AOYUAN-ELECTRONICS" },
	{ 0x54E2E0, "Pace plc", "PACE-PLC" },
	{ 0x54E3B0, "JVL Industri Elektronik", "JVL-INDUSTRI-ELEKTRONIK" },
	{ 0x54E43A, "Apple, Inc.", "APPLE" },
	{ 0x54E4BD, "FN-LINK TECHNOLOGY LIMITED", "FN-LINK" },
	{ 0x54E63F, "ShenZhen LingKeWeiEr Technology Co., Ltd.", "SHENZHEN-LINGKEWEIER" },
	{ 0x54E6FC, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x54EAA8, "Apple, Inc.", "APPLE" },
	{ 0x54EE75, "Wistron InfoComm(Kunshan)Co.,Ltd.", "WISTRON-INFOCOMM-KUNSHAN" },
	{ 0x54EF92, "Shenzhen Elink Technology Co., LTD", "SHENZHEN-ELINK" },
	{ 0x54F5B6, "ORIENTAL PACIFIC INTERNATIONAL LIMITED", "ORIENTAL-PACIFIC" },
	{ 0x54F666, "Berthold Technologies GmbH and Co.KG", "BERTHOLD-TECHNOLOGIES-AND" },
	{ 0x54F876, "ABB AG", "ABB" },
	{ 0x54FA3E, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x54FB58, "WISEWARE, Lda", "WISEWARE-LDA" },
	{ 0x54FDBF, "Scheidt & Bachmann GmbH", "SCHEIDT-BACHMANN" },
	{ 0x54FFCF, "Mopria Alliance", "MOPRIA-ALLIANCE" },
	{ 0x5804CB, "Tianjin Huisun Technology Co.,Ltd.", "TIANJIN-HUISUN" },
	{ 0x580528, "LABRIS NETWORKS", "LABRIS-NETWORKS" },
	{ 0x580556, "Elettronica GF S.r.L.", "ELETTRONICA-GF-S-R-L" },
	{ 0x5808FA, "Fiber Optic & telecommunication INC.", "FIBER-OPTIC-TELECOMMUNICATION" },
	{ 0x580943, "PRIVATE", "PRIVATE" },
	{ 0x5809E5, "Kivic Inc.", "KIVIC" },
	{ 0x580A20, "Cisco", "CISCO" },
	{ 0x58108C, "Intelbras", "INTELBRAS" },
	{ 0x581243, "AcSiP Technology Corp.", "ACSIP" },
	{ 0x581626, "Avaya, Inc", "AVAYA" },
	{ 0x58170C, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x581CBD, "Affinegy", "AFFINEGY" },
	{ 0x581D91, "Advanced Mobile Telecom co.,ltd.", "ADVANCED-MOBILE-TELECOM" },
	{ 0x581F28, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x581F67, "Open-m technology limited", "OPEN-M" },
	{ 0x581FAA, "Apple", "APPLE" },
	{ 0x581FEF, "Tuttnaer LTD", "TUTTNAER" },
	{ 0x582136, "KMB systems, s.r.o.", "KMB-S-R-O" },
	{ 0x58238C, "Technicolor CH USA", "TECHNICOLOR-CH-USA" },
	{ 0x582AF7, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x582EFE, "Lighting Science Group", "LIGHTING-SCIENCE-GROUP" },
	{ 0x582F42, "Universal Electric Corporation", "UNIVERSAL-ELECTRIC" },
	{ 0x58343B, "Glovast Technology Ltd.", "GLOVAST" },
	{ 0x5835D9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x583CC6, "Omneality Ltd.", "OMNEALITY" },
	{ 0x583F54, "LG Electronics (Mobile Communications)", "LG-ELECTRONICS-MOBILE-COMMUNICATION" },
	{ 0x5842E4, "Sigma International General Medical Apparatus, LLC.", "SIGMA-GENERAL-MEDICAL-APPARATUS-LLC" },
	{ 0x58468F, "Koncar Electronics and Informatics", "KONCAR-ELECTRONICS-AND-INFORMATICS" },
	{ 0x5846E1, "Baxter Healthcare", "BAXTER-HEALTHCARE" },
	{ 0x584704, "Shenzhen Webridge Technology Co.,Ltd", "SHENZHEN-WEBRIDGE" },
	{ 0x5848C0, "COFLEC", "COFLEC" },
	{ 0x58493B, "Palo Alto Networks", "PALO-ALTO-NETWORKS" },
	{ 0x5849BA, "Chitai Electronic Corp.", "CHITAI" },
	{ 0x584C19, "Chongqing Guohong Technology Development Company Limited", "CHONGQING-GUOHONG-DEVELOPMENT-COMPANY" },
	{ 0x584CEE, "Digital One Technologies, Limited", "DIGITAL-ONE-TECHNOLOGIES" },
	{ 0x585076, "Linear Equipamentos Eletronicos SA", "LINEAR-EQUIPAMENTOS-ELETRONICOS" },
	{ 0x5850AB, "TLS Corporation", "TLS" },
	{ 0x5850E6, "Best Buy Corporation", "BEST-BUY" },
	{ 0x5855CA, "Apple", "APPLE" },
	{ 0x5856E8, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x58570D, "Danfoss Solar Inverters", "DANFOSS-SOLAR-INVERTERS" },
	{ 0x58639A, "TPL SYSTEMES", "TPL-SYSTEMES" },
	{ 0x5865E6, "INFOMARK CO., LTD.", "INFOMARK" },
	{ 0x5866BA, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x58671A, "BARNES&NOBLE.COM", "BARNES-NOBLE-COM" },
	{ 0x58677F, "Clare Controls Inc.", "CLARE-CONTROLS" },
	{ 0x58696C, "Fujian Ruijie Networks co, ltd", "FUJIAN-RUIJIE-NETWORKS" },
	{ 0x5869F9, "Fusion Transactive Ltd.", "FUSION-TRANSACTIVE" },
	{ 0x586AB1, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x586D8F, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x586ED6, "PRIVATE", "PRIVATE" },
	{ 0x5870C6, "Shanghai Xiaoyi Technology Co., Ltd.", "SHANGHAI-XIAOYI" },
	{ 0x587521, "CJSC RTSoft", "CJSC-RTSOFT" },
	{ 0x587675, "Beijing ECHO Technologies Co.,Ltd", "BEIJING-ECHO-TECHNOLOGIES" },
	{ 0x5876C5, "DIGI I'S LTD", "DIGI-I-S" },
	{ 0x587A4D, "Stonesoft Corporation", "STONESOFT" },
	{ 0x587BE9, "AirPro Technology India Pvt. Ltd", "AIRPRO-INDIA-PVT" },
	{ 0x587E61, "Hisense Electric Co., Ltd", "HISENSE-ELECTRIC" },
	{ 0x587F66, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x587FB7, "SONAR INDUSTRIAL CO., LTD.", "SONAR-INDUSTRIAL" },
	{ 0x587FC8, "S2M", "S2M" },
	{ 0x5884E4, "IP500 Alliance e.V.", "IP500-ALLIANCE-E-V" },
	{ 0x58856E, "QSC AG", "QSC" },
	{ 0x58874C, "LITE-ON CLEAN ENERGY TECHNOLOGY CORP.", "LITE-ON-CLEAN-ENERGY" },
	{ 0x5887E2, "Shenzhen Coship Electronics Co., Ltd.", "SHENZHEN-COSHIP-ELECTRONICS" },
	{ 0x588D09, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x5891CF, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x58920D, "Kinetic Avionics Limited", "KINETIC-AVIONICS" },
	{ 0x589396, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x58946B, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x5894CF, "Vertex Standard LMR, Inc.", "VERTEX-STANDARD-LMR" },
	{ 0x58971E, "Cisco", "CISCO" },
	{ 0x589835, "Technicolor", "TECHNICOLOR" },
	{ 0x58986F, "Revolution Display", "REVOLUTION-DISPLAY" },
	{ 0x589B0B, "Shineway Technologies, Inc.", "SHINEWAY-TECHNOLOGIES" },
	{ 0x589CFC, "FreeBSD Foundation", "FREEBSD-FOUNDATION" },
	{ 0x58A2B5, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x58A76F, "iD corporation", "ID" },
	{ 0x58A839, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x58B035, "Apple", "APPLE" },
	{ 0x58B0D4, "ZuniData Systems Inc.", "ZUNIDATA" },
	{ 0x58B961, "SOLEM Electronique", "SOLEM-ELECTRONIQUE" },
	{ 0x58B9E1, "Crystalfontz America, Inc.", "CRYSTALFONTZ-AMERICA" },
	{ 0x58BC27, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x58BDA3, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x58BDF9, "Sigrand", "SIGRAND" },
	{ 0x58BFEA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x58C232, "NEC Corporation", "NEC" },
	{ 0x58C38B, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x58CF4B, "Lufkin Industries", "LUFKIN-INDUSTRIES" },
	{ 0x58D071, "BW Broadcast", "BW-BROADCAST" },
	{ 0x58D08F, "IEEE 1904.1 Working Group", "IEEE-1904-1-WORKING-GROUP" },
	{ 0x58D6D3, "Dairy Cheq Inc", "DAIRY-CHEQ" },
	{ 0x58DB8D, "Fast Co., Ltd.", "FAST" },
	{ 0x58E02C, "Micro Technic A/S", "MICRO-TECHNIC-A/S" },
	{ 0x58E326, "Compass Technologies Inc.", "COMPASS-TECHNOLOGIES" },
	{ 0x58E476, "CENTRON COMMUNICATIONS TECHNOLOGIES FUJIAN CO.,LTD", "CENTRON-COMMUNICATION-TECHNOLOGIES-FUJIAN" },
	{ 0x58E636, "EVRsafe Technologies", "EVRSAFE-TECHNOLOGIES" },
	{ 0x58E747, "Deltanet AG", "DELTANET" },
	{ 0x58E808, "AUTONICS CORPORATION", "AUTONICS" },
	{ 0x58EB14, "Proteus Digital Health", "PROTEUS-DIGITAL-HEALTH" },
	{ 0x58ECE1, "Newport Corporation", "NEWPORT" },
	{ 0x58EECE, "Icon Time Systems", "ICON-TIME" },
	{ 0x58F387, "HCCP", "HCCP" },
	{ 0x58F39C, "Cisco", "CISCO" },
	{ 0x58F67B, "Xia Men UnionCore Technology LTD.", "XIA-MEN-UNIONCORE" },
	{ 0x58F6BF, "Kyoto University", "KYOTO-UNIVERSITY" },
	{ 0x58F98E, "SECUDOS GmbH", "SECUDOS" },
	{ 0x58FCDB, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x58FD20, "Bravida Sakerhet AB", "BRAVIDA-SAKERHET-AB" },
	{ 0x5C026A, "Applied Vision Corporation", "APPLIED-VISION" },
	{ 0x5C076F, "Thought Creator", "THOUGHT-CREATOR" },
	{ 0x5C0A5B, "SAMSUNG ELECTRO-MECHANICS CO., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x5C0CBB, "CELIZION Inc.", "CELIZION" },
	{ 0x5C0E8B, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0x5C1193, "Seal One AG", "SEAL-ONE" },
	{ 0x5C1437, "Thyssenkrupp Aufzugswerke GmbH", "THYSSENKRUPP-AUFZUGSWERKE" },
	{ 0x5C1515, "ADVAN", "ADVAN" },
	{ 0x5C15E1, "AIDC TECHNOLOGY (S) PTE LTD", "AIDC-S-PTE" },
	{ 0x5C16C7, "Big Switch Networks", "BIG-SWITCH-NETWORKS" },
	{ 0x5C1737, "I-View Now, LLC.", "I-VIEW-NOW-LLC" },
	{ 0x5C17D3, "LGE", "LGE" },
	{ 0x5C18B5, "Talon Communications", "TALON-COMMUNICATION" },
	{ 0x5C20D0, "Asoni Communication Co., Ltd.", "ASONI-COMMUNICATION" },
	{ 0x5C22C4, "DAE EUN ELETRONICS CO., LTD", "DAE-EUN-ELETRONICS" },
	{ 0x5C2479, "Baltech AG", "BALTECH" },
	{ 0x5C254C, "Avire Global Pte Ltd", "AVIRE-GLOBAL-PTE" },
	{ 0x5C260A, "Dell Inc.", "DELL" },
	{ 0x5C2AEF, "Open Access Pty Ltd", "OPEN-ACCESS-PTY" },
	{ 0x5C2BF5, "Vivint", "VIVINT" },
	{ 0x5C2E59, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x5C2ED2, "ABC(XiSheng) Electronics Co.,Ltd", "ABC-XISHENG-ELECTRONICS" },
	{ 0x5C313E, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x5C3327, "Spazio Italia srl", "SPAZIO-ITALIA-SRL" },
	{ 0x5C335C, "Swissphone Telecom AG", "SWISSPHONE-TELECOM" },
	{ 0x5C338E, "Alpha Networkc Inc.", "ALPHA-NETWORKC" },
	{ 0x5C353B, "Compal Broadband Networks Inc.", "COMPAL-BROADBAND-NETWORKS" },
	{ 0x5C35DA, "There Corporation Oy", "THERE-OY" },
	{ 0x5C36B8, "TCL King Electrical Appliances (Huizhou) Ltd.", "TCL-KING-ELECTRICAL-APPLIANCES-HUIZHOU" },
	{ 0x5C38E0, "Shanghai Super Electronics Technology Co.,LTD", "SHANGHAI-SUPER-ELECTRONICS" },
	{ 0x5C3B35, "Gehirn Inc.", "GEHIRN" },
	{ 0x5C3C27, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x5C4058, "Jefferson Audio Video Systems, Inc.", "JEFFERSON-AUDIO-VIDEO" },
	{ 0x5C41E7, "Wiatec International Ltd.", "WIATEC" },
	{ 0x5C43D2, "HAZEMEYER", "HAZEMEYER" },
	{ 0x5C4A26, "Enguity Technology Corp", "ENGUITY" },
	{ 0x5C4CA9, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x5C5015, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x5C514F, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x5C56ED, "3pleplay Electronics Private Limited", "3PLEPLAY-ELECTRONICS-PRIVATE" },
	{ 0x5C571A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x5C57C8, "Nokia Corporation", "NOKIA" },
	{ 0x5C5948, "Apple", "APPLE" },
	{ 0x5C5B35, "Mist Systems, Inc.", "MIST" },
	{ 0x5C5BC2, "YIK Corporation", "YIK" },
	{ 0x5C5EAB, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x5C63BF, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x5C6984, "NUVICO", "NUVICO" },
	{ 0x5C6A7D, "KENTKART EGE ELEKTRONIK SAN. VE TIC. LTD. STI.", "KENTKART-EGE-ELEKTRONIK-SAN-VE-TIC-STI" },
	{ 0x5C6B32, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x5C6B4F, "PRIVATE", "PRIVATE" },
	{ 0x5C6D20, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x5C6F4F, "S.A. SISTEL", "SISTEL" },
	{ 0x5C7757, "Haivision Network Video", "HAIVISION-NETWORK-VIDEO" },
	{ 0x5C7D5E, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x5C8486, "Brightsource Industries Israel LTD", "BRIGHTSOURCE-INDUSTRIES-ISRAEL" },
	{ 0x5C864A, "Secret Labs LLC", "SECRET-LABS-LLC" },
	{ 0x5C8778, "Cybertelbridge co.,ltd", "CYBERTELBRIDGE" },
	{ 0x5C899A, "TP-LINK TECHNOLOGIES CO.,LTD", "TP-LINK-TECHNOLOGIES" },
	{ 0x5C89D4, "Beijing Banner Electric Co.,Ltd", "BEIJING-BANNER-ELECTRIC" },
	{ 0x5C8A38, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x5C8D4E, "Apple", "APPLE" },
	{ 0x5C8FE0, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x5C93A2, "Liteon Technology Corporation", "LITEON" },
	{ 0x5C95AE, "Apple", "APPLE" },
	{ 0x5C966A, "RTNET", "RTNET" },
	{ 0x5C969D, "Apple", "APPLE" },
	{ 0x5C97F3, "Apple", "APPLE" },
	{ 0x5C9AD8, "Fujitsu Limited", "FUJITSU" },
	{ 0x5CA178, "TableTop Media (dba Ziosk)", "TABLETOP-MEDIA-DBA-ZIOSK" },
	{ 0x5CA39D, "SAMSUNG ELECTRO-MECHANICS CO., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x5CA3EB, "Lokel s.r.o.", "LOKEL-S-R-O" },
	{ 0x5CA48A, "Cisco", "CISCO" },
	{ 0x5CAAFD, "Sonos, Inc.", "SONOS" },
	{ 0x5CAC4C, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x5CB43E, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x5CB524, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x5CB6CC, "NovaComm Technologies Inc.", "NOVACOMM-TECHNOLOGIES" },
	{ 0x5CB8CB, "Allis Communications", "ALLIS-COMMUNICATION" },
	{ 0x5CB901, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x5CBD9E, "HONGKONG MIRACLE EAGLE TECHNOLOGY(GROUP) LIMITED", "HONGKONG-MIRACLE-EAGLE-GROUP" },
	{ 0x5CC213, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x5CC5D4, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x5CC6D0, "Skyworth Digital technology(shenzhen)co.ltd.", "SKYWORTH-DIGITAL-SHENZHEN" },
	{ 0x5CC9D3, "PALLADIUM ENERGY ELETRONICA DA AMAZONIA LTDA", "PALLADIUM-ENERGY-ELETRONICA-DA-AMAZONIA-LTDA" },
	{ 0x5CCA32, "Theben AG", "THEBEN" },
	{ 0x5CCCFF, "Techroutes Network Pvt Ltd", "TECHROUTES-NETWORK-PVT" },
	{ 0x5CCEAD, "CDYNE Corporation", "CDYNE" },
	{ 0x5CD135, "Xtreme Power Systems", "XTREME-POWER" },
	{ 0x5CD2E4, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x5CD41B, "UCZOON Technology Co., LTD", "UCZOON" },
	{ 0x5CD4AB, "Zektor", "ZEKTOR" },
	{ 0x5CD61F, "Qardio, Inc", "QARDIO" },
	{ 0x5CD998, "D-Link Corporation", "D-LINK" },
	{ 0x5CDAD4, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x5CDC96, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x5CDD70, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x5CE0C5, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x5CE0CA, "FeiTian United (Beijing) System Technology Co., Ltd.", "FEITIAN-UNITED-BEIJING-SYSTEM" },
	{ 0x5CE0F6, "NIC.br- Nucleo de Informacao e Coordenacao do Ponto BR", "NIC-BR--NUCLEO-DE-INFORMACAO-E-COORDENACAO-DO-PONTO-BR" },
	{ 0x5CE223, "Delphin Technology AG", "DELPHIN" },
	{ 0x5CE286, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x5CE2F4, "AcSiP Technology Corp.", "ACSIP" },
	{ 0x5CE7BF, "New Singularity International Technical Development Co.,Ltd", "NEW-SINGULARITY-TECHNICAL-DEVELOPMENT" },
	{ 0x5CE8EB, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x5CEB4E, "R. STAHL HMI Systems GmbH", "R-STAHL-HMI" },
	{ 0x5CEB68, "Cheerstar Technology Co., Ltd", "CHEERSTAR" },
	{ 0x5CEE79, "Global Digitech Co LTD", "GLOBAL-DIGITECH" },
	{ 0x5CF207, "Speco Technologies", "SPECO-TECHNOLOGIES" },
	{ 0x5CF370, "CC&C Technologies, Inc", "CC-C-TECHNOLOGIES" },
	{ 0x5CF3FC, "IBM Corp", "IBM" },
	{ 0x5CF4AB, "ZyXEL Communications Corp", "ZYXEL-COMMUNICATION" },
	{ 0x5CF50D, "Institute of microelectronic applications", "INSTITUTE-OF-MICROELECTRONIC-APPLICATIONS" },
	{ 0x5CF5DA, "Apple", "APPLE" },
	{ 0x5CF6DC, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x5CF7C3, "SYNTECH (HK) TECHNOLOGY LIMITED", "SYNTECH-HK" },
	{ 0x5CF8A1, "Murata Manufactuaring Co.,Ltd.", "MURATA-MANUFACTUARING" },
	{ 0x5CF938, "Apple, Inc", "APPLE" },
	{ 0x5CF96A, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x5CF9DD, "Dell Inc", "DELL" },
	{ 0x5CF9F0, "Atomos Engineering P/L", "ATOMOS-ENGINEERING-P/L" },
	{ 0x5CFC66, "Cisco", "CISCO" },
	{ 0x5CFF35, "Wistron Corporation", "WISTRON" },
	{ 0x5CFFFF, "Shenzhen Kezhonglong Optoelectronic Technology Co., Ltd", "SHENZHEN-KEZHONGLONG-OPTOELECTRONIC" },
	{ 0x600292, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x6002B4, "Wistron NeWeb Corp.", "WISTRON-NEWEB" },
	{ 0x600308, "Apple", "APPLE" },
	{ 0x600347, "Billion Electric Co. Ltd.", "BILLION-ELECTRIC" },
	{ 0x600417, "POSBANK CO.,LTD", "POSBANK" },
	{ 0x600F77, "SilverPlus, Inc", "SILVERPLUS" },
	{ 0x601199, "Siama Systems Inc", "SIAMA" },
	{ 0x601283, "Soluciones Tecnologicas para la Salud y el Bienestar SA", "SOLUCIONES-TECNOLOGICAS-PARA-LA-SALUD-Y-EL-BIENESTAR" },
	{ 0x60128B, "CANON INC.", "CANON" },
	{ 0x6015C7, "IdaTech", "IDATECH" },
	{ 0x60190C, "RRAMAC", "RRAMAC" },
	{ 0x601929, "VOLTRONIC POWER TECHNOLOGY(SHENZHEN) CORP.", "VOLTRONIC-POWER-SHENZHEN" },
	{ 0x601970, "HUIZHOU QIAOXING ELECTRONICS TECHNOLOGY CO., LTD.", "HUIZHOU-QIAOXING-ELECTRONICS" },
	{ 0x601D0F, "Midnite Solar", "MIDNITE-SOLAR" },
	{ 0x601E02, "EltexAlatau", "ELTEXALATAU" },
	{ 0x602103, "STCUBE.INC", "STCUBE" },
	{ 0x6021C0, "Murata Manufactuaring Co.,Ltd.", "MURATA-MANUFACTUARING" },
	{ 0x6024C1, "Jiangsu Zhongxun Electronic Technology Co., Ltd", "JIANGSU-ZHONGXUN" },
	{ 0x602A54, "CardioTek B.V.", "CARDIOTEK" },
	{ 0x602AD0, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x6032F0, "Mplus technology", "MPLUS" },
	{ 0x60334B, "Apple", "APPLE" },
	{ 0x603553, "Buwon Technology", "BUWON" },
	{ 0x603696, "The Sapling Company", "THE-SAPLING-COMPANY" },
	{ 0x6036DD, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x60380E, "Alps Electric Co.,", "ALPS-ELECTRIC" },
	{ 0x60391F, "ABB Ltd", "ABB" },
	{ 0x603FC5, "COX CO., LTD", "COX" },
	{ 0x6044F5, "Easy Digital Ltd.", "EASY-DIGITAL" },
	{ 0x60455E, "Liptel s.r.o.", "LIPTEL-S-R-O" },
	{ 0x6045BD, "Microsoft", "MICROSOFT" },
	{ 0x604616, "XIAMEN VANN INTELLIGENT CO., LTD", "XIAMEN-VANN-INTELLIGENT" },
	{ 0x6047D4, "FORICS Electronic Technology Co., Ltd.", "FORICS" },
	{ 0x604826, "Newbridge Technologies Int. Ltd.", "NEWBRIDGE-TECHNOLOGIES-INT" },
	{ 0x604A1C, "SUYIN Corporation", "SUYIN" },
	{ 0x6050C1, "Kinetek Sports", "KINETEK-SPORTS" },
	{ 0x60512C, "TCT mobile limited", "TCT-MOBILE" },
	{ 0x6052D0, "FACTS Engineering", "FACTS-ENGINEERING" },
	{ 0x605464, "Eyedro Green Solutions Inc.", "EYEDRO-GREEN-SOLUTIONS" },
	{ 0x605718, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x605BB4, "AzureWave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x60601F, "SZ DJI TECHNOLOGY CO.,LTD", "SZ-DJI" },
	{ 0x6063FD, "Transcend Communication Beijing Co.,Ltd.", "TRANSCEND-COMMUNICATION-BEIJING" },
	{ 0x6064A1, "RADiflow Ltd.", "RADIFLOW" },
	{ 0x606720, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x606944, "Apple, Inc", "APPLE" },
	{ 0x60699B, "isepos GmbH", "ISEPOS" },
	{ 0x606BBD, "Samsung Electronics Co., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x606C66, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x60735C, "Cisco", "CISCO" },
	{ 0x60748D, "Atmaca Elektronik", "ATMACA-ELEKTRONIK" },
	{ 0x607688, "Velodyne", "VELODYNE" },
	{ 0x6077E2, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x60812B, "Custom Control Concepts", "CUSTOM-CONTROL-CONCEPTS" },
	{ 0x6081F9, "Helium Systems, Inc", "HELIUM" },
	{ 0x6083B2, "GkWare e.K.", "GKWARE-E-K" },
	{ 0x60843B, "Soladigm, Inc.", "SOLADIGM" },
	{ 0x608645, "Avery Weigh-Tronix, LLC", "AVERY-WEIGH-TRONIX-LLC" },
	{ 0x60893C, "Thermo Fisher Scientific P.O.A.", "THERMO-FISHER-SCIENTIFIC-P-O-A" },
	{ 0x6089B1, "Key Digital Systems", "KEY-DIGITAL" },
	{ 0x6089B7, "KAEL MÜHENDİSLİK ELEKTRONİK TİCARET SANAYİ LİMİTED ŞİRKETİ", "KAEL-MÜHENDİSLİK-ELEKTRONİK-TİCARET-SANAYİ-LİMİTED-ŞİRKETİ" },
	{ 0x608C2B, "Hanson Technology", "HANSON" },
	{ 0x608D17, "Sentrus Government Systems Division, Inc", "SENTRUS-GOVERNMENT-DIVISION" },
	{ 0x608F5C, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x609084, "DSSD Inc", "DSSD" },
	{ 0x609217, "Apple", "APPLE" },
	{ 0x609620, "PRIVATE", "PRIVATE" },
	{ 0x6099D1, "Vuzix / Lenovo", "VUZIX-/-LENOVO" },
	{ 0x609AA4, "GVI SECURITY INC.", "GVI-SECURITY" },
	{ 0x609C9F, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x609E64, "Vivonic GmbH", "VIVONIC" },
	{ 0x609F9D, "CloudSwitch", "CLOUDSWITCH" },
	{ 0x60A10A, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x60A44C, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x60A8FE, "Nokia Solutions and Networks", "NOKIA-SOLUTIONS-AND-NETWORKS" },
	{ 0x60A9B0, "Merchandising Technologies, Inc", "MERCHANDISING-TECHNOLOGIES" },
	{ 0x60AF6D, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x60B185, "ATH system", "ATH-SYSTEM" },
	{ 0x60B3C4, "Elber Srl", "ELBER-SRL" },
	{ 0x60B606, "Phorus", "PHORUS" },
	{ 0x60B617, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0x60B933, "Deutron Electronics Corp.", "DEUTRON-ELECTRONICS" },
	{ 0x60B982, "RO.VE.R. Laboratories S.p.A.", "RO-VE-R-LABORATORIES-S-P-A" },
	{ 0x60BB0C, "Beijing HuaqinWorld Technology Co,Ltd", "BEIJING-HUAQINWORLD" },
	{ 0x60BC4C, "EWM Hightec Welding GmbH", "EWM-HIGHTEC-WELDING" },
	{ 0x60BD91, "Move Innovation", "MOVE-INNOVATION" },
	{ 0x60BEB5, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0x60C1CB, "Fujian Great Power PLC Equipment Co.,Ltd", "FUJIAN-GREAT-POWER-PLC-EQUIPMENT" },
	{ 0x60C397, "2Wire Inc", "2WIRE" },
	{ 0x60C547, "Apple", "APPLE" },
	{ 0x60C5A8, "Beijing LT Honway Technology Co.,Ltd", "BEIJING-LT-HONWAY" },
	{ 0x60C798, "Verifone, Inc.", "VERIFONE" },
	{ 0x60C980, "Trymus", "TRYMUS" },
	{ 0x60CBFB, "AirScape Inc.", "AIRSCAPE" },
	{ 0x60CDA9, "Abloomy", "ABLOOMY" },
	{ 0x60CDC5, "Taiwan Carol Electronics., Ltd", "TAIWAN-CAROL-ELECTRONICS" },
	{ 0x60D0A9, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x60D1AA, "Vishal Telecommunications Pvt Ltd", "VISHAL-TELECOMMUNICATIONS-PVT" },
	{ 0x60D2B9, "REALAND BIO CO., LTD.", "REALAND-BIO" },
	{ 0x60D30A, "Quatius Limited", "QUATIUS" },
	{ 0x60D819, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x60D9A0, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x60D9C7, "Apple", "APPLE" },
	{ 0x60DA23, "Estech Co.,Ltd", "ESTECH" },
	{ 0x60DB2A, "HNS", "HNS" },
	{ 0x60DE44, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x60E00E, "SHINSEI ELECTRONICS CO LTD", "SHINSEI-ELECTRONICS" },
	{ 0x60E327, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x60E6BC, "Sino-Telecom Technology Co.,Ltd.", "SINO-TELECOM" },
	{ 0x60E701, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x60E956, "Ayla Networks, Inc", "AYLA-NETWORKS" },
	{ 0x60EB69, "Quanta computer Inc.", "QUANTA" },
	{ 0x60F13D, "JABLOCOM s.r.o.", "JABLOCOM-S-R-O" },
	{ 0x60F189, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x60F281, "TRANWO TECHNOLOGY CO., LTD.", "TRANWO" },
	{ 0x60F2EF, "VisionVera International Co., Ltd.", "VISIONVERA" },
	{ 0x60F3DA, "Logic Way GmbH", "LOGIC-WAY" },
	{ 0x60F494, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x60F59C, "CRU-Dataport", "CRU-DATAPORT" },
	{ 0x60F673, "TERUMO CORPORATION", "TERUMO" },
	{ 0x60F81D, "Apple", "APPLE" },
	{ 0x60FACD, "Apple", "APPLE" },
	{ 0x60FB42, "Apple", "APPLE" },
	{ 0x60FE1E, "China Palms Telecom.Ltd", "CHINA-PALMS-TELECOM" },
	{ 0x60FE20, "2 Wire", "2-WIRE" },
	{ 0x60FEC5, "Apple", "APPLE" },
	{ 0x60FEF9, "Thomas & Betts", "THOMAS-BETTS" },
	{ 0x60FFDD, "C.E. ELECTRONICS, INC", "C-E-ELECTRONICS" },
	{ 0x64002D, "Powerlinq Co., LTD", "POWERLINQ" },
	{ 0x6400F1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x6405BE, "NEW LIGHT LED", "NEW-LIGHT-LED" },
	{ 0x64094C, "Beijing Superbee Wireless Technology Co.,Ltd", "BEIJING-SUPERBEE-WIRELESS" },
	{ 0x640980, "XIAOMI Electronics,CO.,LTD", "XIAOMI-ELECTRONICS" },
	{ 0x640B4A, "Digital Telecom Technology Limited", "DIGITAL-TELECOM" },
	{ 0x640E36, "TAZTAG", "TAZTAG" },
	{ 0x640E94, "Pluribus Networks, Inc.", "PLURIBUS-NETWORKS" },
	{ 0x640F28, "2wire", "2WIRE" },
	{ 0x641084, "HEXIUM Technical Development Co., Ltd.", "HEXIUM-TECHNICAL-DEVELOPMENT" },
	{ 0x641225, "Cisco", "CISCO" },
	{ 0x64167F, "Polycom", "POLYCOM" },
	{ 0x64168D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x6416F0, "Shehzhen Huawei Communication Technologies Co., Ltd.", "SHEHZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x641A22, "Heliospectra/Woodhill Investments", "HELIOSPECTRA/WOODHILL-INVESTMENTS" },
	{ 0x641C67, "DIGIBRAS INDUSTRIA DO BRASILS/A", "DIGIBRAS-INDUSTRIA-DO-BRASILS/A" },
	{ 0x641E81, "Dowslake Microsystems", "DOWSLAKE-MICROSYSTEMS" },
	{ 0x64200C, "Apple", "APPLE" },
	{ 0x642184, "Nippon Denki Kagaku Co.,LTD", "NIPPON-DENKI-KAGAKU" },
	{ 0x642216, "Shandong Taixin Electronic co.,Ltd", "SHANDONG-TAIXIN" },
	{ 0x642400, "Xorcom Ltd.", "XORCOM" },
	{ 0x642737, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x642DB7, "SEUNGIL ELECTRONICS", "SEUNGIL-ELECTRONICS" },
	{ 0x643150, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x64317E, "Dexin Corporation", "DEXIN" },
	{ 0x643409, "BITwave Pte Ltd", "BITWAVE-PTE" },
	{ 0x643AB1, "SICHUAN TIANYI COMHEART TELECOMCO.,LTD", "SICHUAN-TIANYI-COMHEART-TELECOMCO" },
	{ 0x643E8C, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x643F5F, "Exablaze", "EXABLAZE" },
	{ 0x644214, "Swisscom Energy Solutions AG", "SWISSCOM-ENERGY-SOLUTIONS" },
	{ 0x644346, "GuangDong Quick Network Computer CO.,LTD", "GUANGDONG-QUICK-NETWORK" },
	{ 0x644BC3, "Shanghai WOASiS Telecommunications Ltd., Co.", "SHANGHAI-WOASIS-TELECOMMUNICATIONS" },
	{ 0x644BF0, "CalDigit, Inc", "CALDIGIT" },
	{ 0x644D70, "dSPACE GmbH", "DSPACE" },
	{ 0x644F74, "LENUS Co., Ltd.", "LENUS" },
	{ 0x644FB0, "Hyunjin.com", "HYUNJIN-COM" },
	{ 0x645106, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x64517E, "LONG BEN (DONGGUAN) ELECTRONIC TECHNOLOGY CO.,LTD.", "LONG-BEN-DONGGUAN" },
	{ 0x645299, "The Chamberlain Group, Inc", "THE-CHAMBERLAIN-GROUP" },
	{ 0x64535D, "Frauscher Sensortechnik", "FRAUSCHER-SENSORTECHNIK" },
	{ 0x645422, "Equinox Payments", "EQUINOX-PAYMENTS" },
	{ 0x645563, "Intelight Inc.", "INTELIGHT" },
	{ 0x64557F, "NSFOCUS Information Technology Co., Ltd.", "NSFOCUS-INFORMATION" },
	{ 0x6455B1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x645601, "TP-LINK TECHNOLOGIES CO.,LTD", "TP-LINK-TECHNOLOGIES" },
	{ 0x6459F8, "Vodafone Omnitel B.V.", "VODAFONE-OMNITEL" },
	{ 0x645A04, "Chicony Electronics Co., Ltd.", "CHICONY-ELECTRONICS" },
	{ 0x645DD7, "Shenzhen Lifesense Medical Electronics Co., Ltd.", "SHENZHEN-LIFESENSE-MEDICAL-ELECTRONICS" },
	{ 0x645EBE, "Yahoo! JAPAN", "YAHOO!-JAPAN" },
	{ 0x645FFF, "Nicolet Neuro", "NICOLET-NEURO" },
	{ 0x646223, "Cellient Co., Ltd.", "CELLIENT" },
	{ 0x64649B, "juniper networks", "JUNIPER-NETWORKS" },
	{ 0x6465C0, "Nuvon, Inc", "NUVON" },
	{ 0x6466B3, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x646707, "Beijing Omnific Technology, Ltd.", "BEIJING-OMNIFIC" },
	{ 0x64680C, "COMTREND", "COMTREND" },
	{ 0x6469BC, "Hytera Communications Co .,ltd", "HYTERA-COMMUNICATION" },
	{ 0x646A52, "Avaya, Inc", "AVAYA" },
	{ 0x646CB2, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x646E6C, "Radio Datacom LLC", "RADIO-DATACOM-LLC" },
	{ 0x646EEA, "Iskratel d.o.o.", "ISKRATEL-D-O-O" },
	{ 0x647002, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x6472D8, "GooWi Technology Co.,Limited", "GOOWI" },
	{ 0x6473E2, "Arbiter Systems, Inc.", "ARBITER" },
	{ 0x647657, "Innovative Security Designs", "INNOVATIVE-SECURITY-DESIGNS" },
	{ 0x6476BA, "Apple", "APPLE" },
	{ 0x647791, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x647BD4, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x647C34, "Ubee Interactive Corp.", "UBEE-INTERACTIVE" },
	{ 0x647D81, "YOKOTA INDUSTRIAL CO,.LTD", "YOKOTA-INDUSTRIAL" },
	{ 0x647FDA, "TEKTELIC Communications Inc.", "TEKTELIC-COMMUNICATION" },
	{ 0x64808B, "VG Controls, Inc.", "VG-CONTROLS" },
	{ 0x648099, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x648125, "Alphatron Marine BV", "ALPHATRON-MARINE" },
	{ 0x648788, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x6487D7, "Pirelli Tyre S.p.A.", "PIRELLI-TYRE-S-P-A" },
	{ 0x6488FF, "Sichuan Changhong Electric Ltd.", "SICHUAN-CHANGHONG-ELECTRIC" },
	{ 0x64899A, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x648D9E, "IVT Electronic Co.,Ltd", "IVT" },
	{ 0x64995D, "LGE", "LGE" },
	{ 0x649968, "Elentec", "ELENTEC" },
	{ 0x6499A0, "AG Elektronik AB", "ELEKTRONIK-AB" },
	{ 0x649ABE, "Apple", "APPLE" },
	{ 0x649B24, "V Technology Co., Ltd.", "V" },
	{ 0x649C81, "Qualcomm iSkoot, Inc.", "QUALCOMM-ISKOOT" },
	{ 0x649C8E, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x649EF3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x649FF7, "Kone OYj", "KONE-OYJ" },
	{ 0x64A0E7, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x64A232, "OOO Samlight", "OOO-SAMLIGHT" },
	{ 0x64A341, "Wonderlan (Beijing) Technology Co., Ltd.", "WONDERLAN-BEIJING" },
	{ 0x64A3CB, "Apple", "APPLE" },
	{ 0x64A769, "HTC Corporation", "HTC" },
	{ 0x64A7DD, "Avaya, Inc", "AVAYA" },
	{ 0x64A837, "Juni Korea Co., Ltd", "JUNI-KOREA" },
	{ 0x64AE0C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x64AE88, "Polytec GmbH", "POLYTEC" },
	{ 0x64B21D, "Chengdu Phycom Tech Co., Ltd.", "CHENGDU-PHYCOM-TECH" },
	{ 0x64B310, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x64B370, "PowerComm Solutions LLC", "POWERCOMM-SOLUTIONS-LLC" },
	{ 0x64B473, "Xiaomi inc.", "XIAOMI" },
	{ 0x64B64A, "ViVOtech, Inc.", "VIVOTECH" },
	{ 0x64B853, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0x64B9E8, "Apple", "APPLE" },
	{ 0x64BABD, "SDJ Technologies, Inc.", "SDJ-TECHNOLOGIES" },
	{ 0x64BC11, "CombiQ AB", "COMBIQ-AB" },
	{ 0x64C5AA, "South African Broadcasting Corporation", "SOUTH-AFRICAN-BROADCASTING" },
	{ 0x64C667, "Barnes&Noble", "BARNES-NOBLE" },
	{ 0x64C6AF, "AXERRA Networks Ltd", "AXERRA-NETWORKS" },
	{ 0x64C944, "LARK Technologies, Inc", "LARK-TECHNOLOGIES" },
	{ 0x64D02D, "Next Generation Integration (NGI)", "NEXT-GENERATION-INTEGRATION-NGI" },
	{ 0x64D1A3, "Sitecom Europe BV", "SITECOM-EUROPE" },
	{ 0x64D241, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x64D4BD, "ALPS ELECTRIC CO.,LTD.", "ALPS-ELECTRIC" },
	{ 0x64D4DA, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x64D814, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x64D912, "Solidica, Inc.", "SOLIDICA" },
	{ 0x64D954, "TAICANG AND W ELECTRONICS CO LTD", "TAICANG-AND-W-ELECTRONICS" },
	{ 0x64D989, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x64DB18, "OpenPattern", "OPENPATTERN" },
	{ 0x64DB81, "Syszone Co., Ltd.", "SYSZONE" },
	{ 0x64DC01, "Static Systems Group PLC", "STATIC-GROUP-PLC" },
	{ 0x64DE1C, "Kingnetic Pte Ltd", "KINGNETIC-PTE" },
	{ 0x64E161, "DEP Corp.", "DEP" },
	{ 0x64E599, "EFM Networks", "EFM-NETWORKS" },
	{ 0x64E625, "Woxu Wireless Co., Ltd", "WOXU-WIRELESS" },
	{ 0x64E682, "Apple", "APPLE" },
	{ 0x64E84F, "Serialway Communication Technology Co. Ltd", "SERIALWAY-COMMUNICATION" },
	{ 0x64E892, "Morio Denki Co., Ltd.", "MORIO-DENKI" },
	{ 0x64E8E6, "global moisture management system", "GLOBAL-MOISTURE-MANAGEMENT-SYSTEM" },
	{ 0x64E950, "Cisco", "CISCO" },
	{ 0x64EAC5, "SiboTech Automation Co., Ltd.", "SIBOTECH-AUTOMATION" },
	{ 0x64EB8C, "Seiko Epson Corporation", "SEIKO-EPSON" },
	{ 0x64ED57, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x64ED62, "WOORI SYSTEMS Co., Ltd", "WOORI" },
	{ 0x64F242, "Gerdes Aktiengesellschaft", "GERDES-AKTIENGESELLSCHAFT" },
	{ 0x64F50E, "Kinion Technology Company Limited", "KINION-COMPANY" },
	{ 0x64F69D, "Cisco", "CISCO" },
	{ 0x64F970, "Kenade Electronics Technology Co.,LTD.", "KENADE-ELECTRONICS" },
	{ 0x64F987, "Avvasi Inc.", "AVVASI" },
	{ 0x64FC8C, "Zonar Systems", "ZONAR" },
	{ 0x680571, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x6805CA, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x680927, "Apple", "APPLE" },
	{ 0x680AD7, "Yancheng Kecheng Optoelectronic Technology Co., Ltd", "YANCHENG-KECHENG-OPTOELECTRONIC" },
	{ 0x68122D, "Special Instrument Development Co., Ltd.", "SPECIAL-INSTRUMENT-DEVELOPMENT" },
	{ 0x681590, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0x6815D3, "Zaklady Elektroniki i Mechaniki Precyzyjnej R&G S.A.", "ZAKLADY-ELEKTRONIKI-I-MECHANIKI-PRECYZYJNEJ-R-G" },
	{ 0x681605, "Systems And Electronic Development FZCO", "AND-DEVELOPMENT-FZCO" },
	{ 0x681729, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x68193F, "Digital Airways", "DIGITAL-AIRWAYS" },
	{ 0x681AB2, "zte corporation", "ZTE" },
	{ 0x681CA2, "Rosewill Inc.", "ROSEWILL" },
	{ 0x681D64, "Sunwave Communications Co., Ltd", "SUNWAVE-COMMUNICATION" },
	{ 0x681E8B, "InfoSight Corporation", "INFOSIGHT" },
	{ 0x681FD8, "Advanced Telemetry", "ADVANCED-TELEMETRY" },
	{ 0x68234B, "Nihon Dengyo Kousaku", "NIHON-DENGYO-KOUSAKU" },
	{ 0x6828BA, "Dejai", "DEJAI" },
	{ 0x6828F6, "Vubiq Networks, Inc.", "VUBIQ-NETWORKS" },
	{ 0x682DDC, "Wuhan Changjiang Electro-Communication Equipment CO.,LTD", "WUHAN-CHANGJIANG-ELECTRO-COMMUNICATION-EQUIPMENT" },
	{ 0x6836B5, "DriveScale, Inc.", "DRIVESCALE" },
	{ 0x683B1E, "Countwise LTD", "COUNTWISE" },
	{ 0x683C7D, "Magic Intelligence Technology Limited", "MAGIC-INTELLIGENCE" },
	{ 0x683EEC, "ERECA", "ERECA" },
	{ 0x684352, "Bhuu Limited", "BHUU" },
	{ 0x684898, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x684B88, "Galtronics Telemetry Inc.", "GALTRONICS-TELEMETRY" },
	{ 0x684CA8, "Shenzhen Herotel Tech. Co., Ltd.", "SHENZHEN-HEROTEL-TECH" },
	{ 0x6851B7, "PowerCloud Systems, Inc.", "POWERCLOUD" },
	{ 0x6854ED, "Alcatel-Lucent - Nuage", "ALCATEL-LUCENT---NUAGE" },
	{ 0x6854F5, "enLighted Inc", "ENLIGHTED" },
	{ 0x68597F, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x685B35, "Apple", "APPLE" },
	{ 0x685B36, "POWERTECH INDUSTRIAL CO., LTD.", "POWERTECH-INDUSTRIAL" },
	{ 0x685D43, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x685E6B, "PowerRay Co., Ltd.", "POWERRAY" },
	{ 0x686359, "Advanced Digital Broadcast SA", "ADVANCED-DIGITAL-BROADCAST" },
	{ 0x68644B, "Apple", "APPLE" },
	{ 0x68692E, "Zycoo Co.,Ltd", "ZYCOO" },
	{ 0x6869F2, "ComAp s.r.o.", "COMAP-S-R-O" },
	{ 0x686E23, "Wi3 Inc.", "WI3" },
	{ 0x686E48, "Prophet Electronic Technology Corp.,Ltd", "PROPHET" },
	{ 0x687251, "Ubiquiti Networks", "UBIQUITI-NETWORKS" },
	{ 0x6872DC, "CETORY.TV Company Limited", "CETORY-TV-COMPANY" },
	{ 0x68764F, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x687848, "Westunitis Co., Ltd.", "WESTUNITIS" },
	{ 0x68784C, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x687924, "ELS-GmbH & Co. KG", "ELS" },
	{ 0x6879ED, "SHARP Corporation", "SHARP" },
	{ 0x687CC8, "Measurement Systems S. de R.L.", "MEASUREMENT-S-DE-R-L" },
	{ 0x687CD5, "Y Soft Corporation, a.s.", "Y-SOFT-A-S" },
	{ 0x687F74, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x68831A, "Pandora Mobility Corporation", "PANDORA-MOBILITY" },
	{ 0x688470, "eSSys Co.,Ltd", "ESSYS" },
	{ 0x688540, "IGI Mobile, Inc.", "IGI-MOBILE" },
	{ 0x68856A, "OuterLink Corporation", "OUTERLINK" },
	{ 0x6886A7, "Cisco", "CISCO" },
	{ 0x6886E7, "Orbotix, Inc.", "ORBOTIX" },
	{ 0x68876B, "INQ Mobile Limited", "INQ-MOBILE" },
	{ 0x688AB5, "EDP Servicos", "EDP-SERVICOS" },
	{ 0x688F84, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x689234, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x689423, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x68967B, "Apple", "APPLE" },
	{ 0x68974B, "Shenzhen Costar Electronics Co. Ltd.", "SHENZHEN-COSTAR-ELECTRONICS" },
	{ 0x6897E8, "Society of Motion Picture &amp; Television Engineers", "SOCIETY-OF-MOTION-PICTURE-AMP-TELEVISION-ENGINEERS" },
	{ 0x6899CD, "Cisco", "CISCO" },
	{ 0x689C5E, "AcSiP Technology Corp.", "ACSIP" },
	{ 0x689C70, "Apple", "APPLE" },
	{ 0x689CE2, "Cisco", "CISCO" },
	{ 0x689E19, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x68A0F6, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x68A1B7, "Honghao Mingchuan Technology (Beijing) CO.,Ltd.", "HONGHAO-MINGCHUAN-BEIJING" },
	{ 0x68A378, "FREEBOX SAS", "FREEBOX-SAS" },
	{ 0x68A3C4, "Liteon Technology Corporation", "LITEON" },
	{ 0x68A40E, "BSH Bosch and Siemens Home Appliances GmbH", "BSH-BOSCH-AND-SIEMENS-HOME-APPLIANCES" },
	{ 0x68A86D, "Apple", "APPLE" },
	{ 0x68AAD2, "DATECS LTD.,", "DATECS" },
	{ 0x68AB8A, "RF IDeas", "RF-IDEAS" },
	{ 0x68AE20, "Apple", "APPLE" },
	{ 0x68AF13, "Futura Mobility", "FUTURA-MOBILITY" },
	{ 0x68B094, "INESA ELECTRON CO.,LTD", "INESA-ELECTRON" },
	{ 0x68B43A, "WaterFurnace International, Inc.", "WATERFURNACE" },
	{ 0x68B599, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x68B6FC, "Hitron Technologies. Inc", "HITRON-TECHNOLOGIES" },
	{ 0x68B8D9, "Act KDE, Inc.", "ACT-KDE" },
	{ 0x68B983, "b-plus GmbH", "B-PLUS" },
	{ 0x68BC0C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x68BDAB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x68C90B, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x68CA00, "Octopus Systems Limited", "OCTOPUS" },
	{ 0x68CC9C, "Mine Site Technologies", "MINE-SITE-TECHNOLOGIES" },
	{ 0x68CD0F, "U Tek Company Limited", "U-TEK-COMPANY" },
	{ 0x68CE4E, "L-3 Communications Infrared Products", "L-3-COMMUNICATION-INFRARED-PRODUCTS" },
	{ 0x68D1FD, "Shenzhen Trimax Technology Co.,Ltd", "SHENZHEN-TRIMAX" },
	{ 0x68D247, "Portalis LC", "PORTALIS-LC" },
	{ 0x68D925, "ProSys Development Services", "PROSYS-DEVELOPMENT-SERVICES" },
	{ 0x68D93C, "Apple", "APPLE" },
	{ 0x68DB67, "Nantong Coship Electronics Co., Ltd", "NANTONG-COSHIP-ELECTRONICS" },
	{ 0x68DB96, "OPWILL Technologies CO .,LTD", "OPWILL-TECHNOLOGIES" },
	{ 0x68DCE8, "PacketStorm Communications", "PACKETSTORM-COMMUNICATION" },
	{ 0x68DFDD, "Xiaomi inc.", "XIAOMI" },
	{ 0x68E166, "PRIVATE", "PRIVATE" },
	{ 0x68E41F, "Unglaube Identech GmbH", "UNGLAUBE-IDENTECH" },
	{ 0x68EBAE, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x68EBC5, "Angstrem Telecom", "ANGSTREM-TELECOM" },
	{ 0x68EC62, "YODO Technology Corp. Ltd.", "YODO" },
	{ 0x68ED43, "Research In Motion", "RESEARCH-IN-MOTION" },
	{ 0x68EE96, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x68EFBD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x68F06D, "ALONG INDUSTRIAL CO., LIMITED", "ALONG-INDUSTRIAL" },
	{ 0x68F0BC, "Shenzhen LiWiFi Technology Co., Ltd", "SHENZHEN-LIWIFI" },
	{ 0x68F125, "Data Controls Inc.", "DATA-CONTROLS" },
	{ 0x68F728, "LCFC(HeFei) Electronics Technology co., ltd", "LCFC-HEFEI-ELECTRONICS" },
	{ 0x68F895, "Redflow Limited", "REDFLOW" },
	{ 0x68FB95, "Generalplus Technology Inc.", "GENERALPLUS" },
	{ 0x68FCB3, "Next Level Security Systems, Inc.", "NEXT-LEVEL-SECURITY" },
	{ 0x6C0273, "Shenzhen Jin Yun Video Equipment Co., Ltd.", "SHENZHEN-JIN-YUN-VIDEO-EQUIPMENT" },
	{ 0x6C0460, "RBH Access Technologies Inc.", "RBH-ACCESS-TECHNOLOGIES" },
	{ 0x6C09D6, "Digiquest Electronics LTD", "DIGIQUEST-ELECTRONICS" },
	{ 0x6C0B84, "Universal Global Scientific Industrial Co.,Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x6C0E0D, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x6C0F6A, "JDC Tech Co., Ltd.", "JDC-TECH" },
	{ 0x6C14F7, "Erhardt+Leimer GmbH", "ERHARDT+LEIMER" },
	{ 0x6C15F9, "Nautronix Limited", "NAUTRONIX" },
	{ 0x6C1811, "Decatur Electronics", "DECATUR-ELECTRONICS" },
	{ 0x6C198F, "D-Link International", "D-LINK" },
	{ 0x6C1E70, "Guangzhou YBDS IT Co.,Ltd", "GUANGZHOU-YBDS-IT" },
	{ 0x6C2056, "Cisco", "CISCO" },
	{ 0x6C22AB, "Ainsworth Game Technology", "AINSWORTH-GAME" },
	{ 0x6C23B9, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x6C25B9, "BBK Electronics Corp., Ltd.,", "BBK-ELECTRONICS" },
	{ 0x6C2995, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x6C2C06, "OOO NPP Systemotechnika-NN", "OOO-NPP-SYSTEMOTECHNIKA-NN" },
	{ 0x6C2E33, "Accelink Technologies Co.,Ltd.", "ACCELINK-TECHNOLOGIES" },
	{ 0x6C2E72, "B&B EXPORTING LIMITED", "B-B-EXPORTING" },
	{ 0x6C2E85, "SAGEMCOM", "SAGEMCOM" },
	{ 0x6C2F2C, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x6C32DE, "Indieon Technologies Pvt. Ltd.", "INDIEON-TECHNOLOGIES-PVT" },
	{ 0x6C33A9, "Magicjack LP", "MAGICJACK-LP" },
	{ 0x6C391D, "Beijing ZhongHuaHun Network Information center", "BEIJING-ZHONGHUAHUN-NETWORK-INFORMATION-CENTER" },
	{ 0x6C3A84, "Shenzhen Aero-Startech. Co.Ltd", "SHENZHEN-AERO-STARTECH" },
	{ 0x6C3BE5, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x6C3C53, "SoundHawk Corp", "SOUNDHAWK" },
	{ 0x6C3E6D, "Apple", "APPLE" },
	{ 0x6C3E9C, "KE Knestel Elektronik GmbH", "KE-KNESTEL-ELEKTRONIK" },
	{ 0x6C4008, "Apple", "APPLE" },
	{ 0x6C40C6, "Nimbus Data Systems, Inc.", "NIMBUS-DATA" },
	{ 0x6C416A, "Cisco", "CISCO" },
	{ 0x6C4418, "Zappware", "ZAPPWARE" },
	{ 0x6C4598, "Antex Electronic Corp.", "ANTEX" },
	{ 0x6C4B7F, "Vossloh-Schwabe Deutschland GmbH", "VOSSLOH-SCHWABE-DEUTSCHLAND" },
	{ 0x6C504D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x6C5779, "Aclima, Inc.", "ACLIMA" },
	{ 0x6C5940, "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.", "SHENZHEN-MERCURY-COMMUNICATION-TECHNOLOGIES" },
	{ 0x6C5A34, "Shenzhen Haitianxiong Electronic Co., Ltd.", "SHENZHEN-HAITIANXIONG" },
	{ 0x6C5AB5, "TCL Technoly Electronics (Huizhou) Co., Ltd.", "TCL-TECHNOLY-ELECTRONICS-HUIZHOU" },
	{ 0x6C5CDE, "SunReports, Inc.", "SUNREPORTS" },
	{ 0x6C5D63, "ShenZhen Rapoo Technology Co., Ltd.", "SHENZHEN-RAPOO" },
	{ 0x6C5E7A, "Ubiquitous Internet Telecom Co., Ltd", "UBIQUITOUS-INTERNET-TELECOM" },
	{ 0x6C5F1C, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x6C6126, "Rinicom Holdings", "RINICOM-HOLDINGS" },
	{ 0x6C626D, "Micro-Star INT'L CO., LTD", "MICRO-STAR" },
	{ 0x6C641A, "Penguin Computing", "PENGUIN-COMPUTING" },
	{ 0x6C6EFE, "Core Logic Inc.", "CORE-LOGIC" },
	{ 0x6C6F18, "Stereotaxis, Inc.", "STEREOTAXIS" },
	{ 0x6C7039, "Novar GmbH", "NOVAR" },
	{ 0x6C709F, "Apple", "APPLE" },
	{ 0x6C71D9, "AzureWave Technologies, Inc", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x6C7660, "KYOCERA Corporation", "KYOCERA" },
	{ 0x6C81FE, "Mitsuba Corporation", "MITSUBA" },
	{ 0x6C8336, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x6C8366, "Nanjing SAC Power Grid Automation Co., Ltd.", "NANJING-SAC-POWER-GRID-AUTOMATION" },
	{ 0x6C8686, "Technonia", "TECHNONIA" },
	{ 0x6C8814, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x6C8B2F, "zte corporation", "ZTE" },
	{ 0x6C8CDB, "Otus Technologies Ltd", "OTUS-TECHNOLOGIES" },
	{ 0x6C8D65, "Wireless Glue Networks, Inc.", "WIRELESS-GLUE-NETWORKS" },
	{ 0x6C90B1, "SanLogic Inc", "SANLOGIC" },
	{ 0x6C92BF, "Inspur Electronic Information Industry Co.,Ltd.", "INSPUR-INFORMATION-INDUSTRY" },
	{ 0x6C94F8, "Apple", "APPLE" },
	{ 0x6C98EB, "Ocedo GmbH", "OCEDO" },
	{ 0x6C9989, "Cisco", "CISCO" },
	{ 0x6C9AC9, "Valentine Research, Inc.", "VALENTINE-RESEARCH" },
	{ 0x6C9B02, "Nokia Corporation", "NOKIA" },
	{ 0x6C9CE9, "Nimble Storage", "NIMBLE-STORAGE" },
	{ 0x6C9CED, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x6CA682, "EDAM information & communications", "EDAM-INFORMATION-COMMUNICATION" },
	{ 0x6CA75F, "zte corporation", "ZTE" },
	{ 0x6CA780, "Nokia Corporation", "NOKIA" },
	{ 0x6CA7FA, "YOUNGBO ENGINEERING INC.", "YOUNGBO-ENGINEERING" },
	{ 0x6CA849, "Avaya, Inc", "AVAYA" },
	{ 0x6CA906, "Telefield Ltd", "TELEFIELD" },
	{ 0x6CA96F, "TransPacket AS", "TRANSPACKET-AS" },
	{ 0x6CAAB3, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x6CAB4D, "Digital Payment Technologies", "DIGITAL-PAYMENT-TECHNOLOGIES" },
	{ 0x6CAC60, "Venetex Corp", "VENETEX" },
	{ 0x6CAD3F, "Hubbell Building Automation, Inc.", "HUBBELL-BUILDING-AUTOMATION" },
	{ 0x6CADEF, "KZ Broadband Technologies, Ltd.", "KZ-BROADBAND-TECHNOLOGIES" },
	{ 0x6CADF8, "Azurewave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x6CAE8B, "IBM Corporation", "IBM" },
	{ 0x6CB0CE, "NETGEAR", "NETGEAR" },
	{ 0x6CB311, "Shenzhen Lianrui Electronics Co.,Ltd", "SHENZHEN-LIANRUI-ELECTRONICS" },
	{ 0x6CB350, "Anhui comhigher tech co.,ltd", "ANHUI-COMHIGHER-TECH" },
	{ 0x6CB56B, "HUMAX.CO.,LTD", "HUMAX" },
	{ 0x6CB7F4, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x6CBEE9, "Alcatel-Lucent-IPD", "ALCATEL-LUCENT-IPD" },
	{ 0x6CBFB5, "Noon Technology Co., Ltd", "NOON" },
	{ 0x6CC1D2, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x6CC217, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x6CC26B, "Apple", "APPLE" },
	{ 0x6CCA08, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x6CD032, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x6CD146, "Smartek d.o.o.", "SMARTEK-D-O-O" },
	{ 0x6CD1B0, "WING SING ELECTRONICS HONG KONG LIMITED", "WING-SING-ELECTRONICS-HONG-KONG" },
	{ 0x6CD68A, "LG Electronics Inc", "LG-ELECTRONICS" },
	{ 0x6CDC6A, "Promethean Limited", "PROMETHEAN" },
	{ 0x6CE01E, "Modcam AB", "MODCAM-AB" },
	{ 0x6CE0B0, "SOUND4", "SOUND4" },
	{ 0x6CE4CE, "Villiger Security Solutions AG", "VILLIGER-SECURITY-SOLUTIONS" },
	{ 0x6CE873, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x6CE907, "Nokia Corporation", "NOKIA" },
	{ 0x6CE983, "Gastron Co., LTD.", "GASTRON" },
	{ 0x6CECA1, "SHENZHEN CLOU ELECTRONICS CO. LTD.", "SHENZHEN-CLOU-ELECTRONICS" },
	{ 0x6CECEB, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x6CF049, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x6CF373, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x6CF37F, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0x6CF5E8, "Mooredoll Inc.", "MOOREDOLL" },
	{ 0x6CF97C, "Nanoptix Inc.", "NANOPTIX" },
	{ 0x6CFA58, "Avaya, Inc", "AVAYA" },
	{ 0x6CFA89, "Cisco", "CISCO" },
	{ 0x6CFAA7, "AMPAK Technology Inc.", "AMPAK" },
	{ 0x6CFDB9, "Proware Technologies Co Ltd.", "PROWARE-TECHNOLOGIES" },
	{ 0x6CFFBE, "MPB Communications Inc.", "MPB-COMMUNICATION" },
	{ 0x700136, "FATEK Automation Corporation", "FATEK-AUTOMATION" },
	{ 0x700258, "01DB-METRAVIB", "01DB-METRAVIB" },
	{ 0x700514, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x700BC0, "Dewav Technology Company", "DEWAV-COMPANY" },
	{ 0x700FC7, "SHENZHEN IKINLOOP TECHNOLOGY CO.,LTD.", "SHENZHEN-IKINLOOP" },
	{ 0x700FEC, "Poindus Systems Corp.", "POINDUS" },
	{ 0x70105C, "Cisco", "CISCO" },
	{ 0x701124, "Apple", "APPLE" },
	{ 0x701404, "Limited Liability Company", "LIABILITY-COMPANY" },
	{ 0x7014A6, "Apple, Inc.", "APPLE" },
	{ 0x70188B, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x701A04, "Liteon Tech Corp.", "LITEON-TECH" },
	{ 0x701AED, "ADVAS CO., LTD.", "ADVAS" },
	{ 0x701D7F, "Comtech Technology Co., Ltd.", "COMTECH" },
	{ 0x702393, "fos4X GmbH", "FOS4X" },
	{ 0x702526, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x702559, "CyberTAN Technology, Inc.", "CYBERTAN" },
	{ 0x702B1D, "E-Domus International Limited", "E-DOMUS" },
	{ 0x702C1F, "Wisol", "WISOL" },
	{ 0x702DD1, "Newings Communication CO., LTD.", "NEWINGS-COMMUNICATION" },
	{ 0x702F4B, "PolyVision Inc.", "POLYVISION" },
	{ 0x702F97, "Aava Mobile Oy", "AAVA-MOBILE-OY" },
	{ 0x703018, "Avaya, Inc", "AVAYA" },
	{ 0x70305D, "Ubiquoss Inc", "UBIQUOSS" },
	{ 0x70305E, "Nanjing Zhongke Menglian Information Technology Co.,LTD", "NANJING-ZHONGKE-MENGLIAN-INFORMATION" },
	{ 0x703187, "ACX GmbH", "ACX" },
	{ 0x7032D5, "Athena Wireless Communications Inc", "ATHENA-WIRELESS-COMMUNICATION" },
	{ 0x703811, "Invensys Rail", "INVENSYS-RAIL" },
	{ 0x7038B4, "Low Tech Solutions", "LOW-TECH-SOLUTIONS" },
	{ 0x7038EE, "Avaya, Inc", "AVAYA" },
	{ 0x703AD8, "Shenzhen Afoundry Electronic Co., Ltd", "SHENZHEN-AFOUNDRY" },
	{ 0x703C39, "SEAWING Kft", "SEAWING-KFT" },
	{ 0x703EAC, "Apple", "APPLE" },
	{ 0x7041B7, "Edwards Lifesciences LLC", "EDWARDS-LIFESCIENCES-LLC" },
	{ 0x704642, "CHYNG HONG ELECTRONIC CO., LTD.", "CHYNG-HONG" },
	{ 0x704AAE, "Xstream Flow (Pty) Ltd", "XSTREAM-FLOW-PTY" },
	{ 0x704AE4, "Rinstrum Pty Ltd", "RINSTRUM-PTY" },
	{ 0x704CED, "TMRG, Inc.", "TMRG" },
	{ 0x704E01, "KWANGWON TECH CO., LTD.", "KWANGWON-TECH" },
	{ 0x704E66, "SHENZHEN FAST TECHNOLOGIES CO.,LTD", "SHENZHEN-FAST-TECHNOLOGIES" },
	{ 0x7052C5, "Avaya, Inc.", "AVAYA" },
	{ 0x70533F, "Alfa Instrumentos Eletronicos Ltda.", "ALFA-INSTRUMENTOS-ELETRONICOS-LTDA" },
	{ 0x7054D2, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x7054F5, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x705681, "Apple", "APPLE" },
	{ 0x705812, "Panasonic AVC Networks Company", "PANASONIC-AVC-NETWORKS-COMPANY" },
	{ 0x705957, "Medallion Instrumentation Systems", "MEDALLION-INSTRUMENTATION" },
	{ 0x705986, "OOO TTV", "OOO-TTV" },
	{ 0x705AB6, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x705B2E, "M2Communication Inc.", "M2COMMUNICATION" },
	{ 0x705CAD, "Konami Gaming Inc", "KONAMI-GAMING" },
	{ 0x705EAA, "Action Target, Inc.", "ACTION-TARGET" },
	{ 0x7060DE, "LaVision GmbH", "LAVISION" },
	{ 0x706173, "Calantec GmbH", "CALANTEC" },
	{ 0x7062B8, "D-Link International", "D-LINK" },
	{ 0x706417, "ORBIS TECNOLOGIA ELECTRICA S.A.", "ORBIS-TECNOLOGIA-ELECTRICA" },
	{ 0x706582, "Suzhou Hanming Technologies Co., Ltd.", "SUZHOU-HANMING-TECHNOLOGIES" },
	{ 0x706879, "Saijo Denki International Co., Ltd.", "SAIJO-DENKI" },
	{ 0x706F81, "PRIVATE", "PRIVATE" },
	{ 0x70704C, "Purple Communications, Inc", "PURPLE-COMMUNICATION" },
	{ 0x7071B3, "Brain Corporation", "BRAIN" },
	{ 0x7071BC, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x70720D, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x70723C, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x7072CF, "EdgeCore Networks", "EDGECORE-NETWORKS" },
	{ 0x7073CB, "Apple", "APPLE" },
	{ 0x707630, "Pace plc.", "PACE-PLC" },
	{ 0x7076DD, "Oxyguard International A/S", "OXYGUARD-A/S" },
	{ 0x7076F0, "LevelOne Communications (India) Private Limited", "LEVELONE-COMMUNICATION-INDIA-PRIVATE" },
	{ 0x7076FF, "KERLINK", "KERLINK" },
	{ 0x707781, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x707BE8, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x707C18, "ADATA Technology Co., Ltd", "ADATA" },
	{ 0x707E43, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x707EDE, "NASTEC LTD.", "NASTEC" },
	{ 0x708105, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x70820E, "as electronics GmbH", "AS-ELECTRONICS" },
	{ 0x70828E, "OleumTech Corporation", "OLEUMTECH" },
	{ 0x7085C6, "Pace plc.", "PACE-PLC" },
	{ 0x708B78, "citygrow technology co., ltd", "CITYGROW" },
	{ 0x708D09, "Nokia Corporation", "NOKIA" },
	{ 0x709383, "Intelligent Optical Network High Tech CO.,LTD.", "INTELLIGENT-OPTICAL-NETWORK-HIGH-TECH" },
	{ 0x7093F8, "Space Monkey, Inc.", "SPACE-MONKEY" },
	{ 0x709756, "Happyelectronics Co.,Ltd", "HAPPYELECTRONICS" },
	{ 0x709A0B, "Italian Institute of Technology", "ITALIAN-INSTITUTE-OF" },
	{ 0x709BA5, "Shenzhen Y&D Electronics Co.,LTD.", "SHENZHEN-Y-D-ELECTRONICS" },
	{ 0x709BFC, "Bryton Inc.", "BRYTON" },
	{ 0x709C8F, "Nero AG", "NERO" },
	{ 0x709E29, "Sony Computer Entertainment Inc.", "SONY-ENTERTAINMENT" },
	{ 0x709E86, "X6D Limited", "X6D" },
	{ 0x709F2D, "zte corporation", "ZTE" },
	{ 0x70A191, "Trendsetter Medical, LLC", "TRENDSETTER-MEDICAL-LLC" },
	{ 0x70A41C, "Advanced Wireless Dynamics S.L.", "ADVANCED-WIRELESS-DYNAMICS-S-L" },
	{ 0x70A66A, "Prox Dynamics AS", "PROX-DYNAMICS-AS" },
	{ 0x70A8E3, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x70AAB2, "Research In Motion", "RESEARCH-IN-MOTION" },
	{ 0x70AD54, "Malvern Instruments Ltd", "MALVERN-INSTRUMENTS" },
	{ 0x70AF25, "Nishiyama Industry Co.,LTD.", "NISHIYAMA-INDUSTRY" },
	{ 0x70B035, "Shenzhen Zowee Technology Co., Ltd", "SHENZHEN-ZOWEE" },
	{ 0x70B08C, "Shenou Communication Equipment Co.,Ltd", "SHENOU-COMMUNICATION-EQUIPMENT" },
	{ 0x70B14E, "Pace plc", "PACE-PLC" },
	{ 0x70B265, "Hiltron s.r.l.", "HILTRON-S-R-L" },
	{ 0x70B3D5, "IEEE REGISTRATION AUTHORITY - Please see OUI36 public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-OUI36-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x70B599, "Embedded Technologies s.r.o.", "EMBEDDED-TECHNOLOGIES-S-R-O" },
	{ 0x70B921, "FiberHome Telecommunication Technologies CO.,LTD", "FIBERHOME-TELECOMMUNICATION-TECHNOLOGIES" },
	{ 0x70BAEF, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x70C6AC, "Bosch Automotive Aftermarket", "BOSCH-AUTOMOTIVE-AFTERMARKET" },
	{ 0x70C76F, "INNO S", "INNO-S" },
	{ 0x70CA9B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x70CD60, "Apple", "APPLE" },
	{ 0x70D4F2, "RIM", "RIM" },
	{ 0x70D57E, "Scalar Corporation", "SCALAR" },
	{ 0x70D5E7, "Wellcore Corporation", "WELLCORE" },
	{ 0x70D6B6, "Metrum Technologies", "METRUM-TECHNOLOGIES" },
	{ 0x70D880, "Upos System sp. z o.o.", "UPOS-SYSTEM-SP-Z-O-O" },
	{ 0x70DA9C, "TECSEN", "TECSEN" },
	{ 0x70DDA1, "Tellabs", "TELLABS" },
	{ 0x70DEE2, "Apple", "APPLE" },
	{ 0x70E027, "HONGYU COMMUNICATION TECHNOLOGY LIMITED", "HONGYU-COMMUNICATION" },
	{ 0x70E139, "3view Ltd", "3VIEW" },
	{ 0x70E24C, "SAE IT-systems GmbH & Co. KG", "SAE-IT" },
	{ 0x70E284, "Wistron InfoComm(Zhongshan) Corporation", "WISTRON-INFOCOMM-ZHONGSHAN" },
	{ 0x70E72C, "Apple", "APPLE" },
	{ 0x70E843, "Beijing C&W Optical Communication Technology Co.,Ltd.", "BEIJING-C-W-OPTICAL-COMMUNICATION" },
	{ 0x70EE50, "Netatmo", "NETATMO" },
	{ 0x70F176, "Data Modul AG", "DATA-MODUL" },
	{ 0x70F196, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0x70F1A1, "Liteon Technology Corporation", "LITEON" },
	{ 0x70F1E5, "Xetawave LLC", "XETAWAVE-LLC" },
	{ 0x70F395, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0x70F927, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x70F96D, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x70FC8C, "OneAccess SA", "ONEACCESS" },
	{ 0x70FF5C, "Cheerzing Communication(Xiamen)Technology Co.,Ltd", "CHEERZING-COMMUNICATION-XIAMEN" },
	{ 0x70FF76, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x7403BD, "Buffalo Inc.", "BUFFALO" },
	{ 0x74042B, "Lenovo Mobile Communication (Wuhan) Company Limited", "LENOVO-MOBILE-COMMUNICATION-WUHAN-COMPANY" },
	{ 0x740ABC, "JSJS Designs (Europe) Limited", "JSJS-DESIGNS-EUROPE" },
	{ 0x740EDB, "Optowiz Co., Ltd", "OPTOWIZ" },
	{ 0x741489, "SRT Wireless", "SRT-WIRELESS" },
	{ 0x7415E2, "Tri-Sen Systems Corporation", "TRI-SEN" },
	{ 0x7419F8, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x741E93, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0x74258A, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x7426AC, "Cisco", "CISCO" },
	{ 0x74273C, "ChangYang Technology (Nanjing) Co., LTD", "CHANGYANG-NANJING" },
	{ 0x7427EA, "Elitegroup Computer Systems Co., Ltd.", "ELITEGROUP" },
	{ 0x7429AF, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x742B0F, "Infinidat Ltd.", "INFINIDAT" },
	{ 0x742B62, "Fujitsu Limited", "FUJITSU" },
	{ 0x742D0A, "Norfolk Elektronik AG", "NORFOLK-ELEKTRONIK" },
	{ 0x742EFC, "DirectPacket Research, Inc,", "DIRECTPACKET-RESEARCH" },
	{ 0x742F68, "Azurewave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x743170, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x743256, "NT-ware Systemprg GmbH", "NT-WARE-SYSTEMPRG" },
	{ 0x74372F, "Tongfang Shenzhen Cloudcomputing Technology Co.,Ltd", "TONGFANG-SHENZHEN-CLOUDCOMPUTING" },
	{ 0x743889, "ANNAX Anzeigesysteme GmbH", "ANNAX-ANZEIGESYSTEME" },
	{ 0x743ECB, "Gentrice tech", "GENTRICE-TECH" },
	{ 0x744401, "NETGEAR", "NETGEAR" },
	{ 0x74458A, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x7446A0, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x744BE9, "EXPLORER HYPERTECH CO.,LTD", "EXPLORER-HYPERTECH" },
	{ 0x744D79, "Arrive Systems Inc.", "ARRIVE" },
	{ 0x7451BA, "XIAOMI INC", "XIAOMI" },
	{ 0x745327, "COMMSEN CO., LIMITED", "COMMSEN" },
	{ 0x74547D, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x745612, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x745798, "TRUMPF Laser GmbH + Co. KG", "TRUMPF-LASER-+" },
	{ 0x745C9F, "TCT mobile ltd.", "TCT-MOBILE" },
	{ 0x745E1C, "PIONEER CORPORATION", "PIONEER" },
	{ 0x745F00, "Samsung Semiconductor Inc.", "SAMSUNG-SEMICONDUCTOR" },
	{ 0x745FAE, "TSL PPL", "TSL-PPL" },
	{ 0x7463DF, "VTS GmbH", "VTS" },
	{ 0x7465D1, "Atlinks", "ATLINKS" },
	{ 0x746630, "T:mi Ytti", "T:MI-YTTI" },
	{ 0x746A3A, "Aperi Corporation", "APERI" },
	{ 0x746A89, "Rezolt Corporation", "REZOLT" },
	{ 0x746A8F, "VS Vision Systems GmbH", "VS-VISION" },
	{ 0x746B82, "MOVEK", "MOVEK" },
	{ 0x746F3D, "Contec GmbH", "CONTEC" },
	{ 0x7472F2, "Chipsip Technology Co., Ltd.", "CHIPSIP" },
	{ 0x747548, "Amazon Technologies Inc.", "AMAZON-TECHNOLOGIES" },
	{ 0x747818, "ServiceAssure", "SERVICEASSURE" },
	{ 0x747B7A, "ETH Inc.", "ETH" },
	{ 0x747DB6, "Aliwei Communications, Inc", "ALIWEI-COMMUNICATION" },
	{ 0x747E1A, "Red Embedded Design Limited", "RED-EMBEDDED-DESIGN" },
	{ 0x747E2D, "Beijing Thomson CITIC Digital Technology Co. LTD.", "BEIJING-THOMSON-CITIC-DIGITAL" },
	{ 0x748114, "Apple", "APPLE" },
	{ 0x74852A, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x74867A, "Dell Inc", "DELL" },
	{ 0x74882A, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x74888B, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x748E08, "Bestek Corp.", "BESTEK" },
	{ 0x748EF8, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x748F1B, "MasterImage 3D", "MASTERIMAGE-3D" },
	{ 0x748F4D, "MEN Mikro Elektronik GmbH", "MEN-MIKRO-ELEKTRONIK" },
	{ 0x749050, "Renesas Electronics Corporation", "RENESAS-ELECTRONICS" },
	{ 0x74911A, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x7491BD, "Four systems Co.,Ltd.", "FOUR" },
	{ 0x7493A4, "Zebra Technologies Corp.", "ZEBRA-TECHNOLOGIES" },
	{ 0x74943D, "AgJunction", "AGJUNCTION" },
	{ 0x749637, "Todaair Electronic Co., Ltd", "TODAAIR" },
	{ 0x749975, "IBM Corporation", "IBM" },
	{ 0x749C52, "Huizhou Desay SV Automotive Co., Ltd.", "HUIZHOU-DESAY-SV-AUTOMOTIVE" },
	{ 0x749CE3, "Art2Wave Canada Inc.", "ART2WAVE-CANADA" },
	{ 0x749DDC, "2Wire", "2WIRE" },
	{ 0x74A02F, "Cisco", "CISCO" },
	{ 0x74A063, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x74A2E6, "Cisco", "CISCO" },
	{ 0x74A34A, "ZIMI CORPORATION", "ZIMI" },
	{ 0x74A4A7, "QRS Music Technologies, Inc.", "QRS-MUSIC-TECHNOLOGIES" },
	{ 0x74A4B5, "Powerleader Science and Technology Co. Ltd.", "POWERLEADER-SCIENCE-AND" },
	{ 0x74A722, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x74ADB7, "China Mobile Group Device Co.,Ltd.", "CHINA-MOBILE-GROUP-DEVICE" },
	{ 0x74AE76, "iNovo Broadband, Inc.", "INOVO-BROADBAND" },
	{ 0x74B00C, "Network Video Technologies, Inc", "NETWORK-VIDEO-TECHNOLOGIES" },
	{ 0x74B9EB, "Fujian JinQianMao Electronic Technology Co.,Ltd", "FUJIAN-JINQIANMAO" },
	{ 0x74BADB, "Longconn Electornics(shenzhen)Co.,Ltd", "LONGCONN-ELECTORNICS-SHENZHEN" },
	{ 0x74BE08, "ATEK Products, LLC", "ATEK-PRODUCTS-LLC" },
	{ 0x74BFA1, "HYUNTECK", "HYUNTECK" },
	{ 0x74C246, "Amazon Technologies Inc.", "AMAZON-TECHNOLOGIES" },
	{ 0x74C621, "Zhejiang Hite Renewable Energy Co.,LTD", "ZHEJIANG-HITE-RENEWABLE-ENERGY" },
	{ 0x74C99A, "Ericsson AB", "ERICSSON-AB" },
	{ 0x74CA25, "Calxeda, Inc.", "CALXEDA" },
	{ 0x74CD0C, "Smith Myers Communications Ltd.", "SMITH-MYERS-COMMUNICATION" },
	{ 0x74CE56, "Packet Force Technology Limited Company", "PACKET-FORCE-COMPANY" },
	{ 0x74D02B, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x74D0DC, "ERICSSON AB", "ERICSSON-AB" },
	{ 0x74D435, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x74D675, "WYMA Tecnologia", "WYMA-TECNOLOGIA" },
	{ 0x74D6EA, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x74D850, "Evrisko Systems", "EVRISKO" },
	{ 0x74DA38, "Edimax Technology Co. Ltd.", "EDIMAX" },
	{ 0x74DBD1, "Ebay Inc", "EBAY" },
	{ 0x74DE2B, "Liteon Technology Corporation", "LITEON" },
	{ 0x74E06E, "Ergophone GmbH", "ERGOPHONE" },
	{ 0x74E14A, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x74E1B6, "Apple", "APPLE" },
	{ 0x74E277, "Vizmonet Pte Ltd", "VIZMONET-PTE" },
	{ 0x74E28C, "Microsoft Corporation", "MICROSOFT" },
	{ 0x74E2F5, "Apple", "APPLE" },
	{ 0x74E424, "APISTE CORPORATION", "APISTE" },
	{ 0x74E50B, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x74E537, "RADSPIN", "RADSPIN" },
	{ 0x74E543, "Liteon Technology Corporation", "LITEON" },
	{ 0x74E6E2, "Dell Inc.", "DELL" },
	{ 0x74E7C6, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x74EA3A, "TP-LINK Technologies Co.,Ltd.", "TP-LINK-TECHNOLOGIES" },
	{ 0x74ECF1, "Acumen", "ACUMEN" },
	{ 0x74F06D, "AzureWave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x74F07D, "BnCOM Co.,Ltd", "BNCOM" },
	{ 0x74F102, "Beijing HCHCOM Technology Co., Ltd", "BEIJING-HCHCOM" },
	{ 0x74F413, "Maxwell Forest", "MAXWELL-FOREST" },
	{ 0x74F612, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x74F726, "Neuron Robotics", "NEURON-ROBOTICS" },
	{ 0x74F85D, "Berkeley Nucleonics Corp", "BERKELEY-NUCLEONICS" },
	{ 0x74FDA0, "Compupal (Group) Corporation", "COMPUPAL-GROUP" },
	{ 0x74FE48, "ADVANTECH CO., LTD.", "ADVANTECH" },
	{ 0x74FF7D, "Wren Sound Systems, LLC", "WREN-SOUND-LLC" },
	{ 0x78028F, "Adaptive Spectrum and Signal Alignment (ASSIA), Inc.", "ADAPTIVE-SPECTRUM-AND-SIGNAL-ALIGNMENT-ASSIA" },
	{ 0x780738, "Z.U.K. Elzab S.A.", "Z-U-K-ELZAB" },
	{ 0x781185, "NBS Payment Solutions Inc.", "NBS-PAYMENT-SOLUTIONS" },
	{ 0x7812B8, "ORANTEK LIMITED", "ORANTEK" },
	{ 0x781881, "AzureWave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x78192E, "NASCENT Technology", "NASCENT" },
	{ 0x7819F7, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x781C5A, "SHARP Corporation", "SHARP" },
	{ 0x781DBA, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x781DFD, "Jabil Inc", "JABIL" },
	{ 0x781FDB, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x78223D, "Affirmed Networks", "AFFIRMED-NETWORKS" },
	{ 0x7824AF, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x782544, "Omnima Limited", "OMNIMA" },
	{ 0x7825AD, "SAMSUNG ELECTRONICS CO., LTD.", "SAMSUNG-ELECTRONICS" },
	{ 0x782BCB, "Dell Inc", "DELL" },
	{ 0x782EEF, "Nokia Corporation", "NOKIA" },
	{ 0x78303B, "Stephen Technologies Co.,Limited", "STEPHEN-TECHNOLOGIES" },
	{ 0x7830E1, "UltraClenz, LLC", "ULTRACLENZ-LLC" },
	{ 0x78312B, "zte corporation", "ZTE" },
	{ 0x7831C1, "Apple", "APPLE" },
	{ 0x78324F, "Millennium Group, Inc.", "MILLENNIUM-GROUP" },
	{ 0x783A84, "Apple", "APPLE" },
	{ 0x783CE3, "Kai-EE", "KAI-EE" },
	{ 0x783D5B, "TELNET Redes Inteligentes S.A.", "TELNET-REDES-INTELIGENTES" },
	{ 0x783E53, "BSkyB Ltd", "BSKYB" },
	{ 0x783F15, "EasySYNC Ltd.", "EASYSYNC" },
	{ 0x7840E4, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x784405, "FUJITU(HONG KONG) ELECTRONIC Co.,LTD.", "FUJITU-HONG-KONG" },
	{ 0x784476, "Zioncom technology co.,ltd", "ZIONCOM" },
	{ 0x784561, "CyberTAN Technology Inc.", "CYBERTAN" },
	{ 0x7845C4, "Dell Inc", "DELL" },
	{ 0x7846C4, "DAEHAP HYPER-TECH", "DAEHAP-HYPER-TECH" },
	{ 0x78471D, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x784859, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x78491D, "The Will-Burt Company", "THE-WILL-BURT-COMPANY" },
	{ 0x784B08, "f.robotics acquisitions ltd", "F-ROBOTICS-ACQUISITIONS" },
	{ 0x784B87, "Murata Manufacturing Co.,Ltd.", "MURATA-MANUFACTURING" },
	{ 0x78510C, "LiveU Ltd.", "LIVEU" },
	{ 0x78521A, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x785262, "Shenzhen Hojy Software Co., Ltd.", "SHENZHEN-HOJY-SOFTWARE" },
	{ 0x78542E, "D-Link International", "D-LINK" },
	{ 0x785517, "SankyuElectronics", "SANKYUELECTRONICS" },
	{ 0x785712, "Mobile Integration Workgroup", "MOBILE-INTEGRATION-WORKGROUP" },
	{ 0x7858F3, "Vachen Co.,Ltd", "VACHEN" },
	{ 0x78593E, "RAFI GmbH & Co.KG", "RAFI" },
	{ 0x78595E, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x785968, "Hon Hai Precision Ind.Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x785C72, "Hioso Technology Co., Ltd.", "HIOSO" },
	{ 0x785F4C, "Argox Information Co., Ltd.", "ARGOX-INFORMATION" },
	{ 0x78617C, "MITSUMI ELECTRIC CO.,LTD", "MITSUMI-ELECTRIC" },
	{ 0x7866AE, "ZTEC Instruments, Inc.", "ZTEC-INSTRUMENTS" },
	{ 0x786A89, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x786C1C, "Apple", "APPLE" },
	{ 0x78719C, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x787E61, "Apple", "APPLE" },
	{ 0x787F62, "GiK mbH", "GIK-MBH" },
	{ 0x78818F, "Server Racks Australia Pty Ltd", "SERVER-RACKS-AUSTRALIA-PTY" },
	{ 0x78843C, "Sony Corporation", "SONY" },
	{ 0x7884EE, "INDRA ESPACIO S.A.", "INDRA-ESPACIO" },
	{ 0x788973, "CMC", "CMC" },
	{ 0x788C54, "Eltek Technologies LTD", "ELTEK-TECHNOLOGIES" },
	{ 0x788DF7, "Hitron Technologies. Inc", "HITRON-TECHNOLOGIES" },
	{ 0x78923E, "Nokia Corporation", "NOKIA" },
	{ 0x78929C, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x789684, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x7898FD, "Q9 Networks Inc.", "Q9-NETWORKS" },
	{ 0x78995C, "Nationz Technologies Inc", "NATIONZ-TECHNOLOGIES" },
	{ 0x789966, "Musilab Electronics (DongGuan)Co.,Ltd.", "MUSILAB-ELECTRONICS-DONGGUAN" },
	{ 0x78998F, "MEDILINE ITALIA SRL", "MEDILINE-ITALIA-SRL" },
	{ 0x789CE7, "Shenzhen Aikede Technology Co., Ltd", "SHENZHEN-AIKEDE" },
	{ 0x789ED0, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x789F4C, "HOERBIGER Elektronik GmbH", "HOERBIGER-ELEKTRONIK" },
	{ 0x789F87, "Siemens AG I IA PP PRM", "SIEMENS-I-IA-PP-PRM" },
	{ 0x78A051, "iiNet Labs Pty Ltd", "IINET-LABS-PTY" },
	{ 0x78A106, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x78A183, "Advidia", "ADVIDIA" },
	{ 0x78A2A0, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x78A351, "SHENZHEN ZHIBOTONG ELECTRONICS CO.,LTD", "SHENZHEN-ZHIBOTONG-ELECTRONICS" },
	{ 0x78A3E4, "Apple", "APPLE" },
	{ 0x78A504, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x78A5DD, "Shenzhen Smarteye Digital Electronics Co., Ltd", "SHENZHEN-SMARTEYE-DIGITAL-ELECTRONICS" },
	{ 0x78A683, "Precidata", "PRECIDATA" },
	{ 0x78A6BD, "DAEYEON Control&Instrument Co,.Ltd", "DAEYEON-CONTROL-INSTRUMENT" },
	{ 0x78A714, "Amphenol", "AMPHENOL" },
	{ 0x78A873, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x78AB60, "ABB Australia", "ABB-AUSTRALIA" },
	{ 0x78ABBB, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x78ACBF, "Igneous Systems", "IGNEOUS" },
	{ 0x78ACC0, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x78AE0C, "Far South Networks", "FAR-SOUTH-NETWORKS" },
	{ 0x78B3B9, "ShangHai sunup lighting CO.,LTD", "SHANGHAI-SUNUP-LIGHTING" },
	{ 0x78B3CE, "Elo touch solutions", "ELO-TOUCH-SOLUTIONS" },
	{ 0x78B5D2, "Ever Treasure Industrial Limited", "EVER-TREASURE-INDUSTRIAL" },
	{ 0x78B6C1, "AOBO Telecom Co.,Ltd", "AOBO-TELECOM" },
	{ 0x78B81A, "INTER SALES A/S", "INTER-SALES-A/S" },
	{ 0x78BAD0, "Shinybow Technology Co. Ltd.", "SHINYBOW" },
	{ 0x78BEB6, "Enhanced Vision", "ENHANCED-VISION" },
	{ 0x78BEBD, "STULZ GmbH", "STULZ" },
	{ 0x78C40E, "H&D Wireless", "H-D-WIRELESS" },
	{ 0x78C4AB, "Shenzhen Runsil Technology Co.,Ltd", "SHENZHEN-RUNSIL" },
	{ 0x78C5E5, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x78C6BB, "Innovasic, Inc.", "INNOVASIC" },
	{ 0x78CA04, "Nokia Corporation", "NOKIA" },
	{ 0x78CA39, "Apple", "APPLE" },
	{ 0x78CA5E, "ELNO", "ELNO" },
	{ 0x78CB33, "DHC Software Co.,Ltd", "DHC-SOFTWARE" },
	{ 0x78CD8E, "SMC Networks Inc", "SMC-NETWORKS" },
	{ 0x78D004, "Neousys Technology Inc.", "NEOUSYS" },
	{ 0x78D129, "Vicos", "VICOS" },
	{ 0x78D34F, "Pace-O-Matic, Inc.", "PACE-O-MATIC" },
	{ 0x78D38D, "HONGKONG YUNLINK TECHNOLOGY LIMITED", "HONGKONG-YUNLINK" },
	{ 0x78D5B5, "NAVIELEKTRO KY", "NAVIELEKTRO-KY" },
	{ 0x78D66F, "Aristocrat Technologies Australia Pty. Ltd.", "ARISTOCRAT-TECHNOLOGIES-AUSTRALIA-PTY" },
	{ 0x78D6F0, "Samsung Electro Mechanics", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x78D752, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x78D99F, "NuCom HK Ltd.", "NUCOM-HK" },
	{ 0x78DA6E, "Cisco", "CISCO" },
	{ 0x78DAB3, "GBO Technology", "GBO" },
	{ 0x78DD08, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x78DDD6, "c-scape", "C-SCAPE" },
	{ 0x78DEE4, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x78E3B5, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x78E400, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x78E7D1, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x78E8B6, "zte corporation", "ZTE" },
	{ 0x78E980, "RainUs Co.,Ltd", "RAINUS" },
	{ 0x78EB14, "SHENZHEN FAST TECHNOLOGIES CO.,LTD", "SHENZHEN-FAST-TECHNOLOGIES" },
	{ 0x78EC22, "Shanghai Qihui Telecom Technology Co., LTD", "SHANGHAI-QIHUI-TELECOM" },
	{ 0x78EC74, "Kyland-USA", "KYLAND-USA" },
	{ 0x78EF4C, "Unetconvergence Co., Ltd.", "UNETCONVERGENCE" },
	{ 0x78F5E5, "BEGA Gantenbrink-Leuchten KG", "BEGA-GANTENBRINK-LEUCHTEN" },
	{ 0x78F5FD, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x78F7BE, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x78F7D0, "Silverbrook Research", "SILVERBROOK-RESEARCH" },
	{ 0x78F944, "PRIVATE", "PRIVATE" },
	{ 0x78FC14, "B Communications Pty Ltd", "B-COMMUNICATION-PTY" },
	{ 0x78FD94, "Apple", "APPLE" },
	{ 0x78FE3D, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x78FE41, "Socus networks", "SOCUS-NETWORKS" },
	{ 0x78FEE2, "Shanghai Diveo Technology Co., Ltd", "SHANGHAI-DIVEO" },
	{ 0x78FF57, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x7C0187, "Curtis Instruments, Inc.", "CURTIS-INSTRUMENTS" },
	{ 0x7C02BC, "Hansung Electronics Co. LTD", "HANSUNG-ELECTRONICS" },
	{ 0x7C034C, "SAGEMCOM", "SAGEMCOM" },
	{ 0x7C03D8, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0x7C0507, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0x7C051E, "RAFAEL LTD.", "RAFAEL" },
	{ 0x7C0623, "Ultra Electronics, CIS", "ULTRA-ELECTRONICS-CIS" },
	{ 0x7C08D9, "Shanghai B-Star Technology Co", "SHANGHAI-B-STAR" },
	{ 0x7C092B, "Bekey A/S", "BEKEY-A/S" },
	{ 0x7C0A50, "J-MEX Inc.", "J-MEX" },
	{ 0x7C0ECE, "Cisco", "CISCO" },
	{ 0x7C11BE, "Apple", "APPLE" },
	{ 0x7C11CD, "QianTang Technology", "QIANTANG" },
	{ 0x7C1476, "Damall Technologies SAS", "DAMALL-TECHNOLOGIES-SAS" },
	{ 0x7C160D, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x7C1A03, "8Locations Co., Ltd.", "8LOCATIONS" },
	{ 0x7C1AFC, "Dalian Co-Edifice Video Technology Co., Ltd", "DALIAN--EDIFICE-VIDEO" },
	{ 0x7C1DD9, "XIAOMI IMC", "XIAOMI-IMC" },
	{ 0x7C1E52, "Microsoft", "MICROSOFT" },
	{ 0x7C1EB3, "2N TELEKOMUNIKACE a.s.", "2N-TELEKOMUNIKACE-A-S" },
	{ 0x7C2048, "KoamTac", "KOAMTAC" },
	{ 0x7C2064, "Alcatel Lucent IPD", "ALCATEL-LUCENT-IPD" },
	{ 0x7C2587, "chaowifi.com", "CHAOWIFI-COM" },
	{ 0x7C2CF3, "Secure Electrans Ltd", "SECURE-ELECTRANS" },
	{ 0x7C2E0D, "Blackmagic Design", "BLACKMAGIC-DESIGN" },
	{ 0x7C2F80, "Gigaset Communications GmbH", "GIGASET-COMMUNICATION" },
	{ 0x7C336E, "MEG Electronics Inc.", "MEG-ELECTRONICS" },
	{ 0x7C386C, "Real Time Logic", "REAL-TIME-LOGIC" },
	{ 0x7C3920, "SSOMA SECURITY", "SSOMA-SECURITY" },
	{ 0x7C3BD5, "Imago Group", "IMAGO-GROUP" },
	{ 0x7C3CB6, "Shenzhen Homecare Technology Co.,Ltd.", "SHENZHEN-HOMECARE" },
	{ 0x7C3E9D, "PATECH", "PATECH" },
	{ 0x7C438F, "E-Band Communications Corp.", "E-BAND-COMMUNICATION" },
	{ 0x7C444C, "Entertainment Solutions, S.L.", "ENTERTAINMENT-SOLUTIONS-S-L" },
	{ 0x7C49B9, "Plexus Manufacturing Sdn Bhd", "PLEXUS-MANUFACTURING-SDN-BHD" },
	{ 0x7C4A82, "Portsmith LLC", "PORTSMITH-LLC" },
	{ 0x7C4AA8, "MindTree Wireless PVT Ltd", "MINDTREE-WIRELESS-PVT" },
	{ 0x7C4B78, "Red Sun Synthesis Pte Ltd", "RED-SUN-SYNTHESIS-PTE" },
	{ 0x7C4C58, "Scale Computing, Inc.", "SCALE-COMPUTING" },
	{ 0x7C4CA5, "BSkyB Ltd", "BSKYB" },
	{ 0x7C4FB5, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x7C534A, "Metamako", "METAMAKO" },
	{ 0x7C55E7, "YSI, Inc.", "YSI" },
	{ 0x7C5CF8, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x7C6097, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x7C6193, "HTC Corporation", "HTC" },
	{ 0x7C669D, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x7C69F6, "Cisco", "CISCO" },
	{ 0x7C6AB3, "IBC TECHNOLOGIES INC.", "IBC-TECHNOLOGIES" },
	{ 0x7C6AC3, "GatesAir, Inc", "GATESAIR" },
	{ 0x7C6ADB, "SafeTone Technology Co.,Ltd", "SAFETONE" },
	{ 0x7C6B33, "Tenyu Tech Co. Ltd.", "TENYU-TECH" },
	{ 0x7C6B52, "Tigaro Wireless", "TIGARO-WIRELESS" },
	{ 0x7C6C39, "PIXSYS SRL", "PIXSYS-SRL" },
	{ 0x7C6C8F, "AMS NEVE LTD", "AMS-NEVE" },
	{ 0x7C6D62, "Apple", "APPLE" },
	{ 0x7C6DF8, "Apple", "APPLE" },
	{ 0x7C6F06, "Caterpillar Trimble Control Technologies", "CATERPILLAR-TRIMBLE-CONTROL-TECHNOLOGIES" },
	{ 0x7C6FF8, "ShenZhen ACTO Digital Video Technology Co.,Ltd.", "SHENZHEN-ACTO-DIGITAL-VIDEO" },
	{ 0x7C70BC, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x7C72E4, "Unikey Technologies", "UNIKEY-TECHNOLOGIES" },
	{ 0x7C7673, "ENMAS GmbH", "ENMAS" },
	{ 0x7C7A53, "Phytrex Technology Corp.", "PHYTREX" },
	{ 0x7C7A91, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x7C7BE4, "Z'SEDAI KENKYUSHO CORPORATION", "Z-SEDAI-KENKYUSHO" },
	{ 0x7C7D41, "Jinmuyu Electronics Co., Ltd.", "JINMUYU-ELECTRONICS" },
	{ 0x7C822D, "Nortec", "NORTEC" },
	{ 0x7C8274, "Shenzhen Hikeen Technology CO.,LTD", "SHENZHEN-HIKEEN" },
	{ 0x7C8306, "Glen Dimplex Nordic as", "GLEN-DIMPLEX-NORDIC-AS" },
	{ 0x7C8D91, "Shanghai Hongzhuo Information Technology co.,LTD", "SHANGHAI-HONGZHUO-INFORMATION" },
	{ 0x7C8EE4, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x7C94B2, "Philips Healthcare PCCI", "PHILIPS-HEALTHCARE-PCCI" },
	{ 0x7C95F3, "Cisco", "CISCO" },
	{ 0x7C9763, "Openmatics s.r.o.", "OPENMATICS-S-R-O" },
	{ 0x7C9A9B, "VSE valencia smart energy", "VSE-VALENCIA-SMART-ENERGY" },
	{ 0x7CA15D, "GN ReSound A/S", "GN-RESOUND-A/S" },
	{ 0x7CA29B, "D.SignT GmbH & Co. KG", "D-SIGNT" },
	{ 0x7CA61D, "MHL, LLC", "MHL-LLC" },
	{ 0x7CACB2, "Bosch Software Innovations GmbH", "BOSCH-SOFTWARE-INNOVATIONS" },
	{ 0x7CAD74, "Cisco", "CISCO" },
	{ 0x7CB03E, "OSRAM GmbH", "OSRAM" },
	{ 0x7CB177, "Satelco AG", "SATELCO" },
	{ 0x7CB21B, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x7CB232, "TCL King High Frequency EI,Co.,LTD", "TCL-KING-HIGH-FREQUENCY-EI" },
	{ 0x7CB542, "ACES Technology", "ACES" },
	{ 0x7CB733, "ASKEY COMPUTER CORP", "ASKEY" },
	{ 0x7CB77B, "Paradigm Electronics Inc", "PARADIGM-ELECTRONICS" },
	{ 0x7CBB6F, "Cosco Electronics Co., Ltd.", "COSCO-ELECTRONICS" },
	{ 0x7CBD06, "AE REFUsol", "AE-REFUSOL" },
	{ 0x7CBF88, "Mobilicom LTD", "MOBILICOM" },
	{ 0x7CBFB1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x7CC3A1, "Apple", "APPLE" },
	{ 0x7CC4EF, "Devialet", "DEVIALET" },
	{ 0x7CC537, "Apple", "APPLE" },
	{ 0x7CC709, "Shenzhen RF-LINK Elec&Technology.,Ltd", "SHENZHEN-RF-LINK-ELEC" },
	{ 0x7CC8AB, "Acro Associates, Inc.", "ACRO-ASSOCIATES" },
	{ 0x7CC8D0, "TIANJIN YAAN TECHNOLOGY CO., LTD.", "TIANJIN-YAAN" },
	{ 0x7CC8D7, "Damalisk", "DAMALISK" },
	{ 0x7CCB0D, "Antaira Technologies, LLC", "ANTAIRA-TECHNOLOGIES-LLC" },
	{ 0x7CCCB8, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x7CCD11, "MS-Magnet", "MS-MAGNET" },
	{ 0x7CCD3C, "Guangzhou Juzing Technology Co., Ltd", "GUANGZHOU-JUZING" },
	{ 0x7CCFCF, "Shanghai SEARI Intelligent System Co., Ltd", "SHANGHAI-SEARI-INTELLIGENT-SYSTEM" },
	{ 0x7CD1C3, "Apple", "APPLE" },
	{ 0x7CD30A, "INVENTEC Corporation", "INVENTEC" },
	{ 0x7CD762, "Freestyle Technology Pty Ltd", "FREESTYLE-PTY" },
	{ 0x7CD844, "Enmotus Inc", "ENMOTUS" },
	{ 0x7CD9FE, "New Cosmos Electric Co., Ltd.", "NEW-COSMOS-ELECTRIC" },
	{ 0x7CDA84, "Dongnian Networks Inc.", "DONGNIAN-NETWORKS" },
	{ 0x7CDD11, "Chongqing MAS SCI&TECH.Co.,Ltd", "CHONGQING-MAS-SCI-TECH" },
	{ 0x7CDD20, "IOXOS Technologies S.A.", "IOXOS-TECHNOLOGIES" },
	{ 0x7CDD90, "Shenzhen Ogemray Technology Co., Ltd.", "SHENZHEN-OGEMRAY" },
	{ 0x7CE044, "NEON Inc", "NEON" },
	{ 0x7CE1FF, "Computer Performance, Inc. DBA Digital Loggers, Inc.", "PERFORMANCE-DBA-DIGITAL-LOGGERS" },
	{ 0x7CE4AA, "PRIVATE", "PRIVATE" },
	{ 0x7CE524, "Quirky, Inc.", "QUIRKY" },
	{ 0x7CE56B, "ESEN Optoelectronics Technology Co.,Ltd.", "ESEN-OPTOELECTRONICS" },
	{ 0x7CE9D3, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x7CEBEA, "ASCT", "ASCT" },
	{ 0x7CEC79, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x7CED8D, "MICROSOFT", "MICROSOFT" },
	{ 0x7CEF18, "Creative Product Design Pty. Ltd.", "CREATIVE-PRODUCT-DESIGN-PTY" },
	{ 0x7CEF8A, "Inhon International Ltd.", "INHON" },
	{ 0x7CF05F, "Apple", "APPLE" },
	{ 0x7CF098, "Bee Beans Technologies, Inc.", "BEE-BEANS-TECHNOLOGIES" },
	{ 0x7CF0BA, "Linkwell Telesystems Pvt Ltd", "LINKWELL-TELESYSTEMS-PVT" },
	{ 0x7CF429, "NUUO Inc.", "NUUO" },
	{ 0x7CF854, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x7CFADF, "Apple", "APPLE" },
	{ 0x7CFE28, "Salutron Inc.", "SALUTRON" },
	{ 0x7CFE4E, "Shenzhen Safe vision Technology Co.,LTD", "SHENZHEN-SAFE-VISION" },
	{ 0x7CFF62, "Huizhou Super Electron Technology Co.,Ltd.", "HUIZHOU-SUPER-ELECTRON" },
	{ 0x80000B, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x800010, "ATT BELL LABORATORIES", "ATT-BELL-LABORATORIES" },
	{ 0x80006E, "Apple", "APPLE" },
	{ 0x800184, "HTC Corporation", "HTC" },
	{ 0x8005DF, "Montage Technology Group Limited", "MONTAGE-GROUP" },
	{ 0x8007A2, "Esson Technology Inc.", "ESSON" },
	{ 0x800902, "Keysight Technologies, Inc.", "KEYSIGHT-TECHNOLOGIES" },
	{ 0x800A06, "COMTEC co.,ltd", "COMTEC" },
	{ 0x800E24, "ForgetBox", "FORGETBOX" },
	{ 0x801440, "Sunlit System Technology Corp", "SUNLIT-SYSTEM" },
	{ 0x8014A8, "Guangzhou V-SOLUTION Electronic Technology Co., Ltd.", "GUANGZHOU-V-SOLUTION" },
	{ 0x8016B7, "Brunel University", "BRUNEL-UNIVERSITY" },
	{ 0x80177D, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x8018A7, "Samsung Eletronics Co., Ltd", "SAMSUNG-ELETRONICS" },
	{ 0x801934, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x801967, "Shanghai Reallytek Information Technology Co.,Ltd", "SHANGHAI-REALLYTEK-INFORMATION" },
	{ 0x801DAA, "Avaya Inc", "AVAYA" },
	{ 0x801F02, "Edimax Technology Co. Ltd.", "EDIMAX" },
	{ 0x8020AF, "Trade FIDES, a.s.", "TRADE-FIDES-A-S" },
	{ 0x802275, "Beijing Beny Wave Technology Co Ltd", "BEIJING-BENY-WAVE" },
	{ 0x802AA8, "Ubiquiti Networks, Inc.", "UBIQUITI-NETWORKS" },
	{ 0x802AFA, "Germaneers GmbH", "GERMANEERS" },
	{ 0x802DE1, "Solarbridge Technologies", "SOLARBRIDGE-TECHNOLOGIES" },
	{ 0x802E14, "azeti Networks AG", "AZETI-NETWORKS" },
	{ 0x802FDE, "Zurich Instruments AG", "ZURICH-INSTRUMENTS" },
	{ 0x803457, "OT Systems Limited", "OT" },
	{ 0x803773, "Netgear Inc", "NETGEAR" },
	{ 0x8038BC, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x8038FD, "LeapFrog Enterprises, Inc.", "LEAPFROG-ENTERPRISES" },
	{ 0x8039E5, "PATLITE CORPORATION", "PATLITE" },
	{ 0x803B9A, "ghe-ces electronic ag", "GHE-CES" },
	{ 0x803F5D, "Winstars Technology Ltd", "WINSTARS" },
	{ 0x803FD6, "bytes at work AG", "BYTES-AT-WORK" },
	{ 0x80414E, "BBK Electronics Corp., Ltd.,", "BBK-ELECTRONICS" },
	{ 0x80427C, "Adolf Tedsen GmbH & Co. KG", "ADOLF-TEDSEN" },
	{ 0x804731, "Packet Design, Inc.", "PACKET-DESIGN" },
	{ 0x8048A5, "SICHUAN TIANYI COMHEART TELECOM CO.,LTD", "SICHUAN-TIANYI-COMHEART-TELECOM" },
	{ 0x804971, "Apple", "APPLE" },
	{ 0x804B20, "Ventilation Control", "VENTILATION-CONTROL" },
	{ 0x804F58, "ThinkEco, Inc.", "THINKECO" },
	{ 0x80501B, "Nokia Corporation", "NOKIA" },
	{ 0x805067, "W & D TECHNOLOGY CORPORATION", "W-D" },
	{ 0x8056F2, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x805719, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x8058C5, "NovaTec Kommunikationstechnik GmbH", "NOVATEC-KOMMUNIKATIONSTECHNIK" },
	{ 0x8059FD, "Noviga", "NOVIGA" },
	{ 0x806007, "RIM", "RIM" },
	{ 0x80618F, "Shenzhen sangfei consumer communications co.,ltd", "SHENZHEN-SANGFEI-CONSUMER-COMMUNICATION" },
	{ 0x806459, "Nimbus Inc.", "NIMBUS" },
	{ 0x8065E9, "BenQ Corporation", "BENQ" },
	{ 0x806629, "Prescope Technologies CO.,LTD.", "PRESCOPE-TECHNOLOGIES" },
	{ 0x806AB0, "Tinno Mobile Technology Corp", "TINNO-MOBILE" },
	{ 0x806C1B, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0x806C8B, "KAESER KOMPRESSOREN AG", "KAESER-KOMPRESSOREN" },
	{ 0x806CBC, "NET New Electronic Technology GmbH", "NET-NEW" },
	{ 0x80711F, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x80717A, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x807459, "K's Co.,Ltd.", "K-S" },
	{ 0x807693, "Newag SA", "NEWAG" },
	{ 0x8079AE, "ShanDong Tecsunrise Co.,Ltd", "SHANDONG-TECSUNRISE" },
	{ 0x807A7F, "ABB Genway Xiamen Electrical Equipment CO., LTD", "ABB-GENWAY-XIAMEN-ELECTRICAL-EQUIPMENT" },
	{ 0x807B1E, "Corsair Components", "CORSAIR-COMPONENTS" },
	{ 0x807D1B, "Neosystem Co. Ltd.", "NEOSYSTEM" },
	{ 0x807DE3, "Chongqing Sichuan Instrument Microcircuit Co.LTD.", "CHONGQING-SICHUAN-INSTRUMENT-MICROCIRCUIT" },
	{ 0x8081A5, "TONGQING COMMUNICATION EQUIPMENT (SHENZHEN) Co.,Ltd", "TONGQING-COMMUNICATION-EQUIPMENT-SHENZHEN" },
	{ 0x808287, "ATCOM Technology Co.Ltd.", "ATCOM" },
	{ 0x808698, "Netronics Technologies Inc.", "NETRONICS-TECHNOLOGIES" },
	{ 0x8086F2, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x808917, "TP-LINK TECHNOLOGIES CO.,LTD", "TP-LINK-TECHNOLOGIES" },
	{ 0x808B5C, "Shenzhen Runhuicheng Technology Co., Ltd", "SHENZHEN-RUNHUICHENG" },
	{ 0x80912A, "Lih Rong electronic Enterprise Co., Ltd.", "LIH-RONG" },
	{ 0x8091C0, "AgileMesh, Inc.", "AGILEMESH" },
	{ 0x80929F, "Apple", "APPLE" },
	{ 0x809393, "Xapt GmbH", "XAPT" },
	{ 0x80946C, "TOKYO RADAR CORPORATION", "TOKYO-RADAR" },
	{ 0x8096B1, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x8096CA, "Hon Hai Precision Ind Co.,Ltd", "HON-HAI-PRECISION-IND" },
	{ 0x80971B, "Altenergy Power System,Inc.", "ALTENERGY-POWER-SYSTEM" },
	{ 0x809B20, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x80A1D7, "Shanghai DareGlobal Technologies Co.,Ltd", "SHANGHAI-DAREGLOBAL-TECHNOLOGIES" },
	{ 0x80A85D, "Osterhout Design Group", "OSTERHOUT-DESIGN-GROUP" },
	{ 0x80AAA4, "USAG", "USAG" },
	{ 0x80AD67, "Kasda Networks Inc", "KASDA-NETWORKS" },
	{ 0x80B219, "ELEKTRON TECHNOLOGY UK LIMITED", "ELEKTRON-UK" },
	{ 0x80B289, "Forworld Electronics Ltd.", "FORWORLD-ELECTRONICS" },
	{ 0x80B32A, "Alstom Grid", "ALSTOM-GRID" },
	{ 0x80B686, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x80B95C, "ELFTECH Co., Ltd.", "ELFTECH" },
	{ 0x80BAAC, "TeleAdapt Ltd", "TELEADAPT" },
	{ 0x80BAE6, "Neets", "NEETS" },
	{ 0x80BBEB, "Satmap Systems Ltd", "SATMAP" },
	{ 0x80BE05, "Apple", "APPLE" },
	{ 0x80C16E, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x80C63F, "Remec Broadband Wireless , LLC", "REMEC-BROADBAND-WIRELESS-LLC" },
	{ 0x80C6AB, "Technicolor USA Inc.", "TECHNICOLOR-USA" },
	{ 0x80C6CA, "Endian s.r.l.", "ENDIAN-S-R-L" },
	{ 0x80C862, "Openpeak, Inc", "OPENPEAK" },
	{ 0x80CEB1, "Theissen Training Systems GmbH", "THEISSEN-TRAINING" },
	{ 0x80CF41, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x80D019, "Embed, Inc", "EMBED" },
	{ 0x80D09B, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x80D18B, "Hangzhou I'converge Technology Co.,Ltd", "HANGZHOU-I-CONVERGE" },
	{ 0x80D21D, "AzureWave Technologies, Inc", "AZUREWAVE-TECHNOLOGIES" },
	{ 0x80D433, "LzLabs GmbH", "LZLABS" },
	{ 0x80D733, "QSR Automations, Inc.", "QSR-AUTOMATIONS" },
	{ 0x80DB31, "Power Quotient International Co., Ltd.", "POWER-QUOTIENT" },
	{ 0x80E4DA, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x80E650, "Apple", "APPLE" },
	{ 0x80EA96, "Apple", "APPLE" },
	{ 0x80EACA, "Dialog Semiconductor Hellas SA", "DIALOG-SEMICONDUCTOR-HELLAS" },
	{ 0x80EE73, "Shuttle Inc.", "SHUTTLE" },
	{ 0x80F25E, "Kyynel", "KYYNEL" },
	{ 0x80F503, "Pace plc", "PACE-PLC" },
	{ 0x80F593, "IRCO Sistemas de Telecomunicación S.A.", "IRCO-SISTEMAS-DE-TELECOMUNICACIóN" },
	{ 0x80F62E, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0x80F8EB, "RayTight", "RAYTIGHT" },
	{ 0x80FA5B, "CLEVO CO.", "CLEVO" },
	{ 0x80FB06, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x80FFA8, "UNIDIS", "UNIDIS" },
	{ 0x8400D2, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x8401A7, "Greyware Automation Products, Inc", "GREYWARE-AUTOMATION-PRODUCTS" },
	{ 0x840B2D, "SAMSUNG ELECTRO-MECHANICS CO., LTD", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x840F45, "Shanghai GMT Digital Technologies Co., Ltd", "SHANGHAI-GMT-DIGITAL-TECHNOLOGIES" },
	{ 0x841715, "GP Electronics (HK) Ltd.", "GP-ELECTRONICS-HK" },
	{ 0x841766, "Weifang GoerTek Electronics Co., Ltd", "WEIFANG-GOERTEK-ELECTRONICS" },
	{ 0x841826, "Osram GmbH", "OSRAM" },
	{ 0x84183A, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x841888, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x841B38, "Shenzhen Excelsecu Data Technology Co.,Ltd", "SHENZHEN-EXCELSECU-DATA" },
	{ 0x841B5E, "NETGEAR", "NETGEAR" },
	{ 0x841E26, "KERNEL-I Co.,LTD", "KERNEL-I" },
	{ 0x842141, "Shenzhen Ginwave Technologies Ltd.", "SHENZHEN-GINWAVE-TECHNOLOGIES" },
	{ 0x84248D, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0x84253F, "Silex Technology, Inc", "SILEX" },
	{ 0x8425A4, "Tariox Limited", "TARIOX" },
	{ 0x8425DB, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x842615, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0x84262B, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x842690, "BEIJING THOUGHT SCIENCE CO.,LTD.", "BEIJING-THOUGHT-SCIENCE" },
	{ 0x8427CE, "Corporation of the Presiding Bishop of The Church of Jesus Christ of Latter-day Saints", "OF-THE-PRESIDING-BISHOP-OF-THE-CHURCH-OF-JESUS-CHRIST-OF-LATTER-DAY-SAINTS" },
	{ 0x842914, "EMPORIA TELECOM Produktions- und VertriebsgesmbH & Co KG", "EMPORIA-TELECOM-PRODUKTIONS--UND-VERTRIEBSGESMBH" },
	{ 0x842999, "Apple", "APPLE" },
	{ 0x842B2B, "Dell Inc.", "DELL" },
	{ 0x842B50, "Huria Co.,Ltd.", "HURIA" },
	{ 0x842BBC, "Modelleisenbahn GmbH", "MODELLEISENBAHN" },
	{ 0x842F75, "Innokas Group", "INNOKAS-GROUP" },
	{ 0x8430E5, "SkyHawke Technologies, LLC", "SKYHAWKE-TECHNOLOGIES-LLC" },
	{ 0x8432EA, "ANHUI WANZTEN P&T CO., LTD", "ANHUI-WANZTEN-P-T" },
	{ 0x843497, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x843611, "hyungseul publishing networks", "HYUNGSEUL-PUBLISHING-NETWORKS" },
	{ 0x843835, "Apple", "APPLE" },
	{ 0x843838, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x843A4B, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x843F4E, "Tri-Tech Manufacturing, Inc.", "TRI-TECH-MANUFACTURING" },
	{ 0x844464, "ServerU Inc", "SERVERU" },
	{ 0x844823, "WOXTER TECHNOLOGY Co. Ltd", "WOXTER" },
	{ 0x844915, "vArmour Networks, Inc.", "VARMOUR-NETWORKS" },
	{ 0x844BB7, "Beijing Sankuai Online Technology Co.,Ltd", "BEIJING-SANKUAI-ONLINE" },
	{ 0x844BF5, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x844F03, "Ablelink Electronics Ltd", "ABLELINK-ELECTRONICS" },
	{ 0x845181, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x8455A5, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0x84569C, "Coho Data, Inc.,", "COHO-DATA" },
	{ 0x845787, "DVR C&C Co., Ltd.", "DVR-C-C" },
	{ 0x845C93, "Chabrier Services", "CHABRIER-SERVICES" },
	{ 0x845DD7, "Shenzhen Netcom Electronics Co.,Ltd", "SHENZHEN-NETCOM-ELECTRONICS" },
	{ 0x8461A0, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x846223, "Shenzhen Coship Electronics Co., Ltd.", "SHENZHEN-COSHIP-ELECTRONICS" },
	{ 0x8462A6, "EuroCB (Phils), Inc.", "EUROCB-PHILS" },
	{ 0x8463D6, "Microsoft Corporation", "MICROSOFT" },
	{ 0x846AED, "Wireless Tsukamoto.,co.LTD", "WIRELESS-TSUKAMOTO" },
	{ 0x846EB1, "Park Assist LLC", "PARK-ASSIST-LLC" },
	{ 0x847207, "I&C Technology", "I-C" },
	{ 0x847303, "Letv Mobile and Intelligent Information Technology (Beijing) Corporation Ltd.", "LETV-MOBILE-AND-INTELLIGENT-INFORMATION-BEIJING" },
	{ 0x84742A, "zte corporation", "ZTE" },
	{ 0x847616, "Addat S.r.o.", "ADDAT-S-R-O" },
	{ 0x84788B, "Apple", "APPLE" },
	{ 0x8478AC, "Cisco", "CISCO" },
	{ 0x847A88, "HTC Corporation", "HTC" },
	{ 0x847E40, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x84802D, "Cisco", "CISCO" },
	{ 0x8482F4, "Beijing Huasun Unicreate Technology Co., Ltd", "BEIJING-HUASUN-UNICREATE" },
	{ 0x848336, "Newrun", "NEWRUN" },
	{ 0x848371, "Avaya, Inc", "AVAYA" },
	{ 0x848433, "Paradox Engineering SA", "PARADOX-ENGINEERING" },
	{ 0x848506, "Apple", "APPLE" },
	{ 0x84850A, "Hella Sonnen- und Wetterschutztechnik GmbH", "HELLA-SONNEN--UND-WETTERSCHUTZTECHNIK" },
	{ 0x8486F3, "Greenvity Communications", "GREENVITY-COMMUNICATION" },
	{ 0x848D84, "Rajant Corporation", "RAJANT" },
	{ 0x848DC7, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0x848E0C, "Apple", "APPLE" },
	{ 0x848E96, "Embertec Pty Ltd", "EMBERTEC-PTY" },
	{ 0x848EDF, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x848F69, "Dell Inc.", "DELL" },
	{ 0x849000, "Arnold & Richter Cine Technik", "ARNOLD-RICHTER-CINE-TECHNIK" },
	{ 0x84930C, "InCoax Networks Europe AB", "INCOAX-NETWORKS-EUROPE-AB" },
	{ 0x84948C, "Hitron Technologies. Inc", "HITRON-TECHNOLOGIES" },
	{ 0x849681, "Cathay Communication Co.,Ltd", "CATHAY-COMMUNICATION" },
	{ 0x8496D8, "Pace plc", "PACE-PLC" },
	{ 0x8497B8, "Memjet Inc.", "MEMJET" },
	{ 0x849CA6, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x849DC5, "Centera Photonics Inc.", "CENTERA-PHOTONICS" },
	{ 0x84A466, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x84A6C8, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x84A783, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x84A8E4, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0x84A991, "Cyber Trans Japan Co.,Ltd.", "CYBER-TRANS-JAPAN" },
	{ 0x84ACA4, "Beijing Novel Super Digital TV Technology Co., Ltd", "BEIJING-NOVEL-SUPER-DIGITAL-TV" },
	{ 0x84AF1F, "Beat System Service Co,. Ltd.", "BEAT-SYSTEM-SERVICE" },
	{ 0x84B153, "Apple", "APPLE" },
	{ 0x84B59C, "Juniper networks", "JUNIPER-NETWORKS" },
	{ 0x84B802, "Cisco", "CISCO" },
	{ 0x84C2E4, "Jiangsu Qinheng Co., Ltd.", "JIANGSU-QINHENG" },
	{ 0x84C3E8, "Vaillant GmbH", "VAILLANT" },
	{ 0x84C727, "Gnodal Ltd", "GNODAL" },
	{ 0x84C7A9, "C3PO S.A.", "C3PO" },
	{ 0x84C8B1, "Incognito Software Inc.", "INCOGNITO-SOFTWARE" },
	{ 0x84C9B2, "D-Link International", "D-LINK" },
	{ 0x84CFBF, "Fairphone", "FAIRPHONE" },
	{ 0x84D32A, "IEEE 1905.1", "IEEE-1905-1" },
	{ 0x84D9C8, "Unipattern Co.,", "UNIPATTERN" },
	{ 0x84DB2F, "Sierra Wireless Inc", "SIERRA-WIRELESS" },
	{ 0x84DBAC, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x84DD20, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x84DDB7, "Cilag GmbH International", "CILAG" },
	{ 0x84DE3D, "Crystal Vision Ltd", "CRYSTAL-VISION" },
	{ 0x84DF0C, "NET2GRID BV", "NET2GRID" },
	{ 0x84DF19, "Chuango Security Technology Corporation", "CHUANGO-SECURITY" },
	{ 0x84E058, "Pace plc", "PACE-PLC" },
	{ 0x84E4D9, "Shenzhen NEED technology Ltd.", "SHENZHEN-NEED" },
	{ 0x84E629, "Bluwan SA", "BLUWAN" },
	{ 0x84E714, "Liang Herng Enterprise,Co.Ltd.", "LIANG-HERNG" },
	{ 0x84EA99, "Vieworks", "VIEWORKS" },
	{ 0x84EB18, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x84ED33, "BBMC Co.,Ltd", "BBMC" },
	{ 0x84F129, "Metrascale Inc.", "METRASCALE" },
	{ 0x84F493, "OMS spol. s.r.o.", "OMS-SPOL-S-R-O" },
	{ 0x84F64C, "Cross Point BV", "CROSS-POINT" },
	{ 0x84FCFE, "Apple", "APPLE" },
	{ 0x84FE9E, "RTC Industries, Inc.", "RTC-INDUSTRIES" },
	{ 0x880355, "Arcadyan Technology Corp.", "ARCADYAN" },
	{ 0x880905, "MTMCommunications", "MTMCOMMUNICATIONS" },
	{ 0x880F10, "Huami Information Technology Co.,Ltd.", "HUAMI-INFORMATION" },
	{ 0x880FB6, "Jabil Circuits India Pvt Ltd,-EHTP unit", "JABIL-CIRCUITS-INDIA-PVT--EHTP-UNIT" },
	{ 0x881036, "Panodic(ShenZhen) Electronics Limted", "PANODIC-SHENZHEN-ELECTRONICS-LIMTED" },
	{ 0x88124E, "Qualcomm Atheros", "QUALCOMM-ATHEROS" },
	{ 0x88142B, "Protonic Holland", "PROTONIC-HOLLAND" },
	{ 0x881544, "Meraki, Inc.", "MERAKI" },
	{ 0x8818AE, "Tamron Co., Ltd", "TAMRON" },
	{ 0x881DFC, "Cisco", "CISCO" },
	{ 0x881FA1, "Apple", "APPLE" },
	{ 0x882012, "LMI Technologies", "LMI-TECHNOLOGIES" },
	{ 0x8821E3, "Nebusens, S.L.", "NEBUSENS-S-L" },
	{ 0x882364, "Watchnet DVR Inc", "WATCHNET-DVR" },
	{ 0x8823FE, "TTTech Computertechnik AG", "TTTECH-COMPUTERTECHNIK" },
	{ 0x88252C, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x882593, "TP-LINK TECHNOLOGIES CO.,LTD", "TP-LINK-TECHNOLOGIES" },
	{ 0x882950, "Dalian Netmoon Tech Develop Co.,Ltd", "DALIAN-NETMOON-TECH-DEVELOP" },
	{ 0x882E5A, "storONE", "STORONE" },
	{ 0x88308A, "Murata Manufactuaring Co.,Ltd.", "MURATA-MANUFACTUARING" },
	{ 0x88329B, "Samsung Electro Mechanics co.,LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x883314, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x8833BE, "Ivenix, Inc.", "IVENIX" },
	{ 0x88354C, "Transics", "TRANSICS" },
	{ 0x883612, "SRC Computers, LLC", "SRC-COMPUTERS-LLC" },
	{ 0x883B8B, "Cheering Connection Co. Ltd.", "CHEERING-CONNECTION" },
	{ 0x8841C1, "ORBISAT DA AMAZONIA IND E AEROL SA", "ORBISAT-DA-AMAZONIA-IND-E-AEROL" },
	{ 0x8841FC, "AirTies Wireless Netowrks", "AIRTIES-WIRELESS-NETOWRKS" },
	{ 0x8843E1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x8844F6, "Nokia Corporation", "NOKIA" },
	{ 0x88462A, "Telechips Inc.", "TELECHIPS" },
	{ 0x884B39, "Siemens AG, Healthcare Sector", "SIEMENS-HEALTHCARE-SECTOR" },
	{ 0x8851FB, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x88532E, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x885395, "Apple", "APPLE" },
	{ 0x8853D4, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x88576D, "XTA Electronics Ltd", "XTA-ELECTRONICS" },
	{ 0x885A92, "Cisco", "CISCO" },
	{ 0x885BDD, "Aerohive Networks Inc.", "AEROHIVE-NETWORKS" },
	{ 0x885C47, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x88615A, "Siano Mobile Silicon Ltd.", "SIANO-MOBILE-SILICON" },
	{ 0x8863DF, "Apple", "APPLE" },
	{ 0x88685C, "Shenzhen ChuangDao & Perpetual Eternal Technology Co.,Ltd", "SHENZHEN-CHUANGDAO-PERPETUAL-ETERNAL" },
	{ 0x886B76, "CHINA HOPEFUL GROUP HOPEFUL ELECTRIC CO.,LTD", "CHINA-HOPEFUL-GROUP-HOPEFUL-ELECTRIC" },
	{ 0x887033, "Hangzhou Silan Microelectronic Inc", "HANGZHOU-SILAN-MICROELECTRONIC" },
	{ 0x88708C, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x8870EF, "SC Professional Trading Co., Ltd.", "SC-PROFESSIONAL-TRADING" },
	{ 0x887398, "K2E Tekpoint", "K2E-TEKPOINT" },
	{ 0x887556, "Cisco", "CISCO" },
	{ 0x88789C, "Game Technologies SA", "GAME-TECHNOLOGIES" },
	{ 0x888603, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x8886A0, "Simton Technologies, Ltd.", "SIMTON-TECHNOLOGIES" },
	{ 0x888717, "CANON INC.", "CANON" },
	{ 0x8887DD, "DarbeeVision Inc.", "DARBEEVISION" },
	{ 0x888914, "All Components Incorporated", "ALL-COMPONENTS-INCORPORATED" },
	{ 0x888964, "GSI Electronics Inc.", "GSI-ELECTRONICS" },
	{ 0x888B5D, "Storage Appliance Corporation", "STORAGE-APPLIANCE" },
	{ 0x888C19, "Brady Corp Asia Pacific Ltd", "BRADY-ASIA-PACIFIC" },
	{ 0x88908D, "Cisco", "CISCO" },
	{ 0x889166, "Viewcooper Corp.", "VIEWCOOPER" },
	{ 0x8891DD, "Racktivity", "RACKTIVITY" },
	{ 0x889471, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x8894F9, "Gemicom Technology, Inc.", "GEMICOM" },
	{ 0x8895B9, "Unified Packet Systems Crop", "UNIFIED-PACKET-CROP" },
	{ 0x889676, "TTC MARCONI s.r.o.", "TTC-MARCONI-S-R-O" },
	{ 0x8897DF, "Entrypass Corporation Sdn. Bhd.", "ENTRYPASS-SDN-BHD" },
	{ 0x889821, "TERAON", "TERAON" },
	{ 0x889B39, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x889CA6, "BTB Korea INC", "BTB-KOREA" },
	{ 0x889FFA, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x88A25E, "juniper networks", "JUNIPER-NETWORKS" },
	{ 0x88A3CC, "Amatis Controls", "AMATIS-CONTROLS" },
	{ 0x88A5BD, "QPCOM INC.", "QPCOM" },
	{ 0x88A73C, "Ragentek Technology Group", "RAGENTEK-GROUP" },
	{ 0x88ACC1, "Generiton Co., Ltd.", "GENERITON" },
	{ 0x88AE1D, "COMPAL INFORMATION(KUNSHAN)CO.,LTD", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0x88B168, "Delta Control GmbH", "DELTA-CONTROL" },
	{ 0x88B1E1, "AirTight Networks, Inc.", "AIRTIGHT-NETWORKS" },
	{ 0x88B627, "Gembird Europe BV", "GEMBIRD-EUROPE" },
	{ 0x88BA7F, "Qfiednet Co., Ltd.", "QFIEDNET" },
	{ 0x88BFD5, "Simple Audio Ltd", "SIMPLE-AUDIO" },
	{ 0x88C36E, "Beijing Ereneben lnformation Technology Limited", "BEIJING-ERENEBEN-LNFORMATION" },
	{ 0x88C626, "Logitech - Ultimate Ears", "LOGITECH---ULTIMATE-EARS" },
	{ 0x88C663, "Apple", "APPLE" },
	{ 0x88C9D0, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x88CB87, "Apple", "APPLE" },
	{ 0x88CEFA, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x88D37B, "FirmTek, LLC", "FIRMTEK-LLC" },
	{ 0x88D7BC, "DEP Company", "DEP-COMPANY" },
	{ 0x88D962, "Canopus Systems US LLC", "CANOPUS-US-LLC" },
	{ 0x88DC96, "SENAO Networks, Inc.", "SENAO-NETWORKS" },
	{ 0x88DD79, "Voltaire", "VOLTAIRE" },
	{ 0x88E0A0, "Shenzhen VisionSTOR Technologies Co., Ltd", "SHENZHEN-VISIONSTOR-TECHNOLOGIES" },
	{ 0x88E0F3, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0x88E161, "Art Beijing Science and Technology Development Co., Ltd.", "ART-BEIJING-SCIENCE-AND-DEVELOPMENT" },
	{ 0x88E3AB, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x88E603, "Avotek corporation", "AVOTEK" },
	{ 0x88E712, "Whirlpool Corporation", "WHIRLPOOL" },
	{ 0x88E7A6, "iKnowledge Integration Corp.", "IKNOWLEDGE-INTEGRATION" },
	{ 0x88E8F8, "YONG TAI ELECTRONIC (DONGGUAN) LTD.", "YONG-TAI-DONGGUAN" },
	{ 0x88E917, "Tamaggo", "TAMAGGO" },
	{ 0x88ED1C, "Cudo Communication Co., Ltd.", "CUDO-COMMUNICATION" },
	{ 0x88F031, "Cisco", "CISCO" },
	{ 0x88F077, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x88F488, "cellon communications technology(shenzhen)Co.,Ltd.", "CELLON-COMMUNICATION-SHENZHEN" },
	{ 0x88F490, "Jetmobile Pte Ltd", "JETMOBILE-PTE" },
	{ 0x88F7C7, "Technicolor USA Inc.", "TECHNICOLOR-USA" },
	{ 0x88FD15, "LINEEYE CO., LTD", "LINEEYE" },
	{ 0x88FED6, "ShangHai WangYong Software Co., Ltd.", "SHANGHAI-WANGYONG-SOFTWARE" },
	{ 0x8C006D, "Apple", "APPLE" },
	{ 0x8C04FF, "Technicolor USA Inc.", "TECHNICOLOR-USA" },
	{ 0x8C0551, "Koubachi AG", "KOUBACHI" },
	{ 0x8C078C, "FLOW DATA INC", "FLOW-DATA" },
	{ 0x8C088B, "Remote Solution", "REMOTE-SOLUTION" },
	{ 0x8C09F4, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x8C0C90, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x8C0CA3, "Amper", "AMPER" },
	{ 0x8C0EE3, "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD.", "GUANGDONG-OPPO-MOBILE-TELECOMMUNICATIONS" },
	{ 0x8C11CB, "ABUS Security-Center GmbH & Co. KG", "ABUS-SECURITY-CENTER" },
	{ 0x8C18D9, "Shenzhen RF Technology Co., Ltd", "SHENZHEN-RF" },
	{ 0x8C1F94, "RF Surgical System Inc.", "RF-SURGICAL-SYSTEM" },
	{ 0x8C210A, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x8C271D, "QuantHouse", "QUANTHOUSE" },
	{ 0x8C278A, "Vocollect Inc", "VOCOLLECT" },
	{ 0x8C2937, "Apple", "APPLE" },
	{ 0x8C2DAA, "Apple", "APPLE" },
	{ 0x8C2F39, "IBA Dosimetry GmbH", "IBA-DOSIMETRY" },
	{ 0x8C3330, "EmFirst Co., Ltd.", "EMFIRST" },
	{ 0x8C3357, "HiteVision Digital Media Technology Co.,Ltd.", "HITEVISION-DIGITAL-MEDIA" },
	{ 0x8C34FD, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x8C3AE3, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x8C3C07, "Skiva Technologies, Inc.", "SKIVA-TECHNOLOGIES" },
	{ 0x8C3C4A, "NAKAYO TELECOMMUNICATIONS,INC.", "NAKAYO-TELECOMMUNICATIONS" },
	{ 0x8C41F2, "RDA Technologies Ltd.", "RDA-TECHNOLOGIES" },
	{ 0x8C4435, "Shanghai BroadMobi Communication Technology Co., Ltd.", "SHANGHAI-BROADMOBI-COMMUNICATION" },
	{ 0x8C4AEE, "GIGA TMS INC", "GIGA-TMS" },
	{ 0x8C4B59, "3D Imaging & Simulations Corp", "3D-IMAGING-SIMULATIONS" },
	{ 0x8C4CDC, "PLANEX COMMUNICATIONS INC.", "PLANEX-COMMUNICATION" },
	{ 0x8C4DB9, "Unmonday Ltd", "UNMONDAY" },
	{ 0x8C4DEA, "Cerio Corporation", "CERIO" },
	{ 0x8C5105, "Shenzhen ireadygo Information Technology CO.,LTD.", "SHENZHEN-IREADYGO-INFORMATION" },
	{ 0x8C53F7, "A&D ENGINEERING CO., LTD.", "A-D-ENGINEERING" },
	{ 0x8C541D, "LGE", "LGE" },
	{ 0x8C569D, "Imaging Solutions Group", "IMAGING-SOLUTIONS-GROUP" },
	{ 0x8C56C5, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x8C57FD, "LVX Western", "LVX-WESTERN" },
	{ 0x8C5877, "Apple", "APPLE" },
	{ 0x8C598B, "C Technologies AB", "C-TECHNOLOGIES-AB" },
	{ 0x8C5AF0, "Exeltech Solar Products", "EXELTECH-SOLAR-PRODUCTS" },
	{ 0x8C5CA1, "d-broad,INC", "D-BROAD" },
	{ 0x8C5D60, "UCI Corporation Co.,Ltd.", "UCI" },
	{ 0x8C5FDF, "Beijing Railway Signal Factory", "BEIJING-RAILWAY-SIGNAL-FACTORY" },
	{ 0x8C604F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x8C640B, "Beyond Devices d.o.o.", "BEYOND-DEVICES-D-O-O" },
	{ 0x8C6422, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x8C6878, "Nortek-AS", "NORTEK-AS" },
	{ 0x8C6AE4, "Viogem Limited", "VIOGEM" },
	{ 0x8C705A, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x8C71F8, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x8C736E, "Fujitsu Limited", "FUJITSU" },
	{ 0x8C76C1, "Goden Tech Limited", "GODEN-TECH" },
	{ 0x8C7712, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x8C7716, "LONGCHEER TELECOMMUNICATION LIMITED", "LONGCHEER-TELECOMMUNICATION" },
	{ 0x8C7967, "zte corporation", "ZTE" },
	{ 0x8C7B9D, "Apple", "APPLE" },
	{ 0x8C7C92, "Apple", "APPLE" },
	{ 0x8C7CB5, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x8C7CFF, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0x8C7EB3, "Lytro, Inc.", "LYTRO" },
	{ 0x8C7F3B, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x8C82A8, "Insigma Technology Co.,Ltd", "INSIGMA" },
	{ 0x8C8401, "PRIVATE", "PRIVATE" },
	{ 0x8C873B, "Leica Camera AG", "LEICA-CAMERA" },
	{ 0x8C89A5, "Micro-Star INT'L CO., LTD", "MICRO-STAR" },
	{ 0x8C8A6E, "ESTUN AUTOMATION TECHNOLOY CO., LTD", "ESTUN-AUTOMATION-TECHNOLOY" },
	{ 0x8C8B83, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x8C8E76, "taskit GmbH", "TASKIT" },
	{ 0x8C90D3, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x8C9109, "Toyoshima Electric Technoeogy(Suzhou) Co.,Ltd.", "TOYOSHIMA-ELECTRIC-TECHNOEOGY-SUZHOU" },
	{ 0x8C9236, "Aus.Linx Technology Co., Ltd.", "AUS-LINX" },
	{ 0x8C94CF, "Encell Technology, Inc.", "ENCELL" },
	{ 0x8CA048, "Beijing NeTopChip Technology Co.,LTD", "BEIJING-NETOPCHIP" },
	{ 0x8CA982, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x8CAE4C, "Plugable Technologies", "PLUGABLE-TECHNOLOGIES" },
	{ 0x8CAE89, "Y-cam Solutions Ltd", "Y-CAM-SOLUTIONS" },
	{ 0x8CB094, "Airtech I&C Co., Ltd", "AIRTECH-I-C" },
	{ 0x8CB64F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x8CB7F7, "Shenzhen UniStrong Science & Technology Co., Ltd", "SHENZHEN-UNISTRONG-SCIENCE" },
	{ 0x8CB82C, "IPitomy Communications", "IPITOMY-COMMUNICATION" },
	{ 0x8CB864, "AcSiP Technology Corp.", "ACSIP" },
	{ 0x8CBEBE, "Xiaomi Technology Co.,Ltd", "XIAOMI" },
	{ 0x8CBF9D, "Shanghai Xinyou Information Technology Ltd. Co.", "SHANGHAI-XINYOU-INFORMATION" },
	{ 0x8CBFA6, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x8CC121, "Panasonic Corporation AVC Networks Company", "PANASONIC-AVC-NETWORKS-COMPANY" },
	{ 0x8CC5E1, "ShenZhen Konka Telecommunication Technology Co.,Ltd", "SHENZHEN-KONKA-TELECOMMUNICATION" },
	{ 0x8CC7AA, "Radinet Communications Inc.", "RADINET-COMMUNICATION" },
	{ 0x8CC7D0, "zhejiang ebang communication co.,ltd", "ZHEJIANG-EBANG-COMMUNICATION" },
	{ 0x8CC8CD, "Samsung Electronics Co., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x8CCDA2, "ACTP, Inc.", "ACTP" },
	{ 0x8CCDE8, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x8CCF5C, "BEFEGA GmbH", "BEFEGA" },
	{ 0x8CD17B, "CG Mobile", "CG-MOBILE" },
	{ 0x8CD3A2, "VisSim AS", "VISSIM-AS" },
	{ 0x8CD628, "Ikor Metering", "IKOR-METERING" },
	{ 0x8CDB25, "ESG Solutions", "ESG-SOLUTIONS" },
	{ 0x8CDCD4, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x8CDD8D, "Wifly-City System Inc.", "WIFLY-CITY-SYSTEM" },
	{ 0x8CDE52, "ISSC Technologies Corp.", "ISSC-TECHNOLOGIES" },
	{ 0x8CDE99, "Comlab Inc.", "COMLAB" },
	{ 0x8CDF9D, "NEC Corporation", "NEC" },
	{ 0x8CE081, "zte corporation", "ZTE" },
	{ 0x8CE748, "PRIVATE", "PRIVATE" },
	{ 0x8CE78C, "DK Networks", "DK-NETWORKS" },
	{ 0x8CE7B3, "Sonardyne International Ltd", "SONARDYNE" },
	{ 0x8CEEC6, "Precepscion Pty. Ltd.", "PRECEPSCION-PTY" },
	{ 0x8CF228, "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.", "SHENZHEN-MERCURY-COMMUNICATION-TECHNOLOGIES" },
	{ 0x8CF813, "ORANGE POLSKA", "ORANGE-POLSKA" },
	{ 0x8CF945, "Power Automation pte Ltd", "POWER-AUTOMATION-PTE" },
	{ 0x8CF9C9, "MESADA Technology Co.,Ltd.", "MESADA" },
	{ 0x8CFABA, "Apple", "APPLE" },
	{ 0x8CFDF0, "QUALCOMM Incorporated", "QUALCOMM-INCORPORATED" },
	{ 0x90004E, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x9000DB, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x90013B, "SAGEMCOM", "SAGEMCOM" },
	{ 0x90028A, "Shenzhen Shidean Legrand Electronic Products Co.,Ltd", "SHENZHEN-SHIDEAN-LEGRAND-PRODUCTS" },
	{ 0x9002A9, "ZHEJIANG DAHUA TECHNOLOGY CO.,LTD", "ZHEJIANG-DAHUA" },
	{ 0x9003B7, "PARROT", "PARROT" },
	{ 0x900917, "Far-sighted mobile", "FAR-SIGHTED-MOBILE" },
	{ 0x900A39, "Wiio, Inc.", "WIIO" },
	{ 0x900A3A, "PSG Plastic Service GmbH", "PSG-PLASTIC-SERVICE" },
	{ 0x900CB4, "Alinket Electronic Technology Co., Ltd", "ALINKET" },
	{ 0x900D66, "Digimore Electronics Co., Ltd", "DIGIMORE-ELECTRONICS" },
	{ 0x900DCB, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x90179B, "Nanomegas", "NANOMEGAS" },
	{ 0x9017AC, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x90185E, "Apex Tool Group GmbH & Co OHG", "APEX-TOOL-GROUP-OHG" },
	{ 0x90187C, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x9018AE, "Shanghai Meridian Technologies, Co. Ltd.", "SHANGHAI-MERIDIAN-TECHNOLOGIES" },
	{ 0x901900, "SCS SA", "SCS" },
	{ 0x901ACA, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x901B0E, "Fujitsu Technology Solutions GmbH", "FUJITSU-SOLUTIONS" },
	{ 0x901D27, "zte corporation", "ZTE" },
	{ 0x901EDD, "GREAT COMPUTER CORPORATION", "GREAT" },
	{ 0x90203A, "BYD Precision Manufacture Co.,Ltd", "BYD-PRECISION-MANUFACTURE" },
	{ 0x902083, "General Engine Management Systems Ltd.", "GENERAL-ENGINE-MANAGEMENT" },
	{ 0x902155, "HTC Corporation", "HTC" },
	{ 0x902181, "Shanghai Huaqin Telecom Technology Co.,Ltd", "SHANGHAI-HUAQIN-TELECOM" },
	{ 0x9027E4, "Apple", "APPLE" },
	{ 0x902B34, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x902CC7, "C-MAX Asia Limited", "C-MAX-ASIA" },
	{ 0x902E1C, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x902E87, "LabJack", "LABJACK" },
	{ 0x9031CD, "Onyx Healthcare Inc.", "ONYX-HEALTHCARE" },
	{ 0x90342B, "Gatekeeper Systems, Inc.", "GATEKEEPER" },
	{ 0x9034FC, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x90356E, "Vodafone Omnitel N.V.", "VODAFONE-OMNITEL-N-V" },
	{ 0x9038DF, "Changzhou Tiannengbo System Co. Ltd.", "CHANGZHOU-TIANNENGBO-SYSTEM" },
	{ 0x903AA0, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x903C92, "Apple", "APPLE" },
	{ 0x903CAE, "Yunnan KSEC Digital Technology Co.,Ltd.", "YUNNAN-KSEC-DIGITAL" },
	{ 0x903D5A, "Shenzhen Wision Technology Holding Limited", "SHENZHEN-WISION-HOLDING" },
	{ 0x903D6B, "Zicon Technology Corp.", "ZICON" },
	{ 0x903EAB, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x904506, "Tokyo Boeki Medisys Inc.", "TOKYO-BOEKI-MEDISYS" },
	{ 0x9046B7, "Vadaro Pte Ltd", "VADARO-PTE" },
	{ 0x904716, "RORZE CORPORATION", "RORZE" },
	{ 0x90489A, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x9049FA, "Intel Corporation", "INTEL" },
	{ 0x904CE5, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x904E2B, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x90507B, "Advanced PANMOBIL Systems GmbH & Co. KG", "ADVANCED-PANMOBIL" },
	{ 0x90513F, "Elettronica Santerno SpA", "ELETTRONICA-SANTERNO-SPA" },
	{ 0x905446, "TES ELECTRONIC SOLUTIONS", "TES-SOLUTIONS" },
	{ 0x9055AE, "Ericsson, EAB/RWI/K", "ERICSSON-EAB/RWI/K" },
	{ 0x905682, "Lenbrook Industries Limited", "LENBROOK-INDUSTRIES" },
	{ 0x905692, "Autotalks Ltd.", "AUTOTALKS" },
	{ 0x9059AF, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x905F2E, "TCT Mobile Limited", "TCT-MOBILE" },
	{ 0x905F8D, "modas GmbH", "MODAS" },
	{ 0x90610C, "Fida International (S) Pte Ltd", "FIDA-S-PTE" },
	{ 0x906717, "Alphion India Private Limited", "ALPHION-INDIA-PRIVATE" },
	{ 0x90671C, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x9067B5, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x9067F3, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x9068C3, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0x906DC8, "DLG Automação Industrial Ltda", "DLG-AUTOMAçãO-INDUSTRIAL-LTDA" },
	{ 0x906EBB, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x907025, "Garea Microsys Co.,Ltd.", "GAREA-MICROSYS" },
	{ 0x907240, "Apple", "APPLE" },
	{ 0x907990, "Benchmark Electronics Romania SRL", "BENCHMARK-ELECTRONICS-ROMANIA-SRL" },
	{ 0x907A0A, "Gebr. Bode GmbH & Co KG", "GEBR-BODE" },
	{ 0x907A28, "Beijing Morncloud Information And Technology Co. Ltd.", "BEIJING-MORNCLOUD-INFORMATION-AND" },
	{ 0x907AF1, "SNUPI Technologies", "SNUPI-TECHNOLOGIES" },
	{ 0x907EBA, "UTEK TECHNOLOGY (SHENZHEN) CO.,LTD", "UTEK-SHENZHEN" },
	{ 0x907F61, "Chicony Electronics Co., Ltd.", "CHICONY-ELECTRONICS" },
	{ 0x908260, "IEEE 1904.1 Working Group", "IEEE-1904-1-WORKING-GROUP" },
	{ 0x90837A, "General Electric Water & Process Technologies", "GENERAL-ELECTRIC-WATER-PROCESS-TECHNOLOGIES" },
	{ 0x90840D, "Apple", "APPLE" },
	{ 0x9088A2, "IONICS TECHNOLOGY ME LTDA", "IONICS-ME-LTDA" },
	{ 0x908C09, "Total Phase", "TOTAL-PHASE" },
	{ 0x908C44, "H.K ZONGMU TECHNOLOGY CO., LTD.", "H-K-ZONGMU" },
	{ 0x908C63, "GZ Weedong Networks Technology Co. , Ltd", "GZ-WEEDONG-NETWORKS" },
	{ 0x908D1D, "GH Technologies", "GH-TECHNOLOGIES" },
	{ 0x908D6C, "Apple", "APPLE" },
	{ 0x908FCF, "UNO System Co., Ltd", "UNO-SYSTEM" },
	{ 0x90903C, "TRISON TECHNOLOGY CORPORATION", "TRISON" },
	{ 0x909060, "RSI VIDEO TECHNOLOGIES", "RSI-VIDEO-TECHNOLOGIES" },
	{ 0x9092B4, "Diehl BGT Defence GmbH & Co. KG", "DIEHL-BGT-DEFENCE" },
	{ 0x9094E4, "D-Link International", "D-LINK" },
	{ 0x909864, "Impex-Sat GmbH&amp;Co KG", "IMPEX-SAT-AMP" },
	{ 0x909916, "ELVEES NeoTek OJSC", "ELVEES-NEOTEK-OJSC" },
	{ 0x909DE0, "Newland Design + Assoc. Inc.", "NEWLAND-DESIGN-+-ASSOC" },
	{ 0x909F33, "EFM Networks", "EFM-NETWORKS" },
	{ 0x909F43, "Accutron Instruments Inc.", "ACCUTRON-INSTRUMENTS" },
	{ 0x90A210, "United Telecoms Ltd", "UNITED-TELECOMS" },
	{ 0x90A2DA, "GHEO SA", "GHEO" },
	{ 0x90A4DE, "Wistron Neweb Corp.", "WISTRON-NEWEB" },
	{ 0x90A783, "JSW PACIFIC CORPORATION", "JSW-PACIFIC" },
	{ 0x90A7C1, "Pakedge Device and Software Inc.", "PAKEDGE-DEVICE-AND-SOFTWARE" },
	{ 0x90AC3F, "BrightSign LLC", "BRIGHTSIGN-LLC" },
	{ 0x90AE1B, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x90B11C, "Dell Inc.", "DELL" },
	{ 0x90B134, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x90B21F, "Apple", "APPLE" },
	{ 0x90B686, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x90B8D0, "Joyent, Inc.", "JOYENT" },
	{ 0x90B931, "Apple, Inc", "APPLE" },
	{ 0x90B97D, "Johnson Outdoors Marine Electronics d/b/a Minnkota", "JOHNSON-OUTDOORS-MARINE-ELECTRONICS-D/B/A-MINNKOTA" },
	{ 0x90C115, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0x90C35F, "Nanjing Jiahao Technology Co., Ltd.", "NANJING-JIAHAO" },
	{ 0x90C682, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x90C792, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x90CC24, "Synaptics, Inc", "SYNAPTICS" },
	{ 0x90CF15, "Nokia Corporation", "NOKIA" },
	{ 0x90CF6F, "Dlogixs Co Ltd", "DLOGIXS" },
	{ 0x90CF7D, "Qingdao Hisense Electric Co.,Ltd.", "QINGDAO-HISENSE-ELECTRIC" },
	{ 0x90D11B, "Palomar Medical Technologies", "PALOMAR-MEDICAL-TECHNOLOGIES" },
	{ 0x90D74F, "Bookeen", "BOOKEEN" },
	{ 0x90D7EB, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x90D852, "Comtec Co., Ltd.", "COMTEC" },
	{ 0x90D92C, "HUG-WITSCHI AG", "HUG-WITSCHI" },
	{ 0x90DA4E, "AVANU", "AVANU" },
	{ 0x90DA6A, "FOCUS H&S Co., Ltd.", "FOCUS-H-S" },
	{ 0x90DB46, "E-LEAD ELECTRONIC CO., LTD", "E-LEAD" },
	{ 0x90DFB7, "s.m.s smart microwave sensors GmbH", "S-M-S-SMART-MICROWAVE-SENSORS" },
	{ 0x90E0F0, "IEEE 1722a Working Group", "IEEE-1722A-WORKING-GROUP" },
	{ 0x90E2BA, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x90E6BA, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0x90E7C4, "HTC Corporation", "HTC" },
	{ 0x90EA60, "SPI Lasers Ltd", "SPI-LASERS" },
	{ 0x90EF68, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0x90F1AA, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x90F1B0, "Hangzhou Anheng Info&Tech CO.,LTD", "HANGZHOU-ANHENG-INFO-TECH" },
	{ 0x90F278, "Radius Gateway", "RADIUS-GATEWAY" },
	{ 0x90F3B7, "Kirisun Communications Co., Ltd.", "KIRISUN-COMMUNICATION" },
	{ 0x90F4C1, "Rand McNally", "RAND-MCNALLY" },
	{ 0x90F652, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x90F72F, "Phillips Machine & Welding Co., Inc.", "PHILLIPS-MACHINE-WELDING" },
	{ 0x90FB5B, "Avaya, Inc", "AVAYA" },
	{ 0x90FBA6, "Hon Hai Precision Ind.Co.Ltd", "HON-HAI-PRECISION-IND" },
	{ 0x90FD61, "Apple", "APPLE" },
	{ 0x90FF79, "Metro Ethernet Forum", "METRO-ETHERNET-FORUM" },
	{ 0x940070, "Nokia Corporation", "NOKIA" },
	{ 0x940149, "AutoHotBox", "AUTOHOTBOX" },
	{ 0x9401C2, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x94049C, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x9405B6, "Liling FullRiver Electronics & Technology Ltd", "LILING-FULLRIVER-ELECTRONICS" },
	{ 0x940B2D, "NetView Technologies(Shenzhen) Co., Ltd", "NETVIEW-TECHNOLOGIES-SHENZHEN" },
	{ 0x940BD5, "Himax Technologies, Inc", "HIMAX-TECHNOLOGIES" },
	{ 0x940C6D, "TP-LINK Technologies Co.,Ltd.", "TP-LINK-TECHNOLOGIES" },
	{ 0x94103E, "Belkin International Inc.", "BELKIN" },
	{ 0x9411DA, "ITF Fröschl GmbH", "ITF-FRöSCHL" },
	{ 0x941673, "Point Core SARL", "POINT-CORE-SARL" },
	{ 0x941D1C, "TLab West Systems AB", "TLAB-WEST-AB" },
	{ 0x942053, "Nokia Corporation", "NOKIA" },
	{ 0x942197, "Stalmart Technology Limited", "STALMART" },
	{ 0x94236E, "Shenzhen Junlan Electronic Ltd", "SHENZHEN-JUNLAN" },
	{ 0x942E17, "Schneider Electric Canada Inc", "SCHNEIDER-ELECTRIC-CANADA" },
	{ 0x942E63, "Finsécur", "FINSéCUR" },
	{ 0x94319B, "Alphatronics BV", "ALPHATRONICS" },
	{ 0x9433DD, "Taco Electronic Solutions, Inc.", "TACO-SOLUTIONS" },
	{ 0x94350A, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x9436E0, "Sichuan Bihong Broadcast &amp; Television New Technologies Co.,Ltd", "SICHUAN-BIHONG-BROADCAST-AMP-TELEVISION-NEW-TECHNOLOGIES" },
	{ 0x9439E5, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x943AF0, "Nokia Corporation", "NOKIA" },
	{ 0x943BB1, "KAONMEDIA", "KAONMEDIA" },
	{ 0x9440A2, "Anywave Communication Technologies, Inc.", "ANYWAVE-COMMUNICATION-TECHNOLOGIES" },
	{ 0x944444, "LG Innotek", "LG-INNOTEK" },
	{ 0x944452, "Belkin International Inc.", "BELKIN" },
	{ 0x944696, "BaudTec Corporation", "BAUDTEC" },
	{ 0x944A09, "BitWise Controls", "BITWISE-CONTROLS" },
	{ 0x944A0C, "Sercomm Corporation", "SERCOMM" },
	{ 0x945047, "Rechnerbetriebsgruppe", "RECHNERBETRIEBSGRUPPE" },
	{ 0x945103, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x9451BF, "Hyundai ESG", "HYUNDAI-ESG" },
	{ 0x945493, "Rigado, LLC", "RIGADO-LLC" },
	{ 0x94592D, "EKE Building Technology Systems Ltd", "EKE-BUILDING" },
	{ 0x945B7E, "TRILOBIT LTDA.", "TRILOBIT-LTDA" },
	{ 0x946124, "Pason Systems", "PASON" },
	{ 0x946269, "Arris Group, Inc.", "ARRIS-GROUP" },
	{ 0x9463D1, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x9470D2, "WINFIRM TECHNOLOGY", "WINFIRM" },
	{ 0x9471AC, "TCT Mobile Limited", "TCT-MOBILE" },
	{ 0x94756E, "QinetiQ North America", "QINETIQ-NORTH-AMERICA" },
	{ 0x94772B, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x947C3E, "Polewall Norge AS", "POLEWALL-NORGE-AS" },
	{ 0x9481A4, "Azuray Technologies", "AZURAY-TECHNOLOGIES" },
	{ 0x94857A, "Evantage Industries Corp", "EVANTAGE-INDUSTRIES" },
	{ 0x9486D4, "Surveillance Pro Corporation", "SURVEILLANCE-PRO" },
	{ 0x94877C, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x948854, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x948B03, "EAGET Innovation and Technology Co., Ltd.", "EAGET-INNOVATION-AND" },
	{ 0x948D50, "Beamex Oy Ab", "BEAMEX-OY-AB" },
	{ 0x948E89, "INDUSTRIAS UNIDAS SA DE CV", "INDUSTRIAS-UNIDAS-DE-CV" },
	{ 0x948FEE, "Hughes Telematics, Inc.", "HUGHES-TELEMATICS" },
	{ 0x949426, "Apple", "APPLE" },
	{ 0x9498A2, "Shanghai LISTEN TECH.LTD", "SHANGHAI-LISTEN-TECH" },
	{ 0x949BFD, "Trans New Technology, Inc.", "TRANS-NEW" },
	{ 0x949C55, "Alta Data Technologies", "ALTA-DATA-TECHNOLOGIES" },
	{ 0x949F3F, "Optek Digital Technology company limited", "OPTEK-DIGITAL-COMPANY" },
	{ 0x949FB4, "ChengDu JiaFaAnTai Technology Co.,Ltd", "CHENGDU-JIAFAANTAI" },
	{ 0x94A1A2, "AMPAK Technology Inc.", "AMPAK" },
	{ 0x94A7B7, "zte corporation", "ZTE" },
	{ 0x94A7BC, "BodyMedia, Inc.", "BODYMEDIA" },
	{ 0x94AAB8, "Joview(Beijing) Technology Co. Ltd.", "JOVIEW-BEIJING" },
	{ 0x94ACCA, "trivum technologies GmbH", "TRIVUM-TECHNOLOGIES" },
	{ 0x94AE61, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0x94AEE3, "Belden Hirschmann Industries (Suzhou) Ltd.", "BELDEN-HIRSCHMANN-INDUSTRIES-SUZHOU" },
	{ 0x94B40F, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0x94B8C5, "RuggedCom Inc.", "RUGGEDCOM" },
	{ 0x94B9B4, "Aptos Technology", "APTOS" },
	{ 0x94BA31, "Visiontec da Amazônia Ltda.", "VISIONTEC-DA-AMAZôNIA-LTDA" },
	{ 0x94BA56, "Shenzhen Coship Electronics Co., Ltd.", "SHENZHEN-COSHIP-ELECTRONICS" },
	{ 0x94BF1E, "eflow Inc. / Smart Device Planning and Development Division", "EFLOW-/-SMART-DEVICE-PLANNING-AND-DEVELOPMENT-DIVISION" },
	{ 0x94BF95, "Shenzhen Coship Electronics Co., Ltd", "SHENZHEN-COSHIP-ELECTRONICS" },
	{ 0x94C014, "Sorter Sp. j. Konrad Grzeszczyk MichaA, Ziomek", "SORTER-SP-J-KONRAD-GRZESZCZYK-MICHAA-ZIOMEK" },
	{ 0x94C038, "Tallac Networks", "TALLAC-NETWORKS" },
	{ 0x94C150, "2Wire Inc", "2WIRE" },
	{ 0x94C3E4, "SCA Schucker Gmbh & Co KG", "SCA-SCHUCKER" },
	{ 0x94C4E9, "PowerLayer Microsystems HongKong Limited", "POWERLAYER-MICROSYSTEMS-HONGKONG" },
	{ 0x94C6EB, "NOVA electronics, Inc.", "NOVA-ELECTRONICS" },
	{ 0x94C7AF, "Raylios Technology", "RAYLIOS" },
	{ 0x94C962, "Teseq AG", "TESEQ" },
	{ 0x94CA0F, "Honeywell Analytics", "HONEYWELL-ANALYTICS" },
	{ 0x94CCB9, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x94CDAC, "Creowave Oy", "CREOWAVE-OY" },
	{ 0x94CE2C, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0x94CE31, "CTS Limited", "CTS" },
	{ 0x94D019, "Cydle Corp.", "CYDLE" },
	{ 0x94D417, "GPI KOREA INC.", "GPI-KOREA" },
	{ 0x94D60E, "shenzhen yunmao information technologies co., ltd", "SHENZHEN-YUNMAO-INFORMATION-TECHNOLOGIES" },
	{ 0x94D723, "Shanghai DareGlobal Technologies Co., Ltd", "SHANGHAI-DAREGLOBAL-TECHNOLOGIES" },
	{ 0x94D771, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x94D93C, "ENELPS", "ENELPS" },
	{ 0x94DB49, "SITCORP", "SITCORP" },
	{ 0x94DBC9, "Azurewave", "AZUREWAVE" },
	{ 0x94DD3F, "A+V Link Technologies, Corp.", "A+V-LINK-TECHNOLOGIES" },
	{ 0x94DE0E, "SmartOptics AS", "SMARTOPTICS-AS" },
	{ 0x94DE80, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0x94DF4E, "Wistron InfoComm(Kunshan)Co.,Ltd.", "WISTRON-INFOCOMM-KUNSHAN" },
	{ 0x94DF58, "IJ Electron CO.,Ltd.", "IJ-ELECTRON" },
	{ 0x94E0D0, "HealthStream Taiwan Inc.", "HEALTHSTREAM-TAIWAN" },
	{ 0x94E226, "D. ORtiz Consulting, LLC", "D-ORTIZ-CONSULTING-LLC" },
	{ 0x94E2FD, "Boge Kompressoren Otto Boge GmbH & Co. KG", "BOGE-KOMPRESSOREN-OTTO-BOGE" },
	{ 0x94E711, "Xirka Dama Persada PT", "XIRKA-DAMA-PERSADA-PT" },
	{ 0x94E848, "FYLDE MICRO LTD", "FYLDE-MICRO" },
	{ 0x94E98C, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x94EB2C, "Google Inc.", "GOOGLE" },
	{ 0x94EBCD, "Research In Motion Limited", "RESEARCH-IN-MOTION" },
	{ 0x94F19E, "HUIZHOU MAORONG INTELLIGENT TECHNOLOGY CO.,LTD", "HUIZHOU-MAORONG-INTELLIGENT" },
	{ 0x94F665, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0x94F692, "Geminico co.,Ltd.", "GEMINICO" },
	{ 0x94F720, "Tianjin Deviser Electronics Instrument Co., Ltd", "TIANJIN-DEVISER-ELECTRONICS-INSTRUMENT" },
	{ 0x94FAE8, "Shenzhen Eycom Technology Co., Ltd", "SHENZHEN-EYCOM" },
	{ 0x94FBB2, "Shenzhen Gongjin Electronics Co.,Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0x94FD1D, "WhereWhen Corp", "WHEREWHEN" },
	{ 0x94FD2E, "Shanghai Uniscope Technologies Co.,Ltd", "SHANGHAI-UNISCOPE-TECHNOLOGIES" },
	{ 0x94FEF4, "SAGEMCOM", "SAGEMCOM" },
	{ 0x980284, "Theobroma Systems GmbH", "THEOBROMA" },
	{ 0x9802D8, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x9803A0, "ABB n.v. Power Quality Products", "ABB-N-V-POWER-QUALITY-PRODUCTS" },
	{ 0x9803D8, "Apple", "APPLE" },
	{ 0x980C82, "Samsung Electro Mechanics", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0x980D2E, "HTC Corporation", "HTC" },
	{ 0x980EE4, "PRIVATE", "PRIVATE" },
	{ 0x981094, "Shenzhen Vsun communication technology Co.,ltd", "SHENZHEN-VSUN-COMMUNICATION" },
	{ 0x9816EC, "IC Intracom", "IC-INTRACOM" },
	{ 0x981DFA, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x98208E, "Definium Technologies", "DEFINIUM-TECHNOLOGIES" },
	{ 0x98262A, "Applied Research Associates, Inc", "APPLIED-RESEARCH-ASSOCIATES" },
	{ 0x98291D, "Jaguar de Mexico, SA de CV", "JAGUAR-DE-MEXICO-DE-CV" },
	{ 0x98293F, "Fujian Start Computer Equipment Co.,Ltd", "FUJIAN-START-EQUIPMENT" },
	{ 0x982CBE, "2Wire", "2WIRE" },
	{ 0x982D56, "Resolution Audio", "RESOLUTION-AUDIO" },
	{ 0x982F3C, "Sichuan Changhong Electric Ltd.", "SICHUAN-CHANGHONG-ELECTRIC" },
	{ 0x983000, "Beijing KEMACOM Technologies Co., Ltd.", "BEIJING-KEMACOM-TECHNOLOGIES" },
	{ 0x983071, "DAIKYUNG VASCOM", "DAIKYUNG-VASCOM" },
	{ 0x98349D, "Krauss Maffei Technologies GmbH", "KRAUSS-MAFFEI-TECHNOLOGIES" },
	{ 0x983571, "Sub10 Systems Ltd", "SUB10" },
	{ 0x9835B8, "Assembled Products Corporation", "ASSEMBLED-PRODUCTS" },
	{ 0x983713, "PT.Navicom Indonesia", "PT-NAVICOM-INDONESIA" },
	{ 0x983B16, "AMPAK Technology Inc", "AMPAK" },
	{ 0x983F9F, "China SSJ (Suzhou) Network Technology Inc.", "CHINA-SSJ-SUZHOU-NETWORK" },
	{ 0x984246, "SOL INDUSTRY PTE., LTD", "SOL-INDUSTRY-PTE" },
	{ 0x9843DA, "INTERTECH", "INTERTECH" },
	{ 0x98473C, "SHANGHAI SUNMON COMMUNICATION TECHNOGY CO.,LTD", "SHANGHAI-SUNMON-COMMUNICATION-TECHNOGY" },
	{ 0x984A47, "CHG Hospital Beds", "CHG-HOSPITAL-BEDS" },
	{ 0x984B4A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x984BE1, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x984C04, "Zhangzhou Keneng Electrical Equipment Co Ltd", "ZHANGZHOU-KENENG-ELECTRICAL-EQUIPMENT" },
	{ 0x984CD3, "Mantis Deposition", "MANTIS-DEPOSITION" },
	{ 0x984E97, "Starlight Marketing (H. K.) Ltd.", "STARLIGHT-MARKETING-H-K" },
	{ 0x984FEE, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x9852B1, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0x9857D3, "HON HAI-CCPBG PRECISION IND.CO.,LTD.", "HON-HAI-CCPBG-PRECISION-IND" },
	{ 0x98588A, "SYSGRATION Ltd.", "SYSGRATION" },
	{ 0x985945, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x985AEB, "Apple, Inc.", "APPLE" },
	{ 0x985C93, "SBG Systems SAS", "SBG-SAS" },
	{ 0x985D46, "PeopleNet Communication", "PEOPLENET-COMMUNICATION" },
	{ 0x985E1B, "ConversDigital Co., Ltd.", "CONVERSDIGITAL" },
	{ 0x986022, "EMW Co., Ltd.", "EMW" },
	{ 0x9866EA, "Industrial Control Communications, Inc.", "INDUSTRIAL-CONTROL-COMMUNICATION" },
	{ 0x986B3D, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0x986CF5, "zte corporation", "ZTE" },
	{ 0x986DC8, "TOSHIBA MITSUBISHI-ELECTRIC INDUSTRIAL SYSTEMS CORPORATION", "TOSHIBA-MITSUBISHI-ELECTRIC-INDUSTRIAL" },
	{ 0x9873C4, "Sage Electronic Engineering LLC", "SAGE-ENGINEERING-LLC" },
	{ 0x9876B6, "Adafruit", "ADAFRUIT" },
	{ 0x987770, "Pep Digital Technology (Guangzhou) Co., Ltd", "PEP-DIGITAL-GUANGZHOU" },
	{ 0x987BF3, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0x987E46, "Emizon Networks Limited", "EMIZON-NETWORKS" },
	{ 0x988217, "Disruptive Ltd", "DISRUPTIVE" },
	{ 0x988389, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0x9886B1, "Flyaudio corporation (China)", "FLYAUDIO-CHINA" },
	{ 0x9889ED, "Anadem Information Inc.", "ANADEM-INFORMATION" },
	{ 0x988B5D, "SAGEM COMMUNICATION", "SAGEM-COMMUNICATION" },
	{ 0x988BAD, "Corintech Ltd.", "CORINTECH" },
	{ 0x988E34, "ZHEJIANG BOXSAM ELECTRONIC CO.,LTD", "ZHEJIANG-BOXSAM" },
	{ 0x988E4A, "NOXUS(BEIJING) TECHNOLOGY CO.,LTD", "NOXUS-BEIJING" },
	{ 0x988EDD, "TE Connectivity Limerick", "TE-CONNECTIVITY-LIMERICK" },
	{ 0x989080, "Linkpower Network System Inc Ltd.", "LINKPOWER-NETWORK-SYSTEM" },
	{ 0x989096, "Dell Inc", "DELL" },
	{ 0x9893CC, "LG Electronics Inc.", "LG-ELECTRONICS" },
	{ 0x989449, "Skyworth Wireless Technology Ltd.", "SKYWORTH-WIRELESS" },
	{ 0x98A7B0, "MCST ZAO", "MCST-ZAO" },
	{ 0x98AAD7, "BLUE WAVE NETWORKING CO LTD", "BLUE-WAVE-NETWORKING" },
	{ 0x98B039, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0x98B8E3, "Apple", "APPLE" },
	{ 0x98BC57, "SVA TECHNOLOGIES CO.LTD", "SVA-TECHNOLOGIES" },
	{ 0x98BC99, "Edeltech Co.,Ltd.", "EDELTECH" },
	{ 0x98BE94, "IBM", "IBM" },
	{ 0x98C0EB, "Global Regency Ltd", "GLOBAL-REGENCY" },
	{ 0x98C845, "PacketAccess", "PACKETACCESS" },
	{ 0x98CDB4, "Virident Systems, Inc.", "VIRIDENT" },
	{ 0x98D331, "Shenzhen Bolutek Technology Co.,Ltd.", "SHENZHEN-BOLUTEK" },
	{ 0x98D686, "Chyi Lee industry Co., ltd.", "CHYI-LEE-INDUSTRY" },
	{ 0x98D6BB, "Apple", "APPLE" },
	{ 0x98D6F7, "LG Electronics", "LG-ELECTRONICS" },
	{ 0x98D88C, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0x98DA92, "Vuzix Corporation", "VUZIX" },
	{ 0x98DCD9, "UNITEC Co., Ltd.", "UNITEC" },
	{ 0x98E0D9, "Apple", "APPLE" },
	{ 0x98E165, "Accutome", "ACCUTOME" },
	{ 0x98E79A, "Foxconn(NanJing) Communication Co.,Ltd.", "FOXCONN-NANJING-COMMUNICATION" },
	{ 0x98EC65, "Cosesy ApS", "COSESY-APS" },
	{ 0x98EECB, "Wistron InfoComm(ZhongShan)Corporation", "WISTRON-INFOCOMM-ZHONGSHAN" },
	{ 0x98F0AB, "Apple", "APPLE" },
	{ 0x98F170, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0x98F537, "zte corporation", "ZTE" },
	{ 0x98F5A9, "OHSUNG ELECTRONICS CO.,LTD.", "OHSUNG-ELECTRONICS" },
	{ 0x98F8C1, "IDT Technology Limited", "IDT" },
	{ 0x98F8DB, "Marini Impianti Industriali s.r.l.", "MARINI-IMPIANTI-INDUSTRIALI-S-R-L" },
	{ 0x98FAE3, "Xiaomi inc.", "XIAOMI" },
	{ 0x98FB12, "Grand Electronics (HK) Ltd", "GRAND-ELECTRONICS-HK" },
	{ 0x98FC11, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0x98FE03, "Ericsson - North America", "ERICSSON---NORTH-AMERICA" },
	{ 0x98FE94, "Apple", "APPLE" },
	{ 0x98FF6A, "OTEC(Shanghai)Technology Co.,Ltd.", "OTEC-SHANGHAI" },
	{ 0x98FFD0, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0x9C0111, "Shenzhen Newabel Electronic Co., Ltd.", "SHENZHEN-NEWABEL" },
	{ 0x9C0298, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x9C039E, "Beijing Winchannel Software Technology Co., Ltd", "BEIJING-WINCHANNEL-SOFTWARE" },
	{ 0x9C0473, "Tecmobile (International) Ltd.", "TECMOBILE" },
	{ 0x9C04EB, "Apple", "APPLE" },
	{ 0x9C066E, "Hytera Communications Corporation Limited", "HYTERA-COMMUNICATION" },
	{ 0x9C0DAC, "Tymphany HK Limited", "TYMPHANY-HK" },
	{ 0x9C1465, "Edata Elektronik San. ve Tic. A.Ş.", "EDATA-ELEKTRONIK-SAN-VE-TIC-A-Ş" },
	{ 0x9C1874, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0x9C1C12, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0x9C1FDD, "Accupix Inc.", "ACCUPIX" },
	{ 0x9C207B, "Apple", "APPLE" },
	{ 0x9C216A, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0x9C220E, "TASCAN Service GmbH", "TASCAN-SERVICE" },
	{ 0x9C2840, "Discovery Technology,LTD..", "DISCOVERY" },
	{ 0x9C28BF, "Continental Automotive Czech Republic s.r.o.", "CONTINENTAL-AUTOMOTIVE-CZECH-REPUBLIC-S-R-O" },
	{ 0x9C28EF, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x9C2A70, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x9C3066, "RWE Effizienz GmbH", "RWE-EFFIZIENZ" },
	{ 0x9C3178, "Foshan Huadian Intelligent Communications Teachnologies Co.,Ltd", "FOSHAN-HUADIAN-INTELLIGENT-COMMUNICATION-TEACHNOLOGIES" },
	{ 0x9C31B6, "Kulite Semiconductor Products Inc", "KULITE-SEMICONDUCTOR-PRODUCTS" },
	{ 0x9C3583, "Nipro Diagnostics, Inc", "NIPRO-DIAGNOSTICS" },
	{ 0x9C35EB, "Apple, Inc.", "APPLE" },
	{ 0x9C37F4, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0x9C3AAF, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x9C3EAA, "EnvyLogic Co.,Ltd.", "ENVYLOGIC" },
	{ 0x9C417C, "Hame Technology Co., Limited", "HAME" },
	{ 0x9C443D, "CHENGDU XUGUANG TECHNOLOGY CO, LTD", "CHENGDU-XUGUANG" },
	{ 0x9C44A6, "SwiftTest, Inc.", "SWIFTTEST" },
	{ 0x9C4563, "DIMEP Sistemas", "DIMEP-SISTEMAS" },
	{ 0x9C4A7B, "Nokia Corporation", "NOKIA" },
	{ 0x9C4CAE, "Mesa Labs", "MESA-LABS" },
	{ 0x9C4E20, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x9C4E36, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0x9C4E8E, "ALT Systems Ltd", "ALT" },
	{ 0x9C4EBF, "BoxCast", "BOXCAST" },
	{ 0x9C53CD, "ENGICAM s.r.l.", "ENGICAM-S-R-L" },
	{ 0x9C541C, "Shenzhen My-power Technology Co.,Ltd", "SHENZHEN-MY-POWER" },
	{ 0x9C54CA, "Zhengzhou VCOM Science and Technology Co.,Ltd", "ZHENGZHOU-VCOM-SCIENCE-AND" },
	{ 0x9C55B4, "I.S.E. S.r.l.", "I-S-E-S-R-L" },
	{ 0x9C5711, "Feitian Xunda(Beijing) Aeronautical Information Technology Co., Ltd.", "FEITIAN-XUNDA-BEIJING-AERONAUTICAL-INFORMATION" },
	{ 0x9C5B96, "NMR Corporation", "NMR" },
	{ 0x9C5C8D, "FIREMAX INDÚSTRIA E COMÉRCIO DE PRODUTOS ELETRÔNICOS LTDA", "FIREMAX-INDÚSTRIA-E-COMÉRCIO-DE-PRODUTOS-ELETRÔNICOS-LTDA" },
	{ 0x9C5D12, "Aerohive Networks Inc", "AEROHIVE-NETWORKS" },
	{ 0x9C5D95, "VTC Electronics Corp.", "VTC-ELECTRONICS" },
	{ 0x9C5E73, "Calibre UK Ltd", "CALIBRE-UK" },
	{ 0x9C611D, "Omni-ID USA, Inc.", "OMNI-ID-USA" },
	{ 0x9C645E, "Harman Consumer Group", "HARMAN-CONSUMER-GROUP" },
	{ 0x9C65B0, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x9C65F9, "AcSiP Technology Corp.", "ACSIP" },
	{ 0x9C6650, "Glodio Technolies Co.,Ltd Tianjin Branch", "GLODIO-TECHNOLIES-TIANJIN-BRANCH" },
	{ 0x9C685B, "Octonion SA", "OCTONION" },
	{ 0x9C6ABE, "QEES ApS.", "QEES-APS" },
	{ 0x9C6C15, "Microsoft Corporation", "MICROSOFT" },
	{ 0x9C7514, "Wildix srl", "WILDIX-SRL" },
	{ 0x9C77AA, "NADASNV", "NADASNV" },
	{ 0x9C79AC, "Suntec Software(Shanghai) Co., Ltd.", "SUNTEC-SOFTWARE-SHANGHAI" },
	{ 0x9C7BD2, "NEOLAB Convergence", "NEOLAB-CONVERGENCE" },
	{ 0x9C807D, "SYSCABLE Korea Inc.", "SYSCABLE-KOREA" },
	{ 0x9C80DF, "Arcadyan Technology Corporation", "ARCADYAN" },
	{ 0x9C86DA, "Phoenix Geophysics Ltd.", "PHOENIX-GEOPHYSICS" },
	{ 0x9C8888, "Simac Techniek NV", "SIMAC-TECHNIEK-NV" },
	{ 0x9C8BF1, "The Warehouse Limited", "THE-WAREHOUSE" },
	{ 0x9C8D1A, "INTEG process group inc", "INTEG-PROCESS-GROUP" },
	{ 0x9C8E99, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0x9C8EDC, "Teracom Limited", "TERACOM" },
	{ 0x9C934E, "Xerox Corporation", "XEROX" },
	{ 0x9C93E4, "PRIVATE", "PRIVATE" },
	{ 0x9C95F8, "SmartDoor Systems, LLC", "SMARTDOOR-LLC" },
	{ 0x9C9726, "Technicolor", "TECHNICOLOR" },
	{ 0x9C9811, "Guangzhou Sunrise Electronics Development Co., Ltd", "GUANGZHOU-SUNRISE-ELECTRONICS-DEVELOPMENT" },
	{ 0x9C9C1D, "Starkey Labs Inc.", "STARKEY-LABS" },
	{ 0x9CA10A, "SCLE SFE", "SCLE-SFE" },
	{ 0x9CA134, "Nike, Inc.", "NIKE" },
	{ 0x9CA3BA, "SAKURA Internet Inc.", "SAKURA-INTERNET" },
	{ 0x9CA577, "Osorno Enterprises Inc.", "OSORNO-ENTERPRISES" },
	{ 0x9CA9E4, "zte corporation", "ZTE" },
	{ 0x9CAD97, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x9CADEF, "Obihai Technology, Inc.", "OBIHAI" },
	{ 0x9CAFCA, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0x9CB008, "Ubiquitous Computing Technology Corporation", "UBIQUITOUS-COMPUTING" },
	{ 0x9CB206, "PROCENTEC", "PROCENTEC" },
	{ 0x9CB654, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0x9CB6D0, "Rivet Networks", "RIVET-NETWORKS" },
	{ 0x9CB70D, "Liteon Technology Corporation", "LITEON" },
	{ 0x9CB793, "Creatcomm Technology Inc.", "CREATCOMM" },
	{ 0x9CBB98, "Shen Zhen RND Electronic Co.,LTD", "SHEN-ZHEN-RND" },
	{ 0x9CBD9D, "SkyDisk, Inc.", "SKYDISK" },
	{ 0x9CBEE0, "Biosoundlab Co., Ltd.", "BIOSOUNDLAB" },
	{ 0x9CC077, "PrintCounts, LLC", "PRINTCOUNTS-LLC" },
	{ 0x9CC0D2, "Conductix-Wampfler GmbH", "CONDUCTIX-WAMPFLER" },
	{ 0x9CC172, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0x9CC7A6, "AVM GmbH", "AVM" },
	{ 0x9CC7D1, "SHARP Corporation", "SHARP" },
	{ 0x9CCAD9, "Nokia Corporation", "NOKIA" },
	{ 0x9CCD82, "CHENG UEI PRECISION INDUSTRY CO.,LTD", "CHENG-UEI-PRECISION-INDUSTRY" },
	{ 0x9CD21E, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0x9CD24B, "zte corporation", "ZTE" },
	{ 0x9CD35B, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x9CD36D, "NETGEAR INC.,", "NETGEAR" },
	{ 0x9CD643, "D-Link International", "D-LINK" },
	{ 0x9CD917, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0x9CDF03, "Harman/Becker Automotive Systems GmbH", "HARMAN/BECKER-AUTOMOTIVE" },
	{ 0x9CE10E, "NCTech Ltd", "NCTECH" },
	{ 0x9CE1D6, "Junger Audio-Studiotechnik GmbH", "JUNGER-AUDIO-STUDIOTECHNIK" },
	{ 0x9CE230, "JULONG CO,.LTD.", "JULONG" },
	{ 0x9CE635, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0x9CE6E7, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0x9CE7BD, "Winduskorea co., Ltd", "WINDUSKOREA" },
	{ 0x9CEBE8, "BizLink (Kunshan) Co.,Ltd", "BIZLINK-KUNSHAN" },
	{ 0x9CF387, "Apple", "APPLE" },
	{ 0x9CF61A, "UTC Fire and Security", "UTC-FIRE-AND-SECURITY" },
	{ 0x9CF67D, "Ricardo Prague, s.r.o.", "RICARDO-PRAGUE-S-R-O" },
	{ 0x9CF8DB, "shenzhen eyunmei technology co,.ltd", "SHENZHEN-EYUNMEI" },
	{ 0x9CF938, "AREVA NP GmbH", "AREVA-NP" },
	{ 0x9CFBF1, "MESOMATIC GmbH & Co.KG", "MESOMATIC" },
	{ 0x9CFFBE, "OTSL Inc.", "OTSL" },
	{ 0xA002DC, "Amazon Technologies Inc.", "AMAZON-TECHNOLOGIES" },
	{ 0xA00363, "Robert Bosch Healthcare GmbH", "ROBERT-BOSCH-HEALTHCARE" },
	{ 0xA00627, "NEXPA System", "NEXPA-SYSTEM" },
	{ 0xA00798, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xA007B6, "Advanced Technical Support, Inc.", "ADVANCED-TECHNICAL-SUPPORT" },
	{ 0xA00ABF, "Wieson Technologies Co., Ltd.", "WIESON-TECHNOLOGIES" },
	{ 0xA00BBA, "SAMSUNG ELECTRO-MECHANICS", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xA00CA1, "SKTB SKiT", "SKTB-SKIT" },
	{ 0xA01290, "Avaya, Inc", "AVAYA" },
	{ 0xA012DB, "TABUCHI ELECTRIC CO.,LTD", "TABUCHI-ELECTRIC" },
	{ 0xA0133B, "HiTi Digital, Inc.", "HITI-DIGITAL" },
	{ 0xA0143D, "PARROT SA", "PARROT" },
	{ 0xA0165C, "Triteka LTD", "TRITEKA" },
	{ 0xA01859, "Shenzhen Yidashi Electronics Co Ltd", "SHENZHEN-YIDASHI-ELECTRONICS" },
	{ 0xA01917, "Bertel S.p.a.", "BERTEL-S-P-A" },
	{ 0xA01C05, "NIMAX TELECOM CO.,LTD.", "NIMAX-TELECOM" },
	{ 0xA01D48, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xA02195, "Samsung Electronics Digital Imaging", "SAMSUNG-ELECTRONICS-DIGITAL-IMAGING" },
	{ 0xA021B7, "NETGEAR", "NETGEAR" },
	{ 0xA0231B, "TeleComp R&D Corp.", "TELECOMP-R-D" },
	{ 0xA02BB8, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xA02EF3, "United Integrated Services Co., Led.", "UNITED-INTEGRATED-SERVICES-LED" },
	{ 0xA0369F, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xA036F0, "Comprehensive Power", "COMPREHENSIVE-POWER" },
	{ 0xA036FA, "Ettus Research LLC", "ETTUS-RESEARCH-LLC" },
	{ 0xA03A75, "PSS Belgium N.V.", "PSS-BELGIUM-N-V" },
	{ 0xA03B1B, "Inspire Tech", "INSPIRE-TECH" },
	{ 0xA04025, "Actioncable, Inc.", "ACTIONCABLE" },
	{ 0xA04041, "SAMWONFA Co.,Ltd.", "SAMWONFA" },
	{ 0xA041A7, "NL Ministry of Defense", "NL-MINISTRY-OF-DEFENSE" },
	{ 0xA0423F, "Tyan Computer Corp", "TYAN" },
	{ 0xA0481C, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xA04CC1, "Helixtech Corp.", "HELIXTECH" },
	{ 0xA04E04, "Nokia Corporation", "NOKIA" },
	{ 0xA04FD4, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0xA051C6, "Avaya, Inc", "AVAYA" },
	{ 0xA0554F, "Cisco", "CISCO" },
	{ 0xA055DE, "Pace plc", "PACE-PLC" },
	{ 0xA056B2, "Harman/Becker Automotive Systems GmbH", "HARMAN/BECKER-AUTOMOTIVE" },
	{ 0xA0593A, "V.D.S. Video Display Systems srl", "V-D-S-VIDEO-DISPLAY-SRL" },
	{ 0xA05AA4, "Grand Products Nevada, Inc.", "GRAND-PRODUCTS-NEVADA" },
	{ 0xA05B21, "ENVINET GmbH", "ENVINET" },
	{ 0xA05DC1, "TMCT Co., LTD.", "TMCT" },
	{ 0xA05DE7, "DIRECTV, Inc.", "DIRECTV" },
	{ 0xA05E6B, "MELPER Co., Ltd.", "MELPER" },
	{ 0xA06391, "Netgear Inc.", "NETGEAR" },
	{ 0xA06518, "VNPT TECHNOLOGY", "VNPT" },
	{ 0xA067BE, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0xA06986, "Wellav Technologies Ltd", "WELLAV-TECHNOLOGIES" },
	{ 0xA06A00, "Verilink Corporation", "VERILINK" },
	{ 0xA06CEC, "RIM", "RIM" },
	{ 0xA06D09, "Intelcan Technosystems Inc.", "INTELCAN-TECHNOSYSTEMS" },
	{ 0xA06E50, "Nanotek Elektronik Sistemler Ltd. Sti.", "NANOTEK-ELEKTRONIK-SISTEMLER-STI" },
	{ 0xA071A9, "Nokia Corporation", "NOKIA" },
	{ 0xA07332, "Cashmaster International Limited", "CASHMASTER" },
	{ 0xA073FC, "Rancore Technologies Private Limited", "RANCORE-TECHNOLOGIES-PRIVATE" },
	{ 0xA07591, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xA07771, "Vialis BV", "VIALIS" },
	{ 0xA078BA, "Pantech Co., Ltd.", "PANTECH" },
	{ 0xA0821F, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xA082C7, "P.T.I Co.,LTD", "P-T-I" },
	{ 0xA0861D, "Chengdu Fuhuaxin Technology co.,Ltd", "CHENGDU-FUHUAXIN" },
	{ 0xA086C6, "XIAOMI INC", "XIAOMI" },
	{ 0xA086EC, "SAEHAN HITEC Co., Ltd", "SAEHAN-HITEC" },
	{ 0xA08869, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xA088B4, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xA089E4, "Skyworth Digital Technology(Shenzhen) Co.,Ltd", "SKYWORTH-DIGITAL-SHENZHEN" },
	{ 0xA08A87, "HuiZhou KaiYue Electronic Co.,Ltd", "HUIZHOU-KAIYUE" },
	{ 0xA08C15, "Gerhard D. Wempe KG", "GERHARD-D-WEMPE" },
	{ 0xA08C9B, "Xtreme Technologies Corp", "XTREME-TECHNOLOGIES" },
	{ 0xA090DE, "VEEDIMS,LLC", "VEEDIMS-LLC" },
	{ 0xA09347, "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD.", "GUANGDONG-OPPO-MOBILE-TELECOMMUNICATIONS" },
	{ 0xA09805, "OpenVox Communication Co Ltd", "OPENVOX-COMMUNICATION" },
	{ 0xA098ED, "Shandong Intelligent Optical Communication Development Co., Ltd.", "SHANDONG-INTELLIGENT-OPTICAL-COMMUNICATION-DEVELOPMENT" },
	{ 0xA0999B, "Apple", "APPLE" },
	{ 0xA09A5A, "Time Domain", "TIME-DOMAIN" },
	{ 0xA09BBD, "Total Aviation Solutions Pty Ltd", "TOTAL-AVIATION-SOLUTIONS-PTY" },
	{ 0xA0A130, "DLI Taiwan Branch office", "DLI-TAIWAN-BRANCH-OFFICE" },
	{ 0xA0A23C, "GPMS", "GPMS" },
	{ 0xA0A3E2, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0xA0A763, "Polytron Vertrieb GmbH", "POLYTRON-VERTRIEB" },
	{ 0xA0A8CD, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xA0AAFD, "EraThink Technologies Corp.", "ERATHINK-TECHNOLOGIES" },
	{ 0xA0ADA1, "JMR Electronics, Inc", "JMR-ELECTRONICS" },
	{ 0xA0B100, "ShenZhen Cando Electronics Co.,Ltd", "SHENZHEN-CANDO-ELECTRONICS" },
	{ 0xA0B3CC, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xA0B437, "GENERAL DYNAMICS C4 SYSEMS", "GENERAL-DYNAMICS-C4-SYSEMS" },
	{ 0xA0B4A5, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0xA0B5DA, "HongKong THTF Co., Ltd", "HONGKONG-THTF" },
	{ 0xA0B662, "Acutvista Innovation Co., Ltd.", "ACUTVISTA-INNOVATION" },
	{ 0xA0B9ED, "Skytap", "SKYTAP" },
	{ 0xA0BAB8, "Pixon Imaging", "PIXON-IMAGING" },
	{ 0xA0BB3E, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xA0BF50, "S.C. ADD-PRODUCTION S.R.L.", "S-C-ADD-PRODUCTION-S-R-L" },
	{ 0xA0BFA5, "CORESYS", "CORESYS" },
	{ 0xA0C2DE, "Costar Video Systems", "COSTAR-VIDEO" },
	{ 0xA0C3DE, "Triton Electronic Systems Ltd.", "TRITON" },
	{ 0xA0C562, "Pace plc", "PACE-PLC" },
	{ 0xA0C6EC, "ShenZhen ANYK Technology Co.,LTD", "SHENZHEN-ANYK" },
	{ 0xA0CEC8, "CE LINK LIMITED", "CE-LINK" },
	{ 0xA0CF5B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xA0D12A, "AXPRO Technology Inc.", "AXPRO" },
	{ 0xA0D3C1, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xA0DA92, "Nanjing Glarun Atten Technology Co. Ltd.", "NANJING-GLARUN-ATTEN" },
	{ 0xA0DC04, "Becker-Antriebe GmbH", "BECKER-ANTRIEBE" },
	{ 0xA0DD97, "PolarLink Technologies, Ltd", "POLARLINK-TECHNOLOGIES" },
	{ 0xA0DDE5, "SHARP Corporation", "SHARP" },
	{ 0xA0DE05, "JSC \"Irbis-T\"", "JSC-IRBIS-T" },
	{ 0xA0E201, "AVTrace Ltd.(China)", "AVTRACE-CHINA" },
	{ 0xA0E25A, "Amicus SK, s.r.o.", "AMICUS-SK-S-R-O" },
	{ 0xA0E295, "DAT System Co.,Ltd", "DAT-SYSTEM" },
	{ 0xA0E453, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0xA0E4CB, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0xA0E534, "Stratec Biomedical AG", "STRATEC-BIOMEDICAL" },
	{ 0xA0E5E9, "enimai Inc", "ENIMAI" },
	{ 0xA0E6F8, "Texas Instruments Inc", "TEXAS-INSTRUMENTS" },
	{ 0xA0E9DB, "Ningbo FreeWings Technologies Co.,Ltd", "NINGBO-FREEWINGS-TECHNOLOGIES" },
	{ 0xA0EB76, "AirCUVE Inc.", "AIRCUVE" },
	{ 0xA0EC80, "zte corporation", "ZTE" },
	{ 0xA0ECF9, "Cisco", "CISCO" },
	{ 0xA0EDCD, "Apple", "APPLE" },
	{ 0xA0EF84, "Seine Image Int'l Co., Ltd", "SEINE-IMAGE" },
	{ 0xA0F217, "GE Medical System(China) Co., Ltd.", "GE-MEDICAL-SYSTEM-CHINA" },
	{ 0xA0F3C1, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xA0F3E4, "Alcatel Lucent IPD", "ALCATEL-LUCENT-IPD" },
	{ 0xA0F419, "Nokia Corporation", "NOKIA" },
	{ 0xA0F450, "HTC Corporation", "HTC" },
	{ 0xA0F459, "FN-LINK TECHNOLOGY LIMITED", "FN-LINK" },
	{ 0xA0F6FD, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xA0F849, "Cisco", "CISCO" },
	{ 0xA0FC6E, "Telegrafia a.s.", "TELEGRAFIA-A-S" },
	{ 0xA0FE91, "AVAT Automation GmbH", "AVAT-AUTOMATION" },
	{ 0xA40130, "ABIsystems Co., LTD", "ABISYSTEMS" },
	{ 0xA4059E, "STA Infinity LLP", "STA-INFINITY-LLP" },
	{ 0xA408EA, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0xA409CB, "Alfred Kaercher GmbH &amp; Co KG", "ALFRED-KAERCHER-AMP" },
	{ 0xA40BED, "Carry Technology Co.,Ltd", "CARRY" },
	{ 0xA40CC3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xA41242, "NEC Platforms, Ltd.", "NEC-PLATFORMS" },
	{ 0xA4134E, "Luxul", "LUXUL" },
	{ 0xA41566, "Wei Fang Goertek Electronics Co.,Ltd", "WEI-FANG-GOERTEK-ELECTRONICS" },
	{ 0xA41731, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xA41875, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xA41BC0, "Fastec Imaging Corporation", "FASTEC-IMAGING" },
	{ 0xA41F72, "Dell Inc.", "DELL" },
	{ 0xA4218A, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0xA42305, "Open Networking Laboratory", "OPEN-NETWORKING-LABORATORY" },
	{ 0xA424B3, "FlatFrog Laboratories AB", "FLATFROG-LABORATORIES-AB" },
	{ 0xA424DD, "Cambrionix Ltd", "CAMBRIONIX" },
	{ 0xA4251B, "Avaya, Inc", "AVAYA" },
	{ 0xA42940, "Shenzhen YOUHUA Technology Co., Ltd", "SHENZHEN-YOUHUA" },
	{ 0xA429B7, "bluesky", "BLUESKY" },
	{ 0xA42B8C, "Netgear Inc", "NETGEAR" },
	{ 0xA42C08, "Masterwork Automodules", "MASTERWORK-AUTOMODULES" },
	{ 0xA433D1, "Fibrlink Communications Co.,Ltd.", "FIBRLINK-COMMUNICATION" },
	{ 0xA434D9, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xA438FC, "Plastic Logic", "PLASTIC-LOGIC" },
	{ 0xA43A69, "Vers Inc", "VERS" },
	{ 0xA43BFA, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xA43D78, "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD", "GUANGDONG-OPPO-MOBILE-TELECOMMUNICATIONS" },
	{ 0xA4466B, "EOC Technology", "EOC" },
	{ 0xA446FA, "AmTRAN Video Corporation", "AMTRAN-VIDEO" },
	{ 0xA44AD3, "ST Electronics(Shanghai) Co.,Ltd", "ST-ELECTRONICS-SHANGHAI" },
	{ 0xA44B15, "Sun Cupid Technology (HK) LTD", "SUN-CUPID-HK" },
	{ 0xA44C11, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xA44E2D, "Adaptive Wireless Solutions, LLC", "ADAPTIVE-WIRELESS-SOLUTIONS-LLC" },
	{ 0xA44E31, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xA44F29, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xA45055, "busware.de", "BUSWARE-DE" },
	{ 0xA4526F, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0xA45602, "fenglian Technology Co.,Ltd.", "FENGLIAN" },
	{ 0xA4561B, "MCOT Corporation", "MCOT" },
	{ 0xA45630, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xA45A1C, "smart-electronic GmbH", "SMART" },
	{ 0xA45C27, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0xA45D36, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xA45DA1, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0xA45E60, "Apple", "APPLE" },
	{ 0xA46032, "MRV Communications (Networks) LTD", "MRV-COMMUNICATION-NETWORKS" },
	{ 0xA46706, "Apple", "APPLE" },
	{ 0xA46C2A, "Cisco", "CISCO" },
	{ 0xA46CC1, "LTi REEnergy GmbH", "LTI-REENERGY" },
	{ 0xA46E79, "DFT System Co.Ltd", "DFT-SYSTEM" },
	{ 0xA470D6, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0xA47733, "Google", "GOOGLE" },
	{ 0xA47760, "Nokia Corporation", "NOKIA" },
	{ 0xA479E4, "KLINFO Corp", "KLINFO" },
	{ 0xA47AA4, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xA47ACF, "VIBICOM COMMUNICATIONS INC.", "VIBICOM-COMMUNICATION" },
	{ 0xA47B85, "ULTIMEDIA Co Ltd,", "ULTIMEDIA" },
	{ 0xA47C14, "ChargeStorm AB", "CHARGESTORM-AB" },
	{ 0xA47C1F, "Cobham plc", "COBHAM-PLC" },
	{ 0xA47E39, "zte corporation", "ZTE" },
	{ 0xA481EE, "Nokia Corporation", "NOKIA" },
	{ 0xA4856B, "Q Electronics Ltd", "Q-ELECTRONICS" },
	{ 0xA4895B, "ARK INFOSOLUTIONS PVT LTD", "ARK-INFOSOLUTIONS-PVT" },
	{ 0xA48CDB, "Lenovo", "LENOVO" },
	{ 0xA49005, "CHINA GREATWALL COMPUTER SHENZHEN CO.,LTD", "CHINA-GREATWALL-SHENZHEN" },
	{ 0xA4934C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xA497BB, "Hitachi Industrial Equipment Systems Co.,Ltd", "HITACHI-INDUSTRIAL-EQUIPMENT" },
	{ 0xA49947, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xA49981, "FuJian Elite Power Tech CO.,LTD.", "FUJIAN-ELITE-POWER-TECH" },
	{ 0xA49A58, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xA49B13, "Burroughs Payment Systems, Inc.", "BURROUGHS-PAYMENT" },
	{ 0xA49D49, "Ketra, Inc.", "KETRA" },
	{ 0xA49EDB, "AutoCrib, Inc.", "AUTOCRIB" },
	{ 0xA49F85, "Lyve Minds, Inc", "LYVE-MINDS" },
	{ 0xA49F89, "Shanghai Rui Rui Communication Technology Co.Ltd.", "SHANGHAI-RUI-RUI-COMMUNICATION" },
	{ 0xA4A1C2, "Ericsson AB (EAB)", "ERICSSON-AB-EAB" },
	{ 0xA4A1E4, "Innotube, Inc.", "INNOTUBE" },
	{ 0xA4A24A, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0xA4A4D3, "Bluebank Communication Technology Co.Ltd", "BLUEBANK-COMMUNICATION" },
	{ 0xA4A80F, "Shenzhen Coship Electronics Co., Ltd.", "SHENZHEN-COSHIP-ELECTRONICS" },
	{ 0xA4AD00, "Ragsdale Technology", "RAGSDALE" },
	{ 0xA4ADB8, "Vitec Group, Camera Dynamics Ltd", "VITEC-GROUP-CAMERA-DYNAMICS" },
	{ 0xA4AE9A, "Maestro Wireless Solutions ltd.", "MAESTRO-WIRELESS-SOLUTIONS" },
	{ 0xA4B121, "Arantia 2010 S.L.", "ARANTIA-2010-S-L" },
	{ 0xA4B197, "Apple", "APPLE" },
	{ 0xA4B1E9, "Technicolor", "TECHNICOLOR" },
	{ 0xA4B1EE, "H. ZANDER GmbH & Co. KG", "H-ZANDER" },
	{ 0xA4B2A7, "Adaxys Solutions AG", "ADAXYS-SOLUTIONS" },
	{ 0xA4B36A, "JSC SDO Chromatec", "JSC-SDO-CHROMATEC" },
	{ 0xA4B818, "PENTA Gesellschaft für elektronische Industriedatenverarbeitung mbH", "PENTA-GESELLSCHAFT-FüR-ELEKTRONISCHE-INDUSTRIEDATENVERARBEITUNG-MBH" },
	{ 0xA4B980, "Parking BOXX Inc.", "PARKING-BOXX" },
	{ 0xA4BADB, "Dell Inc.", "DELL" },
	{ 0xA4BBAF, "Lime Instruments", "LIME-INSTRUMENTS" },
	{ 0xA4BE61, "EutroVision System, Inc.", "EUTROVISION-SYSTEM" },
	{ 0xA4C0C7, "ShenZhen Hitom Communication Technology Co..LTD", "SHENZHEN-HITOM-COMMUNICATION" },
	{ 0xA4C0E1, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0xA4C2AB, "Hangzhou LEAD-IT Information & Technology Co.,Ltd", "HANGZHOU-LEAD-IT-INFORMATION" },
	{ 0xA4C361, "Apple", "APPLE" },
	{ 0xA4C494, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xA4C7DE, "Cambridge Industries(Group) Co.,Ltd.", "CAMBRIDGE-INDUSTRIES-GROUP" },
	{ 0xA4D094, "Erwin Peters Systemtechnik GmbH", "ERWIN-PETERS-SYSTEMTECHNIK" },
	{ 0xA4D18F, "Shenzhen Skyee Optical Fiber Communication Technology Ltd.", "SHENZHEN-SKYEE-OPTICAL-FIBER-COMMUNICATION" },
	{ 0xA4D1D1, "ECOtality North America", "ECOTALITY-NORTH-AMERICA" },
	{ 0xA4D1D2, "Apple", "APPLE" },
	{ 0xA4D3B5, "GLITEL Stropkov, s.r.o.", "GLITEL-STROPKOV-S-R-O" },
	{ 0xA4D856, "Gimbal, Inc", "GIMBAL" },
	{ 0xA4DA3F, "Bionics Corp.", "BIONICS" },
	{ 0xA4DB2E, "Kingspan Environmental Ltd", "KINGSPAN-ENVIRONMENTAL" },
	{ 0xA4DB30, "Liteon Technology Corporation", "LITEON" },
	{ 0xA4DE50, "Total Walther GmbH", "TOTAL-WALTHER" },
	{ 0xA4E0E6, "FILIZOLA S.A. PESAGEM E AUTOMACAO", "FILIZOLA-PESAGEM-E-AUTOMACAO" },
	{ 0xA4E32E, "Silicon & Software Systems Ltd.", "SILICON-SOFTWARE" },
	{ 0xA4E391, "DENY FONTAINE", "DENY-FONTAINE" },
	{ 0xA4E4B8, "BlackBerry Limited", "BLACKBERRY" },
	{ 0xA4E731, "Nokia Corporation", "NOKIA" },
	{ 0xA4E7E4, "Connex GmbH", "CONNEX" },
	{ 0xA4E991, "SISTEMAS AUDIOVISUALES ITELSIS S.L.", "SISTEMAS-AUDIOVISUALES-ITELSIS-S-L" },
	{ 0xA4E9A3, "Honest Technology Co., Ltd", "HONEST" },
	{ 0xA4EBD3, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xA4ED4E, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xA4EE57, "SEIKO EPSON CORPORATION", "SEIKO-EPSON" },
	{ 0xA4EF52, "Telewave Co., Ltd.", "TELEWAVE" },
	{ 0xA4F3C1, "Open Source Robotics Foundation, Inc.", "OPEN-SOURCE-ROBOTICS-FOUNDATION" },
	{ 0xA4F522, "CHOFU SEISAKUSHO CO.,LTD", "CHOFU-SEISAKUSHO" },
	{ 0xA4F7D0, "LAN Accessories Co., Ltd.", "LAN-ACCESSORIES" },
	{ 0xA4FB8D, "Hangzhou Dunchong Technology Co.Ltd", "HANGZHOU-DUNCHONG" },
	{ 0xA4FCCE, "Security Expert Ltd.", "SECURITY-EXPERT" },
	{ 0xA80180, "IMAGO Technologies GmbH", "IMAGO-TECHNOLOGIES" },
	{ 0xA80600, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xA80C0D, "Cisco", "CISCO" },
	{ 0xA81374, "Panasonic Corporation AVC Networks Company", "PANASONIC-AVC-NETWORKS-COMPANY" },
	{ 0xA8154D, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xA816B2, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xA81758, "Elektronik System i Umeå AB", "ELEKTRONIK-SYSTEM-I-UMEå-AB" },
	{ 0xA81B18, "XTS CORP", "XTS" },
	{ 0xA81B5A, "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD.", "GUANGDONG-OPPO-MOBILE-TELECOMMUNICATIONS" },
	{ 0xA81B5D, "Foxtel Management Pty Ltd", "FOXTEL-MANAGEMENT-PTY" },
	{ 0xA81D16, "AzureWave Technologies, Inc", "AZUREWAVE-TECHNOLOGIES" },
	{ 0xA81FAF, "KRYPTON POLSKA", "KRYPTON-POLSKA" },
	{ 0xA82066, "Apple", "APPLE" },
	{ 0xA824EB, "ZAO NPO Introtest", "ZAO-NPO-INTROTEST" },
	{ 0xA826D9, "HTC Corporation", "HTC" },
	{ 0xA8294C, "Precision Optical Transceivers, Inc.", "PRECISION-OPTICAL-TRANSCEIVERS" },
	{ 0xA82BD6, "Shina System Co., Ltd", "SHINA-SYSTEM" },
	{ 0xA830AD, "Wei Fang Goertek Electronics Co.,Ltd", "WEI-FANG-GOERTEK-ELECTRONICS" },
	{ 0xA8329A, "Digicom Futuristic Technologies Ltd.", "DIGICOM-FUTURISTIC-TECHNOLOGIES" },
	{ 0xA83944, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0xA84041, "Dragino Technology Co., Limited", "DRAGINO" },
	{ 0xA84481, "Nokia Corporation", "NOKIA" },
	{ 0xA845E9, "Firich Enterprises CO., LTD.", "FIRICH-ENTERPRISES" },
	{ 0xA849A5, "Lisantech Co., Ltd.", "LISANTECH" },
	{ 0xA854B2, "Wistron Neweb Corp.", "WISTRON-NEWEB" },
	{ 0xA8556A, "Pocketnet Technology Inc.", "POCKETNET" },
	{ 0xA8574E, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xA85B78, "Apple", "APPLE" },
	{ 0xA85BB0, "Shenzhen Dehoo Technology Co.,Ltd", "SHENZHEN-DEHOO" },
	{ 0xA85BF3, "Audivo GmbH", "AUDIVO" },
	{ 0xA861AA, "Cloudview Limited", "CLOUDVIEW" },
	{ 0xA862A2, "JIWUMEDIA CO., LTD.", "JIWUMEDIA" },
	{ 0xA863DF, "DISPLAIRE CORPORATION", "DISPLAIRE" },
	{ 0xA863F2, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xA86405, "nimbus 9, Inc", "NIMBUS-9" },
	{ 0xA865B2, "DONGGUAN YISHANG ELECTRONIC TECHNOLOGY CO., LIMITED", "DONGGUAN-YISHANG" },
	{ 0xA8667F, "Apple, Inc.", "APPLE" },
	{ 0xA86A6F, "RIM", "RIM" },
	{ 0xA870A5, "UniComm Inc.", "UNICOMM" },
	{ 0xA875D6, "FreeTek International Co., Ltd.", "FREETEK" },
	{ 0xA875E2, "Aventura Technologies, Inc.", "AVENTURA-TECHNOLOGIES" },
	{ 0xA8776F, "Zonoff", "ZONOFF" },
	{ 0xA87B39, "Nokia Corporation", "NOKIA" },
	{ 0xA87C01, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0xA87E33, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0xA88038, "ShenZhen MovingComm Technology Co., Limited", "SHENZHEN-MOVINGCOMM" },
	{ 0xA881F1, "BMEYE B.V.", "BMEYE" },
	{ 0xA8827F, "CIBN Oriental Network(Beijing) CO.,Ltd", "CIBN-ORIENTAL-NETWORK-BEIJING" },
	{ 0xA886DD, "Apple, Inc.", "APPLE" },
	{ 0xA88792, "Broadband Antenna Tracking Systems", "BROADBAND-ANTENNA-TRACKING" },
	{ 0xA887ED, "ARC Wireless LLC", "ARC-WIRELESS-LLC" },
	{ 0xA88808, "Apple", "APPLE" },
	{ 0xA88CEE, "MicroMade Galka i Drozdz sp.j.", "MICROMADE-GALKA-I-DROZDZ-SP-J" },
	{ 0xA88D7B, "SunDroid Global limited.", "SUNDROID-GLOBAL" },
	{ 0xA88E24, "Apple", "APPLE" },
	{ 0xA89008, "Beijing Yuecheng Technology Co. Ltd.", "BEIJING-YUECHENG" },
	{ 0xA8922C, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xA893E6, "JIANGXI JINGGANGSHAN CKING COMMUNICATION TECHNOLOGY CO.,LTD", "JIANGXI-JINGGANGSHAN-CKING-COMMUNICATION" },
	{ 0xA895B0, "Aker Subsea Ltd", "AKER-SUBSEA" },
	{ 0xA8968A, "Apple", "APPLE" },
	{ 0xA897DC, "IBM", "IBM" },
	{ 0xA898C6, "Shinbo Co., Ltd.", "SHINBO" },
	{ 0xA8995C, "aizo ag", "AIZO" },
	{ 0xA89B10, "inMotion Ltd.", "INMOTION" },
	{ 0xA89D21, "Cisco", "CISCO" },
	{ 0xA89DD2, "Shanghai DareGlobal Technologies Co., Ltd", "SHANGHAI-DAREGLOBAL-TECHNOLOGIES" },
	{ 0xA89FBA, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xA8A668, "zte corporation", "ZTE" },
	{ 0xA8AD3D, "Alcatel-Lucent Shanghai Bell Co., Ltd", "ALCATEL-LUCENT-SHANGHAI-BELL" },
	{ 0xA8B0AE, "LEONI", "LEONI" },
	{ 0xA8B1D4, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xA8B9B3, "ESSYS", "ESSYS" },
	{ 0xA8BBCF, "Apple", "APPLE" },
	{ 0xA8BD1A, "Honey Bee (Hong Kong) Limited", "HONEY-BEE-HONG-KONG" },
	{ 0xA8BD3A, "UNIONMAN TECHNOLOGY CO.,LTD", "UNIONMAN" },
	{ 0xA8C222, "TM-Research Inc.", "TM-RESEARCH" },
	{ 0xA8CB95, "EAST BEST CO., LTD.", "EAST-BEST" },
	{ 0xA8CCC5, "Saab AB (publ)", "SAAB-AB-PUBL" },
	{ 0xA8CE90, "CVC", "CVC" },
	{ 0xA8D0E3, "Systech Electronics Ltd.", "SYSTECH-ELECTRONICS" },
	{ 0xA8D0E5, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0xA8D236, "Lightware Visual Engineering", "LIGHTWARE-VISUAL-ENGINEERING" },
	{ 0xA8D3C8, "Wachendorff Elektronik GmbH & Co. KG", "WACHENDORFF-ELEKTRONIK" },
	{ 0xA8D409, "USA 111 Inc", "USA-111" },
	{ 0xA8D88A, "Wyconn", "WYCONN" },
	{ 0xA8E018, "Nokia Corporation", "NOKIA" },
	{ 0xA8E3EE, "Sony Computer Entertainment Inc.", "SONY-ENTERTAINMENT" },
	{ 0xA8E539, "Moimstone Co.,Ltd", "MOIMSTONE" },
	{ 0xA8EF26, "Tritonwave", "TRITONWAVE" },
	{ 0xA8F038, "SHEN ZHEN SHI JIN HUA TAI ELECTRONICS CO.,LTD", "SHEN-ZHEN-SHI-JIN-HUA-TAI-ELECTRONICS" },
	{ 0xA8F274, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xA8F470, "Fujian Newland Communication Science Technologies Co.,Ltd.", "FUJIAN-NEWLAND-COMMUNICATION-SCIENCE-TECHNOLOGIES" },
	{ 0xA8F7E0, "PLANET Technology Corporation", "PLANET" },
	{ 0xA8F94B, "Eltex Enterprise Ltd.", "ELTEX" },
	{ 0xA8FAD8, "Apple", "APPLE" },
	{ 0xA8FB70, "WiseSec L.t.d", "WISESEC-L-T-D" },
	{ 0xA8FCB7, "Consolidated Resource Imaging", "CONSOLIDATED-RESOURCE-IMAGING" },
	{ 0xAA0000, "DIGITAL EQUIPMENT CORPORATION", "DIGITAL-EQUIPMENT" },
	{ 0xAA0001, "DIGITAL EQUIPMENT CORPORATION", "DIGITAL-EQUIPMENT" },
	{ 0xAA0002, "DIGITAL EQUIPMENT CORPORATION", "DIGITAL-EQUIPMENT" },
	{ 0xAA0003, "DIGITAL EQUIPMENT CORPORATION", "DIGITAL-EQUIPMENT" },
	{ 0xAA0004, "DIGITAL EQUIPMENT CORPORATION", "DIGITAL-EQUIPMENT" },
	{ 0xAC0142, "Uriel Technologies SIA", "URIEL-TECHNOLOGIES-SIA" },
	{ 0xAC02CA, "HI Solutions, Inc.", "HI-SOLUTIONS" },
	{ 0xAC02CF, "RW Tecnologia Industria e Comercio Ltda", "RW-TECNOLOGIA-INDUSTRIA-E-COMERCIO-LTDA" },
	{ 0xAC02EF, "Comsis", "COMSIS" },
	{ 0xAC0613, "Senselogix Ltd", "SENSELOGIX" },
	{ 0xAC0A61, "Labor S.r.L.", "LABOR-S-R-L" },
	{ 0xAC0DFE, "Ekon GmbH - myGEKKO", "EKON---MYGEKKO" },
	{ 0xAC11D3, "Suzhou HOTEK Video Technology Co. Ltd", "SUZHOU-HOTEK-VIDEO" },
	{ 0xAC1461, "ATAW Co., Ltd.", "ATAW" },
	{ 0xAC14D2, "wi-daq, inc.", "WI-DAQ" },
	{ 0xAC162D, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xAC1702, "Fibar Group sp. z o.o.", "FIBAR-GROUP-SP-Z-O-O" },
	{ 0xAC1826, "SEIKO EPSON CORPORATION", "SEIKO-EPSON" },
	{ 0xAC199F, "SUNGROW POWER SUPPLY CO.,LTD.", "SUNGROW-POWER-SUPPLY" },
	{ 0xAC20AA, "DMATEK Co., Ltd.", "DMATEK" },
	{ 0xAC220B, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xAC2DA3, "TXTR GmbH", "TXTR" },
	{ 0xAC2FA8, "Humannix Co.,Ltd.", "HUMANNIX" },
	{ 0xAC319D, "Shenzhen TG-NET Botone Technology Co.,Ltd.", "SHENZHEN-TG-NET-BOTONE" },
	{ 0xAC34CB, "Shanhai GBCOM Communication Technology Co. Ltd", "SHANHAI-GBCOM-COMMUNICATION" },
	{ 0xAC3613, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xAC3870, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0xAC3A7A, "Roku", "ROKU" },
	{ 0xAC3C0B, "Apple", "APPLE" },
	{ 0xAC3CB4, "Nilan A/S", "NILAN-A/S" },
	{ 0xAC3D05, "Instorescreen Aisa", "INSTORESCREEN-AISA" },
	{ 0xAC3D75, "HANGZHOU ZHIWAY TECHNOLOGIES CO.,LTD.", "HANGZHOU-ZHIWAY-TECHNOLOGIES" },
	{ 0xAC3FA4, "TAIYO YUDEN CO.,LTD", "TAIYO-YUDEN" },
	{ 0xAC40EA, "C&T Solution Inc.", "C-T-SOLUTION" },
	{ 0xAC4122, "Eclipse Electronic Systems Inc.", "ECLIPSE" },
	{ 0xAC44F2, "Revolabs Inc", "REVOLABS" },
	{ 0xAC4723, "Genelec", "GENELEC" },
	{ 0xAC4AFE, "Hisense Broadband Multimedia Technology Co.,Ltd.", "HISENSE-BROADBAND-MULTIMEDIA" },
	{ 0xAC4BC8, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0xAC4E91, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xAC4FFC, "SVS-VISTEK GmbH", "SVS-VISTEK" },
	{ 0xAC5036, "Pi-Coral Inc", "PI-CORAL" },
	{ 0xAC5135, "MPI TECH", "MPI-TECH" },
	{ 0xAC51EE, "Cambridge Communication Systems Ltd", "CAMBRIDGE-COMMUNICATION" },
	{ 0xAC54EC, "IEEE P1823 Standards Working Group", "IEEE-P1823-STANDARDS-WORKING-GROUP" },
	{ 0xAC562C, "LAVA INTERNATIONAL(H.K) LIMITED", "LAVA-H-K" },
	{ 0xAC583B, "Human Assembler, Inc.", "HUMAN-ASSEMBLER" },
	{ 0xAC5D10, "Pace Americas", "PACE-AMERICAS" },
	{ 0xAC5E8C, "Utillink", "UTILLINK" },
	{ 0xAC6123, "Drivven, Inc.", "DRIVVEN" },
	{ 0xAC6706, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0xAC6BAC, "Jenny Science AG", "JENNY-SCIENCE" },
	{ 0xAC6E1A, "Shenzhen Gongjin Electronics Co.,Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0xAC6F4F, "Enspert Inc", "ENSPERT" },
	{ 0xAC6FBB, "TATUNG Technology Inc.", "TATUNG" },
	{ 0xAC6FD9, "Valueplus Inc.", "VALUEPLUS" },
	{ 0xAC7236, "Lexking Technology Co., Ltd.", "LEXKING" },
	{ 0xAC7289, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xAC7A42, "iConnectivity", "ICONNECTIVITY" },
	{ 0xAC7A4D, "ALPS ELECTRIC CO.,LTD.", "ALPS-ELECTRIC" },
	{ 0xAC7BA1, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xAC7F3E, "Apple", "APPLE" },
	{ 0xAC80D6, "Hexatronic AB", "HEXATRONIC-AB" },
	{ 0xAC8112, "Gemtek Technology Co., Ltd.", "GEMTEK" },
	{ 0xAC81F3, "Nokia Corporation", "NOKIA" },
	{ 0xAC8317, "Shenzhen Furtunetel Communication Co., Ltd", "SHENZHEN-FURTUNETEL-COMMUNICATION" },
	{ 0xAC83F0, "ImmediaTV Corporation", "IMMEDIATV" },
	{ 0xAC853D, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xAC8674, "Open Mesh, Inc.", "OPEN-MESH" },
	{ 0xAC867E, "Create New Technology (HK) Limited Company", "CREATE-NEW-HK-COMPANY" },
	{ 0xAC87A3, "Apple", "APPLE" },
	{ 0xAC8ACD, "ROGER D.Wensker, G.Wensker sp.j.", "ROGER-D-WENSKER-G-WENSKER-SP-J" },
	{ 0xAC8D14, "Smartrove Inc", "SMARTROVE" },
	{ 0xAC932F, "Nokia Corporation", "NOKIA" },
	{ 0xAC9403, "Envision Peripherals Inc", "ENVISION-PERIPHERALS" },
	{ 0xAC9A96, "Lantiq Deutschland GmbH", "LANTIQ-DEUTSCHLAND" },
	{ 0xAC9B0A, "Sony Corporation", "SONY" },
	{ 0xAC9B84, "Smak Tecnologia e Automacao", "SMAK-TECNOLOGIA-E-AUTOMACAO" },
	{ 0xAC9CE4, "Alcatel-Lucent Shanghai Bell Co., Ltd", "ALCATEL-LUCENT-SHANGHAI-BELL" },
	{ 0xAC9E17, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xACA016, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xACA213, "Shenzhen Bilian electronic CO.,LTD", "SHENZHEN-BILIAN" },
	{ 0xACA22C, "Baycity Technologies Ltd", "BAYCITY-TECHNOLOGIES" },
	{ 0xACA31E, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0xACA430, "Peerless AV", "PEERLESS-AV" },
	{ 0xACA919, "TrekStor GmbH", "TREKSTOR" },
	{ 0xACA9A0, "Audioengine, Ltd.", "AUDIOENGINE" },
	{ 0xACAB8D, "Lyngso Marine A/S", "LYNGSO-MARINE-A/S" },
	{ 0xACABBF, "AthenTek Inc.", "ATHENTEK" },
	{ 0xACB313, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xACB57D, "Liteon Technology Corporation", "LITEON" },
	{ 0xACB74F, "METEL s.r.o.", "METEL-S-R-O" },
	{ 0xACB859, "Uniband Electronic Corp,", "UNIBAND" },
	{ 0xACBD0B, "IMAC CO.,LTD", "IMAC" },
	{ 0xACBE75, "Ufine Technologies Co.,Ltd.", "UFINE-TECHNOLOGIES" },
	{ 0xACBEB6, "Visualedge Technology Co., Ltd.", "VISUALEDGE" },
	{ 0xACC2EC, "CLT INT'L IND. CORP.", "CLT-IND" },
	{ 0xACC595, "Graphite Systems", "GRAPHITE" },
	{ 0xACC698, "Kohzu Precision Co., Ltd.", "KOHZU-PRECISION" },
	{ 0xACC73F, "VITSMO CO., LTD.", "VITSMO" },
	{ 0xACC935, "Ness Corporation", "NESS" },
	{ 0xACCA54, "Telldus Technologies AB", "TELLDUS-TECHNOLOGIES-AB" },
	{ 0xACCA8E, "ODA Technologies", "ODA-TECHNOLOGIES" },
	{ 0xACCAAB, "Virtual Electric Inc", "VIRTUAL-ELECTRIC" },
	{ 0xACCABA, "Midokura Co., Ltd.", "MIDOKURA" },
	{ 0xACCB09, "Hefcom Metering (Pty) Ltd", "HEFCOM-METERING-PTY" },
	{ 0xACCC8E, "Axis Communications AB", "AXIS-COMMUNICATION-AB" },
	{ 0xACCE8F, "HWA YAO TECHNOLOGIES CO., LTD", "HWA-YAO-TECHNOLOGIES" },
	{ 0xACCF23, "Hi-flying electronics technology Co.,Ltd", "HI-FLYING-ELECTRONICS" },
	{ 0xACCF5C, "Apple", "APPLE" },
	{ 0xACD074, "Espressif Inc.", "ESPRESSIF" },
	{ 0xACD180, "Crexendo Business Solutions, Inc.", "CREXENDO-BUSINESS-SOLUTIONS" },
	{ 0xACD1B8, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xACD364, "ABB SPA, ABB SACE DIV.", "ABB-SPA-ABB-SACE-DIV" },
	{ 0xACD657, "Shaanxi Guolian Digital TV Technology Co., Ltd.", "SHAANXI-GUOLIAN-DIGITAL-TV" },
	{ 0xACD9D6, "tci GmbH", "TCI" },
	{ 0xACDBDA, "Shenzhen Geniatech Inc, Ltd", "SHENZHEN-GENIATECH" },
	{ 0xACDE48, "PRIVATE", "PRIVATE" },
	{ 0xACE010, "Liteon Technology Corporation", "LITEON" },
	{ 0xACE069, "ISAAC Instruments", "ISAAC-INSTRUMENTS" },
	{ 0xACE215, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xACE348, "MadgeTech, Inc", "MADGETECH" },
	{ 0xACE42E, "SK hynix", "SK-HYNIX" },
	{ 0xACE64B, "Shenzhen Baojia Battery Technology Co., Ltd.", "SHENZHEN-BAOJIA-BATTERY" },
	{ 0xACE87B, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xACE87E, "Bytemark Computer Consulting Ltd", "BYTEMARK-CONSULTING" },
	{ 0xACE97F, "IoT Tech Limited", "IOT-TECH" },
	{ 0xACE9AA, "Hay Systems Ltd", "HAY" },
	{ 0xACEA6A, "GENIX INFOCOMM CO., LTD.", "GENIX-INFOCOMM" },
	{ 0xACEE3B, "6harmonics Inc", "6HARMONICS" },
	{ 0xACF0B2, "Becker Electronics Taiwan Ltd.", "BECKER-ELECTRONICS-TAIWAN" },
	{ 0xACF1DF, "D-Link International", "D-LINK" },
	{ 0xACF2C5, "Cisco", "CISCO" },
	{ 0xACF7F3, "XIAOMI CORPORATION", "XIAOMI" },
	{ 0xACF97E, "ELESYS INC.", "ELESYS" },
	{ 0xACFD93, "Weifang GoerTek Electronics Co., Ltd.", "WEIFANG-GOERTEK-ELECTRONICS" },
	{ 0xACFDCE, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xACFDEC, "Apple, Inc", "APPLE" },
	{ 0xB000B4, "Cisco", "CISCO" },
	{ 0xB00594, "Liteon Technology Corporation", "LITEON" },
	{ 0xB008BF, "Vital Connect, Inc.", "VITAL-CONNECT" },
	{ 0xB009D3, "Avizia", "AVIZIA" },
	{ 0xB01041, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xB01203, "Dynamics Hong Kong Limited", "DYNAMICS-HONG-KONG" },
	{ 0xB01266, "Futaba-Kikaku", "FUTABA-KIKAKU" },
	{ 0xB01408, "LIGHTSPEED INTERNATIONAL CO.", "LIGHTSPEED" },
	{ 0xB01743, "EDISON GLOBAL CIRCUITS LLC", "EDISON-GLOBAL-CIRCUITS-LLC" },
	{ 0xB01B7C, "Ontrol A.S.", "ONTROL-A-S" },
	{ 0xB01C91, "Elim Co", "ELIM" },
	{ 0xB01F81, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xB024F3, "Progeny Systems", "PROGENY" },
	{ 0xB025AA, "PRIVATE", "PRIVATE" },
	{ 0xB03495, "Apple", "APPLE" },
	{ 0xB0358D, "Nokia Corporation", "NOKIA" },
	{ 0xB03829, "Siliconware Precision Industries Co., Ltd.", "SILICONWARE-PRECISION-INDUSTRIES" },
	{ 0xB03850, "Nanjing CAS-ZDC IOT SYSTEM CO.,LTD", "NANJING-CAS-ZDC-IOT-SYSTEM" },
	{ 0xB0435D, "NuLEDs, Inc.", "NULEDS" },
	{ 0xB04515, "mira fitness,LLC.", "MIRA-FITNESS-LLC" },
	{ 0xB04519, "TCT mobile ltd", "TCT-MOBILE" },
	{ 0xB04545, "YACOUB Automation GmbH", "YACOUB-AUTOMATION" },
	{ 0xB046FC, "MitraStar Technology Corp.", "MITRASTAR" },
	{ 0xB0487A, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xB0495F, "OMRON HEALTHCARE Co., Ltd.", "OMRON-HEALTHCARE" },
	{ 0xB04C05, "Fresenius Medical Care Deutschland GmbH", "FRESENIUS-MEDICAL-CARE-DEUTSCHLAND" },
	{ 0xB050BC, "SHENZHEN BASICOM ELECTRONIC CO.,LTD.", "SHENZHEN-BASICOM" },
	{ 0xB0518E, "Holl technology CO.Ltd.", "HOLL" },
	{ 0xB05706, "Vallox Oy", "VALLOX-OY" },
	{ 0xB058C4, "Broadcast Microwave Services, Inc", "BROADCAST-MICROWAVE-SERVICES" },
	{ 0xB05B1F, "THERMO FISHER SCIENTIFIC S.P.A.", "THERMO-FISHER-SCIENTIFIC-S-P-A" },
	{ 0xB05B67, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xB05CE5, "Nokia Corporation", "NOKIA" },
	{ 0xB061C7, "Ericsson-LG Enterprise", "ERICSSON-LG" },
	{ 0xB06563, "Shanghai Railway Communication Factory", "SHANGHAI-RAILWAY-COMMUNICATION-FACTORY" },
	{ 0xB065BD, "Apple", "APPLE" },
	{ 0xB068B6, "Hangzhou OYE Technology Co. Ltd", "HANGZHOU-OYE" },
	{ 0xB06971, "DEI Sales, Inc.", "DEI-SALES" },
	{ 0xB06CBF, "3ality Digital Systems GmbH", "3ALITY-DIGITAL" },
	{ 0xB0750C, "QA Cafe", "QA-CAFE" },
	{ 0xB0754D, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0xB075D5, "ZTE Corporation", "ZTE" },
	{ 0xB077AC, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xB07908, "Cummings Engineering", "CUMMINGS-ENGINEERING" },
	{ 0xB0793C, "Revolv Inc", "REVOLV" },
	{ 0xB07994, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0xB07D62, "Dipl.-Ing. H. Horstmann GmbH", "DIPL--ING-H-HORSTMANN" },
	{ 0xB0808C, "Laser Light Engines", "LASER-LIGHT-ENGINES" },
	{ 0xB081D8, "I-sys Corp", "I-SYS" },
	{ 0xB083FE, "Dell Inc", "DELL" },
	{ 0xB0869E, "Chloride S.r.L", "CHLORIDE-S-R-L" },
	{ 0xB08807, "Strata Worldwide", "STRATA-WORLDWIDE" },
	{ 0xB08991, "LGE", "LGE" },
	{ 0xB08E1A, "URadio Systems Co., Ltd", "URADIO" },
	{ 0xB09074, "Fulan Electronics Limited", "FULAN-ELECTRONICS" },
	{ 0xB09134, "Taleo", "TALEO" },
	{ 0xB09137, "ISis ImageStream Internet Solutions, Inc", "ISIS-IMAGESTREAM-INTERNET-SOLUTIONS" },
	{ 0xB0966C, "Lanbowan Technology Ltd.", "LANBOWAN" },
	{ 0xB0973A, "E-Fuel Corporation", "E-FUEL" },
	{ 0xB0989F, "LG CNS", "LG-CNS" },
	{ 0xB09928, "Fujitsu Limited", "FUJITSU" },
	{ 0xB09AE2, "STEMMER IMAGING GmbH", "STEMMER-IMAGING" },
	{ 0xB09BD4, "GNH Software India Private Limited", "GNH-SOFTWARE-INDIA-PRIVATE" },
	{ 0xB09FBA, "Apple", "APPLE" },
	{ 0xB0A10A, "Pivotal Systems Corporation", "PIVOTAL" },
	{ 0xB0A37E, "Qingdao Haier Electronics Co.,Ltd", "QINGDAO-HAIER-ELECTRONICS" },
	{ 0xB0A72A, "Ensemble Designs, Inc.", "ENSEMBLE-DESIGNS" },
	{ 0xB0A737, "Roku, Inc.", "ROKU" },
	{ 0xB0A86E, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0xB0AA36, "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD.", "GUANGDONG-OPPO-MOBILE-TELECOMMUNICATIONS" },
	{ 0xB0AA77, "Cisco", "CISCO" },
	{ 0xB0ACFA, "Fujitsu Limited", "FUJITSU" },
	{ 0xB0ADAA, "Avaya, Inc", "AVAYA" },
	{ 0xB0B2DC, "Zyxel Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0xB0B32B, "Slican Sp. z o.o.", "SLICAN-SP-Z-O-O" },
	{ 0xB0B448, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xB0B8D5, "Nanjing Nengrui Auto Equipment CO.,Ltd", "NANJING-NENGRUI-AUTO-EQUIPMENT" },
	{ 0xB0BD6D, "Echostreams Innovative Solutions", "ECHOSTREAMS-INNOVATIVE-SOLUTIONS" },
	{ 0xB0BDA1, "ZAKLAD ELEKTRONICZNY SIMS", "ZAKLAD-ELEKTRONICZNY-SIMS" },
	{ 0xB0BF99, "WIZITDONGDO", "WIZITDONGDO" },
	{ 0xB0C287, "Technicolor CH USA Inc", "TECHNICOLOR-CH-USA" },
	{ 0xB0C4E7, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xB0C554, "D-Link International", "D-LINK" },
	{ 0xB0C559, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB0C69A, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0xB0C745, "Buffalo Inc.", "BUFFALO" },
	{ 0xB0C83F, "Jiangsu Cynray IOT Co., Ltd.", "JIANGSU-CYNRAY-IOT" },
	{ 0xB0C8AD, "People Power Company", "PEOPLE-POWER-COMPANY" },
	{ 0xB0C95B, "Beijing Symtech CO.,LTD", "BEIJING-SYMTECH" },
	{ 0xB0CE18, "Zhejiang shenghui lighting co.,Ltd", "ZHEJIANG-SHENGHUI-LIGHTING" },
	{ 0xB0CF4D, "MI-Zone Technology Ireland", "MI-ZONE-IRELAND" },
	{ 0xB0D09C, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB0D2F5, "Vello Systems, Inc.", "VELLO" },
	{ 0xB0D59D, "Shenzhen Zowee Technology Co., Ltd", "SHENZHEN-ZOWEE" },
	{ 0xB0D7C5, "Logipix Ltd", "LOGIPIX" },
	{ 0xB0DA00, "CERA ELECTRONIQUE", "CERA-ELECTRONIQUE" },
	{ 0xB0DF3A, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB0E03C, "TCT mobile ltd", "TCT-MOBILE" },
	{ 0xB0E2E5, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0xB0E39D, "CAT SYSTEM CO.,LTD.", "CAT-SYSTEM" },
	{ 0xB0E50E, "NRG SYSTEMS INC", "NRG" },
	{ 0xB0E754, "2Wire", "2WIRE" },
	{ 0xB0E892, "SEIKO EPSON CORPORATION", "SEIKO-EPSON" },
	{ 0xB0E97E, "Advanced Micro Peripherals", "ADVANCED-MICRO-PERIPHERALS" },
	{ 0xB0EC71, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB0EC8F, "GMX SAS", "GMX-SAS" },
	{ 0xB0ECE1, "PRIVATE", "PRIVATE" },
	{ 0xB0EE45, "AzureWave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0xB0F1BC, "Dhemax Ingenieros Ltda", "DHEMAX-INGENIEROS-LTDA" },
	{ 0xB0FAEB, "Cisco", "CISCO" },
	{ 0xB0FEBD, "PRIVATE", "PRIVATE" },
	{ 0xB4009C, "CableWorld Ltd.", "CABLEWORLD" },
	{ 0xB40142, "GCI Science & Technology Co.,LTD", "GCI-SCIENCE" },
	{ 0xB40418, "Smartchip Integrated Inc.", "SMARTCHIP-INTEGRATED" },
	{ 0xB40566, "SP Best Corporation Co., LTD.", "SP-BEST" },
	{ 0xB407F9, "SAMSUNG ELECTRO-MECHANICS", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xB40832, "TC Communications", "TC-COMMUNICATION" },
	{ 0xB40AC6, "DEXON Systems Ltd.", "DEXON" },
	{ 0xB40B44, "Smartisan Technology Co., Ltd.", "SMARTISAN" },
	{ 0xB40B7A, "Brusa Elektronik AG", "BRUSA-ELEKTRONIK" },
	{ 0xB40C25, "Palo Alto Networks", "PALO-ALTO-NETWORKS" },
	{ 0xB40E96, "HERAN", "HERAN" },
	{ 0xB40EDC, "LG-Ericsson Co.,Ltd.", "LG-ERICSSON" },
	{ 0xB41489, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xB41513, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xB41780, "DTI Group Ltd", "DTI-GROUP" },
	{ 0xB418D1, "Apple", "APPLE" },
	{ 0xB41DEF, "Internet Laboratories, Inc.", "INTERNET-LABORATORIES" },
	{ 0xB4211D, "Beijing GuangXin Technology Co., Ltd", "BEIJING-GUANGXIN" },
	{ 0xB4218A, "Dog Hunter LLC", "DOG-HUNTER-LLC" },
	{ 0xB424E7, "Codetek Technology Co.,Ltd", "CODETEK" },
	{ 0xB428F1, "E-Prime Co., Ltd.", "E-PRIME" },
	{ 0xB42A39, "ORBIT MERRET, spol. s r. o.", "ORBIT-MERRET-SPOL-S-R-O" },
	{ 0xB42C92, "Zhejiang Weirong Electronic Co., Ltd", "ZHEJIANG-WEIRONG" },
	{ 0xB42CBE, "Direct Payment Solutions Limited", "DIRECT-PAYMENT-SOLUTIONS" },
	{ 0xB43052, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xB431B8, "Aviwest", "AVIWEST" },
	{ 0xB4346C, "MATSUNICHI DIGITAL TECHNOLOGY (HONG KONG) LIMITED", "MATSUNICHI-DIGITAL-HONG-KONG" },
	{ 0xB43564, "Fujian Tian Cheng Electron Science & Technical Development Co.,Ltd.", "FUJIAN-TIAN-CHENG-ELECTRON-SCIENCE-TECHNICAL-DEVELOPMENT" },
	{ 0xB435F7, "Zhejiang Pearmain Electronics Co.ltd.", "ZHEJIANG-PEARMAIN-ELECTRONICS" },
	{ 0xB43741, "Consert, Inc.", "CONSERT" },
	{ 0xB43934, "Pen Generations, Inc.", "PEN-GENERATIONS" },
	{ 0xB439D6, "ProCurve Networking by HP", "PROCURVE-NETWORKING-BY-HP" },
	{ 0xB43A28, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB43DB2, "Degreane Horizon", "DEGREANE-HORIZON" },
	{ 0xB43E3B, "Viableware, Inc", "VIABLEWARE" },
	{ 0xB4417A, "ShenZhen Gongjin Electronics Co.,Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0xB4430D, "Broadlink Pty Ltd", "BROADLINK-PTY" },
	{ 0xB4475E, "Avaya, Inc", "AVAYA" },
	{ 0xB44CC2, "NR ELECTRIC CO., LTD", "NR-ELECTRIC" },
	{ 0xB451F9, "NB Software", "NB-SOFTWARE" },
	{ 0xB45253, "Seagate Technology", "SEAGATE" },
	{ 0xB4527D, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0xB4527E, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0xB45570, "Borea", "BOREA" },
	{ 0xB45861, "CRemote, LLC", "CREMOTE-LLC" },
	{ 0xB45CA4, "Thing-talk Wireless Communication Technologies Corporation Limited", "THING-TALK-WIRELESS-COMMUNICATION-TECHNOLOGIES" },
	{ 0xB461FF, "Lumigon A/S", "LUMIGON-A/S" },
	{ 0xB46238, "Exablox", "EXABLOX" },
	{ 0xB46293, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB462AD, "raytest GmbH", "RAYTEST" },
	{ 0xB46698, "Zealabs srl", "ZEALABS-SRL" },
	{ 0xB467E9, "Qingdao GoerTek Technology Co., Ltd.", "QINGDAO-GOERTEK" },
	{ 0xB46D35, "Dalian Seasky Automation Co;Ltd", "DALIAN-SEASKY-AUTOMATION" },
	{ 0xB47356, "Hangzhou Treebear Networking Co., Ltd.", "HANGZHOU-TREEBEAR-NETWORKING" },
	{ 0xB4749F, "askey computer corp", "ASKEY" },
	{ 0xB4750E, "Belkin International Inc.", "BELKIN" },
	{ 0xB479A7, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xB47C29, "Shenzhen Guzidi Technology Co.,Ltd", "SHENZHEN-GUZIDI" },
	{ 0xB47F5E, "Foresight Manufacture (S) Pte Ltd", "FORESIGHT-MANUFACTURE-S-PTE" },
	{ 0xB48255, "Research Products Corporation", "RESEARCH-PRODUCTS" },
	{ 0xB4827B, "AKG Acoustics GmbH", "AKG-ACOUSTICS" },
	{ 0xB482C5, "Relay2, Inc.", "RELAY2" },
	{ 0xB482FE, "Askey Computer Corp", "ASKEY" },
	{ 0xB48547, "Amptown System Company GmbH", "AMPTOWN-SYSTEM-COMPANY" },
	{ 0xB48910, "Coster T.E. S.P.A.", "COSTER-T-E-S-P-A" },
	{ 0xB4944E, "WeTelecom Co., Ltd.", "WETELECOM" },
	{ 0xB49842, "zte corporation", "ZTE" },
	{ 0xB4994C, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xB499BA, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0xB49DB4, "Axion Technologies Inc.", "AXION-TECHNOLOGIES" },
	{ 0xB49EAC, "Imagik Int'l Corp", "IMAGIK" },
	{ 0xB49EE6, "SHENZHEN TECHNOLOGY CO LTD", "SHENZHEN" },
	{ 0xB4A4B5, "Zen Eye Co.,Ltd", "ZEN-EYE" },
	{ 0xB4A4E3, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xB4A5A9, "MODI GmbH", "MODI" },
	{ 0xB4A828, "Shenzhen Concox Information Technology Co., Ltd", "SHENZHEN-CONCOX-INFORMATION" },
	{ 0xB4A82B, "Histar Digital Electronics Co., Ltd.", "HISTAR-DIGITAL-ELECTRONICS" },
	{ 0xB4A95A, "Avaya, Inc", "AVAYA" },
	{ 0xB4A9FE, "GHIA Technology (Shenzhen) LTD", "GHIA-SHENZHEN" },
	{ 0xB4AA4D, "Ensequence, Inc.", "ENSEQUENCE" },
	{ 0xB4AB2C, "MtM Technology Corporation", "MTM" },
	{ 0xB4AE6F, "Circle Reliance, Inc DBA Cranberry Networks", "CIRCLE-RELIANCE-DBA-CRANBERRY-NETWORKS" },
	{ 0xB4B017, "Avaya, Inc", "AVAYA" },
	{ 0xB4B265, "DAEHO I&T", "DAEHO-I-T" },
	{ 0xB4B362, "ZTE Corporation", "ZTE" },
	{ 0xB4B52F, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xB4B542, "Hubbell Power Systems, Inc.", "HUBBELL-POWER" },
	{ 0xB4B5AF, "Minsung Electronics", "MINSUNG-ELECTRONICS" },
	{ 0xB4B676, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xB4B859, "Texa Spa", "TEXA-SPA" },
	{ 0xB4B88D, "Thuh Company", "THUH-COMPANY" },
	{ 0xB4C44E, "VXL eTech Pvt Ltd", "VXL-ETECH-PVT" },
	{ 0xB4C799, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0xB4C810, "UMPI Elettronica", "UMPI-ELETTRONICA" },
	{ 0xB4CCE9, "PROSYST", "PROSYST" },
	{ 0xB4CEF6, "HTC Corporation", "HTC" },
	{ 0xB4CFDB, "Shenzhen Jiuzhou Electric Co.,LTD", "SHENZHEN-JIUZHOU-ELECTRIC" },
	{ 0xB4D8A9, "BetterBots", "BETTERBOTS" },
	{ 0xB4D8DE, "iota Computing, Inc.", "IOTA-COMPUTING" },
	{ 0xB4DD15, "ControlThings Oy Ab", "CONTROLTHINGS-OY-AB" },
	{ 0xB4DF3B, "Chromlech", "CHROMLECH" },
	{ 0xB4DFFA, "Litemax Electronics Inc.", "LITEMAX-ELECTRONICS" },
	{ 0xB4E0CD, "Fusion-io, Inc", "FUSION-IO" },
	{ 0xB4E1EB, "PRIVATE", "PRIVATE" },
	{ 0xB4E9B0, "Cisco", "CISCO" },
	{ 0xB4ED19, "Pie Digital, Inc.", "PIE-DIGITAL" },
	{ 0xB4ED54, "Wohler Technologies", "WOHLER-TECHNOLOGIES" },
	{ 0xB4EEB4, "ASKEY COMPUTER CORP", "ASKEY" },
	{ 0xB4EED4, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xB4EF39, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB4F0AB, "Apple", "APPLE" },
	{ 0xB4F2E8, "Pace plc", "PACE-PLC" },
	{ 0xB4F323, "PETATEL INC.", "PETATEL" },
	{ 0xB4FC75, "SEMA Electronics(HK) CO.,LTD", "SEMA-ELECTRONICS-HK" },
	{ 0xB4FE8C, "Centro Sicurezza Italia SpA", "CENTRO-SICUREZZA-ITALIA-SPA" },
	{ 0xB80305, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xB80415, "Bayan Audio", "BAYAN-AUDIO" },
	{ 0xB808CF, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xB8098A, "Apple", "APPLE" },
	{ 0xB80B9D, "ROPEX Industrie-Elektronik GmbH", "ROPEX-INDUSTRIE-ELEKTRONIK" },
	{ 0xB81413, "Keen High Holding(HK) Ltd.", "KEEN-HIGH-HOLDING-HK" },
	{ 0xB81619, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xB817C2, "Apple", "APPLE" },
	{ 0xB8186F, "ORIENTAL MOTOR CO., LTD.", "ORIENTAL-MOTOR" },
	{ 0xB81999, "Nesys", "NESYS" },
	{ 0xB820E7, "Guangzhou Horizontal Information & Network Integration Co. Ltd", "GUANGZHOU-HORIZONTAL-INFORMATION-NETWORK-INTEGRATION" },
	{ 0xB82410, "Magneti Marelli Slovakia s.r.o.", "MAGNETI-MARELLI-SLOVAKIA-S-R-O" },
	{ 0xB8241A, "SWEDA INFORMATICA LTDA", "SWEDA-INFORMATICA-LTDA" },
	{ 0xB8266C, "ANOV France", "ANOV-FRANCE" },
	{ 0xB826D4, "Furukawa Industrial S.A. Produtos Elétricos", "FURUKAWA-INDUSTRIAL-PRODUTOS-ELéTRICOS" },
	{ 0xB827EB, "Raspberry Pi Foundation", "RASPBERRY-PI-FOUNDATION" },
	{ 0xB8288B, "Parker Hannifin", "PARKER-HANNIFIN" },
	{ 0xB829F7, "Blaster Tech", "BLASTER-TECH" },
	{ 0xB82A72, "Dell Inc", "DELL" },
	{ 0xB82ADC, "EFR Europäische Funk-Rundsteuerung GmbH", "EFR-EUROPäISCHE-FUNK-RUNDSTEUERUNG" },
	{ 0xB82CA0, "Honeywell HomMed", "HONEYWELL-HOMMED" },
	{ 0xB830A8, "Road-Track Telematics Development", "ROAD-TRACK-TELEMATICS-DEVELOPMENT" },
	{ 0xB836D8, "Videoswitch", "VIDEOSWITCH" },
	{ 0xB83861, "Cisco", "CISCO" },
	{ 0xB838CA, "Kyokko Tsushin System CO.,LTD", "KYOKKO-TSUSHIN-SYSTEM" },
	{ 0xB83A7B, "Worldplay (Canada) Inc.", "WORLDPLAY-CANADA" },
	{ 0xB83D4E, "Shenzhen Cultraview Digital Technology Co.,Ltd Shanghai Branch", "SHENZHEN-CULTRAVIEW-DIGITAL-SHANGHAI-BRANCH" },
	{ 0xB83E59, "Roku, Inc", "ROKU" },
	{ 0xB8415F, "ASP AG", "ASP" },
	{ 0xB843E4, "Vlatacom", "VLATACOM" },
	{ 0xB847C6, "SanJet Technology Corp.", "SANJET" },
	{ 0xB84FD5, "Microsoft Corporation", "MICROSOFT" },
	{ 0xB85510, "Zioncom Electronics (Shenzhen) Ltd.", "ZIONCOM-ELECTRONICS-SHENZHEN" },
	{ 0xB856BD, "ITT LLC", "ITT-LLC" },
	{ 0xB85810, "NUMERA, INC.", "NUMERA" },
	{ 0xB85A73, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB85AF7, "Ouya, Inc", "OUYA" },
	{ 0xB85AFE, "Handaer Communication Technology (Beijing) Co., Ltd", "HANDAER-COMMUNICATION-BEIJING" },
	{ 0xB85E7B, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB86091, "Onnet Technologies and Innovations LLC", "ONNET-TECHNOLOGIES-AND-INNOVATIONS-LLC" },
	{ 0xB8616F, "Accton Wireless Broadband(AWB), Corp.", "ACCTON-WIRELESS-BROADBAND-AWB" },
	{ 0xB8621F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xB863BC, "ROBOTIS, Co, Ltd", "ROBOTIS" },
	{ 0xB86491, "CK Telecom Ltd", "CK-TELECOM" },
	{ 0xB8653B, "Bolymin, Inc.", "BOLYMIN" },
	{ 0xB86B23, "Toshiba", "TOSHIBA" },
	{ 0xB86CE8, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB870F4, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0xB87424, "Viessmann Elektronik GmbH", "VIESSMANN-ELEKTRONIK" },
	{ 0xB87447, "Convergence Technologies", "CONVERGENCE-TECHNOLOGIES" },
	{ 0xB875C0, "PayPal, Inc.", "PAYPAL" },
	{ 0xB8763F, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xB877C3, "Decagon Devices, Inc.", "DECAGON-DEVICES" },
	{ 0xB8782E, "Apple", "APPLE" },
	{ 0xB87879, "Roche Diagnostics GmbH", "ROCHE-DIAGNOSTICS" },
	{ 0xB8797E, "Secure Meters (UK) Limited", "SECURE-METERS-UK" },
	{ 0xB87AC9, "Siemens Ltd.", "SIEMENS" },
	{ 0xB87CF2, "Aerohive Networks Inc.", "AEROHIVE-NETWORKS" },
	{ 0xB8871E, "Good Mind Industries Co., Ltd.", "GOOD-MIND-INDUSTRIES" },
	{ 0xB887A8, "Step Ahead Innovations Inc.", "STEP-AHEAD-INNOVATIONS" },
	{ 0xB888E3, "COMPAL INFORMATION (KUNSHAN) CO., LTD", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0xB889CA, "ILJIN ELECTRIC Co., Ltd.", "ILJIN-ELECTRIC" },
	{ 0xB88A60, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xB88D12, "Apple", "APPLE" },
	{ 0xB88E3A, "Infinite Technologies JLT", "INFINITE-TECHNOLOGIES-JLT" },
	{ 0xB88EC6, "Stateless Networks", "STATELESS-NETWORKS" },
	{ 0xB88F14, "Analytica GmbH", "ANALYTICA" },
	{ 0xB8921D, "BG T&A", "BG-T-A" },
	{ 0xB894D2, "Retail Innovation HTT AB", "RETAIL-INNOVATION-HTT-AB" },
	{ 0xB89674, "AllDSP GmbH & Co. KG", "ALLDSP" },
	{ 0xB8975A, "BIOSTAR Microtech Int'l Corp.", "BIOSTAR-MICROTECH" },
	{ 0xB898B0, "Atlona Inc.", "ATLONA" },
	{ 0xB898F7, "Gionee Communication Equipment Co,Ltd.ShenZhen", "GIONEE-COMMUNICATION-EQUIPMENT-SHENZHEN" },
	{ 0xB89919, "7signal Solutions, Inc", "7SIGNAL-SOLUTIONS" },
	{ 0xB89ACD, "ELITE OPTOELECTRONIC(ASIA)CO.,LTD", "ELITE-OPTOELECTRONIC-ASIA" },
	{ 0xB89AED, "OceanServer Technology, Inc", "OCEANSERVER" },
	{ 0xB89BC9, "SMC Networks Inc", "SMC-NETWORKS" },
	{ 0xB89BE4, "ABB Power Systems Power Generation", "ABB-POWER-POWER-GENERATION" },
	{ 0xB8A386, "D-Link International", "D-LINK" },
	{ 0xB8A3E0, "BenRui Technology Co.,Ltd", "BENRUI" },
	{ 0xB8A8AF, "Logic S.p.A.", "LOGIC-S-P-A" },
	{ 0xB8AC6F, "Dell Inc", "DELL" },
	{ 0xB8AD3E, "BLUECOM", "BLUECOM" },
	{ 0xB8AE6E, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0xB8AEED, "Elitegroup Computer Systems Co., Ltd.", "ELITEGROUP" },
	{ 0xB8AF67, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0xB8B1C7, "BT&COM CO.,LTD", "BT-COM" },
	{ 0xB8B42E, "Gionee Communication Equipment Co,Ltd.ShenZhen", "GIONEE-COMMUNICATION-EQUIPMENT-SHENZHEN" },
	{ 0xB8B7D7, "2GIG Technologies", "2GIG-TECHNOLOGIES" },
	{ 0xB8B81E, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xB8B94E, "Shenzhen iBaby Labs, Inc.", "SHENZHEN-IBABY-LABS" },
	{ 0xB8BA68, "Xi'an Jizhong Digital Communication Co.,Ltd", "XI-AN-JIZHONG-DIGITAL-COMMUNICATION" },
	{ 0xB8BA72, "Cynove", "CYNOVE" },
	{ 0xB8BB6D, "ENERES Co.,Ltd.", "ENERES" },
	{ 0xB8BC1B, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xB8BD79, "TrendPoint Systems", "TRENDPOINT" },
	{ 0xB8BEBF, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xB8BF83, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xB8C1A2, "Dragon Path Technologies Co., Limited", "DRAGON-PATH-TECHNOLOGIES" },
	{ 0xB8C3BF, "Henan Chengshi NetWork Technology Co.，Ltd", "HENAN-CHENGSHI-NETWORK-，" },
	{ 0xB8C46F, "PRIMMCON INDUSTRIES INC", "PRIMMCON-INDUSTRIES" },
	{ 0xB8C68E, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xB8C716, "Fiberhome Telecommunication Technologies Co.,LTD", "FIBERHOME-TELECOMMUNICATION-TECHNOLOGIES" },
	{ 0xB8C75D, "Apple", "APPLE" },
	{ 0xB8C855, "Shanghai GBCOM Communication Technology Co.,Ltd.", "SHANGHAI-GBCOM-COMMUNICATION" },
	{ 0xB8CA3A, "Dell Inc", "DELL" },
	{ 0xB8CD93, "Penetek, Inc", "PENETEK" },
	{ 0xB8CDA7, "Maxeler Technologies Ltd.", "MAXELER-TECHNOLOGIES" },
	{ 0xB8D06F, "GUANGZHOU HKUST FOK YING TUNG RESEARCH INSTITUTE", "GUANGZHOU-HKUST-FOK-YING-TUNG-RESEARCH-INSTITUTE" },
	{ 0xB8D49D, "M Seven System Ltd.", "M-SEVEN-SYSTEM" },
	{ 0xB8D812, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xB8D9CE, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xB8DAF1, "Strahlenschutz- Entwicklungs- und Ausruestungsgesellschaft mbH", "STRAHLENSCHUTZ--ENTWICKLUNGS--UND-AUSRUESTUNGSGESELLSCHAFT-MBH" },
	{ 0xB8DAF7, "Advanced Photonics, Inc.", "ADVANCED-PHOTONICS" },
	{ 0xB8DC87, "IAI Corporation", "IAI" },
	{ 0xB8DF6B, "SpotCam Co., Ltd.", "SPOTCAM" },
	{ 0xB8E589, "Payter BV", "PAYTER" },
	{ 0xB8E625, "2Wire", "2WIRE" },
	{ 0xB8E779, "9Solutions Oy", "9SOLUTIONS-OY" },
	{ 0xB8E856, "Apple", "APPLE" },
	{ 0xB8E937, "Sonos, Inc.", "SONOS" },
	{ 0xB8EE65, "Liteon Technology Corporation", "LITEON" },
	{ 0xB8EE79, "YWire Technologies, Inc.", "YWIRE-TECHNOLOGIES" },
	{ 0xB8F080, "SPS, INC.", "SPS" },
	{ 0xB8F317, "iSun Smasher Communications Private Limited", "ISUN-SMASHER-COMMUNICATION-PRIVATE" },
	{ 0xB8F4D0, "Herrmann Ultraschalltechnik GmbH & Co. Kg", "HERRMANN-ULTRASCHALLTECHNIK" },
	{ 0xB8F5E7, "WayTools, LLC", "WAYTOOLS-LLC" },
	{ 0xB8F6B1, "Apple", "APPLE" },
	{ 0xB8F732, "Aryaka Networks Inc", "ARYAKA-NETWORKS" },
	{ 0xB8F828, "Changshu Gaoshida Optoelectronic Technology Co. Ltd.", "CHANGSHU-GAOSHIDA-OPTOELECTRONIC" },
	{ 0xB8F934, "Sony Ericsson Mobile Communications AB", "SONY-ERICSSON-MOBILE-COMMUNICATION-AB" },
	{ 0xB8FD32, "Zhejiang ROICX Microelectronics", "ZHEJIANG-ROICX-MICROELECTRONICS" },
	{ 0xB8FF61, "Apple", "APPLE" },
	{ 0xB8FF6F, "Shanghai Typrotech Technology Co.Ltd", "SHANGHAI-TYPROTECH" },
	{ 0xB8FFFE, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xBC0200, "Stewart Audio", "STEWART-AUDIO" },
	{ 0xBC0543, "AVM GmbH", "AVM" },
	{ 0xBC0DA5, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xBC0F2B, "FORTUNE TECHGROUP CO.,LTD", "FORTUNE-TECHGROUP" },
	{ 0xBC0F64, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xBC125E, "Beijing WisVideo INC.", "BEIJING-WISVIDEO" },
	{ 0xBC1401, "Hitron Technologies. Inc", "HITRON-TECHNOLOGIES" },
	{ 0xBC1485, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xBC14EF, "ITON Technology Limited", "ITON" },
	{ 0xBC15A6, "Taiwan Jantek Electronics,Ltd.", "TAIWAN-JANTEK-ELECTRONICS" },
	{ 0xBC1665, "Cisco", "CISCO" },
	{ 0xBC16F5, "Cisco", "CISCO" },
	{ 0xBC1A67, "YF Technology Co., Ltd", "YF" },
	{ 0xBC20A4, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xBC20BA, "Inspur (Shandong) Electronic Information Co., Ltd", "INSPUR-SHANDONG-INFORMATION" },
	{ 0xBC25E0, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xBC25F0, "3D Display Technologies Co., Ltd.", "3D-DISPLAY-TECHNOLOGIES" },
	{ 0xBC261D, "HONG KONG TECON TECHNOLOGY", "HONG-KONG-TECON" },
	{ 0xBC2846, "NextBIT Computing Pvt. Ltd.", "NEXTBIT-COMPUTING-PVT" },
	{ 0xBC28D6, "Rowley Associates Limited", "ROWLEY-ASSOCIATES" },
	{ 0xBC2B6B, "Beijing Haier IC Design Co.,Ltd", "BEIJING-HAIER-IC-DESIGN" },
	{ 0xBC2BD7, "Revogi Innovation Co., Ltd.", "REVOGI-INNOVATION" },
	{ 0xBC2C55, "Bear Flag Design, Inc.", "BEAR-FLAG-DESIGN" },
	{ 0xBC2D98, "ThinGlobal LLC", "THINGLOBAL-LLC" },
	{ 0xBC305B, "Dell Inc.", "DELL" },
	{ 0xBC307D, "Wistron Neweb Corp.", "WISTRON-NEWEB" },
	{ 0xBC3400, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xBC35E5, "Hydro Systems Company", "HYDRO-COMPANY" },
	{ 0xBC38D2, "Pandachip Limited", "PANDACHIP" },
	{ 0xBC39A6, "CSUN System Technology Co.,LTD", "CSUN-SYSTEM" },
	{ 0xBC3AEA, "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD.", "GUANGDONG-OPPO-MOBILE-TELECOMMUNICATIONS" },
	{ 0xBC3BAF, "Apple", "APPLE" },
	{ 0xBC3E13, "Accordance Systems Inc.", "ACCORDANCE" },
	{ 0xBC4100, "Codaco Electronic s.r.o.", "CODACO-S-R-O" },
	{ 0xBC4377, "Hang Zhou Huite Technology Co.,ltd.", "HANG-ZHOU-HUITE" },
	{ 0xBC4486, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xBC4760, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xBC4B79, "SensingTek", "SENSINGTEK" },
	{ 0xBC4CC4, "Apple", "APPLE" },
	{ 0xBC4DFB, "Hitron Technologies. Inc", "HITRON-TECHNOLOGIES" },
	{ 0xBC4E3C, "CORE STAFF CO., LTD.", "CORE-STAFF" },
	{ 0xBC4E5D, "ZhongMiao Technology Co., Ltd.", "ZHONGMIAO" },
	{ 0xBC51FE, "Swann Communications Pty Ltd", "SWANN-COMMUNICATION-PTY" },
	{ 0xBC52B4, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0xBC52B7, "Apple", "APPLE" },
	{ 0xBC54F9, "Drogoo Technology Co., Ltd.", "DROGOO" },
	{ 0xBC5C4C, "ELECOM CO.,LTD.", "ELECOM" },
	{ 0xBC5FF4, "ASRock Incorporation", "ASROCK-INCORPORATION" },
	{ 0xBC6010, "Qingdao Hisense Communications Co.,Ltd", "QINGDAO-HISENSE-COMMUNICATION" },
	{ 0xBC629F, "Telenet Systems P. Ltd.", "TELENET-P" },
	{ 0xBC6641, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xBC671C, "Cisco", "CISCO" },
	{ 0xBC6778, "Apple", "APPLE" },
	{ 0xBC6784, "Environics Oy", "ENVIRONICS-OY" },
	{ 0xBC6A16, "tdvine", "TDVINE" },
	{ 0xBC6A29, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xBC6A2F, "Henge Docks LLC", "HENGE-DOCKS-LLC" },
	{ 0xBC6B4D, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0xBC6E64, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0xBC6E76, "Green Energy Options Ltd", "GREEN-ENERGY-OPTIONS" },
	{ 0xBC71C1, "XTrillion, Inc.", "XTRILLION" },
	{ 0xBC72B1, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xBC74D7, "HangZhou JuRu Technology CO.,LTD", "HANGZHOU-JURU" },
	{ 0xBC764E, "Rackspace US, Inc.", "RACKSPACE-US" },
	{ 0xBC7670, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0xBC7737, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xBC779F, "SBM Co., Ltd.", "SBM" },
	{ 0xBC79AD, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xBC7DD1, "Radio Data Comms", "RADIO-DATA-COMMS" },
	{ 0xBC811F, "Ingate Systems", "INGATE" },
	{ 0xBC8199, "BASIC Co.,Ltd.", "BASIC" },
	{ 0xBC83A7, "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LT", "SHENZHEN-CHUANGWEI-RGB-ELECTRONICS-LT" },
	{ 0xBC851F, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xBC8556, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xBC8893, "VILLBAU Ltd.", "VILLBAU" },
	{ 0xBC8B55, "NPP ELIKS America Inc. DBA T&M Atlantic", "NPP-ELIKS-AMERICA-DBA-T-M-ATLANTIC" },
	{ 0xBC8CCD, "Samsung Electro Mechanics co.,LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xBC8D0E, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0xBC926B, "Apple", "APPLE" },
	{ 0xBC9680, "Shenzhen Gongjin Electronics Co.,Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0xBC9889, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0xBC99BC, "FonSee Technology Inc.", "FONSEE" },
	{ 0xBC9CC5, "Beijing Huafei Technology Co., Ltd.", "BEIJING-HUAFEI" },
	{ 0xBC9DA5, "DASCOM Europe GmbH", "DASCOM-EUROPE" },
	{ 0xBCA4E1, "Nabto", "NABTO" },
	{ 0xBCA9D6, "Cyber-Rain, Inc.", "CYBER-RAIN" },
	{ 0xBCAEC5, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xBCB181, "SHARP CORPORATION", "SHARP" },
	{ 0xBCB1F3, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xBCB308, "HONGKONG RAGENTEK COMMUNICATION TECHNOLOGY CO.,LIMITED", "HONGKONG-RAGENTEK-COMMUNICATION" },
	{ 0xBCB852, "Cybera, Inc.", "CYBERA" },
	{ 0xBCBAE1, "AREC Inc.", "AREC" },
	{ 0xBCBBC9, "Kellendonk Elektronik GmbH", "KELLENDONK-ELEKTRONIK" },
	{ 0xBCBC46, "SKS Welding Systems GmbH", "SKS-WELDING" },
	{ 0xBCC168, "DinBox Sverige AB", "DINBOX-SVERIGE-AB" },
	{ 0xBCC23A, "Thomson Video Networks", "THOMSON-VIDEO-NETWORKS" },
	{ 0xBCC342, "Panasonic System Networks Co., Ltd.", "PANASONIC-SYSTEM-NETWORKS" },
	{ 0xBCC493, "Cisco", "CISCO" },
	{ 0xBCC61A, "SPECTRA EMBEDDED SYSTEMS", "SPECTRA-EMBEDDED" },
	{ 0xBCC6DB, "Nokia Corporation", "NOKIA" },
	{ 0xBCC810, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0xBCCAB5, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xBCCD45, "VOISMART", "VOISMART" },
	{ 0xBCCFCC, "HTC Corporation", "HTC" },
	{ 0xBCD165, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0xBCD177, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xBCD1D3, "Tinno Mobile Technology Corp", "TINNO-MOBILE" },
	{ 0xBCD5B6, "d2d technologies", "D2D-TECHNOLOGIES" },
	{ 0xBCD940, "ASR Co,.Ltd.", "ASR" },
	{ 0xBCE09D, "Eoslink", "EOSLINK" },
	{ 0xBCE59F, "WATERWORLD Technology Co.,LTD", "WATERWORLD" },
	{ 0xBCE767, "Quanzhou TDX Electronics Co., Ltd", "QUANZHOU-TDX-ELECTRONICS" },
	{ 0xBCEA2B, "CityCom GmbH", "CITYCOM" },
	{ 0xBCEAFA, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xBCEC23, "SHENZHEN CHUANGWEI-RGB ELECTRONICS CO.,LTD", "SHENZHEN-CHUANGWEI-RGB-ELECTRONICS" },
	{ 0xBCEE7B, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xBCF1F2, "Cisco", "CISCO" },
	{ 0xBCF2AF, "devolo AG", "DEVOLO" },
	{ 0xBCF5AC, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xBCF61C, "Geomodeling Wuxi Technology Co. Ltd.", "GEOMODELING-WUXI" },
	{ 0xBCF685, "D-Link International", "D-LINK" },
	{ 0xBCF811, "Xiamen DNAKE Technology Co.,Ltd", "XIAMEN-DNAKE" },
	{ 0xBCFE8C, "Altronic, LLC", "ALTRONIC-LLC" },
	{ 0xBCFFAC, "TOPCON CORPORATION", "TOPCON" },
	{ 0xC005C2, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xC00D7E, "Additech, Inc.", "ADDITECH" },
	{ 0xC011A6, "Fort-Telecom ltd.", "FORT-TELECOM" },
	{ 0xC01242, "Alpha Security Products", "ALPHA-SECURITY-PRODUCTS" },
	{ 0xC0143D, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xC01885, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xC01ADA, "Apple", "APPLE" },
	{ 0xC01E9B, "Pixavi AS", "PIXAVI-AS" },
	{ 0xC02250, "PRIVATE", "PRIVATE" },
	{ 0xC02506, "AVM GmbH", "AVM" },
	{ 0xC0255C, "Cisco", "CISCO" },
	{ 0xC02567, "Nexxt Solutions", "NEXXT-SOLUTIONS" },
	{ 0xC027B9, "Beijing National Railway Research & Design Institute of Signal & Communication Co., Ltd.", "BEIJING-NATIONAL-RAILWAY-RESEARCH-DESIGN-INSTITUTE-OF-SIGNAL-COMMUNICATION" },
	{ 0xC02973, "Audyssey Laboratories Inc.", "AUDYSSEY-LABORATORIES" },
	{ 0xC029F3, "XySystem", "XYSYSTEM" },
	{ 0xC02BFC, "iNES. applied informatics GmbH", "INES-APPLIED-INFORMATICS" },
	{ 0xC02C7A, "Shen Zhen Horn audio Co., Ltd.", "SHEN-ZHEN-HORN-AUDIO" },
	{ 0xC0335E, "Microsoft", "MICROSOFT" },
	{ 0xC034B4, "Gigastone Corporation", "GIGASTONE" },
	{ 0xC03580, "A&R TECH", "A-R-TECH" },
	{ 0xC035BD, "Velocytech Aps", "VELOCYTECH-APS" },
	{ 0xC035C5, "Prosoft Systems LTD", "PROSOFT" },
	{ 0xC03896, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xC038F9, "Nokia Danmark A/S", "NOKIA-DANMARK-A/S" },
	{ 0xC03B8F, "Minicom Digital Signage", "MINICOM-DIGITAL-SIGNAGE" },
	{ 0xC03D46, "Shanghai Mochui Network Technology Co., Ltd", "SHANGHAI-MOCHUI-NETWORK" },
	{ 0xC03E0F, "BSkyB Ltd", "BSKYB" },
	{ 0xC03F0E, "NETGEAR", "NETGEAR" },
	{ 0xC03F2A, "Biscotti, Inc.", "BISCOTTI" },
	{ 0xC03FD5, "Elitegroup Computer Systems Co., LTD", "ELITEGROUP" },
	{ 0xC041F6, "LG Electronics Inc", "LG-ELECTRONICS" },
	{ 0xC04301, "Epec Oy", "EPEC-OY" },
	{ 0xC044E3, "Shenzhen Sinkna Electronics Co., LTD", "SHENZHEN-SINKNA-ELECTRONICS" },
	{ 0xC0493D, "MAITRISE TECHNOLOGIQUE", "MAITRISE-TECHNOLOGIQUE" },
	{ 0xC04A00, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xC04DF7, "SERELEC", "SERELEC" },
	{ 0xC05627, "Belkin International, Inc.", "BELKIN" },
	{ 0xC056E3, "Hangzhou Hikvision Digital Technology Co.,Ltd.", "HANGZHOU-HIKVISION-DIGITAL" },
	{ 0xC057BC, "Avaya, Inc", "AVAYA" },
	{ 0xC058A7, "Pico Systems Co., Ltd.", "PICO" },
	{ 0xC05E6F, "V. Stonkaus firma \"Kodinis Raktas\"", "V-STONKAUS-FIRMA-KODINIS-RAKTAS" },
	{ 0xC05E79, "SHENZHEN HUAXUN ARK TECHNOLOGIES CO.,LTD", "SHENZHEN-HUAXUN-ARK-TECHNOLOGIES" },
	{ 0xC06118, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xC0626B, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xC06394, "Apple", "APPLE" },
	{ 0xC064C6, "Nokia Corporation", "NOKIA" },
	{ 0xC06599, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xC067AF, "Cisco", "CISCO" },
	{ 0xC06C0F, "Dobbs Stanford", "DOBBS-STANFORD" },
	{ 0xC06C6D, "MagneMotion, Inc.", "MAGNEMOTION" },
	{ 0xC07009, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xC07BBC, "Cisco", "CISCO" },
	{ 0xC07E40, "SHENZHEN XDK COMMUNICATION EQUIPMENT CO.,LTD", "SHENZHEN-XDK-COMMUNICATION-EQUIPMENT" },
	{ 0xC08170, "Effigis GeoSolutions", "EFFIGIS-GEOSOLUTIONS" },
	{ 0xC0830A, "2Wire", "2WIRE" },
	{ 0xC0847A, "Apple", "APPLE" },
	{ 0xC0885B, "SnD Tech Co., Ltd.", "SND-TECH" },
	{ 0xC08ADE, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0xC08B6F, "S I Sistemas Inteligentes Eletrônicos Ltda", "S-I-SISTEMAS-INTELIGENTES-ELETRôNICOS-LTDA" },
	{ 0xC08C60, "Cisco", "CISCO" },
	{ 0xC09132, "Patriot Memory", "PATRIOT-MEMORY" },
	{ 0xC09134, "ProCurve Networking by HP", "PROCURVE-NETWORKING-BY-HP" },
	{ 0xC09879, "Acer Inc.", "ACER" },
	{ 0xC098E5, "University of Michigan", "UNIVERSITY-OF-MICHIGAN" },
	{ 0xC09A71, "XIAMEN MEITU MOBILE TECHNOLOGY CO.LTD", "XIAMEN-MEITU-MOBILE" },
	{ 0xC09C92, "COBY", "COBY" },
	{ 0xC09D26, "Topicon HK Lmd.", "TOPICON-HK-LMD" },
	{ 0xC09F42, "Apple", "APPLE" },
	{ 0xC0A0BB, "D-Link International", "D-LINK" },
	{ 0xC0A0C7, "FAIRFIELD INDUSTRIES", "FAIRFIELD-INDUSTRIES" },
	{ 0xC0A0DE, "Multi Touch Oy", "MULTI-TOUCH-OY" },
	{ 0xC0A0E2, "Eden Innovations", "EDEN-INNOVATIONS" },
	{ 0xC0A26D, "Abbott Point of Care", "ABBOTT-POINT-OF-CARE" },
	{ 0xC0A364, "3D Systems Massachusetts", "3D-MASSACHUSETTS" },
	{ 0xC0A39E, "EarthCam, Inc.", "EARTHCAM" },
	{ 0xC0AA68, "OSASI Technos Inc.", "OSASI-TECHNOS" },
	{ 0xC0AC54, "SAGEMCOM", "SAGEMCOM" },
	{ 0xC0B339, "Comigo Ltd.", "COMIGO" },
	{ 0xC0B357, "Yoshiki Electronics Industry Ltd.", "YOSHIKI-ELECTRONICS-INDUSTRY" },
	{ 0xC0B8B1, "BitBox Ltd", "BITBOX" },
	{ 0xC0BAE6, "Application Solutions (Electronics and Vision) Ltd", "APPLICATION-SOLUTIONS-ELECTRONICS-AND-VISION" },
	{ 0xC0BD42, "ZPA Smart Energy a.s.", "ZPA-SMART-ENERGY-A-S" },
	{ 0xC0BDD1, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xC0C1C0, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0xC0C3B6, "Automatic Systems", "AUTOMATIC" },
	{ 0xC0C520, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0xC0C569, "SHANGHAI LYNUC CNC TECHNOLOGY CO.,LTD", "SHANGHAI-LYNUC-CNC" },
	{ 0xC0C687, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0xC0C946, "MITSUYA LABORATORIES INC.", "MITSUYA-LABORATORIES" },
	{ 0xC0CB38, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xC0CECD, "Apple", "APPLE" },
	{ 0xC0CFA3, "Creative Electronics & Software, Inc.", "CREATIVE-ELECTRONICS-SOFTWARE" },
	{ 0xC0D044, "SAGEMCOM", "SAGEMCOM" },
	{ 0xC0D962, "Askey Computer Corp.", "ASKEY" },
	{ 0xC0DA74, "Hangzhou Sunyard Technology Co., Ltd.", "HANGZHOU-SUNYARD" },
	{ 0xC0DC6A, "Qingdao Eastsoft Communication Technology Co.,LTD", "QINGDAO-EASTSOFT-COMMUNICATION" },
	{ 0xC0DF77, "Conrad Electronic SE", "CONRAD-SE" },
	{ 0xC0E422, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xC0E54E, "DENX Computer Systems GmbH", "DENX" },
	{ 0xC0EAE4, "Sonicwall", "SONICWALL" },
	{ 0xC0EEFB, "OnePlus Tech (Shenzhen) Ltd", "ONEPLUS-TECH-SHENZHEN" },
	{ 0xC0F1C4, "Pacidal Corporation Ltd.", "PACIDAL" },
	{ 0xC0F2FB, "Apple", "APPLE" },
	{ 0xC0F79D, "Powercode", "POWERCODE" },
	{ 0xC0F8DA, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xC0F991, "GME Standard Communications P/L", "GME-STANDARD-COMMUNICATION-P/L" },
	{ 0xC0FFD4, "Netgear Inc", "NETGEAR" },
	{ 0xC40006, "Lipi Data Systems Ltd.", "LIPI-DATA" },
	{ 0xC40142, "MaxMedia Technology Limited", "MAXMEDIA" },
	{ 0xC4017C, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0xC401B1, "SeekTech INC", "SEEKTECH" },
	{ 0xC401CE, "PRESITION (2000) CO., LTD.", "PRESITION-2000" },
	{ 0xC40415, "NETGEAR INC.,", "NETGEAR" },
	{ 0xC40528, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xC4072F, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xC4084A, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0xC40880, "Shenzhen UTEPO Tech Co., Ltd.", "SHENZHEN-UTEPO-TECH" },
	{ 0xC40938, "Fujian Star-net Communication Co., Ltd", "FUJIAN-STAR-NET-COMMUNICATION" },
	{ 0xC40ACB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xC40E45, "ACK Networks,Inc.", "ACK-NETWORKS" },
	{ 0xC40F09, "Hermes electronic GmbH", "HERMES" },
	{ 0xC4108A, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0xC413E2, "Aerohive Networks Inc.", "AEROHIVE-NETWORKS" },
	{ 0xC4143C, "Cisco", "CISCO" },
	{ 0xC416FA, "Prysm Inc", "PRYSM" },
	{ 0xC417FE, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xC4198B, "Dominion Voting Systems Corporation", "DOMINION-VOTING" },
	{ 0xC419EC, "Qualisys AB", "QUALISYS-AB" },
	{ 0xC41ECE, "HMI Sources Ltd.", "HMI-SOURCES" },
	{ 0xC421C8, "KYOCERA Corporation", "KYOCERA" },
	{ 0xC4237A, "WhizNets Inc.", "WHIZNETS" },
	{ 0xC4242E, "Galvanic Applied Sciences Inc", "GALVANIC-APPLIED-SCIENCES" },
	{ 0xC42628, "Airo Wireless", "AIRO-WIRELESS" },
	{ 0xC42795, "Technicolor USA Inc.", "TECHNICOLOR-USA" },
	{ 0xC4291D, "KLEMSAN ELEKTRIK ELEKTRONIK SAN.VE TIC.AS.", "KLEMSAN-ELEKTRIK-ELEKTRONIK-SAN-VE-TIC-AS" },
	{ 0xC42C03, "Apple", "APPLE" },
	{ 0xC42F90, "Hangzhou Hikvision Digital Technology Co.,Ltd.", "HANGZHOU-HIKVISION-DIGITAL" },
	{ 0xC4346B, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xC4366C, "LG Innotek", "LG-INNOTEK" },
	{ 0xC436DA, "Rusteletech Ltd.", "RUSTELETECH" },
	{ 0xC438D3, "TAGATEC CO.,LTD", "TAGATEC" },
	{ 0xC4393A, "SMC Networks Inc", "SMC-NETWORKS" },
	{ 0xC43A9F, "Siconix Inc.", "SICONIX" },
	{ 0xC43ABE, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0xC43C3C, "CYBELEC SA", "CYBELEC" },
	{ 0xC43DC7, "NETGEAR", "NETGEAR" },
	{ 0xC44202, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xC4438F, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xC44567, "SAMBON PRECISON and ELECTRONICS", "SAMBON-PRECISON-AND-ELECTRONICS" },
	{ 0xC445EC, "Shanghai Yali Electron Co.,LTD", "SHANGHAI-YALI-ELECTRON" },
	{ 0xC44619, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xC44838, "Satcom Direct, Inc.", "SATCOM-DIRECT" },
	{ 0xC44AD0, "FIREFLIES SYSTEMS", "FIREFLIES" },
	{ 0xC44B44, "Omniprint Inc.", "OMNIPRINT" },
	{ 0xC44BD1, "Wallys Communications Teachnologies Co.,Ltd.", "WALLYS-COMMUNICATION-TEACHNOLOGIES" },
	{ 0xC44E1F, "BlueN", "BLUEN" },
	{ 0xC44EAC, "Shenzhen Shiningworth Technology Co., Ltd.", "SHENZHEN-SHININGWORTH" },
	{ 0xC45006, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xC45444, "QUANTA COMPUTER INC.", "QUANTA" },
	{ 0xC455A6, "Cadac Holdings Ltd", "CADAC-HOLDINGS" },
	{ 0xC455C2, "Bach-Simpson", "BACH-SIMPSON" },
	{ 0xC45600, "Galleon Embedded Computing", "GALLEON-EMBEDDED-COMPUTING" },
	{ 0xC456FE, "Lava International Ltd.", "LAVA" },
	{ 0xC4576E, "Samsung Electronics Co.,LTD", "SAMSUNG-ELECTRONICS" },
	{ 0xC458C2, "Shenzhen TATFOOK Technology Co., Ltd.", "SHENZHEN-TATFOOK" },
	{ 0xC45976, "Fugoo Coorporation", "FUGOO-COORPORATION" },
	{ 0xC45DD8, "HDMI Forum", "HDMI-FORUM" },
	{ 0xC46044, "Everex Electronics Limited", "EVEREX-ELECTRONICS" },
	{ 0xC4626B, "ZPT Vigantice", "ZPT-VIGANTICE" },
	{ 0xC462EA, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xC46354, "U-Raku, Inc.", "U-RAKU" },
	{ 0xC46413, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xC467B5, "Libratone A/S", "LIBRATONE-A/S" },
	{ 0xC4693E, "Turbulence Design Inc.", "TURBULENCE-DESIGN" },
	{ 0xC46AB7, "Xiaomi Technology,Inc.", "XIAOMI" },
	{ 0xC46BB4, "myIDkey", "MYIDKEY" },
	{ 0xC46DF1, "DataGravity", "DATAGRAVITY" },
	{ 0xC46E1F, "TP-LINK TECHNOLOGIES CO.,LTD", "TP-LINK-TECHNOLOGIES" },
	{ 0xC47130, "Fon Technology S.L.", "FON-S-L" },
	{ 0xC471FE, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xC47295, "Cisco", "CISCO" },
	{ 0xC4731E, "Samsung Eletronics Co., Ltd", "SAMSUNG-ELETRONICS" },
	{ 0xC47B2F, "Beijing JoinHope Image Technology Ltd.", "BEIJING-JOINHOPE-IMAGE" },
	{ 0xC47BA3, "NAVIS Inc.", "NAVIS" },
	{ 0xC47D4F, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xC47DCC, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0xC47DFE, "A.N. Solutions GmbH", "A-N-SOLUTIONS" },
	{ 0xC47F51, "Inventek Systems", "INVENTEK" },
	{ 0xC4823F, "Fujian Newland Auto-ID Tech. Co,.Ltd.", "FUJIAN-NEWLAND-AUTO-ID-TECH" },
	{ 0xC4824E, "Changzhou Uchip Electronics Co., LTD.", "CHANGZHOU-UCHIP-ELECTRONICS" },
	{ 0xC48508, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xC488E5, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xC48E8F, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xC4913A, "Shenzhen Sanland Electronic Co., ltd.", "SHENZHEN-SANLAND" },
	{ 0xC4924C, "KEISOKUKI CENTER CO.,LTD.", "KEISOKUKI-CENTER" },
	{ 0xC49300, "8Devices", "8DEVICES" },
	{ 0xC49313, "100fio networks technology llc", "100FIO-NETWORKS-LLC" },
	{ 0xC49380, "Speedytel technology", "SPEEDYTEL" },
	{ 0xC495A2, "SHENZHEN WEIJIU INDUSTRY AND TRADE DEVELOPMENT CO., LTD", "SHENZHEN-WEIJIU-INDUSTRY-AND-TRADE-DEVELOPMENT" },
	{ 0xC49805, "Minieum Networks, Inc", "MINIEUM-NETWORKS" },
	{ 0xC49A02, "LG Electronics (Mobile Communicaitons)", "LG-ELECTRONICS-MOBILE-COMMUNICAITONS" },
	{ 0xC4A81D, "D-Link International", "D-LINK" },
	{ 0xC4AAA1, "SUMMIT DEVELOPMENT, spol.s r.o.", "SUMMIT-DEVELOPMENT-SPOL-S-R-O" },
	{ 0xC4AD21, "MEDIAEDGE Corporation", "MEDIAEDGE" },
	{ 0xC4B512, "General Electric Digital Energy", "GENERAL-ELECTRIC-DIGITAL-ENERGY" },
	{ 0xC4BA99, "I+ME Actia Informatik und Mikro-Elektronik GmbH", "I+ME-ACTIA-INFORMATIK-UND-MIKRO-ELEKTRONIK" },
	{ 0xC4BAA3, "Beijing Winicssec Technologies Co., Ltd.", "BEIJING-WINICSSEC-TECHNOLOGIES" },
	{ 0xC4BD6A, "SKF GmbH", "SKF" },
	{ 0xC4BE84, "Texas Instruments.", "TEXAS-INSTRUMENTS" },
	{ 0xC4C0AE, "MIDORI ELECTRONIC CO., LTD.", "MIDORI" },
	{ 0xC4C19F, "National Oilwell Varco Instrumentation, Monitoring, and Optimization (NOV IMO)", "NATIONAL-OILWELL-VARCO-INSTRUMENTATION-MONITORING-AND-OPTIMIZATION-NOV-IMO" },
	{ 0xC4C755, "Beijing HuaqinWorld Technology Co.,Ltd", "BEIJING-HUAQINWORLD" },
	{ 0xC4C919, "Energy Imports Ltd", "ENERGY-IMPORTS" },
	{ 0xC4C9EC, "Gugaoo HK Limited", "GUGAOO-HK" },
	{ 0xC4CAD9, "Hangzhou H3C Technologies Co., Limited", "HANGZHOU-H3C-TECHNOLOGIES" },
	{ 0xC4CD45, "Beijing Boomsense Technology CO.,LTD.", "BEIJING-BOOMSENSE" },
	{ 0xC4D489, "JiangSu Joyque Information Industry Co.,Ltd", "JIANGSU-JOYQUE-INFORMATION-INDUSTRY" },
	{ 0xC4D655, "Tercel technology co.,ltd", "TERCEL" },
	{ 0xC4D987, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xC4DA26, "NOBLEX SA", "NOBLEX" },
	{ 0xC4E032, "IEEE 1904.1 Working Group", "IEEE-1904-1-WORKING-GROUP" },
	{ 0xC4E17C, "U2S co.", "U2S" },
	{ 0xC4E7BE, "SCSpro Co.,Ltd", "SCSPRO" },
	{ 0xC4E92F, "AB Sciex", "AB-SCIEX" },
	{ 0xC4E984, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xC4EBE3, "RRCN SAS", "RRCN-SAS" },
	{ 0xC4EDBA, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xC4EEAE, "VSS Monitoring", "VSS-MONITORING" },
	{ 0xC4EEF5, "Oclaro, Inc.", "OCLARO" },
	{ 0xC4F464, "Spica international", "SPICA" },
	{ 0xC4F57C, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0xC4FCE4, "DishTV NZ Ltd", "DISHTV-NZ" },
	{ 0xC80084, "Cisco", "CISCO" },
	{ 0xC80210, "LG Innotek", "LG-INNOTEK" },
	{ 0xC80258, "ITW GSE ApS", "ITW-GSE-APS" },
	{ 0xC802A6, "Beijing Newmine Technology", "BEIJING-NEWMINE" },
	{ 0xC80718, "TDSi", "TDSI" },
	{ 0xC808E9, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xC80AA9, "Quanta Computer Inc.", "QUANTA" },
	{ 0xC80E77, "Le Shi Zhi Xin Electronic Technology (Tianjin) Co.,Ltd", "LE-SHI-ZHI-XIN-TIANJIN" },
	{ 0xC80E95, "OmniLync Inc.", "OMNILYNC" },
	{ 0xC81479, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xC816BD, "HISENSE ELECTRIC CO.,LTD.", "HISENSE-ELECTRIC" },
	{ 0xC819F7, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xC81AFE, "DLOGIC GmbH", "DLOGIC" },
	{ 0xC81B6B, "Innova Security", "INNOVA-SECURITY" },
	{ 0xC81E8E, "ADV Security (S) Pte Ltd", "ADV-SECURITY-S-PTE" },
	{ 0xC81F66, "Dell Inc", "DELL" },
	{ 0xC8208E, "Storagedata", "STORAGEDATA" },
	{ 0xC8292A, "Barun Electronics", "BARUN-ELECTRONICS" },
	{ 0xC82A14, "Apple", "APPLE" },
	{ 0xC82E94, "Halfa Enterprise Co., Ltd.", "HALFA" },
	{ 0xC83168, "eZEX corporation", "EZEX" },
	{ 0xC83232, "Hunting Innova", "HUNTING-INNOVA" },
	{ 0xC8334B, "Apple", "APPLE" },
	{ 0xC835B8, "Ericsson, EAB/RWI/K", "ERICSSON-EAB/RWI/K" },
	{ 0xC83A35, "Tenda Technology Co., Ltd.", "TENDA" },
	{ 0xC83B45, "JRI-Maxant", "JRI-MAXANT" },
	{ 0xC83D97, "Nokia Corporation", "NOKIA" },
	{ 0xC83E99, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xC83EA7, "KUNBUS GmbH", "KUNBUS" },
	{ 0xC83FB4, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xC84529, "IMK Networks Co.,Ltd", "IMK-NETWORKS" },
	{ 0xC84544, "Shanghai Enlogic Electric Technology Co., Ltd.", "SHANGHAI-ENLOGIC-ELECTRIC" },
	{ 0xC8458F, "Wyler AG", "WYLER" },
	{ 0xC848F5, "MEDISON Xray Co., Ltd", "MEDISON-XRAY" },
	{ 0xC84C75, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xC85645, "Intermas France", "INTERMAS-FRANCE" },
	{ 0xC85663, "Sunflex Europe GmbH", "SUNFLEX-EUROPE" },
	{ 0xC86000, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xC864C7, "zte corporation", "ZTE" },
	{ 0xC86C1E, "Display Systems Ltd", "DISPLAY" },
	{ 0xC86C87, "Zyxel Communications Corp", "ZYXEL-COMMUNICATION" },
	{ 0xC86CB6, "Optcom Co., Ltd.", "OPTCOM" },
	{ 0xC86F1D, "Apple", "APPLE" },
	{ 0xC87248, "Aplicom Oy", "APLICOM-OY" },
	{ 0xC87B5B, "zte corporation", "ZTE" },
	{ 0xC87CBC, "Valink Co., Ltd.", "VALINK" },
	{ 0xC87D77, "Shenzhen Kingtech Communication Equipment Co.,Ltd", "SHENZHEN-KINGTECH-COMMUNICATION-EQUIPMENT" },
	{ 0xC87E75, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xC88439, "Sunrise Technologies", "SUNRISE-TECHNOLOGIES" },
	{ 0xC88447, "Beautiful Enterprise Co., Ltd", "BEAUTIFUL" },
	{ 0xC88550, "Apple", "APPLE" },
	{ 0xC8873B, "Net Optics", "NET-OPTICS" },
	{ 0xC88A83, "Dongguan HuaHong Electronics Co.,Ltd", "DONGGUAN-HUAHONG-ELECTRONICS" },
	{ 0xC88B47, "Nolangroup S.P.A con Socio Unico", "NOLANGROUP-S-P-A-CON-SOCIO-UNICO" },
	{ 0xC8903E, "Pakton Technologies", "PAKTON-TECHNOLOGIES" },
	{ 0xC891F9, "SAGEMCOM", "SAGEMCOM" },
	{ 0xC89346, "MXCHIP Company Limited", "MXCHIP-COMPANY" },
	{ 0xC89383, "Embedded Automation, Inc.", "EMBEDDED-AUTOMATION" },
	{ 0xC894D2, "Jiangsu Datang Electronic Products Co., Ltd", "JIANGSU-DATANG-PRODUCTS" },
	{ 0xC8979F, "Nokia Corporation", "NOKIA" },
	{ 0xC89C1D, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xC89CDC, "ELITEGROUP COMPUTER SYSTEM CO., LTD.", "ELITEGROUP-SYSTEM" },
	{ 0xC89F1D, "SHENZHEN COMMUNICATION TECHNOLOGIES CO.,LTD", "SHENZHEN-COMMUNICATION-TECHNOLOGIES" },
	{ 0xC89F42, "VDII Innovation AB", "VDII-INNOVATION-AB" },
	{ 0xC8A030, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xC8A1B6, "Shenzhen Longway Technologies Co., Ltd", "SHENZHEN-LONGWAY-TECHNOLOGIES" },
	{ 0xC8A1BA, "Neul Ltd", "NEUL" },
	{ 0xC8A620, "Nebula, Inc", "NEBULA" },
	{ 0xC8A70A, "Verizon Business", "VERIZON-BUSINESS" },
	{ 0xC8A729, "SYStronics Co., Ltd.", "SYSTRONICS" },
	{ 0xC8A823, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xC8AA21, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xC8AACC, "PRIVATE", "PRIVATE" },
	{ 0xC8AE9C, "Shanghai TYD Elecronic Technology Co. Ltd", "SHANGHAI-TYD-ELECRONIC" },
	{ 0xC8AF40, "marco Systemanalyse und Entwicklung GmbH", "MARCO-SYSTEMANALYSE-UND-ENTWICKLUNG" },
	{ 0xC8B373, "Cisco-Linksys, LLC", "CISCO-LINKSYS-LLC" },
	{ 0xC8B5B7, "Apple", "APPLE" },
	{ 0xC8BA94, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xC8BBD3, "Embrane", "EMBRANE" },
	{ 0xC8BCC8, "Apple", "APPLE" },
	{ 0xC8BE19, "D-Link International", "D-LINK" },
	{ 0xC8C126, "ZPM Industria e Comercio Ltda", "ZPM-INDUSTRIA-E-COMERCIO-LTDA" },
	{ 0xC8C13C, "RuggedTek Hangzhou Co., Ltd", "RUGGEDTEK-HANGZHOU" },
	{ 0xC8C50E, "Shenzhen Primestone Network Technologies.Co., Ltd.", "SHENZHEN-PRIMESTONE-NETWORK-TECHNOLOGIES" },
	{ 0xC8C791, "Zero1.tv GmbH", "ZERO1-TV" },
	{ 0xC8CBB8, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xC8CD72, "SAGEMCOM", "SAGEMCOM" },
	{ 0xC8D019, "Shanghai Tigercel Communication Technology Co.,Ltd", "SHANGHAI-TIGERCEL-COMMUNICATION" },
	{ 0xC8D10B, "Nokia Corporation", "NOKIA" },
	{ 0xC8D15E, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xC8D1D1, "AGAiT Technology Corporation", "AGAIT" },
	{ 0xC8D2C1, "Jetlun (Shenzhen) Corporation", "JETLUN-SHENZHEN" },
	{ 0xC8D3A3, "D-Link International", "D-LINK" },
	{ 0xC8D429, "Muehlbauer AG", "MUEHLBAUER" },
	{ 0xC8D590, "FLIGHT DATA SYSTEMS", "FLIGHT-DATA" },
	{ 0xC8D5FE, "Shenzhen Zowee Technology Co., Ltd", "SHENZHEN-ZOWEE" },
	{ 0xC8D719, "Cisco Consumer Products, LLC", "CISCO-CONSUMER-PRODUCTS-LLC" },
	{ 0xC8D779, "Qingdao Haier Telecom Co.，Ltd", "QINGDAO-HAIER-TELECOM-，" },
	{ 0xC8DDC9, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0xC8DE51, "Integra Networks, Inc.", "INTEGRA-NETWORKS" },
	{ 0xC8DF7C, "Nokia Corporation", "NOKIA" },
	{ 0xC8E0EB, "Apple", "APPLE" },
	{ 0xC8E130, "Milkyway Group Ltd", "MILKYWAY-GROUP" },
	{ 0xC8E1A7, "Vertu Corporation Limited", "VERTU" },
	{ 0xC8E42F, "Technical Research Design and Development", "TECHNICAL-RESEARCH-DESIGN-AND-DEVELOPMENT" },
	{ 0xC8E7D8, "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.", "SHENZHEN-MERCURY-COMMUNICATION-TECHNOLOGIES" },
	{ 0xC8EE08, "TANGTOP TECHNOLOGY CO.,LTD", "TANGTOP" },
	{ 0xC8EE75, "Pishion International Co. Ltd", "PISHION" },
	{ 0xC8EEA6, "Shenzhen SHX Technology Co., Ltd", "SHENZHEN-SHX" },
	{ 0xC8EF2E, "Beijing Gefei Tech. Co., Ltd", "BEIJING-GEFEI-TECH" },
	{ 0xC8F36B, "Yamato Scale Co.,Ltd.", "YAMATO-SCALE" },
	{ 0xC8F386, "Shenzhen Xiaoniao Technology Co.,Ltd", "SHENZHEN-XIAONIAO" },
	{ 0xC8F406, "Avaya, Inc", "AVAYA" },
	{ 0xC8F650, "Apple", "APPLE" },
	{ 0xC8F68D, "S.E.TECHNOLOGIES LIMITED", "S-E-TECHNOLOGIES" },
	{ 0xC8F704, "Building Block Video", "BUILDING-BLOCK-VIDEO" },
	{ 0xC8F733, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xC8F981, "Seneca s.r.l.", "SENECA-S-R-L" },
	{ 0xC8F9F9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xC8FB26, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0xC8FE30, "Bejing DAYO Mobile Communication Technology Ltd.", "BEJING-DAYO-MOBILE-COMMUNICATION" },
	{ 0xC8FF77, "Dyson Limited", "DYSON" },
	{ 0xCC0080, "BETTINI SRL", "BETTINI-SRL" },
	{ 0xCC03FA, "Technicolor CH USA", "TECHNICOLOR-CH-USA" },
	{ 0xCC047C, "G-WAY Microwave", "G-WAY-MICROWAVE" },
	{ 0xCC04B4, "Select Comfort", "SELECT-COMFORT" },
	{ 0xCC051B, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xCC07AB, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xCC07E4, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0xCC08E0, "Apple", "APPLE" },
	{ 0xCC09C8, "IMAQLIQ LTD", "IMAQLIQ" },
	{ 0xCC0CDA, "Miljovakt AS", "MILJOVAKT-AS" },
	{ 0xCC0DEC, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0xCC10A3, "Beijing Nan Bao Technology Co., Ltd.", "BEIJING-NAN-BAO" },
	{ 0xCC14A6, "Yichun MyEnergy Domain, Inc", "YICHUN-MYENERGY-DOMAIN" },
	{ 0xCC187B, "Manzanita Systems, Inc.", "MANZANITA" },
	{ 0xCC19A8, "PT Inovação e Sistemas SA", "PT-INOVAçãO-E-SISTEMAS" },
	{ 0xCC1AFA, "zte corporation", "ZTE" },
	{ 0xCC1EFF, "Metrological Group BV", "METROLOGICAL-GROUP" },
	{ 0xCC2218, "InnoDigital Co., Ltd.", "INNODIGITAL" },
	{ 0xCC262D, "Verifi, LLC", "VERIFI-LLC" },
	{ 0xCC29F5, "Apple", "APPLE" },
	{ 0xCC2A80, "Micro-Biz intelligence solutions Co.,Ltd", "MICRO-BIZ-INTELLIGENCE-SOLUTIONS" },
	{ 0xCC2D8C, "LG ELECTRONICS INC", "LG-ELECTRONICS" },
	{ 0xCC3080, "VAIO Corporation", "VAIO" },
	{ 0xCC33BB, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0xCC3429, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xCC34D7, "GEWISS S.P.A.", "GEWISS-S-P-A" },
	{ 0xCC3540, "Technicolor USA Inc.", "TECHNICOLOR-USA" },
	{ 0xCC398C, "Shiningtek", "SHININGTEK" },
	{ 0xCC3A61, "SAMSUNG ELECTRO MECHANICS CO., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xCC3C3F, "SA.S.S. Datentechnik AG", "S-S-DATENTECHNIK" },
	{ 0xCC3D82, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xCC3E5F, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xCC3F1D, "Intesis Software SL", "INTESIS-SOFTWARE-SL" },
	{ 0xCC43E3, "Trump s.a.", "TRUMP" },
	{ 0xCC4703, "Intercon Systems Co., Ltd.", "INTERCON" },
	{ 0xCC4AE1, "Fourtec -Fourier Technologies", "FOURTEC--FOURIER-TECHNOLOGIES" },
	{ 0xCC4BFB, "Hellberg Safety AB", "HELLBERG-SAFETY-AB" },
	{ 0xCC4E24, "Brocade Communications Systems, Inc.", "BROCADE-COMMUNICATION" },
	{ 0xCC4EEC, "HUMAX Co., Ltd.", "HUMAX" },
	{ 0xCC501C, "KVH Industries, Inc.", "KVH-INDUSTRIES" },
	{ 0xCC5076, "Ocom Communications, Inc.", "OCOM-COMMUNICATION" },
	{ 0xCC52AF, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0xCC53B5, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xCC5459, "OnTime Networks AS", "ONTIME-NETWORKS-AS" },
	{ 0xCC55AD, "RIM", "RIM" },
	{ 0xCC593E, "TOUMAZ LTD", "TOUMAZ" },
	{ 0xCC5C75, "Weightech Com. Imp. Exp. Equip. Pesagem Ltda", "WEIGHTECH-COM-IMP-EXP-EQUIP-PESAGEM-LTDA" },
	{ 0xCC5D4E, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0xCC5D57, "Information System Research Institute,Inc.", "INFORMATION-SYSTEM-RESEARCH-INSTITUTE" },
	{ 0xCC60BB, "Empower RF Systems", "EMPOWER-RF" },
	{ 0xCC65AD, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xCC69B0, "Global Traffic Technologies, LLC", "GLOBAL-TRAFFIC-TECHNOLOGIES-LLC" },
	{ 0xCC6B98, "Minetec Wireless Technologies", "MINETEC-WIRELESS-TECHNOLOGIES" },
	{ 0xCC6BF1, "Sound Masking Inc.", "SOUND-MASKING" },
	{ 0xCC6DA0, "Roku, Inc.", "ROKU" },
	{ 0xCC6DEF, "TJK Tietolaite Oy", "TJK-TIETOLAITE-OY" },
	{ 0xCC720F, "Viscount Systems Inc.", "VISCOUNT" },
	{ 0xCC7498, "Filmetrics Inc.", "FILMETRICS" },
	{ 0xCC7669, "SEETECH", "SEETECH" },
	{ 0xCC785F, "Apple", "APPLE" },
	{ 0xCC7A30, "CMAX Wireless Co., Ltd.", "CMAX-WIRELESS" },
	{ 0xCC7B35, "zte corporation", "ZTE" },
	{ 0xCC7D37, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xCC7EE7, "Panasonic AVC Networks Company", "PANASONIC-AVC-NETWORKS-COMPANY" },
	{ 0xCC856C, "SHENZHEN MDK DIGITAL TECHNOLOGY CO.,LTD", "SHENZHEN-MDK-DIGITAL" },
	{ 0xCC89FD, "Nokia Corporation", "NOKIA" },
	{ 0xCC8CE3, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xCC9093, "Hansong Tehnologies", "HANSONG-TEHNOLOGIES" },
	{ 0xCC912B, "TE Connectivity Touch Solutions", "TE-CONNECTIVITY-TOUCH-SOLUTIONS" },
	{ 0xCC944A, "Pfeiffer Vacuum GmbH", "PFEIFFER-VACUUM" },
	{ 0xCC95D7, "VIZIO, Inc", "VIZIO" },
	{ 0xCC9635, "LVS Co.,Ltd.", "LVS" },
	{ 0xCC96A0, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0xCC9E00, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0xCC9F35, "Transbit Sp. z o.o.", "TRANSBIT-SP-Z-O-O" },
	{ 0xCCA0E5, "DZG Metering GmbH", "DZG-METERING" },
	{ 0xCCA223, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xCCA374, "Guangdong Guanglian Electronic Technology Co.Ltd", "GUANGDONG-GUANGLIAN" },
	{ 0xCCA462, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xCCA4AF, "Shenzhen Sowell Technology Co., LTD", "SHENZHEN-SOWELL" },
	{ 0xCCA614, "AIFA TECHNOLOGY CORP.", "AIFA" },
	{ 0xCCAF78, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xCCB255, "D-Link International", "D-LINK" },
	{ 0xCCB3F8, "FUJITSU ISOTEC LIMITED", "FUJITSU-ISOTEC" },
	{ 0xCCB55A, "Fraunhofer ITWM", "FRAUNHOFER-ITWM" },
	{ 0xCCB691, "NECMagnusCommunications", "NECMAGNUSCOMMUNICATIONS" },
	{ 0xCCB888, "AnB Securite s.a.", "ANB-SECURITE" },
	{ 0xCCB8F1, "EAGLE KINGDOM TECHNOLOGIES LIMITED", "EAGLE-KINGDOM-TECHNOLOGIES" },
	{ 0xCCBD35, "Steinel GmbH", "STEINEL" },
	{ 0xCCBDD3, "Ultimaker B.V.", "ULTIMAKER" },
	{ 0xCCBE71, "OptiLogix BV", "OPTILOGIX" },
	{ 0xCCC104, "Applied Technical Systems", "APPLIED-TECHNICAL" },
	{ 0xCCC3EA, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0xCCC50A, "SHENZHEN DAJIAHAO TECHNOLOGY CO.,LTD", "SHENZHEN-DAJIAHAO" },
	{ 0xCCC62B, "Tri-Systems Corporation", "TRI" },
	{ 0xCCC8D7, "CIAS Elettronica srl", "CIAS-ELETTRONICA-SRL" },
	{ 0xCCCC4E, "Sun Fountainhead USA. Corp", "SUN-FOUNTAINHEAD-USA" },
	{ 0xCCCC81, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xCCCD64, "SM-Electronic GmbH", "SM" },
	{ 0xCCCE40, "Janteq Corp", "JANTEQ" },
	{ 0xCCD29B, "Shenzhen Bopengfa Elec&Technology CO.,Ltd", "SHENZHEN-BOPENGFA-ELEC" },
	{ 0xCCD539, "Cisco", "CISCO" },
	{ 0xCCD811, "Aiconn Technology Corporation", "AICONN" },
	{ 0xCCD8C1, "Cisco", "CISCO" },
	{ 0xCCD9E9, "SCR Engineers Ltd.", "SCR-ENGINEERS" },
	{ 0xCCE17F, "juniper networks", "JUNIPER-NETWORKS" },
	{ 0xCCE1D5, "Buffalo Inc.", "BUFFALO" },
	{ 0xCCE798, "My Social Stuff", "MY-SOCIAL-STUFF" },
	{ 0xCCE7DF, "American Magnetics, Inc.", "AMERICAN-MAGNETICS" },
	{ 0xCCE8AC, "SOYEA Technology Co.,Ltd.", "SOYEA" },
	{ 0xCCEA1C, "DCONWORKS Co., Ltd", "DCONWORKS" },
	{ 0xCCEED9, "Deto Mechatronic GmbH", "DETO-MECHATRONIC" },
	{ 0xCCEF48, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xCCF3A5, "Chi Mei Communication Systems, Inc", "CHI-MEI-COMMUNICATION" },
	{ 0xCCF407, "EUKREA ELECTROMATIQUE SARL", "EUKREA-ELECTROMATIQUE-SARL" },
	{ 0xCCF538, "3isysnetworks", "3ISYSNETWORKS" },
	{ 0xCCF67A, "Ayecka Communication Systems LTD", "AYECKA-COMMUNICATION" },
	{ 0xCCF841, "Lumewave", "LUMEWAVE" },
	{ 0xCCF8F0, "Xi'an HISU Multimedia Technology Co.,Ltd.", "XI-AN-HISU-MULTIMEDIA" },
	{ 0xCCF954, "Avaya, Inc", "AVAYA" },
	{ 0xCCF9E8, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xCCFA00, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xCCFB65, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0xCCFC6D, "RIZ TRANSMITTERS", "RIZ-TRANSMITTERS" },
	{ 0xCCFCB1, "Wireless Technology, Inc.", "WIRELESS" },
	{ 0xCCFE3C, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xD00790, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD00AAB, "Yokogawa Digital Computer Corporation", "YOKOGAWA-DIGITAL" },
	{ 0xD00EA4, "Porsche Cars North America", "PORSCHE-CARS-NORTH-AMERICA" },
	{ 0xD00ED9, "TAICANG T AND W ELECTRONICS CO LTD", "TAICANG-T-AND-W-ELECTRONICS" },
	{ 0xD01242, "BIOS Corporation", "BIOS" },
	{ 0xD0131E, "Sunrex Technology Corp", "SUNREX" },
	{ 0xD0154A, "zte corporation", "ZTE" },
	{ 0xD0176A, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xD01AA7, "UniPrint", "UNIPRINT" },
	{ 0xD01CBB, "Beijing Ctimes Digital Technology Co., Ltd.", "BEIJING-CTIMES-DIGITAL" },
	{ 0xD02212, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xD022BE, "Samsung Electro Mechanics co.,LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xD023DB, "Apple", "APPLE" },
	{ 0xD02516, "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.", "SHENZHEN-MERCURY-COMMUNICATION-TECHNOLOGIES" },
	{ 0xD02788, "Hon Hai Precision Ind.Co.Ltd", "HON-HAI-PRECISION-IND" },
	{ 0xD02C45, "littleBits Electronics, Inc.", "LITTLEBITS-ELECTRONICS" },
	{ 0xD02DB3, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xD03110, "Ingenic Semiconductor Co.,Ltd", "INGENIC-SEMICONDUCTOR" },
	{ 0xD03761, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD03972, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD039B3, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xD046DC, "Southwest Research Institute", "SOUTHWEST-RESEARCH-INSTITUTE" },
	{ 0xD048F3, "DATTUS Inc", "DATTUS" },
	{ 0xD04CC1, "SINTRONES Technology Corp.", "SINTRONES" },
	{ 0xD04F7E, "Apple", "APPLE" },
	{ 0xD05099, "ASRock Incorporation", "ASROCK-INCORPORATION" },
	{ 0xD05162, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0xD052A8, "Physical Graph Corporation", "PHYSICAL-GRAPH" },
	{ 0xD05349, "Liteon Technology Co., Ltd.", "LITEON" },
	{ 0xD0542D, "Cambridge Industries(Group) Co.,Ltd.", "CAMBRIDGE-INDUSTRIES-GROUP" },
	{ 0xD0574C, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xD05785, "Pantech Co., Ltd.", "PANTECH" },
	{ 0xD057A1, "Werma Signaltechnik GmbH & Co. KG", "WERMA-SIGNALTECHNIK" },
	{ 0xD05875, "Active Control Technology Inc.", "ACTIVE-CONTROL" },
	{ 0xD059C3, "CeraMicro Technology Corporation", "CERAMICRO" },
	{ 0xD059E4, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xD05A0F, "I-BT DIGITAL CO.,LTD", "I-BT-DIGITAL" },
	{ 0xD05AF1, "Shenzhen Pulier Tech CO.,Ltd", "SHENZHEN-PULIER-TECH" },
	{ 0xD05BA8, "zte corporation", "ZTE" },
	{ 0xD05C7A, "Sartura d.o.o.", "SARTURA-D-O-O" },
	{ 0xD05FB8, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD05FCE, "Hitachi Data Systems", "HITACHI-DATA" },
	{ 0xD062A0, "China Essence Technology (Zhumadian) Co., Ltd.", "CHINA-ESSENCE-ZHUMADIAN" },
	{ 0xD0634D, "Meiko Maschinenbau GmbH &amp; Co. KG", "MEIKO-MASCHINENBAU-AMP" },
	{ 0xD063B4, "SolidRun Ltd.", "SOLIDRUN" },
	{ 0xD0667B, "Samsung Electronics Co., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0xD067E5, "Dell Inc", "DELL" },
	{ 0xD0699E, "LUMINEX Lighting Control Equipment", "LUMINEX-LIGHTING-CONTROL-EQUIPMENT" },
	{ 0xD069D0, "Verto Medical Solutions, LLC", "VERTO-MEDICAL-SOLUTIONS-LLC" },
	{ 0xD06A1F, "BSE CO.,LTD.", "BSE" },
	{ 0xD06F4A, "TOPWELL INTERNATIONAL HOLDINGS LIMITED", "TOPWELL-HOLDINGS" },
	{ 0xD072DC, "Cisco", "CISCO" },
	{ 0xD0737F, "Mini-Circuits", "MINI-CIRCUITS" },
	{ 0xD0738E, "DONG OH PRECISION CO., LTD.", "DONG-OH-PRECISION" },
	{ 0xD073D5, "LIFI LABS MANAGEMENT PTY LTD", "LIFI-LABS-MANAGEMENT-PTY" },
	{ 0xD075BE, "Reno A&E", "RENO-A-E" },
	{ 0xD07650, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xD07AB5, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xD07DE5, "Forward Pay Systems, Inc.", "FORWARD-PAY" },
	{ 0xD07E28, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xD07E35, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xD083D4, "XTel ApS", "XTEL-APS" },
	{ 0xD084B0, "Sagemcom", "SAGEMCOM" },
	{ 0xD08999, "APCON, Inc.", "APCON" },
	{ 0xD08A55, "Skullcandy", "SKULLCANDY" },
	{ 0xD08B7E, "Passif Semiconductor", "PASSIF-SEMICONDUCTOR" },
	{ 0xD08CB5, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD08CFF, "UPWIS AB", "UPWIS-AB" },
	{ 0xD0929E, "Microsoft Corporation", "MICROSOFT" },
	{ 0xD093F8, "Stonestreet One LLC", "STONESTREET-ONE-LLC" },
	{ 0xD095C7, "Pantech Co., Ltd.", "PANTECH" },
	{ 0xD09B05, "Emtronix", "EMTRONIX" },
	{ 0xD09C30, "Foster Electric Company, Limited", "FOSTER-ELECTRIC-COMPANY" },
	{ 0xD09D0A, "LINKCOM", "LINKCOM" },
	{ 0xD09DAB, "TCT mobile ltd", "TCT-MOBILE" },
	{ 0xD0A0D6, "Chengdu TD Tech Ltd.", "CHENGDU-TD-TECH" },
	{ 0xD0A311, "Neuberger Gebäudeautomation GmbH", "NEUBERGER-GEBäUDEAUTOMATION" },
	{ 0xD0A5A6, "Cisco", "CISCO" },
	{ 0xD0A637, "Apple", "APPLE" },
	{ 0xD0AEEC, "Alpha Networks Inc.", "ALPHA-NETWORKS" },
	{ 0xD0AFB6, "Linktop Technology Co., LTD", "LINKTOP" },
	{ 0xD0B33F, "SHENZHEN TINNO MOBILE TECHNOLOGY CO.,LTD.", "SHENZHEN-TINNO-MOBILE" },
	{ 0xD0B498, "Robert Bosch LLC Automotive Electronics", "ROBERT-BOSCH-LLC-AUTOMOTIVE-ELECTRONICS" },
	{ 0xD0B523, "Bestcare Cloucal Corp.", "BESTCARE-CLOUCAL" },
	{ 0xD0B53D, "SEPRO ROBOTIQUE", "SEPRO-ROBOTIQUE" },
	{ 0xD0B5C2, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD0BB80, "SHL Telemedicine International Ltd.", "SHL-TELEMEDICINE" },
	{ 0xD0BD01, "DS International", "DS" },
	{ 0xD0BE2C, "CNSLink Co., Ltd.", "CNSLINK" },
	{ 0xD0BF9C, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xD0C0BF, "Actions Microelectronics Co., Ltd", "ACTIONS-MICROELECTRONICS" },
	{ 0xD0C1B1, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xD0C282, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xD0C42F, "Tamagawa Seiki Co.,Ltd.", "TAMAGAWA-SEIKI" },
	{ 0xD0C789, "Cisco", "CISCO" },
	{ 0xD0C7C0, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xD0CDE1, "Scientech Electronics", "SCIENTECH-ELECTRONICS" },
	{ 0xD0CF5E, "Energy Micro AS", "ENERGY-MICRO-AS" },
	{ 0xD0D0FD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xD0D212, "K2NET Co.,Ltd.", "K2NET" },
	{ 0xD0D286, "Beckman Coulter K.K.", "BECKMAN-COULTER-K-K" },
	{ 0xD0D3FC, "Mios, Ltd.", "MIOS" },
	{ 0xD0D412, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0xD0D471, "MVTECH co., Ltd", "MVTECH" },
	{ 0xD0D6CC, "Wintop", "WINTOP" },
	{ 0xD0DB32, "Nokia Corporation", "NOKIA" },
	{ 0xD0DF9A, "Liteon Technology Corporation", "LITEON" },
	{ 0xD0DFB2, "Genie Networks Limited", "GENIE-NETWORKS" },
	{ 0xD0DFC7, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xD0E140, "Apple, Inc", "APPLE" },
	{ 0xD0E347, "Yoga", "YOGA" },
	{ 0xD0E40B, "Wearable Inc.", "WEARABLE" },
	{ 0xD0E54D, "Pace plc", "PACE-PLC" },
	{ 0xD0E782, "Azurewave Technologies, Inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0xD0EB03, "Zhehua technology limited", "ZHEHUA" },
	{ 0xD0EB9E, "Seowoo Inc.", "SEOWOO" },
	{ 0xD0F0DB, "Ericsson", "ERICSSON" },
	{ 0xD0F27F, "SteadyServ Technoligies, LLC", "STEADYSERV-TECHNOLIGIES-LLC" },
	{ 0xD0F73B, "Helmut Mauell GmbH", "HELMUT-MAUELL" },
	{ 0xD0FA1D, "Qihoo 360 Technology Co.,Ltd", "QIHOO-360" },
	{ 0xD0FF50, "Texas Instruments, Inc", "TEXAS-INSTRUMENTS" },
	{ 0xD4000D, "Phoenix Broadband Technologies, LLC.", "PHOENIX-BROADBAND-TECHNOLOGIES-LLC" },
	{ 0xD40057, "MC Technologies GmbH", "MC-TECHNOLOGIES" },
	{ 0xD40129, "Broadcom Corporation", "BROADCOM" },
	{ 0xD4016D, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xD4024A, "Delphian Systems LLC", "DELPHIAN-LLC" },
	{ 0xD40598, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xD40B1A, "HTC Corporation", "HTC" },
	{ 0xD40BB9, "Solid Semecs bv.", "SOLID-SEMECS" },
	{ 0xD40FB2, "Applied Micro Electronics AME bv", "APPLIED-MICRO-ELECTRONICS-AME" },
	{ 0xD41090, "iNFORM Systems AG", "INFORM" },
	{ 0xD410CF, "Huanshun Network Science and Technology Co., Ltd.", "HUANSHUN-NETWORK-SCIENCE-AND" },
	{ 0xD411D6, "ShotSpotter, Inc.", "SHOTSPOTTER" },
	{ 0xD41296, "Anobit Technologies Ltd.", "ANOBIT-TECHNOLOGIES" },
	{ 0xD412BB, "Quadrant Components Inc. Ltd", "QUADRANT-COMPONENTS" },
	{ 0xD4136F, "Asia Pacific Brands", "ASIA-PACIFIC-BRANDS" },
	{ 0xD41C1C, "RCF S.P.A.", "RCF-S-P-A" },
	{ 0xD41E35, "TOHO Electronics INC.", "TOHO-ELECTRONICS" },
	{ 0xD41F0C, "JAI Oy", "JAI-OY" },
	{ 0xD4206D, "HTC Corporation", "HTC" },
	{ 0xD42122, "Sercomm Corporation", "SERCOMM" },
	{ 0xD4223F, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0xD4224E, "Alcatel Lucent", "ALCATEL-LUCENT" },
	{ 0xD42751, "Infopia Co., Ltd", "INFOPIA" },
	{ 0xD428B2, "ioBridge, Inc.", "IOBRIDGE" },
	{ 0xD429EA, "Zimory GmbH", "ZIMORY" },
	{ 0xD42C3D, "Sky Light Digital Limited", "SKY-LIGHT-DIGITAL" },
	{ 0xD42F23, "Akenori PTE Ltd", "AKENORI-PTE" },
	{ 0xD4319D, "Sinwatec", "SINWATEC" },
	{ 0xD43266, "Fike Corporation", "FIKE" },
	{ 0xD437D7, "zte corporation", "ZTE" },
	{ 0xD43A65, "IGRS Engineering Lab Ltd.", "IGRS-ENGINEERING-LAB" },
	{ 0xD43AE9, "DONGGUAN ipt INDUSTRIAL CO., LTD", "DONGGUAN-IPT-INDUSTRIAL" },
	{ 0xD43D67, "Carma Industries Inc.", "CARMA-INDUSTRIES" },
	{ 0xD43D7E, "Micro-Star Int'l Co, Ltd", "MICRO-STAR" },
	{ 0xD443A8, "Changzhou Haojie Electric Co., Ltd.", "CHANGZHOU-HAOJIE-ELECTRIC" },
	{ 0xD44B5E, "TAIYO YUDEN CO., LTD.", "TAIYO-YUDEN" },
	{ 0xD44C24, "Vuppalamritha Magnetic Components LTD", "VUPPALAMRITHA-MAGNETIC-COMPONENTS" },
	{ 0xD44C9C, "Shenzhen YOOBAO Technology Co.Ltd", "SHENZHEN-YOOBAO" },
	{ 0xD44CA7, "Informtekhnika & Communication, LLC", "INFORMTEKHNIKA-COMMUNICATION-LLC" },
	{ 0xD44F80, "Kemper Digital GmbH", "KEMPER-DIGITAL" },
	{ 0xD4507A, "CEIVA Logic, Inc", "CEIVA-LOGIC" },
	{ 0xD4522A, "TangoWiFi.com", "TANGOWIFI-COM" },
	{ 0xD45251, "IBT Ingenieurbureau Broennimann Thun", "IBT-INGENIEURBUREAU-BROENNIMANN-THUN" },
	{ 0xD45297, "nSTREAMS Technologies, Inc.", "NSTREAMS-TECHNOLOGIES" },
	{ 0xD453AF, "VIGO System S.A.", "VIGO-SYSTEM" },
	{ 0xD45556, "Fiber Mountain Inc.", "FIBER-MOUNTAIN" },
	{ 0xD45AB2, "Galleon Systems", "GALLEON" },
	{ 0xD45C70, "Wi-Fi Alliance", "WI-FI-ALLIANCE" },
	{ 0xD45D42, "Nokia Corporation", "NOKIA" },
	{ 0xD46132, "Pro Concept Manufacturer Co.,Ltd.", "PRO-CONCEPT-MANUFACTURER" },
	{ 0xD464F7, "CHENGDU USEE DIGITAL TECHNOLOGY CO., LTD", "CHENGDU-USEE-DIGITAL" },
	{ 0xD466A8, "Riedo Networks GmbH", "RIEDO-NETWORKS" },
	{ 0xD46761, "SAHAB TECHNOLOGY", "SAHAB" },
	{ 0xD467E7, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0xD4684D, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0xD46867, "Neoventus Design Group", "NEOVENTUS-DESIGN-GROUP" },
	{ 0xD468BA, "Shenzhen Sundray Technologies Company Limited", "SHENZHEN-SUNDRAY-TECHNOLOGIES-COMPANY" },
	{ 0xD46A91, "Snap AV", "SNAP-AV" },
	{ 0xD46AA8, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xD46CBF, "Goodrich ISR", "GOODRICH-ISR" },
	{ 0xD46CDA, "CSM GmbH", "CSM" },
	{ 0xD46D50, "Cisco", "CISCO" },
	{ 0xD46E5C, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xD46F42, "WAXESS USA Inc", "WAXESS-USA" },
	{ 0xD479C3, "Cameronet GmbH & Co. KG", "CAMERONET" },
	{ 0xD47B35, "NEO Monitors AS", "NEO-MONITORS-AS" },
	{ 0xD47B75, "HARTING Electronics GmbH", "HARTING-ELECTRONICS" },
	{ 0xD481CA, "iDevices, LLC", "IDEVICES-LLC" },
	{ 0xD4823E, "Argosy Technologies, Ltd.", "ARGOSY-TECHNOLOGIES" },
	{ 0xD48564, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0xD487D8, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xD48890, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xD48CB5, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xD48DD9, "Meld Technology, Inc", "MELD" },
	{ 0xD48F33, "Microsoft Corporation", "MICROSOFT" },
	{ 0xD48FAA, "Sogecam Industrial, S.A.", "SOGECAM-INDUSTRIAL" },
	{ 0xD491AF, "Electroacustica General Iberica, S.A.", "ELECTROACUSTICA-GENERAL-IBERICA" },
	{ 0xD49398, "Nokia Corporation", "NOKIA" },
	{ 0xD493A0, "Fidelix Oy", "FIDELIX-OY" },
	{ 0xD4945A, "COSMO CO., LTD", "COSMO" },
	{ 0xD494A1, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD49524, "Clover Network, Inc.", "CLOVER-NETWORK" },
	{ 0xD496DF, "SUNGJIN C&T CO.,LTD", "SUNGJIN-C-T" },
	{ 0xD4970B, "XIAOMI CORPORATION", "XIAOMI" },
	{ 0xD49A20, "Apple", "APPLE" },
	{ 0xD49C28, "JayBird Gear LLC", "JAYBIRD-GEAR-LLC" },
	{ 0xD49C8E, "University of FUKUI", "UNIVERSITY-OF-FUKUI" },
	{ 0xD49E6D, "Wuhan Zhongyuan Huadian Science & Technology Co.,", "WUHAN-ZHONGYUAN-HUADIAN-SCIENCE" },
	{ 0xD4A02A, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xD4A425, "SMAX Technology Co., Ltd.", "SMAX" },
	{ 0xD4A499, "InView Technology Corporation", "INVIEW" },
	{ 0xD4A928, "GreenWave Reality Inc", "GREENWAVE-REALITY" },
	{ 0xD4AAFF, "MICRO WORLD", "MICRO-WORLD" },
	{ 0xD4AC4E, "BODi rS, LLC", "BODI-RS-LLC" },
	{ 0xD4AD2D, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0xD4AE52, "Dell Inc", "DELL" },
	{ 0xD4B110, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xD4B43E, "Messcomp Datentechnik GmbH", "MESSCOMP-DATENTECHNIK" },
	{ 0xD4BED9, "Dell Inc", "DELL" },
	{ 0xD4BF2D, "SE Controls Asia Pacific Ltd", "SE-CONTROLS-ASIA-PACIFIC" },
	{ 0xD4BF7F, "UPVEL", "UPVEL" },
	{ 0xD4C1FC, "Nokia Corporation", "NOKIA" },
	{ 0xD4C766, "Acentic GmbH", "ACENTIC" },
	{ 0xD4C9EF, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xD4CA6D, "Routerboard.com", "ROUTERBOARD-COM" },
	{ 0xD4CA6E, "u-blox AG", "U-BLOX" },
	{ 0xD4CBAF, "Nokia Corporation", "NOKIA" },
	{ 0xD4CEB8, "Enatel LTD", "ENATEL" },
	{ 0xD4CFF9, "Shenzhen Sen5 Technology Co., Ltd.", "SHENZHEN-SEN5" },
	{ 0xD4D184, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0xD4D249, "Power Ethernet", "POWER-ETHERNET" },
	{ 0xD4D50D, "Southwest Microwave, Inc", "SOUTHWEST-MICROWAVE" },
	{ 0xD4D748, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xD4D7A9, "Shanghai Kaixiang Info Tech LTD", "SHANGHAI-KAIXIANG-INFO-TECH" },
	{ 0xD4D898, "Korea CNO Tech Co., Ltd", "KOREA-CNO-TECH" },
	{ 0xD4D919, "GoPro", "GOPRO" },
	{ 0xD4DF57, "Alpinion Medical Systems", "ALPINION-MEDICAL" },
	{ 0xD4E08E, "ValueHD Corporation", "VALUEHD" },
	{ 0xD4E32C, "S. Siedle & Sohne", "S-SIEDLE-SOHNE" },
	{ 0xD4E33F, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0xD4E8B2, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xD4EA0E, "Avaya, Inc", "AVAYA" },
	{ 0xD4EC0C, "Harley-Davidson Motor Company", "HARLEY-DAVIDSON-MOTOR-COMPANY" },
	{ 0xD4EC86, "LinkedHope Intelligent Technologies Co., Ltd", "LINKEDHOPE-INTELLIGENT-TECHNOLOGIES" },
	{ 0xD4EE07, "HIWIFI Co., Ltd.", "HIWIFI" },
	{ 0xD4F027, "Navetas Energy Management", "NAVETAS-ENERGY-MANAGEMENT" },
	{ 0xD4F0B4, "Napco Security Technologies", "NAPCO-SECURITY-TECHNOLOGIES" },
	{ 0xD4F143, "IPROAD.,Inc", "IPROAD" },
	{ 0xD4F46F, "Apple", "APPLE" },
	{ 0xD4F513, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD4F63F, "IEA S.R.L.", "IEA-S-R-L" },
	{ 0xD4F9A1, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xD8004D, "Apple", "APPLE" },
	{ 0xD8052E, "Skyviia Corporation", "SKYVIIA" },
	{ 0xD806D1, "Honeywell Fire System (Shanghai) Co,. Ltd.", "HONEYWELL-FIRE-SYSTEM-SHANGHAI" },
	{ 0xD808F5, "Arcadia Networks Co. Ltd.", "ARCADIA-NETWORKS" },
	{ 0xD809C3, "Cercacor Labs", "CERCACOR-LABS" },
	{ 0xD80CCF, "C.G.V. S.A.S.", "C-G-V-S" },
	{ 0xD80DE3, "FXI TECHNOLOGIES AS", "FXI-TECHNOLOGIES-AS" },
	{ 0xD8150D, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xD8160A, "Nippon Electro-Sensory Devices", "NIPPON-ELECTRO-SENSORY-DEVICES" },
	{ 0xD8182B, "Conti Temic Microelectronic GmbH", "CONTI-TEMIC-MICROELECTRONIC" },
	{ 0xD819CE, "Telesquare", "TELESQUARE" },
	{ 0xD81BFE, "TWINLINX CORPORATION", "TWINLINX" },
	{ 0xD81C14, "Compacta International, Ltd.", "COMPACTA" },
	{ 0xD81D72, "Apple", "APPLE" },
	{ 0xD81EDE, "B&W Group Ltd", "B-W-GROUP" },
	{ 0xD824BD, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xD82522, "Pace plc", "PACE-PLC" },
	{ 0xD826B9, "Guangdong Coagent Electronics S &T Co., Ltd.", "GUANGDONG-COAGENT-ELECTRONICS-S-T" },
	{ 0xD8270C, "MaxTronic International Co., Ltd.", "MAXTRONIC" },
	{ 0xD828C9, "General Electric Consumer and Industrial", "GENERAL-ELECTRIC-CONSUMER-AND-INDUSTRIAL" },
	{ 0xD82916, "Ascent Communication Technology", "ASCENT-COMMUNICATION" },
	{ 0xD82986, "Best Wish Technology LTD", "BEST-WISH" },
	{ 0xD82A15, "Leitner SpA", "LEITNER-SPA" },
	{ 0xD82A7E, "Nokia Corporation", "NOKIA" },
	{ 0xD82D9B, "Shenzhen G.Credit Communication Technology Co., Ltd", "SHENZHEN-G-CREDIT-COMMUNICATION" },
	{ 0xD82DE1, "Tricascade Inc.", "TRICASCADE" },
	{ 0xD83062, "Apple", "APPLE" },
	{ 0xD831CF, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xD8337F, "Office FA.com Co.,Ltd.", "OFFICE-FA-COM" },
	{ 0xD83C69, "Tinno Mobile Technology Corp", "TINNO-MOBILE" },
	{ 0xD842AC, "Shanghai Feixun Communication Co.,Ltd.", "SHANGHAI-FEIXUN-COMMUNICATION" },
	{ 0xD84606, "Silicon Valley Global Marketing", "SILICON-VALLEY-GLOBAL-MARKETING" },
	{ 0xD8490B, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xD8492F, "CANON INC.", "CANON" },
	{ 0xD84A87, "OI ELECTRIC CO.,LTD", "OI-ELECTRIC" },
	{ 0xD84B2A, "Cognitas Technologies, Inc.", "COGNITAS-TECHNOLOGIES" },
	{ 0xD850E6, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xD8543A, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD854A2, "Aerohive Networks Inc.", "AEROHIVE-NETWORKS" },
	{ 0xD855A3, "zte corporation", "ZTE" },
	{ 0xD857EF, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xD858D7, "CZ.NIC, z.s.p.o.", "CZ-NIC-Z-S-P-O" },
	{ 0xD85D4C, "TP-LINK Technologies Co.,Ltd.", "TP-LINK-TECHNOLOGIES" },
	{ 0xD85D84, "CAx soft GmbH", "CAX-SOFT" },
	{ 0xD85DE2, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xD85DFB, "PRIVATE", "PRIVATE" },
	{ 0xD86194, "Objetivos y Sevicios de Valor Añadido", "OBJETIVOS-Y-SEVICIOS-DE-VALOR-AÑADIDO" },
	{ 0xD862DB, "Eno Inc.", "ENO" },
	{ 0xD86595, "Toy's Myth Inc.", "TOY-S-MYTH" },
	{ 0xD866C6, "Shenzhen Daystar Technology Co.,ltd", "SHENZHEN-DAYSTAR" },
	{ 0xD866EE, "BOXIN COMMUNICATION CO.,LTD.", "BOXIN-COMMUNICATION" },
	{ 0xD867D9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xD86960, "Steinsvik", "STEINSVIK" },
	{ 0xD86BF7, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0xD86CE9, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0xD87157, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0xD87495, "zte corporation", "ZTE" },
	{ 0xD87533, "Nokia Corporation", "NOKIA" },
	{ 0xD8760A, "Escort, Inc.", "ESCORT" },
	{ 0xD878E5, "KUHN SA", "KUHN" },
	{ 0xD87988, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xD87CDD, "SANIX INCORPORATED", "SANIX-INCORPORATED" },
	{ 0xD87EB1, "x.o.ware, inc.", "X-O-WARE" },
	{ 0xD88039, "Microchip Technology Inc.", "MICROCHIP" },
	{ 0xD881CE, "AHN INC.", "AHN" },
	{ 0xD88466, "Extreme Networks", "EXTREME-NETWORKS" },
	{ 0xD88A3B, "UNIT-EM", "UNIT-EM" },
	{ 0xD88D5C, "Elentec", "ELENTEC" },
	{ 0xD890E8, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xD89341, "General Electric Global Research", "GENERAL-ELECTRIC-GLOBAL-RESEARCH" },
	{ 0xD8952F, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD89685, "GoPro", "GOPRO" },
	{ 0xD89695, "Apple", "APPLE" },
	{ 0xD896E0, "Alibaba Cloud Computing Ltd.", "ALIBABA-CLOUD-COMPUTING" },
	{ 0xD8973B, "Artesyn Embedded Technologies", "ARTESYN-EMBEDDED-TECHNOLOGIES" },
	{ 0xD89760, "C2 Development, Inc.", "C2-DEVELOPMENT" },
	{ 0xD8977C, "Grey Innovation", "GREY-INNOVATION" },
	{ 0xD897BA, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0xD89D67, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xD89DB9, "eMegatech International Corp.", "EMEGATECH" },
	{ 0xD89E3F, "Apple", "APPLE" },
	{ 0xD8A25E, "Apple", "APPLE" },
	{ 0xD8ADDD, "Sonavation, Inc.", "SONAVATION" },
	{ 0xD8AE90, "Itibia Technologies", "ITIBIA-TECHNOLOGIES" },
	{ 0xD8AF3B, "Hangzhou Bigbright Integrated communications system Co.,Ltd", "HANGZHOU-BIGBRIGHT-INTEGRATED-COMMUNICATION-SYSTEM" },
	{ 0xD8AFF1, "Panasonic Appliances Company", "PANASONIC-APPLIANCES-COMPANY" },
	{ 0xD8B02E, "Guangzhou Zonerich Business Machine Co., Ltd", "GUANGZHOU-ZONERICH-BUSINESS-MACHINE" },
	{ 0xD8B04C, "Jinan USR IOT Technology Co., Ltd.", "JINAN-USR-IOT" },
	{ 0xD8B12A, "Panasonic Mobile Communications Co., Ltd.", "PANASONIC-MOBILE-COMMUNICATION" },
	{ 0xD8B377, "HTC Corporation", "HTC" },
	{ 0xD8B6B7, "Comtrend Corporation", "COMTREND" },
	{ 0xD8B6C1, "NetworkAccountant, Inc.", "NETWORKACCOUNTANT" },
	{ 0xD8B6D6, "Blu Tether Limited", "BLU-TETHER" },
	{ 0xD8B8F6, "Nantworks", "NANTWORKS" },
	{ 0xD8B90E, "Triple Domain Vision Co.,Ltd.", "TRIPLE-DOMAIN-VISION" },
	{ 0xD8BB2C, "Apple", "APPLE" },
	{ 0xD8BF4C, "Victory Concept Electronics Limited", "VICTORY-CONCEPT-ELECTRONICS" },
	{ 0xD8C068, "Netgenetech.co.,ltd.", "NETGENETECH" },
	{ 0xD8C3FB, "DETRACOM", "DETRACOM" },
	{ 0xD8C691, "Hichan Technology Corp.", "HICHAN" },
	{ 0xD8C7C8, "Aruba Networks", "ARUBA-NETWORKS" },
	{ 0xD8C99D, "EA DISPLAY LIMITED", "EA-DISPLAY" },
	{ 0xD8CB8A, "Micro-Star INTL CO., LTD.", "MICRO-STAR-INTL" },
	{ 0xD8CF9C, "Apple", "APPLE" },
	{ 0xD8D1CB, "Apple", "APPLE" },
	{ 0xD8D27C, "JEMA ENERGY, SA", "JEMA-ENERGY" },
	{ 0xD8D385, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0xD8D43C, "Sony Corporation", "SONY" },
	{ 0xD8D5B9, "Rainforest Automation, Inc.", "RAINFOREST-AUTOMATION" },
	{ 0xD8D67E, "GSK CNC EQUIPMENT CO.,LTD", "GSK-CNC-EQUIPMENT" },
	{ 0xD8DA52, "APATOR S.A.", "APATOR" },
	{ 0xD8DCE9, "Kunshan Erlab ductless filtration system Co.,Ltd", "KUNSHAN-ERLAB-DUCTLESS-FILTRATION-SYSTEM" },
	{ 0xD8DD5F, "BALMUDA Inc.", "BALMUDA" },
	{ 0xD8DDFD, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xD8DECE, "ISUNG CO.,LTD", "ISUNG" },
	{ 0xD8DF0D, "beroNet GmbH", "BERONET" },
	{ 0xD8E3AE, "CIRTEC MEDICAL SYSTEMS", "CIRTEC-MEDICAL" },
	{ 0xD8E56D, "TCT Mobile Limited", "TCT-MOBILE" },
	{ 0xD8E72B, "NetScout Systems, Inc.", "NETSCOUT" },
	{ 0xD8E743, "Wush, Inc", "WUSH" },
	{ 0xD8E952, "KEOPSYS", "KEOPSYS" },
	{ 0xD8EB97, "TRENDnet, Inc.", "TRENDNET" },
	{ 0xD8EE78, "Moog Protokraft", "MOOG-PROTOKRAFT" },
	{ 0xD8F0F2, "Zeebo Inc", "ZEEBO" },
	{ 0xD8F710, "Libre Wireless Technologies Inc.", "LIBRE-WIRELESS-TECHNOLOGIES" },
	{ 0xD8FB11, "AXACORE", "AXACORE" },
	{ 0xD8FC93, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xD8FE8F, "IDFone Co., Ltd.", "IDFONE" },
	{ 0xD8FEE3, "D-Link International", "D-LINK" },
	{ 0xDC0265, "Meditech Kft", "MEDITECH-KFT" },
	{ 0xDC028E, "zte corporation", "ZTE" },
	{ 0xDC052F, "National Products Inc.", "NATIONAL-PRODUCTS" },
	{ 0xDC0575, "SIEMENS ENERGY AUTOMATION", "SIEMENS-ENERGY-AUTOMATION" },
	{ 0xDC05ED, "Nabtesco Corporation", "NABTESCO" },
	{ 0xDC07C1, "HangZhou QiYang Technology Co.,Ltd.", "HANGZHOU-QIYANG" },
	{ 0xDC0914, "Talk-A-Phone Co.", "TALK-A-PHONE" },
	{ 0xDC0B1A, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0xDC0EA1, "COMPAL INFORMATION (KUNSHAN) CO., LTD", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0xDC15DB, "Ge Ruili Intelligent Technology ( Beijing ) Co., Ltd.", "GE-RUILI-INTELLIGENT-BEIJING" },
	{ 0xDC16A2, "Medtronic Diabetes", "MEDTRONIC-DIABETES" },
	{ 0xDC175A, "Hitachi High-Technologies Corporation", "HITACHI-HIGH-TECHNOLOGIES" },
	{ 0xDC1792, "Captivate Network", "CAPTIVATE-NETWORK" },
	{ 0xDC1D9F, "U & B tech", "U-B-TECH" },
	{ 0xDC1DD4, "Microstep-MIS spol. s r.o.", "MICROSTEP-MIS-SPOL-S-R-O" },
	{ 0xDC1EA3, "Accensus LLC", "ACCENSUS-LLC" },
	{ 0xDC2008, "ASD Electronics Ltd", "ASD-ELECTRONICS" },
	{ 0xDC2A14, "Shanghai Longjing Technology Co.", "SHANGHAI-LONGJING" },
	{ 0xDC2B61, "Apple", "APPLE" },
	{ 0xDC2B66, "InfoBLOCK S.A. de C.V.", "INFOBLOCK-DE-C-V" },
	{ 0xDC2BCA, "Zera GmbH", "ZERA" },
	{ 0xDC2C26, "Iton Technology Limited", "ITON" },
	{ 0xDC2E6A, "HCT. Co., Ltd.", "HCT" },
	{ 0xDC2F03, "Step forward Group Co., Ltd.", "STEP-FORWARD-GROUP" },
	{ 0xDC309C, "Heyrex Limited", "HEYREX" },
	{ 0xDC3350, "TechSAT GmbH", "TECHSAT" },
	{ 0xDC3714, "Apple, Inc.", "APPLE" },
	{ 0xDC37D2, "Hunan HKT Electronic Technology Co., Ltd", "HUNAN-HKT" },
	{ 0xDC38E1, "Juniper networks", "JUNIPER-NETWORKS" },
	{ 0xDC3979, "Skyport Systems", "SKYPORT" },
	{ 0xDC3A5E, "Roku, Inc", "ROKU" },
	{ 0xDC3C2E, "Manufacturing System Insights, Inc.", "MANUFACTURING-SYSTEM-INSIGHTS" },
	{ 0xDC3C84, "Ticom Geomatics, Inc.", "TICOM-GEOMATICS" },
	{ 0xDC3E51, "Solberg & Andersen AS", "SOLBERG-ANDERSEN-AS" },
	{ 0xDC3EF8, "Nokia Corporation", "NOKIA" },
	{ 0xDC4517, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xDC49C9, "CASCO SIGNAL LTD", "CASCO-SIGNAL" },
	{ 0xDC4EDE, "SHINYEI TECHNOLOGY CO., LTD.", "SHINYEI" },
	{ 0xDC537C, "Compal Broadband Networks, Inc.", "COMPAL-BROADBAND-NETWORKS" },
	{ 0xDC56E6, "Shenzhen Bococom Technology Co.,LTD", "SHENZHEN-BOCOCOM" },
	{ 0xDC5726, "Power-One", "POWER-ONE" },
	{ 0xDC5E36, "Paterson Technology", "PATERSON" },
	{ 0xDC60A1, "Teledyne DALSA Professional Imaging", "TELEDYNE-DALSA-PROFESSIONAL-IMAGING" },
	{ 0xDC647C, "C.R.S. iiMotion GmbH", "C-R-S-IIMOTION" },
	{ 0xDC663A, "Apacer Technology Inc.", "APACER" },
	{ 0xDC6F00, "Livescribe, Inc.", "LIVESCRIBE" },
	{ 0xDC6F08, "Bay Storage Technology", "BAY-STORAGE" },
	{ 0xDC7014, "PRIVATE", "PRIVATE" },
	{ 0xDC7144, "Samsung Electro Mechanics", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xDC7B94, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xDC825B, "JANUS, spol. s r.o.", "JANUS-SPOL-S-R-O" },
	{ 0xDC85DE, "Azurewave Technologies., inc.", "AZUREWAVE-TECHNOLOGIES" },
	{ 0xDC86D8, "Apple, Inc", "APPLE" },
	{ 0xDC9B1E, "Intercom, Inc.", "INTERCOM" },
	{ 0xDC9B9C, "Apple", "APPLE" },
	{ 0xDC9C52, "Sapphire Technology Limited.", "SAPPHIRE" },
	{ 0xDC9FA4, "Nokia Corporation", "NOKIA" },
	{ 0xDC9FDB, "Ubiquiti Networks, Inc.", "UBIQUITI-NETWORKS" },
	{ 0xDCA5F4, "Cisco", "CISCO" },
	{ 0xDCA6BD, "Beijing Lanbo Technology Co., Ltd.", "BEIJING-LANBO" },
	{ 0xDCA7D9, "Compressor Controls Corp", "COMPRESSOR-CONTROLS" },
	{ 0xDCA8CF, "New Spin Golf, LLC.", "NEW-SPIN-GOLF-LLC" },
	{ 0xDCA971, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xDCA989, "MACANDC", "MACANDC" },
	{ 0xDCAD9E, "GreenPriz", "GREENPRIZ" },
	{ 0xDCAE04, "CELOXICA Ltd", "CELOXICA" },
	{ 0xDCB058, "Burkert Werke GmbH", "BURKERT-WERKE" },
	{ 0xDCB4C4, "Microsoft XCG", "MICROSOFT-XCG" },
	{ 0xDCBF90, "HUIZHOU QIAOXING TELECOMMUNICATION INDUSTRY CO.,LTD.", "HUIZHOU-QIAOXING-TELECOMMUNICATION-INDUSTRY" },
	{ 0xDCC0DB, "Shenzhen Kaiboer Technology Co., Ltd.", "SHENZHEN-KAIBOER" },
	{ 0xDCC101, "SOLiD Technologies, Inc.", "SOLID-TECHNOLOGIES" },
	{ 0xDCC422, "Systembase Limited", "SYSTEMBASE" },
	{ 0xDCC622, "BUHEUNG SYSTEM", "BUHEUNG-SYSTEM" },
	{ 0xDCC793, "Nokia Corporation", "NOKIA" },
	{ 0xDCCBA8, "Explora Technologies Inc", "EXPLORA-TECHNOLOGIES" },
	{ 0xDCCE41, "FE GLOBAL HONG KONG LIMITED", "FE-GLOBAL-HONG-KONG" },
	{ 0xDCCEBC, "Shenzhen JSR Technology Co.,Ltd.", "SHENZHEN-JSR" },
	{ 0xDCCF94, "Beijing Rongcheng Hutong Technology Co., Ltd.", "BEIJING-RONGCHENG-HUTONG" },
	{ 0xDCD0F7, "Bentek Systems Ltd.", "BENTEK" },
	{ 0xDCD2FC, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xDCD321, "HUMAX co.,tld", "HUMAX-TLD" },
	{ 0xDCD52A, "Sunny Heart Limited", "SUNNY-HEART" },
	{ 0xDCD87F, "Shenzhen JoinCyber Telecom Equipment Ltd", "SHENZHEN-JOINCYBER-TELECOM-EQUIPMENT" },
	{ 0xDCDA4F, "GETCK TECHNOLOGY, INC", "GETCK" },
	{ 0xDCDECA, "Akyllor", "AKYLLOR" },
	{ 0xDCE026, "Patrol Tag, Inc", "PATROL-TAG" },
	{ 0xDCE1AD, "Shenzhen Wintop Photoelectric Technology Co., Ltd", "SHENZHEN-WINTOP-PHOTOELECTRIC" },
	{ 0xDCE2AC, "Lumens Digital Optics Inc.", "LUMENS-DIGITAL-OPTICS" },
	{ 0xDCE578, "Experimental Factory of Scientific Engineering and Special Design Department", "EXPERIMENTAL-FACTORY-OF-SCIENTIFIC-ENGINEERING-AND-SPECIAL-DESIGN-DEPARTMENT" },
	{ 0xDCE71C, "AUG Elektronik GmbH", "AUG-ELEKTRONIK" },
	{ 0xDCEC06, "Heimi Network Technology Co., Ltd.", "HEIMI-NETWORK" },
	{ 0xDCF05D, "Letta Teknoloji", "LETTA-TEKNOLOJI" },
	{ 0xDCF110, "Nokia Corporation", "NOKIA" },
	{ 0xDCF755, "SITRONIK", "SITRONIK" },
	{ 0xDCF858, "Lorent Networks, Inc.", "LORENT-NETWORKS" },
	{ 0xDCFAD5, "STRONG Ges.m.b.H.", "STRONG-GES-M-B-H" },
	{ 0xDCFB02, "Buffalo Inc.", "BUFFALO" },
	{ 0xE005C5, "TP-LINK Technologies Co.,Ltd.", "TP-LINK-TECHNOLOGIES" },
	{ 0xE006E6, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xE00B28, "Inovonics", "INOVONICS" },
	{ 0xE00C7F, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0xE00DB9, "PRIVATE", "PRIVATE" },
	{ 0xE0107F, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0xE0143E, "Modoosis Inc.", "MODOOSIS" },
	{ 0xE01877, "Fujitsu Limited", "FUJITSU" },
	{ 0xE0191D, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xE01C41, "Aerohive Networks Inc.", "AEROHIVE-NETWORKS" },
	{ 0xE01CEE, "Bravo Tech, Inc.", "BRAVO-TECH" },
	{ 0xE01D38, "Beijing HuaqinWorld Technology Co.,Ltd", "BEIJING-HUAQINWORLD" },
	{ 0xE01D3B, "Cambridge Industries(Group) Co.,Ltd", "CAMBRIDGE-INDUSTRIES-GROUP" },
	{ 0xE01E07, "Anite Telecoms US. Inc", "ANITE-TELECOMS-US" },
	{ 0xE01F0A, "Xslent Energy Technologies. LLC", "XSLENT-ENERGY-TECHNOLOGIES-LLC" },
	{ 0xE0247F, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xE02538, "Titan Pet Products", "TITAN-PET-PRODUCTS" },
	{ 0xE02630, "Intrigue Technologies, Inc.", "INTRIGUE-TECHNOLOGIES" },
	{ 0xE02636, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0xE0271A, "TTC Next-generation Home Network System WG", "TTC-NEXT-GENERATION-HOME-NETWORK-SYSTEM-WG" },
	{ 0xE02A82, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0xE02F6D, "Cisco", "CISCO" },
	{ 0xE03005, "Alcatel-Lucent Shanghai Bell Co., Ltd", "ALCATEL-LUCENT-SHANGHAI-BELL" },
	{ 0xE031D0, "SZ Telstar CO., LTD", "SZ-TELSTAR" },
	{ 0xE03560, "Challenger Supply Holdings, LLC", "CHALLENGER-SUPPLY-HOLDINGS-LLC" },
	{ 0xE036E3, "Stage One International Co., Ltd.", "STAGE-ONE" },
	{ 0xE039D7, "Plexxi, Inc.", "PLEXXI" },
	{ 0xE03C5B, "SHENZHEN JIAXINJIE ELECTRON CO.,LTD", "SHENZHEN-JIAXINJIE-ELECTRON" },
	{ 0xE03E44, "Broadcom Corporation", "BROADCOM" },
	{ 0xE03E4A, "Cavanagh Group International", "CAVANAGH-GROUP" },
	{ 0xE03E7D, "data-complex GmbH", "DATA-COMPLEX" },
	{ 0xE03F49, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xE0469A, "NETGEAR", "NETGEAR" },
	{ 0xE04B45, "Hi-P Electronics Pte Ltd", "HI-P-ELECTRONICS-PTE" },
	{ 0xE05597, "Emergent Vision Technologies Inc.", "EMERGENT-VISION-TECHNOLOGIES" },
	{ 0xE056F4, "AxesNetwork Solutions inc.", "AXESNETWORK-SOLUTIONS" },
	{ 0xE0589E, "Laerdal Medical", "LAERDAL-MEDICAL" },
	{ 0xE05B70, "Innovid, Co., Ltd.", "INNOVID" },
	{ 0xE05DA6, "Detlef Fink Elektronik & Softwareentwicklung", "DETLEF-FINK-ELEKTRONIK-SOFTWAREENTWICKLUNG" },
	{ 0xE05FB9, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xE061B2, "HANGZHOU ZENOINTEL TECHNOLOGY CO., LTD", "HANGZHOU-ZENOINTEL" },
	{ 0xE06290, "Jinan Jovision Science & Technology Co., Ltd.", "JINAN-JOVISION-SCIENCE" },
	{ 0xE063E5, "Sony Mobile Communications AB", "SONY-MOBILE-COMMUNICATION-AB" },
	{ 0xE064BB, "DigiView S.r.l.", "DIGIVIEW-S-R-L" },
	{ 0xE06678, "Apple", "APPLE" },
	{ 0xE067B3, "C-Data Technology Co., Ltd", "C-DATA" },
	{ 0xE06995, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0xE0750A, "ALPS ERECTORIC CO.,LTD.", "ALPS-ERECTORIC" },
	{ 0xE0757D, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0xE076D0, "AMPAK Technology, Inc.", "AMPAK" },
	{ 0xE07C62, "Whistle Labs, Inc.", "WHISTLE-LABS" },
	{ 0xE07F53, "TECHBOARD SRL", "TECHBOARD-SRL" },
	{ 0xE07F88, "EVIDENCE Network SIA", "EVIDENCE-NETWORK-SIA" },
	{ 0xE08177, "GreenBytes, Inc.", "GREENBYTES" },
	{ 0xE087B1, "Nata-Info Ltd.", "NATA-INFO" },
	{ 0xE0885D, "Technicolor CH USA Inc", "TECHNICOLOR-CH-USA" },
	{ 0xE0899D, "Cisco", "CISCO" },
	{ 0xE08A7E, "Exponent", "EXPONENT" },
	{ 0xE08E3C, "Aztech Electronics Pte Ltd", "AZTECH-ELECTRONICS-PTE" },
	{ 0xE08FEC, "REPOTEC CO., LTD.", "REPOTEC" },
	{ 0xE09153, "XAVi Technologies Corp.", "XAVI-TECHNOLOGIES" },
	{ 0xE091F5, "NETGEAR", "NETGEAR" },
	{ 0xE09467, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xE09579, "ORTHOsoft inc, d/b/a Zimmer CAS", "ORTHOSOFT-D/B/A-ZIMMER-CAS" },
	{ 0xE09796, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xE097F2, "Atomax Inc.", "ATOMAX" },
	{ 0xE09971, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE09D31, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xE09DB8, "PLANEX COMMUNICATIONS INC.", "PLANEX-COMMUNICATION" },
	{ 0xE0A198, "NOJA Power Switchgear Pty Ltd", "NOJA-POWER-SWITCHGEAR-PTY" },
	{ 0xE0A1D7, "SFR", "SFR" },
	{ 0xE0A30F, "Pevco", "PEVCO" },
	{ 0xE0A670, "Nokia Corporation", "NOKIA" },
	{ 0xE0AAB0, "GENERAL VISION ELECTRONICS CO. LTD.", "GENERAL-VISION-ELECTRONICS" },
	{ 0xE0ABFE, "Orb Networks, Inc.", "ORB-NETWORKS" },
	{ 0xE0ACF1, "Cisco", "CISCO" },
	{ 0xE0AE5E, "ALPS Co,. Ltd.", "ALPS" },
	{ 0xE0AEB2, "Bender GmbH &amp; Co.KG", "BENDER-AMP" },
	{ 0xE0AEED, "LOENK", "LOENK" },
	{ 0xE0AF4B, "Pluribus Networks, Inc.", "PLURIBUS-NETWORKS" },
	{ 0xE0B2F1, "FN-LINK TECHNOLOGY LIMITED", "FN-LINK" },
	{ 0xE0B52D, "Apple", "APPLE" },
	{ 0xE0B70A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xE0B7B1, "Pace plc", "PACE-PLC" },
	{ 0xE0B9A5, "Azurewave", "AZUREWAVE" },
	{ 0xE0B9BA, "Apple", "APPLE" },
	{ 0xE0BC43, "C2 Microsystems, Inc.", "C2-MICROSYSTEMS" },
	{ 0xE0C286, "Aisai Communication Technology Co., Ltd.", "AISAI-COMMUNICATION" },
	{ 0xE0C2B7, "Masimo Corporation", "MASIMO" },
	{ 0xE0C3F3, "ZTE Corporation", "ZTE" },
	{ 0xE0C6B3, "MilDef AB", "MILDEF-AB" },
	{ 0xE0C79D, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xE0C86A, "SHENZHEN TW-SCIE Co., Ltd", "SHENZHEN-TW-SCIE" },
	{ 0xE0C922, "Jireh Energy Tech., Ltd.", "JIREH-ENERGY-TECH" },
	{ 0xE0C97A, "Apple", "APPLE" },
	{ 0xE0CA4D, "Shenzhen Unistar Communication Co.,LTD", "SHENZHEN-UNISTAR-COMMUNICATION" },
	{ 0xE0CA94, "Askey Computer", "ASKEY" },
	{ 0xE0CB1D, "PRIVATE", "PRIVATE" },
	{ 0xE0CB4E, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xE0CBEE, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE0CEC3, "ASKEY COMPUTER CORP", "ASKEY" },
	{ 0xE0CF2D, "Gemintek Corporation", "GEMINTEK" },
	{ 0xE0D10A, "Katoudenkikougyousyo co ltd", "KATOUDENKIKOUGYOUSYO" },
	{ 0xE0D173, "Cisco", "CISCO" },
	{ 0xE0D1E6, "Aliph dba Jawbone", "ALIPH-DBA-JAWBONE" },
	{ 0xE0D31A, "EQUES Technology Co., Limited", "EQUES" },
	{ 0xE0D7BA, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xE0D9A2, "Hippih aps", "HIPPIH-APS" },
	{ 0xE0DADC, "JVC KENWOOD Corporation", "JVC-KENWOOD" },
	{ 0xE0DB55, "Dell Inc", "DELL" },
	{ 0xE0DB88, "Open Standard Digital-IF Interface for SATCOM Systems", "OPEN-STANDARD-DIGITAL-IF-INTERFACE-FOR-SATCOM" },
	{ 0xE0DCA0, "Siemens Electrical Apparatus Ltd., Suzhou Chengdu Branch", "SIEMENS-ELECTRICAL-APPARATUS-SUZHOU-CHENGDU-BRANCH" },
	{ 0xE0E5CF, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xE0E631, "SNB TECHNOLOGIES LIMITED", "SNB-TECHNOLOGIES" },
	{ 0xE0E751, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0xE0E8E8, "Olive Telecommunication Pvt. Ltd", "OLIVE-TELECOMMUNICATION-PVT" },
	{ 0xE0ED1A, "vastriver Technology Co., Ltd", "VASTRIVER" },
	{ 0xE0EDC7, "Shenzhen Friendcom Technology Development Co., Ltd", "SHENZHEN-FRIENDCOM-DEVELOPMENT" },
	{ 0xE0EE1B, "Panasonic Automotive Systems Company of America", "PANASONIC-AUTOMOTIVE-COMPANY-OF-AMERICA" },
	{ 0xE0EF25, "Lintes Technology Co., Ltd.", "LINTES" },
	{ 0xE0F211, "Digitalwatt", "DIGITALWATT" },
	{ 0xE0F379, "Vaddio", "VADDIO" },
	{ 0xE0F5C6, "Apple", "APPLE" },
	{ 0xE0F5CA, "CHENG UEI PRECISION INDUSTRY CO.,LTD.", "CHENG-UEI-PRECISION-INDUSTRY" },
	{ 0xE0F847, "Apple", "APPLE" },
	{ 0xE0F9BE, "Cloudena Corp.", "CLOUDENA" },
	{ 0xE0FAEC, "Platan sp. z o.o. sp. k.", "PLATAN-SP-Z-O-O-SP-K" },
	{ 0xE0FFF7, "Softiron Inc.", "SOFTIRON" },
	{ 0xE40439, "TomTom Software Ltd", "TOMTOM-SOFTWARE" },
	{ 0xE4115B, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xE41218, "ShenZhen Rapoo Technology Co., Ltd.", "SHENZHEN-RAPOO" },
	{ 0xE4121D, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE41289, "topsystem Systemhaus GmbH", "TOPSYSTEM-SYSTEMHAUS" },
	{ 0xE41C4B, "V2 TECHNOLOGY, INC.", "V2" },
	{ 0xE41D2D, "Mellanox Technologies, Inc.", "MELLANOX-TECHNOLOGIES" },
	{ 0xE41F13, "IBM Corp", "IBM" },
	{ 0xE42354, "SHENZHEN FUZHI SOFTWARE TECHNOLOGY CO.,LTD", "SHENZHEN-FUZHI-SOFTWARE" },
	{ 0xE425E7, "Apple", "APPLE" },
	{ 0xE425E9, "Color-Chip", "COLOR-CHIP" },
	{ 0xE42771, "Smartlabs", "SMARTLABS" },
	{ 0xE42AD3, "Magneti Marelli S.p.A. Powertrain", "MAGNETI-MARELLI-S-P-A-POWERTRAIN" },
	{ 0xE42C56, "Lilee Systems, Ltd.", "LILEE" },
	{ 0xE42D02, "TCT Mobile Limited", "TCT-MOBILE" },
	{ 0xE42F26, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0xE42FF6, "Unicore communication Inc.", "UNICORE-COMMUNICATION" },
	{ 0xE432CB, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE43593, "Hangzhou GoTo technology Co.Ltd", "HANGZHOU-GOTO" },
	{ 0xE435FB, "Sabre Technology (Hull) Ltd", "SABRE-HULL" },
	{ 0xE437D7, "HENRI DEPAEPE S.A.S.", "HENRI-DEPAEPE-S" },
	{ 0xE438F2, "Advantage Controls", "ADVANTAGE-CONTROLS" },
	{ 0xE43FA2, "Wuxi DSP Technologies Inc.", "WUXI-DSP-TECHNOLOGIES" },
	{ 0xE440E2, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE441E6, "Ottec Technology GmbH", "OTTEC" },
	{ 0xE446BD, "C&C TECHNIC TAIWAN CO., LTD.", "C-C-TECHNIC-TAIWAN" },
	{ 0xE448C7, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0xE44C6C, "Shenzhen Guo Wei Electronic Co,. Ltd.", "SHENZHEN-GUO-WEI" },
	{ 0xE44E18, "Gardasoft VisionLimited", "GARDASOFT-VISIONLIMITED" },
	{ 0xE44F29, "MA Lighting Technology GmbH", "MA-LIGHTING" },
	{ 0xE44F5F, "EDS Elektronik Destek San.Tic.Ltd.Sti", "EDS-ELEKTRONIK-DESTEK-SAN-TIC-STI" },
	{ 0xE455EA, "Dedicated Computing", "DEDICATED-COMPUTING" },
	{ 0xE45614, "Suttle Apparatus", "SUTTLE-APPARATUS" },
	{ 0xE457A8, "Stuart Manufacturing, Inc.", "STUART-MANUFACTURING" },
	{ 0xE458E7, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE45AA2, "vivo Mobile Communication Co., Ltd.", "VIVO-MOBILE-COMMUNICATION" },
	{ 0xE45D52, "Avaya, Inc", "AVAYA" },
	{ 0xE46449, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xE467BA, "Danish Interpretation Systems A/S", "DANISH-INTERPRETATION-A/S" },
	{ 0xE468A3, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xE4695A, "Dictum Health, Inc.", "DICTUM-HEALTH" },
	{ 0xE46C21, "messMa GmbH", "MESSMA" },
	{ 0xE47185, "Securifi Ltd", "SECURIFI" },
	{ 0xE4751E, "Getinge Sterilization AB", "GETINGE-STERILIZATION-AB" },
	{ 0xE47723, "zte corporation", "ZTE" },
	{ 0xE4776B, "AARTESYS AG", "AARTESYS" },
	{ 0xE477D4, "Minrray Industry Co.,Ltd", "MINRRAY-INDUSTRY" },
	{ 0xE47CF9, "Samsung Electronics Co., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0xE47D5A, "Beijing Hanbang Technology Corp.", "BEIJING-HANBANG" },
	{ 0xE47FB2, "Fujitsu Limited", "FUJITSU" },
	{ 0xE48184, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0xE481B3, "Shenzhen ACT Industrial Co.,Ltd.", "SHENZHEN-ACT-INDUSTRIAL" },
	{ 0xE48399, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xE48501, "Geberit International AG", "GEBERIT" },
	{ 0xE48AD5, "RF WINDOW CO., LTD.", "RF-WINDOW" },
	{ 0xE48B7F, "Apple", "APPLE" },
	{ 0xE48C0F, "Discovery Insure", "DISCOVERY-INSURE" },
	{ 0xE48D8C, "Routerboard.com", "ROUTERBOARD-COM" },
	{ 0xE49069, "Rockwell Automation", "ROCKWELL-AUTOMATION" },
	{ 0xE4907E, "Motorola Mobility LLC, a Lenovo Company", "MOTOROLA-MOBILITY-LLC-A-LENOVO-COMPANY" },
	{ 0xE492E7, "Gridlink Tech. Co.,Ltd.", "GRIDLINK-TECH" },
	{ 0xE492FB, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE4956E, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xE496AE, "ALTOGRAPHICS Inc.", "ALTOGRAPHICS" },
	{ 0xE497F0, "Shanghai VLC Technologies Ltd. Co.", "SHANGHAI-VLC-TECHNOLOGIES" },
	{ 0xE498D6, "Apple, Inc", "APPLE" },
	{ 0xE4A5EF, "TRON LINK ELECTRONICS CO., LTD.", "TRON-LINK-ELECTRONICS" },
	{ 0xE4A7FD, "Cellco Partnership", "CELLCO-PARTNERSHIP" },
	{ 0xE4AB46, "UAB Selteka", "UAB-SELTEKA" },
	{ 0xE4AD7D, "SCL Elements", "SCL-ELEMENTS" },
	{ 0xE4AFA1, "HES-SO", "HES-SO" },
	{ 0xE4B021, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE4BAD9, "360 Fly Inc.", "360-FLY" },
	{ 0xE4C146, "Objetivos y Servicios de Valor A", "OBJETIVOS-Y-SERVICIOS-DE-VALOR-A" },
	{ 0xE4C62B, "Airware", "AIRWARE" },
	{ 0xE4C63D, "Apple, Inc.", "APPLE" },
	{ 0xE4C6E6, "Mophie, LLC", "MOPHIE-LLC" },
	{ 0xE4C722, "Cisco", "CISCO" },
	{ 0xE4C806, "Ceiec Electric Technology Inc.", "CEIEC-ELECTRIC" },
	{ 0xE4CE70, "Health & Life co., Ltd.", "HEALTH-LIFE" },
	{ 0xE4CE8F, "Apple", "APPLE" },
	{ 0xE4D332, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xE4D3F1, "Cisco", "CISCO" },
	{ 0xE4D53D, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xE4D71D, "Oraya Therapeutics", "ORAYA-THERAPEUTICS" },
	{ 0xE4DD79, "En-Vision America, Inc.", "EN-VISION-AMERICA" },
	{ 0xE4E0C5, "Samsung Electronics Co., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0xE4E409, "LEIFHEIT AG", "LEIFHEIT" },
	{ 0xE4EC10, "Nokia Corporation", "NOKIA" },
	{ 0xE4EEFD, "MR&D Manufacturing", "MR-D-MANUFACTURING" },
	{ 0xE4F365, "Time-O-Matic, Inc.", "TIME-O-MATIC" },
	{ 0xE4F3E3, "Shanghai iComhome Co.,Ltd.", "SHANGHAI-ICOMHOME" },
	{ 0xE4F4C6, "NETGEAR", "NETGEAR" },
	{ 0xE4F7A1, "Datafox GmbH", "DATAFOX" },
	{ 0xE4F8EF, "Samsung Elec Co.,Ltd", "SAMSUNG-ELEC" },
	{ 0xE4F939, "Minxon Hotel Technology INC.", "MINXON-HOTEL" },
	{ 0xE4FA1D, "PAD Peripheral Advanced Design Inc.", "PAD-PERIPHERAL-ADVANCED-DESIGN" },
	{ 0xE4FED9, "EDMI Europe Ltd", "EDMI-EUROPE" },
	{ 0xE4FFDD, "ELECTRON INDIA", "ELECTRON-INDIA" },
	{ 0xE8039A, "Samsung Electronics CO., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0xE8040B, "Apple", "APPLE" },
	{ 0xE80410, "PRIVATE", "PRIVATE" },
	{ 0xE80462, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xE804F3, "Throughtek Co., Ltd.", "THROUGHTEK" },
	{ 0xE8056D, "Nortel Networks", "NORTEL-NETWORKS" },
	{ 0xE80688, "Apple", "APPLE" },
	{ 0xE807BF, "SHENZHEN BOOMTECH INDUSTRY CO.,LTD", "SHENZHEN-BOOMTECH-INDUSTRY" },
	{ 0xE8088B, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xE80B13, "Akib Systems Taiwan, INC", "AKIB-TAIWAN" },
	{ 0xE80C38, "DAEYOUNG INFORMATION SYSTEM CO., LTD", "DAEYOUNG-INFORMATION-SYSTEM" },
	{ 0xE80C75, "Syncbak, Inc.", "SYNCBAK" },
	{ 0xE8102E, "Really Simple Software, Inc", "REALLY-SIMPLE-SOFTWARE" },
	{ 0xE81132, "Samsung Electronics CO., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0xE81324, "GuangZhou Bonsoninfo System CO.,LTD", "GUANGZHOU-BONSONINFO-SYSTEM" },
	{ 0xE8150E, "Nokia Corporation", "NOKIA" },
	{ 0xE8162B, "IDEO Security Co., Ltd.", "IDEO-SECURITY" },
	{ 0xE817FC, "NIFTY Corporation", "NIFTY" },
	{ 0xE81863, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xE82877, "TMY Co., Ltd.", "TMY" },
	{ 0xE828D5, "Cots Technology", "COTS" },
	{ 0xE82AEA, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xE82E24, "Out of the Fog Research LLC", "OUT-OF-THE-FOG-RESEARCH-LLC" },
	{ 0xE83381, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xE83935, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xE839DF, "Askey Computer", "ASKEY" },
	{ 0xE83A97, "OCZ Technology Group", "OCZ-GROUP" },
	{ 0xE83EB6, "RIM", "RIM" },
	{ 0xE83EFB, "GEODESIC LTD.", "GEODESIC" },
	{ 0xE83EFC, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xE84040, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xE840F2, "PEGATRON CORPORATION", "PEGATRON" },
	{ 0xE843B6, "QNAP Systems, Inc.", "QNAP" },
	{ 0xE8447E, "Bitdefender SRL", "BITDEFENDER-SRL" },
	{ 0xE8481F, "Advanced Automotive Antennas", "ADVANCED-AUTOMOTIVE-ANTENNAS" },
	{ 0xE84E06, "EDUP INTERNATIONAL (HK) CO., LTD", "EDUP-HK" },
	{ 0xE84E84, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE84ECE, "Nintendo Co., Ltd.", "NINTENDO" },
	{ 0xE8508B, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xE8516E, "TSMART Inc.", "TSMART" },
	{ 0xE8519D, "Yeonhab Precision Co.,LTD", "YEONHAB-PRECISION" },
	{ 0xE85484, "NEO INFORMATION SYSTEMS CO., LTD.", "NEO-INFORMATION" },
	{ 0xE856D6, "NCTech Ltd", "NCTECH" },
	{ 0xE85AA7, "LLC Emzior", "LLC-EMZIOR" },
	{ 0xE85B5B, "LG ELECTRONICS INC", "LG-ELECTRONICS" },
	{ 0xE85BF0, "Imaging Diagnostics", "IMAGING-DIAGNOSTICS" },
	{ 0xE85D6B, "Luminate Wireless", "LUMINATE-WIRELESS" },
	{ 0xE85E53, "Infratec Datentechnik GmbH", "INFRATEC-DATENTECHNIK" },
	{ 0xE8611F, "Dawning Information Industry Co.,Ltd", "DAWNING-INFORMATION-INDUSTRY" },
	{ 0xE8617E, "Liteon Technology Corporation", "LITEON" },
	{ 0xE86183, "Black Diamond Advanced Technology, LLC", "BLACK-DIAMOND-ADVANCED-LLC" },
	{ 0xE861BE, "Melec Inc.", "MELEC" },
	{ 0xE866C4, "Datawise Systems", "DATAWISE" },
	{ 0xE86CDA, "Supercomputers and Neurocomputers Research Center", "SUPERCOMPUTERS-AND-NEUROCOMPUTERS-RESEARCH-CENTER" },
	{ 0xE86D52, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xE86D54, "Digit Mobile Inc", "DIGIT-MOBILE" },
	{ 0xE86D6E, "Control & Display Systems Ltd t/a CDSRail", "CONTROL-DISPLAY-T/A-CDSRAIL" },
	{ 0xE8718D, "Elsys Equipamentos Eletronicos Ltda", "ELSYS-EQUIPAMENTOS-ELETRONICOS-LTDA" },
	{ 0xE874E6, "ADB BROADBAND ITALIA", "ADB-BROADBAND-ITALIA" },
	{ 0xE8757F, "FIRS Technologies(Shenzhen) Co., Ltd", "FIRS-TECHNOLOGIES-SHENZHEN" },
	{ 0xE878A1, "BEOVIEW INTERCOM DOO", "BEOVIEW-INTERCOM-DOO" },
	{ 0xE87AF3, "S5 Tech S.r.l.", "S5-TECH-S-R-L" },
	{ 0xE8802E, "Apple", "APPLE" },
	{ 0xE880D8, "GNTEK Electronics Co.,Ltd.", "GNTEK-ELECTRONICS" },
	{ 0xE887A3, "Loxley Public Company Limited", "LOXLEY-PUBLIC-COMPANY" },
	{ 0xE8892C, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xE88D28, "Apple", "APPLE" },
	{ 0xE88DF5, "ZNYX Networks, Inc.", "ZNYX-NETWORKS" },
	{ 0xE88E60, "NSD Corporation", "NSD" },
	{ 0xE89120, "Motorola Mobility LLC, a Lenovo Company", "MOTOROLA-MOBILITY-LLC-A-LENOVO-COMPANY" },
	{ 0xE89218, "Arcontia International AB", "ARCONTIA-AB" },
	{ 0xE892A4, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xE8944C, "Cogent Healthcare Systems Ltd", "COGENT-HEALTHCARE" },
	{ 0xE894F6, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xE89606, "testo Instruments (Shenzhen) Co., Ltd.", "TESTO-INSTRUMENTS-SHENZHEN" },
	{ 0xE8995A, "PiiGAB, Processinformation i Goteborg AB", "PIIGAB-PROCESSINFORMATION-I-GOTEBORG-AB" },
	{ 0xE899C4, "HTC Corporation", "HTC" },
	{ 0xE89A8F, "Quanta Computer Inc.", "QUANTA" },
	{ 0xE89AFF, "Fujian Landi Commercial Equipment Co.,Ltd", "FUJIAN-LANDI-COMMERCIAL-EQUIPMENT" },
	{ 0xE89D87, "Toshiba", "TOSHIBA" },
	{ 0xE8A364, "Signal Path International / Peachtree Audio", "SIGNAL-PATH-/-PEACHTREE-AUDIO" },
	{ 0xE8A4C1, "Deep Sea Electronics PLC", "DEEP-SEA-ELECTRONICS-PLC" },
	{ 0xE8ABFA, "Shenzhen Reecam Tech.Ltd.", "SHENZHEN-REECAM-TECH" },
	{ 0xE8B1FC, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xE8B4AE, "Shenzhen C&D Electronics Co.,Ltd", "SHENZHEN-C-D-ELECTRONICS" },
	{ 0xE8B748, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xE8BA70, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xE8BB3D, "Sino Prime-Tech Limited", "SINO-PRIME-TECH" },
	{ 0xE8BBA8, "GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD.", "GUANGDONG-OPPO-MOBILE-TELECOMMUNICATIONS" },
	{ 0xE8BE81, "SAGEMCOM", "SAGEMCOM" },
	{ 0xE8C229, "H-Displays (MSC) Bhd", "H-DISPLAYS-MSC-BHD" },
	{ 0xE8C320, "Austco Communication Systems Pty Ltd", "AUSTCO-COMMUNICATION-PTY" },
	{ 0xE8C74F, "Liteon Technology Corporation", "LITEON" },
	{ 0xE8CBA1, "Nokia Corporation", "NOKIA" },
	{ 0xE8CC18, "D-Link International", "D-LINK" },
	{ 0xE8CC32, "Micronet LTD", "MICRONET" },
	{ 0xE8CD2D, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xE8CE06, "SkyHawke Technologies, LLC.", "SKYHAWKE-TECHNOLOGIES-LLC" },
	{ 0xE8D0FA, "MKS Instruments Deutschland GmbH", "MKS-INSTRUMENTS-DEUTSCHLAND" },
	{ 0xE8D483, "ULTIMATE Europe Transportation Equipment GmbH", "ULTIMATE-EUROPE-TRANSPORTATION-EQUIPMENT" },
	{ 0xE8D4E0, "Beijing BenyWave Technology Co., Ltd.", "BEIJING-BENYWAVE" },
	{ 0xE8DA96, "Zhuhai Tianrui Electrical Power Tech. Co., Ltd.", "ZHUHAI-TIANRUI-ELECTRICAL-POWER-TECH" },
	{ 0xE8DAAA, "VideoHome Technology Corp.", "VIDEOHOME" },
	{ 0xE8DE27, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xE8DFF2, "PRF Co., Ltd.", "PRF" },
	{ 0xE8E08F, "GRAVOTECH MARKING SAS", "GRAVOTECH-MARKING-SAS" },
	{ 0xE8E0B7, "Toshiba", "TOSHIBA" },
	{ 0xE8E1E2, "Energotest", "ENERGOTEST" },
	{ 0xE8E5D6, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xE8E732, "Alcatel-Lucent", "ALCATEL-LUCENT" },
	{ 0xE8E770, "Warp9 Tech Design, Inc.", "WARP9-TECH-DESIGN" },
	{ 0xE8E776, "Shenzhen Kootion Technology Co., Ltd", "SHENZHEN-KOOTION" },
	{ 0xE8E875, "iS5 Communications Inc.", "IS5-COMMUNICATION" },
	{ 0xE8EA6A, "StarTech.com", "STARTECH-COM" },
	{ 0xE8EADA, "Denkovi Assembly Electroncs LTD", "DENKOVI-ASSEMBLY-ELECTRONCS" },
	{ 0xE8ED05, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xE8EDF3, "Cisco", "CISCO" },
	{ 0xE8EF89, "OPMEX Tech.", "OPMEX-TECH" },
	{ 0xE8F1B0, "SAGEMCOM SAS", "SAGEMCOM-SAS" },
	{ 0xE8F226, "MILLSON CUSTOM SOLUTIONS INC.", "MILLSON-CUSTOM-SOLUTIONS" },
	{ 0xE8F2E3, "Starcor Beijing Co.,Limited", "STARCOR-BEIJING" },
	{ 0xE8F928, "RFTECH SRL", "RFTECH-SRL" },
	{ 0xE8FC60, "ELCOM Innovations Private Limited", "ELCOM-INNOVATIONS-PRIVATE" },
	{ 0xE8FCAF, "NETGEAR INC.,", "NETGEAR" },
	{ 0xEC0EC4, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xEC0ED6, "ITECH INSTRUMENTS SAS", "ITECH-INSTRUMENTS-SAS" },
	{ 0xEC1120, "FloDesign Wind Turbine Corporation", "FLODESIGN-WIND-TURBINE" },
	{ 0xEC1127, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xEC13B2, "Netonix", "NETONIX" },
	{ 0xEC14F6, "BioControl AS", "BIOCONTROL-AS" },
	{ 0xEC172F, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xEC1766, "Research Centre Module", "RESEARCH-CENTRE-MODULE" },
	{ 0xEC1A59, "Belkin International Inc.", "BELKIN" },
	{ 0xEC1D7F, "zte corporation", "ZTE" },
	{ 0xEC219F, "VidaBox LLC", "VIDABOX-LLC" },
	{ 0xEC2257, "JiangSu NanJing University Electronic Information Technology Co.,Ltd", "JIANGSU-NANJING-UNIVERSITY-INFORMATION" },
	{ 0xEC2280, "D-Link International", "D-LINK" },
	{ 0xEC233D, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xEC2368, "IntelliVoice Co.,Ltd.", "INTELLIVOICE" },
	{ 0xEC24B8, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xEC26CA, "TP-LINK TECHNOLOGIES CO.,LTD", "TP-LINK-TECHNOLOGIES" },
	{ 0xEC2AF0, "Ypsomed AG", "YPSOMED" },
	{ 0xEC2C49, "University of Tokyo", "UNIVERSITY-OF-TOKYO" },
	{ 0xEC2E4E, "HITACHI-LG DATA STORAGE INC", "HITACHI-LG-DATA-STORAGE" },
	{ 0xEC3091, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xEC3586, "Apple", "APPLE" },
	{ 0xEC3BF0, "NovelSat", "NOVELSAT" },
	{ 0xEC3C5A, "SHEN ZHEN HENG SHENG HUI DIGITAL TECHNOLOGY CO.,LTD", "SHEN-ZHEN-HENG-SHENG-HUI-DIGITAL" },
	{ 0xEC3C88, "MCNEX Co.,Ltd.", "MCNEX" },
	{ 0xEC3E09, "PERFORMANCE DESIGNED PRODUCTS, LLC", "PERFORMANCE-DESIGNED-PRODUCTS-LLC" },
	{ 0xEC3F05, "Institute 706, The Second Academy China Aerospace Science & Industry Corp", "INSTITUTE-706-THE-SECOND-ACADEMY-CHINA-AEROSPACE-SCIENCE-INDUSTRY" },
	{ 0xEC42F0, "ADL Embedded Solutions, Inc.", "ADL-EMBEDDED-SOLUTIONS" },
	{ 0xEC43E6, "AWCER Ltd.", "AWCER" },
	{ 0xEC43F6, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0xEC4476, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xEC4644, "TTK SAS", "TTK-SAS" },
	{ 0xEC4670, "Meinberg Funkuhren GmbH & Co. KG", "MEINBERG-FUNKUHREN" },
	{ 0xEC473C, "Redwire, LLC", "REDWIRE-LLC" },
	{ 0xEC4993, "Qihan Technology Co., Ltd", "QIHAN" },
	{ 0xEC4C4D, "ZAO NPK RoTeK", "ZAO-NPK-ROTEK" },
	{ 0xEC4F82, "Calix Inc.", "CALIX" },
	{ 0xEC542E, "Shanghai XiMei Electronic Technology Co. Ltd", "SHANGHAI-XIMEI" },
	{ 0xEC55F9, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xEC59E7, "Microsoft Corporation", "MICROSOFT" },
	{ 0xEC5A86, "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd", "YULONG-TELECOMMUNICATION-SCIENTIFIC-SHENZHEN" },
	{ 0xEC5C69, "MITSUBISHI HEAVY INDUSTRIES MECHATRONICS SYSTEMS,LTD.", "MITSUBISHI-HEAVY-INDUSTRIES-MECHATRONICS" },
	{ 0xEC6264, "Global411 Internet Services, LLC", "GLOBAL411-INTERNET-SERVICES-LLC" },
	{ 0xEC63E5, "ePBoard Design LLC", "EPBOARD-DESIGN-LLC" },
	{ 0xEC66D1, "B&W Group LTD", "B-W-GROUP" },
	{ 0xEC6C9F, "Chengdu Volans Technology CO.,LTD", "CHENGDU-VOLANS" },
	{ 0xEC71DB, "Shenzhen Baichuan Digital Technology Co., Ltd.", "SHENZHEN-BAICHUAN-DIGITAL" },
	{ 0xEC74BA, "Hirschmann Automation and Control GmbH", "HIRSCHMANN-AUTOMATION-AND-CONTROL" },
	{ 0xEC7C74, "Justone Technologies Co., Ltd.", "JUSTONE-TECHNOLOGIES" },
	{ 0xEC7D9D, "MEI", "MEI" },
	{ 0xEC8009, "NovaSparks", "NOVASPARKS" },
	{ 0xEC836C, "RM Tech Co., Ltd.", "RM-TECH" },
	{ 0xEC852F, "Apple", "APPLE" },
	{ 0xEC888F, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xEC8892, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0xEC89F5, "Lenovo Mobile Communication Technology Ltd.", "LENOVO-MOBILE-COMMUNICATION" },
	{ 0xEC8A4C, "zte corporation", "ZTE" },
	{ 0xEC8EAD, "DLX", "DLX" },
	{ 0xEC9233, "Eddyfi NDT Inc", "EDDYFI-NDT" },
	{ 0xEC9327, "MEMMERT GmbH + Co. KG", "MEMMERT-+" },
	{ 0xEC9681, "2276427 Ontario Inc", "2276427-ONTARIO" },
	{ 0xEC986C, "Lufft Mess- und Regeltechnik GmbH", "LUFFT-MESS--UND-REGELTECHNIK" },
	{ 0xEC98C1, "Beijing Risbo Network Technology Co.,Ltd", "BEIJING-RISBO-NETWORK" },
	{ 0xEC9A74, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xEC9B5B, "Nokia Corporation", "NOKIA" },
	{ 0xEC9ECD, "Artesyn Embedded Technologies", "ARTESYN-EMBEDDED-TECHNOLOGIES" },
	{ 0xECA29B, "Kemppi Oy", "KEMPPI-OY" },
	{ 0xECA86B, "ELITEGROUP COMPUTER SYSTEMS CO., LTD.", "ELITEGROUP" },
	{ 0xECB106, "Acuro Networks, Inc", "ACURO-NETWORKS" },
	{ 0xECB1D7, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xECB541, "SHINANO E and E Co.Ltd.", "SHINANO-E-AND-E" },
	{ 0xECB907, "CloudGenix Inc", "CLOUDGENIX" },
	{ 0xECBAFE, "GIROPTIC", "GIROPTIC" },
	{ 0xECBBAE, "Digivoice Tecnologia em Eletronica Ltda", "DIGIVOICE-TECNOLOGIA-EM-ELETRONICA-LTDA" },
	{ 0xECBD09, "FUSION Electronics Ltd", "FUSION-ELECTRONICS" },
	{ 0xECC38A, "Accuenergy (CANADA) Inc", "ACCUENERGY-CANADA" },
	{ 0xECC882, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xECCB30, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xECCD6D, "Allied Telesis, Inc.", "ALLIED-TELESIS" },
	{ 0xECD00E, "MiraeRecognition Co., Ltd.", "MIRAERECOGNITION" },
	{ 0xECD040, "GEA Farm Technologies GmbH", "GEA-FARM-TECHNOLOGIES" },
	{ 0xECD19A, "Zhuhai Liming Industries Co., Ltd", "ZHUHAI-LIMING-INDUSTRIES" },
	{ 0xECD925, "RAMI", "RAMI" },
	{ 0xECD950, "IRT SA", "IRT" },
	{ 0xECD9D1, "Shenzhen TG-NET Botone Technology Co.,Ltd.", "SHENZHEN-TG-NET-BOTONE" },
	{ 0xECDE3D, "Lamprey Networks, Inc.", "LAMPREY-NETWORKS" },
	{ 0xECE09B, "Samsung electronics CO., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0xECE1A9, "Cisco", "CISCO" },
	{ 0xECE2FD, "SKG Electric Group(Thailand) Co., Ltd.", "SKG-ELECTRIC-GROUP-THAILAND" },
	{ 0xECE512, "tado GmbH", "TADO" },
	{ 0xECE555, "Hirschmann Automation", "HIRSCHMANN-AUTOMATION" },
	{ 0xECE744, "Omntec mfg. inc", "OMNTEC-MFG" },
	{ 0xECE90B, "SISTEMA SOLUCOES ELETRONICAS LTDA - EASYTECH", "SISTEMA-SOLUCOES-ELETRONICAS-LTDA---EASYTECH" },
	{ 0xECE915, "STI Ltd", "STI" },
	{ 0xECE9F8, "Guang Zhou TRI-SUN Electronics Technology Co., Ltd", "GUANG-ZHOU-TRI-SUN-ELECTRONICS" },
	{ 0xECEA03, "DARFON LIGHTING CORP", "DARFON-LIGHTING" },
	{ 0xECF00E, "Abocom", "ABOCOM" },
	{ 0xECF236, "NEOMONTANA ELECTRONICS", "NEOMONTANA-ELECTRONICS" },
	{ 0xECF35B, "Nokia Corporation", "NOKIA" },
	{ 0xECF4BB, "Dell Inc", "DELL" },
	{ 0xECF72B, "HD DIGITAL TECH CO., LTD.", "HD-DIGITAL-TECH" },
	{ 0xECFAAA, "The IMS Company", "THE-IMS-COMPANY" },
	{ 0xECFC55, "A. Eberle GmbH & Co. KG", "A-EBERLE" },
	{ 0xECFE7E, "BlueRadios, Inc.", "BLUERADIOS" },
	{ 0xF0007F, "Janz - Contadores de Energia, SA", "JANZ---CONTADORES-DE-ENERGIA" },
	{ 0xF0022B, "Chrontel", "CHRONTEL" },
	{ 0xF00248, "SmarteBuilding", "SMARTEBUILDING" },
	{ 0xF00786, "Shandong Bittel Electronics Co., Ltd", "SHANDONG-BITTEL-ELECTRONICS" },
	{ 0xF008F1, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xF013C3, "SHENZHEN FENDA TECHNOLOGY CO., LTD", "SHENZHEN-FENDA" },
	{ 0xF015A0, "KyungDong One Co., Ltd.", "KYUNGDONG-ONE" },
	{ 0xF01C13, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xF01C2D, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0xF01E34, "ORICO Technologies Co., Ltd", "ORICO-TECHNOLOGIES" },
	{ 0xF01FAF, "Dell Inc", "DELL" },
	{ 0xF0219D, "Cal-Comp Electronics & Communications Company Ltd.", "CAL-COMP-ELECTRONICS-COMMUNICATION-COMPANY" },
	{ 0xF02329, "SHOWA DENKI CO.,LTD.", "SHOWA-DENKI" },
	{ 0xF02405, "OPUS High Technology Corporation", "OPUS-HIGH" },
	{ 0xF02408, "Talaris (Sweden) AB", "TALARIS-SWEDEN-AB" },
	{ 0xF02475, "Apple", "APPLE" },
	{ 0xF02572, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xF025B7, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xF0264C, "Dr. Sigrist AG", "DR-SIGRIST" },
	{ 0xF02765, "Murata Manufactuaring Co.,Ltd.", "MURATA-MANUFACTUARING" },
	{ 0xF02929, "Cisco", "CISCO" },
	{ 0xF02A23, "Creative Next Design", "CREATIVE-NEXT-DESIGN" },
	{ 0xF02A61, "Waldo Networks, Inc.", "WALDO-NETWORKS" },
	{ 0xF02FD8, "Bi2-Vision", "BI2-VISION" },
	{ 0xF0321A, "Mita-Teknik A/S", "MITA-TEKNIK-A/S" },
	{ 0xF037A1, "Huike Electronics (SHENZHEN) CO., LTD.", "HUIKE-ELECTRONICS-SHENZHEN" },
	{ 0xF03A4B, "Bloombase, Inc.", "BLOOMBASE" },
	{ 0xF03A55, "Omega Elektronik AS", "OMEGA-ELEKTRONIK-AS" },
	{ 0xF03D29, "Actility", "ACTILITY" },
	{ 0xF03FF8, "R L Drake", "R-L-DRAKE" },
	{ 0xF04335, "DVN(Shanghai)Ltd.", "DVN-SHANGHAI" },
	{ 0xF04A2B, "PYRAMID Computer GmbH", "PYRAMID" },
	{ 0xF04B6A, "Scientific Production Association Siberian Arsenal, Ltd.", "SCIENTIFIC-PRODUCTION-ASSOCIATION-SIBERIAN-ARSENAL" },
	{ 0xF04BF2, "JTECH Communications, Inc.", "JTECH-COMMUNICATION" },
	{ 0xF04DA2, "Dell Inc.", "DELL" },
	{ 0xF04F7C, "PRIVATE", "PRIVATE" },
	{ 0xF05849, "CareView Communications", "CAREVIEW-COMMUNICATION" },
	{ 0xF05A09, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xF05D89, "Dycon Limited", "DYCON" },
	{ 0xF05DC8, "Duracell Powermat", "DURACELL-POWERMAT" },
	{ 0xF05F5A, "Getriebebau NORD GmbH and Co. KG", "GETRIEBEBAU-NORD-AND" },
	{ 0xF06130, "Advantage Pharmacy Services, LLC", "ADVANTAGE-PHARMACY-SERVICES-LLC" },
	{ 0xF0620D, "Shenzhen Egreat Tech Corp.,Ltd", "SHENZHEN-EGREAT-TECH" },
	{ 0xF06281, "ProCurve Networking by HP", "PROCURVE-NETWORKING-BY-HP" },
	{ 0xF065DD, "Primax Electronics Ltd.", "PRIMAX-ELECTRONICS" },
	{ 0xF06853, "Integrated Corporation", "INTEGRATED" },
	{ 0xF06BCA, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xF0728C, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xF073AE, "PEAK-System Technik", "PEAK-SYSTEM-TECHNIK" },
	{ 0xF0761C, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0xF07765, "Sourcefire, Inc", "SOURCEFIRE" },
	{ 0xF077D0, "Xcellen", "XCELLEN" },
	{ 0xF07959, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xF07BCB, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xF07D68, "D-Link Corporation", "D-LINK" },
	{ 0xF07F06, "Cisco", "CISCO" },
	{ 0xF07F0C, "Leopold Kostal GmbH &Co. KG", "LEOPOLD-KOSTAL" },
	{ 0xF081AF, "IRZ AUTOMATION TECHNOLOGIES LTD", "IRZ-AUTOMATION-TECHNOLOGIES" },
	{ 0xF08261, "SAGEMCOM", "SAGEMCOM" },
	{ 0xF0842F, "ADB Broadband Italia", "ADB-BROADBAND-ITALIA" },
	{ 0xF084C9, "zte corporation", "ZTE" },
	{ 0xF08A28, "JIANGSU HENGSION ELECTRONIC S and T CO.,LTD", "JIANGSU-HENGSION-S-AND-T" },
	{ 0xF08BFE, "COSTEL.,CO.LTD", "COSTEL" },
	{ 0xF08CFB, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0xF08EDB, "VeloCloud Networks", "VELOCLOUD-NETWORKS" },
	{ 0xF0921C, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xF0933A, "NxtConect", "NXTCONECT" },
	{ 0xF093C5, "Garland Technology", "GARLAND" },
	{ 0xF099BF, "Apple", "APPLE" },
	{ 0xF09CBB, "RaonThink Inc.", "RAONTHINK" },
	{ 0xF09CE9, "Aerohive Networks Inc", "AEROHIVE-NETWORKS" },
	{ 0xF09E63, "Cisco", "CISCO" },
	{ 0xF09FC2, "Ubiquiti Networks, Inc.", "UBIQUITI-NETWORKS" },
	{ 0xF0A225, "PRIVATE", "PRIVATE" },
	{ 0xF0A764, "GST Co., Ltd.", "GST" },
	{ 0xF0ACA4, "HBC-radiomatic", "HBC-RADIOMATIC" },
	{ 0xF0AD4E, "Globalscale Technologies, Inc.", "GLOBALSCALE-TECHNOLOGIES" },
	{ 0xF0AE51, "Xi3 Corp", "XI3" },
	{ 0xF0B052, "Ruckus Wireless", "RUCKUS-WIRELESS" },
	{ 0xF0B479, "Apple", "APPLE" },
	{ 0xF0B6EB, "Poslab Technology Co., Ltd.", "POSLAB" },
	{ 0xF0BCC8, "MaxID (Pty) Ltd", "MAXID-PTY" },
	{ 0xF0BDF1, "Sipod Inc.", "SIPOD" },
	{ 0xF0BF97, "Sony Corporation", "SONY" },
	{ 0xF0C1F1, "Apple, Inc.", "APPLE" },
	{ 0xF0C24C, "Zhejiang FeiYue Digital Technology Co., Ltd", "ZHEJIANG-FEIYUE-DIGITAL" },
	{ 0xF0C27C, "Mianyang Netop Telecom Equipment Co.,Ltd.", "MIANYANG-NETOP-TELECOM-EQUIPMENT" },
	{ 0xF0C88C, "LeddarTech Inc.", "LEDDARTECH" },
	{ 0xF0CBA1, "Apple", "APPLE" },
	{ 0xF0D14F, "LINEAR LLC", "LINEAR-LLC" },
	{ 0xF0D1A9, "Apple", "APPLE" },
	{ 0xF0D3A7, "CobaltRay Co., Ltd", "COBALTRAY" },
	{ 0xF0D3E7, "Sensometrix SA", "SENSOMETRIX" },
	{ 0xF0D657, "ECHOSENS", "ECHOSENS" },
	{ 0xF0D767, "Axema Passagekontroll AB", "AXEMA-PASSAGEKONTROLL-AB" },
	{ 0xF0DA7C, "RLH INDUSTRIES,INC.", "RLH-INDUSTRIES" },
	{ 0xF0DB30, "Yottabyte", "YOTTABYTE" },
	{ 0xF0DBE2, "Apple", "APPLE" },
	{ 0xF0DBF8, "Apple", "APPLE" },
	{ 0xF0DCE2, "Apple", "APPLE" },
	{ 0xF0DE71, "Shanghai EDO Technologies Co.,Ltd.", "SHANGHAI-EDO-TECHNOLOGIES" },
	{ 0xF0DEB9, "ShangHai Y&Y Electronics Co., Ltd", "SHANGHAI-Y-Y-ELECTRONICS" },
	{ 0xF0DEF1, "Wistron InfoComm (Kunshan)Co", "WISTRON-INFOCOMM-KUNSHAN" },
	{ 0xF0E5C3, "Drägerwerk AG & Co. KG aA", "DRäGERWERK-AA" },
	{ 0xF0E77E, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xF0EBD0, "Shanghai Feixun Communication Co.,Ltd.", "SHANGHAI-FEIXUN-COMMUNICATION" },
	{ 0xF0EC39, "Essec", "ESSEC" },
	{ 0xF0ED1E, "Bilkon Bilgisayar Kontrollu Cih. Im.Ltd.", "BILKON-BILGISAYAR-KONTROLLU-CIH-IM" },
	{ 0xF0EEBB, "VIPAR GmbH", "VIPAR" },
	{ 0xF0F002, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xF0F260, "Mobitec AB", "MOBITEC-AB" },
	{ 0xF0F336, "TP-LINK TECHNOLOGIES CO.,LTD", "TP-LINK-TECHNOLOGIES" },
	{ 0xF0F5AE, "Adaptrum Inc.", "ADAPTRUM" },
	{ 0xF0F61C, "Apple", "APPLE" },
	{ 0xF0F644, "Whitesky Science & Technology Co.,Ltd.", "WHITESKY-SCIENCE" },
	{ 0xF0F669, "Motion Analysis Corporation", "MOTION-ANALYSIS" },
	{ 0xF0F755, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xF0F7B3, "Phorm", "PHORM" },
	{ 0xF0F842, "KEEBOX, Inc.", "KEEBOX" },
	{ 0xF0F9F7, "IES GmbH & Co. KG", "IES" },
	{ 0xF0FDA0, "Acurix Networks LP", "ACURIX-NETWORKS-LP" },
	{ 0xF0FE6B, "Shanghai High-Flying Electronics Technology Co., Ltd", "SHANGHAI-HIGH-FLYING-ELECTRONICS" },
	{ 0xF40321, "BeNeXt B.V.", "BENEXT" },
	{ 0xF4032F, "Reduxio Systems", "REDUXIO" },
	{ 0xF4044C, "ValenceTech Limited", "VALENCETECH" },
	{ 0xF40669, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xF4068D, "devolo AG", "DEVOLO" },
	{ 0xF406A5, "Hangzhou Bianfeng Networking Technology Co., Ltd.", "HANGZHOU-BIANFENG-NETWORKING" },
	{ 0xF409D8, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xF40B93, "Research In Motion", "RESEARCH-IN-MOTION" },
	{ 0xF40E11, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xF40F1B, "Cisco", "CISCO" },
	{ 0xF40F9B, "WAVELINK", "WAVELINK" },
	{ 0xF41563, "F5 Networks, Inc.", "F5-NETWORKS" },
	{ 0xF415FD, "Shanghai Pateo Electronic Equipment Manufacturing Co., Ltd.", "SHANGHAI-PATEO-EQUIPMENT-MANUFACTURING" },
	{ 0xF41BA1, "Apple", "APPLE" },
	{ 0xF41E26, "Simon-Kaloi Engineering", "SIMON-KALOI-ENGINEERING" },
	{ 0xF41F0B, "YAMABISHI Corporation", "YAMABISHI" },
	{ 0xF41FC2, "Cisco", "CISCO" },
	{ 0xF42012, "Cuciniale GmbH", "CUCINIALE" },
	{ 0xF42833, "MMPC Inc.", "MMPC" },
	{ 0xF42853, "Zioncom Electronics (Shenzhen) Ltd.", "ZIONCOM-ELECTRONICS-SHENZHEN" },
	{ 0xF42896, "SPECTO PAINEIS ELETRONICOS LTDA", "SPECTO-PAINEIS-ELETRONICOS-LTDA" },
	{ 0xF42981, "vivo Mobile Communication Co., Ltd.", "VIVO-MOBILE-COMMUNICATION" },
	{ 0xF42C56, "SENOR TECH CO LTD", "SENOR-TECH" },
	{ 0xF436E1, "Abilis Systems SARL", "ABILIS-SARL" },
	{ 0xF437B7, "Apple", "APPLE" },
	{ 0xF43814, "Shanghai Howell Electronic Co.,Ltd", "SHANGHAI-HOWELL" },
	{ 0xF43D80, "FAG Industrial Services GmbH", "FAG-INDUSTRIAL-SERVICES" },
	{ 0xF43E61, "Shenzhen Gongjin Electronics Co., Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0xF43E9D, "Benu Networks, Inc.", "BENU-NETWORKS" },
	{ 0xF44227, "S & S Research Inc.", "S-S-RESEARCH" },
	{ 0xF44450, "BND Co., Ltd.", "BND" },
	{ 0xF445ED, "Portable Innovation Technology Ltd.", "PORTABLE-INNOVATION" },
	{ 0xF44713, "Leading Public Performance Co., Ltd.", "LEADING-PUBLIC-PERFORMANCE" },
	{ 0xF4472A, "Nanjing Rousing Sci. and Tech. Industrial Co., Ltd", "NANJING-ROUSING-SCI-AND-TECH-INDUSTRIAL" },
	{ 0xF44848, "Amscreen Group Ltd", "AMSCREEN-GROUP" },
	{ 0xF44E05, "Cisco", "CISCO" },
	{ 0xF44EFD, "Actions Semiconductor Co.,Ltd.(Cayman Islands)", "ACTIONS-SEMICONDUCTOR-CAYMAN-ISLANDS" },
	{ 0xF450EB, "Telechips Inc", "TELECHIPS" },
	{ 0xF45214, "Mellanox Technologies, Inc.", "MELLANOX-TECHNOLOGIES" },
	{ 0xF45433, "Rockwell Automation", "ROCKWELL-AUTOMATION" },
	{ 0xF45595, "HENGBAO Corporation LTD.", "HENGBAO" },
	{ 0xF4559C, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xF455E0, "Niceway CNC Technology Co.,Ltd.Hunan Province", "NICEWAY-CNC-HUNAN-PROVINCE" },
	{ 0xF45842, "Boxx TV Ltd", "BOXX-TV" },
	{ 0xF45F69, "Matsufu Electronics distribution Company", "MATSUFU-ELECTRONICS-DISTRIBUTION-COMPANY" },
	{ 0xF45FD4, "Cisco SPVTG", "CISCO-SPVTG" },
	{ 0xF45FF7, "DQ Technology Inc.", "DQ" },
	{ 0xF4600D, "Panoptic Technology, Inc", "PANOPTIC" },
	{ 0xF46349, "Diffon Corporation", "DIFFON" },
	{ 0xF4645D, "Toshiba", "TOSHIBA" },
	{ 0xF46A92, "SHENZHEN FAST TECHNOLOGIES CO.,LTD", "SHENZHEN-FAST-TECHNOLOGIES" },
	{ 0xF46ABC, "Adonit Corp. Ltd.", "ADONIT" },
	{ 0xF46D04, "ASUSTek COMPUTER INC.", "ASUSTEK" },
	{ 0xF46DE2, "zte corporation", "ZTE" },
	{ 0xF473CA, "Conversion Sound Inc.", "CONVERSION-SOUND" },
	{ 0xF47626, "Viltechmeda UAB", "VILTECHMEDA-UAB" },
	{ 0xF47A4E, "Woojeon&Handan", "WOOJEON-HANDAN" },
	{ 0xF47ACC, "SolidFire, Inc.", "SOLIDFIRE" },
	{ 0xF47B5E, "Samsung Eletronics Co., Ltd", "SAMSUNG-ELETRONICS" },
	{ 0xF47F35, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xF48139, "CANON INC.", "CANON" },
	{ 0xF48771, "Infoblox", "INFOBLOX" },
	{ 0xF48E09, "Nokia Corporation", "NOKIA" },
	{ 0xF490CA, "Tensorcom", "TENSORCOM" },
	{ 0xF490EA, "Deciso B.V.", "DECISO" },
	{ 0xF49461, "NexGen Storage", "NEXGEN-STORAGE" },
	{ 0xF49466, "CountMax, ltd", "COUNTMAX" },
	{ 0xF499AC, "WEBER Schraubautomaten GmbH", "WEBER-SCHRAUBAUTOMATEN" },
	{ 0xF49F54, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xF49FF3, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xF4A294, "EAGLE WORLD DEVELOPMENT CO., LIMITED", "EAGLE-WORLD-DEVELOPMENT" },
	{ 0xF4A52A, "Hawa Technologies Inc", "HAWA-TECHNOLOGIES" },
	{ 0xF4ACC1, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xF4B164, "Lightning Telecommunications Technology Co. Ltd", "LIGHTNING-TELECOMMUNICATIONS" },
	{ 0xF4B381, "WindowMaster A/S", "WINDOWMASTER-A/S" },
	{ 0xF4B52F, "Juniper networks", "JUNIPER-NETWORKS" },
	{ 0xF4B549, "Yeastar Technology Co., Ltd.", "YEASTAR" },
	{ 0xF4B6E5, "TerraSem Co.,Ltd", "TERRASEM" },
	{ 0xF4B72A, "TIME INTERCONNECT LTD", "TIME-INTERCONNECT" },
	{ 0xF4B7E2, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xF4B85E, "Texas INstruments", "TEXAS-INSTRUMENTS" },
	{ 0xF4BD7C, "Chengdu jinshi communication Co., LTD", "CHENGDU-JINSHI-COMMUNICATION" },
	{ 0xF4C447, "Coagent International Enterprise Limited", "COAGENT" },
	{ 0xF4C6D7, "blackned GmbH", "BLACKNED" },
	{ 0xF4C714, "Shenzhen Huawei Communication Technologies Co., Ltd", "SHENZHEN-HUAWEI-COMMUNICATION-TECHNOLOGIES" },
	{ 0xF4C795, "WEY Elektronik AG", "WEY-ELEKTRONIK" },
	{ 0xF4CAE5, "FREEBOX SA", "FREEBOX" },
	{ 0xF4CD90, "Vispiron Rotec GmbH", "VISPIRON-ROTEC" },
	{ 0xF4CE46, "Hewlett-Packard Company", "HEWLETT-PACKARD-COMPANY" },
	{ 0xF4CFE2, "Cisco", "CISCO" },
	{ 0xF4D032, "Yunnan Ideal Information&Technology.,Ltd", "YUNNAN-IDEAL-INFORMATION" },
	{ 0xF4D261, "SEMOCON Co., Ltd", "SEMOCON" },
	{ 0xF4D9FB, "Samsung Electronics CO., LTD", "SAMSUNG-ELECTRONICS" },
	{ 0xF4DC4D, "Beijing CCD Digital Technology Co., Ltd", "BEIJING-CCD-DIGITAL" },
	{ 0xF4DCDA, "Zhuhai Jiahe Communication Technology Co., limited", "ZHUHAI-JIAHE-COMMUNICATION" },
	{ 0xF4DCF9, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xF4DD9E, "GoPro", "GOPRO" },
	{ 0xF4E142, "Delta Elektronika BV", "DELTA-ELEKTRONIKA" },
	{ 0xF4E3FB, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xF4E6D7, "Solar Power Technologies, Inc.", "SOLAR-POWER-TECHNOLOGIES" },
	{ 0xF4EA67, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xF4EC38, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xF4EE14, "SHENZHEN MERCURY COMMUNICATION TECHNOLOGIES CO.,LTD.", "SHENZHEN-MERCURY-COMMUNICATION-TECHNOLOGIES" },
	{ 0xF4F15A, "Apple", "APPLE" },
	{ 0xF4F1E1, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0xF4F26D, "TP-LINK TECHNOLOGIES CO.,LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xF4F5A5, "Nokia corporation", "NOKIA" },
	{ 0xF4F5E8, "Google", "GOOGLE" },
	{ 0xF4F646, "Dediprog Technology Co. Ltd.", "DEDIPROG" },
	{ 0xF4F951, "Apple", "APPLE" },
	{ 0xF4FC32, "Texas Instruments", "TEXAS-INSTRUMENTS" },
	{ 0xF4FD2B, "ZOYI Company", "ZOYI-COMPANY" },
	{ 0xF80113, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xF80278, "IEEE REGISTRATION AUTHORITY - Please see MAM public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAM-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0xF80332, "Khomp", "KHOMP" },
	{ 0xF8042E, "Samsung Electro Mechanics co., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xF8051C, "DRS Imaging and Targeting Solutions", "DRS-IMAGING-AND-TARGETING-SOLUTIONS" },
	{ 0xF80BBE, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xF80BD0, "Datang Telecom communication terminal (Tianjin) Co., Ltd.", "DATANG-TELECOM-COMMUNICATION-TERMINAL-TIANJIN" },
	{ 0xF80CF3, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xF80D43, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xF80DEA, "ZyCast Technology Inc.", "ZYCAST" },
	{ 0xF80F41, "Wistron InfoComm(ZhongShan) Corporation", "WISTRON-INFOCOMM-ZHONGSHAN" },
	{ 0xF80F84, "Natural Security SAS", "NATURAL-SECURITY-SAS" },
	{ 0xF81037, "Atopia Systems, LP", "ATOPIA-LP" },
	{ 0xF81547, "Avaya, Inc", "AVAYA" },
	{ 0xF81654, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xF81897, "2Wire", "2WIRE" },
	{ 0xF81A67, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xF81CE5, "Telefonbau Behnke GmbH", "TELEFONBAU-BEHNKE" },
	{ 0xF81D93, "Longdhua(Beijing) Controls Technology Co.,Ltd", "LONGDHUA-BEIJING-CONTROLS" },
	{ 0xF81EDF, "Apple", "APPLE" },
	{ 0xF82285, "Cypress Technology CO., LTD.", "CYPRESS" },
	{ 0xF82441, "Yeelink", "YEELINK" },
	{ 0xF82793, "Apple, Inc", "APPLE" },
	{ 0xF82BC8, "Jiangsu Switter Co., Ltd", "JIANGSU-SWITTER" },
	{ 0xF82EDB, "RTW GmbH & Co. KG", "RTW" },
	{ 0xF82F5B, "eGauge Systems LLC", "EGAUGE-LLC" },
	{ 0xF82FA8, "Hon Hai Precision Ind. Co.,Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xF83094, "Alcatel-Lucent Telecom Limited", "ALCATEL-LUCENT-TELECOM" },
	{ 0xF8313E, "endeavour GmbH", "ENDEAVOUR" },
	{ 0xF83376, "Good Mind Innovation Co., Ltd.", "GOOD-MIND-INNOVATION" },
	{ 0xF83553, "Magenta Research Ltd.", "MAGENTA-RESEARCH" },
	{ 0xF835DD, "Gemtek Technology Co., Ltd.", "GEMTEK" },
	{ 0xF83D4E, "Softlink Automation System Co., Ltd", "SOFTLINK-AUTOMATION-SYSTEM" },
	{ 0xF83DFF, "Huawei Technologies Co., Ltd", "HUAWEI-TECHNOLOGIES" },
	{ 0xF842FB, "Yasuda Joho Co.,ltd.", "YASUDA-JOHO" },
	{ 0xF84360, "PRIVATE", "PRIVATE" },
	{ 0xF845AD, "Konka Group Co., Ltd.", "KONKA-GROUP" },
	{ 0xF8462D, "SYNTEC Incorporation", "SYNTEC-INCORPORATION" },
	{ 0xF8472D, "X2gen Digital Corp. Ltd", "X2GEN-DIGITAL" },
	{ 0xF84897, "Hitachi, Ltd.", "HITACHI" },
	{ 0xF84A73, "EUMTECH CO., LTD", "EUMTECH" },
	{ 0xF84A7F, "Innometriks Inc", "INNOMETRIKS" },
	{ 0xF84ABF, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xF84F57, "Cisco", "CISCO" },
	{ 0xF85063, "Verathon", "VERATHON" },
	{ 0xF8516D, "Denwa Technology Corp.", "DENWA" },
	{ 0xF852DF, "VNL Europe AB", "VNL-EUROPE-AB" },
	{ 0xF854AF, "ECI Telecom Ltd.", "ECI-TELECOM" },
	{ 0xF8572E, "Core Brands, LLC", "CORE-BRANDS-LLC" },
	{ 0xF85BC9, "M-Cube Spa", "M-CUBE-SPA" },
	{ 0xF85C45, "IC Nexus Co. Ltd.", "IC-NEXUS" },
	{ 0xF85F2A, "Nokia Corporation", "NOKIA" },
	{ 0xF862AA, "xn systems", "XN" },
	{ 0xF86601, "Suzhou Chi-tek information technology Co., Ltd", "SUZHOU-CHI-TEK-INFORMATION" },
	{ 0xF866D1, "Hon Hai Precision Ind. Co., Ltd.", "HON-HAI-PRECISION-IND" },
	{ 0xF866F2, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xF86971, "Seibu Electric Co.,", "SEIBU-ELECTRIC" },
	{ 0xF86ECF, "Arcx Inc", "ARCX" },
	{ 0xF871FE, "The Goldman Sachs Group, Inc.", "THE-GOLDMAN-SACHS-GROUP" },
	{ 0xF872EA, "Cisco", "CISCO" },
	{ 0xF87394, "NETGEAR INC.,", "NETGEAR" },
	{ 0xF8769B, "Neopis Co., Ltd.", "NEOPIS" },
	{ 0xF87AEF, "Rosonix Technology, Inc.", "ROSONIX" },
	{ 0xF87B62, "FASTWEL INTERNATIONAL CO., LTD. Taiwan Branch", "FASTWEL-TAIWAN-BRANCH" },
	{ 0xF87B7A, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xF87B8C, "Amped Wireless", "AMPED-WIRELESS" },
	{ 0xF8811A, "OVERKIZ", "OVERKIZ" },
	{ 0xF88479, "Yaojin Technology(Shenzhen)Co.,Ltd", "YAOJIN-SHENZHEN" },
	{ 0xF884F2, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xF88C1C, "KAISHUN ELECTRONIC TECHNOLOGY CO., LTD. BEIJING", "KAISHUN-BEIJING" },
	{ 0xF88DEF, "Tenebraex", "TENEBRAEX" },
	{ 0xF88E85, "COMTREND CORPORATION", "COMTREND" },
	{ 0xF88FCA, "Google Fiber, Inc", "GOOGLE-FIBER" },
	{ 0xF8912A, "GLP German Light Products GmbH", "GLP-GERMAN-LIGHT-PRODUCTS" },
	{ 0xF893F3, "VOLANS", "VOLANS" },
	{ 0xF89550, "Proton Products Chengdu Ltd", "PROTON-PRODUCTS-CHENGDU" },
	{ 0xF897CF, "DAESHIN-INFORMATION TECHNOLOGY CO., LTD.", "DAESHIN-INFORMATION" },
	{ 0xF89955, "Fortress Technology Inc", "FORTRESS" },
	{ 0xF89D0D, "Control Technology Inc.", "CONTROL" },
	{ 0xF89FB8, "YAZAKI Energy System Corporation", "YAZAKI-ENERGY-SYSTEM" },
	{ 0xF8A03D, "Dinstar Technologies Co., Ltd.", "DINSTAR-TECHNOLOGIES" },
	{ 0xF8A2B4, "RHEWA-WAAGENFABRIK August Freudewald GmbH &amp;Co. KG", "RHEWA-WAAGENFABRIK-AUGUST-FREUDEWALD-AMP" },
	{ 0xF8A45F, "Beijing Xiaomi communications co.,ltd", "BEIJING-XIAOMI-COMMUNICATION" },
	{ 0xF8A963, "COMPAL INFORMATION (KUNSHAN) CO., LTD.", "COMPAL-INFORMATION-KUNSHAN" },
	{ 0xF8A9D0, "LG Electronics", "LG-ELECTRONICS" },
	{ 0xF8A9DE, "PUISSANCE PLUS", "PUISSANCE-PLUS" },
	{ 0xF8AA8A, "Axview Technology (Shenzhen) Co.,Ltd", "AXVIEW-SHENZHEN" },
	{ 0xF8AC6D, "Deltenna Ltd", "DELTENNA" },
	{ 0xF8B156, "Dell Inc", "DELL" },
	{ 0xF8B2F3, "GUANGZHOU BOSMA TECHNOLOGY CO.,LTD", "GUANGZHOU-BOSMA" },
	{ 0xF8B599, "Guangzhou CHNAVS Digital Technology Co.,Ltd", "GUANGZHOU-CHNAVS-DIGITAL" },
	{ 0xF8BC12, "Dell Inc", "DELL" },
	{ 0xF8BC41, "Rosslare Enterprises Limited", "ROSSLARE-ENTERPRISES" },
	{ 0xF8C001, "Juniper Networks", "JUNIPER-NETWORKS" },
	{ 0xF8C091, "Highgates Technology", "HIGHGATES" },
	{ 0xF8C288, "Cisco", "CISCO" },
	{ 0xF8C397, "NZXT Corp. Ltd.", "NZXT" },
	{ 0xF8C678, "Carefusion", "CAREFUSION" },
	{ 0xF8C96C, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0xF8CFC5, "Motorola Mobility LLC, a Lenovo Company", "MOTOROLA-MOBILITY-LLC-A-LENOVO-COMPANY" },
	{ 0xF8D0AC, "Sony Computer Entertainment Inc.", "SONY-ENTERTAINMENT" },
	{ 0xF8D0BD, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xF8D111, "TP-LINK TECHNOLOGIES CO., LTD.", "TP-LINK-TECHNOLOGIES" },
	{ 0xF8D3A9, "AXAN Networks", "AXAN-NETWORKS" },
	{ 0xF8D462, "Pumatronix Equipamentos Eletronicos Ltda.", "PUMATRONIX-EQUIPAMENTOS-ELETRONICOS-LTDA" },
	{ 0xF8D756, "Simm Tronic Limited", "SIMM-TRONIC" },
	{ 0xF8D7BF, "REV Ritter GmbH", "REV-RITTER" },
	{ 0xF8DADF, "EcoTech, Inc.", "ECOTECH" },
	{ 0xF8DAE2, "Beta LaserMike", "BETA-LASERMIKE" },
	{ 0xF8DAF4, "Taishan Online Technology Co., Ltd.", "TAISHAN-ONLINE" },
	{ 0xF8DB4C, "PNY Technologies, INC.", "PNY-TECHNOLOGIES" },
	{ 0xF8DB7F, "HTC Corporation", "HTC" },
	{ 0xF8DB88, "Dell Inc", "DELL" },
	{ 0xF8DC7A, "Variscite LTD", "VARISCITE" },
	{ 0xF8DFA8, "ZTE Corporation", "ZTE" },
	{ 0xF8E079, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0xF8E4FB, "Actiontec Electronics, Inc", "ACTIONTEC-ELECTRONICS" },
	{ 0xF8E7B5, "µTech Tecnologia LTDA", "µTECH-TECNOLOGIA-LTDA" },
	{ 0xF8E811, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xF8E903, "D-Link International", "D-LINK" },
	{ 0xF8E968, "Egker Kft.", "EGKER-KFT" },
	{ 0xF8EA0A, "Dipl.-Math. Michael Rauch", "DIPL--MATH-MICHAEL-RAUCH" },
	{ 0xF8EDA5, "ARRIS Group, Inc.", "ARRIS-GROUP" },
	{ 0xF8F005, "Newport Media Inc.", "NEWPORT-MEDIA" },
	{ 0xF8F014, "RackWare Inc.", "RACKWARE" },
	{ 0xF8F082, "Orion Networks International, Inc", "ORION-NETWORKS" },
	{ 0xF8F1B6, "Motorola Mobility LLC", "MOTOROLA-MOBILITY-LLC" },
	{ 0xF8F25A, "G-Lab GmbH", "G-LAB" },
	{ 0xF8F7D3, "International Communications Corporation", "COMMUNICATION" },
	{ 0xF8F7FF, "SYN-TECH SYSTEMS INC", "SYN-TECH" },
	{ 0xF8FB2F, "Santur Corporation", "SANTUR" },
	{ 0xF8FE5C, "Reciprocal Labs Corp", "RECIPROCAL-LABS" },
	{ 0xF8FEA8, "Technico Japan Corporation", "TECHNICO-JAPAN" },
	{ 0xF8FF5F, "Shenzhen Communication Technology Co.,Ltd", "SHENZHEN-COMMUNICATION" },
	{ 0xFC0012, "Toshiba Samsung Storage Technolgoy Korea Corporation", "TOSHIBA-SAMSUNG-STORAGE-TECHNOLGOY-KOREA" },
	{ 0xFC019E, "VIEVU", "VIEVU" },
	{ 0xFC01CD, "FUNDACION TEKNIKER", "FUNDACION-TEKNIKER" },
	{ 0xFC0647, "Cortland Research, LLC", "CORTLAND-RESEARCH-LLC" },
	{ 0xFC07A0, "LRE Medical GmbH", "LRE-MEDICAL" },
	{ 0xFC0877, "Prentke Romich Company", "PRENTKE-ROMICH-COMPANY" },
	{ 0xFC09D8, "ACTEON Group", "ACTEON-GROUP" },
	{ 0xFC09F6, "GUANGDONG TONZE ELECTRIC CO.,LTD", "GUANGDONG-TONZE-ELECTRIC" },
	{ 0xFC0A81, "Zebra Technologies Inc", "ZEBRA-TECHNOLOGIES" },
	{ 0xFC0FE6, "Sony Computer Entertainment Inc.", "SONY-ENTERTAINMENT" },
	{ 0xFC10BD, "Control Sistematizado S.A.", "CONTROL-SISTEMATIZADO" },
	{ 0xFC1186, "Logic3 plc", "LOGIC3-PLC" },
	{ 0xFC1349, "Global Apps Corp.", "GLOBAL-APPS" },
	{ 0xFC15B4, "Hewlett Packard", "HEWLETT-PACKARD" },
	{ 0xFC1607, "Taian Technology(Wuxi) Co.,Ltd.", "TAIAN-WUXI" },
	{ 0xFC1794, "InterCreative Co., Ltd", "INTERCREATIVE" },
	{ 0xFC1910, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xFC19D0, "Cloud Vision Networks Technology Co.,Ltd.", "CLOUD-VISION-NETWORKS" },
	{ 0xFC1BFF, "V-ZUG AG", "V-ZUG" },
	{ 0xFC1D59, "I Smart Cities HK Ltd", "I-SMART-CITIES-HK" },
	{ 0xFC1D84, "Autobase", "AUTOBASE" },
	{ 0xFC1E16, "IPEVO corp", "IPEVO" },
	{ 0xFC1F19, "SAMSUNG ELECTRO-MECHANICS CO., LTD.", "SAMSUNG-ELECTRO-MECHANICS" },
	{ 0xFC1FC0, "EURECAM", "EURECAM" },
	{ 0xFC229C, "Han Kyung I Net Co.,Ltd.", "HAN-KYUNG-I-NET" },
	{ 0xFC2325, "EosTek (Shenzhen) Co., Ltd.", "EOSTEK-SHENZHEN" },
	{ 0xFC253F, "Apple", "APPLE" },
	{ 0xFC27A2, "TRANS ELECTRIC CO., LTD.", "TRANS-ELECTRIC" },
	{ 0xFC2A54, "Connected Data, Inc.", "CONNECTED-DATA" },
	{ 0xFC2E2D, "Lorom Industrial Co.LTD.", "LOROM-INDUSTRIAL" },
	{ 0xFC2F40, "Calxeda, Inc.", "CALXEDA" },
	{ 0xFC3288, "CELOT Wireless Co., Ltd", "CELOT-WIRELESS" },
	{ 0xFC3598, "Favite Inc.", "FAVITE" },
	{ 0xFC35E6, "Visteon corp", "VISTEON" },
	{ 0xFC3D93, "LONGCHEER TELECOMMUNICATION LIMITED", "LONGCHEER-TELECOMMUNICATION" },
	{ 0xFC3FAB, "Henan Lanxin Technology Co., Ltd", "HENAN-LANXIN" },
	{ 0xFC4463, "Universal Audio, Inc", "UNIVERSAL-AUDIO" },
	{ 0xFC4499, "Swarco LEA d.o.o.", "SWARCO-LEA-D-O-O" },
	{ 0xFC455F, "JIANGXI SHANSHUI OPTOELECTRONIC TECHNOLOGY CO.,LTD", "JIANGXI-SHANSHUI-OPTOELECTRONIC" },
	{ 0xFC48EF, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xFC4AE9, "Castlenet Technology Inc.", "CASTLENET" },
	{ 0xFC4B1C, "INTERSENSOR S.R.L.", "INTERSENSOR-S-R-L" },
	{ 0xFC4BBC, "Sunplus Technology Co., Ltd.", "SUNPLUS" },
	{ 0xFC4DD4, "Universal Global Scientific Industrial Co., Ltd.", "UNIVERSAL-GLOBAL-SCIENTIFIC-INDUSTRIAL" },
	{ 0xFC5090, "SIMEX Sp. z o.o.", "SIMEX-SP-Z-O-O" },
	{ 0xFC52CE, "Control iD", "CONTROL-ID" },
	{ 0xFC58FA, "Shen Zhen Shi Xin Zhong Xin Technology Co.,Ltd.", "SHEN-ZHEN-SHI-XIN-ZHONG-XIN" },
	{ 0xFC5B24, "Weibel Scientific A/S", "WEIBEL-SCIENTIFIC-A/S" },
	{ 0xFC5B26, "MikroBits", "MIKROBITS" },
	{ 0xFC5B39, "Cisco", "CISCO" },
	{ 0xFC6018, "Zhejiang Kangtai Electric Co., Ltd.", "ZHEJIANG-KANGTAI-ELECTRIC" },
	{ 0xFC6198, "NEC Personal Products, Ltd", "NEC-PERSONAL-PRODUCTS" },
	{ 0xFC626E, "Beijing MDC Telecom", "BEIJING-MDC-TELECOM" },
	{ 0xFC62B9, "ALPS ERECTRIC CO.,LTD", "ALPS-ERECTRIC" },
	{ 0xFC683E, "Directed Perception, Inc", "DIRECTED-PERCEPTION" },
	{ 0xFC6C31, "LXinstruments GmbH", "LXINSTRUMENTS" },
	{ 0xFC6DC0, "BME CORPORATION", "BME" },
	{ 0xFC6FB7, "Pace plc", "PACE-PLC" },
	{ 0xFC7516, "D-Link International", "D-LINK" },
	{ 0xFC75E6, "Handreamnet", "HANDREAMNET" },
	{ 0xFC790B, "Hitachi High Technologies America, Inc.", "HITACHI-HIGH-TECHNOLOGIES-AMERICA" },
	{ 0xFC7CE7, "FCI USA LLC", "FCI-USA-LLC" },
	{ 0xFC8329, "Trei technics", "TREI-TECHNICS" },
	{ 0xFC8399, "Avaya, Inc", "AVAYA" },
	{ 0xFC8B97, "Shenzhen Gongjin Electronics Co.,Ltd", "SHENZHEN-GONGJIN-ELECTRONICS" },
	{ 0xFC8E7E, "Pace plc", "PACE-PLC" },
	{ 0xFC8F90, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xFC8FC4, "Intelligent Technology Inc.", "INTELLIGENT" },
	{ 0xFC923B, "Nokia Corporation", "NOKIA" },
	{ 0xFC946C, "UBIVELOX", "UBIVELOX" },
	{ 0xFC94E3, "Technicolor USA Inc.", "TECHNICOLOR-USA" },
	{ 0xFC9947, "Cisco", "CISCO" },
	{ 0xFC9AFA, "Motus Global Inc.", "MOTUS-GLOBAL" },
	{ 0xFC9FAE, "Fidus Systems Inc", "FIDUS" },
	{ 0xFC9FE1, "CONWIN.Tech. Ltd", "CONWIN-TECH" },
	{ 0xFCA13E, "Samsung Electronics", "SAMSUNG-ELECTRONICS" },
	{ 0xFCA22A, "PT. Callysta Multi Engineering", "PT-CALLYSTA-MULTI-ENGINEERING" },
	{ 0xFCA841, "Avaya, Inc", "AVAYA" },
	{ 0xFCA9B0, "MIARTECH (SHANGHAI),INC.", "MIARTECH-SHANGHAI" },
	{ 0xFCAA14, "GIGA-BYTE TECHNOLOGY CO.,LTD.", "GIGA-BYTE" },
	{ 0xFCAD0F, "QTS NETWORKS", "QTS-NETWORKS" },
	{ 0xFCAF6A, "Qulsar Inc", "QULSAR" },
	{ 0xFCAFAC, "Panasonic System LSI", "PANASONIC-SYSTEM-LSI" },
	{ 0xFCB0C4, "Shanghai DareGlobal Technologies Co., Ltd", "SHANGHAI-DAREGLOBAL-TECHNOLOGIES" },
	{ 0xFCB4E6, "ASKEY COMPUTER CORP.", "ASKEY" },
	{ 0xFCB698, "Cambridge Industries(Group) Co.,Ltd.", "CAMBRIDGE-INDUSTRIES-GROUP" },
	{ 0xFCBBA1, "Shenzhen Minicreate Technology Co.,Ltd", "SHENZHEN-MINICREATE" },
	{ 0xFCC23D, "Atmel Corporation", "ATMEL" },
	{ 0xFCC2DE, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0xFCC734, "Samsung Electronics Co.,Ltd", "SAMSUNG-ELECTRONICS" },
	{ 0xFCC897, "ZTE Corporation", "ZTE" },
	{ 0xFCCCE4, "Ascon Ltd.", "ASCON" },
	{ 0xFCCF62, "IBM Corp", "IBM" },
	{ 0xFCD4F2, "The Coca Cola Company", "THE-COCA-COLA-COMPANY" },
	{ 0xFCD4F6, "Messana Air.Ray Conditioning s.r.l.", "MESSANA-AIR-RAY-CONDITIONING-S-R-L" },
	{ 0xFCD5D9, "Shenzhen SDMC Technology Co., Ltd.", "SHENZHEN-SDMC" },
	{ 0xFCD6BD, "Robert Bosch GmbH", "ROBERT-BOSCH" },
	{ 0xFCD817, "Beijing Hesun Technologies Co.Ltd.", "BEIJING-HESUN-TECHNOLOGIES" },
	{ 0xFCDB96, "ENERVALLEY CO., LTD", "ENERVALLEY" },
	{ 0xFCDBB3, "Murata Manufacturing Co., Ltd.", "MURATA-MANUFACTURING" },
	{ 0xFCDC4A, "G-Wearables Corp.", "G-WEARABLES" },
	{ 0xFCDD55, "Shenzhen WeWins wireless Co.,Ltd", "SHENZHEN-WEWINS-WIRELESS" },
	{ 0xFCE186, "A3M Co., LTD", "A3M" },
	{ 0xFCE192, "Sichuan Jinwangtong Electronic Science&Technology Co,.Ltd", "SICHUAN-JINWANGTONG-SCIENCE" },
	{ 0xFCE1D9, "Stable Imaging Solutions LLC", "STABLE-IMAGING-SOLUTIONS-LLC" },
	{ 0xFCE1FB, "Array Networks", "ARRAY-NETWORKS" },
	{ 0xFCE23F, "CLAY PAKY SPA", "CLAY-PAKY-SPA" },
	{ 0xFCE33C, "HUAWEI TECHNOLOGIES CO.,LTD", "HUAWEI-TECHNOLOGIES" },
	{ 0xFCE557, "Nokia Corporation", "NOKIA" },
	{ 0xFCE892, "Hangzhou Lancable Technology Co.,Ltd", "HANGZHOU-LANCABLE" },
	{ 0xFCE998, "Apple", "APPLE" },
	{ 0xFCEDB9, "Arrayent", "ARRAYENT" },
	{ 0xFCF152, "Sony Corporation", "SONY" },
	{ 0xFCF1CD, "OPTEX-FA CO.,LTD.", "OPTEX-FA" },
	{ 0xFCF528, "ZyXEL Communications Corporation", "ZYXEL-COMMUNICATION" },
	{ 0xFCF647, "Fiberhome Telecommunication Tech.Co.,Ltd.", "FIBERHOME-TELECOMMUNICATION-TECH" },
	{ 0xFCF8AE, "Intel Corporate", "INTEL-CORPORATE" },
	{ 0xFCF8B7, "TRONTEQ Electronic", "TRONTEQ" },
	{ 0xFCFAF7, "Shanghai Baud Data Communication Co.,Ltd.", "SHANGHAI-BAUD-DATA-COMMUNICATION" },
	{ 0xFCFBFB, "CISCO SYSTEMS, INC.", "CISCO" },
	{ 0xFCFE77, "Hitachi Reftechno, Inc.", "HITACHI-REFTECHNO" },
	{ 0xFCFFAA, "IEEE REGISTRATION AUTHORITY - Please see MAL public listing for more information.", "IEEE-REGISTRATION-AUTHORITY---PLEASE-SEE-MAL-PUBLIC-LISTING-FOR-MORE-INFORMATION" },
	{ 0x525400, "possible QEMU VM", "QEMU-VIRTUAL" },

};
