/*
 * Project       : ipv6calc
 * File          : dbieee_iab.h
 * Version       : $Id: dbieee_iab.h,v 1.77 2015/02/20 07:00:22 ds6peter Exp $
 * Generated     : Fri Feb 20 07:39:36 2015
 * Data copyright: IEEE
 *
 * Information:
 *  Additional header file for libipv6calc_db_wrapper_BuiltIn.c
 */

/*@unused@*/ static const char* libieee_iab_status __attribute__ ((__unused__)) = "IAB/20150218";


static const s_ieee_iab libieee_iab[] = {
	{ 0x0050C2, 0x000000, 0x000FFF, "T.L.S. Corp.", "T-L-S" },
	{ 0x0050C2, 0x001000, 0x001FFF, "JMBS Developpements", "JMBS-DEVELOPPEMENTS" },
	{ 0x0050C2, 0x002000, 0x002FFF, "Integrated Automation Solutions", "INTEGRATED-AUTOMATION-SOLUTIONS" },
	{ 0x0050C2, 0x003000, 0x003FFF, "Microsoft", "MICROSOFT" },
	{ 0x0050C2, 0x004000, 0x004FFF, "SCI Technology Inc.", "SCI" },
	{ 0x0050C2, 0x005000, 0x005FFF, "GD California, Inc.", "GD-CALIFORNIA" },
	{ 0x0050C2, 0x006000, 0x006FFF, "Project Management Enterprises, Inc.", "PROJECT-MANAGEMENT-ENTERPRISES" },
	{ 0x0050C2, 0x007000, 0x007FFF, "Clive Green & Co. Ltd.", "CLIVE-GREEN" },
	{ 0x0050C2, 0x008000, 0x008FFF, "Portable Add-Ons", "PORTABLE-ADD-ONS" },
	{ 0x0050C2, 0x009000, 0x009FFF, "Datakinetics Ltd.", "DATAKINETICS" },
	{ 0x0050C2, 0x00A000, 0x00AFFF, "Tharsys", "THARSYS" },
	{ 0x0050C2, 0x00B000, 0x00BFFF, "IO Limited", "IO" },
	{ 0x0050C2, 0x00C000, 0x00CFFF, "Vbrick Systems Inc.", "VBRICK" },
	{ 0x0050C2, 0x00D000, 0x00DFFF, "Opus Telecom Inc.", "OPUS-TELECOM" },
	{ 0x0050C2, 0x00E000, 0x00EFFF, "TTTech", "TTTECH" },
	{ 0x0050C2, 0x00F000, 0x00FFFF, "XLN-t", "XLN-T" },
	{ 0x0050C2, 0x010000, 0x010FFF, "Moisture Systems", "MOISTURE" },
	{ 0x0050C2, 0x011000, 0x011FFF, "Bihl + Wiedemann GmbH", "BIHL-+-WIEDEMANN" },
	{ 0x0050C2, 0x012000, 0x012FFF, "Floware System Solutions Ltd.", "FLOWARE-SYSTEM-SOLUTIONS" },
	{ 0x0050C2, 0x013000, 0x013FFF, "Sensys Technologies Inc.", "SENSYS-TECHNOLOGIES" },
	{ 0x0050C2, 0x014000, 0x014FFF, "Canal +", "CANAL-+" },
	{ 0x0050C2, 0x015000, 0x015FFF, "Leroy Automatique Industrielle", "LEROY-AUTOMATIQUE-INDUSTRIELLE" },
	{ 0x0050C2, 0x016000, 0x016FFF, "DSP Design Ltd.", "DSP-DESIGN" },
	{ 0x0050C2, 0x017000, 0x017FFF, "Hunter Technology Inc.", "HUNTER" },
	{ 0x0050C2, 0x018000, 0x018FFF, "CAD-UL GmbH", "CAD-UL" },
	{ 0x0050C2, 0x019000, 0x019FFF, "Emtac Technology Corp.", "EMTAC" },
	{ 0x0050C2, 0x01A000, 0x01AFFF, "Skylake Talix", "SKYLAKE-TALIX" },
	{ 0x0050C2, 0x01B000, 0x01BFFF, "Cross Products Ltd.", "CROSS-PRODUCTS" },
	{ 0x0050C2, 0x01C000, 0x01CFFF, "Tadiran Scopus", "TADIRAN-SCOPUS" },
	{ 0x0050C2, 0x01D000, 0x01DFFF, "Princeton Gamma Tech", "PRINCETON-GAMMA-TECH" },
	{ 0x0050C2, 0x01E000, 0x01EFFF, "CallTech International Limited", "CALLTECH" },
	{ 0x0050C2, 0x01F000, 0x01FFFF, "KBS Industrieelektronik GmbH", "KBS-INDUSTRIEELEKTRONIK" },
	{ 0x0050C2, 0x020000, 0x020FFF, "Icon Research Ltd.", "ICON-RESEARCH" },
	{ 0x0050C2, 0x021000, 0x021FFF, "DRS Technologies Canada Co.", "DRS-TECHNOLOGIES-CANADA" },
	{ 0x0050C2, 0x022000, 0x022FFF, "Ashling Microsystems Ltd.", "ASHLING-MICROSYSTEMS" },
	{ 0x0050C2, 0x023000, 0x023FFF, "Zabacom, Inc.", "ZABACOM" },
	{ 0x0050C2, 0x024000, 0x024FFF, "IPITEK", "IPITEK" },
	{ 0x0050C2, 0x025000, 0x025FFF, "Teracom Telematica Ltda.", "TERACOM-TELEMATICA-LTDA" },
	{ 0x0050C2, 0x026000, 0x026FFF, "Abatis Systems Corp.", "ABATIS" },
	{ 0x0050C2, 0x027000, 0x027FFF, "Industrial Control Links", "INDUSTRIAL-CONTROL-LINKS" },
	{ 0x0050C2, 0x028000, 0x028FFF, "The Frensch Corporation (Pty) Ltd.", "THE-FRENSCH-PTY" },
	{ 0x0050C2, 0x029000, 0x029FFF, "Grossenbacher Systeme AG", "GROSSENBACHER-SYSTEME" },
	{ 0x0050C2, 0x02A000, 0x02AFFF, "VersaLogic Corp.", "VERSALOGIC" },
	{ 0x0050C2, 0x02B000, 0x02BFFF, "Nova Engineering Inc.", "NOVA-ENGINEERING" },
	{ 0x0050C2, 0x02C000, 0x02CFFF, "Narrowband Telecommunications", "NARROWBAND-TELECOMMUNICATIONS" },
	{ 0x0050C2, 0x02D000, 0x02DFFF, "Innocor LTD", "INNOCOR" },
	{ 0x0050C2, 0x02E000, 0x02EFFF, "Turtle Mountain Corp", "TURTLE-MOUNTAIN" },
	{ 0x0050C2, 0x02F000, 0x02FFFF, "Sinetica Corp", "SINETICA" },
	{ 0x0050C2, 0x030000, 0x030FFF, "Lockheed Martin Tactical Defense Systems Eagan", "LOCKHEED-MARTIN-TACTICAL-DEFENSE-EAGAN" },
	{ 0x0050C2, 0x031000, 0x031FFF, "Eloquence Ltd", "ELOQUENCE" },
	{ 0x0050C2, 0x032000, 0x032FFF, "MotionIO", "MOTIONIO" },
	{ 0x0050C2, 0x033000, 0x033FFF, "Doble Engineering Company", "DOBLE-ENGINEERING-COMPANY" },
	{ 0x0050C2, 0x034000, 0x034FFF, "Ing. Buero W. Kanis GmbH", "ING-BUERO-W-KANIS" },
	{ 0x0050C2, 0x035000, 0x035FFF, "Alliant Techsystems, Inc.", "ALLIANT-TECHSYSTEMS" },
	{ 0x0050C2, 0x036000, 0x036FFF, "Arcturus Networks Inc.", "ARCTURUS-NETWORKS" },
	{ 0x0050C2, 0x037000, 0x037FFF, "E.I.S.M.", "E-I-S-M" },
	{ 0x0050C2, 0x038000, 0x038FFF, "Etheira Technologies", "ETHEIRA-TECHNOLOGIES" },
	{ 0x0050C2, 0x039000, 0x039FFF, "Apex Signal Corp", "APEX-SIGNAL" },
	{ 0x0050C2, 0x03A000, 0x03AFFF, "PLLB Elettronica SPA", "PLLB-ELETTRONICA-SPA" },
	{ 0x0050C2, 0x03B000, 0x03BFFF, "VNR Electronique SA", "VNR-ELECTRONIQUE" },
	{ 0x0050C2, 0x03C000, 0x03CFFF, "BrainBoxes Ltd", "BRAINBOXES" },
	{ 0x0050C2, 0x03D000, 0x03DFFF, "ISDN Gateway Technology AG", "ISDN-GATEWAY" },
	{ 0x0050C2, 0x03E000, 0x03EFFF, "MSU UK Ltd", "MSU-UK" },
	{ 0x0050C2, 0x03F000, 0x03FFFF, "Celotek Corp", "CELOTEK" },
	{ 0x0050C2, 0x040000, 0x040FFF, "MiSPO Co., Ltd.", "MISPO" },
	{ 0x0050C2, 0x041000, 0x041FFF, "Damler Chrysler Rail System (Signal) AB", "DAMLER-CHRYSLER-RAIL-SYSTEM-SIGNAL-AB" },
	{ 0x0050C2, 0x042000, 0x042FFF, "B.E.A.R. Solutions (Australasia) Pty, Ltd", "B-E-A-R-SOLUTIONS-AUSTRALASIA-PTY" },
	{ 0x0050C2, 0x043000, 0x043FFF, "Curtis, Inc.", "CURTIS" },
	{ 0x0050C2, 0x044000, 0x044FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x045000, 0x045FFF, "Chase Manhattan Bank", "CHASE-MANHATTAN-BANK" },
	{ 0x0050C2, 0x046000, 0x046FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x047000, 0x047FFF, "B. R. Electronics", "B-R-ELECTRONICS" },
	{ 0x0050C2, 0x048000, 0x048FFF, "Cybectec Inc.", "CYBECTEC" },
	{ 0x0050C2, 0x049000, 0x049FFF, "Computer Concepts Corp", "CONCEPTS" },
	{ 0x0050C2, 0x04A000, 0x04AFFF, "Telecom Analysis Systems, LP", "TELECOM-ANALYSIS-LP" },
	{ 0x0050C2, 0x04B000, 0x04BFFF, "Tecstar Demo Systems Division", "TECSTAR-DEMO-DIVISION" },
	{ 0x0050C2, 0x04C000, 0x04CFFF, "New Standard Engineering NV", "NEW-STANDARD-ENGINEERING-NV" },
	{ 0x0050C2, 0x04E000, 0x04EFFF, "Industrial Electronic Engineers, Inc.", "INDUSTRIAL-ENGINEERS" },
	{ 0x0050C2, 0x04F000, 0x04FFFF, "Luma Corporation", "LUMA" },
	{ 0x0050C2, 0x050000, 0x050FFF, "Dataprobe, Inc.", "DATAPROBE" },
	{ 0x0050C2, 0x051000, 0x051FFF, "JSR Ultrasonics", "JSR-ULTRASONICS" },
	{ 0x0050C2, 0x052000, 0x052FFF, "Mayo Foundation", "MAYO-FOUNDATION" },
	{ 0x0050C2, 0x054000, 0x054FFF, "Optionexist Limited", "OPTIONEXIST" },
	{ 0x0050C2, 0x055000, 0x055FFF, "San Castle Technologies, Inc.", "SAN-CASTLE-TECHNOLOGIES" },
	{ 0x0050C2, 0x056000, 0x056FFF, "Base 2", "BASE-2" },
	{ 0x0050C2, 0x057000, 0x057FFF, "Lite F GmBH", "LITE-F" },
	{ 0x0050C2, 0x058000, 0x058FFF, "Vision Research, Inc.", "VISION-RESEARCH" },
	{ 0x0050C2, 0x059000, 0x059FFF, "Austco Communication Systems Pty, Ltd", "AUSTCO-COMMUNICATION-PTY" },
	{ 0x0050C2, 0x05A000, 0x05AFFF, "Sonifex Ltd", "SONIFEX" },
	{ 0x0050C2, 0x05B000, 0x05BFFF, "Radiometer Medical A/S", "RADIOMETER-MEDICAL-A/S" },
	{ 0x0050C2, 0x05C000, 0x05CFFF, "Nortel Networks PLC (UK)", "NORTEL-NETWORKS-PLC-UK" },
	{ 0x0050C2, 0x05D000, 0x05DFFF, "Ignitus Communications, LLC", "IGNITUS-COMMUNICATION-LLC" },
	{ 0x0050C2, 0x05E000, 0x05EFFF, "DIVA Systems", "DIVA" },
	{ 0x0050C2, 0x05F000, 0x05FFFF, "Malden Electronics Ltd", "MALDEN-ELECTRONICS" },
	{ 0x0050C2, 0x060000, 0x060FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x061000, 0x061FFF, "Simple Network Magic Corporation", "SIMPLE-NETWORK-MAGIC" },
	{ 0x0050C2, 0x062000, 0x062FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x063000, 0x063FFF, "Ticketmaster Corp", "TICKETMASTER" },
	{ 0x0050C2, 0x064000, 0x064FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x065000, 0x065FFF, "Clever Devices, Ltd.", "CLEVER-DEVICES" },
	{ 0x0050C2, 0x066000, 0x066FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x067000, 0x067FFF, "Riverlink Computers, Ltd.", "RIVERLINK-COMPUTERS" },
	{ 0x0050C2, 0x068000, 0x068FFF, "Seabridge", "SEABRIDGE" },
	{ 0x0050C2, 0x069000, 0x069FFF, "EC Elettronica S.R.L.", "EC-ELETTRONICA-S-R-L" },
	{ 0x0050C2, 0x06A000, 0x06AFFF, "Unimark", "UNIMARK" },
	{ 0x0050C2, 0x06B000, 0x06BFFF, "NCast Corporation", "NCAST" },
	{ 0x0050C2, 0x06C000, 0x06CFFF, "WaveCom Electronics, Inc.", "WAVECOM-ELECTRONICS" },
	{ 0x0050C2, 0x06D000, 0x06DFFF, "Advanced Signal Corp.", "ADVANCED-SIGNAL" },
	{ 0x0050C2, 0x06E000, 0x06EFFF, "Avtron Manufacturing Inc.", "AVTRON-MANUFACTURING" },
	{ 0x0050C2, 0x06F000, 0x06FFFF, "Digital Services Group", "DIGITAL-SERVICES-GROUP" },
	{ 0x0050C2, 0x070000, 0x070FFF, "Katchall Technologies Group", "KATCHALL-TECHNOLOGIES-GROUP" },
	{ 0x0050C2, 0x071000, 0x071FFF, "NetVision Telecom", "NETVISION-TELECOM" },
	{ 0x0050C2, 0x072000, 0x072FFF, "Neuberger Gebaeudeautomation GmbH & Co.", "NEUBERGER-GEBAEUDEAUTOMATION" },
	{ 0x0050C2, 0x073000, 0x073FFF, "Alstom Signalling Ltd.", "ALSTOM-SIGNALLING" },
	{ 0x0050C2, 0x074000, 0x074FFF, "Edge Tech Co., Ltd.", "EDGE-TECH" },
	{ 0x0050C2, 0x075000, 0x075FFF, "ENTTEC Pty Ltd.", "ENTTEC-PTY" },
	{ 0x0050C2, 0x076000, 0x076FFF, "Litton Guidance & Control Systems", "LITTON-GUIDANCE-CONTROL" },
	{ 0x0050C2, 0x077000, 0x077FFF, "Saco Smartvision Inc.", "SACO-SMARTVISION" },
	{ 0x0050C2, 0x078000, 0x078FFF, "Reselec AG", "RESELEC" },
	{ 0x0050C2, 0x079000, 0x079FFF, "Flextel S.p.A", "FLEXTEL-S-P-A" },
	{ 0x0050C2, 0x07A000, 0x07AFFF, "RadioTel", "RADIOTEL" },
	{ 0x0050C2, 0x07B000, 0x07BFFF, "Trikon Technologies Ltd.", "TRIKON-TECHNOLOGIES" },
	{ 0x0050C2, 0x07C000, 0x07CFFF, "PLLB elettronica spa", "PLLB-ELETTRONICA-SPA" },
	{ 0x0050C2, 0x07D000, 0x07DFFF, "Caspian Networks", "CASPIAN-NETWORKS" },
	{ 0x0050C2, 0x07E000, 0x07EFFF, "JL-teknik", "JL-TEKNIK" },
	{ 0x0050C2, 0x07F000, 0x07FFFF, "Dunti Corporation", "DUNTI" },
	{ 0x0050C2, 0x080000, 0x080FFF, "AIM", "AIM" },
	{ 0x0050C2, 0x081000, 0x081FFF, "Matuschek Messtechnik GmbH", "MATUSCHEK-MESSTECHNIK" },
	{ 0x0050C2, 0x082000, 0x082FFF, "GFI Chrono Time", "GFI-CHRONO-TIME" },
	{ 0x0050C2, 0x083000, 0x083FFF, "ARD SA", "ARD" },
	{ 0x0050C2, 0x084000, 0x084FFF, "DIALOG4 System Engineering GmbH", "DIALOG4-SYSTEM-ENGINEERING" },
	{ 0x0050C2, 0x085000, 0x085FFF, "Crossport Systems", "CROSSPORT" },
	{ 0x0050C2, 0x086000, 0x086FFF, "Validyne Engineering Corp.", "VALIDYNE-ENGINEERING" },
	{ 0x0050C2, 0x087000, 0x087FFF, "Monitor Business Machines Ltd.", "MONITOR-BUSINESS-MACHINES" },
	{ 0x0050C2, 0x088000, 0x088FFF, "TELINC Corporation", "TELINC" },
	{ 0x0050C2, 0x089000, 0x089FFF, "Fenwal Italia S.P.A.", "FENWAL-ITALIA-S-P-A" },
	{ 0x0050C2, 0x08A000, 0x08AFFF, "Rising Edge Technologies", "RISING-EDGE-TECHNOLOGIES" },
	{ 0x0050C2, 0x08B000, 0x08BFFF, "HYPERCHIP Inc.", "HYPERCHIP" },
	{ 0x0050C2, 0x08C000, 0x08CFFF, "IP Unity", "IP-UNITY" },
	{ 0x0050C2, 0x08D000, 0x08DFFF, "Kylink Communications Corp.", "KYLINK-COMMUNICATION" },
	{ 0x0050C2, 0x08E000, 0x08EFFF, "BSQUARE", "BSQUARE" },
	{ 0x0050C2, 0x08F000, 0x08FFFF, "General Industries Argentina", "GENERAL-INDUSTRIES-ARGENTINA" },
	{ 0x0050C2, 0x090000, 0x090FFF, "Invensys Controls Network Systems", "INVENSYS-CONTROLS-NETWORK" },
	{ 0x0050C2, 0x091000, 0x091FFF, "StorLogic, Inc.", "STORLOGIC" },
	{ 0x0050C2, 0x092000, 0x092FFF, "DigitAll World Co., Ltd", "DIGITALL-WORLD" },
	{ 0x0050C2, 0x093000, 0x093FFF, "KOREALINK", "KOREALINK" },
	{ 0x0050C2, 0x094000, 0x094FFF, "Analytical Spectral Devices, Inc.", "ANALYTICAL-SPECTRAL-DEVICES" },
	{ 0x0050C2, 0x095000, 0x095FFF, "SEATECH", "SEATECH" },
	{ 0x0050C2, 0x096000, 0x096FFF, "Utronix Elektronikutreckling AB", "UTRONIX-ELEKTRONIKUTRECKLING-AB" },
	{ 0x0050C2, 0x097000, 0x097FFF, "IMV Invertomatic", "IMV-INVERTOMATIC" },
	{ 0x0050C2, 0x098000, 0x098FFF, "EPEL Industrial, S.A.", "EPEL-INDUSTRIAL" },
	{ 0x0050C2, 0x099000, 0x099FFF, "Case Information & Communications", "CASE-INFORMATION-COMMUNICATION" },
	{ 0x0050C2, 0x09A000, 0x09AFFF, "NBO Development Center Sekusui Chemical Co. Ltd.", "NBO-DEVELOPMENT-CENTER-SEKUSUI-CHEMICAL" },
	{ 0x0050C2, 0x09B000, 0x09BFFF, "Seffle Instrument AB", "SEFFLE-INSTRUMENT-AB" },
	{ 0x0050C2, 0x09C000, 0x09CFFF, "RF Applications, Inc.", "RF-APPLICATIONS" },
	{ 0x0050C2, 0x09D000, 0x09DFFF, "ZELPOS", "ZELPOS" },
	{ 0x0050C2, 0x09E000, 0x09EFFF, "Infinitec Networks, Inc.", "INFINITEC-NETWORKS" },
	{ 0x0050C2, 0x09F000, 0x09FFFF, "MetaWave Vedeo Systems", "METAWAVE-VEDEO" },
	{ 0x0050C2, 0x0A0000, 0x0A0FFF, "CYNAPS", "CYNAPS" },
	{ 0x0050C2, 0x0A1000, 0x0A1FFF, "Visable Genetics, Inc.", "VISABLE-GENETICS" },
	{ 0x0050C2, 0x0A2000, 0x0A2FFF, "Jäger Computergesteuerte Messtechnik GmbH", "JAEGER-COMPUTERGESTEUERTE-MESSTECHNIK" },
	{ 0x0050C2, 0x0A3000, 0x0A3FFF, "BaSyTec GmbH", "BASYTEC" },
	{ 0x0050C2, 0x0A4000, 0x0A4FFF, "Bounty Systems Pty Ltd.", "BOUNTY-PTY" },
	{ 0x0050C2, 0x0A5000, 0x0A5FFF, "Mobiltex Data Ltd.", "MOBILTEX-DATA" },
	{ 0x0050C2, 0x0A6000, 0x0A6FFF, "Arula Systems, Inc.", "ARULA" },
	{ 0x0050C2, 0x0A7000, 0x0A7FFF, "WaterCove Networks", "WATERCOVE-NETWORKS" },
	{ 0x0050C2, 0x0A8000, 0x0A8FFF, "Kaveri Networks", "KAVERI-NETWORKS" },
	{ 0x0050C2, 0x0A9000, 0x0A9FFF, "Radiant Networks Plc", "RADIANT-NETWORKS-PLC" },
	{ 0x0050C2, 0x0AA000, 0x0AAFFF, "Log-In, Inc.", "LOG-IN" },
	{ 0x0050C2, 0x0AB000, 0x0ABFFF, "Fastware.Net, LLC", "FASTWARE-NET-LLC" },
	{ 0x0050C2, 0x0AC000, 0x0ACFFF, "Honeywell GNO", "HONEYWELL-GNO" },
	{ 0x0050C2, 0x0AD000, 0x0ADFFF, "BMC Messsysteme GmbH", "BMC-MESSSYSTEME" },
	{ 0x0050C2, 0x0AE000, 0x0AEFFF, "Zarak Systems Corp.", "ZARAK" },
	{ 0x0050C2, 0x0AF000, 0x0AFFFF, "Latus Lightworks, Inc.", "LATUS-LIGHTWORKS" },
	{ 0x0050C2, 0x0B0000, 0x0B0FFF, "LMI Technologies, Inc.", "LMI-TECHNOLOGIES" },
	{ 0x0050C2, 0x0B1000, 0x0B1FFF, "Beeline Networks, Inc.", "BEELINE-NETWORKS" },
	{ 0x0050C2, 0x0B2000, 0x0B2FFF, "R F Micro Devices", "R-F-MICRO-DEVICES" },
	{ 0x0050C2, 0x0B3000, 0x0B3FFF, "SMX Corporation", "SMX" },
	{ 0x0050C2, 0x0B4000, 0x0B4FFF, "Wavefly Corporation", "WAVEFLY" },
	{ 0x0050C2, 0x0B5000, 0x0B5FFF, "Extreme Copper, Inc.", "EXTREME-COPPER" },
	{ 0x0050C2, 0x0B6000, 0x0B6FFF, "ApSecure Technologies (Canada), Inc.", "APSECURE-TECHNOLOGIES-CANADA" },
	{ 0x0050C2, 0x0B7000, 0x0B7FFF, "RYMIC", "RYMIC" },
	{ 0x0050C2, 0x0B8000, 0x0B8FFF, "LAN Controls, Inc.", "LAN-CONTROLS" },
	{ 0x0050C2, 0x0B9000, 0x0B9FFF, "Helmut Mauell GmbH", "HELMUT-MAUELL" },
	{ 0x0050C2, 0x0BA000, 0x0BAFFF, "Pro-Active", "PRO-ACTIVE" },
	{ 0x0050C2, 0x0BB000, 0x0BBFFF, "MAZet GmbH", "MAZET" },
	{ 0x0050C2, 0x0BC000, 0x0BCFFF, "Infolink Software AG", "INFOLINK-SOFTWARE" },
	{ 0x0050C2, 0x0BD000, 0x0BDFFF, "Tattile", "TATTILE" },
	{ 0x0050C2, 0x0BE000, 0x0BEFFF, "Stella Electronics & Tagging", "STELLA-ELECTRONICS-TAGGING" },
	{ 0x0050C2, 0x0BF000, 0x0BFFFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x0C0000, 0x0C0FFF, "Imigix Ltd.", "IMIGIX" },
	{ 0x0050C2, 0x0C1000, 0x0C1FFF, "Casabyte", "CASABYTE" },
	{ 0x0050C2, 0x0C2000, 0x0C2FFF, "Alchemy Semiconductor, Inc.", "ALCHEMY-SEMICONDUCTOR" },
	{ 0x0050C2, 0x0C3000, 0x0C3FFF, "Tonbu, Inc.", "TONBU" },
	{ 0x0050C2, 0x0C4000, 0x0C4FFF, "InterEpoch Technology, Inc.", "INTEREPOCH" },
	{ 0x0050C2, 0x0C5000, 0x0C5FFF, "SAIA Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x0C6000, 0x0C6FFF, "Advanced Medical Information Technologies, Inc.", "ADVANCED-MEDICAL-INFORMATION-TECHNOLOGIES" },
	{ 0x0050C2, 0x0C7000, 0x0C7FFF, "TransComm Technology System, Inc.", "TRANSCOMM-SYSTEM" },
	{ 0x0050C2, 0x0C8000, 0x0C8FFF, "The Trane Company", "THE-TRANE-COMPANY" },
	{ 0x0050C2, 0x0C9000, 0x0C9FFF, "DSS Networks, Inc.", "DSS-NETWORKS" },
	{ 0x0050C2, 0x0CA000, 0x0CAFFF, "J D Richards", "J-D-RICHARDS" },
	{ 0x0050C2, 0x0CB000, 0x0CBFFF, "STUDIEL", "STUDIEL" },
	{ 0x0050C2, 0x0CC000, 0x0CCFFF, "AlphaMedia Co., Ltd", "ALPHAMEDIA" },
	{ 0x0050C2, 0x0CD000, 0x0CDFFF, "LINET OY", "LINET-OY" },
	{ 0x0050C2, 0x0CE000, 0x0CEFFF, "RFL Electronics, Inc.", "RFL-ELECTRONICS" },
	{ 0x0050C2, 0x0CF000, 0x0CFFFF, "PCSC", "PCSC" },
	{ 0x0050C2, 0x0D0000, 0x0D0FFF, "Telefrang AB", "TELEFRANG-AB" },
	{ 0x0050C2, 0x0D1000, 0x0D1FFF, "Renaissance Networking, Inc.", "RENAISSANCE-NETWORKING" },
	{ 0x0050C2, 0x0D2000, 0x0D2FFF, "Real World Computing Partnership", "REAL-WORLD-COMPUTING-PARTNERSHIP" },
	{ 0x0050C2, 0x0D3000, 0x0D3FFF, "Lake Technology, Ltd.", "LAKE" },
	{ 0x0050C2, 0x0D4000, 0x0D4FFF, "Palm, Inc.", "PALM" },
	{ 0x0050C2, 0x0D5000, 0x0D5FFF, "Zelax", "ZELAX" },
	{ 0x0050C2, 0x0D6000, 0x0D6FFF, "Inco Startec GmbH", "INCO-STARTEC" },
	{ 0x0050C2, 0x0D7000, 0x0D7FFF, "Summit Avionics, Inc.", "SUMMIT-AVIONICS" },
	{ 0x0050C2, 0x0D8000, 0x0D8FFF, "Charlotte's Web Networks", "CHARLOTTE-S-WEB-NETWORKS" },
	{ 0x0050C2, 0x0D9000, 0x0D9FFF, "Loewe Opta GmbH", "LOEWE-OPTA" },
	{ 0x0050C2, 0x0DA000, 0x0DAFFF, "Motion Analysis Corp.", "MOTION-ANALYSIS" },
	{ 0x0050C2, 0x0DB000, 0x0DBFFF, "Cyberex", "CYBEREX" },
	{ 0x0050C2, 0x0DC000, 0x0DCFFF, "Elbit Systems Ltd.", "ELBIT" },
	{ 0x0050C2, 0x0DD000, 0x0DDFFF, "Interisa Electronica, S.A.", "INTERISA-ELECTRONICA" },
	{ 0x0050C2, 0x0DE000, 0x0DEFFF, "Frederick Engineering", "FREDERICK-ENGINEERING" },
	{ 0x0050C2, 0x0DF000, 0x0DFFFF, "Innovation Institute, Inc.", "INNOVATION-INSTITUTE" },
	{ 0x0050C2, 0x0E0000, 0x0E0FFF, "EMAC, Inc.", "EMAC" },
	{ 0x0050C2, 0x0E1000, 0x0E1FFF, "Inspiration Technology P/L", "INSPIRATION-P/L" },
	{ 0x0050C2, 0x0E2000, 0x0E2FFF, "Visual Circuits Corp.", "VISUAL-CIRCUITS" },
	{ 0x0050C2, 0x0E3000, 0x0E3FFF, "Lanex S.A.", "LANEX" },
	{ 0x0050C2, 0x0E4000, 0x0E4FFF, "Collabo Tec. Co., Ltd.", "COLLABO-TEC" },
	{ 0x0050C2, 0x0E5000, 0x0E5FFF, "Clearwater Networks", "CLEARWATER-NETWORKS" },
	{ 0x0050C2, 0x0E6000, 0x0E6FFF, "RouteFree, Inc.", "ROUTEFREE" },
	{ 0x0050C2, 0x0E7000, 0x0E7FFF, "Century Geophysical Corp.", "CENTURY-GEOPHYSICAL" },
	{ 0x0050C2, 0x0E8000, 0x0E8FFF, "Audio Design Associates, Inc.", "AUDIO-DESIGN-ASSOCIATES" },
	{ 0x0050C2, 0x0E9000, 0x0E9FFF, "Smartmedia LLC", "SMARTMEDIA-LLC" },
	{ 0x0050C2, 0x0EA000, 0x0EAFFF, "iReady Corporation", "IREADY" },
	{ 0x0050C2, 0x0EB000, 0x0EBFFF, "iREZ Technologies LLC", "IREZ-TECHNOLOGIES-LLC" },
	{ 0x0050C2, 0x0EC000, 0x0ECFFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x0ED000, 0x0EDFFF, "Valley Products Corporation", "VALLEY-PRODUCTS" },
	{ 0x0050C2, 0x0EE000, 0x0EEFFF, "Industrial Indexing Systems, Inc.", "INDUSTRIAL-INDEXING" },
	{ 0x0050C2, 0x0EF000, 0x0EFFFF, "Movaz Networks, Inc.", "MOVAZ-NETWORKS" },
	{ 0x0050C2, 0x0F0000, 0x0F0FFF, "VHB Technologies, Inc.", "VHB-TECHNOLOGIES" },
	{ 0x0050C2, 0x0F1000, 0x0F1FFF, "Polyvision Corporation", "POLYVISION" },
	{ 0x0050C2, 0x0F2000, 0x0F2FFF, "KMS Systems, Inc.", "KMS" },
	{ 0x0050C2, 0x0F3000, 0x0F3FFF, "Young Computer Co., Ltd.", "YOUNG" },
	{ 0x0050C2, 0x0F4000, 0x0F4FFF, "Sysnet Co., Ltd.", "SYSNET" },
	{ 0x0050C2, 0x0F5000, 0x0F5FFF, "Spectra Technologies Holding Co., Ltd.", "SPECTRA-TECHNOLOGIES-HOLDING" },
	{ 0x0050C2, 0x0F6000, 0x0F6FFF, "Carl Baasel Lasertechnik GmbH", "CARL-BAASEL-LASERTECHNIK" },
	{ 0x0050C2, 0x0F7000, 0x0F7FFF, "Foss NIRSystems, Inc.", "FOSS-NIRSYSTEMS" },
	{ 0x0050C2, 0x0F8000, 0x0F8FFF, "Tecnint HTE S.r.L.", "TECNINT-HTE-S-R-L" },
	{ 0x0050C2, 0x0F9000, 0x0F9FFF, "Raven Industries", "RAVEN-INDUSTRIES" },
	{ 0x0050C2, 0x0FA000, 0x0FAFFF, "GE Lubrizol, LLC", "GE-LUBRIZOL-LLC" },
	{ 0x0050C2, 0x0FB000, 0x0FBFFF, "PIUSYS Co., Ltd.", "PIUSYS" },
	{ 0x0050C2, 0x0FC000, 0x0FCFFF, "Kimmon Manufacturing Co., Ltd.", "KIMMON-MANUFACTURING" },
	{ 0x0050C2, 0x0FD000, 0x0FDFFF, "Inducomp Corporation", "INDUCOMP" },
	{ 0x0050C2, 0x0FE000, 0x0FEFFF, "Energy ICT", "ENERGY-ICT" },
	{ 0x0050C2, 0x0FF000, 0x0FFFFF, "IPAXS Corporation", "IPAXS" },
	{ 0x0050C2, 0x100000, 0x100FFF, "Corelatus A.B.", "CORELATUS-A-B" },
	{ 0x0050C2, 0x101000, 0x101FFF, "LAUD Electronic Design AS", "LAUD-DESIGN-AS" },
	{ 0x0050C2, 0x102000, 0x102FFF, "Million Tech Development Ltd.", "MILLION-TECH-DEVELOPMENT" },
	{ 0x0050C2, 0x103000, 0x103FFF, "Green Hills Software, Inc.", "GREEN-HILLS-SOFTWARE" },
	{ 0x0050C2, 0x104000, 0x104FFF, "Adescom Inc.", "ADESCOM" },
	{ 0x0050C2, 0x105000, 0x105FFF, "Lumentis AB", "LUMENTIS-AB" },
	{ 0x0050C2, 0x106000, 0x106FFF, "MATSUOKA", "MATSUOKA" },
	{ 0x0050C2, 0x107000, 0x107FFF, "NewHer Systems", "NEWHER" },
	{ 0x0050C2, 0x108000, 0x108FFF, "Balogh S.A.", "BALOGH" },
	{ 0x0050C2, 0x109000, 0x109FFF, "ITK Dr. Kassen GmbH", "ITK-DR-KASSEN" },
	{ 0x0050C2, 0x10A000, 0x10AFFF, "Quinx AG", "QUINX" },
	{ 0x0050C2, 0x10B000, 0x10BFFF, "MarekMicro GmbH", "MAREKMICRO" },
	{ 0x0050C2, 0x10C000, 0x10CFFF, "Photonic Bridges, Inc.", "PHOTONIC-BRIDGES" },
	{ 0x0050C2, 0x10D000, 0x10DFFF, "Implementa GmbH", "IMPLEMENTA" },
	{ 0x0050C2, 0x10E000, 0x10EFFF, "Unipower AB", "UNIPOWER-AB" },
	{ 0x0050C2, 0x10F000, 0x10FFFF, "Perceptics Corp.", "PERCEPTICS" },
	{ 0x0050C2, 0x110000, 0x110FFF, "QuesCom", "QUESCOM" },
	{ 0x0050C2, 0x111000, 0x111FFF, "Endusis Limited", "ENDUSIS" },
	{ 0x0050C2, 0x112000, 0x112FFF, "Compuworx", "COMPUWORX" },
	{ 0x0050C2, 0x113000, 0x113FFF, "Ace Electronics, Inc.", "ACE-ELECTRONICS" },
	{ 0x0050C2, 0x114000, 0x114FFF, "Quest Innovations", "QUEST-INNOVATIONS" },
	{ 0x0050C2, 0x115000, 0x115FFF, "Vidco, Inc.", "VIDCO" },
	{ 0x0050C2, 0x116000, 0x116FFF, "DSP Design, Ltd.", "DSP-DESIGN" },
	{ 0x0050C2, 0x117000, 0x117FFF, "Wintegra Ltd.", "WINTEGRA" },
	{ 0x0050C2, 0x118000, 0x118FFF, "Microbit 2.0 AB", "MICROBIT-2-0-AB" },
	{ 0x0050C2, 0x119000, 0x119FFF, "Global Opto Communication Tech. Corp", "GLOBAL-OPTO-COMMUNICATION-TECH" },
	{ 0x0050C2, 0x11A000, 0x11AFFF, "Teamaxess Ticketing GmbH", "TEAMAXESS-TICKETING" },
	{ 0x0050C2, 0x11B000, 0x11BFFF, "Digital Vision AB", "DIGITAL-VISION-AB" },
	{ 0x0050C2, 0x11C000, 0x11CFFF, "Stonefly Networks", "STONEFLY-NETWORKS" },
	{ 0x0050C2, 0x11D000, 0x11DFFF, "Destiny Networks, Inc.", "DESTINY-NETWORKS" },
	{ 0x0050C2, 0x11E000, 0x11EFFF, "Volvo Car Corporation", "VOLVO-CAR" },
	{ 0x0050C2, 0x11F000, 0x11FFFF, "CSS Industrie Computer GmbH", "CSS-INDUSTRIE" },
	{ 0x0050C2, 0x120000, 0x120FFF, "XStore, Inc.", "XSTORE" },
	{ 0x0050C2, 0x121000, 0x121FFF, "COE Limited", "COE" },
	{ 0x0050C2, 0x122000, 0x122FFF, "Diva Systems", "DIVA" },
	{ 0x0050C2, 0x123000, 0x123FFF, "Seranoa Networks, Inc.", "SERANOA-NETWORKS" },
	{ 0x0050C2, 0x124000, 0x124FFF, "Tokai Soft Corporation", "TOKAI-SOFT" },
	{ 0x0050C2, 0x125000, 0x125FFF, "Tecwings GmBh", "TECWINGS" },
	{ 0x0050C2, 0x126000, 0x126FFF, "Marvell Hispana S.L.", "MARVELL-HISPANA-S-L" },
	{ 0x0050C2, 0x127000, 0x127FFF, "TPA Traffic & Parking Automation BV", "TPA-TRAFFIC-PARKING-AUTOMATION" },
	{ 0x0050C2, 0x128000, 0x128FFF, "Pycon, Inc.", "PYCON" },
	{ 0x0050C2, 0x129000, 0x129FFF, "TTPCom Ltd.", "TTPCOM" },
	{ 0x0050C2, 0x12A000, 0x12AFFF, "Symbolic Sound Corp.", "SYMBOLIC-SOUND" },
	{ 0x0050C2, 0x12B000, 0x12BFFF, "Dong A Eltek Co., Ltd.", "DONG-A-ELTEK" },
	{ 0x0050C2, 0x12C000, 0x12CFFF, "Delta Tau Data Systems, Inc.", "DELTA-TAU-DATA" },
	{ 0x0050C2, 0x12D000, 0x12DFFF, "Megisto Systems, Inc.", "MEGISTO" },
	{ 0x0050C2, 0x12E000, 0x12EFFF, "RUNCOM", "RUNCOM" },
	{ 0x0050C2, 0x12F000, 0x12FFFF, "HAAG-STREIT AG", "HAAG-STREIT" },
	{ 0x0050C2, 0x130000, 0x130FFF, "U.S. Traffic Corporation", "U-S-TRAFFIC" },
	{ 0x0050C2, 0x131000, 0x131FFF, "InBus Engineering, Inc.", "INBUS-ENGINEERING" },
	{ 0x0050C2, 0x132000, 0x132FFF, "Procon Electronics", "PROCON-ELECTRONICS" },
	{ 0x0050C2, 0x133000, 0x133FFF, "ChipWrights, Inc.", "CHIPWRIGHTS" },
	{ 0x0050C2, 0x134000, 0x134FFF, "DRS Photronics", "DRS-PHOTRONICS" },
	{ 0x0050C2, 0x135000, 0x135FFF, "ELAD SRL", "ELAD-SRL" },
	{ 0x0050C2, 0x136000, 0x136FFF, "Tensilica, Inc.", "TENSILICA" },
	{ 0x0050C2, 0x137000, 0x137FFF, "Uniwell Systems (UK) Ltd.", "UNIWELL-UK" },
	{ 0x0050C2, 0x138000, 0x138FFF, "Delphin Technology AG", "DELPHIN" },
	{ 0x0050C2, 0x139000, 0x139FFF, "SR Research Ltd.", "SR-RESEARCH" },
	{ 0x0050C2, 0x13A000, 0x13AFFF, "Tex Computer SRL", "TEX-SRL" },
	{ 0x0050C2, 0x13B000, 0x13BFFF, "Vaisala Oyj", "VAISALA-OYJ" },
	{ 0x0050C2, 0x13C000, 0x13CFFF, "NBG Industrial Automation B.V.", "NBG-INDUSTRIAL-AUTOMATION" },
	{ 0x0050C2, 0x13D000, 0x13DFFF, "Formula One Management Ltd.", "FORMULA-ONE-MANAGEMENT" },
	{ 0x0050C2, 0x13E000, 0x13EFFF, "AVerMedia Systems, Inc.", "AVERMEDIA" },
	{ 0x0050C2, 0x13F000, 0x13FFFF, "Sentito Networks", "SENTITO-NETWORKS" },
	{ 0x0050C2, 0x140000, 0x140FFF, "ITS, Inc.", "ITS" },
	{ 0x0050C2, 0x141000, 0x141FFF, "Time Terminal Adductor Group AB", "TIME-TERMINAL-ADDUCTOR-GROUP-AB" },
	{ 0x0050C2, 0x142000, 0x142FFF, "Instrumeter A/S", "INSTRUMETER-A/S" },
	{ 0x0050C2, 0x143000, 0x143FFF, "AARTESYS AG", "AARTESYS" },
	{ 0x0050C2, 0x144000, 0x144FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x145000, 0x145FFF, "ELC Lighting", "ELC-LIGHTING" },
	{ 0x0050C2, 0x146000, 0x146FFF, "APCON, Inc.", "APCON" },
	{ 0x0050C2, 0x147000, 0x147FFF, "UniSUR", "UNISUR" },
	{ 0x0050C2, 0x148000, 0x148FFF, "Alltec GmbH", "ALLTEC" },
	{ 0x0050C2, 0x149000, 0x149FFF, "Haag-Streit AG", "HAAG-STREIT" },
	{ 0x0050C2, 0x14A000, 0x14AFFF, "DYCEC, S.A.", "DYCEC" },
	{ 0x0050C2, 0x14B000, 0x14BFFF, "HECUBA Elektronik", "HECUBA-ELEKTRONIK" },
	{ 0x0050C2, 0x14C000, 0x14CFFF, "Optibase Ltd.", "OPTIBASE" },
	{ 0x0050C2, 0x14D000, 0x14DFFF, "wellink, Ltd.", "WELLINK" },
	{ 0x0050C2, 0x14E000, 0x14EFFF, "Corinex Global", "CORINEX-GLOBAL" },
	{ 0x0050C2, 0x14F000, 0x14FFFF, "Telephonics Corp.", "TELEPHONICS" },
	{ 0x0050C2, 0x150000, 0x150FFF, "Torse", "TORSE" },
	{ 0x0050C2, 0x151000, 0x151FFF, "Redux Communications Ltd.", "REDUX-COMMUNICATION" },
	{ 0x0050C2, 0x152000, 0x152FFF, "AirVast Technology Inc.", "AIRVAST" },
	{ 0x0050C2, 0x153000, 0x153FFF, "Advanced Devices SpA", "ADVANCED-DEVICES-SPA" },
	{ 0x0050C2, 0x154000, 0x154FFF, "Jostra AB", "JOSTRA-AB" },
	{ 0x0050C2, 0x155000, 0x155FFF, "Enea Real Time AB", "ENEA-REAL-TIME-AB" },
	{ 0x0050C2, 0x156000, 0x156FFF, "CommServ Solutions Inc.", "COMMSERV-SOLUTIONS" },
	{ 0x0050C2, 0x157000, 0x157FFF, "nCore, Inc.", "NCORE" },
	{ 0x0050C2, 0x158000, 0x158FFF, "Communication Solutions, Inc.", "COMMUNICATION-SOLUTIONS" },
	{ 0x0050C2, 0x159000, 0x159FFF, "Standard Comm. Corp.", "STANDARD-COMM" },
	{ 0x0050C2, 0x15A000, 0x15AFFF, "Plextek Limited", "PLEXTEK" },
	{ 0x0050C2, 0x15B000, 0x15BFFF, "Dune Networks", "DUNE-NETWORKS" },
	{ 0x0050C2, 0x15C000, 0x15CFFF, "Aoptix Technologies", "AOPTIX-TECHNOLOGIES" },
	{ 0x0050C2, 0x15D000, 0x15DFFF, "Cepheid", "CEPHEID" },
	{ 0x0050C2, 0x15E000, 0x15EFFF, "Celite Systems, Inc.", "CELITE" },
	{ 0x0050C2, 0x15F000, 0x15FFFF, "Pulsar GmbH", "PULSAR" },
	{ 0x0050C2, 0x160000, 0x160FFF, "TTI - Telecom International Ltd.", "TTI---TELECOM" },
	{ 0x0050C2, 0x161000, 0x161FFF, "J&B Engineering Group S.L.", "J-B-ENGINEERING-GROUP-S-L" },
	{ 0x0050C2, 0x162000, 0x162FFF, "Teseda Corporation", "TESEDA" },
	{ 0x0050C2, 0x163000, 0x163FFF, "Computerwise, Inc.", "COMPUTERWISE" },
	{ 0x0050C2, 0x164000, 0x164FFF, "Acunia N.V.", "ACUNIA-N-V" },
	{ 0x0050C2, 0x165000, 0x165FFF, "IPCAST", "IPCAST" },
	{ 0x0050C2, 0x166000, 0x166FFF, "Infineer Ltd.", "INFINEER" },
	{ 0x0050C2, 0x167000, 0x167FFF, "Precision Filters, Inc.", "PRECISION-FILTERS" },
	{ 0x0050C2, 0x168000, 0x168FFF, "ExtremeSpeed Inc.", "EXTREMESPEED" },
	{ 0x0050C2, 0x169000, 0x169FFF, "Nordson Corp.", "NORDSON" },
	{ 0x0050C2, 0x16A000, 0x16AFFF, "Time Domain", "TIME-DOMAIN" },
	{ 0x0050C2, 0x16B000, 0x16BFFF, "Masterclock, Inc.", "MASTERCLOCK" },
	{ 0x0050C2, 0x16C000, 0x16CFFF, "Brijing Embedor Embedded Internet Tech. Co. Ltd.", "BRIJING-EMBEDOR-EMBEDDED-INTERNET-TECH" },
	{ 0x0050C2, 0x16D000, 0x16DFFF, "Postec Data Systems Ltd.", "POSTEC-DATA" },
	{ 0x0050C2, 0x16E000, 0x16EFFF, "PMC", "PMC" },
	{ 0x0050C2, 0x16F000, 0x16FFFF, "Dickson Technologies", "DICKSON-TECHNOLOGIES" },
	{ 0x0050C2, 0x170000, 0x170FFF, "Taishodo Seiko Co., Ltd.", "TAISHODO-SEIKO" },
	{ 0x0050C2, 0x171000, 0x171FFF, "Quantronix, Inc.", "QUANTRONIX" },
	{ 0x0050C2, 0x172000, 0x172FFF, "SAET I.S. S.r.l.", "SAET-I-S-S-R-L" },
	{ 0x0050C2, 0x173000, 0x173FFF, "DeMeTec GmbH", "DEMETEC" },
	{ 0x0050C2, 0x174000, 0x174FFF, "N&P Technologies", "N-P-TECHNOLOGIES" },
	{ 0x0050C2, 0x175000, 0x175FFF, "Sei S.p.A.", "SEI-S-P-A" },
	{ 0x0050C2, 0x176000, 0x176FFF, "Wavium AB", "WAVIUM-AB" },
	{ 0x0050C2, 0x177000, 0x177FFF, "Unicoi Systems", "UNICOI" },
	{ 0x0050C2, 0x178000, 0x178FFF, "Partner Voxstream A/S", "PARTNER-VOXSTREAM-A/S" },
	{ 0x0050C2, 0x179000, 0x179FFF, "Verifiber LLC", "VERIFIBER-LLC" },
	{ 0x0050C2, 0x17A000, 0x17AFFF, "WOLF Industrial Systems Inc.", "WOLF-INDUSTRIAL" },
	{ 0x0050C2, 0x17B000, 0x17BFFF, "Broadstorm Telecom", "BROADSTORM-TELECOM" },
	{ 0x0050C2, 0x17C000, 0x17CFFF, "Jeffress Engineering Pty Ltd", "JEFFRESS-ENGINEERING-PTY" },
	{ 0x0050C2, 0x17D000, 0x17DFFF, "Cognex Corporation", "COGNEX" },
	{ 0x0050C2, 0x17E000, 0x17EFFF, "Binary Wave Technologies Inc.", "BINARY-WAVE-TECHNOLOGIES" },
	{ 0x0050C2, 0x17F000, 0x17FFFF, "PDQ Manufacturing", "PDQ-MANUFACTURING" },
	{ 0x0050C2, 0x180000, 0x180FFF, "Zultys Technologies", "ZULTYS-TECHNOLOGIES" },
	{ 0x0050C2, 0x181000, 0x181FFF, "Task 84 Spa", "TASK-84-SPA" },
	{ 0x0050C2, 0x182000, 0x182FFF, "wolf-inf-tec", "WOLF-INF-TEC" },
	{ 0x0050C2, 0x183000, 0x183FFF, "Mixbaal S.A. de C.V.", "MIXBAAL-DE-C-V" },
	{ 0x0050C2, 0x184000, 0x184FFF, "H M Computing Limited", "H-M-COMPUTING" },
	{ 0x0050C2, 0x185000, 0x185FFF, "Optical Wireless Link Inc.", "OPTICAL-WIRELESS-LINK" },
	{ 0x0050C2, 0x186000, 0x186FFF, "Pantec Engineering AG", "PANTEC-ENGINEERING" },
	{ 0x0050C2, 0x187000, 0x187FFF, "Cyan Technology Ltd", "CYAN" },
	{ 0x0050C2, 0x188000, 0x188FFF, "dresden-elektronik", "DRESDEN-ELEKTRONIK" },
	{ 0x0050C2, 0x189000, 0x189FFF, "CC Systems AB", "CC-AB" },
	{ 0x0050C2, 0x18A000, 0x18AFFF, "Basler Electric Company", "BASLER-ELECTRIC-COMPANY" },
	{ 0x0050C2, 0x18B000, 0x18BFFF, "Teradyne Inc.", "TERADYNE" },
	{ 0x0050C2, 0x18C000, 0x18CFFF, "Technodrive srl", "TECHNODRIVE-SRL" },
	{ 0x0050C2, 0x18D000, 0x18DFFF, "CCII Systems (Pty) Ltd", "CCII-PTY" },
	{ 0x0050C2, 0x18E000, 0x18EFFF, "SPARR ELECTRONICS LTD", "SPARR-ELECTRONICS" },
	{ 0x0050C2, 0x18F000, 0x18FFFF, "MATSUI MFG CO.,LTD", "MATSUI-MFG" },
	{ 0x0050C2, 0x190000, 0x190FFF, "Goerlitz AG", "GOERLITZ" },
	{ 0x0050C2, 0x191000, 0x191FFF, "Partner Voxstream A/S", "PARTNER-VOXSTREAM-A/S" },
	{ 0x0050C2, 0x192000, 0x192FFF, "Advanced Concepts, Inc.", "ADVANCED-CONCEPTS" },
	{ 0x0050C2, 0x193000, 0x193FFF, "LaserBit Communications Corp.", "LASERBIT-COMMUNICATION" },
	{ 0x0050C2, 0x194000, 0x194FFF, "COMINFO", "COMINFO" },
	{ 0x0050C2, 0x195000, 0x195FFF, "Momentum Data Systems", "MOMENTUM-DATA" },
	{ 0x0050C2, 0x196000, 0x196FFF, "Netsynt Spa", "NETSYNT-SPA" },
	{ 0x0050C2, 0x197000, 0x197FFF, "EPM Tecnologia e Equipamentos", "EPM-TECNOLOGIA-E-EQUIPAMENTOS" },
	{ 0x0050C2, 0x198000, 0x198FFF, "PotsTek, Inc", "POTSTEK" },
	{ 0x0050C2, 0x199000, 0x199FFF, "Survalent Technology Corporation", "SURVALENT" },
	{ 0x0050C2, 0x19A000, 0x19AFFF, "AZIO TECHNOLOGY CO.", "AZIO" },
	{ 0x0050C2, 0x19B000, 0x19BFFF, "Wilcoxon Research, Inc.", "WILCOXON-RESEARCH" },
	{ 0x0050C2, 0x19C000, 0x19CFFF, "Artec Design", "ARTEC-DESIGN" },
	{ 0x0050C2, 0x19D000, 0x19DFFF, "ELECTREX S.R.L", "ELECTREX-S-R-L" },
	{ 0x0050C2, 0x19E000, 0x19EFFF, "Paltronics, Inc.", "PALTRONICS" },
	{ 0x0050C2, 0x19F000, 0x19FFFF, "Fleetwood Electronics Ltd", "FLEETWOOD-ELECTRONICS" },
	{ 0x0050C2, 0x1A0000, 0x1A0FFF, "SCA Data Systems", "SCA-DATA" },
	{ 0x0050C2, 0x1A1000, 0x1A1FFF, "Portalplayer, Inc", "PORTALPLAYER" },
	{ 0x0050C2, 0x1A2000, 0x1A2FFF, "ABB Switzerland Inc", "ABB-SWITZERLAND" },
	{ 0x0050C2, 0x1A3000, 0x1A3FFF, "Tidel Engineering, L.P.", "TIDEL-ENGINEERING-L-P" },
	{ 0x0050C2, 0x1A4000, 0x1A4FFF, "Protech Optronics Co. Ltd.", "PROTECH-OPTRONICS" },
	{ 0x0050C2, 0x1A5000, 0x1A5FFF, "NORCO", "NORCO" },
	{ 0x0050C2, 0x1A6000, 0x1A6FFF, "RF Code", "RF-CODE" },
	{ 0x0050C2, 0x1A7000, 0x1A7FFF, "Alpha Beta Technologies, Inc.", "ALPHA-BETA-TECHNOLOGIES" },
	{ 0x0050C2, 0x1A8000, 0x1A8FFF, "ANOVA BROADBAND", "ANOVA-BROADBAND" },
	{ 0x0050C2, 0x1A9000, 0x1A9FFF, "Axotec Technologies GmbH", "AXOTEC-TECHNOLOGIES" },
	{ 0x0050C2, 0x1AA000, 0x1AAFFF, "BitBox Ltd", "BITBOX" },
	{ 0x0050C2, 0x1AB000, 0x1ABFFF, "Streaming Networks", "STREAMING-NETWORKS" },
	{ 0x0050C2, 0x1AC000, 0x1ACFFF, "Beckmann+Egle GmbH", "BECKMANN+EGLE" },
	{ 0x0050C2, 0x1AD000, 0x1ADFFF, "Remia s.r.o.", "REMIA-S-R-O" },
	{ 0x0050C2, 0x1AE000, 0x1AEFFF, "Home Director, Inc", "HOME-DIRECTOR" },
	{ 0x0050C2, 0x1AF000, 0x1AFFFF, "PESA Switching Systems, Inc.", "PESA-SWITCHING" },
	{ 0x0050C2, 0x1B0000, 0x1B0FFF, "BLANKOM Antennentechnik GmbH", "BLANKOM-ANTENNENTECHNIK" },
	{ 0x0050C2, 0x1B1000, 0x1B1FFF, "Axes Technologies", "AXES-TECHNOLOGIES" },
	{ 0x0050C2, 0x1B2000, 0x1B2FFF, "SIGOS Systemintegration GmbH", "SIGOS-SYSTEMINTEGRATION" },
	{ 0x0050C2, 0x1B3000, 0x1B3FFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x0050C2, 0x1B4000, 0x1B4FFF, "DSP Group Inc.", "DSP-GROUP" },
	{ 0x0050C2, 0x1B5000, 0x1B5FFF, "Thrane & Thrane A/S", "THRANE-THRANE-A/S" },
	{ 0x0050C2, 0x1B6000, 0x1B6FFF, "DTS, Inc.", "DTS" },
	{ 0x0050C2, 0x1B7000, 0x1B7FFF, "MosChip USA", "MOSCHIP-USA" },
	{ 0x0050C2, 0x1B8000, 0x1B8FFF, "Electronic Systems Development", "DEVELOPMENT" },
	{ 0x0050C2, 0x1B9000, 0x1B9FFF, "EmCom Technology Inc.", "EMCOM" },
	{ 0x0050C2, 0x1BA000, 0x1BAFFF, "INTERZEAG AG", "INTERZEAG" },
	{ 0x0050C2, 0x1BB000, 0x1BBFFF, "Email Metering", "EMAIL-METERING" },
	{ 0x0050C2, 0x1BC000, 0x1BCFFF, "DINEC International", "DINEC" },
	{ 0x0050C2, 0x1BD000, 0x1BDFFF, "AIOI Systems Co., Ltd.", "AIOI" },
	{ 0x0050C2, 0x1BE000, 0x1BEFFF, "Sedia Electronics", "SEDIA-ELECTRONICS" },
	{ 0x0050C2, 0x1BF000, 0x1BFFFF, "International Test & Engineering Services Co.,Ltd.", "TEST-ENGINEERING-SERVICES" },
	{ 0x0050C2, 0x1C0000, 0x1C0FFF, "WillMonius Inc.", "WILLMONIUS" },
	{ 0x0050C2, 0x1C1000, 0x1C1FFF, "InfinitiNetworks Inc.", "INFINITINETWORKS" },
	{ 0x0050C2, 0x1C2000, 0x1C2FFF, "Weltronics Corp.", "WELTRONICS" },
	{ 0x0050C2, 0x1C3000, 0x1C3FFF, "TT electronic manufacturing services ltd", "TT-MANUFACTURING-SERVICES" },
	{ 0x0050C2, 0x1C4000, 0x1C4FFF, "Palm Solutions Group", "PALM-SOLUTIONS-GROUP" },
	{ 0x0050C2, 0x1C5000, 0x1C5FFF, "Flander Oy", "FLANDER-OY" },
	{ 0x0050C2, 0x1C6000, 0x1C6FFF, "Remco Italia Spa", "REMCO-ITALIA-SPA" },
	{ 0x0050C2, 0x1C7000, 0x1C7FFF, "TWIN DEVELOPMENT S.A.", "TWIN-DEVELOPMENT" },
	{ 0x0050C2, 0x1C8000, 0x1C8FFF, "Euphony technology CO., LTD.", "EUPHONY" },
	{ 0x0050C2, 0x1C9000, 0x1C9FFF, "modas GmbH", "MODAS" },
	{ 0x0050C2, 0x1CA000, 0x1CAFFF, "EVER Sp. z o.o.", "EVER-SP-Z-O-O" },
	{ 0x0050C2, 0x1CB000, 0x1CBFFF, "quantumBEAM Limited", "QUANTUMBEAM" },
	{ 0x0050C2, 0x1CC000, 0x1CCFFF, "WaveIP ltd.", "WAVEIP" },
	{ 0x0050C2, 0x1CD000, 0x1CDFFF, "INCAA Informatica Italia srl", "INCAA-INFORMATICA-ITALIA-SRL" },
	{ 0x0050C2, 0x1CE000, 0x1CEFFF, "Datatek Applications, Inc.", "DATATEK-APPLICATIONS" },
	{ 0x0050C2, 0x1CF000, 0x1CFFFF, "LIFETIME MEMORY PRODUCTS, INC.", "LIFETIME-MEMORY-PRODUCTS" },
	{ 0x0050C2, 0x1D0000, 0x1D0FFF, "Yazaki North America, Inc.", "YAZAKI-NORTH-AMERICA" },
	{ 0x0050C2, 0x1D1000, 0x1D1FFF, "Benchmark Electronics", "BENCHMARK-ELECTRONICS" },
	{ 0x0050C2, 0x1D2000, 0x1D2FFF, "Shenyang Internet Technology Inc", "SHENYANG-INTERNET" },
	{ 0x0050C2, 0x1D3000, 0x1D3FFF, "Synopsys", "SYNOPSYS" },
	{ 0x0050C2, 0x1D4000, 0x1D4FFF, "Phase IV Engineering Inc.", "PHASE-IV-ENGINEERING" },
	{ 0x0050C2, 0x1D5000, 0x1D5FFF, "Redpoint Controls", "REDPOINT-CONTROLS" },
	{ 0x0050C2, 0x1D6000, 0x1D6FFF, "shanghai trend intelligent systems CO.,LTD", "SHANGHAI-TREND-INTELLIGENT" },
	{ 0x0050C2, 0x1D7000, 0x1D7FFF, "Pleora Technologies Inc.", "PLEORA-TECHNOLOGIES" },
	{ 0x0050C2, 0x1D8000, 0x1D8FFF, "Guardian Controls International", "GUARDIAN-CONTROLS" },
	{ 0x0050C2, 0x1D9000, 0x1D9FFF, "EDC", "EDC" },
	{ 0x0050C2, 0x1DA000, 0x1DAFFF, "GFI Chrono Time", "GFI-CHRONO-TIME" },
	{ 0x0050C2, 0x1DB000, 0x1DBFFF, "Applied Systems Engineering, Inc.", "APPLIED-ENGINEERING" },
	{ 0x0050C2, 0x1DC000, 0x1DCFFF, "Imarda New Zealand Limited", "IMARDA-NEW-ZEALAND" },
	{ 0x0050C2, 0x1DD000, 0x1DDFFF, "Peiker acustic GmbH & Co. KG", "PEIKER-ACUSTIC" },
	{ 0x0050C2, 0x1DE000, 0x1DEFFF, "ReliOn Inc.", "RELION" },
	{ 0x0050C2, 0x1DF000, 0x1DFFFF, "Lulea University of Technology", "LULEA-UNIVERSITY-OF" },
	{ 0x0050C2, 0x1E0000, 0x1E0FFF, "Cognex Corporation", "COGNEX" },
	{ 0x0050C2, 0x1E1000, 0x1E1FFF, "Automaatiotekniikka Seppo Saari Oy", "AUTOMAATIOTEKNIIKKA-SEPPO-SAARI-OY" },
	{ 0x0050C2, 0x1E2000, 0x1E2FFF, "DIGITRONIC Automationsanlagen GmbH", "DIGITRONIC-AUTOMATIONSANLAGEN" },
	{ 0x0050C2, 0x1E3000, 0x1E3FFF, "Bluesocket, Inc.", "BLUESOCKET" },
	{ 0x0050C2, 0x1E4000, 0x1E4FFF, "Soronti, Inc.", "SORONTI" },
	{ 0x0050C2, 0x1E5000, 0x1E5FFF, "DORLET S.A.", "DORLET" },
	{ 0x0050C2, 0x1E6000, 0x1E6FFF, "United Tri-Tech Corporation", "UNITED-TRI-TECH" },
	{ 0x0050C2, 0x1E7000, 0x1E7FFF, "Smith Meter, Inc.", "SMITH-METER" },
	{ 0x0050C2, 0x1E8000, 0x1E8FFF, "Metrotech", "METROTECH" },
	{ 0x0050C2, 0x1E9000, 0x1E9FFF, "Ranch Networks", "RANCH-NETWORKS" },
	{ 0x0050C2, 0x1EA000, 0x1EAFFF, "DAVE S.r.L.", "DAVE-S-R-L" },
	{ 0x0050C2, 0x1EB000, 0x1EBFFF, "Data Respons A/S", "DATA-RESPONS-A/S" },
	{ 0x0050C2, 0x1EC000, 0x1ECFFF, "COSMO co.,ltd.", "COSMO" },
	{ 0x0050C2, 0x1ED000, 0x1EDFFF, "EMKA-electronic AG", "EMKA" },
	{ 0x0050C2, 0x1EE000, 0x1EEFFF, "Perto Periféricos de Automação S.A.", "PERTO-PERIFéRICOS-DE-AUTOMAçãO" },
	{ 0x0050C2, 0x1EF000, 0x1EFFFF, "M2 Technology Pty Ltd", "M2-PTY" },
	{ 0x0050C2, 0x1F0000, 0x1F0FFF, "EXI Wireless Systems Inc.", "EXI-WIRELESS" },
	{ 0x0050C2, 0x1F1000, 0x1F1FFF, "SKY Computers, Inc.", "SKY-COMPUTERS" },
	{ 0x0050C2, 0x1F2000, 0x1F2FFF, "Tattile srl", "TATTILE-SRL" },
	{ 0x0050C2, 0x1F3000, 0x1F3FFF, "Radionor Communications AS", "RADIONOR-COMMUNICATION-AS" },
	{ 0x0050C2, 0x1F4000, 0x1F4FFF, "Covia, Inc", "COVIA" },
	{ 0x0050C2, 0x1F5000, 0x1F5FFF, "Abest Communication Corp.", "ABEST-COMMUNICATION" },
	{ 0x0050C2, 0x1F6000, 0x1F6FFF, "BAE SYSTEMS Controls", "BAE-CONTROLS" },
	{ 0x0050C2, 0x1F7000, 0x1F7FFF, "ARC'Créations", "ARC-CRéATIONS" },
	{ 0x0050C2, 0x1F8000, 0x1F8FFF, "ULTRACKER TECHNOLOGY", "ULTRACKER" },
	{ 0x0050C2, 0x1F9000, 0x1F9FFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0x1FA000, 0x1FAFFF, "SP Controls, Inc", "SP-CONTROLS" },
	{ 0x0050C2, 0x1FB000, 0x1FBFFF, "Willowglen Systems Inc.", "WILLOWGLEN" },
	{ 0x0050C2, 0x1FC000, 0x1FCFFF, "EDD Srl", "EDD-SRL" },
	{ 0x0050C2, 0x1FD000, 0x1FDFFF, "SouthWing S.L.", "SOUTHWING-S-L" },
	{ 0x0050C2, 0x1FE000, 0x1FEFFF, "Safetran Traffic Systems Inc.", "SAFETRAN-TRAFFIC" },
	{ 0x0050C2, 0x1FF000, 0x1FFFFF, "Product Design Dept., Sohwa Corporation", "PRODUCT-DESIGN-DEPT-SOHWA" },
	{ 0x0050C2, 0x200000, 0x200FFF, "Whittier Mailing Products, Inc.", "WHITTIER-MAILING-PRODUCTS" },
	{ 0x0050C2, 0x201000, 0x201FFF, "OlympusNDT", "OLYMPUSNDT" },
	{ 0x0050C2, 0x202000, 0x202FFF, "Audio Riders Oy", "AUDIO-RIDERS-OY" },
	{ 0x0050C2, 0x203000, 0x203FFF, "Vocality International Ltd", "VOCALITY" },
	{ 0x0050C2, 0x204000, 0x204FFF, "Algodue Elettronica srl", "ALGODUE-ELETTRONICA-SRL" },
	{ 0x0050C2, 0x205000, 0x205FFF, "SystIng", "SYSTING" },
	{ 0x0050C2, 0x206000, 0x206FFF, "Windmill Innovations", "WINDMILL-INNOVATIONS" },
	{ 0x0050C2, 0x207000, 0x207FFF, "Solectron Ind.Com.Servs.Exportadora do Brasil Ltda.", "SOLECTRON-IND-COM-SERVS-EXPORTADORA-DO-BRASIL-LTDA" },
	{ 0x0050C2, 0x208000, 0x208FFF, "nNovia, Inc.", "NNOVIA" },
	{ 0x0050C2, 0x209000, 0x209FFF, "LK Ltd", "LK" },
	{ 0x0050C2, 0x20A000, 0x20AFFF, "Ferrari electronic AG", "FERRARI" },
	{ 0x0050C2, 0x20B000, 0x20BFFF, "Rafael", "RAFAEL" },
	{ 0x0050C2, 0x20C000, 0x20CFFF, "Communication and Telemechanical Systems Company Limited", "COMMUNICATION-AND-TELEMECHANICAL-COMPANY" },
	{ 0x0050C2, 0x20D000, 0x20DFFF, "Varisys Ltd", "VARISYS" },
	{ 0x0050C2, 0x20E000, 0x20EFFF, "PYRAMID Computer Systeme GmbH", "PYRAMID-SYSTEME" },
	{ 0x0050C2, 0x20F000, 0x20FFFF, "OMICRON electronics GmbH", "OMICRON-ELECTRONICS" },
	{ 0x0050C2, 0x210000, 0x210FFF, "Innovics Wireless Inc", "INNOVICS-WIRELESS" },
	{ 0x0050C2, 0x211000, 0x211FFF, "Hochschule für Technik, Wirtschaft und Kultur Leipzig (FH)", "HOCHSCHULE-FUER-TECHNIK-WIRTSCHAFT-UND-KULTUR-LEIPZIG-FH" },
	{ 0x0050C2, 0x212000, 0x212FFF, "4Links Limited", "4LINKS" },
	{ 0x0050C2, 0x213000, 0x213FFF, "SysAware S.A.R.L.", "SYSAWARE-R-L" },
	{ 0x0050C2, 0x214000, 0x214FFF, "Oshimi System Design Inc.", "OSHIMI-SYSTEM-DESIGN" },
	{ 0x0050C2, 0x215000, 0x215FFF, "VoiceCom AG", "VOICECOM" },
	{ 0x0050C2, 0x216000, 0x216FFF, "Level Control Systems", "LEVEL-CONTROL" },
	{ 0x0050C2, 0x217000, 0x217FFF, "Linn Products Ltd", "LINN-PRODUCTS" },
	{ 0x0050C2, 0x218000, 0x218FFF, "Nansen S. A. - Instrumentos de Precisão", "NANSEN-S-A---INSTRUMENTOS-DE-PRECISãO" },
	{ 0x0050C2, 0x219000, 0x219FFF, "Aeroflex GmbH", "AEROFLEX" },
	{ 0x0050C2, 0x21A000, 0x21AFFF, "MST SYSTEMS LIMITED", "MST" },
	{ 0x0050C2, 0x21B000, 0x21BFFF, "General Dynamics Decision Systems", "GENERAL-DYNAMICS-DECISION" },
	{ 0x0050C2, 0x21C000, 0x21CFFF, "Fracarro Radioindustrie SPA", "FRACARRO-RADIOINDUSTRIE-SPA" },
	{ 0x0050C2, 0x21D000, 0x21DFFF, "ESG Elektroniksystem u. Logistik GmbH", "ESG-ELEKTRONIKSYSTEM-U-LOGISTIK" },
	{ 0x0050C2, 0x21E000, 0x21EFFF, "Applied Technologies Associates", "APPLIED-TECHNOLOGIES-ASSOCIATES" },
	{ 0x0050C2, 0x21F000, 0x21FFFF, "Monitor Business Machines Ltd", "MONITOR-BUSINESS-MACHINES" },
	{ 0x0050C2, 0x220000, 0x220FFF, "Serveron Corporation", "SERVERON" },
	{ 0x0050C2, 0x221000, 0x221FFF, "Getinge IT Solutions ApS", "GETINGE-IT-SOLUTIONS-APS" },
	{ 0x0050C2, 0x222000, 0x222FFF, "imo-elektronik GmbH", "IMO-ELEKTRONIK" },
	{ 0x0050C2, 0x223000, 0x223FFF, "visicontrol GmbH", "VISICONTROL" },
	{ 0x0050C2, 0x224000, 0x224FFF, "PANNOCOM Ltd.", "PANNOCOM" },
	{ 0x0050C2, 0x225000, 0x225FFF, "Pigeon Point Systems LLC", "PIGEON-POINT-LLC" },
	{ 0x0050C2, 0x226000, 0x226FFF, "Ross Video Limited", "ROSS-VIDEO" },
	{ 0x0050C2, 0x227000, 0x227FFF, "Intelligent Photonics Control", "INTELLIGENT-PHOTONICS-CONTROL" },
	{ 0x0050C2, 0x228000, 0x228FFF, "Intelligent Media Technologies, Inc.", "INTELLIGENT-MEDIA-TECHNOLOGIES" },
	{ 0x0050C2, 0x229000, 0x229FFF, "eko systems inc.", "EKO" },
	{ 0x0050C2, 0x22A000, 0x22AFFF, "Crescendo Networks", "CRESCENDO-NETWORKS" },
	{ 0x0050C2, 0x22B000, 0x22BFFF, "Riegl Laser Measurement Systems GmbH", "RIEGL-LASER-MEASUREMENT" },
	{ 0x0050C2, 0x22C000, 0x22CFFF, "Intrinsity", "INTRINSITY" },
	{ 0x0050C2, 0x22D000, 0x22DFFF, "asetek Inc.", "ASETEK" },
	{ 0x0050C2, 0x22E000, 0x22EFFF, "LORD INGENIERIE", "LORD-INGENIERIE" },
	{ 0x0050C2, 0x22F000, 0x22FFFF, "HTEC Limited", "HTEC" },
	{ 0x0050C2, 0x230000, 0x230FFF, "AutoTOOLS group Co. Ltd.", "AUTOTOOLS-GROUP" },
	{ 0x0050C2, 0x231000, 0x231FFF, "Legra Systems, Inc.", "LEGRA" },
	{ 0x0050C2, 0x232000, 0x232FFF, "SIMET", "SIMET" },
	{ 0x0050C2, 0x233000, 0x233FFF, "EdenTree Technologies, Inc.", "EDENTREE-TECHNOLOGIES" },
	{ 0x0050C2, 0x234000, 0x234FFF, "Silverback Systems", "SILVERBACK" },
	{ 0x0050C2, 0x235000, 0x235FFF, "POLIMAR ELEKTRONIK LTD.", "POLIMAR-ELEKTRONIK" },
	{ 0x0050C2, 0x236000, 0x236FFF, "JLCooper Electronics", "JLCOOPER-ELECTRONICS" },
	{ 0x0050C2, 0x237000, 0x237FFF, "Tandata Systems Ltd", "TANDATA" },
	{ 0x0050C2, 0x238000, 0x238FFF, "Schwer+Kopka GmbH", "SCHWER+KOPKA" },
	{ 0x0050C2, 0x239000, 0x239FFF, "Stins Coman", "STINS-COMAN" },
	{ 0x0050C2, 0x23A000, 0x23AFFF, "Chantry Networks", "CHANTRY-NETWORKS" },
	{ 0x0050C2, 0x23B000, 0x23BFFF, "Envara", "ENVARA" },
	{ 0x0050C2, 0x23C000, 0x23CFFF, "Wheatstone Corporation", "WHEATSTONE" },
	{ 0x0050C2, 0x23D000, 0x23DFFF, "Gauging Systems Inc", "GAUGING" },
	{ 0x0050C2, 0x23E000, 0x23EFFF, "Kallastra Inc.", "KALLASTRA" },
	{ 0x0050C2, 0x23F000, 0x23FFFF, "Halliburton - NUMAR", "HALLIBURTON---NUMAR" },
	{ 0x0050C2, 0x240000, 0x240FFF, "Geoquip Ltd", "GEOQUIP" },
	{ 0x0050C2, 0x241000, 0x241FFF, "Contronics Automacao Ltda", "CONTRONICS-AUTOMACAO-LTDA" },
	{ 0x0050C2, 0x242000, 0x242FFF, "MDS SCIEX", "MDS-SCIEX" },
	{ 0x0050C2, 0x243000, 0x243FFF, "RGB Spectrum", "RGB-SPECTRUM" },
	{ 0x0050C2, 0x244000, 0x244FFF, "intec GmbH", "INTEC" },
	{ 0x0050C2, 0x245000, 0x245FFF, "Hauppauge Computer Works, Inc.", "HAUPPAUGE-WORKS" },
	{ 0x0050C2, 0x246000, 0x246FFF, "Hardmeier", "HARDMEIER" },
	{ 0x0050C2, 0x247000, 0x247FFF, "Gradual Tecnologia Ltda.", "GRADUAL-TECNOLOGIA-LTDA" },
	{ 0x0050C2, 0x248000, 0x248FFF, "Dixtal Biomedica Ind. Com. Ltda.", "DIXTAL-BIOMEDICA-IND-COM-LTDA" },
	{ 0x0050C2, 0x249000, 0x249FFF, "Bender GmbH & Co. KG", "BENDER" },
	{ 0x0050C2, 0x24A000, 0x24AFFF, "CDS Rail", "CDS-RAIL" },
	{ 0x0050C2, 0x24B000, 0x24BFFF, "Azimuth Systems, Inc.", "AZIMUTH" },
	{ 0x0050C2, 0x24C000, 0x24CFFF, "Supertel", "SUPERTEL" },
	{ 0x0050C2, 0x24D000, 0x24DFFF, "METTLER-TOLEDO HI-SPEED", "METTLER-TOLEDO-HI-SPEED" },
	{ 0x0050C2, 0x24E000, 0x24EFFF, "Scharff Weisberg Systems Integration Inc", "SCHARFF-WEISBERG-INTEGRATION" },
	{ 0x0050C2, 0x24F000, 0x24FFFF, "Macronet s.r.l.", "MACRONET-S-R-L" },
	{ 0x0050C2, 0x250000, 0x250FFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0x251000, 0x251FFF, "DGT Sp. z o.o.", "DGT-SP-Z-O-O" },
	{ 0x0050C2, 0x252000, 0x252FFF, "ads-tec GmbH", "ADS-TEC" },
	{ 0x0050C2, 0x253000, 0x253FFF, "DSM-Messtechnik GmbH", "DSM-MESSTECHNIK" },
	{ 0x0050C2, 0x254000, 0x254FFF, "Thales Communications Ltd", "THALES-COMMUNICATION" },
	{ 0x0050C2, 0x255000, 0x255FFF, "STMicroelectronics (R&D) Ltd", "STMICROELECTRONICS-R-D" },
	{ 0x0050C2, 0x256000, 0x256FFF, "Information Technology Corp.", "INFORMATION" },
	{ 0x0050C2, 0x257000, 0x257FFF, "Digicast Networks", "DIGICAST-NETWORKS" },
	{ 0x0050C2, 0x258000, 0x258FFF, "Spacesaver Corporation", "SPACESAVER" },
	{ 0x0050C2, 0x259000, 0x259FFF, "Omicron Ceti AB", "OMICRON-CETI-AB" },
	{ 0x0050C2, 0x25A000, 0x25AFFF, "Zendex Corporation", "ZENDEX" },
	{ 0x0050C2, 0x25B000, 0x25BFFF, "Winford Engineering", "WINFORD-ENGINEERING" },
	{ 0x0050C2, 0x25C000, 0x25CFFF, "Softhill Technologies Ltd.", "SOFTHILL-TECHNOLOGIES" },
	{ 0x0050C2, 0x25D000, 0x25DFFF, "RDTECH", "RDTECH" },
	{ 0x0050C2, 0x25E000, 0x25EFFF, "MITE Hradec Kralove, s.r.o.", "MITE-HRADEC-KRALOVE-S-R-O" },
	{ 0x0050C2, 0x25F000, 0x25FFFF, "Handtmann Maschinenfabrik GmbH&Co.KG", "HANDTMANN-MASCHINENFABRIK" },
	{ 0x0050C2, 0x260000, 0x260FFF, "BIOTAGE", "BIOTAGE" },
	{ 0x0050C2, 0x261000, 0x261FFF, "Tattile Srl", "TATTILE-SRL" },
	{ 0x0050C2, 0x262000, 0x262FFF, "Shanghai Gaozhi Science&Technology Development Ltd.", "SHANGHAI-GAOZHI-SCIENCE-DEVELOPMENT" },
	{ 0x0050C2, 0x263000, 0x263FFF, "Vansco Electronics Oy", "VANSCO-ELECTRONICS-OY" },
	{ 0x0050C2, 0x264000, 0x264FFF, "Confidence Direct Ltd", "CONFIDENCE-DIRECT" },
	{ 0x0050C2, 0x265000, 0x265FFF, "BELIK S.P.R.L.", "BELIK-S-P-R-L" },
	{ 0x0050C2, 0x266000, 0x266FFF, "ATOM GIKEN Co.,Ltd.", "ATOM-GIKEN" },
	{ 0x0050C2, 0x267000, 0x267FFF, "Allen Martin Conservation Ltd", "ALLEN-MARTIN-CONSERVATION" },
	{ 0x0050C2, 0x268000, 0x268FFF, "Parabit Systems", "PARABIT" },
	{ 0x0050C2, 0x269000, 0x269FFF, "Technisyst Pty Ltd", "TECHNISYST-PTY" },
	{ 0x0050C2, 0x26A000, 0x26AFFF, "FG SYNERYS", "FG-SYNERYS" },
	{ 0x0050C2, 0x26B000, 0x26BFFF, "Continental Gateway Limited", "CONTINENTAL-GATEWAY" },
	{ 0x0050C2, 0x26C000, 0x26CFFF, "Crystal Vision Ltd", "CRYSTAL-VISION" },
	{ 0x0050C2, 0x26D000, 0x26DFFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x0050C2, 0x26E000, 0x26EFFF, "ZP Engineering srl", "ZP-ENGINEERING-SRL" },
	{ 0x0050C2, 0x26F000, 0x26FFFF, "Digital Recorders Inc", "DIGITAL-RECORDERS" },
	{ 0x0050C2, 0x270000, 0x270FFF, "S4 Technology Pty Ltd", "S4-PTY" },
	{ 0x0050C2, 0x271000, 0x271FFF, "VLSIP TECHNOLOGIES INC.", "VLSIP-TECHNOLOGIES" },
	{ 0x0050C2, 0x272000, 0x272FFF, "Verex Technology", "VEREX" },
	{ 0x0050C2, 0x273000, 0x273FFF, "Servicios Condumex, S. A. de C. V.", "SERVICIOS-CONDUMEX-S-A-DE-C-V" },
	{ 0x0050C2, 0x274000, 0x274FFF, "Fundación TECNALIA Research & Innovation", "FUNDACIóN-TECNALIA-RESEARCH-INNOVATION" },
	{ 0x0050C2, 0x275000, 0x275FFF, "Extreme Engineering Solutions", "EXTREME-ENGINEERING-SOLUTIONS" },
	{ 0x0050C2, 0x276000, 0x276FFF, "Tieline Research Pty Ltd", "TIELINE-RESEARCH-PTY" },
	{ 0x0050C2, 0x277000, 0x277FFF, "T/R Systems, Inc.", "T/R" },
	{ 0x0050C2, 0x278000, 0x278FFF, "Replicom Ltd.", "REPLICOM" },
	{ 0x0050C2, 0x279000, 0x279FFF, "PATLITE Corporation", "PATLITE" },
	{ 0x0050C2, 0x27A000, 0x27AFFF, "Maestro Pty Ltd", "MAESTRO-PTY" },
	{ 0x0050C2, 0x27B000, 0x27BFFF, "LinkSecurity A/S", "LINKSECURITY-A/S" },
	{ 0x0050C2, 0x27C000, 0x27CFFF, "Danlaw Inc", "DANLAW" },
	{ 0x0050C2, 0x27D000, 0x27DFFF, "ALLIED TELESIS K.K.", "ALLIED-TELESIS-K-K" },
	{ 0x0050C2, 0x27E000, 0x27EFFF, "AnaLogic Computers Ltd.", "ANALOGIC-COMPUTERS" },
	{ 0x0050C2, 0x27F000, 0x27FFFF, "Air Broadband Communications, Inc.", "AIR-BROADBAND-COMMUNICATION" },
	{ 0x0050C2, 0x280000, 0x280FFF, "AGECODAGIS SARL", "AGECODAGIS-SARL" },
	{ 0x0050C2, 0x281000, 0x281FFF, "CabTronix GmbH", "CABTRONIX" },
	{ 0x0050C2, 0x282000, 0x282FFF, "Telvent", "TELVENT" },
	{ 0x0050C2, 0x283000, 0x283FFF, "ANSITEX CORP.", "ANSITEX" },
	{ 0x0050C2, 0x284000, 0x284FFF, "Micronet Ltd.", "MICRONET" },
	{ 0x0050C2, 0x285000, 0x285FFF, "Littwin GmbH & Co KG", "LITTWIN" },
	{ 0x0050C2, 0x286000, 0x286FFF, "ATEME", "ATEME" },
	{ 0x0050C2, 0x287000, 0x287FFF, "TECNEW Electronics Engineering Cr., Ltd.", "TECNEW-ELECTRONICS-ENGINEERING-CR" },
	{ 0x0050C2, 0x288000, 0x288FFF, "RPM Systems Corporation", "RPM" },
	{ 0x0050C2, 0x289000, 0x289FFF, "Rototype S.p.A.", "ROTOTYPE-S-P-A" },
	{ 0x0050C2, 0x28A000, 0x28AFFF, "Real Time Systems", "REAL-TIME" },
	{ 0x0050C2, 0x28B000, 0x28BFFF, "Orion Technologies, Incorporated", "ORION-TECHNOLOGIES-INCORPORATED" },
	{ 0x0050C2, 0x28C000, 0x28CFFF, "Futaba Corporation", "FUTABA" },
	{ 0x0050C2, 0x28D000, 0x28DFFF, "AXODE SA", "AXODE" },
	{ 0x0050C2, 0x28E000, 0x28EFFF, "TATTILE SRL", "TATTILE-SRL" },
	{ 0x0050C2, 0x28F000, 0x28FFFF, "Spellman High Voltage Electronics Corp", "SPELLMAN-HIGH-VOLTAGE-ELECTRONICS" },
	{ 0x0050C2, 0x290000, 0x290FFF, "EBNEURO SPA", "EBNEURO-SPA" },
	{ 0x0050C2, 0x291000, 0x291FFF, "CHAUVIN ARNOUX", "CHAUVIN-ARNOUX" },
	{ 0x0050C2, 0x292000, 0x292FFF, "AMIRIX Systems", "AMIRIX" },
	{ 0x0050C2, 0x293000, 0x293FFF, "IP Unity", "IP-UNITY" },
	{ 0x0050C2, 0x294000, 0x294FFF, "EPSa GmbH", "EPSA" },
	{ 0x0050C2, 0x295000, 0x295FFF, "LOGOSOL, INC.", "LOGOSOL" },
	{ 0x0050C2, 0x296000, 0x296FFF, "OpVista", "OPVISTA" },
	{ 0x0050C2, 0x297000, 0x297FFF, "KINETICS", "KINETICS" },
	{ 0x0050C2, 0x298000, 0x298FFF, "Harvad University", "HARVAD-UNIVERSITY" },
	{ 0x0050C2, 0x299000, 0x299FFF, "CAD-UL GmbH", "CAD-UL" },
	{ 0x0050C2, 0x29A000, 0x29AFFF, "Packet Techniques Inc.", "PACKET-TECHNIQUES" },
	{ 0x0050C2, 0x29B000, 0x29BFFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0x29C000, 0x29CFFF, "2N TELEKOMUNIKACE a.s.", "2N-TELEKOMUNIKACE-A-S" },
	{ 0x0050C2, 0x29D000, 0x29DFFF, "Globe Wireless", "GLOBE-WIRELESS" },
	{ 0x0050C2, 0x29E000, 0x29EFFF, "SELEX Communications Ltd", "SELEX-COMMUNICATION" },
	{ 0x0050C2, 0x29F000, 0x29FFFF, "Baudisch Electronic GmbH", "BAUDISCH" },
	{ 0x0050C2, 0x2A0000, 0x2A0FFF, "Sterling Industry Consult GmbH", "STERLING-INDUSTRY-CONSULT" },
	{ 0x0050C2, 0x2A1000, 0x2A1FFF, "Infinetix Corp", "INFINETIX" },
	{ 0x0050C2, 0x2A2000, 0x2A2FFF, "Epelsa, SL", "EPELSA-SL" },
	{ 0x0050C2, 0x2A3000, 0x2A3FFF, "West-Com Nurse Call Systems, Inc.", "WEST-COM-NURSE-CALL" },
	{ 0x0050C2, 0x2A4000, 0x2A4FFF, "Xipher Embedded Networking", "XIPHER-EMBEDDED-NETWORKING" },
	{ 0x0050C2, 0x2A5000, 0x2A5FFF, "Septier Communication Ltd", "SEPTIER-COMMUNICATION" },
	{ 0x0050C2, 0x2A6000, 0x2A6FFF, "Brannstroms Elektronik AB", "BRANNSTROMS-ELEKTRONIK-AB" },
	{ 0x0050C2, 0x2A7000, 0x2A7FFF, "Micro System Architecturing srl", "MICRO-SYSTEM-ARCHITECTURING-SRL" },
	{ 0x0050C2, 0x2A8000, 0x2A8FFF, "DVTel Israel Ltd.", "DVTEL-ISRAEL" },
	{ 0x0050C2, 0x2A9000, 0x2A9FFF, "Dr. Staiger, Mohilo + Co GmbH", "DR-STAIGER-MOHILO-+" },
	{ 0x0050C2, 0x2AA000, 0x2AAFFF, "DEUTA Werke GmbH", "DEUTA-WERKE" },
	{ 0x0050C2, 0x2AB000, 0x2ABFFF, "AUM Infotech Private Limited", "AUM-INFOTECH-PRIVATE" },
	{ 0x0050C2, 0x2AC000, 0x2ACFFF, "BBI Engineering, Inc.", "BBI-ENGINEERING" },
	{ 0x0050C2, 0x2AD000, 0x2ADFFF, "ABB T&D Spa", "ABB-T-D-SPA" },
	{ 0x0050C2, 0x2AE000, 0x2AEFFF, "Quest Retail Technology Pty Ltd", "QUEST-RETAIL-PTY" },
	{ 0x0050C2, 0x2AF000, 0x2AFFFF, "CSA Computer & Antriebstechnik GmbH", "CSA-ANTRIEBSTECHNIK" },
	{ 0x0050C2, 0x2B0000, 0x2B0FFF, "Telda Electronics", "TELDA-ELECTRONICS" },
	{ 0x0050C2, 0x2B1000, 0x2B1FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x2B2000, 0x2B2FFF, "Smiths Detection", "SMITHS-DETECTION" },
	{ 0x0050C2, 0x2B3000, 0x2B3FFF, "Embedded Systems Design", "EMBEDDED-DESIGN" },
	{ 0x0050C2, 0x2B4000, 0x2B4FFF, "Polatis Ltd", "POLATIS" },
	{ 0x0050C2, 0x2B5000, 0x2B5FFF, "Hobbes Computer Network Accessories", "HOBBES-NETWORK-ACCESSORIES" },
	{ 0x0050C2, 0x2B6000, 0x2B6FFF, "Softier Inc.", "SOFTIER" },
	{ 0x0050C2, 0x2B7000, 0x2B7FFF, "Rafi GmbH & Co. KG", "RAFI" },
	{ 0x0050C2, 0x2B8000, 0x2B8FFF, "Admiral Secure Products, Ltd.", "ADMIRAL-SECURE-PRODUCTS" },
	{ 0x0050C2, 0x2B9000, 0x2B9FFF, "Richmond Sound Design Ltd.", "RICHMOND-SOUND-DESIGN" },
	{ 0x0050C2, 0x2BA000, 0x2BAFFF, "NORCO INDUSTRIAL TECHNOLOGY INC", "NORCO-INDUSTRIAL" },
	{ 0x0050C2, 0x2BB000, 0x2BBFFF, "TA Instruments Ltd", "TA-INSTRUMENTS" },
	{ 0x0050C2, 0x2BC000, 0x2BCFFF, "Uster Technologies AG", "USTER-TECHNOLOGIES" },
	{ 0x0050C2, 0x2BD000, 0x2BDFFF, "StorLink Semi", "STORLINK-SEMI" },
	{ 0x0050C2, 0x2BE000, 0x2BEFFF, "Lipowsky Industrie-Elektronik GmbH", "LIPOWSKY-INDUSTRIE-ELEKTRONIK" },
	{ 0x0050C2, 0x2BF000, 0x2BFFFF, "PERAX", "PERAX" },
	{ 0x0050C2, 0x2C0000, 0x2C0FFF, "Magellan Technology Pty Ltd", "MAGELLAN-PTY" },
	{ 0x0050C2, 0x2C1000, 0x2C1FFF, "Stage Tec Entwicklungsgesellschaft für professionelle Audio", "STAGE-TEC-ENTWICKLUNGSGESELLSCHAFT-FUER-PROFESSIONELLE-AUDIO" },
	{ 0x0050C2, 0x2C2000, 0x2C2FFF, "Smarteye Corporation", "SMARTEYE" },
	{ 0x0050C2, 0x2C3000, 0x2C3FFF, "Digital SP Ltd", "DIGITAL-SP" },
	{ 0x0050C2, 0x2C4000, 0x2C4FFF, "Invensys Energy Systens (NZ) Limited", "INVENSYS-ENERGY-SYSTENS-NZ" },
	{ 0x0050C2, 0x2C5000, 0x2C5FFF, "Elman srl", "ELMAN-SRL" },
	{ 0x0050C2, 0x2C6000, 0x2C6FFF, "Initial Electronic Security Systems", "INITIAL-SECURITY" },
	{ 0x0050C2, 0x2C7000, 0x2C7FFF, "Siliquent Technologies Ltd", "SILIQUENT-TECHNOLOGIES" },
	{ 0x0050C2, 0x2C8000, 0x2C8FFF, "SELCO", "SELCO" },
	{ 0x0050C2, 0x2C9000, 0x2C9FFF, "Roseman Engineering Ltd.", "ROSEMAN-ENGINEERING" },
	{ 0x0050C2, 0x2CA000, 0x2CAFFF, "PUTERCOM CO., LTD", "PUTERCOM" },
	{ 0x0050C2, 0x2CB000, 0x2CBFFF, "FACTS Engineering LLC", "FACTS-ENGINEERING-LLC" },
	{ 0x0050C2, 0x2CC000, 0x2CCFFF, "EMBEDDED TOOLSMITHS", "EMBEDDED-TOOLSMITHS" },
	{ 0x0050C2, 0x2CD000, 0x2CDFFF, "DataWind Research", "DATAWIND-RESEARCH" },
	{ 0x0050C2, 0x2CE000, 0x2CEFFF, "Ross Video Limited", "ROSS-VIDEO" },
	{ 0x0050C2, 0x2CF000, 0x2CFFFF, "Diseño de Sistemas en Silicio S.A.", "DISEñO-DE-SISTEMAS-EN-SILICIO" },
	{ 0x0050C2, 0x2D0000, 0x2D0FFF, "Worth Data, Inc.", "WORTH-DATA" },
	{ 0x0050C2, 0x2D1000, 0x2D1FFF, "Miritek, Inc.", "MIRITEK" },
	{ 0x0050C2, 0x2D2000, 0x2D2FFF, "AIRNET COMMUNICATIONS CORP", "AIRNET-COMMUNICATION" },
	{ 0x0050C2, 0x2D3000, 0x2D3FFF, "Gerber Scientific Products, Inc.", "GERBER-SCIENTIFIC-PRODUCTS" },
	{ 0x0050C2, 0x2D4000, 0x2D4FFF, "Integrated System Solution Corp.", "INTEGRATED-SYSTEM-SOLUTION" },
	{ 0x0050C2, 0x2D5000, 0x2D5FFF, "PIXY AG", "PIXY" },
	{ 0x0050C2, 0x2D6000, 0x2D6FFF, "WIS Technologies", "WIS-TECHNOLOGIES" },
	{ 0x0050C2, 0x2D7000, 0x2D7FFF, "Neo Electronics Ltd", "NEO-ELECTRONICS" },
	{ 0x0050C2, 0x2D8000, 0x2D8FFF, "SYN-TECH SYSTEMS INC", "SYN-TECH" },
	{ 0x0050C2, 0x2D9000, 0x2D9FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x2DA000, 0x2DAFFF, "PYRAMID Computer GmbH", "PYRAMID" },
	{ 0x0050C2, 0x2DB000, 0x2DBFFF, "AutoTOOLS group Co. Ltd.", "AUTOTOOLS-GROUP" },
	{ 0x0050C2, 0x2DC000, 0x2DCFFF, "Wiener, Plein & Baus GmbH", "WIENER-PLEIN-BAUS" },
	{ 0x0050C2, 0x2DD000, 0x2DDFFF, "Westek Technology Ltd", "WESTEK" },
	{ 0x0050C2, 0x2DE000, 0x2DEFFF, "Research Applications", "RESEARCH-APPLICATIONS" },
	{ 0x0050C2, 0x2DF000, 0x2DFFFF, "MICREL-NKE", "MICREL-NKE" },
	{ 0x0050C2, 0x2E0000, 0x2E0FFF, "Baxter Healthcare", "BAXTER-HEALTHCARE" },
	{ 0x0050C2, 0x2E1000, 0x2E1FFF, "Access IS", "ACCESS-IS" },
	{ 0x0050C2, 0x2E2000, 0x2E2FFF, "Ballard Technology, Inc.", "BALLARD" },
	{ 0x0050C2, 0x2E3000, 0x2E3FFF, "MG Industrieelektronik GmbH", "MG-INDUSTRIEELEKTRONIK" },
	{ 0x0050C2, 0x2E4000, 0x2E4FFF, "iamba LTD.", "IAMBA" },
	{ 0x0050C2, 0x2E5000, 0x2E5FFF, "Transtech DSP", "TRANSTECH-DSP" },
	{ 0x0050C2, 0x2E6000, 0x2E6FFF, "DALSA", "DALSA" },
	{ 0x0050C2, 0x2E7000, 0x2E7FFF, "SafeView, Inc.", "SAFEVIEW" },
	{ 0x0050C2, 0x2E8000, 0x2E8FFF, "S.M.V. Systemelektronik GmbH", "S-M-V-SYSTEMELEKTRONIK" },
	{ 0x0050C2, 0x2E9000, 0x2E9FFF, "SRI International", "SRI" },
	{ 0x0050C2, 0x2EA000, 0x2EAFFF, "QUBIsoft S.r.l.", "QUBISOFT-S-R-L" },
	{ 0x0050C2, 0x2EB000, 0x2EBFFF, "Lingg & Janke OHG", "LINGG-JANKE-OHG" },
	{ 0x0050C2, 0x2EC000, 0x2ECFFF, "CHENGDU BOOK DIGITAL CO., LTD", "CHENGDU-BOOK-DIGITAL" },
	{ 0x0050C2, 0x2ED000, 0x2EDFFF, "4RF Communications Ltd", "4RF-COMMUNICATION" },
	{ 0x0050C2, 0x2EE000, 0x2EEFFF, "SHF Communication Technologies AG", "SHF-COMMUNICATION-TECHNOLOGIES" },
	{ 0x0050C2, 0x2EF000, 0x2EFFFF, "Profline B.V.", "PROFLINE" },
	{ 0x0050C2, 0x2F0000, 0x2F0FFF, "LECO Corporation", "LECO" },
	{ 0x0050C2, 0x2F1000, 0x2F1FFF, "Geometrics, Inc.", "GEOMETRICS" },
	{ 0x0050C2, 0x2F2000, 0x2F2FFF, "Eurotek Srl", "EUROTEK-SRL" },
	{ 0x0050C2, 0x2F3000, 0x2F3FFF, "Crossbow Technology, Inc.", "CROSSBOW" },
	{ 0x0050C2, 0x2F4000, 0x2F4FFF, "Efficient Channel Coding", "EFFICIENT-CHANNEL-CODING" },
	{ 0x0050C2, 0x2F5000, 0x2F5FFF, "ADChips", "ADCHIPS" },
	{ 0x0050C2, 0x2F6000, 0x2F6FFF, "Clifford Chance LLP", "CLIFFORD-CHANCE-LLP" },
	{ 0x0050C2, 0x2F7000, 0x2F7FFF, "GILLAM-FEI S.A.", "GILLAM-FEI" },
	{ 0x0050C2, 0x2F8000, 0x2F8FFF, "SavvyCorp.com Ltd", "SAVVYCORP-COM" },
	{ 0x0050C2, 0x2F9000, 0x2F9FFF, "Digilent Inc.", "DIGILENT" },
	{ 0x0050C2, 0x2FA000, 0x2FAFFF, "Tornado Modular Systems, Ltd", "TORNADO-MODULAR" },
	{ 0x0050C2, 0x2FB000, 0x2FBFFF, "Arthur Industries Inc., dba On Hold Media Group", "ARTHUR-INDUSTRIES-DBA-ON-HOLD-MEDIA-GROUP" },
	{ 0x0050C2, 0x2FC000, 0x2FCFFF, "Blackline Systems Corporation", "BLACKLINE" },
	{ 0x0050C2, 0x2FD000, 0x2FDFFF, "American Microsystems LTD", "AMERICAN-MICROSYSTEMS" },
	{ 0x0050C2, 0x2FE000, 0x2FEFFF, "Saab AB", "SAAB-AB" },
	{ 0x0050C2, 0x2FF000, 0x2FFFFF, "Patria Advanced Solutions", "PATRIA-ADVANCED-SOLUTIONS" },
	{ 0x0050C2, 0x300000, 0x300FFF, "Soredex Instrumentarium Oyj", "SOREDEX-INSTRUMENTARIUM-OYJ" },
	{ 0x0050C2, 0x301000, 0x301FFF, "Delphi Display Systems, Inc.", "DELPHI-DISPLAY" },
	{ 0x0050C2, 0x302000, 0x302FFF, "EuroDesign embedded technologies GmbH", "EURODESIGN-EMBEDDED-TECHNOLOGIES" },
	{ 0x0050C2, 0x303000, 0x303FFF, "CI Systems Ltd.", "CI" },
	{ 0x0050C2, 0x304000, 0x304FFF, "COMERSON S.r.l.", "COMERSON-S-R-L" },
	{ 0x0050C2, 0x305000, 0x305FFF, "Symbium Corporation", "SYMBIUM" },
	{ 0x0050C2, 0x306000, 0x306FFF, "Noran Tel Communications Ltd.", "NORAN-TEL-COMMUNICATION" },
	{ 0x0050C2, 0x307000, 0x307FFF, "UNIONDIGITAL.,CO.LTD", "UNIONDIGITAL" },
	{ 0x0050C2, 0x308000, 0x308FFF, "FiveCo", "FIVECO" },
	{ 0x0050C2, 0x309000, 0x309FFF, "Rackmaster Systems, Inc.", "RACKMASTER" },
	{ 0x0050C2, 0x30A000, 0x30AFFF, "Innings Telecom Inc.", "INNINGS-TELECOM" },
	{ 0x0050C2, 0x30B000, 0x30BFFF, "VX Technologies Inc.", "VX-TECHNOLOGIES" },
	{ 0x0050C2, 0x30C000, 0x30CFFF, "TEAMLOG", "TEAMLOG" },
	{ 0x0050C2, 0x30D000, 0x30DFFF, "SETARAM", "SETARAM" },
	{ 0x0050C2, 0x30E000, 0x30EFFF, "Obvius", "OBVIUS" },
	{ 0x0050C2, 0x30F000, 0x30FFFF, "Digicontrole Lda", "DIGICONTROLE-LDA" },
	{ 0x0050C2, 0x310000, 0x310FFF, "CYBERTRON CO., LTD.", "CYBERTRON" },
	{ 0x0050C2, 0x311000, 0x311FFF, "Comodo", "COMODO" },
	{ 0x0050C2, 0x312000, 0x312FFF, "Dese Technologies SL", "DESE-TECHNOLOGIES-SL" },
	{ 0x0050C2, 0x313000, 0x313FFF, "SAIA Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x314000, 0x314FFF, "MicroBee Systems, Inc", "MICROBEE" },
	{ 0x0050C2, 0x315000, 0x315FFF, "ifak system GmbH", "IFAK-SYSTEM" },
	{ 0x0050C2, 0x316000, 0x316FFF, "Dataline AB", "DATALINE-AB" },
	{ 0x0050C2, 0x317000, 0x317FFF, "Cosine Systems, Inc.", "COSINE" },
	{ 0x0050C2, 0x318000, 0x318FFF, "Milmega Ltd", "MILMEGA" },
	{ 0x0050C2, 0x319000, 0x319FFF, "Invatron Systems Corp.", "INVATRON" },
	{ 0x0050C2, 0x31A000, 0x31AFFF, "Zodiak Data Systems", "ZODIAK-DATA" },
	{ 0x0050C2, 0x31B000, 0x31BFFF, "Datacon", "DATACON" },
	{ 0x0050C2, 0x31C000, 0x31CFFF, "Casa Systems Inc.", "CASA" },
	{ 0x0050C2, 0x31D000, 0x31DFFF, "Imarda New Zealand Limited", "IMARDA-NEW-ZEALAND" },
	{ 0x0050C2, 0x31E000, 0x31EFFF, "C3-ilex, LLC", "C3-ILEX-LLC" },
	{ 0x0050C2, 0x31F000, 0x31FFFF, "Geotech Instruments, LLC", "GEOTECH-INSTRUMENTS-LLC" },
	{ 0x0050C2, 0x320000, 0x320FFF, "DTASENSOR S.p.A.", "DTASENSOR-S-P-A" },
	{ 0x0050C2, 0x321000, 0x321FFF, "UXP", "UXP" },
	{ 0x0050C2, 0x322000, 0x322FFF, "BQT Solutions (Australia) Limited", "BQT-SOLUTIONS-AUSTRALIA" },
	{ 0x0050C2, 0x323000, 0x323FFF, "Red Rock Networks", "RED-ROCK-NETWORKS" },
	{ 0x0050C2, 0x324000, 0x324FFF, "ODIXION", "ODIXION" },
	{ 0x0050C2, 0x325000, 0x325FFF, "Federal Aviation Administration", "FEDERAL-AVIATION-ADMINISTRATION" },
	{ 0x0050C2, 0x326000, 0x326FFF, "Navionics S.p.A.", "NAVIONICS-S-P-A" },
	{ 0x0050C2, 0x327000, 0x327FFF, "Dornier GmbH", "DORNIER" },
	{ 0x0050C2, 0x328000, 0x328FFF, "I.C.S. Electronics Limited", "I-C-S-ELECTRONICS" },
	{ 0x0050C2, 0x329000, 0x329FFF, "Imax", "IMAX" },
	{ 0x0050C2, 0x32A000, 0x32AFFF, "PHYTEC Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x32B000, 0x32BFFF, "Digital Multimedia Technologies Spa", "DIGITAL-MULTIMEDIA-TECHNOLOGIES-SPA" },
	{ 0x0050C2, 0x32C000, 0x32CFFF, "Integrated Silicon Solution (Taiwan), Inc.", "INTEGRATED-SILICON-SOLUTION-TAIWAN" },
	{ 0x0050C2, 0x32D000, 0x32DFFF, "Consens Zeiterfassung GMBH", "CONSENS-ZEITERFASSUNG" },
	{ 0x0050C2, 0x32E000, 0x32EFFF, "MANUSA-GEST, S.L.", "MANUSA-GEST-S-L" },
	{ 0x0050C2, 0x32F000, 0x32FFFF, "PULTRONICS", "PULTRONICS" },
	{ 0x0050C2, 0x330000, 0x330FFF, "Sicon S.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0x331000, 0x331FFF, "Broadcast Sports Inc", "BROADCAST-SPORTS" },
	{ 0x0050C2, 0x332000, 0x332FFF, "PUNJAB COMMUNICATIONS LTD", "PUNJAB-COMMUNICATION" },
	{ 0x0050C2, 0x333000, 0x333FFF, "Radix Corporation", "RADIX" },
	{ 0x0050C2, 0x334000, 0x334FFF, "Picture Elements, Inc.", "PICTURE-ELEMENTS" },
	{ 0x0050C2, 0x335000, 0x335FFF, "Nimcat Networks", "NIMCAT-NETWORKS" },
	{ 0x0050C2, 0x336000, 0x336FFF, "Golden River Traffic", "GOLDEN-RIVER-TRAFFIC" },
	{ 0x0050C2, 0x337000, 0x337FFF, "ETI", "ETI" },
	{ 0x0050C2, 0x338000, 0x338FFF, "Ernitec A/S", "ERNITEC-A/S" },
	{ 0x0050C2, 0x339000, 0x339FFF, "CEGELEC SUD EST", "CEGELEC-SUD-EST" },
	{ 0x0050C2, 0x33A000, 0x33AFFF, "United Telecoms Ltd", "UNITED-TELECOMS" },
	{ 0x0050C2, 0x33B000, 0x33BFFF, "MultimediaLED", "MULTIMEDIALED" },
	{ 0x0050C2, 0x33C000, 0x33CFFF, "SkipJam", "SKIPJAM" },
	{ 0x0050C2, 0x33D000, 0x33DFFF, "General Dynamics Decision Systems", "GENERAL-DYNAMICS-DECISION" },
	{ 0x0050C2, 0x33E000, 0x33EFFF, "CA Technology, Inc", "CA" },
	{ 0x0050C2, 0x33F000, 0x33FFFF, "EXYS bvba", "EXYS-BVBA" },
	{ 0x0050C2, 0x340000, 0x340FFF, "Virtu", "VIRTU" },
	{ 0x0050C2, 0x341000, 0x341FFF, "Novx Systems", "NOVX" },
	{ 0x0050C2, 0x342000, 0x342FFF, "St. Michael Strategies", "ST-MICHAEL-STRATEGIES" },
	{ 0x0050C2, 0x343000, 0x343FFF, "ABB Xiamen Switchgear Co. Ltd.", "ABB-XIAMEN-SWITCHGEAR" },
	{ 0x0050C2, 0x344000, 0x344FFF, "ads-tec GmbH", "ADS-TEC" },
	{ 0x0050C2, 0x345000, 0x345FFF, "ACT", "ACT" },
	{ 0x0050C2, 0x346000, 0x346FFF, "biokeysystem", "BIOKEYSYSTEM" },
	{ 0x0050C2, 0x347000, 0x347FFF, "Row Seven Ltd", "ROW-SEVEN" },
	{ 0x0050C2, 0x348000, 0x348FFF, "KoolSpan, Inc.", "KOOLSPAN" },
	{ 0x0050C2, 0x349000, 0x349FFF, "SSI Schaefer Peem", "SSI-SCHAEFER-PEEM" },
	{ 0x0050C2, 0x34A000, 0x34AFFF, "NIE Corporation", "NIE" },
	{ 0x0050C2, 0x34B000, 0x34BFFF, "Ecutel Systems, Inc.", "ECUTEL" },
	{ 0x0050C2, 0x34C000, 0x34CFFF, "Chuo Electric Works Co., LTD.", "CHUO-ELECTRIC-WORKS" },
	{ 0x0050C2, 0x34D000, 0x34DFFF, "BMK professional electronics GmbH", "BMK-PROFESSIONAL-ELECTRONICS" },
	{ 0x0050C2, 0x34E000, 0x34EFFF, "ABB Power Technologies S.p.A. Unità  Operativa SACE (PTMV)", "ABB-POWER-TECHNOLOGIES-S-P-A-UNITà -OPERATIVA-SACE-PTMV" },
	{ 0x0050C2, 0x34F000, 0x34FFFF, "North Pole Engineering, Inc.", "NORTH-POLE-ENGINEERING" },
	{ 0x0050C2, 0x350000, 0x350FFF, "Kinesys Projects Limited", "KINESYS-PROJECTS" },
	{ 0x0050C2, 0x351000, 0x351FFF, "Finesystem Co., Ltd", "FINESYSTEM" },
	{ 0x0050C2, 0x352000, 0x352FFF, "LUCEO", "LUCEO" },
	{ 0x0050C2, 0x353000, 0x353FFF, "Crossing Informationssysteme GmbH", "CROSSING-INFORMATIONSSYSTEME" },
	{ 0x0050C2, 0x354000, 0x354FFF, "Advanced IP Communications", "ADVANCED-IP-COMMUNICATION" },
	{ 0x0050C2, 0x355000, 0x355FFF, "IHM", "IHM" },
	{ 0x0050C2, 0x356000, 0x356FFF, "Baytech Cinema", "BAYTECH-CINEMA" },
	{ 0x0050C2, 0x357000, 0x357FFF, "Athena Semiconductor", "ATHENA-SEMICONDUCTOR" },
	{ 0x0050C2, 0x358000, 0x358FFF, "ALCEA", "ALCEA" },
	{ 0x0050C2, 0x359000, 0x359FFF, "Kramer Electronics Ltd.", "KRAMER-ELECTRONICS" },
	{ 0x0050C2, 0x35A000, 0x35AFFF, "Advanced Si-Net Co., LTD.", "ADVANCED-SI-NET" },
	{ 0x0050C2, 0x35B000, 0x35BFFF, "VLSIP TECHNOLOGIES, INC", "VLSIP-TECHNOLOGIES" },
	{ 0x0050C2, 0x35C000, 0x35CFFF, "Ratotec GmbH", "RATOTEC" },
	{ 0x0050C2, 0x35D000, 0x35DFFF, "NetTest A/S", "NETTEST-A/S" },
	{ 0x0050C2, 0x35E000, 0x35EFFF, "Jobin Yvon,Inc", "JOBIN-YVON" },
	{ 0x0050C2, 0x35F000, 0x35FFFF, "F.Imm. S.r.L.", "F-IMM-S-R-L" },
	{ 0x0050C2, 0x360000, 0x360FFF, "Digital Receiver Technology, Inc.", "DIGITAL-RECEIVER" },
	{ 0x0050C2, 0x361000, 0x361FFF, "Contec", "CONTEC" },
	{ 0x0050C2, 0x362000, 0x362FFF, "AZD Praha s.r.o.", "AZD-PRAHA-S-R-O" },
	{ 0x0050C2, 0x363000, 0x363FFF, "Septentrio nv/sa", "SEPTENTRIO-NV/" },
	{ 0x0050C2, 0x364000, 0x364FFF, "TATTILE SRL", "TATTILE-SRL" },
	{ 0x0050C2, 0x365000, 0x365FFF, "VPG", "VPG" },
	{ 0x0050C2, 0x366000, 0x366FFF, "Vanguard Technology Corp.", "VANGUARD" },
	{ 0x0050C2, 0x367000, 0x367FFF, "CANMAX Technology Ltd.", "CANMAX" },
	{ 0x0050C2, 0x368000, 0x368FFF, "ASPEL S.A.", "ASPEL" },
	{ 0x0050C2, 0x369000, 0x369FFF, "Always On Wireless", "ALWAYS-ON-WIRELESS" },
	{ 0x0050C2, 0x36A000, 0x36AFFF, "Optronic Partner pr AB", "OPTRONIC-PARTNER-PR-AB" },
	{ 0x0050C2, 0x36B000, 0x36BFFF, "Minerva Technology Inc", "MINERVA" },
	{ 0x0050C2, 0x36C000, 0x36CFFF, "RISCO Group", "RISCO-GROUP" },
	{ 0x0050C2, 0x36D000, 0x36DFFF, "Oplink Communications", "OPLINK-COMMUNICATION" },
	{ 0x0050C2, 0x36E000, 0x36EFFF, "Minicom Advanced Systems Ltd", "MINICOM-ADVANCED" },
	{ 0x0050C2, 0x36F000, 0x36FFFF, "XIMEA s.r.o.", "XIMEA-S-R-O" },
	{ 0x0050C2, 0x370000, 0x370FFF, "Europe Technologies", "EUROPE-TECHNOLOGIES" },
	{ 0x0050C2, 0x371000, 0x371FFF, "DIGITAL ART SYSTEM", "DIGITAL-ART-SYSTEM" },
	{ 0x0050C2, 0x372000, 0x372FFF, "ELV Elektronik AG", "ELV-ELEKTRONIK" },
	{ 0x0050C2, 0x373000, 0x373FFF, "Companion Worlds, inc.", "COMPANION-WORLDS" },
	{ 0x0050C2, 0x374000, 0x374FFF, "Owasys Advanced Wireless Devices", "OWASYS-ADVANCED-WIRELESS-DEVICES" },
	{ 0x0050C2, 0x375000, 0x375FFF, "TIR Systems Ltd.", "TIR" },
	{ 0x0050C2, 0x376000, 0x376FFF, "CLEODE", "CLEODE" },
	{ 0x0050C2, 0x377000, 0x377FFF, "Xycom VME", "XYCOM-VME" },
	{ 0x0050C2, 0x378000, 0x378FFF, "Daintree Networks Inc", "DAINTREE-NETWORKS" },
	{ 0x0050C2, 0x379000, 0x379FFF, "Control LAN S.A.", "CONTROL-LAN" },
	{ 0x0050C2, 0x37A000, 0x37AFFF, "IDA Corporation", "IDA" },
	{ 0x0050C2, 0x37B000, 0x37BFFF, "freescale semiconductor", "FREESCALE-SEMICONDUCTOR" },
	{ 0x0050C2, 0x37C000, 0x37CFFF, "MODIA SYSTEMS Co., Ltd", "MODIA" },
	{ 0x0050C2, 0x37D000, 0x37DFFF, "VeroTrak Inc.", "VEROTRAK" },
	{ 0x0050C2, 0x37E000, 0x37EFFF, "Ni.Co. S.r.l.", "NI-S-R-L" },
	{ 0x0050C2, 0x37F000, 0x37FFFF, "Foresearch", "FORESEARCH" },
	{ 0x0050C2, 0x380000, 0x380FFF, "EKE-Electronics Ltd.", "EKE-ELECTRONICS" },
	{ 0x0050C2, 0x381000, 0x381FFF, "Realtime Engineering AG", "REALTIME-ENGINEERING" },
	{ 0x0050C2, 0x382000, 0x382FFF, "Colorado vNet", "COLORADO-VNET" },
	{ 0x0050C2, 0x383000, 0x383FFF, "ICS Electronics", "ICS-ELECTRONICS" },
	{ 0x0050C2, 0x384000, 0x384FFF, "Wireless Reading Systems Holding ASA", "WIRELESS-READING-HOLDING-ASA" },
	{ 0x0050C2, 0x385000, 0x385FFF, "SUNGJIN NEOTECH Co.Ltd.", "SUNGJIN-NEOTECH" },
	{ 0x0050C2, 0x386000, 0x386FFF, "Precision System Science Co.,Ltd", "PRECISION-SYSTEM-SCIENCE" },
	{ 0x0050C2, 0x387000, 0x387FFF, "Inoteska s.r.o.", "INOTESKA-S-R-O" },
	{ 0x0050C2, 0x388000, 0x388FFF, "IEE Inc", "IEE" },
	{ 0x0050C2, 0x389000, 0x389FFF, "Exavio Inc.", "EXAVIO" },
	{ 0x0050C2, 0x38A000, 0x38AFFF, "Embedtronics Enterprise", "EMBEDTRONICS" },
	{ 0x0050C2, 0x38B000, 0x38BFFF, "InterBridge,Inc.", "INTERBRIDGE" },
	{ 0x0050C2, 0x38C000, 0x38CFFF, "EPSILON SRL", "EPSILON-SRL" },
	{ 0x0050C2, 0x38D000, 0x38DFFF, "A&G Soluzioni Digitali", "A-G-SOLUZIONI-DIGITALI" },
	{ 0x0050C2, 0x38E000, 0x38EFFF, "Nordic Alarm AB", "NORDIC-ALARM-AB" },
	{ 0x0050C2, 0x38F000, 0x38FFFF, "TTC Telecom", "TTC-TELECOM" },
	{ 0x0050C2, 0x390000, 0x390FFF, "TC Communications", "TC-COMMUNICATION" },
	{ 0x0050C2, 0x391000, 0x391FFF, "Esensors, Inc.", "ESENSORS" },
	{ 0x0050C2, 0x392000, 0x392FFF, "PHYTEC Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x393000, 0x393FFF, "SYS TEC electronic GmbH", "SYS-TEC" },
	{ 0x0050C2, 0x394000, 0x394FFF, "Embedit A/S", "EMBEDIT-A/S" },
	{ 0x0050C2, 0x395000, 0x395FFF, "vidisys gmbh", "VIDISYS" },
	{ 0x0050C2, 0x396000, 0x396FFF, "RapidWave Inc.", "RAPIDWAVE" },
	{ 0x0050C2, 0x397000, 0x397FFF, "MANGO DSP Ltd.", "MANGO-DSP" },
	{ 0x0050C2, 0x398000, 0x398FFF, "InHand Electronics, Inc.", "INHAND-ELECTRONICS" },
	{ 0x0050C2, 0x399000, 0x399FFF, "Advanced Micro Controls Inc.", "ADVANCED-MICRO-CONTROLS" },
	{ 0x0050C2, 0x39A000, 0x39AFFF, "Optical Air Data Systems", "OPTICAL-AIR-DATA" },
	{ 0x0050C2, 0x39B000, 0x39BFFF, "YUYAMA MFG. CO., LTD.", "YUYAMA-MFG" },
	{ 0x0050C2, 0x39C000, 0x39CFFF, "TIYODA MFG CO.,LTD.", "TIYODA-MFG" },
	{ 0x0050C2, 0x39D000, 0x39DFFF, "DigitalDeck, Inc.", "DIGITALDECK" },
	{ 0x0050C2, 0x39E000, 0x39EFFF, "A.R.G ElectroDesign Ltd", "A-R-G-ELECTRODESIGN" },
	{ 0x0050C2, 0x39F000, 0x39FFFF, "Isensix", "ISENSIX" },
	{ 0x0050C2, 0x3A0000, 0x3A0FFF, "StreetFire Sound Labs, LLC", "STREETFIRE-SOUND-LABS-LLC" },
	{ 0x0050C2, 0x3A1000, 0x3A1FFF, "Samsoft", "SAMSOFT" },
	{ 0x0050C2, 0x3A2000, 0x3A2FFF, "Vegas Amusement", "VEGAS-AMUSEMENT" },
	{ 0x0050C2, 0x3A3000, 0x3A3FFF, "Star Link Communication Pvt. Ltd.", "STAR-LINK-COMMUNICATION-PVT" },
	{ 0x0050C2, 0x3A4000, 0x3A4FFF, "Silvertree Engineering Ltd", "SILVERTREE-ENGINEERING" },
	{ 0x0050C2, 0x3A5000, 0x3A5FFF, "LabJack Corporation", "LABJACK" },
	{ 0x0050C2, 0x3A6000, 0x3A6FFF, "IntelliDesign Pty Ltd", "INTELLIDESIGN-PTY" },
	{ 0x0050C2, 0x3A7000, 0x3A7FFF, "Elektrotechnik & Elektronik Oltmann GmbH", "ELEKTROTECHNIK-ELEKTRONIK-OLTMANN" },
	{ 0x0050C2, 0x3A8000, 0x3A8FFF, "Engim, Inc.", "ENGIM" },
	{ 0x0050C2, 0x3A9000, 0x3A9FFF, "Westronic Systems Inc.", "WESTRONIC" },
	{ 0x0050C2, 0x3AA000, 0x3AAFFF, "Networked Robotics Corporation", "NETWORKED-ROBOTICS" },
	{ 0x0050C2, 0x3AB000, 0x3ABFFF, "taskit Rechnertechnik GmbH", "TASKIT-RECHNERTECHNIK" },
	{ 0x0050C2, 0x3AC000, 0x3ACFFF, "InAccess Networks", "INACCESS-NETWORKS" },
	{ 0x0050C2, 0x3AD000, 0x3ADFFF, "Spirent Communications (Scotland) Limited", "SPIRENT-COMMUNICATION-SCOTLAND" },
	{ 0x0050C2, 0x3AE000, 0x3AEFFF, "Hankuk Tapi Computer Co., Ltd", "HANKUK-TAPI" },
	{ 0x0050C2, 0x3AF000, 0x3AFFFF, "Norbit ODM AS", "NORBIT-ODM-AS" },
	{ 0x0050C2, 0x3B0000, 0x3B0FFF, "Microtarget Tecnologia Digital Ltda.", "MICROTARGET-TECNOLOGIA-DIGITAL-LTDA" },
	{ 0x0050C2, 0x3B1000, 0x3B1FFF, "RDC Specstroy-Svyaz Ltd", "RDC-SPECSTROY-SVYAZ" },
	{ 0x0050C2, 0x3B2000, 0x3B2FFF, "Tennessee Valley Authority", "TENNESSEE-VALLEY-AUTHORITY" },
	{ 0x0050C2, 0x3B3000, 0x3B3FFF, "Media Lab., Inc.", "MEDIA-LAB" },
	{ 0x0050C2, 0x3B4000, 0x3B4FFF, "Contrôle Analytique inc.", "CONTRôLE-ANALYTIQUE" },
	{ 0x0050C2, 0x3B5000, 0x3B5FFF, "NEC TOKIN Corporation", "NEC-TOKIN" },
	{ 0x0050C2, 0x3B6000, 0x3B6FFF, "Arecont Vision, LLC", "ARECONT-VISION-LLC" },
	{ 0x0050C2, 0x3B7000, 0x3B7FFF, "Mindspeed Technologies", "MINDSPEED-TECHNOLOGIES" },
	{ 0x0050C2, 0x3B8000, 0x3B8FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x3B9000, 0x3B9FFF, "Gilbarco Autotank AB", "GILBARCO-AUTOTANK-AB" },
	{ 0x0050C2, 0x3BA000, 0x3BAFFF, "PHYTEC Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x3BB000, 0x3BBFFF, "IMAGO Technologies GmbH", "IMAGO-TECHNOLOGIES" },
	{ 0x0050C2, 0x3BC000, 0x3BCFFF, "Tyzx, Inc.", "TYZX" },
	{ 0x0050C2, 0x3BD000, 0x3BDFFF, "Bigbang L.T.D.", "BIGBANG-L-T-D" },
	{ 0x0050C2, 0x3BE000, 0x3BEFFF, "Pauly Steuer- und Regelanlagen GmbH & Co. KG", "PAULY-STEUER--UND-REGELANLAGEN" },
	{ 0x0050C2, 0x3BF000, 0x3BFFFF, "Audio Processing Technology Ltd", "AUDIO-PROCESSING" },
	{ 0x0050C2, 0x3C0000, 0x3C0FFF, "EDA Industries Srl", "EDA-INDUSTRIES-SRL" },
	{ 0x0050C2, 0x3C1000, 0x3C1FFF, "MicroTek Electronics, Inc.", "MICROTEK-ELECTRONICS" },
	{ 0x0050C2, 0x3C2000, 0x3C2FFF, "Casabyte Inc.", "CASABYTE" },
	{ 0x0050C2, 0x3C3000, 0x3C3FFF, "4g Technologies, L.P.", "4G-TECHNOLOGIES-L-P" },
	{ 0x0050C2, 0x3C4000, 0x3C4FFF, "Sypris Electronics", "SYPRIS-ELECTRONICS" },
	{ 0x0050C2, 0x3C5000, 0x3C5FFF, "Silicon Optix Canada Inc.", "SILICON-OPTIX-CANADA" },
	{ 0x0050C2, 0x3C6000, 0x3C6FFF, "Net Optics", "NET-OPTICS" },
	{ 0x0050C2, 0x3C7000, 0x3C7FFF, "Salent Technologies Ltd", "SALENT-TECHNOLOGIES" },
	{ 0x0050C2, 0x3C8000, 0x3C8FFF, "Wheels of Zeus Inc.", "WHEELS-OF-ZEUS" },
	{ 0x0050C2, 0x3C9000, 0x3C9FFF, "Dilax Intelcom AG", "DILAX-INTELCOM" },
	{ 0x0050C2, 0x3CA000, 0x3CAFFF, "ABB Inc.", "ABB" },
	{ 0x0050C2, 0x3CB000, 0x3CBFFF, "Analytica GmbH", "ANALYTICA" },
	{ 0x0050C2, 0x3CC000, 0x3CCFFF, "LINKWELL TELESYSTEMS PRIVATE LIMITED", "LINKWELL-TELESYSTEMS-PRIVATE" },
	{ 0x0050C2, 0x3CD000, 0x3CDFFF, "Micro-Measurements", "MICRO-MEASUREMENTS" },
	{ 0x0050C2, 0x3CE000, 0x3CEFFF, "Ward Leonard Electric Company", "WARD-LEONARD-ELECTRIC-COMPANY" },
	{ 0x0050C2, 0x3CF000, 0x3CFFFF, "Technovare Systems, Inc.", "TECHNOVARE" },
	{ 0x0050C2, 0x3D0000, 0x3D0FFF, "Micro-Robotics Limited", "MICRO-ROBOTICS" },
	{ 0x0050C2, 0x3D1000, 0x3D1FFF, "Braintronics BV", "BRAINTRONICS" },
	{ 0x0050C2, 0x3D2000, 0x3D2FFF, "Adilec Enginyeria SL", "ADILEC-ENGINYERIA-SL" },
	{ 0x0050C2, 0x3D3000, 0x3D3FFF, "American LED-gible Inc.", "AMERICAN-LED-GIBLE" },
	{ 0x0050C2, 0x3D4000, 0x3D4FFF, "Wisnu and Supak Co.Ltd.", "WISNU-AND-SUPAK" },
	{ 0x0050C2, 0x3D5000, 0x3D5FFF, "Fluke Biomedical, Radiation Management Services", "FLUKE-BIOMEDICAL-RADIATION-MANAGEMENT-SERVICES" },
	{ 0x0050C2, 0x3D6000, 0x3D6FFF, "Comlab Inc.", "COMLAB" },
	{ 0x0050C2, 0x3D7000, 0x3D7FFF, "TTC TELEKOMUNIKACE Ltd", "TTC-TELEKOMUNIKACE" },
	{ 0x0050C2, 0x3D8000, 0x3D8FFF, "Key Systems , Inc.", "KEY" },
	{ 0x0050C2, 0x3D9000, 0x3D9FFF, "Bavaria Digital Technik GmbH", "BAVARIA-DIGITAL-TECHNIK" },
	{ 0x0050C2, 0x3DA000, 0x3DAFFF, "M5 Data Limited", "M5-DATA" },
	{ 0x0050C2, 0x3DB000, 0x3DBFFF, "Osmetech Inc.", "OSMETECH" },
	{ 0x0050C2, 0x3DC000, 0x3DCFFF, "3D perception", "3D-PERCEPTION" },
	{ 0x0050C2, 0x3DD000, 0x3DDFFF, "ELMIC GmbH", "ELMIC" },
	{ 0x0050C2, 0x3DE000, 0x3DEFFF, "ABB Power Technologies", "ABB-POWER-TECHNOLOGIES" },
	{ 0x0050C2, 0x3DF000, 0x3DFFFF, "BiODE Inc.", "BIODE" },
	{ 0x0050C2, 0x3E0000, 0x3E0FFF, "Oy Stinghorn Ltd", "OY-STINGHORN" },
	{ 0x0050C2, 0x3E1000, 0x3E1FFF, "NeuLion Incorporated", "NEULION-INCORPORATED" },
	{ 0x0050C2, 0x3E2000, 0x3E2FFF, "SysNova", "SYSNOVA" },
	{ 0x0050C2, 0x3E3000, 0x3E3FFF, "CSIRO - Division of Exploration and Mining", "CSIRO---DIVISION-OF-EXPLORATION-AND-MINING" },
	{ 0x0050C2, 0x3E4000, 0x3E4FFF, "CUE, a.s.", "CUE-A-S" },
	{ 0x0050C2, 0x3E5000, 0x3E5FFF, "Vacon Plc", "VACON-PLC" },
	{ 0x0050C2, 0x3E6000, 0x3E6FFF, "CRDE", "CRDE" },
	{ 0x0050C2, 0x3E7000, 0x3E7FFF, "Revolution Education Ltd", "REVOLUTION-EDUCATION" },
	{ 0x0050C2, 0x3E8000, 0x3E8FFF, "Conformative Systems, Inc.", "CONFORMATIVE" },
	{ 0x0050C2, 0x3E9000, 0x3E9FFF, "MedAvant Healthcare", "MEDAVANT-HEALTHCARE" },
	{ 0x0050C2, 0x3EA000, 0x3EAFFF, "Alro Information Systems SA", "ALRO-INFORMATION" },
	{ 0x0050C2, 0x3EB000, 0x3EBFFF, "ISS International", "ISS" },
	{ 0x0050C2, 0x3EC000, 0x3ECFFF, "Teneros", "TENEROS" },
	{ 0x0050C2, 0x3ED000, 0x3EDFFF, "The Board Room Inc.", "THE-BOARD-ROOM" },
	{ 0x0050C2, 0x3EE000, 0x3EEFFF, "Commoca, Inc", "COMMOCA" },
	{ 0x0050C2, 0x3EF000, 0x3EFFFF, "PAT Industries, DBA Pacific Advanced Technology", "PAT-INDUSTRIES-DBA-PACIFIC-ADVANCED" },
	{ 0x0050C2, 0x3F0000, 0x3F0FFF, "megatec electronic GmbH", "MEGATEC" },
	{ 0x0050C2, 0x3F1000, 0x3F1FFF, "Salland Electronics Holding BV", "SALLAND-ELECTRONICS-HOLDING" },
	{ 0x0050C2, 0x3F2000, 0x3F2FFF, "STL GmbH", "STL" },
	{ 0x0050C2, 0x3F3000, 0x3F3FFF, "Hytec Geraetebau GmbH", "HYTEC-GERAETEBAU" },
	{ 0x0050C2, 0x3F4000, 0x3F4FFF, "MC TECHNOLOGY GmbH", "MC" },
	{ 0x0050C2, 0x3F5000, 0x3F5FFF, "Phaedrus Limited", "PHAEDRUS" },
	{ 0x0050C2, 0x3F6000, 0x3F6FFF, "dAFTdATA Limited", "DAFTDATA" },
	{ 0x0050C2, 0x3F7000, 0x3F7FFF, "Advantage R&D", "ADVANTAGE-R-D" },
	{ 0x0050C2, 0x3F8000, 0x3F8FFF, "Superna Ltd", "SUPERNA" },
	{ 0x0050C2, 0x3F9000, 0x3F9FFF, "Sintium Ltd", "SINTIUM" },
	{ 0x0050C2, 0x3FA000, 0x3FAFFF, "Tumsan", "TUMSAN" },
	{ 0x0050C2, 0x3FB000, 0x3FBFFF, "Pigeon Point Systems LLC", "PIGEON-POINT-LLC" },
	{ 0x0050C2, 0x3FC000, 0x3FCFFF, "Weinberger Deutschland GmbH", "WEINBERGER-DEUTSCHLAND" },
	{ 0x0050C2, 0x3FD000, 0x3FDFFF, "HARTMANN software GbR", "HARTMANN-SOFTWARE-GBR" },
	{ 0x0050C2, 0x3FE000, 0x3FEFFF, "HaiVision Systems Inc", "HAIVISION" },
	{ 0x0050C2, 0x3FF000, 0x3FFFFF, "Cast Iron Systems", "CAST-IRON" },
	{ 0x0050C2, 0x400000, 0x400FFF, "SmartMotor AS", "SMARTMOTOR-AS" },
	{ 0x0050C2, 0x401000, 0x401FFF, "Promess Incorporated", "PROMESS-INCORPORATED" },
	{ 0x0050C2, 0x402000, 0x402FFF, "Numeron Sp. z o.o.", "NUMERON-SP-Z-O-O" },
	{ 0x0050C2, 0x403000, 0x403FFF, "Rohde & Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0x404000, 0x404FFF, "NanShanBridge Co.Ltd", "NANSHANBRIDGE" },
	{ 0x0050C2, 0x405000, 0x405FFF, "Guralp Systems Limited", "GURALP" },
	{ 0x0050C2, 0x406000, 0x406FFF, "CoreStreet, Ltd", "CORESTREET" },
	{ 0x0050C2, 0x407000, 0x407FFF, "AIE Etudes", "AIE-ETUDES" },
	{ 0x0050C2, 0x408000, 0x408FFF, "TERN, Inc.", "TERN" },
	{ 0x0050C2, 0x409000, 0x409FFF, "KTEC LTD", "KTEC" },
	{ 0x0050C2, 0x40A000, 0x40AFFF, "Contec Steuerungstechnik & Automation GmbH", "CONTEC-STEUERUNGSTECHNIK-AUTOMATION" },
	{ 0x0050C2, 0x40B000, 0x40BFFF, "Center VOSPI JSC", "CENTER-VOSPI-JSC" },
	{ 0x0050C2, 0x40C000, 0x40CFFF, "Applied Materials UK Ltd", "APPLIED-MATERIALS-UK" },
	{ 0x0050C2, 0x40D000, 0x40DFFF, "Afonics Fibreoptics Ltd", "AFONICS-FIBREOPTICS" },
	{ 0x0050C2, 0x40E000, 0x40EFFF, "ads-tec GmbH", "ADS-TEC" },
	{ 0x0050C2, 0x40F000, 0x40FFFF, "BIR,INC.", "BIR" },
	{ 0x0050C2, 0x410000, 0x410FFF, "Grossenbacher Systeme AG", "GROSSENBACHER-SYSTEME" },
	{ 0x0050C2, 0x411000, 0x411FFF, "Multimessage Systems Ltd.", "MULTIMESSAGE" },
	{ 0x0050C2, 0x412000, 0x412FFF, "TSB Solutions Inc.", "TSB-SOLUTIONS" },
	{ 0x0050C2, 0x413000, 0x413FFF, "Goodrich", "GOODRICH" },
	{ 0x0050C2, 0x414000, 0x414FFF, "Talleres de Escoriaza SA", "TALLERES-DE-ESCORIAZA" },
	{ 0x0050C2, 0x415000, 0x415FFF, "SensoTech GmbH", "SENSOTECH" },
	{ 0x0050C2, 0x416000, 0x416FFF, "SELCO s.r.l.", "SELCO-S-R-L" },
	{ 0x0050C2, 0x417000, 0x417FFF, "QT systems ab", "QT-AB" },
	{ 0x0050C2, 0x418000, 0x418FFF, "Planea Oy", "PLANEA-OY" },
	{ 0x0050C2, 0x419000, 0x419FFF, "Mecsel Oy", "MECSEL-OY" },
	{ 0x0050C2, 0x41A000, 0x41AFFF, "Bluewater Systems Ltd", "BLUEWATER" },
	{ 0x0050C2, 0x41B000, 0x41BFFF, "LogiM GmbH Software und Entwicklung", "LOGIM-SOFTWARE-UND-ENTWICKLUNG" },
	{ 0x0050C2, 0x41C000, 0x41CFFF, "Infrasafe, Inc.", "INFRASAFE" },
	{ 0x0050C2, 0x41D000, 0x41DFFF, "Altronic, Inc.", "ALTRONIC" },
	{ 0x0050C2, 0x41E000, 0x41EFFF, "Videotek Sistemas Eletronicos Ltda.", "VIDEOTEK-SISTEMAS-ELETRONICOS-LTDA" },
	{ 0x0050C2, 0x41F000, 0x41FFFF, "Avionica, Inc", "AVIONICA" },
	{ 0x0050C2, 0x420000, 0x420FFF, "Boundless Technologies", "BOUNDLESS-TECHNOLOGIES" },
	{ 0x0050C2, 0x421000, 0x421FFF, "EFSYS", "EFSYS" },
	{ 0x0050C2, 0x422000, 0x422FFF, "Gekeler Martina", "GEKELER-MARTINA" },
	{ 0x0050C2, 0x423000, 0x423FFF, "Power-One Inc.", "POWER-ONE" },
	{ 0x0050C2, 0x424000, 0x424FFF, "Metrolab Technology SA", "METROLAB" },
	{ 0x0050C2, 0x425000, 0x425FFF, "Pinnacle Technology", "PINNACLE" },
	{ 0x0050C2, 0x426000, 0x426FFF, "STOM System", "STOM-SYSTEM" },
	{ 0x0050C2, 0x427000, 0x427FFF, "Scheidt & Bachmann GmbH", "SCHEIDT-BACHMANN" },
	{ 0x0050C2, 0x428000, 0x428FFF, "Roxar A/S", "ROXAR-A/S" },
	{ 0x0050C2, 0x429000, 0x429FFF, "Matthews Australasia", "MATTHEWS-AUSTRALASIA" },
	{ 0x0050C2, 0x42A000, 0x42AFFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x0050C2, 0x42B000, 0x42BFFF, "VLSIP TECHNOLOGIES, INC.", "VLSIP-TECHNOLOGIES" },
	{ 0x0050C2, 0x42C000, 0x42CFFF, "Trapeze ITS U.S.A., LLC", "TRAPEZE-ITS-U-LLC" },
	{ 0x0050C2, 0x42D000, 0x42DFFF, "Argo-Tech", "ARGO-TECH" },
	{ 0x0050C2, 0x42E000, 0x42EFFF, "Oelmann Elektronik GmbH", "OELMANN-ELEKTRONIK" },
	{ 0x0050C2, 0x42F000, 0x42FFFF, "Win4NET", "WIN4NET" },
	{ 0x0050C2, 0x430000, 0x430FFF, "Arcom Digital", "ARCOM-DIGITAL" },
	{ 0x0050C2, 0x431000, 0x431FFF, "Octatron, Inc.", "OCTATRON" },
	{ 0x0050C2, 0x432000, 0x432FFF, "Topway Industries Ltd.", "TOPWAY-INDUSTRIES" },
	{ 0x0050C2, 0x433000, 0x433FFF, "Idetech Europe S.A.", "IDETECH-EUROPE" },
	{ 0x0050C2, 0x434000, 0x434FFF, "ImperativeNetworks", "IMPERATIVENETWORKS" },
	{ 0x0050C2, 0x435000, 0x435FFF, "ADATEL TELECOMUNICACIONES S.A.", "ADATEL-TELECOMUNICACIONES" },
	{ 0x0050C2, 0x436000, 0x436FFF, "Satellite Services BV", "SATELLITE-SERVICES" },
	{ 0x0050C2, 0x437000, 0x437FFF, "PowerWAN, Inc", "POWERWAN" },
	{ 0x0050C2, 0x438000, 0x438FFF, "Telecom Protection Technologies Limited", "TELECOM-PROTECTION-TECHNOLOGIES" },
	{ 0x0050C2, 0x439000, 0x439FFF, "Peleton Photonic Systems", "PELETON-PHOTONIC" },
	{ 0x0050C2, 0x43A000, 0x43AFFF, "ProDesign GmbH", "PRODESIGN" },
	{ 0x0050C2, 0x43B000, 0x43BFFF, "A3IP", "A3IP" },
	{ 0x0050C2, 0x43C000, 0x43CFFF, "Ducommun LaBarge Technologies, Inc", "DUCOMMUN-LABARGE-TECHNOLOGIES" },
	{ 0x0050C2, 0x43D000, 0x43DFFF, "Ann Arbor Sensor Systems LLC", "ANN-ARBOR-SENSOR-LLC" },
	{ 0x0050C2, 0x43E000, 0x43EFFF, "Coppercom", "COPPERCOM" },
	{ 0x0050C2, 0x43F000, 0x43FFFF, "ARVOO Imaging Products BV", "ARVOO-IMAGING-PRODUCTS" },
	{ 0x0050C2, 0x440000, 0x440FFF, "Advanced Modular Computers Ltd.", "ADVANCED-MODULAR-COMPUTERS" },
	{ 0x0050C2, 0x441000, 0x441FFF, "Sammi Information Systems Co.,Ltd", "SAMMI-INFORMATION" },
	{ 0x0050C2, 0x442000, 0x442FFF, "Pico Computing, Inc.", "PICO-COMPUTING" },
	{ 0x0050C2, 0x443000, 0x443FFF, "Pickering Laboratories", "PICKERING-LABORATORIES" },
	{ 0x0050C2, 0x444000, 0x444FFF, "Offshore Systems Ltd", "OFFSHORE" },
	{ 0x0050C2, 0x445000, 0x445FFF, "MICRONIC s.r.o.", "MICRONIC-S-R-O" },
	{ 0x0050C2, 0x446000, 0x446FFF, "Micro Technic A-S", "MICRO-TECHNIC-A-S" },
	{ 0x0050C2, 0x447000, 0x447FFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0x448000, 0x448FFF, "Comtech Systems Inc.", "COMTECH" },
	{ 0x0050C2, 0x449000, 0x449FFF, "BLEILE DATENTECHNIK GmbH", "BLEILE-DATENTECHNIK" },
	{ 0x0050C2, 0x44A000, 0x44AFFF, "ELETTRONICA SANTERNO SPA", "ELETTRONICA-SANTERNO-SPA" },
	{ 0x0050C2, 0x44B000, 0x44BFFF, "Solace Systems, Inc.", "SOLACE" },
	{ 0x0050C2, 0x44C000, 0x44CFFF, "Computime Systems UK Ltd.", "COMPUTIME-UK" },
	{ 0x0050C2, 0x44D000, 0x44DFFF, "Electro-Matic Products, Inc.", "ELECTRO-MATIC-PRODUCTS" },
	{ 0x0050C2, 0x44E000, 0x44EFFF, "QQ Technology,Inc", "QQ" },
	{ 0x0050C2, 0x44F000, 0x44FFFF, "kippdata GmbH", "KIPPDATA" },
	{ 0x0050C2, 0x450000, 0x450FFF, "Enconair Ecological Chambers Inc.", "ENCONAIR-ECOLOGICAL-CHAMBERS" },
	{ 0x0050C2, 0x451000, 0x451FFF, "HAMEG GmbH", "HAMEG" },
	{ 0x0050C2, 0x452000, 0x452FFF, "SCAME SISTEMI s.r.l.", "SCAME-SISTEMI-S-R-L" },
	{ 0x0050C2, 0x453000, 0x453FFF, "Erhardt + Leimer GmbH", "ERHARDT-+-LEIMER" },
	{ 0x0050C2, 0x454000, 0x454FFF, "Brivo Systems, LLC", "BRIVO-LLC" },
	{ 0x0050C2, 0x455000, 0x455FFF, "AirCell, Inc.", "AIRCELL" },
	{ 0x0050C2, 0x456000, 0x456FFF, "DRDC Valcartier", "DRDC-VALCARTIER" },
	{ 0x0050C2, 0x457000, 0x457FFF, "Danbridge", "DANBRIDGE" },
	{ 0x0050C2, 0x458000, 0x458FFF, "HRZ data GmbH", "HRZ-DATA" },
	{ 0x0050C2, 0x459000, 0x459FFF, "PHYTEC Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x45A000, 0x45AFFF, "Funkwerk plettac electronic GmbH", "FUNKWERK-PLETTAC" },
	{ 0x0050C2, 0x45B000, 0x45BFFF, "Matra Electronique", "MATRA-ELECTRONIQUE" },
	{ 0x0050C2, 0x45C000, 0x45CFFF, "Deister Electronic GmbH", "DEISTER" },
	{ 0x0050C2, 0x45D000, 0x45DFFF, "Digital Engineering, Inc.", "DIGITAL-ENGINEERING" },
	{ 0x0050C2, 0x45E000, 0x45EFFF, "Halliburton - Sperry Drilling Service", "HALLIBURTON---SPERRY-DRILLING-SERVICE" },
	{ 0x0050C2, 0x45F000, 0x45FFFF, "T2C Marketing AB", "T2C-MARKETING-AB" },
	{ 0x0050C2, 0x460000, 0x460FFF, "Vitelnet", "VITELNET" },
	{ 0x0050C2, 0x461000, 0x461FFF, "TATTILE SRL", "TATTILE-SRL" },
	{ 0x0050C2, 0x462000, 0x462FFF, "CT Company", "CT-COMPANY" },
	{ 0x0050C2, 0x463000, 0x463FFF, "Codem Systems, Inc.", "CODEM" },
	{ 0x0050C2, 0x464000, 0x464FFF, "XYTAC system technologies", "XYTAC-SYSTEM-TECHNOLOGIES" },
	{ 0x0050C2, 0x465000, 0x465FFF, "PDTS GmbH", "PDTS" },
	{ 0x0050C2, 0x466000, 0x466FFF, "LONAP Limited", "LONAP" },
	{ 0x0050C2, 0x467000, 0x467FFF, "United Western Technologies", "UNITED-WESTERN-TECHNOLOGIES" },
	{ 0x0050C2, 0x468000, 0x468FFF, "Network I/O", "NETWORK-I/O" },
	{ 0x0050C2, 0x469000, 0x469FFF, "BiPOM Electronics, Inc.", "BIPOM-ELECTRONICS" },
	{ 0x0050C2, 0x46A000, 0x46AFFF, "ISE GmbH", "ISE" },
	{ 0x0050C2, 0x46B000, 0x46BFFF, "EASYTECH GmbH", "EASYTECH" },
	{ 0x0050C2, 0x46C000, 0x46CFFF, "CAMCO GmbH", "CAMCO" },
	{ 0x0050C2, 0x46D000, 0x46DFFF, "Paul Scherrer Institut (PSI)", "PAUL-SCHERRER-INSTITUT-PSI" },
	{ 0x0050C2, 0x46E000, 0x46EFFF, "Avenir Technologies Inc.", "AVENIR-TECHNOLOGIES" },
	{ 0x0050C2, 0x46F000, 0x46FFFF, "Neuroware", "NEUROWARE" },
	{ 0x0050C2, 0x470000, 0x470FFF, "Cybectec inc.", "CYBECTEC" },
	{ 0x0050C2, 0x471000, 0x471FFF, "Pixtree Technologies, inc.", "PIXTREE-TECHNOLOGIES" },
	{ 0x0050C2, 0x472000, 0x472FFF, "KOP Ltd", "KOP" },
	{ 0x0050C2, 0x473000, 0x473FFF, "Sensus Metering Systems Israel", "SENSUS-METERING-ISRAEL" },
	{ 0x0050C2, 0x474000, 0x474FFF, "Venue 1, Inc.", "VENUE-1" },
	{ 0x0050C2, 0x475000, 0x475FFF, "ISEPOS GmbH", "ISEPOS" },
	{ 0x0050C2, 0x476000, 0x476FFF, "Ascon S.p.a.", "ASCON-S-P-A" },
	{ 0x0050C2, 0x477000, 0x477FFF, "SEV Tidsystem AB", "SEV-TIDSYSTEM-AB" },
	{ 0x0050C2, 0x478000, 0x478FFF, "Metafix Inc.", "METAFIX" },
	{ 0x0050C2, 0x479000, 0x479FFF, "Unlimited Bandwidth LLC", "UNLIMITED-BANDWIDTH-LLC" },
	{ 0x0050C2, 0x47A000, 0x47AFFF, "Efficient Channel Coding", "EFFICIENT-CHANNEL-CODING" },
	{ 0x0050C2, 0x47B000, 0x47BFFF, "Pitney Bowes, Inc", "PITNEY-BOWES" },
	{ 0x0050C2, 0x47C000, 0x47CFFF, "AUCONET GmbH", "AUCONET" },
	{ 0x0050C2, 0x47D000, 0x47DFFF, "WIT Inc", "WIT" },
	{ 0x0050C2, 0x47E000, 0x47EFFF, "Energie Umwelt Systemtechnik GmbH", "ENERGIE-UMWELT-SYSTEMTECHNIK" },
	{ 0x0050C2, 0x47F000, 0x47FFFF, "BRIT Inc.", "BRIT" },
	{ 0x0050C2, 0x480000, 0x480FFF, "SELKOM GmbH", "SELKOM" },
	{ 0x0050C2, 0x481000, 0x481FFF, "Computer Sciences Corp", "SCIENCES" },
	{ 0x0050C2, 0x482000, 0x482FFF, "PRIAMUS SYSTEM TECHNOLOGIES AG", "PRIAMUS-SYSTEM-TECHNOLOGIES" },
	{ 0x0050C2, 0x483000, 0x483FFF, "SES", "SES" },
	{ 0x0050C2, 0x484000, 0x484FFF, "Kooltech LLC", "KOOLTECH-LLC" },
	{ 0x0050C2, 0x485000, 0x485FFF, "PHYTEC Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x486000, 0x486FFF, "Safegate International AB", "SAFEGATE-AB" },
	{ 0x0050C2, 0x487000, 0x487FFF, "Eridon Corporation", "ERIDON" },
	{ 0x0050C2, 0x488000, 0x488FFF, "DA SISTEMI SPA", "DA-SISTEMI-SPA" },
	{ 0x0050C2, 0x489000, 0x489FFF, "EREE Electronique", "EREE-ELECTRONIQUE" },
	{ 0x0050C2, 0x48A000, 0x48AFFF, "Mobile Matrix, Inc.", "MOBILE-MATRIX" },
	{ 0x0050C2, 0x48B000, 0x48BFFF, "ADS-TEC GmbH", "ADS-TEC" },
	{ 0x0050C2, 0x48C000, 0x48CFFF, "UNITON AG", "UNITON" },
	{ 0x0050C2, 0x48D000, 0x48DFFF, "Metron Sp. z o.o.", "METRON-SP-Z-O-O" },
	{ 0x0050C2, 0x48E000, 0x48EFFF, "Teledyne Tekmar", "TELEDYNE-TEKMAR" },
	{ 0x0050C2, 0x48F000, 0x48FFFF, "DENGYOSHA co.,LTD.", "DENGYOSHA" },
	{ 0x0050C2, 0x490000, 0x490FFF, "Cloanto Corporation", "CLOANTO" },
	{ 0x0050C2, 0x491000, 0x491FFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0x492000, 0x492FFF, "TRAFSYS AS", "TRAFSYS-AS" },
	{ 0x0050C2, 0x493000, 0x493FFF, "Artis GmbH", "ARTIS" },
	{ 0x0050C2, 0x494000, 0x494FFF, "Ultimate Technology, Inc.", "ULTIMATE" },
	{ 0x0050C2, 0x495000, 0x495FFF, "VAZA Elektronik AB", "VAZA-ELEKTRONIK-AB" },
	{ 0x0050C2, 0x496000, 0x496FFF, "Acutelogic Corporation", "ACUTELOGIC" },
	{ 0x0050C2, 0x497000, 0x497FFF, "Advanced Driver Information Technology GmbH", "ADVANCED-DRIVER-INFORMATION" },
	{ 0x0050C2, 0x498000, 0x498FFF, "Quartet Technology, Inc.", "QUARTET" },
	{ 0x0050C2, 0x499000, 0x499FFF, "Trellia Networks", "TRELLIA-NETWORKS" },
	{ 0x0050C2, 0x49A000, 0x49AFFF, "TelASIC Communications, Inc.", "TELASIC-COMMUNICATION" },
	{ 0x0050C2, 0x49B000, 0x49BFFF, "vg controls, inc", "VG-CONTROLS" },
	{ 0x0050C2, 0x49C000, 0x49CFFF, "Envisacor Technologies Inc.", "ENVISACOR-TECHNOLOGIES" },
	{ 0x0050C2, 0x49D000, 0x49DFFF, "Critical Link", "CRITICAL-LINK" },
	{ 0x0050C2, 0x49E000, 0x49EFFF, "Armorlink CO .Ltd", "ARMORLINK" },
	{ 0x0050C2, 0x49F000, 0x49FFFF, "GCS, Inc", "GCS" },
	{ 0x0050C2, 0x4A0000, 0x4A0FFF, "Advanced technologies & Engineering (pty) Ltd", "ADVANCED-TECHNOLOGIES-ENGINEERING-PTY" },
	{ 0x0050C2, 0x4A1000, 0x4A1FFF, "Pigeon Point Systems LLC", "PIGEON-POINT-LLC" },
	{ 0x0050C2, 0x4A2000, 0x4A2FFF, "SPECS GmbH", "SPECS" },
	{ 0x0050C2, 0x4A3000, 0x4A3FFF, "Protium Technologies, Inc.", "PROTIUM-TECHNOLOGIES" },
	{ 0x0050C2, 0x4A4000, 0x4A4FFF, "IEEE P1609 WG", "IEEE-P1609-WG" },
	{ 0x0050C2, 0x4A5000, 0x4A5FFF, "Teledyne Monitor Labs", "TELEDYNE-MONITOR-LABS" },
	{ 0x0050C2, 0x4A6000, 0x4A6FFF, "BUYANG ELECTRONICS INDUSTRIAL CO., LTD.", "BUYANG-ELECTRONICS-INDUSTRIAL" },
	{ 0x0050C2, 0x4A7000, 0x4A7FFF, "iseg Spezialelektronik GmbH", "ISEG-SPEZIALELEKTRONIK" },
	{ 0x0050C2, 0x4A8000, 0x4A8FFF, "CYJAYA Korea", "CYJAYA-KOREA" },
	{ 0x0050C2, 0x4A9000, 0x4A9FFF, "Faber Electronics BV", "FABER-ELECTRONICS" },
	{ 0x0050C2, 0x4AA000, 0x4AAFFF, "HEINEN ELEKTRONIK GmbH", "HEINEN-ELEKTRONIK" },
	{ 0x0050C2, 0x4AB000, 0x4ABFFF, "JVF Communications Ltd", "JVF-COMMUNICATION" },
	{ 0x0050C2, 0x4AC000, 0x4ACFFF, "Doramu Co.,Ltd.", "DORAMU" },
	{ 0x0050C2, 0x4AD000, 0x4ADFFF, "OpenPeak, Inc.", "OPENPEAK" },
	{ 0x0050C2, 0x4AE000, 0x4AEFFF, "ads-tec GmbH", "ADS-TEC" },
	{ 0x0050C2, 0x4AF000, 0x4AFFFF, "Orbis Oy", "ORBIS-OY" },
	{ 0x0050C2, 0x4B0000, 0x4B0FFF, "Esmart Distribution Pte Ltd", "ESMART-DISTRIBUTION-PTE" },
	{ 0x0050C2, 0x4B1000, 0x4B1FFF, "Nsfocus Information Technology Co.,Ltd", "NSFOCUS-INFORMATION" },
	{ 0x0050C2, 0x4B2000, 0x4B2FFF, "TESLA, a.s.", "TESLA-A-S" },
	{ 0x0050C2, 0x4B3000, 0x4B3FFF, "ANSA Corporation", "ANSA" },
	{ 0x0050C2, 0x4B4000, 0x4B4FFF, "Matrix Audio Designs", "MATRIX-AUDIO-DESIGNS" },
	{ 0x0050C2, 0x4B5000, 0x4B5FFF, "Valley Tecnologia", "VALLEY-TECNOLOGIA" },
	{ 0x0050C2, 0x4B6000, 0x4B6FFF, "General Resources Co., LTD.", "GENERAL-RESOURCES" },
	{ 0x0050C2, 0x4B7000, 0x4B7FFF, "GFI Chrono Time", "GFI-CHRONO-TIME" },
	{ 0x0050C2, 0x4B8000, 0x4B8FFF, "Shenzhen Hongdian Technologies.,Ltd", "SHENZHEN-HONGDIAN-TECHNOLOGIES" },
	{ 0x0050C2, 0x4B9000, 0x4B9FFF, "Rose Technologies", "ROSE-TECHNOLOGIES" },
	{ 0x0050C2, 0x4BA000, 0x4BAFFF, "Mistletoe Technologies", "MISTLETOE-TECHNOLOGIES" },
	{ 0x0050C2, 0x4BB000, 0x4BBFFF, "Protonic Holland", "PROTONIC-HOLLAND" },
	{ 0x0050C2, 0x4BC000, 0x4BCFFF, "Saia Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x4BD000, 0x4BDFFF, "Argon ST", "ARGON-ST" },
	{ 0x0050C2, 0x4BE000, 0x4BEFFF, "Digital Dynamics, Inc.", "DIGITAL-DYNAMICS" },
	{ 0x0050C2, 0x4BF000, 0x4BFFFF, "Westinghouse Rail Systems Ltd", "WESTINGHOUSE-RAIL" },
	{ 0x0050C2, 0x4C0000, 0x4C0FFF, "Bio-logic Systems Corp", "BIO-LOGIC" },
	{ 0x0050C2, 0x4C1000, 0x4C1FFF, "Movaz Networks, Inc.", "MOVAZ-NETWORKS" },
	{ 0x0050C2, 0x4C2000, 0x4C2FFF, "Elbit Systems", "ELBIT" },
	{ 0x0050C2, 0x4C3000, 0x4C3FFF, "Quantum3D, Inc.", "QUANTUM3D" },
	{ 0x0050C2, 0x4C4000, 0x4C4FFF, "Black Diamond Video, Inc.", "BLACK-DIAMOND-VIDEO" },
	{ 0x0050C2, 0x4C5000, 0x4C5FFF, "eXray Broadband Inc.", "EXRAY-BROADBAND" },
	{ 0x0050C2, 0x4C6000, 0x4C6FFF, "Rubin Ltd.", "RUBIN" },
	{ 0x0050C2, 0x4C7000, 0x4C7FFF, "Transbit Sp.z o.o.", "TRANSBIT-SP-Z-O-O" },
	{ 0x0050C2, 0x4C8000, 0x4C8FFF, "Neets", "NEETS" },
	{ 0x0050C2, 0x4C9000, 0x4C9FFF, "Scirocco AB", "SCIROCCO-AB" },
	{ 0x0050C2, 0x4CA000, 0x4CAFFF, "Yarg Biometrics Limited", "YARG-BIOMETRICS" },
	{ 0x0050C2, 0x4CB000, 0x4CBFFF, "Verint Systems Ltd", "VERINT" },
	{ 0x0050C2, 0x4CC000, 0x4CCFFF, "ImpediMed Limited", "IMPEDIMED" },
	{ 0x0050C2, 0x4CD000, 0x4CDFFF, "Bixi Systems AG", "BIXI" },
	{ 0x0050C2, 0x4CE000, 0x4CEFFF, "Open Date Equipment Limited", "OPEN-DATE-EQUIPMENT" },
	{ 0x0050C2, 0x4CF000, 0x4CFFFF, "Ziehl-Abegg AG", "ZIEHL-ABEGG" },
	{ 0x0050C2, 0x4D0000, 0x4D0FFF, "RCS Energy Management Ltd", "RCS-ENERGY-MANAGEMENT" },
	{ 0x0050C2, 0x4D1000, 0x4D1FFF, "SLICAN sp. z o.o.", "SLICAN-SP-Z-O-O" },
	{ 0x0050C2, 0x4D2000, 0x4D2FFF, "Twoway CATV SERVICE INC.", "TWOWAY-CATV-SERVICE" },
	{ 0x0050C2, 0x4D3000, 0x4D3FFF, "ELPROC sp. z o.o.", "ELPROC-SP-Z-O-O" },
	{ 0x0050C2, 0x4D4000, 0x4D4FFF, "Herholdt Controls srl", "HERHOLDT-CONTROLS-SRL" },
	{ 0x0050C2, 0x4D5000, 0x4D5FFF, "SEBA Design Pty Ltd", "SEBA-DESIGN-PTY" },
	{ 0x0050C2, 0x4D6000, 0x4D6FFF, "Ingenieurbüro Schober", "INGENIEURBUERO-SCHOBER" },
	{ 0x0050C2, 0x4D7000, 0x4D7FFF, "Delta Tau Data Systems, Inc.", "DELTA-TAU-DATA" },
	{ 0x0050C2, 0x4D8000, 0x4D8FFF, "Avantry Ltd.", "AVANTRY" },
	{ 0x0050C2, 0x4D9000, 0x4D9FFF, "GE Security Kampro", "GE-SECURITY-KAMPRO" },
	{ 0x0050C2, 0x4DA000, 0x4DAFFF, "MEDIORNET GmbH", "MEDIORNET" },
	{ 0x0050C2, 0x4DB000, 0x4DBFFF, "Alfing Montagetechnik GmbH", "ALFING-MONTAGETECHNIK" },
	{ 0x0050C2, 0x4DC000, 0x4DCFFF, "Ace Electronics Inc.", "ACE-ELECTRONICS" },
	{ 0x0050C2, 0x4DD000, 0x4DDFFF, "Truteq Wireless (PTY) Ltd.", "TRUTEQ-WIRELESS-PTY" },
	{ 0x0050C2, 0x4DE000, 0x4DEFFF, "General Dynamics C4 Systems", "GENERAL-DYNAMICS-C4" },
	{ 0x0050C2, 0x4DF000, 0x4DFFFF, "Thermo Electron", "THERMO-ELECTRON" },
	{ 0x0050C2, 0x4E0000, 0x4E0FFF, "Telematrix", "TELEMATRIX" },
	{ 0x0050C2, 0x4E1000, 0x4E1FFF, "SS Telecoms CC", "SS-TELECOMS-CC" },
	{ 0x0050C2, 0x4E2000, 0x4E2FFF, "Applied Research Laboratories: UT", "APPLIED-RESEARCH-LABORATORIES:-UT" },
	{ 0x0050C2, 0x4E3000, 0x4E3FFF, "Romteck Pty Ltd", "ROMTECK-PTY" },
	{ 0x0050C2, 0x4E4000, 0x4E4FFF, "Embigence GmbH", "EMBIGENCE" },
	{ 0x0050C2, 0x4E5000, 0x4E5FFF, "Sedo Systems Ltd", "SEDO" },
	{ 0x0050C2, 0x4E6000, 0x4E6FFF, "Photonic Bridges Inc.", "PHOTONIC-BRIDGES" },
	{ 0x0050C2, 0x4E7000, 0x4E7FFF, "Computerized Elevator Contol", "COMPUTERIZED-ELEVATOR-CONTOL" },
	{ 0x0050C2, 0x4E8000, 0x4E8FFF, "SATEL sp. z o.o.", "SATEL-SP-Z-O-O" },
	{ 0x0050C2, 0x4E9000, 0x4E9FFF, "Seachange international", "SEACHANGE" },
	{ 0x0050C2, 0x4EA000, 0x4EAFFF, "PMC", "PMC" },
	{ 0x0050C2, 0x4EB000, 0x4EBFFF, "Mandozzi Elettronica SA", "MANDOZZI-ELETTRONICA" },
	{ 0x0050C2, 0x4EC000, 0x4ECFFF, "Thales Defence and Security Systems GmbH", "THALES-DEFENCE-AND-SECURITY" },
	{ 0x0050C2, 0x4ED000, 0x4EDFFF, "Lab X Technologies, LLC", "LAB-X-TECHNOLOGIES-LLC" },
	{ 0x0050C2, 0x4EE000, 0x4EEFFF, "Beijing Corelogic Communication Co., Ltd.", "BEIJING-CORELOGIC-COMMUNICATION" },
	{ 0x0050C2, 0x4EF000, 0x4EFFFF, "Creative Retail Entertainment", "CREATIVE-RETAIL-ENTERTAINMENT" },
	{ 0x0050C2, 0x4F0000, 0x4F0FFF, "MedAvant Healthcare", "MEDAVANT-HEALTHCARE" },
	{ 0x0050C2, 0x4F1000, 0x4F1FFF, "Packet Island Inc.", "PACKET-ISLAND" },
	{ 0x0050C2, 0x4F2000, 0x4F2FFF, "Tantronic AG", "TANTRONIC" },
	{ 0x0050C2, 0x4F3000, 0x4F3FFF, "Autronica Fire & Security", "AUTRONICA-FIRE-SECURITY" },
	{ 0x0050C2, 0x4F4000, 0x4F4FFF, "O2RUN", "O2RUN" },
	{ 0x0050C2, 0x4F5000, 0x4F5FFF, "Monroe Electronics, Inc.", "MONROE-ELECTRONICS" },
	{ 0x0050C2, 0x4F6000, 0x4F6FFF, "REAL D", "REAL-D" },
	{ 0x0050C2, 0x4F7000, 0x4F7FFF, "WaveIP Ltd.", "WAVEIP" },
	{ 0x0050C2, 0x4F8000, 0x4F8FFF, "Prodco International Inc.", "PRODCO" },
	{ 0x0050C2, 0x4F9000, 0x4F9FFF, "RTDS Technologies Inc.", "RTDS-TECHNOLOGIES" },
	{ 0x0050C2, 0x4FA000, 0x4FAFFF, "Cambridge Technology, Inc.", "CAMBRIDGE" },
	{ 0x0050C2, 0x4FB000, 0x4FBFFF, "BES Technology Group", "BES-GROUP" },
	{ 0x0050C2, 0x4FC000, 0x4FCFFF, "Hwayoung RF Solution Inc", "HWAYOUNG-RF-SOLUTION" },
	{ 0x0050C2, 0x4FD000, 0x4FDFFF, "Network Automation mxc AB", "NETWORK-AUTOMATION-MXC-AB" },
	{ 0x0050C2, 0x4FE000, 0x4FEFFF, "GEM ELETTRONICA Srl", "GEM-ELETTRONICA-SRL" },
	{ 0x0050C2, 0x4FF000, 0x4FFFFF, "Dakty GmbH", "DAKTY" },
	{ 0x0050C2, 0x500000, 0x500FFF, "Orenco Systems, Inc.", "ORENCO" },
	{ 0x0050C2, 0x501000, 0x501FFF, "IBEX UK Limited", "IBEX-UK" },
	{ 0x0050C2, 0x502000, 0x502FFF, "Criterion Systems Limited", "CRITERION" },
	{ 0x0050C2, 0x503000, 0x503FFF, "RESPIRONICS INC.", "RESPIRONICS" },
	{ 0x0050C2, 0x504000, 0x504FFF, "Aphex Systems Ltd.", "APHEX" },
	{ 0x0050C2, 0x505000, 0x505FFF, "Computerwise, Inc.", "COMPUTERWISE" },
	{ 0x0050C2, 0x506000, 0x506FFF, "7+ Kft", "7+-KFT" },
	{ 0x0050C2, 0x507000, 0x507FFF, "Micro Connect Pty Ltd", "MICRO-CONNECT-PTY" },
	{ 0x0050C2, 0x508000, 0x508FFF, "PUTERCOM ENTERPRISE CO., LTD.", "PUTERCOM" },
	{ 0x0050C2, 0x509000, 0x509FFF, "Hillcrest Laboratories, Inc.", "HILLCREST-LABORATORIES" },
	{ 0x0050C2, 0x50A000, 0x50AFFF, "Monitor Business Machines Ltd", "MONITOR-BUSINESS-MACHINES" },
	{ 0x0050C2, 0x50B000, 0x50BFFF, "Logic Beach Inc", "LOGIC-BEACH" },
	{ 0x0050C2, 0x50C000, 0x50CFFF, "AIRWISE TECHNOLOGY CO., LTD.", "AIRWISE" },
	{ 0x0050C2, 0x50D000, 0x50DFFF, "Clearsonics Pty. Ltd.", "CLEARSONICS-PTY" },
	{ 0x0050C2, 0x50E000, 0x50EFFF, "Fibresavers Corporation", "FIBRESAVERS" },
	{ 0x0050C2, 0x50F000, 0x50FFFF, "Polystar Instruments AB", "POLYSTAR-INSTRUMENTS-AB" },
	{ 0x0050C2, 0x510000, 0x510FFF, "Summit Developmen", "SUMMIT-DEVELOPMEN" },
	{ 0x0050C2, 0x511000, 0x511FFF, "Tecna Srl", "TECNA-SRL" },
	{ 0x0050C2, 0x512000, 0x512FFF, "Linear Acoustic, Inc", "LINEAR-ACOUSTIC" },
	{ 0x0050C2, 0x513000, 0x513FFF, "Genie Network Resource Management Inc.", "GENIE-NETWORK-RESOURCE-MANAGEMENT" },
	{ 0x0050C2, 0x514000, 0x514FFF, "Tadian Electronics Systems LTD", "TADIAN-ELECTRONICS" },
	{ 0x0050C2, 0x515000, 0x515FFF, "Monaghan Engineering, Inc.", "MONAGHAN-ENGINEERING" },
	{ 0x0050C2, 0x516000, 0x516FFF, "SOWA ELECTRIC CO., LTD.", "SOWA-ELECTRIC" },
	{ 0x0050C2, 0x517000, 0x517FFF, "Solid State Logic", "SOLID-STATE-LOGIC" },
	{ 0x0050C2, 0x518000, 0x518FFF, "Christ Elektronik GmbH", "CHRIST-ELEKTRONIK" },
	{ 0x0050C2, 0x519000, 0x519FFF, "DBMCORP, Inc.", "DBMCORP" },
	{ 0x0050C2, 0x51A000, 0x51AFFF, "SpeasTech, Inc.", "SPEASTECH" },
	{ 0x0050C2, 0x51B000, 0x51BFFF, "Beta Lasermike Ltd", "BETA-LASERMIKE" },
	{ 0x0050C2, 0x51C000, 0x51CFFF, "TOA Systems", "TOA" },
	{ 0x0050C2, 0x51D000, 0x51DFFF, "VELUX", "VELUX" },
	{ 0x0050C2, 0x51E000, 0x51EFFF, "Alcon Technologies", "ALCON-TECHNOLOGIES" },
	{ 0x0050C2, 0x51F000, 0x51FFFF, "Traquair Data Systems, Inc.", "TRAQUAIR-DATA" },
	{ 0x0050C2, 0x520000, 0x520FFF, "McCain Traffic Supply", "MCCAIN-TRAFFIC-SUPPLY" },
	{ 0x0050C2, 0x521000, 0x521FFF, "ARIS TECHNOLOGIES", "ARIS-TECHNOLOGIES" },
	{ 0x0050C2, 0x522000, 0x522FFF, "Mark IV IDS Corp.", "MARK-IV-IDS" },
	{ 0x0050C2, 0x523000, 0x523FFF, "AMRDEC Prototype Integration Facility", "AMRDEC-PROTOTYPE-INTEGRATION-FACILITY" },
	{ 0x0050C2, 0x524000, 0x524FFF, "Motec Pty Ltd", "MOTEC-PTY" },
	{ 0x0050C2, 0x525000, 0x525FFF, "VASTech", "VASTECH" },
	{ 0x0050C2, 0x526000, 0x526FFF, "AC SYSTEMS, s.r.o.", "AC-S-R-O" },
	{ 0x0050C2, 0x527000, 0x527FFF, "IRTrans", "IRTRANS" },
	{ 0x0050C2, 0x528000, 0x528FFF, "tattile srl", "TATTILE-SRL" },
	{ 0x0050C2, 0x529000, 0x529FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x52A000, 0x52AFFF, "OMNITRONICS PTY LTD", "OMNITRONICS-PTY" },
	{ 0x0050C2, 0x52B000, 0x52BFFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0x52C000, 0x52CFFF, "VITEC MULTIMEDIA", "VITEC-MULTIMEDIA" },
	{ 0x0050C2, 0x52D000, 0x52DFFF, "Smartcom-Bulgaria AD", "SMARTCOM-BULGARIA-AD" },
	{ 0x0050C2, 0x52E000, 0x52EFFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x0050C2, 0x52F000, 0x52FFFF, "Gesellschaft für Rationalisierung und Rechentechnik mbH", "GESELLSCHAFT-FUER-RATIONALISIERUNG-UND-RECHENTECHNIK-MBH" },
	{ 0x0050C2, 0x530000, 0x530FFF, "Innovation, Institute, Inc", "INNOVATION-INSTITUTE" },
	{ 0x0050C2, 0x531000, 0x531FFF, "Orion Technologies, Incorporated", "ORION-TECHNOLOGIES-INCORPORATED" },
	{ 0x0050C2, 0x532000, 0x532FFF, "NVE Corporation", "NVE" },
	{ 0x0050C2, 0x533000, 0x533FFF, "NanShanBridge Co.Ltd", "NANSHANBRIDGE" },
	{ 0x0050C2, 0x534000, 0x534FFF, "Hyundai J. Comm", "HYUNDAI-J-COMM" },
	{ 0x0050C2, 0x535000, 0x535FFF, "MMS Servis s.r.o.", "MMS-SERVIS-S-R-O" },
	{ 0x0050C2, 0x536000, 0x536FFF, "C2 DIAGNOSTICS", "C2-DIAGNOSTICS" },
	{ 0x0050C2, 0x537000, 0x537FFF, "DST CONTROL AB", "DST-CONTROL-AB" },
	{ 0x0050C2, 0x538000, 0x538FFF, "EtherTek Circuits", "ETHERTEK-CIRCUITS" },
	{ 0x0050C2, 0x539000, 0x539FFF, "Detection Technology Inc.", "DETECTION" },
	{ 0x0050C2, 0x53A000, 0x53AFFF, "Image Control Design Limited", "IMAGE-CONTROL-DESIGN" },
	{ 0x0050C2, 0x53B000, 0x53BFFF, "Teleks Co. Ltd.", "TELEKS" },
	{ 0x0050C2, 0x53C000, 0x53CFFF, "Marposs SPA", "MARPOSS-SPA" },
	{ 0x0050C2, 0x53D000, 0x53DFFF, "Digital communications Technologies", "DIGITAL-COMMUNICATION-TECHNOLOGIES" },
	{ 0x0050C2, 0x53E000, 0x53EFFF, "Honeywell GNO", "HONEYWELL-GNO" },
	{ 0x0050C2, 0x53F000, 0x53FFFF, "Ellips B.V.", "ELLIPS" },
	{ 0x0050C2, 0x540000, 0x540FFF, "Mesure Controle Commande", "MESURE-CONTROLE-COMMANDE" },
	{ 0x0050C2, 0x541000, 0x541FFF, "WAVES SYSTEM", "WAVES-SYSTEM" },
	{ 0x0050C2, 0x542000, 0x542FFF, "AVerMedia Technologies, Inc.", "AVERMEDIA-TECHNOLOGIES" },
	{ 0x0050C2, 0x543000, 0x543FFF, "DIGI SESN AG", "DIGI-SESN" },
	{ 0x0050C2, 0x544000, 0x544FFF, "Zetera", "ZETERA" },
	{ 0x0050C2, 0x545000, 0x545FFF, "SecuInfo Co., Ltd.", "SECUINFO" },
	{ 0x0050C2, 0x546000, 0x546FFF, "Universidad de Chile Facultad de Medicina", "UNIVERSIDAD-DE-CHILE-FACULTAD-DE-MEDICINA" },
	{ 0x0050C2, 0x547000, 0x547FFF, "BLANKOM Antennentechnik GmbH", "BLANKOM-ANTENNENTECHNIK" },
	{ 0x0050C2, 0x548000, 0x548FFF, "I.T.W. Betaprint", "I-T-W-BETAPRINT" },
	{ 0x0050C2, 0x549000, 0x549FFF, "Netsynt S.p.A.", "NETSYNT-S-P-A" },
	{ 0x0050C2, 0x54A000, 0x54AFFF, "IPTC Tech. Comm. AB", "IPTC-TECH-COMM-AB" },
	{ 0x0050C2, 0x54B000, 0x54BFFF, "Innopsys", "INNOPSYS" },
	{ 0x0050C2, 0x54C000, 0x54CFFF, "Sintecnos srl", "SINTECNOS-SRL" },
	{ 0x0050C2, 0x54D000, 0x54DFFF, "Silent System", "SILENT-SYSTEM" },
	{ 0x0050C2, 0x54E000, 0x54EFFF, "Convergent Design", "CONVERGENT-DESIGN" },
	{ 0x0050C2, 0x54F000, 0x54FFFF, "Valtronic SA", "VALTRONIC" },
	{ 0x0050C2, 0x550000, 0x550FFF, "LJU Automatisierungstechnik GmbH", "LJU-AUTOMATISIERUNGSTECHNIK" },
	{ 0x0050C2, 0x551000, 0x551FFF, "Innovative Neurotroncs", "INNOVATIVE-NEUROTRONCS" },
	{ 0x0050C2, 0x552000, 0x552FFF, "Elfiq Inc.", "ELFIQ" },
	{ 0x0050C2, 0x553000, 0x553FFF, "ATH system", "ATH-SYSTEM" },
	{ 0x0050C2, 0x554000, 0x554FFF, "Weinzierl Engineering GmbH", "WEINZIERL-ENGINEERING" },
	{ 0x0050C2, 0x555000, 0x555FFF, "Control Alternative Solutions, Inc.", "CONTROL-ALTERNATIVE-SOLUTIONS" },
	{ 0x0050C2, 0x556000, 0x556FFF, "Freiburger BlickZentrum", "FREIBURGER-BLICKZENTRUM" },
	{ 0x0050C2, 0x557000, 0x557FFF, "Netcomsec Co Ltd", "NETCOMSEC" },
	{ 0x0050C2, 0x558000, 0x558FFF, "Bedo Elektronik GmbH", "BEDO-ELEKTRONIK" },
	{ 0x0050C2, 0x559000, 0x559FFF, "Fail Safe Solutions LLC", "FAIL-SAFE-SOLUTIONS-LLC" },
	{ 0x0050C2, 0x55A000, 0x55AFFF, "Valde Systems, Inc.", "VALDE" },
	{ 0x0050C2, 0x55B000, 0x55BFFF, "MATRIX TELECOM PVT. LTD.", "MATRIX-TELECOM-PVT" },
	{ 0x0050C2, 0x55C000, 0x55CFFF, "ads-tec GmbH", "ADS-TEC" },
	{ 0x0050C2, 0x55D000, 0x55DFFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0x55E000, 0x55EFFF, "HANZAS ELEKTRONIKA, SIA", "HANZAS-ELEKTRONIKA-SIA" },
	{ 0x0050C2, 0x55F000, 0x55FFFF, "Moog Broad Reach", "MOOG-BROAD-REACH" },
	{ 0x0050C2, 0x560000, 0x560FFF, "Procon Electronics", "PROCON-ELECTRONICS" },
	{ 0x0050C2, 0x561000, 0x561FFF, "Seitec Elektronik GmbH", "SEITEC-ELEKTRONIK" },
	{ 0x0050C2, 0x562000, 0x562FFF, "C21 Systems Limited", "C21" },
	{ 0x0050C2, 0x563000, 0x563FFF, "ORTRAT, S.L.", "ORTRAT-S-L" },
	{ 0x0050C2, 0x564000, 0x564FFF, "Last Mile Gear", "LAST-MILE-GEAR" },
	{ 0x0050C2, 0x565000, 0x565FFF, "WORKPOWER TECNOLOGIA ELETRONICA LTDA-EPP", "WORKPOWER-TECNOLOGIA-ELETRONICA-LTDA-EPP" },
	{ 0x0050C2, 0x566000, 0x566FFF, "ubinetsys.co..ltd", "UBINETSYS" },
	{ 0x0050C2, 0x567000, 0x567FFF, "Tess GmbH", "TESS" },
	{ 0x0050C2, 0x568000, 0x568FFF, "GeoFocus, LLC", "GEOFOCUS-LLC" },
	{ 0x0050C2, 0x569000, 0x569FFF, "Twinwin Technplogy Co.,Ltd.", "TWINWIN-TECHNPLOGY" },
	{ 0x0050C2, 0x56A000, 0x56AFFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0x56B000, 0x56BFFF, "Dataton Utvecklings AB", "DATATON-UTVECKLINGS-AB" },
	{ 0x0050C2, 0x56C000, 0x56CFFF, "Targeted Technologies, LLC", "TARGETED-TECHNOLOGIES-LLC" },
	{ 0x0050C2, 0x56D000, 0x56DFFF, "Computrol Fuel Systems Inc.", "COMPUTROL-FUEL" },
	{ 0x0050C2, 0x56E000, 0x56EFFF, "LAB-EL ELEKTRONIKA LABORATORYJNA S.J.", "LAB-EL-ELEKTRONIKA-LABORATORYJNA-S-J" },
	{ 0x0050C2, 0x56F000, 0x56FFFF, "GMA, LLC", "GMA-LLC" },
	{ 0x0050C2, 0x570000, 0x570FFF, "Ellex Medical Pty Ltd", "ELLEX-MEDICAL-PTY" },
	{ 0x0050C2, 0x571000, 0x571FFF, "Oberon Service srl", "OBERON-SERVICE-SRL" },
	{ 0x0050C2, 0x572000, 0x572FFF, "Chell Instruments Ltd", "CHELL-INSTRUMENTS" },
	{ 0x0050C2, 0x573000, 0x573FFF, "DATAMICRO Co., Ltd.", "DATAMICRO" },
	{ 0x0050C2, 0x574000, 0x574FFF, "Ingeniería AlmudÃ­ S.L.", "INGENIERíA-ALMUDÃ­-S-L" },
	{ 0x0050C2, 0x575000, 0x575FFF, "SOLYSTIC", "SOLYSTIC" },
	{ 0x0050C2, 0x576000, 0x576FFF, "Visi-tech Systems Ltd", "VISI-TECH" },
	{ 0x0050C2, 0x577000, 0x577FFF, "Advanced Software Technologies", "ADVANCED-SOFTWARE-TECHNOLOGIES" },
	{ 0x0050C2, 0x578000, 0x578FFF, "Delphi Display Systems, Inc.", "DELPHI-DISPLAY" },
	{ 0x0050C2, 0x579000, 0x579FFF, "Gastager Systemtechnik GmbH", "GASTAGER-SYSTEMTECHNIK" },
	{ 0x0050C2, 0x57A000, 0x57AFFF, "Pigeon Point Systems LLC", "PIGEON-POINT-LLC" },
	{ 0x0050C2, 0x57B000, 0x57BFFF, "ptswitch", "PTSWITCH" },
	{ 0x0050C2, 0x57C000, 0x57CFFF, "éolane", "éOLANE" },
	{ 0x0050C2, 0x57D000, 0x57DFFF, "Sierra Video Systems", "SIERRA-VIDEO" },
	{ 0x0050C2, 0x57E000, 0x57EFFF, "Digital Way", "DIGITAL-WAY" },
	{ 0x0050C2, 0x57F000, 0x57FFFF, "Orderite, Inc.", "ORDERITE" },
	{ 0x0050C2, 0x580000, 0x580FFF, "Buyang Electronics Industrial co.,Ltd.", "BUYANG-ELECTRONICS-INDUSTRIAL" },
	{ 0x0050C2, 0x581000, 0x581FFF, "Devitech ApS", "DEVITECH-APS" },
	{ 0x0050C2, 0x582000, 0x582FFF, "AllSun A/S", "ALLSUN-A/S" },
	{ 0x0050C2, 0x583000, 0x583FFF, "Jünger Audio-Studiotechnik GmbH", "JUENGER-AUDIO-STUDIOTECHNIK" },
	{ 0x0050C2, 0x584000, 0x584FFF, "Toyota Motorsport GmbH", "TOYOTA-MOTORSPORT" },
	{ 0x0050C2, 0x585000, 0x585FFF, "Wireless Cables Inc", "WIRELESS-CABLES" },
	{ 0x0050C2, 0x586000, 0x586FFF, "Genetix Ltd", "GENETIX" },
	{ 0x0050C2, 0x587000, 0x587FFF, "Dynalco", "DYNALCO" },
	{ 0x0050C2, 0x588000, 0x588FFF, "Federal Electronics", "FEDERAL-ELECTRONICS" },
	{ 0x0050C2, 0x589000, 0x589FFF, "HORIBA MEDICAL", "HORIBA-MEDICAL" },
	{ 0x0050C2, 0x58A000, 0x58AFFF, "Dixell S.p.a.", "DIXELL-S-P-A" },
	{ 0x0050C2, 0x58B000, 0x58BFFF, "Innovative Dynamics GmbH", "INNOVATIVE-DYNAMICS" },
	{ 0x0050C2, 0x58C000, 0x58CFFF, "Lattice Semiconductor Corp. (LPA)", "LATTICE-SEMICONDUCTOR-LPA" },
	{ 0x0050C2, 0x58D000, 0x58DFFF, "ZAO", "ZAO" },
	{ 0x0050C2, 0x58E000, 0x58EFFF, "Penny & Giles Aerospace Ltd", "PENNY-GILES-AEROSPACE" },
	{ 0x0050C2, 0x58F000, 0x58FFFF, "XoIP Systems Pty Ltd", "XOIP-PTY" },
	{ 0x0050C2, 0x590000, 0x590FFF, "EM Motorsport Ltd", "EM-MOTORSPORT" },
	{ 0x0050C2, 0x591000, 0x591FFF, "Grosvenor Technology Ltd", "GROSVENOR" },
	{ 0x0050C2, 0x592000, 0x592FFF, "PaloDEx Group Oy", "PALODEX-GROUP-OY" },
	{ 0x0050C2, 0x593000, 0x593FFF, "Broadlight", "BROADLIGHT" },
	{ 0x0050C2, 0x594000, 0x594FFF, "Pixel Velocity, Inc.", "PIXEL-VELOCITY" },
	{ 0x0050C2, 0x595000, 0x595FFF, "Callpod, Inc.", "CALLPOD" },
	{ 0x0050C2, 0x596000, 0x596FFF, "SPANSION", "SPANSION" },
	{ 0x0050C2, 0x597000, 0x597FFF, "Nautel Limited", "NAUTEL" },
	{ 0x0050C2, 0x598000, 0x598FFF, "Bundesamt für Strahlenschutz", "BUNDESAMT-FUER-STRAHLENSCHUTZ" },
	{ 0x0050C2, 0x599000, 0x599FFF, "Fen Technology Limited", "FEN" },
	{ 0x0050C2, 0x59A000, 0x59AFFF, "MultiTrode Pty Ltd", "MULTITRODE-PTY" },
	{ 0x0050C2, 0x59B000, 0x59BFFF, "SAPEC", "SAPEC" },
	{ 0x0050C2, 0x59C000, 0x59CFFF, "DELSAT GROUP S.A.", "DELSAT-GROUP" },
	{ 0x0050C2, 0x59D000, 0x59DFFF, "DSS Networks, Inc.", "DSS-NETWORKS" },
	{ 0x0050C2, 0x59E000, 0x59EFFF, "Legerity", "LEGERITY" },
	{ 0x0050C2, 0x59F000, 0x59FFFF, "ads-tec GmbH", "ADS-TEC" },
	{ 0x0050C2, 0x5A0000, 0x5A0FFF, "Rudolph Technologies, Inc.", "RUDOLPH-TECHNOLOGIES" },
	{ 0x0050C2, 0x5A1000, 0x5A1FFF, "Vestfold Butikkdata AS", "VESTFOLD-BUTIKKDATA-AS" },
	{ 0x0050C2, 0x5A2000, 0x5A2FFF, "iNET Systems Inc.", "INET" },
	{ 0x0050C2, 0x5A3000, 0x5A3FFF, "LUMEL S.A.", "LUMEL" },
	{ 0x0050C2, 0x5A4000, 0x5A4FFF, "Federal State Unitary Enterprise Experimental Factory for Sc", "FEDERAL-STATE-UNITARY-EXPERIMENTAL-FACTORY-FOR-SC" },
	{ 0x0050C2, 0x5A5000, 0x5A5FFF, "Equipos de Telecomunicación Optoelectronicos, S.A.", "EQUIPOS-DE-TELECOMUNICACIóN-OPTOELECTRONICOS" },
	{ 0x0050C2, 0x5A6000, 0x5A6FFF, "Plastic Logic", "PLASTIC-LOGIC" },
	{ 0x0050C2, 0x5A7000, 0x5A7FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x5A8000, 0x5A8FFF, "ETAP NV", "ETAP-NV" },
	{ 0x0050C2, 0x5A9000, 0x5A9FFF, "AYC Telecom Ltd", "AYC-TELECOM" },
	{ 0x0050C2, 0x5AA000, 0x5AAFFF, "Transenna AB", "TRANSENNA-AB" },
	{ 0x0050C2, 0x5AB000, 0x5ABFFF, "Eaton Corporation Electrical Group Data Center Solutions - Pulizzi", "EATON-ELECTRICAL-GROUP-DATA-CENTER-SOLUTIONS---PULIZZI" },
	{ 0x0050C2, 0x5AC000, 0x5ACFFF, "Kinemetrics, Inc.", "KINEMETRICS" },
	{ 0x0050C2, 0x5AD000, 0x5ADFFF, "Emcom Systems", "EMCOM" },
	{ 0x0050C2, 0x5AE000, 0x5AEFFF, "CPS EUROPE B.V.", "CPS-EUROPE" },
	{ 0x0050C2, 0x5AF000, 0x5AFFFF, "DORLET S.A.", "DORLET" },
	{ 0x0050C2, 0x5B0000, 0x5B0FFF, "INCOTEC GmbH", "INCOTEC" },
	{ 0x0050C2, 0x5B1000, 0x5B1FFF, "Rosta Ltd", "ROSTA" },
	{ 0x0050C2, 0x5B2000, 0x5B2FFF, "Syntronic AB", "SYNTRONIC-AB" },
	{ 0x0050C2, 0x5B3000, 0x5B3FFF, "HITECOM System", "HITECOM-SYSTEM" },
	{ 0x0050C2, 0x5B4000, 0x5B4FFF, "Terrascience Systems Ltd.", "TERRASCIENCE" },
	{ 0x0050C2, 0x5B5000, 0x5B5FFF, "RAFAEL", "RAFAEL" },
	{ 0x0050C2, 0x5B6000, 0x5B6FFF, "Kontron (Beijing) Technology Co.,Ltd.", "KONTRON-BEIJING" },
	{ 0x0050C2, 0x5B7000, 0x5B7FFF, "AVerMedia Technologies, Inc.", "AVERMEDIA-TECHNOLOGIES" },
	{ 0x0050C2, 0x5B8000, 0x5B8FFF, "WestfaliaSurge GmbH", "WESTFALIASURGE" },
	{ 0x0050C2, 0x5B9000, 0x5B9FFF, "Taiwan Video & Monitor", "TAIWAN-VIDEO-MONITOR" },
	{ 0x0050C2, 0x5BA000, 0x5BAFFF, "SAIA Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x5BB000, 0x5BBFFF, "UNIC TECHNOLOGIES INC", "UNIC-TECHNOLOGIES" },
	{ 0x0050C2, 0x5BC000, 0x5BCFFF, "Guangzhou Hui Si Information Technologies Inc.", "GUANGZHOU-HUI-SI-INFORMATION-TECHNOLOGIES" },
	{ 0x0050C2, 0x5BD000, 0x5BDFFF, "Nomus Comm-Systems", "NOMUS-COMM" },
	{ 0x0050C2, 0x5BE000, 0x5BEFFF, "Card Access Services Pty Ltd", "CARD-ACCESS-SERVICES-PTY" },
	{ 0x0050C2, 0x5BF000, 0x5BFFFF, "Techimp Systems S.r.l.", "TECHIMP-S-R-L" },
	{ 0x0050C2, 0x5C0000, 0x5C0FFF, "Pyott-Boone Electronics", "PYOTT-BOONE-ELECTRONICS" },
	{ 0x0050C2, 0x5C1000, 0x5C1FFF, "R. L. Drake Company", "R-L-DRAKE-COMPANY" },
	{ 0x0050C2, 0x5C2000, 0x5C2FFF, "Intuitive Surgical", "INTUITIVE-SURGICAL" },
	{ 0x0050C2, 0x5C3000, 0x5C3FFF, "KS System GmbH", "KS-SYSTEM" },
	{ 0x0050C2, 0x5C4000, 0x5C4FFF, "ProMik GmbH", "PROMIK" },
	{ 0x0050C2, 0x5C5000, 0x5C5FFF, "Radiant Imaging, Inc.", "RADIANT-IMAGING" },
	{ 0x0050C2, 0x5C6000, 0x5C6FFF, "Technische Alternative GmbH", "TECHNISCHE-ALTERNATIVE" },
	{ 0x0050C2, 0x5C7000, 0x5C7FFF, "InSync Technology Ltd", "INSYNC" },
	{ 0x0050C2, 0x5C8000, 0x5C8FFF, "Georgia Tech Research Institute", "GEORGIA-TECH-RESEARCH-INSTITUTE" },
	{ 0x0050C2, 0x5C9000, 0x5C9FFF, "Shenzhen Quanlong Technique Co.Ltd", "SHENZHEN-QUANLONG-TECHNIQUE" },
	{ 0x0050C2, 0x5CA000, 0x5CAFFF, "Buyang Electronics Industrial Co., Ltd.", "BUYANG-ELECTRONICS-INDUSTRIAL" },
	{ 0x0050C2, 0x5CB000, 0x5CBFFF, "Kobold Sistemi s.r.l.", "KOBOLD-SISTEMI-S-R-L" },
	{ 0x0050C2, 0x5CC000, 0x5CCFFF, "ENSEO", "ENSEO" },
	{ 0x0050C2, 0x5CD000, 0x5CDFFF, "RADA Electronics Industries Ltd.", "RADA-ELECTRONICS-INDUSTRIES" },
	{ 0x0050C2, 0x5CE000, 0x5CEFFF, "Roke Manor Research Ltd", "ROKE-MANOR-RESEARCH" },
	{ 0x0050C2, 0x5CF000, 0x5CFFFF, "Innomed Medical Inc", "INNOMED-MEDICAL" },
	{ 0x0050C2, 0x5D0000, 0x5D0FFF, "Automata Spa", "AUTOMATA-SPA" },
	{ 0x0050C2, 0x5D1000, 0x5D1FFF, "Meucci Solutions", "MEUCCI-SOLUTIONS" },
	{ 0x0050C2, 0x5D2000, 0x5D2FFF, "DA-Design Oy", "DA-DESIGN-OY" },
	{ 0x0050C2, 0x5D3000, 0x5D3FFF, "Wexiodisk AB", "WEXIODISK-AB" },
	{ 0x0050C2, 0x5D4000, 0x5D4FFF, "Buyang Electronics Industrial Co., Ltd.", "BUYANG-ELECTRONICS-INDUSTRIAL" },
	{ 0x0050C2, 0x5D5000, 0x5D5FFF, "Cannon Technologies", "CANNON-TECHNOLOGIES" },
	{ 0x0050C2, 0x5D6000, 0x5D6FFF, "BioAccess Tecnologia em Biometria Ltda.", "BIOACCESS-TECNOLOGIA-EM-BIOMETRIA-LTDA" },
	{ 0x0050C2, 0x5D7000, 0x5D7FFF, "Synrad, Inc.", "SYNRAD" },
	{ 0x0050C2, 0x5D8000, 0x5D8FFF, "TECHNIFOR SAS", "TECHNIFOR-SAS" },
	{ 0x0050C2, 0x5D9000, 0x5D9FFF, "Crimson Microsystems, Inc.", "CRIMSON-MICROSYSTEMS" },
	{ 0x0050C2, 0x5DA000, 0x5DAFFF, "TONNA ELECTRONIQUE", "TONNA-ELECTRONIQUE" },
	{ 0x0050C2, 0x5DB000, 0x5DBFFF, "CEGELEC SUD EST", "CEGELEC-SUD-EST" },
	{ 0x0050C2, 0x5DC000, 0x5DCFFF, "RM Michaelides Software & Elektronik GmbH", "RM-MICHAELIDES-SOFTWARE-ELEKTRONIK" },
	{ 0x0050C2, 0x5DD000, 0x5DDFFF, "SomerData ltd", "SOMERDATA" },
	{ 0x0050C2, 0x5DE000, 0x5DEFFF, "Magal Senstar Inc.", "MAGAL-SENSTAR" },
	{ 0x0050C2, 0x5DF000, 0x5DFFFF, "Gnutek Ltd.", "GNUTEK" },
	{ 0x0050C2, 0x5E0000, 0x5E0FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x5E1000, 0x5E1FFF, "Ittiam Systems (P) Ltd", "ITTIAM-P" },
	{ 0x0050C2, 0x5E2000, 0x5E2FFF, "PYRAMID Computer GmbH", "PYRAMID" },
	{ 0x0050C2, 0x5E3000, 0x5E3FFF, "Computechnic AG", "COMPUTECHNIC" },
	{ 0x0050C2, 0x5E4000, 0x5E4FFF, "Buyang Electronics Industrial Co., Ltd.", "BUYANG-ELECTRONICS-INDUSTRIAL" },
	{ 0x0050C2, 0x5E5000, 0x5E5FFF, "Stresstech OY", "STRESSTECH-OY" },
	{ 0x0050C2, 0x5E6000, 0x5E6FFF, "Musatel", "MUSATEL" },
	{ 0x0050C2, 0x5E7000, 0x5E7FFF, "EADS TEST & SERVICES", "EADS-TEST-SERVICES" },
	{ 0x0050C2, 0x5E8000, 0x5E8FFF, "Info-Chip Communications Ltd.", "INFO-CHIP-COMMUNICATION" },
	{ 0x0050C2, 0x5E9000, 0x5E9FFF, "Micro Technology Services Inc.", "MICRO-SERVICES" },
	{ 0x0050C2, 0x5EA000, 0x5EAFFF, "Micro Elektronische Producten", "MICRO-ELEKTRONISCHE-PRODUCTEN" },
	{ 0x0050C2, 0x5EB000, 0x5EBFFF, "Garper Telecomunicaciones, S.L.", "GARPER-TELECOMUNICACIONES-S-L" },
	{ 0x0050C2, 0x5EC000, 0x5ECFFF, "ASiS Technologies Pte Ltd", "ASIS-TECHNOLOGIES-PTE" },
	{ 0x0050C2, 0x5ED000, 0x5EDFFF, "AQUAROTTER A FRANKE COMPANY", "AQUAROTTER-A-FRANKE-COMPANY" },
	{ 0x0050C2, 0x5EE000, 0x5EEFFF, "Condre Corporation", "CONDRE" },
	{ 0x0050C2, 0x5EF000, 0x5EFFFF, "pikkerton GmbH", "PIKKERTON" },
	{ 0x0050C2, 0x5F0000, 0x5F0FFF, "DIAS Infrared GmbH", "DIAS-INFRARED" },
	{ 0x0050C2, 0x5F1000, 0x5F1FFF, "Technomarine JSC", "TECHNOMARINE-JSC" },
	{ 0x0050C2, 0x5F2000, 0x5F2FFF, "ESEM Grünau GmbH & Co. KG", "ESEM-GRUENAU" },
	{ 0x0050C2, 0x5F3000, 0x5F3FFF, "POSNET Polska S.A.", "POSNET-POLSKA" },
	{ 0x0050C2, 0x5F4000, 0x5F4FFF, "TeamProjects BV", "TEAMPROJECTS" },
	{ 0x0050C2, 0x5F5000, 0x5F5FFF, "Genesis inc", "GENESIS" },
	{ 0x0050C2, 0x5F6000, 0x5F6FFF, "CAMBRIDGE CONSULTANTS LTD", "CAMBRIDGE-CONSULTANTS" },
	{ 0x0050C2, 0x5F7000, 0x5F7FFF, "Metrologic Group", "METROLOGIC-GROUP" },
	{ 0x0050C2, 0x5F8000, 0x5F8FFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0x5F9000, 0x5F9FFF, "ROTHARY Solutions AG", "ROTHARY-SOLUTIONS" },
	{ 0x0050C2, 0x5FA000, 0x5FAFFF, "LEA d.o.o.", "LEA-D-O-O" },
	{ 0x0050C2, 0x5FB000, 0x5FBFFF, "All-Systems Electronics Pty Ltd", "ALL--ELECTRONICS-PTY" },
	{ 0x0050C2, 0x5FC000, 0x5FCFFF, "FilmLight Limited", "FILMLIGHT" },
	{ 0x0050C2, 0x5FD000, 0x5FDFFF, "MEG Electronic Inc.", "MEG" },
	{ 0x0050C2, 0x5FE000, 0x5FEFFF, "Novacomm", "NOVACOMM" },
	{ 0x0050C2, 0x5FF000, 0x5FFFFF, "Gazelle Monitoring Systems", "GAZELLE-MONITORING" },
	{ 0x0050C2, 0x600000, 0x600FFF, "Protec Fire Detection plc", "PROTEC-FIRE-DETECTION-PLC" },
	{ 0x0050C2, 0x601000, 0x601FFF, "MedAvant Healthcare", "MEDAVANT-HEALTHCARE" },
	{ 0x0050C2, 0x602000, 0x602FFF, "CHAUVIN ARNOUX", "CHAUVIN-ARNOUX" },
	{ 0x0050C2, 0x603000, 0x603FFF, "Cerus Corp", "CERUS" },
	{ 0x0050C2, 0x604000, 0x604FFF, "HCJB Global", "HCJB-GLOBAL" },
	{ 0x0050C2, 0x605000, 0x605FFF, "Swistec GmbH", "SWISTEC" },
	{ 0x0050C2, 0x606000, 0x606FFF, "Shenzhen Huazhong Technology Inc", "SHENZHEN-HUAZHONG" },
	{ 0x0050C2, 0x607000, 0x607FFF, "Telecom FM", "TELECOM-FM" },
	{ 0x0050C2, 0x608000, 0x608FFF, "Silex Industrial Automation Ltd.", "SILEX-INDUSTRIAL-AUTOMATION" },
	{ 0x0050C2, 0x609000, 0x609FFF, "Toptech Systems, Inc.", "TOPTECH" },
	{ 0x0050C2, 0x60A000, 0x60AFFF, "Gradual Tecnologia Ltda.", "GRADUAL-TECNOLOGIA-LTDA" },
	{ 0x0050C2, 0x60B000, 0x60BFFF, "Shanghai QianJin Electronic Equipment Co. Ltd.", "SHANGHAI-QIANJIN-EQUIPMENT" },
	{ 0x0050C2, 0x60C000, 0x60CFFF, "IDENTIC AB", "IDENTIC-AB" },
	{ 0x0050C2, 0x60D000, 0x60DFFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0x60E000, 0x60EFFF, "Automation and Control Technology, Inc.", "AUTOMATION-AND-CONTROL" },
	{ 0x0050C2, 0x60F000, 0x60FFFF, "Kommunikations- & Sicherheitssysteme Gesellschaft m.b.H", "KOMMUNIKATIONS--SICHERHEITSSYSTEME-GESELLSCHAFT-M-B-H" },
	{ 0x0050C2, 0x610000, 0x610FFF, "FDT Manufacturing, LLC", "FDT-MANUFACTURING-LLC" },
	{ 0x0050C2, 0x611000, 0x611FFF, "Brookhaven National Laboratory", "BROOKHAVEN-NATIONAL-LABORATORY" },
	{ 0x0050C2, 0x612000, 0x612FFF, "IHP-GmbH", "IHP" },
	{ 0x0050C2, 0x613000, 0x613FFF, "TATTILE SRL", "TATTILE-SRL" },
	{ 0x0050C2, 0x614000, 0x614FFF, "Proserv", "PROSERV" },
	{ 0x0050C2, 0x615000, 0x615FFF, "Axis Electronics", "AXIS-ELECTRONICS" },
	{ 0x0050C2, 0x616000, 0x616FFF, "Honeywell", "HONEYWELL" },
	{ 0x0050C2, 0x617000, 0x617FFF, "NARINET, INC.", "NARINET" },
	{ 0x0050C2, 0x618000, 0x618FFF, "Intergrated Security Mfg. Ltd", "INTERGRATED-SECURITY-MFG" },
	{ 0x0050C2, 0x619000, 0x619FFF, "Linkbit, Inc.", "LINKBIT" },
	{ 0x0050C2, 0x61A000, 0x61AFFF, "Communication Components Inc.", "COMMUNICATION-COMPONENTS" },
	{ 0x0050C2, 0x61B000, 0x61BFFF, "NCI Technologies Inc.", "NCI-TECHNOLOGIES" },
	{ 0x0050C2, 0x61C000, 0x61CFFF, "TestPro Systems, Inc.", "TESTPRO" },
	{ 0x0050C2, 0x61D000, 0x61DFFF, "Sutus Inc", "SUTUS" },
	{ 0x0050C2, 0x61E000, 0x61EFFF, "LESTER ELECTRONICS LTD", "LESTER-ELECTRONICS" },
	{ 0x0050C2, 0x61F000, 0x61FFFF, "Imagine Communications", "IMAGINE-COMMUNICATION" },
	{ 0x0050C2, 0x620000, 0x620FFF, "Harman/Becker Automotive Systems GmbH", "HARMAN/BECKER-AUTOMOTIVE" },
	{ 0x0050C2, 0x621000, 0x621FFF, "Version-T", "VERSION-T" },
	{ 0x0050C2, 0x622000, 0x622FFF, "2N TELEKOMUNIKACE a.s.", "2N-TELEKOMUNIKACE-A-S" },
	{ 0x0050C2, 0x623000, 0x623FFF, "SAFELINE SL", "SAFELINE-SL" },
	{ 0x0050C2, 0x624000, 0x624FFF, "Comtest Networks", "COMTEST-NETWORKS" },
	{ 0x0050C2, 0x625000, 0x625FFF, "EBNeuro SpA", "EBNEURO-SPA" },
	{ 0x0050C2, 0x626000, 0x626FFF, "Winsys Informatica ltda", "WINSYS-INFORMATICA-LTDA" },
	{ 0x0050C2, 0x627000, 0x627FFF, "JungleSystem Co., Ltd.", "JUNGLESYSTEM" },
	{ 0x0050C2, 0x628000, 0x628FFF, "DARE Development", "DARE-DEVELOPMENT" },
	{ 0x0050C2, 0x629000, 0x629FFF, "MacDonald Humfrey (Products) Ltd", "MACDONALD-HUMFREY-PRODUCTS" },
	{ 0x0050C2, 0x62A000, 0x62AFFF, "Prisma Engineering srl", "PRISMA-ENGINEERING-SRL" },
	{ 0x0050C2, 0x62B000, 0x62BFFF, "First Control Systems AB", "FIRST-CONTROL-AB" },
	{ 0x0050C2, 0x62C000, 0x62CFFF, "AirMatrix, Inc.", "AIRMATRIX" },
	{ 0x0050C2, 0x62D000, 0x62DFFF, "Procon Electronics", "PROCON-ELECTRONICS" },
	{ 0x0050C2, 0x62E000, 0x62EFFF, "TDM Ingénierie", "TDM-INGéNIERIE" },
	{ 0x0050C2, 0x62F000, 0x62FFFF, "QES", "QES" },
	{ 0x0050C2, 0x630000, 0x630FFF, "Aurora Flight Sciences", "AURORA-FLIGHT-SCIENCES" },
	{ 0x0050C2, 0x631000, 0x631FFF, "Fraunhofer IIS", "FRAUNHOFER-IIS" },
	{ 0x0050C2, 0x632000, 0x632FFF, "RoseTechnology A/S", "ROSETECHNOLOGY-A/S" },
	{ 0x0050C2, 0x633000, 0x633FFF, "Rice University", "RICE-UNIVERSITY" },
	{ 0x0050C2, 0x634000, 0x634FFF, "Sohon Inc", "SOHON" },
	{ 0x0050C2, 0x635000, 0x635FFF, "Shockfish SA", "SHOCKFISH" },
	{ 0x0050C2, 0x636000, 0x636FFF, "dSPACE GmbH", "DSPACE" },
	{ 0x0050C2, 0x637000, 0x637FFF, "Omnitrol Networks, Inc.", "OMNITROL-NETWORKS" },
	{ 0x0050C2, 0x638000, 0x638FFF, "HUNGAROCOM Telecommunication Ltd.", "HUNGAROCOM-TELECOMMUNICATION" },
	{ 0x0050C2, 0x639000, 0x639FFF, "Qstreams Networks Inc.", "QSTREAMS-NETWORKS" },
	{ 0x0050C2, 0x63A000, 0x63AFFF, "3DSP Corporation", "3DSP" },
	{ 0x0050C2, 0x63B000, 0x63BFFF, "Powis Corporation", "POWIS" },
	{ 0x0050C2, 0x63C000, 0x63CFFF, "dPict Imaging, Inc.", "DPICT-IMAGING" },
	{ 0x0050C2, 0x63D000, 0x63DFFF, "IDERs Inc", "IDERS" },
	{ 0x0050C2, 0x63E000, 0x63EFFF, "T2 Communication Ltd", "T2-COMMUNICATION" },
	{ 0x0050C2, 0x63F000, 0x63FFFF, "Speech Technology Center, Ltd.", "SPEECH-CENTER" },
	{ 0x0050C2, 0x640000, 0x640FFF, "IAC", "IAC" },
	{ 0x0050C2, 0x641000, 0x641FFF, "NEO Information Systems Co., Ltd.", "NEO-INFORMATION" },
	{ 0x0050C2, 0x642000, 0x642FFF, "Stanton Technologies Sdn Bhd", "STANTON-TECHNOLOGIES-SDN-BHD" },
	{ 0x0050C2, 0x643000, 0x643FFF, "Enatel Limited", "ENATEL" },
	{ 0x0050C2, 0x644000, 0x644FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x645000, 0x645FFF, "The Software Group Limited", "THE-SOFTWARE-GROUP" },
	{ 0x0050C2, 0x646000, 0x646FFF, "TRUTOUCH TECHNOLOGIES INC", "TRUTOUCH-TECHNOLOGIES" },
	{ 0x0050C2, 0x647000, 0x647FFF, "R&D Technology Solutionz Limited", "R-D-SOLUTIONZ" },
	{ 0x0050C2, 0x648000, 0x648FFF, "Fidelity Comtech, Inc.", "FIDELITY-COMTECH" },
	{ 0x0050C2, 0x649000, 0x649FFF, "Pan-STARRS", "PAN-STARRS" },
	{ 0x0050C2, 0x64A000, 0x64AFFF, "CPqD", "CPQD" },
	{ 0x0050C2, 0x64B000, 0x64BFFF, "MangoDSP", "MANGODSP" },
	{ 0x0050C2, 0x64C000, 0x64CFFF, "CIS Corporation", "CIS" },
	{ 0x0050C2, 0x64D000, 0x64DFFF, "Tera Information System Labs", "TERA-INFORMATION-SYSTEM-LABS" },
	{ 0x0050C2, 0x64E000, 0x64EFFF, "Northern Power", "NORTHERN-POWER" },
	{ 0x0050C2, 0x64F000, 0x64FFFF, "MA Lighting Technology GmbH", "MA-LIGHTING" },
	{ 0x0050C2, 0x650000, 0x650FFF, "Liquid Breaker, LLC", "LIQUID-BREAKER-LLC" },
	{ 0x0050C2, 0x651000, 0x651FFF, "STAER SPA", "STAER-SPA" },
	{ 0x0050C2, 0x652000, 0x652FFF, "Wideco Sweden AB", "WIDECO-SWEDEN-AB" },
	{ 0x0050C2, 0x653000, 0x653FFF, "Doble Engineering", "DOBLE-ENGINEERING" },
	{ 0x0050C2, 0x654000, 0x654FFF, "PaloDEx Group Oy", "PALODEX-GROUP-OY" },
	{ 0x0050C2, 0x655000, 0x655FFF, "Physik Instrumente (PI) GmbH&Co.KG", "PHYSIK-INSTRUMENTE-PI" },
	{ 0x0050C2, 0x656000, 0x656FFF, "LDA Audio Video Profesional", "LDA-AUDIO-VIDEO-PROFESIONAL" },
	{ 0x0050C2, 0x657000, 0x657FFF, "MONYTEL S.A.", "MONYTEL" },
	{ 0x0050C2, 0x658000, 0x658FFF, "OpenPKG GmbH", "OPENPKG" },
	{ 0x0050C2, 0x659000, 0x659FFF, "Dorsett Technologies, Inc.", "DORSETT-TECHNOLOGIES" },
	{ 0x0050C2, 0x65A000, 0x65AFFF, "Hisstema AB", "HISSTEMA-AB" },
	{ 0x0050C2, 0x65B000, 0x65BFFF, "Silverbrook Research", "SILVERBROOK-RESEARCH" },
	{ 0x0050C2, 0x65C000, 0x65CFFF, "VTZ d.o.o.", "VTZ-D-O-O" },
	{ 0x0050C2, 0x65D000, 0x65DFFF, "Redfone Communications LLC", "REDFONE-COMMUNICATION-LLC" },
	{ 0x0050C2, 0x65E000, 0x65EFFF, "Cantion A/S", "CANTION-A/S" },
	{ 0x0050C2, 0x65F000, 0x65FFFF, "Invocon, Inc.", "INVOCON" },
	{ 0x0050C2, 0x660000, 0x660FFF, "IZISOFT", "IZISOFT" },
	{ 0x0050C2, 0x661000, 0x661FFF, "P.C.E.", "P-C-E" },
	{ 0x0050C2, 0x662000, 0x662FFF, "Asia Pacific Card & System Sdn Bhd", "ASIA-PACIFIC-CARD-SYSTEM-SDN-BHD" },
	{ 0x0050C2, 0x663000, 0x663FFF, "COE Limited", "COE" },
	{ 0x0050C2, 0x664000, 0x664FFF, "Westel Wireless Systems", "WESTEL-WIRELESS" },
	{ 0x0050C2, 0x665000, 0x665FFF, "NetworkSound, Inc", "NETWORKSOUND" },
	{ 0x0050C2, 0x666000, 0x666FFF, "Xworks NZ Limited", "XWORKS-NZ" },
	{ 0x0050C2, 0x667000, 0x667FFF, "Vocality International Ltd", "VOCALITY" },
	{ 0x0050C2, 0x668000, 0x668FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x669000, 0x669FFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x0050C2, 0x66A000, 0x66AFFF, "ABB Xiamen Transmission and Distribution Automation Equipmen", "ABB-XIAMEN-TRANSMISSION-AND-DISTRIBUTION-AUTOMATION-EQUIPMEN" },
	{ 0x0050C2, 0x66B000, 0x66BFFF, "flsystem", "FLSYSTEM" },
	{ 0x0050C2, 0x66C000, 0x66CFFF, "DESY", "DESY" },
	{ 0x0050C2, 0x66D000, 0x66DFFF, "DIGITEK S.p.A.", "DIGITEK-S-P-A" },
	{ 0x0050C2, 0x66E000, 0x66EFFF, "Linear Systems Ltd.", "LINEAR" },
	{ 0x0050C2, 0x66F000, 0x66FFFF, "Nilan A/S", "NILAN-A/S" },
	{ 0x0050C2, 0x670000, 0x670FFF, "Naim Audio", "NAIM-AUDIO" },
	{ 0x0050C2, 0x671000, 0x671FFF, "Skyline Products, Inc", "SKYLINE-PRODUCTS" },
	{ 0x0050C2, 0x672000, 0x672FFF, "DDS Elettronica srl", "DDS-ELETTRONICA-SRL" },
	{ 0x0050C2, 0x673000, 0x673FFF, "Ferrari electronic AG", "FERRARI" },
	{ 0x0050C2, 0x674000, 0x674FFF, "Protech Optronics Co., Ltd.", "PROTECH-OPTRONICS" },
	{ 0x0050C2, 0x675000, 0x675FFF, "Kenton Research Ltd", "KENTON-RESEARCH" },
	{ 0x0050C2, 0x676000, 0x676FFF, "EDS", "EDS" },
	{ 0x0050C2, 0x677000, 0x677FFF, "ProconX Pty Ltd", "PROCONX-PTY" },
	{ 0x0050C2, 0x678000, 0x678FFF, "IHM", "IHM" },
	{ 0x0050C2, 0x679000, 0x679FFF, "Industrial Vacuum Systems", "INDUSTRIAL-VACUUM" },
	{ 0x0050C2, 0x67A000, 0x67AFFF, "CC Systems AB", "CC-AB" },
	{ 0x0050C2, 0x67B000, 0x67BFFF, "Sparton Electronics", "SPARTON-ELECTRONICS" },
	{ 0x0050C2, 0x67C000, 0x67CFFF, "AirCell, Inc.", "AIRCELL" },
	{ 0x0050C2, 0x67D000, 0x67DFFF, "ESA Messtechnik GmbH", "ESA-MESSTECHNIK" },
	{ 0x0050C2, 0x67E000, 0x67EFFF, "SAIA Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x67F000, 0x67FFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x680000, 0x680FFF, "Honey Network Research Limited", "HONEY-NETWORK-RESEARCH" },
	{ 0x0050C2, 0x681000, 0x681FFF, "Owasys Advanced Wireless Devices", "OWASYS-ADVANCED-WIRELESS-DEVICES" },
	{ 0x0050C2, 0x682000, 0x682FFF, "Commet AB", "COMMET-AB" },
	{ 0x0050C2, 0x683000, 0x683FFF, "MEGGITT Safety System", "MEGGITT-SAFETY-SYSTEM" },
	{ 0x0050C2, 0x684000, 0x684FFF, "REASON Tecnologia S.A.", "REASON-TECNOLOGIA" },
	{ 0x0050C2, 0x685000, 0x685FFF, "Datamars SA", "DATAMARS" },
	{ 0x0050C2, 0x686000, 0x686FFF, "ANNAX Anzeigesysteme GmbH", "ANNAX-ANZEIGESYSTEME" },
	{ 0x0050C2, 0x687000, 0x687FFF, "Access Specialties, Inc", "ACCESS-SPECIALTIES" },
	{ 0x0050C2, 0x688000, 0x688FFF, "Elk Products", "ELK-PRODUCTS" },
	{ 0x0050C2, 0x689000, 0x689FFF, "RF Code, Inc.", "RF-CODE" },
	{ 0x0050C2, 0x68A000, 0x68AFFF, "Zhuhai Jiahe Electronics Co.,LTD", "ZHUHAI-JIAHE-ELECTRONICS" },
	{ 0x0050C2, 0x68B000, 0x68BFFF, "SIMTEK INC.", "SIMTEK" },
	{ 0x0050C2, 0x68C000, 0x68CFFF, "Isochron Inc", "ISOCHRON" },
	{ 0x0050C2, 0x68D000, 0x68DFFF, "CXR Larus Corporation", "CXR-LARUS" },
	{ 0x0050C2, 0x68E000, 0x68EFFF, "SELCO", "SELCO" },
	{ 0x0050C2, 0x68F000, 0x68FFFF, "BERTRONIC SRL", "BERTRONIC-SRL" },
	{ 0x0050C2, 0x690000, 0x690FFF, "GHL Systems Berhad", "GHL-BERHAD" },
	{ 0x0050C2, 0x691000, 0x691FFF, "Interopix, Inc.", "INTEROPIX" },
	{ 0x0050C2, 0x692000, 0x692FFF, "Mate Media Access Technologies", "MATE-MEDIA-ACCESS-TECHNOLOGIES" },
	{ 0x0050C2, 0x693000, 0x693FFF, "Tech Comm, Inc.", "TECH-COMM" },
	{ 0x0050C2, 0x694000, 0x694FFF, "Initel srl", "INITEL-SRL" },
	{ 0x0050C2, 0x695000, 0x695FFF, "Purelink Technology, inc.", "PURELINK" },
	{ 0x0050C2, 0x696000, 0x696FFF, "Casabyte Inc.", "CASABYTE" },
	{ 0x0050C2, 0x697000, 0x697FFF, "Monarch Instrument", "MONARCH-INSTRUMENT" },
	{ 0x0050C2, 0x698000, 0x698FFF, "Navtech Radar Ltd", "NAVTECH-RADAR" },
	{ 0x0050C2, 0x699000, 0x699FFF, "Bulletendpoints Enterprises Inc", "BULLETENDPOINTS-ENTERPRISES" },
	{ 0x0050C2, 0x69A000, 0x69AFFF, "StoreTech Limited", "STORETECH" },
	{ 0x0050C2, 0x69B000, 0x69BFFF, "Tsien (UK) Ltd", "TSIEN-UK" },
	{ 0x0050C2, 0x69C000, 0x69CFFF, "Bug Labs, Inc.", "BUG-LABS" },
	{ 0x0050C2, 0x69D000, 0x69DFFF, "Dvation.co.,Ltd", "DVATION" },
	{ 0x0050C2, 0x69E000, 0x69EFFF, "Ideus AB", "IDEUS-AB" },
	{ 0x0050C2, 0x69F000, 0x69FFFF, "Total RF, LLC", "TOTAL-RF-LLC" },
	{ 0x0050C2, 0x6A0000, 0x6A0FFF, "GFP Lab S.r.l.", "GFP-LAB-S-R-L" },
	{ 0x0050C2, 0x6A1000, 0x6A1FFF, "PRICOL LIMITED", "PRICOL" },
	{ 0x0050C2, 0x6A2000, 0x6A2FFF, "Cadi Scientific Pte Ltd", "CADI-SCIENTIFIC-PTE" },
	{ 0x0050C2, 0x6A3000, 0x6A3FFF, "CreaTech Electronics Co.", "CREATECH-ELECTRONICS" },
	{ 0x0050C2, 0x6A4000, 0x6A4FFF, "TELETASK", "TELETASK" },
	{ 0x0050C2, 0x6A5000, 0x6A5FFF, "FHF Funke+Huster Fernsig GmbH", "FHF-FUNKE+HUSTER-FERNSIG" },
	{ 0x0050C2, 0x6A6000, 0x6A6FFF, "Victory Concept Industries Ltd.", "VICTORY-CONCEPT-INDUSTRIES" },
	{ 0x0050C2, 0x6A7000, 0x6A7FFF, "Hoer GmbH & Co. Industrie-Electronic KG", "HOER-INDUSTRIE" },
	{ 0x0050C2, 0x6A8000, 0x6A8FFF, "Delcan Technologies, Inc", "DELCAN-TECHNOLOGIES" },
	{ 0x0050C2, 0x6A9000, 0x6A9FFF, "Armida Technologies Corporation", "ARMIDA-TECHNOLOGIES" },
	{ 0x0050C2, 0x6AA000, 0x6AAFFF, "Ifox - Industria e Comercio Ltda", "IFOX---INDUSTRIA-E-COMERCIO-LTDA" },
	{ 0x0050C2, 0x6AB000, 0x6ABFFF, "Softwareentwicklung", "SOFTWAREENTWICKLUNG" },
	{ 0x0050C2, 0x6AC000, 0x6ACFFF, "Thales UK", "THALES-UK" },
	{ 0x0050C2, 0x6AD000, 0x6ADFFF, "Heim- & Bürokommunikation", "HEIM--BUEROKOMMUNIKATION" },
	{ 0x0050C2, 0x6AE000, 0x6AEFFF, "Qualisys AB", "QUALISYS-AB" },
	{ 0x0050C2, 0x6AF000, 0x6AFFFF, "Nanoradio AB", "NANORADIO-AB" },
	{ 0x0050C2, 0x6B0000, 0x6B0FFF, "Smart Key International Limited", "SMART-KEY" },
	{ 0x0050C2, 0x6B1000, 0x6B1FFF, "Burk Technology", "BURK" },
	{ 0x0050C2, 0x6B2000, 0x6B2FFF, "Edgeware AB", "EDGEWARE-AB" },
	{ 0x0050C2, 0x6B3000, 0x6B3FFF, "4RF Communications Ltd", "4RF-COMMUNICATION" },
	{ 0x0050C2, 0x6B4000, 0x6B4FFF, "SOMESCA", "SOMESCA" },
	{ 0x0050C2, 0x6B5000, 0x6B5FFF, "TRIUMF", "TRIUMF" },
	{ 0x0050C2, 0x6B6000, 0x6B6FFF, "CommoDaS GmbH", "COMMODAS" },
	{ 0x0050C2, 0x6B7000, 0x6B7FFF, "System LSI CO.Ltd.", "SYSTEM-LSI" },
	{ 0x0050C2, 0x6B8000, 0x6B8FFF, "Epec Oy", "EPEC-OY" },
	{ 0x0050C2, 0x6B9000, 0x6B9FFF, "unipo GmbH", "UNIPO" },
	{ 0x0050C2, 0x6BA000, 0x6BAFFF, "Fertron Controle e Automacao Industrial Ltda.", "FERTRON-CONTROLE-E-AUTOMACAO-INDUSTRIAL-LTDA" },
	{ 0x0050C2, 0x6BB000, 0x6BBFFF, "Ele.Mag S.r.l.", "ELE-MAG-S-R-L" },
	{ 0x0050C2, 0x6BC000, 0x6BCFFF, "Paraytec Ltd", "PARAYTEC" },
	{ 0x0050C2, 0x6BD000, 0x6BDFFF, "Mitron Oy", "MITRON-OY" },
	{ 0x0050C2, 0x6BE000, 0x6BEFFF, "ESTEC Co.,Ltd.", "ESTEC" },
	{ 0x0050C2, 0x6BF000, 0x6BFFFF, "Optoplan as", "OPTOPLAN-AS" },
	{ 0x0050C2, 0x6C0000, 0x6C0FFF, "GLOSTER SANTE EUROPE", "GLOSTER-SANTE-EUROPE" },
	{ 0x0050C2, 0x6C1000, 0x6C1FFF, "RADIUS Sweden AB", "RADIUS-SWEDEN-AB" },
	{ 0x0050C2, 0x6C2000, 0x6C2FFF, "HoseoTelnet Inc...", "HOSEOTELNET" },
	{ 0x0050C2, 0x6C3000, 0x6C3FFF, "iTRACS Corporation", "ITRACS" },
	{ 0x0050C2, 0x6C4000, 0x6C4FFF, "REXXON GmbH", "REXXON" },
	{ 0x0050C2, 0x6C5000, 0x6C5FFF, "Oerlikon Contraves AG", "OERLIKON-CONTRAVES" },
	{ 0x0050C2, 0x6C6000, 0x6C6FFF, "MedAvant Healthcare Solutions", "MEDAVANT-HEALTHCARE-SOLUTIONS" },
	{ 0x0050C2, 0x6C7000, 0x6C7FFF, "QuickCircuit Ltd.", "QUICKCIRCUIT" },
	{ 0x0050C2, 0x6C8000, 0x6C8FFF, "B&S MEDIA CO., LTD.", "B-S-MEDIA" },
	{ 0x0050C2, 0x6C9000, 0x6C9FFF, "NETAMI", "NETAMI" },
	{ 0x0050C2, 0x6CA000, 0x6CAFFF, "Dynamic Hearing Pty Ltd", "DYNAMIC-HEARING-PTY" },
	{ 0x0050C2, 0x6CB000, 0x6CBFFF, "Stream Processors", "STREAM-PROCESSORS" },
	{ 0x0050C2, 0x6CC000, 0x6CCFFF, "Widmer Time Recorder Co., Inc.", "WIDMER-TIME-RECORDER" },
	{ 0x0050C2, 0x6CD000, 0x6CDFFF, "RGM SPA", "RGM-SPA" },
	{ 0x0050C2, 0x6CE000, 0x6CEFFF, "EMITALL Surveillance S.A,", "EMITALL-SURVEILLANCE" },
	{ 0x0050C2, 0x6CF000, 0x6CFFFF, "Microway", "MICROWAY" },
	{ 0x0050C2, 0x6D0000, 0x6D0FFF, "EDS Systemtechnik", "EDS-SYSTEMTECHNIK" },
	{ 0x0050C2, 0x6D1000, 0x6D1FFF, "Schnick-Schnack-Systems GmbH", "SCHNICK-SCHNACK" },
	{ 0x0050C2, 0x6D2000, 0x6D2FFF, "Lumistar Incorporated", "LUMISTAR-INCORPORATED" },
	{ 0x0050C2, 0x6D3000, 0x6D3FFF, "DigiSensory technologies Pty Ltd", "DIGISENSORY-TECHNOLOGIES-PTY" },
	{ 0x0050C2, 0x6D4000, 0x6D4FFF, "Etani Electronics Co.,Ltd.", "ETANI-ELECTRONICS" },
	{ 0x0050C2, 0x6D5000, 0x6D5FFF, "Becker Electronics GmbH", "BECKER-ELECTRONICS" },
	{ 0x0050C2, 0x6D6000, 0x6D6FFF, "ADL Electronics Ltd.", "ADL-ELECTRONICS" },
	{ 0x0050C2, 0x6D7000, 0x6D7FFF, "Mavenir System, Inc.", "MAVENIR-SYSTEM" },
	{ 0x0050C2, 0x6D8000, 0x6D8FFF, "BL Healthcare, Inc.", "BL-HEALTHCARE" },
	{ 0x0050C2, 0x6D9000, 0x6D9FFF, "Ajeco Oy", "AJECO-OY" },
	{ 0x0050C2, 0x6DA000, 0x6DAFFF, "Techno Fittings S.r.l.", "TECHNO-FITTINGS-S-R-L" },
	{ 0x0050C2, 0x6DB000, 0x6DBFFF, "Gebhardt Ventilatoren GmbH", "GEBHARDT-VENTILATOREN" },
	{ 0x0050C2, 0x6DC000, 0x6DCFFF, "L-3 Communications Mobile-Vision, Inc.", "L-3-COMMUNICATION-MOBILE-VISION" },
	{ 0x0050C2, 0x6DD000, 0x6DDFFF, "Zmicro Systems Inc", "ZMICRO" },
	{ 0x0050C2, 0x6DE000, 0x6DEFFF, "Laser Tools & Technics Corp.", "LASER-TOOLS-TECHNICS" },
	{ 0x0050C2, 0x6DF000, 0x6DFFFF, "QR Sciences Ltd", "QR-SCIENCES" },
	{ 0x0050C2, 0x6E0000, 0x6E0FFF, "FIRSTTRUST Co.,Ltd.", "FIRSTTRUST" },
	{ 0x0050C2, 0x6E1000, 0x6E1FFF, "NewOnSys Ltd.", "NEWONSYS" },
	{ 0x0050C2, 0x6E2000, 0x6E2FFF, "PHYTEC Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x6E3000, 0x6E3FFF, "Miros AS", "MIROS-AS" },
	{ 0x0050C2, 0x6E4000, 0x6E4FFF, "MangoDSP", "MANGODSP" },
	{ 0x0050C2, 0x6E5000, 0x6E5FFF, "Boeckeler Instruments, Inc.", "BOECKELER-INSTRUMENTS" },
	{ 0x0050C2, 0x6E6000, 0x6E6FFF, "Lanetco", "LANETCO" },
	{ 0x0050C2, 0x6E7000, 0x6E7FFF, "Axis Network Technology", "AXIS-NETWORK" },
	{ 0x0050C2, 0x6E8000, 0x6E8FFF, "Anymax", "ANYMAX" },
	{ 0x0050C2, 0x6E9000, 0x6E9FFF, "Bando electronic communication Co.Lltd", "BANDO-COMMUNICATION-LLTD" },
	{ 0x0050C2, 0x6EA000, 0x6EAFFF, "FIRSTEC SA", "FIRSTEC" },
	{ 0x0050C2, 0x6EB000, 0x6EBFFF, "Harrison Audio, LLC", "HARRISON-AUDIO-LLC" },
	{ 0x0050C2, 0x6EC000, 0x6ECFFF, "Netistix Technologies Corporation", "NETISTIX-TECHNOLOGIES" },
	{ 0x0050C2, 0x6ED000, 0x6EDFFF, "Sechan Electronics, Inc.", "SECHAN-ELECTRONICS" },
	{ 0x0050C2, 0x6EE000, 0x6EEFFF, "Interactive Electronic Systems", "INTERACTIVE" },
	{ 0x0050C2, 0x6EF000, 0x6EFFFF, "Pneumopartners LaenneXT SA", "PNEUMOPARTNERS-LAENNEXT" },
	{ 0x0050C2, 0x6F0000, 0x6F0FFF, "Stanley Security Solutions, Inc.", "STANLEY-SECURITY-SOLUTIONS" },
	{ 0x0050C2, 0x6F1000, 0x6F1FFF, "ITS Telecom", "ITS-TELECOM" },
	{ 0x0050C2, 0x6F2000, 0x6F2FFF, "Laser Electronics Ltd", "LASER-ELECTRONICS" },
	{ 0x0050C2, 0x6F3000, 0x6F3FFF, "E3Switch LLC", "E3SWITCH-LLC" },
	{ 0x0050C2, 0x6F4000, 0x6F4FFF, "Cryogenic Control Systems, Inc.", "CRYOGENIC-CONTROL" },
	{ 0x0050C2, 0x6F5000, 0x6F5FFF, "Kitron Microelectronics AB", "KITRON-MICROELECTRONICS-AB" },
	{ 0x0050C2, 0x6F6000, 0x6F6FFF, "AV SatCom AS", "AV-SATCOM-AS" },
	{ 0x0050C2, 0x6F7000, 0x6F7FFF, "infoplan Gesellschaftfür Informationssysteme mbH", "INFOPLAN-GESELLSCHAFTFUER-INFORMATIONSSYSTEME-MBH" },
	{ 0x0050C2, 0x6F8000, 0x6F8FFF, "RV Technology Limited", "RV" },
	{ 0x0050C2, 0x6F9000, 0x6F9FFF, "Revox GmbH", "REVOX" },
	{ 0x0050C2, 0x6FA000, 0x6FAFFF, "DCN", "DCN" },
	{ 0x0050C2, 0x6FB000, 0x6FBFFF, "WaveIP", "WAVEIP" },
	{ 0x0050C2, 0x6FC000, 0x6FCFFF, "Acte Sp. z o.o.", "ACTE-SP-Z-O-O" },
	{ 0x0050C2, 0x6FD000, 0x6FDFFF, "SAIA Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x6FE000, 0x6FEFFF, "Blue Origin", "BLUE-ORIGIN" },
	{ 0x0050C2, 0x6FF000, 0x6FFFFF, "St. Michael Strategies Inc.", "ST-MICHAEL-STRATEGIES" },
	{ 0x0050C2, 0x700000, 0x700FFF, "GEM-MED SL", "GEM-MED-SL" },
	{ 0x0050C2, 0x701000, 0x701FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x702000, 0x702FFF, "SPM Instrument AB", "SPM-INSTRUMENT-AB" },
	{ 0x0050C2, 0x703000, 0x703FFF, "SAE IT-systems GmbH & Co. KG", "SAE-IT" },
	{ 0x0050C2, 0x704000, 0x704FFF, "The Dini Group, La Jolla inc.", "THE-DINI-GROUP-LA-JOLLA" },
	{ 0x0050C2, 0x705000, 0x705FFF, "Hauch & Bach ApS", "HAUCH-BACH-APS" },
	{ 0x0050C2, 0x706000, 0x706FFF, "DioDigiWorks. CO., LTD.", "DIODIGIWORKS" },
	{ 0x0050C2, 0x707000, 0x707FFF, "DTech Labs Inc", "DTECH-LABS" },
	{ 0x0050C2, 0x708000, 0x708FFF, "Smartek d.o.o.", "SMARTEK-D-O-O" },
	{ 0x0050C2, 0x709000, 0x709FFF, "RO.VE.R. Laboratories S.p.A", "RO-VE-R-LABORATORIES-S-P-A" },
	{ 0x0050C2, 0x70A000, 0x70AFFF, "Efficient Channel Coding", "EFFICIENT-CHANNEL-CODING" },
	{ 0x0050C2, 0x70B000, 0x70BFFF, "B.E.A.R. Solutions (Australasia) Pty, Ltd", "B-E-A-R-SOLUTIONS-AUSTRALASIA-PTY" },
	{ 0x0050C2, 0x70C000, 0x70CFFF, "Exertus", "EXERTUS" },
	{ 0x0050C2, 0x70D000, 0x70DFFF, "ela-soft GmbH & Co. KG", "ELA-SOFT" },
	{ 0x0050C2, 0x70E000, 0x70EFFF, "AUDICO SYSTEMS OY", "AUDICO-OY" },
	{ 0x0050C2, 0x70F000, 0x70FFFF, "Zumbach Electronic AG", "ZUMBACH" },
	{ 0x0050C2, 0x710000, 0x710FFF, "Wharton Electronics Ltd", "WHARTON-ELECTRONICS" },
	{ 0x0050C2, 0x711000, 0x711FFF, "LINKIT S.R.L.", "LINKIT-S-R-L" },
	{ 0x0050C2, 0x712000, 0x712FFF, "Pasan SA", "PASAN" },
	{ 0x0050C2, 0x713000, 0x713FFF, "3DX-Ray Limited", "3DX-RAY" },
	{ 0x0050C2, 0x714000, 0x714FFF, "T.E.AM., S. A.", "T-E-AM-S-A" },
	{ 0x0050C2, 0x715000, 0x715FFF, "RIEXINGER Elektronik", "RIEXINGER-ELEKTRONIK" },
	{ 0x0050C2, 0x716000, 0x716FFF, "MITROL S.R.L.", "MITROL-S-R-L" },
	{ 0x0050C2, 0x717000, 0x717FFF, "MB Connect Line GmbH", "MB-CONNECT-LINE" },
	{ 0x0050C2, 0x718000, 0x718FFF, "illunis LLC", "ILLUNIS-LLC" },
	{ 0x0050C2, 0x719000, 0x719FFF, "ennovatis GmbH", "ENNOVATIS" },
	{ 0x0050C2, 0x71A000, 0x71AFFF, "Logus Broadband Wireless Solutions Inc.", "LOGUS-BROADBAND-WIRELESS-SOLUTIONS" },
	{ 0x0050C2, 0x71B000, 0x71BFFF, "ADVA Optical Networking", "ADVA-OPTICAL-NETWORKING" },
	{ 0x0050C2, 0x71C000, 0x71CFFF, "Elmec Inc.", "ELMEC" },
	{ 0x0050C2, 0x71D000, 0x71DFFF, "MG s.r.l.", "MG-S-R-L" },
	{ 0x0050C2, 0x71E000, 0x71EFFF, "ASKI Industrie Elektronik Ges.m.b.H.", "ASKI-INDUSTRIE-ELEKTRONIK-GES-M-B-H" },
	{ 0x0050C2, 0x71F000, 0x71FFFF, "ASC telecom AG", "ASC-TELECOM" },
	{ 0x0050C2, 0x720000, 0x720FFF, "Colorado Engineering Inc.", "COLORADO-ENGINEERING" },
	{ 0x0050C2, 0x721000, 0x721FFF, "Spectrum Communications FZE", "SPECTRUM-COMMUNICATION-FZE" },
	{ 0x0050C2, 0x722000, 0x722FFF, "Centric TSolve BV", "CENTRIC-TSOLVE" },
	{ 0x0050C2, 0x723000, 0x723FFF, "Power Electronics", "POWER-ELECTRONICS" },
	{ 0x0050C2, 0x724000, 0x724FFF, "HSC-Regelungstechnik GmbH", "HSC-REGELUNGSTECHNIK" },
	{ 0x0050C2, 0x725000, 0x725FFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x0050C2, 0x726000, 0x726FFF, "eta systemi CKB", "ETA-SYSTEMI-CKB" },
	{ 0x0050C2, 0x727000, 0x727FFF, "Pelweckyj Videotechnik GmbH", "PELWECKYJ-VIDEOTECHNIK" },
	{ 0x0050C2, 0x728000, 0x728FFF, "InterDigital Canada Ltd", "INTERDIGITAL-CANADA" },
	{ 0x0050C2, 0x729000, 0x729FFF, "SP Controls, Inc", "SP-CONTROLS" },
	{ 0x0050C2, 0x72A000, 0x72AFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x72B000, 0x72BFFF, "Sequestered Solutions", "SEQUESTERED-SOLUTIONS" },
	{ 0x0050C2, 0x72C000, 0x72CFFF, "Richard Griessbach Feinmechanik GmbH", "RICHARD-GRIESSBACH-FEINMECHANIK" },
	{ 0x0050C2, 0x72D000, 0x72DFFF, "Physical Acoustics Corporation", "PHYSICAL-ACOUSTICS" },
	{ 0x0050C2, 0x72E000, 0x72EFFF, "SNCF EIM PAYS DE LOIRE", "SNCF-EIM-PAYS-DE-LOIRE" },
	{ 0x0050C2, 0x72F000, 0x72FFFF, "Priority Electronics Ltd", "PRIORITY-ELECTRONICS" },
	{ 0x0050C2, 0x730000, 0x730FFF, "haber & koenig electronics gmbh", "HABER-KOENIG-ELECTRONICS" },
	{ 0x0050C2, 0x731000, 0x731FFF, "Spirent Communications", "SPIRENT-COMMUNICATION" },
	{ 0x0050C2, 0x732000, 0x732FFF, "Schlumberger K.K.", "SCHLUMBERGER-K-K" },
	{ 0x0050C2, 0x733000, 0x733FFF, "Cimetrics Research Pty Ltd", "CIMETRICS-RESEARCH-PTY" },
	{ 0x0050C2, 0x734000, 0x734FFF, "CardioMEMS Inc.", "CARDIOMEMS" },
	{ 0x0050C2, 0x735000, 0x735FFF, "Ant Lamp, Inc", "ANT-LAMP" },
	{ 0x0050C2, 0x736000, 0x736FFF, "Nika Ltd", "NIKA" },
	{ 0x0050C2, 0x737000, 0x737FFF, "Teradici Corporation", "TERADICI" },
	{ 0x0050C2, 0x738000, 0x738FFF, "Miracom Technology Co., Ltd.", "MIRACOM" },
	{ 0x0050C2, 0x739000, 0x739FFF, "Tattile srl", "TATTILE-SRL" },
	{ 0x0050C2, 0x73A000, 0x73AFFF, "Naturela Ltd.", "NATURELA" },
	{ 0x0050C2, 0x73B000, 0x73BFFF, "On Air Networks", "ON-AIR-NETWORKS" },
	{ 0x0050C2, 0x73C000, 0x73CFFF, "Simicon", "SIMICON" },
	{ 0x0050C2, 0x73D000, 0x73DFFF, "cryptiris", "CRYPTIRIS" },
	{ 0x0050C2, 0x73E000, 0x73EFFF, "Quantec Networks GmbH", "QUANTEC-NETWORKS" },
	{ 0x0050C2, 0x73F000, 0x73FFFF, "MEDAV GmbH", "MEDAV" },
	{ 0x0050C2, 0x740000, 0x740FFF, "McQuay China", "MCQUAY-CHINA" },
	{ 0x0050C2, 0x741000, 0x741FFF, "Dain", "DAIN" },
	{ 0x0050C2, 0x742000, 0x742FFF, "Fantuzzi Reggiane", "FANTUZZI-REGGIANE" },
	{ 0x0050C2, 0x743000, 0x743FFF, "Elektro-Top 3000 Ltd.", "ELEKTRO-TOP-3000" },
	{ 0x0050C2, 0x744000, 0x744FFF, "Avonaco Systems, Inc.", "AVONACO" },
	{ 0x0050C2, 0x745000, 0x745FFF, "ACISA", "ACISA" },
	{ 0x0050C2, 0x746000, 0x746FFF, "Realtronix Company", "REALTRONIX-COMPANY" },
	{ 0x0050C2, 0x747000, 0x747FFF, "CDSA Dam Neck", "CDSA-DAM-NECK" },
	{ 0x0050C2, 0x748000, 0x748FFF, "Letechnic Ltd", "LETECHNIC" },
	{ 0x0050C2, 0x749000, 0x749FFF, "Affolter Technologies SA", "AFFOLTER-TECHNOLOGIES" },
	{ 0x0050C2, 0x74A000, 0x74AFFF, "MONITOR ELECTRONICS LTD", "MONITOR-ELECTRONICS" },
	{ 0x0050C2, 0x74B000, 0x74BFFF, "STAR-Dundee Ltd", "STAR-DUNDEE" },
	{ 0x0050C2, 0x74C000, 0x74CFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x74D000, 0x74DFFF, "Beceem Communications, Inc.", "BECEEM-COMMUNICATION" },
	{ 0x0050C2, 0x74E000, 0x74EFFF, "TRONICO", "TRONICO" },
	{ 0x0050C2, 0x74F000, 0x74FFFF, "German Technologies", "GERMAN-TECHNOLOGIES" },
	{ 0x0050C2, 0x750000, 0x750FFF, "Brightlights Intellectual Property Ltd", "BRIGHTLIGHTS-INTELLECTUAL-PROPERTY" },
	{ 0x0050C2, 0x751000, 0x751FFF, "e&s Engineering & Software GmbH", "E-S-ENGINEERING-SOFTWARE" },
	{ 0x0050C2, 0x752000, 0x752FFF, "LOBER, S.A.", "LOBER" },
	{ 0x0050C2, 0x753000, 0x753FFF, "ABB", "ABB" },
	{ 0x0050C2, 0x754000, 0x754FFF, "Abeo Corporation", "ABEO" },
	{ 0x0050C2, 0x755000, 0x755FFF, "Teletek Electronics", "TELETEK-ELECTRONICS" },
	{ 0x0050C2, 0x756000, 0x756FFF, "Chesapeake Sciences Corp", "CHESAPEAKE-SCIENCES" },
	{ 0x0050C2, 0x757000, 0x757FFF, "E S P Technologies Ltd", "E-S-P-TECHNOLOGIES" },
	{ 0x0050C2, 0x758000, 0x758FFF, "AixSolve GmbH", "AIXSOLVE" },
	{ 0x0050C2, 0x759000, 0x759FFF, "Sequentric Energy Systems, LLC", "SEQUENTRIC-ENERGY-LLC" },
	{ 0x0050C2, 0x75A000, 0x75AFFF, "Gaisler Research AB", "GAISLER-RESEARCH-AB" },
	{ 0x0050C2, 0x75B000, 0x75BFFF, "DMT System S.p.A.", "DMT-SYSTEM-S-P-A" },
	{ 0x0050C2, 0x75C000, 0x75CFFF, "STÖRK-TRONIC Störk GmbH&Co. KG", "STOERK-TRONIC-STOERK" },
	{ 0x0050C2, 0x75D000, 0x75DFFF, "Fluid Analytics, Inc.", "FLUID-ANALYTICS" },
	{ 0x0050C2, 0x75E000, 0x75EFFF, "Sky-Skan, Incorporated", "SKY-SKAN-INCORPORATED" },
	{ 0x0050C2, 0x75F000, 0x75FFFF, "B. Rexroth the identity company GmbH", "B-REXROTH-THE-IDENTITY-COMPANY" },
	{ 0x0050C2, 0x760000, 0x760FFF, "AR'S CO., LTD.", "AR-S" },
	{ 0x0050C2, 0x761000, 0x761FFF, "Elbit Systems of America - Fort Worth Operations", "ELBIT-OF-AMERICA---FORT-WORTH-OPERATIONS" },
	{ 0x0050C2, 0x762000, 0x762FFF, "Assembly Contracts Limited", "ASSEMBLY-CONTRACTS" },
	{ 0x0050C2, 0x763000, 0x763FFF, "XtendWave", "XTENDWAVE" },
	{ 0x0050C2, 0x764000, 0x764FFF, "Argus-Spectrum", "ARGUS-SPECTRUM" },
	{ 0x0050C2, 0x765000, 0x765FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x766000, 0x766FFF, "Gutermann Technology GmbH", "GUTERMANN" },
	{ 0x0050C2, 0x767000, 0x767FFF, "EID", "EID" },
	{ 0x0050C2, 0x768000, 0x768FFF, "Control Service do Brasil Ltda", "CONTROL-SERVICE-DO-BRASIL-LTDA" },
	{ 0x0050C2, 0x769000, 0x769FFF, "BES GmbH", "BES" },
	{ 0x0050C2, 0x76A000, 0x76AFFF, "Digidrive Audio Limited", "DIGIDRIVE-AUDIO" },
	{ 0x0050C2, 0x76B000, 0x76BFFF, "Putercom Enterprise Co., LTD.", "PUTERCOM" },
	{ 0x0050C2, 0x76C000, 0x76CFFF, "EFG CZ spol. s r.o.", "EFG-CZ-SPOL-S-R-O" },
	{ 0x0050C2, 0x76D000, 0x76DFFF, "Mobilisme", "MOBILISME" },
	{ 0x0050C2, 0x76E000, 0x76EFFF, "Crinia Corporation", "CRINIA" },
	{ 0x0050C2, 0x76F000, 0x76FFFF, "Control and Robotics Solutions", "CONTROL-AND-ROBOTICS-SOLUTIONS" },
	{ 0x0050C2, 0x770000, 0x770FFF, "Cadex Electronics Inc.", "CADEX-ELECTRONICS" },
	{ 0x0050C2, 0x771000, 0x771FFF, "ZigBee Alliance", "ZIGBEE-ALLIANCE" },
	{ 0x0050C2, 0x772000, 0x772FFF, "IES Elektronikentwicklung", "IES-ELEKTRONIKENTWICKLUNG" },
	{ 0x0050C2, 0x773000, 0x773FFF, "Pointe Conception Medical Inc.", "POINTE-CONCEPTION-MEDICAL" },
	{ 0x0050C2, 0x774000, 0x774FFF, "GeoSIG Ltd.", "GEOSIG" },
	{ 0x0050C2, 0x775000, 0x775FFF, "Laserdyne Technologies", "LASERDYNE-TECHNOLOGIES" },
	{ 0x0050C2, 0x776000, 0x776FFF, "Integrated Security Corporation", "INTEGRATED-SECURITY" },
	{ 0x0050C2, 0x777000, 0x777FFF, "Euro Display Srl", "EURO-DISPLAY-SRL" },
	{ 0x0050C2, 0x778000, 0x778FFF, "SunGard Vivista", "SUNGARD-VIVISTA" },
	{ 0x0050C2, 0x779000, 0x779FFF, "Coral Telecom Ltd", "CORAL-TELECOM" },
	{ 0x0050C2, 0x77A000, 0x77AFFF, "Smith Meter, Inc", "SMITH-METER" },
	{ 0x0050C2, 0x77B000, 0x77BFFF, "Itibia Technologies, Inc.", "ITIBIA-TECHNOLOGIES" },
	{ 0x0050C2, 0x77C000, 0x77CFFF, "ATEC SRL", "ATEC-SRL" },
	{ 0x0050C2, 0x77D000, 0x77DFFF, "Lincoln Industrial", "LINCOLN-INDUSTRIAL" },
	{ 0x0050C2, 0x77E000, 0x77EFFF, "Cominfo Inc.", "COMINFO" },
	{ 0x0050C2, 0x77F000, 0x77FFFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0x780000, 0x780FFF, "IQ Solutions GmbH & Co. KG", "IQ-SOLUTIONS" },
	{ 0x0050C2, 0x781000, 0x781FFF, "Starling Advanced Communications", "STARLING-ADVANCED-COMMUNICATION" },
	{ 0x0050C2, 0x782000, 0x782FFF, "Phytec Mestechnik GmbH", "PHYTEC-MESTECHNIK" },
	{ 0x0050C2, 0x783000, 0x783FFF, "NORMA systems GmbH", "NORMA" },
	{ 0x0050C2, 0x784000, 0x784FFF, "Lewis Controls Inc.", "LEWIS-CONTROLS" },
	{ 0x0050C2, 0x785000, 0x785FFF, "Icon Time Systems", "ICON-TIME" },
	{ 0x0050C2, 0x786000, 0x786FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x787000, 0x787FFF, "Austco Communication Systems Pty Ltd", "AUSTCO-COMMUNICATION-PTY" },
	{ 0x0050C2, 0x788000, 0x788FFF, "HOSA TECHNOLOGY, INC.", "HOSA" },
	{ 0x0050C2, 0x789000, 0x789FFF, "Rosslare Enterprises Limited", "ROSSLARE-ENTERPRISES" },
	{ 0x0050C2, 0x78A000, 0x78AFFF, "LEVEL TELECOM", "LEVEL-TELECOM" },
	{ 0x0050C2, 0x78B000, 0x78BFFF, "OMICRON electronics GmbH", "OMICRON-ELECTRONICS" },
	{ 0x0050C2, 0x78C000, 0x78CFFF, "Giga-tronics, Inc.", "GIGA-TRONICS" },
	{ 0x0050C2, 0x78D000, 0x78DFFF, "Telairity", "TELAIRITY" },
	{ 0x0050C2, 0x78E000, 0x78EFFF, "GLOBALCOM ENGINEERING SRL", "GLOBALCOM-ENGINEERING-SRL" },
	{ 0x0050C2, 0x78F000, 0x78FFFF, "ELMAR electronic", "ELMAR" },
	{ 0x0050C2, 0x790000, 0x790FFF, "GE Security-Kampro", "GE-SECURITY-KAMPRO" },
	{ 0x0050C2, 0x791000, 0x791FFF, "M Squared Lasers Limited", "M-SQUARED-LASERS" },
	{ 0x0050C2, 0x792000, 0x792FFF, "SMARTRO Co.,Ltd.", "SMARTRO" },
	{ 0x0050C2, 0x793000, 0x793FFF, "Enertex Bayern GmbH", "ENERTEX-BAYERN" },
	{ 0x0050C2, 0x794000, 0x794FFF, "COMSONICS, INC.", "COMSONICS" },
	{ 0x0050C2, 0x795000, 0x795FFF, "Ameli Spa", "AMELI-SPA" },
	{ 0x0050C2, 0x796000, 0x796FFF, "DORLET S.A.", "DORLET" },
	{ 0x0050C2, 0x797000, 0x797FFF, "Tiefenbach Control Systems GmbH", "TIEFENBACH-CONTROL" },
	{ 0x0050C2, 0x798000, 0x798FFF, "Indefia", "INDEFIA" },
	{ 0x0050C2, 0x799000, 0x799FFF, "AAVD", "AAVD" },
	{ 0x0050C2, 0x79A000, 0x79AFFF, "JMC America, LLC", "JMC-AMERICA-LLC" },
	{ 0x0050C2, 0x79B000, 0x79BFFF, "Schniewindt GmbH & Co. KG", "SCHNIEWINDT" },
	{ 0x0050C2, 0x79C000, 0x79CFFF, "Vital Systems Inc", "VITAL" },
	{ 0x0050C2, 0x79D000, 0x79DFFF, "MiraTrek", "MIRATREK" },
	{ 0x0050C2, 0x79E000, 0x79EFFF, "Benshaw Canada Controls, Inc.", "BENSHAW-CANADA-CONTROLS" },
	{ 0x0050C2, 0x79F000, 0x79FFFF, "ZAO NPC", "ZAO-NPC" },
	{ 0x0050C2, 0x7A0000, 0x7A0FFF, "MedAvant Healthcare", "MEDAVANT-HEALTHCARE" },
	{ 0x0050C2, 0x7A1000, 0x7A1FFF, "Field Design Service", "FIELD-DESIGN-SERVICE" },
	{ 0x0050C2, 0x7A2000, 0x7A2FFF, "RaySat Israel LTD", "RAYSAT-ISRAEL" },
	{ 0x0050C2, 0x7A3000, 0x7A3FFF, "ABB Transmission and Distribution Automation Equipment (Xiam", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTOMATION-EQUIPMENT-XIAM" },
	{ 0x0050C2, 0x7A4000, 0x7A4FFF, "Calibre UK LTD", "CALIBRE-UK" },
	{ 0x0050C2, 0x7A5000, 0x7A5FFF, "Quantum Medical Imaging", "QUANTUM-MEDICAL-IMAGING" },
	{ 0x0050C2, 0x7A6000, 0x7A6FFF, "ASIANA IDT", "ASIANA-IDT" },
	{ 0x0050C2, 0x7A7000, 0x7A7FFF, "Guidance Navigation Limited", "GUIDANCE-NAVIGATION" },
	{ 0x0050C2, 0x7A8000, 0x7A8FFF, "Integrated Design Tools, Inc.", "INTEGRATED-DESIGN-TOOLS" },
	{ 0x0050C2, 0x7A9000, 0x7A9FFF, "Delta Tau Data Systems, Inc", "DELTA-TAU-DATA" },
	{ 0x0050C2, 0x7AA000, 0x7AAFFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0x7AB000, 0x7ABFFF, "General Microsystems Sdn Bhd", "GENERAL-MICROSYSTEMS-SDN-BHD" },
	{ 0x0050C2, 0x7AC000, 0x7ACFFF, "IUSA SA DE CV", "IUSA-DE-CV" },
	{ 0x0050C2, 0x7AD000, 0x7ADFFF, "Turun Turvatekniikka Oy", "TURUN-TURVATEKNIIKKA-OY" },
	{ 0x0050C2, 0x7AE000, 0x7AEFFF, "Global Tel-Link", "GLOBAL-TEL-LINK" },
	{ 0x0050C2, 0x7AF000, 0x7AFFFF, "C2 Microsystems", "C2-MICROSYSTEMS" },
	{ 0x0050C2, 0x7B0000, 0x7B0FFF, "IMP Telekom", "IMP-TELEKOM" },
	{ 0x0050C2, 0x7B1000, 0x7B1FFF, "ATEME", "ATEME" },
	{ 0x0050C2, 0x7B2000, 0x7B2FFF, "A.D.I Video technologies", "A-D-I-VIDEO-TECHNOLOGIES" },
	{ 0x0050C2, 0x7B3000, 0x7B3FFF, "Elmec, Inc.", "ELMEC" },
	{ 0x0050C2, 0x7B4000, 0x7B4FFF, "T 1 Engineering", "T-1-ENGINEERING" },
	{ 0x0050C2, 0x7B5000, 0x7B5FFF, "DIT-MCO International", "DIT-MCO" },
	{ 0x0050C2, 0x7B6000, 0x7B6FFF, "Alstom (Schweiz) AG", "ALSTOM-SCHWEIZ" },
	{ 0x0050C2, 0x7B7000, 0x7B7FFF, "TATTILE SRL", "TATTILE-SRL" },
	{ 0x0050C2, 0x7B8000, 0x7B8FFF, "Design 2000 Pty Ltd", "DESIGN-2000-PTY" },
	{ 0x0050C2, 0x7B9000, 0x7B9FFF, "Technovare Systems, Inc.", "TECHNOVARE" },
	{ 0x0050C2, 0x7BA000, 0x7BAFFF, "Infodev Electronic Designers Intl.", "INFODEV-DESIGNERS-INTL" },
	{ 0x0050C2, 0x7BB000, 0x7BBFFF, "InRay Solutions Ltd.", "INRAY-SOLUTIONS" },
	{ 0x0050C2, 0x7BC000, 0x7BCFFF, "EIDOS SPA", "EIDOS-SPA" },
	{ 0x0050C2, 0x7BD000, 0x7BDFFF, "PROMATE ELECTRONIC CO.LTD", "PROMATE" },
	{ 0x0050C2, 0x7BE000, 0x7BEFFF, "Powerlinx, Inc.", "POWERLINX" },
	{ 0x0050C2, 0x7BF000, 0x7BFFFF, "Zoe Medical", "ZOE-MEDICAL" },
	{ 0x0050C2, 0x7C0000, 0x7C0FFF, "European Industrial Electronics B.V.", "EUROPEAN-INDUSTRIAL-ELECTRONICS" },
	{ 0x0050C2, 0x7C1000, 0x7C1FFF, "Primary Integration Encorp LLC", "PRIMARY-INTEGRATION-ENCORP-LLC" },
	{ 0x0050C2, 0x7C2000, 0x7C2FFF, "DSR Information Technologies Ltd.", "DSR-INFORMATION-TECHNOLOGIES" },
	{ 0x0050C2, 0x7C3000, 0x7C3FFF, "AST INCORPORATED", "AST-INCORPORATED" },
	{ 0x0050C2, 0x7C4000, 0x7C4FFF, "MoBaCon", "MOBACON" },
	{ 0x0050C2, 0x7C5000, 0x7C5FFF, "Venture Research Inc.", "VENTURE-RESEARCH" },
	{ 0x0050C2, 0x7C6000, 0x7C6FFF, "Lyngdorf Audio Aps", "LYNGDORF-AUDIO-APS" },
	{ 0x0050C2, 0x7C7000, 0x7C7FFF, "Pyrosequencing AB", "PYROSEQUENCING-AB" },
	{ 0x0050C2, 0x7C8000, 0x7C8FFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0x7C9000, 0x7C9FFF, "Bluebell Opticom Limited", "BLUEBELL-OPTICOM" },
	{ 0x0050C2, 0x7CA000, 0x7CAFFF, "CEDAR Audio Limited", "CEDAR-AUDIO" },
	{ 0x0050C2, 0x7CB000, 0x7CBFFF, "ViewPlus Technologies, Inc.", "VIEWPLUS-TECHNOLOGIES" },
	{ 0x0050C2, 0x7CC000, 0x7CCFFF, "SWECO JAPS AB", "SWECO-JAPS-AB" },
	{ 0x0050C2, 0x7CD000, 0x7CDFFF, "Precision MicroControl Corporation", "PRECISION-MICROCONTROL" },
	{ 0x0050C2, 0x7CE000, 0x7CEFFF, "AirCell Inc.", "AIRCELL" },
	{ 0x0050C2, 0x7CF000, 0x7CFFFF, "Emitech Corporation", "EMITECH" },
	{ 0x0050C2, 0x7D0000, 0x7D0FFF, "Radar Tronic ltd.", "RADAR-TRONIC" },
	{ 0x0050C2, 0x7D1000, 0x7D1FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x7D2000, 0x7D2FFF, "Bittitalo Oy", "BITTITALO-OY" },
	{ 0x0050C2, 0x7D3000, 0x7D3FFF, "Highrail Systems Limited", "HIGHRAIL" },
	{ 0x0050C2, 0x7D4000, 0x7D4FFF, "WR Systems, Ltd.", "WR" },
	{ 0x0050C2, 0x7D5000, 0x7D5FFF, "Deuta-Werke GmbH", "DEUTA-WERKE" },
	{ 0x0050C2, 0x7D6000, 0x7D6FFF, "International Mining Technologies", "MINING-TECHNOLOGIES" },
	{ 0x0050C2, 0x7D7000, 0x7D7FFF, "Newtec A/S", "NEWTEC-A/S" },
	{ 0x0050C2, 0x7D8000, 0x7D8FFF, "InnoScan K/S", "INNOSCAN-K/S" },
	{ 0x0050C2, 0x7D9000, 0x7D9FFF, "Volumatic Limited", "VOLUMATIC" },
	{ 0x0050C2, 0x7DA000, 0x7DAFFF, "HTEC Limited", "HTEC" },
	{ 0x0050C2, 0x7DB000, 0x7DBFFF, "Mueller Elektronik", "MUELLER-ELEKTRONIK" },
	{ 0x0050C2, 0x7DC000, 0x7DCFFF, "aiXtrusion GmbH", "AIXTRUSION" },
	{ 0x0050C2, 0x7DD000, 0x7DDFFF, "LS Elektronik AB", "LS-ELEKTRONIK-AB" },
	{ 0x0050C2, 0x7DE000, 0x7DEFFF, "Cascade Technologies Ltd", "CASCADE-TECHNOLOGIES" },
	{ 0x0050C2, 0x7DF000, 0x7DFFFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x7E0000, 0x7E0FFF, "C&D Technologies, Inc", "C-D-TECHNOLOGIES" },
	{ 0x0050C2, 0x7E1000, 0x7E1FFF, "Zeltiq Aesthetics, Inc.", "ZELTIQ-AESTHETICS" },
	{ 0x0050C2, 0x7E2000, 0x7E2FFF, "DIGITROL LTD", "DIGITROL" },
	{ 0x0050C2, 0x7E3000, 0x7E3FFF, "Progentech Limited", "PROGENTECH" },
	{ 0x0050C2, 0x7E4000, 0x7E4FFF, "Meta Vision Systems Ltd.", "META-VISION" },
	{ 0x0050C2, 0x7E5000, 0x7E5FFF, "Nystrom Engineering", "NYSTROM-ENGINEERING" },
	{ 0x0050C2, 0x7E6000, 0x7E6FFF, "Empirix Italy S.p.A.", "EMPIRIX-ITALY-S-P-A" },
	{ 0x0050C2, 0x7E7000, 0x7E7FFF, "V2Green, Inc.", "V2GREEN" },
	{ 0x0050C2, 0x7E8000, 0x7E8FFF, "Mistral Solutions Pvt. Ltd", "MISTRAL-SOLUTIONS-PVT" },
	{ 0x0050C2, 0x7E9000, 0x7E9FFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0x7EA000, 0x7EAFFF, "Monitor Business Machines Ltd.", "MONITOR-BUSINESS-MACHINES" },
	{ 0x0050C2, 0x7EB000, 0x7EBFFF, "Sesol Industrial Computer", "SESOL-INDUSTRIAL" },
	{ 0x0050C2, 0x7EC000, 0x7ECFFF, "Lyngsoe Systems", "LYNGSOE" },
	{ 0x0050C2, 0x7ED000, 0x7EDFFF, "Genesis Automation Inc.", "GENESIS-AUTOMATION" },
	{ 0x0050C2, 0x7EE000, 0x7EEFFF, "NH Research", "NH-RESEARCH" },
	{ 0x0050C2, 0x7EF000, 0x7EFFFF, "GFI Chrono Time", "GFI-CHRONO-TIME" },
	{ 0x0050C2, 0x7F0000, 0x7F0FFF, "Network Harbor, Inc.", "NETWORK-HARBOR" },
	{ 0x0050C2, 0x7F1000, 0x7F1FFF, "STUHL Regelsysteme GmbH", "STUHL-REGELSYSTEME" },
	{ 0x0050C2, 0x7F2000, 0x7F2FFF, "Logotherm Regelsysteme GmbH", "LOGOTHERM-REGELSYSTEME" },
	{ 0x0050C2, 0x7F3000, 0x7F3FFF, "SOREC", "SOREC" },
	{ 0x0050C2, 0x7F4000, 0x7F4FFF, "Wireless Cables Inc", "WIRELESS-CABLES" },
	{ 0x0050C2, 0x7F5000, 0x7F5FFF, "ACE Carwash Systems", "ACE-CARWASH" },
	{ 0x0050C2, 0x7F6000, 0x7F6FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x7F7000, 0x7F7FFF, "MangoDSP", "MANGODSP" },
	{ 0x0050C2, 0x7F8000, 0x7F8FFF, "Wise Industria de Telecomunicações Ldta.", "WISE-INDUSTRIA-DE-TELECOMUNICAçõES-LDTA" },
	{ 0x0050C2, 0x7F9000, 0x7F9FFF, "Karl DUNGS GmbH & Co. KG", "KARL-DUNGS" },
	{ 0x0050C2, 0x7FA000, 0x7FAFFF, "AutomationX GmbH", "AUTOMATIONX" },
	{ 0x0050C2, 0x7FB000, 0x7FBFFF, "Qtron Pty Ltd", "QTRON-PTY" },
	{ 0x0050C2, 0x7FC000, 0x7FCFFF, "TIS Dialog LLC", "TIS-DIALOG-LLC" },
	{ 0x0050C2, 0x7FD000, 0x7FDFFF, "Adeneo", "ADENEO" },
	{ 0x0050C2, 0x7FE000, 0x7FEFFF, "Wireless Cables Inc.", "WIRELESS-CABLES" },
	{ 0x0050C2, 0x7FF000, 0x7FFFFF, "Shenzhen MaiWei Cable TV Equipment CO.,LTD.", "SHENZHEN-MAIWEI-CABLE-TV-EQUIPMENT" },
	{ 0x0050C2, 0x800000, 0x800FFF, "Delphi Display Systems, Inc.", "DELPHI-DISPLAY" },
	{ 0x0050C2, 0x801000, 0x801FFF, "JANUS srl", "JANUS-SRL" },
	{ 0x0050C2, 0x802000, 0x802FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0x803000, 0x803FFF, "dB Broadcast Limited", "DB-BROADCAST" },
	{ 0x0050C2, 0x804000, 0x804FFF, "SoftSwitching Technologies", "SOFTSWITCHING-TECHNOLOGIES" },
	{ 0x0050C2, 0x805000, 0x805FFF, "MultimediaLED", "MULTIMEDIALED" },
	{ 0x0050C2, 0x806000, 0x806FFF, "CET", "CET" },
	{ 0x0050C2, 0x807000, 0x807FFF, "TECHNOMARK", "TECHNOMARK" },
	{ 0x0050C2, 0x808000, 0x808FFF, "ITB CompuPhase", "ITB-COMPUPHASE" },
	{ 0x0050C2, 0x809000, 0x809FFF, "Varma Electronics Oy", "VARMA-ELECTRONICS-OY" },
	{ 0x0050C2, 0x80A000, 0x80AFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x80B000, 0x80BFFF, "Open Video, Inc.", "OPEN-VIDEO" },
	{ 0x0050C2, 0x80C000, 0x80CFFF, "Luxpert Technologies Co., Ltd.", "LUXPERT-TECHNOLOGIES" },
	{ 0x0050C2, 0x80D000, 0x80DFFF, "Acube Systems s.r.l.", "ACUBE-S-R-L" },
	{ 0x0050C2, 0x80E000, 0x80EFFF, "Bruno International Ltd.", "BRUNO" },
	{ 0x0050C2, 0x80F000, 0x80FFFF, "Selekron Microcontrol s.l.", "SELEKRON-MICROCONTROL-S-L" },
	{ 0x0050C2, 0x810000, 0x810FFF, "Alphion Corporation", "ALPHION" },
	{ 0x0050C2, 0x811000, 0x811FFF, "Open System Solutions Limited", "OPEN-SYSTEM-SOLUTIONS" },
	{ 0x0050C2, 0x812000, 0x812FFF, "Femto SA", "FEMTO" },
	{ 0x0050C2, 0x813000, 0x813FFF, "Intelleflex Corporation", "INTELLEFLEX" },
	{ 0x0050C2, 0x814000, 0x814FFF, "Telvent", "TELVENT" },
	{ 0x0050C2, 0x815000, 0x815FFF, "microC Design SRL", "MICROC-DESIGN-SRL" },
	{ 0x0050C2, 0x816000, 0x816FFF, "Intelight Inc.", "INTELIGHT" },
	{ 0x0050C2, 0x817000, 0x817FFF, "Odin TeleSystems Inc", "ODIN-TELESYSTEMS" },
	{ 0x0050C2, 0x818000, 0x818FFF, "Wireless Value BV", "WIRELESS-VALUE" },
	{ 0x0050C2, 0x819000, 0x819FFF, "Cabinplant A/S", "CABINPLANT-A/S" },
	{ 0x0050C2, 0x81A000, 0x81AFFF, "InfoGLOBAL", "INFOGLOBAL" },
	{ 0x0050C2, 0x81B000, 0x81BFFF, "Brain Tech Co., Ltd", "BRAIN-TECH" },
	{ 0x0050C2, 0x81C000, 0x81CFFF, "Telcom", "TELCOM" },
	{ 0x0050C2, 0x81D000, 0x81DFFF, "IT SALUX CO., LTD.", "IT-SALUX" },
	{ 0x0050C2, 0x81E000, 0x81EFFF, "Channelot Ltd.", "CHANNELOT" },
	{ 0x0050C2, 0x81F000, 0x81FFFF, "2N TELEKOMUNIKACE a.s.", "2N-TELEKOMUNIKACE-A-S" },
	{ 0x0050C2, 0x820000, 0x820FFF, "TESCAN, s.r.o.", "TESCAN-S-R-O" },
	{ 0x0050C2, 0x821000, 0x821FFF, "MISCO Refractometer", "MISCO-REFRACTOMETER" },
	{ 0x0050C2, 0x822000, 0x822FFF, "Winner Technology Co, Ltd.", "WINNER" },
	{ 0x0050C2, 0x823000, 0x823FFF, "Robot Visual Systems GmbH", "ROBOT-VISUAL" },
	{ 0x0050C2, 0x824000, 0x824FFF, "SMT d.o.o.", "SMT-D-O-O" },
	{ 0x0050C2, 0x825000, 0x825FFF, "Funkwerk Information Technologies Karlsfeld GmbH", "FUNKWERK-INFORMATION-TECHNOLOGIES-KARLSFELD" },
	{ 0x0050C2, 0x826000, 0x826FFF, "HEWI Heinrich Wilke GmbH", "HEWI-HEINRICH-WILKE" },
	{ 0x0050C2, 0x827000, 0x827FFF, "Enero Solutions inc.", "ENERO-SOLUTIONS" },
	{ 0x0050C2, 0x828000, 0x828FFF, "SLICAN sp. z o.o.", "SLICAN-SP-Z-O-O" },
	{ 0x0050C2, 0x829000, 0x829FFF, "Intellectronika", "INTELLECTRONIKA" },
	{ 0x0050C2, 0x82A000, 0x82AFFF, "VDC Display Systems", "VDC-DISPLAY" },
	{ 0x0050C2, 0x82B000, 0x82BFFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x82C000, 0x82CFFF, "Vitel Net", "VITEL-NET" },
	{ 0x0050C2, 0x82D000, 0x82DFFF, "Elmec, Inc.", "ELMEC" },
	{ 0x0050C2, 0x82E000, 0x82EFFF, "LogiCom GmbH", "LOGICOM" },
	{ 0x0050C2, 0x82F000, 0x82FFFF, "Momentum Data Systems", "MOMENTUM-DATA" },
	{ 0x0050C2, 0x830000, 0x830FFF, "CompuShop Services LLC", "COMPUSHOP-SERVICES-LLC" },
	{ 0x0050C2, 0x831000, 0x831FFF, "St Jude Medical, Inc.", "ST-JUDE-MEDICAL" },
	{ 0x0050C2, 0x832000, 0x832FFF, "S1nn GmbH & Co. KG", "S1NN" },
	{ 0x0050C2, 0x833000, 0x833FFF, "LaserLinc, Inc.", "LASERLINC" },
	{ 0x0050C2, 0x834000, 0x834FFF, "ANTEK GmbH", "ANTEK" },
	{ 0x0050C2, 0x835000, 0x835FFF, "Communications Laboratories Inc", "COMMUNICATION-LABORATORIES" },
	{ 0x0050C2, 0x836000, 0x836FFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x0050C2, 0x837000, 0x837FFF, "ID-KARTA s.r.o.", "ID-KARTA-S-R-O" },
	{ 0x0050C2, 0x838000, 0x838FFF, "T PROJE MUHENDISLIK DIS. TIC. LTD. STI.", "T-PROJE-MUHENDISLIK-DIS-TIC-STI" },
	{ 0x0050C2, 0x839000, 0x839FFF, "IMS Röntgensysteme GmbH", "IMS-ROENTGENSYSTEME" },
	{ 0x0050C2, 0x83A000, 0x83AFFF, "Syr-Tec Engineering & Marketing", "SYR-TEC-ENGINEERING-MARKETING" },
	{ 0x0050C2, 0x83B000, 0x83BFFF, "O. Bay AG", "O-BAY" },
	{ 0x0050C2, 0x83C000, 0x83CFFF, "hema electronic GmbH", "HEMA" },
	{ 0x0050C2, 0x83D000, 0x83DFFF, "beroNet GmbH", "BERONET" },
	{ 0x0050C2, 0x83E000, 0x83EFFF, "KPE spol. s r.o.", "KPE-SPOL-S-R-O" },
	{ 0x0050C2, 0x83F000, 0x83FFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x840000, 0x840FFF, "Residential Control Systems", "RESIDENTIAL-CONTROL" },
	{ 0x0050C2, 0x841000, 0x841FFF, "Connection Electronics Ltd.", "CONNECTION-ELECTRONICS" },
	{ 0x0050C2, 0x842000, 0x842FFF, "Quantum Controls BV", "QUANTUM-CONTROLS" },
	{ 0x0050C2, 0x843000, 0x843FFF, "Xtensor Systems Inc.", "XTENSOR" },
	{ 0x0050C2, 0x844000, 0x844FFF, "Prodigy Electronics Limited", "PRODIGY-ELECTRONICS" },
	{ 0x0050C2, 0x845000, 0x845FFF, "VisualSonics Inc.", "VISUALSONICS" },
	{ 0x0050C2, 0x846000, 0x846FFF, "ESP-Planning Co.", "ESP-PLANNING" },
	{ 0x0050C2, 0x847000, 0x847FFF, "Lars Morich Kommunikationstechnik GmbH", "LARS-MORICH-KOMMUNIKATIONSTECHNIK" },
	{ 0x0050C2, 0x848000, 0x848FFF, "DASA ROBOT Co., Ltd.", "DASA-ROBOT" },
	{ 0x0050C2, 0x849000, 0x849FFF, "Design Analysis Associates, Inc.", "DESIGN-ANALYSIS-ASSOCIATES" },
	{ 0x0050C2, 0x84A000, 0x84AFFF, "Keystone Electronic Solutions", "KEYSTONE-SOLUTIONS" },
	{ 0x0050C2, 0x84B000, 0x84BFFF, "TASK SISTEMAS DE COMPUTACAO LTDA", "TASK-SISTEMAS-DE-COMPUTACAO-LTDA" },
	{ 0x0050C2, 0x84C000, 0x84CFFF, "Performance Motion Devices", "PERFORMANCE-MOTION-DEVICES" },
	{ 0x0050C2, 0x84D000, 0x84DFFF, "BMTI", "BMTI" },
	{ 0x0050C2, 0x84E000, 0x84EFFF, "DRACO SYSTEMS", "DRACO" },
	{ 0x0050C2, 0x84F000, 0x84FFFF, "Gamber-Johnson LLC", "GAMBER-JOHNSON-LLC" },
	{ 0x0050C2, 0x850000, 0x850FFF, "K.K. Rocky", "K-K-ROCKY" },
	{ 0x0050C2, 0x851000, 0x851FFF, "SPJ Embedded Technologies Pvt. Ltd.", "SPJ-EMBEDDED-TECHNOLOGIES-PVT" },
	{ 0x0050C2, 0x852000, 0x852FFF, "eInfochips Ltd.", "EINFOCHIPS" },
	{ 0x0050C2, 0x853000, 0x853FFF, "Ettus Research LLC", "ETTUS-RESEARCH-LLC" },
	{ 0x0050C2, 0x854000, 0x854FFF, "Ratioplast-Optoelectronics GmbH", "RATIOPLAST-OPTOELECTRONICS" },
	{ 0x0050C2, 0x855000, 0x855FFF, "Rohde & Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0x856000, 0x856FFF, "CT Company", "CT-COMPANY" },
	{ 0x0050C2, 0x857000, 0x857FFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0x858000, 0x858FFF, "Wireless Acquisition LLC", "WIRELESS-ACQUISITION-LLC" },
	{ 0x0050C2, 0x859000, 0x859FFF, "Nuvation", "NUVATION" },
	{ 0x0050C2, 0x85A000, 0x85AFFF, "ART s.r.l.", "ART-S-R-L" },
	{ 0x0050C2, 0x85B000, 0x85BFFF, "Boreste", "BORESTE" },
	{ 0x0050C2, 0x85C000, 0x85CFFF, "B S E", "B-S-E" },
	{ 0x0050C2, 0x85D000, 0x85DFFF, "Ing. Knauseder Mechatronik GmbH", "ING-KNAUSEDER-MECHATRONIK" },
	{ 0x0050C2, 0x85E000, 0x85EFFF, "Radiometer Medical ApS", "RADIOMETER-MEDICAL-APS" },
	{ 0x0050C2, 0x85F000, 0x85FFFF, "General Dynamics C4 Systems", "GENERAL-DYNAMICS-C4" },
	{ 0x0050C2, 0x860000, 0x860FFF, "Eutron S.p.A.", "EUTRON-S-P-A" },
	{ 0x0050C2, 0x861000, 0x861FFF, "Grantronics Pty Ltd", "GRANTRONICS-PTY" },
	{ 0x0050C2, 0x862000, 0x862FFF, "Elsys AG", "ELSYS" },
	{ 0x0050C2, 0x863000, 0x863FFF, "Advanced Technology Solutions", "ADVANCED-SOLUTIONS" },
	{ 0x0050C2, 0x864000, 0x864FFF, "ATG Automatisierungstechnik GERA GmbH", "ATG-AUTOMATISIERUNGSTECHNIK-GERA" },
	{ 0x0050C2, 0x865000, 0x865FFF, "Persy Control Services B.v.", "PERSY-CONTROL-SERVICES" },
	{ 0x0050C2, 0x866000, 0x866FFF, "Saia Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x867000, 0x867FFF, "Syntronics", "SYNTRONICS" },
	{ 0x0050C2, 0x868000, 0x868FFF, "Aethon, Inc.", "AETHON" },
	{ 0x0050C2, 0x869000, 0x869FFF, "Funkwerk plettac electronic GmbH", "FUNKWERK-PLETTAC" },
	{ 0x0050C2, 0x86A000, 0x86AFFF, "USM Systems, Ltd", "USM" },
	{ 0x0050C2, 0x86B000, 0x86BFFF, "OMB Sistemas Electronicos S.A.", "OMB-SISTEMAS-ELECTRONICOS" },
	{ 0x0050C2, 0x86C000, 0x86CFFF, "Condigi Televagt A/S", "CONDIGI-TELEVAGT-A/S" },
	{ 0x0050C2, 0x86D000, 0x86DFFF, "Tieline Research Pty Ltd", "TIELINE-RESEARCH-PTY" },
	{ 0x0050C2, 0x86E000, 0x86EFFF, "HANYANG ELECTRIC CP., LTD", "HANYANG-ELECTRIC-CP" },
	{ 0x0050C2, 0x86F000, 0x86FFFF, "b-plus GmbH", "B-PLUS" },
	{ 0x0050C2, 0x870000, 0x870FFF, "LOGEL S.R.L.", "LOGEL-S-R-L" },
	{ 0x0050C2, 0x871000, 0x871FFF, "R-S-I Elektrotechnik GmbH & Co. KG", "R-S-I-ELEKTROTECHNIK" },
	{ 0x0050C2, 0x872000, 0x872FFF, "Oliotalo Oy", "OLIOTALO-OY" },
	{ 0x0050C2, 0x873000, 0x873FFF, "XRONET Corporation", "XRONET" },
	{ 0x0050C2, 0x874000, 0x874FFF, "Arcos Technologies Ltd.", "ARCOS-TECHNOLOGIES" },
	{ 0x0050C2, 0x875000, 0x875FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x876000, 0x876FFF, "Privatquelle Gruber GmbH & CO KG", "PRIVATQUELLE-GRUBER" },
	{ 0x0050C2, 0x877000, 0x877FFF, "Motion Analysis Corp", "MOTION-ANALYSIS" },
	{ 0x0050C2, 0x878000, 0x878FFF, "Acoustic Research Laboratories Pty Ltd", "ACOUSTIC-RESEARCH-LABORATORIES-PTY" },
	{ 0x0050C2, 0x879000, 0x879FFF, "MILESYS", "MILESYS" },
	{ 0x0050C2, 0x87A000, 0x87AFFF, "Spectrum Management, LC", "SPECTRUM-MANAGEMENT-LC" },
	{ 0x0050C2, 0x87B000, 0x87BFFF, "UAVNavigation S.L.", "UAVNAVIGATION-S-L" },
	{ 0x0050C2, 0x87C000, 0x87CFFF, "Arcontia AB", "ARCONTIA-AB" },
	{ 0x0050C2, 0x87D000, 0x87DFFF, "AT&T Government Solutions", "AT-T-GOVERNMENT-SOLUTIONS" },
	{ 0x0050C2, 0x87E000, 0x87EFFF, "SCM PRODUCTS, INC.", "SCM-PRODUCTS" },
	{ 0x0050C2, 0x87F000, 0x87FFFF, "Optoelettronica Italia S.r.l.", "OPTOELETTRONICA-ITALIA-S-R-L" },
	{ 0x0050C2, 0x880000, 0x880FFF, "Creation Technologies Chicago", "CREATION-TECHNOLOGIES-CHICAGO" },
	{ 0x0050C2, 0x881000, 0x881FFF, "InnoTrans Communications, Inc.", "INNOTRANS-COMMUNICATION" },
	{ 0x0050C2, 0x882000, 0x882FFF, "WARECUBE,INC.", "WARECUBE" },
	{ 0x0050C2, 0x883000, 0x883FFF, "Neocontrol Soluções em Automação", "NEOCONTROL-SOLUçõES-EM-AUTOMAçãO" },
	{ 0x0050C2, 0x884000, 0x884FFF, "IP Thinking A/S", "IP-THINKING-A/S" },
	{ 0x0050C2, 0x885000, 0x885FFF, "OOO NTK IMOS", "OOO-NTK-IMOS" },
	{ 0x0050C2, 0x886000, 0x886FFF, "Transas Scandinavia AB", "TRANSAS-SCANDINAVIA-AB" },
	{ 0x0050C2, 0x887000, 0x887FFF, "Inventis Technology Pty Limited", "INVENTIS-PTY" },
	{ 0x0050C2, 0x888000, 0x888FFF, "IADEA CORPORATION", "IADEA" },
	{ 0x0050C2, 0x889000, 0x889FFF, "ACS MOTION CONTROL", "ACS-MOTION-CONTROL" },
	{ 0x0050C2, 0x88A000, 0x88AFFF, "Continental Electronics Corp.", "CONTINENTAL-ELECTRONICS" },
	{ 0x0050C2, 0x88B000, 0x88BFFF, "Hollis Electronics Company LLC", "HOLLIS-ELECTRONICS-COMPANY-LLC" },
	{ 0x0050C2, 0x88C000, 0x88CFFF, "Z-App Systems", "Z-APP" },
	{ 0x0050C2, 0x88D000, 0x88DFFF, "L3 Communications Nova Engineering", "L3-COMMUNICATION-NOVA-ENGINEERING" },
	{ 0x0050C2, 0x88E000, 0x88EFFF, "Cardinal Scale Mfg Co", "CARDINAL-SCALE-MFG" },
	{ 0x0050C2, 0x88F000, 0x88FFFF, "Keynote SIGOS GmbH", "KEYNOTE-SIGOS" },
	{ 0x0050C2, 0x890000, 0x890FFF, "BAE Systems Hägglunds AB", "BAE-HAEGGLUNDS-AB" },
	{ 0x0050C2, 0x891000, 0x891FFF, "Admiral Secure Products, Ltd.", "ADMIRAL-SECURE-PRODUCTS" },
	{ 0x0050C2, 0x892000, 0x892FFF, "Trakce a.s.", "TRAKCE-A-S" },
	{ 0x0050C2, 0x893000, 0x893FFF, "EIZO Technologies GmbH", "EIZO-TECHNOLOGIES" },
	{ 0x0050C2, 0x894000, 0x894FFF, "Shockfish SA", "SHOCKFISH" },
	{ 0x0050C2, 0x895000, 0x895FFF, "Marine Communications Limited", "MARINE-COMMUNICATION" },
	{ 0x0050C2, 0x896000, 0x896FFF, "Blankom", "BLANKOM" },
	{ 0x0050C2, 0x897000, 0x897FFF, "ODF Optronics, Inc.", "ODF-OPTRONICS" },
	{ 0x0050C2, 0x898000, 0x898FFF, "Veeco Process Equipment, Inc.", "VEECO-PROCESS-EQUIPMENT" },
	{ 0x0050C2, 0x899000, 0x899FFF, "Inico Technologies Ltd.", "INICO-TECHNOLOGIES" },
	{ 0x0050C2, 0x89A000, 0x89AFFF, "Neptune Technology Group, Inc.", "NEPTUNE-GROUP" },
	{ 0x0050C2, 0x89B000, 0x89BFFF, "Sensata Technologies, Inc.", "SENSATA-TECHNOLOGIES" },
	{ 0x0050C2, 0x89C000, 0x89CFFF, "Mediana", "MEDIANA" },
	{ 0x0050C2, 0x89D000, 0x89DFFF, "Systemtechnik GmbH", "SYSTEMTECHNIK" },
	{ 0x0050C2, 0x89E000, 0x89EFFF, "Broadcast Electronics", "BROADCAST-ELECTRONICS" },
	{ 0x0050C2, 0x89F000, 0x89FFFF, "Datalink Technologies Gateways Inc.", "DATALINK-TECHNOLOGIES-GATEWAYS" },
	{ 0x0050C2, 0x8A0000, 0x8A0FFF, "Specialized Communications Corp.", "SPECIALIZED-COMMUNICATION" },
	{ 0x0050C2, 0x8A1000, 0x8A1FFF, "Intune Networks Limited", "INTUNE-NETWORKS" },
	{ 0x0050C2, 0x8A2000, 0x8A2FFF, "UAVISION Engenharia de Sistemas", "UAVISION-ENGENHARIA-DE-SISTEMAS" },
	{ 0x0050C2, 0x8A3000, 0x8A3FFF, "RTW GmbH & Co.KG", "RTW" },
	{ 0x0050C2, 0x8A4000, 0x8A4FFF, "BALOGH T.A.G Corporation", "BALOGH-T-A-G" },
	{ 0x0050C2, 0x8A5000, 0x8A5FFF, "Mocon, Inc.", "MOCON" },
	{ 0x0050C2, 0x8A6000, 0x8A6FFF, "SELCO", "SELCO" },
	{ 0x0050C2, 0x8A7000, 0x8A7FFF, "PIXEYE LTD", "PIXEYE" },
	{ 0x0050C2, 0x8A8000, 0x8A8FFF, "ALTEK ELECTRONICS", "ALTEK-ELECTRONICS" },
	{ 0x0050C2, 0x8A9000, 0x8A9FFF, "Intelligent Security Systems", "INTELLIGENT-SECURITY" },
	{ 0x0050C2, 0x8AA000, 0x8AAFFF, "ATS Elektronik GmbH", "ATS-ELEKTRONIK" },
	{ 0x0050C2, 0x8AB000, 0x8ABFFF, "Nanomotion Ltd.", "NANOMOTION" },
	{ 0x0050C2, 0x8AC000, 0x8ACFFF, "Telsa s.r.l", "TELSA-S-R-L" },
	{ 0x0050C2, 0x8AD000, 0x8ADFFF, "Thales Communications, Inc", "THALES-COMMUNICATION" },
	{ 0x0050C2, 0x8AE000, 0x8AEFFF, "DESARROLLO DE SISTEMAS INTEGRADOS DE CONTROL S.A.", "DESARROLLO-DE-SISTEMAS-INTEGRADOS-DE-CONTROL" },
	{ 0x0050C2, 0x8AF000, 0x8AFFFF, "Xelerated", "XELERATED" },
	{ 0x0050C2, 0x8B0000, 0x8B0FFF, "BK Innovation, Inc.", "BK-INNOVATION" },
	{ 0x0050C2, 0x8B1000, 0x8B1FFF, "RingCube Technologies, Inc.", "RINGCUBE-TECHNOLOGIES" },
	{ 0x0050C2, 0x8B2000, 0x8B2FFF, "SERVAIND SA.", "SERVAIND" },
	{ 0x0050C2, 0x8B3000, 0x8B3FFF, "VTQ Videtronik GmbH", "VTQ-VIDETRONIK" },
	{ 0x0050C2, 0x8B4000, 0x8B4FFF, "Sandar Telecast AS", "SANDAR-TELECAST-AS" },
	{ 0x0050C2, 0x8B5000, 0x8B5FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x8B6000, 0x8B6FFF, "Shadrinskiy Telefonny Zavod", "SHADRINSKIY-TELEFONNY-ZAVOD" },
	{ 0x0050C2, 0x8B7000, 0x8B7FFF, "Calnex Solutions Limited", "CALNEX-SOLUTIONS" },
	{ 0x0050C2, 0x8B8000, 0x8B8FFF, "DSS Networks, Inc.", "DSS-NETWORKS" },
	{ 0x0050C2, 0x8B9000, 0x8B9FFF, "ACD Elektronik Gmbh", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0x8BA000, 0x8BAFFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0x8BB000, 0x8BBFFF, "smtag international ag", "SMTAG" },
	{ 0x0050C2, 0x8BC000, 0x8BCFFF, "Honeywell Sensotec", "HONEYWELL-SENSOTEC" },
	{ 0x0050C2, 0x8BD000, 0x8BDFFF, "Matrix Switch Corporation", "MATRIX-SWITCH" },
	{ 0x0050C2, 0x8BE000, 0x8BEFFF, "The Pennsylvania State University", "THE-PENNSYLVANIA-STATE-UNIVERSITY" },
	{ 0x0050C2, 0x8BF000, 0x8BFFFF, "ARISTO Graphic Systeme GmbH & Co. KG", "ARISTO-GRAPHIC-SYSTEME" },
	{ 0x0050C2, 0x8C0000, 0x8C0FFF, "S.C.E. s.r.l.", "S-C-E-S-R-L" },
	{ 0x0050C2, 0x8C1000, 0x8C1FFF, "Heraeus Noblelight GmbH", "HERAEUS-NOBLELIGHT" },
	{ 0x0050C2, 0x8C2000, 0x8C2FFF, "Access Control Systems JSC", "ACCESS-CONTROL-JSC" },
	{ 0x0050C2, 0x8C3000, 0x8C3FFF, "Byte Paradigm", "BYTE-PARADIGM" },
	{ 0x0050C2, 0x8C4000, 0x8C4FFF, "Soldig Industria e Comercio de Equipamentos Eletronicos LTDA", "SOLDIG-INDUSTRIA-E-COMERCIO-DE-EQUIPAMENTOS-ELETRONICOS-LTDA" },
	{ 0x0050C2, 0x8C5000, 0x8C5FFF, "Vortex Engineering pvt ltd", "VORTEX-ENGINEERING-PVT" },
	{ 0x0050C2, 0x8C6000, 0x8C6FFF, "Gradual Tecnologia Ltda.", "GRADUAL-TECNOLOGIA-LTDA" },
	{ 0x0050C2, 0x8C7000, 0x8C7FFF, "Tattile srl", "TATTILE-SRL" },
	{ 0x0050C2, 0x8C8000, 0x8C8FFF, "Pumatronix Equipamentos Eletrônicos Ltda", "PUMATRONIX-EQUIPAMENTOS-ELETRôNICOS-LTDA" },
	{ 0x0050C2, 0x8C9000, 0x8C9FFF, "A+S Aktuatorik und Sensorik GmbH", "A+S-AKTUATORIK-UND-SENSORIK" },
	{ 0x0050C2, 0x8CA000, 0x8CAFFF, "Altair semiconductor Ltd", "ALTAIR-SEMICONDUCTOR" },
	{ 0x0050C2, 0x8CB000, 0x8CBFFF, "Beonic Corporation", "BEONIC" },
	{ 0x0050C2, 0x8CC000, 0x8CCFFF, "LyconSys GmbH & Co.KG", "LYCONSYS" },
	{ 0x0050C2, 0x8CD000, 0x8CDFFF, "Cambridge Sound Management, LLC", "CAMBRIDGE-SOUND-MANAGEMENT-LLC" },
	{ 0x0050C2, 0x8CE000, 0x8CEFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x8CF000, 0x8CFFFF, "GigaLinx Ltd.", "GIGALINX" },
	{ 0x0050C2, 0x8D0000, 0x8D0FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x8D1000, 0x8D1FFF, "Bachmann Monitoring GmbH", "BACHMANN-MONITORING" },
	{ 0x0050C2, 0x8D2000, 0x8D2FFF, "TTi Ltd", "TTI" },
	{ 0x0050C2, 0x8D3000, 0x8D3FFF, "IFAM GmbH", "IFAM" },
	{ 0x0050C2, 0x8D4000, 0x8D4FFF, "Internet Protocolo Lógica SL", "INTERNET-PROTOCOLO-LóGICA-SL" },
	{ 0x0050C2, 0x8D5000, 0x8D5FFF, "Peek Traffic Corp", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0x8D6000, 0x8D6FFF, "UltraVision Security Systems, Inc.", "ULTRAVISION-SECURITY" },
	{ 0x0050C2, 0x8D7000, 0x8D7FFF, "Polygon Informatics Ltd.", "POLYGON-INFORMATICS" },
	{ 0x0050C2, 0x8D8000, 0x8D8FFF, "Array Technologies Inc", "ARRAY-TECHNOLOGIES" },
	{ 0x0050C2, 0x8D9000, 0x8D9FFF, "Industrial Control and Communication Limited", "INDUSTRIAL-CONTROL-AND-COMMUNICATION" },
	{ 0x0050C2, 0x8DA000, 0x8DAFFF, "DOCUTEMP, INC", "DOCUTEMP" },
	{ 0x0050C2, 0x8DB000, 0x8DBFFF, "DCOM Network Technology (Pty) Ltd", "DCOM-NETWORK-PTY" },
	{ 0x0050C2, 0x8DC000, 0x8DCFFF, "Frame Systems Limited", "FRAME" },
	{ 0x0050C2, 0x8DD000, 0x8DDFFF, "GIMCON", "GIMCON" },
	{ 0x0050C2, 0x8DE000, 0x8DEFFF, "Coherix, Inc", "COHERIX" },
	{ 0x0050C2, 0x8DF000, 0x8DFFFF, "Dipl.-Ing. W. Nophut GmbH", "DIPL--ING-W-NOPHUT" },
	{ 0x0050C2, 0x8E0000, 0x8E0FFF, "Shenzhen Pennda Technologies Co., Ltd.", "SHENZHEN-PENNDA-TECHNOLOGIES" },
	{ 0x0050C2, 0x8E1000, 0x8E1FFF, "Deutscher Weterdienst", "DEUTSCHER-WETERDIENST" },
	{ 0x0050C2, 0x8E2000, 0x8E2FFF, "Wireless Cables Inc", "WIRELESS-CABLES" },
	{ 0x0050C2, 0x8E3000, 0x8E3FFF, "bioMérieux Italia S.p.A.", "BIOMéRIEUX-ITALIA-S-P-A" },
	{ 0x0050C2, 0x8E4000, 0x8E4FFF, "MaCaPS International Limited", "MACAPS" },
	{ 0x0050C2, 0x8E5000, 0x8E5FFF, "Berthel GmbH", "BERTHEL" },
	{ 0x0050C2, 0x8E6000, 0x8E6FFF, "Sandel Avionics, Inc.", "SANDEL-AVIONICS" },
	{ 0x0050C2, 0x8E7000, 0x8E7FFF, "MKT Systemtechnik", "MKT-SYSTEMTECHNIK" },
	{ 0x0050C2, 0x8E8000, 0x8E8FFF, "Friedrich Kuhnt GmbH", "FRIEDRICH-KUHNT" },
	{ 0x0050C2, 0x8E9000, 0x8E9FFF, "UNIDATA", "UNIDATA" },
	{ 0x0050C2, 0x8EA000, 0x8EAFFF, "ATEME", "ATEME" },
	{ 0x0050C2, 0x8EB000, 0x8EBFFF, "C-COM Satellite Systems Inc.", "C-COM-SATELLITE" },
	{ 0x0050C2, 0x8EC000, 0x8ECFFF, "Balfour Beatty Rail GmbH", "BALFOUR-BEATTY-RAIL" },
	{ 0x0050C2, 0x8ED000, 0x8EDFFF, "AT-Automation Technology GmbH", "AT-AUTOMATION" },
	{ 0x0050C2, 0x8EE000, 0x8EEFFF, "PCSC", "PCSC" },
	{ 0x0050C2, 0x8EF000, 0x8EFFFF, "Technologies Sensio Inc", "TECHNOLOGIES-SENSIO" },
	{ 0x0050C2, 0x8F0000, 0x8F0FFF, "Xentras Communications", "XENTRAS-COMMUNICATION" },
	{ 0x0050C2, 0x8F1000, 0x8F1FFF, "Detection Technologies Ltd.", "DETECTION-TECHNOLOGIES" },
	{ 0x0050C2, 0x8F2000, 0x8F2FFF, "Schneider Electric GmbH", "SCHNEIDER-ELECTRIC" },
	{ 0x0050C2, 0x8F3000, 0x8F3FFF, "Curtis Door Systems Inc", "CURTIS-DOOR" },
	{ 0x0050C2, 0x8F4000, 0x8F4FFF, "Critical Link", "CRITICAL-LINK" },
	{ 0x0050C2, 0x8F5000, 0x8F5FFF, "tec5 AG", "TEC5" },
	{ 0x0050C2, 0x8F6000, 0x8F6FFF, "K-MAC Corp.", "K-MAC" },
	{ 0x0050C2, 0x8F7000, 0x8F7FFF, "TGE Co., Ltd.", "TGE" },
	{ 0x0050C2, 0x8F8000, 0x8F8FFF, "RMSD LTD", "RMSD" },
	{ 0x0050C2, 0x8F9000, 0x8F9FFF, "Honeywell International", "HONEYWELL" },
	{ 0x0050C2, 0x8FA000, 0x8FAFFF, "TELIUM s.c.", "TELIUM-S-C" },
	{ 0x0050C2, 0x8FB000, 0x8FBFFF, "Alfred Kuhse GmbH", "ALFRED-KUHSE" },
	{ 0x0050C2, 0x8FC000, 0x8FCFFF, "Symetrics Industries", "SYMETRICS-INDUSTRIES" },
	{ 0x0050C2, 0x8FD000, 0x8FDFFF, "Sindoma Müh Mim Ýnþ Elk San Tic Ltd.", "SINDOMA-MUEH-MIM-ÝNþ-ELK-SAN-TIC" },
	{ 0x0050C2, 0x8FE000, 0x8FEFFF, "Cross Country Systems AB", "CROSS-COUNTRY-AB" },
	{ 0x0050C2, 0x8FF000, 0x8FFFFF, "Luceat", "LUCEAT" },
	{ 0x0050C2, 0x900000, 0x900FFF, "Magor Communications Corp", "MAGOR-COMMUNICATION" },
	{ 0x0050C2, 0x901000, 0x901FFF, "Research Applications Incorp", "RESEARCH-APPLICATIONS-INCORP" },
	{ 0x0050C2, 0x902000, 0x902FFF, "China Railway Signal & Communication Corp.", "CHINA-RAILWAY-SIGNAL-COMMUNICATION" },
	{ 0x0050C2, 0x903000, 0x903FFF, "EcoAxis Systems Pvt. Ltd.", "ECOAXIS-PVT" },
	{ 0x0050C2, 0x904000, 0x904FFF, "R2Sonic, LLC", "R2SONIC-LLC" },
	{ 0x0050C2, 0x905000, 0x905FFF, "Link Communications, Inc", "LINK-COMMUNICATION" },
	{ 0x0050C2, 0x906000, 0x906FFF, "Gidel", "GIDEL" },
	{ 0x0050C2, 0x907000, 0x907FFF, "Cristal Controles Ltee", "CRISTAL-CONTROLES-LTEE" },
	{ 0x0050C2, 0x908000, 0x908FFF, "Codex Digital Ltd", "CODEX-DIGITAL" },
	{ 0x0050C2, 0x909000, 0x909FFF, "Elisra Electronic Systems", "ELISRA" },
	{ 0x0050C2, 0x90A000, 0x90AFFF, "Board Level Limited", "BOARD-LEVEL" },
	{ 0x0050C2, 0x90B000, 0x90BFFF, "E.ON ES Sverige AB", "E-ON-ES-SVERIGE-AB" },
	{ 0x0050C2, 0x90C000, 0x90CFFF, "LSS GmbH", "LSS" },
	{ 0x0050C2, 0x90D000, 0x90DFFF, "EVK DI Kerschhaggl GmbH", "EVK-DI-KERSCHHAGGL" },
	{ 0x0050C2, 0x90E000, 0x90EFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x90F000, 0x90FFFF, "INTEGRA Biosciences AG", "INTEGRA-BIOSCIENCES" },
	{ 0x0050C2, 0x910000, 0x910FFF, "Autotank AB", "AUTOTANK-AB" },
	{ 0x0050C2, 0x911000, 0x911FFF, "Vapor Rail", "VAPOR-RAIL" },
	{ 0x0050C2, 0x912000, 0x912FFF, "ASSET InterTech, Inc.", "ASSET-INTERTECH" },
	{ 0x0050C2, 0x913000, 0x913FFF, "Selex Ex Ltd", "SELEX-EX" },
	{ 0x0050C2, 0x914000, 0x914FFF, "IO-Connect", "IO-CONNECT" },
	{ 0x0050C2, 0x915000, 0x915FFF, "Verint Systems Ltd.", "VERINT" },
	{ 0x0050C2, 0x916000, 0x916FFF, "CHK GridSense P/L", "CHK-GRIDSENSE-P/L" },
	{ 0x0050C2, 0x917000, 0x917FFF, "CIRTEM", "CIRTEM" },
	{ 0x0050C2, 0x918000, 0x918FFF, "Design Lightning Corp", "DESIGN-LIGHTNING" },
	{ 0x0050C2, 0x919000, 0x919FFF, "AHV Systems, Inc.", "AHV" },
	{ 0x0050C2, 0x91A000, 0x91AFFF, "Xtone Networks", "XTONE-NETWORKS" },
	{ 0x0050C2, 0x91B000, 0x91BFFF, "Embedded Data Systems, LLC", "EMBEDDED-DATA-LLC" },
	{ 0x0050C2, 0x91C000, 0x91CFFF, "MangoDSP", "MANGODSP" },
	{ 0x0050C2, 0x91D000, 0x91DFFF, "Rosendahl Studiotechnik GmbH", "ROSENDAHL-STUDIOTECHNIK" },
	{ 0x0050C2, 0x91E000, 0x91EFFF, "Automation Tec", "AUTOMATION-TEC" },
	{ 0x0050C2, 0x91F000, 0x91FFFF, "2NCOMM DESIGN SRL", "2NCOMM-DESIGN-SRL" },
	{ 0x0050C2, 0x920000, 0x920FFF, "Rogue Engineering Inc.", "ROGUE-ENGINEERING" },
	{ 0x0050C2, 0x921000, 0x921FFF, "iQue RFID Technologies BV", "IQUE-RFID-TECHNOLOGIES" },
	{ 0x0050C2, 0x922000, 0x922FFF, "Metrum Sweden AB", "METRUM-SWEDEN-AB" },
	{ 0x0050C2, 0x923000, 0x923FFF, "Amicus Wireless", "AMICUS-WIRELESS" },
	{ 0x0050C2, 0x924000, 0x924FFF, "Link Electric & Safety Control Co.", "LINK-ELECTRIC-SAFETY-CONTROL" },
	{ 0x0050C2, 0x925000, 0x925FFF, "PHB Eletronica Ltda.", "PHB-ELETRONICA-LTDA" },
	{ 0x0050C2, 0x926000, 0x926FFF, "DITEST FAHRZEUGDIAGNOSE GMBH", "DITEST-FAHRZEUGDIAGNOSE" },
	{ 0x0050C2, 0x927000, 0x927FFF, "ATIS group s.r.o.", "ATIS-GROUP-S-R-O" },
	{ 0x0050C2, 0x928000, 0x928FFF, "Cinetix GmbH", "CINETIX" },
	{ 0x0050C2, 0x929000, 0x929FFF, "Flight Deck Resources", "FLIGHT-DECK-RESOURCES" },
	{ 0x0050C2, 0x92A000, 0x92AFFF, "Rohde & Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0x92B000, 0x92BFFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x0050C2, 0x92C000, 0x92CFFF, "Exatrol Corporation", "EXATROL" },
	{ 0x0050C2, 0x92D000, 0x92DFFF, "APProSoftware.com", "APPROSOFTWARE-COM" },
	{ 0x0050C2, 0x92E000, 0x92EFFF, "Goanna Technologies Pty Ltd", "GOANNA-TECHNOLOGIES-PTY" },
	{ 0x0050C2, 0x92F000, 0x92FFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x930000, 0x930FFF, "NETA Elektronik AS", "NETA-ELEKTRONIK-AS" },
	{ 0x0050C2, 0x931000, 0x931FFF, "Korea Telecom Internet Solutions (KTIS)", "KOREA-TELECOM-INTERNET-SOLUTIONS-KTIS" },
	{ 0x0050C2, 0x932000, 0x932FFF, "SMAVIS Inc.", "SMAVIS" },
	{ 0x0050C2, 0x933000, 0x933FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x934000, 0x934FFF, "Xilar Corp.", "XILAR" },
	{ 0x0050C2, 0x935000, 0x935FFF, "Image Video", "IMAGE-VIDEO" },
	{ 0x0050C2, 0x936000, 0x936FFF, "Margaritis Engineering", "MARGARITIS-ENGINEERING" },
	{ 0x0050C2, 0x937000, 0x937FFF, "BigBear", "BIGBEAR" },
	{ 0x0050C2, 0x938000, 0x938FFF, "Postec Data Systems Ltd", "POSTEC-DATA" },
	{ 0x0050C2, 0x939000, 0x939FFF, "Mosaic Dynamic Solutions", "MOSAIC-DYNAMIC-SOLUTIONS" },
	{ 0x0050C2, 0x93A000, 0x93AFFF, "ALPHATRONICS nv", "ALPHATRONICS-NV" },
	{ 0x0050C2, 0x93B000, 0x93BFFF, "Reliatronics Inc.", "RELIATRONICS" },
	{ 0x0050C2, 0x93C000, 0x93CFFF, "FractureCode Corporation", "FRACTURECODE" },
	{ 0x0050C2, 0x93D000, 0x93DFFF, "Lighting Science Group Corporation", "LIGHTING-SCIENCE-GROUP" },
	{ 0x0050C2, 0x93E000, 0x93EFFF, "RCS Communication Test Systems Ltd.", "RCS-COMMUNICATION-TEST" },
	{ 0x0050C2, 0x93F000, 0x93FFFF, "TSB Solutions Inc.", "TSB-SOLUTIONS" },
	{ 0x0050C2, 0x940000, 0x940FFF, "Phitek Systems Ltd.", "PHITEK" },
	{ 0x0050C2, 0x941000, 0x941FFF, "Rolbit", "ROLBIT" },
	{ 0x0050C2, 0x942000, 0x942FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x943000, 0x943FFF, "QuanZhou TDX Electronics Co., Ltd.", "QUANZHOU-TDX-ELECTRONICS" },
	{ 0x0050C2, 0x944000, 0x944FFF, "Wireonair A/S", "WIREONAIR-A/S" },
	{ 0x0050C2, 0x945000, 0x945FFF, "Ex-i Flow Measurement Ltd.", "EX-I-FLOW-MEASUREMENT" },
	{ 0x0050C2, 0x946000, 0x946FFF, "MEGWARE Computer GmbH", "MEGWARE" },
	{ 0x0050C2, 0x947000, 0x947FFF, "IMEXHIGHWAY cvba", "IMEXHIGHWAY-CVBA" },
	{ 0x0050C2, 0x948000, 0x948FFF, "ELECTRONIA", "ELECTRONIA" },
	{ 0x0050C2, 0x949000, 0x949FFF, "taskit GmbH", "TASKIT" },
	{ 0x0050C2, 0x94A000, 0x94AFFF, "TRUMEDIA TECHNOLOGIES", "TRUMEDIA-TECHNOLOGIES" },
	{ 0x0050C2, 0x94B000, 0x94BFFF, "Piller engineering Ltd.", "PILLER-ENGINEERING" },
	{ 0x0050C2, 0x94C000, 0x94CFFF, "TEMIX", "TEMIX" },
	{ 0x0050C2, 0x94D000, 0x94DFFF, "C&H technology ltd.", "C-H" },
	{ 0x0050C2, 0x94E000, 0x94EFFF, "Zynix Original Sdn. Bhd.", "ZYNIX-ORIGINAL-SDN-BHD" },
	{ 0x0050C2, 0x94F000, 0x94FFFF, "IT-Designers GmbH", "IT-DESIGNERS" },
	{ 0x0050C2, 0x950000, 0x950FFF, "Tele and Radio Research Institute", "TELE-AND-RADIO-RESEARCH-INSTITUTE" },
	{ 0x0050C2, 0x951000, 0x951FFF, "EL.C.A. soc. coop.", "EL-C-A-SOC-COOP" },
	{ 0x0050C2, 0x952000, 0x952FFF, "Tech Fass s.r.o.", "TECH-FASS-S-R-O" },
	{ 0x0050C2, 0x953000, 0x953FFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0x954000, 0x954FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x955000, 0x955FFF, "Roessmann Engineering", "ROESSMANN-ENGINEERING" },
	{ 0x0050C2, 0x956000, 0x956FFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0x957000, 0x957FFF, "STRATEC Control Systems", "STRATEC-CONTROL" },
	{ 0x0050C2, 0x958000, 0x958FFF, "Sensoptics Ltd", "SENSOPTICS" },
	{ 0x0050C2, 0x959000, 0x959FFF, "DECTRIS Ltd.", "DECTRIS" },
	{ 0x0050C2, 0x95A000, 0x95AFFF, "TechnoAP", "TECHNOAP" },
	{ 0x0050C2, 0x95B000, 0x95BFFF, "AS Solar GmbH", "AS-SOLAR" },
	{ 0x0050C2, 0x95C000, 0x95CFFF, "Resurgent Health & Medical", "RESURGENT-HEALTH-MEDICAL" },
	{ 0x0050C2, 0x95D000, 0x95DFFF, "full electronic system", "FULL-SYSTEM" },
	{ 0x0050C2, 0x95E000, 0x95EFFF, "BEEcube Inc.", "BEECUBE" },
	{ 0x0050C2, 0x95F000, 0x95FFFF, "METRONIC APARATURA KONTROLNO - POMIAROWA", "METRONIC-APARATURA-KONTROLNO---POMIAROWA" },
	{ 0x0050C2, 0x960000, 0x960FFF, "kuroneko dennnou kenkyuushitsu", "KURONEKO-DENNNOU-KENKYUUSHITSU" },
	{ 0x0050C2, 0x961000, 0x961FFF, "Picsolve International Limited", "PICSOLVE" },
	{ 0x0050C2, 0x962000, 0x962FFF, "Shockfish SA", "SHOCKFISH" },
	{ 0x0050C2, 0x963000, 0x963FFF, "Lécureux SA", "LéCUREUX" },
	{ 0x0050C2, 0x964000, 0x964FFF, "IQ Automation GmbH", "IQ-AUTOMATION" },
	{ 0x0050C2, 0x965000, 0x965FFF, "Emitech Corporation", "EMITECH" },
	{ 0x0050C2, 0x966000, 0x966FFF, "PCM Industries", "PCM-INDUSTRIES" },
	{ 0x0050C2, 0x967000, 0x967FFF, "Watthour Engineering Co., Inc.", "WATTHOUR-ENGINEERING" },
	{ 0x0050C2, 0x968000, 0x968FFF, "BuLogics, Inc.", "BULOGICS" },
	{ 0x0050C2, 0x969000, 0x969FFF, "Gehrke Kommunikationssysteme GmbH", "GEHRKE-KOMMUNIKATIONSSYSTEME" },
	{ 0x0050C2, 0x96A000, 0x96AFFF, "Elektrobit Wireless Communications Ltd", "ELEKTROBIT-WIRELESS-COMMUNICATION" },
	{ 0x0050C2, 0x96B000, 0x96BFFF, "Electronic Media Services Ltd", "MEDIA-SERVICES" },
	{ 0x0050C2, 0x96C000, 0x96CFFF, "Aqua Cooler Pty Ltd", "AQUA-COOLER-PTY" },
	{ 0x0050C2, 0x96D000, 0x96DFFF, "Keene Electronics Ltd.", "KEENE-ELECTRONICS" },
	{ 0x0050C2, 0x96E000, 0x96EFFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0x96F000, 0x96FFFF, "Varec Inc.", "VAREC" },
	{ 0x0050C2, 0x970000, 0x970FFF, "Tsuji Electronics Co.,Ltd", "TSUJI-ELECTRONICS" },
	{ 0x0050C2, 0x971000, 0x971FFF, "Ipitek", "IPITEK" },
	{ 0x0050C2, 0x972000, 0x972FFF, "Switch Science (Panini Keikaku)", "SWITCH-SCIENCE-PANINI-KEIKAKU" },
	{ 0x0050C2, 0x973000, 0x973FFF, "Systèmes Pran", "SYSTèMES-PRAN" },
	{ 0x0050C2, 0x974000, 0x974FFF, "EMAC, INC.", "EMAC" },
	{ 0x0050C2, 0x975000, 0x975FFF, "Pyramid Technical Consultants", "PYRAMID-TECHNICAL-CONSULTANTS" },
	{ 0x0050C2, 0x976000, 0x976FFF, "SANDS INSTRUMENTATION INDIA PVT LTD", "SANDS-INSTRUMENTATION-INDIA-PVT" },
	{ 0x0050C2, 0x977000, 0x977FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x978000, 0x978FFF, "LOGITAL DIGITAL MEDIA srl", "LOGITAL-DIGITAL-MEDIA-SRL" },
	{ 0x0050C2, 0x979000, 0x979FFF, "Far South Networks (Pty) Ltd", "FAR-SOUTH-NETWORKS-PTY" },
	{ 0x0050C2, 0x97A000, 0x97AFFF, "KST Technology Co., Ltd", "KST" },
	{ 0x0050C2, 0x97B000, 0x97BFFF, "SMARTQUANTUM SA", "SMARTQUANTUM" },
	{ 0x0050C2, 0x97C000, 0x97CFFF, "Creacon Technologies B.V.", "CREACON-TECHNOLOGIES" },
	{ 0x0050C2, 0x97D000, 0x97DFFF, "Soehnle Professional GmbH & Co.KG", "SOEHNLE-PROFESSIONAL" },
	{ 0x0050C2, 0x97E000, 0x97EFFF, "RF Industries", "RF-INDUSTRIES" },
	{ 0x0050C2, 0x97F000, 0x97FFFF, "C&I Co.Ltd", "C-I" },
	{ 0x0050C2, 0x980000, 0x980FFF, "Digital Payment Technologies", "DIGITAL-PAYMENT-TECHNOLOGIES" },
	{ 0x0050C2, 0x981000, 0x981FFF, "Novotronik GmbH", "NOVOTRONIK" },
	{ 0x0050C2, 0x982000, 0x982FFF, "Triple Ring Technologies, Inc.", "TRIPLE-RING-TECHNOLOGIES" },
	{ 0x0050C2, 0x983000, 0x983FFF, "Bogart Engineering", "BOGART-ENGINEERING" },
	{ 0x0050C2, 0x984000, 0x984FFF, "Atel Corporation", "ATEL" },
	{ 0x0050C2, 0x985000, 0x985FFF, "Earnestcom Sdn Bhd", "EARNESTCOM-SDN-BHD" },
	{ 0x0050C2, 0x986000, 0x986FFF, "DSCI", "DSCI" },
	{ 0x0050C2, 0x987000, 0x987FFF, "Joinsoon Electronics MFG. Co., Ltd", "JOINSOON-ELECTRONICS-MFG" },
	{ 0x0050C2, 0x988000, 0x988FFF, "Pantel International", "PANTEL" },
	{ 0x0050C2, 0x989000, 0x989FFF, "Psigenics Corporation", "PSIGENICS" },
	{ 0x0050C2, 0x98A000, 0x98AFFF, "MEV Limited", "MEV" },
	{ 0x0050C2, 0x98B000, 0x98BFFF, "TI2000 TECNOLOGIA INFORMATICA 2000", "TI2000-TECNOLOGIA-INFORMATICA-2000" },
	{ 0x0050C2, 0x98C000, 0x98CFFF, "MGM-Devices Oy", "MGM-DEVICES-OY" },
	{ 0x0050C2, 0x98D000, 0x98DFFF, "Mecos AG", "MECOS" },
	{ 0x0050C2, 0x98E000, 0x98EFFF, "Link Technologies, Inc", "LINK-TECHNOLOGIES" },
	{ 0x0050C2, 0x98F000, 0x98FFFF, "BELIK S.P.R.L.", "BELIK-S-P-R-L" },
	{ 0x0050C2, 0x990000, 0x990FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x991000, 0x991FFF, "UGL Limited", "UGL" },
	{ 0x0050C2, 0x992000, 0x992FFF, "IDT Sound Processing Corporation", "IDT-SOUND-PROCESSING" },
	{ 0x0050C2, 0x993000, 0x993FFF, "UNETCONVERGENCE CO., LTD", "UNETCONVERGENCE" },
	{ 0x0050C2, 0x994000, 0x994FFF, "Xafax Nederland bv", "XAFAX-NEDERLAND" },
	{ 0x0050C2, 0x995000, 0x995FFF, "Inter Control Hermann Köhler Elektrik GmbH&Co.KG", "INTER-CONTROL-HERMANN-KOEHLER-ELEKTRIK" },
	{ 0x0050C2, 0x996000, 0x996FFF, "Commercial Timesharing Inc.", "COMMERCIAL-TIMESHARING" },
	{ 0x0050C2, 0x997000, 0x997FFF, "Depro Électronique", "DEPRO-ÉLECTRONIQUE" },
	{ 0x0050C2, 0x998000, 0x998FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0x999000, 0x999FFF, "Cambustion Ltd", "CAMBUSTION" },
	{ 0x0050C2, 0x99A000, 0x99AFFF, "Miromico AG", "MIROMICO" },
	{ 0x0050C2, 0x99B000, 0x99BFFF, "Bettini srl", "BETTINI-SRL" },
	{ 0x0050C2, 0x99C000, 0x99CFFF, "CaTs3 Limited", "CATS3" },
	{ 0x0050C2, 0x99D000, 0x99DFFF, "Powersense A/S", "POWERSENSE-A/S" },
	{ 0x0050C2, 0x99E000, 0x99EFFF, "Engage Technologies", "ENGAGE-TECHNOLOGIES" },
	{ 0x0050C2, 0x99F000, 0x99FFFF, "Sietron Elektronik", "SIETRON-ELEKTRONIK" },
	{ 0x0050C2, 0x9A0000, 0x9A0FFF, "Trs Systems, Inc.", "TRS" },
	{ 0x0050C2, 0x9A1000, 0x9A1FFF, "ComAp s.r.o", "COMAP-S-R-O" },
	{ 0x0050C2, 0x9A2000, 0x9A2FFF, "SAMsystems GmbH", "SAMSYSTEMS" },
	{ 0x0050C2, 0x9A3000, 0x9A3FFF, "Computerwise, Inc.", "COMPUTERWISE" },
	{ 0x0050C2, 0x9A4000, 0x9A4FFF, "Entwicklung Hard- & Software", "ENTWICKLUNG-HARD--SOFTWARE" },
	{ 0x0050C2, 0x9A5000, 0x9A5FFF, "Conolog Corporation", "CONOLOG" },
	{ 0x0050C2, 0x9A6000, 0x9A6FFF, "Metodo2", "METODO2" },
	{ 0x0050C2, 0x9A7000, 0x9A7FFF, "Thales Communications & Security S.A.", "THALES-COMMUNICATION-SECURITY" },
	{ 0x0050C2, 0x9A8000, 0x9A8FFF, "DOMIS SA", "DOMIS" },
	{ 0x0050C2, 0x9A9000, 0x9A9FFF, "General Dynamics C4 Systems", "GENERAL-DYNAMICS-C4" },
	{ 0x0050C2, 0x9AA000, 0x9AAFFF, "TEKO TELECOM SpA", "TEKO-TELECOM-SPA" },
	{ 0x0050C2, 0x9AB000, 0x9ABFFF, "Electrodata Inc.", "ELECTRODATA" },
	{ 0x0050C2, 0x9AC000, 0x9ACFFF, "Questek Australia Pty Ltd", "QUESTEK-AUSTRALIA-PTY" },
	{ 0x0050C2, 0x9AD000, 0x9ADFFF, "Chronos Technology Ltd.", "CHRONOS" },
	{ 0x0050C2, 0x9AE000, 0x9AEFFF, "Esensors, Inc.", "ESENSORS" },
	{ 0x0050C2, 0x9AF000, 0x9AFFFF, "KRESS-NET Krzysztof Rutecki", "KRESS-NET-KRZYSZTOF-RUTECKI" },
	{ 0x0050C2, 0x9B0000, 0x9B0FFF, "Ebru GmbH", "EBRU" },
	{ 0x0050C2, 0x9B1000, 0x9B1FFF, "Bon Hora GmbH", "BON-HORA" },
	{ 0x0050C2, 0x9B2000, 0x9B2FFF, "TempSys", "TEMPSYS" },
	{ 0x0050C2, 0x9B3000, 0x9B3FFF, "Kahler Automation", "KAHLER-AUTOMATION" },
	{ 0x0050C2, 0x9B4000, 0x9B4FFF, "EUKREA ELECTROMATIQUE SARL", "EUKREA-ELECTROMATIQUE-SARL" },
	{ 0x0050C2, 0x9B5000, 0x9B5FFF, "Telegamma srl", "TELEGAMMA-SRL" },
	{ 0x0050C2, 0x9B6000, 0x9B6FFF, "ACTECH", "ACTECH" },
	{ 0x0050C2, 0x9B7000, 0x9B7FFF, "St. Michael Strategies", "ST-MICHAEL-STRATEGIES" },
	{ 0x0050C2, 0x9B8000, 0x9B8FFF, "Sound Player Systems e.K.", "SOUND-PLAYER-E-K" },
	{ 0x0050C2, 0x9B9000, 0x9B9FFF, "ISA - Intelligent Sensing Anywhere, S.A.", "ISA---INTELLIGENT-SENSING-ANYWHERE" },
	{ 0x0050C2, 0x9BA000, 0x9BAFFF, "Connor-Winfield", "CONNOR-WINFIELD" },
	{ 0x0050C2, 0x9BB000, 0x9BBFFF, "OMICRON electronics GmbH", "OMICRON-ELECTRONICS" },
	{ 0x0050C2, 0x9BC000, 0x9BCFFF, "Vester Elektronik GmbH", "VESTER-ELEKTRONIK" },
	{ 0x0050C2, 0x9BD000, 0x9BDFFF, "Sensitron Semiconductor", "SENSITRON-SEMICONDUCTOR" },
	{ 0x0050C2, 0x9BE000, 0x9BEFFF, "Xad Communications Ltd", "XAD-COMMUNICATION" },
	{ 0x0050C2, 0x9BF000, 0x9BFFFF, "2N TELEKOMUNIKACE a.s.", "2N-TELEKOMUNIKACE-A-S" },
	{ 0x0050C2, 0x9C0000, 0x9C0FFF, "Stuyts Engineering Haarlem BV", "STUYTS-ENGINEERING-HAARLEM" },
	{ 0x0050C2, 0x9C1000, 0x9C1FFF, "Tattile srl", "TATTILE-SRL" },
	{ 0x0050C2, 0x9C2000, 0x9C2FFF, "Team Enginers", "TEAM-ENGINERS" },
	{ 0x0050C2, 0x9C3000, 0x9C3FFF, "GE Security-Kampro", "GE-SECURITY-KAMPRO" },
	{ 0x0050C2, 0x9C4000, 0x9C4FFF, "Vitel Net", "VITEL-NET" },
	{ 0x0050C2, 0x9C5000, 0x9C5FFF, "Scansonic MI GmbH", "SCANSONIC-MI" },
	{ 0x0050C2, 0x9C6000, 0x9C6FFF, "Protronic GmbH", "PROTRONIC" },
	{ 0x0050C2, 0x9C7000, 0x9C7FFF, "Kumera Drives Oy", "KUMERA-DRIVES-OY" },
	{ 0x0050C2, 0x9C8000, 0x9C8FFF, "ethermetrics", "ETHERMETRICS" },
	{ 0x0050C2, 0x9C9000, 0x9C9FFF, "LUMINEX Lighting Control Equipment", "LUMINEX-LIGHTING-CONTROL-EQUIPMENT" },
	{ 0x0050C2, 0x9CA000, 0x9CAFFF, "ESAB-ATAS GmbH", "ESAB-ATAS" },
	{ 0x0050C2, 0x9CB000, 0x9CBFFF, "NIS-time GmbH", "NIS-TIME" },
	{ 0x0050C2, 0x9CC000, 0x9CCFFF, "Hirotech, Inc", "HIROTECH" },
	{ 0x0050C2, 0x9CD000, 0x9CDFFF, "Uwe Schneider GmbH", "UWE-SCHNEIDER" },
	{ 0x0050C2, 0x9CE000, 0x9CEFFF, "Ronan Engineering", "RONAN-ENGINEERING" },
	{ 0x0050C2, 0x9CF000, 0x9CFFFF, "Intuitive Surgical, Inc", "INTUITIVE-SURGICAL" },
	{ 0x0050C2, 0x9D0000, 0x9D0FFF, "J. DITTRICH ELEKTRONIC GmbH & Co. KG", "J-DITTRICH-ELEKTRONIC" },
	{ 0x0050C2, 0x9D1000, 0x9D1FFF, "Bladelius Design Group AB", "BLADELIUS-DESIGN-GROUP-AB" },
	{ 0x0050C2, 0x9D2000, 0x9D2FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0x9D3000, 0x9D3FFF, "Telemetrie Elektronik GmbH", "TELEMETRIE-ELEKTRONIK" },
	{ 0x0050C2, 0x9D4000, 0x9D4FFF, "FIRST", "FIRST" },
	{ 0x0050C2, 0x9D5000, 0x9D5FFF, "Netpower Labs AB", "NETPOWER-LABS-AB" },
	{ 0x0050C2, 0x9D6000, 0x9D6FFF, "Innovation, Institute, Inc", "INNOVATION-INSTITUTE" },
	{ 0x0050C2, 0x9D7000, 0x9D7FFF, "Melex Inc.", "MELEX" },
	{ 0x0050C2, 0x9D8000, 0x9D8FFF, "SAMSUNG HEAVY INDUSTRIES CO.,LTD.", "SAMSUNG-HEAVY-INDUSTRIES" },
	{ 0x0050C2, 0x9D9000, 0x9D9FFF, "CNS Systems, Inc.", "CNS" },
	{ 0x0050C2, 0x9DA000, 0x9DAFFF, "NEUTRONIK e.K.", "NEUTRONIK-E-K" },
	{ 0x0050C2, 0x9DB000, 0x9DBFFF, "Walter Grotkasten", "WALTER-GROTKASTEN" },
	{ 0x0050C2, 0x9DC000, 0x9DCFFF, "FTM Marketing Limited", "FTM-MARKETING" },
	{ 0x0050C2, 0x9DD000, 0x9DDFFF, "Institut Dr. Foerster", "INSTITUT-DR-FOERSTER" },
	{ 0x0050C2, 0x9DE000, 0x9DEFFF, "CHAUVIN ARNOUX", "CHAUVIN-ARNOUX" },
	{ 0x0050C2, 0x9DF000, 0x9DFFFF, "CODEC Co., Ltd.", "CODEC" },
	{ 0x0050C2, 0x9E0000, 0x9E0FFF, "DST Swiss AG", "DST-SWISS" },
	{ 0x0050C2, 0x9E1000, 0x9E1FFF, "Enreduce Energy Control AB", "ENREDUCE-ENERGY-CONTROL-AB" },
	{ 0x0050C2, 0x9E2000, 0x9E2FFF, "E-ViEWS SAFETY SYSTEMS, INC", "E-VIEWS-SAFETY" },
	{ 0x0050C2, 0x9E3000, 0x9E3FFF, "SAI Informationstechnik", "SAI-INFORMATIONSTECHNIK" },
	{ 0x0050C2, 0x9E4000, 0x9E4FFF, "Pyxis Controls WLL", "PYXIS-CONTROLS-WLL" },
	{ 0x0050C2, 0x9E5000, 0x9E5FFF, "Halliburton Far East Pte Ltd", "HALLIBURTON-FAR-EAST-PTE" },
	{ 0x0050C2, 0x9E6000, 0x9E6FFF, "Kumho Electric, Inc.", "KUMHO-ELECTRIC" },
	{ 0x0050C2, 0x9E7000, 0x9E7FFF, "DORLET S.A.", "DORLET" },
	{ 0x0050C2, 0x9E8000, 0x9E8FFF, "Hammock Corporation", "HAMMOCK" },
	{ 0x0050C2, 0x9E9000, 0x9E9FFF, "Ciemme Sistemi Spa", "CIEMME-SISTEMI-SPA" },
	{ 0x0050C2, 0x9EA000, 0x9EAFFF, "SISMODULAR - Engenharia, Lda", "SISMODULAR---ENGENHARIA-LDA" },
	{ 0x0050C2, 0x9EB000, 0x9EBFFF, "AFORE Solutions Inc.", "AFORE-SOLUTIONS" },
	{ 0x0050C2, 0x9EC000, 0x9ECFFF, "Rohde & Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0x9ED000, 0x9EDFFF, "Picell B.V.", "PICELL" },
	{ 0x0050C2, 0x9EE000, 0x9EEFFF, "Michael Stevens & Partners Ltd", "MICHAEL-STEVENS-PARTNERS" },
	{ 0x0050C2, 0x9EF000, 0x9EFFFF, "WoKa-Elektronik GmbH", "WOKA-ELEKTRONIK" },
	{ 0x0050C2, 0x9F0000, 0x9F0FFF, "Veracity UK Ltd", "VERACITY-UK" },
	{ 0x0050C2, 0x9F1000, 0x9F1FFF, "IDEAS s.r.l.", "IDEAS-S-R-L" },
	{ 0x0050C2, 0x9F2000, 0x9F2FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0x9F3000, 0x9F3FFF, "Vision Technologies, Inc.", "VISION-TECHNOLOGIES" },
	{ 0x0050C2, 0x9F4000, 0x9F4FFF, "FSR Inc.", "FSR" },
	{ 0x0050C2, 0x9F5000, 0x9F5FFF, "Commex Technologies", "COMMEX-TECHNOLOGIES" },
	{ 0x0050C2, 0x9F6000, 0x9F6FFF, "Ion Sense Inc.", "ION-SENSE" },
	{ 0x0050C2, 0x9F7000, 0x9F7FFF, "Dave Jones Design", "DAVE-JONES-DESIGN" },
	{ 0x0050C2, 0x9F8000, 0x9F8FFF, "Austco Communication Systems Pty Ltd", "AUSTCO-COMMUNICATION-PTY" },
	{ 0x0050C2, 0x9F9000, 0x9F9FFF, "ABB Transmission and Distribution Auto Eqip(Xiamen.China)", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTO-EQIP-XIAMEN-CHINA" },
	{ 0x0050C2, 0x9FA000, 0x9FAFFF, "Teranex A Division of Silicon Optix", "TERANEX-A-DIVISION-OF-SILICON-OPTIX" },
	{ 0x0050C2, 0x9FB000, 0x9FBFFF, "Villbau Kft.", "VILLBAU-KFT" },
	{ 0x0050C2, 0x9FC000, 0x9FCFFF, "ECTEC INC.", "ECTEC" },
	{ 0x0050C2, 0x9FD000, 0x9FDFFF, "Bitt technology-A Ltd.", "BITT--A" },
	{ 0x0050C2, 0x9FE000, 0x9FEFFF, "SPECTRA EMBEDDED SYSTEMS", "SPECTRA-EMBEDDED" },
	{ 0x0050C2, 0x9FF000, 0x9FFFFF, "Humphrey Products", "HUMPHREY-PRODUCTS" },
	{ 0x0050C2, 0xA00000, 0xA00FFF, "Technovare Systems", "TECHNOVARE" },
	{ 0x0050C2, 0xA01000, 0xA01FFF, "Patronics International LTD", "PATRONICS" },
	{ 0x0050C2, 0xA02000, 0xA02FFF, "Reference, LLC.", "REFERENCE-LLC" },
	{ 0x0050C2, 0xA03000, 0xA03FFF, "EEG Enterprises Inc", "EEG-ENTERPRISES" },
	{ 0x0050C2, 0xA04000, 0xA04FFF, "TP Radio", "TP-RADIO" },
	{ 0x0050C2, 0xA05000, 0xA05FFF, "Adgil Design Inc.", "ADGIL-DESIGN" },
	{ 0x0050C2, 0xA06000, 0xA06FFF, "Cloos Schweisstechnik GmbH", "CLOOS-SCHWEISSTECHNIK" },
	{ 0x0050C2, 0xA07000, 0xA07FFF, "Dynon Instruments", "DYNON-INSTRUMENTS" },
	{ 0x0050C2, 0xA08000, 0xA08FFF, "LabJack Corporation", "LABJACK" },
	{ 0x0050C2, 0xA09000, 0xA09FFF, "Innovative American Technology", "INNOVATIVE-AMERICAN" },
	{ 0x0050C2, 0xA0A000, 0xA0AFFF, "ACD Elektronik Gmbh", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0xA0B000, 0xA0BFFF, "I.D.S. Ingegneria Dei Sistemi S.p.A.", "I-D-S-INGEGNERIA-DEI-SISTEMI-S-P-A" },
	{ 0x0050C2, 0xA0C000, 0xA0CFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xA0D000, 0xA0DFFF, "CHARLYROBOT", "CHARLYROBOT" },
	{ 0x0050C2, 0xA0E000, 0xA0EFFF, "Engicam srl", "ENGICAM-SRL" },
	{ 0x0050C2, 0xA0F000, 0xA0FFFF, "Visualware Inc", "VISUALWARE" },
	{ 0x0050C2, 0xA10000, 0xA10FFF, "Essential Design & Integration P/L", "ESSENTIAL-DESIGN-INTEGRATION-P/L" },
	{ 0x0050C2, 0xA11000, 0xA11FFF, "OJSC Rawenstvo", "OJSC-RAWENSTVO" },
	{ 0x0050C2, 0xA12000, 0xA12FFF, "HCE Engineering S.r.l.", "HCE-ENGINEERING-S-R-L" },
	{ 0x0050C2, 0xA13000, 0xA13FFF, "Talyst, Inc.", "TALYST" },
	{ 0x0050C2, 0xA14000, 0xA14FFF, "Elbit Systems of America - Tallahassee Operations", "ELBIT-OF-AMERICA---TALLAHASSEE-OPERATIONS" },
	{ 0x0050C2, 0xA15000, 0xA15FFF, "Industrial Computing Ltd", "INDUSTRIAL-COMPUTING" },
	{ 0x0050C2, 0xA16000, 0xA16FFF, "Baudisch Electronic GmbH", "BAUDISCH" },
	{ 0x0050C2, 0xA17000, 0xA17FFF, "Winners Satellite Electronics Corp.", "WINNERS-SATELLITE-ELECTRONICS" },
	{ 0x0050C2, 0xA18000, 0xA18FFF, "Eoslink", "EOSLINK" },
	{ 0x0050C2, 0xA19000, 0xA19FFF, "Icon Time Systems", "ICON-TIME" },
	{ 0x0050C2, 0xA1A000, 0xA1AFFF, "DDL", "DDL" },
	{ 0x0050C2, 0xA1B000, 0xA1BFFF, "Realtime Systems Ltd.", "REALTIME" },
	{ 0x0050C2, 0xA1C000, 0xA1CFFF, "Microtechnica", "MICROTECHNICA" },
	{ 0x0050C2, 0xA1D000, 0xA1DFFF, "SAMH Engineering Services", "SAMH-ENGINEERING-SERVICES" },
	{ 0x0050C2, 0xA1E000, 0xA1EFFF, "MAMAC Systems, Inc.", "MAMAC" },
	{ 0x0050C2, 0xA1F000, 0xA1FFFF, "Flight Data Systems Pty Ltd", "FLIGHT-DATA-PTY" },
	{ 0x0050C2, 0xA20000, 0xA20FFF, "Quorum Technologies Ltd", "QUORUM-TECHNOLOGIES" },
	{ 0x0050C2, 0xA21000, 0xA21FFF, "ISAC SRL", "ISAC-SRL" },
	{ 0x0050C2, 0xA22000, 0xA22FFF, "Nippon Manufacturing Service Corporation (abbreviated as 'nms')", "NIPPON-MANUFACTURING-SERVICE-ABBREVIATED-AS-NMS" },
	{ 0x0050C2, 0xA23000, 0xA23FFF, "Agility Mfg, Inc.", "AGILITY-MFG" },
	{ 0x0050C2, 0xA24000, 0xA24FFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0xA25000, 0xA25FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xA26000, 0xA26FFF, "Preferred Oil, LLC", "PREFERRED-OIL-LLC" },
	{ 0x0050C2, 0xA27000, 0xA27FFF, "meconet e. K.", "MECONET-E-K" },
	{ 0x0050C2, 0xA28000, 0xA28FFF, "KENDA ELECTRONIC SYSTEMS LIMITED", "KENDA" },
	{ 0x0050C2, 0xA29000, 0xA29FFF, "Luminex Corporation", "LUMINEX" },
	{ 0x0050C2, 0xA2A000, 0xA2AFFF, "Custom Control Concepts", "CUSTOM-CONTROL-CONCEPTS" },
	{ 0x0050C2, 0xA2B000, 0xA2BFFF, "APRILIA RACING S.R.L.", "APRILIA-RACING-S-R-L" },
	{ 0x0050C2, 0xA2C000, 0xA2CFFF, "KWS-Electronic GmbH", "KWS" },
	{ 0x0050C2, 0xA2D000, 0xA2DFFF, "Inventure Inc.", "INVENTURE" },
	{ 0x0050C2, 0xA2E000, 0xA2EFFF, "Yuyama Mfg. Co., Ltd.", "YUYAMA-MFG" },
	{ 0x0050C2, 0xA2F000, 0xA2FFFF, "DragonFly Scientific LLC", "DRAGONFLY-SCIENTIFIC-LLC" },
	{ 0x0050C2, 0xA30000, 0xA30FFF, "D-TA Systems", "D-TA" },
	{ 0x0050C2, 0xA31000, 0xA31FFF, "Coolit Systems, Inc.", "COOLIT" },
	{ 0x0050C2, 0xA32000, 0xA32FFF, "Harris Designs of NRV, Inc.", "HARRIS-DESIGNS-OF-NRV" },
	{ 0x0050C2, 0xA33000, 0xA33FFF, "Fuji Firmware", "FUJI-FIRMWARE" },
	{ 0x0050C2, 0xA34000, 0xA34FFF, "Casabyte Inc.", "CASABYTE" },
	{ 0x0050C2, 0xA35000, 0xA35FFF, "Appareo Systems, LLC", "APPAREO-LLC" },
	{ 0x0050C2, 0xA36000, 0xA36FFF, "Shenzhen Shangji electronic Co.Ltd", "SHENZHEN-SHANGJI" },
	{ 0x0050C2, 0xA37000, 0xA37FFF, "Software Systems Plus", "SOFTWARE-PLUS" },
	{ 0x0050C2, 0xA38000, 0xA38FFF, "Tred Displays", "TRED-DISPLAYS" },
	{ 0x0050C2, 0xA39000, 0xA39FFF, "Industrial Data Products Ltd", "INDUSTRIAL-DATA-PRODUCTS" },
	{ 0x0050C2, 0xA3A000, 0xA3AFFF, "Telecor Inc.", "TELECOR" },
	{ 0x0050C2, 0xA3B000, 0xA3BFFF, "IPcontrols GmbH", "IPCONTROLS" },
	{ 0x0050C2, 0xA3C000, 0xA3CFFF, "Brähler ICS Konferenztechnik AG", "BRAEHLER-ICS-KONFERENZTECHNIK" },
	{ 0x0050C2, 0xA3D000, 0xA3DFFF, "OWANDY", "OWANDY" },
	{ 0x0050C2, 0xA3E000, 0xA3EFFF, "DUEVI SNC DI MORA E SANTESE", "DUEVI-SNC-DI-MORA-E-SANTESE" },
	{ 0x0050C2, 0xA3F000, 0xA3FFFF, "LHA Systems CC", "LHA-CC" },
	{ 0x0050C2, 0xA40000, 0xA40FFF, "Mosberger Consulting LLC", "MOSBERGER-CONSULTING-LLC" },
	{ 0x0050C2, 0xA41000, 0xA41FFF, "Meiryo Denshi Corp.", "MEIRYO-DENSHI" },
	{ 0x0050C2, 0xA42000, 0xA42FFF, "RealVision Inc.", "REALVISION" },
	{ 0x0050C2, 0xA43000, 0xA43FFF, "NKS Co.Ltd.", "NKS" },
	{ 0x0050C2, 0xA44000, 0xA44FFF, "TORC Technologies", "TORC-TECHNOLOGIES" },
	{ 0x0050C2, 0xA45000, 0xA45FFF, "Sofradir-EC", "SOFRADIR-EC" },
	{ 0x0050C2, 0xA46000, 0xA46FFF, "Softronics Ltd.", "SOFTRONICS" },
	{ 0x0050C2, 0xA47000, 0xA47FFF, "PRIMETECH ENGINEERING CORP.", "PRIMETECH-ENGINEERING" },
	{ 0x0050C2, 0xA48000, 0xA48FFF, "Thales Optronics Limited", "THALES-OPTRONICS" },
	{ 0x0050C2, 0xA49000, 0xA49FFF, "Wayne Dalton Corp.", "WAYNE-DALTON" },
	{ 0x0050C2, 0xA4A000, 0xA4AFFF, "DITRON S.r.l.", "DITRON-S-R-L" },
	{ 0x0050C2, 0xA4B000, 0xA4BFFF, "L-3 Communications Mobile-Vision, Inc.", "L-3-COMMUNICATION-MOBILE-VISION" },
	{ 0x0050C2, 0xA4C000, 0xA4CFFF, "VasoNova, Inc.", "VASONOVA" },
	{ 0x0050C2, 0xA4D000, 0xA4DFFF, "LevelStar LLC.", "LEVELSTAR-LLC" },
	{ 0x0050C2, 0xA4E000, 0xA4EFFF, "Conduant Corporation", "CONDUANT" },
	{ 0x0050C2, 0xA4F000, 0xA4FFFF, "Deuta GmbH", "DEUTA" },
	{ 0x0050C2, 0xA50000, 0xA50FFF, "i-RED Infrarot Systeme GmbH", "I-RED-INFRAROT-SYSTEME" },
	{ 0x0050C2, 0xA51000, 0xA51FFF, "Y-products co.ltd.", "Y-PRODUCTS" },
	{ 0x0050C2, 0xA52000, 0xA52FFF, "The VON Corporation", "THE-VON" },
	{ 0x0050C2, 0xA53000, 0xA53FFF, "Quality & Design", "QUALITY-DESIGN" },
	{ 0x0050C2, 0xA54000, 0xA54FFF, "Diamond Point International (Europe) Ltd", "DIAMOND-POINT-EUROPE" },
	{ 0x0050C2, 0xA55000, 0xA55FFF, "Arrowvale Electronics", "ARROWVALE-ELECTRONICS" },
	{ 0x0050C2, 0xA56000, 0xA56FFF, "ReaMetrix, Inc.", "REAMETRIX" },
	{ 0x0050C2, 0xA57000, 0xA57FFF, "Juice Technologies, LLC", "JUICE-TECHNOLOGIES-LLC" },
	{ 0x0050C2, 0xA58000, 0xA58FFF, "EPL", "EPL" },
	{ 0x0050C2, 0xA59000, 0xA59FFF, "GSP Sprachtechnologie GmbH", "GSP-SPRACHTECHNOLOGIE" },
	{ 0x0050C2, 0xA5A000, 0xA5AFFF, "ITAS A/S", "ITAS-A/S" },
	{ 0x0050C2, 0xA5B000, 0xA5BFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xA5C000, 0xA5CFFF, "JSC Component-ASU", "JSC-COMPONENT-ASU" },
	{ 0x0050C2, 0xA5D000, 0xA5DFFF, "MECC CO., LTD.", "MECC" },
	{ 0x0050C2, 0xA5E000, 0xA5EFFF, "Ansen Investment Holdings Ltd.", "ANSEN-INVESTMENT-HOLDINGS" },
	{ 0x0050C2, 0xA5F000, 0xA5FFFF, "Alga Microwave Inc", "ALGA-MICROWAVE" },
	{ 0x0050C2, 0xA60000, 0xA60FFF, "Arrow Central Europe GmbH - Division Spoerle", "ARROW-CENTRAL-EUROPE---DIVISION-SPOERLE" },
	{ 0x0050C2, 0xA61000, 0xA61FFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0xA62000, 0xA62FFF, "Grossenbacher Systeme AG", "GROSSENBACHER-SYSTEME" },
	{ 0x0050C2, 0xA63000, 0xA63FFF, "EMS Industries", "EMS-INDUSTRIES" },
	{ 0x0050C2, 0xA64000, 0xA64FFF, "tetronik GmbH AEN", "TETRONIK-AEN" },
	{ 0x0050C2, 0xA65000, 0xA65FFF, "Mark-O-Print GmbH", "MARK-O-PRINT" },
	{ 0x0050C2, 0xA66000, 0xA66FFF, "DVTech", "DVTECH" },
	{ 0x0050C2, 0xA67000, 0xA67FFF, "GSS Avionics Limited", "GSS-AVIONICS" },
	{ 0x0050C2, 0xA68000, 0xA68FFF, "X-Pert Paint Mixing Systems", "X-PERT-PAINT-MIXING" },
	{ 0x0050C2, 0xA69000, 0xA69FFF, "Advanced Integrated Systems", "ADVANCED-INTEGRATED" },
	{ 0x0050C2, 0xA6A000, 0xA6AFFF, "Infocrossing", "INFOCROSSING" },
	{ 0x0050C2, 0xA6B000, 0xA6BFFF, "Explorer Inc.", "EXPLORER" },
	{ 0x0050C2, 0xA6C000, 0xA6CFFF, "Figment Design Laboratories", "FIGMENT-DESIGN-LABORATORIES" },
	{ 0x0050C2, 0xA6D000, 0xA6DFFF, "DTV Innovations", "DTV-INNOVATIONS" },
	{ 0x0050C2, 0xA6E000, 0xA6EFFF, "Screen Technics Pty Limited", "SCREEN-TECHNICS-PTY" },
	{ 0x0050C2, 0xA6F000, 0xA6FFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xA70000, 0xA70FFF, "Reliable System Services Corp", "RELIABLE-SYSTEM-SERVICES" },
	{ 0x0050C2, 0xA71000, 0xA71FFF, "Purite Ltd", "PURITE" },
	{ 0x0050C2, 0xA72000, 0xA72FFF, "Gamber-Johnson LLC.", "GAMBER-JOHNSON-LLC" },
	{ 0x0050C2, 0xA73000, 0xA73FFF, "KYOEI ENGINEERING Co.,Ltd.", "KYOEI-ENGINEERING" },
	{ 0x0050C2, 0xA74000, 0xA74FFF, "DSP DESIGN LTD", "DSP-DESIGN" },
	{ 0x0050C2, 0xA75000, 0xA75FFF, "JTL Systems Ltd.", "JTL" },
	{ 0x0050C2, 0xA76000, 0xA76FFF, "Roesch & Walter Industrie-Elektronik GmbH", "ROESCH-WALTER-INDUSTRIE-ELEKTRONIK" },
	{ 0x0050C2, 0xA77000, 0xA77FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xA78000, 0xA78FFF, "Apantac LLC", "APANTAC-LLC" },
	{ 0x0050C2, 0xA79000, 0xA79FFF, "Saintronic", "SAINTRONIC" },
	{ 0x0050C2, 0xA7A000, 0xA7AFFF, "DetNet South Africa PTY (LTD)", "DETNET-SOUTH-AFRICA-PTY" },
	{ 0x0050C2, 0xA7B000, 0xA7BFFF, "Orange Tree Technologies", "ORANGE-TREE-TECHNOLOGIES" },
	{ 0x0050C2, 0xA7C000, 0xA7CFFF, "Pneu-Logic Corporation", "PNEU-LOGIC" },
	{ 0x0050C2, 0xA7D000, 0xA7DFFF, "Vitel Net", "VITEL-NET" },
	{ 0x0050C2, 0xA7E000, 0xA7EFFF, "Independent Project Engineering Ltd", "INDEPENDENT-PROJECT-ENGINEERING" },
	{ 0x0050C2, 0xA7F000, 0xA7FFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xA80000, 0xA80FFF, "ARD SA", "ARD" },
	{ 0x0050C2, 0xA81000, 0xA81FFF, "BPC circuits Ltd", "BPC-CIRCUITS" },
	{ 0x0050C2, 0xA82000, 0xA82FFF, "CT Company", "CT-COMPANY" },
	{ 0x0050C2, 0xA83000, 0xA83FFF, "Techno Sobi Co. Ltd.", "TECHNO-SOBI" },
	{ 0x0050C2, 0xA84000, 0xA84FFF, "Lino Manfrotto +Co spa", "LINO-MANFROTTO-+-SPA" },
	{ 0x0050C2, 0xA85000, 0xA85FFF, "JOYSYSTEM", "JOYSYSTEM" },
	{ 0x0050C2, 0xA86000, 0xA86FFF, "LIMAB AB", "LIMAB-AB" },
	{ 0x0050C2, 0xA87000, 0xA87FFF, "Littlemore Scientific", "LITTLEMORE-SCIENTIFIC" },
	{ 0x0050C2, 0xA88000, 0xA88FFF, "S-SYS", "S-SYS" },
	{ 0x0050C2, 0xA89000, 0xA89FFF, "CA Traffic Ltd", "CA-TRAFFIC" },
	{ 0x0050C2, 0xA8A000, 0xA8AFFF, "Audio Engineering Ltd.", "AUDIO-ENGINEERING" },
	{ 0x0050C2, 0xA8B000, 0xA8BFFF, "Navicron Oy", "NAVICRON-OY" },
	{ 0x0050C2, 0xA8C000, 0xA8CFFF, "Redwire, LLC", "REDWIRE-LLC" },
	{ 0x0050C2, 0xA8D000, 0xA8DFFF, "Frontier Electronic Systems Corp.", "FRONTIER" },
	{ 0x0050C2, 0xA8E000, 0xA8EFFF, "BFI Industrie-Elektronik GmbH & Co.KG", "BFI-INDUSTRIE-ELEKTRONIK" },
	{ 0x0050C2, 0xA8F000, 0xA8FFFF, "Quantum3D, Inc.", "QUANTUM3D" },
	{ 0x0050C2, 0xA90000, 0xA90FFF, "S.two Corporation", "S-TWO" },
	{ 0x0050C2, 0xA91000, 0xA91FFF, "Ceron Tech Co.,LTD", "CERON-TECH" },
	{ 0x0050C2, 0xA92000, 0xA92FFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0xA93000, 0xA93FFF, "SPX Dehydration & Filtration", "SPX-DEHYDRATION-FILTRATION" },
	{ 0x0050C2, 0xA94000, 0xA94FFF, "Par-Tech, Inc.", "PAR-TECH" },
	{ 0x0050C2, 0xA95000, 0xA95FFF, "INNOVACIONES Microelectrónicas SL (AnaFocus)", "INNOVACIONES-MICROELECTRóNICAS-SL-ANAFOCUS" },
	{ 0x0050C2, 0xA96000, 0xA96FFF, "FEP SRL", "FEP-SRL" },
	{ 0x0050C2, 0xA97000, 0xA97FFF, "MICROSYSTEMES", "MICROSYSTEMES" },
	{ 0x0050C2, 0xA98000, 0xA98FFF, "Sentry 360 Security", "SENTRY-360-SECURITY" },
	{ 0x0050C2, 0xA99000, 0xA99FFF, "Haivision Systems Inc", "HAIVISION" },
	{ 0x0050C2, 0xA9A000, 0xA9AFFF, "Absolutron. LLC", "ABSOLUTRON-LLC" },
	{ 0x0050C2, 0xA9B000, 0xA9BFFF, "PDQ Manufacturing Inc.", "PDQ-MANUFACTURING" },
	{ 0x0050C2, 0xA9C000, 0xA9CFFF, "Eberspächer Electronics GmbH & Co. KG", "EBERSPAECHER-ELECTRONICS" },
	{ 0x0050C2, 0xA9D000, 0xA9DFFF, "Joehl & Koeferli AG", "JOEHL-KOEFERLI" },
	{ 0x0050C2, 0xA9E000, 0xA9EFFF, "Procon Engineering Limited", "PROCON-ENGINEERING" },
	{ 0x0050C2, 0xA9F000, 0xA9FFFF, "YellowSoft Co., Ltd.", "YELLOWSOFT" },
	{ 0x0050C2, 0xAA0000, 0xAA0FFF, "Smith Meter, Inc.", "SMITH-METER" },
	{ 0x0050C2, 0xAA1000, 0xAA1FFF, "ELREM ELECTRONIC AG", "ELREM" },
	{ 0x0050C2, 0xAA2000, 0xAA2FFF, "ELPA sas", "ELPA-SAS" },
	{ 0x0050C2, 0xAA3000, 0xAA3FFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0xAA4000, 0xAA4FFF, "PSi Printer Systems international GmbH", "PSI-PRINTER" },
	{ 0x0050C2, 0xAA5000, 0xAA5FFF, "Tampere University of Technology", "TAMPERE-UNIVERSITY-OF" },
	{ 0x0050C2, 0xAA6000, 0xAA6FFF, "Bassett Electronic Systems ltd", "BASSETT" },
	{ 0x0050C2, 0xAA7000, 0xAA7FFF, "Endeas Oy", "ENDEAS-OY" },
	{ 0x0050C2, 0xAA8000, 0xAA8FFF, "Nexans Cabling Solutions", "NEXANS-CABLING-SOLUTIONS" },
	{ 0x0050C2, 0xAA9000, 0xAA9FFF, "SAN GIORGIO S.E.I.N. srl", "SAN-GIORGIO-S-E-I-N-SRL" },
	{ 0x0050C2, 0xAAA000, 0xAAAFFF, "Flexible Picture Systems", "FLEXIBLE-PICTURE" },
	{ 0x0050C2, 0xAAB000, 0xAABFFF, "BRS Sistemas Eletrônicos", "BRS-SISTEMAS-ELETRôNICOS" },
	{ 0x0050C2, 0xAAC000, 0xAACFFF, "VisiCon GmbH", "VISICON" },
	{ 0x0050C2, 0xAAD000, 0xAADFFF, "Update Systems Inc.", "UPDATE" },
	{ 0x0050C2, 0xAAE000, 0xAAEFFF, "OUTLINE srl", "OUTLINE-SRL" },
	{ 0x0050C2, 0xAAF000, 0xAAFFFF, "Santa Barbara Instrument Group", "SANTA-BARBARA-INSTRUMENT-GROUP" },
	{ 0x0050C2, 0xAB0000, 0xAB0FFF, "FRAKO Kondensatoren- und Anlagenbau GmbH", "FRAKO-KONDENSATOREN--UND-ANLAGENBAU" },
	{ 0x0050C2, 0xAB1000, 0xAB1FFF, "Bitmanufaktur GmbH", "BITMANUFAKTUR" },
	{ 0x0050C2, 0xAB2000, 0xAB2FFF, "ProCom Systems, Inc.", "PROCOM" },
	{ 0x0050C2, 0xAB3000, 0xAB3FFF, "Compañía de Instrumentacion y control, S.L.", "COMPAñíA-DE-INSTRUMENTACION-Y-CONTROL-S-L" },
	{ 0x0050C2, 0xAB4000, 0xAB4FFF, "n3k Informatik GmbH", "N3K-INFORMATIK" },
	{ 0x0050C2, 0xAB5000, 0xAB5FFF, "METTLER-TOLEDO HI-SPEED", "METTLER-TOLEDO-HI-SPEED" },
	{ 0x0050C2, 0xAB6000, 0xAB6FFF, "Gygax Embedded Engineering GEE.ch", "GYGAX-EMBEDDED-ENGINEERING-GEE-CH" },
	{ 0x0050C2, 0xAB7000, 0xAB7FFF, "Twinfalls Technologies", "TWINFALLS-TECHNOLOGIES" },
	{ 0x0050C2, 0xAB8000, 0xAB8FFF, "AHM Limited (CLiKAPAD)", "AHM-CLIKAPAD" },
	{ 0x0050C2, 0xAB9000, 0xAB9FFF, "Showtacle", "SHOWTACLE" },
	{ 0x0050C2, 0xABA000, 0xABAFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xABB000, 0xABBFFF, "Volantic AB", "VOLANTIC-AB" },
	{ 0x0050C2, 0xABC000, 0xABCFFF, "Barrick", "BARRICK" },
	{ 0x0050C2, 0xABD000, 0xABDFFF, "Monitor Business Machines Ltd.", "MONITOR-BUSINESS-MACHINES" },
	{ 0x0050C2, 0xABE000, 0xABEFFF, "AP Labs", "AP-LABS" },
	{ 0x0050C2, 0xABF000, 0xABFFFF, "MCC Computer Company", "MCC-COMPANY" },
	{ 0x0050C2, 0xAC0000, 0xAC0FFF, "DS PRO Audio Ltda", "DS-PRO-AUDIO-LTDA" },
	{ 0x0050C2, 0xAC1000, 0xAC1FFF, "DAISHIN-DENSHI Co., Ltd", "DAISHIN-DENSHI" },
	{ 0x0050C2, 0xAC2000, 0xAC2FFF, "OpenXS B.V.", "OPENXS" },
	{ 0x0050C2, 0xAC3000, 0xAC3FFF, "Diversified Control, Inc.", "DIVERSIFIED-CONTROL" },
	{ 0x0050C2, 0xAC4000, 0xAC4FFF, "Orion Technologies, Incorporated", "ORION-TECHNOLOGIES-INCORPORATED" },
	{ 0x0050C2, 0xAC5000, 0xAC5FFF, "E-Motion System, Inc.", "E-MOTION-SYSTEM" },
	{ 0x0050C2, 0xAC6000, 0xAC6FFF, "Marathon Products, Inc.", "MARATHON-PRODUCTS" },
	{ 0x0050C2, 0xAC7000, 0xAC7FFF, "WaveIP", "WAVEIP" },
	{ 0x0050C2, 0xAC8000, 0xAC8FFF, "Palladio Systeme GmbH", "PALLADIO-SYSTEME" },
	{ 0x0050C2, 0xAC9000, 0xAC9FFF, "Steinbeis-Transferzentrum Embedded Design und Networking", "STEINBEIS-TRANSFERZENTRUM-EMBEDDED-DESIGN-UND-NETWORKING" },
	{ 0x0050C2, 0xACA000, 0xACAFFF, "Soft & Control Technology s.r.o.", "SOFT-CONTROL-S-R-O" },
	{ 0x0050C2, 0xACB000, 0xACBFFF, "U-CARE INC.", "U-CARE" },
	{ 0x0050C2, 0xACC000, 0xACCFFF, "StockerYale", "STOCKERYALE" },
	{ 0x0050C2, 0xACD000, 0xACDFFF, "MeshWorks Wireless Oy", "MESHWORKS-WIRELESS-OY" },
	{ 0x0050C2, 0xACE000, 0xACEFFF, "ChronoLogic Pty. Ltd.", "CHRONOLOGIC-PTY" },
	{ 0x0050C2, 0xACF000, 0xACFFFF, "SP Controls, Inc", "SP-CONTROLS" },
	{ 0x0050C2, 0xAD0000, 0xAD0FFF, "Geonautics Australia Pty Ltd", "GEONAUTICS-AUSTRALIA-PTY" },
	{ 0x0050C2, 0xAD1000, 0xAD1FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xAD2000, 0xAD2FFF, "Rafael", "RAFAEL" },
	{ 0x0050C2, 0xAD3000, 0xAD3FFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0xAD4000, 0xAD4FFF, "Global Rainmakers Inc.", "GLOBAL-RAINMAKERS" },
	{ 0x0050C2, 0xAD5000, 0xAD5FFF, "Mighty Lube Systematic Lubrication, Inc.", "MIGHTY-LUBE-SYSTEMATIC-LUBRICATION" },
	{ 0x0050C2, 0xAD6000, 0xAD6FFF, "Unisensor A/S", "UNISENSOR-A/S" },
	{ 0x0050C2, 0xAD7000, 0xAD7FFF, "Air Monitors Ltd", "AIR-MONITORS" },
	{ 0x0050C2, 0xAD8000, 0xAD8FFF, "Incyma", "INCYMA" },
	{ 0x0050C2, 0xAD9000, 0xAD9FFF, "elettrondata srl", "ELETTRONDATA-SRL" },
	{ 0x0050C2, 0xADA000, 0xADAFFF, "Essepie Srl", "ESSEPIE-SRL" },
	{ 0x0050C2, 0xADB000, 0xADBFFF, "GO engineering GmbH", "GO-ENGINEERING" },
	{ 0x0050C2, 0xADC000, 0xADCFFF, "Synthesechemie Dr. Penth GmbH", "SYNTHESECHEMIE-DR-PENTH" },
	{ 0x0050C2, 0xADD000, 0xADDFFF, "General Dynamics C4 Sysems", "GENERAL-DYNAMICS-C4-SYSEMS" },
	{ 0x0050C2, 0xADE000, 0xADEFFF, "Neoptix Inc.", "NEOPTIX" },
	{ 0x0050C2, 0xADF000, 0xADFFFF, "Altinex, Inc", "ALTINEX" },
	{ 0x0050C2, 0xAE0000, 0xAE0FFF, "AT4 wireless.S.A", "AT4-WIRELESS" },
	{ 0x0050C2, 0xAE1000, 0xAE1FFF, "EVERCARE", "EVERCARE" },
	{ 0x0050C2, 0xAE2000, 0xAE2FFF, "Power Medical Interventions", "POWER-MEDICAL-INTERVENTIONS" },
	{ 0x0050C2, 0xAE3000, 0xAE3FFF, "PSD", "PSD" },
	{ 0x0050C2, 0xAE4000, 0xAE4FFF, "Advanced Electronic Designs, Inc.", "ADVANCED-DESIGNS" },
	{ 0x0050C2, 0xAE5000, 0xAE5FFF, "ABS Gesellschaft f. Automatisierung, Bildverarbeitung und Software mbH", "ABS-GESELLSCHAFT-F-AUTOMATISIERUNG-BILDVERARBEITUNG-UND-SOFTWARE-MBH" },
	{ 0x0050C2, 0xAE6000, 0xAE6FFF, "VECOM USA", "VECOM-USA" },
	{ 0x0050C2, 0xAE7000, 0xAE7FFF, "Redwood Systems", "REDWOOD" },
	{ 0x0050C2, 0xAE8000, 0xAE8FFF, "Bit-Lab PTY LTD", "BIT-LAB-PTY" },
	{ 0x0050C2, 0xAE9000, 0xAE9FFF, "ClearCorp Enterprises, Inc", "CLEARCORP-ENTERPRISES" },
	{ 0x0050C2, 0xAEA000, 0xAEAFFF, "EMBEDIA", "EMBEDIA" },
	{ 0x0050C2, 0xAEB000, 0xAEBFFF, "UMLogics Corporation", "UMLOGICS" },
	{ 0x0050C2, 0xAEC000, 0xAECFFF, "Fritz Pauker Ingenieure GmbH", "FRITZ-PAUKER-INGENIEURE" },
	{ 0x0050C2, 0xAED000, 0xAEDFFF, "3Roam", "3ROAM" },
	{ 0x0050C2, 0xAEE000, 0xAEEFFF, "IPtec, Inc.", "IPTEC" },
	{ 0x0050C2, 0xAEF000, 0xAEFFFF, "National CineMedia", "NATIONAL-CINEMEDIA" },
	{ 0x0050C2, 0xAF0000, 0xAF0FFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0xAF1000, 0xAF1FFF, "Green Goose", "GREEN-GOOSE" },
	{ 0x0050C2, 0xAF2000, 0xAF2FFF, "ACD Elektronik Gmbh", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0xAF3000, 0xAF3FFF, "Palomar Products, Inc.", "PALOMAR-PRODUCTS" },
	{ 0x0050C2, 0xAF4000, 0xAF4FFF, "Dixell S.p.A.", "DIXELL-S-P-A" },
	{ 0x0050C2, 0xAF5000, 0xAF5FFF, "Kramara s.r.o.", "KRAMARA-S-R-O" },
	{ 0x0050C2, 0xAF6000, 0xAF6FFF, "Energid", "ENERGID" },
	{ 0x0050C2, 0xAF7000, 0xAF7FFF, "Midwest Microwave Solutions Inc.", "MIDWEST-MICROWAVE-SOLUTIONS" },
	{ 0x0050C2, 0xAF8000, 0xAF8FFF, "Global Satellite Engineering", "GLOBAL-SATELLITE-ENGINEERING" },
	{ 0x0050C2, 0xAF9000, 0xAF9FFF, "Ingenieurbuero Bickele und Buehler GmbH", "INGENIEURBUERO-BICKELE-UND-BUEHLER" },
	{ 0x0050C2, 0xAFA000, 0xAFAFFF, "Absolute Fire Solutions Inc.", "ABSOLUTE-FIRE-SOLUTIONS" },
	{ 0x0050C2, 0xAFB000, 0xAFBFFF, "Vocality International Ltd", "VOCALITY" },
	{ 0x0050C2, 0xAFC000, 0xAFCFFF, "Odus Technologies SA", "ODUS-TECHNOLOGIES" },
	{ 0x0050C2, 0xAFD000, 0xAFDFFF, "HomeScenario, Inc.", "HOMESCENARIO" },
	{ 0x0050C2, 0xAFE000, 0xAFEFFF, "Trolex Limited", "TROLEX" },
	{ 0x0050C2, 0xAFF000, 0xAFFFFF, "XoByte LLC", "XOBYTE-LLC" },
	{ 0x0050C2, 0xB00000, 0xB00FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xB01000, 0xB01FFF, "HSR Harald L. Reuter", "HSR-HARALD-L-REUTER" },
	{ 0x0050C2, 0xB02000, 0xB02FFF, "MASTER CO LTD", "MASTER" },
	{ 0x0050C2, 0xB03000, 0xB03FFF, "Spider Tecnologia Ind. e Com. Ltda.", "SPIDER-TECNOLOGIA-IND-E-COM-LTDA" },
	{ 0x0050C2, 0xB04000, 0xB04FFF, "Ubiquiti Networks", "UBIQUITI-NETWORKS" },
	{ 0x0050C2, 0xB05000, 0xB05FFF, "POLA s.r.l.", "POLA-S-R-L" },
	{ 0x0050C2, 0xB06000, 0xB06FFF, "CompuDesigns, Inc.", "COMPUDESIGNS" },
	{ 0x0050C2, 0xB07000, 0xB07FFF, "FARECO", "FARECO" },
	{ 0x0050C2, 0xB08000, 0xB08FFF, "Goerlitz AG", "GOERLITZ" },
	{ 0x0050C2, 0xB09000, 0xB09FFF, "Harper Chalice Group Limited", "HARPER-CHALICE-GROUP" },
	{ 0x0050C2, 0xB0A000, 0xB0AFFF, "Indutherm Giesstechnologie GmbH", "INDUTHERM-GIESSTECHNOLOGIE" },
	{ 0x0050C2, 0xB0B000, 0xB0BFFF, "Honeywell", "HONEYWELL" },
	{ 0x0050C2, 0xB0C000, 0xB0CFFF, "SMARTB TECHNOLOGIES", "SMARTB-TECHNOLOGIES" },
	{ 0x0050C2, 0xB0D000, 0xB0DFFF, "Japan Electronics System, Inc", "JAPAN-ELECTRONICS-SYSTEM" },
	{ 0x0050C2, 0xB0E000, 0xB0EFFF, "KYAB Lulea AB", "KYAB-LULEA-AB" },
	{ 0x0050C2, 0xB0F000, 0xB0FFFF, "NARA Controls Inc.", "NARA-CONTROLS" },
	{ 0x0050C2, 0xB10000, 0xB10FFF, "Marine Entertainment Systems Ltd", "MARINE-ENTERTAINMENT" },
	{ 0x0050C2, 0xB11000, 0xB11FFF, "EXEL s.r.l", "EXEL-S-R-L" },
	{ 0x0050C2, 0xB12000, 0xB12FFF, "CM Elektronik GmbH", "CM-ELEKTRONIK" },
	{ 0x0050C2, 0xB13000, 0xB13FFF, "Measy Electronics Co., Ltd.", "MEASY-ELECTRONICS" },
	{ 0x0050C2, 0xB14000, 0xB14FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xB15000, 0xB15FFF, "PhotoTelesis LP", "PHOTOTELESIS-LP" },
	{ 0x0050C2, 0xB16000, 0xB16FFF, "Neothings, Inc.", "NEOTHINGS" },
	{ 0x0050C2, 0xB17000, 0xB17FFF, "Elcoteq Design Center Oy", "ELCOTEQ-DESIGN-CENTER-OY" },
	{ 0x0050C2, 0xB18000, 0xB18FFF, "Rosslare Enterprises Limited", "ROSSLARE-ENTERPRISES" },
	{ 0x0050C2, 0xB19000, 0xB19FFF, "Polytron Corporation", "POLYTRON" },
	{ 0x0050C2, 0xB1A000, 0xB1AFFF, "ELCUS", "ELCUS" },
	{ 0x0050C2, 0xB1B000, 0xB1BFFF, "Integrated Control Corp.", "INTEGRATED-CONTROL" },
	{ 0x0050C2, 0xB1C000, 0xB1CFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xB1D000, 0xB1DFFF, "Telegenix", "TELEGENIX" },
	{ 0x0050C2, 0xB1E000, 0xB1EFFF, "Abbott Medical Optics", "ABBOTT-MEDICAL-OPTICS" },
	{ 0x0050C2, 0xB1F000, 0xB1FFFF, "SCA Schucker GmbH & Co.", "SCA-SCHUCKER" },
	{ 0x0050C2, 0xB20000, 0xB20FFF, "FIVE9 NETWORK SYSTEMS LLC", "FIVE9-NETWORK-LLC" },
	{ 0x0050C2, 0xB21000, 0xB21FFF, "Phytron-Elektronik GmbH", "PHYTRON-ELEKTRONIK" },
	{ 0x0050C2, 0xB22000, 0xB22FFF, "FarSite Communications Limited", "FARSITE-COMMUNICATION" },
	{ 0x0050C2, 0xB23000, 0xB23FFF, "Ronyo Technologies s.r.o.", "RONYO-TECHNOLOGIES-S-R-O" },
	{ 0x0050C2, 0xB24000, 0xB24FFF, "Teledyne Defence Limited", "TELEDYNE-DEFENCE" },
	{ 0x0050C2, 0xB25000, 0xB25FFF, "Triax A/S", "TRIAX-A/S" },
	{ 0x0050C2, 0xB26000, 0xB26FFF, "Elko Systems", "ELKO" },
	{ 0x0050C2, 0xB27000, 0xB27FFF, "ATEME", "ATEME" },
	{ 0x0050C2, 0xB28000, 0xB28FFF, "Micromax Pty. Ltd.", "MICROMAX-PTY" },
	{ 0x0050C2, 0xB29000, 0xB29FFF, "Integra LifeSciences (Ireland) Ltd", "INTEGRA-LIFESCIENCES-IRELAND" },
	{ 0x0050C2, 0xB2A000, 0xB2AFFF, "Trench Austria GmbH", "TRENCH-AUSTRIA" },
	{ 0x0050C2, 0xB2B000, 0xB2BFFF, "CosmoData Informatica Ltda.", "COSMODATA-INFORMATICA-LTDA" },
	{ 0x0050C2, 0xB2C000, 0xB2CFFF, "Concepteers, LLC", "CONCEPTEERS-LLC" },
	{ 0x0050C2, 0xB2D000, 0xB2DFFF, "Datasat Digital Entertainment", "DATASAT-DIGITAL-ENTERTAINMENT" },
	{ 0x0050C2, 0xB2E000, 0xB2EFFF, "ACT", "ACT" },
	{ 0x0050C2, 0xB2F000, 0xB2FFFF, "IntelliVision Technologies, Corp", "INTELLIVISION-TECHNOLOGIES" },
	{ 0x0050C2, 0xB30000, 0xB30FFF, "Applied Micro Electronics AME BV", "APPLIED-MICRO-ELECTRONICS-AME" },
	{ 0x0050C2, 0xB31000, 0xB31FFF, "Shop Safe AG", "SHOP-SAFE" },
	{ 0x0050C2, 0xB32000, 0xB32FFF, "Byres Security Inc", "BYRES-SECURITY" },
	{ 0x0050C2, 0xB33000, 0xB33FFF, "Numcore Ltd", "NUMCORE" },
	{ 0x0050C2, 0xB34000, 0xB34FFF, "Meisol co.,ltd", "MEISOL" },
	{ 0x0050C2, 0xB35000, 0xB35FFF, "haneron", "HANERON" },
	{ 0x0050C2, 0xB36000, 0xB36FFF, "CRDE", "CRDE" },
	{ 0x0050C2, 0xB37000, 0xB37FFF, "IAdea Corporation", "IADEA" },
	{ 0x0050C2, 0xB38000, 0xB38FFF, "Grenmore Ltd", "GRENMORE" },
	{ 0x0050C2, 0xB39000, 0xB39FFF, "siXis, Inc.", "SIXIS" },
	{ 0x0050C2, 0xB3A000, 0xB3AFFF, "Nikon Systems Inc.", "NIKON" },
	{ 0x0050C2, 0xB3B000, 0xB3BFFF, "Sportvision Inc.", "SPORTVISION" },
	{ 0x0050C2, 0xB3C000, 0xB3CFFF, "JanasCard", "JANASCARD" },
	{ 0x0050C2, 0xB3D000, 0xB3DFFF, "AMS", "AMS" },
	{ 0x0050C2, 0xB3E000, 0xB3EFFF, "Sage Consultants", "SAGE-CONSULTANTS" },
	{ 0x0050C2, 0xB3F000, 0xB3FFFF, "M-Tronic Design and Technology GmbH", "M-TRONIC-DESIGN-AND" },
	{ 0x0050C2, 0xB40000, 0xB40FFF, "Tecnint HTE SRL", "TECNINT-HTE-SRL" },
	{ 0x0050C2, 0xB41000, 0xB41FFF, "Tata Power Company, Strategic Electronics Division", "TATA-POWER-COMPANY-STRATEGIC-ELECTRONICS-DIVISION" },
	{ 0x0050C2, 0xB42000, 0xB42FFF, "ETM Electromatic Incorporated", "ETM-ELECTROMATIC-INCORPORATED" },
	{ 0x0050C2, 0xB43000, 0xB43FFF, "J-Systems Inc.", "J" },
	{ 0x0050C2, 0xB44000, 0xB44FFF, "Ampcontrol Pty Ltd", "AMPCONTROL-PTY" },
	{ 0x0050C2, 0xB45000, 0xB45FFF, "Efftronics Systems (P) Ltd", "EFFTRONICS-P" },
	{ 0x0050C2, 0xB46000, 0xB46FFF, "Mobileye", "MOBILEYE" },
	{ 0x0050C2, 0xB47000, 0xB47FFF, "MCS MICRONIC Computer Systeme GmbH", "MCS-MICRONIC-SYSTEME" },
	{ 0x0050C2, 0xB48000, 0xB48FFF, "MTD GmbH", "MTD" },
	{ 0x0050C2, 0xB49000, 0xB49FFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x0050C2, 0xB4A000, 0xB4AFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xB4B000, 0xB4BFFF, "Chitose Co.,Ltd", "CHITOSE" },
	{ 0x0050C2, 0xB4C000, 0xB4CFFF, "ElectroCom", "ELECTROCOM" },
	{ 0x0050C2, 0xB4D000, 0xB4DFFF, "Troll Systems Corporation", "TROLL" },
	{ 0x0050C2, 0xB4E000, 0xB4EFFF, "AixControl GmbH", "AIXCONTROL" },
	{ 0x0050C2, 0xB4F000, 0xB4FFFF, "Sencon UK Ltd.", "SENCON-UK" },
	{ 0x0050C2, 0xB50000, 0xB50FFF, "SELCO", "SELCO" },
	{ 0x0050C2, 0xB51000, 0xB51FFF, "Aeroflex GmbH", "AEROFLEX" },
	{ 0x0050C2, 0xB52000, 0xB52FFF, "SMH Technologies", "SMH-TECHNOLOGIES" },
	{ 0x0050C2, 0xB53000, 0xB53FFF, "Prodco", "PRODCO" },
	{ 0x0050C2, 0xB54000, 0xB54FFF, "APG Cash Drawer, LLC", "APG-CASH-DRAWER-LLC" },
	{ 0x0050C2, 0xB55000, 0xB55FFF, "SANYO ELECTRONIC INDUSTRIES CO.,LTD", "SANYO-INDUSTRIES" },
	{ 0x0050C2, 0xB56000, 0xB56FFF, "SINOVIA SA", "SINOVIA" },
	{ 0x0050C2, 0xB57000, 0xB57FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xB58000, 0xB58FFF, "Real D", "REAL-D" },
	{ 0x0050C2, 0xB59000, 0xB59FFF, "SLICAN sp. z o.o.", "SLICAN-SP-Z-O-O" },
	{ 0x0050C2, 0xB5A000, 0xB5AFFF, "GREEN Center s.r.o.", "GREEN-CENTER-S-R-O" },
	{ 0x0050C2, 0xB5B000, 0xB5BFFF, "Timberline Mfg Company", "TIMBERLINE-MFG-COMPANY" },
	{ 0x0050C2, 0xB5C000, 0xB5CFFF, "ADI Video Technologies", "ADI-VIDEO-TECHNOLOGIES" },
	{ 0x0050C2, 0xB5D000, 0xB5DFFF, "Plitron Manufacturing Inc.", "PLITRON-MANUFACTURING" },
	{ 0x0050C2, 0xB5E000, 0xB5EFFF, "Palgiken Co.,Ltd.", "PALGIKEN" },
	{ 0x0050C2, 0xB5F000, 0xB5FFFF, "North Bridge Technologies", "NORTH-BRIDGE-TECHNOLOGIES" },
	{ 0x0050C2, 0xB60000, 0xB60FFF, "OOO NPF ATIS", "OOO-NPF-ATIS" },
	{ 0x0050C2, 0xB61000, 0xB61FFF, "Nayos LTD", "NAYOS" },
	{ 0x0050C2, 0xB62000, 0xB62FFF, "Measurement Technology NW", "MEASUREMENT-NW" },
	{ 0x0050C2, 0xB63000, 0xB63FFF, "RO.VE.R. Laboratories S.p.A", "RO-VE-R-LABORATORIES-S-P-A" },
	{ 0x0050C2, 0xB64000, 0xB64FFF, "FEW Bauer GmbH", "FEW-BAUER" },
	{ 0x0050C2, 0xB65000, 0xB65FFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0xB66000, 0xB66FFF, "Deuta-Werke GmbH", "DEUTA-WERKE" },
	{ 0x0050C2, 0xB67000, 0xB67FFF, "RC Systems Co. Inc.", "RC" },
	{ 0x0050C2, 0xB68000, 0xB68FFF, "Electronic Systems Protection, Inc.", "PROTECTION" },
	{ 0x0050C2, 0xB69000, 0xB69FFF, "Thetis S.p.A.", "THETIS-S-P-A" },
	{ 0x0050C2, 0xB6A000, 0xB6AFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xB6B000, 0xB6BFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xB6C000, 0xB6CFFF, "Drinelec", "DRINELEC" },
	{ 0x0050C2, 0xB6D000, 0xB6DFFF, "Sound Metrics Corp", "SOUND-METRICS" },
	{ 0x0050C2, 0xB6E000, 0xB6EFFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0xB6F000, 0xB6FFFF, "CT Company", "CT-COMPANY" },
	{ 0x0050C2, 0xB70000, 0xB70FFF, "Nisshin Electronics co.,ltd.", "NISSHIN-ELECTRONICS" },
	{ 0x0050C2, 0xB71000, 0xB71FFF, "Digitale Analoge COMponenten West Electronic Vertriebs GmbH", "DIGITALE-ANALOGE-COMPONENTEN-WEST-VERTRIEBS" },
	{ 0x0050C2, 0xB72000, 0xB72FFF, "Advanced Desktop Systems Ltd", "ADVANCED-DESKTOP" },
	{ 0x0050C2, 0xB73000, 0xB73FFF, "ARKRAY, Inc. Kyoto Laboratory", "ARKRAY-KYOTO-LABORATORY" },
	{ 0x0050C2, 0xB74000, 0xB74FFF, "AXED Jakubowski Wojciechowski sp.j.", "AXED-JAKUBOWSKI-WOJCIECHOWSKI-SP-J" },
	{ 0x0050C2, 0xB75000, 0xB75FFF, "Blankom", "BLANKOM" },
	{ 0x0050C2, 0xB76000, 0xB76FFF, "ITF Fröschl GmbH", "ITF-FROESCHL" },
	{ 0x0050C2, 0xB77000, 0xB77FFF, "KRISTECH", "KRISTECH" },
	{ 0x0050C2, 0xB78000, 0xB78FFF, "Folink", "FOLINK" },
	{ 0x0050C2, 0xB79000, 0xB79FFF, "MITSUYA LABORATORIES INC.", "MITSUYA-LABORATORIES" },
	{ 0x0050C2, 0xB7A000, 0xB7AFFF, "Schnoor Industrieelektronik GmbH & Co. KG", "SCHNOOR-INDUSTRIEELEKTRONIK" },
	{ 0x0050C2, 0xB7B000, 0xB7BFFF, "QUARTECH CORPORATION", "QUARTECH" },
	{ 0x0050C2, 0xB7C000, 0xB7CFFF, "Bettini srl", "BETTINI-SRL" },
	{ 0x0050C2, 0xB7D000, 0xB7DFFF, "ELETECH Srl", "ELETECH-SRL" },
	{ 0x0050C2, 0xB7E000, 0xB7EFFF, "NARETRENDS", "NARETRENDS" },
	{ 0x0050C2, 0xB7F000, 0xB7FFFF, "Enatel", "ENATEL" },
	{ 0x0050C2, 0xB80000, 0xB80FFF, "iScreen LLC", "ISCREEN-LLC" },
	{ 0x0050C2, 0xB81000, 0xB81FFF, "GHL Advanced Technolgy GmbH & Co. KG", "GHL-ADVANCED-TECHNOLGY" },
	{ 0x0050C2, 0xB82000, 0xB82FFF, "TANABIKI Inc.", "TANABIKI" },
	{ 0x0050C2, 0xB83000, 0xB83FFF, "Advanced Storage Concepts, Inc.", "ADVANCED-STORAGE-CONCEPTS" },
	{ 0x0050C2, 0xB84000, 0xB84FFF, "Innovate Software Solutions Pvt Ltd", "INNOVATE-SOFTWARE-SOLUTIONS-PVT" },
	{ 0x0050C2, 0xB85000, 0xB85FFF, "SilverNet", "SILVERNET" },
	{ 0x0050C2, 0xB86000, 0xB86FFF, "ASTO", "ASTO" },
	{ 0x0050C2, 0xB87000, 0xB87FFF, "EMAC, Inc.", "EMAC" },
	{ 0x0050C2, 0xB88000, 0xB88FFF, "Gigatronik Köln GmbH", "GIGATRONIK-KOELN" },
	{ 0x0050C2, 0xB89000, 0xB89FFF, "ENTEC Electric & Electronic Co., LTD.", "ENTEC-ELECTRIC" },
	{ 0x0050C2, 0xB8A000, 0xB8AFFF, "MicroPoise", "MICROPOISE" },
	{ 0x0050C2, 0xB8B000, 0xB8BFFF, "FBB", "FBB" },
	{ 0x0050C2, 0xB8C000, 0xB8CFFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xB8D000, 0xB8DFFF, "CEMSI", "CEMSI" },
	{ 0x0050C2, 0xB8E000, 0xB8EFFF, "WAC (Israel) Ltd.", "WAC-ISRAEL" },
	{ 0x0050C2, 0xB8F000, 0xB8FFFF, "Gentec", "GENTEC" },
	{ 0x0050C2, 0xB90000, 0xB90FFF, "NAONWORKS Co., Ltd", "NAONWORKS" },
	{ 0x0050C2, 0xB91000, 0xB91FFF, "Finnet-Service Ltd.", "FINNET-SERVICE" },
	{ 0x0050C2, 0xB92000, 0xB92FFF, "ABB Transmission and Distribution Auto Eqip(Xiamen.China)", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTO-EQIP-XIAMEN-CHINA" },
	{ 0x0050C2, 0xB93000, 0xB93FFF, "EMC PARTNER AG", "EMC-PARTNER" },
	{ 0x0050C2, 0xB94000, 0xB94FFF, "Tritech International Ltd", "TRITECH" },
	{ 0x0050C2, 0xB95000, 0xB95FFF, "Rx Monitoring Services", "RX-MONITORING-SERVICES" },
	{ 0x0050C2, 0xB96000, 0xB96FFF, "Onix Electronic Systems Inc", "ONIX" },
	{ 0x0050C2, 0xB97000, 0xB97FFF, "ikerlan", "IKERLAN" },
	{ 0x0050C2, 0xB98000, 0xB98FFF, "Southwest Research Institute", "SOUTHWEST-RESEARCH-INSTITUTE" },
	{ 0x0050C2, 0xB99000, 0xB99FFF, "Greenlight Innovation Corp.", "GREENLIGHT-INNOVATION" },
	{ 0x0050C2, 0xB9A000, 0xB9AFFF, "Talo, NV Inc", "TALO-NV" },
	{ 0x0050C2, 0xB9B000, 0xB9BFFF, "Telventy Energia S.A.", "TELVENTY-ENERGIA" },
	{ 0x0050C2, 0xB9C000, 0xB9CFFF, "Dave srl", "DAVE-SRL" },
	{ 0x0050C2, 0xB9D000, 0xB9DFFF, "W. Vershoven GmbH", "W-VERSHOVEN" },
	{ 0x0050C2, 0xB9E000, 0xB9EFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xB9F000, 0xB9FFFF, "AUDIOSCOPE 2K SRL", "AUDIOSCOPE-2K-SRL" },
	{ 0x0050C2, 0xBA0000, 0xBA0FFF, "txtr GmbH", "TXTR" },
	{ 0x0050C2, 0xBA1000, 0xBA1FFF, "Transtechnik GmbH & Co.KG", "TRANSTECHNIK" },
	{ 0x0050C2, 0xBA2000, 0xBA2FFF, "Logical Tools s.r.l.", "LOGICAL-TOOLS-S-R-L" },
	{ 0x0050C2, 0xBA3000, 0xBA3FFF, "DSP DESIGN LTD", "DSP-DESIGN" },
	{ 0x0050C2, 0xBA4000, 0xBA4FFF, "CUSTOS MOBILE S.L.", "CUSTOS-MOBILE-S-L" },
	{ 0x0050C2, 0xBA5000, 0xBA5FFF, "InterCel Pty Ltd", "INTERCEL-PTY" },
	{ 0x0050C2, 0xBA6000, 0xBA6FFF, "Jomitek", "JOMITEK" },
	{ 0x0050C2, 0xBA7000, 0xBA7FFF, "RaumComputer Entwicklungs- und Vertriebs GmbH", "RAUMCOMPUTER-ENTWICKLUNGS--UND-VERTRIEBS" },
	{ 0x0050C2, 0xBA8000, 0xBA8FFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0xBA9000, 0xBA9FFF, "SISS Technology Inc.", "SISS" },
	{ 0x0050C2, 0xBAA000, 0xBAAFFF, "NetworkFX Communications, LLC", "NETWORKFX-COMMUNICATION-LLC" },
	{ 0x0050C2, 0xBAB000, 0xBABFFF, "iDeal Teknoloji Bilisim Cozumleri A.S.", "IDEAL-TEKNOLOJI-BILISIM-COZUMLERI-A-S" },
	{ 0x0050C2, 0xBAC000, 0xBACFFF, "VITECO VNPT JSC", "VITECO-VNPT-JSC" },
	{ 0x0050C2, 0xBAD000, 0xBADFFF, "Prediktor AS", "PREDIKTOR-AS" },
	{ 0x0050C2, 0xBAE000, 0xBAEFFF, "Fiber Connections Inc.", "FIBER-CONNECTIONS" },
	{ 0x0050C2, 0xBAF000, 0xBAFFFF, "MangoDSP", "MANGODSP" },
	{ 0x0050C2, 0xBB0000, 0xBB0FFF, "Gainbrain", "GAINBRAIN" },
	{ 0x0050C2, 0xBB1000, 0xBB1FFF, "Pro4tech", "PRO4TECH" },
	{ 0x0050C2, 0xBB2000, 0xBB2FFF, "St Michael Strategies Inc", "ST-MICHAEL-STRATEGIES" },
	{ 0x0050C2, 0xBB3000, 0xBB3FFF, "ClimateWell AB (publ)", "CLIMATEWELL-AB-PUBL" },
	{ 0x0050C2, 0xBB4000, 0xBB4FFF, "JSC Electrical Equipment Factory", "JSC-ELECTRICAL-EQUIPMENT-FACTORY" },
	{ 0x0050C2, 0xBB5000, 0xBB5FFF, "MROAD INFORMATION SYSTEM", "MROAD-INFORMATION-SYSTEM" },
	{ 0x0050C2, 0xBB6000, 0xBB6FFF, "Quarch Technology Ltd", "QUARCH" },
	{ 0x0050C2, 0xBB7000, 0xBB7FFF, "General Dynamics C4 Systems", "GENERAL-DYNAMICS-C4" },
	{ 0x0050C2, 0xBB8000, 0xBB8FFF, "MoeTronix", "MOETRONIX" },
	{ 0x0050C2, 0xBB9000, 0xBB9FFF, "Toptech Systems, Inc.", "TOPTECH" },
	{ 0x0050C2, 0xBBA000, 0xBBAFFF, "Systemteq Limited", "SYSTEMTEQ" },
	{ 0x0050C2, 0xBBB000, 0xBBBFFF, "GHL Systems Berhad", "GHL-BERHAD" },
	{ 0x0050C2, 0xBBC000, 0xBBCFFF, "ImpactSystems", "IMPACTSYSTEMS" },
	{ 0x0050C2, 0xBBD000, 0xBBDFFF, "ITS Telecom", "ITS-TELECOM" },
	{ 0x0050C2, 0xBBE000, 0xBBEFFF, "Onlinepizza Norden AB", "ONLINEPIZZA-NORDEN-AB" },
	{ 0x0050C2, 0xBBF000, 0xBBFFFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xBC0000, 0xBC0FFF, "Galaxia Electronics", "GALAXIA-ELECTRONICS" },
	{ 0x0050C2, 0xBC1000, 0xBC1FFF, "Sentec Ltd", "SENTEC" },
	{ 0x0050C2, 0xBC2000, 0xBC2FFF, "XSLENT Energy Technologies LLC", "XSLENT-ENERGY-TECHNOLOGIES-LLC" },
	{ 0x0050C2, 0xBC3000, 0xBC3FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xBC4000, 0xBC4FFF, "Wheatstone Corporation", "WHEATSTONE" },
	{ 0x0050C2, 0xBC5000, 0xBC5FFF, "Toptechnology SRL", "TOPTECHNOLOGY-SRL" },
	{ 0x0050C2, 0xBC6000, 0xBC6FFF, "MireroTack", "MIREROTACK" },
	{ 0x0050C2, 0xBC7000, 0xBC7FFF, "PTS GmbH", "PTS" },
	{ 0x0050C2, 0xBC8000, 0xBC8FFF, "AGWTech Ltd", "AGWTECH" },
	{ 0x0050C2, 0xBC9000, 0xBC9FFF, "Nextmove Technologies", "NEXTMOVE-TECHNOLOGIES" },
	{ 0x0050C2, 0xBCA000, 0xBCAFFF, "Aitecsystem Co.,Ltd.", "AITECSYSTEM" },
	{ 0x0050C2, 0xBCB000, 0xBCBFFF, "ARTEIXO TELECOM", "ARTEIXO-TELECOM" },
	{ 0x0050C2, 0xBCC000, 0xBCCFFF, "VVDN TECHNOLOGIES PVT. LTD.", "VVDN-TECHNOLOGIES-PVT" },
	{ 0x0050C2, 0xBCD000, 0xBCDFFF, "Highlight Parking Systems Ltd", "HIGHLIGHT-PARKING" },
	{ 0x0050C2, 0xBCE000, 0xBCEFFF, "TV Portal Co., Ltd.", "TV-PORTAL" },
	{ 0x0050C2, 0xBCF000, 0xBCFFFF, "Epiko, elektronski sistemi d.o.o.", "EPIKO-ELEKTRONSKI-SISTEMI-D-O-O" },
	{ 0x0050C2, 0xBD0000, 0xBD0FFF, "EDC wifi", "EDC-WIFI" },
	{ 0x0050C2, 0xBD1000, 0xBD1FFF, "Ariem Technologies Pvt Ltd", "ARIEM-TECHNOLOGIES-PVT" },
	{ 0x0050C2, 0xBD2000, 0xBD2FFF, "Percello Ltd.", "PERCELLO" },
	{ 0x0050C2, 0xBD3000, 0xBD3FFF, "Postjet Systems Ltd", "POSTJET" },
	{ 0x0050C2, 0xBD4000, 0xBD4FFF, "Global Security Devices", "GLOBAL-SECURITY-DEVICES" },
	{ 0x0050C2, 0xBD5000, 0xBD5FFF, "RF-Embedded GmbH", "RF-EMBEDDED" },
	{ 0x0050C2, 0xBD6000, 0xBD6FFF, "BG Systems, Inc.", "BG" },
	{ 0x0050C2, 0xBD7000, 0xBD7FFF, "SLAT", "SLAT" },
	{ 0x0050C2, 0xBD8000, 0xBD8FFF, "b.a.b-technologie gmbh", "B-A-B-TECHNOLOGIE" },
	{ 0x0050C2, 0xBD9000, 0xBD9FFF, "AMS Controls, Inc.", "AMS-CONTROLS" },
	{ 0x0050C2, 0xBDA000, 0xBDAFFF, "Digital Lumens", "DIGITAL-LUMENS" },
	{ 0x0050C2, 0xBDB000, 0xBDBFFF, "GasTOPS Ltd.", "GASTOPS" },
	{ 0x0050C2, 0xBDC000, 0xBDCFFF, "SS Systems LLC", "SS-LLC" },
	{ 0x0050C2, 0xBDE000, 0xBDEFFF, "Evo-Teh d.o.o.", "EVO-TEH-D-O-O" },
	{ 0x0050C2, 0xBDF000, 0xBDFFFF, "Euro-Konsult Sp. z o.o.", "EURO-KONSULT-SP-Z-O-O" },
	{ 0x0050C2, 0xBE0000, 0xBE0FFF, "Phaedrus Limited", "PHAEDRUS" },
	{ 0x0050C2, 0xBE1000, 0xBE1FFF, "TATTILE SRL", "TATTILE-SRL" },
	{ 0x0050C2, 0xBE2000, 0xBE2FFF, "Convergent Bioscience Ltd.", "CONVERGENT-BIOSCIENCE" },
	{ 0x0050C2, 0xBE3000, 0xBE3FFF, "Jiskoot Ltd", "JISKOOT" },
	{ 0x0050C2, 0xBE4000, 0xBE4FFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0xBE5000, 0xBE5FFF, "RF Code, Inc", "RF-CODE" },
	{ 0x0050C2, 0xBE6000, 0xBE6FFF, "Docobo Ltd", "DOCOBO" },
	{ 0x0050C2, 0xBE7000, 0xBE7FFF, "Genetec Inc.", "GENETEC" },
	{ 0x0050C2, 0xBE8000, 0xBE8FFF, "VEHICLE TESTING EQUIPMENT, S.L.", "VEHICLE-TESTING-EQUIPMENT-S-L" },
	{ 0x0050C2, 0xBE9000, 0xBE9FFF, "ZUCCHETTI SPA", "ZUCCHETTI-SPA" },
	{ 0x0050C2, 0xBEA000, 0xBEAFFF, "Daeyoung inc.", "DAEYOUNG" },
	{ 0x0050C2, 0xBEB000, 0xBEBFFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0xBEC000, 0xBECFFF, "DRS Laruel Technologies", "DRS-LARUEL-TECHNOLOGIES" },
	{ 0x0050C2, 0xBED000, 0xBEDFFF, "Touch Revolution Inc.", "TOUCH-REVOLUTION" },
	{ 0x0050C2, 0xBEE000, 0xBEEFFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0xBEF000, 0xBEFFFF, "SOCIEDAD IBERICA DE CONSTRUCCIONES ELECTRICAS, S.A. (SICE)", "SOCIEDAD-IBERICA-DE-CONSTRUCCIONES-ELECTRICAS-SICE" },
	{ 0x0050C2, 0xBF0000, 0xBF0FFF, "AIM", "AIM" },
	{ 0x0050C2, 0xBF1000, 0xBF1FFF, "Amatic Industries GmbH", "AMATIC-INDUSTRIES" },
	{ 0x0050C2, 0xBF2000, 0xBF2FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xBF3000, 0xBF3FFF, "Wanco Inc.", "WANCO" },
	{ 0x0050C2, 0xBF4000, 0xBF4FFF, "Monarch Innovative Technologies Pvt Ltd", "MONARCH-INNOVATIVE-TECHNOLOGIES-PVT" },
	{ 0x0050C2, 0xBF5000, 0xBF5FFF, "AILES ELECTRONICS CO., LTD.", "AILES-ELECTRONICS" },
	{ 0x0050C2, 0xBF6000, 0xBF6FFF, "NOLAM EMBEDDED SYSTEMS", "NOLAM-EMBEDDED" },
	{ 0x0050C2, 0xBF7000, 0xBF7FFF, "Phytec Messtechnik GmbH", "PHYTEC-MESSTECHNIK" },
	{ 0x0050C2, 0xBF8000, 0xBF8FFF, "Crtiical Link", "CRTIICAL-LINK" },
	{ 0x0050C2, 0xBF9000, 0xBF9FFF, "Vitel Net", "VITEL-NET" },
	{ 0x0050C2, 0xBFA000, 0xBFAFFF, "Rohde & Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xBFB000, 0xBFBFFF, "ECS Srl", "ECS-SRL" },
	{ 0x0050C2, 0xBFC000, 0xBFCFFF, "Altronix Corporation", "ALTRONIX" },
	{ 0x0050C2, 0xBFD000, 0xBFDFFF, "Ernemann Cine Tec GmbH", "ERNEMANN-CINE-TEC" },
	{ 0x0050C2, 0xBFE000, 0xBFEFFF, "Ingeteam Paneles S.A.U.", "INGETEAM-PANELES-U" },
	{ 0x0050C2, 0xBFF000, 0xBFFFFF, "I.S.A. S.r.l.", "I-S-R-L" },
	{ 0x0050C2, 0xC00000, 0xC00FFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0xC01000, 0xC01FFF, "QUERCUS TECHNOLOGIES, S.L.", "QUERCUS-TECHNOLOGIES-S-L" },
	{ 0x0050C2, 0xC02000, 0xC02FFF, "Hanning Elektro-Werke GmbH & Co. KG", "HANNING-ELEKTRO-WERKE" },
	{ 0x0050C2, 0xC03000, 0xC03FFF, "Volumatic Limited.", "VOLUMATIC" },
	{ 0x0050C2, 0xC04000, 0xC04FFF, "SoGEME", "SOGEME" },
	{ 0x0050C2, 0xC05000, 0xC05FFF, "Doppler Systems LLC", "DOPPLER-LLC" },
	{ 0x0050C2, 0xC06000, 0xC06FFF, "ANALOG WAY", "ANALOG-WAY" },
	{ 0x0050C2, 0xC07000, 0xC07FFF, "CIO Informatique Industrielle", "CIO-INFORMATIQUE-INDUSTRIELLE" },
	{ 0x0050C2, 0xC08000, 0xC08FFF, "juiceboss", "JUICEBOSS" },
	{ 0x0050C2, 0xC09000, 0xC09FFF, "Globe Wireless", "GLOBE-WIRELESS" },
	{ 0x0050C2, 0xC0A000, 0xC0AFFF, "ABB Transmission and Distribution Auto Eqip(Xiamen.China)", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTO-EQIP-XIAMEN-CHINA" },
	{ 0x0050C2, 0xC0B000, 0xC0BFFF, "ProSourcing GmbH", "PROSOURCING" },
	{ 0x0050C2, 0xC0C000, 0xC0CFFF, "Altierre", "ALTIERRE" },
	{ 0x0050C2, 0xC0D000, 0xC0DFFF, "Fr. SauterAG", "FR-SAUTERAG" },
	{ 0x0050C2, 0xC0E000, 0xC0EFFF, "AVItronic GmbH", "AVITRONIC" },
	{ 0x0050C2, 0xC0F000, 0xC0FFFF, "DYCEC, S.A.", "DYCEC" },
	{ 0x0050C2, 0xC10000, 0xC10FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xC11000, 0xC11FFF, "ART Antriebs- und Regeltechnik GmbH", "ART-ANTRIEBS--UND-REGELTECHNIK" },
	{ 0x0050C2, 0xC12000, 0xC12FFF, "OKI DENKI BOHSAI CO.,LTD.", "OKI-DENKI-BOHSAI" },
	{ 0x0050C2, 0xC13000, 0xC13FFF, "Dantec Dynamics A/S", "DANTEC-DYNAMICS-A/S" },
	{ 0x0050C2, 0xC14000, 0xC14FFF, "Spectronix Corporation", "SPECTRONIX" },
	{ 0x0050C2, 0xC15000, 0xC15FFF, "INO - Institut National d'Optique", "INO---INSTITUT-NATIONAL-D-OPTIQUE" },
	{ 0x0050C2, 0xC16000, 0xC16FFF, "OMICRON electronics GmbH", "OMICRON-ELECTRONICS" },
	{ 0x0050C2, 0xC17000, 0xC17FFF, "Axis-Shield PoC AS", "AXIS-SHIELD-POC-AS" },
	{ 0x0050C2, 0xC18000, 0xC18FFF, "Linuxstamp Designs, LLC", "LINUXSTAMP-DESIGNS-LLC" },
	{ 0x0050C2, 0xC19000, 0xC19FFF, "Ibercomp SA", "IBERCOMP" },
	{ 0x0050C2, 0xC1A000, 0xC1AFFF, "SAM Co., Ltd.", "SAM" },
	{ 0x0050C2, 0xC1B000, 0xC1BFFF, "Graesslin GmbH", "GRAESSLIN" },
	{ 0x0050C2, 0xC1C000, 0xC1CFFF, "Becton Dickinson", "BECTON-DICKINSON" },
	{ 0x0050C2, 0xC1D000, 0xC1DFFF, "Powerbase Energy Systems Inc.", "POWERBASE-ENERGY" },
	{ 0x0050C2, 0xC1E000, 0xC1EFFF, "Peperoni-Light", "PEPERONI-LIGHT" },
	{ 0x0050C2, 0xC1F000, 0xC1FFFF, "Specialist Electronics Services Ltd", "SPECIALIST-ELECTRONICS-SERVICES" },
	{ 0x0050C2, 0xC20000, 0xC20FFF, "SRC Computers, LLC", "SRC-COMPUTERS-LLC" },
	{ 0x0050C2, 0xC21000, 0xC21FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0xC22000, 0xC22FFF, "Audient Ltd", "AUDIENT" },
	{ 0x0050C2, 0xC23000, 0xC23FFF, "Vidicon LLC", "VIDICON-LLC" },
	{ 0x0050C2, 0xC24000, 0xC24FFF, "Qualnetics Corporation", "QUALNETICS" },
	{ 0x0050C2, 0xC25000, 0xC25FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0xC26000, 0xC26FFF, "Austco Communication Systems Pty Ltd", "AUSTCO-COMMUNICATION-PTY" },
	{ 0x0050C2, 0xC27000, 0xC27FFF, "Qtechnology A/S", "QTECHNOLOGY-A/S" },
	{ 0x0050C2, 0xC28000, 0xC28FFF, "ELREHA GmbH", "ELREHA" },
	{ 0x0050C2, 0xC29000, 0xC29FFF, "Newtel Engineering S.r.l.", "NEWTEL-ENGINEERING-S-R-L" },
	{ 0x0050C2, 0xC2A000, 0xC2AFFF, "RealTime Systems Ltd", "REALTIME" },
	{ 0x0050C2, 0xC2B000, 0xC2BFFF, "Z-App Systems, Inc.", "Z-APP" },
	{ 0x0050C2, 0xC2C000, 0xC2CFFF, "bach-messtechnik gmbh", "BACH-MESSTECHNIK" },
	{ 0x0050C2, 0xC2D000, 0xC2DFFF, "Digitale Analoge COMponenten West Electronic Vertriebs GmbH", "DIGITALE-ANALOGE-COMPONENTEN-WEST-VERTRIEBS" },
	{ 0x0050C2, 0xC2E000, 0xC2EFFF, "DISMUNTEL SAL", "DISMUNTEL-SAL" },
	{ 0x0050C2, 0xC2F000, 0xC2FFFF, "REFLEX CES", "REFLEX-CES" },
	{ 0x0050C2, 0xC30000, 0xC30FFF, "Wagner Group GmbH", "WAGNER-GROUP" },
	{ 0x0050C2, 0xC31000, 0xC31FFF, "4D Technology Corporation", "4D" },
	{ 0x0050C2, 0xC32000, 0xC32FFF, "Procon Electronics", "PROCON-ELECTRONICS" },
	{ 0x0050C2, 0xC34000, 0xC34FFF, "Kyuhen", "KYUHEN" },
	{ 0x0050C2, 0xC35000, 0xC35FFF, "Insitu, Inc.", "INSITU" },
	{ 0x0050C2, 0xC36000, 0xC36FFF, "SET GmbH", "SET" },
	{ 0x0050C2, 0xC37000, 0xC37FFF, "B.E.A.R. Solutions (Australasia) Pty, Ltd", "B-E-A-R-SOLUTIONS-AUSTRALASIA-PTY" },
	{ 0x0050C2, 0xC38000, 0xC38FFF, "Computer Automation Technology Inc", "AUTOMATION" },
	{ 0x0050C2, 0xC39000, 0xC39FFF, "SECAD SA", "SECAD" },
	{ 0x0050C2, 0xC3A000, 0xC3AFFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0xC3B000, 0xC3BFFF, "ELEKTRO-AUTOMATIK GmbH & Co. KG", "ELEKTRO-AUTOMATIK" },
	{ 0x0050C2, 0xC3C000, 0xC3CFFF, "ELSIST S.r.l.", "ELSIST-S-R-L" },
	{ 0x0050C2, 0xC3D000, 0xC3DFFF, "PLA ELECTRO APPLIANCES PVT. LTD.", "PLA-ELECTRO-APPLIANCES-PVT" },
	{ 0x0050C2, 0xC3E000, 0xC3EFFF, "Sysacom", "SYSACOM" },
	{ 0x0050C2, 0xC3F000, 0xC3FFFF, "ANXeBusiness Corporation", "ANXEBUSINESS" },
	{ 0x0050C2, 0xC40000, 0xC40FFF, "BAE Systems Bofors AB", "BAE-BOFORS-AB" },
	{ 0x0050C2, 0xC41000, 0xC41FFF, "COMPRION GmbH", "COMPRION" },
	{ 0x0050C2, 0xC42000, 0xC42FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xC43000, 0xC43FFF, "Cammegh Limited", "CAMMEGH" },
	{ 0x0050C2, 0xC44000, 0xC44FFF, "Beijing Zhongherongzhi Elec.&Tech.Co.,Ltd.", "BEIJING-ZHONGHERONGZHI-ELEC-TECH" },
	{ 0x0050C2, 0xC45000, 0xC45FFF, "Galvamat & Unican Technologies SA", "GALVAMAT-UNICAN-TECHNOLOGIES" },
	{ 0x0050C2, 0xC46000, 0xC46FFF, "QNE GmbH & Co. KG", "QNE" },
	{ 0x0050C2, 0xC47000, 0xC47FFF, "Weltek Technologies Co. Ltd.", "WELTEK-TECHNOLOGIES" },
	{ 0x0050C2, 0xC48000, 0xC48FFF, "Cytek Media Systems, INC.", "CYTEK-MEDIA" },
	{ 0x0050C2, 0xC49000, 0xC49FFF, "Elektronic Thoma GmbH", "ELEKTRONIC-THOMA" },
	{ 0x0050C2, 0xC4A000, 0xC4AFFF, "Herrick Technology Laboratories, Inc.", "HERRICK-LABORATORIES" },
	{ 0x0050C2, 0xC4B000, 0xC4BFFF, "R.V.R. elettronica s.p.a.", "R-V-R-ELETTRONICA-S-P-A" },
	{ 0x0050C2, 0xC4C000, 0xC4CFFF, "Lancier Monitoring GmbH", "LANCIER-MONITORING" },
	{ 0x0050C2, 0xC4D000, 0xC4DFFF, "Industrial Automation Systems", "INDUSTRIAL-AUTOMATION" },
	{ 0x0050C2, 0xC4E000, 0xC4EFFF, "Elaso AG", "ELASO" },
	{ 0x0050C2, 0xC4F000, 0xC4FFFF, "Powersense A/S", "POWERSENSE-A/S" },
	{ 0x0050C2, 0xC50000, 0xC50FFF, "Beceem Communications, Inc.", "BECEEM-COMMUNICATION" },
	{ 0x0050C2, 0xC51000, 0xC51FFF, "InForce Computing, Inc.", "INFORCE-COMPUTING" },
	{ 0x0050C2, 0xC52000, 0xC52FFF, "Smartfield, Inc.", "SMARTFIELD" },
	{ 0x0050C2, 0xC53000, 0xC53FFF, "Eilersen Electric A/S", "EILERSEN-ELECTRIC-A/S" },
	{ 0x0050C2, 0xC54000, 0xC54FFF, "HPC Platform", "HPC-PLATFORM" },
	{ 0x0050C2, 0xC55000, 0xC55FFF, "Watterott electronic", "WATTEROTT" },
	{ 0x0050C2, 0xC56000, 0xC56FFF, "Spirent Communications", "SPIRENT-COMMUNICATION" },
	{ 0x0050C2, 0xC57000, 0xC57FFF, "High Speed Design, Inc.", "HIGH-SPEED-DESIGN" },
	{ 0x0050C2, 0xC58000, 0xC58FFF, "Foerster-Technik GmbH", "FOERSTER-TECHNIK" },
	{ 0x0050C2, 0xC59000, 0xC59FFF, "SKD System AB", "SKD-SYSTEM-AB" },
	{ 0x0050C2, 0xC5A000, 0xC5AFFF, "Commotive A/S", "COMMOTIVE-A/S" },
	{ 0x0050C2, 0xC5B000, 0xC5BFFF, "MICRO TECHNICA", "MICRO-TECHNICA" },
	{ 0x0050C2, 0xC5C000, 0xC5CFFF, "WAVECOM ELEKTRONIK AG", "WAVECOM-ELEKTRONIK" },
	{ 0x0050C2, 0xC5D000, 0xC5DFFF, "SweMet AB", "SWEMET-AB" },
	{ 0x0050C2, 0xC5E000, 0xC5EFFF, "CellPlus technologies, Inc.", "CELLPLUS-TECHNOLOGIES" },
	{ 0x0050C2, 0xC5F000, 0xC5FFFF, "Icon Time Systems", "ICON-TIME" },
	{ 0x0050C2, 0xC60000, 0xC60FFF, "Integration Technologies Limited", "INTEGRATION-TECHNOLOGIES" },
	{ 0x0050C2, 0xC61000, 0xC61FFF, "HaiVision Systems Incorporated", "HAIVISION-INCORPORATED" },
	{ 0x0050C2, 0xC62000, 0xC62FFF, "Zeus Systems Private Limited", "ZEUS-PRIVATE" },
	{ 0x0050C2, 0xC63000, 0xC63FFF, "Potter Electric Signal Company", "POTTER-ELECTRIC-SIGNAL-COMPANY" },
	{ 0x0050C2, 0xC64000, 0xC64FFF, "Pal Software Service Co.,Ltd.", "PAL-SOFTWARE-SERVICE" },
	{ 0x0050C2, 0xC65000, 0xC65FFF, "Micro I/O Servicos de Electronica, Lda", "MICRO-I/O-SERVICOS-DE-ELECTRONICA-LDA" },
	{ 0x0050C2, 0xC66000, 0xC66FFF, "KS Beschallungstechnik GmbH", "KS-BESCHALLUNGSTECHNIK" },
	{ 0x0050C2, 0xC67000, 0xC67FFF, "Practical Control Ltd", "PRACTICAL-CONTROL" },
	{ 0x0050C2, 0xC68000, 0xC68FFF, "Broadsoft PacketSmart, Inc.", "BROADSOFT-PACKETSMART" },
	{ 0x0050C2, 0xC69000, 0xC69FFF, "REBO CO.,LTD.", "REBO" },
	{ 0x0050C2, 0xC6A000, 0xC6AFFF, "ELECTRONICA KELD", "ELECTRONICA-KELD" },
	{ 0x0050C2, 0xC6B000, 0xC6BFFF, "SiGarden Sp z o.o.", "SIGARDEN-SP-Z-O-O" },
	{ 0x0050C2, 0xC6C000, 0xC6CFFF, "DORLET S.A.", "DORLET" },
	{ 0x0050C2, 0xC6D000, 0xC6DFFF, "Deansoft CO., Ltd.", "DEANSOFT" },
	{ 0x0050C2, 0xC6E000, 0xC6EFFF, "TBS Holding AG", "TBS-HOLDING" },
	{ 0x0050C2, 0xC6F000, 0xC6FFFF, "MSB Elektronik und Geraetebau GmbH", "MSB-ELEKTRONIK-UND-GERAETEBAU" },
	{ 0x0050C2, 0xC70000, 0xC70FFF, "Wilke Technology GmbH", "WILKE" },
	{ 0x0050C2, 0xC71000, 0xC71FFF, "Sequoia Technology Group Ltd", "SEQUOIA-GROUP" },
	{ 0x0050C2, 0xC72000, 0xC72FFF, "Quail", "QUAIL" },
	{ 0x0050C2, 0xC73000, 0xC73FFF, "Industry Controls, Inc.", "INDUSTRY-CONTROLS" },
	{ 0x0050C2, 0xC74000, 0xC74FFF, "Wapice Ltd.", "WAPICE" },
	{ 0x0050C2, 0xC75000, 0xC75FFF, "Rovsing A/S", "ROVSING-A/S" },
	{ 0x0050C2, 0xC76000, 0xC76FFF, "GridManager A/S", "GRIDMANAGER-A/S" },
	{ 0x0050C2, 0xC77000, 0xC77FFF, "AIM Co.,Ltd", "AIM" },
	{ 0x0050C2, 0xC78000, 0xC78FFF, "9Solutions Oy", "9SOLUTIONS-OY" },
	{ 0x0050C2, 0xC79000, 0xC79FFF, "CODESYSTEM Co.,Ltd", "CODESYSTEM" },
	{ 0x0050C2, 0xC7A000, 0xC7AFFF, "Protonic Holland", "PROTONIC-HOLLAND" },
	{ 0x0050C2, 0xC7B000, 0xC7BFFF, "Honeywell", "HONEYWELL" },
	{ 0x0050C2, 0xC7C000, 0xC7CFFF, "Scienlab Electronic Systems GmbH", "SCIENLAB" },
	{ 0x0050C2, 0xC7D000, 0xC7DFFF, "TAE Antriebstechnik GmbH", "TAE-ANTRIEBSTECHNIK" },
	{ 0x0050C2, 0xC7E000, 0xC7EFFF, "Buerkert Werke GmbH", "BUERKERT-WERKE" },
	{ 0x0050C2, 0xC7F000, 0xC7FFFF, "Kinects Solutions Inc", "KINECTS-SOLUTIONS" },
	{ 0x0050C2, 0xC80000, 0xC80FFF, "Reko-vek", "REKO-VEK" },
	{ 0x0050C2, 0xC81000, 0xC81FFF, "Odyssee Systemes SAS", "ODYSSEE-SYSTEMES-SAS" },
	{ 0x0050C2, 0xC82000, 0xC82FFF, "Kyosha Industries", "KYOSHA-INDUSTRIES" },
	{ 0x0050C2, 0xC83000, 0xC83FFF, "Gronic Systems GmbH", "GRONIC" },
	{ 0x0050C2, 0xC84000, 0xC84FFF, "DOMIS", "DOMIS" },
	{ 0x0050C2, 0xC85000, 0xC85FFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0xC86000, 0xC86FFF, "Bruckner & Jarosch Ingenieurgesellschaft mbH", "BRUCKNER-JAROSCH-INGENIEURGESELLSCHAFT-MBH" },
	{ 0x0050C2, 0xC87000, 0xC87FFF, "LECO Corporation", "LECO" },
	{ 0x0050C2, 0xC88000, 0xC88FFF, "CSI Controles e Sistemas Industriais Ltda.", "CSI-CONTROLES-E-SISTEMAS-INDUSTRIAIS-LTDA" },
	{ 0x0050C2, 0xC89000, 0xC89FFF, "Creative Micro Design", "CREATIVE-MICRO-DESIGN" },
	{ 0x0050C2, 0xC8A000, 0xC8AFFF, "Automated Media Services, Inc.", "AUTOMATED-MEDIA-SERVICES" },
	{ 0x0050C2, 0xC8B000, 0xC8BFFF, "OCAS AS", "OCAS-AS" },
	{ 0x0050C2, 0xC8C000, 0xC8CFFF, "Lanmark Controls Inc.", "LANMARK-CONTROLS" },
	{ 0x0050C2, 0xC8D000, 0xC8DFFF, "Emergency Message Controls LLC", "EMERGENCY-MESSAGE-CONTROLS-LLC" },
	{ 0x0050C2, 0xC8E000, 0xC8EFFF, "SDD ITG", "SDD-ITG" },
	{ 0x0050C2, 0xC8F000, 0xC8FFFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xC90000, 0xC90FFF, "REALD", "REALD" },
	{ 0x0050C2, 0xC91000, 0xC91FFF, "Media Technologies Ltd.", "MEDIA-TECHNOLOGIES" },
	{ 0x0050C2, 0xC92000, 0xC92FFF, "EMAC, Inc.", "EMAC" },
	{ 0x0050C2, 0xC93000, 0xC93FFF, "SENSAIR Pty Ltd", "SENSAIR-PTY" },
	{ 0x0050C2, 0xC94000, 0xC94FFF, "ISIS ENGINEERING, S.A.", "ISIS-ENGINEERING" },
	{ 0x0050C2, 0xC95000, 0xC95FFF, "IPSES S.r.l.", "IPSES-S-R-L" },
	{ 0x0050C2, 0xC96000, 0xC96FFF, "CyberCraft", "CYBERCRAFT" },
	{ 0x0050C2, 0xC97000, 0xC97FFF, "MSTRONIC CO., LTD.", "MSTRONIC" },
	{ 0x0050C2, 0xC98000, 0xC98FFF, "Criticare Systems, Inc", "CRITICARE" },
	{ 0x0050C2, 0xC99000, 0xC99FFF, "HJPC Corporation dba Pactron", "HJPC-DBA-PACTRON" },
	{ 0x0050C2, 0xC9A000, 0xC9AFFF, "PACOMP Sp. z o.o.", "PACOMP-SP-Z-O-O" },
	{ 0x0050C2, 0xC9B000, 0xC9BFFF, "Sm electronic co.", "SM" },
	{ 0x0050C2, 0xC9C000, 0xC9CFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xC9D000, 0xC9DFFF, "Radius Sweden AB", "RADIUS-SWEDEN-AB" },
	{ 0x0050C2, 0xC9E000, 0xC9EFFF, "Rohde & Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xC9F000, 0xC9FFFF, "xxter b.v.", "XXTER" },
	{ 0x0050C2, 0xCA0000, 0xCA0FFF, "Kiefer technic GmbH", "KIEFER-TECHNIC" },
	{ 0x0050C2, 0xCA1000, 0xCA1FFF, "Wayne Kerr Electronics", "WAYNE-KERR-ELECTRONICS" },
	{ 0x0050C2, 0xCA2000, 0xCA2FFF, "The Logical Company", "THE-LOGICAL-COMPANY" },
	{ 0x0050C2, 0xCA3000, 0xCA3FFF, "CT Company", "CT-COMPANY" },
	{ 0x0050C2, 0xCA4000, 0xCA4FFF, "Vox Technologies", "VOX-TECHNOLOGIES" },
	{ 0x0050C2, 0xCA5000, 0xCA5FFF, "YOKOWO CO.,LTD", "YOKOWO" },
	{ 0x0050C2, 0xCA6000, 0xCA6FFF, "Vidisys GmbH", "VIDISYS" },
	{ 0x0050C2, 0xCA7000, 0xCA7FFF, "Thermo Fisher Scientific", "THERMO-FISHER-SCIENTIFIC" },
	{ 0x0050C2, 0xCA8000, 0xCA8FFF, "Systems With Intelligence Inc.", "WITH-INTELLIGENCE" },
	{ 0x0050C2, 0xCA9000, 0xCA9FFF, "Intelligent Devices", "INTELLIGENT-DEVICES" },
	{ 0x0050C2, 0xCAA000, 0xCAAFFF, "DSP DESIGN LTD", "DSP-DESIGN" },
	{ 0x0050C2, 0xCAB000, 0xCABFFF, "SAE IT-systems GmbH & Co. KG", "SAE-IT" },
	{ 0x0050C2, 0xCAC000, 0xCACFFF, "PURVIS Systems Incorporated", "PURVIS-INCORPORATED" },
	{ 0x0050C2, 0xCAD000, 0xCADFFF, "Pacific Coast Engineering", "PACIFIC-COAST-ENGINEERING" },
	{ 0x0050C2, 0xCAE000, 0xCAEFFF, "Campbell Scientific Canada Corp.", "CAMPBELL-SCIENTIFIC-CANADA" },
	{ 0x0050C2, 0xCAF000, 0xCAFFFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0xCB0000, 0xCB0FFF, "Konsmetal S.A.", "KONSMETAL" },
	{ 0x0050C2, 0xCB1000, 0xCB1FFF, "ZK Celltest Inc", "ZK-CELLTEST" },
	{ 0x0050C2, 0xCB2000, 0xCB2FFF, "Moravian Instruments", "MORAVIAN-INSTRUMENTS" },
	{ 0x0050C2, 0xCB3000, 0xCB3FFF, "Deuta-Werke GmbH", "DEUTA-WERKE" },
	{ 0x0050C2, 0xCB4000, 0xCB4FFF, "GEA Farm Technologies GmbH", "GEA-FARM-TECHNOLOGIES" },
	{ 0x0050C2, 0xCB5000, 0xCB5FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0xCB6000, 0xCB6FFF, "Krontek Pty Ltd", "KRONTEK-PTY" },
	{ 0x0050C2, 0xCB7000, 0xCB7FFF, "inotech GmbH", "INOTECH" },
	{ 0x0050C2, 0xCB8000, 0xCB8FFF, "Raith GmbH", "RAITH" },
	{ 0x0050C2, 0xCB9000, 0xCB9FFF, "Micro Technic A/S", "MICRO-TECHNIC-A/S" },
	{ 0x0050C2, 0xCBA000, 0xCBAFFF, "DELTA TAU DATA SYSTEMS", "DELTA-TAU-DATA" },
	{ 0x0050C2, 0xCBB000, 0xCBBFFF, "Coptonix GmbH", "COPTONIX" },
	{ 0x0050C2, 0xCBC000, 0xCBCFFF, "CP ELETRONICA SA", "CP-ELETRONICA" },
	{ 0x0050C2, 0xCBD000, 0xCBDFFF, "Hi Tech Electronics Ltd", "HI-TECH-ELECTRONICS" },
	{ 0x0050C2, 0xCBE000, 0xCBEFFF, "CODE BLUE CORPORATION", "CODE-BLUE" },
	{ 0x0050C2, 0xCBF000, 0xCBFFFF, "Megacon AB", "MEGACON-AB" },
	{ 0x0050C2, 0xCC0000, 0xCC0FFF, "World Time Solutions Limited", "WORLD-TIME-SOLUTIONS" },
	{ 0x0050C2, 0xCC1000, 0xCC1FFF, "Level 3 Communications", "LEVEL-3-COMMUNICATION" },
	{ 0x0050C2, 0xCC2000, 0xCC2FFF, "ConectaIP Tecnologia S.L.", "CONECTAIP-TECNOLOGIA-S-L" },
	{ 0x0050C2, 0xCC3000, 0xCC3FFF, "viscount systems inc.", "VISCOUNT" },
	{ 0x0050C2, 0xCC4000, 0xCC4FFF, "General Dynamics C4S", "GENERAL-DYNAMICS-C4S" },
	{ 0x0050C2, 0xCC5000, 0xCC5FFF, "Tecnovum AG", "TECNOVUM" },
	{ 0x0050C2, 0xCC6000, 0xCC6FFF, "KDT", "KDT" },
	{ 0x0050C2, 0xCC7000, 0xCC7FFF, "TOPROOT Technology Corp. Ltd.,", "TOPROOT" },
	{ 0x0050C2, 0xCC8000, 0xCC8FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0xCC9000, 0xCC9FFF, "Promess GmbH", "PROMESS" },
	{ 0x0050C2, 0xCCA000, 0xCCAFFF, "SANMINA SHENZHEN", "SANMINA-SHENZHEN" },
	{ 0x0050C2, 0xCCB000, 0xCCBFFF, "CaptiveAire Systems Inc.", "CAPTIVEAIRE" },
	{ 0x0050C2, 0xCCC000, 0xCCCFFF, "Smartech-technology", "SMARTECH" },
	{ 0x0050C2, 0xCCD000, 0xCCDFFF, "FUJI DATA SYSTEM Co.,Ltd.", "FUJI-DATA-SYSTEM" },
	{ 0x0050C2, 0xCCE000, 0xCCEFFF, "Mac-Gray Corporation", "MAC-GRAY" },
	{ 0x0050C2, 0xCCF000, 0xCCFFFF, "TASK SISTEMAS DE COMPUTACAO LTDA", "TASK-SISTEMAS-DE-COMPUTACAO-LTDA" },
	{ 0x0050C2, 0xCD0000, 0xCD0FFF, "MME Mueller Mikroelektronik", "MME-MUELLER-MIKROELEKTRONIK" },
	{ 0x0050C2, 0xCD1000, 0xCD1FFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0xCD2000, 0xCD2FFF, "SIM2 Multimedia S.p.A.", "SIM2-MULTIMEDIA-S-P-A" },
	{ 0x0050C2, 0xCD3000, 0xCD3FFF, "Covidence A/S", "COVIDENCE-A/S" },
	{ 0x0050C2, 0xCD4000, 0xCD4FFF, "SCHRAML GmbH", "SCHRAML" },
	{ 0x0050C2, 0xCD5000, 0xCD5FFF, "Arcos Technologies Ltd.", "ARCOS-TECHNOLOGIES" },
	{ 0x0050C2, 0xCD6000, 0xCD6FFF, "Arktan Systems", "ARKTAN" },
	{ 0x0050C2, 0xCD7000, 0xCD7FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xCD8000, 0xCD8FFF, "IT-IS International Ltd.", "IT-IS" },
	{ 0x0050C2, 0xCD9000, 0xCD9FFF, "NDC Infrared Engineering, Inc.", "NDC-INFRARED-ENGINEERING" },
	{ 0x0050C2, 0xCDA000, 0xCDAFFF, "taskit GmbH", "TASKIT" },
	{ 0x0050C2, 0xCDB000, 0xCDBFFF, "RUTTER INC", "RUTTER" },
	{ 0x0050C2, 0xCDC000, 0xCDCFFF, "ABB Transmission and Distribution Auto Eqip(Xiamen.China)", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTO-EQIP-XIAMEN-CHINA" },
	{ 0x0050C2, 0xCDD000, 0xCDDFFF, "K.C.C. SHOKAI LIMITED", "K-C-C-SHOKAI" },
	{ 0x0050C2, 0xCDE000, 0xCDEFFF, "Axotec Technologies GmbH", "AXOTEC-TECHNOLOGIES" },
	{ 0x0050C2, 0xCDF000, 0xCDFFFF, "CoreEL TEchnologies (I) Pvt Ltd", "COREEL-TECHNOLOGIES-I-PVT" },
	{ 0x0050C2, 0xCE0000, 0xCE0FFF, "Industrial Control Links, Inc.", "INDUSTRIAL-CONTROL-LINKS" },
	{ 0x0050C2, 0xCE1000, 0xCE1FFF, "Satellink Inc.", "SATELLINK" },
	{ 0x0050C2, 0xCE2000, 0xCE2FFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0xCE3000, 0xCE3FFF, "Industrial Automatics Design Bureau", "INDUSTRIAL-AUTOMATICS-DESIGN-BUREAU" },
	{ 0x0050C2, 0xCE4000, 0xCE4FFF, "TEKTRONIK", "TEKTRONIK" },
	{ 0x0050C2, 0xCE5000, 0xCE5FFF, "Maretron, LLP", "MARETRON-LLP" },
	{ 0x0050C2, 0xCE6000, 0xCE6FFF, "APLICA TECHNOLOGIES", "APLICA-TECHNOLOGIES" },
	{ 0x0050C2, 0xCE7000, 0xCE7FFF, "Echola Systems", "ECHOLA" },
	{ 0x0050C2, 0xCE8000, 0xCE8FFF, "Thomas & Betts", "THOMAS-BETTS" },
	{ 0x0050C2, 0xCE9000, 0xCE9FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0xCEA000, 0xCEAFFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xCEB000, 0xCEBFFF, "Toyon Research Corporation", "TOYON-RESEARCH" },
	{ 0x0050C2, 0xCEC000, 0xCECFFF, "Erhardt+Leimer GmbH", "ERHARDT+LEIMER" },
	{ 0x0050C2, 0xCED000, 0xCEDFFF, "AeroMechanical Services Ltd, FLYHT", "AEROMECHANICAL-SERVICES-FLYHT" },
	{ 0x0050C2, 0xCEE000, 0xCEEFFF, "EMBED-IT OG", "EMBED-IT-OG" },
	{ 0x0050C2, 0xCEF000, 0xCEFFFF, "Lupatecnologia e Sistemas Ltda", "LUPATECNOLOGIA-E-SISTEMAS-LTDA" },
	{ 0x0050C2, 0xCF0000, 0xCF0FFF, "Inviso B.V.", "INVISO" },
	{ 0x0050C2, 0xCF1000, 0xCF1FFF, "TelGaAs, Inc.", "TELGAAS" },
	{ 0x0050C2, 0xCF2000, 0xCF2FFF, "Weiss Robotics GmbH & Co. KG", "WEISS-ROBOTICS" },
	{ 0x0050C2, 0xCF3000, 0xCF3FFF, "Daiken Automacao Ltda", "DAIKEN-AUTOMACAO-LTDA" },
	{ 0x0050C2, 0xCF4000, 0xCF4FFF, "Baudisch Electronic GmbH", "BAUDISCH" },
	{ 0x0050C2, 0xCF5000, 0xCF5FFF, "AirCell Inc.", "AIRCELL" },
	{ 0x0050C2, 0xCF6000, 0xCF6FFF, "Epec Oy", "EPEC-OY" },
	{ 0x0050C2, 0xCF7000, 0xCF7FFF, "Armour Home Electronics LTD", "ARMOUR-HOME-ELECTRONICS" },
	{ 0x0050C2, 0xCF8000, 0xCF8FFF, "beks Kommunikacios Technika kft", "BEKS-KOMMUNIKACIOS-TECHNIKA-KFT" },
	{ 0x0050C2, 0xCF9000, 0xCF9FFF, "Elbit Systems of America", "ELBIT-OF-AMERICA" },
	{ 0x0050C2, 0xCFA000, 0xCFAFFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0xCFB000, 0xCFBFFF, "New Embedded Technology", "NEW-EMBEDDED" },
	{ 0x0050C2, 0xCFC000, 0xCFCFFF, "Tritium Pty Ltd", "TRITIUM-PTY" },
	{ 0x0050C2, 0xCFD000, 0xCFDFFF, "AIRFOLC,INC.", "AIRFOLC" },
	{ 0x0050C2, 0xCFE000, 0xCFEFFF, "Techleader", "TECHLEADER" },
	{ 0x0050C2, 0xCFF000, 0xCFFFFF, "Infrasafe, Inc.", "INFRASAFE" },
	{ 0x0050C2, 0xD00000, 0xD00FFF, "Bodensee Gravitymeter Geosystem GmbH", "BODENSEE-GRAVITYMETER-GEOSYSTEM" },
	{ 0x0050C2, 0xD01000, 0xD01FFF, "Aanderaa Data Instruments", "AANDERAA-DATA-INSTRUMENTS" },
	{ 0x0050C2, 0xD02000, 0xD02FFF, "SURVALENT TECHNOLOGY CORP", "SURVALENT" },
	{ 0x0050C2, 0xD03000, 0xD03FFF, "Peekel Instruments B.V.", "PEEKEL-INSTRUMENTS" },
	{ 0x0050C2, 0xD04000, 0xD04FFF, "Tehama Wireless", "TEHAMA-WIRELESS" },
	{ 0x0050C2, 0xD06000, 0xD06FFF, "nCk Research LLC", "NCK-RESEARCH-LLC" },
	{ 0x0050C2, 0xD07000, 0xD07FFF, "IAF GmbH", "IAF" },
	{ 0x0050C2, 0xD08000, 0xD08FFF, "Reimesch Kommunikationssysteme GmbH", "REIMESCH-KOMMUNIKATIONSSYSTEME" },
	{ 0x0050C2, 0xD09000, 0xD09FFF, "Guardtec, Inc.", "GUARDTEC" },
	{ 0x0050C2, 0xD0A000, 0xD0AFFF, "Airpoint Co., Ltd.", "AIRPOINT" },
	{ 0x0050C2, 0xD0B000, 0xD0BFFF, "CODACO ELECTRONIC s.r.o.", "CODACO-S-R-O" },
	{ 0x0050C2, 0xD0C000, 0xD0CFFF, "JVL Industri Elektronik", "JVL-INDUSTRI-ELEKTRONIK" },
	{ 0x0050C2, 0xD0D000, 0xD0DFFF, "DECA Card Engineering GmbH", "DECA-CARD-ENGINEERING" },
	{ 0x0050C2, 0xD0E000, 0xD0EFFF, "Weinert Engineering GmbH", "WEINERT-ENGINEERING" },
	{ 0x0050C2, 0xD0F000, 0xD0FFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xD10000, 0xD10FFF, "Rosslare Enterprises Ltd.", "ROSSLARE-ENTERPRISES" },
	{ 0x0050C2, 0xD11000, 0xD11FFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x0050C2, 0xD12000, 0xD12FFF, "Tokyo Weld Co.,Ltd.", "TOKYO-WELD" },
	{ 0x0050C2, 0xD13000, 0xD13FFF, "GUNMA ELECTRONICS CO LTD", "GUNMA-ELECTRONICS" },
	{ 0x0050C2, 0xD14000, 0xD14FFF, "SAET I.S.", "SAET-I-S" },
	{ 0x0050C2, 0xD15000, 0xD15FFF, "MSR-Office GmbH", "MSR-OFFICE" },
	{ 0x0050C2, 0xD16000, 0xD16FFF, "Imricor Medical Systems, Inc.", "IMRICOR-MEDICAL" },
	{ 0x0050C2, 0xD17000, 0xD17FFF, "CUE, a.s.", "CUE-A-S" },
	{ 0x0050C2, 0xD18000, 0xD18FFF, "Glyn GmbH & Co.KG", "GLYN" },
	{ 0x0050C2, 0xD19000, 0xD19FFF, "Applied Medical Technologies, Inc DBA AirClean Systems", "APPLIED-MEDICAL-TECHNOLOGIES-DBA-AIRCLEAN" },
	{ 0x0050C2, 0xD1A000, 0xD1AFFF, "GILLAM-FEI S.A.", "GILLAM-FEI" },
	{ 0x0050C2, 0xD1B000, 0xD1BFFF, "TECHKON GmbH", "TECHKON" },
	{ 0x0050C2, 0xD1C000, 0xD1CFFF, "Recon Dynamics, LLC", "RECON-DYNAMICS-LLC" },
	{ 0x0050C2, 0xD1D000, 0xD1DFFF, "Moco Media Pty Ltd", "MOCO-MEDIA-PTY" },
	{ 0x0050C2, 0xD1E000, 0xD1EFFF, "Tobila Systems, Inc.", "TOBILA" },
	{ 0x0050C2, 0xD1F000, 0xD1FFFF, "Olympus NDT Canada Inc.", "OLYMPUS-NDT-CANADA" },
	{ 0x0050C2, 0xD20000, 0xD20FFF, "7+ Kft", "7+-KFT" },
	{ 0x0050C2, 0xD21000, 0xD21FFF, "Innovative Circuit Technology", "INNOVATIVE-CIRCUIT" },
	{ 0x0050C2, 0xD22000, 0xD22FFF, "eMDee Technology, Inc.", "EMDEE" },
	{ 0x0050C2, 0xD23000, 0xD23FFF, "Bluestone Technology GmbH", "BLUESTONE" },
	{ 0x0050C2, 0xD24000, 0xD24FFF, "Expro North Sea", "EXPRO-NORTH-SEA" },
	{ 0x0050C2, 0xD25000, 0xD25FFF, "VAF Instruments BV", "VAF-INSTRUMENTS" },
	{ 0x0050C2, 0xD26000, 0xD26FFF, "RCH GROUP", "RCH-GROUP" },
	{ 0x0050C2, 0xD27000, 0xD27FFF, "Fr.Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0xD28000, 0xD28FFF, "Digitale Analoge COMponenten West Electronic Vertriebs GmbH", "DIGITALE-ANALOGE-COMPONENTEN-WEST-VERTRIEBS" },
	{ 0x0050C2, 0xD29000, 0xD29FFF, "Axible Technologies", "AXIBLE-TECHNOLOGIES" },
	{ 0x0050C2, 0xD2A000, 0xD2AFFF, "Millennium Electronics Pty.Ltd.", "MILLENNIUM-ELECTRONICS-PTY" },
	{ 0x0050C2, 0xD2B000, 0xD2BFFF, "Video Tech Laboratories, Inc.", "VIDEO-TECH-LABORATORIES" },
	{ 0x0050C2, 0xD2C000, 0xD2CFFF, "Schneider Electric Motion USA", "SCHNEIDER-ELECTRIC-MOTION-USA" },
	{ 0x0050C2, 0xD2D000, 0xD2DFFF, "CADI SCIENTIFIC PTE LTD", "CADI-SCIENTIFIC-PTE" },
	{ 0x0050C2, 0xD2E000, 0xD2EFFF, "RS Gesellschaft fur Informationstechnik mbH & Co KG", "RS-GESELLSCHAFT-FUR-INFORMATIONSTECHNIK-MBH" },
	{ 0x0050C2, 0xD2F000, 0xD2FFFF, "Key Systems, Inc.", "KEY" },
	{ 0x0050C2, 0xD30000, 0xD30FFF, "ACTIV Financial Systems, Inc.", "ACTIV-FINANCIAL" },
	{ 0x0050C2, 0xD31000, 0xD31FFF, "UNGAVA Technologies Inc.", "UNGAVA-TECHNOLOGIES" },
	{ 0x0050C2, 0xD32000, 0xD32FFF, "RealTime Systems Ltd", "REALTIME" },
	{ 0x0050C2, 0xD33000, 0xD33FFF, "Maddalena S.p.A", "MADDALENA-S-P-A" },
	{ 0x0050C2, 0xD34000, 0xD34FFF, "GAON TECH corp.", "GAON-TECH" },
	{ 0x0050C2, 0xD35000, 0xD35FFF, "UG Systems GmbH & Co. KG", "UG" },
	{ 0x0050C2, 0xD36000, 0xD36FFF, "Enatel Limited", "ENATEL" },
	{ 0x0050C2, 0xD37000, 0xD37FFF, "LJT & Associates, Inc.", "LJT-ASSOCIATES" },
	{ 0x0050C2, 0xD38000, 0xD38FFF, "Kyowa Electronics Co.,Ltd.", "KYOWA-ELECTRONICS" },
	{ 0x0050C2, 0xD39000, 0xD39FFF, "Apex NV", "APEX-NV" },
	{ 0x0050C2, 0xD3A000, 0xD3AFFF, "WellSense Technologies", "WELLSENSE-TECHNOLOGIES" },
	{ 0x0050C2, 0xD3B000, 0xD3BFFF, "Gitsn Inc.", "GITSN" },
	{ 0x0050C2, 0xD3C000, 0xD3CFFF, "ASSYSTEM France", "ASSYSTEM-FRANCE" },
	{ 0x0050C2, 0xD3D000, 0xD3DFFF, "Tellabs Operations Inc.", "TELLABS-OPERATIONS" },
	{ 0x0050C2, 0xD3E000, 0xD3EFFF, "Synatec Electronic GmbH", "SYNATEC" },
	{ 0x0050C2, 0xD3F000, 0xD3FFFF, "CSS, LLC", "CSS-LLC" },
	{ 0x0050C2, 0xD40000, 0xD40FFF, "demmel products", "DEMMEL-PRODUCTS" },
	{ 0x0050C2, 0xD41000, 0xD41FFF, "AREA ENERGY, INC.", "AREA-ENERGY" },
	{ 0x0050C2, 0xD42000, 0xD42FFF, "Hagenuk KMT GmbH", "HAGENUK-KMT" },
	{ 0x0050C2, 0xD43000, 0xD43FFF, "DSP4YOU Ltd", "DSP4YOU" },
	{ 0x0050C2, 0xD44000, 0xD44FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xD45000, 0xD45FFF, "Technagon GmbH", "TECHNAGON" },
	{ 0x0050C2, 0xD46000, 0xD46FFF, "Thales Nederland BV", "THALES-NEDERLAND" },
	{ 0x0050C2, 0xD47000, 0xD47FFF, "Rohde & Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xD48000, 0xD48FFF, "Watermark Estate Management Services, LLC", "WATERMARK-ESTATE-MANAGEMENT-SERVICES-LLC" },
	{ 0x0050C2, 0xD49000, 0xD49FFF, "Smith Meter, Inc", "SMITH-METER" },
	{ 0x0050C2, 0xD4A000, 0xD4AFFF, "ATH system", "ATH-SYSTEM" },
	{ 0x0050C2, 0xD4B000, 0xD4BFFF, "Indra Australia", "INDRA-AUSTRALIA" },
	{ 0x0050C2, 0xD4C000, 0xD4CFFF, "DALOG Diagnosesysteme GmbH", "DALOG-DIAGNOSESYSTEME" },
	{ 0x0050C2, 0xD4D000, 0xD4DFFF, "Yardney Technical Products Inc.", "YARDNEY-TECHNICAL-PRODUCTS" },
	{ 0x0050C2, 0xD4E000, 0xD4EFFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xD4F000, 0xD4FFFF, "SECOM GmbH", "SECOM" },
	{ 0x0050C2, 0xD50000, 0xD50FFF, "Solbrig Electronics, Inc.", "SOLBRIG-ELECTRONICS" },
	{ 0x0050C2, 0xD51000, 0xD51FFF, "BETTINI SRL", "BETTINI-SRL" },
	{ 0x0050C2, 0xD52000, 0xD52FFF, "F+D Feinwerk- und Drucktechnik GmbH", "F+D-FEINWERK--UND-DRUCKTECHNIK" },
	{ 0x0050C2, 0xD53000, 0xD53FFF, "Telemerkki Oy", "TELEMERKKI-OY" },
	{ 0x0050C2, 0xD54000, 0xD54FFF, "ABtrack s.r.l.", "ABTRACK-S-R-L" },
	{ 0x0050C2, 0xD55000, 0xD55FFF, "Sterna Security", "STERNA-SECURITY" },
	{ 0x0050C2, 0xD56000, 0xD56FFF, "SELEX Communications Limited", "SELEX-COMMUNICATION" },
	{ 0x0050C2, 0xD57000, 0xD57FFF, "Hijikata Denki Corp.", "HIJIKATA-DENKI" },
	{ 0x0050C2, 0xD58000, 0xD58FFF, "NIK-ELEKTRONIKA Ltd", "NIK-ELEKTRONIKA" },
	{ 0x0050C2, 0xD59000, 0xD59FFF, "BUANCO SYSTEM A/S", "BUANCO-SYSTEM-A/S" },
	{ 0x0050C2, 0xD5A000, 0xD5AFFF, "Embedded Monitoring Systems Ltd.", "EMBEDDED-MONITORING" },
	{ 0x0050C2, 0xD5B000, 0xD5BFFF, "Infinition Inc.", "INFINITION" },
	{ 0x0050C2, 0xD5C000, 0xD5CFFF, "Ibetor S.L.", "IBETOR-S-L" },
	{ 0x0050C2, 0xD5D000, 0xD5DFFF, "GLOBALCOM ENGINEERING SRL", "GLOBALCOM-ENGINEERING-SRL" },
	{ 0x0050C2, 0xD5E000, 0xD5EFFF, "infinitec co., ltd.", "INFINITEC" },
	{ 0x0050C2, 0xD5F000, 0xD5FFFF, "Embedded Solution Co., Ltd.", "EMBEDDED-SOLUTION" },
	{ 0x0050C2, 0xD60000, 0xD60FFF, "Nihon Kessho Koogaku Co., Ltd.", "NIHON-KESSHO-KOOGAKU" },
	{ 0x0050C2, 0xD61000, 0xD61FFF, "system2 GmbH", "SYSTEM2" },
	{ 0x0050C2, 0xD62000, 0xD62FFF, "EMAC, Inc.", "EMAC" },
	{ 0x0050C2, 0xD63000, 0xD63FFF, "DATAREGIS S.A.", "DATAREGIS" },
	{ 0x0050C2, 0xD64000, 0xD64FFF, "TV1 GmbH", "TV1" },
	{ 0x0050C2, 0xD65000, 0xD65FFF, "TX Technology Corp", "TX" },
	{ 0x0050C2, 0xD66000, 0xD66FFF, "Uvax Concepts", "UVAX-CONCEPTS" },
	{ 0x0050C2, 0xD67000, 0xD67FFF, "KLING & FREITAG GmbH", "KLING-FREITAG" },
	{ 0x0050C2, 0xD68000, 0xD68FFF, "HiSpeed Data, Inc.", "HISPEED-DATA" },
	{ 0x0050C2, 0xD69000, 0xD69FFF, "GHL Systems Bhd", "GHL-BHD" },
	{ 0x0050C2, 0xD6A000, 0xD6AFFF, "A&T Corporation, Electrics Group , LAS R&D Unit,", "A-T-ELECTRICS-GROUP-LAS-R-D-UNIT" },
	{ 0x0050C2, 0xD6B000, 0xD6BFFF, "Nemec Automation", "NEMEC-AUTOMATION" },
	{ 0x0050C2, 0xD6C000, 0xD6CFFF, "ALPHA Corporation", "ALPHA" },
	{ 0x0050C2, 0xD6D000, 0xD6DFFF, "Pro-Digital Industria Eletronica", "PRO-DIGITAL-INDUSTRIA-ELETRONICA" },
	{ 0x0050C2, 0xD6E000, 0xD6EFFF, "BC Illumination, Inc.", "BC-ILLUMINATION" },
	{ 0x0050C2, 0xD6F000, 0xD6FFFF, "Imtron Messtechnik GmbH", "IMTRON-MESSTECHNIK" },
	{ 0x0050C2, 0xD70000, 0xD70FFF, "C. Rob. Hammerstein GmbH & Co. KG", "C-ROB-HAMMERSTEIN" },
	{ 0x0050C2, 0xD71000, 0xD71FFF, "EMAC, Inc.", "EMAC" },
	{ 0x0050C2, 0xD72000, 0xD72FFF, "Scale-Tron, Inc.", "SCALE-TRON" },
	{ 0x0050C2, 0xD73000, 0xD73FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xD74000, 0xD74FFF, "Computech International", "COMPUTECH" },
	{ 0x0050C2, 0xD75000, 0xD75FFF, "Collectric AB", "COLLECTRIC-AB" },
	{ 0x0050C2, 0xD76000, 0xD76FFF, "Telvent", "TELVENT" },
	{ 0x0050C2, 0xD77000, 0xD77FFF, "Fr.SauterAG", "FR-SAUTERAG" },
	{ 0x0050C2, 0xD78000, 0xD78FFF, "P4Q Electronics", "P4Q-ELECTRONICS" },
	{ 0x0050C2, 0xD79000, 0xD79FFF, "DSI RF Systems, Inc.", "DSI-RF" },
	{ 0x0050C2, 0xD7A000, 0xD7AFFF, "Transbit Sp. z o.o.", "TRANSBIT-SP-Z-O-O" },
	{ 0x0050C2, 0xD7B000, 0xD7BFFF, "OWITA GmbH", "OWITA" },
	{ 0x0050C2, 0xD7C000, 0xD7CFFF, "Microcubs Systems Pvt Ltd", "MICROCUBS-PVT" },
	{ 0x0050C2, 0xD7D000, 0xD7DFFF, "Voltech Instruments", "VOLTECH-INSTRUMENTS" },
	{ 0x0050C2, 0xD7E000, 0xD7EFFF, "LYNX Technik AG", "LYNX-TECHNIK" },
	{ 0x0050C2, 0xD7F000, 0xD7FFFF, "HMI Technologies", "HMI-TECHNOLOGIES" },
	{ 0x0050C2, 0xD80000, 0xD80FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xD81000, 0xD81FFF, "TATTILE SRL", "TATTILE-SRL" },
	{ 0x0050C2, 0xD82000, 0xD82FFF, "Audio Authority Corp", "AUDIO-AUTHORITY" },
	{ 0x0050C2, 0xD83000, 0xD83FFF, "Blankom", "BLANKOM" },
	{ 0x0050C2, 0xD84000, 0xD84FFF, "ABB Transmission and Distribution Auto Eqip(Xiamen.China)", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTO-EQIP-XIAMEN-CHINA" },
	{ 0x0050C2, 0xD85000, 0xD85FFF, "VITEC", "VITEC" },
	{ 0x0050C2, 0xD86000, 0xD86FFF, "ECOMM ERA", "ECOMM-ERA" },
	{ 0x0050C2, 0xD87000, 0xD87FFF, "Electrolight Shivuk (1994) Ltd.", "ELECTROLIGHT-SHIVUK-1994" },
	{ 0x0050C2, 0xD88000, 0xD88FFF, "T+A elektroakustik GmbH & Co KG", "T+A-ELEKTROAKUSTIK" },
	{ 0x0050C2, 0xD89000, 0xD89FFF, "Visual Telecommunication Network, Inc", "VISUAL-TELECOMMUNICATION-NETWORK" },
	{ 0x0050C2, 0xD8A000, 0xD8AFFF, "OptoLink Industria e Comercio Ltda", "OPTOLINK-INDUSTRIA-E-COMERCIO-LTDA" },
	{ 0x0050C2, 0xD8B000, 0xD8BFFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0xD8C000, 0xD8CFFF, "iRphotonics", "IRPHOTONICS" },
	{ 0x0050C2, 0xD8D000, 0xD8DFFF, "CS-Instruments", "CS-INSTRUMENTS" },
	{ 0x0050C2, 0xD8E000, 0xD8EFFF, "LSD Science&Technology Co.,Ltd.", "LSD-SCIENCE" },
	{ 0x0050C2, 0xD8F000, 0xD8FFFF, "Syes srl", "SYES-SRL" },
	{ 0x0050C2, 0xD90000, 0xD90FFF, "Dumps Electronic", "DUMPS" },
	{ 0x0050C2, 0xD91000, 0xD91FFF, "CHAUVIN ARNOUX", "CHAUVIN-ARNOUX" },
	{ 0x0050C2, 0xD92000, 0xD92FFF, "Manz", "MANZ" },
	{ 0x0050C2, 0xD93000, 0xD93FFF, "Axlon AB", "AXLON-AB" },
	{ 0x0050C2, 0xD94000, 0xD94FFF, "Software Effect Enterprises, Inc", "SOFTWARE-EFFECT-ENTERPRISES" },
	{ 0x0050C2, 0xD95000, 0xD95FFF, "Honeywell", "HONEYWELL" },
	{ 0x0050C2, 0xD96000, 0xD96FFF, "CONTEC GmbH", "CONTEC" },
	{ 0x0050C2, 0xD97000, 0xD97FFF, "ERS electronic GmbH", "ERS" },
	{ 0x0050C2, 0xD98000, 0xD98FFF, "Rong Shun Xuan Corp.", "RONG-SHUN-XUAN" },
	{ 0x0050C2, 0xD99000, 0xD99FFF, "T-Industry, s.r.o.", "T-INDUSTRY-S-R-O" },
	{ 0x0050C2, 0xD9A000, 0xD9AFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xD9B000, 0xD9BFFF, "Intuitive Surgical, Inc", "INTUITIVE-SURGICAL" },
	{ 0x0050C2, 0xD9C000, 0xD9CFFF, "Gamber Johnson LLC", "GAMBER-JOHNSON-LLC" },
	{ 0x0050C2, 0xD9D000, 0xD9DFFF, "Mistral Solutions Pvt. Ltd", "MISTRAL-SOLUTIONS-PVT" },
	{ 0x0050C2, 0xD9E000, 0xD9EFFF, "Vocality International", "VOCALITY" },
	{ 0x0050C2, 0xD9F000, 0xD9FFFF, "BitWise Controls", "BITWISE-CONTROLS" },
	{ 0x0050C2, 0xDA0000, 0xDA0FFF, "Precision Remotes", "PRECISION-REMOTES" },
	{ 0x0050C2, 0xDA1000, 0xDA1FFF, "MangoDSP", "MANGODSP" },
	{ 0x0050C2, 0xDA2000, 0xDA2FFF, "metraTec GmbH", "METRATEC" },
	{ 0x0050C2, 0xDA3000, 0xDA3FFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x0050C2, 0xDA4000, 0xDA4FFF, "Deuta-Werke GmbH", "DEUTA-WERKE" },
	{ 0x0050C2, 0xDA5000, 0xDA5FFF, "megatec electronic GmbH", "MEGATEC" },
	{ 0x0050C2, 0xDA6000, 0xDA6FFF, "Manitowoc Ice", "MANITOWOC-ICE" },
	{ 0x0050C2, 0xDA7000, 0xDA7FFF, "Capton", "CAPTON" },
	{ 0x0050C2, 0xDA8000, 0xDA8FFF, "Sine Systems, Inc.", "SINE" },
	{ 0x0050C2, 0xDA9000, 0xDA9FFF, "Tieline Research Pty Ltd", "TIELINE-RESEARCH-PTY" },
	{ 0x0050C2, 0xDAA000, 0xDAAFFF, "M & PAUL, INC", "M-PAUL" },
	{ 0x0050C2, 0xDAB000, 0xDABFFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x0050C2, 0xDAC000, 0xDACFFF, "RFL Electronics", "RFL-ELECTRONICS" },
	{ 0x0050C2, 0xDAD000, 0xDADFFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xDAE000, 0xDAEFFF, "Spang Power Electronics", "SPANG-POWER-ELECTRONICS" },
	{ 0x0050C2, 0xDAF000, 0xDAFFFF, "eumig industrie-tv GmbH", "EUMIG-INDUSTRIE-TV" },
	{ 0x0050C2, 0xDB0000, 0xDB0FFF, "IMAGO Technologies GmbH", "IMAGO-TECHNOLOGIES" },
	{ 0x0050C2, 0xDB1000, 0xDB1FFF, "RF Code, Inc", "RF-CODE" },
	{ 0x0050C2, 0xDB2000, 0xDB2FFF, "SoftwareCannery", "SOFTWARECANNERY" },
	{ 0x0050C2, 0xDB3000, 0xDB3FFF, "LAUDA DR. R. WOBSER GMBH & CO. KG", "LAUDA-DR-R-WOBSER" },
	{ 0x0050C2, 0xDB4000, 0xDB4FFF, "ZAO NPC Kompjuternie Technologii", "ZAO-NPC-KOMPJUTERNIE-TECHNOLOGII" },
	{ 0x0050C2, 0xDB5000, 0xDB5FFF, "DSP DESIGN LTD", "DSP-DESIGN" },
	{ 0x0050C2, 0xDB6000, 0xDB6FFF, "PROSOFT-SYSTEMS LTD", "PROSOFT" },
	{ 0x0050C2, 0xDB7000, 0xDB7FFF, "SOREL GmbH Mikroelektronik", "SOREL-MIKROELEKTRONIK" },
	{ 0x0050C2, 0xDB8000, 0xDB8FFF, "Comsat VertriebsgmbH", "COMSAT-VERTRIEBSGMBH" },
	{ 0x0050C2, 0xDB9000, 0xDB9FFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0xDBA000, 0xDBAFFF, "M.P. Electronics", "M-P-ELECTRONICS" },
	{ 0x0050C2, 0xDBB000, 0xDBBFFF, "Esensors, Inc.", "ESENSORS" },
	{ 0x0050C2, 0xDBC000, 0xDBCFFF, "Nantes Systems Private Limited", "NANTES-PRIVATE" },
	{ 0x0050C2, 0xDBD000, 0xDBDFFF, "Margento R&D", "MARGENTO-R-D" },
	{ 0x0050C2, 0xDBE000, 0xDBEFFF, "WITHSYSTEM Co.,Ltd", "WITHSYSTEM" },
	{ 0x0050C2, 0xDBF000, 0xDBFFFF, "One-Nemoto Engineering Corporation", "ONE-NEMOTO-ENGINEERING" },
	{ 0x0050C2, 0xDC0000, 0xDC0FFF, "Security Services Group (SSG)", "SECURITY-SERVICES-GROUP-SSG" },
	{ 0x0050C2, 0xDC1000, 0xDC1FFF, "Acrux Technology Limited", "ACRUX" },
	{ 0x0050C2, 0xDC2000, 0xDC2FFF, "TESSERA TECHNOLOGY INC.", "TESSERA" },
	{ 0x0050C2, 0xDC3000, 0xDC3FFF, "ZED Ziegler Electronic Devices GmbH", "ZED-ZIEGLER-DEVICES" },
	{ 0x0050C2, 0xDC4000, 0xDC4FFF, "Keith & Koep GmbH", "KEITH-KOEP" },
	{ 0x0050C2, 0xDC5000, 0xDC5FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xDC6000, 0xDC6FFF, "Fluid Components International", "FLUID-COMPONENTS" },
	{ 0x0050C2, 0xDC7000, 0xDC7FFF, "AGT Holdings Limited", "AGT-HOLDINGS" },
	{ 0x0050C2, 0xDC8000, 0xDC8FFF, "T2M2 GmbH", "T2M2" },
	{ 0x0050C2, 0xDC9000, 0xDC9FFF, "KinotonGmbH", "KINOTONGMBH" },
	{ 0x0050C2, 0xDCA000, 0xDCAFFF, "Tele Data Control", "TELE-DATA-CONTROL" },
	{ 0x0050C2, 0xDCB000, 0xDCBFFF, "CT Company", "CT-COMPANY" },
	{ 0x0050C2, 0xDCC000, 0xDCCFFF, "Instrumentel Limited", "INSTRUMENTEL" },
	{ 0x0050C2, 0xDCD000, 0xDCDFFF, "dilitronics GmbH", "DILITRONICS" },
	{ 0x0050C2, 0xDCE000, 0xDCEFFF, "Mecsel Oy", "MECSEL-OY" },
	{ 0x0050C2, 0xDCF000, 0xDCFFFF, "MCS Engenharia ltda", "MCS-ENGENHARIA-LTDA" },
	{ 0x0050C2, 0xDD0000, 0xDD0FFF, "IDC Solutions Pty Ltd", "IDC-SOLUTIONS-PTY" },
	{ 0x0050C2, 0xDD1000, 0xDD1FFF, "Brankamp GmbH", "BRANKAMP" },
	{ 0x0050C2, 0xDD2000, 0xDD2FFF, "Electronic Applications, Inc.", "APPLICATIONS" },
	{ 0x0050C2, 0xDD3000, 0xDD3FFF, "Rohde & Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xDD4000, 0xDD4FFF, "SYSTECH", "SYSTECH" },
	{ 0x0050C2, 0xDD5000, 0xDD5FFF, "Friend Spring Industrial Co., Ltd.", "FRIEND-SPRING-INDUSTRIAL" },
	{ 0x0050C2, 0xDD6000, 0xDD6FFF, "Transas Marine International AB", "TRANSAS-MARINE-AB" },
	{ 0x0050C2, 0xDD7000, 0xDD7FFF, "Tornado Modular Systems", "TORNADO-MODULAR" },
	{ 0x0050C2, 0xDD8000, 0xDD8FFF, "Selex Systems Integration Inc", "SELEX-INTEGRATION" },
	{ 0x0050C2, 0xDD9000, 0xDD9FFF, "Metraware", "METRAWARE" },
	{ 0x0050C2, 0xDDA000, 0xDDAFFF, "rbz robot design s.l.", "RBZ-ROBOT-DESIGN-S-L" },
	{ 0x0050C2, 0xDDB000, 0xDDBFFF, "LUCEO", "LUCEO" },
	{ 0x0050C2, 0xDDC000, 0xDDCFFF, "Vision & Control GmbH", "VISION-CONTROL" },
	{ 0x0050C2, 0xDDD000, 0xDDDFFF, "A&A GENERAL SRL", "A-A-GENERAL-SRL" },
	{ 0x0050C2, 0xDDE000, 0xDDEFFF, "DRS ITS", "DRS-ITS" },
	{ 0x0050C2, 0xDDF000, 0xDDFFFF, "Device GmbH", "DEVICE" },
	{ 0x0050C2, 0xDE0000, 0xDE0FFF, "INTERNET PROTOCOLO LOGICA SL", "INTERNET-PROTOCOLO-LOGICA-SL" },
	{ 0x0050C2, 0xDE1000, 0xDE1FFF, "ABB Transmission and Distribution Auto Eqip(Xiamen.China)", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTO-EQIP-XIAMEN-CHINA" },
	{ 0x0050C2, 0xDE2000, 0xDE2FFF, "SEQUTEC INC", "SEQUTEC" },
	{ 0x0050C2, 0xDE3000, 0xDE3FFF, "Breakaway Systems LLC", "BREAKAWAY-LLC" },
	{ 0x0050C2, 0xDE4000, 0xDE4FFF, "EGS Technologies Ltd", "EGS-TECHNOLOGIES" },
	{ 0x0050C2, 0xDE5000, 0xDE5FFF, "Neets", "NEETS" },
	{ 0x0050C2, 0xDE6000, 0xDE6FFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0xDE7000, 0xDE7FFF, "Elan Systems", "ELAN" },
	{ 0x0050C2, 0xDE8000, 0xDE8FFF, "Visual Productions", "VISUAL-PRODUCTIONS" },
	{ 0x0050C2, 0xDE9000, 0xDE9FFF, "Dacom West GmbH", "DACOM-WEST" },
	{ 0x0050C2, 0xDEA000, 0xDEAFFF, "Cerner Corporation", "CERNER" },
	{ 0x0050C2, 0xDEB000, 0xDEBFFF, "Ruwisch & Kollegen GmbH", "RUWISCH-KOLLEGEN" },
	{ 0x0050C2, 0xDEC000, 0xDECFFF, "VendNovation LLC", "VENDNOVATION-LLC" },
	{ 0x0050C2, 0xDED000, 0xDEDFFF, "Lee Laser", "LEE-LASER" },
	{ 0x0050C2, 0xDEF000, 0xDEFFFF, "Powersense A/S", "POWERSENSE-A/S" },
	{ 0x0050C2, 0xDF0000, 0xDF0FFF, "Koncar Electrical Engineering Institute", "KONCAR-ELECTRICAL-ENGINEERING-INSTITUTE" },
	{ 0x0050C2, 0xDF1000, 0xDF1FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xDF2000, 0xDF2FFF, "Ocean Sonics", "OCEAN-SONICS" },
	{ 0x0050C2, 0xDF3000, 0xDF3FFF, "INSEVIS GmbH", "INSEVIS" },
	{ 0x0050C2, 0xDF4000, 0xDF4FFF, "Potter Electric Signal", "POTTER-ELECTRIC-SIGNAL" },
	{ 0x0050C2, 0xDF5000, 0xDF5FFF, "EtherLight", "ETHERLIGHT" },
	{ 0x0050C2, 0xDF6000, 0xDF6FFF, "HINO ENGINEERING, INC", "HINO-ENGINEERING" },
	{ 0x0050C2, 0xDF7000, 0xDF7FFF, "Combilent", "COMBILENT" },
	{ 0x0050C2, 0xDF8000, 0xDF8FFF, "Tommotek (WA) Pty Ltd.", "TOMMOTEK-WA-PTY" },
	{ 0x0050C2, 0xDF9000, 0xDF9FFF, "Jenny Science AG", "JENNY-SCIENCE" },
	{ 0x0050C2, 0xDFA000, 0xDFAFFF, "MAC Valves, Inc.", "MAC-VALVES" },
	{ 0x0050C2, 0xDFB000, 0xDFBFFF, "BETTINI SRL", "BETTINI-SRL" },
	{ 0x0050C2, 0xDFC000, 0xDFCFFF, "I-Evo Ltd", "I-EVO" },
	{ 0x0050C2, 0xDFD000, 0xDFDFFF, "Wotbox ltd", "WOTBOX" },
	{ 0x0050C2, 0xDFE000, 0xDFEFFF, "Xitek Design Limited", "XITEK-DESIGN" },
	{ 0x0050C2, 0xDFF000, 0xDFFFFF, "TANTAL ELECTRONICA, SL", "TANTAL-ELECTRONICA-SL" },
	{ 0x0050C2, 0xE00000, 0xE00FFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x0050C2, 0xE01000, 0xE01FFF, "Tyco Traffic & Transportation", "TYCO-TRAFFIC-TRANSPORTATION" },
	{ 0x0050C2, 0xE02000, 0xE02FFF, "Cleverscope", "CLEVERSCOPE" },
	{ 0x0050C2, 0xE03000, 0xE03FFF, "ICU Scandinavia Schweiz GmbH", "ICU-SCANDINAVIA-SCHWEIZ" },
	{ 0x0050C2, 0xE04000, 0xE04FFF, "Sec.Eng Systems Pty Ltd", "SEC-ENG-PTY" },
	{ 0x0050C2, 0xE05000, 0xE05FFF, "NOCOSIUM", "NOCOSIUM" },
	{ 0x0050C2, 0xE06000, 0xE06FFF, "Ebner Electronic GmbH", "EBNER" },
	{ 0x0050C2, 0xE07000, 0xE07FFF, "Protagon Process Technologies GmbH", "PROTAGON-PROCESS-TECHNOLOGIES" },
	{ 0x0050C2, 0xE08000, 0xE08FFF, "KST Technology", "KST" },
	{ 0x0050C2, 0xE09000, 0xE09FFF, "ATEME", "ATEME" },
	{ 0x0050C2, 0xE0A000, 0xE0AFFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0xE0B000, 0xE0BFFF, "Seartech", "SEARTECH" },
	{ 0x0050C2, 0xE0C000, 0xE0CFFF, "YOUHO ELECTRIC IND.,LTD.", "YOUHO-ELECTRIC-IND" },
	{ 0x0050C2, 0xE0D000, 0xE0DFFF, "Unixmedia Srl", "UNIXMEDIA-SRL" },
	{ 0x0050C2, 0xE0E000, 0xE0EFFF, "PMAC JAPAN", "PMAC-JAPAN" },
	{ 0x0050C2, 0xE0F000, 0xE0FFFF, "Trentino Systems", "TRENTINO" },
	{ 0x0050C2, 0xE10000, 0xE10FFF, "Radinetworks Co., Ltd", "RADINETWORKS" },
	{ 0x0050C2, 0xE11000, 0xE11FFF, "RadioMobile Inc", "RADIOMOBILE" },
	{ 0x0050C2, 0xE12000, 0xE12FFF, "Kago Electronics BV", "KAGO-ELECTRONICS" },
	{ 0x0050C2, 0xE13000, 0xE13FFF, "Automation Assist Japan Company", "AUTOMATION-ASSIST-JAPAN-COMPANY" },
	{ 0x0050C2, 0xE14000, 0xE14FFF, "Calixto Systems Pvt Ltd", "CALIXTO-PVT" },
	{ 0x0050C2, 0xE15000, 0xE15FFF, "IHI Scube Co.,Ltd", "IHI-SCUBE" },
	{ 0x0050C2, 0xE16000, 0xE16FFF, "Jetstream Ltd.", "JETSTREAM" },
	{ 0x0050C2, 0xE17000, 0xE17FFF, "Gall Tankdatensysteme GmbH", "GALL-TANKDATENSYSTEME" },
	{ 0x0050C2, 0xE18000, 0xE18FFF, "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTOMATION-EQUIPMENT-XIAMEN" },
	{ 0x0050C2, 0xE19000, 0xE19FFF, "Zoe Medical", "ZOE-MEDICAL" },
	{ 0x0050C2, 0xE1A000, 0xE1AFFF, "Rosslare Enterprises Limited", "ROSSLARE-ENTERPRISES" },
	{ 0x0050C2, 0xE1B000, 0xE1BFFF, "Embedded Labs", "EMBEDDED-LABS" },
	{ 0x0050C2, 0xE1C000, 0xE1CFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xE1D000, 0xE1DFFF, "Holdline Tecnologia e Sistemas Ltda", "HOLDLINE-TECNOLOGIA-E-SISTEMAS-LTDA" },
	{ 0x0050C2, 0xE1E000, 0xE1EFFF, "Lo-Q plc", "LO-Q-PLC" },
	{ 0x0050C2, 0xE1F000, 0xE1FFFF, "ELVEES", "ELVEES" },
	{ 0x0050C2, 0xE20000, 0xE20FFF, "Divelbiss Corporation", "DIVELBISS" },
	{ 0x0050C2, 0xE21000, 0xE21FFF, "Norwia AS", "NORWIA-AS" },
	{ 0x0050C2, 0xE22000, 0xE22FFF, "Michael Riedel Transformatorenbau GmbH", "MICHAEL-RIEDEL-TRANSFORMATORENBAU" },
	{ 0x0050C2, 0xE23000, 0xE23FFF, "VITEC", "VITEC" },
	{ 0x0050C2, 0xE24000, 0xE24FFF, "DiTEST Fahrzeugdiagnose GmbH", "DITEST-FAHRZEUGDIAGNOSE" },
	{ 0x0050C2, 0xE25000, 0xE25FFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0xE26000, 0xE26FFF, "Cinetix s.r.l.", "CINETIX-S-R-L" },
	{ 0x0050C2, 0xE27000, 0xE27FFF, "CONTROL SYSTEMS Srl", "CONTROL-SRL" },
	{ 0x0050C2, 0xE28000, 0xE28FFF, "Teplovodokhran", "TEPLOVODOKHRAN" },
	{ 0x0050C2, 0xE29000, 0xE29FFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0xE2A000, 0xE2AFFF, "Rohde&Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xE2B000, 0xE2BFFF, "Plant Integrity Limited", "PLANT-INTEGRITY" },
	{ 0x0050C2, 0xE2C000, 0xE2CFFF, "EN ElectronicNetwork Hamburg GmbH", "EN-ELECTRONICNETWORK-HAMBURG" },
	{ 0x0050C2, 0xE2D000, 0xE2DFFF, "Funkwerk IT Karlsfeld GmbH", "FUNKWERK-IT-KARLSFELD" },
	{ 0x0050C2, 0xE2E000, 0xE2EFFF, "DS! Ingenieurbuero", "DS!-INGENIEURBUERO" },
	{ 0x0050C2, 0xE2F000, 0xE2FFFF, "Beam Ltd", "BEAM" },
	{ 0x0050C2, 0xE30000, 0xE30FFF, "Goennheimer Elektronic GmbH", "GOENNHEIMER-ELEKTRONIC" },
	{ 0x0050C2, 0xE31000, 0xE31FFF, "ENSIS Co., Ltd.", "ENSIS" },
	{ 0x0050C2, 0xE32000, 0xE32FFF, "Oshoksh Corporation", "OSHOKSH" },
	{ 0x0050C2, 0xE33000, 0xE33FFF, "Morita Technical Center Company", "MORITA-TECHNICAL-CENTER-COMPANY" },
	{ 0x0050C2, 0xE34000, 0xE34FFF, "HGL Dynamics", "HGL-DYNAMICS" },
	{ 0x0050C2, 0xE35000, 0xE35FFF, "Omnica Corporation", "OMNICA" },
	{ 0x0050C2, 0xE36000, 0xE36FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xE37000, 0xE37FFF, "FUJI DATA SYSTEM Co., Ltd", "FUJI-DATA-SYSTEM" },
	{ 0x0050C2, 0xE38000, 0xE38FFF, "Aesir Copenhagen", "AESIR-COPENHAGEN" },
	{ 0x0050C2, 0xE39000, 0xE39FFF, "Telemetrics Inc.", "TELEMETRICS" },
	{ 0x0050C2, 0xE3A000, 0xE3AFFF, "Vocality International", "VOCALITY" },
	{ 0x0050C2, 0xE3B000, 0xE3BFFF, "Nanosolution Inc.", "NANOSOLUTION" },
	{ 0x0050C2, 0xE3C000, 0xE3CFFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0xE3D000, 0xE3DFFF, "Baudisch Electronic GmbH", "BAUDISCH" },
	{ 0x0050C2, 0xE3E000, 0xE3EFFF, "Monnit Corp.", "MONNIT" },
	{ 0x0050C2, 0xE3F000, 0xE3FFFF, "VISITO S.R.L.", "VISITO-S-R-L" },
	{ 0x0050C2, 0xE40000, 0xE40FFF, "Ecrin Systems", "ECRIN" },
	{ 0x0050C2, 0xE41000, 0xE41FFF, "Higeco S.r.l.", "HIGECO-S-R-L" },
	{ 0x0050C2, 0xE42000, 0xE42FFF, "Wings for Media SL", "WINGS-FOR-MEDIA-SL" },
	{ 0x0050C2, 0xE43000, 0xE43FFF, "Technica Engineering GmbH", "TECHNICA-ENGINEERING" },
	{ 0x0050C2, 0xE44000, 0xE44FFF, "DEUTA-WERKE GmbH", "DEUTA-WERKE" },
	{ 0x0050C2, 0xE45000, 0xE45FFF, "Stichting Sunrise", "STICHTING-SUNRISE" },
	{ 0x0050C2, 0xE46000, 0xE46FFF, "Industrea Mining Technology", "INDUSTREA-MINING" },
	{ 0x0050C2, 0xE47000, 0xE47FFF, "ENIKA.CZ", "ENIKA-CZ" },
	{ 0x0050C2, 0xE48000, 0xE48FFF, "ITW Reyflex North America", "ITW-REYFLEX-NORTH-AMERICA" },
	{ 0x0050C2, 0xE49000, 0xE49FFF, "CTF TECHNOLOGIES DO BRASIL LTDA", "CTF-TECHNOLOGIES-DO-BRASIL-LTDA" },
	{ 0x0050C2, 0xE4A000, 0xE4AFFF, "GHL Systems Bhd", "GHL-BHD" },
	{ 0x0050C2, 0xE4B000, 0xE4BFFF, "Rohde&Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xE4C000, 0xE4CFFF, "Applied Micro Electronics AME BV", "APPLIED-MICRO-ELECTRONICS-AME" },
	{ 0x0050C2, 0xE4D000, 0xE4DFFF, "PCSC", "PCSC" },
	{ 0x0050C2, 0xE4E000, 0xE4EFFF, "Institute For Information Industry", "INSTITUTE-FOR-INFORMATION-INDUSTRY" },
	{ 0x0050C2, 0xE4F000, 0xE4FFFF, "Wine Technology Marlborough", "WINE-MARLBOROUGH" },
	{ 0x0050C2, 0xE50000, 0xE50FFF, "Tattile srl", "TATTILE-SRL" },
	{ 0x0050C2, 0xE51000, 0xE51FFF, "Motec Pty Ltd", "MOTEC-PTY" },
	{ 0x0050C2, 0xE52000, 0xE52FFF, "Famas System S.p.A.", "FAMAS-SYSTEM-S-P-A" },
	{ 0x0050C2, 0xE53000, 0xE53FFF, "NEXT video systems Hard- and Software Development GmbH", "NEXT-VIDEO-HARD--AND-SOFTWARE-DEVELOPMENT" },
	{ 0x0050C2, 0xE54000, 0xE54FFF, "Arcos Technologies LTD", "ARCOS-TECHNOLOGIES" },
	{ 0x0050C2, 0xE55000, 0xE55FFF, "TTi Ltd", "TTI" },
	{ 0x0050C2, 0xE56000, 0xE56FFF, "RFENGINE CO., LTD.", "RFENGINE" },
	{ 0x0050C2, 0xE57000, 0xE57FFF, "EOLANE MONTCEAU", "EOLANE-MONTCEAU" },
	{ 0x0050C2, 0xE58000, 0xE58FFF, "Agri-hitech LLC", "AGRI-HITECH-LLC" },
	{ 0x0050C2, 0xE59000, 0xE59FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xE5A000, 0xE5AFFF, "FUTEC INC.", "FUTEC" },
	{ 0x0050C2, 0xE5B000, 0xE5BFFF, "CAIPO Automazione Industriale s.r.l.", "CAIPO-AUTOMAZIONE-INDUSTRIALE-S-R-L" },
	{ 0x0050C2, 0xE5C000, 0xE5CFFF, "MCOPIA Co., Ltd", "MCOPIA" },
	{ 0x0050C2, 0xE5D000, 0xE5DFFF, "T8 Ltd", "T8" },
	{ 0x0050C2, 0xE5E000, 0xE5EFFF, "OREP", "OREP" },
	{ 0x0050C2, 0xE5F000, 0xE5FFFF, "Pantec Engineering AG", "PANTEC-ENGINEERING" },
	{ 0x0050C2, 0xE60000, 0xE60FFF, "TAIYO SEIKI CO.,LTD.", "TAIYO-SEIKI" },
	{ 0x0050C2, 0xE61000, 0xE61FFF, "Detech Electronics ApS", "DETECH-ELECTRONICS-APS" },
	{ 0x0050C2, 0xE62000, 0xE62FFF, "SAE IT-systems GmbH & Co. KG", "SAE-IT" },
	{ 0x0050C2, 0xE63000, 0xE63FFF, "Prima sistemi", "PRIMA-SISTEMI" },
	{ 0x0050C2, 0xE64000, 0xE64FFF, "Edgeware AB", "EDGEWARE-AB" },
	{ 0x0050C2, 0xE65000, 0xE65FFF, "IB Elektronik GmbH", "IB-ELEKTRONIK" },
	{ 0x0050C2, 0xE66000, 0xE66FFF, "EMAC, Inc.", "EMAC" },
	{ 0x0050C2, 0xE67000, 0xE67FFF, "Critical Link, LLC", "CRITICAL-LINK-LLC" },
	{ 0x0050C2, 0xE68000, 0xE68FFF, "Kyoritsu Electric Corporation", "KYORITSU-ELECTRIC" },
	{ 0x0050C2, 0xE69000, 0xE69FFF, "Netmaker", "NETMAKER" },
	{ 0x0050C2, 0xE6A000, 0xE6AFFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x0050C2, 0xE6B000, 0xE6BFFF, "Sika Technology AG", "SIKA" },
	{ 0x0050C2, 0xE6C000, 0xE6CFFF, "SAMSUNG Electronics Co.,Ltd.(LED Division)", "SAMSUNG-ELECTRONICS-LED-DIVISION" },
	{ 0x0050C2, 0xE6D000, 0xE6DFFF, "Allerta Inc", "ALLERTA" },
	{ 0x0050C2, 0xE6E000, 0xE6EFFF, "Power-One Italia S.p.A", "POWER-ONE-ITALIA-S-P-A" },
	{ 0x0050C2, 0xE6F000, 0xE6FFFF, "Leyden Engineering", "LEYDEN-ENGINEERING" },
	{ 0x0050C2, 0xE70000, 0xE70FFF, "DORLET S.A.", "DORLET" },
	{ 0x0050C2, 0xE71000, 0xE71FFF, "traffic network solutions s.l", "TRAFFIC-NETWORK-SOLUTIONS-S-L" },
	{ 0x0050C2, 0xE73000, 0xE73FFF, "ACS Motion Control Ltd.", "ACS-MOTION-CONTROL" },
	{ 0x0050C2, 0xE74000, 0xE74FFF, "Will corp.", "WILL" },
	{ 0x0050C2, 0xE75000, 0xE75FFF, "FSM AG", "FSM" },
	{ 0x0050C2, 0xE76000, 0xE76FFF, "Embedded Solution Bank Co., Ltd.", "EMBEDDED-SOLUTION-BANK" },
	{ 0x0050C2, 0xE77000, 0xE77FFF, "Fr. Sauter AG", "FR-SAUTER" },
	{ 0x0050C2, 0xE78000, 0xE78FFF, "TASK SISTEMAS DE COMPUTACAO LTDA", "TASK-SISTEMAS-DE-COMPUTACAO-LTDA" },
	{ 0x0050C2, 0xE79000, 0xE79FFF, "MCS MICRONIC Computer Systeme GmbH", "MCS-MICRONIC-SYSTEME" },
	{ 0x0050C2, 0xE7A000, 0xE7AFFF, "Lightel", "LIGHTEL" },
	{ 0x0050C2, 0xE7B000, 0xE7BFFF, "ATOM GIKEN Co.,Ltd.", "ATOM-GIKEN" },
	{ 0x0050C2, 0xE7C000, 0xE7CFFF, "sp controls, inc", "SP-CONTROLS" },
	{ 0x0050C2, 0xE7D000, 0xE7DFFF, "AEL Microsystems Limited", "AEL-MICROSYSTEMS" },
	{ 0x0050C2, 0xE7E000, 0xE7EFFF, "Swareflex GmbH", "SWAREFLEX" },
	{ 0x0050C2, 0xE7F000, 0xE7FFFF, "LS Control A/S", "LS-CONTROL-A/S" },
	{ 0x0050C2, 0xE80000, 0xE80FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xE81000, 0xE81FFF, "Adaptive Technologies, Inc.", "ADAPTIVE-TECHNOLOGIES" },
	{ 0x0050C2, 0xE82000, 0xE82FFF, "Xplore Technologies Corp", "XPLORE-TECHNOLOGIES" },
	{ 0x0050C2, 0xE83000, 0xE83FFF, "Witree Co.,Ltd", "WITREE" },
	{ 0x0050C2, 0xE84000, 0xE84FFF, "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTOMATION-EQUIPMENT-XIAMEN" },
	{ 0x0050C2, 0xE85000, 0xE85FFF, "Cosmo Life Co.,Ltd", "COSMO-LIFE" },
	{ 0x0050C2, 0xE86000, 0xE86FFF, "Multisuns Corporation", "MULTISUNS" },
	{ 0x0050C2, 0xE87000, 0xE87FFF, "Lamson Safes & Security", "LAMSON-SAFES-SECURITY" },
	{ 0x0050C2, 0xE88000, 0xE88FFF, "Pivitec, LLC", "PIVITEC-LLC" },
	{ 0x0050C2, 0xE89000, 0xE89FFF, "PROTEQSEN", "PROTEQSEN" },
	{ 0x0050C2, 0xE8A000, 0xE8AFFF, "Macronet s.r.l.", "MACRONET-S-R-L" },
	{ 0x0050C2, 0xE8B000, 0xE8BFFF, "RPA Electronic Solutions, Inc.", "RPA-SOLUTIONS" },
	{ 0x0050C2, 0xE8C000, 0xE8CFFF, "Epec Oy", "EPEC-OY" },
	{ 0x0050C2, 0xE8D000, 0xE8DFFF, "SystemAdvanced Co,Ltd", "SYSTEMADVANCED" },
	{ 0x0050C2, 0xE8E000, 0xE8EFFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x0050C2, 0xE8F000, 0xE8FFFF, "STT Condigi A/S", "STT-CONDIGI-A/S" },
	{ 0x0050C2, 0xE90000, 0xE90FFF, "GS Elektromedizinische Geraete G. Stemple GmbH", "GS-ELEKTROMEDIZINISCHE-GERAETE-G-STEMPLE" },
	{ 0x0050C2, 0xE91000, 0xE91FFF, "DSP DESIGN LTD", "DSP-DESIGN" },
	{ 0x0050C2, 0xE92000, 0xE92FFF, "CT Company", "CT-COMPANY" },
	{ 0x0050C2, 0xE93000, 0xE93FFF, "Perceptive Pixel Inc.", "PERCEPTIVE-PIXEL" },
	{ 0x0050C2, 0xE94000, 0xE94FFF, "ANA-U GmbH", "ANA-U" },
	{ 0x0050C2, 0xE95000, 0xE95FFF, "Dlite Comercio, Importadora e Serviços de Automação Ltda", "DLITE-COMERCIO-IMPORTADORA-E-SERVIçOS-DE-AUTOMAçãO-LTDA" },
	{ 0x0050C2, 0xE96000, 0xE96FFF, "PROYECSON S.A.", "PROYECSON" },
	{ 0x0050C2, 0xE97000, 0xE97FFF, "Arista Systems Corporation", "ARISTA" },
	{ 0x0050C2, 0xE98000, 0xE98FFF, "i3 International Inc.", "I3" },
	{ 0x0050C2, 0xE99000, 0xE99FFF, "UV Networks, Inc.", "UV-NETWORKS" },
	{ 0x0050C2, 0xE9A000, 0xE9AFFF, "Solace Systems", "SOLACE" },
	{ 0x0050C2, 0xE9B000, 0xE9BFFF, "Hentschel System GmbH", "HENTSCHEL-SYSTEM" },
	{ 0x0050C2, 0xE9C000, 0xE9CFFF, "SPARQ systems", "SPARQ" },
	{ 0x0050C2, 0xE9D000, 0xE9DFFF, "nicai-systems", "NICAI" },
	{ 0x0050C2, 0xE9E000, 0xE9EFFF, "American Microsystems, Ltd.", "AMERICAN-MICROSYSTEMS" },
	{ 0x0050C2, 0xE9F000, 0xE9FFFF, "DataSoft Corporation", "DATASOFT" },
	{ 0x0050C2, 0xEA0000, 0xEA0FFF, "Robert Bosch Healthcare, Inc.", "ROBERT-BOSCH-HEALTHCARE" },
	{ 0x0050C2, 0xEA1000, 0xEA1FFF, "TEX COMPUTER SRL", "TEX-SRL" },
	{ 0x0050C2, 0xEA2000, 0xEA2FFF, "ThinkRF Corp", "THINKRF" },
	{ 0x0050C2, 0xEA3000, 0xEA3FFF, "Subsea Systems, Inc.", "SUBSEA" },
	{ 0x0050C2, 0xEA4000, 0xEA4FFF, "head", "HEAD" },
	{ 0x0050C2, 0xEA5000, 0xEA5FFF, "Aerodata AG", "AERODATA" },
	{ 0x0050C2, 0xEA6000, 0xEA6FFF, "Powersense A/S", "POWERSENSE-A/S" },
	{ 0x0050C2, 0xEA7000, 0xEA7FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xEA8000, 0xEA8FFF, "MB Connect Line GmbH", "MB-CONNECT-LINE" },
	{ 0x0050C2, 0xEA9000, 0xEA9FFF, "METTLER-TOLEDO HI-SPEED", "METTLER-TOLEDO-HI-SPEED" },
	{ 0x0050C2, 0xEAA000, 0xEAAFFF, "BAE Systems", "BAE" },
	{ 0x0050C2, 0xEAB000, 0xEABFFF, "Warp9 Tech Design, Inc.", "WARP9-TECH-DESIGN" },
	{ 0x0050C2, 0xEAC000, 0xEACFFF, "Alias ip", "ALIAS-IP" },
	{ 0x0050C2, 0xEAD000, 0xEADFFF, "Rohde&Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xEAE000, 0xEAEFFF, "Alyrica Networks", "ALYRICA-NETWORKS" },
	{ 0x0050C2, 0xEAF000, 0xEAFFFF, "Aircell", "AIRCELL" },
	{ 0x0050C2, 0xEB0000, 0xEB0FFF, "Pulse Communication Systems Pvt. Ltd.", "PULSE-COMMUNICATION-PVT" },
	{ 0x0050C2, 0xEB1000, 0xEB1FFF, "PDU EXPERT UK LTD", "PDU-EXPERT-UK" },
	{ 0x0050C2, 0xEB2000, 0xEB2FFF, "Otaki Electric Corporation", "OTAKI-ELECTRIC" },
	{ 0x0050C2, 0xEB3000, 0xEB3FFF, "AR RF/Microwave Instrumentation", "AR-RF/MICROWAVE-INSTRUMENTATION" },
	{ 0x0050C2, 0xEB4000, 0xEB4FFF, "Wishtek Technology, Inc.", "WISHTEK" },
	{ 0x0050C2, 0xEB5000, 0xEB5FFF, "Covidence A/S", "COVIDENCE-A/S" },
	{ 0x0050C2, 0xEB6000, 0xEB6FFF, "Monsoon Solutions, Inc.", "MONSOON-SOLUTIONS" },
	{ 0x0050C2, 0xEB7000, 0xEB7FFF, "Saab AB", "SAAB-AB" },
	{ 0x0050C2, 0xEB8000, 0xEB8FFF, "dspnor", "DSPNOR" },
	{ 0x0050C2, 0xEB9000, 0xEB9FFF, "ALPHA-MOS", "ALPHA-MOS" },
	{ 0x0050C2, 0xEBA000, 0xEBAFFF, "West-Com Nurse Call Systems, Inc.", "WEST-COM-NURSE-CALL" },
	{ 0x0050C2, 0xEBB000, 0xEBBFFF, "TimeTerminal Adductor Group AB", "TIMETERMINAL-ADDUCTOR-GROUP-AB" },
	{ 0x0050C2, 0xEBC000, 0xEBCFFF, "Diehl AKO Stiftung & Co. KG", "DIEHL-AKO-STIFTUNG" },
	{ 0x0050C2, 0xEBD000, 0xEBDFFF, "Droplet Measurement Technologies", "DROPLET-MEASUREMENT-TECHNOLOGIES" },
	{ 0x0050C2, 0xEBE000, 0xEBEFFF, "Global Tecnologia LTDA.", "GLOBAL-TECNOLOGIA-LTDA" },
	{ 0x0050C2, 0xEBF000, 0xEBFFFF, "CIVOLUTION", "CIVOLUTION" },
	{ 0x0050C2, 0xEC0000, 0xEC0FFF, "UgMO Technologies", "UGMO-TECHNOLOGIES" },
	{ 0x0050C2, 0xEC1000, 0xEC1FFF, "ANT Group s.r.l", "ANT-GROUP-S-R-L" },
	{ 0x0050C2, 0xEC2000, 0xEC2FFF, "Ixonos Plc", "IXONOS-PLC" },
	{ 0x0050C2, 0xEC3000, 0xEC3FFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x0050C2, 0xEC4000, 0xEC4FFF, "Logical Electromechanical Sys Inc.", "LOGICAL-ELECTROMECHANICAL-SYS" },
	{ 0x0050C2, 0xEC5000, 0xEC5FFF, "RSUPPORT Co., Ltd.", "RSUPPORT" },
	{ 0x0050C2, 0xEC6000, 0xEC6FFF, "INFRONICS SYSTEMS LIMITED", "INFRONICS" },
	{ 0x0050C2, 0xEC7000, 0xEC7FFF, "LIQUID ROBOTICS, INC", "LIQUID-ROBOTICS" },
	{ 0x0050C2, 0xEC8000, 0xEC8FFF, "IBERNEX INGENIERIA, S.L.", "IBERNEX-INGENIERIA-S-L" },
	{ 0x0050C2, 0xEC9000, 0xEC9FFF, "Amsterdam Scientific Instruments BV", "AMSTERDAM-SCIENTIFIC-INSTRUMENTS" },
	{ 0x0050C2, 0xECA000, 0xECAFFF, "BitWise Controls", "BITWISE-CONTROLS" },
	{ 0x0050C2, 0xECB000, 0xECBFFF, "FAL Corp", "FAL" },
	{ 0x0050C2, 0xECC000, 0xECCFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xECD000, 0xECDFFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0xECE000, 0xECEFFF, "easii ic adiis", "EASII-IC-ADIIS" },
	{ 0x0050C2, 0xECF000, 0xECFFFF, "TAIWAN HIPLUS CORPORATION", "TAIWAN-HIPLUS" },
	{ 0x0050C2, 0xED0000, 0xED0FFF, "Nippon Systemware Co.,Ltd.", "NIPPON-SYSTEMWARE" },
	{ 0x0050C2, 0xED1000, 0xED1FFF, "Arcontia Technology AB", "ARCONTIA-AB" },
	{ 0x0050C2, 0xED2000, 0xED2FFF, "Klangspektrum GmbH", "KLANGSPEKTRUM" },
	{ 0x0050C2, 0xED3000, 0xED3FFF, "ECO MONITORING UTILITY SYSTEMS LTD", "ECO-MONITORING-UTILITY" },
	{ 0x0050C2, 0xED4000, 0xED4FFF, "TAMAGAWA ELECTRONICS CO.,LTD.", "TAMAGAWA-ELECTRONICS" },
	{ 0x0050C2, 0xED5000, 0xED5FFF, "RFL Electronics Inc.", "RFL-ELECTRONICS" },
	{ 0x0050C2, 0xED6000, 0xED6FFF, "Cat AB", "CAT-AB" },
	{ 0x0050C2, 0xED7000, 0xED7FFF, "FBT Elettronica spa", "FBT-ELETTRONICA-SPA" },
	{ 0x0050C2, 0xED8000, 0xED8FFF, "AVocation Systems, Inc.", "AVOCATION" },
	{ 0x0050C2, 0xED9000, 0xED9FFF, "Plasmatronics pty ltd", "PLASMATRONICS-PTY" },
	{ 0x0050C2, 0xEDA000, 0xEDAFFF, "Joint Stock Company Svyaz Inginiring M", "JOINT-STOCK-COMPANY-SVYAZ-INGINIRING-M" },
	{ 0x0050C2, 0xEDB000, 0xEDBFFF, "BELIK S.P.R.L.", "BELIK-S-P-R-L" },
	{ 0x0050C2, 0xEDC000, 0xEDCFFF, "Eyelock Corporation", "EYELOCK" },
	{ 0x0050C2, 0xEDD000, 0xEDDFFF, "EBNEURO SPA", "EBNEURO-SPA" },
	{ 0x0050C2, 0xEDE000, 0xEDEFFF, "Smart Grid Networks", "SMART-GRID-NETWORKS" },
	{ 0x0050C2, 0xEDF000, 0xEDFFFF, "Monitor Business Machines", "MONITOR-BUSINESS-MACHINES" },
	{ 0x0050C2, 0xEE0000, 0xEE0FFF, "osf Hansjuergen Meier GmbH & Co. KG", "OSF-HANSJUERGEN-MEIER" },
	{ 0x0050C2, 0xEE1000, 0xEE1FFF, "Procon Electronics", "PROCON-ELECTRONICS" },
	{ 0x0050C2, 0xEE2000, 0xEE2FFF, "System Industrie Electronic GmbH", "SYSTEM-INDUSTRIE" },
	{ 0x0050C2, 0xEE3000, 0xEE3FFF, "Tecnint HTE Srl", "TECNINT-HTE-SRL" },
	{ 0x0050C2, 0xEE4000, 0xEE4FFF, "Rohde&Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xEE5000, 0xEE5FFF, "Cytec Zylindertechnik GmbH", "CYTEC-ZYLINDERTECHNIK" },
	{ 0x0050C2, 0xEE6000, 0xEE6FFF, "B:TECH, a. s.", "B:TECH-A-S" },
	{ 0x0050C2, 0xEE7000, 0xEE7FFF, "Syes srl", "SYES-SRL" },
	{ 0x0050C2, 0xEE8000, 0xEE8FFF, "Kamacho Scale Co., Ltd.", "KAMACHO-SCALE" },
	{ 0x0050C2, 0xEE9000, 0xEE9FFF, "QUANTA S.r.l.", "QUANTA-S-R-L" },
	{ 0x0050C2, 0xEEA000, 0xEEAFFF, "Positioneering Limited", "POSITIONEERING" },
	{ 0x0050C2, 0xEEB000, 0xEEBFFF, "fibrisTerre GmbH", "FIBRISTERRE" },
	{ 0x0050C2, 0xEEC000, 0xEECFFF, "Yuyama Mfg. Co., Ltd.", "YUYAMA-MFG" },
	{ 0x0050C2, 0xEED000, 0xEEDFFF, "Future Design Controls, Inc", "FUTURE-DESIGN-CONTROLS" },
	{ 0x0050C2, 0xEEE000, 0xEEEFFF, "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTOMATION-EQUIPMENT-XIAMEN" },
	{ 0x0050C2, 0xEEF000, 0xEEFFFF, "IDTRONIC GmbH", "IDTRONIC" },
	{ 0x0050C2, 0xEF0000, 0xEF0FFF, "Homaetrix Ltd", "HOMAETRIX" },
	{ 0x0050C2, 0xEF1000, 0xEF1FFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xEF2000, 0xEF2FFF, "Specialty Microwave Corp", "SPECIALTY-MICROWAVE" },
	{ 0x0050C2, 0xEF3000, 0xEF3FFF, "Smart Power Electronics GmbH & Co. KG", "SMART-POWER-ELECTRONICS" },
	{ 0x0050C2, 0xEF4000, 0xEF4FFF, "RO.VE.R. Laboratories S.p.A", "RO-VE-R-LABORATORIES-S-P-A" },
	{ 0x0050C2, 0xEF5000, 0xEF5FFF, "Human Network Labs, Inc.", "HUMAN-NETWORK-LABS" },
	{ 0x0050C2, 0xEF6000, 0xEF6FFF, "Netline Communication Technologies", "NETLINE-COMMUNICATION-TECHNOLOGIES" },
	{ 0x0050C2, 0xEF7000, 0xEF7FFF, "Amstelland Electronic BV", "AMSTELLAND" },
	{ 0x0050C2, 0xEF8000, 0xEF8FFF, "HCL Technologies", "HCL-TECHNOLOGIES" },
	{ 0x0050C2, 0xEF9000, 0xEF9FFF, "HORIBA ABX", "HORIBA-ABX" },
	{ 0x0050C2, 0xEFA000, 0xEFAFFF, "Predictive Sensor Technology", "PREDICTIVE-SENSOR" },
	{ 0x0050C2, 0xEFB000, 0xEFBFFF, "Norbit ODM AS", "NORBIT-ODM-AS" },
	{ 0x0050C2, 0xEFC000, 0xEFCFFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0xEFD000, 0xEFDFFF, "Sanmina", "SANMINA" },
	{ 0x0050C2, 0xEFE000, 0xEFEFFF, "PLR Information Systems Ltd.", "PLR-INFORMATION" },
	{ 0x0050C2, 0xEFF000, 0xEFFFFF, "Zephyrus Electronics LTD.", "ZEPHYRUS-ELECTRONICS" },
	{ 0x0050C2, 0xF00000, 0xF00FFF, "Syscom Instruments", "SYSCOM-INSTRUMENTS" },
	{ 0x0050C2, 0xF01000, 0xF01FFF, "Mango DSP, Inc", "MANGO-DSP" },
	{ 0x0050C2, 0xF02000, 0xF02FFF, "BMR", "BMR" },
	{ 0x0050C2, 0xF03000, 0xF03FFF, "Wren Sound Systems", "WREN-SOUND" },
	{ 0x0050C2, 0xF04000, 0xF04FFF, "KINKI ROENTGEN INDUSTRIAL CO.,LTD", "KINKI-ROENTGEN-INDUSTRIAL" },
	{ 0x0050C2, 0xF05000, 0xF05FFF, "ESI Ventures", "ESI-VENTURES" },
	{ 0x0050C2, 0xF06000, 0xF06FFF, "Micro-Key BV", "MICRO-KEY" },
	{ 0x0050C2, 0xF07000, 0xF07FFF, "Icon Research Ltd", "ICON-RESEARCH" },
	{ 0x0050C2, 0xF08000, 0xF08FFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x0050C2, 0xF09000, 0xF09FFF, "Wheatstone Corporation", "WHEATSTONE" },
	{ 0x0050C2, 0xF0A000, 0xF0AFFF, "HASCOM International Pty Ltd", "HASCOM-PTY" },
	{ 0x0050C2, 0xF0B000, 0xF0BFFF, "Treehaven Technologies, Inc.", "TREEHAVEN-TECHNOLOGIES" },
	{ 0x0050C2, 0xF0C000, 0xF0CFFF, "SKYCHANNEL LTD", "SKYCHANNEL" },
	{ 0x0050C2, 0xF0D000, 0xF0DFFF, "Bluetest AB", "BLUETEST-AB" },
	{ 0x0050C2, 0xF0E000, 0xF0EFFF, "Micro Technic A/S", "MICRO-TECHNIC-A/S" },
	{ 0x0050C2, 0xF0F000, 0xF0FFFF, "AeroVision Avionics, Inc.", "AEROVISION-AVIONICS" },
	{ 0x0050C2, 0xF10000, 0xF10FFF, "Wincor Nixdorf Sp. z o.o.", "WINCOR-NIXDORF-SP-Z-O-O" },
	{ 0x0050C2, 0xF11000, 0xF11FFF, "Organis GmbH", "ORGANIS" },
	{ 0x0050C2, 0xF12000, 0xF12FFF, "General Industrial Controls Pvt Ltd", "GENERAL-INDUSTRIAL-CONTROLS-PVT" },
	{ 0x0050C2, 0xF13000, 0xF13FFF, "Packet Plus, Inc.", "PACKET-PLUS" },
	{ 0x0050C2, 0xF14000, 0xF14FFF, "VISION SYSTEMS AERONAUTIC", "VISION-AERONAUTIC" },
	{ 0x0050C2, 0xF15000, 0xF15FFF, "Sascal Displays Ltd", "SASCAL-DISPLAYS" },
	{ 0x0050C2, 0xF16000, 0xF16FFF, "Peter Huber Kältemaschinenbau GmbH", "PETER-HUBER-KAELTEMASCHINENBAU" },
	{ 0x0050C2, 0xF17000, 0xF17FFF, "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd", "ABB-TRANSMISSION-AND-DISTRIBUTION-AUTOMATION-EQUIPMENT-XIAMEN" },
	{ 0x0050C2, 0xF18000, 0xF18FFF, "Vitec Multimedia", "VITEC-MULTIMEDIA" },
	{ 0x0050C2, 0xF19000, 0xF19FFF, "Netlink Bilisim Sistemleri San. ve Tic. Ltd. Sti.", "NETLINK-BILISIM-SISTEMLERI-SAN-VE-TIC-STI" },
	{ 0x0050C2, 0xF1A000, 0xF1AFFF, "Aqua Management", "AQUA-MANAGEMENT" },
	{ 0x0050C2, 0xF1B000, 0xF1BFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xF1C000, 0xF1CFFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x0050C2, 0xF1D000, 0xF1DFFF, "Grossenbacher Systeme AG", "GROSSENBACHER-SYSTEME" },
	{ 0x0050C2, 0xF1E000, 0xF1EFFF, "Dell'Orto S.P.A.", "DELL-ORTO-S-P-A" },
	{ 0x0050C2, 0xF1F000, 0xF1FFFF, "Verified Energy, LLC.", "VERIFIED-ENERGY-LLC" },
	{ 0x0050C2, 0xF20000, 0xF20FFF, "Unfors Instruments AB", "UNFORS-INSTRUMENTS-AB" },
	{ 0x0050C2, 0xF21000, 0xF21FFF, "SEITEC Co. Ltd", "SEITEC" },
	{ 0x0050C2, 0xF22000, 0xF22FFF, "Harland Simon plc", "HARLAND-SIMON-PLC" },
	{ 0x0050C2, 0xF23000, 0xF23FFF, "Electro-Motive Diesel", "ELECTRO-MOTIVE-DIESEL" },
	{ 0x0050C2, 0xF24000, 0xF24FFF, "CT Company", "CT-COMPANY" },
	{ 0x0050C2, 0xF25000, 0xF25FFF, "Samway Electronic SRL", "SAMWAY-SRL" },
	{ 0x0050C2, 0xF26000, 0xF26FFF, "WaveIP", "WAVEIP" },
	{ 0x0050C2, 0xF27000, 0xF27FFF, "ELAN SYSTEMS", "ELAN" },
	{ 0x0050C2, 0xF28000, 0xF28FFF, "Vertex Antennentechnik GmbH", "VERTEX-ANTENNENTECHNIK" },
	{ 0x0050C2, 0xF29000, 0xF29FFF, "RADYNE CORPORATION", "RADYNE" },
	{ 0x0050C2, 0xF2A000, 0xF2AFFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0xF2B000, 0xF2BFFF, "Bio Guard component & technologies", "BIO-GUARD-COMPONENT-TECHNOLOGIES" },
	{ 0x0050C2, 0xF2C000, 0xF2CFFF, "Terratel Technology s.r.o.", "TERRATEL-S-R-O" },
	{ 0x0050C2, 0xF2D000, 0xF2DFFF, "Robert Bosch Healthcare Systems, Inc.", "ROBERT-BOSCH-HEALTHCARE" },
	{ 0x0050C2, 0xF2E000, 0xF2EFFF, "H&L Instruments, LLC", "H-L-INSTRUMENTS-LLC" },
	{ 0x0050C2, 0xF2F000, 0xF2FFFF, "Arcos Technologies LTD", "ARCOS-TECHNOLOGIES" },
	{ 0x0050C2, 0xF30000, 0xF30FFF, "Miris AB", "MIRIS-AB" },
	{ 0x0050C2, 0xF31000, 0xF31FFF, "Ruetz Technologies GmbH", "RUETZ-TECHNOLOGIES" },
	{ 0x0050C2, 0xF32000, 0xF32FFF, "Net4Things", "NET4THINGS" },
	{ 0x0050C2, 0xF33000, 0xF33FFF, "Applied Micro Electronics AME BV", "APPLIED-MICRO-ELECTRONICS-AME" },
	{ 0x0050C2, 0xF34000, 0xF34FFF, "Sequip S+E GmbH", "SEQUIP-S+E" },
	{ 0x0050C2, 0xF35000, 0xF35FFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x0050C2, 0xF36000, 0xF36FFF, "Visitech AS", "VISITECH-AS" },
	{ 0x0050C2, 0xF37000, 0xF37FFF, "Rosslare Enterprises Limited", "ROSSLARE-ENTERPRISES" },
	{ 0x0050C2, 0xF38000, 0xF38FFF, "AeroControl, Inc.", "AEROCONTROL" },
	{ 0x0050C2, 0xF39000, 0xF39FFF, "Inforce Computing, Inc.", "INFORCE-COMPUTING" },
	{ 0x0050C2, 0xF3A000, 0xF3AFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xF3B000, 0xF3BFFF, "TAMS firmware co.", "TAMS-FIRMWARE" },
	{ 0x0050C2, 0xF3C000, 0xF3CFFF, "Vemco Sp. z o. o.", "VEMCO-SP-Z-O-O" },
	{ 0x0050C2, 0xF3D000, 0xF3DFFF, "Project service S.a.s", "PROJECT-SERVICE-S" },
	{ 0x0050C2, 0xF3E000, 0xF3EFFF, "Vtron Pty Ltd", "VTRON-PTY" },
	{ 0x0050C2, 0xF3F000, 0xF3FFFF, "DENSEI COMMUNICATION Inc.", "DENSEI-COMMUNICATION" },
	{ 0x0050C2, 0xF40000, 0xF40FFF, "iBWorld co.,ltd.", "IBWORLD" },
	{ 0x0050C2, 0xF41000, 0xF41FFF, "FairyDevices Inc.", "FAIRYDEVICES" },
	{ 0x0050C2, 0xF42000, 0xF42FFF, "DSPCon", "DSPCON" },
	{ 0x0050C2, 0xF43000, 0xF43FFF, "Special Systems Engineering Center LLC", "SPECIAL-ENGINEERING-CENTER-LLC" },
	{ 0x0050C2, 0xF44000, 0xF44FFF, "Steinbichler Optotechnik GmbH", "STEINBICHLER-OPTOTECHNIK" },
	{ 0x0050C2, 0xF45000, 0xF45FFF, "HUSTY M.Styczen J.Hupert Sp.J.", "HUSTY-M-STYCZEN-J-HUPERT-SP-J" },
	{ 0x0050C2, 0xF46000, 0xF46FFF, "Reason Tecnologia S.A.", "REASON-TECNOLOGIA" },
	{ 0x0050C2, 0xF47000, 0xF47FFF, "cadac,inc.", "CADAC" },
	{ 0x0050C2, 0xF48000, 0xF48FFF, "Midas Technology DBA Phoenix Audio Technologies", "MIDAS-DBA-PHOENIX-AUDIO-TECHNOLOGIES" },
	{ 0x0050C2, 0xF49000, 0xF49FFF, "Green Instruments A/S", "GREEN-INSTRUMENTS-A/S" },
	{ 0x0050C2, 0xF4A000, 0xF4AFFF, "Z-App Systems, Inc.", "Z-APP" },
	{ 0x0050C2, 0xF4B000, 0xF4BFFF, "Supranet", "SUPRANET" },
	{ 0x0050C2, 0xF4C000, 0xF4CFFF, "Enistic Limited", "ENISTIC" },
	{ 0x0050C2, 0xF4D000, 0xF4DFFF, "KNOWHOW INFOCOM INC.", "KNOWHOW-INFOCOM" },
	{ 0x0050C2, 0xF4E000, 0xF4EFFF, "Heinzinger electronic GmbH", "HEINZINGER" },
	{ 0x0050C2, 0xF4F000, 0xF4FFFF, "BAP Precision Ltd.", "BAP-PRECISION" },
	{ 0x0050C2, 0xF50000, 0xF50FFF, "Moritex Corporation", "MORITEX" },
	{ 0x0050C2, 0xF51000, 0xF51FFF, "NDC Infrared Engineering, Inc.", "NDC-INFRARED-ENGINEERING" },
	{ 0x0050C2, 0xF52000, 0xF52FFF, "Rohde&Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xF53000, 0xF53FFF, "BAYCOM OPTO-ELECTRONICS TECHNOLOGY CO.,LTD.", "BAYCOM-OPTO-ELECTRONICS" },
	{ 0x0050C2, 0xF54000, 0xF54FFF, "Hella Gutmann Solutions GmbH", "HELLA-GUTMANN-SOLUTIONS" },
	{ 0x0050C2, 0xF55000, 0xF55FFF, "Honeywell International Inc.", "HONEYWELL" },
	{ 0x0050C2, 0xF56000, 0xF56FFF, "Monsoon Solutions, Inc.", "MONSOON-SOLUTIONS" },
	{ 0x0050C2, 0xF57000, 0xF57FFF, "Reach Technologies Inc.", "REACH-TECHNOLOGIES" },
	{ 0x0050C2, 0xF58000, 0xF58FFF, "IEEE-SA", "IEEE" },
	{ 0x0050C2, 0xF59000, 0xF59FFF, "G3 Technologies", "G3-TECHNOLOGIES" },
	{ 0x0050C2, 0xF5A000, 0xF5AFFF, "Sentry 360 Security", "SENTRY-360-SECURITY" },
	{ 0x0050C2, 0xF5B000, 0xF5BFFF, "Saia-Burgess Controls AG", "SAIA-BURGESS-CONTROLS" },
	{ 0x0050C2, 0xF5C000, 0xF5CFFF, "DSP DESIGN LTD", "DSP-DESIGN" },
	{ 0x0050C2, 0xF5D000, 0xF5DFFF, "SMARTB TECHNOLOGIES", "SMARTB-TECHNOLOGIES" },
	{ 0x0050C2, 0xF5E000, 0xF5EFFF, "Y-cam Solutions Ltd", "Y-CAM-SOLUTIONS" },
	{ 0x0050C2, 0xF5F000, 0xF5FFFF, "BORYEU TECHNOLOGY CO.,LTD", "BORYEU" },
	{ 0x0050C2, 0xF60000, 0xF60FFF, "Deckma GmbH", "DECKMA" },
	{ 0x0050C2, 0xF61000, 0xF61FFF, "Brauch Elektronik GmbH&Co.KG", "BRAUCH-ELEKTRONIK" },
	{ 0x0050C2, 0xF62000, 0xF62FFF, "EMAC, Inc.", "EMAC" },
	{ 0x0050C2, 0xF63000, 0xF63FFF, "Triax A/S", "TRIAX-A/S" },
	{ 0x0050C2, 0xF64000, 0xF64FFF, "Chrisso Technologies LLC", "CHRISSO-TECHNOLOGIES-LLC" },
	{ 0x0050C2, 0xF65000, 0xF65FFF, "Telebyte Inc.", "TELEBYTE" },
	{ 0x0050C2, 0xF66000, 0xF66FFF, "GWT LLC", "GWT-LLC" },
	{ 0x0050C2, 0xF67000, 0xF67FFF, "Celestial Audio", "CELESTIAL-AUDIO" },
	{ 0x0050C2, 0xF68000, 0xF68FFF, "NEWTEC A/S", "NEWTEC-A/S" },
	{ 0x0050C2, 0xF69000, 0xF69FFF, "Safe Place Solutions Ltd", "SAFE-PLACE-SOLUTIONS" },
	{ 0x0050C2, 0xF6A000, 0xF6AFFF, "OFI Inc. (dba 2D2C)", "OFI-DBA-2D2C" },
	{ 0x0050C2, 0xF6B000, 0xF6BFFF, "Algodue Elettronica Srl", "ALGODUE-ELETTRONICA-SRL" },
	{ 0x0050C2, 0xF6C000, 0xF6CFFF, "Pro Design Electronic GmbH", "PRO-DESIGN" },
	{ 0x0050C2, 0xF6D000, 0xF6DFFF, "Pro Design Electronic GmbH", "PRO-DESIGN" },
	{ 0x0050C2, 0xF6E000, 0xF6EFFF, "Smith Meter, Inc.", "SMITH-METER" },
	{ 0x0050C2, 0xF6F000, 0xF6FFFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x0050C2, 0xF70000, 0xF70FFF, "Noralta Technologies Inc", "NORALTA-TECHNOLOGIES" },
	{ 0x0050C2, 0xF71000, 0xF71FFF, "RF CODE, INC", "RF-CODE" },
	{ 0x0050C2, 0xF72000, 0xF72FFF, "MaxDeTec AG", "MAXDETEC" },
	{ 0x0050C2, 0xF73000, 0xF73FFF, "DELTACAST.TV", "DELTACAST-TV" },
	{ 0x0050C2, 0xF74000, 0xF74FFF, "Thor Technologies Pty Ltd", "THOR-TECHNOLOGIES-PTY" },
	{ 0x0050C2, 0xF75000, 0xF75FFF, "PumpWell Solutions Ltd.", "PUMPWELL-SOLUTIONS" },
	{ 0x0050C2, 0xF76000, 0xF76FFF, "Rong Jie(FuZhou)Electronics Co.,Ltd", "RONG-JIE-FUZHOU-ELECTRONICS" },
	{ 0x0050C2, 0xF77000, 0xF77FFF, "SYSTEMTECHNIK GmbH", "SYSTEMTECHNIK" },
	{ 0x0050C2, 0xF78000, 0xF78FFF, "Gets MSS S.A.", "GETS-MSS" },
	{ 0x0050C2, 0xF79000, 0xF79FFF, "Tattile SRL", "TATTILE-SRL" },
	{ 0x0050C2, 0xF7A000, 0xF7AFFF, "C3 LLC", "C3-LLC" },
	{ 0x0050C2, 0xF7B000, 0xF7BFFF, "MCM Electronics", "MCM-ELECTRONICS" },
	{ 0x0050C2, 0xF7C000, 0xF7CFFF, "Atonometrics, Inc.", "ATONOMETRICS" },
	{ 0x0050C2, 0xF7D000, 0xF7DFFF, "D-Hike Electroncs Technology Co.,Ltd", "D-HIKE-ELECTRONCS" },
	{ 0x0050C2, 0xF7E000, 0xF7EFFF, "TruTeq Wireless (Pty) Ltd", "TRUTEQ-WIRELESS-PTY" },
	{ 0x0050C2, 0xF7F000, 0xF7FFFF, "Dynamic Design", "DYNAMIC-DESIGN" },
	{ 0x0050C2, 0xF80000, 0xF80FFF, "SYS TEC electronic GmbH", "SYS-TEC" },
	{ 0x0050C2, 0xF81000, 0xF81FFF, "PLDA", "PLDA" },
	{ 0x0050C2, 0xF82000, 0xF82FFF, "Sincair Systems International", "SINCAIR" },
	{ 0x0050C2, 0xF83000, 0xF83FFF, "GSP Sprachtechnologie GmbH", "GSP-SPRACHTECHNOLOGIE" },
	{ 0x0050C2, 0xF84000, 0xF84FFF, "Dynon Instruments", "DYNON-INSTRUMENTS" },
	{ 0x0050C2, 0xF85000, 0xF85FFF, "Enetics, Inc.", "ENETICS" },
	{ 0x0050C2, 0xF86000, 0xF86FFF, "Audio Power Labs", "AUDIO-POWER-LABS" },
	{ 0x0050C2, 0xF87000, 0xF87FFF, "Vaisala Oyj", "VAISALA-OYJ" },
	{ 0x0050C2, 0xF88000, 0xF88FFF, "RTC Manufacturing Inc.", "RTC-MANUFACTURING" },
	{ 0x0050C2, 0xF89000, 0xF89FFF, "CSA Engineering AG", "CSA-ENGINEERING" },
	{ 0x0050C2, 0xF8A000, 0xF8AFFF, "EMAC, Inc.", "EMAC" },
	{ 0x0050C2, 0xF8B000, 0xF8BFFF, "comlet Verteilte Systeme GmbH", "COMLET-VERTEILTE-SYSTEME" },
	{ 0x0050C2, 0xF8C000, 0xF8CFFF, "UBSTechnology Co., Ltd", "UBSTECHNOLOGY" },
	{ 0x0050C2, 0xF8D000, 0xF8DFFF, "GUANGDONG EAST POWER CO.,LTD.", "GUANGDONG-EAST-POWER" },
	{ 0x0050C2, 0xF8E000, 0xF8EFFF, "GPO", "GPO" },
	{ 0x0050C2, 0xF8F000, 0xF8FFFF, "Computerwise, Inc.", "COMPUTERWISE" },
	{ 0x0050C2, 0xF90000, 0xF90FFF, "SecureTech Systems, Inc.", "SECURETECH" },
	{ 0x0050C2, 0xF91000, 0xF91FFF, "RE2 Inc", "RE2" },
	{ 0x0050C2, 0xF92000, 0xF92FFF, "CONET Solutions GmbH", "CONET-SOLUTIONS" },
	{ 0x0050C2, 0xF93000, 0xF93FFF, "Baudisch Electronic GmbH", "BAUDISCH" },
	{ 0x0050C2, 0xF94000, 0xF94FFF, "Digital Barriers", "DIGITAL-BARRIERS" },
	{ 0x0050C2, 0xF95000, 0xF95FFF, "TTi LTD (Thurlby Thandar Instruments LTD)", "TTI-THURLBY-THANDAR-INSTRUMENTS" },
	{ 0x0050C2, 0xF96000, 0xF96FFF, "JLCooper Electronics", "JLCOOPER-ELECTRONICS" },
	{ 0x0050C2, 0xF97000, 0xF97FFF, "Sicon s.r.l.", "SICON-S-R-L" },
	{ 0x0050C2, 0xF98000, 0xF98FFF, "Infotech North America", "INFOTECH-NORTH-AMERICA" },
	{ 0x0050C2, 0xF99000, 0xF99FFF, "Dr. Neumann elektronik GmbH", "DR-NEUMANN-ELEKTRONIK" },
	{ 0x0050C2, 0xF9A000, 0xF9AFFF, "Telvent", "TELVENT" },
	{ 0x0050C2, 0xF9B000, 0xF9BFFF, "NEWELL TECHNOLOGIES LIMITED", "NEWELL-TECHNOLOGIES" },
	{ 0x0050C2, 0xF9C000, 0xF9CFFF, "R&D KOMETEH", "R-D-KOMETEH" },
	{ 0x0050C2, 0xF9D000, 0xF9DFFF, "JSC Kaluga Teletypes Manufacturing Plant", "JSC-KALUGA-TELETYPES-MANUFACTURING-PLANT" },
	{ 0x0050C2, 0xF9E000, 0xF9EFFF, "Matsusada Precision Inc.", "MATSUSADA-PRECISION" },
	{ 0x0050C2, 0xF9F000, 0xF9FFFF, "Nanjing SAC Power Grid Automation Co., Ltd.", "NANJING-SAC-POWER-GRID-AUTOMATION" },
	{ 0x0050C2, 0xFA0000, 0xFA0FFF, "Amplus Communication Pte Ltd", "AMPLUS-COMMUNICATION-PTE" },
	{ 0x0050C2, 0xFA1000, 0xFA1FFF, "N-Hands GmbH und Co KG", "N-HANDS-UND" },
	{ 0x0050C2, 0xFA2000, 0xFA2FFF, "Power-One", "POWER-ONE" },
	{ 0x0050C2, 0xFA3000, 0xFA3FFF, "Xemex NV", "XEMEX-NV" },
	{ 0x0050C2, 0xFA5000, 0xFA5FFF, "Intuitive Surgical, Inc.", "INTUITIVE-SURGICAL" },
	{ 0x0050C2, 0xFA6000, 0xFA6FFF, "Hilkom digital GmbH", "HILKOM-DIGITAL" },
	{ 0x0050C2, 0xFA7000, 0xFA7FFF, "Exelis Inc.", "EXELIS" },
	{ 0x0050C2, 0xFA8000, 0xFA8FFF, "Yash SiQure Technologies India Pvt. Ltd.", "YASH-SIQURE-TECHNOLOGIES-INDIA-PVT" },
	{ 0x0050C2, 0xFA9000, 0xFA9FFF, "Hijet Print d.o.o.", "HIJET-PRINT-D-O-O" },
	{ 0x0050C2, 0xFAA000, 0xFAAFFF, "YJSYSTEM", "YJSYSTEM" },
	{ 0x0050C2, 0xFAB000, 0xFABFFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x0050C2, 0xFAC000, 0xFACFFF, "ADETEL GROUP", "ADETEL-GROUP" },
	{ 0x0050C2, 0xFAD000, 0xFADFFF, "Finishing Brands", "FINISHING-BRANDS" },
	{ 0x0050C2, 0xFAE000, 0xFAEFFF, "ATI Automacao Telecomunicacoes e Informatica Ltda", "ATI-AUTOMACAO-TELECOMUNICACOES-E-INFORMATICA-LTDA" },
	{ 0x0050C2, 0xFAF000, 0xFAFFFF, "Vremya-CH JSC", "VREMYA-CH-JSC" },
	{ 0x0050C2, 0xFB0000, 0xFB0FFF, "Tateishi Kobisha Co.LTD", "TATEISHI-KOBISHA" },
	{ 0x0050C2, 0xFB1000, 0xFB1FFF, "MATELEX", "MATELEX" },
	{ 0x0050C2, 0xFB2000, 0xFB2FFF, "Preston Industries dba PolyScience", "PRESTON-INDUSTRIES-DBA-POLYSCIENCE" },
	{ 0x0050C2, 0xFB3000, 0xFB3FFF, "CT Company", "CT-COMPANY" },
	{ 0x0050C2, 0xFB4000, 0xFB4FFF, "MC-monitoring SA", "MC-MONITORING" },
	{ 0x0050C2, 0xFB5000, 0xFB5FFF, "Assembly Contracts Limited", "ASSEMBLY-CONTRACTS" },
	{ 0x0050C2, 0xFB6000, 0xFB6FFF, "ARGUS-SPECTRUM", "ARGUS-SPECTRUM" },
	{ 0x0050C2, 0xFB7000, 0xFB7FFF, "Pounce Consulting", "POUNCE-CONSULTING" },
	{ 0x0050C2, 0xFB8000, 0xFB8FFF, "TECHNO CO.,LTD.", "TECHNO" },
	{ 0x0050C2, 0xFB9000, 0xFB9FFF, "Coral Telecom Ltd", "CORAL-TELECOM" },
	{ 0x0050C2, 0xFBA000, 0xFBAFFF, "Elbit Systems of America", "ELBIT-OF-AMERICA" },
	{ 0x0050C2, 0xFBB000, 0xFBBFFF, "ACIDA GmbH", "ACIDA" },
	{ 0x0050C2, 0xFBC000, 0xFBCFFF, "Leroy Somer", "LEROY-SOMER" },
	{ 0x0050C2, 0xFBD000, 0xFBDFFF, "FHF Funke+Huster Fernsig GmbH", "FHF-FUNKE+HUSTER-FERNSIG" },
	{ 0x0050C2, 0xFBE000, 0xFBEFFF, "senTec Elektronik GmbH", "SENTEC-ELEKTRONIK" },
	{ 0x0050C2, 0xFBF000, 0xFBFFFF, "MYLOGIC", "MYLOGIC" },
	{ 0x0050C2, 0xFC0000, 0xFC0FFF, "Rohde&Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x0050C2, 0xFC1000, 0xFC1FFF, "Motec Pty Ltd", "MOTEC-PTY" },
	{ 0x0050C2, 0xFC2000, 0xFC2FFF, "ELTA", "ELTA" },
	{ 0x0050C2, 0xFC3000, 0xFC3FFF, "HSDC Sp. z o.o.", "HSDC-SP-Z-O-O" },
	{ 0x0050C2, 0xFC4000, 0xFC4FFF, "Kyowadensi", "KYOWADENSI" },
	{ 0x0050C2, 0xFC5000, 0xFC5FFF, "Sakura Seiki Co.,Ltd.", "SAKURA-SEIKI" },
	{ 0x0050C2, 0xFC6000, 0xFC6FFF, "Critical Link", "CRITICAL-LINK" },
	{ 0x0050C2, 0xFC7000, 0xFC7FFF, "SERCOM Regeltechniek", "SERCOM-REGELTECHNIEK" },
	{ 0x0050C2, 0xFC8000, 0xFC8FFF, "Far South Networks", "FAR-SOUTH-NETWORKS" },
	{ 0x0050C2, 0xFC9000, 0xFC9FFF, "Mehta Tech, Inc.", "MEHTA-TECH" },
	{ 0x0050C2, 0xFCA000, 0xFCAFFF, "Telemisis Ltd", "TELEMISIS" },
	{ 0x0050C2, 0xFCB000, 0xFCBFFF, "Propagation Systems Limited", "PROPAGATION" },
	{ 0x0050C2, 0xFCC000, 0xFCCFFF, "Soudronic AG", "SOUDRONIC" },
	{ 0x0050C2, 0xFCD000, 0xFCDFFF, "Jinyoung Contech", "JINYOUNG-CONTECH" },
	{ 0x0050C2, 0xFCE000, 0xFCEFFF, "KOYO ELECTRIC", "KOYO-ELECTRIC" },
	{ 0x0050C2, 0xFCF000, 0xFCFFFF, "DINTEK Shanghai Electronic Ltd", "DINTEK-SHANGHAI" },
	{ 0x0050C2, 0xFD0000, 0xFD0FFF, "Simple Solutions", "SIMPLE-SOLUTIONS" },
	{ 0x0050C2, 0xFD1000, 0xFD1FFF, "Enyx SA", "ENYX" },
	{ 0x0050C2, 0xFD2000, 0xFD2FFF, "Autonomic Controls. Inc", "AUTONOMIC-CONTROLS" },
	{ 0x0050C2, 0xFD3000, 0xFD3FFF, "Aster Electric Co.,Ltd.", "ASTER-ELECTRIC" },
	{ 0x0050C2, 0xFD4000, 0xFD4FFF, "Insitu, Inc.", "INSITU" },
	{ 0x0050C2, 0xFD5000, 0xFD5FFF, "American Microsystems, Ltd.", "AMERICAN-MICROSYSTEMS" },
	{ 0x0050C2, 0xFD6000, 0xFD6FFF, "City Computing Ltd", "CITY-COMPUTING" },
	{ 0x0050C2, 0xFD7000, 0xFD7FFF, "Deuta-Werke GmbH", "DEUTA-WERKE" },
	{ 0x0050C2, 0xFD8000, 0xFD8FFF, "Ease Inc.", "EASE" },
	{ 0x0050C2, 0xFD9000, 0xFD9FFF, "Figment Design Laboratories", "FIGMENT-DESIGN-LABORATORIES" },
	{ 0x0050C2, 0xFDA000, 0xFDAFFF, "ELAN SYSTEMS", "ELAN" },
	{ 0x0050C2, 0xFDB000, 0xFDBFFF, "The Security Center Inc", "THE-SECURITY-CENTER" },
	{ 0x0050C2, 0xFDC000, 0xFDCFFF, "QUERCUS TECHNOLOGIES, S.L.", "QUERCUS-TECHNOLOGIES-S-L" },
	{ 0x0050C2, 0xFDD000, 0xFDDFFF, "Toptech Systems, Inc.", "TOPTECH" },
	{ 0x0050C2, 0xFDE000, 0xFDEFFF, "Peek Traffic", "PEEK-TRAFFIC" },
	{ 0x0050C2, 0xFDF000, 0xFDFFFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x0050C2, 0xFE0000, 0xFE0FFF, "Azurtest", "AZURTEST" },
	{ 0x0050C2, 0xFE1000, 0xFE1FFF, "dotOcean", "DOTOCEAN" },
	{ 0x0050C2, 0xFE2000, 0xFE2FFF, "Pulsotronic Anlagentechnik GmbH", "PULSOTRONIC-ANLAGENTECHNIK" },
	{ 0x0050C2, 0xFE3000, 0xFE3FFF, "PRIVATE", "PRIVATE" },
	{ 0x0050C2, 0xFE4000, 0xFE4FFF, "RTT Mobile Interpretation", "RTT-MOBILE-INTERPRETATION" },
	{ 0x0050C2, 0xFE5000, 0xFE5FFF, "Scandinova Systems AB", "SCANDINOVA-AB" },
	{ 0x0050C2, 0xFE6000, 0xFE6FFF, "Exibea AB", "EXIBEA-AB" },
	{ 0x0050C2, 0xFE7000, 0xFE7FFF, "Erhardt+Leimer GmbH", "ERHARDT+LEIMER" },
	{ 0x0050C2, 0xFE8000, 0xFE8FFF, "Mango DSP, Inc.", "MANGO-DSP" },
	{ 0x0050C2, 0xFE9000, 0xFE9FFF, "MB Connect Line GmbH", "MB-CONNECT-LINE" },
	{ 0x0050C2, 0xFEA000, 0xFEAFFF, "Brunel GmbH Section Communications", "BRUNEL-SECTION-COMMUNICATION" },
	{ 0x0050C2, 0xFEB000, 0xFEBFFF, "Axible Technologies", "AXIBLE-TECHNOLOGIES" },
	{ 0x0050C2, 0xFEC000, 0xFECFFF, "First System Technology Co., Ltd.", "FIRST-SYSTEM" },
	{ 0x0050C2, 0xFED000, 0xFEDFFF, "LOGISOL Kft.", "LOGISOL-KFT" },
	{ 0x0050C2, 0xFEE000, 0xFEEFFF, "Sparks Instruments SA", "SPARKS-INSTRUMENTS" },
	{ 0x0050C2, 0xFEF000, 0xFEFFFF, "Task Sistemas de Computacao", "TASK-SISTEMAS-DE-COMPUTACAO" },
	{ 0x0050C2, 0xFF0000, 0xFF0FFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x0050C2, 0xFF1000, 0xFF1FFF, "DiTEST FAHRZEUGDIAGNOSE GMBH", "DITEST-FAHRZEUGDIAGNOSE" },
	{ 0x0050C2, 0xFF2000, 0xFF2FFF, "GLOBALCOM ENGINEERING SRL", "GLOBALCOM-ENGINEERING-SRL" },
	{ 0x0050C2, 0xFF3000, 0xFF3FFF, "CONTROL SYSTEMS Srl", "CONTROL-SRL" },
	{ 0x0050C2, 0xFF4000, 0xFF4FFF, "Burk Technology", "BURK" },
	{ 0x0050C2, 0xFF5000, 0xFF5FFF, "Flexkom Internet Pazarlama Bilipim ve Eoitim Hiz.Inp.Mim.Muh.Oto.Enerji San. Tic. A.p.", "FLEXKOM-INTERNET-PAZARLAMA-BILIPIM-VE-EOITIM-HIZ-INP-MIM-MUH-OTO-ENERJI-SAN-TIC-A-P" },
	{ 0x0050C2, 0xFF6000, 0xFF6FFF, "Booyco Electronics", "BOOYCO-ELECTRONICS" },
	{ 0x0050C2, 0xFF7000, 0xFF7FFF, "Human Intech", "HUMAN-INTECH" },
	{ 0x0050C2, 0xFF8000, 0xFF8FFF, "KST technology", "KST" },
	{ 0x0050C2, 0xFF9000, 0xFF9FFF, "Penttech AB", "PENTTECH-AB" },
	{ 0x0050C2, 0xFFA000, 0xFFAFFF, "Nupoint Systems Inc.", "NUPOINT" },
	{ 0x0050C2, 0xFFB000, 0xFFBFFF, "SEFRAM", "SEFRAM" },
	{ 0x0050C2, 0xFFC000, 0xFFCFFF, "Spirent Communications", "SPIRENT-COMMUNICATION" },
	{ 0x0050C2, 0xFFD000, 0xFFDFFF, "Touchless Biometric Systems AG", "TOUCHLESS-BIOMETRIC" },
	{ 0x0050C2, 0xFFE000, 0xFFEFFF, "Sensata Technologies", "SENSATA-TECHNOLOGIES" },
	{ 0x0050C2, 0xFFF000, 0xFFFFFF, "MSR-Solutions GmbH", "MSR-SOLUTIONS" },
	{ 0x40D855, 0x000000, 0x000FFF, "XRONOS.INC", "XRONOS" },
	{ 0x40D855, 0x001000, 0x001FFF, "Vemotion", "VEMOTION" },
	{ 0x40D855, 0x002000, 0x002FFF, "Hangzhou Chenxiao Technologies Co. Ltd.", "HANGZHOU-CHENXIAO-TECHNOLOGIES" },
	{ 0x40D855, 0x003000, 0x003FFF, "AlphaNavigation coltd", "ALPHANAVIGATION-COLTD" },
	{ 0x40D855, 0x004000, 0x004FFF, "CR Magnetics, Inc.", "CR-MAGNETICS" },
	{ 0x40D855, 0x005000, 0x005FFF, "Monarch Instrument", "MONARCH-INSTRUMENT" },
	{ 0x40D855, 0x006000, 0x006FFF, "Bactest Limited", "BACTEST" },
	{ 0x40D855, 0x007000, 0x007FFF, "Digital Audio SA", "DIGITAL-AUDIO" },
	{ 0x40D855, 0x008000, 0x008FFF, "Kaori Industria Eletronica Ltda", "KAORI-INDUSTRIA-ELETRONICA-LTDA" },
	{ 0x40D855, 0x009000, 0x009FFF, "ClearSite Communications Inc.", "CLEARSITE-COMMUNICATION" },
	{ 0x40D855, 0x00A000, 0x00AFFF, "Sarana Sistem Mikro", "SARANA-SISTEM-MIKRO" },
	{ 0x40D855, 0x00B000, 0x00BFFF, "Aircell", "AIRCELL" },
	{ 0x40D855, 0x00C000, 0x00CFFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x40D855, 0x00D000, 0x00DFFF, "HuNS", "HUNS" },
	{ 0x40D855, 0x00E000, 0x00EFFF, "Brightwell Dispensers", "BRIGHTWELL-DISPENSERS" },
	{ 0x40D855, 0x00F000, 0x00FFFF, "DIGITAL DYNAMICS, INC.", "DIGITAL-DYNAMICS" },
	{ 0x40D855, 0x010000, 0x010FFF, "APG CASH DRAWER", "APG-CASH-DRAWER" },
	{ 0x40D855, 0x011000, 0x011FFF, "Flexim Security Oy", "FLEXIM-SECURITY-OY" },
	{ 0x40D855, 0x012000, 0x012FFF, "Sencon Inc.", "SENCON" },
	{ 0x40D855, 0x013000, 0x013FFF, "Grande Vitesse Systems", "GRANDE-VITESSE" },
	{ 0x40D855, 0x014000, 0x014FFF, "Toni Studio", "TONI-STUDIO" },
	{ 0x40D855, 0x015000, 0x015FFF, "BITMILL srl", "BITMILL-SRL" },
	{ 0x40D855, 0x016000, 0x016FFF, "Par-Tech, Inc.", "PAR-TECH" },
	{ 0x40D855, 0x017000, 0x017FFF, "Franke Aquarotter GmbH", "FRANKE-AQUAROTTER" },
	{ 0x40D855, 0x018000, 0x018FFF, "STANEO SAS", "STANEO-SAS" },
	{ 0x40D855, 0x019000, 0x019FFF, "Nautel Limited", "NAUTEL" },
	{ 0x40D855, 0x01A000, 0x01AFFF, "MEGGITT DEFENSE SYSTEMS INC.", "MEGGITT-DEFENSE" },
	{ 0x40D855, 0x01B000, 0x01BFFF, "Audio Enhancement", "AUDIO-ENHANCEMENT" },
	{ 0x40D855, 0x01C000, 0x01CFFF, "BERG Cloud Limited", "BERG-CLOUD" },
	{ 0x40D855, 0x01D000, 0x01DFFF, "Scharco Elektronik GmbH", "SCHARCO-ELEKTRONIK" },
	{ 0x40D855, 0x01E000, 0x01EFFF, "A2S", "A2S" },
	{ 0x40D855, 0x01F000, 0x01FFFF, "Sitep Italia Spa", "SITEP-ITALIA-SPA" },
	{ 0x40D855, 0x020000, 0x020FFF, "ENTEC Electric & Electronic CO., LTD.", "ENTEC-ELECTRIC" },
	{ 0x40D855, 0x021000, 0x021FFF, "SMT D.O.O.", "SMT-D-O-O" },
	{ 0x40D855, 0x022000, 0x022FFF, "Digimerge Technology Inc", "DIGIMERGE" },
	{ 0x40D855, 0x023000, 0x023FFF, "Shanghai o-solution electronics & Technology Co., Ltd.", "SHANGHAI-O-SOLUTION-ELECTRONICS" },
	{ 0x40D855, 0x024000, 0x024FFF, "Electrical Geodesics Incorporated", "ELECTRICAL-GEODESICS-INCORPORATED" },
	{ 0x40D855, 0x025000, 0x025FFF, "Rosemount Analytical", "ROSEMOUNT-ANALYTICAL" },
	{ 0x40D855, 0x026000, 0x026FFF, "Symetrics Industries", "SYMETRICS-INDUSTRIES" },
	{ 0x40D855, 0x027000, 0x027FFF, "GRUPO EPELSA S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x40D855, 0x028000, 0x028FFF, "Integrated Control Corp.", "INTEGRATED-CONTROL" },
	{ 0x40D855, 0x029000, 0x029FFF, "Depro Electronique", "DEPRO-ELECTRONIQUE" },
	{ 0x40D855, 0x02A000, 0x02AFFF, "Tinkerforge GmbH", "TINKERFORGE" },
	{ 0x40D855, 0x02B000, 0x02BFFF, "Nomatronics", "NOMATRONICS" },
	{ 0x40D855, 0x02C000, 0x02CFFF, "InventLab s.c.", "INVENTLAB-S-C" },
	{ 0x40D855, 0x02D000, 0x02DFFF, "Elgama Sistemos", "ELGAMA-SISTEMOS" },
	{ 0x40D855, 0x02E000, 0x02EFFF, "Circuitec Ind. Equip. Eletr. Ltda", "CIRCUITEC-IND-EQUIP-ELETR-LTDA" },
	{ 0x40D855, 0x02F000, 0x02FFFF, "Adva Technologies", "ADVA-TECHNOLOGIES" },
	{ 0x40D855, 0x030000, 0x030FFF, "Tecnologias Plexus", "TECNOLOGIAS-PLEXUS" },
	{ 0x40D855, 0x031000, 0x031FFF, "Dommel GmbH", "DOMMEL" },
	{ 0x40D855, 0x032000, 0x032FFF, "BETTINI SRL", "BETTINI-SRL" },
	{ 0x40D855, 0x033000, 0x033FFF, "Ermes Elettronica s.r.l.", "ERMES-ELETTRONICA-S-R-L" },
	{ 0x40D855, 0x034000, 0x034FFF, "Dacom West GmbH", "DACOM-WEST" },
	{ 0x40D855, 0x035000, 0x035FFF, "Mesotech International, Inc.", "MESOTECH" },
	{ 0x40D855, 0x036000, 0x036FFF, "Schweers informationstechnologie GmbH", "SCHWEERS-INFORMATIONSTECHNOLOGIE" },
	{ 0x40D855, 0x037000, 0x037FFF, "Software Workshop", "SOFTWARE-WORKSHOP" },
	{ 0x40D855, 0x038000, 0x038FFF, "Special Measurements Labs LLC", "SPECIAL-MEASUREMENTS-LABS-LLC" },
	{ 0x40D855, 0x039000, 0x039FFF, "CI Systems Ltd", "CI" },
	{ 0x40D855, 0x03A000, 0x03AFFF, "Socus networks", "SOCUS-NETWORKS" },
	{ 0x40D855, 0x03B000, 0x03BFFF, "Telcomkorea", "TELCOMKOREA" },
	{ 0x40D855, 0x03C000, 0x03CFFF, "Computer System Co.,Ltd", "SYSTEM" },
	{ 0x40D855, 0x03D000, 0x03DFFF, "Tekelek Europe Ltd", "TEKELEK-EUROPE" },
	{ 0x40D855, 0x03E000, 0x03EFFF, "Vishay Celtron Technologies, Inc.", "VISHAY-CELTRON-TECHNOLOGIES" },
	{ 0x40D855, 0x03F000, 0x03FFFF, "UniSVR Global Information Technology Corp.", "UNISVR-GLOBAL-INFORMATION" },
	{ 0x40D855, 0x040000, 0x040FFF, "GHL Systems Berhad", "GHL-BERHAD" },
	{ 0x40D855, 0x041000, 0x041FFF, "T.Q.M. Itaca Technology s.r.l.", "T-Q-M-ITACA-S-R-L" },
	{ 0x40D855, 0x042000, 0x042FFF, "Mango Communicaitons Inc.", "MANGO-COMMUNICAITONS" },
	{ 0x40D855, 0x043000, 0x043FFF, "SchulerControl GmbH", "SCHULERCONTROL" },
	{ 0x40D855, 0x044000, 0x044FFF, "An Chen Computer Co. Ltd.", "AN-CHEN" },
	{ 0x40D855, 0x045000, 0x045FFF, "Genadsystem", "GENADSYSTEM" },
	{ 0x40D855, 0x046000, 0x046FFF, "Circuitlink Pty Ltd", "CIRCUITLINK-PTY" },
	{ 0x40D855, 0x047000, 0x047FFF, "Dos&Donts SRL", "DOS-DONTS-SRL" },
	{ 0x40D855, 0x048000, 0x048FFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x40D855, 0x049000, 0x049FFF, "Thermo Fisher Scientific", "THERMO-FISHER-SCIENTIFIC" },
	{ 0x40D855, 0x04A000, 0x04AFFF, "Gateway Technologies SA de CV", "GATEWAY-TECHNOLOGIES-DE-CV" },
	{ 0x40D855, 0x04B000, 0x04BFFF, "Vital Tech Industria e Comercio Ltda", "VITAL-TECH-INDUSTRIA-E-COMERCIO-LTDA" },
	{ 0x40D855, 0x04C000, 0x04CFFF, "Serveron Corporation", "SERVERON" },
	{ 0x40D855, 0x04D000, 0x04DFFF, "MACHINEPERFORMANCE ApS", "MACHINEPERFORMANCE-APS" },
	{ 0x40D855, 0x04E000, 0x04EFFF, "Honeywell Aerospace/Intelligent Automation Corp.", "HONEYWELL-AEROSPACE/INTELLIGENT-AUTOMATION" },
	{ 0x40D855, 0x04F000, 0x04FFFF, "Haein S&S Co., Ltd", "HAEIN-S-S" },
	{ 0x40D855, 0x050000, 0x050FFF, "ATG UV Technology", "ATG-UV" },
	{ 0x40D855, 0x051000, 0x051FFF, "CS Instruments Asia", "CS-INSTRUMENTS-ASIA" },
	{ 0x40D855, 0x052000, 0x052FFF, "DAN ELECTRONICS SYSTEM (P) LIMITED", "DAN-ELECTRONICS-SYSTEM-P" },
	{ 0x40D855, 0x053000, 0x053FFF, "Amantys Ltd", "AMANTYS" },
	{ 0x40D855, 0x054000, 0x054FFF, "VITEC", "VITEC" },
	{ 0x40D855, 0x055000, 0x055FFF, "Helmholtz Zentrum Dresden Rossendorf e.V.", "HELMHOLTZ-ZENTRUM-DRESDEN-ROSSENDORF-E-V" },
	{ 0x40D855, 0x056000, 0x056FFF, "GROUP 57", "GROUP-57" },
	{ 0x40D855, 0x057000, 0x057FFF, "Tammermatic Group Oy", "TAMMERMATIC-GROUP-OY" },
	{ 0x40D855, 0x058000, 0x058FFF, "Energy Team S.p.A.", "ENERGY-TEAM-S-P-A" },
	{ 0x40D855, 0x059000, 0x059FFF, "COLONIAL ASSEMBLY and DESIGN", "COLONIAL-ASSEMBLY-AND-DESIGN" },
	{ 0x40D855, 0x05A000, 0x05AFFF, "Ultra Electronics Flightline Systems", "ULTRA-ELECTRONICS-FLIGHTLINE" },
	{ 0x40D855, 0x05B000, 0x05BFFF, "Data Flow Systems, Inc.", "DATA-FLOW" },
	{ 0x40D855, 0x05C000, 0x05CFFF, "Rosslare Enterprises Limited", "ROSSLARE-ENTERPRISES" },
	{ 0x40D855, 0x05D000, 0x05DFFF, "Leica Biosystems", "LEICA-BIOSYSTEMS" },
	{ 0x40D855, 0x05E000, 0x05EFFF, "inoage GmbH", "INOAGE" },
	{ 0x40D855, 0x05F000, 0x05FFFF, "EPSa GmbH", "EPSA" },
	{ 0x40D855, 0x060000, 0x060FFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x40D855, 0x061000, 0x061FFF, "Cominfo, Inc.", "COMINFO" },
	{ 0x40D855, 0x062000, 0x062FFF, "Tech Source Inc", "TECH-SOURCE" },
	{ 0x40D855, 0x063000, 0x063FFF, "Protonic Holland", "PROTONIC-HOLLAND" },
	{ 0x40D855, 0x064000, 0x064FFF, "HIPODROMO DE AGUA CALIENTE, S.A. DE C.V.", "HIPODROMO-DE-AGUA-CALIENTE-DE-C-V" },
	{ 0x40D855, 0x065000, 0x065FFF, "Parallel Wireless", "PARALLEL-WIRELESS" },
	{ 0x40D855, 0x066000, 0x066FFF, "TeraTron GmbH", "TERATRON" },
	{ 0x40D855, 0x067000, 0x067FFF, "Tronic Control ltd.", "TRONIC-CONTROL" },
	{ 0x40D855, 0x068000, 0x068FFF, "Oki Seatec Co., Ltd.", "OKI-SEATEC" },
	{ 0x40D855, 0x069000, 0x069FFF, "Smartcom-Bulgaria AD", "SMARTCOM-BULGARIA-AD" },
	{ 0x40D855, 0x06A000, 0x06AFFF, "elgris", "ELGRIS" },
	{ 0x40D855, 0x06B000, 0x06BFFF, "BRS Sistemas Eletronicos", "BRS-SISTEMAS-ELETRONICOS" },
	{ 0x40D855, 0x06C000, 0x06CFFF, "Rohde&Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x40D855, 0x06D000, 0x06DFFF, "BroadSoft, INC", "BROADSOFT" },
	{ 0x40D855, 0x06E000, 0x06EFFF, "C-COM Satellite Systems Inc.", "C-COM-SATELLITE" },
	{ 0x40D855, 0x06F000, 0x06FFFF, "DORLET SA", "DORLET" },
	{ 0x40D855, 0x070000, 0x070FFF, "JSC Electrical Equipment Factory", "JSC-ELECTRICAL-EQUIPMENT-FACTORY" },
	{ 0x40D855, 0x071000, 0x071FFF, "TATTILE SRL", "TATTILE-SRL" },
	{ 0x40D855, 0x072000, 0x072FFF, "CT Company", "CT-COMPANY" },
	{ 0x40D855, 0x073000, 0x073FFF, "Diamond Technologies, Inc", "DIAMOND-TECHNOLOGIES" },
	{ 0x40D855, 0x074000, 0x074FFF, "Sphere Medical Ltd", "SPHERE-MEDICAL" },
	{ 0x40D855, 0x075000, 0x075FFF, "Teraflops", "TERAFLOPS" },
	{ 0x40D855, 0x076000, 0x076FFF, "INTERNET PROTOCOLO LOGICA SL", "INTERNET-PROTOCOLO-LOGICA-SL" },
	{ 0x40D855, 0x077000, 0x077FFF, "TOEC TECHNOLOGY CO.,LTD", "TOEC" },
	{ 0x40D855, 0x078000, 0x078FFF, "NACHI-FUJIKOSHI CORP", "NACHI-FUJIKOSHI" },
	{ 0x40D855, 0x079000, 0x079FFF, "DelfiSolutions A/S", "DELFISOLUTIONS-A/S" },
	{ 0x40D855, 0x07A000, 0x07AFFF, "4embedded", "4EMBEDDED" },
	{ 0x40D855, 0x07B000, 0x07BFFF, "IPS Technology Limited", "IPS" },
	{ 0x40D855, 0x07C000, 0x07CFFF, "Agramkow Fluid Systems A/S", "AGRAMKOW-FLUID-A/S" },
	{ 0x40D855, 0x07D000, 0x07DFFF, "Wuxi SiNeng New Energy Co., Ltd.", "WUXI-SINENG-NEW-ENERGY" },
	{ 0x40D855, 0x07E000, 0x07EFFF, "TESCOM CORPORATION", "TESCOM" },
	{ 0x40D855, 0x07F000, 0x07FFFF, "Wheatstone Corporation", "WHEATSTONE" },
	{ 0x40D855, 0x080000, 0x080FFF, "Honeywell", "HONEYWELL" },
	{ 0x40D855, 0x081000, 0x081FFF, "Sicon srl", "SICON-SRL" },
	{ 0x40D855, 0x082000, 0x082FFF, "ard sa", "ARD" },
	{ 0x40D855, 0x083000, 0x083FFF, "DELOPT", "DELOPT" },
	{ 0x40D855, 0x084000, 0x084FFF, "Papendorf Software Engineering GmbH", "PAPENDORF-SOFTWARE-ENGINEERING" },
	{ 0x40D855, 0x085000, 0x085FFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x40D855, 0x086000, 0x086FFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x40D855, 0x087000, 0x087FFF, "Bestel China", "BESTEL-CHINA" },
	{ 0x40D855, 0x088000, 0x088FFF, "JEL SYSTEM CO., LTD.", "JEL-SYSTEM" },
	{ 0x40D855, 0x089000, 0x089FFF, "WUHAN XINGTUXINKE ELECTRONIC CO.,LTD", "WUHAN-XINGTUXINKE" },
	{ 0x40D855, 0x08A000, 0x08AFFF, "Leder Elektronik Design", "LEDER-ELEKTRONIK-DESIGN" },
	{ 0x40D855, 0x08B000, 0x08BFFF, "MeshWorks Wireless Oy", "MESHWORKS-WIRELESS-OY" },
	{ 0x40D855, 0x08C000, 0x08CFFF, "Magnescale Co.,Ltd", "MAGNESCALE" },
	{ 0x40D855, 0x08D000, 0x08DFFF, "Boehme Nachrichtentechnik", "BOEHME-NACHRICHTENTECHNIK" },
	{ 0x40D855, 0x08E000, 0x08EFFF, "Lyngsoe Systems", "LYNGSOE" },
	{ 0x40D855, 0x08F000, 0x08FFFF, "Excelitas", "EXCELITAS" },
	{ 0x40D855, 0x090000, 0x090FFF, "Axxess Identification Ltd", "AXXESS-IDENTIFICATION" },
	{ 0x40D855, 0x091000, 0x091FFF, "KDT", "KDT" },
	{ 0x40D855, 0x092000, 0x092FFF, "Wasserbauer GmbH", "WASSERBAUER" },
	{ 0x40D855, 0x093000, 0x093FFF, "Sentry 360 Security", "SENTRY-360-SECURITY" },
	{ 0x40D855, 0x094000, 0x094FFF, "Nomad Digital Limited", "NOMAD-DIGITAL" },
	{ 0x40D855, 0x095000, 0x095FFF, "Heart Force Medical", "HEART-FORCE-MEDICAL" },
	{ 0x40D855, 0x096000, 0x096FFF, "Comtel Electronics GmbH", "COMTEL-ELECTRONICS" },
	{ 0x40D855, 0x097000, 0x097FFF, "Burton Technical Services LLC", "BURTON-TECHNICAL-SERVICES-LLC" },
	{ 0x40D855, 0x098000, 0x098FFF, "Dave Srl", "DAVE-SRL" },
	{ 0x40D855, 0x099000, 0x099FFF, "idcell co.ltd", "IDCELL" },
	{ 0x40D855, 0x09A000, 0x09AFFF, "CoherentPlus Sdn Bhd", "COHERENTPLUS-SDN-BHD" },
	{ 0x40D855, 0x09B000, 0x09BFFF, "Tokyo Denki Gijutsu Kogyo", "TOKYO-DENKI-GIJUTSU-KOGYO" },
	{ 0x40D855, 0x09C000, 0x09CFFF, "Keyware Solutions Inc.", "KEYWARE-SOLUTIONS" },
	{ 0x40D855, 0x09D000, 0x09DFFF, "EMAC, Inc.", "EMAC" },
	{ 0x40D855, 0x09E000, 0x09EFFF, "NanoPulse, Inc.", "NANOPULSE" },
	{ 0x40D855, 0x09F000, 0x09FFFF, "Bascules Robbe nv", "BASCULES-ROBBE-NV" },
	{ 0x40D855, 0x0A0000, 0x0A0FFF, "Quantronix, Inc.", "QUANTRONIX" },
	{ 0x40D855, 0x0A1000, 0x0A1FFF, "ADVALY SYSTEM Inc.", "ADVALY-SYSTEM" },
	{ 0x40D855, 0x0A2000, 0x0A2FFF, "Xemex NV", "XEMEX-NV" },
	{ 0x40D855, 0x0A3000, 0x0A3FFF, "Telefrank GmbH", "TELEFRANK" },
	{ 0x40D855, 0x0A4000, 0x0A4FFF, "Resch Electronic Innovation GmbH", "RESCH-INNOVATION" },
	{ 0x40D855, 0x0A5000, 0x0A5FFF, "WooshCom Corporation", "WOOSHCOM" },
	{ 0x40D855, 0x0A6000, 0x0A6FFF, "Alumbra Produtos Elétricos e Eletrônicos Ltda", "ALUMBRA-PRODUTOS-ELéTRICOS-E-ELETRôNICOS-LTDA" },
	{ 0x40D855, 0x0A7000, 0x0A7FFF, "First Design System Inc.", "FIRST-DESIGN-SYSTEM" },
	{ 0x40D855, 0x0A8000, 0x0A8FFF, "Baudisch Electronic GmbH", "BAUDISCH" },
	{ 0x40D855, 0x0A9000, 0x0A9FFF, "Apantac LLC", "APANTAC-LLC" },
	{ 0x40D855, 0x0AA000, 0x0AAFFF, "Thermal Imaging Radar, LLC", "THERMAL-IMAGING-RADAR-LLC" },
	{ 0x40D855, 0x0AB000, 0x0ABFFF, "Enel doo Belgrade", "ENEL-DOO-BELGRADE" },
	{ 0x40D855, 0x0AC000, 0x0ACFFF, "Fraunhofer HHI", "FRAUNHOFER-HHI" },
	{ 0x40D855, 0x0AD000, 0x0ADFFF, "Space Micro", "SPACE-MICRO" },
	{ 0x40D855, 0x0AE000, 0x0AEFFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x40D855, 0x0AF000, 0x0AFFFF, "EnVerv Inc.", "ENVERV" },
	{ 0x40D855, 0x0B0000, 0x0B0FFF, "Micrologic", "MICROLOGIC" },
	{ 0x40D855, 0x0B1000, 0x0B1FFF, "Nanjing TIANSU Automation Control System Co., Ltd.", "NANJING-TIANSU-AUTOMATION-CONTROL-SYSTEM" },
	{ 0x40D855, 0x0B2000, 0x0B2FFF, "Ever Trend Technology Development Limited", "EVER-TREND-DEVELOPMENT" },
	{ 0x40D855, 0x0B3000, 0x0B3FFF, "T.W.S. srl", "T-W-S-SRL" },
	{ 0x40D855, 0x0B4000, 0x0B4FFF, "MITSUBISHI ELECTRIC SYSTEM & SERVICE CO.,LTD.", "MITSUBISHI-ELECTRIC-SYSTEM-SERVICE" },
	{ 0x40D855, 0x0B5000, 0x0B5FFF, "DATA SHARING CONSULTING", "DATA-SHARING-CONSULTING" },
	{ 0x40D855, 0x0B6000, 0x0B6FFF, "Telvent", "TELVENT" },
	{ 0x40D855, 0x0B7000, 0x0B7FFF, "ACD Elektronik GmbH", "ACD-ELEKTRONIK" },
	{ 0x40D855, 0x0B8000, 0x0B8FFF, "Ferlin Trading BV", "FERLIN-TRADING" },
	{ 0x40D855, 0x0B9000, 0x0B9FFF, "WxBR Sistemas de Telecomunicacoes Ltda", "WXBR-SISTEMAS-DE-TELECOMUNICACOES-LTDA" },
	{ 0x40D855, 0x0BA000, 0x0BAFFF, "PCH Engineering A/S", "PCH-ENGINEERING-A/S" },
	{ 0x40D855, 0x0BB000, 0x0BBFFF, "Whiptail", "WHIPTAIL" },
	{ 0x40D855, 0x0BC000, 0x0BCFFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x40D855, 0x0BD000, 0x0BDFFF, "iCOGNIZE GmbH", "ICOGNIZE" },
	{ 0x40D855, 0x0BE000, 0x0BEFFF, "Manufacturing System Insights Inc", "MANUFACTURING-SYSTEM-INSIGHTS" },
	{ 0x40D855, 0x0BF000, 0x0BFFFF, "Shenzhen SETEC Power Co.,Ltd", "SHENZHEN-SETEC-POWER" },
	{ 0x40D855, 0x0C0000, 0x0C0FFF, "ACT", "ACT" },
	{ 0x40D855, 0x0C1000, 0x0C1FFF, "Xepto Computing Inc", "XEPTO-COMPUTING" },
	{ 0x40D855, 0x0C2000, 0x0C2FFF, "SC Techswarm SRL", "SC-TECHSWARM-SRL" },
	{ 0x40D855, 0x0C3000, 0x0C3FFF, "APG CASH DRAWER", "APG-CASH-DRAWER" },
	{ 0x40D855, 0x0C4000, 0x0C4FFF, "Inspired Systems", "INSPIRED" },
	{ 0x40D855, 0x0C5000, 0x0C5FFF, "M.M. Elektrolab", "M-M-ELEKTROLAB" },
	{ 0x40D855, 0x0C6000, 0x0C6FFF, "comtime GmbH", "COMTIME" },
	{ 0x40D855, 0x0C7000, 0x0C7FFF, "insensiv GmbH", "INSENSIV" },
	{ 0x40D855, 0x0C8000, 0x0C8FFF, "Mettler Toledo Hispeed", "METTLER-TOLEDO-HISPEED" },
	{ 0x40D855, 0x0C9000, 0x0C9FFF, "QUANTAFLOW", "QUANTAFLOW" },
	{ 0x40D855, 0x0CA000, 0x0CAFFF, "NEUTRIK AG", "NEUTRIK" },
	{ 0x40D855, 0x0CB000, 0x0CBFFF, "ReliOn Inc", "RELION" },
	{ 0x40D855, 0x0CC000, 0x0CCFFF, "ATEME", "ATEME" },
	{ 0x40D855, 0x0CD000, 0x0CDFFF, "Logical Product", "LOGICAL-PRODUCT" },
	{ 0x40D855, 0x0CE000, 0x0CEFFF, "EST Analytical", "EST-ANALYTICAL" },
	{ 0x40D855, 0x0CF000, 0x0CFFFF, "Clark-MXR, Inc.", "CLARK-MXR" },
	{ 0x40D855, 0x0D0000, 0x0D0FFF, "Icraft Oy", "ICRAFT-OY" },
	{ 0x40D855, 0x0D1000, 0x0D1FFF, "Cantada Inc", "CANTADA" },
	{ 0x40D855, 0x0D2000, 0x0D2FFF, "ELAN SYSTEMS", "ELAN" },
	{ 0x40D855, 0x0D3000, 0x0D3FFF, "LECO Corporation", "LECO" },
	{ 0x40D855, 0x0D4000, 0x0D4FFF, "Mitsubishi Heavy Industries, Ltd.", "MITSUBISHI-HEAVY-INDUSTRIES" },
	{ 0x40D855, 0x0D5000, 0x0D5FFF, "Shimizu Electric Co., Ltd.", "SHIMIZU-ELECTRIC" },
	{ 0x40D855, 0x0D6000, 0x0D6FFF, "deskontrol electronics", "DESKONTROL-ELECTRONICS" },
	{ 0x40D855, 0x0D7000, 0x0D7FFF, "Avant Technologies,Inc", "AVANT-TECHNOLOGIES" },
	{ 0x40D855, 0x0D8000, 0x0D8FFF, "NEXT! s.c. S.Piela B.Dryja", "NEXT!-S-C-S-PIELA-B-DRYJA" },
	{ 0x40D855, 0x0D9000, 0x0D9FFF, "YUKO ELECTRIC CO.,LTD", "YUKO-ELECTRIC" },
	{ 0x40D855, 0x0DA000, 0x0DAFFF, "Devialet SA", "DEVIALET" },
	{ 0x40D855, 0x0DB000, 0x0DBFFF, "Top Connect OU", "TOP-CONNECT-OU" },
	{ 0x40D855, 0x0DC000, 0x0DCFFF, "NVS Technologies Inc", "NVS-TECHNOLOGIES" },
	{ 0x40D855, 0x0DD000, 0x0DDFFF, "Embed Limited", "EMBED" },
	{ 0x40D855, 0x0DE000, 0x0DEFFF, "VPG", "VPG" },
	{ 0x40D855, 0x0DF000, 0x0DFFFF, "Xadi Inc", "XADI" },
	{ 0x40D855, 0x0E0000, 0x0E0FFF, "Richter", "RICHTER" },
	{ 0x40D855, 0x0E1000, 0x0E1FFF, "STV Electronic GmbH", "STV" },
	{ 0x40D855, 0x0E2000, 0x0E2FFF, "Keocko International", "KEOCKO" },
	{ 0x40D855, 0x0E3000, 0x0E3FFF, "Medigus Ltd", "MEDIGUS" },
	{ 0x40D855, 0x0E4000, 0x0E4FFF, "ARAGO SYSTEMS", "ARAGO" },
	{ 0x40D855, 0x0E5000, 0x0E5FFF, "Triton Electronics LTD", "TRITON-ELECTRONICS" },
	{ 0x40D855, 0x0E6000, 0x0E6FFF, "Kyoritsu Electric Corp.", "KYORITSU-ELECTRIC" },
	{ 0x40D855, 0x0E7000, 0x0E7FFF, "LIGHTSTAR", "LIGHTSTAR" },
	{ 0x40D855, 0x0E8000, 0x0E8FFF, "HEITEC AG", "HEITEC" },
	{ 0x40D855, 0x0E9000, 0x0E9FFF, "HAMEG GmbH", "HAMEG" },
	{ 0x40D855, 0x0EA000, 0x0EAFFF, "A-Z-E", "A-Z-E" },
	{ 0x40D855, 0x0EB000, 0x0EBFFF, "WANTECH Networks", "WANTECH-NETWORKS" },
	{ 0x40D855, 0x0EC000, 0x0ECFFF, "Sentry 360 Security", "SENTRY-360-SECURITY" },
	{ 0x40D855, 0x0ED000, 0x0EDFFF, "IntelliDesign Pty Ltd", "INTELLIDESIGN-PTY" },
	{ 0x40D855, 0x0EE000, 0x0EEFFF, "Siegmar Zander HuSWare", "SIEGMAR-ZANDER-HUSWARE" },
	{ 0x40D855, 0x0EF000, 0x0EFFFF, "GeneSys Elektronik GmbH", "GENESYS-ELEKTRONIK" },
	{ 0x40D855, 0x0F0000, 0x0F0FFF, "Redwood Systems", "REDWOOD" },
	{ 0x40D855, 0x0F1000, 0x0F1FFF, "Grossenbacher Systeme AG", "GROSSENBACHER-SYSTEME" },
	{ 0x40D855, 0x0F2000, 0x0F2FFF, "SigmaPhi Electronics", "SIGMAPHI-ELECTRONICS" },
	{ 0x40D855, 0x0F3000, 0x0F3FFF, "ECON Systems Inc.", "ECON" },
	{ 0x40D855, 0x0F4000, 0x0F4FFF, "MB Connect Line GmbH", "MB-CONNECT-LINE" },
	{ 0x40D855, 0x0F5000, 0x0F5FFF, "CST Group", "CST-GROUP" },
	{ 0x40D855, 0x0F6000, 0x0F6FFF, "PRIVATE", "PRIVATE" },
	{ 0x40D855, 0x0F7000, 0x0F7FFF, "Comline Elektronik Elektrotechnik GmbH", "COMLINE-ELEKTRONIK-ELEKTROTECHNIK" },
	{ 0x40D855, 0x0F8000, 0x0F8FFF, "Better Place", "BETTER-PLACE" },
	{ 0x40D855, 0x0F9000, 0x0F9FFF, "Invisua Lighting BV", "INVISUA-LIGHTING" },
	{ 0x40D855, 0x0FA000, 0x0FAFFF, "Marmitek BV", "MARMITEK" },
	{ 0x40D855, 0x0FB000, 0x0FBFFF, "InfoMac Sp. z o. o. Sp. k.", "INFOMAC-SP-Z-O-O-SP-K" },
	{ 0x40D855, 0x0FC000, 0x0FCFFF, "eumig industrie-tv GmbH", "EUMIG-INDUSTRIE-TV" },
	{ 0x40D855, 0x0FD000, 0x0FDFFF, "MONOGRAM technologies ltd", "MONOGRAM-TECHNOLOGIES" },
	{ 0x40D855, 0x0FE000, 0x0FEFFF, "Cytech Technology Pte Ltd", "CYTECH-PTE" },
	{ 0x40D855, 0x0FF000, 0x0FFFFF, "YUYAMA　MFG．ＣＯ．，ＬＴＤ．", "YUYAMA　MFG．ＣＯ．，ＬＴＤ．" },
	{ 0x40D855, 0x100000, 0x100FFF, "TASK SISTEMAS DE COMPUTACAO S.A.", "TASK-SISTEMAS-DE-COMPUTACAO" },
	{ 0x40D855, 0x101000, 0x101FFF, "e.p.g. Elettronica Srl", "E-P-G-ELETTRONICA-SRL" },
	{ 0x40D855, 0x102000, 0x102FFF, "Power Electronics", "POWER-ELECTRONICS" },
	{ 0x40D855, 0x103000, 0x103FFF, "Peek Traffic Corporation", "PEEK-TRAFFIC" },
	{ 0x40D855, 0x104000, 0x104FFF, "IMPLE SISTEMAS ELETRONICOS EMBARCADOS LTDA", "IMPLE-SISTEMAS-ELETRONICOS-EMBARCADOS-LTDA" },
	{ 0x40D855, 0x105000, 0x105FFF, "Tieline Research Pty Ltd", "TIELINE-RESEARCH-PTY" },
	{ 0x40D855, 0x106000, 0x106FFF, "Orbital A/S", "ORBITAL-A/S" },
	{ 0x40D855, 0x107000, 0x107FFF, "Smith Meter, Inc", "SMITH-METER" },
	{ 0x40D855, 0x108000, 0x108FFF, "ALPHA DESIGN CO.,LTD.", "ALPHA-DESIGN" },
	{ 0x40D855, 0x109000, 0x109FFF, "Rosslare Enterprises Limited", "ROSSLARE-ENTERPRISES" },
	{ 0x40D855, 0x10A000, 0x10AFFF, "DAVIS DERBY LIMITED", "DAVIS-DERBY" },
	{ 0x40D855, 0x10B000, 0x10BFFF, "So-Cool Corporation.", "SO-COOL" },
	{ 0x40D855, 0x10C000, 0x10CFFF, "Contrans TI sp. z o.o.", "CONTRANS-TI-SP-Z-O-O" },
	{ 0x40D855, 0x10D000, 0x10DFFF, "Rite-Tech Industrial CO., Ltd.", "RITE-TECH-INDUSTRIAL" },
	{ 0x40D855, 0x10E000, 0x10EFFF, "HKS-Prozesstechnik GmbH", "HKS-PROZESSTECHNIK" },
	{ 0x40D855, 0x10F000, 0x10FFFF, "CAVALRY STORAGE INC", "CAVALRY-STORAGE" },
	{ 0x40D855, 0x110000, 0x110FFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x40D855, 0x111000, 0x111FFF, "Grupo Epelsa S.L.", "GRUPO-EPELSA-S-L" },
	{ 0x40D855, 0x112000, 0x112FFF, "Halliburton - Sperry Drilling Service", "HALLIBURTON---SPERRY-DRILLING-SERVICE" },
	{ 0x40D855, 0x113000, 0x113FFF, "Testbook Ltd", "TESTBOOK" },
	{ 0x40D855, 0x114000, 0x114FFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x40D855, 0x115000, 0x115FFF, "MESA Electronic GmbH", "MESA" },
	{ 0x40D855, 0x116000, 0x116FFF, "Uniscan LLC", "UNISCAN-LLC" },
	{ 0x40D855, 0x117000, 0x117FFF, "RCS Energy Management Limited", "RCS-ENERGY-MANAGEMENT" },
	{ 0x40D855, 0x118000, 0x118FFF, "University of Nebraska -- Lincoln", "UNIVERSITY-OF-NEBRASKA----LINCOLN" },
	{ 0x40D855, 0x119000, 0x119FFF, "OOO Group of Industrial Technologies", "OOO-GROUP-OF-INDUSTRIAL-TECHNOLOGIES" },
	{ 0x40D855, 0x11A000, 0x11AFFF, "Sicon srl", "SICON-SRL" },
	{ 0x40D855, 0x11B000, 0x11BFFF, "nanoTRONIC GmbH", "NANOTRONIC" },
	{ 0x40D855, 0x11C000, 0x11CFFF, "DEUTA-WERKE GmbH", "DEUTA-WERKE" },
	{ 0x40D855, 0x11D000, 0x11DFFF, "ACD Elektronik GmBH", "ACD-ELEKTRONIK" },
	{ 0x40D855, 0x11E000, 0x11EFFF, "CEMSI, Inc.", "CEMSI" },
	{ 0x40D855, 0x11F000, 0x11FFFF, "KOMPAN Pawel Sokolowski", "KOMPAN-PAWEL-SOKOLOWSKI" },
	{ 0x40D855, 0x120000, 0x120FFF, "ObjectFab GmbH", "OBJECTFAB" },
	{ 0x40D855, 0x121000, 0x121FFF, "shanghai Anjian Information technology co. , ltd.", "SHANGHAI-ANJIAN-INFORMATION" },
	{ 0x40D855, 0x122000, 0x122FFF, "ATX Networks Ltd.", "ATX-NETWORKS" },
	{ 0x40D855, 0x123000, 0x123FFF, "ZAO NPC Kompjuternie Technologii", "ZAO-NPC-KOMPJUTERNIE-TECHNOLOGII" },
	{ 0x40D855, 0x124000, 0x124FFF, "Debug s.r.l.", "DEBUG-S-R-L" },
	{ 0x40D855, 0x125000, 0x125FFF, "Scandyna A/S", "SCANDYNA-A/S" },
	{ 0x40D855, 0x126000, 0x126FFF, "TTI LTD", "TTI" },
	{ 0x40D855, 0x127000, 0x127FFF, "LIGHTSTAR", "LIGHTSTAR" },
	{ 0x40D855, 0x128000, 0x128FFF, "Akse srl", "AKSE-SRL" },
	{ 0x40D855, 0x129000, 0x129FFF, "DSP DESIGN", "DSP-DESIGN" },
	{ 0x40D855, 0x12A000, 0x12AFFF, "Jadpod Communication Company Limited", "JADPOD-COMMUNICATION-COMPANY" },
	{ 0x40D855, 0x12B000, 0x12BFFF, "Mango DSP, Inc.", "MANGO-DSP" },
	{ 0x40D855, 0x12C000, 0x12CFFF, "NSP Europe Ltd", "NSP-EUROPE" },
	{ 0x40D855, 0x12D000, 0x12DFFF, "Biotage Sweden AB", "BIOTAGE-SWEDEN-AB" },
	{ 0x40D855, 0x12E000, 0x12EFFF, "Canfield Scientific, Inc.", "CANFIELD-SCIENTIFIC" },
	{ 0x40D855, 0x12F000, 0x12FFFF, "PRIVATE", "PRIVATE" },
	{ 0x40D855, 0x130000, 0x130FFF, "GSP Sprachtechnologie GmbH", "GSP-SPRACHTECHNOLOGIE" },
	{ 0x40D855, 0x131000, 0x131FFF, "EMAC, INC.", "EMAC" },
	{ 0x40D855, 0x132000, 0x132FFF, "AeroVision Avionics, Inc", "AEROVISION-AVIONICS" },
	{ 0x40D855, 0x133000, 0x133FFF, "Tattile srl", "TATTILE-SRL" },
	{ 0x40D855, 0x134000, 0x134FFF, "digitech GmbH &amp; Co. KG", "DIGITECH-AMP" },
	{ 0x40D855, 0x135000, 0x135FFF, "GLOBALCOM ENGINEERING SRL", "GLOBALCOM-ENGINEERING-SRL" },
	{ 0x40D855, 0x136000, 0x136FFF, "Devriecom B.V.", "DEVRIECOM" },
	{ 0x40D855, 0x137000, 0x137FFF, "GDE Polska", "GDE-POLSKA" },
	{ 0x40D855, 0x138000, 0x138FFF, "Calon Associates Limited", "CALON-ASSOCIATES" },
	{ 0x40D855, 0x139000, 0x139FFF, "WOW System", "WOW-SYSTEM" },
	{ 0x40D855, 0x13A000, 0x13AFFF, "Supplier Ind. e Com de Eletroeletrônicos", "SUPPLIER-IND-E-COM-DE-ELETROELETRôNICOS" },
	{ 0x40D855, 0x13B000, 0x13BFFF, "Davin Technologies Co.,Ltd", "DAVIN-TECHNOLOGIES" },
	{ 0x40D855, 0x13C000, 0x13CFFF, "shanghai anjian Information technology co. , ltd.", "SHANGHAI-ANJIAN-INFORMATION" },
	{ 0x40D855, 0x13D000, 0x13DFFF, "Perm Scientific-Industrial Instrument Making Company JSC", "PERM-SCIENTIFIC-INDUSTRIAL-INSTRUMENT-MAKING-COMPANY-JSC" },
	{ 0x40D855, 0x13E000, 0x13EFFF, "hanatech", "HANATECH" },
	{ 0x40D855, 0x13F000, 0x13FFFF, "Zhejiang Wellsun Electric Meter Co.,Ltd", "ZHEJIANG-WELLSUN-ELECTRIC-METER" },
	{ 0x40D855, 0x140000, 0x140FFF, "InnoTrans Communications, Inc", "INNOTRANS-COMMUNICATION" },
	{ 0x40D855, 0x141000, 0x141FFF, "Key Systems, Inc.", "KEY" },
	{ 0x40D855, 0x142000, 0x142FFF, "Tetracore, Inc.", "TETRACORE" },
	{ 0x40D855, 0x143000, 0x143FFF, "Tokyo Drawing Ltd.", "TOKYO-DRAWING" },
	{ 0x40D855, 0x144000, 0x144FFF, "Venco", "VENCO" },
	{ 0x40D855, 0x145000, 0x145FFF, "Weber Marking Systems GmbH", "WEBER-MARKING" },
	{ 0x40D855, 0x146000, 0x146FFF, "Pleiger Elektronik GmbH and Co. KG", "PLEIGER-ELEKTRONIK-AND" },
	{ 0x40D855, 0x147000, 0x147FFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x40D855, 0x148000, 0x148FFF, "SEIKO TIME SYSTEMS INC.", "SEIKO-TIME" },
	{ 0x40D855, 0x149000, 0x149FFF, "Engage Technologies", "ENGAGE-TECHNOLOGIES" },
	{ 0x40D855, 0x14A000, 0x14AFFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x40D855, 0x14B000, 0x14BFFF, "Vocality International", "VOCALITY" },
	{ 0x40D855, 0x14C000, 0x14CFFF, "PLT", "PLT" },
	{ 0x40D855, 0x14D000, 0x14DFFF, "SOMFY SAS", "SOMFY-SAS" },
	{ 0x40D855, 0x14E000, 0x14EFFF, "Marposs S.p.A", "MARPOSS-S-P-A" },
	{ 0x40D855, 0x14F000, 0x14FFFF, "TDS Software Solutions Pty Ltd", "TDS-SOFTWARE-SOLUTIONS-PTY" },
	{ 0x40D855, 0x150000, 0x150FFF, "SHIKINO HIGH-TECH", "SHIKINO-HIGH-TECH" },
	{ 0x40D855, 0x151000, 0x151FFF, "Progress Rail Services, Inspection and Information Systems", "PROGRESS-RAIL-SERVICES-INSPECTION-AND-INFORMATION" },
	{ 0x40D855, 0x152000, 0x152FFF, "Home Automation Europe", "HOME-AUTOMATION-EUROPE" },
	{ 0x40D855, 0x153000, 0x153FFF, "BlinkPipe Ltd", "BLINKPIPE" },
	{ 0x40D855, 0x154000, 0x154FFF, "iart", "IART" },
	{ 0x40D855, 0x155000, 0x155FFF, "Telefrang AB", "TELEFRANG-AB" },
	{ 0x40D855, 0x156000, 0x156FFF, "Emphysys, Inc.", "EMPHYSYS" },
	{ 0x40D855, 0x157000, 0x157FFF, "Hitachi Power Solutions Co., Ltd.", "HITACHI-POWER-SOLUTIONS" },
	{ 0x40D855, 0x158000, 0x158FFF, "Exibea AB", "EXIBEA-AB" },
	{ 0x40D855, 0x159000, 0x159FFF, "PLATINUM GmbH", "PLATINUM" },
	{ 0x40D855, 0x15A000, 0x15AFFF, "DORLET S.A.U", "DORLET-U" },
	{ 0x40D855, 0x15B000, 0x15BFFF, "SQF Spezialelektronik GmbH", "SQF-SPEZIALELEKTRONIK" },
	{ 0x40D855, 0x15C000, 0x15CFFF, "Spectratech Inc.", "SPECTRATECH" },
	{ 0x40D855, 0x15D000, 0x15DFFF, "Actronic Technologies", "ACTRONIC-TECHNOLOGIES" },
	{ 0x40D855, 0x15E000, 0x15EFFF, "Prodco International Inc.", "PRODCO" },
	{ 0x40D855, 0x15F000, 0x15FFFF, "CT COMPANY", "CT-COMPANY" },
	{ 0x40D855, 0x160000, 0x160FFF, "Thermo Fisher Sceintific", "THERMO-FISHER-SCEINTIFIC" },
	{ 0x40D855, 0x161000, 0x161FFF, "Solidscape Inc", "SOLIDSCAPE" },
	{ 0x40D855, 0x162000, 0x162FFF, "LUNA-NEXUS", "LUNA-NEXUS" },
	{ 0x40D855, 0x163000, 0x163FFF, "KMtronic LTD", "KMTRONIC" },
	{ 0x40D855, 0x164000, 0x164FFF, "NFT Automatisierungssysteme GmbH", "NFT-AUTOMATISIERUNGSSYSTEME" },
	{ 0x40D855, 0x165000, 0x165FFF, "TECHBOARD SRL", "TECHBOARD-SRL" },
	{ 0x40D855, 0x166000, 0x166FFF, "Anhui Jiante Network Technology Co., Ltd.", "ANHUI-JIANTE-NETWORK" },
	{ 0x40D855, 0x167000, 0x167FFF, "Assembly Contracts Ltd", "ASSEMBLY-CONTRACTS" },
	{ 0x40D855, 0x168000, 0x168FFF, "OPASCA Systems GmbH", "OPASCA" },
	{ 0x40D855, 0x169000, 0x169FFF, "Photop Koncent", "PHOTOP-KONCENT" },
	{ 0x40D855, 0x16A000, 0x16AFFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x40D855, 0x16B000, 0x16BFFF, "TECHWAY", "TECHWAY" },
	{ 0x40D855, 0x16C000, 0x16CFFF, "PRIVATE", "PRIVATE" },
	{ 0x40D855, 0x16D000, 0x16DFFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x40D855, 0x16E000, 0x16EFFF, "Secuinfo Co.Ltd", "SECUINFO" },
	{ 0x40D855, 0x16F000, 0x16FFFF, "BrightLeaf Power", "BRIGHTLEAF-POWER" },
	{ 0x40D855, 0x170000, 0x170FFF, "ICS Eletronics", "ICS-ELETRONICS" },
	{ 0x40D855, 0x171000, 0x171FFF, "Sicon srl", "SICON-SRL" },
	{ 0x40D855, 0x172000, 0x172FFF, "YAWATA ELECTRIC INDUSTRIAL CO.,LTD.", "YAWATA-ELECTRIC-INDUSTRIAL" },
	{ 0x40D855, 0x173000, 0x173FFF, "Contec Steuerungstechnik & Automation GmbH", "CONTEC-STEUERUNGSTECHNIK-AUTOMATION" },
	{ 0x40D855, 0x174000, 0x174FFF, "EcoGuard AB", "ECOGUARD-AB" },
	{ 0x40D855, 0x175000, 0x175FFF, "AHB Systeme GmbH", "AHB-SYSTEME" },
	{ 0x40D855, 0x176000, 0x176FFF, "Schneider Electric Motion, Inc. USA", "SCHNEIDER-ELECTRIC-MOTION-USA" },
	{ 0x40D855, 0x177000, 0x177FFF, "TRI Engineering co.,ltd.", "TRI-ENGINEERING" },
	{ 0x40D855, 0x178000, 0x178FFF, "REDER Domotic GmbH", "REDER-DOMOTIC" },
	{ 0x40D855, 0x179000, 0x179FFF, "Servo-Robot Inc.", "SERVO-ROBOT" },
	{ 0x40D855, 0x17A000, 0x17AFFF, "ARGUS-SPECTRUM", "ARGUS-SPECTRUM" },
	{ 0x40D855, 0x17B000, 0x17BFFF, "LUCEO", "LUCEO" },
	{ 0x40D855, 0x17C000, 0x17CFFF, "Critical Link", "CRITICAL-LINK" },
	{ 0x40D855, 0x17D000, 0x17DFFF, "Kiwigrid GmbH", "KIWIGRID" },
	{ 0x40D855, 0x17E000, 0x17EFFF, "TOKHATEC", "TOKHATEC" },
	{ 0x40D855, 0x17F000, 0x17FFFF, "Telvent", "TELVENT" },
	{ 0x40D855, 0x180000, 0x180FFF, "BroadSoft Inc", "BROADSOFT" },
	{ 0x40D855, 0x181000, 0x181FFF, "eROCCA", "EROCCA" },
	{ 0x40D855, 0x182000, 0x182FFF, "Georg Neumann GmbH", "GEORG-NEUMANN" },
	{ 0x40D855, 0x183000, 0x183FFF, "EMAC, Inc.", "EMAC" },
	{ 0x40D855, 0x184000, 0x184FFF, "Satkirit Ltd", "SATKIRIT" },
	{ 0x40D855, 0x185000, 0x185FFF, "Standard Change Makers", "STANDARD-CHANGE-MAKERS" },
	{ 0x40D855, 0x186000, 0x186FFF, "KST technology", "KST" },
	{ 0x40D855, 0x187000, 0x187FFF, "CDEX Corp.", "CDEX" },
	{ 0x40D855, 0x188000, 0x188FFF, "Array Corporation", "ARRAY" },
	{ 0x40D855, 0x189000, 0x189FFF, "Yoozma Corporation", "YOOZMA" },
	{ 0x40D855, 0x18A000, 0x18AFFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x40D855, 0x18B000, 0x18BFFF, "Diagnosys Test Systems Ltd", "DIAGNOSYS-TEST" },
	{ 0x40D855, 0x18C000, 0x18CFFF, "EOS S.r.l.", "EOS-S-R-L" },
	{ 0x40D855, 0x18D000, 0x18DFFF, "Zoe Medical", "ZOE-MEDICAL" },
	{ 0x40D855, 0x18E000, 0x18EFFF, "Kerun Visual Technology Co., Ltd.(Shenzhen)", "KERUN-VISUAL-SHENZHEN" },
	{ 0x40D855, 0x18F000, 0x18FFFF, "Beat Sensing co. , ltd.", "BEAT-SENSING" },
	{ 0x40D855, 0x190000, 0x190FFF, "Spider Tecnologia Ind. e Com Ltda", "SPIDER-TECNOLOGIA-IND-E-COM-LTDA" },
	{ 0x40D855, 0x191000, 0x191FFF, "Soukai Electric", "SOUKAI-ELECTRIC" },
	{ 0x40D855, 0x192000, 0x192FFF, "GENERAL DYNAMICS C4 SYSTEMS", "GENERAL-DYNAMICS-C4" },
	{ 0x40D855, 0x193000, 0x193FFF, "FORZA SILICON CORP.", "FORZA-SILICON" },
	{ 0x40D855, 0x194000, 0x194FFF, "RF Code", "RF-CODE" },
	{ 0x40D855, 0x195000, 0x195FFF, "TONNA ELECTRONIQUE", "TONNA-ELECTRONIQUE" },
	{ 0x40D855, 0x196000, 0x196FFF, "Advanced Micro Controls Inc.", "ADVANCED-MICRO-CONTROLS" },
	{ 0x40D855, 0x197000, 0x197FFF, "Berg Cloud Limited", "BERG-CLOUD" },
	{ 0x40D855, 0x198000, 0x198FFF, "devboards GmbH", "DEVBOARDS" },
	{ 0x40D855, 0x199000, 0x199FFF, "PRESSOL Schmiergeraete GmbH", "PRESSOL-SCHMIERGERAETE" },
	{ 0x40D855, 0x19A000, 0x19AFFF, "Rohde&Schwarz Topex SA", "ROHDE-SCHWARZ-TOPEX" },
	{ 0x40D855, 0x19B000, 0x19BFFF, "Northern Star Technologies", "NORTHERN-STAR-TECHNOLOGIES" },
	{ 0x40D855, 0x19C000, 0x19CFFF, "Parris Service Corporation", "PARRIS-SERVICE" },
	{ 0x40D855, 0x19D000, 0x19DFFF, "EMAC, Inc.", "EMAC" },
	{ 0x40D855, 0x19E000, 0x19EFFF, "Thirdwayv Inc.", "THIRDWAYV" },
	{ 0x40D855, 0x19F000, 0x19FFFF, "Patria Aviation Oy", "PATRIA-AVIATION-OY" },
	{ 0x40D855, 0x1A0000, 0x1A0FFF, "Futaba Corporation", "FUTABA" },
	{ 0x40D855, 0x1A1000, 0x1A1FFF, "KRONOTECH SRL", "KRONOTECH-SRL" },
	{ 0x40D855, 0x1A2000, 0x1A2FFF, "HIPODROMO DE AGUA CALIENTE, S.A. DE C.V.", "HIPODROMO-DE-AGUA-CALIENTE-DE-C-V" },
	{ 0x40D855, 0x1A3000, 0x1A3FFF, "Noritake Itron Corporation", "NORITAKE-ITRON" },
	{ 0x40D855, 0x1A4000, 0x1A4FFF, "cibite AG", "CIBITE" },
	{ 0x40D855, 0x1A5000, 0x1A5FFF, "DemoPad", "DEMOPAD" },
	{ 0x40D855, 0x1A6000, 0x1A6FFF, "RB-LINK Wireless", "RB-LINK-WIRELESS" },
	{ 0x40D855, 0x1A7000, 0x1A7FFF, "ENTEC Electric & Electronic CO., LTD", "ENTEC-ELECTRIC" },
	{ 0x40D855, 0x1A8000, 0x1A8FFF, "Multiobrabotka", "MULTIOBRABOTKA" },
	{ 0x40D855, 0x1A9000, 0x1A9FFF, "Lubino s.r.o.", "LUBINO-S-R-O" },
	{ 0x40D855, 0x1AA000, 0x1AAFFF, "Broachlink Technology Co.,Limited", "BROACHLINK" },
	{ 0x40D855, 0x1AB000, 0x1ABFFF, "Rosslare Enterprises Limited", "ROSSLARE-ENTERPRISES" },
	{ 0x40D855, 0x1AC000, 0x1ACFFF, "ELAN SYSTEMS", "ELAN" },
	{ 0x40D855, 0x1AD000, 0x1ADFFF, "WICHER DIGITAL TECHNIK", "WICHER-DIGITAL-TECHNIK" },
	{ 0x40D855, 0x1AE000, 0x1AEFFF, "Autonomous Solutions, Inc", "AUTONOMOUS-SOLUTIONS" },
	{ 0x40D855, 0x1AF000, 0x1AFFFF, "Vigitron Inc.", "VIGITRON" },
	{ 0x40D855, 0x1B0000, 0x1B0FFF, "Shin-ei Electronic Measuring Co.,Ltd.", "SHIN-EI-MEASURING" },
	{ 0x40D855, 0x1B1000, 0x1B1FFF, "Logos 01 S.r.l.", "LOGOS-01-S-R-L" },
	{ 0x40D855, 0x1B2000, 0x1B2FFF, "AGE A. Gilg Elektronik", "AGE-A-GILG-ELEKTRONIK" },
	{ 0x40D855, 0x1B3000, 0x1B3FFF, "BETTINI SRL", "BETTINI-SRL" },
	{ 0x40D855, 0x1B4000, 0x1B4FFF, "Inforce Computing Inc.", "INFORCE-COMPUTING" },
	{ 0x40D855, 0x1B5000, 0x1B5FFF, "A+EC Klein Ingenieurbuero", "A+EC-KLEIN-INGENIEURBUERO" },
	{ 0x40D855, 0x1B6000, 0x1B6FFF, "Magic Systems", "MAGIC" },
	{ 0x40D855, 0x1B7000, 0x1B7FFF, "TEWS Elektronik GmbH & Co. KG", "TEWS-ELEKTRONIK" },
	{ 0x40D855, 0x1B8000, 0x1B8FFF, "Orion Systems, Inc", "ORION" },
	{ 0x40D855, 0x1B9000, 0x1B9FFF, "Beking Industrieele automatisering", "BEKING-INDUSTRIEELE-AUTOMATISERING" },
	{ 0x40D855, 0x1BA000, 0x1BAFFF, "Creative Lighting And Sound Systems Pty Ltd", "CREATIVE-LIGHTING-AND-SOUND-PTY" },
	{ 0x40D855, 0x1BB000, 0x1BBFFF, "Micromega Dynamics SA", "MICROMEGA-DYNAMICS" },
	{ 0x40D855, 0x1BC000, 0x1BCFFF, "KbDevice,Inc.", "KBDEVICE" },
	{ 0x40D855, 0x1BD000, 0x1BDFFF, "HORIBA ABX", "HORIBA-ABX" },
	{ 0x40D855, 0x1BE000, 0x1BEFFF, "PEEK TRAFFIC", "PEEK-TRAFFIC" },
	{ 0x40D855, 0x1BF000, 0x1BFFFF, "shanghai mingding information tech co.Ltd", "SHANGHAI-MINGDING-INFORMATION-TECH" },
	{ 0x40D855, 0x1C0000, 0x1C0FFF, "NPB Automation AB", "NPB-AUTOMATION-AB" },
	{ 0x40D855, 0x1C1000, 0x1C1FFF, "Triamec Motion AG", "TRIAMEC-MOTION" },
	{ 0x40D855, 0x1C2000, 0x1C2FFF, "Digital Display Systems", "DIGITAL-DISPLAY" },
	{ 0x40D855, 0x1C3000, 0x1C3FFF, "Cornfed Systems LLC", "CORNFED-LLC" },
	{ 0x40D855, 0x1C4000, 0x1C4FFF, "QED Advanced Systems Limited", "QED-ADVANCED" },
	{ 0x40D855, 0x1C5000, 0x1C5FFF, "PRIVATE", "PRIVATE" },
	{ 0x40D855, 0x1C6000, 0x1C6FFF, "Device Solutions Ltd", "DEVICE-SOLUTIONS" },
	{ 0x40D855, 0x1C7000, 0x1C7FFF, "Wexiödisk AB", "WEXIOEDISK-AB" },
	{ 0x40D855, 0x1C8000, 0x1C8FFF, "Sensata Technologies", "SENSATA-TECHNOLOGIES" },
	{ 0x40D855, 0x1C9000, 0x1C9FFF, "Andy-L Ltd.", "ANDY-L" },
	{ 0x40D855, 0x1CA000, 0x1CAFFF, "Rigel Engineering", "RIGEL-ENGINEERING" },
	{ 0x40D855, 0x1CB000, 0x1CBFFF, "MG S.r.l.", "MG-S-R-L" },
	{ 0x40D855, 0x1CC000, 0x1CCFFF, "NKT Photonics A/S", "NKT-PHOTONICS-A/S" },
	{ 0x40D855, 0x1CD000, 0x1CDFFF, "YXLON International A/S", "YXLON-A/S" },
	{ 0x40D855, 0x1CE000, 0x1CEFFF, "Peter Huber", "PETER-HUBER" },
	{ 0x40D855, 0x1CF000, 0x1CFFFF, "Omnik New Energy Co., Ltd", "OMNIK-NEW-ENERGY" },
	{ 0x40D855, 0x1D0000, 0x1D0FFF, "Webeasy BV", "WEBEASY" },
	{ 0x40D855, 0x1D1000, 0x1D1FFF, "Founder Broadband Network Service Co.,Ltd.", "FOUNDER-BROADBAND-NETWORK-SERVICE" },
	{ 0x40D855, 0x1D2000, 0x1D2FFF, "InventLab s.c.", "INVENTLAB-S-C" },
	{ 0x40D855, 0x1D3000, 0x1D3FFF, "Kaluga Teletypes Manufacturing Plant", "KALUGA-TELETYPES-MANUFACTURING-PLANT" },
	{ 0x40D855, 0x1D4000, 0x1D4FFF, "Prisma Engineering srl", "PRISMA-ENGINEERING-SRL" },
	{ 0x40D855, 0x1D5000, 0x1D5FFF, "FST21 Ltd.", "FST21" },
	{ 0x40D855, 0x1D6000, 0x1D6FFF, "EMS Computers Pty Ltd", "EMS-COMPUTERS-PTY" },
	{ 0x40D855, 0x1D7000, 0x1D7FFF, "Wheatstone Corporation", "WHEATSTONE" },
	{ 0x40D855, 0x1D8000, 0x1D8FFF, "Owl Computing Technologies, Inc.", "OWL-COMPUTING-TECHNOLOGIES" },
	{ 0x40D855, 0x1D9000, 0x1D9FFF, "Commercial Wireless Systems International LLC.", "COMMERCIAL-WIRELESS-LLC" },
	{ 0x40D855, 0x1DA000, 0x1DAFFF, "Energy Technology and Control Ltd.", "ENERGY-AND-CONTROL" },
	{ 0x40D855, 0x1DB000, 0x1DBFFF, "NIPPON TECHNO LAB.,INC,", "NIPPON-TECHNO-LAB" },
	{ 0x40D855, 0x1DC000, 0x1DCFFF, "Aplex Technology Inc.", "APLEX" },
	{ 0x40D855, 0x1DD000, 0x1DDFFF, "BaOpt Benelux bv", "BAOPT-BENELUX" },
	{ 0x40D855, 0x1DE000, 0x1DEFFF, "Vidisys GmbH", "VIDISYS" },
	{ 0x40D855, 0x1DF000, 0x1DFFFF, "Chengdu Meihuan Technology Co.,Ltd", "CHENGDU-MEIHUAN" },
	{ 0x40D855, 0x1E0000, 0x1E0FFF, "Embedded Technology Corporation", "EMBEDDED" },
	{ 0x40D855, 0x1E1000, 0x1E1FFF, "AD QUALITE", "AD-QUALITE" },
	{ 0x40D855, 0x1E2000, 0x1E2FFF, "ELNEC s.r.o.", "ELNEC-S-R-O" },
	{ 0x40D855, 0x1E3000, 0x1E3FFF, "Mega Electronics Ltd", "MEGA-ELECTRONICS" },
	{ 0x40D855, 0x1E4000, 0x1E4FFF, "STEK Ltd", "STEK" },
	{ 0x40D855, 0x50D000, 0x50DFFF, "Shenzhen MaiWei Cable TV Equipment CO.,LTD.", "SHENZHEN-MAIWEI-CABLE-TV-EQUIPMENT" },
	{ 0x40D855, 0xEE6000, 0xEE6FFF, "Narinet, Inc.", "NARINET" },

};
